// FILE: ./assets/js/util/CRM_Utilities.js
function CRM_Utilities(ScreenService, templateManager, ZHPromise, restEngine, General, ValidationEngine) {

/*function CRM_Utilities(ScreenService, templateManager, Operador_Repository, ZHPromise, restEngine, General, ValidationEngine) {
*/

    /**
     * Função para limpar todos os valores de uma widget recebida
     * como parâmetro.
     * 
     * @param {obejct} widget widget na qual será removido os valores
     * de todas as fields.
     * 
    */
    this.cleanWidgetFields = function(widget){
        widget.fields.forEach(function(field){
            field.value("");    
        });   
    };
    
    
    this.CATCH_EXCEPTION_ERROR = function(error) {
        return 'Ocorreu um erro inesperado, contate o administrador informando o trecho abaixo: <br><br>' + error + '<br><br>';
    };

    
    /**
     * Função para retornar as informações do
     * operador logado.
     * 
     * @return {object} loggedOperatorInfos objeto
     * que contém todas as informações do operador
     * logado.
    */
    this.getLoggedOperatorInfos = function(){
        
        var loggedOperatorInfos = templateManager.LOGGEDOPERATOR;
        
        return loggedOperatorInfos;
            
    };
	
	/**
     * Função para setar as informações do operador 
     * logado em um array na widget recebida.
     * 
     * @param {object} destinationObject objeto destino 
     * das informações do operador logado.
     * 
    */
    /*
	this.setOperatorLoggedInfosInCurrentRow = function(destinationObject){
        
        return Operador_Repository.getOperadorLogado().then(function(operator){
        
            if(!operator[0]){
                
                ScreenService.showMessage("Não foi possível carregar os dados do operador logado. Entre em contato com um administrador sobre o erro.");
                return;
            }
        
            destinationObject.OPERATORLOGGED = operator[0];
            
            }, 
            function(error){
                ScreenService.showMessage("Não foi possível carregar os dados do operador logado. Entre em contato com um administrador sobre o erro.");
            }
        );  
    }; 
    */
    /**
     * Retorna o index da widget localizada, pelo nome
     * recebido como parâmetro da função, no array de widgets,
     * também recebido como parâmetro.
     * 
     * @param {object} widgets array de widgets onde é feito
     * a busca.
     * 
     * @ param {string} widgetName nome da widget a ser
     * localizada.
     * 
     * @param {int} indexWidgetObject index da widget localizada
     * dentro do array de wigets.
    */
    this.getWidgetPosition = function(widgets, widgetName){
        
        var indexWidgetObject;
        
        widgets.forEach(function(widget, indexWidget){
            
            if(widget.name === widgetName){
                indexWidgetObject = indexWidget;
                return;
            }
                
        });
        
        return indexWidgetObject;
            
    };
    
    /**
     * Função para identificar o index de um objeto
     * dentro de um dataSourceFilter, de acordo com 
     * seu nome.
     * 
     * @param {object} dataSourceFilter objeto da busca.
     * 
     * @param {String} name propriedade no objeto pelo qual
     * é feito a busca.
     * 
     * @return {int} indexFilterObject index Do objeto 
     * encontrado.
     * 
    */
    this.getFilterPosition = function(dataSourceFilter, name){
        var indexFilterObject;
        
        dataSourceFilter.forEach(function(value,i){
            if(value.name === name){
                indexFilterObject = i;
                return;
            }    
        });
        return indexFilterObject;
    };
    
    /**
     * Função para identificar o index de um objeto
     * dentro de um FieldGroups, de acordo com 
     * seu nome.
     * 
     * @param {object} fieldGroup objeto da busca.
     * 
     * @param {String} name propriedade no objeto 
     * pelo qual é feito a busca.
     * 
     * @return {int} indexFieldGroupObject index Do objeto 
     * encontrado.
    */
    this.getFieldGroupPosition = function(fieldGroups, name){
      
        var indexFieldGroupObject;
      
        fieldGroups.forEach(function(value, i){
            if(value.name === name){
                indexFieldGroupObject = i;
            } 
        });
        
        return indexFieldGroupObject;
        
    };
    
    /**
     * Procedimento para recuperar a posição
     * de um objeto dentro de um array.
     * 
     * @param {object} array array onde será feito
     * a varredura.
     * 
     * @param {object} objectName nome do objeto a ser
     * buscado.
    */
    this.getObjectPositionInArrayByName = function(array, objectName){
        
        var indexFieldObject;
      
        array.forEach(function(value, i){
            if(value.name === objectName){
                indexFieldObject = i;
            } 
        });
        
        return indexFieldObject;
        
    };
    
    /**
     * Limpa um field de acordo com seu template.
     * 
     * @param {object} field terá seu valor limpo
    */
    this.clearFieldValue = function(field){
        
        var templateField = field.template;
        
        if(templateField === "field/select.html"){
            
            if(field.value() && field.value() !== ""){
                
                field.clearByOutData();
            }
            
        }else{
            
            field.clearValue();
            
        }
        
    };
    
     /**
     * Função que habilita a criação de um novo código
     * para o registro que será salvo.
     * 
     * @param {object(widget)} widget widget ativa que
     * terá seu parâmetro alterados
     * 
     * @param {String} tableName tabela referência para
     * gerar o novo código
     * 
     * @param {String} tablePk nome da Primary Key da tabela
     * que recebera o código gerado.
    */
    this.enableNewNumericalCode = function(widget, tableName, tablePk){
        
        if(widget.currentRow.__is_new){
            
            var paramsNewCode = widget.paramsNewCode;
            
            paramsNewCode.__is_new_code = true;
            paramsNewCode.__new_code_pk = tableName;
            paramsNewCode.__table_pk = tablePk;
            
        }
        
    };
    
     /**
     * Define os parâmetros para gerar um novo código
     * para o registro a ser salvo.
     * 
     * @param {object} row objeto que será persistido.
     * 
     * @param {String} tableName tabela referência para
     * gerar o novo código
     * 
     * @param {String} tablePk nome da Primary Key da tabela
     * que recebera o código gerado.
     * 
     * @param {string} columnToReturn coluna a ser retornada.
    */
    this.setColumnToReturnParamsOnRow = function(row, tableName, tablePk, columnToReturn){
        
        row.__is_new_code = true;
        row.__new_code_pk = tableName;
        row.__table_pk = tablePk;
        row.__new_code_length = 20;
        row.__columns_to_return = [columnToReturn];
        
    };
    
    /**
     * Função que desabilita a criação de um novo código.
     * 
     * @param {object(widget)} widget widget ativa
     * que terá seus parâmetros desabilitados.
    */
    this.desableNewNumericalCode = function(widget){
    
        var paramsNewCode = widget.paramsNewCode;
        
        paramsNewCode.__is_new_code = false;
        paramsNewCode.__new_code_pk = "";
        paramsNewCode.__table_pk = "";
        
    };
    
    /**
     * Função para setar a coluna a ser retornada
     * na resposta do 'save'.
     * 
     * @param {object} currentRow currentRow que receberá
     * a propriedade com o nome da coluna a ser retornada.
     * 
     * @param {string} columnName nome da coluna a ser
     * retornada.
    */
    this.setColumnToReturnInCurrentRow = function(currentRow, columnName){
    
        currentRow.__columns_to_return = [columnName];

    };
    
    /**
     * Função para remover todos os caracteres
     * não numéricos de um determinado valor,
     * caso o valor recebido seja vazio, uma
     * String vazia é retornada.
     * 
     * @param {String} valueWithCurrencyMask valor que será retornado
     * sem caracteres não numéricos.
     * 
     * @return {String} valueWithoutSymbols valor
     * recebido como parâmetro sem caracteres não
     * numéricos.
     * 
    */
    this.removeValueMask = function(valueWithMask){
        
        if(valueWithMask){
        
            var valueWithoutSymbols = valueWithMask.replace(/[^0-9]/g, "");
            
            return valueWithoutSymbols;
        
        }else{
            
            return "";
            
        }
    };
    
    /**
     * Função para removar mascarás de moedas do valor
     * recebido como parâmetro.
     * 
     * @param {String} valueWithCurrencyMask valor que 
     * será retornado sem máscara de moeda.
     * 
     * @return {String} valueWithoutSymbols valor
     * recebido como parâmetro sem máscara de moeda.
     * 
    */
    this.removeValueCurrencyMask = function(valueWithCurrencyMask){
        
        if(valueWithCurrencyMask){
            if(typeof valueWithCurrencyMask === "string") {
                var valuesWithoutCurrencyMask = $.currency.unmaskString(valueWithCurrencyMask, {"thousands": "."});    
                return valuesWithoutCurrencyMask;
            }else{
                //Retorna o pŕoprio valor se não vier uma string
                return valueWithCurrencyMask;
            }
        }else{
            
            return "";
            
        }
        
            
    };
    
    /**
     * Função para localizar uma widget dentro
     * de um array de widgets pelo seu nome;
     * 
     * @param {object} widgets arrays de widgets, que
     * será percorrido pela busca.
     * 
     * @param {string} widgetName nome da widget a ser
     * procurada.
     * 
     * @return {object} foundWidget widget encontrada
     * no array de widgets recebido como parâmetro.
    */
    this.getWidgetByName = function(widgets, widgetName){
        
        var foundWidget;
        
        widgets.map(function(widget){
            
            if(widget.name === widgetName){
                foundWidget = widget;
            }
            else if(widget.widgets && widget.widgets.length > 0 && !foundWidget){
                
                foundWidget = this.getWidgetByName(widget.widgets, widgetName);
                
            }
            
        }.bind(this));
        
        return foundWidget;
        
    };
    
    /**
     * Função para setar o filtro do número da organização
     * padrão e do número da organização do operador logado
     * na field recebida como parâmetro, e logo em seguida,
     * recarregar a field.
     * 
     * @param {object} field field que receberá o filtro.
    */
    this.setDefaultOrgFilterAndOperatorOrgFilter = function(field){
        
        var fieldFilter = field.dataSourceFilter;
        var loggedOperatorInfos = this.getLoggedOperatorInfos();
        var valueWithDefaultOrgAndJobOrg = [getLocalVar("CRM_NRORGPADRAO"), loggedOperatorInfos.NRORGTRAB];
        
        if(!fieldFilter){
            
            field.dataSourceFilter = [];
            fieldFilter = field.dataSourceFilter;
            
        }
        
        var nrorgFilterIndex = this.getFilterPosition(fieldFilter, "NRORG");
        
        if(nrorgFilterIndex >= 0){
            fieldFilter[nrorgFilterIndex].value = valueWithDefaultOrgAndJobOrg;
        }else{
            fieldFilter.push(
                {
                    "name": "NRORG",
                    "operator": "IN",
                    "value": valueWithDefaultOrgAndJobOrg
                }
            );
        }
        
    };
    
    /**
     * Função para setar o filtro de organização padrão
     * para atribuir o valor do parâmetro 'P_NRORGPADRAO'
     * nas querys do dataSource em caso.
     * 
     * @param {object} filterDestination destino onde o filtro
     * será definido.
    */
    this.setDefaultOrgNumberFilterValue = function(filterDestination){
        
        var filterDestinationFilter = filterDestination.dataSourceFilter;
        
        if(!filterDestinationFilter){
            
            filterDestination.dataSourceFilter = [];
            filterDestinationFilter = filterDestination.dataSourceFilter;
            
        }
        
        filterDestinationFilter.push(
            {
                "name": "P_NRORGPADRAO",
                "operator": "IN",
                "value": [getLocalVar("CRM_NRORGPADRAO")]
            }
        );
        
    };
    
    /**
     * Função para remover um objeto do array(filtro)
     * recebido como parâmetro.
     * 
     * @param {object} filter array com os objetos(filtros).
     * 
     * @param {int} objectIndex índice do objecto a ser removido
     * do array(filtro).
    */
    this.deleteFilterObject = function(filter, objectIndex){
        
        if(objectIndex === 0){
            filter.shift();    
        }else if(objectIndex === filter.length - 1){
            filter.pop();
        }else{
            filter = filter.slice(0,objectIndex).concat(filter.slice(objectIndex + 1));    
        }
        
        return filter;
    };
   
    /**
     * Função para retornar a data e hora atual.
     * 
     * @return {string} data e hora atual.
    */
    this.getDataHoraAtual = function() {
        return moment().format('DD/MM/YYYY HH:mm');            
    };
    
    /**
     * Função para retornar a data atual.
     * 
     * @return {string} data atual.
    */
    this.getCurrentDate = function(){
        return moment().format('DD/MM/YYYY');
    };
    
    /**
     * Função para retornar a hora atual.
     * 
     * @return {string} hora atual.
    */
    this.getCurrentTime = function(){
        return moment().format('HH:ss');
    };
    
    /**
     * Função para retornar a data atual com
     * o horário máximo.
     * 
     * @return {string} data atual.
    */
    this.getCurrentDateWithMaxTime = function(){
        return moment().format('DD/MM/YYYY 23:59');
    };
    
    /**
     * Função para retornar a data atual com
     * o horário mínimo.
     * 
     * @return {string} data atual.
    */
    this.getCurrentDateWithMinTime = function(){
        return moment().format('DD/MM/YYYY 00:00');
    };
    
    /**
     * Procedimento para retornar uma data
     * subtraida pela quantidade de dias recebida
     * como parâmetro.
     * 
     * @param {string} date data s ser subtraida.
     * 
     * @param {number} numberOfDays dias como subtraendo.
     * 
     * @return {string} data subtraida.
    */
    this.subtractDateByDays = function(date,numberOfDays){
        return moment(date, 'DD/MM/YYYY').subtract(numberOfDays, 'day').format('DD/MM/YYYY');
    };
    
    /**
     * Procedimento para retornar uma data
     * subtraida pela quantidade de mêses recebido
     * como parâmetro.
     * 
     * @param {string} date data s ser subtraida.
     * 
     * @param {number} numberOfMonth mêses como subtraendo.
     * 
     * @return {string} data subtraida.
    */
    this.subtractDateByMonth = function(date,numberOfMonth){
        return moment(date, 'DD/MM/YYYY').subtract(numberOfMonth, 'month').format('DD/MM/YYYY');
    };
    
    /**
     * Adiciona um número de dias a uma data.
     * 
     * @param {string} date data se ser modificada.
     * 
     * @param {number} numberOfDays número de dias a ser
     * adicionado.
     * 
     * @param {string} format formato da data recebida.
     * 
     * @return  {string} data com os dias adicionado.
    */
    this.addDaysToDate = function(date, numberOfDays, format){
        return moment(date, format).add(numberOfDays, 'day').format(format);
    };
    
    /**
     * Retornar a data de uma string no formato
     * 'dateTime'.
     * 
     * @param {string} dateTime string no formato
     * 'dateTime'.
    */
    this.getDateInDateTime = function(dateTime){
        return moment(dateTime, 'DD/MM/YYYY HH:ss').format('DD/MM/YYYY');
    };
    
    /**
     * Retornar o horário de uma string no formato
     * 'dateTime'.
     * 
     * @param {string} dateTime string no formato
     * 'dateTime'.
    */
    this.getTimeInDateTime = function(dateTime){
        return moment(dateTime, 'DD/MM/YYYY HH:ss').format('HH:ss');
    };
    
    /**
     * Altera o formato da data de acordo com o
     * formato recebido como parâmetro.
     * 
     * @param {string} date data que será modificada.
     * 
     * @param {string} currentFormat formato atual da
     * data recebida.
     * 
     * @param {string} newFormat formato que será aplicado
     * na data recebida.
     * 
     * @return {string} data recebida formatada de acordo com
     * o novo padrão também recebido.
    */
    this.changeDateFormat = function(date, currentFormat, newFormat){
        return moment(date, currentFormat).format('YYYY-MM-DD');
    };
    
    /**
     * Função para filtrar os valores de um
     * campo de localização pelo seu local de origem
     * (Ex.:Minas Gerais (estado selecionado)
     * Cidades disponíveis: Contagem, Belo Horizonte, Betin).
     * 
     * @param {object} widget widget onde se localiza o campo
     * de localidade.
     * 
     * @param {object} field campo de localidade que receberá
     * o filtro pelo seu local de origem.
    */
    this.setPlaceFilterByPlaceOrigin = function(widget, field){
        
        var simplifiedFieldName = this.getPartOfTheExpressionFromLeftToRight(field.name, 4);
        var currentRow = widget.currentRow;
        var filters = [];
        var filterReference;
        
        switch(simplifiedFieldName){
            
            case "NMES":
                filterReference = "CDPAIS";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.CDPAIS));
            break;
            
            case "NMMU":
                filterReference = "SGESTADO";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.SGESTADO));
                filterReference = "CDPAIS";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.CDPAIS));
            break;
            
            case "NMBA":
                filterReference = "CDMUNICIPIO";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.CDMUNICIPIO));
                filterReference = "SGESTADO";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.SGESTADO));
                filterReference = "CDPAIS";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.CDPAIS));
            break;
            
        }
        
        if(filters){
            filters.forEach(function(filter){
                var placeNameFilterIndex = this.getFilterPosition(field.dataSourceFilter, filter.name);
            
                if(placeNameFilterIndex >= 0){
                    field.dataSourceFilter[placeNameFilterIndex].value = currentRow[filter.name];
                }else{
                    field.dataSourceFilter.push(filter);   
                }
            }.bind(this));
        }
    };
    
    /**
     * Função que recebe uma expressão e retorna
     * a mesma expressão simplicada(o número de 
     * caracteres recebido como parâmetro, começando
     * da esquerda para a direita.)
     * 
     * @param {string} string expressão a ser simplificada.
     * 
     * @para {number} numbersOfChars número de caracteres
     * que será retornado da expressão recebida com parâmetro.
    */
    this.getPartOfTheExpressionFromLeftToRight = function(expression, numbersOfChars){
        
        eval('var regEx = /(\\w{'+numbersOfChars+'})(\\w*)/');
        
        var newExpression = expression.replace(regEx, "$1");
        return newExpression;
    };
    
    /**
     * Função que retorna um 'objeto filtro'
     * com o operador de igualdade.
     * 
     * @param {string} name nome de referência para
     * o filtro.
     * 
     * @param {string/number} value valor buscado pelo
     * filtro.
    */
    this.createFilterWithEqualityOperator = function(name,value){
        
        var filter = {
            "name": name,
            "operator": "=",
            "value": value
        };
            
        return filter;
    };

    /**
     * Valida a inscrição informada pelo usuário
     * e seta na currentRow da widget o código do
     * tipo da inscrição e a máscara da inscrição,
     * caso a inscrição seja inválida
     * ele limpa os campos e manda uma mensagem.
     * 
     * @param {object} widget        : widget que possui a currentRow
     *                                  a ser modificada
     * 
     * @param {string} inscricao     : string com o nome do field 
     *                                  que possui a Inscrição
     * 
     * @param {string} tipoInscricao : string com o nome do field 
     *                                  do tipo de inscrição
    */
    this.validaInscricao = function(widget, inscricao, tipoInscricao){
        
        var currentRow = widget.currentRow;
        
        var resultValidate = General.inscricaoFormat(currentRow[inscricao]);
        
        if(resultValidate.valid && resultValidate.value){
            
            currentRow[tipoInscricao] = resultValidate.tipo.cod;
            
            currentRow[inscricao] = resultValidate.value;
            
        }else{
            
            ScreenService.showMessage('Número de inscrição inválido');
            
            currentRow[inscricao]     = "";
            currentRow[tipoInscricao] = "";
        }    
        
    };
    
    this.validaCPF = function(widget, inscricao){
        
        var currentRow = widget.currentRow;
        
        var resultValidate = General.validateCPF(currentRow[inscricao]);
        
        if(resultValidate){
            resultValidate = General.inscricaoFormat(currentRow[inscricao]);
            currentRow[inscricao] = resultValidate.value;
            
        }else{
            
            ScreenService.showMessage('CPF inválido');
            
            currentRow[inscricao]     = "";
        }    
        
    };
    
    /**
     * Busca as parametrizações do sistema
     * de acordo com o número da organização
     * do operador logado no template Manager.
     * 
     * @return {object} array com as parametrizações
     * da organização do operador logado.
    */
    this.getCrmParametrization = function(){
        
        return templateManager.CRM_PARAMETRIZACAO; 
      
    };
    
    /**
     * Esconde as ações funcionais do container
     * ativo.
    */
    this.hideFunctionalActions = function(){
        
        var containerActions = templateManager.container.actions;
        
        containerActions.forEach(function(action){
                
            if(action.label !== "Voltar"){
                action.isVisible = false;
                action.disable = true;
            }
            
        });    
    
    };
    
    /**
     * Retorna as configurações padrão das ações
     * do container ativo.
     * 
     * @param {object} widget em caso.
    */
    this.returnContainerDefaultActions = function(widget){
        
        var containerActions = templateManager.container.actions;

        containerActions.forEach(function(action){
            
            if(Boolean(action.disable)){
                action.isVisible = true;
                delete action.disable;
            }
            
        });
        
    };
    
    /**
     * Define o filtro(regra) nos fields definidos de acordo
     * com a revenda do operador logado.
     * 
     * @param {object} objects array com os objetos a receberem
     * a regra.
    */
    this.setResaleRulesInRestrictObjectsByLoggedOperatorResale = function(objects) {
        
        objects.forEach(function(field){
            this.setObjectFilterWhenResaleLoggedOperator(field);    
        }.bind(this));
       
    };
    
    /**
     * Defini os filtros de revenda do obejto recebido
     * como parâmetro (field || widget).
     * 
     * @param {object} objeto object a ser modificado.
    */
    this.setObjectFilterWhenResaleLoggedOperator = function(object) {
        
        if(this.isResaleLoggedOperator()) {
            
            this.changeObjectOperTypeFilterToResaleOper(object);
            this.setObjectResaleFilter(object);
        }
        
    };
    
    /**
     * Retorna um valor lógico apontando se o operador
     * logado pertence a uma revenda.
     * 
    */
    this.isResaleLoggedOperator = function(){
        
        var loggedOperatorInfos = this.getLoggedOperatorInfos();
        
        return (Boolean(loggedOperatorInfos.NRREVENDA) && loggedOperatorInfos.IDTPOPERADOR === 'R');
        
    };
    
    /**
     * Altera o filtro de tipo de operador do objeto
     * recebido como parâmetro.
     * 
     * @param {object} object objeto a ser modificado.
    */
    this.changeObjectOperTypeFilterToResaleOper = function(object) {
        
        var objectFilter = object.dataSourceFilter;
        var idtooperadorFilterIndex = this.getObjectPositionInArrayByName(objectFilter, 'IDTPOPERADOR');
        var resaleTypeOperator = 'R';
        
        if(idtooperadorFilterIndex === 0 || Boolean(idtooperadorFilterIndex)){
            objectFilter[idtooperadorFilterIndex].value = resaleTypeOperator;    
        }
      
    };
    
    /**
     * Define um filtro no objeto recebido como parâmetro
     * atribuindo o número da revenda o qual o operador
     * logado pertence.
     * 
     * @param {object} object objeto a ser modificado.
    */
    this.setObjectResaleFilter = function(object) {
        
        var loggedOperatorInfos = this.getLoggedOperatorInfos();    
        
        var resaleFilter = {
            name: "NRREVENDA",
            operator: "=",
            value: loggedOperatorInfos.NRREVENDA
        };
        
        object.dataSourceFilter.push(resaleFilter);
        
    };
    
        
    /*
     * Deixa primeira letra das palavras de uma frase maíusculas
    */
    this.capitalizeFirstLetter = function(string){
        string = string.split(' ');
        string.forEach(function(partOfString, index, string){
            partOfString = partOfString.charAt(0).toUpperCase() + partOfString.toLowerCase().slice(1);    
            string[index] = partOfString;
        });
        return string.join(' ');
    };
    
    /*
     * registra a validação do tipo 'phone'
    */
    this.registerPhoneValidation = function(){
        
        var phoneValidation = function(value, validationAttrs) {
            var retorno;
            if(value === null){
                retorno =  {"valid":true};
            }else{    
                var numberSizeReceived = value.replace(/[^0-9]/g,'').length;
                retorno = {
                    "valid": (numberSizeReceived === 10 || numberSizeReceived === 11 || numberSizeReceived === 12 || numberSizeReceived === 0),
                    "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Invalid phone.")
                };    
            }
            return retorno;
        };
            	      
        ValidationEngine.registerValidation('phone', phoneValidation);
    };
    
    this.registerMinCurrencyValidation = function(){
        
        var minCurrency = function(value, validationAttrs) {
            var limit = parseFloat(validationAttrs.limit);
    		value = parseFloat(value.replace(/[^0-9]/g, '').slice(0, -2));
    		
    		return {
    			"valid": limit <= value || (isNaN(limit) && isNaN(value)),
    			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Minimum value: ") + validationAttrs.limit
    		};
        };
            	      
        ValidationEngine.registerValidation('minCurrency', minCurrency);
    };
    
    /**
     * Inicializa as actions da widget e suas 
     * filhas recursivamente.
     * 
     * @param {object} widget widget de evolução.
     * 
    */    
    this.factoryWidgetActions = function(widget){
        if(!Util.isEmptyOrBlank(widget.widgets)){
            widget.widgets.forEach(function(inWidget){
                this.factoryWidgetActions(inWidget);
            }.bind(this));
        }
        widget.actions.forEach(function(action){
           metaDataFactory.actionFactory(action, widget);
        });
        widget.activate();
    };
    
    /**
     * Seta o datasource filter das tabs dinamicas 
     * baseado num campo da widget principal.
     * 
     * @param {object} widget widget dinamica
     * @param {object} widget widget principal
    */
    this.setDataSourceFilterOnDynamicTabs = function(tabWidget, activeWidget){
        
        var NRSEQCONTATO = activeWidget.currentRow.NRSEQCONTATO;
        tabWidget.dataSourceFilter[0].value = NRSEQCONTATO;
        if(Util.isEmptyOrBlank(tabWidget.currentRow.NRSEQCONTATO))
            tabWidget.currentRow.NRSEQCONTATO = NRSEQCONTATO;
        tabWidget.reload();
        
    };
    
    /**
     * Seta um elemento em uma posição definida dentro de um
     * array.
     * 
     * @param {all} element elemento a ser inserido no array.
     * 
     * @param {object} array array a ser modificado.
     * 
     * @param {int} position posição que o elemento entrará no
     * array.
     * 
     * @return {object} array modificado.
    */
    this.setElementOnArrayDefinedPosition = function(element, array, position){
        
        if(!Boolean(position)){
            array.push(element);
        } else {
            array.splice(position - 1, 0, element);    
        }
        
        return array;
        
    };
    
    /**
     * Verifica se a primeira data recebida como parâmtro
     * é maior que a segunda data, também recebida como
     * parâmetro.
     * Valida um intervalo de datas.
     * 
     * @param {string} initDate data inicial do intervalo.
     * 
     * @param {string} endDate data final do intervalo.
     * 
     * @param {string} format formato das datas recebidas.
     * 
     * @retur {boolean} valor lógico indicando se a primeira
     * data é maior ou não que a segunda.
    */
    this.validateIntervalDates = function(initDate, endDate, format){
        
        if(moment(initDate, format).format(format) > moment(endDate, format).format(format)){
            return false;
        }
        
        return true;
    };
    
    /**
     * Define a propriedade 'readOnly' do campo recebido como
     * parâmetro.
     * 
     * @param {object} field campo a ser modificado.
     * 
     * @param {boolean} isReadOnly valor lógico definindo a 
     * propriedade do campo.
    */
    this.setFieldReadOnlyProperty = function(field, isReadOnly){
        field.readOnly = isReadOnly;  
    };
    
    /**
     * Aplica o método de inserção na widget recebida como parâmetro e
     * seta na currentRow a coluna retornada.
     * 
     * @param {object} widgtet widget que sofrerá a rotina de inserção.
     * 
     * @param {string} columnToReturn coluna a ser retornada na currentRow.
    */
    this.saveAndSetColumnToBeRetunrOnCurrentRow = function(widget, columnToReturn){
        
        var currentRow = widget.currentRow;
        
        this.setColumnToReturnInCurrentRow(currentRow, columnToReturn);
        
        return General.widgetSave(widget).then(function(response){
            currentRow[columnToReturn] = response.dataset.columnsToReturn[0][columnToReturn];    
        });
        
    };
    
    /**
     * Sobreescreve a função relacionada a uma ação
     * da widget recebida como parâmetro.
     * 
     * @param {object} widget widget que terá sua ação
     * modificadas.
     * 
     * @param {string} actionName nome da ação a ser 
     * modificada.
     * 
     * @param {string} functionCall chamada da função a ser colocada
     * na action.
     * EX.: ExemploController.exmploFuncao();
     * Vale lembrar que a chamada da função é atribuida no JSON da
     * widget, sendo assim, os parâmetros devem seguir as regras de
     * chamdas em JSON.
    */
    this.overwriteAction = function(widget, actionName, functionCall){
        
        if(!Boolean(widget.getAction(actionName).events[0].defaultCode)){
            widget.getAction(actionName).events[0].defaultCode = widget.getAction(actionName).events[0].code;
        }
        
        widget.getAction(actionName).events[0].code = functionCall;
        
    };
    
    /**
     * Retorna a função padrão de uma ação (quando a mesma
     * foi sobrescrita).
     * 
     * @param {object} widget widget que terá sua ação 
     * modificada.
     * 
     * @param {string} actionName nome da ação a ser 
     * modificada.
    */
    this.setActionDefaultCode = function(widget, actionName){
        
        widget.getAction(actionName).events[0].code = widget.getAction(actionName).events[0].defaultCode;
        
    };
    /* Função que verifica se o caractere digitado é do tipo NUMBER ou STRING,
    caso seja STRING, substitui o valor para vazio, caso seja numero, passa o valor digitado*/ 
    this.onlyNumbers = function(field) {
		const fieldValue = field.value();
		if (fieldValue) {
			var fieldElements = fieldValue.split("");
			fieldElements.forEach(function (value, index) {
				if (isNaN(parseInt(value))) {
					fieldElements.splice(index,1);
					field.value(fieldElements.join(''));
				}
			});
		}
	};
	
	this.changeVisibilityOrgTrab = function(nrOrgRegistro, widget, actions, fields) {

        var visible = true;
        var nrOrgTrab = getLocalVar("CRM_NRORGTRAB");

        if (nrOrgRegistro == nrOrgTrab || widget.currentRow.__is_new) {
            visible = true;
        }
        else {
            visible = false;
        }

        if (!Util.isUndefinedOrNull(actions)) {
            actions.forEach(function(action) {
                if (widget.getAction(action)) {
                    widget.getAction(action).isVisible = visible;
                }
            });
        }

        if (!Util.isUndefinedOrNull(fields)) {
            fields.forEach(function(field) {
                if (widget.getField(field)) {
                    widget.getField(field).readOnly = !visible;
                }
            });
        }
    };
    
    /**
     * Faz com que o select-native respeite o outData, setando 
     * os capos na currentRow
    */
    this.outDataNativeFields = function(args){
        var field      = args.owner.field;
	    field.dataSource.data.forEach(function(option){
	        if(option[field.valueField] === args.row[field.name]){
	            //Limpa a antiga row
	            Object.keys(field.outData).forEach(function(key){
                    if(args.row[field.outData[key]] !== undefined) delete args.row[field.outData[key]];    	                     
	            });
	            Object.keys(option).forEach(function(key){
	                if(field.outData[key] !== undefined){
	                    args.row[field.outData[key]] = option[key];
	                }
	            });
	        }      
	    });
	};
	
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('CRM_Utilities', CRM_Utilities);
});

// FILE: ./assets/js/util/evolutionTabController.js
function evolutionTabController(ScreenService, templateManager, metaDataFactory, ZHPromise, General){
    
    /**
     * Tranforma o JSON da tab em uma widget de fato.
     * 
     * @param {object} widget widget que irá receber a tab.
     * 
     * @param {string} nrparcnegocio nome do campo com o número
     * do parceiro de negócio.
     * 
     * @return widget evolucao construida.
     * 
    */
    function _factoryEvolutionTab(activeWidget) {
        
        
        var evolutionTabWidgetJson = {
            "id": "1722319758311315390513",
            "container":activeWidget.container,
            "parent":activeWidget,
            "name": "WDGT_EVOLUCAO",
            "label": "Evoluções",
            "isVisible": true,
            "showDescriptionOrder": false,
            "template": "widget/master_detail/grid.html",
            "dataSource": {
                "name": "/evolcont_contato",
                "rest": true,
                "lazyLoad": true
            },
            "fieldGroups": [
                {
                    "name": "FG_AGENDAR_TAREFA",
                    "label": "Próxima Tarefa",
                    "collapsible": true,
                    "opened": true
                }
            ],
            "dataSourceFilter": [
                {
                    "name": "NRSEQCONTATO",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": "@templateManager.LOGGEDOPERATOR.NRORG@"
                }
            ],
            "fields": [
                {
                    "id": "02",
                    "name": "NRSEQCONTATO",
                    "label": "Contato",
                    "template": "field/datetime.html",
                    "showOnForm": false,
                    "showOnList": false,
                    "isVisible": false,
                    "readOnly": false,
                    "class": "6",
                    "defaultValue":"@General.getFieldValue('NRSEQCONTATO', '"+activeWidget.name+"')@",
                    "events": []
                },
                {
                    "id": "03",
                    "name": "NRORG",
                    "label": "Org",
                    "template": "field/datetime.html",
                    "showOnForm": false,
                    "showOnList": false,
                    "isVisible": false,
                    "readOnly": false,
                    "class": "6",
                    "defaultValue":"@getLocalVar('CRM_NRORGTRAB')@",
                    "events": []
                },
                {
                    "id": "04",
                    "name": "DTHREVOLCONT",
                    "label": "Data da Evolução",
                    "template": "field/datetime.html",
                    "showOnForm": true,
                    "showOnList": true,
                    "isVisible": true,
                    "readOnly": false,
                    "defaultValue":"@CRM_Utilities.getDataHoraAtual()@",
                    "class": "6",
                    "events": []
                },
                {
                    "id": "05",
                    "name": "IDSENEVOCONT",
                    "label": "Sentido",
                    "template": "field/select-native.html",
                    "showOnForm": true,
                    "showOnList": true,
                    "class": "6",
                    "valueField": "IDSENEVOCONT",
                    "descriptionField": "NMSENEVOLCONT",
                    "defaultValue": "S",
                    "isVisible": true,
                    "events": [],
                    "validations":{
                        "required":true
                    },
                    "filterProperties":{
                        "validations":{}
                    },
                    "hideEmptyValue":true,
                    "dataSource": {
                        "data": [
                            {
                                "NMSENEVOLCONT": "Entrada",
                                "IDSENEVOCONT":"E"
                            },
                            {
                                "NMSENEVOLCONT": "Saída",
                                "IDSENEVOCONT":"S"
                            }
                        ]
                    }
                },
                {
                    "id": "06",
                    "name": "NMOPERADOR",
                    "label": "Operador Responsável",
                    "template": "field/select.html",
                    "class": "12",
                    "isVisible": true,
                    "showOnList": true,
                    "showOnView": true,
                    "showOnForm": true,
                    "readOnly": false,
                    "dataSource": {
                        "name": "product#/operador",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "defaultValue": "@getLocalVar('CRM_NMOPERADOR')@",
                    "itemsPerPage": 50,
                    "dataSourceFilter": [
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('CRM_NRORGTRAB')@"
                        },
                        {
                            "name": "IDATIVO",
                            "operator": "=",
                            "value": "S"
                        }
                    ],
                    "descriptionField": "NMOPERADOR",
                    "valueField": "CDOPEREVOL",
                    "outData":{
                        "CDOPERADOR" : "CDOPEREVOL",
                        "NMOPERADOR" : "NMOPERADOR"
                    },
                    "showSelectAs": "grid",
                    "fields": [
                        {
                            
                            "name": "NMOPERADOR",
                            "label": "Operador",
                            "showOnList": true
                        }
                    ],
                    "validations": {},
                    "events": []
                },
                {
                    "id": "07",
                    "name": "CDOPEREVOL",
                    "label": "Código Operador",
                    "template": "field/datetime.html",
                    "showOnForm": false,
                    "showOnList": false,
                    "isVisible": false,
                    "readOnly": false,
                    "class": "6",
                    "defaultValue":"@templateManager.LOGGEDOPERATOR.CDOPERADOR@",
                    "events": []
                },
                {
                    "id": "08",
                    "name": "NMTIPOCONT",
                    "label": "Tipo de Contato",
                    "template": "field/select.html",
                    "showSelectAs": "grid",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "class": "6",
                    "readOnly": false,
                    "dataSource": {
                        "name": "product#/crud/tipocont",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "itemsPerPage": 50,
                    "dataSourceFilter": [
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('CRM_NRORGTRAB')@"
                        },
                        {
                            "name": "IDATIVO",
                            "operator": "=",
                            "value": "S"
                        }
                    ],
                    "descriptionField": "NMTIPOCONT",
                    "valueField": "CDTIPOCONT",
                    "fields": [
                        {
                            
                            "name": "NMTIPOCONT",
                            "label": "Tipo de Contato",
                            "showOnList": true
                        }
                    ],
                    "validations":{  
                        "required":true
                    },
                    "events": []
                },
                {
                    "id": "09",
                    "name": "NMMOTIEVOCON",
                    "label": "Motivo da Evolução",
                    "template": "field/select.html",
                    "class": "6",
                    "isVisible": true,
                    "showOnList": true,
                    "showOnView": true,
                    "showOnForm": true,
                    "readOnly": false,
                    "dataSource": {
                        "name": "product#/motievolcont_crm_perfiresult_crm_perfioper",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "itemsPerPage": 50,
                    "dataSourceFilter": [
                        {
                            "name": "P_NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('CRM_NRORGTRAB')@"
                        }
                    ],
                    "outData": {
                        "NMMOTIEVOCON": "NMMOTIEVOCON",
                        "CDMOTIEVOCON": "CDMOTIEVOCON",
                        "IDENVIAEMAIL": "IDENVIAEMAIL",
                        "NMANEXOPADRAO": "NMANEXOPADRAO",
                        "DSCAMINHOANEXOPADRAO": "DSCAMINHOANEXOPADRAO"
                    },
                    "descriptionField": "NMMOTIEVOCON",
                    "valueField": "CDMOTIEVOCON",
                    "showSelectAs": "grid",
                    "fields": [
                        {
                            
                            "name": "NMMOTIEVOCON",
                            "label": "Motivo da Evolução",
                            "showOnList": true
                        }
                    ],
                    "validations":{  
                        "required":true
                    },
                    "events":[
                        {
                            "name":"FieldOnChange",
                            "code":"evolutionTabController.checkMotievolcont(args.owner.field.widget)"
                        }
                    ]
                },
                {
                    "id": "010",
                    "name": "NMINTERLOCUTOR",
                    "label": "Interlocutor",
                    "template": "field/select.html",
                    "showSelectAs": "grid",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "readOnly": false,
                    "class": "6",
                    "valueField": "NRSEQINTERLOCUTOR",
                    "descriptionField": "NMINTERLOCUTOR",
                    "dataSource": {
                        "name": "product#/crm_interlocutor",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "itemsPerPage": 50,
                    "dataSourceFilter": [
                   
                        {
                            "name": "NRPARCNEGOCIO",
                            "operator": "=",
                            "value": "@General.getFieldValue('NRPARCNEGOCIO', '"+activeWidget.name+"')@"
                        },
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('CRM_NRORGTRAB')@"
                        }
                    ],
                    "outData": {
                        "NRSEQINTERLOCUTOR": "NRSEQINTERLOCUTOR",
                        "EMAILINTERLOC": "EMAILINTERLOC",
                        "NRTELEFONE1": "NRTELEFONE1",
                        "NMINTERLOCUTOR": "NMINTERLOCUTOR"
                    },
                    "fields": [
                        {
                            
                            "name": "NMINTERLOCUTOR",
                            "label": "Nome Interlocutor",
                            "isVisible": true,
                            "showOnList": true
                        }
                    ],
                    "events": [
                        {
                            
                            "name": "FieldAfterSelectOpen",
                            "code": "args.owner.field.reload()"
                        }
                    ],
                    "filterProperties": {
                        "isVisible": false,
                        "showOnForm": false
                    }
                },
                {
                    "id": "12932085129618812781121755128",
                    "name": "ESTADOINTERLOCUTOR",
                    "label": "Estado (Interlocutor)",
                    "template": "field/text-edit.html",
                    "showOnForm": false,
                    "showOnView": false,
                    "showOnList": true,
                    "isVisible": true,
                    "events": []
                },
                {
                    "id": "12932085112296181287811127559",
                    "name": "CIDADEINTERLOCUTOR",
                    "label": "Cidade (Interlocutor)",
                    "template": "field/text-edit.html",
                    "showOnForm": false,
                    "showOnView": false,
                    "showOnList": true,
                    "isVisible": true,
                    "events": []
                },
                {
                    "id": "011",
                    "name": "NMSEQRESULEVOL",
                    "label": "Resultado",
                    "template": "field/select.html",
                    "showSelectAs": "grid",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "class": "6",
                    "dataSource": {
                        "name": "product#/crm_resultevol_crm_perfiresult_crm_perfioper",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "itemsPerPage": 50,
                    "dataSourceFilter": [
                        {  
                            "name":"P_NRORG",
                            "operator":"=",
                            "value":"@getLocalVar('CRM_NRORGTRAB')@"
                        },
                        {  
                            "name":"IDRESULTEVOLUCAO",
                            "operator":"=",
                            "value":"SIM"
                        }
                    ],
                    "descriptionField": "DESCRESULEVOL",
                    "valueField": "NRSEQRESULEVOL",
                    "fields": [
                        {
                            
                            "name": "DESCRESULEVOL",
                            "label": "Resultado da Evolução",
                            "showOnList": true
                        }
                    ],
                    "outData": {
                        "IDGERPROXATIV": "NEWTASK",
                        "DESCRESULEVOL": "NMSEQRESULEVOL",
                        "NRSEQRESULEVOL":"NRSEQRESULEVOL"
                    },
                    "validations":{  
                        "required":true
                    },
                    "events": [
                        {
                            
                            "name": "FieldOnChange",
                            "code": "EvolucaoController.checkNewTask(args.owner.field.widget)"
                        }
                    ]
                },
                {
                    "id": "012",
                    "name": "TXEVOLCONT",
                    "label": "Descrição",
                    "template": "field/textarea.html",
                    "isVisible": true,
                    "showOnForm": true,
                    "class": "12",
                    "validations":{  
                        "required":true
                    },
                    "textRows": 5
                },
                {
                    "id": "013",
                    "name": "DTPLANFILACONTA",
                    "label": "Data da Próxima Tarefa",
                    "template": "field/datetime.html",
                    "showOnForm": true,
                    "showOnList": false,
                    "isVisible": true,
                    "readOnly": true,
                    "class": "6",
                    "fieldGroup": "FG_AGENDAR_TAREFA",
                    "validations": {},
                    "events": []
                },
                {
                    "id": "014",
                    "name": "NMOPERADORREAL",
                    "label": "Operador",
                    "template": "field/select.html",
                    "showSelectAs": "grid",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnView": true,
                    "showOnList": false,
                    "readOnly": true,
                    "fieldGroup": "FG_AGENDAR_TAREFA",
                    "class": "6",
                    "dataSource": {
                        "name": "product#/operador",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "defaultValue": "@getLocalVar('CRM_NMOPERADOR')@",
                    "itemsPerPage": 50,
                    "dataSourceFilter": [
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('CRM_NRORGTRAB')@"
                        },
                        {
                            "name": "IDATIVO",
                            "operator": "=",
                            "value": "S"
                        }
                    ],
                    "descriptionField": "NMOPERADORREAL",
                    "valueField": "CDOPERREAL",
                    "fields": [
                        {
                            
                            "name": "NMOPERADOR",
                            "label": "Operador",
                            "showOnList": true
                        }
                    ],
                    "outData": {
                        "NMOPERADOR": "NMOPERADORREAL",
                        "CDOPERADOR": "CDOPERREAL"
                    },
                    "validations": {},
                    "events": []
                },
                {
                    "id": "015",
                    "name": "CDOPERREAL",
                    "label": "Código Operador",
                    "template": "field/datetime.html",
                    "showOnForm": false,
                    "showOnList": false,
                    "isVisible": false,
                    "readOnly": false,
                    "class": "6",
                    "defaultValue":"@templateManager.LOGGEDOPERATOR.CDOPERADOR@",
                    "events": []
                },
                {
                    "id": "016",
                    "name": "DSFILACONTATO",
                    "label": "Assunto da Tarefa",
                    "template": "field/text-edit.html",
                    "showOnForm": true,
                    "fieldGroup": "FG_AGENDAR_TAREFA",
                    "readOnly": true,
                    "showOnList": true,
                    "isVisible": true,
                    "defaultValue": "RETORNAR CONTATO",
                    "class": "12",
                    "events": []
                }
            ],
            "actions": [
                {
                    "id": "017",
                    "name": "BACK_VIEW",
                    "label": "Voltar",
                    "isVisible": true,
                    "preventHide":true,
                    "showAsAction": "back_view",
                    "events": [
                        {
                            
                            "name": "ActionEvent",
                            "code": "General.widgetGoBackToList(args.owner.widget);"
                        }
                    ]
                },
                {
                    "id": "018",
                    "name": "BACK_LIST",
                    "label": "Voltar",
                    "isVisible": true,
                    "preventHide":true,
                    "showAsAction": "back_list",
                    "events": [
                        {
                            
                            "name": "ActionEvent",
                            "code": "General.widgetGoBackToParent(args.owner.widget);"
                        }
                    ]
                },
                {
                    "id": "019",
                    "name": "BACK_EDIT",
                    "label": "Cancelar",
                    "isVisible": true,
                    "showAsAction": "back_edit",
                    "events": [
                        {
                            
                            "name": "ActionEvent",
                            "code": "General.widgetCancelForm(args.owner.widget)"
                        }
                    ]
                },
                {
                    "id": "020",
                    "name": "EDIT",
                    "label": "Editar",
                    "icon": "pencil",
                    "isVisible": true,
                    "showAsAction": "view",
                    "events": [
                        {
                            
                            "name": "ActionEvent",
                            "code": "General.widgetEdit(args.owner.widget)"
                        }
                    ]
                },
                {
                    "id": "021",
                    "name": "ADD",
                    "label": "Adicionar",
                    "icon": "plus",
                    "showAsAction": "always|list",
                    "events": [
                        {
                            
                            "name": "ActionEvent",
                            "code": "General.widgetNewRow(args.owner.widget)"
                        }
                    ]
                },
                {
                    "id": "022",
                    "name": "SAVE",
                    "label": "Salvar",
                    "showAsAction": "never_edit",
                    "events": [
                        {
                            
                            "name": "ActionEvent",
                            "code": "EvolucaoController.checkEvolution(args.owner.widget)"
                        }
                    ]
                },
                {
                    "id": "023",
                    "name": "DELETE",
                    "label": "Excluir",
                    "icon": "trash",
                    "isVisible": false,
                    "preventHide": true,
                    "showAsAction": "view",
                    "events": [
                        {
                            "id": "024",
                            "name": "ActionEvent",
                            "code": "General.deleteCrud(args.owner.widget);"
                        }
                    ]
                }
            ],
            "events": [
                {
                    "id": "025",
                    "name": "WidgetOnEnterOnce",
                    "code": "CRM_Utilities.factoryWidgetActions(args.owner)"
                },
                {
                    "id": "026",
                    "name": "WidgetOnEnter",
                    "code": "CRM_Utilities.setDataSourceFilterOnDynamicTabs(args.owner, templateManager.container.getWidget('" + activeWidget.name + "'))"
                }
            ],
            "widgets": [
                {
                    "id": "1722319758311315390514",
                    "container":activeWidget.container,
                    "name": "WDGT_ANEXOS_EVOLUCAO",
                    "label": "Anexos",
                    "isVisible": true,
                    "showDescriptionOrder": false,
                    "template": "widget/master_detail/grid.html",
                    "dataSource": {
                        "name": "/crm_anexocontato",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "dataSourceFilter": [
                        {
                            "name": "NRSEQCONTATO",
                            "operator": "=",
                            "value": "@General.getFieldValue('NRSEQCONTATO', 'WDGT_EVOLUCAO')@"
                        },
                        {
                            "name": "NRORIGEM",
                            "operator": "=",
                            "value": "@General.getFieldValue('NRSEQEVOLCONT', 'WDGT_EVOLUCAO')@"
                        }
                    ],
                    "events": [
                        {
                            "id": "17223197583113153905171",
                            "name": "WidgetOnEnter",
                            "code": "args.owner.reload()"
                        }
                    ],
                    "actions": [
                        {
                            "id": "1722319758311315390516",
                            "name": "DOWNLOAD",
                            "label": "Download",
                            "isVisible": true,
                            "showAsAction": "never_view",
                            "events": [
                                {
                                    "id": "030",
                                    "name": "ActionEvent",
                                    "code": "EvolucaoController.downloadFile(args.owner.widget);"
                                }
                            ]
                        },
                        {
                            "id": "1722319758311315390517",
                            "name": "ADD",
                            "label": "Adicionar",
                            "icon": "plus",
                            "showAsAction": "always|list",
                            "events": [
                                {
                                    "id": "032",
                                    "name": "ActionEvent",
                                    "code": "EvolucaoController.openAttachSwipe();"
                                }
                            ]
                        },
                        {
                            "id": "1722319758311315390518",
                            "name": "BACK_LIST",
                            "label": "Voltar",
                            "isVisible": true,
                            "preventHide":true,
                            "showAsAction": "back_list",
                            "events": [
                                {
                                    "id": "035",
                                    "name": "ActionEvent",
                                    "code": "General.widgetGoBackToParent(args.owner.widget);"
                                }
                            ]
                        },
                        {
                            "id": "172231975831131539051911",
                            "name": "BACK_VIEW",
                            "label": "Voltar",
                            "isVisible": true,
                            "preventHide":true,
                            "showAsAction": "back_view",
                            "events": [
                                {
                                    "id": "037",
                                    "name": "ActionEvent",
                                    "code": "General.widgetGoBackToList(args.owner.widget);"
                                }
                            ]
                        },
                        {
                            "id": "1722319758311315390591",
                            "name": "DELETE",
                            "label": "Excluir",
                            "icon": "trash",
                            "showAsAction": "view",
                            "events": [
                                {
                                    "id": "039",
                                    "name": "ActionEvent",
                                    "code": "General.deleteCrud(args.owner.widget);"
                                }
                            ]
                        }
                    ],
                    "fields": [
                        {
                            "id": "1722319758311315390592",
                            "name": "NRORIGEM",
                            "label": "Código",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "showOnList": false,
                            "isVisible": false,
                            "readOnly": false
                        },
                        {
                            "id": "1722319758311315390593",
                            "name": "NMARQUIVO",
                            "label": "Nome do arquivo",
                            "class": "12",
                            "isVisible": true,
                            "showOnList": true,
                            "showOnView": true,
                            "readOnly": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "1722319758311315390594",   
                            "name": "DSTIPO",
                            "label": "Extensão",
                            "class": "6",
                            "isVisible": true,
                            "showOnList": true,
                            "showOnView": true,
                            "readOnly": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "1722319758311315390595",
                            "name": "VRTAMANHO",
                            "label": "Tamanho do arquivo(bytes)",
                            "class": "6",
                            "isVisible": true,
                            "showOnList": true,
                            "showOnView": true,
                            "readOnly": false,
                            "template": "field/text-edit.html"
                        }
                    ],
                    "widgets": [
                        {
                            "id": "1722319758311315390515",
                            "container":activeWidget.container,
                            "name": "WDGT_SELECAO_ARQUIVOS",
                            "label": "Anexar",
                            "isVisible": false,
                            "template": "widget/form.html",
                            "dataSource": {},
                            "actions": [
                                {
                                    "id": "1722319758311315390519112",
                                    "name": "uploadFile",
                                    "isVisible": true,
                                    "label": "Anexar",
                                    "showAsAction": "never",
                                    "events": [
                                        {
                                            "id": "046",
                                            "name": "ActionEvent",
                                            "code": "EvolucaoController.uploadEvolutionFiles(args.owner.widget);"
                                        }
                                    ]
                                },
                                {
                                    "id": "1722319758311315390519113",
                                    "name": "BACK",
                                    "label": "Cancelar",
                                    "isVisible": true,
                                    "showAsAction": "back",
                                    "events": [
                                        {
                                            "id": "1722319758311315390519114",
                                            "name": "ActionEvent",
                                            "code": "EvolucaoController.closeAttachSwipe(args.owner.widget);"
                                        }
                                    ]
                                }
                            ],
                            "events": [
                                {
                                    "id": "1722319758311315390597",
                                    "name": "WidgetOnEnter",
                                    "code": "args.owner.newRow();"
                                }
                            ],
                            "fields": [
                                {
                                    "id": "1722319758311315390598",
                                    "name": "DSCAMINHO",
                                    "placeholder": "Anexar arquivo",
                                    "class": "10 col-xs-offset-1",
                                    "isVisible": true,
                                    "readOnly": false,
                                    "descriptionOrder": 1,
                                    "validations": {
                                        "required": true
                                    },
                                    "template": "field/file-upload.html",
                                    "isMultiple": true
                                }
                            ]
                        }
                    ]
                }
            ]
        };
        
        var evolutionTabWidget = metaDataFactory.widgetFactory(evolutionTabWidgetJson);
        
        return evolutionTabWidget;
        
    }
    
    
    
    /**
     * Insere a widget de evoluções na widget recebida como 
     * parâmetro.
     * 
     * @param {object} widget widget que receberá a tab.
     * 
     * @param {string} nrparcnegocio nome referente ao
     * campo de parceiro de negócio.
    */
    this.setEvolutionTabOnActiveWidget = function(widget){
        
        widget.widgets.push(_factoryEvolutionTab(widget));
        
    };
    
    /**
     * Sobreescreve qualquer action da tab.
     * 
     * @param {string} actionName nome da action que será sobrescrita.
     *   BACK_VIEW -> Voltar para o grid da tab.
     *   BACK_LIST -> Voltar para o grid principal.
     *   BACK_EDIT -> Cancelar Edição.
     *   EDIT      -> Editar.
     *   ADD       -> Nova Evolução.
     *   SAVE      -> Salvar.
     *   DELETE    -> Excluir.
     * 
     * @param {string} code nome do controller + nome do metodo a ser executado.
     * Ex.:XYController.xyMethod()
    */
    this.overwriteAction = function(actionName, code){
        var evolutionWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        if(!evolutionWidget.getAction(actionName).events[0].defaultCode)
            evolutionWidget.getAction(actionName).events[0].defaultCode = evolutionWidget.getAction(actionName).events[0].code;
        templateManager.container.getWidget('WDGT_EVOLUCAO').getAction(actionName).events[0].code = code;    
    };
    
    /**
     * Volta para o evento defalut de qualquer action.
     * 
     * @param {string} actionName nome da action que será sobrescrita.
     *   BACK_VIEW -> Voltar para o grid da tab.
     *   BACK_LIST -> Voltar para o grid principal.
     *   BACK_EDIT -> Cancelar Edição.
     *   EDIT      -> Editar.
     *   ADD       -> Nova Evolução.
     *   SAVE      -> Salvar.
     *   DELETE    -> Excluir.
     * 
    */
    this.defaultAction = function(actionName){
        var evolutionWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        if(evolutionWidget.getAction(actionName).events[0].defaultCode)
            evolutionWidget.getAction(actionName).events[0].code = evolutionWidget.getAction(actionName).events[0].defaultCode;
    };
    
    /**
     * Executa comandos após toda a rotina save da evolução.
     * 
     * @param {array} arrayOfMethods array de strings, cada posição 
     * um comando a ser executado.
     * Ex.:[
     *      "ScreenService.closeSwipe(true)",
     *      "XYController.xyMethod()"
     *    ]
    */
    this.setMethodsAfterSave = function(arrayOfMethods){
        var evolutionWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        if(evolutionWidget.afterSave)
            evolutionWidget.afterSave.push(arrayOfMethods);
        else
            evolutionWidget.afterSave = arrayOfMethods;
    };
    
    
    /**
     * Builda novos fieldGroups na tab e chama o metodo de inserção de fields.
     * 
     * @param {array}  arrayOfFields array de fields(json).
     * @param {string} fieldGroupName nome do fieldGroup a ser inserido.
     * 
    */
    this.buildDynamicFieldGroup = function(arrayOfFields, fieldGroupName, fieldGroupLabel){
        var evolutionWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        
        var fieldGroup = {
            "name":fieldGroupName,
            "label":fieldGroupLabel,
            "colapsible":true,
            "opened":true,
            "isVisible":true,
            "dynamic":true
        };
        if(this.checkFieldGroup(fieldGroup, evolutionWidget.fieldGroups) === -1){
            if(evolutionWidget.fieldGroups)
                evolutionWidget.fieldGroups.push(fieldGroup);
            else
                evolutionWidget.fieldGroups = [fieldGroup];
            
            evolutionWidget.fieldGroups = evolutionWidget.fieldGroups.map(function(item) {
    			item.widget = evolutionWidget;
    			return Util.extend(new FieldGroup(), item);
    		});
    		//Chama o metodo de inserção
            _insertFields(arrayOfFields, evolutionWidget);
        }
    };
    
    this.checkFieldGroup = function(fieldGroup, fieldGroups){
        var indexField = -1;
        fieldGroups.forEach(function(item, index){
            if(item.name === fieldGroup.name)
                indexField = index;
        });    
        return indexField;
    }
    
    /**
     * Builda os novos Fields na widget
     * 
     * @param {array}  arrayOfFields array de fields(json).
     * @param {string} fieldGroupName nome do fieldGroup a ser inserido.
     * 
    */
    function _insertFields(arrayOfFields, widget){
        arrayOfFields.forEach(function(field){
            field.dynamic = true;
            metaDataFactory.fieldFactory(field, widget, true);  
        });
    }
    
    /**
     * Esconde determinados fieldGroups
     * 
     * @param {string/true} fieldGroupName nome do field
     * group a ser escondido, se = true, esconde todos.
     * 
    */
    this.hideFieldGroup = function(fieldGroupName){
        var evolutionWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        
        if(typeof fieldGroupName === "string"){
            evolutionWidget.fieldGroups.forEach(function(fg){
                if(fg.name === fieldGroupName && fg.dynamic){
                    fg.isVisible = false;
                }
            });    
        }else if(fieldGroupName && typeof fieldGroupName === "boolean"){
            evolutionWidget.fieldGroups.forEach(function(fg){
                if(fg.dynamic)
                    fg.isVisible = false;
            });   
        }
        
        evolutionWidget.fields.forEach(function(field){
            if(field.fieldGroup && (field.fieldGroup === fieldGroupName || fieldGroupName === true)){
                if(field.dynamic)
                    field.isVisible = false;
            }
        });
    };
    
    /**
     * Mostra determinados fieldGroups
     * 
     * @param {string/true} fieldGroupName nome do field
     * group a ser mostrado, se = true, mostra todos.
     * 
    */
    this.showFieldGroup = function(fieldGroupName){
        var evolutionWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        
        if(typeof fieldGroupName === "string"){
            evolutionWidget.fieldGroups.forEach(function(fg){
                if(fg.name === fieldGroupName && fg.dynamic)
                    fg.isVisible = true;
            });    
        }else if(fieldGroupName && typeof fieldGroupName === "boolean"){
            evolutionWidget.fieldGroups.forEach(function(fg){
                if(fg.dynamic)
                    fg.isVisible = true;
            });   
        }
        
        evolutionWidget.fields.forEach(function(field){
            if(field.fieldGroup && (field.fieldGroup === fieldGroupName || fieldGroupName === true)){
                if(field.dynamic)
                    field.isVisible = true;
            }
        });
    };
    
    /**
     * Verifica se o motivo selecionado permite enviar email
     * para o interlocutor
     * 
     * @param {object} evolWidget  Widget de Evolução
     * 
    */
    this.checkMotievolcont = function(evolWidget){
        widgetMail.fields.forEach(function(field, index){
            if(field.name === 'DSCAMINHO')
                widgetMail.fields.splice(index,1);
        });
        if(evolWidget.currentRow.IDENVIAEMAIL === 'S'){
            //Salva o Anexo default no popup de email
            var file = {
                name: evolWidget.currentRow.NMANEXOPADRAO || '',
                path: evolWidget.currentRow.DSCAMINHOANEXOPADRAO || ''
            };
            this.insertAttachField(file);    
        }
    };
    
    this.insertAttachField = function(file){
        var field = {
            "name": "DSCAMINHO",
            "placeholder": "Anexar arquivo",
            "class": "12",
            "isVisible": true,
            "readOnly": false,
            "validations": {},
            "template": "field/file-upload.html",
            "isMultiple": true,
            "file":file
        };
        widgetMail.fields.push(field);     
    };
    
    var widgetMail = 
        {
            "name": "mensagens",
            "label": "Notificação de Interlocutor",
            "isVisible": false,
            "showAsTab": false,
            "template": "widget/form.html",
            "events": [
                {
                    "id": "6182828512039293853658",
                    "name": "WidgetOnEnter",
                    "code": "args.owner.reload();"
                }
            ],
            "actions": [
                {
                    "id": "618282851568652067654",
                    "name": "enviar",
                    "isVisible": true,
                    "label": "Enviar",
                    "showAsAction": "never",
                    "events": [
                        {
                            "id": "6182828514145614549655",
                            "name": "ActionEvent",
                            "code": "evolutionTabController.notificarInterlocutor(args.owner.widget, args.owner.widget.backuprow)"
                        }
                    ]
                },
                {
                    "id": "6182828511372890396656",
                    "name": "cancel",
                    "label": "Voltar",
                    "showAsAction": "back",
                    "events": [
                        {
                            "id": "6182828512205401251657",
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup(args.owner.widget);evolutionTabController.removeMessagesWidget(args.owner.widget.parent);"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "id": "6182828513466738669652",
                    "name": "DSTITULOMENSAGEM",
                    "label": "Assunto da Mensagem",
                    "isVisible": true,
                    "class": 12,
                    "descriptionOrder": 1,
                    "events": [],
                    "showOnList": true,
                    "showOnForm": true,
                    "showOnView": true,
                    "template": "field/text-edit.html",
                    "validations": {
                        "required": {},
                        "maxLength": {
                            "length": 100
                        }
                    }
                },
                {
                    "id": "6182828511660973795653",
                    "name": "DSMENSAGEM",
                    "label": "Mensagem",
                    "class": 12,
                    "placeholder": null,
                    "isVisible": true,
                    "showOnList": false,
                    "showOnForm": true,
                    "showOnView": true,
                    "template": "zh-rich-text#field/rich-text.html",
                    "height": 150,
                    "toolbar": {
                        "fontname": true
                    },
                    "validations": {
                        "required": {}
                    },
                    "events": [],
                    "filterProperty": {
                        "isVisible": false,
                        "showOnForm": false
                    }
                }
            ]
        };

    this.removeMessagesWidget = function(widget){
        widget.widgets.forEach(function(wid, index){
            if(wid.name === "mensagens")
                widget.widgets.splice(index, 1);
        });      
        widget.activate();
    };
    
    this.openWidgetSendMail = function(widget){
        var promise = ZHPromise.defer();
        if(widget.currentRow.EMAILINTERLOC){
            
            widgetMail.container = widget.container;
            widgetMail = metaDataFactory.widgetFactory(widgetMail);
            widgetMail.parent = widget;
            widget.widgets.push(widgetMail);
            
            widgetMail.currentRow.DSMENSAGEM = '<table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee" style="background-color: #eeeeee;"> <tbody> <tr> <td> <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="0274BA" class="container590" style="background-color: #0274ba; text-align: center;"> <tbody> <tr> <td> <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580"> <tbody> <tr> <td align="center" valign="middle"> <table border="0" align="middle" cellpadding="0" cellspacing="0" style="border-collapse:collapse; mso-table-lspace:0pt; mso-table-rspace:0pt;" class=""> <tbody> <tr> <td align="center" valign="middle"> <p style="color: #ffffff; font-size: 25px; padding:20px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 25px; padding-top:45px;">Notificação de Evolução</p> <p style="color: #ffffff; font-size: 16px; font-family: "Questrial", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"> [DATAEVOLUCAO] </p> <p style="color: #ffffff; font-size: 16px; font-family: "Questrial", Helvetica, sans-serif; line-height: 15px; padding-top:15px;">&nbsp;</p> <p style="color: #ffffff; font-size: 16px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"> <br> </p> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> <table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee"> <tbody> <tr> <td align="center"> <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="ffffff" class="container590 bg_color"> <tbody> <tr> <td> <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580"> <tbody> <tr> <td align="left" style="color: #39434b; font-size: 20px; font-family: " Montserrat ", Helvetica, sans-serif; font-weight: 400; padding-bottom:15px; padding-top:20px; line-height: 35px;" class="title_color"> <div style="padding:20px; line-height: 22px"> Prezado(a) [INTERLOCUTOR]</div> </td> </tr> <tr> <td align="left" class="title_color" style="color: #848999; font-size: 14px; font-family: " Questrial ", sans-serif; mso-line-height-rule: exactly; line-height: 24px; padding-bottom:15px;"> <div style="line-height: 24px;"> <br> </div> </td> </tr> <tr> <td height="10" style="font-size: 10px; line-height: 10px;">&nbsp;</td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> ';
            //Seta o anexo default no field de upload
            widgetMail.currentRow.DSCAMINHO  = [widgetMail.getField('DSCAMINHO').file];
            setTimeout(function(){
                widgetMail.getField('DSCAMINHO').fileList = widgetMail.currentRow.DSCAMINHO;
            },100);
            widgetMail.currentRow.DATAEVOLUCAO = widget.currentRow.DTHREVOLCONT;
            widgetMail.backuprow = angular.copy(widget.currentRow);
            ScreenService.openPopup(widgetMail).then(function(){
                promise.resolve();
                widgetMail.activate();
            });
        }else{
            ScreenService.showMessage('Interlocutor não possui email cadastrado!').then(function(){
               promise.resolve(); 
            });
        }
        return promise.promise;
    };
    
    this.notificarInterlocutor = function(widget, row){
        var evolWidget = templateManager.container.getWidget('evolucao');
        if(!evolWidget){
            evolWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
            evolWidget.currentRow = row;    
        }
        
        var messagesWidget = templateManager.container.getWidget('mensagens');
        messagesWidget.fields.forEach(function(field, index){
            if(field.name === 'DSCAMINHO')
                messagesWidget.fields.splice(index,1);
        }); 
        
        var files = messagesWidget.currentRow.DSCAMINHO;
        var rows = [];
        
        files.forEach(function(file, fileIndex){
            rows.push(this.setValuesForFileUpload({DSCAMINHO:widget.currentRow.DSCAMINHO}, fileIndex, evolWidget.currentRow));
            //Se for o anexo default
            if(!Util.isEmptyOrBlank(file.path)){
                rows[0].DEFAULTATTACH = true;
            }
            
        }.bind(this));
        
        General.callRestEngine("product#/saveAttachEvol", "DataSet", rows).then(function() {
            
                ScreenService.closePopup(true);
                widget.currentRow.NMINTERLOCUTOR  = evolWidget.currentRow.NMINTERLOCUTOR;
                widget.currentRow.DSEMAILINTERLOC = evolWidget.currentRow.EMAILINTERLOC;
                widget.currentRow.NRORG           = templateManager.LOGGEDOPERATOR.NRORG;
                widget.currentRow.NMOPERADORRESP  = evolWidget.currentRow.NMOPERADOR;
                widget.currentRow.NRSEQEVOLCONT   = evolWidget.currentRow.NRSEQEVOLCONT;
                General.callRestEngine("product#/notificaInterlocutorEvolucao", "Row", widget.currentRow);
                this.removeMessagesWidget(evolWidget);
                evolWidget.activate();
                
        }.bind(this));
        if(templateManager.promise)
            templateManager.promise.resolve();
    };
    
    /**
     * Função para retornar uma row configurada
     * com as informações do anexo referente
     * ao ID, recebido como parâmetro, no array
     * de anexos na currentRow.
     * 
     * @param {object} widget widget onde foi selecionado
     * os anexos.Array
     * 
     * @param {int} fileIndex index do anexo no array de 
     * anexos presente na currentRow.
     * 
     * @return {object} currentRow row configurada com as informações
     * do anexo referente ao ID recebido.
    */
    this.setValuesForFileUpload = function(row, fileIndex){
        
        var currentRow = angular.copy(row);
        var evolutionWidget = templateManager.container.getWidget('evolucao');
        if(!evolutionWidget){
            evolutionWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        }
        var loggedOperatorInfos = templateManager.LOGGEDOPERATOR;
        
        currentRow.NRSEQCONTATO = evolutionWidget.currentRow.NRSEQCONTATO;
        currentRow.NMCOMPLETO = currentRow.DSCAMINHO[fileIndex].name;
        currentRow.NMARQUIVO = this.getFileNameWithoutExtension(currentRow.DSCAMINHO[fileIndex].name);
        currentRow.DSTIPO = this.getFileExtension(currentRow.DSCAMINHO[fileIndex].name);
        currentRow.VRTAMANHO = currentRow.DSCAMINHO[fileIndex].size;
        currentRow.IDATIVO = "S";
        currentRow.NRORG = loggedOperatorInfos.NRORG;
        currentRow.CDOPERADOR = loggedOperatorInfos.CDOPERADOR;
        currentRow.IDORIGEM = "EVOLUCAO";
        currentRow.NRORIGEM = evolutionWidget.currentRow.NRSEQEVOLCONT;
        
        return currentRow;
        
    };
    
    /**
     * Função para retornar a extenção de um arquivo
     * extraida do seu nome.
     * 
     * @param {string} fileName nome completo do arquivo.
    */
    this.getFileExtension = function(fileName){
        
        return fileName.replace(/^(.*)\.(.*)$/, "$2");
        
    };
    
    /**
     * Função para retornar o nome do arquivo sem
     * sua extenção.
     * 
     * @param {string} fileName nome completo do arquivo.
    */
    this.getFileNameWithoutExtension = function(fileName){
        
        return fileName.replace(/^(.*)\.(.*)$/, "$1");
            
    };
	
}
Configuration(function(ContextRegister){
    ContextRegister.register('evolutionTabController', evolutionTabController);
});

// FILE: ./assets/js/util/PartnerInfosModalController.js
function PartnerInfosModalController(CRM_Utilities, ScreenService, templateManager, metaDataFactory){
    
    /**
     * Builda o modal.
     * 
     * @param {object} widget widget que irá receber o modal.
     * 
     * @param {string} nrparcnegocio nome do campo com o número
     * do parceiro de negócio.
     * 
     * @return widget modal construida.
     * 
    */
    this.factoryModalWidget = function(activeWidget, nrparcnegocio) {
        
        /*Modificado em 22/09 por Arthur
            Adicionada propriedade 'preventHide',
            que garante que a widget Dados do Parceiro 
            não tenha suas actions setadas como invisíveis em chamados já 
            concluidos
        */
        var modalWidgetJson = {
            "id":"17223197583113",
            "name": "WDGT_MODAL_PARTNER_INFOS",
            "label": "Informações do Parceiro",
            "container": activeWidget.container,
            "parent": activeWidget,
            "template": "widget/tabbedWidget.html",
            "defaultTab": "WDGT_PARTNER_DATA",
            "showAsModal": true,
            "preventHide":true,
            "showAsTab":false,
            "isVisible": true,
            "events":[
                {
                    "id":"17223197583113124",
                    "name":"WidgetOnEnterOnce",
                    "code":"AbaInterlocutorController.setInterlocutorWidgetOnActiveWidget(args.owner);ABALogotipoParceiroController.setLogoWidgetOnActiveWidget(args.owner, 5);PartnerInfosModalController.changeTemplateTabInterlocutor(args.owner);"
                }
            ],
            "widgets":[
                {
                    "id":"17223197583113123",
                    "name": "WDGT_PARTNER_DATA",
                    "label": "Dados",
                    "container": activeWidget.container,
                    "template": "widget/form-view.html",
                    "isVisible": true,
                    "dataSource": {
                        "name": "product#/crm_candidato_prospect_parcnegocio",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "dataSourceFilter": [
                        {
                            "name": "NRPARCNEGOCIO",
                            "operator": "=",
                            "value": ""
                        }
                    ],
                    "actions": [],
                    "events": [
                        {
                            "id":"17223197583113125",
                            "name": "WidgetOnEnter",
                            "code": "PartnerInfosModalController.setModalFilterByActivePartner(args.owner.parent, templateManager.container.getWidget('" + activeWidget.name + "'), '" + nrparcnegocio + "');"
                        },
                        {
                            "id":"17223197583113126",
                            "name": "WidgetAfterLoadDataSource",
                            "code": "args.owner.moveToFirst();"
                        }
                    ],
                    "fields": [
                        {
                            "id":"17223197583113127",
                            "name": "NOMEPRINCIPAL",
                            "label": "Nome",
                            "template": "field/text-edit.html",
                            "class": "12",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113128",
                            "name": "NOMEFANTASIA",
                            "label": "Nome Fantasia",
                            "template": "field/text-edit.html",
                            "class": "12",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113129",
                            "name": "INSCRICAO",
                            "label": "Inscrição",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113130",
                            "name": "TIPOINSC",
                            "label": "Tipo Inscrição",
                            "template": "field/select-native.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true,
                            "valueField": "value",
                            "descriptionField": "name",
                            "validations": {},
                            "events": [],
                            "dataSource": {
                                "data": [
                                    {
                                        "name": "Física",
                                        "value": "F"
                                    },
                                    {
                                        "name": "Jurídica",
                                        "value": "J"
                                    },
                                    {
                                        "name": "Outros",
                                        "value": "O"
                                    },
                                    {
                                        "name": "Inscrição",
                                        "value": "I"
                                    }
                                ]
                            }
                        },
                        {
                            "id":"17223197583113131",
                            "name": "NMOPERRESPONSAVEL",
                            "label": "Operador Responsável",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113132",
                            "name": "TIPOPARCEIRO",
                            "label": "Tipo Parceiro",
                            "template": "field/select-native.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true,
                            "valueField": "value",
                            "descriptionField": "name",
                            "dataSource": {
                                "data": [
                                    {
                                        "name": "Lead",
                                        "value": "L"
                                    },
                                    {
                                        "name": "Prospect",
                                        "value": "P"
                                    },
                                    {
                                        "name": "Cliente",
                                        "value": "C"
                                    }
                                ]
                            }
                        },
                        {
                            "id":"17223197583113133",
                            "name": "DSSEGMER",
                            "label": "Segmento de Mercado",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113134",
                            "name": "DSATIVCLI",
                            "label": "Ramo de Atividade",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113135",
                            "name": "DSTPATIVCLIENTE",
                            "label": "Tipo de Atividade",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113136",
                            "name": "NMMODODIVULG",
                            "label": "Como Chegou Até Nós",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113137",
                            "name": "IDATIVO",
                            "label": "Ativo",
                            "template": "field/select-native.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true,
                            "valueField": "value",
                            "descriptionField": "name",
                            "validations": {},
                            "dataSource": {
                                "data": [
                                    {
                                        "name": "Sim",
                                        "value": "S"
                                    },
                                    {
                                        "name": "Não",
                                        "value": "N"
                                    }
                                ]
                            }
                        },
                        {
                            "id":"17223197583113138",
                            "name": "NRFUNCIONARIOS",
                            "label": "Nr. Funcionários",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true
                        }
                    ],
                    "widgets":[]
                },
                {
                    "id": "14418982341257778598851",
                    "name": "WDGT_DADOS_ADICIONAIS",
                    "container": activeWidget.container,
                    "label": "Dados Adicionais",
                    "showCheckbox": false,
                    "isVisible": true,
                    "itemsPerPage": 30,
                    "template": "widget/form-view.html",
                    "templateToForm":"widget/form.html",
                    "showDescriptionOrder": false,
                    "dataSource": {
                        "rest": true,
                        "lazyLoad": true,
                        "name": "product#/crud/ger_parcnegocioinfo",
                        "itemsPerPage": 100
                    },
                    "dataSourceFilter": [
                        {
                            "name": "P_NRPARCNEGOCIO",
                            "operator": "=",
                            "value": ""
                        },
                        {
                            "name": "P_NRORG",
                            "operator": "=",
                            "value": "@templateManager.LOGGEDOPERATOR.NRORG@"
                        }
                    ],
                    "actions": [],
                    "events": [
                        {
                            "id": "14418982343495474723876",
                            "name": "WidgetOnEnter",
                            "code": "PartnerInfosModalController.setCurrentRowDadosAdicionais(args.owner)"
                        }
                    ],
                    "fields": [   
                        {
                            "id":"17223197583113139",
                            "name": "NMRESPGERAL",
                            "label": "Responsável Geral",
                            "class":"12",
                            "isVisible": true,
                            "showOnForm": true,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113140",
                            "name": "NMPONTOATENDIMENTO",
                            "label": "Ponto de Atendimento",
                            "class":"12",
                            "isVisible": true,
                            "showOnForm": true,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113141",
                            "name": "NRPONTUACAO",
                            "label": "Pontuação",
                            "class":"6",
                            "isVisible": true,
                            "showOnForm": true,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113142",
                            "name": "CDINTEGRACAO",
                            "label": "Núm. Integração",
                            "class":"6",
                            "isVisible": true,
                            "showOnForm": true,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113143",
                            "name": "DTINIRELACIONA",
                            "label": "Data de Início do Relacionamento",
                            "class":"6",
                            "isVisible": true,
                            "showOnForm": true,
                            "dateFormat": "DD/MM/YYYY",
                            "template": "field/calendar.html"
                        },
                        {
                            "id":"17223197583113143",
                            "name": "NMMODORECEBINFO",
                            "label": "Como Deseja Receber Informações",
                            "class":"6",
                            "isVisible": true,
                            "showOnForm": true,
                            "template": "field/text-edit.html"
                        }
                    ],
                    "widgets": []
                },
                {
                    "id":"17223197583113144",
                    "name": "WDGT_PARTNER_ADDRESS",
                    "label": "Endereços",
                    "container": activeWidget.container,
                    "template": "widget/grid.html",
                    "isVisible": true,
                    "dataSource": {
                        "name": "product#/crud/enderecoparc_localizacao_tipoendereco",
                        "rest": true,
                        "itemsPerPage": 50,
                        "lazyLoad": true
                    },
                    "dataSourceFilter": [
                        {
                            "name": "NRPARCNEGOCIO",
                            "operator": "=",
                            "value": ""
                        }
                    ],
                    "actions": [],
                    "events": [
                        {
                            "id":"17223197583113144",
                            "name": "WidgetOnEnter",
                            "code": "args.owner.reload();PartnerInfosModalController.resizeWindow()"
                        }
                    ],
                    "fields": [
                        {
                            "id":"17223197583113145",
                            "label": "Tipo de Endereço",
                            "name": "NMTIPOENDERECO",
                            "template": "field/text-edit.html",
                            "isVisible": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "valueField": "CDTIPOENDERECO",
                            "descriptionField": "NMTIPOENDERECO",
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id":"17223197583113146",
                            "name": "NRCEPENDERECO",
                            "label": "CEP",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnList": true,
                            "showOnForm": true,
                            "events": [],
                            "mask": {
                                "type": "fix",
                                "params": {
                                    "cleanMaskOnModel": true,
                                    "mask": "99999-999"
                                }
                            }
                        },
                        {
                            "id":"17223197583113147",
                            "name": "DSENDERECO",
                            "label": "Endereço",
                            "validations": {
                                "required": true
                            },
                            "maxlength": 250,
                            "class": "10",
                            "showOnList": true,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113147",
                            "name": "NRIMOVELENDERECO",
                            "label": "Número",
                            "class": "2",
                            "maxlength": 10,
                            "isVisible": true,
                            "showOnList": true,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113148",
                            "name": "DSCOMPLEENDERECO",
                            "label": "Complemento",
                            "class": "12",
                            "maxlength": 100,
                            "isVisible": true,
                            "showOnList": true,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113149",
                            "name": "NMPAIS",
                            "label": "País",
                            "template": "field/select.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnList": true,
                            "dataSource": {
                                "name": "product#/pais",
                                "lazyLoad": true
                            },
                            "dataSourceFilter": [],
                            "outData": {
                                "CDPAIS": "CDPAIS",
                                "NMPAIS": "NMPAIS"
                            },
                            "fields": [
                                {
                                    "id":"17223197583113150",
                                    "label": "Código",
                                    "name": "CDPAIS",
                                    "class": "4",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                },
                                {
                                    "id":"17223197583113151",
                                    "label": "País",
                                    "name": "NMPAIS",
                                    "class": "8",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                }
                            ],
                            "events": []
                        },
                        {
                            "id":"17223197583113152",
                            "name": "NMESTADO",
                            "label": "Estado",
                            "template": "field/select.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnList": true,
                            "dataSource": {
                                "name": "product#/estado",
                                "lazyLoad": true
                            },
                            "dataSourceFilter": [],
                            "outData": {
                                "SGESTADO": "SGESTADO",
                                "NMESTADO": "NMESTADO"
                            },
                            "fields": [
                                {
                                    "id":"17223197583113153",
                                    "label": "Sigla",
                                    "name": "SGESTADO",
                                    "class": "4",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                },
                                {
                                    "id":"17223197583113153",
                                    "label": "Estado",
                                    "name": "NMESTADO",
                                    "class": "8",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                }
                            ],
                            "events": []
                        },
                        {
                            "id":"17223197583113154",
                            "name": "NMMUNICIPIO",
                            "label": "Município",
                            "template": "field/select.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnList": true,
                            "dataSource": {
                                "name": "product#/municipio",
                                "lazyLoad": true
                            },
                            "dataSourceFilter": [],
                            "outData": {
                                "CDMUNICIPIO": "CDMUNICIPIO",
                                "NMMUNICIPIO": "NMMUNICIPIO"
                            },
                            "fields": [
                                {
                                    "id":"17223197583113155",
                                    "label": "Código",
                                    "name": "CDMUNICIPIO",
                                    "class": "4",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                },
                                {
                                    "id":"17223197583113156",
                                    "label": "Município",
                                    "name": "NMMUNICIPIO",
                                    "class": "8",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                }
                            ],
                            "events": []
                        },
                        {
                            "id":"17223197583113157",
                            "name": "NMBAIRRO",
                            "label": "Bairro",
                            "template": "field/select.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnList": true,
                            "dataSource": {
                                "name": "product#/bairro",
                                "lazyLoad": true
                            },
                            "dataSourceFilter": [],
                            "outData": {
                                "CDBAIRRO": "CDBAIRRO",
                                "NMBAIRRO": "NMBAIRRO"
                            },
                            "events": []
                        }
                    ],
                    "widgets": []
                },
                {
                    "id":"17223197583113158",
                    "name": "WDGT_PARTNER_CONTACTS",
                    "label": "Contatos",
                    "container": activeWidget.container,
                    "template": "widget/grid.html",
                    "isVisible": false,
                    "dataSource": {
                        "rest": true,
                        "name": "product#/crm_relacionaparc",
                        "lazyLoad": true
                    },
                    "dataSourceFilter": [
                        {
                            "name": "NRPARCNEGOCIO",
                            "operator": "=",
                            "value": ""
                        },
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('CRM_NRORG')@"
                        },
                        {
                            "name": "P_NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('CRM_NRORG')@"
                        }
                    ],
                    "actions": [],
                    "events": [
                        {
                            "id":"17223197583113159",
                            "name": "WidgetOnEnter",
                            "code": "args.owner.reload();PartnerInfosModalController.resizeWindow()"
                        }
                    ],
                    "fields": [
                        {
                            "id":"17223197583113160",
                            "name": "NOMECONT",
                            "label": "Nome",
                            "isVisible": true,
                            "showOnList": true,
                            "showOnForm": true,
                            "validations": {
                                "required": true
                            },
                            "class": 10,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113161",
                            "name": "EMAIL",
                            "label": "Email",
                            "isVisible": true,
                            "showOnList": true,
                            "showOnView": true,
                            "showOnForm": true,
                            "class": 6,
                            "template": "field/text-edit.html",
                            "events": []
                        },
                        {
                            "id":"17223197583113162",
                            "name": "TELEFONE",
                            "label": "Telefone",
                            "isVisible": true,
                            "showOnList": true,
                            "showOnView": true,
                            "showOnForm": true,
                            "class": 6,
                            "template": "field/text-edit.html",
                            "events": []
                        },
                        {
                            "id":"17223197583113163",
                            "name": "NMTIPOPARCEIRO",
                            "label": "Tipo de Parceiro",
                            "template": "field/select.html",
                            "isVisible": true,
                            "showOnList": true,
                            "class": "12",
                            "valueField": "TIPODEPARCEIRO",
                            "descriptionField": "NMTIPODEPARCEIRO",
                            "dataSource": {
                                "rest": true,
                                "name": "product#/crm_tipoParceiro",
                                "lazyLoad": true
                            },
                            "outData": {
                                "NMTIPODEPARCEIRO": "NMTIPODEPARCEIRO",
                                "TIPODEPARCEIRO": "TIPODEPARCEIRO"
                            },
                            "dataSourceFilter": [],
                            "events": []
                        }
                    ],
                    "widgets": []
                },
                {
                    "id": "14418982341257778598852",
                    "name": "WDGT_PARTNER_COMU",
                    "container": activeWidget.container,
                    "label": "Comunicação",
                    "showCheckbox": false,
                    "isVisible": true,
                    "itemsPerPage": 30,
                    "template": "widget/master_detail/grid.html",
                    "showDescriptionOrder": false,
                    "dataSource": {
                        "rest": true,
                        "lazyLoad": true,
                        "name": "product#/crm_comunicaparc",
                        "itemsPerPage": 100
                    },
                    "dataSourceFilter": [
                        {
                            "name": "P_NRPARCNEGOCIO",
                            "operator": "=",
                            "value": ""
                        },
                        {
                            "name": "P_NRORG",
                            "operator": "=",
                            "value": "@templateManager.LOGGEDOPERATOR.NRORG@"
                        },
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": "@templateManager.LOGGEDOPERATOR.NRORG@"
                        }
                    ],
                    "actions": [],
                    "events": [
                        {
                            "id": "14418982343495474723875",
                            "name": "WidgetOnEnter",
                            "code": "args.owner.reload();PartnerInfosModalController.resizeWindow()"
                        }
                    ],
                    "fields": [
                        {
                            "id": "14418982341628670838852",
                            "name": "IDATIVO",
                            "label": "Ativo",
                            "isVisible": false,
                            "showOnList": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "14418982342800647217853",
                            "name": "NRCOMUNICAPARC",
                            "label": "NRCOMUNICAPARC",
                            "isVisible": false,
                            "showOnList": false,
                            "showOnForm": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "14418982341061414688854",
                            "name": "NRPARCNEGOCIO",
                            "label": "NRPARCNEGOCIO",
                            "isVisible": false,
                            "showOnList": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "14418982344140770823855",
                            "name": "NRENDERECOPARC",
                            "label": "NRENDERECOPARC",
                            "isVisible": false,
                            "showOnList": false,
                            "showOnForm": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "1441898234664443810856",
                            "name": "CDCOMPLECOMUPARC",
                            "label": "CDCOMPLECOMUPARC",
                            "isVisible": false,
                            "showOnList": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "14418982341567386213857",
                            "name": "CDFORMACOMU",
                            "label": "Forma de Comunicação",
                            "isVisible": false,
                            "showOnList": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "1441898234614649406858",
                            "name": "NMFORMACOMU",
                            "label": "Forma de Comunicação",
                            "class": "12",
                            "order": "CDFORMACOMU",
                            "maxlength": 4,
                            "isVisible": true,
                            "showOnList": true,
                            "showOnForm": true,
                            "readOnly": false,
                            "defaultValue": null,
                            "template": "field/select.html",
                            "showSelectAs": "grid",
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "placeholder": null,
                            "dataSource": {
                                "rest": true,
                                "localStorage": true,
                                "name": "product#/formacomu",
                                "itemsPerPage": 50,
                                "lazyLoad": true
                            },
                            "dataSourceFilter": [
                                {
                                    "name": "NRORG",
                                    "operator": "IN",
                                    "value": "@[getLocalVar('CRM_NRORGTRAB'),getLocalVar('CRM_NRORGPADRAO') ]@"
                                }
                            ],
                            "outData": {
                                "CDFORMACOMU": "CDFORMACOMU",
                                "NMFORMACOMU": "NMFORMACOMU"
                            },
                            "fields": [
                                {
                                    "id": "14418982344259089695859",
                                    "label": "Código",
                                    "name": "CDFORMACOMU",
                                    "class": "4",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                },
                                {
                                    "id": "14418982341859185194860",
                                    "label": "Forma de Comunicação",
                                    "name": "NMFORMACOMU",
                                    "class": "8",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                }
                            ],
                            "events": []
                        },
                        {
                            "id": "14418982341691438344861",
                            "name": "CDPREFIXCOMUPARC",
                            "label": "Prefixo",
                            "class": "2",
                            "isVisible": true,
                            "showOnList": true,
                            "maxlength": 10,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "14418982342324818248862",
                            "name": "CDCOMUNICAPARC",
                            "label": "Descrição",
                            "class": "10",
                            "isVisible": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "maxlength": 250,
                            "template": "field/text-edit.html"
                        }
                    ],
                    "widgets": []
                }
			],
			"actions": [
			    {
			        "id":"17223197583113164",
                    "name": "BACK_MODAL",
                    "label": "Voltar",
                    "icon": "back",
                    "isVisible": true,
                    "showAsAction": "back",
                    "events": [
                        {
                            "id":"17223197583113165",
                           	"name": "ActionEvent",
                            "code": "ScreenService.closePopup(true)"
                        }
                    ]
                }
            ]
        };
        
        var modalWidget = metaDataFactory.widgetFactory(modalWidgetJson);
        
        return modalWidget;
        
    };
    
    /**
     * Abri o modal com as informações do parceiro.
    */
    this.openModal = function(){
        
        ScreenService.openPopup(templateManager.container.getWidget('WDGT_MODAL_PARTNER_INFOS')).then(function(){
            var widget = templateManager.container.getWidget('WDGT_PARTNER_DATA');
            //this.setDataWidgetCurrentRow(widget);
        }.bind(this));
        
    };
    
    /**
     * Define o filtro das widgets do modal informativo
     * de acordo com o parceiro selectionado na widget
     * ativa.
     * 
     * @param {object} modalWidget widget do modal informativo.
     * 
     * @param {object} activeWidget widget ativa onde o parceiro
     * foi selecionado.
     * 
     * @param {string} partnerFieldName nome do campo referente ao
     * parceiro de negócio.
    */
    this.setModalFilterByActivePartner = function(modalWidget, activeWidget, partnerFieldName){
        
        var selectedPartner = activeWidget.currentRow[partnerFieldName];
        
        var modalWidgets = modalWidget.widgets;
        var dataWidgetIndex = CRM_Utilities.getObjectPositionInArrayByName(modalWidgets, "WDGT_PARTNER_DATA");
        var addressWidgetIndex = CRM_Utilities.getObjectPositionInArrayByName(modalWidgets, "WDGT_PARTNER_ADDRESS");     
        var contactsWidgetIndex = CRM_Utilities.getObjectPositionInArrayByName(modalWidgets, "WDGT_PARTNER_CONTACTS");
        var comunicationWidgetIndex = CRM_Utilities.getObjectPositionInArrayByName(modalWidgets, "WDGT_PARTNER_COMU");
        var additionalWidgetIndex = CRM_Utilities.getObjectPositionInArrayByName(modalWidgets, "WDGT_DADOS_ADICIONAIS");
        var dataWidget = modalWidgets[dataWidgetIndex]; 
        var addressWidget = modalWidgets[addressWidgetIndex];     
        var contactsWidget = modalWidgets[contactsWidgetIndex];
        var comunicationWidget = modalWidgets[comunicationWidgetIndex];
        var additionalWidget = modalWidgets[additionalWidgetIndex];
        
        var dataWidgetNrparcnegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(dataWidget.dataSourceFilter, "NRPARCNEGOCIO");
        var addressWidgetNrparcnegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(addressWidget.dataSourceFilter, "NRPARCNEGOCIO");
        var contactsWidgetNrparcnegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(contactsWidget.dataSourceFilter, "NRPARCNEGOCIO");
        var comunicationWidgetNrparcnegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(comunicationWidget.dataSourceFilter, "P_NRPARCNEGOCIO");
        var additionalWidgetNrparcnegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(additionalWidget.dataSourceFilter, "P_NRPARCNEGOCIO");
        
        dataWidget.dataSourceFilter[dataWidgetNrparcnegocioFilterIndex].value = selectedPartner;
        addressWidget.dataSourceFilter[addressWidgetNrparcnegocioFilterIndex].value = selectedPartner;
        contactsWidget.dataSourceFilter[contactsWidgetNrparcnegocioFilterIndex].value = selectedPartner;
        comunicationWidget.dataSourceFilter[comunicationWidgetNrparcnegocioFilterIndex].value = selectedPartner;
        additionalWidget.dataSourceFilter[additionalWidgetNrparcnegocioFilterIndex].value = selectedPartner;
        
        dataWidget.reload();
        
    };
    
    /**
     * Atribui o parceiro retornado no data source do modal
     * na currentRow da widget de informações do parceiro
     * ('WDGT_PARTNER_DATA').
     * 
     * @param {object} widget widget de dados do parceiro.
     * ('WDGT_PARTNER_DATA').
    */
    this.setDataWidgetCurrentRow = function(widget){

        widget.moveToFirst();
        
    };
    
    /**
     * Insere a widget modal na widget recebida como 
     * parâmetro.
     * 
     * @param {object} widget widget que receberá o modal.
     * 
     * @param {string} nrparcnegocio nome referente ao
     * campo de parceiro de negócio.
    */
    this.setModalWidgetOnActiveWidget = function(widget, nrparcnegocio){
        
        this.createModalAction(widget);
        widget.widgets.push(this.factoryModalWidget(widget, nrparcnegocio));
        
    };
    
    /**
     * Cria a action para abrir o modal com os dados
     *  do parceiro na widget.
     * 
     * @param {object} widget widget que receberá o modal.
     * 
    */
    this.createModalAction = function(widget){
        /*Modificado em 22/09 por Arthur
            Adicionada propriedade 'preventHide',
            que garante que a action de Dados do Parceiro 
            não seja setada como invisível em chamados já 
            concluidos
        */
        var modalAction = {
            "id":"17223197583113166",
            "name": "ACTION_MODAL_INFOS_PARCEIRO",
            "label": "Consultar Dados do Parceiro",
            "isVisible": true,
            "preventHide":true,
            "showAsAction": "never_view",
            "events": [
                {
                    "name": "ActionEvent",
                    "code": "PartnerInfosModalController.openModal();"
                }
            ]
        };
        
        if((widget.template.search("master_detail") === -1) && widget.template.search("kanban") === -1){
            modalAction.showAsAction = "never_list";
        }
        
        metaDataFactory.actionFactory(modalAction, widget, true);
        
    };
    
    
    /**
     * Muda o template da tab de interlocutor, necessário
     * para evitar o click na row, que abria o swipe.
     * 
     * @param {object} widget widget modal.
     * 
    */
    this.changeTemplateTabInterlocutor = function(widget){
        var interlocutorTab = templateManager.container.getWidget('WDGT_INTERLOCUTOR');
        interlocutorTab.template = 'widget/grid.html';
        templateManager.updateTemplate();
        interlocutorTab.activate();
    };
    
    this.setCurrentRowDadosAdicionais = function(widget){
        widget.reload().then(function(){
            if(!Util.isEmptyOrBlank(widget.dataSource.data))
                widget.currentRow = widget.dataSource.data[0];
        });
    }
    
    /**
     * Evita Bug de tamanho de colunas inferior ao tamanho do registro
    */
    this.resizeWindow = function(){
        $(window).resize();
    };
	
}
Configuration(function(ContextRegister){
    ContextRegister.register('PartnerInfosModalController', PartnerInfosModalController);
});

// FILE: ./assets/js/util/SelectUtil.js
function SelectUtil(ScreenService, metaDataFactory, EventChildPropagation) {
    
    var self = this;
    
    var CROSSTAB_TEMPLATE = 'crosstab.html';
	var TREE_TEMPLATE = 'tree/grid.html';
	var TREE_LIST_TEMPLATE = 'tree/list.html';
	var GANTT_TEMPLATE = 'tree/gantt.html';
	
	var _isTemplate = function(widget, template) {
		return widget.template && widget.template.indexOf(template) !== -1;
	};
	
	this.isTree = function(widget) {
		return _isTemplate(widget,TREE_TEMPLATE) || _isTemplate(widget, TREE_LIST_TEMPLATE);
	};
	
	var isLastChild = function(row){
		return row._childCount === 0;
	};
	
	var setChanged = function(field) {
		field.widget.currentRow.changed = true;
	};
    
    this.construirWidgetSelect = function(field, newActions) {
        if (!field.selectWidget) {
            var actions = [{
    			"label": ScreenService.i18n("Cancel"),
    			"showAsAction": "back",
    			"isVisible": true,
    			"showOnForm": true,
    			"showOnList": true,
    			"hideIcon": true,
    			"events": [{
    				"name": "ActionEvent",
    				"code": "ScreenService.closePopup(); args.owner.widget.afterClosePopup(args.owner.widget.selectField);",
    				"id": 9998
    			}]
    		}];
    
    		if (field.showAddButtonAs == 'action') {
    			actions.push({
    				"icon": "plus",
    				"label": field.addButtonText,
    				"showAsAction": "always|edit|checked_rows|view",
    				"isVisible": true,
    				"showOnForm": true,
    				"showOnList": true,
    				"events": [{
    					"name": "ActionEvent",
    					"code": "args.owner.widget.onAdd()",
    					"id": 9999
    				}]
    			});
    		}
    		
    		actions = actions.concat(newActions);

    		var template;
    		if (!field.showSelectAs) {
    			template = "widget/list.html";
    		} else if (field.showSelectAs === "tree") {
    			template = "widget/tree/grid.html";
    		} else {
    			template = "widget/" + field.showSelectAs + ".html";
    		}
    
    		var fields;
    		if (field.fields && field.fields.length) {
    			fields = field.fields;
    		} else {
    			fields = [{
    				"label": field.sourceLabel,
    				"name": field.descriptionField,
    				"showOnList": true,
    				"mask": field.mask
    			}];
    		}
            
			var widgetEvents = [{
				"name": "WidgetAfterMoveRow",
				"code": "args.owner.setSelected(args);",
				"id": 9999
			}];
			field.selectWidget = metaDataFactory.widgetFactory({
				"id": 9009,
				"name": "Widget",
				"label": field.label,
				"template": template,
				"container": field.widget.container,
				"isVisible": true,
				"popupNoBlock": false,
				"showDescriptionOrder": false,
				"floatingControl": {
					"openSearch": !Util.isMobile(),
					"defaultSearchField": field.defaultSearchField || field.descriptionField,
					"pdfAction": false,
					"xlsAction": false,
					"csvAction": false,
					"searchAction": true,
					"customizationAction": {
						"filter": false,
						"layout": false,
						"view": false
					}
				},

				"parentProperty": field.parentProperty,
				"primaryProperty": field.primaryProperty,
				"expandingProperty": field.expandingProperty,
				"breadcrumbProperty": field.breadcrumbProperty,

				"order": field.order,
				"reverse": field.reverse,
				"bringGroupsCollapsed": field.bringGroupsCollapsed,
				"orderFromGroups": field.orderFromGroups,
				"selectField": field,
				"showCheckbox": false,
				"onAdd": field.onAdd,
				"showAddButtonAs": field.showAddButtonAs,
				"addButtonText": field.addButtonText,
				"noMatchText": field.noMatchText,
				"searchNoMatchText": field.searchNoMatchText,
				"fields": fields,
				"dataSource": field.dataSource,
				"dataSourceFilter": field.dataSourceFilter,
				"itemsPerPage": field.itemsPerPage,
				"searchWithPagination": Util.isUndefinedOrNull(field.searchWithPagination) ? true : field.searchWithPagination,
				"hasPagination": Util.isUndefinedOrNull(field.hasPagination) ? true : field.hasPagination,
				"onDemand": field.onDemand,
				"allowSelectOnlyLastChild": field.allowSelectOnlyLastChild,
				"searchCriteria": field.searchCriteria,

				"actions": actions,
				"events": EventChildPropagation.propagate(widgetEvents, field),

				"styles": field.selectWidgetStyles
			});
			
			field.selectWidget.setSelected = this.setSelected;
			field.selectWidget.afterClosePopup = this.afterClosePopup;
		}
    };
    
    this.setSelected = function(args) {
		var field = args.owner.selectField;
		if(!field.allowSelectOnlyLastChild || (field.allowSelectOnlyLastChild && self.isTree(field.selectWidget) && isLastChild(args.row))){
			ScreenService.closePopup();

			var row = args.row;
			
			if (field.element) {
				field.element.focus();
			}

			(field.outData ? self.changeByOut : self.changeByName)(field, row);
			field.dataSource.clearCheckedRows();
			field.dataSource.addCheckedRows(row);
			setChanged(field);

			if (field._afterSelected) {
				field._afterSelected(row);
			}
			if (field.change) {
				field.change();
			}
		}
	};
	
	this.changeByOut = function(field, row) {
		if (field.widget.currentRow) {
			var outData = field.outData || {};

			Object.keys(outData).forEach(function(key) {
				if (Array.isArray(outData[key])) {
					outData[key].forEach(function(valueKey) {
						field.widget.currentRow[valueKey] = row[key];
					});
				} else {
					field.widget.currentRow[outData[key]] = row[key];
				}
			});
		}
	};
	
	this.changeByName = function(field, row) {
      var valueField = field.valueField || field.name;
      var descriptionField = field.descriptionField || field.name;
      field.widget.currentRow[valueField] = row[valueField];
      field.widget.currentRow[field.name] = row[descriptionField];
      field.widget.inlineEdit();
    };
    
    this.afterClosePopup = function(field) {
		if (field.element) {
			field.element.focus();
		}
	};
}

Configuration(function(ContextRegister) {
    ContextRegister.register("SelectUtil", SelectUtil);
});

// FILE: ./assets/js/util/Utilities.js
function Utilities(ScreenService, General, restEngine, ZHPromise, templateManager, ZhFilterPreferences, ZhUtil) {

    /**
     * Carrega os registros da widget passada quando ela estiver em modo list.
     * 
     * @param {object} widget
     */
    this.carregarWidgetOnEnterTab = function (widget) {
        if (widget.mode == "list") widget.reload();
    };

    /**
     * Carrega na currentRow os dados do dataSource em uma 
     * widget com template form.html
     * @params{Object} widget - widget de form
     * @params{bool} alterarAction - determina se as actions de add e edit também serão validadas
    **/
    this.carregarFormOnEnterTab = function (widget, alterarAction) {
        if (widget.dataSource && widget.dataSource.data && widget.dataSource.data.length > 0)
            widget.currentRow = widget.dataSource.data[0];
        else if (widget.active)
            this.formWidgetOnEnterTab(widget, alterarAction);
    };

    this.setValoresPadroesParaCampoSelectMultiple = function (field, value, index) {
        var data = this.procurarDataSource(field, value, index);
        field.value(this.criarArrayPeloDataSet(data, field.valueField));
    };

    this.procurarDataSource = function (field, values, index) {
        if (!field.dataSource || !field.dataSource.data) {
            return [];
        } else {
            return field.dataSource.data.filter(function (row) {
                var found = false;
                values.forEach(function (value) {
                    if (row[index] == value) {
                        found = true;
                    }
                });
                return found;
            });
        }
    };

    this.criarArrayPeloDataSet = function (dataSet, index) {
        var array = [];
        dataSet.forEach(function (row) {
            array.push(row[index]);
        });
        return array;
    };
    /**
     * Cria uma linha na widget para cada registro selecionado no campo select-multiple passado, 
     * a fim de inserir múltiplos registros de uma só vez na tabela.
     * 
     * @param {object} widget - Widget da tela
     * @param {string} nomeCampo - Nome do campo (field) select-multiple
     * 
     * @return {object[]}
     */
    this.gerarLinhasParaInsertMultiplo = function (widget, nomeCampo, camposOutData, usaNovoCodigo) {
        if (!camposOutData) {
            camposOutData = widget.getField(nomeCampo).outData;

            if (!camposOutData) {
                camposOutData = { nomeCampo: nomeCampo };
            }
        }

        var registros = widget.currentRow[nomeCampo];
        var rows = [];

        for (var i = 0; i < registros.length; i++) {
            var currentRow = angular.copy(widget.currentRow);
            currentRow.__is_new = true;

            if (usaNovoCodigo && !currentRow.__new_code_pk) {
                if (widget.paramsNewCode) {
                    currentRow.__using_unit = widget.paramsNewCode.__using_unit || false;
                    currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
                    currentRow.__table_pk = widget.paramsNewCode.__table_pk;
                    currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
                    currentRow.__cast_number = widget.paramsNewCode.__cast_number || false;
                    if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                        widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
                }
            }

            var keys = Object.keys(camposOutData);
            var values = Object.values(camposOutData);

            for (var j = 0; j < keys.length; j++) {
                currentRow[values[j]] = currentRow[values[j]][i];
            }

            rows.push(currentRow);
        }

        return rows;
    };

    /**
     * Insere registros múltiplos.
     * 
     * @param {object} widget
     * @param {string} nomeCampo - Nome do campo (field) select-multiple na current row
     * @param {string} rota - Rota de "save" da widget
     * 
     * @return promise
     */
    this.inserirMultiplos = function (widget, nomeCampo, camposOutData, rota) {
        if (!rota)
            rota = widget.dataSource.name + "/save";

        var rows = this.gerarLinhasParaInsertMultiplo(widget, nomeCampo, camposOutData, false);
        return General.callRestEngine(rota, "DataSet", rows);
    };

    this.inserirMultiplosComNovoCodigo = function (widget, nomeCampo, camposOutData, rota) {
        if (!rota)
            rota = widget.dataSource.name + "/save";

        var rows = this.gerarLinhasParaInsertMultiplo(widget, nomeCampo, camposOutData, true);
        return General.callRestEngine(rota, "DataSet", rows);
    };

    /**
     * Ao selecionar algum item da árvore, seus itens pai também são selecionados.
     * 
     * @param {object} widget
     */
    this.selecionarTodosItensPai = function (widget) {
        var currentRow = widget.selectedRow;
        this.selecionarPai(widget, currentRow);
    };

    /**
     * Seleciona a linha pai da que foi passada.
     * 
     * @param {object} widget
     * @param {object} row
     */
    this.selecionarPai = function (widget, row) {
        widget.selectAll([row]);
        if (row._parent) {
            this.selecionarPai(widget, row._parent);
        }
    };

    /**
     * Retorna todos os itens pai de um elemento de uma árvore.
     * 
     * @param {object} row
     * 
     * @return {object[]}
     */
    this.getItensPai = function (row) {
        var itensPai = [];
        return this.getItensPaiAux(itensPai, row);
    };

    /**
     * Função auxiliar da "getItensPai", que vai adicionando os itens pai recursivamente em um array.
     * 
     * @param {object[]} itensPai
     * @param {object} row
     * 
     * @return {object[]}
     */
    this.getItensPaiAux = function (itensPai, row) {
        if (row._parent) {
            itensPai.push(row._parent);
            itensPai = this.getItensPaiAux(itensPai, row._parent);
        }
        return itensPai;
    };

    /**
     * Retorna o maior item pai (item mais acima) de um elemento de uma árvore.
     * 
     * @param {object} row
     * @return {object}
     */
    this.getMaiorItemPai = function (row) {
        return (row._parent) ? this.getMaiorItemPai(row._parent) : row;
    };

    /**
     * Retorna os itens raíz da árvore, ou seja, os itens que não tem item pai.
     * 
     * @param {object[]} dataSource
     * @return {object[]}
     */
    this.getItensRaizArvore = function (rows) {
        return rows.filter(function (row) {
            return !row._parent;
        });
    };

    /**
     * Abre um relatório BIRT
     */
    this.openReport = function (route, reportName, params) {
        params.__report_name = reportName;

        var reportParams = {
            "requestType": "Row",
            "row": params
        };

        return restEngine.post(route, reportParams, null, null, null, null, 300000).then(function (response) {
            if (response && response.messages) {
                var report = response.messages.pop();
                window.open(window.serviceUrl+report.message);
            }
        }).catch(function (response) {
            if (response.data && response.data.error) {
                ScreenService.showMessage(ScreenService.i18n(response.data.error), 'E');
            }
        });
    };

    /**
     * Se o NRORG do registro for igual a zero, o registro fica inalterável.
     */
    this.validarNrorgZero = function (widget) {
        if (!widget.currentRow.__is_new) {
            var actions = [
                widget.getAction("edit"),
                widget.getAction("delete"),
            ];

            var nrorg = widget.currentRow.NRORG;
            if (nrorg !== undefined) {
                var visivel = (nrorg == 0) ? false : true;

                actions.forEach(function (act) {
                    act.isVisible = visivel;
                });
            }
        }
    };

    /**
     * Função para identificar o index de um objeto
     * dentro de um FieldGroups, de acordo com 
     * seu nome.
     * 
     * @param {object} fieldGroup objeto da busca.
     * 
     * @param {String} name propriedade no objeto 
     * pelo qual é feito a busca.
     * 
     * @return {int} indexFieldGroupObject index Do objeto 
     * encontrado.
    */
    this.getFieldGroupPosition = function (fieldGroups, name) {
        var indexFieldGroupObject;

        fieldGroups.forEach(function (value, i) {
            if (value.name === name) {
                indexFieldGroupObject = i;
            }
        });
        return indexFieldGroupObject;
    };

    /**
     * Função para identificar o index de um objeto
     * dentro de um dataSourceFilter, de acordo com 
     * seu nome.
     * 
     * @param {object} dataSourceFilter objeto da busca.
     * 
     * @param {String} name propriedade no objeto pelo qual
     * é feito a busca.
     * 
     * @return {int} indexFilterObject index Do objeto 
     * encontrado.
     * 
    */
    this.getFilterPosition = function (dataSourceFilter, name) {
        var indexFilterObject;

        dataSourceFilter.forEach(function (value, i) {
            if (value.name === name) {
                indexFilterObject = i;
                return;
            }
        });

        return indexFilterObject;
    };

    this.callRestEngineWithTimeout = function (route, requestType, requestValue, miliseconds) {
        var defer = ZHPromise.defer();
        var params = {};

        switch (requestType) {
            case "Row":
                params = {
                    "requestType": "Row",
                    "row": requestValue
                };
                break;

            case "DataSet":
                params = {
                    "requestType": "DataSet",
                    "dataset": requestValue
                };
                break;

            case "FilterData":
                params = {
                    "requestType": "FilterData",
                    "filter": requestValue
                };
                break;

            default:
                return defer.reject("Erro na passagem de parâmetros para a requisição. O valor para requestType deve ser 'Row', 'DataSet' ou 'FilterData'.");
        }

        if (miliseconds)
            return restEngine.post(route, params, null, null, null, null, miliseconds);
        else
            return restEngine.post(route, params);
    };


    /**
    * Verifica se o e-mail é válido.
    *
    * retorna true para e-mail válido, false para inválido
    */

    this.verificaEmail = function (email) {
        var valida = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
        var retorno = valida.test(email);

        return retorno;
    };

    /**
     * Valida no banco se o dados que estao sendo inseridos ja não existem nos campos dos quais estão sendo gravados
     * 
     * Retona true se nao existe ou false se já existir na promise
     * 
     * @param {object} currenteRow da tela.
     * 
     * @param {Array} Lista de campos que serão verificados
     * 
     * @return {String} Rota do dataSource
     */
    this.validarCamposDuplicados = function (widget, pkFieldNames, validationFieldNames, findRoute) {
        var promise = ZHPromise.defer();

        if (!findRoute)
            findRoute = widget.dataSource.name;

        var params = {
            "DATASOURCE": findRoute,
            "COLUNAS_PK": pkFieldNames,
            "COLUNAS_VALIDACAO": validationFieldNames,
            "ROW": widget.currentRow
        };
        General.callRestEngine("product#/validacoes_gerais/validarRegistrosDuplicados", "Row", params).then(function (response) {

            if (response.dataset.EXISTE[0]) {
                promise.resolve(true);
            } else {
                promise.resolve(false);
            }
        })
            .catch(function (error) {
                promise.reject(error);
            });
        return promise;
    };

    /**
     * Constroi a função Criteria que nao existe no app.js para ser concatenado ao arquivo
     * (fundametal para o funcionamento do calendário e do filtro do mapa)
     * 
     * @param {object} widget widget 
    */
    this.buildCriteria = function (widget) {

        widget.setSearchCriteria = function (searchCriteria, forceReload) {
            if (!forceReload) {
                widget._noReloadFromSearchCriteria = true;
            }
            widget.searchCriteria = {};
            widget.searchList = "";
            if (searchCriteria) {
                searchCriteria.forEach(function (field) {
                    var fieldValue = field.value;
                    if (fieldValue[fieldValue.length - 1] == '%') {
                        fieldValue = fieldValue.substring(0, (fieldValue.length - 1));
                    }
                    if (fieldValue[0] == "%") {
                        fieldValue = fieldValue.substring(1, fieldValue.length);
                    }
                    this.widget.searchCriteria[field.name] = fieldValue;
                });
            }
        };
    };

    /**
     * Retorna o valor do field na widget informada. 
     * Caso não informe a widget, será considerada a widget ativa.
     */

    this.getFieldValue = function (name, widgetName) {
		var widget;
		if (widgetName) {
		    widget = templateManager.container.getWidget(widgetName);
		}
	    else {
	        widget = templateManager.container.getActiveWidget();
	    }
		    
		if (widget && widget.currentRow) {
			return widget.currentRow[name];
		}
		else {
			return "";
		}
	};
	
	 /* Função para confirmar e setar na currentRow 
     * do endereço em cadastro, o endereço retornado
     * da pesquisa pelo CEP no banco.
     * 
     * @param {string} CEP cep que será utilizado 
     *  para encontrar o endereço
     * 
    */
    this.buscarCep = function(CEP) {

        CEP = CEP.replace(/[\D]+/g, '');

        var params = {
            'CEP': CEP
        };

        if (CEP.length == 8) {
            return General.callRestEngine('/findAddress', "Row", params);
        } else {
            ScreenService.showMessage('CEP Inválido.');
        }
    };

    this.transformaHorasEmSegundos = function (horas) {
        horas = horas.split(':');
        return horas[0] * 3600 + horas[1] * 60;
    };


    /**
     * Mostra mensagem de erro, sendo ela uma string ou um objeto de erro de promise.
     */
    this.showErrorMessage = function (error) {
        console.error(error);
        var message = error;
        if (typeof error === 'object') {
            if (error.data && error.data.error) {
                message = error.data.error;
            }
            else if (error.message) {
                message = error.message;
            }
        }

        if (typeof message === 'object' && message.data == null) {
            message = 'Tempo limite excedido. Por favor, tente novamente.';
        }
        if (!message) {
            message = 'Ocorreu um erro inesperado. Entre em contato com o administrador do sistema.';
        }
        else if (message.indexOf('Exception: Exception: ') != -1) {
            message = message.replace('Exception: Exception: ', '');
        }

        ScreenService.showMessage(message, 'error');
    };

    /**
     * Salva a currentRow da widget e seta os valores da "__columns_to_return" na currentRow.
     * O parâmetro "showError" define se, caso ocorra algum erro na requisição do save, o erro será exibido.
     */
    this.widgetSaveAndSetColumnsToReturn = function(widget, showError) {
        if (showError == undefined) {
            showError = true;
        }

        var deferred = ZHPromise.defer();

        var columnsToReturn = null;
        if (widget.paramsNewCode && widget.paramsNewCode.__columns_to_return) {
            columnsToReturn = widget.paramsNewCode.__columns_to_return;
        }
        else {
            columnsToReturn = widget.currentRow.__columns_to_return;
        }

        General.widgetSave(widget).then(function (response) {
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                if (Array.isArray(columnsToReturn)) {
                    columnsToReturn.forEach(function (column) {
                        widget.currentRow[column] = response.dataset.columnsToReturn[0][column];
                    });
                }
            }

            deferred.resolve(response);
        },
            function (error) {
                if (showError !== false) {
                    this.showErrorMessage(error);
                }

                deferred.reject(error);
            }.bind(this));

        return deferred.promise;
    };
    /**
     * Manda buscar e coloca na row as informações 
     * em uma widget de template form.html
     * @params{Object} widget - widget de form
     * @params{bool} alterarAction - determina se as actions de add e edit também serão validadas
    **/
    this.formWidgetOnEnterTab = function (widget, alteraActions) {
        widget.list();
        var actAdicionar = widget.getAction("add");
        var actEditar = widget.getAction("edit");
        var actExcluir = widget.getAction("delete");
        var deferred = ZHPromise.defer();

        widget.reload().then(function () {
            if (widget.dataSource.data.length > 0) {
                widget.currentRow = widget.dataSource.data[0];
                if (alteraActions) {
                    actAdicionar.isVisible = false;
                    actEditar.isVisible = true;
                    if (actExcluir) actExcluir.isVisible = true;
                }
            } else {
                widget.currentRow = {};
                if (alteraActions) {
                    actAdicionar.isVisible = true;
                    actEditar.isVisible = false;
                    if (actExcluir) actExcluir.isVisible = false;
                }
            }

            widget.view();

            deferred.resolve();
        })
            .catch(function (error) {
                this.showErrorMessage(error);
                deferred.reject(error);
            });

        return deferred.promise;

    };

    this.formWidgetNewRow = function (widget) {
        widget.newRow();
        widget.template = 'widget/form.html';
    };

    this.formWidgetEdit = function (widget) {
        var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];

        //faz backup da currentRow para permitir a recuperação dos dados
        widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
        widget.edit();
        widget.template = 'widget/form.html';
    };

    this.formWidgetCancel = function (widget) {
        if (widget.currentRow.__is_new) {
            widget.dataSource.data.pop();
            widget.currentRow = {};
        } else {
            Object.keys(widget.backupRow).forEach(function (key) {
                widget.currentRow[key] = widget.backupRow[key];
            });
        }
        widget.view();
        widget.template = 'widget/form-view.html';
    };

    this.formWidgetSaveAndSetColumnsToReturn = function(widget, showError) {
        if (showError == undefined) {
            showError = true;
        }

        var deferred = ZHPromise.defer();

        var columnsToReturn = null;
        if (widget.paramsNewCode && widget.paramsNewCode.__columns_to_return) {
            columnsToReturn = widget.paramsNewCode.__columns_to_return;
        }
        else {
            columnsToReturn = widget.currentRow.__columns_to_return;
        }

        General.widgetSave(widget).then(function (response) {
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                if (Array.isArray(columnsToReturn)) {
                    columnsToReturn.forEach(function (column) {
                        widget.currentRow[column] = response.dataset.columnsToReturn[0][column];
                    });
                }
            }

            widget.view();
            widget.template = "widget/form-view.html";

            deferred.resolve(response);
        })
            .catch(function (error) {
                if (showError !== false) {
                    this.showErrorMessage(error);
                }

                deferred.reject(error);
            }.bind(this));

        return deferred.promise;
    };

    /**
     * Abre a tela com a arvore de elementos aberta
     * @params{Object} widget widget que será aberta
    **/
    this.abreArvoreElementos = function (widget) {
        var deferred = ZHPromise.defer();
        if (widget.mode === "list") {
            widget.reload().then(function () {
                try {
                    widget.dataSource.data.map(function (dado) {
                        dado._opened = true;
                    });

                    templateManager.updateTemplate();
                    deferred.resolve();
                } catch (e) {
                    deferred.reject();
                }
            }).catch(function (error) {
                deferred.reject();
            });
        }
        return deferred.promise;
    };

    this.tornaCamposSelectMultipleObrigatorios = function (widget, grupoDeCamposEscolhidos) {
        var vazio = true;
        var contem = false;
        var contA = 0;
        var contC = 0;
        var contD = 0;
        grupoDeCamposEscolhidos.forEach(function (campoNome) {
            widget.fields.forEach(function (field) {
                if (campoNome == field.name) {
                    if (!field.validations.required)
                        contA += 1;
                    else
                        contC += 1;
                }
            });
        });

        grupoDeCamposEscolhidos.forEach(function (campo) {
            if (Array.isArray(widget.getField(campo).value())) {
                if (widget.getField(campo).value().length > 0) {
                    contB += 1;
                    contem = true;
                }
                else
                    contB -= 1;

                if (widget.getField(campo).value().length == 0)
                    contD += 1;
            } else {
                if (widget.getField(campo).value())
                    contB += 1;
                else
                    contB -= 1;

                if (!widget.getField(campo).value())
                    contD += 1;
            }
        });

        if (contB == 0 || contD == grupoDeCamposEscolhidos.length)
            vazio = false;

        if (contA == grupoDeCamposEscolhidos.length) {
            grupoDeCamposEscolhidos.forEach(function (campo) {
                widget.getField(campo).validations = { "required": true };
            });
            contA = 1;
        }
        if (contC == grupoDeCamposEscolhidos.length && !vazio && !contem) {
            grupoDeCamposEscolhidos.forEach(function (campo) {
                widget.getField(campo).validations = {};
            });
        }
    };
    /**
    * Instrução de Uso:
    * 
    * Seta validation como obrigatorio para os fields que foram escolhidos e passado seus nomes em um Array 
    * Para o objetivo completo desta função é nescessario chamar estes metodo em cada field escolida através 
    * do evento FieldOnChange destes Fields.
    * Para o tipo de campo que esta na Widget verificar se existe campo Select Multiplo e seta na variavel true ou false
    * 
    * @params{Object, Array} widget widget que contem as fields e "grupoDeCamposEscolhidos" um array com os nomes dod fields.
   **/
    var contB = 0;
    this.tornaCamposEscolhidosObrigatorios = function (widget, grupoDeCamposEscolhidos, _is_multSearch) {
        if (_is_multSearch) {
            this.tornaCamposSelectMultipleObrigatorios(widget, grupoDeCamposEscolhidos);
        } else {
            var vazio = true;
            var contA = 0;
            var contC = 0;
            var contD = 0;
            grupoDeCamposEscolhidos.forEach(function (campoNome) {
                widget.fields.forEach(function (field) {
                    if (campoNome == field.name) {
                        if (!field.validations.required)
                            contA += 1;
                        else
                            contC += 1;
                    }
                });
            });

            grupoDeCamposEscolhidos.forEach(function (campo) {
                if (widget.getField(campo).value())
                    contB += 1;
                else
                    contB -= 1;

                if (!widget.getField(campo).value())
                    contD += 1;
            });

            if (contB == 0 || contD == grupoDeCamposEscolhidos.length)
                vazio = false;

            if (contA == grupoDeCamposEscolhidos.length) {
                grupoDeCamposEscolhidos.forEach(function (campo) {
                    widget.getField(campo).validations = { "required": true };
                });
                contA = 1;
            }
            if (contC == grupoDeCamposEscolhidos.length && !vazio) {
                grupoDeCamposEscolhidos.forEach(function (campo) {
                    widget.getField(campo).validations = {};
                });
            }
        }
    };

    /**
   * Instrução de Uso:
   * 
   * verifica se os campos que estão visiveis na widget estão prenchidos para ser ultilizado na validação da ação de save da tela
   * 
   * @params{Object} widget widget que contem as fields para filtrar os campos visiveis
  **/

    this.verificaCamposPrenchidos = function (widget) {
        var retorno = false;
        if (!retorno) {
            widget.fields.forEach(function (field) {
                if (field.isVisible && field.showOnForm) {
                    if (Array.isArray(field.value())) {
                        if (field.value().length > 0)
                            retorno = true;
                    } else
                        if (field.value())
                            retorno = true;
                }
            });
        }
        return retorno;
    };

    this.masterDetailNewRow = function () {
        var widget = templateManager.container.getActiveWidget();

        this.changeWidgetVisibility(widget.widgets, false);

        widget.newRow();
    };

    this.changeWidgetVisibility = function (widgets, isVisible) {
        widgets.forEach(function (widget) {
            widget.isVisible = isVisible;
        });
    };

    this.masterDetailSave = function () {
        var widget = templateManager.container.getActiveWidget();

        this.changeWidgetVisibility(widget.widgets, true);
        this.formChangeTemplate();

        if (widget.isValid()) {
            widget.currentRow.changed = true;

            return widget.saveRow().then(function (response) {
                widget.currentRow.changed = false;
                widget.currentRow.__is_new = false;
                ScreenService.showMessage("Registro salvo com sucesso");
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
            });
        }
    };

    this.formChangeTemplate = function () {
        var widget = templateManager.container.getActiveWidget();

        if ((widget.template != "widget/form.html") && (widget.template != "widget/form-view.html")) return false;

        if (!widget.templateToForm) widget.templateToForm = "widget/form.html";
        if (!widget.templateToView) widget.templateToForm = "widget/form-view.html";

        if (widget.template == widget.templateToForm) widget.template = widget.templateToView;
        else if (widget.template == widget.templateToView) widget.template = widget.templateToForm;

        templateManager.updateTemplate();

        return true;
    };

    this.masterDetailGoBackToList = function () {
        var widget = templateManager.container.getActiveWidget();

        ScreenService.closeSwipe();
        widget.list();
    };

    this.masterDetailCancelForm = function () {
        var widget = templateManager.container.getActiveWidget();

        this.changeWidgetVisibility(widget.widgets, true);

        if (widget.currentRow.__is_new) {
            var indexOfCurrentRow = widget.dataSource.data.indexOf(widget.currentRow);
            if (indexOfCurrentRow !== -1) {
                widget.dataSource.data.pop();
            }
            if (!this.formChangeTemplate()) widget.list();
            else widget.list();

            ScreenService.closeSwipe();
        } else {
            Util.extend(widget.currentRow, widget.backupRow);
            this.formChangeTemplate();
            widget.view();
        }
    };

    this.masterDetailEdit = function () {
        var widget = templateManager.container.getActiveWidget();

        this.changeWidgetVisibility(widget.widgets, false);
        this.formChangeTemplate();

        widget.backupRow = Util.clone(widget.currentRow);
        widget.edit();
    };

    this.masterDetailDelete = function () {
        try {
            var widget = templateManager.container.getActiveWidget();
            ScreenService.confirmMessage("Deseja deletar o registro?", "CONFIRMATION", function () {
                widget.remove();
                this.masterDetailGoBackToList();
                ScreenService.showMessage("Registro excluído com sucesso");
            }.bind(this));
        } catch (e) {
            ScreenService.showMessage(e, 'ERROR');
        }
    };

    /**
     * Aplica o filtro dos valores default na tela 
     * durante a primeira entrada
    **/
    this.filterAfterApply = function(args, fieldNames, filter) {
        if(filter == {} || filter == undefined) {
            if(fieldNames.length <= 0){
                args.owner.fields.forEach(function(field){
                    fieldNames.push(field.name);
                });
            }
            filter = this.getRowDefaultValuesFields(args.owner, fieldNames);
        }

        ZhFilterPreferences.setOriginalFilter(args.owner.id, filter);
        ZhFilterPreferences.applyOriginalFilter(args.owner);
    };

    /**
     * Habilita a field de select multiplo apenas quando 
     * o usuário estiver inserindo um novo registro
     * @param {Object} widget widget que contem a field e a row
     * @param {String} nameFieldMultiple nome da field de select multiple
     * @param {String} nameField nome da field que deverá aparecer no lugar da field de select multiple
    **/
    this.EdicaoFieldSelectMultiple = function (widget, nameFieldMultiple, nameField) {
        if (widget.currentRow.__is_new) {
            widget.getField(nameFieldMultiple).isVisible = true;
            widget.getField(nameFieldMultiple).showOnForm = true;
            widget.getField(nameField).showOnForm = false;
        } else {
            widget.getField(nameFieldMultiple).isVisible = false;
            widget.getField(nameFieldMultiple).showOnForm = false;
            widget.getField(nameField).showOnForm = true;
        }
    };

    /**
     * Substitui a visualização de uma widget por outra
     * @param {Object} widget widget que ficará invisível
     * @param {String} nameWidget nome da widget que ficará visivel
    **/
    this.alteraModoVisualizacaoWdg = function (widget, nameWidget) {
        var wdgAlteracao = templateManager.container.getWidget(nameWidget);
        widget.isVisible = false;
        wdgAlteracao.isVisible = true;
        templateManager.updateTemplate();
        wdgAlteracao.activate();
        wdgAlteracao.reload();
    };

    this.iniciarAutocomplete = function (field) {
        field.initializeField();
    };
    /**
     * Retira os valores repetidos de um array
     * @params{Array} arrayData - array de dados
    **/
    this.retirarValoresDuplicados = function (arrayData) {
        return arrayData.filter(function (value, index) {
            return arrayData.indexOf(value) === index;
        });
    };

    /**
     * Retira a mascara de formatação dos valores 
     * e o inicializa com 0 caso seja null
     * @params{string} vrUnitario - valor a ser formatado 
     * @params{number} casas - casas decimais do numero a ser retornado
    **/
    this.tirarMascaraValores = function(vrUnitario, casas) {
        if (casas == undefined) {
            casas = 2;
        }

        if (!vrUnitario) {
            vrUnitario = 0;
        } else {
            vrUnitario = vrUnitario.toString();
            if (vrUnitario.includes(",")) {
                vrUnitario = vrUnitario.replace(/\./g, "").replace(",", ".");
            }
        }

        if (typeof vrUnitario === 'string' || vrUnitario instanceof String) {
            vrUnitario = parseFloat(vrUnitario);
        }

        return parseFloat(vrUnitario.toFixed(casas));
    };

    this.calcularValoresDesconto = function (widget, nomeCampoValorTotal, nomeCampoPercDesconto, nomeCampoValorDesconto, nomeCampoValorTotalComDesconto, campoAlterado) {
        var valorTotal = this.tirarMascaraValores(widget.currentRow[nomeCampoValorTotal]);
        var valorPercDesconto = this.tirarMascaraValores(widget.currentRow[nomeCampoPercDesconto]);
        var valorDesconto = this.tirarMascaraValores(widget.currentRow[nomeCampoValorDesconto]);
        var valorTotalComDesconto = 0;

        if (valorPercDesconto || valorDesconto) {
            if (campoAlterado == nomeCampoPercDesconto) {
                valorDesconto = valorTotal * valorPercDesconto / 100;
                widget.currentRow[nomeCampoValorDesconto] = valorDesconto.toFixed(2);
                valorTotalComDesconto = valorTotal - valorDesconto;
            }
            else {
                valorPercDesconto = valorDesconto / valorTotal * 100;
                widget.currentRow[nomeCampoPercDesconto] = valorPercDesconto.toFixed(2);
                valorTotalComDesconto = valorTotal - valorDesconto;
            }

            widget.currentRow[nomeCampoValorTotalComDesconto] = valorTotalComDesconto.toFixed(2);
        }
        else {
            widget.currentRow[nomeCampoValorTotalComDesconto] = valorTotal;
        }
    };

    this.widgetCancelForm = function (widget) {
        widget.currentRow.changed = false;
        General.widgetCancelForm(widget);
    };

    /**
     * Seta o idativo no filtro das widgets
     * @params{Object} widget - widget que terá o filtro inicializado
     * @params{bool} idUtilizaDefaultValue - identifica se utilizará o valor do defaulvalue das fields
     * @params{array} nmFields - array de nome das fields que irão compor o filtro
     *  - caso esse parametro venha null e o idUtilizaDefaultValue = true ele usará todas as fields visiveis da widget
    **/
    this.setRowFilterWidgets = function (widget, idUtilizaDefaultValue, nmFields) {
        var currentRow = {};
        if (idUtilizaDefaultValue) {
            if (!nmFields) {
                widget.fields.forEach(function (field) {
                    nmFields.push(field.name);
                });
            }
            currentRow = this.getRowDefaultValuesFields(widget, nmFields);
        }
        ZhFilterPreferences.openFilterWidget(widget, currentRow);
    };

    /**
     * Retorna uma row com todos os valores do defaultvalue das fields descritas que irão aparecer na tela
     * @params{Object} widget - widget solicitante
     * @params{array} nmFields - array com os nomes das fields que irão compor o row
    **/
    this.getRowDefaultValuesFields = function (widget, nmFields) {
        var row = {};
        widget.fields.forEach(function (field) {
            if (nmFields.includes(field.name)) {
                var name = field.name;
                var fieldTemplate = field.template;
                var isVisible = (field.filterProperties && field.filterProperties.isVisible) ? field.filterProperties.isVisible : field.isVisible;
                if (field.filterProperties && isVisible && field.filterProperties.defaultValue) {
                    if (field.filterProperties.name) name = field.filterProperties.name;
                    if (field.filterProperties.template) fieldTemplate = field.filterProperties.template;

                    row[name] = ZhUtil.evaluate(field.filterProperties.defaultValue);
                } else {
                    if (field.isVisible && field.showOnForm && field.defaultValue)
                        row[field.name] = ZhUtil.evaluate(field.defaultValue);
                }

                if (fieldTemplate === "field/select-multiple.html" && !Array.isArray(row[name]))
                    row[name] = [row[name]];
            }
        });
        return row;
    };
    /**
     * Recebe um número de qualquer tipo e retorna uma string do mesmo 
     * formatado
    **/
    this.formatarNumero = function (numero) {
        return parseFloat(numero).toLocaleString('pt-BR', { minimumFractionDigits: 2 });
    };
    /**
     * Retorna o tipo de um arquivo
     * @params{str} type - extensao do arquivo
    **/
    this.getMimeTypesFile = function (type) {
        var extencoes = {
            ".pdf": "application/pdf",
            ".xlsx": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
            ".xls": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
            ".doc": "application/msword",
            ".docx": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
            ".png": "image/png",
            ".jpg": "image/jpeg",
            ".zip": "application/zip"
        };

        if (extencoes["." + type.replace(".", "")])
            return extencoes["." + type.replace(".", "")];

        return type;
    };

    /**
     * Faz o download de um arquivo salvo no servidor 
     * @params{string} nmarquivo - nome do arquivo
     * @params{string} dscaminho - caminho do arquivo no fileServer
     * @params{string} dstipo - tipo do arquivo
    **/
    this.downloadFile = function (nmarquivo, dscaminho, dstipo) {
        var params = {
            "DSCAMINHOARQ": dscaminho,
            "NMARQUIVO": nmarquivo,
            "DSTIPO": this.getMimeTypesFile(dstipo)
        };

        General.callRestEngine("product#/documentos/getUrlAnexos", "Row", params).then(function (success) {
            var file = success.dataset.URLDOCUMENTO;
            window.open(window.backendIndexUrl + file);
            ScreenService.closePopup();
        }).catch(function (error) {
            this.showErrorMessage();
        }.bind(this));
    };
    /**
     * Adiciona valores a row do filtro sem mante-los como default nas fields
     * @params{Object} widget     - widget que terá o filtro inicializado
     * @params{array} nmFields    - array de nome das fields que irão compor o filtro
     * @params{array} valueFields - valores que serão aplicados nas rows
    **/
    this.setRowFilterWidgetSemDefaultValue = function (widget, nameFields, valueFields) {
        var currentRow = {};
        for (var i = 0; i < nameFields.length; i++) {
            if (valueFields.length >= i)
                currentRow[nameFields[i]] = valueFields[i];
        }

        return ZhFilterPreferences.openFilterWidget(widget, currentRow);
    };

    /**
     * Retorna um período de doze meses anteriores a data atual
    **/
    this.getUltimosDozeMesesPeriodoInclusivo = function () {
        var dtAtual = moment().endOf('month');
        var periodo = {};

        periodo.end = dtAtual.format('DD/MM/YYYY');
        periodo.start = dtAtual.subtract(1, 'year').format('DD/MM/YYYY');
        return periodo;
    };

    /**
     * Retorna um período de doze meses anteriores a data atual
    **/
    this.getUltimosDozeMesesPeriodoExclusivo = function () {
        var dtAtual = moment().subtract(1, 'months').endOf('month');
        var periodo = {};

        periodo.end = dtAtual.format('DD/MM/YYYY');
        periodo.start = dtAtual.subtract(1, 'year').format('DD/MM/YYYY');
        return periodo;
    };

    /**
     * Valida se o percetual fornecido está entre 0 e 100
     * 
     * @params{Object} widget - widget que contem o campo de percentual
     * @params{String} nmCampoPercentual - nome do campo de percentual
    **/
    this.validarPercentual = function (widget, nmCampoPercentual) {
        if (widget.currentRow[nmCampoPercentual]) {
            var percentual = this.tirarMascaraValores(widget.currentRow[nmCampoPercentual]);
            if (percentual < 0 || percentual > 100) {
                ScreenService.showMessage("O percentual informado deve ser um número entre 0 e 100.");
                widget.currentRow[nmCampoPercentual] = 0;
            }
        }
    };

    /**
     * Retorna um período de 7 dias anteriores a data atual
    **/
    this.getUltimaSemana = function () {
        var dtAtual = moment().subtract(1, 'months').endOf('month');
        var periodo = {};

        periodo.end = dtAtual.format('DD/MM/YYYY');
        periodo.start = dtAtual.subtract(7, 'days').format('DD/MM/YYYY');
        return periodo;
    };

    /**
     * Retorna um período de 1 ano atrás até ontem.
     */
    this.getUltimoAnoAteOntem = function () {
        var dtFinal = moment().subtract(1, 'days');
        var periodo = {};

        periodo.end = dtFinal.format('DD/MM/YYYY');
        periodo.start = dtFinal.subtract(1, 'year').add(1, 'days').format('DD/MM/YYYY');
        return periodo;
    };

    /**
     * Compara se 2 arrays são iguais.
     * @returns {boolean}
     */
    this.arraysSaoIguais = function (arr1, arr2) {
        // if the other array is a falsy value, return
        if (!arr2)
            return false;

        // compare lengths - can save a lot of time 
        if (arr1.length != arr2.length)
            return false;

        for (var i = 0, l = arr1.length; i < l; i++) {
            // Check if we have nested arrays
            if (arr1[i] instanceof Array && arr2[i] instanceof Array) {
                // recurse into the nested arrays
                if (!arr1[i].equals(arr2[i]))
                    return false;
            }
            else if (arr1[i] != arr2[i]) {
                // Warning - two different object instances will never be equal: {x:20} != {x:20}
                return false;
            }
        }
        return true;
    };

    this.getDiasEntreDatas = function (dataIni, dataFin) {
        var prazo = null;
        if (dataIni.length > 0 && dataFin.length > 0) {
            dataIni = moment(dataIni, "DD/MM/YYYY");
            dataFin = moment(dataFin, "DD/MM/YYYY");
            prazo = dataFin.diff(dataIni, "days");
        }
        return prazo;
    };

    this.limparChangedDataSource = function (widget) {
        widget.dataSource.data.forEach(function (row) {
            row.changed = false;
        });
    };

    this.widgetNewRow = function (widget) {
        this.limparChangedDataSource(widget);
        General.widgetNewRow(widget);
    };


    this.widgetEdit = function (widget) {
        this.limparChangedDataSource(widget);
        General.widgetEdit(widget);
    };

    /**
     * Limpa o campo de data
     * 
     * @param {string} no do campo, mesmo nome presente no current row.
    */
    this.clearCalendarField = function(nomeDoCampo){
        setTimeout(function(){
            var campo = document.querySelector('.zh-field-' + nomeDoCampo);
            if(campo){
                var filhos = campo.children;
                if (filhos) {
                    var fechar = filhos[1].children[0];
                    if (fechar) {
                        fechar.click();
                    }
                }
            }
        }, 250);

    };

    /**
     * Retorna a diferença em dias entre 2 datas.
     * @param {string} dataini 
     * @param {string} datafin 
     * @returns int
     */
    this.calculaDiferencaDatasEmDias = function (dataini, datafin) {
        if (dataini && datafin) {
            dataini = moment(dataini.split(" ")[0], "DD/MM/YYYY");
            datafin = moment(datafin.split(" ")[0], "DD/MM/YYYY");

            return datafin.diff(dataini, "days");
        }
        return null;
    };




    /*
    Opção de salvar a tarefa no google agenda 
    link de modelo:
    https://calendar.google.com/calendar/u/0/r/eventedit?text=Example+Google+Calendar+Event&details=More+help+see:+https://support.google.com/calendar/thread/81344786&dates=20201231T160000/20201231T170000&recur=RRULE:FREQ%3DWEEKLY;UNTIL%3D20210603&ctz=America/Toronto
    */
    this.SaveOnGoogleAgenda = function(params) {
        var title = params.title || "Sem título";
        var endDate = params.endDate || null;
        var description = params.description || null;

        ScreenService.confirmMessage('Deseja salvar a terefa no Google Agendas?', 'QUESTION', function (yes) {
            var url = this.GenerateLinkGoogleAgenda(title, endDate, description);
            console.log(url);
            window.open(url, '_blank');
        }.bind(this));
    }    

    this.GenerateLinkGoogleAgenda = function (title, endDate, description){

        //Pega o timezone do navegador
        var timeZone = Intl.DateTimeFormat().resolvedOptions().timeZone;
        var startDate = this.formatDateGoogle(new Date().toLocaleString().replace(',',''));
        
        var params = {};
        // params.text = widget.currentRow.DSFILACONTATO+" - "+widget.currentRow.NMPARCNEGOCIO.substring(0, 40);

        // Definir os parâmetros
        params.text = title || "Sem título"; // Default se não houver título
        if (description) params.details = description;
        if (timeZone) params.ctz = timeZone;
                
        if (endDate) {
            var endDateFormatted = this.formatDateGoogle(endDate);
            params.dates = startDate + "/" + endDateFormatted;
        } else {
            params.dates = startDate + "/" + startDate;
        }

        var queryString = new URLSearchParams(params).toString();
        var link = "https://calendar.google.com/calendar/u/0/r/eventedit?" + queryString;

        return link;
    }

    // Formata a data para o padrão do Google
    this.formatDateGoogle = function (date){

        // Dividir a data e a hora
        var parts = date.split(" ");
        var datePart = parts[0];
        var timePart = parts[1];

        // Dividir a data no formato DD/MM/YYYY
        var dateComponents = datePart.split("/");
        var day = dateComponents[0];
        var month = dateComponents[1];
        var year = dateComponents[2];

        // Remover os dois pontos da hora (HH:mm:ss)
        var formattedTime = timePart.replace(/:/g, "");

        // Retornar no formato YYYYMMDDTHHmmss
        return year + month + day + "T" + formattedTime;
    }

}

Configuration(function (ContextRegister) {
    ContextRegister.register("Utilities", Utilities);
});


// FILE: ./assets/js/util/UtilitiesMetas.js
function UtilitiesMetas(General, ScreenService, Utilities, templateManager){
    this.corSituacaoMeta = function (element, row) {
        if(row.REALIZADO < row.VRMETATPPERFIL){
            element.find('span').css('color', '#FF0000');
        } else {
            element.find('span').css('color', '#000');
        }
    }

    /**
     * Retorna a quantidade de dias de acordo 
     * com o código do período fixo
     * @param {Object} widget widget tabbed principal
     **/
    this.getDiasPeriodo = function(widget){
        switch (widget.currentRow.CDPERIODOFIXO) {
            case '2': return 7;
            case '3': return 15;
            case '4': return 30;
            case '5': return 90;
            case '6': return 180;
            case '7': return 365;
            default: return 1;
        }
    } 
    
    /**
     * Calcula a quantidade de períodos há a partir 
     * das datas passadas pelo usuário
     * @param {Object} widget widget tabbed principal
     **/
    this.calculaQtPeriodos = function (widget){
        var periodoIni = widget.currentRow.PERIODO.start ? moment(widget.currentRow.PERIODO.start, "DD/MM/YYYY") : moment().subtract(30, "days");
        var periodoFin = widget.currentRow.PERIODO.end ? moment(widget.currentRow.PERIODO.end, "DD/MM/YYYY") : moment();
        var dias       = periodoFin.diff(periodoIni, "days");
        var qtperiodos = 1;
        if(widget.currentRow.TPPERIODOMETA === "1")
            qtperiodos =  dias/this.getDiasPeriodo(widget);
        else 
            qtperiodos = dias/widget.currentRow.QTDIASCORRIDOS;
        if(qtperiodos < 1)
            return 1;
        else 
            return parseInt(qtperiodos);
    }

    /**
     * Coloca os valores do filtro nas widgets
     **/
    this.filtraDadosTela = function(){
        var wdgPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        var filter = [];

        if(wdgPrincipal.currentRow.NRPARAMSMETAS){
            if(!wdgPrincipal.currentRow.PERIODO && wdgPrincipal.currentRow.NRPARAMSMETAS){
                wdgPrincipal.currentRow.PERIODO = {
                    "end"   : moment().endOf('month').format('DD/MM/YYYY'),
                    "start" : moment().startOf('month').format('DD/MM/YYYY'),
                    "period": ""
                };
            }

            var qtperiodo = this.calculaQtPeriodos(wdgPrincipal);
            filter = [
                {
                    name: "NRPARAMSMETAS",
                    operator: "=",
                    value: wdgPrincipal.currentRow.NRPARAMSMETAS
                },
                {
                    name: "PERIODOINI",
                    operator: wdgPrincipal.currentRow.PERIODOINI ? "=" : "IS_NULL",
                    value: wdgPrincipal.currentRow.PERIODO.start 
                },
                {
                    name: "PERIODOFIN",
                    operator: wdgPrincipal.currentRow.PERIODOFIN ? "=" : "IS_NULL",
                    value: wdgPrincipal.currentRow.PERIODO.end
                },
                {
                    name: "QTPERIODOS",
                    operator: "=",
                    value: qtperiodo
                }
            ];
        }

        wdgPrincipal.widgets.forEach(function(widget){
            widget.dataSourceFilter = [widget.dataSourceFilter[0], widget.dataSourceFilter[1]];
            widget.dataSourceFilter = widget.dataSourceFilter.concat(filter);
        });
    };

    /**
     * Valida as datas do período buscado
     * @param {Object} widget widget tabbed principal
     * @param {Object} field field de data que será validada
    **/
    this.validarData = function(widget){
        if(widget.currentRow.PERIODO){
            if(moment(widget.currentRow.PERIODO.end, "DD/MM/YYYY").diff(moment(widget.currentRow.PERIODO.start, "DD/MM/YYYY")) < 0){
                ScreenService.showMessage(ScreenService.i18n("Operação bloqueada.<br>A data final do período precisa ser maior do que a data inicial.")).then(function(){
                    widget.currentRow[field.name] = "";
                });
            }
        }
    }

    this.alteraViewValores = function(wdgTabbed){
        wdgTabbed.widgets.forEach(function(widget){
            if(widget.template === 'zh-charts#widget/zh-chart.html'){
                widget.chartSeries[0].serieYAxisFormat = (wdgTabbed.currentRow.TPMEDIDAMETA === '1') ? "currency" : "number"; 
                if(widget.isActive())
                    widget.updateChart();
            } else {
                widget.getField("REALIZADO").mask.params.showSymbol  = (wdgTabbed.currentRow.TPMEDIDAMETA === '2') ? false : "R$";
                widget.getField("DIFERENCA").mask.params.showSymbol  = (wdgTabbed.currentRow.TPMEDIDAMETA === '2') ? false : "R$";
                widget.getField("VRMETATPPERFIL").mask.params.showSymbol  = (wdgTabbed.currentRow.TPMEDIDAMETA === '2') ? false : "R$";
            }
        });        
        templateManager.container.getActiveWidget().reload();
    }
}

Configuration(function(ContextRegister){
    ContextRegister.register('UtilitiesMetas', UtilitiesMetas);
});

// FILE: ./assets/js/util/UtilitiesPropostaContrato.js
/**
 * Utilities de funções iguais entre o contrato e a proposta
**/
function UtilitiesPropostaContrato(ScreenService, templateManager, General, Utilities, ZHPromise) {
        
    /**
     * Abre a widget que permite criar um novo serviço durante a 
     * seleceção do serviço para o item de formação de preço 
     * na proposta ou no contrato
     * @params{String} wdgName nome da widget que abrirá o popup
    **/
    this.criarNovoServicoSelecao = function(wdgName){
        ScreenService.closePopup();
        var widServico       = templateManager.container.getWidget(wdgName);
        widServico.isVisible = true;
        ScreenService.openPopup(widServico).then(function(){
            widServico.edit();
            widServico.newRow();
            widServico.activate();
        });
    };

    /**
     * Salva o novo serviço inserido durante a busca 
     * na proposta ou no contrato
     * @params{Object} widget widget popup servico proposta
    **/
    this.saveService = function(widget){
        var servico = widget.currentRow.NMSERVORCA;
        var widgetItens = templateManager.container.getWidget("itemFormacaoPreco");
        widget.currentRow.__columns_to_return = ["CDSERVORCA", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if(response.dataset){
                widgetItens.currentRow.CDSERVORCA = response.dataset.columnsToReturn[0].CDSERVORCA;
                widgetItens.currentRow.NMSERVORCA = servico;
            }

            widget.isVisible = false;
            ScreenService.closePopup();
            widgetItens.activate();
        }.bind(this))
        .catch(function(error){
            console.log(error);
        });
    };
    
    /**
     * Habilita a widget para permitir a edição de um item do cálculo de 
     * formção de preço
     * @params{Object} widget widget editar calculo de formação de preço
    **/
    this.editarItensFormacaoPrec = function(widget){
        if(!widget.currentRow.IDALTERAFORMPREC || widget.currentRow.IDALTERAFORMPREC === "N"){
            ScreenService.showMessage("Elemento não pode ser alterado.<br>");
        } else {
            var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
    
            widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
            widget.edit();
            widget.template = "widget/form.html";
        }
    };
    
        
    /**
     * Volta para a widget de item de formação de preço
     * @params{Object} widget widget editar cálculo de formação de preço
     * @params{String} actionName nome da action que está sendo utilizada
    **/
    this.voltarItensFormacaoPrec = function(widget, actionName){
        if(actionName === "VoltarWdgFormPrec"){
            var tabbedFormacao = templateManager.container.getWidget("wdgTabbedPrincipalFormPreco");
            var itemFormacao   = templateManager.container.getWidget("itemFormacaoPreco");

            tabbedFormacao.isVisible = false;
            widget.isVisible = false;
            widget.dataSource.clearCheckedRows();
            widget.dataSource.data = [];
   
            this.habilitaEdicaoWidgetItemFormacao(itemFormacao, false);
            ScreenService.closeSwipe().then(function(){
                itemFormacao.activate();
            });  
        } else {
            if(actionName === "voltar_grid_arvore"){
                widget.template = "widget/tree/grid.html";
                widget.list();
            } else {
                if(!widget.currentRow.__is_new) {
                    Object.keys(widget.backupRow).forEach(function(key) {
                        widget.currentRow[key] = widget.backupRow[key];
                    });
                    
                    widget.changed  = false;
                    widget.template = "widget/form-view.html";
                    widget.view();
                } else {
                    if(widget.dataSource.data.indexOf(widget.currentRow) !== -1)
                        widget.dataSource.data.pop();
                    
                    widget.template = "widget/tree/grid.html";
                    widget.list();
                }
            }
        }
    };
    
    /**
     * Apaga um item de formação de preço da árvore auxiliar
     * @params{Object} widget widget editar formação de preço
    **/
    this.excluiItensArvore = function(widget){
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
            if(widget.dataSource.name === "product#/fct_calcformprecserv_widget"){
                var index = widget.dataSource.data.indexOf(widget.currentRow);
                widget.dataSource.data.splice(index, 1);
                this.voltarItensFormacaoPrec(widget, "voltar_grid_arvore");
            } else {
                widget.removeRow().then(function() {
                    widget.template = "widget/tree/grid.html";
                    widget.list();

                    var params = {};
                    if(widget.dataSource.name === "product#/crud/fct_calcformprecserv_itpro_widget")
                        params = {
                            "NRFORMPRECOSERVICO" : widget.currentRow.NRFORMPRECOSERVICO,
                            "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO",
                            "NRITEMORCAMENTO"    :  null
                        };
                    else 
                        params = {
                            "NRFORMPRECOSERVICO" : widget.currentRow.NRFORMPRECOSERVICO,
                            "TABELA"             : "FCT_CALCFORMPRECSERV_ITCONT",
                            "NRITEMOCONTRATO"    :  null
                        };
                    General.callRestEngine("product#/fct_calcformprecserv_widget/recalcularArvore", "Row", params).then(function(){
                        Utilities.abreArvoreElementos(widget);
                    }.bind(this))
                    .catch(function(){
                        ScreenService.showMessage("Falha ao recalcular o valor da Formação de Preço.");
                    });
                });
            }
        }.bind(this), 
        function (error) {
			Utilities.showErrorMessage(error);
        });
    };
    
    /**
     * habilita o modo view da widget sem abrim um novo swipe
     * @params{Object} widget widget editar itens formação
    **/
    this.habilitarViewItensFormacao = function(widget) {
        widget.template = "widget/form-view.html";
        widget.view();
        widget.activate();
    };
    
    /**
     * Seleciona todos os dados da árvore para salvá-los
     * @params{Object} widget widget editar formação de preço
     * @params{Array} rows array de rows selecionadas
    **/
    this.buscaArvoreFormacaoCompleta = function(widget, rows){
        widget.dataSource.data.forEach(function(data){
            rows.filter(function(row){
                if(row === data._parent) {
                    if (rows.indexOf(data) === -1) {
                        rows.push(data);
                    }
                }
            });
        });
        
        return rows;
    };
    
    this.calculaItem = function(widget, tableName, nritemorcamento, nritemocontrato){
        var defered = ZHPromise.defer();
        var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
        if (tipoCalculo == 'E') 
            this.calculaExpressao(widget, tableName, nritemorcamento, nritemocontrato).then(function(){
                this.calculaValorTotal(widget);
                defered.resolve();
            }.bind(this));
        else 
            if (tipoCalculo == 'S'){
                defered.resolve();
            } else {
                this.calculaValorTotal(widget);
                defered.resolve();
            }
        return defered;
    };

    /**
     * Realiza o cálculo descrito no campo "Expressão" e retorna para o campo "Valor Unitário"
     * 
     * @param {object} widget widget
     */
    this.calculaExpressao = function(widget, tableName, nritemorcamento, nritemocontrato){
        var defered = ZHPromise.defer();
        var NrFormPrecoServico = widget.currentRow.NRFORMPRECOSERVICO;
        var expressao = widget.currentRow.DSEXPRESSAOCALC;
        var percentual = widget.currentRow.VRPERCCALCULO;
        var vrunit = 0;
        var params = {
            "NRFORMPRECOSERVICO" : NrFormPrecoServico,
            "TABELA"             : tableName,
            "NRITEMORCAMENTO"    : nritemorcamento,
            "NRITEMOCONTRATO"    : nritemocontrato
        };
        General.callRestEngine('product#/fct_calcformprecserv_widget/buscaIdentValores', "Row", params).then(function(response){
            var identVal = response.dataset.IDENTVAL;
            identVal.forEach(function(identfica){
                var identificador = '['+identfica.IDITEMCALCULO+']';
                var valor = identfica.VRTOTAL;
                expressao = expressao.split(identificador).join(valor);    
            });
            expressao = expressao.split("%").join("/100");
            if (percentual != null){
               vrunit = eval(expressao);
               vrunit = (vrunit*percentual)/100;
            } else {
                vrunit = eval(expressao);
            }
            widget.currentRow.VRUNITARIO =  vrunit;
            defered.resolve();
        }, function(error) {
            Utilities.showErrorMessage(error);
            defered.reject();
        });
        return defered;
    };

    /**
     * Realiza o cálculo do Valor Total
     * 
     * @param {object} widget widget
     */
     
    this.calculaValorTotal = function(widget){
        var vrUnit = widget.currentRow.VRUNITARIO;
        if (vrUnit) {
            vrUnit = vrUnit.toString();
            var primeiraPos = vrUnit.indexOf(".");
            var ultimaPos = vrUnit.lastIndexOf(".");
            if (primeiraPos == ultimaPos) {
                if (vrUnit.includes(",")) {
                    vrUnit = parseFloat(vrUnit.replace(".", "").replace(",", "."));
                } else {
                    vrUnit = parseFloat(vrUnit.replace(",", "."));
                }
            } else {
                var parteInicial = vrUnit.substr(0, primeiraPos + 1);
                var parteFinal = vrUnit.substr(primeiraPos + 1);
                vrUnit = parseFloat(parteInicial.replace(".", "") + parteFinal);
            }
        } else {
            vrUnit = 0;
        }
        var qtCalc    = widget.currentRow.QTCALCULO;
        var usamedida = widget.currentRow.IDUSAMEDIDA;
        var qtmedida  = widget.currentRow.QTMEDIDACALC;
        var vrTot     = 0;
        if (usamedida == 'S'){
            vrTot = (vrUnit*qtCalc)*qtmedida ; 
        } else {
            vrTot = vrUnit*qtCalc;
        }
        widget.currentRow.VRTOTAL = vrTot;
    };
    
    /**
     * Valida se os escopos escolhidos possuem os dados necessários 
     * a geração da negociação de valores da proposta.
     * @params{Object} widget - widget de proposta
     * @param{String} nmFieldEscopo - nome do campo de escopo
     * @params{String} tipoDeEscopo - tipo de escopo (ORC, CONT)
    **/
    this.validarParamsFaturamentoEscopo = function(widget, nmFieldEscopo, tipoDeEscopo) {
        var nrtipoorcamento  = (tipoDeEscopo == "ORC") ? "NRTIPOORCAMENTO" : "NRTIPOORCAMENTO_CONT";
        var nmtipoorcamento  = (tipoDeEscopo == "ORC") ? "NMTIPOORCAMENTO" : "NMTIPOORCAMENTO_CONT";
        var fieldIsArray     = (["NRTIPOORCAMENTO", "NRTIPOORCAMENTO_CONT"].includes(nmFieldEscopo));
 
        if((!fieldIsArray && widget.currentRow[nrtipoorcamento]) || (fieldIsArray && widget.currentRow[nrtipoorcamento].length > 0)) {
            var params = { "NRTIPOORCAMENTO": widget.currentRow[nrtipoorcamento] };
            General.callRestEngine("product#/fct_tipoorcamento_widget/validarParamsFaturamentoEscopo", "Row", params)
            .then().catch(function(error) {
                Utilities.showErrorMessage(error);
                
                widget.currentRow[nrtipoorcamento] = fieldIsArray ? [] : null;
                widget.currentRow[nmtipoorcamento] = fieldIsArray ? [] : null;
                widget.getField(nmFieldEscopo).readOnly = false;
            });
        }
    };
    
    /**
     * valida se o operador logado pode diminuir a quantidade de medida dos itens
    **/
    this.validarPermissaoAlteracaoQtMedida = function() {
        var deferred = ZHPromise.defer();
        General.callRestEngine("product#/fct_orcamento_widget/validarPermissaoAlteracaoQtMedida", "Row", {}).then(function(response) {
            var permiteAlteracao = (response && response.dataset && response.dataset.fct_orcamento_widget.PERMITEDIMINUICAO);
            deferred.resolve(permiteAlteracao);
        }).catch(function(error) {
            Utilities.showErrorMessage(error);
            deferred.reject();
        });
        
        return deferred.promise;
    };

    this.setarMedidaVariavelEscopo = function(wdgItens, propostaOuContrato) {
        var nrtipoorcamento = (propostaOuContrato == "P") ? wdgItens.currentRow.NRTIPOORCAMENTO : templateManager.container.getWidget("gridContrato").currentRow.NRTIPOORCAMENTO;
        var tipoprodserv =  (propostaOuContrato == "P") ? wdgItens.currentRow.TIPOPRODSERV : wdgItens.currentRow.IDPRODSERV;
        var cdprodserv = wdgItens.currentRow.CDPRODSERV;
        var nomeFieldQtdMedida = (propostaOuContrato == "P") ? "QTMEDIDAORCA" : "QTMEDIDACONTR";

        if (nrtipoorcamento && tipoprodserv == "S" && cdprodserv) {
            var params = {
                "NRTIPOORCAMENTO": nrtipoorcamento,
                "CDSERVORCA"     : cdprodserv
            };

            General.callRestEngine("product#/fct_itemorcamento_widget/getMedidaVariavelEscopo", "Row", params).then(function(response) {
                if (response.dataset && response.dataset.retorno) {
                    wdgItens.currentRow.CDTPMEDIDASERV                = response.dataset.retorno.CDTPMEDIDAVARIAVEL;
                    wdgItens.currentRow.CDTPMEDIDAVARIAVEL            = response.dataset.retorno.CDTPMEDIDAVARIAVEL;
                    wdgItens.currentRow.DSTPMEDIDASERV                = response.dataset.retorno.DSTPMEDIDAVARIAVEL;
                    wdgItens.currentRow.IDMULTMEDVRTOT                = "S";
                    wdgItens.getField("DSTPMEDIDASERV").readOnly      = true;
                    wdgItens.getField("DSTPMEDIDASERV").validations   = {required: true};
                    wdgItens.getField("IDMULTMEDVRTOT").readOnly      = true;
                    wdgItens.getField(nomeFieldQtdMedida).readOnly    = false;
                    wdgItens.getField(nomeFieldQtdMedida).validations = {required: true};
                    wdgItens.getField(nomeFieldQtdMedida).validations = {required: true};
                    wdgItens.currentRow.INSERIR_MEDIDA                = true;
                } else {
                    wdgItens.currentRow.CDTPMEDIDASERV           = "";
                    wdgItens.currentRow.CDTPMEDIDAVARIAVEL       = "";
                    wdgItens.currentRow.DSTPMEDIDASERV           = "";
                    wdgItens.getField("DSTPMEDIDASERV").readOnly = false;
                    wdgItens.getField("IDMULTMEDVRTOT").readOnly = false;
                    wdgItens.currentRow.INSERIR_MEDIDA           = false;
                }
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
        else {
            wdgItens.currentRow.CDTPMEDIDASERV = "";
            wdgItens.currentRow.CDTPMEDIDAVARIAVEL = "";
            wdgItens.currentRow.DSTPMEDIDASERV = "";
            wdgItens.getField("DSTPMEDIDASERV").readOnly = false;
            wdgItens.getField("IDMULTMEDVRTOT").readOnly = false;
            wdgItens.currentRow.INSERIR_MEDIDA = false;
        }
    };
    
    /**
     * Controla o readOnly das fields na widget de itens de formação de preço
     * para manter a coerencia durante a edição 
     * @params{Object} widget - widget de itens de formação
     * @params{bool} readOnly - se o campo será habilitado ou não
    **/
    this.habilitaEdicaoWidgetItemFormacao = function(widget, readOnly) {
        widget.fields.forEach(function(field) {
            if(field.name != "NMTIPOORCAMENTO")
                field.readOnly = readOnly;
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("UtilitiesPropostaContrato", UtilitiesPropostaContrato);
});

// FILE: ./assets/js/util/ValidacoesParametrosFaturamentoUtil.js
function ValidacoesParametrosFaturamentoUtil(General, ZHPromise, Utilities, ScreenService, templateManager) {
    
    this.validarSePeriodoDeCompetenciaExiste = function(widget, nomeTabela, nomeCampoPK, nomeCampoPai) {
        var deferred = ZHPromise.defer();
        
        var valorCampoPK = widget.currentRow[nomeCampoPK];
        if (Util.isEmptyOrBlank(valorCampoPK))
            valorCampoPK = -1;
        
        var valorCampoPai = widget.currentRow[nomeCampoPai];
        
        var params = {
            "NOME_TABELA"    : nomeTabela,
            "NOME_CAMPO_PK"  : nomeCampoPK,
            "VALOR_CAMPO_PK" : valorCampoPK,
            "NOME_CAMPO_PAI" : nomeCampoPai,
            "VALOR_CAMPO_PAI": valorCampoPai,
            "NRDIACOMPETINI" : widget.currentRow.NRDIACOMPETINI,
            "NRDIACOMPETFIN" : widget.currentRow.NRDIACOMPETFIN,
            "IDMESCOMPETINI" : widget.currentRow.IDMESCOMPETINI,
            "IDMESCOMPETFIN" : widget.currentRow.IDMESCOMPETFIN
        };
        
        General.callRestEngine("product#/validacoes_parametros_faturamento/validarSePeriodoDeCompetenciaExiste", "Row", params)
        .then(function(success) {
            deferred.resolve(success.dataset.resultado.existe);
        })
        .catch(function(error) {
            deferred.reject(error);
        });
        
        return deferred.promise;
    };
    
    /* ========================== FUNCOES NOVO FATURAMENTO ========================== */
    
    this.controlarTipoCalculoVencimento = function(wdgParametrosFaturamento) {
        var idtpcalcvenc = wdgParametrosFaturamento.currentRow.IDTPCALCVENC;
        var fieldDiaVencimento = wdgParametrosFaturamento.getField("NRDIACALCVENC");
        var fieldMesCalcVenc = wdgParametrosFaturamento.getField("IDMESCALCVENC");
        var fieldDiasMinimosFatVenc = wdgParametrosFaturamento.getField("QTDIASFATUVENC");
        
        if (idtpcalcvenc == "1") { //Dia fixo para a data de vencimento
            fieldDiaVencimento.label = "Dia de Vencimento";
            
            fieldMesCalcVenc.readOnly = false;
            fieldMesCalcVenc.validations = {"required": true};
            
            fieldDiasMinimosFatVenc.readOnly = false;
            fieldDiasMinimosFatVenc.validations = {"required": true};
        }
        else if (idtpcalcvenc == "2") { //Quantidade de dias após a data de emissão
            fieldDiaVencimento.label = "Quantidade de Dias";
            
            fieldMesCalcVenc.readOnly = true;
            fieldMesCalcVenc.value("");
            fieldMesCalcVenc.validations = {};
            
            fieldDiasMinimosFatVenc.readOnly = true;
            fieldDiasMinimosFatVenc.value("");
            fieldDiasMinimosFatVenc.validations = {};
        }
    };
    
    this.wdgParametrosFaturamentoNewRow = function(wdgParametrosFaturamento, nameWdgCompetencia) {
        General.widgetNewRow(wdgParametrosFaturamento);
        this.controlarTipoCalculoVencimento(wdgParametrosFaturamento);
        
        if(nameWdgCompetencia)
            this.controlarCamposTipoFaturamento(wdgParametrosFaturamento, nameWdgCompetencia);
    };
    
    this.validarPeriodoCompetencia = function(wdgParametrosFaturamento) {
        var diaInicial = wdgParametrosFaturamento.currentRow.NRDIACOMPETINI;
        var diaFinal   = wdgParametrosFaturamento.currentRow.NRDIACOMPETFIN;
        var mesInicial = parseInt(wdgParametrosFaturamento.currentRow.IDMESCOMPETINI);
        var mesFinal   = parseInt(wdgParametrosFaturamento.currentRow.IDMESCOMPETFIN);
        
        if (!Util.isEmptyOrBlank(diaInicial) && !Util.isEmptyOrBlank(diaFinal)) {
            if (mesInicial == mesFinal && diaInicial > diaFinal) {
                ScreenService.showMessage("Operação bloqueada.<br/>O dia da competência inicial deve anterior que o dia final.");
                wdgParametrosFaturamento.currentRow.NRDIACOMPETINI = "";
                wdgParametrosFaturamento.currentRow.NRDIACOMPETFIN = "";
            }
            else if (mesInicial > mesFinal) {
                ScreenService.showMessage("Operação bloqueada.<br/>O dia da competência inicial deve anterior que o dia final.");
                wdgParametrosFaturamento.currentRow.NRDIACOMPETINI = "";
                wdgParametrosFaturamento.currentRow.NRDIACOMPETFIN = "";
            }
        }
    };
    
    this.controlarCamposTipoFaturamento = function(wdgParametrosFaturamento, nameWdgCompetencia) {
        var idtpfatura = wdgParametrosFaturamento.currentRow.IDTPFATURA;
        
        var fieldTipoMedidaVar = wdgParametrosFaturamento.getField("DSTPMEDIDAVARIAVEL");
        if (fieldTipoMedidaVar) {
            if (idtpfatura == "X") {
                fieldTipoMedidaVar.readOnly = false;
            }
            else {
                Object.values(fieldTipoMedidaVar.outData).forEach(function(field) {
                    wdgParametrosFaturamento.currentRow[field] = null;
                });
                fieldTipoMedidaVar.readOnly = true;
            }
        }

        if(nameWdgCompetencia) {
            var wdgCompetencia = templateManager.container.getWidget(nameWdgCompetencia);
            wdgCompetencia.isVisible = (idtpfatura && idtpfatura != "P");
            wdgParametrosFaturamento.getField("QTDIASAPURACAO").isVisible = (idtpfatura == "P");
            wdgParametrosFaturamento.getField("IDTPCALCVENC").isVisible   = (idtpfatura == "P");
            wdgParametrosFaturamento.getField("NRDIACALCVENC").isVisible  = (idtpfatura == "P");
            wdgParametrosFaturamento.getField("IDMESCALCVENC").isVisible  = (idtpfatura == "P");
            wdgParametrosFaturamento.getField("QTDIASFATUVENC").isVisible = (idtpfatura == "P");
            wdgParametrosFaturamento.getField("QTPARCELASFATU").isVisible = (idtpfatura == "P");
            
            templateManager.updateTemplate();
        } else {
            var campoQtdParcelas = wdgParametrosFaturamento.getField("QTPARCELASFATU");
            if (idtpfatura == "P") { //Parcelamento (Quantidade de Parcelas)
                campoQtdParcelas.readOnly = false;
                campoQtdParcelas.validations = {"required": true};
            } else { //Periódico
                campoQtdParcelas.readOnly = true;
                campoQtdParcelas.value("");
                campoQtdParcelas.validations = {};
            }
        }
    };

    this.controlarCamposParcelamentoAutomatico = function(widget) {
        var idTpFatura = widget.currentRow.IDTPFATURA;
        var qtdParcelas = widget.getField("QTPARCELASPARAUTO");
        var qtdDiasIntervalo = widget.getField("QTDIASINTERVALOPARAUTO");
        var diaVencimento = widget.getField("NRDIAVENCIMENTOPARAUTO");
        var qtdCarencia = widget.getField("QTCARENCIADIASPARAUTO");

        if (idTpFatura == "P") {
            qtdParcelas.readOnly = false;
            qtdDiasIntervalo.readOnly = false;
            diaVencimento.readOnly = false;
            qtdCarencia.readOnly = false;
        }
        else {
            qtdParcelas.readOnly = true;
            qtdDiasIntervalo.readOnly = true;
            diaVencimento.readOnly = true;
            qtdCarencia.readOnly = true;
        }
    };
    
    this.wdgCapaParametrosFaturamentoOnEnter = function(wdgCapaParametrosFaturamento, nameWdgCompetencia) {
        wdgCapaParametrosFaturamento.list();
        var actAdicionar = wdgCapaParametrosFaturamento.getAction("add");
        var actEditar    = wdgCapaParametrosFaturamento.getAction("edit");
        var actExcluir   = wdgCapaParametrosFaturamento.getAction("delete");
        
        wdgCapaParametrosFaturamento.reload().then(function() {
            if (wdgCapaParametrosFaturamento.dataSource.data.length > 0) {
                wdgCapaParametrosFaturamento.currentRow = wdgCapaParametrosFaturamento.dataSource.data[0];
                actAdicionar.isVisible = false;
                actEditar.isVisible    = true;
                if (actExcluir) actExcluir.isVisible = true;
            } else {
                wdgCapaParametrosFaturamento.currentRow = {};
                actAdicionar.isVisible = true;
                actEditar.isVisible    = false;
                if (actExcluir) actExcluir.isVisible = false;
            }

            wdgCapaParametrosFaturamento.view();
            if(nameWdgCompetencia){
                this.controlarCamposTipoFaturamento(wdgCapaParametrosFaturamento, nameWdgCompetencia);
                this.controlarTipoCalculoVencimento(wdgCapaParametrosFaturamento);
            }
            
        }.bind(this));
    };
    
    this.wdgCapaParametrosFaturamentoNewRow = function(wdgCapaParametrosFaturamento, nomeWidgetPai, nomeWidgetParamFatTabbed, nameWdgCompetencia) {
        altararAcessibilidadeWidgetsFaturamento(wdgCapaParametrosFaturamento.name, nomeWidgetPai, nomeWidgetParamFatTabbed, false);
        Utilities.formWidgetNewRow(wdgCapaParametrosFaturamento);
        templateManager.container.getWidget(nomeWidgetParamFatTabbed).edit();
        this.controlarCamposTipoFaturamento(wdgCapaParametrosFaturamento, nameWdgCompetencia);
    };
    
    this.wdgCapaParametrosFaturamentoEdit = function(wdgCapaParametrosFaturamento, nomeWidgetPai, nomeWidgetParamFatTabbed, nameWdgCompetencia) {
        altararAcessibilidadeWidgetsFaturamento(wdgCapaParametrosFaturamento.name, nomeWidgetPai, nomeWidgetParamFatTabbed, false);
        Utilities.formWidgetEdit(wdgCapaParametrosFaturamento);
        templateManager.container.getWidget(nomeWidgetParamFatTabbed).edit();
        this.controlarCamposTipoFaturamento(wdgCapaParametrosFaturamento, nameWdgCompetencia);
    };
    
    this.wdgCapaParametrosFaturamentoCancel = function(wdgCapaParametrosFaturamento, nomeWidgetPai, nomeWidgetParamFatTabbed) {
        altararAcessibilidadeWidgetsFaturamento(wdgCapaParametrosFaturamento.name, nomeWidgetPai, nomeWidgetParamFatTabbed, true);
        Utilities.formWidgetCancel(wdgCapaParametrosFaturamento);
        templateManager.container.getWidget(nomeWidgetParamFatTabbed).list();
    };
    
    this.wdgCapaParametrosFaturamentoDelete = function(widget, wdgNameWidgetTabbed) {
		ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
			widget.removeRow().then(function () {
				this.wdgCapaParametrosFaturamentoOnEnter(widget, wdgNameWidgetTabbed);
			}.bind(this),
			function (error) {
				if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
					ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
				}
				else {
					Utilities.showErrorMessage(error);
				}
			});
		}.bind(this));
    };
    
    this.wdgCapaParametrosFaturamentoSave = function(wdgCapaParametrosFaturamento, nomeWidgetPai, nomeWidgetParamFatTabbed, nameWidgetCompetencia) {
        Utilities.formWidgetSaveAndSetColumnsToReturn(wdgCapaParametrosFaturamento).then(function() {
            altararAcessibilidadeWidgetsFaturamento(wdgCapaParametrosFaturamento.name, nomeWidgetPai, nomeWidgetParamFatTabbed, true);
            this.wdgCapaParametrosFaturamentoOnEnter(wdgCapaParametrosFaturamento, nameWidgetCompetencia);
            templateManager.container.getWidget(nomeWidgetParamFatTabbed).list();
        }.bind(this));
    };
    
    var altararAcessibilidadeWidgetsFaturamento = function(nomeWidgetCapa, nomeWidgetPai, nomeWidgetParamFatTabbed, visibilidade) {
        var wdgModeloContrato = templateManager.container.getWidget(nomeWidgetPai);
        
        var widgets = wdgModeloContrato.widgets.filter(function(wid) {
            return wid.name != nomeWidgetParamFatTabbed;
        }).concat(wdgModeloContrato);
        
        var wdgParametrosFaturamentoTabbed = templateManager.container.getWidget(nomeWidgetParamFatTabbed);
        widgets = widgets.concat(wdgParametrosFaturamentoTabbed.widgets.filter(function(wid) {
            return wid.name != nomeWidgetCapa;
        }));
        
        General.changeWidgetAcessibility(widgets, visibilidade);
    };
    
    this.wdgParametrosFaturamentoSave = function(wdgParametrosFaturamento, nomeCampoPai) {
        if (wdgParametrosFaturamento.isValid()) {
            var nomeTabela = wdgParametrosFaturamento.paramsNewCode.__new_code_pk;
            var nomeCampoPK = wdgParametrosFaturamento.paramsNewCode.__table_pk;
            
            this.validarSePeriodoDeCompetenciaExiste(wdgParametrosFaturamento, nomeTabela, nomeCampoPK, nomeCampoPai)
            .then(function(existe) {
                if (existe === false) {
                    Utilities.widgetSaveAndSetColumnsToReturn(wdgParametrosFaturamento);
                }
                else {
                    Utilities.showErrorMessage("Operação bloqueada. <br/>Já existe um registro para o período de competência informado.");
                }
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("ValidacoesParametrosFaturamentoUtil", ValidacoesParametrosFaturamentoUtil);
});

// FILE: ./assets/js/repositories/ger00004500_Prospect_Repository.js
function ger00004500_Prospect_Repository(restEngine){
    
    this.callInsertTaskProspect = function(paramsRestEngine, currentRow){
        
        return restEngine.post('/insertNextTaskProspect', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
    };
    
    this.selectContactProspect = function(paramsRestEngine, currentRow){
        
        return restEngine.post('/selectProspectNrContato', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
    };
    
    this.requestForForInsertNrseqcontatoForNewOpportunity = function(paramsRestEngine){
        
        return restEngine.post('/insertNrseqcontatoForNewOpportunity', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
        
    };
    
    this.requestForUpdatePartnerType = function(paramsRestEngine){
        
        return restEngine.post('/updatePartnerType', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
        
    };
    
    this.requestForInsertPartnershipOfTypeClient = function(paramsRestEngine){
    
        return restEngine.post('/insertPartnershipOfTypeClient', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
        
    };
    
    this.requestForInsertClient = function(paramsRestEngine){
        
        return restEngine.post('/insertClient', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
        
    };
    
    this.requestForDeactivateProspect = function(paramsRestEngine){
        
        return restEngine.post('/deactivateProspect', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
          
    };
    
    this.requestForInsertClientAddress = function(paramsRestEngine){
    
        return restEngine.post('/insertClientAddress', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
        
    };
    
    this.requestForSelectProspectAddress = function(paramsRestEngine){
      
        return restEngine.post('/selectProspectAddress', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
         
    };
    
    this.requestForSetTaskEndDate = function(paramsRestEngine){
      
        return restEngine.post('/setTaskEndDate/Prospect', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
         
    };
    
    this.requestForDeleteEvolutionFilesByOriginNumber = function(paramsRestEngine){
        
        return restEngine.post('/deleteFilesByOriginNumber', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
        
    };
    
    this.requestForDeactivateTask = function(paramsRestEngine){
      
        return restEngine.post('/deactivateProspectTask', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
         
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("ger00004500_Prospect_Repository", ger00004500_Prospect_Repository);
});


// FILE: ./assets/js/repositories/Operador_Repository.js
var Operador_Repository = function(localStorageEngine) {
    
    this.getOperadorLogado = function() {
        var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }
        ];
        
        return localStorageEngine.download("product#/crud/operador", filter);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Operador_Repository", Operador_Repository);
});

// FILE: ./assets/js/repositories/Sm_Percentual_Desconto_Operador_Repository.js
function Sm_Percentual_Desconto_Operador_Repository(restEngine, ScreenService){
   
    this.validaPercentDescontoOper = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_percdescoper_widget/ValidaPercentDescontoOper', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm_Percentual_Desconto_Operador_Repository", Sm_Percentual_Desconto_Operador_Repository);
});


// FILE: ./assets/js/repositories/Sm00001000_PropostaAlimenticia_Repository.js
var Sm00001000_PropostaAlimenticia_Repository = function(localStorageEngine) {
    /**
     * Salva no localStorage o dataSource de uma tela
     * @params{String} name - nome da widget
     * @params{Array} data - dataSource
    **/
    this.saveDataLocalStorageEngine = function(name, data) {
        localStorageEngine._setItem(name, data);
    };
    /**
     * retorna do localStorage um dataSource de uma tela
     * @params{String} name - nome do data na sessao
    **/
    this.getDataLocalStorageEngine = function(name) {
        return localStorageEngine.getDataSourceItems(name);
    };
    /**
     * deleta no localStorage um dataSource de uma tela
     * @params{String} name - nome do data na sessao
    **/
    this.removeDataLocalStorageEngine = function(name) {
        localStorageEngine._removeAll(name);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00001000_PropostaAlimenticia_Repository", Sm00001000_PropostaAlimenticia_Repository);
});

// FILE: ./assets/js/repositories/Sm00002000_Modelo_Proposta_Repository.js
function Sm00002000_Modelo_Proposta_Repository(restEngine, ScreenService){
   
    /**
     * A função "validaEscopoProposta" verifica se o escopo 
     * que está sendo relacionado já está relacionado ao modelo.
     */
    
    this.validaEscopoProposta = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_relacescopomodelo_widget/validaEscopoProposta', paramsRestEngine);
        
    };
    
    /**
     * A função "validaCompProposta" verifica se a composição 
     * que está sendo relacionada já está relacionada ao modelo.
     */
    
    this.validaCompProposta = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_relorcamodcomposicao_widget/validaCompProposta', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00002000_Modelo_Proposta_Repository", Sm00002000_Modelo_Proposta_Repository);
});


// FILE: ./assets/js/repositories/Sm00003000_Escopo_Proposta_Repository.js
function Sm00003000_Escopo_Proposta_Repository(restEngine, ScreenService){
   
     /**
     * A função "validaServicoTabela" verifica se o serviço 
     * que está sendo relacionado já está relacionado na tabela.
     */
     
    this.validaServicoEscopo = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_itescopoproposta_widget/validaServicoEscopo', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00003000_Escopo_Proposta_Repository", Sm00003000_Escopo_Proposta_Repository);
});


// FILE: ./assets/js/repositories/Sm00004000_Contrato_Repository.js
function Sm00004000_Contrato_Repository(restEngine, ScreenService){
   
     /**
     * A função "copiaVersionaContrato" executa a procedure
     * "FCT_COPIA_VERSIONA_CONTRATO".
     */
     
    this.copiaVersionaContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_contrato_widget/copiaVersionaContrato', paramsRestEngine);
        
    };
    
     /**
     * A função "encerraContrato" executa a procedure
     * "FCT_ENCERRA_CONTRATO".
     */
     
    this.encerraContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_contrato_widget/encerraContrato', paramsRestEngine);
        
    };
    
    
     /**
     * A função "geraParcelasContrato" executa a procedure
     * "FCT_VENCPREV_CONT".
     */
     
    this.geraParcelasContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_contrato_widget/geraParcelasContrato', paramsRestEngine);
        
    };
    
     /**
     * A função "cancelaParcelasContrato" executa a procedure
     * "FCT_CANC_VENCPREV_CONT".
     */
     
    this.cancelaParcelasContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_contrato_widget/cancelaParcelasContrato', paramsRestEngine);
        
    };
    
    /**
     * A função "alteraVigenciaContrato" altera os dados de vigência do contrato
     */
     
    this.alteraVigenciaContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_prorrogcontrato_widget/alteraVigenciaContrato', paramsRestEngine);
        
    };
    
    
    /**
     * A função "validaDadosProrrog" valida se ja existem alterações de vigência para o contrato
     */
     
    this.validaDadosProrrog = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_prorrogcontrato_widget/validaDadosProrrog', paramsRestEngine);
        
    };
    
    /**
     * A função "alteraDadosProrrog" inativa as alterações de vigência do contrato
     */
     
    this.alteraDadosProrrog = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_prorrogcontrato_widget/alteraDadosProrrog', paramsRestEngine);
        
    };
    
    /**
     * A função "recebeParamFatModCont" recebe os dados de Faturamento do Modelo de Contrato
     */
     
    this.recebeParamFatModCont = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_paramfatmodcont_widget/recebeParamFatModCont', paramsRestEngine);
        
    };
    
    /**
     * A função "recebeModReajustContr" recebe os dados de Reajuste do Modelo de Contrato
     */
     
    this.recebeModReajustContr = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_modreajustcontr_operador/recebeModReajustContr', paramsRestEngine);
        
    };
    
    /**
     * A função "recebeModIndContr" recebe os dados de Indice de Reajuste do Modelo de Contrato
     */
     
    this.recebeModIndContr = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_modindcontr_widget/recebeModIndContr', paramsRestEngine);
        
    };
    
    /**
     * A função "recebeModeloDocContr" recebe os dados de Documento do Modelo de Contrato
     */
     
    this.recebeModeloDocContr = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_modelodoccontr_widget/recebeModeloDocContr', paramsRestEngine);
        
    };
    
    /**
     * A função "geraCodigoContrato" executa a procedure  "CRIA_MNEMONICO".
     */
     
    this.geraCodigoContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_contrato_widget/geraCodigoContrato', paramsRestEngine);
        
    };
    
    /**
     * A função "geraDocumentoContrato" executa a procedure "FCT_SUB_MNE_DOC".
     */
     
    this.geraDocumentoContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_contrato_widget/geraDocumentoContrato', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00004000_Contrato_Repository", Sm00004000_Contrato_Repository);
});


// FILE: ./assets/js/repositories/Sm00004005_Parametrizacao_Contrato_Repository.js
function Sm00004005_Parametrizacao_Contrato_Repository(restEngine, ScreenService){
   
     /**
     * A função "insereDadosContrato" insere os dados de parametização 
     * de fatiuramento na tabela de Contrato (FCT_CONTRATO).
     */
    
    this.insereDadosContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_paramfatcont_widget/insereDadosContrato', paramsRestEngine);
        
    };
    
    
    this.insereDadosItContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_paramfatitcont_widget/atualizarDadosItContrato', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00004005_Parametrizacao_Contrato_Repository", Sm00004005_Parametrizacao_Contrato_Repository);
});


// FILE: ./assets/js/repositories/Sm00006000_Elemento_Repository.js
function Sm00006000_Elemento_Repository(restEngine, ScreenService){
   
    /**
     * A função "validaServicoRelacionado" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço.
     */
    
    this.validaServicoRelacionado = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_relacservico_widget/validaServicoRelacionado', paramsRestEngine);
        
    };
    
     /**
     * A função "validaServicoRelacionadoUnit" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço para inclusão 
     * de um só item.
     */
    
    this.validaServicoRelacionadoUnit = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_relacservico_widget/validaServicoRelacionadoUnit', paramsRestEngine);
        
    };
    
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006000_Elemento_Repository", Sm00006000_Elemento_Repository);
});


// FILE: ./assets/js/repositories/Sm00006000_Servicos_Repository.js
function Sm00006000_Servicos_Repository(restEngine, ScreenService){
   
     /**
     * A função "validaServicoRelacionado" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço.
     */
     
    this.validaServicoRelacionado = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_relacservico_widget/validaServicoRelacionado', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006000_Servicos_Repository", Sm00006000_Servicos_Repository);
});


// FILE: ./assets/js/repositories/Sm00006001_Tabela_Preco_Servico_Repository.js
function Sm00006001_Tabela_Preco_Servico_Repository(restEngine, ScreenService){
   
     /**
     * A função "validaServicoTabela" verifica se o serviço 
     * que está sendo relacionado já está relacionado na tabela.
     */
     
    this.validaServicoTabela = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_itprecbaseserv_widget/validaServicoTabela', paramsRestEngine);
        
    };
    
      /**
     * A função "validaTipoMedidaVariavel" verifica se o tipo de medida variável
     * ja está criado para a faixa existente.
     */
     
    this.validaTipoMedidaVariavel = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_variacaoprecbaseserv_widget/validaTipoMedidaVariavel', paramsRestEngine);
        
    };
    
     /**
     * A função "sincronizarVariacaoPreco" executa a procedure "FCT_SINCRONIZAR_VARIACAO_PRECO".
     */
     
    this.sincronizarVariacaoPreco = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_itprecbaseserv_widget/sincronizarVariacaoPreco', paramsRestEngine);
        
    };
    
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006001_Tabela_Preco_Servico_Repository", Sm00006001_Tabela_Preco_Servico_Repository);
});


// FILE: ./assets/js/repositories/Sm00006002_Tipo_de_Medida_Repository.js
function Sm00006002_Tipo_de_Medida_Repository(restEngine){
   
     /**
     * A função "validaTipoMedidaVariavel" verifica se o tipo de medida variável
     * ja está criado para a faixa existente.
     */
     
    this.validaTipoMedidaVariavel = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_variacaotipomedida_fct_tipomedidaserv/validaTipoMedidaVariavel', paramsRestEngine);
        
    };
    
    /**
     * A função "validaExsiteMedidaVariavel" verifica se existem medidas variaveis 
     * para o tipo de medida atual.
     */
     
    this.validaExsiteMedidaVariavel = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_variacaotipomedida_fct_tipomedidaserv/validaExsiteMedidaVariavel', paramsRestEngine);
        
    };
    
     /**
     * A função "validaExsiteVariacao" verifica se o tipo de medida variavel esá sendo 
     * utilizado.
     */
     
    this.validaExsiteVariacao = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_variacaotipomedida_fct_tipomedidaserv/validaExsiteVariacao', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006002_Tipo_de_Medida_Repository", Sm00006002_Tipo_de_Medida_Repository);
});


// FILE: ./assets/js/repositories/Sm00009000_Contrato_Modelo_Repository.js
function Sm00009000_Contrato_Modelo_Repository(restEngine, ScreenService){
   
     /**
     * A função "insereDadosModContrato" insere os dados de parametização 
     * de fatiuramento na tabela de Modelo de Contrato (FCT_MODELOCONTRATO).
     */
    
    this.insereDadosModContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_paramfatmodcont_widget/insereDadosModContrato', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00009000_Contrato_Modelo_Repository", Sm00009000_Contrato_Modelo_Repository);
});


// FILE: ./assets/js/repositories/Sm00034000_Maximo_Carencia_Proposta_Repository.js
function Sm00034000_Maximo_Carencia_Proposta_Repository(restEngine, ScreenService){
   
    /**
     * A função "validaCarenciaOperador" verifica se já existe 
     * uma carencia cadastrada para o operador indicado.
     */
    
    this.validaCarenciaOperador = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_carenciaoper_parcnegocio/validaCarenciaOperador', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00034000_Maximo_Carencia_Proposta_Repository", Sm00034000_Maximo_Carencia_Proposta_Repository);
});


// FILE: ./assets/js/services/ger00004500_Propspect_Service.js
function ger00004500_Prospect_Service(ger00004500_Prospect_Repository, CRM_Utilities){
    
    this.callForInsertProspect = function(currentRow){
        
        var paramsRestEngine = {
            "requestType": "Row",
            "row": currentRow
        };
        
        return ger00004500_Prospect_Repository.callInsertTaskProspect(paramsRestEngine, currentRow);
        
    };
    
    this.callForSelectProspect = function(currentRow){
        
        var paramsRestEngine = {
            "requestType": "Row",
            "row": currentRow
        };
        
        return ger00004500_Prospect_Repository.selectContactProspect(paramsRestEngine, currentRow);
        
    };
    
    this.callForInsertNrseqcontatoForNewOpportunity = function(currentRow){
        
        var paramsRestEngine = {
            "requestType": "Row",
            "row": currentRow
        };
        
        ger00004500_Prospect_Repository.requestForForInsertNrseqcontatoForNewOpportunity(paramsRestEngine);
        
    };
    
    this.callForUpdatePartnerType = function(currentRow){
    
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForUpdatePartnerType(paramsRestEngine);
        
    };
    
    this.callForInsertPartnershipOfTypeClient = function(currentRow){
    
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForInsertPartnershipOfTypeClient(paramsRestEngine);
        
    };
    
    this.callForInsertClient = function(currentRow){
        
        currentRow.NRINSJURPROS = CRM_Utilities.removeValueMask(currentRow.NRINSJURPROS);
        currentRow.CDCLIENTE = this.concatInscriptionTypeWithInscription(currentRow.IDTPIJURPROS, currentRow.NRINSJURPROS);
        
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForInsertClient(paramsRestEngine);
        
    };
    
    this.concatInscriptionTypeWithInscription = function(inscriptionType, inscription){
        
        return inscriptionType + inscription;
            
    };
    
    this.callForDeactivateProspect = function(currentRow){
    
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForDeactivateProspect(paramsRestEngine);
    };
    
    this.callForInsertClientAddress = function(currentRow){
        
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForInsertClientAddress(paramsRestEngine);
        
    };
    
    this.callForSelectProspectAddress = function(currentRow){
        var paramsRestEngine = {
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForSelectProspectAddress(paramsRestEngine);
    };
    
    this.callForSetTaskEndDate = function(widget){
        
        var currentRow = widget.currentRow;
        
        currentRow.NRSEQFILACONTA = widget.NRSEQFILACONTA;
        
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForSetTaskEndDate(paramsRestEngine);
          
    };
    
    this.callForDeleteEvolutionFilesByOriginNumber = function(widget){
        
        var currentRow = widget.currentRow;
        
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForDeleteEvolutionFilesByOriginNumber(paramsRestEngine);
        
    };
    
    this.callForDeactivateTask = function(widget){
        
        var currentRow = widget.currentRow;
        
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForDeactivateTask(paramsRestEngine);
          
    };
    
}
Configuration(function (ContextRegister) {
    ContextRegister.register("ger00004500_Prospect_Service", ger00004500_Prospect_Service);
});


// FILE: ./assets/js/services/Sm_Percentual_Desconto_Operador_Service.js
function Sm_Percentual_Desconto_Operador_Service(Sm_Percentual_Desconto_Operador_Repository){
    
    
    this.validaPercentDescontoOper = function(parcOperador){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'NRPARCNEGOCIO': parcOperador}
        };
        
        var requestPromisse = Sm_Percentual_Desconto_Operador_Repository.validaPercentDescontoOper(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm_Percentual_Desconto_Operador_Service", Sm_Percentual_Desconto_Operador_Service);
});


// FILE: ./assets/js/services/Sm00001000_LancamentoProposta_Service.js
function Sm00001000_LancamentoProposta_Service(General, ZHPromise, Utilities) {
    
    /**
     * 
     */
    this.validarElementosParaConclusao = function (row) {
        var params = {
            "NRSEQORCAMENTO"  : row.NRSEQORCAMENTO,
            "NRORG"           : row.NRORG,
            "NRORCAMENTO"     : row.NRORCAMENTO,
            "PERIODOEXTENDIDO": true,
            "DTFINVIGCONTR"   : row.DTFINPROPOSTA

        };
        return General.callRestEngine("product#/fct_orcamentoaliment_widget/concluirProposta", "Row", params);

    };

    this.validarPropostaParaConclusao = function(currentRow) {
        var msgErro = "";
        
        // Início da validação da Proposta
        
        if (!currentRow.NMPARCINTERLOCUTOR) {
            msgErro += "Interlocutor deve ser informado.<br/><br/>";
        } else {
            if (!currentRow.DSEMAIL) {
                msgErro += "O Interlocutor informado não possui e-mail cadastrado.<br/><br/>";
            }else{
                
                var validaEmail = Utilities.verificaEmail(currentRow.DSEMAIL);  
                
                if (validaEmail == false){
                    
                     msgErro += "O e-mail informado para o interlocutor não é valido.<br/><br/>";
                }
            }
        }

        if (!currentRow.NMPARCINTERLOCUTORFAT) {
            msgErro += "Interlocutor de faturamento deve ser informado.<br/><br/>";
        }
        else {
            if (!currentRow.DSEMAILFAT) {
                msgErro += "O Interlocutor de faturamento informado não possui e-mail cadastrado.<br/><br/>";
            }else{
                
                var validaEmail = Utilities.verificaEmail(currentRow.DSEMAILFAT);  
                
                if (validaEmail == false){
                    
                     msgErro += "O e-mail informado para o interlocutor de faturamento não é valido.<br/><br/>";
                }
            }
        }
       
        
        if (!currentRow.DTVALIDADE) {
            msgErro += "Validade da Proposta (Data) deve ser informada.<br/><br/>";
        }
        
        if (!currentRow.QTDIASVALIDADE && currentRow.QTDIASVALIDADE != 0) {
            msgErro += "Validade da Proposta (Dias) deve ser informada.<br/><br/>";
        }
        
        // Fim da validação da Proposta
        
        if (msgErro.length > 0) {
            msgErro = msgErro.substring(0, msgErro.length-10);
        }
        
        return msgErro;
    };

    this.validarMoedaItens = function(nrseqorcamento) {
        var deferred = ZHPromise.defer();
        var params = {
            "NRSEQORCAMENTO": nrseqorcamento
        };
        
        General.callRestEngine("product#/fct_orcamento_widget/getQuantidadeItensMoedaDiferente", "Row", params).then(function(response) {
            if (response.dataset.retorno.qtd > 0) {
                deferred.reject("Operação bloqueada! <br><br>Os itens da proposta foram inseridos com uma diferente da moeda atual da proposta. Favor atualizar o valor dos itens.");
            }
            else {
                deferred.resolve();
            }
        }).catch(function(error) {
            deferred.reject(error);
        });

        return deferred.promise;
    };
    
    /**
     * 
     */
    this.validarItensPropostaParaConclusao = function(nrseqorcamento, nrorg) {
        var deferred = ZHPromise.defer();
        var msgErro = "";
        
        var filtro = [
            {
                "name": "NRSEQORCAMENTO",
                "operator": "=",
                "value": nrseqorcamento
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": nrorg
            }
        ];
        
        General.callRestEngine("product#/crud/fct_itemorcamento_widget", "FilterData", filtro).then(function(success) {
            if (success.dataset) {
                
                var itens = success.dataset.fct_itemorcamento_widget;
                
                // Início da validação dos Itens
                for (var i = 0; i < itens.length; i++) {
                    
                    var tipo = (itens[i].IDADICITEMORC == "S") ? "Adicional " : "Item ";
                    
                    if (itens[i].IDFATURAORCA == "O") {
                        if (!itens[i].CDMODPGTO) {
                            msgErro += tipo + itens[i].NMITEMORCAMENTO + ": Modalidade de Pagamento deve ser informada.<br/><br/>";
                        }
                    }
                    
                    if (itens[i].IDFATURAORCA == "P") {
                        if (!itens[i].QTPARCELASFATU) {
                            msgErro += tipo + itens[i].NMITEMORCAMENTO + ": Quantidade de Parcelas deve ser informado.<br/><br/>";
                        }
                        
                        if (!itens[i].NRDIAVENCPARCELASFATU) {
                            msgErro += tipo + itens[i].NMITEMORCAMENTO + ": Dia para Vencimento deve ser informado.<br/><br/>";
                        }
                    }

                    if (!itens[i].VRTOTALITEMORCA && itens[i].VRTOTALITEMORCA != 0) {
                        msgErro += tipo + itens[i].NMITEMORCAMENTO + ": Valor Total deve ser informado.<br/><br/>";
                    }
                }
                // Fim da validação dos Itens
            }
            
            if (msgErro) {
                msgErro = msgErro.substring(0, msgErro.length-10);
                deferred.reject(msgErro);
            }
            else {
                deferred.resolve();
            }
        },
        function(error) {
            console.log(error);
            deferred.reject(error.data.error);
        });
        
        return deferred.promise;
    };
    
    this.getComposicaoDoElemento = function(row) {
        if (row._parent.TIPO == "C") {
            return row._parent;
        }
        else {
            return this.getComposicaoDoElemento(row._parent);
        }
    };
    
    
    
    this.validaPropReneg = function(nrseqorcamento){
        
        var params = {
            "NRSEQORCAMENTO": nrseqorcamento

         };
         
         General.callRestEngine("product#/fct_orcamento_widget/validaPropReneg", "Row", params).then(function(success){ 
            
             var validaPropReneg = success.dataset.PROPOSTA.VALIDA;    
             
             return validaPropReneg;
             
         }.bind(this),
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
        });
    }
    
    
    /**
     * Validação que obriga o usuário selecionar os elementos da composição que são obrigatórios.
     * 
     * @param {object[]} rows
     * 
     * @return {string}
     */
    this.validarObrigatoriedadeItensComposicao = function(rows) {
        var promessa = ZHPromise.defer();
        var msg      = null;
        
        var existeLinhaEscolhida = rows.some(function(row) {
            return row.__isSelected == true;
        });
        
        if (existeLinhaEscolhida) {
            var composicoesObrigatoriasNaoEscolhidas = rows.filter(function(row) {
                if (row.TIPO == "C" && row.IDSERVOBRIGCOMP == "S" && row.__isSelected == false) {
                    if (row._parent) {
                        var composicao = this.getComposicaoDoElemento(row);
                        return composicao.__isSelected;
                    }
                    else {
                        return true;
                    }
                }
            }.bind(this));
            
            if (composicoesObrigatoriasNaoEscolhidas.length == 0) {
                
                var elementosObrigatoriosNaoEscolhidos = rows.filter(function(row) {
                    if (row.TIPO == "E" && row._childCount == 0 && row.IDSERVOBRIGCOMP == "S" && row.__isSelected == false) {
                        if (row._parent) {
                            var composicao = this.getComposicaoDoElemento(row);
                            return composicao.__isSelected;
                        }
                        else {
                            return true;
                        }
                    }
                    else {
                        return false;
                    }
                }.bind(this));
                
                if (elementosObrigatoriosNaoEscolhidos.length == 0) {
                    promessa.resolve(null);
                }
                else {
                    msg = "Os seguintes elementos são obrigatórios:<br/><br/>";
                    
                    elementosObrigatoriosNaoEscolhidos.forEach(function(campo) {
                        var itensPai = Utilities.getItensPai(campo);
                        
                        if (itensPai.length > 0) {
                            itensPai.reverse();
                            itensPai.forEach(function(pai) {
                                msg += pai.NMELEMENTO + " > ";
                            });
                        }
                        msg += "<u>" + campo.NMELEMENTO + "</u><br/><br/>";
                    });
                    
                    promessa.resolve(msg.substr(0, msg.length - 10));
                }
                
            }
            else {
                msg = "As seguintes composições são obrigatórias:<br/><br/>";
                composicoesObrigatoriasNaoEscolhidas.forEach(function(item) {
                    var itensPai = Utilities.getItensPai(item);
                        
                    if (itensPai.length > 0) {
                        itensPai.reverse();
                        itensPai.forEach(function(pai) {
                            msg += pai.NMELEMENTO + " > ";
                        });
                    }
                    
                    msg += "<u>" + item.NMELEMENTO + "</u><br/><br/>";
                });
                
                promessa.resolve(msg.substr(0, msg.length - 10));
            }
        
        } else {
            promessa.resolve("Operação bloqueada.<br/><br/>Nenhum elemento foi selecionado.");
        }
        
        return promessa.promise;
    };
    
    this.validarVendedor = function(nrseqorcamento, nrvendedororca, nrparcvendedor) {
        if (!nrvendedororca)
            nrvendedororca = -1;
        
        var params = {
            "NRSEQORCAMENTO": nrseqorcamento,
            "NRVENDEDORORCA": nrvendedororca,
            "NRPARCVENDEDOR": nrparcvendedor
        };
        
        return General.callRestEngine("product#/fct_vendedororca_widget/validarVendedor", "Row", params);
    };
    
    this.getPorcentagemVendedores = function(nrseqorcamento, nrorg) {
        var params = {
            "NRSEQORCAMENTO": nrseqorcamento,
            "NRORG": nrorg
        };
        
        return General.callRestEngine("product#/fct_vendedororca_widget/getPorcentagemVendedores", "Row", params);
    };
    
    this.validarTipoEndereco = function(nrseqorcamento, nrenderecoorca, cdtipoendereco) {
        if (!nrenderecoorca)
            nrenderecoorca = -1;
        
        var params = {
            "NRSEQORCAMENTO": nrseqorcamento,
            "NRENDERECOORCA": nrenderecoorca,
            "CDTIPOENDERECO": cdtipoendereco
        };
        
        return General.callRestEngine("product#/fct_enderecoorca_widget/validarTipoEndereco", "Row", params);
    };

    this.validarNegociacaoValores = function(nrseqorcamento) {
        var promise = ZHPromise.defer();
        var params = {
            "NRSEQORCAMENTO": nrseqorcamento
        };

        General.callRestEngine("product#/fct_orcamento_widget/validarNegociacaoValores", "Row", params).then(function(response) {
            promise.resolve(response.dataset.validacao.resultado);
        })
        .catch(function(error) {
            console.error(error);
            promise.reject(error.data.error);
        });

        return promise.promise;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00001000_LancamentoProposta_Service", Sm00001000_LancamentoProposta_Service);
});

// FILE: ./assets/js/services/Sm00002000_Modelo_Proposta_Service.js
function Sm00002000_Modelo_Proposta_Service(Sm00002000_Modelo_Proposta_Repository){
    
    /**
     * A função "validaEscopoProposta" verifica se o escopo 
     * que está sendo relacionado já está relacionado ao modelo.
     */
    
    this.validaEscopoProposta = function(tipoOrcamento, orcaModelo){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'NRTIPOORCAMENTO': tipoOrcamento,
                    'NRORCAMENTOMODELO': orcaModelo
            }
        };
        
        var requestPromisse = Sm00002000_Modelo_Proposta_Repository.validaEscopoProposta(paramsRestEngine);
        
        return requestPromisse;
    };
    
    
    /**
     * A função "validaCompProposta" verifica se a composição 
     * que está sendo relacionada já está relacionada ao modelo.
     */
    
    this.validaCompProposta = function(compServico, orcaModelo){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'NRCOMPSERVICO': compServico,
                    'NRORCAMENTOMODELO': orcaModelo
            }
        };
        
        var requestPromisse = Sm00002000_Modelo_Proposta_Repository.validaCompProposta(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00002000_Modelo_Proposta_Service", Sm00002000_Modelo_Proposta_Service);
});


// FILE: ./assets/js/services/Sm00003000_Escopo_Proposta_Service.js
function Sm00003000_Escopo_Proposta_Service(Sm00003000_Escopo_Proposta_Repository){
    
    /**
     * A função "validaServicoTabela" verifica se o serviço 
     * que está sendo relacionado já está relacionado na tabela.
     */
    
    this.validaServicoEscopo = function(servOrca, escopoOrca){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {
                'CDSERVORCA': servOrca,
                'NRTIPOORCAMENTO': escopoOrca
            }
        };
        
        var requestPromisse = Sm00003000_Escopo_Proposta_Repository.validaServicoEscopo(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00003000_Escopo_Proposta_Service", Sm00003000_Escopo_Proposta_Service);
});


// FILE: ./assets/js/services/Sm00004000_Contrato_Service.js
function Sm00004000_Contrato_Service(Sm00004000_Contrato_Repository) {

    /**
     * A função "copiaVersionaContrato" executa a procedure
     * "FCT_COPIA_VERSIONA_CONTRATO".
     */
    this.copiaVersionaContrato = function(nrSeqContrato, idCopiaVersiona, adicionaAgrupamento, idParcelaConsiderada, idReplicarNegociacao, dtRenegociaCont, dtFimDoNovoContrato) {
        
        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'NRSEQCONTRATO'       : nrSeqContrato,
                'TIPO'                : idCopiaVersiona,
                'ADCAGRUPAMENTO'      : adicionaAgrupamento,
                'IDPARCELACONSIDERADA': idParcelaConsiderada,
                'IDREPLICARNEGOCIACAO': idReplicarNegociacao,
                'DTRENEGOCIACONT'     : dtRenegociaCont,
                "DTFIMCONTRATO"       : dtFimDoNovoContrato
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.copiaVersionaContrato(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * Executa a procedure "FCT_ENCERRA_CONTRATO".
     */
    this.encerraContrato = function(nrSeqContrato, idSituacao, dtrecisao, idtpexclusaoparcela, nrtiporescisao, idDocX, dtpedrescisaocontr, nrmotivorescisao, dsjustificaencerra, vrpendenciafin) {
        
        if(!dtrecisao)
            dtrecisao = moment().format("DD/MM/YYYY");
            
        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'NRSEQCONTRATO'       : nrSeqContrato,
                "IDSITUACAOCONTRATO"  : idSituacao,
                "IDTPEXCLUSAOPARCELA" : idtpexclusaoparcela,
                "DTRECISAOCONTR"      : dtrecisao,
                "NRTIPORESCISAO"      : nrtiporescisao,
                "IDDOCX"              : idDocX,
                "DTPEDRESCISAOCONTR"  : dtpedrescisaocontr,
                "NRMOTIVORESCISAO"    : nrmotivorescisao,
                "DSJUSTIFICAENCERRA"  : dsjustificaencerra,
                "VRPENDENCIAFIN"      : vrpendenciafin
            }
        };
        
        var requestPromisse = Sm00004000_Contrato_Repository.encerraContrato(paramsRestEngine);
        return requestPromisse;
    };

    /**
     * Executa a procedure "FCT_VENCPREV_CONT".
     */
    this.geraParcelasContrato = function(nrSeqContrato) {

        var paramsRestEngine = {
            "requestType": "Row",
            "row": {
                "NRSEQCONTRATO": nrSeqContrato
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.geraParcelasContrato(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "cancelaParcelasContrato" executa a procedure
     * "FCT_CANC_VENCPREV_CONT".
     */
    this.cancelaParcelasContrato = function(nrSeqContrato) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'NRSEQCONTRATO': nrSeqContrato
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.cancelaParcelasContrato(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "alteraVigenciaContrato" altera os dados de vigência do contrato
     */
    this.alteraVigenciaContrato = function(dtInicVigContr, dtFinVigContr, seqContrato) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'DTINIVIGENCONT': dtInicVigContr,
                'DTFINVIGENCONT': dtFinVigContr,
                'NRSEQCONTRATO': seqContrato
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.alteraVigenciaContrato(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "validaDadosProrrog" valida se ja existem alterações de vigência para o contrato
     */
    this.validaDadosProrrog = function(seqContrato) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': { 'NRSEQCONTRATO': seqContrato }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.validaDadosProrrog(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "alteraDadosProrrog" inativa as alterações de vigência do contrato
     */

    this.alteraDadosProrrog = function(seqContrato, seqprorrog) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'NRSEQCONTRATO': seqContrato,
                'NRSEQPRORRCONT': seqprorrog
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.alteraDadosProrrog(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "recebeParamFatModCont" recebe os dados de Faturamento do Modelo de Contrato
     */

    this.recebeParamFatModCont = function(seqModContrato) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': { 'NRSEQMODCONTRA': seqModContrato }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.recebeParamFatModCont(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "recebeModReajustContr" recebe os dados de Reajuste do Modelo de Contrato
     */

    this.recebeModReajustContr = function(seqModContrato) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': { 'NRSEQMODCONTRA': seqModContrato }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.recebeModReajustContr(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "recebeModIndContr" recebe os dados de Indice de Reajuste do Modelo de Contrato
     */

    this.recebeModIndContr = function(seqModContrato, seqModReajus) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'NRSEQMODCONTRA': seqModContrato,
                'NRSEQMODREAJUS': seqModReajus
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.recebeModIndContr(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "recebeModeloDocContr" recebe os dados de Documento do Modelo de Contrato
     */

    this.recebeModeloDocContr = function(seqModContrato) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': { 'NRSEQMODCONTRA': seqModContrato }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.recebeModeloDocContr(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "geraCodigoContrato" executa a procedure "CRIA_MNEMONICO".
     */

    this.geraCodigoContrato = function(seqContrato) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'NRSEQCONTRATO': seqContrato
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.geraCodigoContrato(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "geraDocumentoContrato" executa a procedure "FCT_SUB_MNE_DOC".
     */

    this.geraDocumentoContrato = function(seqContrato, nrSeqDocCont) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'NRSEQCONTRATO': seqContrato,
                'NRSEQCONTDOC': nrSeqDocCont
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.geraDocumentoContrato(paramsRestEngine);

        return requestPromisse;
    };


}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00004000_Contrato_Service", Sm00004000_Contrato_Service);
});


// FILE: ./assets/js/services/Sm00004005_Parametrizacao_Contrato_Service.js
function Sm00004005_Parametrizacao_Contrato_Service(Sm00004005_Parametrizacao_Contrato_Repository){
    
  /**
     * A função "insereDadosContrato" insere os dados de parametização 
     * de fatiuramento na tabela de Contrato (FCT_CONTRATO).
     */
    
    this.insereDadosContrato = function(IdFaturaContr, CdModPgto, QtParcelasFatu, NrDiaVencParcelasFatu, dtInicFatContr, idQtDiasCompCon, nrQtDiasCompCon, idCompetCont, nrDiaCompetCont, idTpDtEmissNf, nrDiaDtEmissNf, idTpDtVenc, nrDiaVenParcFat, idTpFatuIncomp, seqContrato){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': { 
                     'IDFATURACONTR' : IdFaturaContr, 
                     'CDMODPGTO': CdModPgto,
                     'QTPARCELASFATU': QtParcelasFatu,
                     'NRDIAVENCPARCELASFATU': NrDiaVencParcelasFatu,
                     'DTINICFATCONTR': dtInicFatContr,
                     'IDQTDIASCOMPCON': idQtDiasCompCon,
                     'NRQTDIASCOMPCON': nrQtDiasCompCon,
                     'IDTPCOMPETCONT': idCompetCont,
                     'NRDIACOMPETCONT': nrDiaCompetCont,
                     'IDTPDTEMISSNF': idTpDtEmissNf,
                     'NRDIADTEMISSNF': nrDiaDtEmissNf,
                     'IDTPDTVENC': idTpDtVenc,
                     'NRDIAVENPARCFAT': nrDiaVenParcFat,
                     'IDTPFATUINCOMP':idTpFatuIncomp,
                     'NRSEQCONTRATO': seqContrato
            }
        };
        
        var requestPromisse = Sm00004005_Parametrizacao_Contrato_Repository.insereDadosContrato(paramsRestEngine);
        
        return requestPromisse;
    };
    
    
    this.insereDadosItContrato = function(IdFaturaContr, CdModPgto, QtParcelasFatu, NrDiaVencParcelasFatu, seqContrato, seqitContrato){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': { 
                     'IDFATURACONTR' : IdFaturaContr, 
                     'CDMODPGTO': CdModPgto,
                     'QTPARCELASFATU': QtParcelasFatu,
                     'NRDIAVENCPARCELASFATU': NrDiaVencParcelasFatu,
                     'NRSEQCONTRATO': seqContrato,
                     'NRITEMOCONTRATO': seqitContrato
            }
        };
        
        var requestPromisse = Sm00004005_Parametrizacao_Contrato_Repository.insereDadosItContrato(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00004005_Parametrizacao_Contrato_Service", Sm00004005_Parametrizacao_Contrato_Service);
});


// FILE: ./assets/js/services/Sm00006000_Elemento_Service.js
function Sm00006000_Elemento_Service(Sm00006000_Elemento_Repository){
    
    /**
     * A função "validaServicoRelacionado" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço.
     */
    
    this.validaServicoRelacionado = function(servOrcaRelac, servOrca, tipoMedida){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'CDSERVORCARELAC': servOrcaRelac,
                    'CDSERVORCA': servOrca,
                    'CDTPMEDIDASERV': tipoMedida
            }
        };
        
        var requestPromisse = Sm00006000_Elemento_Repository.validaServicoRelacionado(paramsRestEngine);
        
        return requestPromisse;
    };
    
    
     /**
     * A função "validaServicoRelacionadoUnit" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço para inclusão 
     * de um só item.
     */
    
    this.validaServicoRelacionadoUnit = function(servOrcaRelac, servOrca, tipoMedida){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'CDSERVORCARELAC': servOrcaRelac,
                    'CDSERVORCA': servOrca,
                    'CDTPMEDIDASERV': tipoMedida
            }
        };
        
        var requestPromisse = Sm00006000_Elemento_Repository.validaServicoRelacionadoUnit(paramsRestEngine);
        
        return requestPromisse;
    };
    
    
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006000_Elemento_Service", Sm00006000_Elemento_Service);
});


// FILE: ./assets/js/services/Sm00006000_Servicos_Service.js
function Sm00006000_Servicos_Service(Sm00006000_Servicos_Repository){
    
    /**
     * A função "validaServicoRelacionado" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço.
     */
     
    this.validaServicoRelacionado = function(servOrcaRelac, servOrca, tipoMedida){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'CDSERVORCARELAC': servOrcaRelac,
                    'CDSERVORCA': servOrca,
                    'CDTPMEDIDASERV': tipoMedida
            }
        };
        
        var requestPromisse = Sm00006000_Servicos_Repository.validaServicoRelacionado(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006000_Servicos_Service", Sm00006000_Servicos_Service);
});


// FILE: ./assets/js/services/Sm00006001_Tabela_Preco_Servico_Service.js
function Sm00006001_Tabela_Preco_Servico_Service(Sm00006001_Tabela_Preco_Servico_Repository){
    
    /**
     * A função "validaServicoTabela" verifica se o serviço 
     * que está sendo relacionado já está relacionado na tabela.
     */
    
    this.validaServicoTabela = function(servOrca, precBaseServ, tipoMedida){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'CDSERVORCA': servOrca,
                    'CDPRECBASESERV': precBaseServ,
                    'CDTPMEDIDASERV': tipoMedida
            }
        };
        
        var requestPromisse = Sm00006001_Tabela_Preco_Servico_Repository.validaServicoTabela(paramsRestEngine);
        
        return requestPromisse;
    };
    
     /**
     * A função "validaTipoMedidaVariavel" verifica se o tipo de medida variável
     * ja está criado para a faixa existente.
     */
    
    this.validaTipoMedidaVariavel = function(nrSeqItPrecBServ, cdTpMedidaServ, CdtpMedidaVariavel, nrVaricacaoPrecBaseServ){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'NRSEQITPRECBSERV' : nrSeqItPrecBServ,
                    'CDTPMEDIDASERV' : cdTpMedidaServ,
                    'CDTPMEDIDAVARIAVEL' : CdtpMedidaVariavel,
                    'NRVARIACAOPRECBASESERV': nrVaricacaoPrecBaseServ,
            }
        };
        
        var requestPromisse = Sm00006001_Tabela_Preco_Servico_Repository.validaTipoMedidaVariavel(paramsRestEngine);
        
        return requestPromisse;
    };
    
     /**
     * A função "sincronizarVariacaoPreco" executa a procedure "FCT_SINCRONIZAR_VARIACAO_PRECO".
     */
    
    this.sincronizarVariacaoPreco = function(nrSeqItPrecBServ, tipo){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'NRSEQITPRECBSERV': nrSeqItPrecBServ,
                    'TIPO': tipo
            }
        };
        
        var requestPromisse = Sm00006001_Tabela_Preco_Servico_Repository.sincronizarVariacaoPreco(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006001_Tabela_Preco_Servico_Service", Sm00006001_Tabela_Preco_Servico_Service);
});


// FILE: ./assets/js/services/Sm00006002_Tipo_de_Medida_Service.js
function Sm00006002_Tipo_de_Medida_Service(Sm00006002_Tipo_de_Medida_Repository){
    
    /**
     * A função "validaTipoMedidaVariavel" verifica se o tipo de medida variável
     * ja está criado para a faixa existente.
     */
    
    this.validaTipoMedidaVariavel = function(cdTpMedidaServ, CdtpMedidaVariavel, nrVaricacaoTpMedida){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'CDTPMEDIDASERV': cdTpMedidaServ,
                    'CDTPMEDIDAVARIAVEL': CdtpMedidaVariavel,
                    'NRVARIACAOTPMEDIDA': nrVaricacaoTpMedida
            }
        };
        
        var requestPromisse = Sm00006002_Tipo_de_Medida_Repository.validaTipoMedidaVariavel(paramsRestEngine);
        
        return requestPromisse;
    };
    
    /**
     * A função "validaExsiteMedidaVariavel" verifica se existem medidas variaveis 
     * para o tipo de medida atual.
     */
     
    this.validaExsiteMedidaVariavel = function(cdTpMedidaServ){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'CDTPMEDIDASERV': cdTpMedidaServ
            }
        };
        
        var requestPromisse = Sm00006002_Tipo_de_Medida_Repository.validaExsiteMedidaVariavel(paramsRestEngine);
        
        return requestPromisse;
    };
    
    
     /**
     * A função "validaExsiteVariacao" verifica se o tipo de medida variavel esá sendo 
     * utilizado.
     */
     
    this.validaExsiteVariacao = function(CdtpMedidaVariavel){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'CDTPMEDIDAVARIAVEL': CdtpMedidaVariavel
            }
        };
        
        var requestPromisse = Sm00006002_Tipo_de_Medida_Repository.validaExsiteVariacao(paramsRestEngine);
        
        return requestPromisse;
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006002_Tipo_de_Medida_Service", Sm00006002_Tipo_de_Medida_Service);
});


// FILE: ./assets/js/services/Sm00009000_Contrato_Modelo_Service.js
function Sm00009000_Contrato_Modelo_Service(Sm00009000_Contrato_Modelo_Repository){
    
  /**
     * A função "insereDadosModContrato" insere os dados de parametização 
     * de fatiuramento na tabela de Modelo de Contrato (FCT_MODELOCONTRATO).
     */
    
    this.insereDadosModContrato = function(dtInicFatContr, idQtDiasCompCon, nrQtDiasCompCon, idCompetCont, nrDiaCompetCont, idTpDtEmissNf, nrDiaDtEmissNf, idTpDtVenc, nrDiaVenParcFat, idTpFatuIncomp, seqModContrato){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': { 'DTINICFATCONTR': dtInicFatContr,
                     'IDQTDIASCOMPCON': idQtDiasCompCon,
                     'NRQTDIASCOMPCON': nrQtDiasCompCon,
                     'IDTPCOMPETCONT': idCompetCont,
                     'NRDIACOMPETCONT': nrDiaCompetCont,
                     'IDTPDTEMISSNF': idTpDtEmissNf,
                     'NRDIADTEMISSNF': nrDiaDtEmissNf,
                     'IDTPDTVENC': idTpDtVenc,
                     'NRDIAVENPARCFAT': nrDiaVenParcFat,
                     'IDTPFATUINCOMP':idTpFatuIncomp,
                     'NRSEQMODCONTRA': seqModContrato
            }
        };
        
        var requestPromisse = Sm00009000_Contrato_Modelo_Repository.insereDadosModContrato(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00009000_Contrato_Modelo_Service", Sm00009000_Contrato_Modelo_Service);
});


// FILE: ./assets/js/services/Sm00034000_Maximo_Carencia_Proposta_Service.js
function Sm00034000_Maximo_Carencia_Proposta_Service(Sm00034000_Maximo_Carencia_Proposta_Repository){
    
    /**
     * A função "validaCarenciaOperador" verifica se já existe 
     * uma carencia cadastrada para o operador indicado.
     */
    
    this.validaCarenciaOperador = function(parcOperador){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'NRPARCNEGOCIO': parcOperador}
        };
        
        var requestPromisse = Sm00034000_Maximo_Carencia_Proposta_Repository.validaCarenciaOperador(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00034000_Maximo_Carencia_Proposta_Service", Sm00034000_Maximo_Carencia_Proposta_Service);
});


// FILE: ./assets/js/controllers/ABEvolucaoController.js
function ABEvolucaoController(CRM_Utilities, templateManager, restEngine, ScreenService, ZHPromise) {
    
    /**
     * Procedimento para definir a permissão para editar
     * o campo 'Data da Evolução' no ambiente de criação
     * de uma evolução.
     * 
     * @para {widget} widget widget referente a tela de
     * evolução.
    */
    
    this.setEvolDateChangePermission = function(widget){
        var currentRow = widget.currentRow;
        
        this.getNrDiasLanceEvol(widget).then(function(response){
            var nrDiasLanceEvol = templateManager.NRDIASLANCEVOL;
            
            if(nrDiasLanceEvol > 0 && currentRow.__is_new){
                widget.getField('DTHREVOLCONT').readOnly = false;
                this.configureEvolDateChangePermission(widget, nrDiasLanceEvol);
            }else{
                widget.getField('DTHREVOLCONT').readOnly = true;
            }
        }.bind(this));
    };
    
    /**
     * Procedimento para buscar, e setar no templateManager
     * (se não já existir no templateManager),
     * o número de dias permitido para uma 
     * evolução ser lançada posteriomento ao dia real.
     * 
     * @param {object} widget widget referente a
     * tela de evoluções.
     * 
     * @return {number} retorna o número de dias pemitido
     * para lançar uma evolução posterior a data real.
    */
    this.getNrDiasLanceEvol = function(widget){
        var nrDiasLanceEvol = templateManager.NRDIASLANCEVOL;
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        var deferred = ZHPromise.defer();
        
        if(nrDiasLanceEvol){
            deferred.resolve();
            
            return deferred;
        }else{
            
            var paramsRestEngine = {
            "requestType": "Row",
            "row": {NRORG: loggedOperatorInfos.NRORG}
            };
        
            return restEngine.post('/getNrDiasLanceEvol', paramsRestEngine,
            function(success){
                templateManager.NRDIASLANCEVOL = success.dataset.DADOS.NRDIASLANCEVOL[0].NRDIASLANCEVOL;
            },undefined,
            function(error){
                CRM_Utilities.CATCH_EXCEPTION_ERROR(error.data.error);
            },undefined,undefined
        );
            
        }
    };
    
    /**
     * Procedimento para configurar as permissões
     * de data máxima e data mínima para lançar
     * uma evolução.
     * 
     * @param {object} widget widget referente a
     * tela de evoluções.
     * 
     * @param {number} numberOfDays número máximo de dias
     * para se lançar uma evolução posteriormente.
    */
    this.configureEvolDateChangePermission = function(widget, numberOfDays){
        var evolDateField = widget.getField('DTHREVOLCONT');
        
        evolDateField.minDate = CRM_Utilities.subtractDateByDays(CRM_Utilities.getCurrentDateWithMinTime(),numberOfDays);
        evolDateField.maxDate = CRM_Utilities.getCurrentDateWithMaxTime();
    };
    
    /**
     * //TODO
    */
    this.configureEvolTimeShowPermission = function(widget){
        var currentRow = widget.currentRow;
        var selectedDateWithoutTime = moment(currentRow.DTHREVOLCONT, 'DD/MM/YYYY HH:mm').format('DD/MM/YYYY');    
        
        if(selectedDateWithoutTime === CRM_Utilities.getCurrentDate()){
            currentRow.DTHREVOLCONT = selectedDateWithoutTime + ' ' + CRM_Utilities.getCurrentTime();
        }else{
            currentRow.DTHREVOLCONT = selectedDateWithoutTime + ' 00:00';
        }
    };
    
    /**
     * Define o filtro da widget de evolução
     * pelo número do parceiro de negócio selecionado
     * anteriormente.
     * 
     * @param {object} widgetName nome da widget de evolução
     * em caso.
    */
    this.setEvolutionWidgetFilterByNrParcNegocio = function(widgetName){
        
        var evolutionWidget = templateManager.container.getWidget(widgetName);
        
        var widgetFilter = evolutionWidget.dataSourceFilter;
        var parentWidgetCurrentRow = evolutionWidget.parent.currentRow;
        var nrParcNegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(widgetFilter, "P_NRPARCNEGOCIO");
        
        widgetFilter[nrParcNegocioFilterIndex].value = parentWidgetCurrentRow.NRPARCNEGOCIO;
        
        evolutionWidget.reload();
        
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ABEvolucaoController', ABEvolucaoController);
});

// FILE: ./assets/js/controllers/Dashboard_Controller.js
function Dashboard_Controller(ScreenService, templateManager, Operador_Repository, General, Utilities){

    //Get Widgets

    var getWidgetAgendamentosPendentes = function() {
        return templateManager.container.getWidget('wdgAgendamentosPendentes');
    };
    var getContainer = function() {
        return templateManager.container;
    };

    var getWidgetTabbedPrincipal = function() {
        return getContainer().getWidget('wdgTabbedPrincipal'); 
    }
    /**
    * Busca o usuário logado e o adiciona o dataSourceFilter
    * @params{Object} widget
    **/
    this.buscaOperadorLogado = function(widget) {
        Operador_Repository.getOperadorLogado().then(function(success) {
            widget.dataSourceFilter[0].value = success[0].CDOPERADOR;
            widget.reload();
        }.bind(this));
    };
    
    /**
    * Troca as cores das series de cada ZH_Charts de acordo com o que veio no DataSource
    * 
    * @param {String, object} String de nomes das Widgets 
    *                         widget widget de cada Zh-Charts
    * */
     this.trocarCorDaSerieZH_charts = function(name, widget) {
        $(document).ready(function() {
            var element = angular.element(name);
            var data = widget.dataSource.data;
            element.find("g.slices .slice").each(function(index, element) {
                $(this).attr("fill", data[index]['SERIESCORES']);
                $(this).attr("fill", data[index]['SERIESCORES']);
            });
            element.find("g.slices .label-value").each(function(index, element) {
                $(this).attr("fill", data[index]['SERIESCORES']);
                $(this).attr("fill", data[index]['SERIESCORES']);
            });
            element.find('.legend-label path').each(function(index, element) {
                $(this).attr("fill", data[index]['SERIESCORES']);
                $(this).attr("fill", data[index]['SERIESCORES']);
            });
        });
    };
    
    
    /**
    * Seta as cores das series de cada Charts de acordo com o que veio no DataSource
    * 
    * @param {object} widget widget de cada Charts
    */
    this.trocarCorDaSerieCharts = function(widget) {
        var cores = widget.dataSource.data;
       
        cores.forEach(function(row) {
            widget.chartColors.push(row.SERIESCORES);
        });

    };

    this.filtraGrid = function(evento){
        var widgetTabbed = getWidgetTabbedPrincipal();
        var widget       = getWidgetAgendamentosPendentes();
        var filial       = widgetTabbed.currentRow.CDFILIAL;
        var operador     = widgetTabbed.currentRow.CDOPERADOR;
        var periodo      = widgetTabbed.currentRow.PERIODO;
        
        widget.dataSourceFilter = [
            {
                "name": "CDOPERADORLOGADO",
                "operator": "=",
                "value":"OPERADOR_LOGADO"
            },
            {
                "name": [
                    {
                        "name": "DTINIAGENDAMENTO",
                        "direction": "DESC"
                    }
                ],
                "operator": "ORDER_BY"
            }
        ];
        
        if(filial && filial.length > 0){
            widget.dataSourceFilter.push({
                "name": "CDFILIAL",
                "operator": "IN",
                "value": filial
            });
        }
        
        if(operador && operador.length > 0) {
            widget.dataSourceFilter.push(
                {
                    "name": "CDOPERINCLUSAO",
                    "operator": "IN",
                    "value": operador
                }
            );
        }
        
        if(periodo && periodo.start && periodo.start.length > 0) {
            widget.dataSourceFilter.push(
                {
                    "name": "DTINIAGENDAMENTO",
                    "operator": "=",
                    "value":  periodo.start
                },
                {
                    "name": "DTFINAGENDAMENTO",
                    "operator": "=",
                    "value": periodo.end
                }
            );
        }
        
    };

    /**
    * Abre a tela de Contratos filtrando pela a serie clicada no Charts
    * 
    * @param {object} widget widget da Charts
    this.abirWidgetContratoEmFiltro = function(widget) {
        var situacao = widget.currentRow.IDSITUACAOCONTRATO;
    
           ScreenService.openWindow('product#sm_00004_000_contrato').then(function() {

            var widgetChamada = templateManager.container.getWidget('gridContrato');
            widgetChamada.dataSourceFilter = [
                {
                    name: "IDSITUACAOCONTRATO",
                    operator: "=",
                    static: true,
                    value: situacao
                }
            ];
            widgetChamada.reload().then(function() {
                ScreenService.closePopup(true);
            });

        });
    };
    */
    
    /**
    * Abre a tela de Propostas filtrando pela a serie clicada no Charts
    * 
    * @param {object} widget widget da Charts
    */
    this.abirWidgetPropostaEmFiltro = function(widget) {
        var situacao = widget.currentRow.IDSITUACAOORCAMENTO;
  
           ScreenService.openWindow('product#sm_00001_000_lancamento_proposta').then(function() {

            var widgetChamada = templateManager.container.getWidget('wdgPropostaGrid');
            widgetChamada.dataSourceFilter = [
                {
                    name: "IDSITUACAOORCAMENTO",
                    operator: "=",
                    static: true,
                    value: situacao
                }
            ];
            widgetChamada.reload().then(function() {
                ScreenService.closePopup(true);
            });

        });
    };
    /**
    * Abre a tela de Recursos Agendados filtrando pela a serie clicada no Zh-charts
    * 
    * @param {object} widget widget da Zh-Charts
    this.abirWidgetRecursoEmFiltro = function(widget) {
        var situacao = widget.currentRow.IDSITUARECGEND;
   
           ScreenService.openWindow('product#sm_00015_004_equipes_de_trabalho').then(function() {

            var widgetChamada = templateManager.container.getWidget('equipeDeTrabalho');
            widgetChamada.dataSourceFilter = [
                {
                    name: "IDSITUARECGEND",
                    operator: "=",
                    static: true,
                    value: situacao
                }
            ];
            widgetChamada.reload().then(function() {
                ScreenService.closePopup(true);
            });

        });
    };
    */
    
    this.deleteRecursosAgendados = function(widget){
      
        General.deleteCrud(widget).then(function() {
            widget.reload();
        },
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });

    };
    
    /**
     * Coloca o mês corrente como defaultValue da field de período
     * @params{Object} widget widget Tabbed Principal 
    **/
    this.setPeriodoDafault = function(widget){
        getLocalVar("loginPromise").then(function() {

            widget.currentRow.PERIODO = {
                "end"   : moment().endOf('month').format('DD/MM/YYYY'),
                "start" : moment().startOf('month').format('DD/MM/YYYY'),
                "period": ""
            };
            
            this.filtraGraficos(widget.getField("PERIODO"));
        
        }.bind(this));
    };

    /**
     * Aplica o filtro nos gáficos da tela
     * @params{Object} field field alterada para filtragem
    **/
    this.filtraGraficos = function(field){
        var wdgDadosProp = templateManager.container.getWidget("wdgDadosPropostas");
        var value = [];
        var index;
        
        if(field.name === "CDFILIAL"){
            index = 4;
            if(field.dataSource.getCheckedRows().length){
                field.dataSource.getCheckedRows().forEach(function(check){
                    value.push(check.CDFILIAL);
                });
                
                value = "_" + value.join("_") + "_";
            } else {
                value = "-1";
            }
        } else {
            if(field.name === "CDOPERADOR"){
                index = 3;
                if(field.dataSource.getCheckedRows().length){
                    field.dataSource.getCheckedRows().forEach(function(check){
                        value.push(check.NRPARCNEGOCIO);
                    });
                        
                    value = "_" + value.join("_") + "_";
                } else {
                    value = "-1";
                }
            } else {
                if(field.name === "PERIODO"){
                    if(field.value()){
                        wdgDadosProp.widgets.forEach(function(widget){
                            widget.dataSourceFilter[1].value = field.value().start;
                            widget.dataSourceFilter[2].value = field.value().end;
                            widget.reload();
                        });
                    } else {
                        wdgDadosProp.widgets.forEach(function(widget){
                            widget.dataSourceFilter[1].value = null; 
                            widget.dataSourceFilter[2].value = null; 
                            widget.reload();
                        });
                    }
                    
                    return;
                }
            }
        }
        
        wdgDadosProp.widgets.forEach(function(widget){
            widget.dataSourceFilter[index].value = value;
            widget.reload();
        });
    };
    
    this.validaReajusteContr = function(widget){
       
        var params = [];
        
        General.callRestEngine("product#/fct_vencprevcont_widget/validarReajuste", "Row", params).then(function(response){
            
            if(response.dataset.dados.length > 0){
            
                ScreenService.showCustomDialog("Atenção!<br><br>Não será possível gerar apurações de faturamento. <br><br>Existem contratos com pendência em reajuste de valores que devem ser regularizados antes da realização das apurações.", 'WARNING', null, [
                {
                    "label": "OK",
                    "code": function() {
                        
                        widget.reload();
                    }
                }
                ]);
            
            }
            else{
                
              widget.reload();    
            }
            
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        }); 
    
    };
}
Configuration(function(ContextRegister)
{
    ContextRegister.register('Dashboard_Controller', Dashboard_Controller);
});

// FILE: ./assets/js/controllers/DashboarSMNEXT_Controller.js
function DashboarSMNEXT_Controller(ScreenService, templateManager, LibUtilities, ZhFilterPreferences, Utilities, eventAggregator){

    window.addEventListener('message',function (event) {
        if(event.data.type == 'DashboardNext-charts'){
            if(event.data.realData.PAGINA == 'Proposta'){
                ScreenService.openWindow('product#sm_00001_100_lancamento_proposta').then(function() {
                    eventAggregator.subscribe("abriuFiltroProposta", function() {
                        var wdgAtiva = templateManager.container.getActiveWidget();
                        var fieldEscopo = wdgAtiva.getField("NRTIPOORCAMENTO");
                        fieldEscopo.reload().then(function() {
                            Utilities.setValoresPadroesParaCampoSelectMultiple(fieldEscopo, [event.data.realData.ESCOPO], 'NMTIPOORCAMENTO');
                            ZhFilterPreferences.applyFilter(wdgAtiva, wdgAtiva.currentRow);
                        });
                    });
                });
            } else if(event.data.realData.PAGINA == 'Contrato'){
                ScreenService.openWindow('product#sm_00004_100_contrato').then(function() {
                    eventAggregator.subscribe("abriuFiltroContrato", function() {
                        var wdgAtiva = templateManager.container.getActiveWidget();
                        var fieldEscopo = wdgAtiva.getField("NRTIPOORCAMENTO");
                        fieldEscopo.reload().then(function() {
                            Utilities.setValoresPadroesParaCampoSelectMultiple(fieldEscopo, [event.data.realData.ESCOPO], 'NMTIPOORCAMENTO');
                            ZhFilterPreferences.applyFilter(wdgAtiva, wdgAtiva.currentRow);
                        });
                    });
                });
            }
        }
    });
    
}

window.Configuration(function(ContextRegister){
    ContextRegister.register('DashboarSMNEXT_Controller', DashboarSMNEXT_Controller);
});


// FILE: ./assets/js/controllers/ger45000_Prospect_Controller.js
function ger45000_Prospect_Controller(ScreenService, Utilities, Operador_Repository, ValidationEngine, templateManager, General, restEngine, Endereco, ger00004500_Prospect_Service, ZHPromise, CRM_Utilities, ABEvolucaoController, PartnerInfosModalController, evolutionTabController) {

    var self = this;


    var opLogado = null;
    var nomeOperador = null;
    var idOperMaster = null;
    var idUtilizaHierOper = null;
    var nrparcnegocioOperLogado = null;
    var tpOperador = null;
    var nrOrgOper = null;

    // Get Widget
    var getWidgetGridProspect = function () {
        return templateManager.container.getWidget('gridProspect');
    };

    this.wdgProspectOnEnterOnce = function (widget) {
        this.buscaOperadorLogado();
        Utilities.setRowFilterWidgetSemDefaultValue(widget, ['IDATIVO', 'NMPARCRESPROSP'], ['S', getLocalVar('CDOPERADOR')]);
    };

    /** Busca o operador logado a sessão atual 
     * @params{Object} widget de proposta
     */
    this.buscaOperadorLogado = function () {
        var deferred = ZHPromise.defer();

        Operador_Repository.getOperadorLogado().then(function (success) {
            opLogado = success[0].CDOPERADOR;
            nomeOperador = success[0].NMOPERADOR;
            nrparcnegocioOperLogado = success[0].NRPARCNEGOCIO;
            tpOperador = success[0].IDTPOPERADOR;
            nrOrgOper = success[0].NRORG;
            this.buscaInformacaoHierarquia();
            deferred.resolve(success);
        }.bind(this));

        return deferred.promise;
    };


    /* busca os dados de hierarquia do usuário logado
    * @ params{Object} widget  
    */
    this.buscaInformacaoHierarquia = function () {
        General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", {}).then(function (success) {
            if (success.dataset.fct_parametrogeral[0]) {
                var paramsFilter = [{
                    name: "CDOPERADOR",
                    operator: "=",
                    value: opLogado
                }];

                idUtilizaHierOper = success.dataset.fct_parametrogeral[0].IDUTILIZAHIEROPER;

                General.callRestEngine("product#/crud/crm_hierarquiaoper", "FilterData", paramsFilter).then(function (success) {
                    if (success.dataset.crm_hierarquiaoper[0]) {
                        idOperMaster = success.dataset.crm_hierarquiaoper[0].IDOPERMASTER;
                    }
                }.bind(this));
            }
        }.bind(this));
    };

    /* Recarrega o field caso o dataSource tenha sido alterado 
    * @params{Object} field Nome Operador
    */
    this.recarregaDadosField = function (fieldNmOp) {
        var widgetAtiva = templateManager.container.getActiveWidget();
        var fieldParc = widgetAtiva.getField("NRPARCNEGOCIO");

        if (widgetAtiva.name === "filterPopup") {
            fieldParc.dataSourceFilter[1] = {
                name: "P_CDOPERADOR",
                operator: "=",
                value: opLogado
            };
        }

        fieldNmOp.reload();
    };


    this.validaInscricao = function (widget, lead, prospect, fromProspect) {
        var inscricao = widget.getField("NRINSJURCAN");
        var tipoInscricao = widget.getField("IDTPIJURCAN");
        if (fromProspect) {
            inscricao = widget.getField("NRINSJURPROS");
            tipoInscricao = widget.getField("IDTPIJURPROS");
        }
        //Variavel para validacao de cpnj raiz
        var valido = true;
        var valueInscricao = inscricao.value().replace(/[^0-9]/g, '');

        CRM_Utilities.validaInscricao(widget, inscricao.name, tipoInscricao.name);
        if (!Util.isEmptyOrBlank(valueInscricao)) {
            this.completeValidation(widget, inscricao, tipoInscricao, lead, prospect, fromProspect);
        }
        if (!fromProspect)
            this.changeLabelDTNASCIFUNDPARC(widget);

    };


    this.completeValidation = function (widget, inscricao, tipoInscricao, lead, prospect, fromProspect) {
        if (fromProspect) {
            widget.currentRow.INSCRICAO = widget.currentRow.NRINSJURPROS.toString().replace(/[^0-9]/g, '');
            widget.currentRow.NOME = widget.currentRow.NMRAZSOCPROS.toString().replace(' ', "%").toUpperCase();
            widget.currentRow.IDTPIJURCAN = widget.currentRow.IDTPIJURPROS;
        } else {
            widget.currentRow.INSCRICAO = widget.currentRow.NRINSJURCAN.toString().replace(/[^0-9]/g, '');
            widget.currentRow.NOME = widget.currentRow.NMCANDIDATO.toString().replace(' ', "%").toUpperCase();
        }

        widget.currentRow.LEAD = lead || false;
        widget.currentRow.PROSPECT = prospect || false;
        this.verificaLeadExistente(widget.currentRow).then(function (response) {
            if (Array.isArray(response.dataset.DADOS.LEADS)) {
                var leads = response.dataset.DADOS.LEADS;
                var nomes = '';
                leads.forEach(function (lead) {
                    nomes += '<br>' + (lead.NMPRINCIPALPARC || lead.NMCANDIDATO) + '<hr>';
                });
                ScreenService.showMessage('Operação bloqueada! <br><br> Esta Inscrição já está vinculada ao(s) seguinte(s) Parceiro(s) de Negócio: <br>' + nomes);
                inscricao.clearValue();
                tipoInscricao.clearValue();
            } else {
                this.autofillData(widget, fromProspect);
            }
        }.bind(this));

    };

    /**
     * Verifica se já existem parceiros vinculados
     * aquele CNPJ ou nome
    */
    this.verificaLeadExistente = function (currentRow) {
        var paramsRestEngine = {
            "requestType": "Row",
            "row": currentRow
        };
        return restEngine.post('/validaLead', paramsRestEngine);
    };


    this.autofillData = function (widget, fromProspect) {

        var currentRow = widget.currentRow;
        if (fromProspect) {
            currentRow.NUMBERINSC = currentRow.NRINSJURPROS ? currentRow.NRINSJURPROS.replace(/[^0-9]/g, '') : "";
        } else {
            currentRow.NUMBERINSC = currentRow.NRINSJURCAN ? currentRow.NRINSJURCAN.replace(/[^0-9]/g, '') : "";
        }

        if (currentRow.NUMBERINSC.length === 14) {

            ScreenService.confirmMessage("Deseja buscar as informações pelo CNPJ?", "question", function () {

                var parametroRestEngine = {
                    'requestType': 'Row',
                    'row': { 'NUMBERINSC': currentRow.NUMBERINSC }
                };

                restEngine.post('/consultaRF', parametroRestEngine,
                    function (success) {
                        var dadosReceitaFederal = success.dataset.CONSULTA;
                        if (Boolean(dadosReceitaFederal.Mensagem) && dadosReceitaFederal.Mensagem === 'Saldo Insuficiente ou Contrato com Pendencias') {
                            ScreenService.showMessage(dadosReceitaFederal.Mensagem);
                        }
                        else if (dadosReceitaFederal.SituacaoRFB !== "ATIVA") {
                            ScreenService.showMessage("Lead Inativo na receita.");
                        } else {
                            this.fillWithCurrentRowReturnedValues(widget, dadosReceitaFederal);
                            ScreenService.showMessage("Consulta realizada com sucesso.");
                        }
                    }.bind(this), undefined,
                    function (error) {
                        console.log(error);
                    }, null, 3000000
                );
                ScreenService.closePopup();
            }.bind(this));
        }

    };

    this.fillWithCurrentRowReturnedValues = function (widget, returnedValues) {

        var convertWidget = templateManager.container.getWidget('gridProspect');

        var currentRow = convertWidget.currentRow;


        currentRow.NMRAZSOCPROS = returnedValues.RazaoSocial;
        currentRow.NMFANTPROS = returnedValues.NomeFantasia;

        currentRow.NMPAIS = "Brasil";
        currentRow.CDPAIS = "0055";
        currentRow.NRCEPENDERECO = returnedValues.Enderecos.CEP;
        currentRow.DSENDERECO = returnedValues.Enderecos.Logradouro;
        currentRow.NRIMOVELENDERECO = parseInt(returnedValues.Enderecos.Numero);
        currentRow.DSCOMPLEENDERECO = returnedValues.Enderecos.Complemento;

        this.localizationFill(currentRow, returnedValues);

        convertWidget.getField('NMESTADO').readOnly = false;
        convertWidget.getField('NMMUNICIPIO').readOnly = false;
        convertWidget.getField('NMBAIRRO').readOnly = false;

        /*
        currentRow.DSTELEFONECAN = (currentRow.DSTELEFONECAN === "") ? returnedValues.Telefone : currentRow.DSTELEFONECAN;
        if((currentRow.DSEMAILCAN === "") && typeof returnedValues.Email !== "undefined"){
            currentRow.DSEMAILCAN = returnedValues.Email;
        }
        currentRow.DATAFUNDACAO = returnedValues.DataFundacao;
        currentRow.LATITUDE = returnedValues.Enderecos.GeoLocalizacao.Latitude;
        currentRow.LONGITUDE = returnedValues.Enderecos.GeoLocalizacao.Longitude;
        
        */
    };

    this.localizationFill = function (currentRow, values) {

        var parametroRestEngine = {
            'requestType': 'Row',
            'row': { 'RESULTCONSUL': values }
        };

        restEngine.post('/ConfirmationLocation', parametroRestEngine,
            function (success) {
                var localization = success.dataset.LOCALIZACAO;
                currentRow.NMESTADO = localization[0].NMESTADO;
                currentRow.SGESTADO = localization[0].SGESTADO;
                currentRow.NMMUNICIPIO = localization[1].NMMUNICIPIO;
                currentRow.CDMUNICIPIO = localization[1].CDMUNICIPIO;
                currentRow.NMBAIRRO = (!Util.isUndefined(localization[2].NMBAIRRO)) ? localization[2].NMBAIRRO : values.Enderecos.BAIRRO;
                currentRow.CDBAIRRO = (!Util.isUndefined(localization[2].CDBAIRRO)) ? localization[2].CDBAIRRO : null;

            }, undefined,
            function (error) {
                ScreenService.showMessage("Operação não concluida, Tente Novamente");
            }
        );

    };

    this.setAuditInfo = function (widget) {  /*Operador_Repository,*/

        var currentRow = widget.currentRow;

        if (widget.currentRow.__is_new) {
            currentRow.DTINCLUSAO = self.momentNow();
            currentRow.CDOPERINCLUSAO = currentRow.NMOPERLOGGED;
        } else {
            currentRow.DTULTATU = self.momentNow();
            currentRow.CDOPERULTATU = currentRow.CODOPERLOGGED;
            currentRow.NMOPERULTATU = currentRow.NMOPERLOGGED;
        }
    };

    this.setOperatorLoggedInCurrentRow = function () {

        /* Operador_Repository.getOperadorLogado().then(function(operator)
         {
             if(self.operatorIsValid(operator) === false)
             {
                 return;
             }
             var widget = templateManager.container.getActiveWidget();
             
             widget.currentRow.CODOPERLOGGED = operator[0].CDOPERADOR;
             widget.currentRow.NMOPERLOGGED = operator[0].NMOPERADOR;
             
         });*/

        var widget = templateManager.container.getActiveWidget();

        widget.currentRow.CODOPERLOGGED = opLogado;
        widget.currentRow.NMOPERLOGGED = nomeOperador;


    };

    this.getFilterPosition = function (dataSourceFilter, name) {
        var indexFilterObject;

        dataSourceFilter.forEach(function (value, i) {
            if (value.name === name) {
                indexFilterObject = i;
                return;
            }
        });
        return indexFilterObject;
    };

    this.operatorIsValid = function (operator) {
        if (operator[0] === undefined || operator[0] === null) {
            ScreenService.showMessage("Erro ao realizar a operação.");
            return false;
        }
        return true;
    };

    this.momentNow = function () {
        return moment(new Date()).format('DD/MM/YYYY HH:mm:ss');
    };

    this.createEqualFilter = function (name, value) {
        var filter = [
            {
                "name": name,
                "operator": "=",
                "value": value
            }
        ];
        return filter;
    };

    this.validationPhone = function (field) {
        var numberSizeReceived = field.value().replace(/[^0-9]/g, '').length;
        if (numberSizeReceived !== 10 && numberSizeReceived !== 11 && numberSizeReceived !== 12) {
            ScreenService.showMessage("Telefone Inválido");
            field.clearValue();
            return false;
        }
    };

    this.setAddressDefaultValue = function (currentRow) {
        currentRow.CDPAIS = "";
        currentRow.SGESTADO = "";
        currentRow.CDMUNICIPIO = "";
        currentRow.CDBAIRRO = "";
    };

    this.setWidgetsVisibility = function (widgetParent, value) {
        widgetParent.widgets.forEach(function (widget) {
            widget.isVisible = value;
        });
    };

    this.verificaEndereco = function (widget) {
        var fields = widget.fields;
        var widgetGridProspect = getWidgetGridProspect();

        widget.currentRow.REPLICAENDERECO = false;
        widget.currentRow.CODOPERLOGGED = widgetGridProspect.currentRow.CODOPERLOGGED;
        fields.forEach(function (field) {
            if (field.fieldGroup == "ENDER_INTER") {
                if (!Util.isEmptyOrBlank(field.value()))
                    widget.currentRow.REPLICAENDERECO = true;
                else
                    field.setValue('');
            }
        });
    };

    this.saveProspectAndSetAudit = function () {

        var widget = templateManager.container.getActiveWidget();
        var incluindo = widget.currentRow.__is_new;
        widget.currentRow.NRINSJURPROS = widget.currentRow.NRINSJURPROS.toString().replace(/[^0-9]/g, '');

        this.setAuditInfo(widget);

        return General.widgetSave(widget).then(function (response) {
            if (response && response.dataset && response.dataset.DADOS) {
                widget.dataSourceFilter.push({ name: 'CDPROSPECT', operator: '=', value: response.dataset.DADOS.PROSPECT });
                widget.reload().then(function () {
                    widget.dataSourceFilter.pop();

                    if (incluindo == true) {
                        ScreenService.confirmMessage("Prospect criado com sucesso.<br><br>Deseja ir para a tela de lançamento de Proposta?", 'CONFIRMATION',
                            function (success) {
                                ScreenService.openWindow('product#sm_00001_100_lancamento_proposta');
                            },
                            function (fail) {
                            });
                    }
                });
            } else {
                widget.reload();
            }
            widget.currentRow.DIASFIMCONTRATO = this.calculaDiasFimContrato(widget.currentRow.DTFIMVIGENCIA);
        }.bind(this));
    };

    this.openSwipeFundirAndSetName = function () {
        var widgetGridProspect = getWidgetGridProspect();
        var widget = templateManager.container.getWidget('FUNDIRPROSP');

        ScreenService.openSwipe(widget);
        ScreenService.closePopup();

        widget.activate();
        widget.newRow();

        widget.currentRow.NMCANDPROSPECT = widgetGridProspect.currentRow.NMRAZSOCPROS;

    };

    this.closeWidgetFundirProsp = function () {
        var widget = templateManager.container.getActiveWidget();

        ScreenService.closeSwipe().then(function () {
            self.cleanWidgetFields(widget);
            widget.parent.view();
            widget.parent.activate();
        });
    };

    this.cleanWidgetFields = function (widget) {
        widget.fields.forEach(function (field) {
            field.value("");
        });
    };

    /**
     * Inicia o mvc para realizar um insert de tarefas
     * na widget de Evolução
    */
    this.evolucaoNewRow = function () {

        var widget = templateManager.container.getWidget("evolucao");
        widget.getField('NMRESULTEVOL').readOnly = false;

        General.widgetNewRow(widget);

        var currentRow = widget.currentRow;

        currentRow.DTHREVOLCONT = (currentRow.DTHREVOLCONT) ? currentRow.DTHREVOLCONT : moment(new Date()).format('DD/MM/YYYY HH:mm');
        currentRow.NMOPERADOR = nomeOperador;
        currentRow.CDOPEREVOL = opLogado;
        currentRow.NRORG = nrOrgOper;

        //    ABEvolucaoController.setEvolDateChangePermission(widget);
        var widgetGridProspect = getWidgetGridProspect();

        currentRow.DTINCLUSAO = self.momentNow();
        currentRow.NRSEQCONTATO = widgetGridProspect.currentRow.NRPARCNEGOCIO;

        currentRow.NRORG = nrOrgOper;
    };



    this.setContactReasonFilter = function (field) {

        var fieldFilter = field.dataSourceFilter;

        var nrorgFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "P_NRORG");
        //var cdoperadorFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "P_CDOPERADOR");

        fieldFilter[nrorgFilterIndex].value = nrOrgOper;
        //fieldFilter[cdoperadorFilterIndex].value = opLogado;

        field.reload();

    };

    this.setInterlocutorFilter = function (field) {
        var widgetGridProspect = getWidgetGridProspect();
        var fieldFilter = field.dataSourceFilter;
        var parentWidgetCurrentRow = field.widgetGridProspect.currentRow;

        var nrnparcnegocioFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "NRPARCNEGOCIO");

        fieldFilter[nrnparcnegocioFilterIndex].value = parentWidgetCurrentRow.NRPARCNEGOCIO;

        field.reload();

    };

    this.setResultFilter = function (field) {

        var fieldFilter = field.dataSourceFilter;

        var nrorgFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "P_NRORG");
        //var cdoperadorFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "P_CDOPERADOR");

        fieldFilter[nrorgFilterIndex].value = nrOrgOper;
        //fieldFilter[cdoperadorFilterIndex].value = opLogado;

        field.reload();
    };



    /**
     * Organiza os campos para salvar a evolução
     * e verifica tarefa já aberta.
     * 
     * @param {object} widget widget de evolução
    */
    this.saveEvolucao = function (widget) {

        var widgetGridProspect = getWidgetGridProspect();
        var currentRow = widget.currentRow;

        currentRow.CDOPEREVOL = opLogado;
        currentRow.IDGERAFATURA = 'NAO';
        currentRow.IDTPEVOLCONT = 'P';
        currentRow.IDGERAFATURA = 'NAO';
        currentRow.IDANEXOEVOLCONT = 'N';
        currentRow.IDSENEVOCONT = "S";
        currentRow.NRPARCNEGOCIO = widgetGridProspect.currentRow.NRPARCNEGOCIO;

        currentRow.CDFILIAL = "";

        widget.getField('NMRESULTEVOL').readOnly = true;

        if (widget.isValid()) {
            if (!currentRow.NRRELACIONAPARC) {
                currentRow.NRRELACIONAPARC = null;
            }

            this.selectProspectContato(widget).then(function (response) {

                if (currentRow.__is_new) {
                    this.getParentField(widget, response.dataset.DADOS.NRSEQCONTATO);
                }

                widget.currentRow.__columns_to_return = ["NRSEQEVOLCONT", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
                General.widgetSave(widget).then(function (response) {
                    if (response.dataset && response.dataset.columnsToReturn[0]) {
                        widget.currentRow.NRSEQEVOLCONT = response.dataset.columnsToReturn[0].NRSEQEVOLCONT;
                        widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                        widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                        widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                        widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                    }
                });

            }.bind(this));
        }
    };

    /**
     * Salva evolução e gera Tarefa
     * 
     * @param {object} widget widget de evolução
    */
    this.saveEvolutionAndInsertTask = function (widget, currentRow) {
        this.insertRowNextTask(widget).then(function (response) {

            if (response) {

                currentRow.NRSEQFILACONTA = response.dataset.DADOS.NRSEQFILACONTA;

            } else if (!currentRow.NRSEQFILACONTA) {

                currentRow.NRSEQFILACONTA = "";

            }

            General.widgetSave(widget).then(function (response) {

                this.cleanNextTaskFieldValues(widget);

                if (response.dataset.DADOS && response.dataset.DADOS.NRSEQEVOLCONT) {
                    currentRow.NRSEQEVOLCONT = response.dataset.DADOS.NRSEQEVOLCONT;
                }
                //Se a evolução é de qualificação
                if (widget.SET_QUALIFICADO) {
                    this.callSetIdQualificadoProspect(widget.SET_QUALIFICADO);
                    delete widget.preventLoad;
                    delete widget.SET_QUALIFICADO;
                }
                if (!Util.isEmptyOrBlank(widget.currentRow.NRSEQINTERLOCUTOR) && widget.currentRow.IDENVIAEMAIL === 'S' && widget.currentRow.DSCAMINHO[0]) {
                    ScreenService.confirmMessage('Deseja enviar um email para ' + widget.currentRow.NMINTERLOCUTOR + ' com o anexo ' + widget.currentRow.DSCAMINHO[0].name + '?', 'QUESTION',
                        function (yes) {
                            evolutionTabController.openWidgetSendMail(widget);
                        });
                }
                widget.reload();
                this.setTaskEndDate(widget);

            }.bind(this)).catch(function (error) {
                if (error.indexOf('REQUIRE_INTERLOCUTOR') !== -1)
                    ScreenService.showMessage("Operação bloqueada! <br><br> Favor informar o interlocutor.");
                else
                    ScreenService.showMessage(error);
            });

        }.bind(this));
    };

    /**
     * Verifica se já existem tarefas relacionadas
     * aquele parceiro
     * 
     * @param {object} widget widget de evolução
    */
    this.verificaTarefasAbertas = function (currentRow) {
        var paramsRestEngine = {
            "requestType": "Row",
            "row": currentRow
        };
        return restEngine.post('/verificaTarefas', paramsRestEngine);
    };

    /**
     * Função para pegar as informações da tarefa
     * na widget de evolução e setar a data de
     * conclusão da tarefa no registro da própria
     * tarefa.
     * 
     * @param {object} widget widget ativa que contém
     * as informações da taréfa que será concluida.
    */
    this.setTaskEndDate = function (widget) {

        if (widget.COMPLETETASK) {

            ger00004500_Prospect_Service.callForSetTaskEndDate(widget).then(function () {

                ScreenService.showMessage("Tarefa Concluída com sucesso.");
                this.deleteTaskInfosInEvolutionWidget(widget);
                this.activateTaskWidget();

            }.bind(this));
        }

    };

    /**
     * Função para ativar a widget de agenda
     * de tarefas.
    */
    this.activateTaskWidget = function () {

        var taskWidget = templateManager.container.getWidget("posvendagrid");

        taskWidget.activate();
    };

    /**
     * Função para deletar as informações da
     * tarefa a ser concluída da widget de evolução.
     * 
     * @param {object} widget widget de evolução.
    */
    this.deleteTaskInfosInEvolutionWidget = function (widget) {

        delete widget.COMPLETETASK;
        delete widget.NRSEQFILACONTA;

    };

    /**
     * Seta o operador para preparar a proxima tarefa
     * da widget de evolução
     * 
     * @param {object} currentRow pega a currentRow
     * da evolução
    */
    this.getParentField = function (widget, nrseqcontato) {
        var widgetGridProspect = getWidgetGridProspect();
        widget.currentRow.NRPARCNEGOCIO = widgetGridProspect.currentRow.NRPARCNEGOCIO;
        widget.currentRow.NRSEQCONTATO = nrseqcontato;
        widget.currentRow.NRCONTATO = nrseqcontato;

    };

    /**
     * Inicia o mvc para realizar select de CONTATO
     * 
     * @param {object} widget widget para passar a currentRow
    */
    this.selectProspectContato = function (widget) {
        return ger00004500_Prospect_Service.callForSelectProspect(widget.currentRow);
    };


    this.actionsNewRow = function (widName) {

        this.closeActionsPopup().then(function () {
            var widget = templateManager.container.getWidget(widName);
            this.activateWidget(widget).then(function () {
                setTimeout(function () {
                    widget.newRow();
                }, 100);
            }.bind(this));
        }.bind(this));


    };

    this.actionEvolutionNewRow = function (widName) {

        this.closeActionsPopup().then(function () {//TESTEWEL
            var widget = templateManager.container.getWidget(widName);

            this.activateWidget(widget).then(function () {

                setTimeout(function () {
                    this.evolucaoNewRow();
                }.bind(this), 100);

            }.bind(this));

        }.bind(this));


    };

    this.closeActionsPopup = function () {
        var deferred = ZHPromise.defer();
        ScreenService.closePopup();
        deferred.resolve();
        return deferred;
    };

    this.activateWidget = function (widget) {
        var deferred = ZHPromise.defer();
        widget.activate();
        deferred.resolve();
        return deferred;
    };

    this.filterOnOpenEvol = function (widget) {
        this.selectProspectContato(widget).then(function (response) {

            var widDataSource = widget.dataSourceFilter;

            var nrseqcontatoFilter = CRM_Utilities.getFilterPosition(widDataSource, "NRSEQCONTATO");

            widDataSource[nrseqcontatoFilter].value = response.dataset.DADOS.NRSEQCONTATO;
            widget.reload();

        }.bind(this));
    };

    this.setEvolutionWidgetFilter = function (widget) {
        var widgetGridProspect = getWidgetGridProspect();
        widget.dataSourceFilter[0].value = widgetGridProspect.currentRow.NRPARCNEGOCIO;

        widget.reload();

    };

    this.reloadTab = function (widget) {
        var widgetGridProspect = getWidgetGridProspect();
        if (widget.currentFilter && widget.currentFilter !== widgetGridProspect.currentRow.NRPARCNEGOCIO)
            widget.reload();
        else if (!widget.currentFilter)
            widget.reload();

        this.setCurrentFilter(widget);
    };

    this.setCurrentFilter = function (widget) {
        var widgetGridProspect = getWidgetGridProspect();
        widget.currentFilter = widgetGridProspect.currentRow.NRPARCNEGOCIO;
    };


    this.getProspectNrSeqContato = function (widget) {

        this.selectProspectContato(widget).then(function (response) {

            widget.currentRow.NRSEQCONTATO = response.dataset.DADOS.NRSEQCONTATO;

        });

    };

    this.setResponsibleOperatorFilter = function (field) {

        var fieldFilter = field.dataSourceFilter;

        var nrorgFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "P_NRORG");
        var nrparcnegocioFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "P_NRPARCNEGOCIO");

        fieldFilter[nrorgFilterIndex].value = nrOrgOper;
        fieldFilter[nrparcnegocioFilterIndex].value = opLogado;


        field.reload();

    };


    this.setColumsToReturnInCurrentRow = function (currentRow, arrayColumns) {

        currentRow.__columns_to_return = arrayColumns;

    };

    this.insertNrseqcontatoForNewOpportunity = function (widget) {

        var currentRow = widget.currentRow;

        ger00004500_Prospect_Service.callForInsertNrseqcontatoForNewOpportunity(currentRow);

    };

    this.convertProspectIntoClient = function (widget) {

        var currentRow = widget.currentRow;

        currentRow.CDOPERADOR = currentRow.CODOPERLOGGED;

        this.validacaoCNPJCliente(widget.currentRow.NRINSJURPROS, widget.currentRow.NRORG).
            then(function (response) {
                if (response.dataset && response.dataset.cnpjcliente.valid === true) {

                    this.updatePartnerType(currentRow);

                    this.insertPartnershipOfTypeClient(currentRow);

                    this.insertClient(currentRow).then(function () {

                        this.insertClientAddress(currentRow);
                        ScreenService.showMessage("Prospect convertido com sucesso.");
                        //BEGIN RD Station
                        var contactWidgets = templateManager.container.getWidget('WDGT_INTERLOCUTOR');
                        contactWidgets.dataSourceFilter.forEach(function (filter) {
                            if (filter.name === 'NRPARCNEGOCIO')
                                filter.value = widget.currentRow.NRPARCNEGOCIO;
                        });
                        contactWidgets.reload().then(function () {
                            if (!Util.isEmptyOrBlank(contactWidgets.dataSource.data)) {
                                contactWidgets.dataSource.data.forEach(function (contact) {
                                    restEngine.post('/IntegracaoRDStation/feedBackConversion', { 'requestType': 'Row', 'row': { EVENT_NAME: 'Cadastro convertido em cliente no CRM', EMAIL: contact.EMAILINTERLOC } }).
                                        then(function () {
                                            restEngine.post('/IntegracaoRDStation/changeFunnel', { 'requestType': 'Row', 'row': 'Client', EMAIL: contac.EMAILINTERLOC });
                                        });
                                });
                            }
                        });
                        //END RD Station
                        ScreenService.closePopup();
                        ScreenService.closeSwipe();
                        widget.reload();
                    }.bind(this));

                    this.deactivateProspect(currentRow);

                } else if (response.dataset && Array.isArray(response.dataset.cnpjcliente.valid)) {

                    var clientes = response.dataset.cnpjcliente.valid;
                    var nomes = '';
                    clientes.forEach(function (cliente) {
                        nomes += '<br>' + cliente.NMRAZSOCCLIE;
                    });
                    ScreenService.showMessage("Operação bloqueada! <br><br> Este CNPJ já está vinculado ao(s) seguinte(s) clientes(s): <br>" + nomes);

                }
            }.bind(this));
    };

    /**
     * Verifica se já existem clientes vinculados
     * com aquele CNPJ
    */
    this.validacaoCNPJCliente = function (CNPJ, NRORG) {
        var paramsRestEngine = {
            "requestType": "Row",
            "row": { 'CNPJ': CNPJ, 'NRORG': NRORG }
        };
        return restEngine.post('/validaCNPJCliente', paramsRestEngine);
    };

    this.updatePartnerType = function (currentRow) {

        ger00004500_Prospect_Service.callForUpdatePartnerType(currentRow);

    };

    this.insertPartnershipOfTypeClient = function (currentRow) {

        ger00004500_Prospect_Service.callForInsertPartnershipOfTypeClient(currentRow);

    };


    this.insertClient = function (currentRow) {

        return ger00004500_Prospect_Service.callForInsertClient(currentRow);

    };

    this.insertClientAddress = function (currentRow) {

        this.selectProspectAddress(currentRow).then(function (response) {
            var address = response.dataset.DADOS.ADDRESS;
            address.forEach(function (addressObject) {
                var reserveCurrentRow = currentRow;

                reserveCurrentRow.NRENDERECOPARC = addressObject.NRENDERECOPARC;
                reserveCurrentRow.CDPAIS = addressObject.CDPAIS;
                reserveCurrentRow.SGESTADO = addressObject.SGESTADO;
                reserveCurrentRow.CDMUNICIPIO = addressObject.CDMUNICIPIO;
                reserveCurrentRow.CDBAIRRO = addressObject.CDBAIRRO;
                reserveCurrentRow.DSENDECLIE = addressObject.DSENDEPROS;
                reserveCurrentRow.IDTPENDECLIE = addressObject.IDTPENDEPROS;
                reserveCurrentRow.NMBAIRCLIE = addressObject.NMBAIRPROS;
                reserveCurrentRow.DSCOMPENDCLI = addressObject.DSCOMPENDPRO;
                reserveCurrentRow.NRCEPCLIE = addressObject.NRCEPPROS;
                reserveCurrentRow.NRTELECLIE = addressObject.NRTELEPROS;
                reserveCurrentRow.NRRAMALCLIE = addressObject.NRRAMALPROS;
                reserveCurrentRow.DSEMAILCLIE = addressObject.DSEMAILPROS;

                ger00004500_Prospect_Service.callForInsertClientAddress(reserveCurrentRow);

            });

        });

    };

    this.selectProspectAddress = function (currentRow) {
        return ger00004500_Prospect_Service.callForSelectProspectAddress(currentRow);
    };

    this.deactivateProspect = function (currentRow) {

        ger00004500_Prospect_Service.callForDeactivateProspect(currentRow);

    };

    this.setEvolutionInterlocutorFieldFilter = function (field) {
        var widgetGridProspect = getWidgetGridProspect();
        var fieldFilter = field.dataSourceFilter;
        var prospectPartnershipNumber = field.widgetGridProspect.currentRow.NRPARCNEGOCIO;

        var nrparcnegocioFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "NRPARCNEGOCIO");

        fieldFilter[nrparcnegocioFilterIndex].value = prospectPartnershipNumber;

        field.reload();

    };


    /**
     * Função para abrir a widget de seleção
     * de arquivos.
    */
    this.openAnnexSwipe = function () {

        var fileUploadWidget = templateManager.container.getWidget('WDGT_SELECAO_ARQUIVOS');

        fileUploadWidget.isVisible = true;
        ScreenService.openSwipe(fileUploadWidget);

    };

    /**
     * Função para salvar os anexos selecionados
     * ao caso em questão.
     * 
     * @param {object} widget widget onde é
     * feito a seleção dos anexos.
    */
    this.uploadEvolutionFiles = function (widget) {
        var widgetGridProspect = getWidgetGridProspect();


        var files = widget.currentRow.DSCAMINHO;

        var rows = [];

        files.forEach(function (file, fileIndex) {

            rows.push(this.setValuesForFileUpload(widget, fileIndex));

        }.bind(this));
        General.callRestEngine("product#/crm_anexocontato/save", "DataSet", rows).then(function () {
            this.closeAnnexSwipe(widget);
            widgetGridProspect.reload();

        }.bind(this));
    };

    /**
     * Função para retornar uma row configurada
     * com as informações do anexo referente
     * ao ID, recebido como parâmetro, no array
     * de anexos na currentRow.
     * 
     * @param {object} widget widget onde foi selecionado
     * os anexos.Array
     * 
     * @param {int} fileIndex index do anexo no array de 
     * anexos presente na currentRow.
     * 
     * @return {object} currentRow row configurada com as informações
     * do anexo referente ao ID recebido.
    */
    this.setValuesForFileUpload = function (widget, fileIndex) {

        var currentRow = angular.copy(widget.currentRow);
        //var evolutionWidget = widget.parent.parent;
        var widgerGridProspect = getWidgetGridProspect();

        currentRow.NRSEQCONTATO = widgerGridProspect.currentRow.NRSEQCONTATO;
        currentRow.NMARQUIVO = this.getFileNameWithoutExtension(currentRow.DSCAMINHO[fileIndex].name);
        currentRow.DSTIPO = this.getFileExtension(currentRow.DSCAMINHO[fileIndex].name);
        currentRow.VRTAMANHO = currentRow.DSCAMINHO[fileIndex].size;
        currentRow.IDATIVO = "S";
        currentRow.NRORG = nrOrgOper;
        currentRow.CDOPERADOR = opLogado;
        currentRow.IDORIGEM = "EVOLUCAO";
        currentRow.NRORIGEM = widgerGridProspect.currentRow.NRSEQEVOLCONT;

        return currentRow;

    };

    /**
     * Função para retornar a extenção de um arquivo
     * extraida do seu nome.
     * 
     * @param {string} fileName nome completo do arquivo.
    */
    this.getFileExtension = function (fileName) {

        return fileName.replace(/^(.*)\.(.*)$/, "$2");

    };

    /**
     * Função para retornar o nome do arquivo sem
     * sua extenção.
     * 
     * @param {string} fileName nome completo do arquivo.
    */
    this.getFileNameWithoutExtension = function (fileName) {

        return fileName.replace(/^(.*)\.(.*)$/, "$1");

    };

    /**
     * Função para fechar o swipe de upload de imagens.
     * 
     * @param {object} widget widget de upload de imagens.
    */
    this.closeAnnexSwipe = function (widget) {
        ScreenService.closeSwipe().then(function () {

            widget.isVisible = false;
            widget.parent.list();
            widget.parent.activate();

        });
    };

    /**
     * Função para fazer o download de uma arquivo
     * salvo no servidor.
     * 
     * @param {object} widget widget que contém
     * as informações do arquivo a ser baixado 
     * em sua currentRow.
    */
    this.downloadFile = function (widget) {
        var nmarquivo = widget.currentRow.NMARQUIVO;
        var dscaminho = widget.currentRow.DSCAMINHOARQ;
        var type = widget.currentRow.DSTIPO;

        Utilities.downloadFile(nmarquivo, dscaminho, type);
    };


    /**
     * Função para deletar uma evolução e 
     * seus anexos.
     * 
     * @param {object} widget widget ativa, no
     * modo view, mostrando os dados da evolução
     * que será deletada.
    */
    this.deleteEvolution = function (widget) {

        var currentRow = widget.currentRow;

        currentRow.NRORIGEM = currentRow.NRSEQEVOLCONT;

        General.deleteCrud(widget).then(function () {

            this.deleteEvolutionFilesByOriginNumber(widget);

        }.bind(this));

    };

    /**
     * Função para deletar os anexos de uma evolução
     * pelo número de origem do anexo.
     * 
     * @param {object} widget widget ativa, no
     * modo view, mostrando os dados da evolução
     * que será deletada.
    */
    this.deleteEvolutionFilesByOriginNumber = function (widget) {

        return ger00004500_Prospect_Service.callForDeleteEvolutionFilesByOriginNumber(widget);

    };


    /**
     * Função para editar uma evolução.
     * 
     * @param {object} widget widget ativa, no
     * modo de visualização, com as informações
     * da evolução a ser editada.
    */
    this.editEvolution = function (widget) {

        General.widgetEdit(widget);

        widget.getField('NMRESULTEVOL').readOnly = true;
        var fieldDTPLANFILACONTA = widget.getField('DTPLANFILACONTA');
        if (fieldDTPLANFILACONTA)
            widget.getField('DTPLANFILACONTA').validations = {};

        //ABEvolucaoController.setEvolDateChangePermission(widget);

    };


    /**
     * Função para desativar a tarefa em caso.
     * 
     * @param {object} widget widget ativa, no modo
     * de visualização, com as informações da tarefa
     * a ser desativada.
    */
    /*
    this.deactivateTask = function(widget){
        
        return ger00004500_Prospect_Service.callForDeactivateTask(widget);
            
    };
    */

    /**
     * Função para setar o bloqueio de um campo.
     * 
     * @param {object} field field a ser manipulada.
     * 
     * @param {boolean} isLocked variável que apontará
     * se o campo será bloqueado ou não;
    */
    this.setFieldLock = function (field, isLocked) {

        field.readOnly = isLocked;

    };

    /**
     * Função para setar o bloqueio do campo 'Código'
     * na widget de interlocutores.
     * 
     * @param {object} widget widget de interlocutores do prospect
     * em caso.
     * 
     * @param {boolean} isLocked variável que apontará
     * se os campos serão bloqueados ou não;
    */
    this.setNegotiationPhaseLock = function (widget, isLocked) {

        var interlocCodeField = widget.getField("NMSEQFASENEGO");

        this.setFieldLock(interlocCodeField, isLocked);

    };

    this.setIDATIVO = function (widget) {
        var filter;
        if (widget.name !== 'gridProspect') {
            filter = templateManager.container.getWidget('gridProspect').__filterPopup;
        } else {
            filter = widget.__filterPopup;
        }
        filter.currentRow.IDATIVO = 'S';
        filter.reload();
    };

    /**
     * Dispara os eventos necessários no evento
     * "WidgetAfterMoveRow" da widget principal
     * (gridProspect).
     * 
     * @param {object} widget widget principal da tela 
     * prospect.
    */
    this.shootsMainWidgetEventsAfterMoveRow = function (widget) {

        this.setOperatorLoggedInCurrentRow();
        this.getProspectNrSeqContato(widget);
        //   this.hideActionsToReservedProspects(widget);
        //   this.setLabelActionQualificado(widget);
    };

    /**
     * Esconde as actions da tela de prospect quando
     * um prospect reservado é selecionado.
     * 
     * @param {object} widget widget principal da tela
     * de prospect (gridProspect).
     * 
    */
    this.hideActionsToReservedProspects = function (widget) {

        var currentRow = widget.currentRow;

        if (this.isReservedProspect(widget) && nrparcnegocioOperLogado !== currentRow.NRPARCRESPROSP) {

            this.hideFunctionalActions();
            currentRow.BLOCKEDPROSPECT = true;
            ScreenService.showMessage("Operação bloqueada.<br/><br/>Este prospect já sendo trabalhado por outro operador.");

        } else if (!Boolean(currentRow.BLOCKEDPROSPECT) || currentRow.BLOCKEDPROSPECT === false) {

            this.returnContainerDefaultAction(widget);
            currentRow.BLOCKEDPROSPECT = false;

        }

    };

    /**
     * Verifica se um prospect está reservado
     * de acordo com a paramentrização do CRM.
     * 
     * @param {object} widget widget principal da tela
     * de prospect (gridProspect).
    */
    this.isReservedProspect = function (widget) {

        var crmParametrization = CRM_Utilities.getCrmParametrization();

        var currentRow = widget.currentRow;

        var reserveTnitDate = CRM_Utilities.subtractDateByDays(currentRow.DTINIRESERVAPROSP, 1);
        var reserveEndDate = CRM_Utilities.addDaysToDate(reserveTnitDate, crmParametrization.NRMAXDIASPROSPBLOQ + 1, 'DD/MM/YYYY');

        var currentDate = CRM_Utilities.getDataHoraAtual();
        var formatedCurrentDate = CRM_Utilities.changeDateFormat(currentDate, 'DD/MM/YYYY HH:mm', 'YYYY-MM-DD');
        var formatedReserveTnitDate = CRM_Utilities.changeDateFormat(reserveTnitDate, 'DD/MM/YYYY HH:mm', 'YYYY-MM-DD');
        var formatedReserveEndDate = CRM_Utilities.changeDateFormat(reserveEndDate, 'DD/MM/YYYY HH:mm', 'YYYY-MM-DD');

        return moment(formatedCurrentDate).isBetween(formatedReserveTnitDate, formatedReserveEndDate);

    };

    /**
     * Esconde as ações funcionais da tela.
    */
    this.hideFunctionalActions = function () {

        var containerActions = templateManager.container.actions;

        containerActions.forEach(function (action) {

            if (action.label !== "Voltar") {
                action.isVisible = false;
                action.disable = true;
            }

        });

    };

    /**
     * Retorna as configurações padrão das ações
     * do container ativo.
     * 
     * @param {object} widget widget principal da tela de
     * prospect com o backup das actions.
    */
    this.returnContainerDefaultAction = function (widget) {

        var containerActions = templateManager.container.actions;

        containerActions.forEach(function (action) {

            if (Boolean(action.disable)) {
                action.isVisible = true;
                delete action.disable;
            }

        });

    };

    this.setLabelActionQualificado = function (widget) {
        var currentRow = widget.currentRow;
        var actionQualificar = widget.getAction('SET_QUALIFICADO');

        if (currentRow.IDQUALIFICADO === 'S') {
            actionQualificar.label = "Desqualificar Prospect";
        } else {
            actionQualificar.label = "Qualificar Prospect";
        }
    };

    this.setIdQualificadoProspect = function (widget) {
        var currentRow = widget.currentRow;
        var actionQualificar = widget.getAction('SET_QUALIFICADO');
        var message = "Tem certeza que deseja " + (actionQualificar.label.split(' ')[0]).toLowerCase() + " o prospect: <br>" + currentRow.NMRAZSOCPROS + " ?";

        ScreenService.confirmMessage(message, "CONFIRMATION",
            function () {
                var evolWidgt = templateManager.container.getWidget('evolucao');
                widget.openTab('evolucao');

                ScreenService.closePopup();

                evolWidgt.preventLoad = true;
                evolWidgt.SET_QUALIFICADO = widget;

                setTimeout(function () {
                    this.evolucaoNewRow();
                }.bind(this), 500);

            }.bind(this),
            function () {
                ScreenService.closePopup();
            });
    };

    this.callSetIdQualificadoProspect = function (widget) {
        var currentRow = widget.currentRow;

        if (currentRow.IDQUALIFICADO === 'S') {
            currentRow.IDQUALIFICADO = 'N';
        } else {
            currentRow.IDQUALIFICADO = 'S';
        }

        var parametroRestEngine = {
            'requestType': 'Row',
            'row': currentRow
        };

        restEngine.post('/setIdQualificadoProspect', parametroRestEngine)
            .then(function (response) {
                widget.reload();
                if (currentRow.IDQUALIFICADO === 'N')
                    ScreenService.showMessage("Prospect Desqualificado com Sucesso!");
                else
                    ScreenService.showMessage("Prospect Qualificado com Sucesso!");
                this.setLabelActionQualificado(widget);
            }.bind(this)).catch(function () {
                currentRow.IDQUALIFICADO = currentRow.IDQUALIFICADO === 'N' ? 'S' : 'N';
                ScreenService.showMessage("Falha na operação!");
            });
    };

    this.calculaDiasFimContrato = function (fimVigencia) {

        if (typeof fimVigencia === "string" && !Util.isEmptyOrBlank(fimVigencia)) {
            //formata data
            fimVigencia = fimVigencia.slice(3, 5) + '/' + fimVigencia.slice(0, 2) + '/' + fimVigencia.slice(6, 10);
            fimVigencia = new Date(fimVigencia);
            var currentDate = new Date();
            //calcula a diferencia em milesimos
            var diferenca = (fimVigencia - currentDate);
            //milesimos em um dia
            var dia = 1000 * 60 * 60 * 24;

            //Se fimVigencia não for numero retorna vazio, se não retorna a diferenca limitada a zero
            return (Math.round(diferenca / dia) < 0) ? 0 : Math.round(diferenca / dia);
        } else {
            return null;
        }

    };

    this.alteraEstadoReadonly = function (widget) {
        var paisField = widget.getField("NMPAIS");
        var estadoField = widget.getField("NMESTADO");
        var currentRow = widget.currentRow;

        if (!paisField.value()) {
            estadoField.readOnly = true;
            currentRow.SGESTADO = null;
            currentRow.NMESTADO = null;
        }
        else {
            estadoField.readOnly = false;
            currentRow.SGESTADO = null;
            currentRow.NMESTADO = null;
        }

        this.alteraMunicipioReadonly(widget);
    };

    this.alteraMunicipioReadonly = function (widget) {
        var estadoField = widget.getField("NMESTADO");
        var municipioField = widget.getField("NMMUNICIPIO");
        var currentRow = widget.currentRow;

        if (!estadoField.value()) {
            municipioField.readOnly = true;
            currentRow.NMMUNICIPIO = null;
            currentRow.CDMUNICIPIO = null;
            currentRow.NMMUNICIPIO = null;
        }
        else {
            municipioField.readOnly = false;
            currentRow.NMMUNICIPIO = null;
            currentRow.CDMUNICIPIO = null;
            currentRow.NMMUNICIPIO = null;
        }

        this.alteraBairroReadonly(widget);
    };

    this.alteraBairroReadonly = function (widget) {
        var municipioField = widget.getField("NMMUNICIPIO");
        var bairroField = widget.getField("NMBAIRRO");
        var currentRow = widget.currentRow;
        if (!municipioField.value()) {
            bairroField.readOnly = true;
            currentRow.NMBAIRROTRAN = null;
            currentRow.CDBAIRRO = null;
            currentRow.NMBAIRRO = null;
            currentRow.NMBAIRROCAN = null;
        }
        else {
            bairroField.readOnly = false;
            currentRow.NMBAIRROTRAN = null;
            currentRow.CDBAIRRO = null;
            currentRow.NMBAIRRO = null;
            currentRow.NMBAIRROCAN = null;
        }
    };

    this.disableSituacaoOport = function (widget) {
        if (widget.currentRow.__is_new) {
            widget.getField('SITUACAO').readOnly = true;
        } else {
            widget.getField('SITUACAO').readOnly = false;
        }
    };

    this.styleNewProspect = function (element) {
        var tr = $(element[0]);
        if (tr.hasClass('tr')) {
            tr.wrap('<a class="newLeadTooltip" title="Prospect recém criado"></a>');
        }
    };

    this.styleProspect = function (element) {
        while ($(element[0]).parent().is('a')) {
            $(element[0]).unwrap('a');
        }
    };

    this.cancelarNovoRegistro = function () {
        ScreenService.closePopup();
    };

    this.criarNovoConcorrente = function (widget) {
        var popupNovoConcorrente = templateManager.container.getWidget('popupNovoConcorrente');
        popupNovoConcorrente.newRow();

        popupNovoConcorrente.template = "widget/form.html";
        popupNovoConcorrente.activate();
        ScreenService.openPopup(popupNovoConcorrente);

    };

    this.prepararNovoConcorrente = function (widget) {

        CRM_Utilities.setColumnToReturnInCurrentRow(widget.currentRow, "NRCONCORRENTE");
        widget.currentRow.DTULTATUBAIR = CRM_Utilities.getCurrentDate();

        widget.currentRow.NRORG = nrOrgOper;
        widget.currentRow.FIELDONADD = true;
        this.salvarConcorrente(widget.currentRow).then(function (response) {
            var widgetConvert = templateManager.container.getWidget("gridProspect");
            widgetConvert.currentRow.NRCONCORRENTE = response.dataset.columnsToReturn[0].NRCONCORRENTE;
            widgetConvert.currentRow.NMRAZSOCCONC = widget.currentRow.NMRAZSOCCONC;
            ScreenService.closePopup();
            ScreenService.closePopup();
            ScreenService.showMessage('Concorrente salvo com sucesso!');
        });
    };


    this.salvarConcorrente = function (currentRow) {
        var paramsRestEngine = {
            "requestType": "Row",
            "row": currentRow
        };

        return restEngine.post('/crm_concorrente/save', paramsRestEngine);
    };


    this.widgetNewRow = function (widget) {
        General.widgetNewRow(widget);


        widget.currentRow.NRORG = nrOrgOper;
        widget.currentRow.NRPARCRESPROSP = nrparcnegocioOperLogado;
        widget.currentRow.NMPARCRESPROSP = nomeOperador;
        widget.currentRow.IDATIVO = 'S';
        widget.currentRow.IDPRIPUBPROS = '';
        widget.currentRow.IDPRIPUBPROS = '';
        widget.currentRow.CDMODODIVULG = '';
        widget.currentRow.CDATIVCLI = '';
        widget.currentRow.CDQUALIF = '';
        widget.currentRow.CDEXPEDIENTE = '';
        widget.currentRow.CDSEGMER = '';
        widget.currentRow.IDSITUPROS = '';
        widget.currentRow.DTNASCPROS = '';
        widget.currentRow.HRINIFUNCPROSP = '';
        widget.currentRow.HRFINFUNCPROSP = '';
        widget.currentRow.IDLIGEXTPROS = '';
        widget.currentRow.NRUNOPPROS = '';
        widget.currentRow.DSCONCORREPROSP = '';
        widget.currentRow.DTENCECONTRATU = '';
        widget.currentRow.DTREVPRECOPROSP = '';
        widget.currentRow.NRSEQTPATIVCLI = '';
        widget.getField('NRINSJURPROS').readOnly = false;
        widget.getField('NMRAZSOCPROS').readOnly = false;


        var params = {
            "OPERADOR": opLogado
        };

        General.callRestEngine("product#/prospect_hierarquiaPerfilDeAcesso/buscaFilialHierarquia", "Row", params).then(function (response) {

            if (response.dataset.DATA != 0) {
                var filial = response.dataset.DATA[0].CDFILIAL;
                var nmfilial = response.dataset.DATA[0].NMFILIAL;

                widget.currentRow.CDFILIAL = filial;
                widget.currentRow.NMFILIAL = nmfilial;
            }

        },
            function (error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });

    };








    /*=================================== ENDEREÇO ==================================*/



    this.saveEnderecoAndSetAudit = function () {

        var widget = templateManager.container.getActiveWidget();
        this.setAuditInfo(widget);
        var insere = false;

        if (widget.currentRow.__is_new == true) {

            insere = true;
        }
        var widgetGridProspect = getWidgetGridProspect();
        // substituição de '.parent' para 'getWidget' (widget.parent.parent.currentRow);
        var row = {
            "INSERE": insere,
            "NRENDERECOPARC": widget.currentRow.NRENDERECOPARC,
            "NRPARCNEGOCIO": widgetGridProspect.currentRow.NRPARCNEGOCIO,
            "CDTIPOENDERECO": widget.currentRow.CDTIPOENDERECO,
            "NRORG": widget.currentRow.NRORG,
            "IDATIVO": "S",
            "DTINCLUSAO": widget.currentRow.DTINCLUSAO,
            "DTULTATU": widget.currentRow.DTULTATU,
            "NRORGINCLUSAO": widget.currentRow.NRORGINCLUSAO,
            "CDOPERINCLUSAO": widgetGridProspect.currentRow.CDOPERINCLUSAO,
            "NRORGULTATU": widget.currentRow.NRORGULTATU,
            "CDOPERULTATU": widget.currentRow.CDOPERULTATU,
            "CDPAIS": widget.currentRow.CDPAIS,
            "SGESTADO": widget.currentRow.SGESTADO,
            "CDMUNICIPIO": widget.currentRow.CDMUNICIPIO,
            "CDBAIRRO": widget.currentRow.CDBAIRRO,
            "NMBAIRROENDERECO": widget.currentRow.NMBAIRRO,
            "DSCOMPLEENDERECO": widget.currentRow.DSCOMPLEENDERECO,
            "NRCEPENDERECO": widget.currentRow.NRCEPENDERECO,
            "DSENDERECO": widget.currentRow.DSENDERECO,
            "NRIMOVELENDERECO": widget.currentRow.NRIMOVELENDERECO,
            "DSREFERENCIAENDE": widget.currentRow.DSREFERENCIAENDE
        };
        console.log(row);
        General.callRestEngine("product#/prospect_hierarquiaPerfilDeAcesso/insereEnderecoProspect", "Row", row).then(function () {

            ScreenService.closeSwipe().then(function () {
                widget.reload();
            });

        },
            function (error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });
    };




    /**
     * Insere a widget de endereço na widget recebida como 
     * parâmetro.
     * 
     * @param {object} widget widget que receberá a a widget
     * de endereço("WDGT_ENDERECOS").
     * 
     * @param {string} nrparcnegocio nome referente ao
     * campo de parceiro de negócio.
    */
    this.setAddressWidgetOnActiveWidget = function (widget, position) {

        widget.widgets = CRM_Utilities.setElementOnArrayDefinedPosition(this.factoryAddressWidget(widget), widget.widgets, position);
        widget.activate();

    };


    this.setAddressWidgetFilters = function (widget) {

        this.setAddressWidgetFilterBySelectedPartner(widget);

        widget.reload();

    };

    /**
     * Define o filtro da widget de enderço pelo parceiro selecionado.
     *
     * @param {object} widget widget de endereço.
    */
    this.setAddressWidgetFilterBySelectedPartner = function (widget) {

        var widgetGridProspect = getWidgetGridProspect();
        var parentWidgetCurrentRow = widgetGridProspect.currentRow;

        var widgetFilter = widget.dataSourceFilter;
        var nrparcnegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(widgetFilter, "NRPARCNEGOCIO");

        if (widgetFilter[nrparcnegocioFilterIndex].value !== parentWidgetCurrentRow.NRPARCNEGOCIO) {
            widgetFilter[nrparcnegocioFilterIndex].value = parentWidgetCurrentRow.NRPARCNEGOCIO;
        }

    };

    /**
     * Define o filtro do campo 'Tipo do Endereço' pelo
     * parceiro selecionado.
     * Definindo o filtro, o usuário consegue cadastrar
     * apenas um endereço por tipo, lembrando que se o usuário
     * for editar um endereço, o tipo previamente cadastra
     * é exibido, este é o motivo do filtro do campo CDTIPOENDERECO.
     * 
     * @param {object} widget widget de endereços
     * ("WDGT_ENDERECOS").
    */
    this.setAddressTypeFieldFilter = function (widget) {
        var widgetGridProspect = getWidgetGridProspect();
        var parentWidgetCurrentRow = widgetGridProspect.currentRow;
        var currentRow = widget.currentRow;

        var relatedPartnerCode = parentWidgetCurrentRow.NRPARCNEGOCIO;
        var relatedOrgCode = parentWidgetCurrentRow.NRORG;
        var currentAddressType = currentRow.CDTIPOENDERECO;

        var addressTypeField = widget.getField("NMTIPOENDERECO");
        var addressTypeFieldFilter = addressTypeField.dataSourceFilter;

        var nrparcnegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(addressTypeFieldFilter, "P_NRPARCNEGOCIO");
        var nrorgFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(addressTypeFieldFilter, "P_NRORG");
        var cdtipoenderecoFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(addressTypeFieldFilter, "P_CDTIPOENDERECO");


        addressTypeFieldFilter[nrparcnegocioFilterIndex].value = relatedPartnerCode;
        addressTypeFieldFilter[nrorgFilterIndex].value = relatedOrgCode;
        addressTypeFieldFilter[cdtipoenderecoFilterIndex].value = currentAddressType ? currentAddressType : "-1";

        addressTypeField.reload();


    };

    /**
     * Verifica se o usuário modificou a localidade, em caso,
     * selecionada.
     * 
     * @param {object} field campo a ser verificado.
    */
    this.SELECTEDPLACES = { COUNTRY: "", STATE: "", CITY: "" };
    this.checkPlaceChange = function (field) {

        var widget = field.widget;
        var currentRow = widget.currentRow;

        switch (field.name) {

            case "NMPAIS":

                if (currentRow.NMPAIS !== this.SELECTEDPLACES.COUNTRY) {

                    this.cleanSelectedPlacesByPlaceHierarchy(widget, "STATE");
                    this.SELECTEDPLACES.COUNTRY = currentRow.NMPAIS;

                }

                break;

            case "NMESTADO":

                if (currentRow.NMESTADO !== this.SELECTEDPLACES.STATE) {

                    this.cleanSelectedPlacesByPlaceHierarchy(widget, "CITY");
                    this.SELECTEDPLACES.STATE = currentRow.NMESTADO;

                }

                break;

            case "NMMUNICIPIO":

                if (currentRow.NMMUNICIPIO !== this.SELECTEDPLACES.COUNTRY) {

                    this.cleanSelectedPlacesByPlaceHierarchy(widget, "NEIGHBORHOOD");
                    this.SELECTEDPLACES.CITY = currentRow.NMMUNICIPIO;

                }

                break;

        }

        this.unlockPlaceFieldsByFilledPlaces(widget);

    };

    /**
     * Atualiza os locais preenchidos na variável
     * relacionada a locais selecionados
     * (SELECTEDPLACES).
     * 
     * @param {object} widget widget de endereços
     * ("WDGT_ENDERECOS").
    */
    this.checkPlacesFilled = function (widget) {

        var currentRow = widget.currentRow;

        this.SELECTEDPLACES.COUNTRY = currentRow.NMPAIS;
        this.SELECTEDPLACES.STATE = currentRow.NMESTADO;
        this.SELECTEDPLACES.CITY = currentRow.NMMUNICIPIO;

    };

    /**
     * Limpa os campo de endereço a partir do local selecionado
     * (olhando a hierarquia de localidade).
     * EX.:: Se o parâmetro 'referencePlace' for 'STATE', o procedimento
     * irá limpar todos os campo a partir de estado (incluindo o estado)
     * na currentRow.
     * 
     * @param {object} widget widget de endereço ('WDGT_ENDERECO').
     * 
     * @param {string} referencePlace localidade referência para a função executar
     * a limpeza.
    */
    this.PLACEHIERARCHY = {
        COUNTRY: ["NMPAIS", "CDPAIS"],
        STATE: ["NMESTADO", "SGESTADO"],
        CITY: ["NMMUNICIPIO", "CDMUNICIPIO"],
        NEIGHBORHOOD: ["NMBAIRRO", "CDBAIRRO"]
    };
    this.cleanSelectedPlacesByPlaceHierarchy = function (widget, referencePlace) {

        var currentRow = widget.currentRow;
        var toClean = false;

        for (var place in this.PLACEHIERARCHY) {

            if (place === referencePlace) {
                toClean = true;
            }

            if (toClean) {
                for (var placeField in this.PLACEHIERARCHY[place]) {
                    currentRow[placeField] = "";
                }
            }
        }

    };

    /**
     * Libera os campos de endereço de acordo com o preenchimento
     * dos mesmos, pois através da regra de negócio do sistema, o
     * usuário deve preencher um endereço de acordo com a hierarquia
     * de localidade.
     * 
     * @param {object} widget widget de endereços("WDGT_ENDEREÇOS").
    */
    this.unlockPlaceFieldsByFilledPlaces = function (widget) {

        var arrayOfPlaces = [
            widget.getField("NMPAIS"),
            widget.getField("NMESTADO"),
            widget.getField("NMMUNICIPIO"),
            widget.getField("NMBAIRRO")
        ];
        var currentRow = widget.currentRow;

        arrayOfPlaces.forEach(function (place, index) {

            if (place.name !== "NMBAIRRO") {

                if (Boolean(currentRow[place.name])) {
                    CRM_Utilities.setFieldReadOnlyProperty(arrayOfPlaces[index + 1], false);
                } else {
                    CRM_Utilities.setFieldReadOnlyProperty(arrayOfPlaces[index + 1], true);
                }

            }

        });

    };

    /**
     * Dispara os eventos necessários ao clicar
     * em um campo de endereço.
     * 
     * @param {object} field campo selecionado.
    */
    this.addressFieldOnClick = function (field) {

        CRM_Utilities.setPlaceFilterByPlaceOrigin(field.widget, field);
        //CRM_Utilities.setDefaultOrgFilterAndOperatorOrgFilter(field);

        field.reload();

    };

    /**
     * Preenche os campos de endereço pelo CEP
     * digitado.
     * 
     * @param {object} widget widget de endereços
     * ("WDGT_ENDEREÇOS").
    */
    this.fillAddressByCepEnde = function (widget) {
        var fields = ["NMPAIS", "CDPAIS", "NMESTADO", "SGESTADO", "NMMUNICIPIO", "CDMUNICIPIO", "NMBAIRRO", "CDBAIRRO", "DSENDERECO"];
        var cep = widget.currentRow.NRCEPENDERECO;

        if (cep) {
            Utilities.buscarCep(cep).then(function (response) {
                var foundAddress = response.dataset.ENDERECO;
                fields.forEach(function (field) {
                    widget.currentRow[field] = foundAddress[field] ? foundAddress[field] : "";
                });

                this.unlockPlaceFieldsByFilledPlaces(widget);
                if (response.messages) {
                    var msg = ScreenService.i18n(response.messages[0].message);
                    ScreenService.showMessage(msg);
                }

                angular.element('#footer').click();
            }.bind(this))
                .catch(function (error) {
                    ScreenService.showMessage("Endereço não encontrado.");
                });
        } else {
            fields.forEach(function (field) {
                widget.currentRow[field] = "";
            });

        }

    };

    /**
     * Habilita o ambiente de inserção
     * de um endereço.
     * 
     * @param {object} widget widget de 
     * endereços ("WDGT_ENDERECOS").
    */
    this.addAddress = function (widget) {

        General.widgetNewRow(widget);

        this.unlockPlaceFieldsByFilledPlaces(widget);
        this.setAddressDefaultValues(widget);

    };

    /**
     * Habilita o ambiente de edição de 
     * um endereço.
     * 
     * @param {object} widget widget de 
     * endereços ("WDGT_ENDERECOS").
    */
    this.editAddress = function (widget) {

        General.widgetEdit(widget);

        this.unlockPlaceFieldsByFilledPlaces(widget);
        this.checkPlacesFilled(widget);

    };

    /**
     * Define os valores padrões de um endereço.
     * 
     * @param {object} widget widget de endereço.
    */
    this.setAddressDefaultValues = function (widget) {
        var widgetGridProspect = getWidgetGridProspect();
        var parentWidgetCurrentRow = widgetGridProspect.currentRow;
        var currentRow = widget.currentRow;

        currentRow.NRPARCNEGOCIO = parentWidgetCurrentRow.NRPARCNEGOCIO;

    };






    /* ============  INTERLOCUTOR  ===================*/


    this.saveInterlocutorAndSetAudit = function () {

        var widget = templateManager.container.getActiveWidget();
        this.setAuditInfo(widget);
        var insere = false;

        if (widget.currentRow.__is_new == true) {

            insere = true;
        }
        var cpf = "";

        if (widget.currentRow.NRCPFINTERLOC != null) {

            cpf = widget.currentRow.NRCPFINTERLOC.replace(/[^0-9]/g, '');
        }

        var cargo = "";

        if (widget.currentRow.CDCARGOINTERLOC != null) {
            cargo = widget.currentRow.CDCARGOINTERLOC;
        }

        var setor = "";

        if (widget.currentRow.CDSETOR != null) {
            setor = widget.currentRow.CDSETOR;
        }
        var widgetGridProspect = getWidgetGridProspect();
        //confuso sobre forma de transmissão de dados
        // console.log(widget.parent.parent.currentRow);
        var row = {
            "INSERE": insere,
            "NRSEQINTERLOCUTOR": widget.currentRow.NRSEQINTERLOCUTOR,
            "NMINTERLOCUTOR": widget.currentRow.NMINTERLOCUTOR,
            "NRPARCNEGOCIO": widgetGridProspect.currentRow.NRPARCNEGOCIO,
            "EMAILINTERLOC": widget.currentRow.EMAILINTERLOC,
            "NRTELEFONE1": widget.currentRow.NRTELEFONE1,
            "NRTELEFONE2": widget.currentRow.NRTELEFONE2,
            "NRTELEFONE3": widget.currentRow.NRTELEFONE3,
            "NRTELEFONE4": widget.currentRow.NRTELEFONE4,
            "CDCARGOINTERLOC": cargo,
            "DTNASCIMENTO": widget.currentRow.DTNASCIMENTO,
            "IDRECEBEPRESENTE": widget.currentRow.IDRECEBEPRESENTE,
            "CDPAIS": widget.currentRow.CDPAIS,
            "SGESTADO": widget.currentRow.SGESTADO,
            "CDMUNICIPIO": widget.currentRow.CDMUNICIPIO,
            "CDBAIRRO": widget.currentRow.CDBAIRRO,
            "NRCEPINTERLOCUTOR": widget.currentRow.NRCEPINTERLOCUTOR,
            "DSENDEINTERLOCUTOR": widget.currentRow.DSENDEINTERLOCUTOR,
            "DSCOMPINTERLOCUTOR": widget.currentRow.DSCOMPINTERLOCUTOR,
            "NRENDEINTERLOCUTOR": widget.currentRow.NRENDEINTERLOCUTOR,
            "NRORG": widget.currentRow.NRORG,
            "IDATIVO": "S",
            "DTINCLUSAO": widget.currentRow.DTINCLUSAO,
            "DTULTATU": widgetGridProspect.currentRow.DTULTATU,
            "NRORGINCLUSAO": widgetGridProspect.currentRow.NRORGINCLUSAO,
            "NRORGULTATU": widgetGridProspect.currentRow.NRORGULTATU,
            "CDOPERINCLUSAO": widgetGridProspect.currentRow.CDOPERINCLUSAO,
            "CDOPERULTATU": widgetGridProspect.currentRow.CDOPERULTATU,
            "CDSETOR": setor,
            "NRCPFINTERLOC": cpf
        };
        console.log(row);

        General.callRestEngine("product#/prospect_hierarquiaPerfilDeAcesso/insereInterlocutorProspect", "Row", row).then(function () {

            ScreenService.closeSwipe().then(function () {
                widget.reload();
            });

        },
            function (error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });
    };



    /**
     * Insere a widget de interlocutor na widget recebida como 
     * parâmetro.
     * 
     * @param {object} widget widget que receberá a widget de interlocutor
     * ("WDGT_INTERLOCUTOR").
     * 
     * @param {string} nrparcnegocio nome referente ao
     * campo de parceiro de negócio.
    */
    this.setInterlocutorWidgetOnActiveWidget = function (widget, position) {

        widget.widgets = CRM_Utilities.setElementOnArrayDefinedPosition(this.factoryInterlocutorWidget(widget), widget.widgets, position);
        widget.activate();

    };

    this.openPopupNovoSetor = function (widget) {
        var widSetor = templateManager.container.getWidget('popUpNovoSetor');
        ScreenService.openPopup(widSetor).then(function () {
            widSetor.edit();
            widSetor.template = "widget/form.html";
            widSetor.activate();
            widSetor.currentRow = {};
            widSetor.actions = widSetor.actions.splice(0, 2);
        });
    };

    this.saveSetor = function (widget) {
        var widInter = templateManager.container.getWidget('WDGT_INTERLOCUTOR');
        CRM_Utilities.setColumnToReturnInCurrentRow(widget.currentRow, 'CDSETOR');

        widget.currentRow.__is_new = true;
        widget.currentRow.CDDEPTO = '';
        General.widgetSave(widget).then(function (response) {
            widInter.currentRow.CDSETOR = response.dataset.columnsToReturn[0].CDSETOR;
            widInter.currentRow.NMSETOR = widget.currentRow.NMSETOR;
            ScreenService.showMessage('Setor adicionado com sucesso!').then(function () {
                ScreenService.closePopup(true);
                widInter.activate();
            });
        });

    };

    /**
     * Define o filtro de acordo com o local referente selecionado,
     * por exemplo, filtra apenas os municípios do estado selecionado.
     * 
     * @param {object} widget widget pai da widget de interlocutor
    */
    this.placeFilter = function (widget, field) {
        var placeName = field.name;
        var currentRow = widget.currentRow;
        var filter;
        var filterCode;

        switch (placeName) {
            case "NMESTADO":
                filterCode = "CDPAIS";
                filter = this.createEqualFilter(filterCode, currentRow.CDPAIS);
                break;

            case "NMMUNICIPIO":
                filterCode = "SGESTADO";
                filter = this.createEqualFilter(filterCode, currentRow.SGESTADO);
                break;

            case "NMBAIRRO":
                filterCode = "CDMUNICIPIO";
                filter = this.createEqualFilter(filterCode, currentRow.CDMUNICIPIO);
                break;
        }

        var localNameIndexFilter = CRM_Utilities.getFilterPosition(field.dataSourceFilter, filterCode);

        if (field.dataSourceFilter[localNameIndexFilter]) {
            field.dataSourceFilter = CRM_Utilities.deleteFilterObject(field.dataSourceFilter, localNameIndexFilter);
        }

        field.dataSourceFilter.push(filter);
        field.reload();
    };

    /**
     * Retorna um objeto "filtro" com o operador
     * de igualdade.
     * 
     * @param {string} name campo referente para filtrar
     * 
     * @param {string || number} value valor a ser filtrado
     * 
     * @return {object} objeto como filtro.s
    */
    this.createEqualFilter = function (name, value) {
        var filter = {
            "name": name,
            "operator": "=",
            "value": value
        };

        return filter;
    };

    /**
     * Verifica se o usuário modificou o país selecionado,
     * caso ele tenha alterado, a função limpa os campos
     * abaixo da hierarquia de localidade.
     * 
     * @param {object} currentRow row do interlocutor a ser
     * cadastrado.
    */
    var selectedCountry;
    this.checkCountryChange = function (currentRow) {
        var currentCountrys = currentRow.NMPAIS.toString();

        if (typeof selectedCountry === "undefined") {
            selectedCountry = currentCountrys;
        } else if (selectedCountry !== currentCountrys) {
            currentRow.NMESTADO = "";
            currentRow.SGESTADO = "";
            currentRow.NMMUNICIPIO = "";
            currentRow.CDMUNICIPIO = "";
            currentRow.NMBAIRRO = "";
            currentRow.CDBAIRRO = "";
            selectedCountry = currentCountrys;
        }
    };

    /**
     * Verifica se o usuário modificou o estado selecionado,
     * caso ele tenha alterado, a função limpa os campos
     * abaixo da hierarquia de localidade.
     * 
     * @param {object} currentRow row do interlocutor a ser
     * cadastrado.
    */
    var selectedState;
    this.checkStateChange = function (currentRow) {
        var currentState = currentRow.NMESTADO.toString();

        if (typeof selectedState === "undefined") {
            selectedState = currentState;
        } else if (selectedState !== currentState) {
            currentRow.NMMUNICIPIO = "";
            currentRow.CDMUNICIPIO = "";
            currentRow.NMBAIRRO = "";
            currentRow.CDBAIRRO = "";
            selectedState = currentState;
        }
    };

    /**
     * Verifica se o usuário modificou o cidade selecionado,
     * caso ele tenha alterado, a função limpa os campos
     * abaixo da hierarquia de localidade.
     * 
     * @param {object} currentRow row do interlocutor a ser
     * cadastrado.
    */
    var selectedCity;
    this.checkCityChange = function (currentRow) {
        var currentMunicipality = currentRow.NMMUNICIPIO.toString();

        if (typeof selectedCity === "undefined") {
            selectedCity = currentMunicipality;
        } else if (selectedCity !== currentMunicipality) {
            currentRow.NMBAIRRO = "";
            currentRow.CDBAIRRO = "";
            selectedCity = currentMunicipality;
        }
    };

    /**
     * Define o bloqueio do campo de estado de acordo 
     * com a hierarquia de localidade.
     * 
     * @param {object} wodget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.alteraEstadoReadonly = function (widget) {
        //Tratando se é filtro ou edit
        if (widget.mode === "edit") {
            var paisField = widget.getField("NMPAIS");
            var estadoField = widget.getField("NMESTADO");
            var currentRow = widget.currentRow;
            if (!paisField.value()) {
                estadoField.readOnly = true;
                currentRow.SGESTADO = "";
                currentRow.NMESTADO = "";
            }
            else {
                estadoField.readOnly = false;
            }
            this.alteraMunicipioReadonly(widget);

        }
    };

    /**
     * Define o bloqueio do campo de município de acordo 
     * com a hierarquia de localidade.
     * 
     * @param {object} wodget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.alteraMunicipioReadonly = function (widget) {
        //Tratando se é filtro ou edit
        if (widget.mode === "edit") {
            var estadoField = widget.getField("NMESTADO");
            var municipioField = widget.getField("NMMUNICIPIO");
            var currentRow = widget.currentRow;
            if (!estadoField.value()) {
                municipioField.readOnly = true;
                currentRow.NMMUNICIPIO = "";
                currentRow.CDMUNICIPIO = "";
                currentRow.NMMUNICIPIO = "";
            }
            else {
                municipioField.readOnly = false;
            }
        }
        this.alteraBairroReadonly(widget);
    };

    /**
     * Define o bloqueio do campo de bairro de acordo 
     * com a hierarquia de localidade.
     * 
     * @param {object} wodget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.alteraBairroReadonly = function (widget) {
        //Tratando se é filtro ou edit
        if (widget.mode === "edit") {
            var municipioField = widget.getField("NMMUNICIPIO");
            var bairroField = widget.getField("NMBAIRRO");
            var currentRow = widget.currentRow;
            if (!municipioField.value()) {
                bairroField.readOnly = true;
                currentRow.NMBAIRRO = "";
                currentRow.CDBAIRRO = "";
                currentRow.NMBAIRRO = "";
            }
            else {
                bairroField.readOnly = false;
            }
        }
    };

    /**
     * Define o filtro dos campos de endereço
     * pela organização padrão do sistema e
     * pela organização do operador logado.
     * 
     * @param {object} widget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.setAddressFieldsFilterByDefaultOrgAndOperatorOrg = function (widget) {

        var addressFields = [widget.getField('NMPAIS'), widget.getField('NMESTADO'), widget.getField('NMMUNICIPIO'), widget.getField('NMBAIRRO')];

        addressFields.forEach(function (field) {
            this.setAddressFieldFilterValue(field);
        }.bind(this));

    };

    /**
     * Define o filtro do campo recebido como parâmtro
     * para buscar registros da organização padrão e
     * da organização do operador logado.
     * 
     * @param {object} field campo de endereço a ser
     * filtrado.
    */
    this.setAddressFieldFilterValue = function (field) {


        var fieldFilter = field.dataSourceFilter;
        var nrorgFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "NRORG");

        if (Util.isUndefined(nrorgFilterIndex)) {
            nrorgFilterIndex = field.dataSourceFilter.push({ name: "NRORG", operator: "IN", value: "" }) - 1;
        }

        fieldFilter[nrorgFilterIndex].value = [nrOrgOper, getLocalVar('CRM_NRORGPADRAO')];

    };

    /**
     * Define os valores iniciais do cadastro de interlocutor.
     * 
     * @param {object} widget widget de interlocutor ('WDGT_INTERLOCUTOR').
    */
    this.setInterlocutorWidgetDefaultValues = function (widget) {
        var currentRow = widget.currentRow;
        var parentWidget = getWidgetGridProspect();

        currentRow.NRPARCNEGOCIO = parentWidget.currentRow.NRPARCNEGOCIO;

    };

    /**
     * Salve um registro de interlocutor.
     * 
     * @param {object} widget widget de interlocutor('WDGT_INTERLOCUTOR').
    */
    this.saveInterlocutor = function (widget) {

        var currentRow = widget.currentRow;

        this.removeInterlocutorPhoneMask(widget);

        if (Util.isEmptyOrBlank(currentRow.NRCPFINTERLOC))
            currentRow.NRCPFINTERLOC = '';

        currentRow.NRCPFINTERLOC = currentRow.NRCPFINTERLOC.replace(/[^0-9]/g, '');
        CRM_Utilities.setColumnToReturnInCurrentRow(currentRow, "NRSEQINTERLOCUTOR");

        General.widgetSave(widget).then(function (response) {

            currentRow.NRSEQINTERLOCUTOR = response.dataset.columnsToReturn[0].NRSEQINTERLOCUTOR;

        });

    };

    /**
     * Retira a máscara dos telefones digitados
     * na currentRow.
     * 
     * @param {object} widget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.removeInterlocutorPhoneMask = function (widget) {

        var currentRow = widget.currentRow;
        var phoneFields = ["NRTELEFONE1", "NRTELEFONE2", "NRTELEFONE3", "NRTELEFONE4"];

        phoneFields.forEach(function (phone) {

            if (currentRow[phone]) {
                currentRow[phone] = currentRow[phone].replace(/[^0-9]/g, '');
            }

        });

    };

    /**
     * Abre um ambiente para edição de um
     * interlocutor.
     * 
     * @param {object} widget widget de interlocutor
     * ('CRM_INTERLOCUTOR').
    */
    this.editInterlocutor = function (widget) {

        General.widgetEdit(widget);

        this.checkAddressFieldsPermission(widget);

    };

    /**
     * Define as permissões dos campos de endereço,
     * se eles estarão bloqueados ou não.
     * 
     * @param {object} widget widget de interlocutor
     * ('CRM_INTERLOCUTOR').
    */
    this.checkAddressFieldsPermission = function (widget) {

        this.alteraEstadoReadonly(widget);
        this.alteraMunicipioReadonly(widget);
        this.alteraBairroReadonly(widget);

    };

    /**
     * Preenche os campos de endereço pelo CEP
     * digitado.
     * 
     * @param {object} widget widget de interlocutor
     * ('WDGT_INTERLOCUTOR')
    this.fillAddressByCepInterloc = function(widget){
        
        var currentRow = widget.currentRow;
        var cep = currentRow.NRCEPINTERLOCUTOR;
        
        if(cep.length === 8){
            
            CRM_Utilities.buscarCep(cep).then(function(response){
            
                var foundAddress = response.dataset.ENDERECO;
                
                currentRow.NMPAIS = "Brasil";
                currentRow.CDPAIS = "0055";
                currentRow.NMESTADO = foundAddress.NMESTADO;
                currentRow.SGESTADO = foundAddress.SGESTADO;
                currentRow.NMMUNICIPIO = foundAddress.NMMUNICIPIO;
                currentRow.CDMUNICIPIO = foundAddress.CDMUNICIPIO;
                currentRow.NMBAIRRO = foundAddress.NMBAIRRO;
                currentRow.CDBAIRRO = foundAddress.CDBAIRRO;
                currentRow.DSENDEINTERLOCUTOR = foundAddress.DSENDERECO;
                
                this.checkAddressFieldsPermission(widget);
                
                TODO Encontrar um jeito melhor de fazer o autocomplete atualizar os fields para os dados da currentRow
                angular.element('#footer').click();
            
            }.bind(this))
            .catch(function(error){
                ScreenService.showMessage("Endereço não encontrado.");
            });    
            
        } else {
            ScreenService.showMessage("CEP Inválido");
        }
        
    };
    */

    /**
     * Define o filtro da aba de interlocutor pelo
     * parceiro de negócio em caso.
     * 
     * @param {object} widget widget de interlocutor.
    */
    this.setInterlocutorWidgetFilter = function (widget) {
        var widgetGridProspect = getWidgetGridProspect();
        var parentWidgetCurrentRow = widgetGridProspect.currentRow;
        /*Adaptação para widget modal 'dados do parceiro'*/
        if (Util.isEmptyOrBlank(Object.keys(parentWidgetCurrentRow))) {
            parentWidgetCurrentRow = widgetGridProspect.widgets[0].currentRow;
        }
        var nrparcnegocioFilterIndex = CRM_Utilities.getFilterPosition(widget.dataSourceFilter, "NRPARCNEGOCIO");
        if (!Boolean(widget.NRPARCEIROFILTRADO) || widget.NRPARCEIROFILTRADO !== parentWidgetCurrentRow.NRPARCNEGOCIO) {

            widget.dataSourceFilter[nrparcnegocioFilterIndex].value = parentWidgetCurrentRow.NRPARCNEGOCIO;
            widget.NRPARCEIROFILTRADO = parentWidgetCurrentRow.NRPARCNEGOCIO;

            widget.reload();

        }

    };

    /**
     * Abre a widget popup de alteração dos operadores responsáveis
     * @params{Object} widget widget popup alterar operadores responsáveis
    **/
    this.abrirPopupAlterarOpers = function (widget) {
        var wdgPopup = templateManager.container.getWidget("wdgPopupAlterarOpers");
        wdgPopup.isVisible = true;
        wdgPopup.getField("CDOPERRESPATUAL").readOnly = false;
        ScreenService.openPopup(wdgPopup).then(function () {
            wdgPopup.activate();
            wdgPopup.edit();
            wdgPopup.newRow();
        });
    };

    /**
     * Cancela a alteração em massa dos operadores responsáveis
     * @params{Object} widget widget alterar operadores responsáveis
    **/
    this.cancelarPopupAlteracoesOpers = function (widget) {
        var wdgProspect = templateManager.container.getWidget("gridProspect");
        var fieldProspect = widget.getField("CDPROSPECT");
        var fieldOper = widget.getField("CDOPERRESPATUAL");

        fieldOper.readOnly = false;
        fieldOper.dataSource.clearCheckedRows();
        if (fieldOper.dataSourceFilter.length > 0) {
            fieldOper.dataSourceFilter.pop();
        }

        fieldProspect.dataSource.clearCheckedRows();
        if (fieldOper.dataSourceFilter.length > 0) {
            fieldProspect.dataSourceFilter.pop();
        }

        widget.dataSource.data = [];
        widget.isVisible = false;
        ScreenService.closePopup();
        wdgProspect.reload();
        wdgProspect.activate();
    };

    /**
     * Filtra as fields de prospect e operador atual
     * @params{Object} widget widget popup altera operadores responsáveis
     * @params{String} fieldAlterada nome da field que sofreu alteração 
    **/
    this.filtraProspectEOperadores = function (widget, fieldAlterada) {
        var fieldProspect = widget.getField("CDPROSPECT");
        var fieldOper = widget.getField("CDOPERRESPATUAL");

        if (widget.currentRow.CDOPERRESPATUAL && !widget.currentRow.CDPROSPECT || (widget.currentRow.CDPROSPECT && widget.currentRow.CDOPERRESPATUAL && fieldAlterada === "CDOPERRESPATUAL")) {
            if (widget.currentRow.CDOPERRESPATUAL && widget.currentRow.CDOPERRESPATUAL.length > 0) {
                fieldProspect.clearValue();
                fieldProspect.dataSourceFilter[0] = {
                    "name": "NRPARCRESPROSP",
                    "operator": "IN",
                    "value": widget.currentRow.NRPARCNEGOCIOATUAL
                };

                fieldOper.readOnly = false;
            } else {
                if (fieldProspect.dataSourceFilter.length > 1) {
                    fieldProspect.dataSourceFilter.pop();
                }
            }

            fieldProspect.reload();

        } else {
            fieldOper.readOnly = true;
            if (!widget.currentRow.CDOPERRESPATUAL) {
                widget.currentRow.NRPARCRESPROSP = [];
                widget.getField("CDPROSPECT").dataSource.getCheckedRows().forEach(function (contrato) {
                    if (widget.currentRow.NRPARCRESPROSP.indexOf(contrato.NRPARCRESPROSP) === -1)
                        widget.currentRow.NRPARCRESPROSP.push(contrato.NRPARCRESPROSP);
                });

                if ((!widget.currentRow.CDOPERRESPATUAL && widget.currentRow.CDPROSPECT) || (widget.currentRow.CDPROSPECT && widget.currentRow.CDOPERRESPATUAL && fieldAlterada === "CDPROSPECT")) {
                    if (widget.currentRow.NRPARCRESPROSP && widget.currentRow.NRPARCRESPROSP.length > 0) {
                        widget.currentRow.CDOPERRESPATUAL = [];
                        fieldOper.dataSourceFilter[0] = {
                            "name": "NRPARCNEGOCIO",
                            "operator": "IN",
                            "value": widget.currentRow.NRPARCRESPROSP
                        };

                        fieldOper.reload().then(function () {
                            fieldOper.dataSource.data.forEach(function (operador) {
                                fieldOper.dataSource.addCheckedRows(operador);
                                widget.currentRow.CDOPERRESPATUAL.push(operador.CDOPERADOR);
                            });
                        });

                    } else {
                        if (fieldOper.dataSourceFilter.length > 0) {
                            fieldOper.dataSourceFilter.pop();
                            fieldOper.readOnly = false;
                        }
                    }

                    fieldOper.reload().then(function () {
                        widget.redraw();
                        templateManager.updateTemplate();
                    });
                }
            }
        }
    };

    /**
     * Salva o novo operador responsável para os prospect selecionados
     * @params{Object} widget widget altera operadores responsáveis
    **/
    this.salvarAlteracoesOpers = function (widget) {
        if (widget.currentRow.CDPROSPECT.length === 0 && widget.currentRow.NRPARCNEGOCIOATUAL.length !== 0) {
            widget.currentRow.CDPROSPECT = [];
            widget.getField("CDPROSPECT").dataSource.data.forEach(function (data) {
                widget.currentRow.CDPROSPECT.push(data.CDPROSPECT);
            });
        }

        if (widget.isValid()) {
            var params = {
                "CDPROSPECT": widget.currentRow.CDPROSPECT,
                "NRPARCRESPROSP": widget.currentRow.NRPARCNEGOCIO,
                "ALTERAPROPOSTA": widget.currentRow.ALTERAPROPOSTA,
                "ALTERACONTRATO": widget.currentRow.ALTERACONTRATO
            };

            General.callRestEngine("product#/prospect_hierarquiaPerfilDeAcesso/salvarAlteracoesOpersResp", "Row", params).then(function () {
                ScreenService.showMessage("Prospects atualizados com sucesso.");
                this.cancelarPopupAlteracoesOpers(widget);
            }.bind(this));
        }
    };

}

Configuration(function (ContextRegister) {
    ContextRegister.register("ger45000_Prospect_Controller", ger45000_Prospect_Controller);
});



// FILE: ./assets/js/controllers/Sm_PercentualMaximoDescontoAcrescimoOperador_Controller.js
function Sm_PercentualMaximoDescontoAcrescimoOperador_Controller(General,Utilities,templateManager, ScreenService) {
    var tipoAcao;
    this.salvarOperacaoAcresOuDescont = function(widget) {
        var gridDescAcresOperador = templateManager.container.getWidget("gridDescAcresOperador");
        var wdgTabbedDescontoAcrecimo = templateManager.container.getWidget("wdgTabbedDescontoAcrecimo");
        if(widget.isValid()){
            var params = {
                "NRPARCNEGOCIO": widget.currentRow.NRPARCNEGOCIO,
                "CDSERVORCADESC": widget.currentRow.CDSERVORCADESC,
                "VRPERCDESCDESC": widget.currentRow.VRPERCDESCDESC,
                "IDTIPODESC": widget.currentRow.IDTIPODESC,
                "CDSERVORCAACRES": widget.currentRow.CDSERVORCAACRES,
                "VRPERCDESCACRES": widget.currentRow.VRPERCDESCACRES,
                "IDTIPOACRES": widget.currentRow.IDTIPOACRES,
                "ROW": widget.currentRow
            };
              
            
            General.callRestEngine("product#/descontoAcrescimo_Operador/salvarOperacaoAcresOuDescont", "Row", params).then(function(response){
                if(response.dataset.TIPOREPEDITO.length == 0){
                    ScreenService.closeSwipe(wdgTabbedDescontoAcrecimo);
                    this.widgetCancelNewRowOperador(gridDescAcresOperador);
                    gridDescAcresOperador.reload();
                }else
                    if(response.dataset.TIPOREPEDITO.length == 1){
                        if(response.dataset.TIPOREPEDITO[0] == "D"){
                            ScreenService.showMessage("Operação bloqueada! <br>Já existe um Desconto cadastrado para o serviço e operador informado.");
                        }
                        if(response.dataset.TIPOREPEDITO[0] == "A"){
                                ScreenService.showMessage("Operação bloqueada! <br>Já existe um Acréscimo cadastrado para o serviço e operador informado.");
                        }
                }else{
                    ScreenService.showMessage("Operação bloqueada! <br>Já existe um Desconto e um Acréscimo cadastrados para os serviços e operador informados.");
                }
                
            }.bind(this)); 
                   
        }
        
    };
   
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Percentual de Desconto e Acrescímo do Operador 
     * para salvar o novo registro
    */
    this.salvarOperacao = function(widget) {
        var pkFieldNames = ["NRSEQDESC"];
        var fieldNames = ["CDSERVORCA", "NRPARCNEGOCIO", "IDTIPO"];
        var datasourcename = widget.dataSource.name;
        
        var gridDescAcresOperador = templateManager.container.getWidget("gridDescAcresOperador");
        widget.currentRow.NRPARCNEGOCIO = gridDescAcresOperador.currentRow.NRPARCNEGOCIO;
        widget.currentRow.NRORG = gridDescAcresOperador.currentRow.NRORG;
     
        widget.currentRow.__columns_to_return = ["NRSEQDESC", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];
        if (widget.currentRow.__is_new) {
            Utilities.validarCamposDuplicados(widget, pkFieldNames, fieldNames, datasourcename).then(function(response){
                if(response){
                    General.widgetSave(widget).then(function(response){
                        if (response.dataset.columnsToReturn) {
                            widget.currentRow.NRSEQDESC = response.dataset.columnsToReturn[0].NRSEQDESC;
                            widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                            widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                            widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                        }
                    }.bind(this));
                }
                else{
                    ScreenService.showMessage("Operação bloqueada.<br/>Já existe um registro cadastrado para o Serviço informado.");
                }
            });
        }
        else {
             General.widgetSave(widget).then(function(response){
                if (response.dataset.columnsToReturn) {
                    widget.currentRow.NRSEQDESCPERF = response.dataset.columnsToReturn[0].NRSEQDESCPERF;
                    widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                    widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                    widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                }
            }.bind(this));
        }
    };
    /**
    * Realiza a validação no field com os limites minimo e máximo para a porcentagem
    * 
    * @param {object} widget widget de 
    * WdgDesconto, WdgAcrescímo e WdgCadastramentoAutomático 
    */
    this.NovoValidarPorcentagemDesc = function(widget,acrescimo){
        var taxa = widget.currentRow.VRPERCDESCDESC;
         if(!acrescimo){
            if(taxa > 100 || taxa < 0){
                widget.currentRow.VRPERCDESCDESC = null;
                ScreenService.showMessage("Taxa invalida. O valor deve estar entre 0 e 100%");
            }
        }else
            if(acrescimo){
               if(taxa < 0){
                widget.currentRow.VRPERCDESCDESC = null;
                ScreenService.showMessage("Taxa invalida. O valor deve ser superior a 0 (zero)");
            } 
            }
    };
     /**
    * Realiza a validação no field com os limites minimo e máximo para a porcentagem
    * 
    * @param {object} widget widget de 
    * WdgDesconto, WdgAcrescímo e WdgCadastramentoAutomático 
    */
    this.NovoValidarPorcentagemAcres = function(widget,acrescimo){
        var taxa = widget.currentRow.VRPERCDESCACRES;
         if(!acrescimo){
            if(taxa > 100 || taxa < 0){
                widget.currentRow.VRPERCDESCACRES = null;
                ScreenService.showMessage("Taxa invalida. O valor deve estar entre 0 e 100%");
            }
        }else
            if(acrescimo){
               if(taxa < 0){
                widget.currentRow.VRPERCDESCACRES = null;
                ScreenService.showMessage("Taxa invalida. O valor deve ser superior a 0 (zero)");
            } 
            }
    };
    
    /**
    * Realiza a validação no field com os limites minimo e máximo para a porcentagem
    * 
    * @param {object} widget widget de 
    * WdgDesconto, WdgAcrescímo e WdgCadastramentoAutomático 
    */
    this.validarPorcentagem = function(widget,acrescimo){
        var taxa = widget.currentRow.VRPERCDESC;
         if(!acrescimo){
            if(taxa > 100 || taxa < 0){
                widget.currentRow.VRPERCDESC = null;
                ScreenService.showMessage("Taxa invalida. O valor deve estar entre 0 e 100%");
            }
        }else
            if(acrescimo){
               if(taxa < 0){
                widget.currentRow.VRPERCDESC = null;
                ScreenService.showMessage("Taxa invalida. O valor deve ser superior a 0 (zero)");
            } 
            }
    };
    /**
    * Realiza a validação no field com os limites minimo e máximo para a porcentagem
    * 
    * @param {object} widget widget de 
    * WdgDesconto, WdgAcrescímo e WdgCadastramentoAutomático 
    */
    this.validarPorcentagemAutomatico = function(widget){
        
        var taxa = widget.currentRow.VRPERCDESC;
        if(tipoAcao == "D"){
            if(taxa > 100 || taxa < 0){
                widget.currentRow.VRPERCDESC = null;
                ScreenService.showMessage("Taxa invalida. O valor deve estar entre 0 e 100%");
            }
        }else
            {
               if(taxa < 0){
                    widget.currentRow.VRPERCDESC = null;
                    ScreenService.showMessage("Taxa invalida. O valor deve ser superior a 0 (zero)");
                } 
            }
    };
    /**
     * Abri a ação de Cadastro Automatico tanto para as widget de Desconto com tambem de Acrescímo
    * 
    * @param {Boolean} Bool da widget de 
    * WdgDesconto e  WdgAcrescímo 
    * 
    * @param {String} string da widget de 
    * WdgDesconto e  WdgAcrescímo 
    */
    this.abrirWdgCadastroAutomatico = function(sim) {
        var WdgcadastroAuto = templateManager.container.getWidget("WdgcadastramentoAutomatico");
   
        WdgcadastroAuto.isVisible = sim;
        ScreenService.openSwipe(WdgcadastroAuto).then(function() {
            WdgcadastroAuto.newRow();
        }.bind(this));
    };
     /**
    *Cancela Cadastramento Automatico 
    * 
    * @param {object} widget widget de 
    * WdgDesconto, WdgAcrescímo e WdgCadastramentoAutomático 
    */
     this.cancelarCadastramentoAutomatico = function(wdgcadastroAuto) {
        wdgcadastroAuto.dataSource.data.pop();
        this.desobrigaCampos(wdgcadastroAuto);
        ScreenService.closeSwipe();
    };
     /**
    *Realisa o Cadastramento Automático
    * 
    * @param {object} wdgcadastroAuto widget de ação
    * Cadastramento Automático 
    */
    this.saveCadastramentoAutomatico = function(wdgcadastroAuto) {
        if(Utilities.verificaCamposPrenchidos(wdgcadastroAuto)){
            if(wdgcadastroAuto.isValid()){
                var gridDescAcresOperador = templateManager.container.getWidget("gridDescAcresOperador");
                var parametros = {
                    "IDTIPODESC": wdgcadastroAuto.currentRow.IDTIPODESC,
                    "NRPARCNEGOCIODESC": wdgcadastroAuto.currentRow.NRPARCNEGOCIODESC,
                    "CDSERVORCADESC": wdgcadastroAuto.currentRow.CDSERVORCADESC,
                    "VRPERCDESCDESC": wdgcadastroAuto.currentRow.VRPERCDESCDESC,
                    "IDTIPOACRES": wdgcadastroAuto.currentRow.IDTIPOACRES,
                    "NRPARCNEGOCIOACRES": wdgcadastroAuto.currentRow.NRPARCNEGOCIOACRES,
                    "CDSERVORCAACRES": wdgcadastroAuto.currentRow.CDSERVORCAACRES,
                    "VRPERCDESCACRES": wdgcadastroAuto.currentRow.VRPERCDESCACRES
                };
                
                General.callRestEngine("product#/fct_percdescoper_widget/saveCadastramentoAutomatico", "Row", parametros).then(function(response) {
                    if(response){
                        gridDescAcresOperador.reload();
                        this.fecharWdgcadastramentoAutomatico(wdgcadastroAuto);
                        ScreenService.showMessage("Processo realizado com sucesso!");
                    }
                }.bind(this))
                .catch(function(error) {
                    console.error(error);
                    var msg = error.error;
                    if (msg == "REGISTRO_DUPLICADO") {
                        ScreenService.showMessage("Operação bloqueada.<br/>Um ou mais serviços selecionados já estão cadastrados para o(s) operador(es) selecionado(s) para Desconto e Acrécimo.");
                    }else
                    if (msg == "D") {
                        ScreenService.showMessage("Operação bloqueada.<br/>Um ou mais serviços selecionados já estão cadastrados para o(s) operador(es) selecionado(s) para Desconto.");
                    }
                    else
                    if (msg == "A") {
                       ScreenService.showMessage("Operação bloqueada.<br/>Um ou mais serviços selecionados já estão cadastrados para o(s) operador(es) selecionado(s) para Acréscimo."); 
                    }
                    else {
                        ScreenService.showMessage(msg);
                    }
                });
            }
        }else
            ScreenService.showMessage("Nenhum campo preenchido!");
    };
    /*
     * Fecha a ação de Cadastramento Automático
    * 
    * @param {object} wdgcadastroAuto widget de ação
    * Cadastramento Automático 
    */
    this.fecharWdgcadastramentoAutomatico = function(wdgcadastroAuto) {
        ScreenService.closeSwipe();
        this.desobrigaCampos(wdgcadastroAuto);
        wdgcadastroAuto.isVisible = false;
    };
    
    this.widgetNewRowOperador = function(widget) {
        widget.template = "widget/master_detail/grid.html";
        templateManager.updateTemplate();
        setTimeout(function() {
            widget.newRow();
        }, 500);
        widget.activate();
        
    };
    this.widgetCancelNewRowOperador = function(widget) {
        General.widgetCancelForm(widget);
        widget.template = "widget/grid.html";
        templateManager.updateTemplate();
        ScreenService.closeSwipe().then(function(){
            widget.activate();
        });
    };
    
    this.wdgDescontoAcrecimofterMoveRow = function(widget) {
        var wdgPrincipal = templateManager.container.getWidget("gridDescAcresOperador");
        wdgPrincipal.descriptionOrder = widget.descriptionOrder;
        
        if(!wdgPrincipal.currentRow.__is_new)
            this.abrirSwipeWidgets();
    };
     /**
     * Abre a widget de Dados da Proposta (view/form)
     */
    this.abrirSwipeWidgets = function() {
        var gridAcrescimo = templateManager.container.getWidget("gridAcrescimo");
        var gridDesconto = templateManager.container.getWidget("gridDesconto");
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedDescontoAcrecimo");
        
        ScreenService.openSwipe(wdgTabbedPrincipal).then(function() {
            gridDesconto.list();
            gridDesconto.reload().then(function() {
                gridDesconto.list();
            });
            wdgTabbedPrincipal.activate();
        });
        Utilities.carregarWidgetOnEnterTab(gridDesconto);
        if(gridAcrescimo.active)
            gridAcrescimo.reload();
    };
    
    this.widgetVoltarView = function(widget) {
        var gridDescAcresOperador = templateManager.container.getWidget("gridDescAcresOperador");
        if(gridDescAcresOperador.currentRow.__is_new){
            this.widgetCancelNewRowOperador(gridDescAcresOperador);
        }
        else{
             if(widget.name == "gridDesconto" && widget.mode == "view")
                ScreenService.closeSwipe().then(function (){
                        widget.activate();
                });
            if(widget.name == "gridDesconto" && widget.mode == "list"){
                ScreenService.closeSwipe();
                gridDescAcresOperador.activate();
            } 
        }
    };
    
    this.widgetPreparaVoltarView = function(widget) {
        var gridDesconto = templateManager.container.getWidget("gridDesconto");
         if(widget.name == "gridAcrescimo" && widget.mode != "list")
            ScreenService.closeSwipe();
        gridDesconto.activate();
         
    };
    this.tornaCamposObrigatoriosDesc = function(widget) {
        var _is_multSearch = false;
        var camposNome = ["NMSERVORCADESC","VRPERCDESCDESC"];
        
        Utilities.tornaCamposEscolhidosObrigatorios(widget, camposNome, _is_multSearch);
    };
    
    this.tornaCamposObrigatoriosAcres = function(widget) {
        var _is_multSearch = false;
        var camposNome = ["NMSERVORCAACRES","VRPERCDESCACRES"];
        
        Utilities.tornaCamposEscolhidosObrigatorios(widget, camposNome, _is_multSearch);
    };
     this.tornaCamposObrigatoriosAutomaticoDesc = function(widget) {
        var _is_multSearch = true;
        var camposNome = ["NRPARCNEGOCIODESC", "CDSERVORCADESC", "VRPERCDESCDESC"];

        Utilities.tornaCamposEscolhidosObrigatorios(widget, camposNome, _is_multSearch);
    };
    this.tornaCamposObrigatoriosAutomaticoAcres = function(widget) {
        var _is_multSearch = true;
        var camposNome = ["NRPARCNEGOCIOACRES", "CDSERVORCAACRES", "VRPERCDESCACRES"];
        
        Utilities.tornaCamposEscolhidosObrigatorios(widget, camposNome, _is_multSearch);
    };
     this.desobrigaCampos = function(widget){
        widget.fields.forEach(function(field){
            field.validations = {};
        }.bind(this));
    }; 
    
}
Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm_PercentualMaximoDescontoAcrescimoOperador_Controller', Sm_PercentualMaximoDescontoAcrescimoOperador_Controller);
});



// FILE: ./assets/js/controllers/Sm_PercentualMaximoDescontoAcrescimoPerfilAcesso_Controller.js
function Sm_PercentualMaximoDescontoAcrescimoPerfilAcesso_Controller(General,Utilities,templateManager, ScreenService) {
    var tipoAcao;
        this.salvarOperacaoAcresOuDescont = function(widget) {
        var gridDescAcresPerfilAcesso = templateManager.container.getWidget("gridDescAcresPerfilAcesso");
        var wdgTabbedDescontoAcrecimo = templateManager.container.getWidget("wdgTabbedDescontoAcrecimo");
        if(widget.isValid()){
            var params = {
                "NRPERFILACESSO": widget.currentRow.NRPERFILACESSO,
                "CDSERVORCADESC": widget.currentRow.CDSERVORCADESC,
                "VRPERCENTUALDESC": widget.currentRow.VRPERCENTUALDESC,
                "IDTIPODESC": widget.currentRow.IDTIPODESC,
                "CDSERVORCAACRES": widget.currentRow.CDSERVORCAACRES,
                "VRPERCENTUALACRES": widget.currentRow.VRPERCENTUALACRES,
                "IDTIPOACRES": widget.currentRow.IDTIPOACRES,
                "ROW": widget.currentRow
            };
              
            
            General.callRestEngine("product#/descontoAcrescimoPerfil_perfilacesso/salvarOperacaoAcresOuDescont", "Row", params).then(function(response){
                if(response.dataset.TIPOREPEDITO.length == 0){
                    ScreenService.closeSwipe(wdgTabbedDescontoAcrecimo);
                    this.widgetCancelNewRowOperador(gridDescAcresPerfilAcesso);
                    gridDescAcresPerfilAcesso.reload();
                }else
                    if(response.dataset.TIPOREPEDITO.length == 1){
                        if(response.dataset.TIPOREPEDITO[0] == "D"){
                            ScreenService.showMessage("Operação bloqueada! <br>Já existe um Desconto cadastrado para o serviço e perfil de acesso informado.");
                        }
                        if(response.dataset.TIPOREPEDITO[0] == "A"){
                                ScreenService.showMessage("Operação bloqueada! <br>Já existe um Acréscimo cadastrado para o serviço e perfil de acesso informado.");
                        }
                }else{
                    ScreenService.showMessage("Operação bloqueada! <br>Já existe um Desconto e um Acréscimo cadastrados para os serviços e perfil de acesso informados.");
                }
                
            }.bind(this)); 
                   
        }
        
    };

    /**
     * Salva o novo registro 
     * 
     * @param {object} widget widget de 
     * Percentual de Desconto e Acrescímo do Perfíl de Acesso 
     * para salvar o novo registro
    */
    this.salvarOperacao = function(widget){
        var pkFieldNames = ["NRSEQDESCPERF"];
        var fieldNames = ["CDSERVORCA", "NRPERFILACESSO", "IDTIPO"];
        var datasourcename = widget.dataSource.name;
        
        var gridDescAcresPerfilAcesso = templateManager.container.getWidget("gridDescAcresPerfilAcesso");
        widget.currentRow.NRPERFILACESSO = gridDescAcresPerfilAcesso.currentRow.NRPERFILACESSO;
        widget.currentRow.NRORG = gridDescAcresPerfilAcesso.currentRow.NRORG;
     
        widget.currentRow.__columns_to_return = ["NRSEQDESCPERF", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];
        if (widget.currentRow.__is_new) {
            Utilities.validarCamposDuplicados(widget, pkFieldNames, fieldNames, datasourcename).then(function(response){
                if(response) {
                    General.widgetSave(widget).then(function(response){
                        if (response.dataset.columnsToReturn) {
                            widget.currentRow.NRSEQDESCPERF = response.dataset.columnsToReturn[0].NRSEQDESCPERF;
                            widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                            widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                            widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                        }
                    }.bind(this));
                }
                else{
                    ScreenService.showMessage("Operação bloqueada.<br/>Já existe um registro cadastrado para o Serviço informado.");
                }
            });
        }
        else {
            General.widgetSave(widget).then(function(response){
                if (response.dataset.columnsToReturn) {
                    widget.currentRow.NRSEQDESCPERF = response.dataset.columnsToReturn[0].NRSEQDESCPERF;
                    widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                    widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                    widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                }
            }.bind(this));
        }
    };
     /**
    * Realiza a validação no field com os limites minimo e máximo para a porcentagem
    * 
    * @param {object} widget widget de 
    * WdgDesconto, WdgAcrescímo e WdgCadastramentoAutomático 
    */
    this.NovoValidarPorcentagemDesc = function(widget,acrescimo){
        var taxa = widget.currentRow.VRPERCENTUALDESC;
         if(!acrescimo){
            if(taxa > 100 || taxa < 0){
                widget.currentRow.VRPERCENTUALDESC = null;
                ScreenService.showMessage("Taxa invalida. O valor deve estar entre 0 e 100%");
            }
        }else
            if(acrescimo){
               if(taxa < 0){
                widget.currentRow.VRPERCENTUALDESC = null;
                ScreenService.showMessage("Taxa invalida. O valor deve ser superior a 0 (zero)");
            } 
            }
    };
     /**
    * Realiza a validação no field com os limites minimo e máximo para a porcentagem
    * 
    * @param {object} widget widget de 
    * WdgDesconto, WdgAcrescímo e WdgCadastramentoAutomático 
    */
    this.NovoValidarPorcentagemAcres = function(widget,acrescimo){
        var taxa = widget.currentRow.VRPERCENTUALACRES;
         if(!acrescimo){
            if(taxa > 100 || taxa < 0){
                widget.currentRow.VRPERCENTUALACRES = null;
                ScreenService.showMessage("Taxa invalida. O valor deve estar entre 0 e 100%");
            }
        }else
            if(acrescimo){
               if(taxa < 0){
                widget.currentRow.VRPERCENTUALACRES = null;
                ScreenService.showMessage("Taxa invalida. O valor deve ser superior a 0 (zero)");
            } 
            }
    };
    /**
    * Realiza a validação no field com os limites minimo e máximo para a porcentagem
    * 
    * @param {object} widget widget de 
    * WdgDesconto, WdgAcrescímo e WdgCadastramentoAutomático 
    */
    this.validarPorcentagem = function(widget,acrescimo){
        
        var taxa = widget.currentRow.VRPERCENTUAL;
        if(!acrescimo){
            if(taxa > 100 || taxa < 0){
                widget.currentRow.VRPERCENTUAL = null;
                ScreenService.showMessage("Taxa invalida. O valor deve estar entre 0 e 100%");
            }
        }else
            if(acrescimo){
               if(taxa < 0){
                widget.currentRow.VRPERCENTUAL = null;
                ScreenService.showMessage("Taxa invalida. O valor deve ser superior a 0 (zero)");
            } 
            }
    };
    /**
    * Realiza a validação no field com os limites minimo e máximo para a porcentagem
    * 
    * @param {object} widget widget de 
    * WdgDesconto, WdgAcrescímo e WdgCadastramentoAutomático 
    */
    this.validarPorcentagemAutomatico = function(widget){
        
        var taxa = widget.currentRow.VRPERCENTUAL;
        if(tipoAcao == "D"){
            if(taxa > 100 || taxa < 0){
                widget.currentRow.VRPERCENTUAL = null;
                ScreenService.showMessage("Taxa invalida. O valor deve estar entre 0 e 100%");
            }
        }else
            {
               if(taxa < 0){
                    widget.currentRow.VRPERCENTUAL = null;
                    ScreenService.showMessage("Taxa invalida. O valor deve ser superior a 0 (zero)");
                } 
            }
    };
    /**
     * Abri a ação de Cadastro Automatico tanto para as widget de Desconto com tambem de Acrescímo
    * 
    * @param {Boolean} Bool da widget de 
    * WdgDesconto e  WdgAcrescímo 
    * 
    * @param {String} string da widget de 
    * WdgDesconto e  WdgAcrescímo 
    */
    this.abrirWdgCadastroAutomatico = function(sim) {
        var WdgcadastroAuto = templateManager.container.getWidget("WdgcadastramentoAutomatico");
   
        WdgcadastroAuto.isVisible = sim;
        ScreenService.openSwipe(WdgcadastroAuto).then(function() {
            WdgcadastroAuto.newRow();
        }.bind(this));
    };
     /**
    *Cancela Cadastramento Automatico 
    * 
    * @param {object} widget widget de 
    * WdgDesconto, WdgAcrescímo e WdgCadastramentoAutomático 
    */
     this.cancelarCadastramentoAutomatico = function(wdgcadastroAuto) {
        wdgcadastroAuto.dataSource.data.pop();
        this.desobrigaCampos(wdgcadastroAuto);
        ScreenService.closeSwipe();
    };
   
     /**
    *Realisa o Cadastramento Automático
    * 
    * @param {object} wdgcadastroAuto widget de ação
    * Cadastramento Automático 
    */
    this.saveCadastramentoAutomatico = function(wdgcadastroAuto) {
        if(Utilities.verificaCamposPrenchidos(wdgcadastroAuto)){
            if(wdgcadastroAuto.isValid()){
                var gridDescAcresPerfilAcesso = templateManager.container.getWidget("gridDescAcresPerfilAcesso");
                var parametros = {
                    "IDTIPODESC": wdgcadastroAuto.currentRow.IDTIPODESC,
                    "NRPERFILACESSODESC": wdgcadastroAuto.currentRow.NRPERFILACESSODESC,
                    "CDSERVORCADESC": wdgcadastroAuto.currentRow.CDSERVORCADESC,
                    "VRPERCENTUALDESC": wdgcadastroAuto.currentRow.VRPERCENTUALDESC,
                    "IDTIPOACRES": wdgcadastroAuto.currentRow.IDTIPOACRES,
                    "NRPERFILACESSOACRES": wdgcadastroAuto.currentRow.NRPERFILACESSOACRES,
                    "CDSERVORCAACRES": wdgcadastroAuto.currentRow.CDSERVORCAACRES,
                    "VRPERCENTUALACRES": wdgcadastroAuto.currentRow.VRPERCENTUALACRES
                };
                
                General.callRestEngine("product#/fct_percdescperfil_widget/saveCadastramentoAutomatico", "Row", parametros).then(function(response) {
                    if(response){
                        gridDescAcresPerfilAcesso.reload();
                        this.fecharWdgcadastramentoAutomatico(wdgcadastroAuto);
                        ScreenService.showMessage("Processo realizado com sucesso!");
                    }
                }.bind(this))
                .catch(function(error) {
                    console.error(error);
                    var msg = error.error;
                    if (msg == "REGISTRO_DUPLICADO") {
                        ScreenService.showMessage("Operação bloqueada.<br/>Um ou mais serviços selecionados já estão cadastrados para o(s) perfil(is) selecionado(s).");
                    }else
                    if (msg == "D") {
                        ScreenService.showMessage("Operação bloqueada.<br/>Um ou mais serviços selecionados já estão cadastrados para o(s) perfil(is) selecionado(s) para Desconto.");
                    }
                    else
                    if (msg == "A") {
                       ScreenService.showMessage("Operação bloqueada.<br/>Um ou mais serviços selecionados já estão cadastrados para o(s) perfil(is) selecionado(s) para Acréscimo."); 
                    }
                    else {
                        ScreenService.showMessage(msg);
                    }
                });
            }
        }else
            ScreenService.showMessage("Nenhum campo preenchido!");
    };
    
    /**
     * Fecha a ação de Cadastramento Automático
     * 
     * @param {object} wdgcadastroAuto widget de ação
     * Cadastramento Automático 
     */
    this.fecharWdgcadastramentoAutomatico = function(wdgcadastroAuto) {
        ScreenService.closeSwipe();
        this.desobrigaCampos(wdgcadastroAuto);
        wdgcadastroAuto.isVisible = false;
    };
    
    this.widgetNewRowOperador = function(widget) {
        widget.template = "widget/master_detail/grid.html";
        templateManager.updateTemplate();
        setTimeout(function() {
            widget.newRow();
        }, 500);
        widget.activate();
        
    };
    this.widgetCancelNewRowOperador = function(widget) {
        General.widgetCancelForm(widget);
        widget.template = "widget/grid.html";
        templateManager.updateTemplate();
        ScreenService.closeSwipe().then(function(){
            widget.activate();
        });
    };
    
    this.wdgDescontoAcrecimofterMoveRow = function(widget) {
        var wdgPrincipal = templateManager.container.getWidget("gridDescAcresPerfilAcesso");
        wdgPrincipal.descriptionOrder = widget.descriptionOrder;
        
        if(!wdgPrincipal.currentRow.__is_new)
            this.abrirSwipeWidgets();
    };
    
      /**
     * Abre a widget de Dados da Proposta (view/form)
     */
    this.abrirSwipeWidgets = function() {
        var gridAcrescimo = templateManager.container.getWidget("gridAcrescimo");
        var gridDesconto = templateManager.container.getWidget("gridDesconto");
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedDescontoAcrecimo");
        
        ScreenService.openSwipe(wdgTabbedPrincipal).then(function() {
            gridDesconto.list();
            gridDesconto.reload().then(function() {
                gridDesconto.list();
            });
            wdgTabbedPrincipal.activate();
        });
        Utilities.carregarWidgetOnEnterTab(gridDesconto);
        if(gridAcrescimo.active)
            gridAcrescimo.reload();
    };
    
    this.widgetVoltarView = function(widget) {
        
        var gridDescAcresPerfilAcesso = templateManager.container.getWidget("gridDescAcresPerfilAcesso");
        if(gridDescAcresPerfilAcesso.currentRow.__is_new){
            this.widgetCancelNewRowOperador(gridDescAcresPerfilAcesso);
        }
        else{
            if(widget.name == "gridDesconto" && widget.mode == "view")
                ScreenService.closeSwipe().then(function (){
                        widget.activate();
                });
            if(widget.name == "gridDesconto" && widget.mode == "list"){
                ScreenService.closeSwipe();
                gridDescAcresPerfilAcesso.activate();
            }                
        }
    };
    this.widgetPreparaVoltarView = function(widget) {
        var gridDesconto = templateManager.container.getWidget("gridDesconto");
        if(widget.name == "gridAcrescimo" && widget.mode != "list")
            ScreenService.closeSwipe();
        gridDesconto.activate();
    };
    
     this.tornaCamposObrigatoriosDesc = function(widget) {
        
        var camposEscolhidos = ["NMSERVORCADESC","VRPERCENTUALDESC"];
        
        Utilities.tornaCamposEscolhidosObrigatorios(widget, camposEscolhidos);
    };
    
    this.tornaCamposObrigatoriosAcres = function(widget) {
        
        var camposEscolhidos = ["NMSERVORCAACRES","VRPERCENTUALACRES"];
        
        Utilities.tornaCamposEscolhidosObrigatorios(widget, camposEscolhidos);
    };
      this.tornaCamposObrigatoriosAutomaticoDesc = function(widget) {
        var _is_multSearch = true;
        var camposNome = ["NRPERFILACESSODESC", "CDSERVORCADESC", "VRPERCENTUALDESC"];

        Utilities.tornaCamposEscolhidosObrigatorios(widget, camposNome, _is_multSearch);
    };
    this.tornaCamposObrigatoriosAutomaticoAcres = function(widget) {
        var _is_multSearch = true;
        var camposNome = ["NRPERFILACESSOACRES", "CDSERVORCAACRES", "VRPERCENTUALACRES"];
        
        Utilities.tornaCamposEscolhidosObrigatorios(widget, camposNome, _is_multSearch);
    };
     this.desobrigaCampos = function(widget){
        widget.fields.forEach(function(field){
            field.validations = {};
        }.bind(this));
    };
}
Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm_PercentualMaximoDescontoAcrescimoPerfilAcesso_Controller', Sm_PercentualMaximoDescontoAcrescimoPerfilAcesso_Controller);
});

// FILE: ./assets/js/controllers/Sm00001000_LancamentoProposta_Controller.js
function Sm00001000_LancamentoProposta_Controller(ScreenService, General, templateManager, Sm00001000_LancamentoProposta_Service, Utilities, ZHPromise, Operador_Repository, Endereco, CRM_Utilities, ger00004500_Prospect_Service, ABEvolucaoController, UtilitiesPropostaContrato, ValidacoesParametrosFaturamentoUtil, ZhFilterPreferences, SelectUtil, restEngine, eventAggregator) {
    var self = this;
    // Campos de saída do modelo de proposta...
    var fieldsModelo = [
        "DSMODPGTO",
        "IDFATURAORCA",
        "NMGRUPOFATURA",
        "QTDIASCONTRATO",
        "QTDIASVALIDADE",
        "DTVALIDADE"
    ];

    var visibilidadeAcoes       = null;
    var opLogado                = null;
    var nomeOperador            = null;
    var idOperMaster            = null;
    var idUtilizaHierOper       = null;
    var nrparcnegocioOperLogado = null;
    var tpOperador              = null;
    var nrOrgOper               = null;
    var is_required             = false;
    var qtdPrenchido            = [];
    var trancado                = false;
    var idAlteraPropConc        = null;
    var servIdMedidaFixa        = [];  
    var idObrigaModProp         = null;

    var cdidiom_padrao = null;
    var nmidioma_padrao = null;
    var nrmoeda_padrao = null;
    var nmmoeda_padrao = null;

    var mudouMoeda = false;

    // widgets
    var getWidgetPropostaGrid = function () {
        return templateManager.container.getWidget('wdgPropostaGrid');
    };
    var getWidgetProposta = function () {
        return templateManager.container.getWidget('wdgProposta');
    };
    var getWidgetEndereco = function () {
        return templateManager.container.getWidget('wdgEndereco');
    };
    var getWidgetItensProposta = function () {
        return templateManager.container.getWidget('wdgItensProposta');
    };
    var getWidgetAnexos = function () {
        return templateManager.container.getWidget('WDGT_ANEXOS_EVOLUCAO');
    };
    var getWidgetEvolucao = function () {
        return templateManager.container.getWidget('evolucao');
    };
    var getWidgetItensPropostaTree = function () {
        return templateManager.container.getWidget('wdgItensPropostaTree');
    };
    var getWidgetWdgElementosProposta = function () {
        return templateManager.container.getWidget('wdgElementosProposta');
    };
    var getWidgetWdgEscopoProposta = function () {
        return templateManager.container.getWidget('wdgEscoposProposta');
    };
    var getWidgetNegociacaoDeValores = function() {
        return templateManager.container.getWidget('wdgNegociacaoValores');
    };
    // Actions
    var getActionEditWidgetProposta = function(){
        return getWidgetProposta().getAction('edit');
    };
    var getActionAddEndereco = function(){
        return getWidgetEndereco().getAction('add');
    };
    var getActionEditEndereco = function(){
        return getWidgetEndereco().getAction('edit');
    };
    var getActionEditEscopoProposta = function(){
        return getWidgetWdgEscopoProposta().getAction('edit');
    };
    var getActionEditEvolucao = function(){
        return getWidgetEvolucao().getAction('EDIT');
    };
    var getActionAdicEvolucao = function(){
        return getWidgetEvolucao().getAction('ADD_EVOLUCAO');
    };
    // fields
    var getFieldModeloDePropostaFromWidgetPropostaGrid = function(){
        return getWidgetProposta().getField('NMORCAMENTOMODELO');
    };
    
    var getDataSourceTipoDesconto = function(somenteP) {
        var data = [
            {
                "VALUE": "P",
                "NAME": "Na quantidade medida total (vendida + adicional)"
            }
        ];
        if (!somenteP) {
            data.push({
                "VALUE": "V",
                "NAME": "Apenas na quantidade vendida"
            });
        }
        
        return data;
    };
    
    var setDataSourceTipoDesconto = function(widget, somenteP) {
        var fieldTipoDesconto = widget.getField("IDTPDESCONTO");
        fieldTipoDesconto.dataSource.data = getDataSourceTipoDesconto(somenteP);
    };
    
    /**
     * Coloca na currentRow os parametrôs de novo código
     * para a inserção de novos registros
     * @params{Object} widget - widget que irá inserir um novo registro
    **/
    this.criarParametrosNewCode = function(widget){
        if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
            widget.currentRow.__using_unit      = widget.paramsNewCode.__using_unit || false;
            widget.currentRow.__new_code_pk     = widget.paramsNewCode.__new_code_pk;
            widget.currentRow.__table_pk        = widget.paramsNewCode.__table_pk;
            widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 1;
            widget.currentRow.__cast_number     = widget.paramsNewCode.__cast_number || false;
            if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
        }
    };
    /** Busca o operador logado a sessão atual 
     * @params{Object} widget de proposta
     */
    // this.wdgPropostaGridOnEnterOnce = function() {
    //     ZhFilterPreferences.wdgPropostaGridOnEnter(templateManager.container.getActiveWidget()).then(function(){
    //         Operador_Repository.getOperadorLogado().then(function(success) {
    //             var filter = templateManager.container.getActiveWidget();
    //             filter.currentRow.IDSITUACAOORCAMENTO = ["ELABORACAO", "CONCLUIDO", "ENVIADO", "ACEITO"];
    //             var operadorResponsavel = filter.getField("CDOPERRESPONSAVEL");
    //             operadorResponsavel.reload().then(function(){
    //                 Utilities.setValoresPadroesParaCampoSelectMultiple(operadorResponsavel, [success[0].CDOPERADOR.toString()], 'CDOPERADOR');
    //                 operadorResponsavel.lazyLoad = true;
    //                 operadorResponsavel.itemsPerPage = 50;
    //             });
                
    //         }.bind(this));
    //     });
    // };
    /** Busca o operador logado a sessão atual 
     * @params{Object} widget de proposta
     */
    this.wdgPropostaGridOnEnterOnce = function() {
        ZhFilterPreferences.openFilterWidget(templateManager.container.getActiveWidget()).then(function(){
            Operador_Repository.getOperadorLogado().then(function(success) {
                var filter = templateManager.container.getActiveWidget();
                filter.currentRow.IDSITUACAOORCAMENTO = ["ELABORACAO", "CONCLUIDO", "ENVIADO", "ACEITO"];
                var operadorResponsavel = filter.getField("CDOPERRESPONSAVEL");
                operadorResponsavel.reload().then(function(){
                    Utilities.setValoresPadroesParaCampoSelectMultiple(operadorResponsavel, [success[0].CDOPERADOR.toString()], 'CDOPERADOR');
                    operadorResponsavel.lazyLoad = true;
                    operadorResponsavel.itemsPerPage = 50;
                });
                eventAggregator.publish("abriuFiltroProposta");
            }.bind(this));
        });
    };

    this.buscaOperadorLogado = function() {
        var deferred = ZHPromise.defer();
        Operador_Repository.getOperadorLogado().then(function(success) {
            opLogado                = success[0].CDOPERADOR;
            nomeOperador            = success[0].NMOPERADOR;
            nrparcnegocioOperLogado = success[0].NRPARCNEGOCIO;
            tpOperador              = success[0].IDTPOPERADOR;
            nrOrgOper               = success[0].NRORG;
            
            this.buscaInformacaoHierarquia();
            deferred.resolve(success);
        }.bind(this));

        return deferred.promise;
    };

    /* Verifica se o usuário tem hierarquia de operadores e caso tenha
     * busca sua posição
     */
    this.verificaHierarquia = function(widgetPropostaGrid, widget) {
        if(idOperMaster === null && idUtilizaHierOper === null)
            this.buscaInformacaoHierarquia();

        if(opLogado) {
            widget.currentRow.CDOPERRESPONSAVEL = opLogado;
            widget.currentRow.CDNOVOOPERRESP    = opLogado;
            widget.currentRow.NMOPERRESP        = nomeOperador;
        } else {
            this.buscaOperadorLogado(widget).then(function(){
                widget.currentRow.CDOPERRESPONSAVEL = opLogado;
                widget.currentRow.CDNOVOOPERRESP    = opLogado;
                widget.currentRow.NMOPERRESP        = nomeOperador;
            });
        }    
    };
    
    /* busca os dados de hierarquia do usuário logado
    * @ params{Object} widget  
    */
    this.buscaInformacaoHierarquia = function() {
        General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", {}).then(function(success) {
            if (success.dataset.fct_parametrogeral && success.dataset.fct_parametrogeral.length > 0) {
                var paramsService = success.dataset.fct_parametrogeral[0];
                
                if(paramsService.IDOBRIGMODORC)
                    idObrigaModProp = (paramsService.IDOBRIGMODORC == 'S');
                
                var paramsFilter = [
                    {
                        name: "CDOPERADOR",
                        operator: "=",
                        value: opLogado
                    }
                ];
                
                idUtilizaHierOper = paramsService.IDUTILIZAHIEROPER;
                General.callRestEngine("product#/crud/crm_hierarquiaoper", "FilterData", paramsFilter).then(function(success) {
                    if (success.dataset.crm_hierarquiaoper && success.dataset.crm_hierarquiaoper.length > 0) 
                        idOperMaster = success.dataset.crm_hierarquiaoper[0].IDOPERMASTER;
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };

    /* Recarrega o field caso o dataSource tenha sido alterado 
     * @params{Object} field Nome Operador
     */
    this.recarregaDadosField = function(fieldNmOp) {
        var widgetAtiva = templateManager.container.getActiveWidget();
        var fieldParc  =  widgetAtiva.getField("NRPARCNEGOCIO");
        
        if (widgetAtiva.name !== "wdgProposta") {
            fieldParc.dataSourceFilter[1] = {
                name: "P_CDOPERADOR",
                operator: "=",
                value: opLogado
            };
        }

        Utilities.iniciarAutocomplete(fieldNmOp);
    };

    this.novaProposta = function() {
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        var wdgProposta = templateManager.container.getWidget("wdgProposta");

        wdgProposta.getField("NMIDIOMA").readOnly = false;
        wdgProposta.getField("NMMOEDA").readOnly = false;

        Operador_Repository.getOperadorLogado().then(function(success) {
            var parametro = {
                "NRPARCNEGOCIO" : success[0].NRPARCNEGOCIO,
                "NRORG"         : success[0].NRORG,
                "PROPOUCONT"    : "P"
            };

            return General.callRestEngine('product#/validarPerfilAcesso', "Row", parametro);
        }).then(function(perfilusuario) {
            if (perfilusuario.dataset.validarperfil.PERFILACESSO) {
                var modelosDeContrato = [];
                var field = getFieldModeloDePropostaFromWidgetPropostaGrid();
                perfilusuario.dataset.validarperfil.PERFILACESSO.forEach(function (modeloContrato) {
                    modelosDeContrato.push(modeloContrato.MODELO);
                });
                field.dataSourceFilter[2] = {
                    "name"     : "NRORCAMENTOMODELO",
                    "operator" : "NOT_IN",
                    "value"    : modelosDeContrato
                };
            }

            setLocalVar("PROPOSTA_NRSEQORCAMENTO", "");
            //setLocalVar("PROPOSTA_NRORG", "");
            
            self.validarObrigatoriedadeModelo();
            return ScreenService.openSwipe(wdgTabbedPrincipal);
        }).then(function() {
            wdgProposta.template = "widget/form.html";
            wdgProposta.newRow();
            self.alterarAcessibilidadeAbasProposta(false);

            //wdgProposta.newRow();
            wdgProposta.template = "widget/form.html";
            wdgTabbedPrincipal.edit();

            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            self.verificaHierarquia(wdgPropostaGrid, wdgProposta);
            wdgPropostaGrid.currentRow = {};

            self.validacoesGeraisProposta(wdgPropostaGrid);
            self.filtroInterlocutor(wdgProposta);
            self.setVendedorPadrao(wdgProposta);
            self.controlaCodProposta(wdgProposta);
            self.assinaturaEletronicaOnChange(wdgProposta);

            var wdgEscoposProposta = templateManager.container.getWidget("wdgEscoposProposta");
            wdgEscoposProposta.isVisible = false;

            //setar idioma e moeda padrao
            return General.callRestEngine("product#/configuracao_regional/getIdiomaMoedaPadrao", "Row", {});
        }).then(function(response) {
            if (response.dataset.retorno.CDIDIOM) {
                cdidiom_padrao = wdgProposta.currentRow.CDIDIOM = response.dataset.retorno.CDIDIOM;
                nmidioma_padrao = wdgProposta.currentRow.NMIDIOMA = response.dataset.retorno.NMIDIOMA;
                nrmoeda_padrao = wdgProposta.currentRow.NRMOEDA = response.dataset.retorno.NRMOEDA;
                nmmoeda_padrao = wdgProposta.currentRow.NMMOEDA = response.dataset.retorno.NMMOEDA;
            }
        }).catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    }

    this.validaPerfilEvolucao = function(){
        var wdgProposta = getWidgetProposta();
        var newitem = getActionEditEvolucao();
        var edititem = getActionAdicEvolucao();
        if(wdgProposta.currentRow.IDSITUACAOORCAMENTO == "CONCLUIDO") {
            newitem.isVisible  = visibilidadeAcoes;
            edititem.isVisible = visibilidadeAcoes;
        }
    };

    /**
     * Seta o usuário logado como default no campo vendedor, caso ele seja um.
     * 
     * @param {object} wdgProposta - Widget de Dados da Proposta (view/form)
     */
    this.setVendedorPadrao = function(wdgProposta) {
        Operador_Repository.getOperadorLogado().then(function(success) {

                var nrparcnegocio = success[0].NRPARCNEGOCIO;
                var params = {
                    "NRPARCNEGOCIO": nrparcnegocio
                };

                General.callRestEngine("product#/fct_orcamento_widget/verificarSeEVendedor", "Row", params).then(function(success) {
                        if (success.dataset.DADOS.vendedor) {
                            wdgProposta.currentRow.NRVENDEDOR = [nrparcnegocio];
                            templateManager.updateTemplate();
                        }
                    },
                    function(error) {
                        console.error(error);
                        ScreenService.showMessage(error.data.error);
                    });
            },
            function(error) {
                console.error(error);
            });
    };

    /**
     * Controla a acessibilidade das abas filhas de Proposta, desabilitando-as em 
     * modo de inserção e habilitando-as em modo de edição.
     * 
     * @param {boolean} enabled
     */
    this.alterarAcessibilidadeAbasProposta = function(enabled) {
        var wdgPropostaTabbed = templateManager.container.getWidget("wdgPropostaTabbed");

        wdgPropostaTabbed.widgets.filter(function(wid) {
                return wid.name != "wdgProposta";
            })
            .forEach(function(widget) {
                widget.enabled = enabled;
            });

        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");

        wdgTabbedPrincipal.widgets.filter(function(wid) {
                return wid.name != "wdgPropostaTabbed";
            })
            .forEach(function(widget) {
                widget.enabled = enabled;
            });

        templateManager.updateTemplate();
    };

    /**
     * Volta para o modo list da tela de proposta.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.voltarParaPropostaGrid = function() {
        var deferred    = ZHPromise.defer(); 
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        wdgProposta.dataSource.data = [];
        wdgProposta.currentRow = {};

        ScreenService.closeSwipe().then(function() {
            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            wdgPropostaGrid.activate();
            wdgPropostaGrid.refreshWidget().then(function() {
                deferred.resolve();
            });
        });
        return deferred.promise;
    };

    /**
     * Cancela o salvamento da Proposta.
     * 
     * @param {object} wdgProposta - Widget de Proposta
     */
    this.cancelarSalvamentoProposta = function(wdgProposta) {
        var wdgItensComp              = templateManager.container.getWidget("wdgItensComposicaoProposta");
        var wdgItensProposta          = templateManager.container.getWidget("wdgItensProposta");
        var wdgItensPropostaInserindo = templateManager.container.getWidget("wdgItensPropostaInserindo");
        var wdgTipoMedidaInserindo    = templateManager.container.getWidget("wdgTipoMedidaPropostaInserindo");
        var wdgPropostaTabbed         = templateManager.container.getWidget("wdgPropostaTabbed");
        var wdgTabbedPrincipal        = templateManager.container.getWidget("wdgTabbedPrincipal");
        //var wdgLocalAtendimento = templateManager.container.getWidget("localAtendProposta");
        wdgPropostaTabbed.view();

        wdgItensComp.isVisible              = false;
        wdgItensPropostaInserindo.isVisible = false;
        wdgTipoMedidaInserindo.isVisible    = false;
        //wdgItensProposta.isVisible = true;
        //wdgLocalAtendimento.isVisible = true;
        //wdgProposta.enabled = true;
        //this.desfiltroEscopo(wdgProposta);
        //General.widgetCancelForm(wdgProposta);
        wdgPropostaTabbed.enabled = true;
        this.manipularCamposModeloProposta(wdgProposta, false);
        wdgProposta.currentRow.changed = false;
        this.alterarAcessibilidadeAbasProposta(true);
        wdgProposta.template = "widget/form-view.html";
        if (wdgProposta.currentRow.__is_new) {
            var indexOfCurrentRow = wdgProposta.dataSource.data.indexOf(wdgProposta.currentRow);
            if (indexOfCurrentRow !== -1) {
                wdgProposta.dataSource.data.pop();
            }

            wdgProposta.list();
            this.voltarParaPropostaGrid();
        } else { //caso a widget estiver em modo de edição, recupera o backup e entra no modo de visualização
            Object.keys(wdgProposta.backupRow).forEach(function(key) {
                wdgProposta.currentRow[key] = wdgProposta.backupRow[key];
            }); //copia dados coluna por coluna para não perder referência do objeto da currentRow
            wdgProposta.view();
            wdgTabbedPrincipal.view();
        }

        var wdgEscoposProposta = templateManager.container.getWidget("wdgEscoposProposta");
        wdgEscoposProposta.isVisible = true;
    };

    /**
     * Faz o controle geral da tela de Proposta de acordo com o registro selecionado.
     * 
     * @param {object} widget - Widget de Proposta (grid)
     */
    this.validacoesGeraisProposta = function(wdgPropostaGrid) {
        var situacao = wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO;
        //var utilizaComposicao = wdgPropostaGrid.currentRow.IDUTILIZACOMP;
        var utilizaComposicao = (wdgPropostaGrid.currentRow.QTITENSTRABSERV > 0) ? "S" : "N";
        var utilizaPrecoVariavel = wdgPropostaGrid.currentRow.IDUTILIZAVARPRECO;

        var wdgProposta = templateManager.container.getWidget("wdgProposta");

        //this.controlarAcoesDocumentoPropostaAbaEscopos(situacao);
        this.controlarCamposInlineEditable(situacao);
        this.controlarAcoesDeCrud(wdgProposta, situacao);
        this.controlarAcoesDeOperacaoProposta(wdgProposta, situacao);
        this.controlaActionEnviarEmail(wdgProposta);
        this.controlarAcoesDeOperacaoParcelasNegociacao(situacao);
        this.controlarVisibilidadeAcaoMemoriaDeCalculo(wdgProposta, utilizaComposicao);
        this.controlarVisibilidadeAbaElementoEditando(utilizaComposicao);
        this.controlarVisibilidadeAbaNomeContratualEditando(utilizaComposicao);
        this.controlarVisibilidadeAbaTipoMedidaVariavelEditando(utilizaComposicao, utilizaPrecoVariavel);
        
        this.controlarEscopoProposta(wdgProposta, false, false);

        var wid = templateManager.container.getWidget("wdgItensProposta");
        this.itensVerificarUtilizaComposicao(wid, utilizaComposicao, situacao);
       // this.controlarDataSourceItensProposta(wid, utilizaComposicao);
        this.controlarDataSourceItensProposta(utilizaComposicao);
        this.controlarCampoVariacaoMultiplicavelItensProposta(wid, utilizaComposicao);

        wid = templateManager.container.getWidget("wdgAdicionais");
        //this.controlarDataSourceItensProposta(wid, utilizaComposicao);
        this.itensVerificarUtilizaComposicao(wid, utilizaComposicao, situacao);
        //this.controlarDataSourceItensProposta(wid, utilizaComposicao);
        // this.validaIdCertsign(); // OTIMIZAÇÂO: estava duplicado portanto foi removido
    };
    
    this.controlarCampoVariacaoMultiplicavelItensProposta = function(wdgItensProposta, utilizaComposicao) {
        var campoVariacaoMultiplicavel = wdgItensProposta.getField("QTMEDIDAVARIAVEL");
        if (utilizaComposicao == "S") {
            campoVariacaoMultiplicavel.isVisible = true;
            campoVariacaoMultiplicavel.showOnList = true;
        }
        else {
            campoVariacaoMultiplicavel.isVisible = false;
            campoVariacaoMultiplicavel.showOnList = false;
        }
    };
    
    /**
     * Atribui o dataSource correto na tela de Itens da Proposta. O nome do 
     * dataSource depende se a proposta utiliza ou não composição de serviço.
     */
    this.controlarDataSourceItensProposta = function(/*wid,*/utilizaComposicao) {
        var widItens = templateManager.container.getWidget("wdgItensProposta");
        var widItensTree = templateManager.container.getWidget("wdgItensPropostaTree");
        
        if (utilizaComposicao == "S") {
            widItens.isVisible = false;
            widItensTree.isVisible = true;
            /*
            wid.dataSource.name = "product#/crud/fct_itemorcamento_arvore_medidas_variaveis";
            wid.template = "widget/master_detail/tree/grid.html";
            wid.templateToForm = "widget/master_detail/tree/grid.html";
            wid.primaryProperty = "IDARVORE";
            wid.parentProperty = "IDARVOREPAI";
            wid.expandingProperty = "NMSERVORCA";
            wid.breadcrumbProperty = "NMSERVORCA";
            templateManager.updateTemplate();
            wid.redraw();*/
        } else {
            widItens.isVisible = true;
            widItensTree.isVisible = false;
            
            /*wid.dataSource.name = "product#/crud/fct_itemorcamento_widget";
            wid.template = "widget/master_detail/grid.html";
            delete wid.primaryProperty;
            delete wid.parentProperty;
            delete wid.expandingProperty;
            delete wid.breadcrumbProperty;
            templateManager.updateTemplate();
            wid.redraw();*/
        }
    };

    /**
     * Permite ou bloqueia os campos "inlineEditable" de acordo com a situação da proposta.
     * @param {string} situacao
     */
    this.controlarCamposInlineEditable = function(situacao) {
        var wdgItensProposta  = templateManager.container.getWidget("wdgItensProposta");
        var wdgTipoMedida     = templateManager.container.getWidget("wdgTipoMedidaPropostaEditando");
        var wdgNomeContratual = templateManager.container.getWidget("wdgNomeContratual");
        
        var fieldsInline = [];
        var widgets      = [wdgItensProposta, wdgTipoMedida, wdgNomeContratual];
        
        widgets.forEach(function(widget) {
            fieldsInline = fieldsInline.concat(widget.fields.filter(function(field) {
                return field.editavelGrid == true;
            }));
        });
        
        if (situacao == "ELABORACAO" || (situacao == 'CONCLUIDO' && idAlteraPropConc == 'S' && visibilidadeAcoes)) {
            fieldsInline.forEach(function(field) {
                if (field.name == 'NMCONTRATUALITEMORCA'){
                    field.inlineEditable = "text";  
                }
                else{
                    field.inlineEditable = "number";
                }
            });
        } else {
            fieldsInline.forEach(function(field) {
                delete field.inlineEditable;
            });
        }
        
        /*var fieldsWdgItensProposta = wdgItensProposta.fields.filter(function(field) {
            return field.editavelGrid == true;
        });
        var fieldsWdgTipoMedida = wdgTipoMedida.fields.filter(function(field) {
            return field.editavelGrid == true;
        });
        var fieldsWdgNmContratual = wdgTipoMedida.fields.filter(function(field) {
            return field.editavelGrid == true;
        });
        
        if (situacao == "ELABORACAO") {
            fieldsWdgItensProposta.forEach(function(field) {
                field.inlineEditable = "number";
            });
            fieldsWdgTipoMedida.forEach(function(field) {
                field.inlineEditable = "number";
            });
            fieldsWdgNmContratual.forEach(function(field) {
                field.inlineEditable = "number";
            });
        }
        else {
            fieldsWdgItensProposta.forEach(function(field) {
                delete field.inlineEditable;
            });
            fieldsWdgTipoMedida.forEach(function(field) {
                delete field.inlineEditable;
            });
            fieldsWdgNmContratual.forEach(function(field) {
                delete field.inlineEditable;
            });
        }*/
    };

    /**
     * Habilita a ação de Memória de Cálculo caso a proposta utilize composição de serviço. 
     * Desabilita se não utilizar.
     * 
     * @param {object} wdgProposta - Widget de Proposta (view)
     * @param {boolean} utilizaComposicao
     */
    this.controlarVisibilidadeAcaoMemoriaDeCalculo = function(wdgProposta, utilizaComposicao) {
        var actMemoriaCalculo = wdgProposta.getAction("memoria_calculo_proposta");
        if (utilizaComposicao == "S") {
            actMemoriaCalculo.isVisible = true;
        }
        else {
            actMemoriaCalculo.isVisible = false;
        }
    };

    /**
     * Esconde as ações do documento da proposta da aba de Elementos
     * quando a proposta estiver em elaboração ou cancelada.
     
    this.controlarAcoesDocumentoPropostaAbaEscopos = function(situacao) {
        var wdgEscopos = templateManager.container.getWidget("wdgEscoposProposta");
        //var actDocumento = wdgEscopos.getAction("documento_proposta");
        var actEnviarPorEmail = wdgEscopos.getAction("enviar_documento");

        var visivel = true;
        if (situacao == "ELABORACAO" || situacao == "CANCELADO") {
            visivel = false;
        }

        //actDocumento.isVisible = visivel;
        actEnviarPorEmail.isVisible = visivel;
    }; */

    /**
     * Habilita a aba de Elementos caso a proposta utilize composição de serviço. 
     * Desabilita se não utilizar.
     */
    this.controlarVisibilidadeAbaElementoEditando = function(utilizaComposicao) {
        var widget = templateManager.container.getWidget("wdgElementosProposta");
        widget.isVisible = (utilizaComposicao == "S");
    };
    /**
     * Controla a visibilidade da widget de nome contratual
     * para aparecer apenas quando a proposta for de composição de serviço
     * @params{String} utilizaComposicao - define se utiliza ou não composição
    **/
    this.controlarVisibilidadeAbaNomeContratualEditando = function(utilizaComposicao){
        var widget = templateManager.container.getWidget("wdgNomeContratual");
        if(widget) widget.isVisible = (utilizaComposicao == 'S');
    };
    /**
     * Habilita a aba de Tipo de Medida Variável caso algum escopo da proposta utilize preço variável. 
     * Desabilita se não utilizar.
     */
    this.controlarVisibilidadeAbaTipoMedidaVariavelEditando = function(utilizaComposicao, utilizaPrecoVariavel) {
        var widget = templateManager.container.getWidget("wdgTipoMedidaPropostaEditando");
        if (utilizaComposicao == "S" && utilizaPrecoVariavel.indexOf("S") > -1) {
            this.validarVisibilidadeWidgetTpMedida(widget);
        } else {
            widget.isVisible = false;
        }
    };
    /**
     * Verifica a situação da Proposta para permitir/bloquear sua edição e/ou exclusão.
     * 
     * @param {object} wdgProposta - Widget de Proposta
     */
    this.controlarAcoesDeCrud = function(wdgProposta, situacao) {
        var actDelete = wdgProposta.getAction("delete");
        var actEdit   = wdgProposta.getAction("edit");
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
         
        if(situacao == "CONCLUIDO"){
            actDelete.isVisible = false;
            this.controlarAcoesDeCrudAux(wdgProposta.parent, (idAlteraPropConc == 'S' && visibilidadeAcoes), situacao);
            this.controlarAcoesDeCrudAux(wdgTabbedPrincipal, (idAlteraPropConc == 'S' && visibilidadeAcoes), situacao);
        } else {
            if (situacao == "ELABORACAO") {
                actDelete.isVisible = true;
                actEdit.isVisible   = true;
                this.controlarAcoesDeCrudAux(wdgProposta.parent, true, situacao);
                this.controlarAcoesDeCrudAux(wdgTabbedPrincipal, true, situacao);
                this.verificarEscopo(wdgProposta);
            } else {
                actEdit.isVisible   = false;
                actDelete.isVisible = (situacao == "CANCELADO");
                this.controlarAcoesDeCrudAux(wdgProposta.parent, false, situacao);
                this.controlarAcoesDeCrudAux(wdgTabbedPrincipal, false, situacao);
            }
        }
    };
    
    /**
     * Verifica se as proposta concluídas podem ser alterada
     * @params {Object} widget
    **/
    this.verificaParamAlteraPropConc = function(widget){
        var deferred = ZHPromise.defer();
        if(idAlteraPropConc){
            deferred.resolve();
        } else {
            var nrorg = getLocalVar("PROPOSTA_NRORG");
            var paramsGeral  = [
                {
                    name: "NRORG",
                    operator: "=",
                    value: nrorg
                }
            ];

            General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", paramsGeral).then(function(success) {
                if(success.dataset.fct_parametrogeral.length !== 0)
                    idAlteraPropConc = success.dataset.fct_parametrogeral[0].IDALTERAPROPCONC;
                
                if(visibilidadeAcoes !== null) {
                    deferred.resolve();
                } else {
                    this.validarPermissaoAlteracaoDoPerfilAcesso(widget, nrorg).then(function(){
                        deferred.resolve();
                    });
                }     
                
            }.bind(this))
            .catch(function(error) {
                console.log(error);
                deferred.reject();
            });
        }
        
        return deferred.promise;
    };
    

    /**
     * Verifica se a proposta utiliza composição de serviços para bloquear a edição dos escopos.
     * 
     * @param {object} wdgProposta - Widget de Proposta (view)
     */
    this.verificarEscopo = function(wdgProposta) {
        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");

        //var utilizaComposicao = wdgPropostaGrid.currentRow.IDUTILIZACOMP;
        var utilizaComposicao = (wdgPropostaGrid.currentRow.QTITENSTRABSERV > 0) ? "S" : "N";
        var wdgEscoposProposta = templateManager.container.getWidget("wdgEscoposProposta");

        wdgEscoposProposta.getAction("add").isVisible = false;
        wdgEscoposProposta.getAction("delete").isVisible = false;
    };

    /**
     * Muda a visibilidade das ações de crud das widgets filhas de proposta
     */
    this.controlarAcoesDeCrudAux = function(widget, visivel, situacao) {
        var widgetsIgnoradas = ["wdgDocumento", "wdgProposta", "wdgPropostaTabbed"];

        widget.widgets.filter(function(wid) {
                return widgetsIgnoradas.indexOf(wid.name) == -1;
            })
            .forEach(function(wid) {
                this.setVisibilidadeDasAcoesDeCrud(wid, visivel, situacao);
                if (wid.widgets.length > 0) {
                    this.controlarAcoesDeCrudAux(wid, visivel, situacao);
                }
            }.bind(this));
    };

    /**
     * Controla as ações do canto direito da tela de Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     * @param string situacao - Situação da Proposta
     */
    this.controlarAcoesDeOperacaoProposta = function(widget, situacao) {
        var actConcluir       = widget.getAction("concluir_proposta");
        var actAceitar        = widget.getAction("aceitar_proposta");
        var actCopiar         = widget.getAction("copiar_proposta");
        var actVersionar      = widget.getAction("versionar_proposta");
        var actCancelar       = widget.getAction("cancelar_proposta");
        var actMemoriaCalculo = widget.getAction("memoria_calculo_proposta");
        /*var actDocProposta = widget.getAction("documento_proposta");
        var actEnviarPorEmail = widget.getAction("enviar_documento");*/

        switch (situacao) {
            case "ELABORACAO":
                {
                    actConcluir.isVisible       = true;
                    actAceitar.isVisible        = false;
                    actCopiar.isVisible         = true;
                    actVersionar.isVisible      = true;
                    actCancelar.isVisible       = true;
                    actMemoriaCalculo.isVisible = true;
                    /*actDocProposta.isVisible  = false;
                    actEnviarPorEmail.isVisible = false;*/
                    break;
                }
            case "CANCELADO":
                {
                   actConcluir.isVisible       = false;
                   actAceitar.isVisible        = false;
                   actCopiar.isVisible         = true;
                   actVersionar.isVisible      = true;
                   actCancelar.isVisible       = false;
                   actMemoriaCalculo.isVisible = false;
                   /* actDocProposta.isVisible = false;
                   actEnviarPorEmail.isVisible = false;*/
                    break;
                }
            case "ACEITO":
                {
                    actConcluir.isVisible       = false;
                    actAceitar.isVisible        = false;
                    actCopiar.isVisible         = true;
                    actVersionar.isVisible      = true;
                    actCancelar.isVisible       = true;
                    actMemoriaCalculo.isVisible = true;
                    /*actDocProposta.isVisible  = true;
                    actEnviarPorEmail.isVisible = true;*/
                    break;
                }
            default:
                { //CONCLUIDO ou ENVIADO
                    actConcluir.isVisible       = false;
                    actAceitar.isVisible        = true;
                    actCopiar.isVisible         = true;
                    actVersionar.isVisible      = true;
                    actCancelar.isVisible       = true;
                    actMemoriaCalculo.isVisible = true;
                    /*actDocProposta.isVisible  = true;
                    actEnviarPorEmail.isVisible = true;*/
                    break;
                }
        }
        // this.validaIdCertsign();
    };

    /**
     * Ativa/desativa todas as actions novo, editar e excluir de uma widget.
     * 
     * @param {object} widget
     * @param {boolean} visivel
     */
    this.setVisibilidadeDasAcoesDeCrud = function(widget, visivel, situacao) {
        var actions               = ["add", "edit", "delete"];
        var wdgsPermiteExclusao   = ["wdgEndereco", "wdgItensProposta"];
        var wdgsPermitemAlteracao = ["wdgEscoposProposta", "wdgElementosProposta", "wdgNegociacaoValores", "wdgEndereco", "wdgItensProposta", "wdgItensPropostaTree"];

        if(situacao === "CONCLUIDO" && wdgsPermitemAlteracao.indexOf(widget.name) !== -1){
            widget.getAction("edit").isVisible = (idAlteraPropConc == 'S' && visibilidadeAcoes);   
            actions = [];

            if (wdgsPermiteExclusao.indexOf(widget.name) !== -1){
                widget.getAction("delete").isVisible = (idAlteraPropConc == 'S' && visibilidadeAcoes);
                widget.getAction("add").isVisible    = (idAlteraPropConc == 'S' && visibilidadeAcoes);
            }
        }
        
        widget.actions.forEach(function(action) {
            if (actions.includes(action.name)) {
                action.isVisible = visivel;
            }
        });
        
    };

    /**
     * Verifica se foi selecionado um Interlocutor para bloquear/liberar o campo Descrição do Interlocutor.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.verificarInterlocutor = function(widget) {
        var fldInterloc = widget.getField("NMPARCINTERLOCUTOR");
        var fldDescInterloc = widget.getField("DSINTERORCA");

        if (fldInterloc.value().length > 0) {
            fldDescInterloc.value("");
            fldDescInterloc.readOnly = true;
        }
        else {
            fldDescInterloc.readOnly = false;
        }
    };

    this.verificarInterlocutorfat = function(widget) {
        var fldInterloc = widget.getField("NMPARCINTERLOCUTORFAT");
        var fldDescInterloc = widget.getField("DSINTERORCA");

        if (fldInterloc.value().length > 0) {
            fldDescInterloc.value("");
            fldDescInterloc.readOnly = true;
        }
        else {
            fldDescInterloc.readOnly = false;
        }
    };

    this.parceiroOnChange = function(wdgProposta) {
        this.filtroInterlocutor(wdgProposta, true);
        this.filtroOportunidade(wdgProposta);
        if (wdgProposta.currentRow.__is_new) {
            if (wdgProposta.currentRow.CDIDIOM_PARC) {
                wdgProposta.currentRow.CDIDIOM = wdgProposta.currentRow.CDIDIOM_PARC;
                wdgProposta.currentRow.NMIDIOMA = wdgProposta.currentRow.NMIDIOMA_PARC;
            }
            else if (cdidiom_padrao) {
                wdgProposta.currentRow.CDIDIOM = cdidiom_padrao;
                wdgProposta.currentRow.NMIDIOMA = nmidioma_padrao;
            }

            if (wdgProposta.currentRow.NRMOEDA_PARC) {
                wdgProposta.currentRow.NRMOEDA = wdgProposta.currentRow.NRMOEDA_PARC;
                wdgProposta.currentRow.NMMOEDA = wdgProposta.currentRow.NMMOEDA_PARC;
                this.moedaOnChange(wdgProposta);
            }
            else if (nrmoeda_padrao) {
                wdgProposta.currentRow.NRMOEDA = nrmoeda_padrao;
                wdgProposta.currentRow.NMMOEDA = nmmoeda_padrao;
                this.moedaOnChange(wdgProposta);
            }
        }
    };

    /**
     * filtra o campo Interlocutor.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.filtroInterlocutor = function(widget, limparCampos) {
        
        var campoInterlocutor = widget.getField("NMPARCINTERLOCUTOR");
        var campoInterlocutorfat = widget.getField("NMPARCINTERLOCUTORFAT"); 
        campoInterlocutorfat.dataSourceFilter[0].value = widget.currentRow.NRPARCNEGOCIO;
        campoInterlocutor.dataSourceFilter[0].value = widget.currentRow.NRPARCNEGOCIO;
        if (limparCampos) {
            widget.currentRow.NMPARCINTERLOCUTOR = null;
            widget.currentRow.DSEMAIL = null;
            widget.currentRow.NRTELEFONE = null;
        }
        
        campoInterlocutor.reload();
        campoInterlocutorfat.reload();
    };

    /**
     * Filtra o campo Oportunidade de Negócio.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.filtroOportunidade = function(widget) {
        var campoOportunidade = widget.getField("NMOPORT");
        campoOportunidade.dataSourceFilter[2].value = widget.currentRow.NRPARCNEGOCIO;
        campoOportunidade.reload();
    };

    /**
     * Salva a proposta.
     * 
     * @param {object} wdgProposta - Widget de Proposta
     */
    this.salvarProposta = function(wdgProposta) {

        wdgProposta.currentRow.__columns_to_return = ["NRSEQORCAMENTO", "NRORCAMENTO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];

        if (wdgProposta.isValid()) {

            var mudouParceiro = false;
            if (wdgProposta.backupRow && wdgProposta.backupRow.NRPARCNEGOCIO !== wdgProposta.currentRow.NRPARCNEGOCIO) {
                mudouParceiro = true;
            }
            
            var mudouInterlocutor = false;
            if (wdgProposta.backupRow && wdgProposta.backupRow.NRSEQINTERLOCUTOR !== wdgProposta.currentRow.NRSEQINTERLOCUTOR) {
                mudouInterlocutor = true;
                var nrinterlocutorAnte =  wdgProposta.backupRow.NRSEQINTERLOCUTOR;
            }

            var mudouInterlocutorfat = false;
            if (wdgProposta.backupRow && wdgProposta.backupRow.NRSEQINTERLOCUTORFAT !== wdgProposta.currentRow.NRSEQINTERLOCUTORFAT) {
                mudouInterlocutorfat = true;
            }
            
            this.salvarWidgetProposta(wdgProposta).then(function(response) {

                wdgProposta.view();
                templateManager.container.getWidget('wdgTabbedPrincipal').view();
                wdgProposta.template = "widget/form-view.html";
                this.alterarAcessibilidadeAbasProposta(true);

                var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                wdgPropostaGrid.reload();

                wdgProposta.currentRow.changed = false;
                delete wdgProposta.backupRow;

                if (wdgProposta.currentRow.__is_new) {
                    wdgProposta.currentRow.__is_new = false;
                    this.inserirAssinaturasOrca(wdgProposta);
                    this.voltarParaPropostaGrid();
                    
                    var wdgEscoposProposta = templateManager.container.getWidget("wdgEscoposProposta");
                    wdgEscoposProposta.isVisible = true;
                } else {
                    if (response.dataset && response.dataset.columnsToReturn[0]) {
                        wdgProposta.currentRow.NRSEQORCAMENTO = response.dataset.columnsToReturn[0].NRSEQORCAMENTO;
                        wdgProposta.currentRow.NRORCAMENTO = response.dataset.columnsToReturn[0].NRORCAMENTO;
                        wdgProposta.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                        wdgProposta.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                        wdgProposta.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                        wdgProposta.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                    }

                    this.manipularCamposModeloProposta(wdgProposta, false);
                    //this.desfiltroEscopo(wdgProposta);
                    this.verificarObrigatoriedadeOportunidade(wdgProposta);

                    wdgProposta.reload();

                    if (mudouParceiro) {
                        var params = {
                            "NRSEQORCAMENTO": wdgProposta.currentRow.NRSEQORCAMENTO,
                            "NRPARCNEGOCIO": wdgProposta.currentRow.NRPARCNEGOCIO
                        };
                        General.callRestEngine("product#/fct_orcamento_widget/atualizarParceiroNegociacoesProposta", "Row", params)
                        .then(null)
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }
                    
                    if (mudouInterlocutor) {
                        var paramsInt = {
                            "NRSEQORCAMENTO": wdgProposta.currentRow.NRSEQORCAMENTO,
                            "NRSEQINTERLOCUTOR": wdgProposta.currentRow.NRSEQINTERLOCUTOR,
                            "NRINTERLOCUTORANTE": nrinterlocutorAnte,
                            "NRORCAMENTOMODELO": wdgProposta.currentRow.NRORCAMENTOMODELO,
                        };
                        General.callRestEngine("product#/fct_orcamento_widget/insereInterlocutor", "Row", paramsInt)
                        .then(null)
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }

                    if (mudouInterlocutorfat) {
                        var paramsInt = {
                            "NRSEQORCAMENTO": wdgProposta.currentRow.NRSEQORCAMENTO,
                            "NRPARCINTERLOCUTORFAT": wdgProposta.currentRow.NRSEQINTERLOCUTORFAT,
                            "NRINTERLOCUTORANTE": nrinterlocutorAnte,
                            "NRORCAMENTOMODELO": wdgProposta.currentRow.NRORCAMENTOMODELO,
                        };
                        General.callRestEngine("product#/fct_orcamento_widget/insereInterlocutor", "Row", paramsInt)
                        .then(null)
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }
                    wdgProposta.reload();
                }
                    
            }.bind(this),
            function(error) {
                console.error(error);
                var errorMessage = 'Timeout';
                if (error.data) errorMessage = error.data.error;
                ScreenService.showMessage(errorMessage);
            });
        } else {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
    };
    
    /**
    * Verifica se a proposta está ligada a uma oportunidade
    * de negócio
    **/
    this.verificaIntegracaoPropOpor = function(rowProposta){
        var params = [
            {
                name: "NRORCAMENTOMODELO",
                operator: "=",
                value: rowProposta.NRORCAMENTOMODELO
            }
        ];
        
        General.callRestEngine("product#/crud/fct_orcamentomodelo_widget", "FilterData", params).then(function(response){
            if(response.dataset.fct_orcamentomodelo_widget[0] && response.dataset.fct_orcamentomodelo_widget[0]["IDOBRIGAOPORTUNIDADE"] === "S"){
                ScreenService.confirmMessage("Deseja gerar uma evolução na oportunidade de negócio vinculada à proposta?", "question", function() {
                    var paramsOrcamento = [
                        {
                            name: "NRSEQORCAMENTO",
                            operator: "=",
                            value: rowProposta.NRSEQORCAMENTO
                        }
                    ];
                            
                    General.callRestEngine("product#/fct_orcamento_widget", "FilterData", paramsOrcamento).then(function(success) {
                        if(success.dataset.fct_orcamento_widget[0]){
                            var paramsService = [
                                {
                                    name: "NRORG",
                                    operator: "=",
                                    value: success.dataset.fct_orcamento_widget[0].NRORG
                                }
                            ];
                            
                            General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", paramsService).then(function(response){
                                
                                if(response.dataset.fct_parametrogeral[0] && response.dataset.fct_parametrogeral[0].IDINTEGRACAOCRM  === "S" && response.dataset.fct_parametrogeral[0].URLINTEGRACAOCRM !== null){
                                    this.abrirWdgEvoluirCRM(success.dataset.fct_orcamento_widget[0]);
                                } else {
                                    ScreenService.showMessage("Não há parametrizações cadastradas para a integração com o CRM.");
                                }
                                
                            }.bind(this))
                            .catch(function(error){
                                console.error(error);
                            });
                        }
                        
                    }.bind(this))
                    .catch(function(error){
                        console.error(error);
                    });       
                }.bind(this));
            }
        }.bind(this));
    };
    
    /**
     * Função disparada ao modificar o campo Modelo de Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.modeloOnChange = function(widget) {
        this.verificarModeloProposta(widget);
        this.calcularDataValidadeProposta(widget);
        this.controlaCodProposta(widget);
        this.escoposOnChange(widget);
        this.controlaEscopoModelo(widget);
        this.validaIdCertsign();
        this.assinaturaEletronicaOnChange(widget);
        this.habilitaCampoRevenda();
    };
    
    /**
     * Controla a field de escopo de acordo com o modelo de orcamento
     * escolhido. 
     * @param {Object} widget widget propostas
    **/
    this.controlaEscopoModelo = function(widget){
        var nmFieldEscopo = (widget.currentRow.IDUTILIZACOMP == "S") ? "NRTIPOORCAMENTO" : "NMTIPOORCAMENTO";
        var fieldEscopo   = widget.getField(nmFieldEscopo);
        var fieldIsArray  = (fieldEscopo.name === "NRTIPOORCAMENTO");
        
        fieldEscopo.reload().then(function(){
            if(fieldEscopo.dataSource.data.length > 1){
                fieldEscopo.readOnly = false;
                widget.currentRow.NRTIPOORCAMENTO = fieldIsArray ? [] : "";
                widget.currentRow.NMTIPOORCAMENTO = fieldIsArray ? [] : "";
                fieldEscopo.dataSource.clearCheckedRows();
            } else {
                if(fieldEscopo.dataSource.data.length == 1){
                    fieldEscopo.readOnly = true;
                    var nrtipoorcamento = fieldEscopo.dataSource.data[0].NRTIPOORCAMENTO;
                    var nmtipoorcamento = fieldEscopo.dataSource.data[0].NMTIPOORCAMENTO;
                    var utilizavariacao = fieldEscopo.dataSource.data[0].IDUTILIZAVARPRECO;
                    
                    widget.currentRow.NRTIPOORCAMENTO   = fieldIsArray ? [nrtipoorcamento] : nrtipoorcamento;
                    widget.currentRow.NMTIPOORCAMENTO   = fieldIsArray ? [nmtipoorcamento] : nmtipoorcamento;
                    widget.currentRow.IDUTILIZAVARPRECO = fieldIsArray ? [utilizavariacao] : utilizavariacao;
                    
                    fieldEscopo.dataSource.addCheckedRows(fieldEscopo.dataSource.data[0]);
                    fieldEscopo.dataSource.data[0].__isSelected = true;
                }
            }
            UtilitiesPropostaContrato.validarParamsFaturamentoEscopo(widget, nmFieldEscopo, "ORC");
        });
    };

    /**
     * Controla campo "Código" da proposta
     */
    this.controlaCodProposta = function(widget) {
        var modProposta = widget.currentRow.NMORCAMENTOMODELO;
        var fieldCodigo = widget.getField("NRORCAMENTO");

        if (modProposta && widget.currentRow.__is_new) {
            fieldCodigo.readOnly = true;
            widget.currentRow.NRORCAMENTO = "";
            fieldCodigo.validations = {};
        }
        else {
            fieldCodigo.readOnly = false;
            fieldCodigo.validations = {
                "required": true
            };
        }
    };

    /**
     * Ao carregar um modelo, é verificado se seus dados não poderão ser alterados e se será
     * obrigatório informar uma oportunidade de negócio.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.verificarModeloProposta = function(widget) {
        var idbloqalteradados = widget.currentRow.IDBLOQALTERADADOS;
        var readOnly = (idbloqalteradados == "NAO") ? true : false;
        this.manipularCamposModeloProposta(widget, readOnly);

        var utilizaComposicao = (widget.currentRow.IDUTILIZACOMP == "S");
        this.controlarEscopoProposta(widget, utilizaComposicao);
        this.controlarWidgetItens(widget, utilizaComposicao);

        this.verificarObrigatoriedadeOportunidade(widget);
    };

    /**
     * Controla o campo de Escopo de Proposta de acordo com o Modelo (se utiliza composição de serviços ou não).
     * 
     * @param {object} widget - Widget de Proposta
     * @param {boolean} utilizaComposicao
     */
    this.controlarEscopoProposta = function(widget, utilizaComposicao, limparCampos) {
        if (limparCampos == undefined) {
            limparCampos = true;
        }

        var fieldCodEscopo    = widget.getField("NRTIPOORCAMENTO");
        var fieldNomeEscopo   = widget.getField("NMTIPOORCAMENTO");
        var fieldUtilVariacao = widget.getField("IDUTILIZAVARPRECO");
        fieldNomeEscopo.readOnly   = false;
        fieldCodEscopo.readOnly    = false;
        this.filtroEscopo(widget, utilizaComposicao);
        if (utilizaComposicao) {
            fieldNomeEscopo.isVisible  = false;
            //fieldNomeEscopo.showOnView = false;
            fieldNomeEscopo.showOnForm = false;
            fieldNomeEscopo.template   = "field/text-edit.html";
            
            fieldCodEscopo.template    = "field/select-multiple.html";
            fieldCodEscopo.validations = {
                "required": true
            };
            fieldCodEscopo.isVisible  = true;
            //fieldCodEscopo.showOnView = true;
            fieldCodEscopo.showOnForm = true;
            fieldCodEscopo.reload().then(function() {
                if (fieldCodEscopo.dataSource.data.length == 1) {
                    fieldCodEscopo.value([fieldCodEscopo.dataSource.data[0].NRTIPOORCAMENTO]);
                    fieldNomeEscopo.value([fieldCodEscopo.dataSource.data[0].NMTIPOORCAMENTO]);
                    fieldUtilVariacao.value([fieldCodEscopo.dataSource.data[0].IDUTILIZAVARPRECO]);
                }
            });
        } else {
            fieldCodEscopo.isVisible = false;
            //fieldCodEscopo.showOnView = false;
            fieldCodEscopo.showOnForm = false;
            fieldCodEscopo.template = "field/text-edit.html";
            fieldCodEscopo.validations = {};

            fieldNomeEscopo.template = "zh-select-autocomplete#field/select-autocomplete.html";
            fieldNomeEscopo.isVisible = true;
            //fieldNomeEscopo.showOnView = true;
            fieldNomeEscopo.showOnForm = true;
        }

        if (limparCampos) {
            fieldNomeEscopo.value("");
            fieldCodEscopo.value("");
        }
    };

    /**
     * filtra o campo Escopo de Proposta quando tem Modelo.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.filtroEscopo = function(widget, utilizaComposicao) {
        var fieldEscopo       = utilizaComposicao ? widget.getField("NRTIPOORCAMENTO") : widget.getField("NMTIPOORCAMENTO");
        var nrorcamentomodelo = widget.currentRow.NRORCAMENTOMODELO;
        if(fieldEscopo.name == "NRTIPOORCAMENTO"){
            fieldEscopo.dataSourceFilter[1].value = nrorcamentomodelo;
        } else {
            if(!nrorcamentomodelo){
                fieldEscopo.dataSource.name = "product#/crud/fct_tipoorcamento_widget";
                fieldEscopo.dataSourceFilter = [fieldEscopo.dataSourceFilter[0]];
            } else {
                fieldEscopo.dataSource.name = "product#/fct_relacescopomodelo_widget";
                fieldEscopo.dataSourceFilter[1] = {
                    name: "NRORCAMENTOMODELO",
                    operator: "=",
                    value: nrorcamentomodelo
                };
            }
        }
        fieldEscopo.reload();
    };

    /**
     * filtra o campo Escopo de Proposta quando tem Modelo.
     * 
     * @param {object} widget - Widget de Proposta
     */
    /*this.desfiltroEscopo = function(widget) {
        var fieldEscopo = widget.getField("NRTIPOORCAMENTO");
        fieldEscopo.dataSource.name = "product#/crud/fct_tipoorcamento_widget";
        delete fieldEscopo.dataSourceFilter[1];
        fieldEscopo.reload();
    };*/

    /**
     * Escolhe qual widget de Itens irá aparecer de acordo com o parâmetro
     * "Utiliza composição de serviços" do Modelo de Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     * @param {boolean} utilizaComposicao
     */
    this.controlarWidgetItens = function(widget, utilizaComposicao) {
        if (utilizaComposicao) {
            widget.getAction("save").isVisible = false;
            widget.getAction("avancar_composicao").isVisible = true;
        }
        else {
            widget.getAction("avancar_composicao").isVisible = false;
            widget.getAction("save").isVisible = true;
        }
    };
    
    /**
     * Valida se o valor alterado na quantidade de medida
     * continua coerente com os demais
     * @params{Object} widget - widget itens proposta
    **/
    this.validarEdicoesInline = function(widget, elementos){
        var deferred = ZHPromise.defer();
        elementos.forEach(function(elemento){
            var quantidade         = elemento.QTITEMORCAMENTO;
            var utilizaQtdeMedida  = elemento.IDMULTMEDVRTOT;
            var qtdeMedida         = elemento.QTMEDIDAORCA_GRID;
            var valorUnitario      = elemento.VRUNITITEMORCA;
            var tipoAjuste         = elemento.IDTPDESCORCA;
            var ajuste             = elemento.VRPERCDESCORCA;
            var qtdeVariacao       = elemento.QTMEDIDAVARIAVEL;
            var multiplicaVariacao = (elemento.IDMULTIPVARIACAO == "S");
            elemento.QTMEDIDAORCA  = elemento.QTMEDIDAORCA_GRID;

            if (qtdeVariacao == undefined || qtdeVariacao == "")
                qtdeVariacao = 1;
        
            if (elemento.QTMEDIDAORCA) {
                if (elemento.IDALTQTDMEDIDAPROP == "4") { //nao altera
                    elemento.QTMEDIDAORCA             = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_inline      = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_GRID        = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_GRID_inline = elemento.QTMEDIDACALC;
                    return deferred.reject("Operação bloqueada.<br/>Não é permitido alterar a Quantidade de Medida do elemento: <br>" + elemento._parent["NMITEMORCAMENTO"] + " -> " + elemento["NMITEMORCAMENTO"]); 
                } else if (elemento.IDALTQTDMEDIDAPROP == "3" && elemento.QTMEDIDAORCA > elemento.QTMEDIDACALC) { //somente reduzir
                    elemento.QTMEDIDAORCA             = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_inline      = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_GRID        = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_GRID_inline = elemento.QTMEDIDACALC;
                    return deferred.reject("Operação bloqueada.<br/>Não é permitido aumentar a Quantidade de Medida do item <br>"+ elemento._parent["NMITEMORCAMENTO"] + " -> " +elemento["NMITEMORCAMENTO"]+". O campo foi atribuído com o valor máximo (" + elemento.QTMEDIDACALC.formatFloat(2) + ").");
                } else if (elemento.IDALTQTDMEDIDAPROP == "2" && elemento.QTMEDIDAORCA < elemento.QTMEDIDACALC) { //somente aumentar
                    elemento.QTMEDIDAORCA             = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_inline      = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_GRID        = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_GRID_inline = elemento.QTMEDIDACALC;
                    return deferred.reject("Operação bloqueada.<br/>Não é permitido reduzir a Quantidade de Medida do item<br> "+ elemento._parent["NMITEMORCAMENTO"] + " -> " +elemento["NMITEMORCAMENTO"]+".<br> O campo foi atribuído com o valor mínimo (" + elemento.QTMEDIDACALC.formatFloat(2) + ").");
                }
            }

            if (quantidade) quantidade = parseFloat(quantidade.toString().replace(",", "."));
            if (qtdeMedida) qtdeMedida = parseFloat(qtdeMedida.toString().replace(",", "."));
            if (valorUnitario) valorUnitario = parseFloat(valorUnitario.toString().replace(",", "."));
            if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));
            var valorSemAjuste = quantidade * valorUnitario;
            if (utilizaQtdeMedida == "S") 
                valorSemAjuste *= qtdeMedida;
            if (multiplicaVariacao) 
                valorSemAjuste *= qtdeVariacao;

            var valorTotal = valorSemAjuste;
            if (tipoAjuste == "V") {
                valorTotal -= ajuste;
            } else if (tipoAjuste == "P") {
                valorTotal -= valorTotal * ajuste / 100;
            } else if (tipoAjuste == "X") {
                valorTotal += ajuste;
            } else if (tipoAjuste == "Y") {
                valorTotal += valorTotal * ajuste / 100;
            }
    
            if(valorTotal < 0)
                return deferred.reject("Operação bloqueada!<br>Não foi possível atualizar o valor da quantidade de medida do elemento, pois o novo valor total não está coerente com o ajuste do mesmo.");
            else 
                return deferred.resolve();
        }.bind(this));
        return deferred.promise;
    };
    
    this.salvarItensInline = function(widget) {
        var itensTipoI = widget.dataSource.data.filter(function(row) {
            return row.IDTIPO == "I" && row.changed == true;
        });
        var itensTipoM = widget.dataSource.data.filter(function(row) {
            return row.IDTIPO == "M" && row.changed == true;
        });
        
        this.validarEdicoesInline(widget, itensTipoM).then(function(){
            var params = {
                "ELEMENTOS": itensTipoM
            };
            
            General.callRestEngine("product#/fct_orcamento_widget/salvarElementosAlteradosPeloUsuario", "Row", params).then(function() {
                params = {};
                if(itensTipoM.length === 0){
                    return General.callRestEngine(widget.dataSource.name + "/save", "DataSet", itensTipoI);
                } else {
                    widget.reload().then(function(){
                        var eleTipoI = [];
                        itensTipoM.forEach(function(elemento){
                            if(elemento.IDARVOREPAI != null)
                                eleTipoI.push(elemento.IDARVOREPAI);
                        });
                        eleTipoI.forEach(function(elementPai){
                            var newValue = 0;
                            var itemPaiAlterado = {};
                            widget.dataSource.data.forEach(function(item){
                                itemPaiAlterado = (item.IDARVORE !== elementPai) ? itemPaiAlterado : item;
                                if(item.IDARVOREPAI == elementPai)
                                    newValue += item.VRTOTALITEMORCA;
                            });
                            var index = itensTipoI.indexOf(itemPaiAlterado);
                            itemPaiAlterado.VRUNITITEMORCA  = newValue;
                            itemPaiAlterado.VRTOTALITEMORCA = newValue;
                            if(index === -1)
                                itensTipoI.push(itemPaiAlterado);
                            else 
                                itensTipoI[index] = itemPaiAlterado;
                        });
                        General.callRestEngine(widget.dataSource.name + "/save", "DataSet", itensTipoI).then(function(){
                            return widget.reload();
                        });
                    });
                }
            }).then(function() {
                widget.list();
                widget.reload();
                var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                this.atualizarValorProposta(wdgPropostaGrid);
            }.bind(this))
            .catch(function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        }.bind(this)).catch(function(error){
            ScreenService.showMessage(error);
        });
    };
    
    /**
     * Atualiza o ajuste dos elementos da arvore de composição de serviço
     * @params{Object} widget widget itens
    **/
    this.replicaAteracaoParaItemPai = function(widget){    
        var row = {
            "NRSEQORCAMENTO"  : widget.currentRow.NRSEQORCAMENTO,
            "CDSERVORCA"      : widget.currentRow.CDSERVORCA
        };
        
        return General.callRestEngine("product#/fct_itemorcamento_widget/replicaAteracaoParaItemPai", "Row", row);
    };
    
    /**
     * Realiza a s validaçoes antes de salvar um registro
     * 
     * @param {object} widget widget de 
     * Item de Proposta para salvar o novo registro.
     */
    this.validaPercentualSaveItens = function(widget) {
        if (widget.isValid()) {
            var vrunititemorcatblpreco = widget.currentRow.VRUNITITEMORCATBLPRECO;
            var vrunititemorca         = widget.currentRow.VRUNITITEMORCA;
            
            vrunititemorcatblpreco = Utilities.tirarMascaraValores(vrunititemorcatblpreco);
            vrunititemorca         = Utilities.tirarMascaraValores(vrunititemorca);

            if (vrunititemorca > vrunititemorcatblpreco) {
                var nritempai = widget.currentRow.NRITEMORCAMENTOPAI;
                var percAcrescimoServico = 0;
                widget.dataSource.data.filter(function(row) {
                    return row.NRITEMORCAMENTOPAI == nritempai;
                }).forEach(function(row) {
                    var percentual = (Utilities.tirarMascaraValores(row.VRUNITITEMORCA) - Utilities.tirarMascaraValores(row.VRUNITITEMORCATBLPRECO)) / Utilities.tirarMascaraValores(row.VRUNITITEMORCATBLPRECO) * 100;
                    percAcrescimoServico += percentual;
                });

                var params = {
                    "CDSERVORCA" : widget.currentRow.CDSERVORCA,
                    "IDTIPO"     : "A"
                };
                
                General.callRestEngine("product#/fct_itemorcamento_widget/validaPorcentualDescontoAcrescimoSave", "Row", params).then(function(response) {
                    var resultado = response.dataset.RESULTADO.VRPERCDESC;
                    if (resultado && percAcrescimoServico > resultado.VRPERCDESC) {
                        var contexto = (resultado.TIPO == 1) ? "Este": "O perfil de acesso deste";
                        ScreenService.showMessage("Operação bloqueada.<br/>"+contexto+" operador não tem autorização para propor acréscimo superior a " + resultado.VRPERCDESC + "%.");
                    } else {
                       //  this.validarDadosRateioItens(widget); Por hora, não está sendo utilizado rateio
                       this.salvarItens(widget);
                    }
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            } else {
              //  this.validarDadosRateioItens(widget);  Por hora, não está sendo utilizado rateio
              this.salvarItens(widget);
            }
        }
        else {
            ScreenService.showMessage("Preencha todos os campos corretamente!");
        }
    };

    /**var nritempai = widget.currentRow.NRITEMORCAMENTOPAI;
            var percAcrescimoServico = 0;
            widget.dataSource.data.filter(function(row) {
                return row.NRITEMORCAMENTOPAI == nritempai;
            })
            .forEach(function(row) {
                var percentual = (Utilities.tirarMascaraValores(row.VRUNITITEMORCA) - Utilities.tirarMascaraValores(row.VRUNITITEMORCATBLPRECO)) / Utilities.tirarMascaraValores(row.VRUNITITEMORCATBLPRECO) * 100;
                percAcrescimoServico += percentual;
            });

            var params = {
                "CDSERVORCA": widget.currentRow.CDSERVORCA,
                "IDTIPO": "A"
            };
            General.callRestEngine("product#/fct_itemorcamento_widget/validaPorcentualDescontoAcrescimoSave", "Row", params)
     * Converte o valor de ajuste para valor percentual
     * 
     * @param {object} widget widget de 
     * Item de Proposta para salvar o novo registro.
     */
    this.converteParaPercentual = function(widget) {
        var percentual        = widget.currentRow.VRPERCDESCORCA;
        var tipoAjuste        = widget.currentRow.IDTPDESCORCA;
        var quantidade        = widget.currentRow.QTITEMORCAMENTO;
        var valorUnitario     = widget.currentRow.VRUNITITEMORCA;
        var utilizaQtdeMedida = widget.currentRow.IDMULTMEDVRTOT;
        var qtdeMedida        = widget.currentRow.QTMEDIDAORCA;
        var valorSemAjuste    = quantidade * valorUnitario;
        if (utilizaQtdeMedida == "S") {
            valorSemAjuste *= qtdeMedida;
        }

        var valorTotal = valorSemAjuste;

        if (tipoAjuste == "V" || tipoAjuste == "X") {
            return (percentual / valorTotal) * 100;
        }
        else
            return percentual;
    };

    /**
     * Salva um item ou um adicional.
     * 
     * @param {object} widget - Widget de Itens ou de Adicionais
     */
    this.salvarItens = function(widget){
        var deferred         = ZHPromise.defer();  
        var wdgProposta      = getWidgetProposta();
        var isnew            = widget.currentRow.__is_new;
        var nrseqorcamento   = widget.currentRow.NRSEQORCAMENTO;
        var quantidade       = widget.currentRow.QTITEMORCAMENTO;
        var quantidadeAntiga = quantidade;

        //widget.currentRow.NRMOEDA = wdgProposta.currentRow.NRMOEDA;
        
        if (widget.backupRow) {
            quantidadeAntiga = widget.backupRow.QTITEMORCAMENTO;
        }
        
        if (widget.currentRow.TIPOPRODSERV == 'S') 
            widget.currentRow.CDSERVORCA = widget.currentRow.CDPRODSERV;
        else 
            widget.currentRow.CDPRODUTO = widget.currentRow.CDPRODSERV;
        
        var idsituacao = wdgProposta.currentRow.IDSITUACAOORCAMENTO;
        widget.currentRow.__columns_to_return = ["NRITEMORCAMENTO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];
        
        if(widget.currentRow.NMDOCITEMORCAMENTO && widget.currentRow.NMDOCITEMORCAMENTO !== widget.currentRow.NMITEMORCAMENTO){
            widget.currentRow.NMITEMORCAMENTO = widget.currentRow.NMDOCITEMORCAMENTO;
        }
        
        if(widget.currentRow.IDTIPO === "I" && !widget.currentRow.VRUNITITEMORCA)
            widget.currentRow.VRUNITITEMORCA = widget.currentRow.VRTOTALITEMORCA;
            
        if(widget.currentRow.VRUNITITEMORCA) widget.currentRow.VRUNITITEMORCA = Utilities.tirarMascaraValores(widget.currentRow.VRUNITITEMORCA, 8);
        if(widget.currentRow.QTMEDIDAORCA)   widget.currentRow.QTMEDIDAORCA   = Utilities.tirarMascaraValores(widget.currentRow.QTMEDIDAORCA);
        if(widget.currentRow.VRPERCDESCORCA) widget.currentRow.VRPERCDESCORCA = Utilities.tirarMascaraValores(widget.currentRow.VRPERCDESCORCA);
        
        
        this.criarParametrosNewCode(widget);
        General.widgetSave(widget).then(function(success) {
            widget.currentRow.QTMEDIDAORCA_GRID = widget.currentRow.QTMEDIDAORCA;

            if (success.dataset && success.dataset.columnsToReturn[0]) {
                widget.currentRow.NRITEMORCAMENTO = success.dataset.columnsToReturn[0].NRITEMORCAMENTO;
                widget.currentRow.NRORG           = success.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO      = success.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO  = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }

            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            
            if (success.dataset.retorno) {
                wdgPropostaGrid.currentRow.VRORCAMENTO = success.dataset.retorno.valorProposta;
                wdgProposta.currentRow.VRORCAMENTO = success.dataset.retorno.valorProposta;
            }
            
            //if (wdgProposta.currentRow.IDUTILIZACOMP === 'S') {
            if (wdgProposta.currentRow.QTITENSTRABSERV > 0) {
                if (widget.currentRow.IDTIPO === "I") {
                    widget.currentRow.VRUNITITEMORCA = "";
                }
                else {
                    this.replicaAteracaoParaItemPai(widget).then(function() {
                        this.atualizarValorItensNegociacaoValores(widget.currentRow).then(function() {
                            this.atualizarValorProposta(wdgPropostaGrid);
                        }.bind(this))
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }.bind(this));
                    
                    General.callRestEngine("product#/fct_orcamento_widget/atualizarMedidaVariavelServico", "Row", widget.currentRow).then(null)
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            }
            else {
                this.atualizarValoresNegociacaoAoEditarItemSemComposicao(widget.currentRow).then(function() {
                    this.atualizarValorProposta(wdgPropostaGrid);
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
            
            // if(["X", "V"].includes(widget.currentRow.IDTPDESCORCA)) 
            //     this.calcularValorTotalItem(widget);

            widget.currentRow.changed = false;
            widget.widgets.forEach(function(wdg){
                wdg.enabled = true;
            });
            
            /*if(!isnew){
                widget.view();
            } else {
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
            }*/
            
            if(idsituacao === "CONCLUIDO" && isnew) {
                this.insereItensAcaoPreventiva(nrseqorcamento, widget.currentRow.NRITEMORCAMENTO, quantidade, idsituacao);
            } else {
                if (!isnew && quantidade !== quantidadeAntiga) {
                    var params = {
                        "NRSEQORCAMENTO" : nrseqorcamento,
                        "NRITEMORCAMENTO": widget.currentRow.NRITEMORCAMENTO,
                        "QTPENDENCIA"    : quantidade
                    };
                    
                    General.callRestEngine("product#/fct_itemorcamento_widget/atualizaQuantidadeItensAgendPendente", "Row", params)
                    .then(null)
                    .catch(function(error){
                        ScreenService.showMessage(error.data.error);
                    });
                }
            }
            
            deferred.resolve();
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            deferred.reject();
            if (error.data)
                ScreenService.showMessage(error.data.error);
            else
                ScreenService.showMessage(error);
        });
        
        return deferred.promise;
    };

    this.atualizarValoresNegociacaoAoEditarItemSemComposicao = function(row) {
        var params = {
            "NRITEMORCAMENTO": row.NRITEMORCAMENTO,
            "VRTOTALITEMORCA": row.VRTOTALITEMORCA
        };

        return General.callRestEngine("product#/fct_orcamento_widget/atualizarValoresNegociacaoAoEditarItemSemComposicao", "Row", params);
    };

    this.excluirItem = function(widget) {
        var widgetProposta  = templateManager.container.getWidget("wdgProposta");
        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
        var params = {
            "NRSEQORCAMENTO"  : widgetProposta.currentRow.NRSEQORCAMENTO,
            "NRITEMORCAMENTO" : widget.currentRow.NRITEMORCAMENTO
        };

        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
            General.callRestEngine("product#/fct_itemorcamento_widget/DeletaItemOrcamentoConcluido", "Row", widget.currentRow).then(function(){
                General.callRestEngine("product#/fct_relacestruturaprop_widget/deletaAlteracoesInutilizadas", "Row", params).then(function(){
                    ScreenService.closeSwipe().then(function () {
                        widget.activate();
                        widget.reload();
                        this.atualizarValorProposta(wdgPropostaGrid); 
                    }.bind(this));
                }.bind(this))
                .catch(function(error){
                    console.error(error);
                });
            }.bind(this))
            .catch(function(error){
                ScreenService.showMessage(error.data.error);
            });
            
        }.bind(this));
    };

    /**
     * Atualiza o valor total da proposta de acordo com a soma dos itens.
     * 
     * @param {object} wdgPropostaGrid - Widget de Proposta (grid)
     */
    this.atualizarValorProposta = function(wdgPropostaGrid) {
        var params = {
            "NRSEQORCAMENTO": wdgPropostaGrid.currentRow.NRSEQORCAMENTO,
            "NRORG": wdgPropostaGrid.currentRow.NRORG
        };

        General.callRestEngine("product#/fct_orcamento_widget/atualizarValorProposta", "Row", params).then(function(success) {
                var valorProposta = success.dataset.FCT_ORCAMENTO.VRORCAMENTO;
                var wdgProposta = templateManager.container.getWidget("wdgProposta");

                wdgPropostaGrid.currentRow.VRORCAMENTO = valorProposta;
                wdgProposta.currentRow.VRORCAMENTO = valorProposta;
            },
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
    };

    /**
     * Realiza o controle dos campos de "Medida" de acordo 
     * com o field "Utiliza Medida?"
     * 
     * @param {object} widget - Widget de Itens para pegar o field "IDMULTMEDVRTOT".
     */
    this.bloquearDadosMedida = function(widget) {
        var fieldUsaQtdMedida = widget.getField("IDMULTMEDVRTOT");
        
        if(!fieldUsaQtdMedida)
            fieldUsaQtdMedida = widget.getField("IDUSAMEDIDA");
            
        if (fieldUsaQtdMedida.readOnly == false) {
            /* Bloqueia a Modalidade de pagamento*/
            if (fieldUsaQtdMedida.value() == 'N') {
                /* Medida */
                widget.getField('DSTPMEDIDASERV').readOnly = true;
                widget.getField('DSTPMEDIDASERV').value(null);
                widget.getField('DSTPMEDIDASERV').validations = {};

                widget.getField('CDTPMEDIDASERV').value(null);
                widget.getField('CDTPMEDIDASERV').validations = {};

                /* Quantidade de Medida */
                widget.getField('QTMEDIDAORCA').readOnly = true;
                widget.getField('QTMEDIDAORCA').value(null);
                widget.getField('QTMEDIDAORCA').validations = {};

            } else {
            /* Desbloqueia a Medida*/

                /* Medida */
                widget.getField('DSTPMEDIDASERV').readOnly = false;
                widget.getField('DSTPMEDIDASERV').validations = { "required": true };

                /* Quantidade de Medida */
                widget.getField('QTMEDIDAORCA').readOnly = false;
                widget.getField('QTMEDIDAORCA').validations = { "required": true };
            }
        }
    };


    /**
     * Realiza o controle dos campos de "Dados do item" de acordo para bloquear 
     * os mesmos caso a proposta seja criada à prtir de uma composição.
     * 
     * @param {object} widget widget de 
     * Itens para pegar o field "IDUTILIZACOMP".
     */
    /*this.bloquearDadosItemComposicao = function(widget) {
        var widgetPropostaGrid = getWidgetPropostaGrid();
        var usaComposicao = widgetPropostaGrid.currentRow.IDUTILIZACOMP;

        // Bloqueia a Modalidade de pagamento
        if (usaComposicao == 'S') {
            // Item 
            widget.getField('NMITEMORCAMENTO').readOnly = true;

            // Quantidade  
            widget.getField('QTITEMORCAMENTO').readOnly = true;

            // Utiliza Quantidade de Medida? 
            widget.getField('IDMULTMEDVRTOT').readOnly = true;

            // Medida  
            widget.getField('DSTPMEDIDASERV').readOnly = true;

            // Quantidade de Medida 
            widget.getField('QTMEDIDAORCA').readOnly = true;

            // Valor Unitário  
            widget.getField('VRUNITITEMORCA').readOnly = true;

            // Tipo de Desconto  
            widget.getField('IDTPDESCORCA').readOnly = true;

            // Desconto  
            widget.getField('VRPERCDESCORCA').readOnly = true;
        }
        else {
            // Item 
            widget.getField('NMITEMORCAMENTO').readOnly = false;

            // Quantidade  
            widget.getField('QTITEMORCAMENTO').readOnly = false;

            // Utiliza Quantidade de Medida? 
            widget.getField('IDMULTMEDVRTOT').readOnly = false;

            // Medida  
            widget.getField('DSTPMEDIDASERV').readOnly = false;

            // Quantidade de Medida 
            widget.getField('QTMEDIDAORCA').readOnly = false;

            // Valor Unitário  
            widget.getField('VRUNITITEMORCA').readOnly = false;

            // Tipo de Desconto  
            widget.getField('IDTPDESCORCA').readOnly = false;

            // Desconto  
            widget.getField('VRPERCDESCORCA').readOnly = false;
        }
    };*/

    /**
     * Habilita/desabilita os campos de saída do Modelo de Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     * @param {boolean} readOnly
     */
    this.manipularCamposModeloProposta = function(widget, readOnly) {
        fieldsModelo.forEach(function(field) {
            widget.getField(field).readOnly = readOnly;
        });
    };

    /**
     * Verifica a obrigatoriedade do campo Oportunidade de Negócio de acordo com o modelo de proposta selecionado.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.verificarObrigatoriedadeOportunidade = function(widget) {
        var idobrigaoportunidade = widget.currentRow.IDOBRIGAOPORTUNIDADE;

        var validations = {};
        if (idobrigaoportunidade == "S")
            validations = {
                "required": true
            };

        var fieldOportunidade = widget.getField("NMOPORT");
        fieldOportunidade.validations = validations;
    };

    /**
     * Coloca a tela de Proposta em modo de edição.
     * 
     * @param {object} wdgProposta - Widget de Proposta
     */
    this.prepararEdicaoProposta = function(wdgProposta) {
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];

        wdgProposta.backupRow = Util.clone(wdgProposta.currentRow, ignoredKeys);
        wdgProposta.edit();
        wdgTabbedPrincipal.edit();

        wdgProposta.template = "widget/form.html";
        //this.bloquearModalidadePagamento(wdgProposta);
        this.filtroInterlocutor(wdgProposta);
        this.filtroOportunidade(wdgProposta);
        //var utilizaComposicao = (wdgProposta.currentRow.IDUTILIZACOMP == "S");
        var utilizaComposicao = (wdgProposta.currentRow.QTITENSTRABSERV > 0);
        this.controlarEscopoProposta(wdgProposta, utilizaComposicao, false);
        wdgProposta.getAction("avancar_composicao").isVisible = false;
        wdgProposta.getAction("save").isVisible = true;

        this.alterarAcessibilidadeAbasProposta(false);
        this.controlaCodProposta(wdgProposta);
        this.assinaturaEletronicaOnChange(wdgProposta);
        mudouMoeda = false;

        var readOnly = wdgProposta.currentRow.IDSITUACAOORCAMENTO !== "ELABORACAO";
        wdgProposta.getField("NMIDIOMA").readOnly = readOnly;
        wdgProposta.getField("NMMOEDA").readOnly = readOnly;
    };

    /**
     * Coloca a tela de Itens ou de Adicionais em modo de edição.
     * 
     * @param {object} widget - Widget de Itens ou de Adicionais
     */
    this.prepararEdicaoItens = function(widget) {
        General.widgetEdit(widget);
        if(templateManager.container.getWidget("wdgProposta").currentRow.QTITENSTRABSERV > 0) {
            this.validarAbreSwipeItens(widget, "W");
            var params = {
                "NRTIPOORCAMENTO": widget.currentRow.NRTIPOORCAMENTO,
                "CDSERVORCA"     : widget.currentRow.CDSERVORCA
            };

            General.callRestEngine("product#/fct_itemorcamento_widget/getMedidaVariavelEscopo", "Row", params).then(function(response) {
                var fieldVrunitario = widget.getField("VRUNITITEMORCA");
                if(response.dataset && response.dataset.retorno) {
                    fieldVrunitario.readOnly = true;
                    fieldVrunitario.help     = "O campo foi desabilitado porque não é possível inserir acréscimos em itens vinculados a serviços faturados por medição.";
                } else {
                    fieldVrunitario.readOnly = false;
                    fieldVrunitario.help     = null;
                }
            });
        } else {
            this.bloquearDadosMedida(widget);
            this.filtrarTipoDeItensManuaisDoEscopo(widget);
            
            var nrtipoorcamento = widget.currentRow.NRTIPOORCAMENTO;
            var tipoprodserv    = widget.currentRow.TIPOPRODSERV;
            var cdprodserv      = widget.currentRow.CDPRODSERV;

            if (nrtipoorcamento && tipoprodserv == "S" && cdprodserv) {
                var params = {
                    "NRTIPOORCAMENTO": nrtipoorcamento,
                    "CDSERVORCA"     : cdprodserv
                };

                General.callRestEngine("product#/fct_itemorcamento_widget/getMedidaVariavelEscopo", "Row", params).then(function(response) {
                    if (response.dataset && response.dataset.retorno) {
                        widget.getField("DSTPMEDIDASERV").readOnly = true;
                        widget.currentRow.INSERIR_MEDIDA = true;
                    } else {
                        widget.getField("DSTPMEDIDASERV").readOnly = false;
                        widget.currentRow.INSERIR_MEDIDA = false;
                    }
                });
            }
        }
    };

    /**
     * Salva um Vendedor.
     * 
     * @param {object} widget - Widget de Vendedor
     */
    this.salvarVendedor = function(widget) {
        if (widget.isValid()) {

            var nrseqorcamento = widget.currentRow.NRSEQORCAMENTO;
            var nrvendedororca = widget.currentRow.NRVENDEDORORCA;
            var nrparcvendedor = widget.currentRow.NRPARCVENDEDOR;

            Sm00001000_LancamentoProposta_Service.validarVendedor(nrseqorcamento, nrvendedororca, nrparcvendedor).then(function(response) {
                    var dados = response.dataset.fct_vendedororca_widget;
                    var idvenpriorca = widget.currentRow.IDVENPRIORCA;

                    if (dados.existeVendedor) {
                        ScreenService.showMessage("Operação bloqueada.<br/><br/>Vendedor já cadastrado para a proposta selecionada.");
                    }
                    else if (dados.existePrincipal && idvenpriorca == "S") {
                        ScreenService.showMessage("Operação bloqueada.<br/><br/>Deve haver apenas um vendedor principal para a proposta selecionada.");
                    }
                    else {
                        widget.currentRow.__columns_to_return = ["NRVENDEDORORCA", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];

                        General.widgetSave(widget).then(function(success) {
                            if (success.dataset && success.dataset.columnsToReturn[0]) {
                                widget.currentRow.NRVENDEDORORCA = success.dataset.columnsToReturn[0].NRVENDEDORORCA;
                                widget.currentRow.NRORG = success.dataset.columnsToReturn[0].NRORG;
                                widget.currentRow.DTINCLUSAO = success.dataset.columnsToReturn[0].DTINCLUSAO;
                                widget.currentRow.CDOPERINCLUSAO = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                            }
                        }).catch(function(error) {
                            console.error(error);
                            if (error.data)
                                ScreenService.showMessage(error.data.error);
                            else
                                ScreenService.showMessage(error);
                        });
                    }
                },
                function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
        }
    };


    /**
     * Calcula a data de validade da proposta a partir dos dias.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.calcularDataValidadeProposta = function(widget) {
        var qtDias = widget.currentRow.QTDIASVALIDADE;
        if (qtDias != null && qtDias >= 0) {
            var dataAtual = moment();
            widget.currentRow.DTVALIDADE = dataAtual.add(qtDias, "day").format("DD/MM/YYYY");
        }
        else {
            widget.currentRow.DTVALIDADE = "";
            widget.currentRow.QTDIASVALIDADE = "";
        }
    };

    /**
     * Calcula os dias de validade da proposta a partir da data.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.calcularDiasValidadeProposta = function(widget) {
        var dataValidade = widget.currentRow.DTVALIDADE;
        if (dataValidade) {
            dataValidade = moment(dataValidade + " 00:00:00", "DD/MM/YYYY HH:mm:ss");
            var dataAtual = moment().format("DD/MM/YYYY");
            dataAtual = moment(dataAtual + " 00:00:00", "DD/MM/YYYY HH:mm:ss");
            var qtDias = dataValidade.diff(dataAtual) / 86400000;
            if (qtDias >= 0) {
                widget.currentRow.QTDIASVALIDADE = qtDias;
            }
            else {
                ScreenService.showMessage("Data de Validade não pode ser inferior à data de hoje.");
                widget.currentRow.DTVALIDADE = "";
                widget.currentRow.QTDIASVALIDADE = "";
            }
        }
        else {
            widget.currentRow.QTDIASVALIDADE = "";
        }
    };

    /**
     * Seta o Modelo e os Escopos de Proposta para o filtro da Composição.
     * 
     * @param {object} widget - Widget de Elementos (inserindo)
     */
    this.setFiltroComposicao = function(widget) {
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var codModelo = wdgProposta.currentRow.NRORCAMENTOMODELO;
        var escopos = wdgProposta.currentRow.NRTIPOORCAMENTO;
        var revenda = wdgProposta.currentRow.NRREVENDA;

        widget.dataSourceFilter[0].value = codModelo;
        widget.dataSourceFilter[1].value = escopos;
        widget.dataSourceFilter[2].value = revenda;

        widget.reload();
    };

    /**
     * Avança para a aba de Elementos (inserindo).
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.avancarComposicao = function(widget) {
        if (widget.isValid()) {
            //widget.enabled = false;
            var wdgPropostaTabbed = templateManager.container.getWidget("wdgPropostaTabbed");
            wdgPropostaTabbed.enabled = false;

            var wdgItensComp = templateManager.container.getWidget("wdgItensComposicaoProposta");
            wdgItensComp.enabled = true;
            wdgItensComp.isVisible = true;
            wdgItensComp.activate().then(function() {
                this.setFiltroComposicao(wdgItensComp);
            }.bind(this));
        }
        else {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
    };

    /**
     * Volta para a aba de Proposta.
     * 
     * @param {object} widget - Widget de Elementos (inserindo)
     */
    this.voltarComposicao = function(widget) {
        widget.enabled = false;
        var wdgPropostaTabbed = templateManager.container.getWidget("wdgPropostaTabbed");
        wdgPropostaTabbed.enabled = true;
        wdgPropostaTabbed.activate();
    };

    /**
     * Volta para a aba de Elementos da Proposta (inserindo).
     * 
     * @param {object} widget - Widget de Tipo de Medida da Proposta (inserindo)
     */
    this.voltarTipoMedida = function(widget) {
        widget.enabled = false;
        var wdgItensComposicaoProposta = templateManager.container.getWidget("wdgItensComposicaoProposta");
        wdgItensComposicaoProposta.enabled = true;

        //var wdgProposta = templateManager.container.getWidget("wdgProposta");
        //wdgProposta.openTab("wdgItensComposicaoProposta");
        wdgItensComposicaoProposta.activate();
    };



    //=============== COMENTADO PORQUE FOI RETIRADO O CAMPO DE VALOR DA TELA ===============
    /**
     * Calcula o valor dos itens Pai.
     * 
     * @param {object} widget - Widget de Elementos (inserindo).
     */
    /*this.calcularValorItensPai = function(widget) {
        setTimeout(function() {
            widget.dataSource.data.forEach(function(row) {
                if (row._childCount > 0) {
                    var dataset = widget.dataSource.data;
                    var codItem = row.NRITCOMPSERVICO;
                    
                    var somaValor = this.getSomaValor(dataset, codItem, 0);
                    row.VRTOTAL = somaValor;
                }
            }.bind(this));
        }.bind(this), 100);
    };*/

    /**
     * Calcula a soma dos valores dos filhos do item de código passado.
     * 
     * @param {Object[]} dataset - Dados carregados na tela de Elementos (inserindo)
     * @param {number} codItem - Código do item da Composição
     * @param {floar} somaValor - Somatório dos Itens da Composição filhos do item do código informado
     * 
     * @return {float}
     */
    /*this.getSomaValor = function(dataset, codItem, somaValor) {
        for (var i=0; i < dataset.length; i++) {
            if (dataset[i]._parent && dataset[i]._parent.NRITCOMPSERVICO == codItem) {
                somaValor += dataset[i].VRTOTAL;
                somaValor = this.getSomaValor(dataset, dataset[i].NRITCOMPSERVICO, somaValor);
            }
        }
        return somaValor;
    };*/

    //====================================================================================



    /**
     * Seta o filtro dos Serviços dos Elementos.
     * 
     * @param {object} widget - Widget de Serviço dos Elementos
     */
    this.setFiltroServicoItensComposicaoProposta = function(widget) {
        var wdgItensComposicaoProposta = templateManager.container.getWidget("wdgItensComposicaoProposta");
        var wdgProposta = templateManager.container.getWidget("wdgProposta");

        widget.dataSourceFilter[0].value = wdgItensComposicaoProposta.currentRow.CDELEMENTO;
        widget.dataSourceFilter[1].value = wdgProposta.currentRow.NRTIPOORCAMENTO;
        widget.dataSourceFilter[2].value = wdgItensComposicaoProposta.currentRow.NRORG;
        widget.reload();
    };
    
    /**
     * Adiciona os itens da proposta na tabela de agendamentos pendentes 
     * com a pendencia de ação preventiva.
     * @params{number} numero do orcamento
    **/
    this.insereItensAcaoPreventiva = function(nrseqorcamento, nritemorcamento, quantidade, situacao){
        var params = {
            "NRSEQORCAMENTO"     : nrseqorcamento,
            "NRITEMORCAMENTO"    : nritemorcamento,
            "QUANTIDADE"         : quantidade,
            "IDSITUACAOORCAMENTO": situacao,
            "ROTINA"             : "PROPOSTA"
        };
        
        General.callRestEngine("product#/fct_agendrecursocont_widget/salvarAgendamentosPendentes", "Row", params)
        .then(null)
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Conclui a Proposta.
     * 
     * @param {object} wdgProposta - Widget de Proposta
     */
    this.concluirProposta = function(wdgProposta) {
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente concluir a proposta?"), 'question', function() {
            var nrseqorcamento = wdgProposta.currentRow.NRSEQORCAMENTO;
            var nrorg = wdgProposta.currentRow.NRORG;

            Sm00001000_LancamentoProposta_Service.validarElementosParaConclusao(wdgProposta.currentRow).then(function (validaElemento) {
                if(validaElemento.dataset.validacao) {
                    var msgErro = Sm00001000_LancamentoProposta_Service.validarPropostaParaConclusao(wdgProposta.currentRow);
                    if (msgErro) {
                        ScreenService.showMessage(msgErro);
                        ScreenService.closePopup();
                    } else {
                        Sm00001000_LancamentoProposta_Service.validarMoedaItens(nrseqorcamento).then(function() {
                            Sm00001000_LancamentoProposta_Service.getPorcentagemVendedores(nrseqorcamento, nrorg).then(function(success) {
                                var somaPorcentagem = success.dataset.fct_vendedororca_widget.porcentagem;
                                if (somaPorcentagem == 100 || somaPorcentagem == 0) {

                                    Sm00001000_LancamentoProposta_Service.validarNegociacaoValores(nrseqorcamento).then(function(response) {
                                        if (response) {
                                            //if (wdgProposta.currentRow.IDUTILIZACOMP == "S") {
                                            if (wdgProposta.currentRow.QTITENSTRABSERV > 0) {
                                                var row = {"NRSEQORCAMENTO": nrseqorcamento};
                                                General.callRestEngine("product#/fct_itemorcamento_widget/validarValoresItensProp", "Row", row).then(function(response){
                                                    if(response.dataset && response.dataset.validarValoresItensProp && response.dataset.validarValoresItensProp.VRORCAMENTO){
                                                        wdgProposta.currentRow.VRORCAMENTO = response.dataset.validarValoresItensProp.VRORCAMENTO;
                                                        ScreenService.showMessage("Operação bloqueada!<br><br>O valor dos itens da proposta foram atualizados devido a mudanças recentes na tabela de preço ou na quantidade de medida do serviço.<br><br>Favor checa-los e tentar concluir novamente.");
                                                    }
                                                    else {
                                                        this.salvarConclusaoProposta(wdgProposta, true);
                                                    }
                                                    this.validaIdCertsign();
                                                }.bind(this),
                                                function (error) {
                                                    if (error.data && error.data.error.indexOf('POSSUI COMPOSICAO INATIVA') !== -1) {
                                                        ScreenService.showMessage("Operação bloqueada!<br>Não é possível concluir propostas ligadas a composições de serviço inativas.<br>Ative-as e tente novamente!");
                                                    } else {
                                                        Utilities.showErrorMessage(error);
                                                    }
                                                });
                                            }
                                            else {
                                                this.salvarConclusaoProposta(wdgProposta, false);
                                            }
                                        }
                                        else {
                                            ScreenService.showMessage('Operação bloqueada.<br/><br/>Os itens da aba "Negociação de Valores" que são de parcelamento não estão com as parcelas criadas corretamente: o valor total da soma das parcelas deve ser igual ao valor do item.');
                                        }
                                    }.bind(this))
                                    .catch(function(error) {
                                        Utilities.showErrorMessage(error);
                                    });
                                }
                                else {
                                    ScreenService.closePopup();
                                    ScreenService.showMessage("Operação bloqueada.<br/><br/>Soma da porcentagem de participação na venda dos vendedores da proposta deve ser igual a 100%.");
                                }
                                    
                            }.bind(this))
                            .catch(function (error) {
                                Utilities.showErrorMessage(error);
                            });
                        }.bind(this))
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }
                } else {
                    ScreenService.closePopup();
                }
            }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
        }.bind(this));
    };
    
    /**
     * @param {object} wdgProposta
     * @param {boolean} utilizaComposicao
     */
    this.salvarConclusaoProposta = function(wdgProposta, utilizaComposicao) {
        var params = { "NRSEQORCAMENTO": wdgProposta.currentRow.NRSEQORCAMENTO };
        General.callRestEngine("product#/fct_orcamento_widget/validarParametrosFaturamentoOrca", "Row", params).then(function(response) {
            var nrseqorcamento    = wdgProposta.currentRow.NRSEQORCAMENTO;
            var nrorcamentomodelo = wdgProposta.currentRow.NRORCAMENTOMODELO;
            var nrorg             = wdgProposta.currentRow.NRORG;
    
            var params = {
                "NRORCAMENTOMODELO" : nrorcamentomodelo,
                "NRORG"             : nrorg
            };
        
            General.callRestEngine("product#/fct_orcamento_widget/validaOrcamentoValidos", "Row", params).then(function(validacao) {
                if (validacao.dataset.modeloInativo.length > 0) {
                    ScreenService.showMessage("Operação bloqueada!<br><br>O modelo de proposta selecionado está inativo");
                } else {
                    
                    wdgProposta.currentRow.changed = true;
                    wdgProposta.currentRow.IDSITUACAOORCAMENTO = "CONCLUIDO";
    
                    wdgProposta.saveRow().then(function () {
                        wdgProposta.currentRow.changed = false;
                        ScreenService.closePopup();
                        
                        // Gera os documentos no ambiente do Certsign
                        
                       var params = {
                            "NRORCAMENTOMODELO" : nrorcamentomodelo,
                        };

                        General.callRestEngine('product#/fct_orcamento_widget/validaGeraDocCertisign', "Row", params).then(function(response) {
                            
                            var idenviadocconclusao = response.dataset.validaGeraDocCertisign.resultado;
                            
                            if (idenviadocconclusao == 'S') {
                            
                                var params = {
                                    "NRSEQORCAMENTO" : nrseqorcamento
                                };
        
                                General.callRestEngine('product#/fct_orcamento_widget/validaIdCertsign', "Row", params).then(function(response) {
                    
                                    var idTokenCertsign = response.dataset.validaIdCertsign.DSTOKENCERTSIGN;
                                    var idAssEletrDigital = response.dataset.validaIdCertsign.IDASSELETRDIGITAL;
                                    
                                    if ((idTokenCertsign != null ) && (idAssEletrDigital == 'S')) {
                                        
                                        var params = {
                                            "NRSEQORCAMENTO"     : nrseqorcamento,
                                            "NRTIPOORCAMENTO"    : wdgProposta.currentRow.NRTIPOORCAMENTO,
                                            "IDSITUACAOORCAMENTO": wdgProposta.currentRow.IDSITUACAOORCAMENTO,
                                            "NRORCAMENTO"        : wdgProposta.currentRow.NRORCAMENTO,
                                            "CDIDIOM"           : wdgProposta.currentRow.CDIDIOM
                                        };
                                    
                                        var timeout = 5 * 60000; //5 min
                                        Utilities.callRestEngineWithTimeout("product#/fct_orcamento_widget/gerarDocumentoCertsign", "Row", params, timeout)
                                        .then(function() {
                                            ScreenService.showMessage("Processo realizado com sucesso!");
                                        })
                                        .catch(function(error) {
                                            var message = error;
                                            if (typeof error === 'object') {
                                                if (error.data && error.data.error) {
                                                    message = error.data.error;
                                                }
                                                else if (error.message) {
                                                    message = error.message;
                                                }
                                            }
                                            
                                            ScreenService.showMessage("Ocorreram erros durante o upload dos documentos para o Portal Certisign. <br/>Erro: " + message);
                                        });
                                    }
                                    else {
                                        ScreenService.showMessage("Processo realizado com sucesso!");
                                    }
                                });
                            }
                        });
                        //salva as informações da variação de preço da proposta
                        if (utilizaComposicao) {
                            params = {
                                "NRSEQORCAMENTO": nrseqorcamento,
                                "NRTIPOORCAMENTO": wdgProposta.currentRow.NRTIPOORCAMENTO,
                                "NRMOEDA": wdgProposta.currentRow.NRMOEDA
                            };
                            General.callRestEngine("product#/fct_orcamento_widget/salvarInfoServicoEPrecoOrca", "Row", params).then(null).catch(function(error) {
                                Utilities.showErrorMessage(error);
                            });
                        }
    
                        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                        wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO = "CONCLUIDO";
                        
                        this.alterarVisibilidadeActionEditProposta(wdgPropostaGrid, nrorg);
                        this.validacoesGeraisProposta(wdgPropostaGrid);
                        this.validaIdCertsign();
                    }.bind(this))
                    .catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            }.bind(this),
            function (error) {
                ScreenService.closePopup();
                Utilities.showErrorMessage(error);
            });
        }.bind(this),
        function (error) {
            Utilities.showErrorMessage(error);
        });
    };

    /**
     * Abre o popup para  ajustificativa de uma renegociação ou um cancelamento
     * de uma porposta já aceita
     * @params{Object} widget widget proposta
    **/
    this.abrePopupDsJustCancReneg = function(widget, action){
        ScreenService.closePopup();
        var wdgPopup = templateManager.container.getWidget("PopupDsJustCancReneg");
        if(action === "cancelar_proposta"){
            wdgPopup.label = "Justificativa de Cancelamento da Proposta";
            wdgPopup.getField("DSJUSTIFCANCRENEG").label = "Justificativa de Cancelamento"; 
        } else { 
            wdgPopup.label = "Justificativa de Renegociação da Proposta";
            wdgPopup.getField("DSJUSTIFCANCRENEG").label = "Justificativa de Renegociação"; 
        }
            
        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function(){
            wdgPopup.activate();
            wdgPopup.newRow();
            wdgPopup.currentRow.NRSEQORCAMENTO   = widget.currentRow.NRSEQORCAMENTO;
            wdgPopup.currentRow.NRORCAMENTO      = widget.currentRow.NRORCAMENTO;
            wdgPopup.currentRow.DSREFERENCIAORCA = widget.currentRow.DSREFERENCIAORCA;
            wdgPopup.currentRow.ACTION           = action;
        }.bind(this));
    };
    
    /**
     * Salva a justificativa de cancelamento ou renegociação da proposta
     * @params{Object} widget widget popup de justificativa de cancelamento ou renegociação
    **/
    this.salvarJustificativaCancReneg = function(widget){
        if(widget.isValid()){
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            wdgProposta.currentRow.DSJUSTIFCANCRENEG = widget.currentRow.DSJUSTIFCANCRENEG;
            wdgProposta.currentRow.DTACEITEORCA      = null;
            wdgProposta.saveRow().then(function(){
                ScreenService.closePopup();
                widget.isVisible = false;
                wdgProposta.activate();
                if(widget.currentRow.ACTION === "cancelar_proposta")
                    this.cancelarProposta(wdgProposta);
                else 
                    this.copiarVersionarProposta(wdgProposta, 'V');
            }.bind(this))
            .catch(function(error){ 
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    /**
     * Cancela o popup e ativa a widget que contem o nome do paramêtro
     * @params{String} widgetName nome da widget que será ativada
    **/
    this.cancelaPopup = function(widgetName){
        ScreenService.closePopup();
        templateManager.container.getWidget(widgetName).activate();
    };
    
    /**
     * Verifica se a proposta escolhida está aceita
     * Caso esteja abre um popup pedindo a justificativa 
     * @params{Object} widget widget proposta
     * @params{String} action nome da action que acionou a função
    **/
    this.validaSituacaoPropCancReneg = function(widget, action){
        if (action === "versionar_proposta" && (widget.currentRow.ISSUEID || widget.currentRow.NRSOLISERV)) {
            ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br><br>Não é possível renegociar propostas ligadas a uma issue ou solicitação."));
        }
        else {
            var msgConfirmacao = null;
            if((widget.currentRow.IDSITUACAOORCAMENTO == "CANCELADO" && action === "versionar_proposta") || action === "cancelar_proposta")
                msgConfirmacao = "Deseja realmente "+ (action === "cancelar_proposta" ? "cancelar" : "renegociar") +" a proposta?";
            else 
                msgConfirmacao = "Ao renegociar esta proposta, sua versão atual ficará como cancelada. Deseja continuar?";
            
            if(widget.currentRow.IDSITUACAOORCAMENTO === "ACEITO"){
                var params = { "NRSEQORCAMENTO": widget.currentRow.NRSEQORCAMENTO };
                General.callRestEngine("product#/fct_orcamento_widget/getContratosDifPendentes", "Row", params).then(function(response){
                    if(response.dataset && response.dataset.fct_orcamento_widget && response.dataset.fct_orcamento_widget.CONTRATOS){
                        ScreenService.showMessage("Operação bloqueada.<br><br> Não é permitido "+ (action === "cancelar_proposta" ? "cancelar" : "renegociar") +" propostas que possuem contratos gerados com situação diferente de \"Pendente\" e \"Reprovado\".");
                    } else {
                        ScreenService.confirmMessage(ScreenService.i18n(msgConfirmacao), 'question', function(){
                            if(action == 'cancelar_proposta') 
                                this.abrePopupDsJustCancReneg(widget, action);
                            else 
                                this.validarRenegociacao(widget);
                        }.bind(this));
                    }
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            } else {
                ScreenService.confirmMessage(ScreenService.i18n(msgConfirmacao), 'question', function(){
                    if(action === "cancelar_proposta") 
                        this.cancelarProposta(widget);
                    else 
                        this.validarRenegociacao(widget);
                }.bind(this));
            }
        }
    };
    
    /**
     * Ação que cancela a Proposta.
     * 
     * @param {object} wdgProposta - Widget de Prposta
     */
    this.cancelarProposta = function(wdgProposta) {
        var params = {
            "NRSEQORCAMENTO": wdgProposta.currentRow.NRSEQORCAMENTO,
            "NRORG": wdgProposta.currentRow.NRORG
        };

        General.callRestEngine("product#/fct_orcamento_widget/cancelarProposta", "Row", params).then(function(success) {
            ScreenService.closePopup();
            wdgProposta.currentRow.IDSITUACAOORCAMENTO = "CANCELADO";
            
            var params = {
                "NRSEQORCAMENTO"  : wdgProposta.currentRow.NRSEQORCAMENTO,
                "NRTIPOORCAMENTO" : wdgProposta.currentRow.NRTIPOORCAMENTO
            };
            
            General.callRestEngine("product#/fct_orcamento_widget/excluirDocumentoCertsign", "Row", params).then(function() {
                ScreenService.showMessage("Processo realizado com sucesso!");
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });

            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO = "CANCELADO";
            this.validacoesGeraisProposta(wdgPropostaGrid);
        }.bind(this),
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Exibe mensagem de confirmação para copiar a Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.confirmarRenegociarCopiarProposta = function(widget, tipo) {
        if (widget.currentRow.ISSUEID || widget.currentRow.NRSOLISERV) {
            ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br><br>Não é possível copiar propostas ligadas a uma issue ou solicitação."));
        } else {
            ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente copiar a proposta?"), 'question', function(){   
                var params = { "NRSEQORCAMENTO" : widget.currentRow.NRSEQORCAMENTO };
                General.callRestEngine("product#/fct_itemorcamento_widget/validarAtivacaoComposicao", "Row", params).then(function(response){
                    this.copiarVersionarProposta(widget, tipo);
                }.bind(this))
                .catch(function(error){
                    if (error.data && error.data.error.indexOf('POSSUI COMPOSICAO INATIVA') !== -1) 
                        ScreenService.showMessage("Operação bloqueada!<br>Não é possível copiar propostas ligadas a composições de serviço inativas.<br>Ative-as e tente novamente!");
                    else 
                        Utilities.showErrorMessage(error);
                });
            }.bind(this));
        }
    };
    
    /**
     * Copia ou versiona a Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     * @param {string} tipo - 'C' para Copiar, 'V' para Versionar
     */
    this.copiarVersionarProposta = function(widget, tipo) {
        var nrseqorcamento = widget.currentRow.NRSEQORCAMENTO;
        var row = widget.currentRow;
        var params = {
            "IDSITUACAOORCAMENTO": widget.currentRow.IDSITUACAOORCAMENTO,
            "NRSEQORCAMENTO"     : widget.currentRow.NRSEQORCAMENTO,
            "NRORG"              : widget.currentRow.NRORG,
            "TIPO"               : tipo
        };
        
        General.callRestEngine("product#/fct_orcamento_widget/copiarVersionarProposta", "Row", params).then(function(response) {
            if(!widget.currentRow.NRCONTATO){
                row.NRSEQORCAMENTO = response.dataset.NRSEORCGERADO[0];
                this.geraContatoProposta(row);
                
                if (tipo == "V") {
                    var params = {
                        "NRSEQORCAMENTO"  : nrseqorcamento,
                        "NRTIPOORCAMENTO" : widget.currentRow.NRTIPOORCAMENTO
                    };
                    
                    General.callRestEngine("product#/fct_orcamento_widget/excluirDocumentoCertsign", "Row", params);
                }
            }
            
            ScreenService.showMessage("Processo realizado com sucesso!");
            ScreenService.closePopup();
            this.voltarParaPropostaGrid();
        }.bind(this),
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Aceita a Proposta.
     * 
     * @param {object} widget - popup de aceitar proposta
     */
    this.aceitarProposta = function(widget) {
        if(!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgProposta = getWidgetProposta();
            var nrseqorcamento = wdgProposta.currentRow.NRSEQORCAMENTO;
            
            var params = {
                "NRSEQORCAMENTO" : nrseqorcamento
            };
            
            General.callRestEngine("product#/fct_orcamento_widget/validaAceiteCertsign", "Row", params).then(function(resultado) {
                
                if (resultado.dataset.validaAceiteCertsign.resultado != 'OK') {
                    ScreenService.showMessage("Operação bloqueada!<br><br>Existem assinaturas eletrônicas/digitais pendentes ou rejeitadas.");
                }
                else {
                    ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente aceitar a proposta?"), 'question', function(){
                        var params = {
                            "NRSEQORCAMENTO": nrseqorcamento,
                            "NRORCAMENTO"   : wdgProposta.currentRow.NRORCAMENTO,
                            "NRORG"         : wdgProposta.currentRow.NRORG,
                            //"IDUTILIZACOMP" : wdgProposta.currentRow.IDUTILIZACOMP,
                            "IDUTILIZACOMP" : (wdgProposta.currentRow.QTITENSTRABSERV > 0) ? "S" : "N",
                            "NRPARCNEGOCIO" : wdgProposta.currentRow.NRPARCNEGOCIO,
                            "DTACEITEORCA"  : widget.currentRow.DTACEITE
                        };
                        
                        Sm00001000_LancamentoProposta_Service.validarNegociacaoValores(nrseqorcamento).then(function(response) {
                            if (response) {
                                General.callRestEngine("product#/fct_orcamento_widget/aceitarProposta", "Row", params).then(function() {
                                    
                                    this.cancelaPopup("wdgProposta");
                                    ScreenService.showMessage("Processo realizado com sucesso!");
                                    ScreenService.closePopup();
                                    wdgProposta.reload().then(function() {
                                        var wdgPropostaGrid = getWidgetPropostaGrid();
                                        
                                        wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO = "ACEITO";
                                        wdgPropostaGrid.currentRow.DTACEITEORCA        = wdgProposta.currentRow.DTACEITEORCA;
                                        
                                        this.validacoesGeraisProposta(wdgPropostaGrid);
                                    }.bind(this));
                                
                                }.bind(this))
                                .catch(function(error) {
                                    Utilities.showErrorMessage(error);
                                });
                            }
                            else {
                                ScreenService.showMessage('Operação bloqueada.<br/><br/>Os itens da aba "Negociação de Valores" que são de parcelamento não estão com as parcelas criadas corretamente: o valor total da soma das parcelas deve ser igual ao valor do item.');
                            }
                        }.bind(this))
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                        
                    }.bind(this));
                }
            }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
        }
    };

    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o País.
     * 
     * @param {object} widget - Widget de Endereço
     */
    this.controlarCamposEnderecoPorPais = function(widget) {
        widget.currentRow.SGESTADO = null;
        widget.currentRow.NMESTADO = null;
        widget.currentRow.CDMUNICIPIO = null;
        widget.currentRow.NMMUNICIPIO = null;
        widget.currentRow.CDBAIRRO = null;
        widget.currentRow.NMBAIRROENDERECO = null;

        var cdpais = widget.currentRow.CDPAIS;

        var campoEstado = widget.getField("NMESTADO");
        var campoMunicipio = widget.getField("NMMUNICIPIO");
        var campoBairro = widget.getField("NMBAIRROENDERECO");

        campoEstado.dataSourceFilter[0].value = cdpais;
        campoEstado.reload();

        campoMunicipio.dataSourceFilter[0].value = cdpais;

        campoBairro.dataSourceFilter[0].value = cdpais;
    };

    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o Estado.
     * 
     * @param {object} widget - Widget de Endereço
     */
    this.controlarCamposEnderecoPorEstado = function(widget) {
        widget.currentRow.CDMUNICIPIO = null;
        widget.currentRow.NMMUNICIPIO = null;
        widget.currentRow.CDBAIRRO = null;
        widget.currentRow.NMBAIRROENDERECO = null;

        var sgestado = widget.currentRow.SGESTADO;

        var campoMunicipio = widget.getField("NMMUNICIPIO");
        var campoBairro = widget.getField("NMBAIRROENDERECO");

        campoMunicipio.dataSourceFilter[1].value = sgestado;
        campoMunicipio.reload();

        campoBairro.dataSourceFilter[1].value = sgestado;
    };

    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o Município.
     * 
     * @param {object} widget - Widget de Endereço
     */
    this.controlarCamposEnderecoPorMunicipio = function(widget) {
        widget.currentRow.CDBAIRRO = null;
        widget.currentRow.NMBAIRROENDERECO = null;

        var cdmunicipio = widget.currentRow.CDMUNICIPIO;

        var campoBairro = widget.getField("NMBAIRROENDERECO");

        campoBairro.dataSourceFilter[2].value = cdmunicipio;
        campoBairro.reload();
    };

    /**
     * Salva o Endereço.
     * 
     * @param {object} widget - Widget de Endereço
     */
    this.salvarEndereco = function(widget) {
        widget.currentRow.__columns_to_return = ["NRENDERECOORCA", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];

        var nrseqorcamento = widget.currentRow.NRSEQORCAMENTO;
        var nrenderecoorca = widget.currentRow.NRENDERECOORCA;
        var cdtipoendereco = widget.currentRow.CDTIPOENDERECO;

        Sm00001000_LancamentoProposta_Service.validarTipoEndereco(nrseqorcamento, nrenderecoorca, cdtipoendereco).then(function(success) {

                if (success.dataset.fct_enderecoorca_widget.existe == false) {

                    General.widgetSave(widget).then(function(success) {
                        if (success.dataset && success.dataset.columnsToReturn[0]) {
                            widget.currentRow.NRENDERECOORCA = success.dataset.columnsToReturn[0].NRENDERECOORCA;
                            widget.currentRow.NRORG = success.dataset.columnsToReturn[0].NRORG;
                            widget.currentRow.DTINCLUSAO = success.dataset.columnsToReturn[0].DTINCLUSAO;
                            widget.currentRow.CDOPERINCLUSAO = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                        }
                    }).catch(function(error) {
                        console.error(error);
                        if (error.data)
                            ScreenService.showMessage(error.data.error);
                        else
                            ScreenService.showMessage(error);
                    });
                }
                else {
                    ScreenService.showMessage("Operação bloqueada.<br/><br/>Já existe um endereço cadastrado para o tipo informado.");
                }
            },
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
    };

    /**
     * Abre a widget de Enviar Email.
     */
    this.abrirWdgEnviarEmail = function(widget) {
        ScreenService.closePopup();

        var nrseqorcamento  = widget.currentRow.NRSEQORCAMENTO;
        var nrorg           = widget.currentRow.NRORG;
        var nrtipoorcamento = widget.currentRow.NRTIPOORCAMENTO;

        var wdgEnviarEmail = templateManager.container.getWidget("wdgEnviarEmail");
        wdgEnviarEmail.isVisible = true;
        this.setDadosPadraoWdgEnviarEmail(wdgEnviarEmail, nrseqorcamento, nrorg, nrtipoorcamento);
    };

    /**
     * Fecha a tela de Enviar E-mail.
     * 
     * @param {object} widget
     */
    this.fecharWdgEnviarEmail = function(widget) {
        this.desobrigaCamposEvolucaoEmail();
        ScreenService.closeSwipe().then(function() {
            widget.isVisible = false;
            getWidgetProposta().activate();
        });
    };

    /**
     * Seta o Remetente e os Destinatários padrão ao abrir a tela de Enviar E-mail.
     * 
     * @param {object} widget - Widget de Enviar E-mail
     */
    this.setDadosPadraoWdgEnviarEmail = function(widget, nrseqorcamento, nrorg, nrtipoorcamento) {
        var params = {
            "NRSEQORCAMENTO"   : nrseqorcamento,
            "NRORG"            : nrorg,
            "NRTIPOORCAMENTO"  : nrtipoorcamento
        };

        General.callRestEngine("product#/fct_orcamento_widget/getDadosPadraoEnviarEmail", "Row", params).then(function(success) {

                ScreenService.openSwipe(widget).then(function() {
                    widget.edit();

                    widget.currentRow = {
                        "NRSEQORCAMENTO" : nrseqorcamento,
                        "NRORG"          : nrorg,
                        "NRTIPOORCAMENTO": nrtipoorcamento,
                        "REMETENTE"      : success.dataset.DADOS.REMETENTE,
                        "DESTINATARIOS"  : success.dataset.DADOS.DESTINATARIOS,
                        "ASSUNTO"        : success.dataset.DADOS.DSASSUNTOEMAIL,
                        "CORPO"          : success.dataset.DADOS.DSCORPOEMAIL
                    };
                });
            },
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
    };

    /**
     * Envia o Documento da Proposta por e-mail.
     * 
     * @param {object} widget
     */
    this.enviarDocumentoPorEmail = function(widget) {
        var widgetProposta = templateManager.container.getWidget("wdgProposta");
        var wdgEnviarEmail = templateManager.container.getWidget("wdgEnviarEmail");
        
        var nmtipocont     = wdgEnviarEmail.currentRow.NMTIPOCONT;
        var nmmotievocon   = wdgEnviarEmail.currentRow.NMMOTIEVOCON;
        var nmseqresulevol = wdgEnviarEmail.currentRow.NMSEQRESULEVOL;
        
        if (widget.isValid()) {
            var destinatarios = widget.currentRow.DESTINATARIOS;
            if (destinatarios.indexOf(";") == -1) {
                destinatarios += ";";
            }
            
            var nrtipoorcamento = widget.currentRow.NRTIPOORCAMENTO;
            if(nrtipoorcamento instanceof Array){
                nrtipoorcamento = 0;
            }
            
            var interlocutor =  widgetProposta.currentRow.NRSEQINTERLOCUTOR;
            var params = {
                "NRSEQORCAMENTO"     : widget.currentRow.NRSEQORCAMENTO,
                "NRORG"              : widget.currentRow.NRORG,
                "NRTIPOORCAMENTO"    : nrtipoorcamento,
                "REMETENTE"          : widget.currentRow.REMETENTE,
                "DESTINATARIOS"      : widget.currentRow.DESTINATARIOS,
                "ASSUNTO"            : widget.currentRow.ASSUNTO,
                "CORPO"              : widget.currentRow.CORPO,
                "IDSITUACAOORCAMENTO": widgetProposta.currentRow.IDSITUACAOORCAMENTO
            };
            
            var idSituacao = widgetProposta.currentRow.IDSITUACAOORCAMENTO;
            var currentRow = widget.currentRow;
            currentRow.DTHREVOLCONT = (currentRow.DTHREVOLCONT)? currentRow.DTHREVOLCONT : moment(new Date()).format('DD/MM/YYYY HH:mm');
            widget.currentRow.NMPARCINTERLOCUTOR = widgetProposta.currentRow.NMPARCINTERLOCUTOR;
            widget.currentRow.TXEVOLCONT = widget.currentRow.CORPO;
            widget.currentRow.NMOPERADOR = nomeOperador;
            widget.currentRow.NRPARCNEGOCIO = widgetProposta.currentRow.NRPARCNEGOCIO;
           
            var timeout = 5 * 60000; //5 min
            Utilities.callRestEngineWithTimeout("product#/fct_orcamento_widget/enviarDocumentoPorEmail", "Row", params, timeout).then(function() {
                ScreenService.showMessage("Processo realizado com sucesso!").then(function(){
                if(nmtipocont && nmmotievocon && nmseqresulevol){
                    this.actionEvolutionNewRowPropostaEmail(interlocutor,idSituacao);
                }
                
                ScreenService.closePopup();
                ScreenService.closeSwipe();
                widget.isVisible = false;
                getWidgetProposta().activate();
                
            }.bind(this),
            function(error) {
                console.error(error);
                var msg = "Timeout";
                if (error.status !== 0 && error.data)
                    msg = error.data.error;
                ScreenService.showMessage(msg);
            });
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
      /**
     * Cria um nova row de propostaPorEmail
     * 
     * @param {interge} widget - métodi enviarDocumentoPorEmail
     */
    this.actionEvolutionNewRowPropostaEmail = function(interlocutor,idSituacao){
        this.closeActionsPopup().then(function(){
            var wdgEnviarEmail = templateManager.container.getWidget("wdgEnviarEmail");
            var corpo          = wdgEnviarEmail.currentRow.CORPO;
            var nmtipocont     = wdgEnviarEmail.currentRow.NMTIPOCONT;
            var nmmotievocon   = wdgEnviarEmail.currentRow.NMMOTIEVOCON;
            var nmseqresulevol = wdgEnviarEmail.currentRow.NMSEQRESULEVOL;
            var cdtipocont     = wdgEnviarEmail.currentRow.CDTIPOCONT;
            var cdmotievocon   = wdgEnviarEmail.currentRow.CDMOTIEVOCON;
            var nrseqresulevol = wdgEnviarEmail.currentRow.NRSEQRESULEVOL;
            this.evolucaoNewRowEmail(corpo, interlocutor, nmtipocont, nmmotievocon, nmseqresulevol, cdtipocont, cdmotievocon, nrseqresulevol, idSituacao);
        }.bind(this));
    };
    
     /**
     * Forma uma widget para evoluir apos o envio do email
     * 
     * @param {interge} widget - métodi actionEvolutionNewRowPropostaEmail
     */
    this.evolucaoNewRowEmail = function(corpo, interlocutor, nmtipocont, nmmotievocon, nmseqresulevol, cdtipocont, cdmotievocon, nrseqresulevol, idSituacao) {
            var widget = templateManager.container.getWidget("evolucao");
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            
            General.widgetNewRow(widget);
            
            var currentRow = widget.currentRow;
            currentRow.DTHREVOLCONT      = (currentRow.DTHREVOLCONT)? currentRow.DTHREVOLCONT : moment(new Date()).format('DD/MM/YYYY HH:mm');
            currentRow.NMOPERADOR        = nomeOperador;
            currentRow.CDOPEREVOL        = opLogado;
            currentRow.NRORG             = nrOrgOper;
            currentRow.NRSEQINTERLOCUTOR = interlocutor;
            currentRow.NMTIPOCONT        = nmtipocont;
            currentRow.NMMOTIEVOCON      = nmmotievocon;
            currentRow.NMSEQRESULEVOL    = nmseqresulevol;
            currentRow.CDTIPOCONT        = cdtipocont;
            currentRow.CDMOTIEVOCON      = cdmotievocon;
            currentRow.TXEVOLCONT        = corpo;
            currentRow.NRSEQRESULEVOL    = nrseqresulevol;
           //    ABEvolucaoController.setEvolDateChangePermission(widget);
            
            currentRow.DTINCLUSAO = self.momentNow();
            currentRow.NRSEQCONTATO = wdgProposta.currentRow.NRSEQCONTATO;
            currentRow.NRORG = nrOrgOper;
            
            widget.currentRow =  currentRow;
            this.saveEvolucaoAposEmail(widget, idSituacao);
    };

    /**
     * Buscas os dados do anexo para para salvar no anexo da evolução
     */
    this.anexarDocumnetosdoEmail = function(idSituacao, nrseqcontato, nrseqevolcont) {
        var wdgEnviarEmail = templateManager.container.getWidget("wdgEnviarEmail");
        var nrtipoorcamento = wdgEnviarEmail.currentRow.NRTIPOORCAMENTO;
            
         var params = {
            "NRSEQORCAMENTO": wdgEnviarEmail.currentRow.NRSEQORCAMENTO,
            "NRTIPOORCAMENTO": nrtipoorcamento,
            "IDSITUACAOORCAMENTO": idSituacao,
            "NRORG": wdgEnviarEmail.currentRow.NRORG
        };
        
        General.callRestEngine("product#/fct_orcamento_widget/getDadosAnexo", "Row", params).then(function(response) {
                var params = [];
                response.dataset.dadoAnexos.forEach(function(row){
                
                    var retorno = this.converteNomePDF(row.DSCAMINHODOCGERADOPDF, row.NMARQUIVO);
                
                    params.push({
                        "NRSEQCONTATO": nrseqcontato,
                        "DSCAMINHOARQ": row.DSCAMINHODOCGERADOPDF,
                        "NMARQUIVO": retorno[0],
                        "DSTIPO": retorno[1],
                        "OPERADOR": opLogado,
                        "VRTAMANHO": row.VRTAMANHO,
                        "NRSEQEVOLCONT": nrseqevolcont
                    });
                }.bind(this));
                
                General.callRestEngine("product#/crm_anexocontato/saveAnexo", "DataSet", params).then(function() {
                    
                }.bind(this));
            }.bind(this),
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });

    };
    
     this.converteNomePDF = function(pdf,nomeArq){
        var position;
        var extensao = pdf;
        position = extensao.indexOf(".");
        extensao = extensao.slice(position,extensao.length);
            
        var nome = nomeArq;
        position = nome.indexOf(".");
        nome = nome.substr(0, position);
        
        nome = nome+extensao;
        
        var retorno = [nome,extensao];
        return retorno;
     };
     
     this.setValuesForFileUploadEvolucao = function(widget, dadoAnexo, nrseqcontato, nrseqevolcont){
         var dscamiho = {
            "path": dadoAnexo.DSCAMINHODOCGERADOPDF
        };
        
        var currentRow = angular.copy(widget.currentRow);
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        currentRow.NRSEQCONTATO = nrseqcontato;
        currentRow.NMARQUIVO = dadoAnexo.NMARQUIVO;
        currentRow.DSTIPO = dadoAnexo.DSTIPO;
        currentRow.VRTAMANHO = dadoAnexo.VRTAMANHO;
        currentRow.DSCAMINHO = dscamiho;
        currentRow.IDATIVO = "S";
        currentRow.NRORG = nrOrgOper;
        currentRow.CDOPERADOR = nrparcnegocioOperLogado;
        currentRow.IDORIGEM = "EVOLUCAO";
        currentRow.NRORIGEM = nrseqevolcont;
        
        return currentRow;
        
    };
   
    /**
     * Salva o Documento.
     * 
     * @param {object} widget - widget de Documento
     */
    this.salvarDocumento = function(widget) {
        widget.currentRow.__columns_to_return = ["NRSEQORCADOC", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];

        var isNew = widget.currentRow.__is_new;
        General.widgetSave(widget).then(function(success) {
            if (success.dataset && success.dataset.columnsToReturn[0]) {
                widget.currentRow.NRSEQORCADOC = success.dataset.columnsToReturn[0].NRSEQORCADOC;
                widget.currentRow.NRORG = success.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = success.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
            if (isNew) {
                this.atualizarDocumento(widget, false);
            }
        }.bind(this));
    };


    /**
     * Atualiza o Documento, chamando a procedure FCT_SUB_MNE_DOC.
     * 
     * @param {object} widget - widget de Documento
     */
    this.atualizarDocumento = function(widget, fecharSwipe) {
        if (fecharSwipe == undefined) {
            fecharSwipe = true;
        }

        var params = {
            "NRSEQORCAMENTO": widget.currentRow.NRSEQORCAMENTO,
            "NRSEQORCADOC": widget.currentRow.NRSEQORCADOC,
            "NRORG": widget.currentRow.NRORG
        };

        General.callRestEngine("product#/fct_orcamento_widget/atualizarDocumento", "Row", params).then(function() {
                ScreenService.showMessage("Processo realizado com sucesso!");
                if (fecharSwipe)
                    ScreenService.closeSwipe();
                widget.reload();
            },
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
    };

    /**
     * Marca os elementos que já foram escolhidos ao clicar em editar.
     * 
     * @param {object} widget
     */
    this.marcarElementosEscolhidos = function(widget, elementos) {
        widget.dataSource.data.forEach(function(row) {
            if(row.TIPO == 'C') row.OLDCOMPPROPOSTA = 'N';
            elementos.forEach(function(elemento){
                if(elemento["IDELEMENTO"] == row["IDELEMENTO"] && elemento["NRCOMPSERVICO"] == row["NRCOMPSERVICO"]){
                    if(row.TIPO == 'C') {
                        row.OLDCOMPPROPOSTA = 'S';
                    }
                    row.__isSelected = true;
                    widget.dataSource.addCheckedRows(row);
                    row.CDCENTCUST = elemento["CDCENTCUST"];
                }
            });
        });
    };
    /**
     * Coloca a tela de Elementos da Proposta em modo de edição.
     * 
     * @param {object} widget
     */
    this.editarElementosProposta = function(widget) {
        widget.template  = "widget/master_detail/tree/grid.html";
        var wdgProposta = getWidgetProposta();
        var eleProposta = Util.clone(widget.dataSource.data, []);
        var params = [
            {
                "name": "NRORCAMENTOMODELO",
                "operator": "=",
                "value": wdgProposta.currentRow.NRORCAMENTOMODELO
            },
            {
                "name": "NRTIPOORCAMENTO",
                "operator": "IN",
                "value": wdgProposta.currentRow.NRTIPOORCAMENTO
            },
            {
                "name": "NRSEQORCAMENTO",
                "operator": "=",
                "value": wdgProposta.currentRow.NRSEQORCAMENTO
            }
        ];
        General.callRestEngine("product#/crud/composicao_elemento_proposta_inserida", 'FilterData', params).then(function(response){
            widget.dataSource.data = response.dataset.composicao_elemento_proposta_inserida;
            widget.edit();
            widget.showCheckbox = true;
            this.marcarElementosEscolhidos(widget, eleProposta);
            this.controlarAcessibilidadeTabbed("edit", false, widget);
        }.bind(this));
    };
    /**
     * Ação de cancelar dos Elementos da Proposta (editando).
     * 
     * @param {object} widget - Widget de Elementos da Proposta (editando)
     */
    this.cancelarElementosProposta = function(widget) {
        widget.dataSourceFilter[0].operator = "=";
        widget.dataSourceFilter[0].value    = "S";

        widget.list();
        widget.reload().then(function() {
            widget.showCheckbox = false;
            widget.unSelectAll();
            this.controlarAcessibilidadeTabbed("view", true, widget);
            widget.activate();
        }.bind(this));
    };
    /**
     * Volta para o grid de elementos da proposta inserida
    **/
    this.voltarGridElementos = function(widget){
        widget.dataSourceFilter[0].operator = "=";
        widget.dataSourceFilter[0].value    = "S";
        ScreenService.closeSwipe().then(function(){
            widget.reload().then(function() {
                widget.list();
                widget.showCheckbox = false;
            });
        });
    };

    /**
     * Atualiza os Elementos da Proposta.
     * 
     * @param {object} widget - Widget de Elementos da Proposta (editando)
     */
    this.atualizarComposicao = function(widget, tptratamento) {
        var rowProposta = templateManager.container.getWidget("wdgProposta").currentRow;
        var elementos   = [];
        var composicoes = [];

        widget.dataSource.data.forEach(function(row) {
            if(row.__isSelected){
                if (row.TIPO == "E") {
                    elementos.push({
                        "NRSEQORCAMENTO" : rowProposta.NRSEQORCAMENTO,
                        "NRCOMPSERVICO"  : row.NRCOMPSERVICO,
                        "NRITCOMPSERVICO": row.NRITCOMPSERVICO,
                        "CDSERVORCA"     : row.CDSERVORCA,
                        "IDELEMENTO"     : row.IDELEMENTO,
                        "IDELEMENTOPAI"  : this.getElementoPaiServico(row),
                        "IDESCOLHIDO"    : (row.__isSelected) ? "S" : "N",
                        "NRORG"          : row.NRORG,
                    });
                } else {
                    composicoes.push({
                        "NRSEQORCAMENTO"   : rowProposta.NRSEQORCAMENTO,
                        "NRCOMPSERVICO"    : row.CDELEMENTO,
                        "NMCOMPSERVICO"    : row.NMELEMENTO,
                        "NRCOMPSERVICOPAI" : row.IDARVOREPAI,
                        "IDCOMPOBRIGATORIA": row.IDSERVOBRIGCOMP,
                        "IDESCOLHIDO"      : (row.__isSelected) ? "S" : "N",
                        "NRORG"            : row.NRORG,
                        "OLDCOMPPROPOSTA"  : row.OLDCOMPPROPOSTA,
                        "CDCENTCUST"       : row.CDCENTCUST
                    });
                }
            }
        }.bind(this));

        var params = {
            "PROPOSTA"    : rowProposta,
            "ELEMENTOS"   : elementos,
            "COMPOSICOES" : composicoes,
            "TPTRATAMENTO": tptratamento
        };

        Sm00001000_LancamentoProposta_Service.validarObrigatoriedadeItensComposicao(widget.dataSource.data, rowProposta.NRSEQORCAMENTO)
        .then(function(msg) {
            if (!msg) {
                General.callRestEngine("product#/fct_orcamento_widget/atualizarComposicaoProposta", "Row", params).then(function(response) {
                        if (response.messages[0].message != null) {
                            ScreenService.showMessage(response.messages[0].message);
                        }
                        this.cancelarElementosProposta(widget);
    
                        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                        this.atualizarValorProposta(wdgPropostaGrid);
    
                    }.bind(this),
                    function(error) {
                        console.error(error);
                        ScreenService.showMessage(error.data.error);
                    });
            }
            else {
                ScreenService.showMessage(msg);
            }
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };

    this.getElementoPaiServico = function(row){
        if(row.__isSelected  && row._parent && row._parent.TIPO == 'C')
            return null;
        return row.IDARVORE;
    };
    /**
     * Seta o filtro da aba de Serviços dos Elementos da Proposta.
     * 
     * @param {object} widget - Widget de Serviços dos Elementos da Proposta
     */
    this.setFiltroServicosElementosProposta = function(widget) {
        var widgetElementosPropostas = getWidgetWdgElementosProposta();
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        widget.dataSourceFilter[0].value = widgetElementosPropostas.currentRow.NRSEQORCAMENTO;
        widget.dataSourceFilter[1].value = widgetElementosPropostas.currentRow.IDELEMENTO;
        widget.dataSourceFilter[2].value = wdgProposta.currentRow.NRTIPOORCAMENTO;
        widget.reload();
    };

    /**
     * Sintetiza as funções em uma mesma função
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.funcoesDoCampoAjuste = function(widget) {
        this.controlarCampoValorDoAjuste(widget);
        this.calcularValorTotalItem(widget);
        if (widget.currentRow.VRPERCDESCORCA) {
            this.adequaAjusteAoCampo(widget);
        }
    };

    /**
     * Previne possiveis erros com a troca seguida no select native do campo Ajuste
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.adequaAjusteAoCampo = function(widget) {
        var quantidade        = widget.currentRow.QTITEMORCAMENTO;
        var utilizaQtdeMedida = widget.currentRow.IDMULTMEDVRTOT;
        var qtdeMedida        = (widget.currentRow.QTMEDIDAORCA) ? parseFloat(widget.currentRow.QTMEDIDAORCA.replace(",", ".")) : 0;
        var valorUnitario     = widget.currentRow.VRUNITITEMORCA;
        var tipoAjuste        = widget.currentRow.IDTPDESCORCA;
        var ajuste            = widget.currentRow.VRPERCDESCORCA;

        var valorSemajuste = quantidade * valorUnitario;
        if (utilizaQtdeMedida == "S") {
            valorSemajuste *= qtdeMedida;
        }
        var valorTotal = valorSemajuste;
        if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));
        if (tipoAjuste == "P" && ajuste > 100) {
            widget.currentRow.VRPERCDESCORCA = null;
            widget.currentRow.VRDESCORCA     = null;
        }
        if (tipoAjuste == "V" && ajuste > valorTotal) {
            widget.currentRow.VRPERCDESCORCA = null;
            widget.currentRow.VRDESCORCA     = null;
        }
    };
    /**
     * Valida se a alteração na quantidade de medida é permitida
     * @params{Object} row - currentRow itens
    **/
    this.validarAlteracaoQtMedida = function(row, nomeCampoMedida) {
        if (row.QTMEDIDACALC !== "" && row.QTMEDIDACALC !== null) {
            UtilitiesPropostaContrato.validarPermissaoAlteracaoQtMedida().then(function(permiteAlteracao) {  
                var qtmedida = row[nomeCampoMedida];
                if (!qtmedida) qtmedida = row.QTMEDIDAORCA;
    
                qtmedida = Utilities.tirarMascaraValores(qtmedida);
                var qtmedidacalc   = Utilities.tirarMascaraValores(row.QTMEDIDACALCULADA);
                var qtmedidabackup = Utilities.tirarMascaraValores(row.QTMEDIDACALC);
                
                if (qtmedida !== qtmedidacalc && row.IDTIPO == 'M') {
                    if (row.IDALTQTDMEDIDAPROP == "4") { //nao altera
                        ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido alterar a Quantidade de Medida deste item.");
                        row.QTMEDIDASERV = qtmedidabackup.formatFloat(2);
                        row.QTMEDIDAORCA = qtmedidabackup.formatFloat(2);
                        return false;
                    } else if (row.IDALTQTDMEDIDAPROP == "3" && qtmedida > qtmedidacalc) { //somente reduzir
                        ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido aumentar a Quantidade de Medida deste item. O campo foi atribuído com o valor máximo (" + qtmedidacalc.formatFloat(2) + ").");
                        row.QTMEDIDASERV = qtmedidabackup.formatFloat(2);
                        row.QTMEDIDAORCA = qtmedidabackup.formatFloat(2);
                        return false;
                    } if (qtmedida < qtmedidacalc) { 
                        if(row.IDALTQTDMEDIDAPROP != "2") {
                            if(!permiteAlteracao) {
                                ScreenService.showMessage("Operação bloqueada.<br/>O operador logado não tem permissão para diminuir a quantidade de medida deste item.");
                                row.QTMEDIDASERV = qtmedidabackup.formatFloat(2);
                                row.QTMEDIDAORCA = qtmedidabackup.formatFloat(2);
                                return false;
                            }
                        } else {
                            if(permiteAlteracao)
                                ScreenService.showMessage("Operação bloqueada.<br/>O escopo de faturamento deste item não permite a diminuição de sua quantidade de medida.<br>Contate um administrador e tente novamente.");
                            else    
                                ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido reduzir a Quantidade de Medida deste item. O campo foi atribuído com o valor mínimo (" + qtmedidacalc.formatFloat(2) + ").");
                            
                            row.QTMEDIDASERV = qtmedidabackup.formatFloat(2);
                            row.QTMEDIDAORCA = qtmedidabackup.formatFloat(2);
                            return false;
                        }
                    }
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
        return true;
    };

    /**
     * Calcula o valor total do item da proposta.
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.calcularValorTotalItem = function(widget) {
        if (widget.mode != "list" && widget.mode != "inline_edit") {
            var quantidade        = widget.currentRow.QTITEMORCAMENTO;
            var utilizaQtdeMedida = widget.currentRow.IDMULTMEDVRTOT;
            var qtdeMedida        = (widget.getField("QTMEDIDAORCA")) ? widget.currentRow.QTMEDIDAORCA : widget.currentRow.QTMEDIDASERV;
            var valorUnitario     = widget.currentRow.VRUNITITEMORCA;
            var tipoAjuste        = widget.currentRow.IDTPDESCORCA;
            
            if (quantidade) quantidade = parseFloat(quantidade.toString().replace(",", "."));
            if (qtdeMedida) qtdeMedida = parseFloat(qtdeMedida.toString().replace(",", "."));
            if (valorUnitario) valorUnitario = parseFloat(valorUnitario.toString().replace(",", "."));

            var valorSemAjuste = quantidade * valorUnitario;
            if (utilizaQtdeMedida == "S") 
                valorSemAjuste *= qtdeMedida;
                
            var valorTotal = valorSemAjuste;
            
            var ajuste         = widget.currentRow.VRDESCORCA;
            var vrperdescorca  = widget.currentRow.VRPERCDESCORCA;
            
            if (ajuste) ajuste               = parseFloat(ajuste.toString().replace(",", "."));
            if (vrperdescorca) vrperdescorca = parseFloat(vrperdescorca.toString().replace(",", "."));
            if(!widget.currentRow.__is_new && ["P", "Y"].includes(tipoAjuste))
                ajuste = valorTotal * vrperdescorca/100;
            else
                vrperdescorca = ajuste * 100/valorSemAjuste;
            
            if(!widget.currentRow.IDTIPO || widget.currentRow.IDTIPO !== "I"){
                if (["V", "P"].includes(tipoAjuste) && (ajuste <= valorTotal || ajuste > 0)) {
                    if(ajuste < 0)
                        ajuste *= -1;
                        
                    valorTotal -= ajuste;
                } else if(["X", "Y"].includes(tipoAjuste)) {
                    valorTotal += ajuste;
                }
            }
            
            widget.currentRow.VRITEMSEMDESC   = valorSemAjuste;
            widget.currentRow.VRTOTALITEMORCA = valorTotal;
            widget.currentRow.VRDESCORCA      = (tipoAjuste !== "N" && ajuste) ? ajuste : null;
            widget.currentRow.VRPERCDESCORCA  = (tipoAjuste !== "N" && ajuste) ? vrperdescorca : null;
            
            templateManager.updateTemplate();
        }
    };
    
    /**
     * Realiza as validaçoes no campo conforme o tipo de ajuste selecionado.
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.validarCampoPorcentagemValor = function(widget) {
        var tipoAjuste   = widget.currentRow.IDTPDESCORCA;
        var vrpercajuste = widget.currentRow.VRPERCDESCORCA;
        var ajuste       = widget.currentRow.VRDESCORCA;

        if (ajuste) ajuste             = parseFloat(ajuste.toString().replace(",", "."));
        if (vrpercajuste) vrpercajuste = parseFloat(vrpercajuste.toString().replace(",", "."));
        if(['P', 'Y'].includes(tipoAjuste))
            ajuste = vrpercajuste;
            
        var valorSemAjuste = widget.currentRow.VRITEMSEMDESC;
        if (tipoAjuste == "P") {
            if (ajuste < 0 || ajuste > 100) {
                ScreenService.showMessage("O valor percentual do desconto deve ser maior que 0 (zero) e inferior a 100%");
                widget.currentRow.VRPERCDESCORCA = null;
                widget.currentRow.VRDESCORCA     = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            }
        } else if (tipoAjuste == "Y") {
            if (ajuste < 0) {
                ScreenService.showMessage("O valor percentual do acréscimo não pode ser inferior a 0 (zero)");
                widget.currentRow.VRPERCDESCORCA  = null;
                widget.currentRow.VRDESCORCA      = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            }
        } else if (tipoAjuste == "V") {
            if (ajuste < 0) {
                ScreenService.showMessage("O valor do ajuste não pode ser inferior a 0 (zero)");
                widget.currentRow.VRPERCDESCORCA  = null;
                widget.currentRow.VRDESCORCA      = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            } else if (ajuste > valorSemAjuste) {
                ScreenService.showMessage("O valor do ajuste não pode ser superior ao valor total");
                widget.currentRow.VRPERCDESCORCA  = null;
                widget.currentRow.VRDESCORCA      = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            }
        } else if (tipoAjuste == "X") {
            if (ajuste < 0) {
                ScreenService.showMessage("O valor do ajuste não pode ser inferior a 0 (zero)");
                widget.currentRow.VRPERCDESCORCA  = null;
                widget.currentRow.VRDESCORCA      = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            }
        }
    };

    /**
     * Controla o campo de desconto de acordo com o tipo.
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.controlarCampoValorDoAjuste = function(widget) {
        var tipoAjuste      = widget.currentRow.IDTPDESCORCA ? widget.currentRow.IDTPDESCORCA : "N";
        var fieldAjuste     = widget.getField("VRDESCORCA");
        var fieldPercAjuste = widget.getField("VRPERCDESCORCA");
        var vrItemSAjuste   = widget.currentRow.VRITEMSEMDESC;
        if(vrItemSAjuste) vrItemSAjuste = parseFloat(vrItemSAjuste.toString().replace(",", "."));
        
        widget.getField('IDTPDESCORCA').readOnly = false;
        if (tipoAjuste == "N") {
            fieldAjuste.readOnly     = (tipoAjuste == "N");
            fieldPercAjuste.readOnly = (tipoAjuste == "N");
            fieldAjuste.value("");
            fieldPercAjuste.value("");
        } else {
            var readOnly = ["P", "Y"].includes(tipoAjuste);
            fieldPercAjuste.readOnly = !readOnly;
            fieldAjuste.readOnly     = readOnly;
            
            var ajuste       = fieldAjuste.value();
            var vrpercajuste = fieldPercAjuste.value();
            if(ajuste) ajuste             = parseFloat(ajuste.toString().replace(",", "."));
            if(vrpercajuste) vrpercajuste = parseFloat(vrpercajuste.toString().replace(",", "."));
            if(ajuste < 0) ajuste         = ajuste * -1;
            
            if(!readOnly){
                fieldPercAjuste.value("");
                if(ajuste) fieldPercAjuste.value(parseFloat(((ajuste * 100)/vrItemSAjuste).toFixed(2)));
            } else {
                fieldAjuste.value("");
                //if(vrpercajuste) fieldAjuste.value(parseFloat((vrItemSAjuste * (vrpercajuste/100)).toFixed(2)));
                if(vrpercajuste) fieldAjuste.value((vrItemSAjuste * (vrpercajuste/100)).toFixed(2).replace(".", ","));
            }
        }
    };

    /**
     * Novo Registro da aba de Itens da Proposta ou de Adicionais.
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.itensNovoRegistro = function(widget) {
        
        ScreenService.showCustomDialog("Como deseja realizar a inclusão dos itens na proposta?", 'CONFIRMATION', null, [
            {
                "label": "Manualmente",
                "code": function() {
                    General.widgetNewRow(widget);
                    this.bloquearDadosMedida(widget);
                    var wdgProposta = getWidgetProposta();
                    var codEscopos  = wdgProposta.currentRow.NRTIPOORCAMENTO;
                    var nomeEscopos = wdgProposta.currentRow.NMTIPOORCAMENTO;
                    var idtipoorca  = wdgProposta.currentRow.IDTIPOORCA;
                    
                    if (!Array.isArray(codEscopos)) {
                        widget.currentRow.NRTIPOORCAMENTO = codEscopos;
                        widget.currentRow.NMTIPOORCAMENTO = nomeEscopos;
                    }
                    else if (codEscopos.length <= 1) {
                        widget.currentRow.NRTIPOORCAMENTO = codEscopos[0];
                        widget.currentRow.NMTIPOORCAMENTO = nomeEscopos[0];
                        widget.currentRow.IDTIPOORCA      = idtipoorca[0];
                        
                    } else {
                        var fieldCodEscopo  = widget.getField("NRTIPOORCAMENTO");
                        var fieldNomeEscopo = widget.getField("NMTIPOORCAMENTO");

                        fieldCodEscopo.template   = "field/select-multiple.html";
                        fieldCodEscopo.isVisible  = true;
                        fieldCodEscopo.showOnForm = true;
                        fieldCodEscopo.outData    = {
                            "IDTIPOORCA"      : "IDTIPOORCA",
                            "NRTIPOORCAMENTO" : "NRTIPOORCAMENTO",
                            "NMTIPOORCAMENTO" : "NMTIPOORCAMENTO"
                        };
                        
                        fieldNomeEscopo.isVisible  = false;
                        fieldNomeEscopo.showOnForm = false;
                    }

                    widget.currentRow.NRMOEDA = wdgProposta.currentRow.NRMOEDA;
                    
                    this.filtrarTipoDeItensManuaisDoEscopo(widget);
                }.bind(this)
            },
            {
                "label": "Formação de Preços",
                "code": function() {
                    this.abrirWdgFormacaoPrec();
                }.bind(this)
            },
            {
                "label": "Cancelar",
                "code": function() {}
            }
        ]);
    };

    /**
     * Verifica se a proposta utiliza composição de serviços para impedir alterações nas abas de Itens da Proposta e de Adicionais.
     * 
     * @param {object} widget - Widget de Itens da Proposta (editando) ou de Adicionais
     * @param {string} utilizaComposicao
     * @param {string} situacao
     */
    this.itensVerificarUtilizaComposicao = function(widget, utilizaComposicao, situacao) {
        var readOnly = (utilizaComposicao == "S");

        widget.fields.filter(function(field) {
                return field.composicao == true;
            })
            .forEach(function(field) {
                field.readOnly = readOnly;
            });

        if (situacao == "ELABORACAO") {
            widget.getAction("add").isVisible = !readOnly;
            widget.getAction("delete").isVisible = !readOnly;
        }
    };

    /**
     * Avança para a aba de Tipo de Medida Variável (inserindo).
     * 
     * @param {object} widget - Widget de Elementos (inserindo)
     */
    this.avancarTipoMedida = function(widget) {
        Sm00001000_LancamentoProposta_Service.validarObrigatoriedadeItensComposicao(widget.dataSource.data)
        .then(function(msg) {
            if (!msg) {
                widget.enabled = false;
    
                var wdgProposta = templateManager.container.getWidget("wdgProposta");
                var idutilizavarpreco = wdgProposta.currentRow.IDUTILIZAVARPRECO;
    
                if ((idutilizavarpreco) && (idutilizavarpreco.indexOf("S") > -1)) {
                    var wdgTipoMedidaInserindo = templateManager.container.getWidget("wdgTipoMedidaPropostaInserindo");
                    wdgTipoMedidaInserindo.enabled = true;
                    wdgTipoMedidaInserindo.isVisible = true;
                    wdgTipoMedidaInserindo.activate().then(function() {
                        this.setFiltroTipoMedida(wdgTipoMedidaInserindo);
                    }.bind(this));
                }
                else {
                    this.avancarItensProposta();
                }
            }
            else {
                ScreenService.showMessage(msg);
            }
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };

    /**
     * Seta o filtro da widget de Tipo de Medida.
     * 
     * @param {object} widget - Widget de Tipo de Medida da Proposta (inserindo)
     */
    this.setFiltroTipoMedida = function(widget) {
        var wdgItensComp = templateManager.container.getWidget("wdgItensComposicaoProposta");
        var idElementos = [];

        var idElementosRaiz = [];
        var elementosRaiz = [];
        wdgItensComp.dataSource.data.forEach(function(row) {
            if (row.TIPO == "E" && row.__isSelected && row._parent.TIPO == "C") {
                if (!idElementosRaiz.includes(row.IDELEMENTO)) {
                    idElementosRaiz.push(row.IDELEMENTO);
                    elementosRaiz.push(row);
                }
            }
        });
        
        wdgItensComp.dataSource.data.forEach(function(row) {
            if (row.TIPO == "E" && row.__isSelected) {
                idElementos.push(row.IDELEMENTO);
            }
        });

        idElementos = idElementos.filter(function(elem, i) {
            return idElementos.indexOf(elem) == i;
        });
        
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var nrtipoorcamento = wdgProposta.currentRow.NRTIPOORCAMENTO;
        var nrmoeda = wdgProposta.currentRow.NRMOEDA;
        
        var params = {
            "ID_ELEMENTOS"  : idElementos,
            "ELEMENTOS_RAIZ": elementosRaiz,
            "ESCOPOS"       : nrtipoorcamento,
            "NRMOEDA"       : nrmoeda
        };
        
        General.callRestEngine("product#/fct_orcamento_widget/getTiposMedidaVariaveisParaInserir", "Row", params).then(function(success) {
            widget.dataSource.data = success.dataset.tipos_medida_variavel;
            widget.edit();
            widget.getAction("save").isVisible = false;
            widget.getAction("cancel").isVisible = false;
        }.bind(this),
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    /**
     * Atualiza os tipos de medida variáveis da proposta.
     * 
     * @param {object} widget - Widget de Tipo de Medida da Proposta (editando)
     */
    this.atualizarTipoDeMedidaProposta = function(widget, idTpTratamento) {
        widget.saveRow().then(function() {
            var rowProposta = templateManager.container.getWidget("wdgProposta").currentRow;
            var params = {
                "PROPOSTA"     : rowProposta,
                "TPTRATAMENTO" : idTpTratamento
            };

            General.callRestEngine("product#/fct_orcamento_widget/atualizarItensPropostaPelosTiposDeMedida", "Row", params).then(function() {
                widget.list();
                widget.reload();
                var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                this.atualizarValorProposta(wdgPropostaGrid);
            }.bind(this),
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            if (error.data)
                ScreenService.showMessage(error.data.error);
            else
                ScreenService.showMessage(error);
        });
    };

    /**
     * Salva os escopos da proposta.
     * 
     * @param {widget} widget - Widget de Escopos da Proposta
     */
    this.salvarEscopoProposta = function(widget) {
        widget.currentRow.__columns_to_return = ["NRRELACESCOPOPROPOSTA", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];

        General.widgetSave(widget).then(function(success) {
                if (success.dataset && success.dataset.columnsToReturn[0]) {
                    var columnsToReturn = success.dataset.columnsToReturn[0];
                    widget.currentRow.NRRELACESCOPOPROPOSTA = columnsToReturn.NRRELACESCOPOPROPOSTA;
                    widget.currentRow.NRORG = columnsToReturn.NRORG;
                    widget.currentRow.DTINCLUSAO = columnsToReturn.DTINCLUSAO;
                    widget.currentRow.CDOPERINCLUSAO = columnsToReturn.CDOPERINCLUSAO;
                    widget.currentRow.NRORGINCLUSAO = columnsToReturn.NRORGINCLUSAO;
                }
            })
            .catch(function(error) {
                console.error(error);
                if (error.data)
                    ScreenService.showMessage(error.data.error);
                else
                    ScreenService.showMessage(error);
            });
    };

    /**
     * Desabilita a widget e avança para a aba de Itens da Proposta (inserindo).
     * 
     * @param {object} widget - Widget de Tipo de Medida da Proposta (inserindo)
     */
    this.desabilitarWidgetEAvancarItensProposta = function(widget) {
        widget.enabled = false;
        this.avancarItensProposta();
    };

    /**
     * Avança para a aba de Itens da Proposta (inserindo).
     */
    this.avancarItensProposta = function() {
        var wdgItensProposta = templateManager.container.getWidget("wdgItensProposta");
        wdgItensProposta.isVisible = false;

        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var wdgItensComp = templateManager.container.getWidget("wdgItensComposicaoProposta");
        var wdgTipoMedidaInserindo = templateManager.container.getWidget("wdgTipoMedidaPropostaInserindo");

        var elementosEscolhidos = [];
        var composicoes = [];

        wdgItensComp.dataSource.data.forEach(function(row) {
            if (row.__isSelected && row.TIPO == "E") {
                elementosEscolhidos.push({
                    "IDELEMENTO": row.IDELEMENTO,
                    "NRCOMPSERVICO": row.NRCOMPSERVICO
                });
            }
            else if (row.TIPO == "C") {
                composicoes.push({
                    "NRCOMPSERVICO": row.CDELEMENTO,
                    "NMCOMPSERVICO": row.NMELEMENTO,
                    "NRCOMPSERVICOPAI": row.IDARVOREPAI,
                    "IDCOMPOBRIGATORIA": row.IDSERVOBRIGCOMP,
                    "CDCENTCUST" : row.CDCENTCUST,
                    "IDESCOLHIDO": row.__isSelected ? "S" : "N"
                });
            }
        });

        var params = {
            "PROPOSTA": wdgProposta.currentRow,
            //"ELEMENTOS": wdgItensComp.dataSource.data,
            "ELEMENTOS_ESCOLHIDOS": elementosEscolhidos,
            "COMPOSICOES": composicoes,
            "MEDIDAS": wdgTipoMedidaInserindo.dataSource.data
        };

        General.callRestEngine("product#/fct_orcamento_widget/getItensPropostaParaInserir", "Row", params).then(function(success) {
                var wdgItensPropostaInserindo = templateManager.container.getWidget("wdgItensPropostaInserindo");
                wdgItensPropostaInserindo.dataSource.data = success.dataset.fct_itemorcamento;
                this.habilitarWidgetNomeContratual(success.dataset.fct_itemorcamento);
        }.bind(this),
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    /* Aplica o filtro de situação da proposta na tela de escopo 
    * para aparecer apenas o documento da situação atual
    * @params{Object} widget escopo proposta
    */
    this.setFilterSituacaoEscopo = function(widgetEscopoProposta){
        var wdgProposta = getWidgetProposta();
        var edititem = getActionEditEscopoProposta();
        var widgetProposta = templateManager.container.getWidget("wdgProposta");
        
        widgetEscopoProposta.dataSourceFilter[2].value = widgetProposta.currentRow.IDSITUACAOORCAMENTO;
        widgetEscopoProposta.reload();

        if(wdgProposta.currentRow.IDSITUACAOORCAMENTO == "CONCLUIDO") {
            edititem.isVisible = visibilidadeAcoes;
        }
    };

    /**
     * Coloca a widget de Itens da Proposta em modo de edição.
     * 
     * @param {object} widget - Widget de Itens da Proposta (inserindo)
     */
    this.wdgItensPropostaInserindoOnActivate = function(widget) {
        widget.edit();
        /*setTimeout(function() {
            widget.getAction("save").isVisible = false;
            widget.getAction("cancel").isVisible = false;
        }, 200);*/
    };

    /**
     * Volta para a aba de Tipo de Medida Variável (inserindo) ou para a aba de Elementos (inserindo).
     * 
     * @param {object} widget - Widget de Itens da Proposta (inserindo)
     */
    this.voltarItensProposta = function(widget) {
        widget.enabled = false;
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var idutilizavarpreco = wdgProposta.currentRow.IDUTILIZAVARPRECO;

        if ((idutilizavarpreco) && (idutilizavarpreco.indexOf("S") > -1)) {
            var wdgTipoMedidaPropostaInserindo = templateManager.container.getWidget("wdgTipoMedidaPropostaInserindo");
            wdgTipoMedidaPropostaInserindo.enabled = true;

            //wdgProposta.openTab("wdgTipoMedidaPropostaInserindo");
            wdgTipoMedidaPropostaInserindo.activate();
        }
        else {
            var wdgItensComposicaoProposta = templateManager.container.getWidget("wdgItensComposicaoProposta");
            wdgItensComposicaoProposta.enabled = true;

            //wdgProposta.openTab("wdgItensComposicaoProposta");
            wdgItensComposicaoProposta.activate();
        }
    };

    /**
     * Seta o campo IDUTILIZAVARPRECO de acordo com os escopos selecionados.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.escoposOnChange = function(widget) {
        var fieldEscopo     = widget.getField("NRTIPOORCAMENTO");
        var nrtipoorcamento = fieldEscopo.value();
        var escopos         = fieldEscopo.dataSource.data;

        var idutilizavarpreco = [];
        escopos.forEach(function(row) {
            if (nrtipoorcamento.indexOf(row.NRTIPOORCAMENTO) > -1) {
                idutilizavarpreco.push(row.IDUTILIZAVARPRECO);
            }
        });

        if (idutilizavarpreco.length == 0)
            idutilizavarpreco = "";

        widget.currentRow.IDUTILIZAVARPRECO = idutilizavarpreco;
        if ((idutilizavarpreco) && (idutilizavarpreco.indexOf("S") == -1)) {
            var wdgTipoMedidaInserindo = templateManager.container.getWidget("wdgTipoMedidaPropostaInserindo");
            wdgTipoMedidaInserindo.isVisible = false;
            wdgTipoMedidaInserindo.enabled   = false;
        }
    };
    
    /*this.calcularValorTotalItensInlineInserindo = function(row) {
        if (row._parent) {
            this.calcularValorTotalItensInlineInserindoFilho(row);
        }
        else {
            this.calcularValorTotalItensInlineInserindoPai(row);
        }
    };

    this.calcularValorTotalItensInlineInserindoPai = function(row) {
        if (row.QTMEDIDASERV != "" && row.QTMEDIDASERV != row.QTMEDIDACALC) {
            ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido alterar a Quantidade de Medida deste item.");
            row.QTMEDIDASERV = row.QTMEDIDACALC;
            row.QTMEDIDASERV_inline = row.QTMEDIDACALC;
        }
        else {
            row.VRTOTALITEMORCA = row.QTITEMORCAMENTO * row.VRUNITITEMORCA * (row.QTMEDIDAVARIAVEL ? parseFloat(row.QTMEDIDAVARIAVEL.toString().replace(",", ".")) : 1);
        }
    };
    var calcularValorUnitarioItemPai = function(row) {
        var widget = templateManager.container.getActiveWidget();
        
        var somaTotalFilhos = 0;
        widget.dataSource.data.forEach(function(item) {
            if (item.IDARVOREPAI == row.IDARVORE) {
                var valorTotal = item.VRTOTALITEMORCA;
                somaTotalFilhos += parseFloat(valorTotal.toString().replace(",", "."));
            }
        });
        
        row.changed = true;
        row.VRUNITITEMORCA = somaTotalFilhos;
    };
    
    /*this.calcularValorTotalItensInlineEditando = function(row, fieldName) {
        if (row._parent) 
            this.calcularValorTotalItensInlineEditandoFilho(row);
        else 
            this.calcularValorTotalItensInlineEditandoPai(row);
    };
    
    this.calcularValorTotalItensInlineEditandoPai = function(row) {
        if (Util.isEmptyOrBlank(row.changed_QTMEDIDAORCA)) {
            row.changed_QTMEDIDAORCA = row.QTMEDIDAORCA;
        }
        
        if (row.QTMEDIDAORCA != row.changed_QTMEDIDAORCA) {
            ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido alterar a Quantidade de Medida deste item.");
            row.QTMEDIDAORCA = row.changed_QTMEDIDAORCA;
            row.QTMEDIDAORCA_inline = row.changed_QTMEDIDAORCA;
        }
        else {
            var quantidade = parseFloat(row.QTITEMORCAMENTO.toString().replace(",", "."));
            var quantidadeMedida = row.QTMEDIDAORCA ? row.QTMEDIDAORCA : 1;
            quantidadeMedida = parseFloat(quantidadeMedida.toString().replace(",", "."));
            var vrunititemorca = parseFloat(row.VRUNITITEMORCA.toString().replace(",", "."));

            var utilizaQtdeMedida = row.IDMULTMEDVRTOT;
            var tipoAjuste = row.IDTPDESCORCA;
            var ajuste = (row.VRPERCDESCORCA == null) ? 0 : row.VRPERCDESCORCA;
            ajuste = parseFloat(ajuste.toString().replace(",", "."));
            
            var valorSemAjuste = quantidade * vrunititemorca;
            if (utilizaQtdeMedida == "S") {
                valorSemAjuste *= quantidadeMedida;
            }
            
            var valorTotal = valorSemAjuste;
    
            if (tipoAjuste == "V" && ajuste <= valorTotal && ajuste >= 0) {
                valorTotal -= ajuste;
            }
            else if (tipoAjuste == "P" && ajuste <= 100 && ajuste >= 0) {
                valorTotal -= valorTotal * ajuste / 100;
            }
            else if (tipoAjuste == "X" && ajuste >= 0) {
                valorTotal += ajuste;
            }
            else if (tipoAjuste == "Y" && ajuste >= 0) {
                valorTotal += valorTotal * ajuste / 100;
            }
    
            row.VRITEMSEMDESC = valorSemAjuste;
            row.VRTOTALITEMORCA = valorTotal;
        }
    };
    
    this.calcularValorTotalItensInlineEditandoFilho = function(row) {
        row.QTMEDIDAORCA = row.QTMEDIDAORCA_GRID;
        
        var qtdeVariacao  = row.QTMEDIDAVARIAVEL;
        var multiplicaVariacao = (row.IDMULTIPVARIACAO == "S");
        
        if (qtdeVariacao == undefined || qtdeVariacao == "")
            qtdeVariacao = 1;
        
        if (row.QTMEDIDAORCA != row.QTMEDIDACALC) {
            if (row.IDALTQTDMEDIDAPROP == "4") { //nao altera
                ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido alterar a Quantidade de Medida deste item.");
                row.QTMEDIDAORCA = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_inline = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_GRID = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_GRID_inline = row.QTMEDIDACALC;
            }
            else if (row.IDALTQTDMEDIDAPROP == "3" && row.QTMEDIDAORCA > row.QTMEDIDACALC) { //somente reduzir
                ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido aumentar a Quantidade de Medida deste item. O campo foi atribuído com o valor máximo (" + row.QTMEDIDACALC.formatFloat(2) + ").");
                row.QTMEDIDAORCA = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_inline = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_GRID = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_GRID_inline = row.QTMEDIDACALC;
            }
            else if (row.IDALTQTDMEDIDAPROP == "2" && row.QTMEDIDAORCA < row.QTMEDIDACALC) { //somente aumentar
                ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido reduzir a Quantidade de Medida deste item. O campo foi atribuído com o valor mínimo (" + row.QTMEDIDACALC.formatFloat(2) + ").");
                row.QTMEDIDAORCA = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_inline = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_GRID = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_GRID_inline = row.QTMEDIDACALC;
            }
        }
        
        row.VRTOTALITEMORCA = row.QTITEMORCAMENTO * row.QTMEDIDAORCA * row.VRUNITITEMORCA;
        if (multiplicaVariacao) {
            row.VRTOTALITEMORCA = row.VRTOTALITEMORCA * qtdeVariacao;
        }
        
        calcularValorUnitarioItemPai(row._parent);
        this.calcularValorTotalItensInlineEditando(row._parent);
    };*/
    
    /**
     * Calcula a quantidade de medida do item de acordo com a soma
     * da quantidade de medida dos seus itens filhos.
     */
    this.calcularValorQuantidadeMedidaItem = function(row, nomeCampoMedida) {
        var widget = templateManager.container.getActiveWidget();
        
        var somaMedidaFilhos = 0;
        widget.dataSource.data.forEach(function(item) {
            if (item.IDARVOREPAI == row.IDARVORE) {
                var qtd = item[nomeCampoMedida];
                somaMedidaFilhos += parseFloat(qtd.toString().replace(",", "."));
                //parseFloat(vrunititemorca.toString().replace(",", "."))
            }
        });
        
        row.changed = true;
        row[nomeCampoMedida] = somaMedidaFilhos;
    };

    /**
     * Salva a Proposta e seus Elementos, Itens e Tipos de Medida Variáveis.
     */
    this.inserirTudo = function() {
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        wdgProposta.currentRow.__columns_to_return = ["NRSEQORCAMENTO", "NRORCAMENTO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        // Usar para inserir o interlocutor no final    
        var nrSeqInterLocutor = null; 
        var nrOrcamentoModelo = null; 
        var nrseqorcamento = null;
        
        this.salvarWidgetProposta(wdgProposta).then(function(success) {
            nrseqorcamento = success.dataset.columnsToReturn[0].NRSEQORCAMENTO;

            wdgProposta.currentRow.NRSEQORCAMENTO = nrseqorcamento;
            wdgProposta.currentRow.NRORCAMENTO = success.dataset.columnsToReturn[0].NRORCAMENTO;

            var wdgTipoMedidaInserindo = templateManager.container.getWidget("wdgTipoMedidaPropostaInserindo");
            var medidas = [];

            var idutilizavarpreco = wdgProposta.currentRow.IDUTILIZAVARPRECO;
            if ((idutilizavarpreco) && (idutilizavarpreco.indexOf("S") > -1)) {
                medidas = wdgTipoMedidaInserindo.dataSource.data;
            }

            var wdgItensComp = templateManager.container.getWidget("wdgItensComposicaoProposta");

            var wdgItensPropostaInserindo = templateManager.container.getWidget("wdgItensPropostaInserindo");
            var itens = wdgItensPropostaInserindo.dataSource.data;

            var params = {
                "NRSEQORCAMENTO"   : nrseqorcamento,
                "NRTIPOORCAMENTO"  : wdgProposta.currentRow.NRTIPOORCAMENTO,
                "NRORCAMENTOMODELO": wdgProposta.currentRow.NRORCAMENTOMODELO,
                "MEDIDAS"          : medidas,
                "ITENS"            : itens,
                "INTERLOCUTOR"     : wdgProposta.currentRow.NRSEQINTERLOCUTOR
            };
            
            nrOrcamentoModelo = wdgProposta.currentRow.NRORCAMENTOMODELO;
            nrSeqInterLocutor = wdgProposta.currentRow.NRSEQINTERLOCUTOR;
            nrSeqInterLocutorfat = wdgProposta.currentRow.NRSEQINTERLOCUTORFAT;
            
            var currentRow = {
                "IDSITUACAOORCAMENTO" : "ELABORACAO",
                "NRSEQORCAMENTO"      : nrseqorcamento,
                "NRTIPOORCAMENTO"     : wdgProposta.currentRow.NRTIPOORCAMENTO,
                "IDUTILIZAVARPRECO"   : wdgProposta.currentRow.IDUTILIZAVARPRECO,
                "IDUTILIZACOMP"       : wdgProposta.currentRow.IDUTILIZACOMP,
                "QTITENSTRABSERV"     : 2,
                "NRORG"               : success.dataset.columnsToReturn[0].NRORG
            };
  
            General.callRestEngine("product#/fct_orcamento_widget/inserirTudo", "Row", params).then(function() {
                    this.voltarParaPropostaGrid().then(function() {
                        var wdgPropostaGrid        = getWidgetPropostaGrid();
                        wdgPropostaGrid.currentRow = currentRow;
                        this.wdgPropostaGridAfterMoveRow(wdgPropostaGrid);
                        //this.abrirWidgetNegociacaoValores(currentRow);
                    }.bind(this));
                }.bind(this))
                .catch(function(error) {
                    console.error(error);
                    var msg = "Timeout";
                    if (error.status !== 0 && error.data)
                        msg = error.data.error;

                    ScreenService.showMessage(msg);
                    this.voltarParaPropostaGrid();
                }.bind(this))
                .finally(function() {
                    wdgProposta.view();
                    wdgProposta.template = "widget/form-view.html";
                    this.alterarAcessibilidadeAbasProposta(true);

                    wdgItensComp.isVisible = false;
                    wdgTipoMedidaInserindo.isVisible = false;
                    wdgItensPropostaInserindo.isVisible = false;

                    wdgProposta.enabled = true;

                    var wdgItensProposta = templateManager.container.getWidget("wdgItensProposta");
                    wdgItensProposta.isVisible = true;
                    wdgItensProposta.enabled = true;

                    var wdgPropostaTabbed = templateManager.container.getWidget("wdgPropostaTabbed");
                    wdgPropostaTabbed.isVisible = true;
                    wdgPropostaTabbed.enabled = true;
                    templateManager.container.getWidget("wdgTabbedPrincipal").view();


                    var wdgEscoposProposta = templateManager.container.getWidget("wdgEscoposProposta");
                    wdgEscoposProposta.isVisible = true;
                    
                    var wdgNomeContratual = templateManager.container.getWidget("wdgNomeContratual");
                    wdgNomeContratual.list();
                    wdgNomeContratual.enabled = true;
                    
                    //var wdgLocalAtendimento = templateManager.container.getWidget("localAtendProposta");
                    //wdgLocalAtendimento.isVisible = true;
                     var paramsInt = {
                    "NRSEQORCAMENTO": nrseqorcamento,
                    "NRSEQINTERLOCUTOR": nrSeqInterLocutor,
                    "NRPARCINTERLOCUTORFAT": nrSeqInterLocutorfat,
                    "NRINTERLOCUTORANTE": null,
                    "NRORCAMENTOMODELO": nrOrcamentoModelo
                };
                General.callRestEngine("product#/fct_orcamento_widget/insereInterlocutor", "Row", paramsInt)
                .then(null)
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
                this.validaIdCertsign();
                }.bind(this));
        }.bind(this),
        function(error) {
            console.error(error);
            if (error.data)
                ScreenService.showMessage(error.data.error);
            else
                ScreenService.showMessage(error);
        });
        // this.salvarProposta(wdgProposta);
    };

    /**
     * Gera o relatório de memória de cálculo da proposta.
     * 
     * @param {object} wdgProposta
     */
    this.gerarRelatorioMemoriaCalculo = function(wdgProposta) {
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente gerar o relátorio de memória de cálculo?"), 'question', function(){
            var nrseqorcamento = wdgProposta.currentRow.NRSEQORCAMENTO;
            var nrorg          = wdgProposta.currentRow.NRORG;
            var nomeRelatorio  = "memoria_calculo_proposta";
            var params = {
                "NRSEQORCAMENTO": nrseqorcamento,
                "NRORG": nrorg,
                "isUrl": true
            };
    
            Utilities.openReport("product#/geraBirtReport", nomeRelatorio, params);
            ScreenService.closePopup();
        }.bind(this));
    };

    this.validaPerfilEndereco = function () {
        var wdgProposta = getWidgetProposta();
        var newitem = getActionAddEndereco();
        var edititem = getActionEditEndereco();
        if(wdgProposta.currentRow.IDSITUACAOORCAMENTO == "CONCLUIDO") {
            newitem.isVisible = visibilidadeAcoes;
            edititem.isVisible = visibilidadeAcoes;
        }
    }

    /**
     * Executa validações da proposta ao trocar de linha.
     * 
     * @param {object} wdgPropostaGrid - Widget de Proposta (grid)
     */
    this.wdgPropostaGridAfterMoveRow = function(wdgPropostaGrid) {
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        wdgTabbedPrincipal.descriptionOrder = wdgPropostaGrid.descriptionOrder;

        var nrseqorcamento = wdgPropostaGrid.currentRow.NRSEQORCAMENTO;
        var nrorg          = wdgPropostaGrid.currentRow.NRORG;

        setLocalVar("PROPOSTA_NRSEQORCAMENTO", nrseqorcamento);
        setLocalVar("PROPOSTA_NRORG", nrorg);

        if(wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO == 'CONCLUIDO')
            this.alterarVisibilidadeActionEditProposta(wdgPropostaGrid, nrorg);
        
        if(idAlteraPropConc) {
            this.validacoesGeraisProposta(wdgPropostaGrid);
        } else {
            this.verificaParamAlteraPropConc(wdgPropostaGrid).then(function(){
                this.validacoesGeraisProposta(wdgPropostaGrid);
            }.bind(this));
        }
        
        
        this.desobrigaCamposEvolucaoEmail();
        this.abrirSwipeProposta();
    };
    /**
     * Valida e muda a visibilidade da action de edição da aba de dados da proposta em 
     * um orcamento concluida
    **/
    this.alterarVisibilidadeActionEditProposta = function(widget, nrorg) {
        if(!idAlteraPropConc) {
            this.verificaParamAlteraPropConc(widget).then(function(){
                this.alterarVisibilidadeActionEditProposta(widget, nrorg);
            }.bind(this));
        } else {
            if(idAlteraPropConc == 'N') {
                getActionEditWidgetProposta().isVisible = false;
            } else {
                if(visibilidadeAcoes !== null) {
                    getActionEditWidgetProposta().isVisible = visibilidadeAcoes;
                } else {
                    this.validarPermissaoAlteracaoDoPerfilAcesso(widget, nrorg).then(function(){
                        getActionEditWidgetProposta().isVisible = visibilidadeAcoes;
                    });
                }
            }
            
            if(idOperMaster === null && idUtilizaHierOper === null)
                this.buscaInformacaoHierarquia();
        }

        this.desobrigaCamposEvolucaoEmail();
    };
    /**
     * valida se o perfil de acesso do operador pode editar as propostas 
     * @params{Object} widget - widget de propostas 
     * @params{integer} nrorg - número da organização 
    **/
    this.validarPermissaoAlteracaoDoPerfilAcesso = function (widget, nrorg) {
        var deferred = ZHPromise.defer();
        var nrparcnegocioOperLogado = ["NRPARCNEGOCIO"];
        Operador_Repository.getOperadorLogado().then(function (success) {
            nrparcnegocioOperLogado[0] = success[0].NRPARCNEGOCIO;
            nrparcnegocioOperLogado[1] = nrorg;
            nrparcnegocioOperLogado[2] = 'P';
            General.callRestEngine('product#/fct_parametrogeral_widget/validarperfilacesso', "Row", nrparcnegocioOperLogado).then(function (perfilusuario) {
                visibilidadeAcoes = perfilusuario.dataset.fct_parametrogeral_widget[0];
                getActionEditWidgetProposta().isVisible = visibilidadeAcoes;
                deferred.resolve();
            }).catch(function(error){
                deferred.reject(error);
            });
        });
        
        return deferred.promise;
    };
    
    this.desobrigaCamposEvolucaoEmail = function() {
       var wdgEnviarEmail = templateManager.container.getWidget("wdgEnviarEmail");
        is_required = this.alternaCampo(wdgEnviarEmail, false);
        wdgEnviarEmail.reload();
    };
    /**
     * Abre a widget de Dados da Proposta (view/form)
     */
    this.abrirSwipeProposta = function() {
        var wdgProposta        = templateManager.container.getWidget("wdgProposta");
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");

        ScreenService.openSwipe(wdgTabbedPrincipal).then(function() {
            wdgTabbedPrincipal.openTab("wdgPropostaTabbed");
            wdgProposta.list();
            wdgProposta.reload().then(function() {
                wdgProposta.view();
                wdgProposta.activate();
                wdgTabbedPrincipal.view();
            });
        });
    };

    /**
     * Seta a currentRow com o registro retornado do banco de dados.
     * 
     * @param {object} wdgProposta - Widget de Dados da Proposta (view/form)
     */
    this.setDadosProposta = function(wdgProposta) {
        wdgProposta.currentRow = wdgProposta.dataSource.data[0];
    };
    
    /**
     * Deleta o contato gerado para as propostas não ligadas a uma oportunidade
     * @params{Object} row currentRow excluida da proposta
    **/
    this.deletaContatoGerado = function(row){
        if(!row.NRSEQOPORT && row.NRSEQCONTATO){
            var params = {
                "NRSEQCONTATO": row.NRSEQCONTATO
            };
            
            General.callRestEngine("product#/fct_orcamento_widget/deletaContatoProposta", "Row", params);
        }
    };

    /**
     * Exclui a Proposta.
     * 
     * @param {object} wdgProposta - Widget de Dados da Proposta (view/form)
     */
    this.excluirProposta = function(wdgProposta) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function() {
            Utilities.validarCamposDuplicados(wdgProposta, null, ["NRSEQCONTATO", "NRORG"], "product#/evolcont_parcnegocio").then(function(valida){
                if(!valida){
                    ScreenService.confirmMessage("Não é possível excluir a proposta porque ela possui registros filhos. Deseja inativar a proposta?", "question",
                    function() {
                        General.callRestEngine("product#/fct_orcamento_widget/inativaPropContato", "Row", wdgProposta.currentRow)
                        .then(null)
                        .catch(function(error){
                            Utilities.showErrorMessage(error);
                        });
                    });
                } else {
                    var params = {
                        "NRSEQORCAMENTO": wdgProposta.currentRow.NRSEQORCAMENTO
                    };
                    General.callRestEngine("product#/fct_orcamento_widget/buscarLinksDocumentosProposta", "Row", params).then(function(response) {
                        
                        wdgProposta.removeRow().then(function() {
                            this.voltarParaPropostaGrid();
                            var links = response.dataset.links;
                            var params2 = [];
                            links.forEach(function(link) {
                                params2.push({
                                    "CAMINHO": link
                                });
                            });

                            General.callRestEngine("product#/file_server/excluirArquivos", "DataSet", params2).then(null).catch(function(error) {
                                Utilities.showErrorMessage(error);
                            });
                            //this.deletaContatoGerado(row);
                        }.bind(this),
                        function(error) {
                            if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                                //ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
                                ScreenService.confirmMessage("Não é possível excluir a proposta porque ela possui registros filhos. Deseja inativar a proposta?", "question",
                                function() {
                                    this.inativarProposta(wdgProposta);
                                }.bind(this),
                                null);
                            } else {
                                Utilities.showErrorMessage(error);
                            }
                        }.bind(this));
                    }.bind(this));
                }
            }.bind(this));
        }.bind(this));
    };
    
    this.inativarProposta = function(wdgProposta) {
        wdgProposta.currentRow.changed = true;
        wdgProposta.currentRow.IDATIVO = "N";
        General.widgetSave(wdgProposta).then(function() {
            this.voltarParaPropostaGrid();
        }.bind(this),
        function(error) {
            console.error(error);
            if (error.data)
                ScreenService.showMessage(error.data.error);
            else
                ScreenService.showMessage(error);
        });
    };
    
    /**
     * Gera um contato para a proposta caso ela não esteja ligada a uma oportunidade
     * @params{Object} widget widget proposta
    **/
    this.geraContatoProposta = function(row){
        var params = {
            "NRSEQORCAMENTO"    : row.NRSEQORCAMENTO,
            "DSREFCONTATO"      : row.NRORCAMENTO            ,
            "NRPARCNEGOCIO"     : row.NRPARCNEGOCIO,
            "NRPARCINTERLOCUTOR": row.NRPARCINTERLOCUTOR,
            "NRTIPOORCAMENTO"   : row.NRTIPOORCAMENTO
        };
        
        return General.callRestEngine("product#/fct_orcamento_widget/geraContato", "Row", params);
    };
    
    /**
     * verifica se ha alteração do contato da proposta
     * @params{Object} widget widget proposta
    **/
    this.verificaAlteracaoContato = function(widget){
        var deferred = ZHPromise.defer();
        
        if(!widget.currentRow.__is_new){
            if(widget.currentRow.NRSEQCONTATO !== widget.backupRow.NRSEQCONTATO){
                ScreenService.confirmMessage("A alteração do contato vinculado a essa proposta pode gerar a perda das evoluções atuais. Deseja continuar?", "question", 
                function(){
                    deferred.resolve();
                }, function(){
                    widget.currentRow.NRSEQOPORT   = widget.backupRow.NRSEQOPORT;
                    widget.currentRow.NRSEQCONTATO = widget.backupRow.NRSEQCONTATO;
                    deferred.resolve();
                });
            } else {
                deferred.resolve();
            }
        }
        else {
            deferred.resolve();
        }
        
        return deferred.promise;
    };
    
    /**
     * Salva a proposta.
     * 
     * @param {object} wdgProposta - Widget de Dados da Proposta (view/form)
     */
    this.salvarWidgetProposta = function(wdgProposta) {
        var deferred   = ZHPromise.defer();
        var contato    = wdgProposta.currentRow.NRSEQCONTATO; 
        var row        = wdgProposta.currentRow;
        var isnew      = wdgProposta.currentRow.__is_new;
        
        this.verificaAlteracaoContato(wdgProposta).then(function(){
            if (wdgProposta.currentRow.__is_new) {
                if (wdgProposta.paramsNewCode && wdgProposta.paramsNewCode.__is_new_code) {
                    wdgProposta.currentRow.__using_unit = wdgProposta.paramsNewCode.__using_unit || false;
                    wdgProposta.currentRow.__new_code_pk = wdgProposta.paramsNewCode.__new_code_pk;
                    wdgProposta.currentRow.__table_pk = wdgProposta.paramsNewCode.__table_pk;
                    wdgProposta.currentRow.__new_code_length = wdgProposta.paramsNewCode.__new_code_length || 5;
                    wdgProposta.currentRow.__cast_number = wdgProposta.paramsNewCode.__cast_number || false;
                }
            }
            
            var rowProposta = {
                "NRORCAMENTOMODELO": wdgProposta.currentRow.NRORCAMENTOMODELO,
                "NRSEQORCAMENTO"   : wdgProposta.currentRow.NRPARCNEGOCIO,
                "NRORG"            : wdgProposta.currentRow.NRORG
            };
            
            wdgProposta.saveRow().then(function(response) {
                    // wdgProposta.view();
                    // wdgProposta.template = "widget/form-view.html";
                    // this.alterarAcessibilidadeAbasProposta(true);
                    if(response.dataset){
                        rowProposta.NRSEQORCAMENTO = response.dataset.NRSEQORCAMENTO[0];
                        row.NRSEQORCAMENTO         = response.dataset.NRSEQORCAMENTO[0];
                        row.NRORCAMENTO            = response.dataset.columnsToReturn[0].NRORCAMENTO;
                        this.verificaIntegracaoPropOpor(rowProposta);
                        if(!contato && isnew)
                            this.geraContatoProposta(row);
                    }
                    
                    templateManager.container.getWidget("wdgTabbedPrincipal").view();
                    deferred.resolve(response);
                    
                }.bind(this),
                function(error) {
                    deferred.reject(error);
                });
        }.bind(this));
        
        return deferred.promise;
    };

    /**
     * Ativa a widget de Proposta (grid)
     * 
     * @param {widget} widget - Widget de Proposta (grid)
     */
    this.wdgPropostaGridOnEnter = function(widget) {
        widget.activate();
    };

    /**
     * Busca o endereço pelo CEP.
     */
    this.buscarEndereco = function(widget) {
        var fields = ["CDPAIS", "NMPAIS", "SGESTADO", "NMESTADO", "CDMUNICIPIO", "NMMUNICIPIO", "CDBAIRRO", "NMBAIRROENDERECO", "CDLOGRADOURO", "DSLOGRADOURO", "DSENDERECO"];
        
        if(widget.currentRow.NRCEPENDERECO){
            Utilities.buscarCep(widget.currentRow.NRCEPENDERECO).then(function(response) {
                    var endereco = response.dataset.ENDERECO;
                    fields.forEach(function(field) {
                        widget.currentRow[field] = (endereco[field]) ? endereco[field] : "";
                    });
                    
                    if(response.messages){
                        var msg = ScreenService.i18n(response.messages[0].message);
                        ScreenService.showMessage(msg);
                    }
                    
                    angular.element('#footer').click();;
                },
                function(error) {
                    console.error(error);
                    if (error.data){
                        var mensagem = error.data.error;
                        if(error.data.error === "Erro de configuração de proxy!")
                            mensagem = "Endereço não encotrado.";
                            
                        ScreenService.showMessage(mensagem);
                    } else {
                        ScreenService.showMessage("Falha na conexão.");
                    }
            });
        } else {
            fields.forEach(function(field) {
                widget.currentRow[field] = "";
            });
        }
    };

    this.cancelarItens = function(widget) {
        widget.list();
        widget.reload();
    };

    /**
     * Valida se os documentos podem ser gerados em DOCX
     * e pergunta com qual extensão o usuário deseja baixar
    **/
    this.validaParametrosDoc = function(widget, idescopo){
        General.callRestEngine("product#/fct_orcamento_widget/getParametroExtensaoDoc", "Row", {"NRSEQORCAMENTO": widget.currentRow.NRSEQORCAMENTO}).then(function(response){
            if(!response.dataset || !response.dataset.IDPERMITEDOCX){
                ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente gerar o documento da proposta?"), 'question', function(){
                    this.gerarRelatorioDocumentoProposta(widget, idescopo, 'P');
                }.bind(this));
            } else {
                var permite = false;
                response.dataset.IDPERMITEDOCX.forEach(function(idpermite){
                    if(idpermite.IDPERMITEDOCX && idpermite.IDPERMITEDOCX === "S"){
                        permite = true;
                    }
                }.bind(this));

                if(!permite){
                    ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente gerar o documento da proposta?"), 'question', function(){
                        this.gerarRelatorioDocumentoProposta(widget, idescopo, 'P');
                    }.bind(this));
                } else {
                    ScreenService.showCustomDialog("Alguns documentos da proposta podem ser gerados em DOCX.<br>Como deseja gerá-los?", 'CONFIRMATION', null, [{
                            "label": "Gerar PDF",
                            "code": function() {
                                this.gerarRelatorioDocumentoProposta(widget, idescopo, 'P');
                            }.bind(this)
                        }, {
                            "label": "Gerar DOCX",
                            "code": function() {
                                this.gerarRelatorioDocumentoProposta(widget, idescopo, 'D');
                            }.bind(this)
                        }, {
                            "label": "Cancelar",
                            "code": function() {}
                        }
                    ]);
                }
            }
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    }

    /**
     * Gera o relatório do documento da proposta.
     */
    this.gerarRelatorioDocumentoProposta = function(widget, idescopo, idExtensao) {
        var nrseqorcamento  = widget.currentRow.NRSEQORCAMENTO;
        var nrtipoorcamento = widget.currentRow.NRTIPOORCAMENTO;
        var idSituacao      = widget.currentRow.IDSITUACAOORCAMENTO;
        var nrorg           = widget.currentRow.NRORG;

        if (idescopo == "N") {
            nrtipoorcamento = 0;
        }
        
        if(!widget.currentRow.IDSITUACAOORCAMENTO){
            var widgetProposta = templateManager.container.getWidget("wdgProposta");
            idSituacao = widgetProposta.currentRow.IDSITUACAOORCAMENTO;
        }

        var params = {
            "NRSEQORCAMENTO"     : nrseqorcamento,
            "NRTIPOORCAMENTO"    : nrtipoorcamento,
            "NRORG"              : nrorg,
            "IDSITUACAOORCAMENTO": idSituacao,
            "IDPERMITEDOCX"      : idExtensao
        };

        var timeout = 5 * 60000; //5 min
        Utilities.callRestEngineWithTimeout("product#/fct_orcamento_widget/gerarDocumentoProposta", "Row", params, timeout).then(function(success) {
            var links = success.dataset.documentos.links;
            links.forEach(function(link) {
                if(link) window.open(window.backendIndexUrl + link);
            });
            
            if(success && success.messages && success.messages.length > 0 && success.messages[0].message)
                ScreenService.showMessage(success.messages[0].message);
                
            ScreenService.closePopup();
        }).catch(function(error) {
            console.error(error);
            var msg = "Timeout";
            if (error.status !== 0 && error.data)
                msg = error.data.error;
            ScreenService.showMessage(msg);
        });
    };


    /* ---=========  FUNÇÕES DA PÁGINA FORMAÇÃO PREÇO =========--- */


    /**
     * A função "abrirWdgFormacaoPrec" abre a tela "Itens de Formação de Preço"
     */

    this.abrirWdgFormacaoPrec = function() {
        ScreenService.closePopup();
        var formacaoPreco = templateManager.container.getWidget("itemFormacaoPreco");
        formacaoPreco.getField("NRCALCFORMPRECSERV").dataSource.clearCheckedRows();
        formacaoPreco.isVisible = true;
        ScreenService.openSwipe(formacaoPreco).then(function() {
            formacaoPreco.edit();
            this.filtraFormPreco(formacaoPreco);
            this.verificaAcessibilidadeFieldServ(formacaoPreco);
            
            var wdgProposta = getWidgetProposta();
            var codEscopos  = wdgProposta.currentRow.NRTIPOORCAMENTO;
            var nomeEscopos = wdgProposta.currentRow.NMTIPOORCAMENTO;
            
            if (!Array.isArray(codEscopos)) {
                formacaoPreco.currentRow.NRTIPOORCAMENTO = codEscopos;
                formacaoPreco.currentRow.NMTIPOORCAMENTO = nomeEscopos;
            } else if (codEscopos.length == 1) {
                formacaoPreco.currentRow.NRTIPOORCAMENTO = codEscopos[0];
                formacaoPreco.currentRow.NMTIPOORCAMENTO = nomeEscopos[0];
            } else {
                var fieldNomeEscopo = formacaoPreco.getField("NMTIPOORCAMENTO");
                fieldNomeEscopo.readOnly = false;
            }
        }.bind(this));
    };
    
    /**
     * Abre a widget que permite a edição do cálculo da formação de preço
     * @params{Object} widget widget Item Formação de preço
    **/
    this.abreWidgetEdicaoCalculoItens = function(widget) {
        var tabbedFormacao = templateManager.container.getWidget("wdgTabbedPrincipalFormPreco");
        var itensCalculo   = templateManager.container.getWidget("editaCalculoItensFormPreco");
        var widgetProposta = templateManager.container.getWidget("wdgProposta");
        
        ScreenService.closePopup();
        if(!widget.currentRow.NMFORMPRECOSERVICO || widget.currentRow.NMFORMPRECOSERVICO.length === 0){
            ScreenService.showMessage("Não é possível editar os itens do cálculo sem indicar uma Formação de Preço.<br> Para continuar selecione uma Formação de Preço <br>");
        } else {
            if(widget.currentRow.NMFORMPRECOSERVICO && widget.currentRow.NMFORMPRECOSERVICO.length > 1){
                ScreenService.showMessage("A edição do Cálculo de Formação de Preço só é permitida quando apenas um tipo de Formação de Preço é selecionado. <br>Para continuar escolha apenas uma Formação de Preço.<br>");
            } else {
                if(widget.currentRow.NRCALCFORMPRECSERV && widget.currentRow.NRCALCFORMPRECSERV.length > 0){
                    ScreenService.showMessage("Não é possível editar os itens do cálculo após selecionar os Itens de Formação de Preço.<br>Para continuar desfaça a seleção de itens atual.<br>");
                } else {
                    var params = {
                        "NRSEQORCAMENTO": widgetProposta.currentRow.NRSEQORCAMENTO,
                        "NRITEMORCAMENTO": null
                    };
                    
                    General.callRestEngine("product#/fct_relacestruturaprop_widget/deletaAlteracoesInutilizadas", "Row", params).then(function(){
                        this.salvarAlteracoesCalculo(itensCalculo, true, null).then(function(){
                            tabbedFormacao.isVisible = true;
                            itensCalculo.isVisible   = true;

                            ScreenService.openSwipe(tabbedFormacao).then(function() {
                                tabbedFormacao.openTab("editaCalculoItensFormPreco");
                                
                                itensCalculo.dataSourceFilter[0].value = widget.currentRow.NRFORMPRECOSERVICO[0];
                                itensCalculo.dataSourceFilter[2].value = templateManager.container.getWidget('wdgProposta').currentRow.NRSEQORCAMENTO;
                                Utilities.abreArvoreElementos(itensCalculo);
                                
                                UtilitiesPropostaContrato.habilitaEdicaoWidgetItemFormacao(widget, true);
                            });    
                        }.bind(this));    
                    }.bind(this));    
                }
            }
        }
    };
    
    /**
     * Habilita a tela para receber um novo item de formação de preço
     * e seta o número de ordenação na tela
     * @params{Object} widget widget editar formação de preço
    **/
    this.adicionarItensFormacao = function(widget){
        var nritemorcamento    = null;
        var nrformprecoservico = null;
        var widgetItensPropostaTree = getWidgetItensPropostaTree();
        var widgetItensProposta = getWidgetItensProposta();
        
        if(widget.mode === "view"){
            var nmElementoSup = widget.currentRow.DSREFECALCULO;
            var nrCalcServPai = widget.currentRow.NRCALCFORMPRECSERV;
            widget.newRow();
            this.bloquearDadosMedidaEdicao(widget);
            widget.currentRow.DSREFECALCULOPAI      = nmElementoSup;
            widget.currentRow.NRCALCFORMPRECSERVPAI = nrCalcServPai;
        } else {
            widget.newRow();
            this.bloquearDadosMedidaEdicao(widget);
        }

        if(widget.name === "editaItensFormPreco"){
            nritemorcamento    = widgetItensProposta.currentRow.NRITEMORCAMENTO;
            nrformprecoservico = widget.dataSource.data[0].NRFORMPRECOSERVICO;
        } else {
            nrformprecoservico = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
            widget.template    = "widget/form.html";
        }
        
        var params = {
            "NRFORMPRECOSERVICO" : nrformprecoservico,
            "NRITEMORCAMENTO"    : nritemorcamento,
            "TABELA"             : (widget.dataSource.name === "product#/fct_calcformprecserv_widget") ? null : "FCT_CALCFORMPRECSERV_ITPRO"
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/recebeOrdenacao', "Row", params).then(function(response){
            if(response.dataset.ORDENACAO){
                var ordenacao = response.dataset.ORDENACAO.IDORDENACAO;
                ordenacao = ordenacao ? (ordenacao + 10) : 10;
                widget.currentRow.IDORDENACAO        = ordenacao;
                widget.currentRow.NRFORMPRECOSERVICO = nrformprecoservico;
                widget.currentRow.NRITEMORCAMENTO    = nritemorcamento;
            } else {
                widget.currentRow.IDORDENACAO = 10;
            }
        }, 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
         /**
     * Realiza o controle dos campos de "Medida" de acordo 
     * com o field "Utiliza Medida?"
     * 
     * @param {object} widget widget de 
     * Itens para pegar o field "IDUSAMEDIDA".
    */
    
    this.bloquearDadosMedidaEdicao = function(widget){
        var usamedida = widget.getField('IDUSAMEDIDA').value();
        
        /* Bloqueia a Modalidade de pagamento*/
        if (usamedida == 'N') {
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = true;
            widget.getField('DSTPMEDIDASERV').setValue(null);
            widget.getField('DSTPMEDIDASERV').validations = {};
            widget.getField('CDTPMEDIDASERV').setValue(null);
            widget.getField('CDTPMEDIDASERV').validations = {};
            /* Quantidade de Medida */
            widget.getField('QTMEDIDACALC').readOnly = true;
            widget.getField('QTMEDIDACALC').setValue(null);
            widget.getField('QTMEDIDACALC').validations = {};
        } else {
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = false;
            widget.getField('DSTPMEDIDASERV').validations = {"required": true};
            /* Quantidade de Medida */
            widget.getField('QTMEDIDACALC').readOnly = false;
            widget.getField('QTMEDIDACALC').validations = {"required": true};
        }
    };
    
    /**
     * Filtra a field de elemento superior para aperecer apenas os elementos
     * já inseridos na árvore
     * @params{Object} widget widget editar formaçao de preço
    **/
    this.filtraElementoSuperiorItensFormacao = function(widget){
        var widgetPropostaGrid = getWidgetPropostaGrid();
        if(!widget.currentRow.DSREFECALCULOPAI){
            var nrformprecoservico = null;
            widget.currentRow.DSREFECALCULOPAI = "";
            
            if(widget.name === "editaItensFormPreco"){
                nrformprecoservico = widget.dataSource.data[0].NRFORMPRECOSERVICO;
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[3].value = getWidgetItensProposta().currentRow.NRITEMORCAMENTO;
            } else {
                nrformprecoservico = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
            }
            
            if(widget.currentRow.NMELEMENTO){
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[0].value = nrformprecoservico;
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[1].value = (widget.currentRow.NRCALCFORMPRECSERV) ? widget.currentRow.NRCALCFORMPRECSERV : -1;
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[2].value = templateManager.container.getWidget("wdgProposta").currentRow.NRSEQORCAMENTO;;             
                widget.getField("DSREFECALCULOPAI").reload();
            }
        }
    };
    
    /**
     * Salva no grid as alterações feitas no item de formação de preco
     * @params{Object} widget widget editar calculo de formação de preço
    **/
    this.salvarLocalmenteAlteracoesFormPreco = function(widget){
        if(widget.isValid()) {
            var isnew       = widget.currentRow.__is_new;
            var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
            
            widget.currentRow.NRSEQORCAMENTO     = templateManager.container.getWidget("wdgProposta").currentRow.NRSEQORCAMENTO;
            widget.currentRow.NRFORMPRECOSERVICO = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
            widget.currentRow.NRITEMORCAMENTO    = null;
            
            if(!widget.currentRow.DSREFECALCULO)
                widget.currentRow.DSREFECALCULO = widget.currentRow.IDITEMCALCULO + " - " + widget.currentRow.NMELEMENTO;
            
            if(!widget.currentRow.NRRELACFORMACAOPROP)
                widget.currentRow.NRRELACFORMACAOPROP = 0;
                
            UtilitiesPropostaContrato.calculaItem(widget, "FCT_CALCFORMPRECSERV_ITPRO", widget.currentRow.NRITEMORCAMENTO, null).then(function(){    
                if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
                    widget.currentRow.__using_unit      = widget.paramsNewCode.__using_unit || false;
                    widget.currentRow.__new_code_pk     = widget.paramsNewCode.__new_code_pk;
                    widget.currentRow.__table_pk        = widget.paramsNewCode.__table_pk;
                    widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
                    widget.currentRow.__cast_number     = widget.paramsNewCode.__cast_number || false;
                    if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                        widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
                }
                
                widget.saveRow().then(function(response) {
                    if(response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0)
                        widget.currentRow.NRCALCFORMPRECSERV = response.dataset.columnsToReturn[0].NRCALCFORMPRECSERV;
            
                    if (tipoCalculo == 'S'){
                        var params = {
                            "NRCALCFORMPRECSERV" : widget.currentRow.NRCALCFORMPRECSERV,
                            "NRITEMORCAMENTO"    : widget.currentRow.NRITEMORCAMENTO,
                            "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO"
                        };
                        
                        General.callRestEngine('product#/fct_calcformprecserv_widget/calcSomatorioFilhos', "Row", params);  
                    }
                    
                    ScreenService.confirmMessage("Deseja recalcular a Formação de Preço?", 'question', 
                        function (success) {
                            widget.template = "widget/tree/grid.html";
                            widget.list();
                            this.recalcularArvore(widget);
                        }.bind(this),
                        function() {
                            if(!isnew) {
                                widget.currentRow.changed = false;
                                UtilitiesPropostaContrato.habilitarViewItensFormacao(widget);
                            } else {
                                widget.template = "widget/tree/grid.html";
                                widget.list();
                                Utilities.abreArvoreElementos(widget);
                            }
                        }
                    );
                }.bind(this));
            }.bind(this));
        }
    };
    
    /**
     * Realiza a validação  do campo "identificador". Não podem haver dois identificadores iguais em uma mesma composição.
     * 
     * @param {object} widget widget
     */

    this.validaIdentificador = function(widget){
        var widgetPropostaGrid = getWidgetPropostaGrid();
        var widgetItensProposta = getWidgetItensProposta();
        var nrformprecoservico = null;
        var nritemorcamento = null;
        var IdItemCalculo = widget.currentRow.IDITEMCALCULO;
        var nrCalcFormPrecServ = widget.currentRow.NRCALCFORMPRECSERV;
        /* === Ordenação === */
        
        if(widget.name === "editaItensFormPreco"){
            nritemorcamento    = widgetItensProposta.currentRow.NRITEMORCAMENTO;
            nrformprecoservico = widget.dataSource.data[0].NRFORMPRECOSERVICO;
        } else {
            nrformprecoservico = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
        }
        
        var params = {
            "NRFORMPRECOSERVICO": nrformprecoservico,
            "NRITEMORCAMENTO"   : nritemorcamento,
            "TABELA"            : (widget.dataSource.name === "product#/fct_calcformprecserv_widget") ? null : "FCT_CALCFORMPRECSERV_ITPRO",
            "IDITEMCALCULO"     : IdItemCalculo,
            "NRCALCFORMPRECSERV": nrCalcFormPrecServ
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/validaIdentificador', "Row", params).then(function(response){
            var identificador = response.dataset.IDENTIFICADOR.VALIDA;
            if (identificador != 0){
                widget.currentRow.NMELEMENTO = "",
                widget.currentRow.IDITEMCALCULO = null;
                ScreenService.showMessage('O identificador "'+IdItemCalculo+'" já está sendo utilizado para outro elemento.'); 
            }
            
        }.bind(this), 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });    
    };
    
    /**
     * Salva os itens selecionados pelo usuário com as novas formaçoes de preço
     * @params{Object} widget widget editar itens formação de preço
    **/
    this.utilizarItensAlteradosFormPrec = function(widget){
        var widgetItensFomacao = templateManager.container.getWidget("itemFormacaoPreco");
        var widgetItensprop    = templateManager.container.getWidget("wdgItensProposta");
        var widgetProposta     = templateManager.container.getWidget("wdgProposta");
        var rows               = widget.getCheckedRows();
        var itensSelecionados  = [];
        var nrformprecoservico = widgetItensFomacao.currentRow.NRFORMPRECOSERVICO;
        var servico            = null;
        
        if(widgetItensFomacao.currentRow.CDSERVORCA)
            servico = widgetItensFomacao.currentRow.CDSERVORCA;
        else 
            servico = widgetItensFomacao.currentRow.CDSERVORCAFORM;
        
        if(widget.dataSource.data.length !== rows.length)
            rows = UtilitiesPropostaContrato.buscaArvoreFormacaoCompleta(widget, rows);
        
        rows.forEach(function(row) {
            row.NRSEQORCAMENTO = widgetProposta.currentRow.NRSEQORCAMENTO;
            if(rows.indexOf(row._parent) == -1) {
                widgetItensFomacao.getField("NRCALCFORMPRECSERV").dataSource.addCheckedRows(row);
                itensSelecionados.push(row.NRCALCFORMPRECSERV);
            }
        });

        widgetItensFomacao.currentRow.NRCALCFORMPRECSERV = itensSelecionados;
        widget.dataSource.data                           = rows;
        
        this.salvaItemformacaopreco(widgetItensFomacao).then(function(response){
            var nrItensProp = null;
            if(response)
                nrItensProp = response;
            
            UtilitiesPropostaContrato.habilitaEdicaoWidgetItemFormacao(widgetItensFomacao, false);
            this.salvarAlteracoesCalculo(widget, false, nrItensProp).then(function(){
                widget.dataSource.clearCheckedRows();
                ScreenService.closePopup();
                
                ScreenService.closeSwipe().then(function(){
                    ScreenService.closeSwipe().then(function(){
                        widgetItensprop.activate();
                        widgetItensprop.reload();
                    });
                });
            }.bind(this))
            .catch(function(error){
                if(nrItensProp && nrItensProp.length > 0) {
                    var itens = [];
                    nrItensProp.forEach(function(item){
                        itens.push(item.NRITEMORCAMENTO);
                    });
                    
                    var params = {
                        "NRSEQORCAMENTO": widgetProposta.currentRow.NRSEQORCAMENTO,
                        "NRITEMORCAMENTO": itens
                    };
                    
                    General.callRestEngine("product#/fct_relacestruturaprop_widget/deletarItensFormacaoProp", "Row", params).then(function(){
                        widgetItensFomacao.getField("NRFORMPRECOSERVICO").dataSource.addCheckedRows(nrformprecoservico);
                        widgetItensFomacao.currentRow.NRFORMPRECOSERVICO = nrformprecoservico;
                        widgetItensFomacao.currentRow.CDSERVORCAFORM     = servico;
                        
                        ScreenService.closePopup();
                        ScreenService.showMessage("Não foi possivel utilizar o(s) item(ns) selecionados.<br/>Erro durante a inserção do(s) mesmo(s) na proposta.");
                    }.bind(this));
                }
            });
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        }); 
    };
    
    /**
     * Manda recalcular a arvore de formação
     * @params{Object} widget widget formação de preço
     * @params{int} nrformprecoservico nr da formação de preço
    **/
    this.recalculaFormacaoItens = function(widget, nrformprecoservico, nritemorcamento){
        var deferred = ZHPromise.defer();
        var params = {
            "NRFORMPRECOSERVICO" : nrformprecoservico,
            "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO",
            "NRITEMORCAMENTO"    :  nritemorcamento
        };
        
        General.callRestEngine("product#/fct_calcformprecserv_widget/recalcularArvore", "Row", params).then(function() {
            Utilities.abreArvoreElementos(widget).then(function(){
                ScreenService.showMessage("Processo realizado com sucesso!");
                deferred.resolve();
            });
        })
        .catch(function(error) {
            Utilities.showErrorMessage(error);
            deferred.reject(error);
        });
        
        return deferred.promise;
    };
    
    
    /**
     * Recalcula a árvore de elementos
     * @params{Object} widget widget editar itens de formação de preço
     * .parent substituido pela widget da grid principal
    **/
    this.recalcularArvore = function(widget) {
          var widgetItensProposta = getWidgetItensProposta();
        if(widget.name === "editaItensFormPreco"){
            this.recalculaFormacaoItens(widget, widget.dataSource.data[0].NRFORMPRECOSERVICO, widgetItensProposta.currentRow.NRITEMORCAMENTO).then(function(){
                this.recalculaVrItem(widget, null, null);
            }.bind(this));
        } else {
            var nrformprecoservico = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
            this.recalculaFormacaoItens(widget, nrformprecoservico, null);
        }
    };
    
    /**
     * salva as edições feitas nas formações auxiliares dos itens
     * @params{Object} widget widget editar itens formação na aba itens
    **/
    this.salvarEdicaoItFormacao = function(widget){
        var isNew = widget.currentRow.__is_new;
        var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
        if(!widget.currentRow.DSREFECALCULO)
            widget.currentRow.DSREFECALCULO = widget.currentRow.IDITEMCALCULO + " - " + widget.currentRow.NMELEMENTO;
        
        if(!widget.currentRow.NRRELACFORMACAOPROP)
            widget.currentRow.NRRELACFORMACAOPROP = 0; 
            
        UtilitiesPropostaContrato.calculaItem(widget, "FCT_CALCFORMPRECSERV_ITPRO", widget.currentRow.NRITEMORCAMENTO, null).then(function(){
            General.widgetSave(widget).then(function(response){
                if(response.dataset && response.dataset.columnsToReturn.length > 0){
                    widget.currentRow.NRSEQORCAMENTO     = response.dataset.columnsToReturn[0].NRSEQORCAMENTO;
                    widget.currentRow.NRCALCFORMPRECSERV = response.dataset.columnsToReturn[0].NRCALCFORMPRECSERV;
                    widget.currentRow.NRFORMPRECOSERVICO = response.dataset.columnsToReturn[0].NRFORMPRECOSERVICO;
                    widget.currentRow.NRITEMORCAMENTO    = response.dataset.columnsToReturn[0].NRITEMORCAMENTO;
                    widget.currentRow.NRORG              = response.dataset.columnsToReturn[0].NRORG;
                }
                if (tipoCalculo == 'S'){
                    var params = {
                        "NRCALCFORMPRECSERV" : widget.currentRow.NRCALCFORMPRECSERV,
                        "NRITEMORCAMENTO"    : widget.currentRow.NRITEMORCAMENTO,
                        "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO"
                    };
                    General.callRestEngine('product#/fct_calcformprecserv_widget/calcSomatorioFilhos', "Row", params);  
                }
                ScreenService.confirmMessage("Deseja recalcular a Formação de Preço?", 'question', function (success) {
                    if(!isNew)
                        ScreenService.closeSwipe();
                    widget.list();
                    this.recalcularArvore(widget);
                }.bind(this),
                function(){
                    this.recalculaVrItem(widget, null, null);
                }.bind(this));
                
            }.bind(this))
            .catch(function(error){
                console.error(error);
            });
        }.bind(this));
    };
    
    /**
     * Verifica se o item do orcamento faz parte de uma formação de preço 
     * auxiliar para habilita a widget de edição de formações auxiliares
     * @params{Object} widget widget item orcamento
    **/
    this.validaWidgetEditaFormAux = function(widget){
        //if(getWidgetProposta().currentRow.IDUTILIZACOMP === 'N'){
        if(getWidgetProposta().currentRow.QTITENSTRABSERV == 0){
            templateManager.container.getWidget("wdgAdicionais").enabled = true
            var params = [
                {
                    name    : "NRITEMORCAMENTO",
                    operator: "=",
                    value   : widget.currentRow.NRITEMORCAMENTO
                }
            ];
            
            General.callRestEngine("product#/crud/fct_calcformprecserv_itpro_widget", "FilterData", params).then(function(response){
                var wdgFormacao = templateManager.container.getWidget("editaItensFormPreco");
                var situacao = templateManager.container.getWidget("wdgProposta").currentRow.IDSITUACAOORCAMENTO;
                
                if(response.dataset && response.dataset.fct_calcformprecserv_itpro_widget.length > 0){
                    wdgFormacao.isVisible = true;
                    wdgFormacao.enabled   = true;
                    if(situacao === "ELABORACAO" || (idAlteraPropConc === "S" && situacao === "CONCLUIDO"))
                        wdgFormacao.actions.forEach(function(action){
                            action.isVisible = true;
                        });
                    else
                        wdgFormacao.actions.forEach(function(action){
                            if(action.name !== "VoltarWdgFormPrec")
                                action.isVisible = false;
                        });
                        
                } else {
                    wdgFormacao.isVisible = false;
                }
            });
        }
    };
    
    /**
     * Exclui um elemento da formçaõ de preço auxiliar e manda
     * editar o valor do item
     * @params{Object} widget widget editar itens formação de preço
    **/
    this.excluirFormacaoItens = function(widget){
        var dado = [widget.currentRow];
        var params = [
            {
                name: "NRCALCFORMPRECSERVPAI",
                operator: "=",
                value: widget.currentRow.NRCALCFORMPRECSERV
            },
            {
                name: "NRSEQORCAMENTO",
                operator: "=",
                value: widget.currentRow.NRSEQORCAMENTO
            },
            {
                name: "NRITEMORCAMENTO",
                operator: "=",
                value: widget.currentRow.NRITEMORCAMENTO
            },
            {
                name: "NRFORMPRECOSERVICO",
                operator: "=",
                value: widget.currentRow.NRFORMPRECOSERVICO
            }
        ]

        General.callRestEngine("product#/crud/fct_calcformprecserv_itpro_widget", "FilterData", params).then(function(response){
            var mensagem;
            if(response.dataset && response.dataset.fct_calcformprecserv_itpro_widget && response.dataset.fct_calcformprecserv_itpro_widget.length > 0)
                mensagem = "Apagar este elemento levará a exclusão dos elementos inferiores a ele.<br>Confirma exclusão?";
            else 
                mensagem = "Delete Register?";

            ScreenService.confirmMessage(ScreenService.i18n(mensagem), "question", function(){
                params = {
                    "NRCALCFORMPRECSERV": widget.currentRow.NRCALCFORMPRECSERV,
                    "NRSEQORCAMENTO"    : widget.currentRow.NRSEQORCAMENTO,
                    "NRITEMORCAMENTO"   : widget.currentRow.NRITEMORCAMENTO,
                    "NRFORMPRECOSERVICO": widget.currentRow.NRFORMPRECOSERVICO
                };

                General.callRestEngine("product#/fct_relacestruturaprop_widget/deletarArvoreFormPrecoItPro", "Row", params).then(function(){
                    ScreenService.confirmMessage(ScreenService.i18n("Deseja recalcular a Formação de Preço?"), 'question', function (success) {
                        this.recalcularArvore(widget);
                    }.bind(this));
                    ScreenService.closeSwipe().then(function () {
                        widget.activate();
                        this.recalculaVrItem(widget, dado, "DELETE");
                    }.bind(this));
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }.bind(this));
        }.bind(this));
    };
    
    /**
     * Edita o valor do item quando a formação de preço 
     * é alterada.
     * @params{Object} widget widget editar itens formação de preco
    **/
    this.recalculaVrItem = function(widget, elementos, acao){
        var widgetItensProposta = getWidgetItensProposta();
        var valor  = 0;

        if(!elementos){
            elementos = widget.dataSource.data.filter(function(dado){
                return dado.NRCALCFORMPRECSERVPAI === null;
            });
        }

        elementos.forEach(function(elemento){
            valor = valor + elemento.VRTOTAL;
        });

        if(acao === "DELETE")
            valor = widgetItensProposta.currentRow.VRUNITITEMORCA - elementos[0].VRTOTAL;

        if(widgetItensProposta.currentRow.VRUNITITEMORCA !== valor){
            widgetItensProposta.currentRow.VRUNITITEMORCA = valor;
            this.calcularValorTotalItem(widgetItensProposta);
            this.salvarItens(widgetItensProposta).then(function(){
                widget.activate();
                Utilities.abreArvoreElementos(widget);
            }).catch(function(error){
                ScreenService.showMessage("Não foi possivel atualizar o valor do item da proposta.");
            });
        }
    };
    
    /**
     * Seta o número do item gerado durante a inserção do item na proposta
     * @params{Object} widget widget editar formaçao de preço
     * @params{Array} nrItensProp array de nr de itens gerados
    **/
    this.setNrItemFormacaoAux = function(widget, nrItensProp){
        var rows = [];
        if(nrItensProp === null) {
            widget.dataSource.data.forEach(function(dado){ 
                dado.NRITEMORCAMENTO = null;
            });
        } else {
            if(nrItensProp.length === 1) {
                widget.dataSource.data.forEach(function(dado){
                    dado.NRITEMORCAMENTO = nrItensProp[0].NRITEMORCAMENTO;
                });
            } else {
                nrItensProp.forEach(function(item){
                    widget.dataSource.data.filter(function(data){
                        if(data.IDELEMENTO === item.IDELEMENTO) {
                            data.NRITEMORCAMENTO = item.NRITEMORCAMENTO;
                            rows.push(data);
                        }
                    });
                });
                
                widget.dataSource.data.forEach(function(data){ 
                    rows.filter(function(row){ 
                        if(data._parent && row === data._parent) {
                            if (rows.indexOf(data) === -1) {
                                data.NRITEMORCAMENTO = data._parent.NRITEMORCAMENTO;
                                rows.push(data);
                            }
                        }
                    });
                });
            }
        }
    };
    
    /**
     * Salva as alterações feitas nos elementos do cálculo
     * @params{Object} widget widget editar itens formação de preço
    **/
    this.salvarAlteracoesCalculo = function(widget, isNew, nrItensProp) {
        var deferred       = ZHPromise.defer();
        var nrseqorcamento = getWidgetProposta().currentRow.NRSEQORCAMENTO;
        if(isNew){
            var params = {
                "NRFORMPRECOSERVICO": templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0],
                "NRSEQORCAMENTO"    : nrseqorcamento,
                "NRITEMORCAMENTO"   : null
            };
            
            General.callRestEngine("product#/fct_relacestruturaprop_widget/insereAlteracaoFormPrec", "Row", params).then(function(){
                deferred.resolve();
            }.bind(this))
            .catch(function(error){
                deferred.reject();
                Utilities.showErrorMessage(error);
            });
        } else {
            var paramsEdit = [];
            try {
                this.setNrItemFormacaoAux(widget, nrItensProp);
                nrItensProp.forEach(function(item){
                    var idItemCalculo = [];
                    var dadosItem = widget.dataSource.data.filter(function(dado){
                        return dado.NRITEMORCAMENTO === item.NRITEMORCAMENTO;
                    }); 
                    
                    dadosItem.forEach(function(dadoItem){
                        idItemCalculo.push(dadoItem.IDITEMCALCULO);
                    });
                    
                    var param = {
                        "NRFORMPRECOSERVICO": widget.dataSource.data[0].NRFORMPRECOSERVICO,
                        "NRSEQORCAMENTO"    : nrseqorcamento,
                        "IDITEMCALCULO"     : idItemCalculo,
                        "NRITEMORCAMENTO"   : item.NRITEMORCAMENTO
                    };
                    
                    paramsEdit.push(param);
                });
            } catch(error){
                deferred.reject();
            }
            
            General.callRestEngine("product#/fct_relacestruturaprop_widget/editaItensFormacaoAux", "DataSet", paramsEdit).then(function() {
                deferred.resolve();
            }).catch(function(error){
                deferred.reject();    
            });
        }
        
        return deferred.promise;
    };
    
    
    /**
     * Verifica se a formação de preço selecionada está habilitada para 
     * permitir a inserção um serviço
     * @params{Object} widget widget item formação de preço
    **/
    this.verificaAcessibilidadeFieldServ = function(widget){
        if(widget.currentRow.NRFORMPRECOSERVICO && widget.currentRow.NRFORMPRECOSERVICO.length === 1){
            widget.getField("NMSERVORCA").readOnly = false;
        } else {
            widget.getField("NMSERVORCA").readOnly = true;
            widget.currentRow.NMSERVORCA = "";
            widget.currentRow.CDSERVORCA = "";
        } 
    };

    /**
     * A função "fecharWdgFormacaoPrec" fecha a tela "Itens de Formação de Preço"
     */
    this.fecharWdgFormacaoPrec = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.isVisible = false;

            /*Limpa os campos*/
            this.limpaCamposFormPrec(widget);

            templateManager.container.getWidget("wdgItensProposta").activate();

        }.bind(this));
    };


    /**
     * A função "filtraItFormPreco" filtra o campo "Itens da Formação de Preço" 
     */

    this.filtraItFormPreco = function(widget) {

        var formPreco = widget.currentRow.NRFORMPRECOSERVICO;

        widget.getField('NRCALCFORMPRECSERV').dataSourceFilter[0].value = formPreco;
        widget.getField('NRCALCFORMPRECSERV').reload();
    };


    /**
     * A Função "filtraFormPreco" filtra o campo "Formação de Preço" 
     */

    this.filtraFormPreco = function(widget) {

        var nrparcnegocio = templateManager.container.getWidget("wdgProposta").currentRow.NRPARCNEGOCIO;

        widget.getField('NRFORMPRECOSERVICO').dataSourceFilter[0].value = nrparcnegocio;
        widget.getField('NRFORMPRECOSERVICO').reload();
    };


    /**
     *  A função "verificaServicosRepetidos" verifica entre as formações que foram marcadas, 
     *  se exitem serviços repetidos.
     */

    this.verificaServicosRepetidos = function(widget) {

        var servRepetidos = [];
        var servicos = widget.currentRow.CDSERVORCAFORM;
        var servAnte = 0;

        servicos.sort();

        servicos.forEach(function(servAux) {

            if (servAux == servAnte) {
                if (!servRepetidos.includes(servAux)) {
                    servRepetidos.push(servAux);
                }
            }

            servAnte = servAux;
        });

        return servRepetidos;
    };


    /**
     *  A função "salvaItemformacaopreco" centraliza o salvamento dos 
     *  itens de formação de preço na proposta.
     */

    this.salvaItemformacaopreco = function(widget) {
        if (widget.isValid()) {
            var servRepetidos = this.verificaServicosRepetidos(widget);
            if (servRepetidos.length != 0) {
                ScreenService.showCustomDialog("Foram selecionados diferentes formações de preço que possuem o mesmo serviço. O que você deseja fazer com os serviços repetidos?", 'CONFIRMATION', null, 
                [
                    {
                        "label": "Somar os Valores",
                        "code": function() {
                            this.validarDadosRateioFormacaoPreco(widget, true);
                        }.bind(this)
                    },
                    {
                        "label": "Incluir Separadamente",
                        "code": function() {
                            this.validarDadosRateioFormacaoPreco(widget, false);
                        }.bind(this)
                    },
                    {
                        "label": "Selecionar Novamente",
                        "code": function() {}
                    }
                ]);
            } else {
                var deferred = ZHPromise.defer();
                this.validarDadosRateioFormacaoPreco(widget, false).then(function(response){
                    deferred.resolve(response);
                }).catch(function(error) {
                    deferred.reject(error);
                });
                
                return deferred.promise;
            }
        }
    };

    /**
     *  A função "insereItemFormPrec" insere os serviços selecionados.
     */
    this.insereItemFormPrec = function(widget, soma) {
        var deferred    = ZHPromise.defer();
        var rowProposta = templateManager.container.getWidget("wdgProposta").currentRow;
        var servicos    = widget.getField("NRCALCFORMPRECSERV").dataSource.getCheckedRows();
        
        if(widget.currentRow.NMSERVORCA){
            // pega o serviço da field serviço da tela e substitui o serviço da formação de preço
            servicos.forEach(function(servico){
                servico.CDSERVORCA = widget.currentRow.CDSERVORCA;
                servico.NMSERVORCA = widget.currentRow.NMSERVORCA;
            });
        }
        
        var params = {
            "SOMA"           : soma,
            "SERVICOS"       : servicos,
            "PROPOSTA"       : rowProposta.NRSEQORCAMENTO,
            "NRTIPOORCAMENTO": widget.currentRow.NRTIPOORCAMENTO
        };

        General.callRestEngine('product#/fct_relacestruturaprop_widget/insereItemFormPrec', "Row", params).then(function(response) {
            this.limpaCamposFormPrec(widget);

            var widgetAtiva     = templateManager.container.getActiveWidget();
            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            
            if(widgetAtiva.name === "itemFormacaoPreco"){
                ScreenService.closeSwipe();
                templateManager.container.getWidget("wdgItensProposta").activate();
                templateManager.container.getWidget("wdgItensProposta").reload();
            }
                
            this.atualizarValorProposta(wdgPropostaGrid);
            var itens = response.dataset.fct_relacestruturaprop_widget;
            if(rowProposta.IDSITUACAOORCAMENTO !== "ELABORACAO") {
                itens.forEach(function(item){
                    this.insereItensAcaoPreventiva(rowProposta.NRSEQORCAMENTO, item.NRITEMORCAMENTO, 1, rowProposta.IDSITUACAOORCAMENTO);
                }.bind(this));
            }
            
            deferred.resolve(itens);
        }.bind(this),
        function(error) {
            Utilities.showErrorMessage(error);
            deferred.reject();
        }.bind(this));

        return deferred.promise;
    };

    /**
     *  A função "controlaVisibItFormPrec" controla a visibilidade da
     *  action "Itens de Formação de preço".
     */
    this.controlaVisibItFormPrec = function(widget) {
        var situacao = templateManager.container.getWidget("wdgProposta").currentRow.IDSITUACAOORCAMENTO;
        //var utilizaComposicao = templateManager.container.getWidget("wdgProposta").currentRow.IDUTILIZACOMP;
        var utilizaComposicao = (templateManager.container.getWidget("wdgProposta").currentRow.QTITENSTRABSERV > 0) ? "S" : "N";

        if ((situacao == 'ELABORACAO') && (utilizaComposicao == 'N')) {
            widget.getAction("formacaoPreco").isVisible = true;
        }
        else {
            widget.getAction("formacaoPreco").isVisible = false;
        }
    };

    /**
     *  A função "limpaCamposFormPrec" Limpa os campos após o fechamento do swipe
     */
    this.limpaCamposFormPrec = function(widget) {
        widget.currentRow.NRFORMPRECOSERVICO = null;
        widget.currentRow.NMFORMPRECOSERVICO = null;
        widget.currentRow.CDSERVORCAFORM     = null;

        widget.currentRow.NRCALCFORMPRECSERV = null;
        widget.currentRow.CDSERVORCA         = null;
        widget.currentRow.NMSERVORCA         = null;
        widget.currentRow.NRESTRUTURAM       = null;
        widget.currentRow.CDHABILITSERV      = null;
        widget.currentRow.NMELEMENTO         = null;
        widget.currentRow.VRUNITARIO         = null;
        widget.currentRow.QTCALCULO          = null;
        widget.currentRow.IDUSAMEDIDA        = null;
        widget.currentRow.CDTPMEDIDASERV     = null;
        widget.currentRow.QTMEDIDACALC       = null;
        widget.currentRow.VRTOTAl            = null;
    };

    /* FUNÇÕES DA WIDGET LOCAL DE ATENDIMENTO */

    this.validaCnpj = function(widget) {

        var cnpj = widget.currentRow.NRINSCRICAOPARC;

        var testaCnpj = General.validateCNPJ(cnpj);

        if (testaCnpj) {

            widget.currentRow.CDTIPOINSCRICAO = 'CNPJ';
        }
        else {
            widget.currentRow.CDTIPOINSCRICAO = 'LIVRE';

        }
    };

    /**
     * Realiza à rotina de inserção de todas as informações em suas respectivas tabelas.
     * 
     * @param {object} widget widget 
     */
    this.inserirNvLocalAtend = function(widget) {

        this.validaCnpj(widget);
        var params = {
            "NMESTRUTURAM": widget.currentRow.NMESTRUTURAMNV,
            "NRESTRUTURAMSUP": widget.currentRow.NRESTRUTURAMSUP,
            "NRTIPOESTRUTURA": widget.currentRow.NRTIPOESTRUTURA,
            "NRINSCRICAOPARC": widget.currentRow.NRINSCRICAOPARC.replace(/[\D]+/g, ''),
            "CDCENTCUST": widget.currentRow.CDCENTCUST,
            "CDTIPOINSCRICAO": widget.currentRow.CDTIPOINSCRICAO,
            "DSENDERECO": widget.currentRow.DSENDERECO,
            "NRIMOVELENDERECO": widget.currentRow.NRIMOVELENDERECO,
            "DSCOMPLEENDERECO": widget.currentRow.DSCOMPLEENDERECO,
            "CDPAIS": widget.currentRow.CDPAIS,
            "SGESTADO": widget.currentRow.SGESTADO,
            "CDMUNICIPIO": widget.currentRow.CDMUNICIPIO,
            "CDBAIRRO": widget.currentRow.CDBAIRRO,
            "NMBAIRROENDERECO": widget.currentRow.NMBAIRROENDERECO,
            "NRCEPENDERECO": widget.currentRow.NRCEPENDERECO,
            "DSREFERENCIAENDE": widget.currentRow.DSREFERENCIAENDE
        };

        General.callRestEngine('product#/estruturam_widget/inserirTudo', "Row", params).then(function(response) {

            widget.currentRow.NRESTRUTURAM = response.dataset.estruturam_widget[0];
            General.widgetSave(widget).then(function() {
                ScreenService.closeSwipe().then(function() {
                    widget.currentRow.NOVOLOCALATEND = "N";
                    this.habilitaNovoLocalAtend(widget);
                    widget.reload();
                }.bind(this));
            }.bind(this));


        }.bind(this)
        , function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Salva o novo registro da página "Local de Atendimento"
     * 
     * @param {object} widget widget de local de atendimento 
     * para salvar o novo registro.
     */
    this.salvarLocalAtend = function(widget) {

        if (widget.isValid()) {
            if (widget.currentRow.NOVOLOCALATEND === 'S') {
                this.inserirNvLocalAtend(widget);
            }
            else {
                Utilities.inserirMultiplos(widget, "NRESTRUTURAM").then(function() {
                        ScreenService.closeSwipe().then(function() {
                            widget.currentRow.NOVOLOCALATEND = "N";
                            this.habilitaNovoLocalAtend(widget);
                            widget.reload();
                        }.bind(this));
                }.bind(this),
                function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
            }

        }
    };

    this.deletarLocalAtend = function(widget) {

        var possuiFilhos = false;
        if (widget.dataSource.data) {
            widget.dataSource.data.forEach(function(dado) {
                if (dado.NRESTRUTURAMSUP != null && dado.NRESTRUTURAMSUP === widget.currentRow.NRESTRUTURAM) {
                    possuiFilhos = true;
                }
            });
        }

        if (possuiFilhos) {
            this.deletarPaieFilhos(widget);
        }else {
            General.deleteCrud(widget).then(function() {
                    widget.reload();
                },
                function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
        }

    };

    /**
     * Exclui os local de atandimento e os filhos.
     * 
     * @param {object} widget widget
     */

    this.deletarPaieFilhos = function(widget) {

        ScreenService.confirmMessage("A exclusão deste local de atendimento também irá excluir os locais de atendimento inferiores a ele. Confirma exclusão?", "question", function() {

            var params = {
                "NRESTRUTURAM": widget.currentRow.NRESTRUTURAM,
                "NRSEQORCAMENTO": widget.currentRow.NRSEQORCAMENTO
            };

            General.callRestEngine('product#/fct_relacestruturaprop_widget/deletarPaieFilhos', "Row", params).then(function() {

                    ScreenService.showMessage('Exclusão realizada com sucesso.');
                    ScreenService.closeSwipe();
                    widget.reload();

                },
                function(error) {
                    console.log(error);
                    ScreenService.showMessage(error.data.error);
                });

        });
    };

    /* Habilita os campos para a inserção de um novo local de atendimento
     * @params{Object} widget local de atendimento
     */
    this.habilitaNovoLocalAtend = function(widgetLocalAtend) {
        var fields = ["NMESTRUTURAMNV", "NRINSCRICAOPARC", "NRTIPOESTRUTURA", "NMTIPOESTRUTURA", "NRESTRUTURAMSUP", "NMESTRUTURAMSUP", "NMCENTCUST", "NRCEPENDERECO", "NMPAIS", "SGESTADO", "NMESTADO", "NMMUNICIPIO", "DSENDERECO", "NRIMOVELENDERECO", "NMBAIRROENDERECO", "DSCOMPLEENDERECO", "DSREFERENCIAENDE"];

        if (widgetLocalAtend.currentRow.NOVOLOCALATEND === 'S') {
            widgetLocalAtend.getField('NRESTRUTURAM').readOnly = true;
            widgetLocalAtend.getField('NRESTRUTURAM').validations = {};
            fields.forEach(function(field) {
                widgetLocalAtend.getField(field).readOnly = false;
            });
            widgetLocalAtend.getField("NMESTRUTURAMNV").validations = {
                "required": true
            };
            widgetLocalAtend.getField("NMTIPOESTRUTURA").validations = {
                "required": true
            };
            this.recuperaEnderecoParcNegocio(widgetLocalAtend);
        }
        else {
            widgetLocalAtend.getField('NRESTRUTURAM').validations = {
                "required": true
            };
            widgetLocalAtend.getField("NMESTRUTURAMNV").validations = {};
            widgetLocalAtend.getField("NMTIPOESTRUTURA").validations = {};
            fields.forEach(function(field) {
                widgetLocalAtend.getField(field).readOnly = true;
                widgetLocalAtend.currentRow[field.name] = "";
            });

            widgetLocalAtend.getField('NRESTRUTURAM').readOnly = false;

        }
    };

    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o País.
     * 
     * @param {object} widget
     */
    this.controlarCamposEnderecoPorPais = function(widget) {
        widget.currentRow.SGESTADO = "";
        widget.currentRow.NMESTADO = "";
        widget.currentRow.CDMUNICIPIO = "";
        widget.currentRow.NMMUNICIPIO = "";
        widget.currentRow.CDBAIRRO = "";
        widget.currentRow.NMBAIRROENDERECO = "";

        var cdpais = widget.currentRow.CDPAIS;

        var campoEstado = widget.getField("NMESTADO");
        var campoMunicipio = widget.getField("NMMUNICIPIO");
        var campoBairro = widget.getField("NMBAIRROENDERECO");

        campoEstado.dataSourceFilter[0].value = cdpais;
        campoEstado.reload();

        campoMunicipio.dataSourceFilter[0].value = cdpais;

        campoBairro.dataSourceFilter[0].value = cdpais;
    };


    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o Estado.
     * 
     * @param {object} widget
     */
    this.controlarCamposEnderecoPorEstado = function(widget) {
        widget.currentRow.CDMUNICIPIO = "";
        widget.currentRow.NMMUNICIPIO = "";
        widget.currentRow.CDBAIRRO = "";
        widget.currentRow.NMBAIRROENDERECO = "";

        var sgestado = widget.currentRow.SGESTADO;

        var campoMunicipio = widget.getField("NMMUNICIPIO");
        var campoBairro = widget.getField("NMBAIRROENDERECO");

        campoMunicipio.dataSourceFilter[1].value = sgestado;
        campoMunicipio.reload();

        campoBairro.dataSourceFilter[1].value = sgestado;
    };

    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o Município.
     * 
     * @param {object} widget
     */
    this.controlarCamposEnderecoPorMunicipio = function(widget) {
        widget.currentRow.CDBAIRRO = "";
        widget.currentRow.NMBAIRROENDERECO = "";

        var cdmunicipio = widget.currentRow.CDMUNICIPIO;

        var campoBairro = widget.getField("NMBAIRROENDERECO");

        campoBairro.dataSourceFilter[2].value = cdmunicipio;
        campoBairro.reload();
    };


    this.recuperaEnderecoParcNegocio = function(wdgNvLocalAtend) {
        var widgetLancProposta = templateManager.container.getWidget("wdgProposta");
        var paramsFilter = [
            {
                name: "NRPARCNEGOCIO",
                operator: "=",
                value: widgetLancProposta.currentRow.NRPARCNEGOCIO
            }
        ];

        General.callRestEngine("product#/crud/enderecoparc", "FilterData", paramsFilter).then(function(success) {
            if (success.dataset && success.dataset.enderecoparc.length > 0) {
                success.dataset.enderecoparc.every(function(dado) {
                    if (dado.CDTIPOENDERECO === "PRINCIPAL") {
                        wdgNvLocalAtend.currentRow.NRCEPENDERECO = dado.NRCEPENDERECO;
                        this.buscarEndereco(wdgNvLocalAtend);
                    }
                }.bind(this));
                
                if(!wdgNvLocalAtend.currentRow.NRCEPENDERECO){
                    wdgNvLocalAtend.currentRow.NRCEPENDERECO = success.dataset.enderecoparc[0].NRCEPENDERECO;
                    this.buscarEndereco(wdgNvLocalAtend);
                }
            }
        }.bind(this));
    };
    
    /* Verifica a action acionada */
    this.verificaAction = function(widget, action){
        if(action === "AVANCAR"){
            this.avancarComposicao(widget);
        } else {
            this.salvarProposta(widget);
        }
    };

    /**
     * Busca o parametro de medida fixada da arvore de serviço
     * @params{Object} widget - widget itens
    **/
    this.getUsaMedidaFixada = function(widget, tpElemento, nrseqorcamento){
        var deferred = ZHPromise.defer();
        if(!tpElemento || (servIdMedidaFixa.length > 0 && servIdMedidaFixa[0]["NRSEQORCAMENTO"] === nrseqorcamento)){
            deferred.resolve();
        } else {
            var arrIdElePai = [];
            var cdservorca  = [];
            widget.dataSource.data.forEach(function(item){
                if(item.IDELEMENTO && !arrIdElePai.includes(item.IDELEMENTO))
                    arrIdElePai.push(item.IDELEMENTO);
                if(item.CDSERVORCA && !cdservorca.includes(item.CDSERVORCA))
                    cdservorca.push(item.CDSERVORCA);
            });
            var params = {
                "IDARVORE"  : arrIdElePai,
                "CDSERVORCA": cdservorca
            };
            General.callRestEngine("product#/fct_orcamento_widget/getIdUsaMedidaFixada", "Row", params).then(function(response){
                if(response.dataset && response.dataset.getIdUsaMedidaFixada)
                    servIdMedidaFixa[0] = {"NRSEQORCAMENTO": nrseqorcamento,
                                           "SERVMEDIDAFIXA": response.dataset.getIdUsaMedidaFixada};
                deferred.resolve();
            }).catch(function(error){
                deferred.reject();
            });
        }
        
        return deferred.promise;
    };
    /**
     * Validação para que o swipe do view não seja aberto quando for selecionada
     * uma linha correspondente a uma medida variável, e não um item de proposta.
     */
    this.validarAbreSwipeItens = function(args, tipe) {
        //if(getWidgetProposta().currentRow.IDUTILIZACOMP === 'S'){
        if(getWidgetProposta().currentRow.QTITENSTRABSERV > 0) {
            var nrseqorcamento = getWidgetProposta().currentRow.NRSEQORCAMENTO;
            var idMedidaFixa   = 'N';
            var widget         = (tipe === "W") ? args : args.owner;
            var isVisible      = widget.currentRow.IDTIPO === "M";
            var qtMdVisivel    = (widget.currentRow.IDTIPO === "M" && idMedidaFixa === 'N' && widget.currentRow.IDMULTIPVARIACAO === "S");

            widget.getField("VRUNITITEMORCA").help = null;
            this.getUsaMedidaFixada(widget, isVisible, nrseqorcamento).then(function(){
                if(widget.currentRow.IDTIPO === "M"){
                    servIdMedidaFixa[0]["SERVMEDIDAFIXA"].forEach(function(cdservorca){
                        if(cdservorca.CDSERVORCARELAC === widget.currentRow.CDSERVORCA)
                            idMedidaFixa = cdservorca.IDMEDIDAFIXA;
                    });
                }
                
                widget.getField("QTMEDIDAORCA").isVisible = isVisible;
                widget.getField("QTMEDIDAORCA").readOnly  = qtMdVisivel;
                this.alterarVisibilidadeCamposItens(widget, idMedidaFixa);
                
                if(widget.getField("VRUNITITEMORCATBLPRECO")) {
                    widget.getField("VRUNITITEMORCATBLPRECO").showOnView = isVisible;
                    widget.getField("VRUNITITEMORCATBLPRECO").showOnForm = isVisible;
                }
                
                widget.getAction("edit").isVisible = (widget.getAction("edit").isVisible && widget.currentRow.IDTIPO !== "A");
            }.bind(this));
        }
    };
    /**
     * Altera a visibilidade dos campos da tela de itens
     * @params{Object} widget - widget itens (inserindo ou editando)
    **/
    this.alterarVisibilidadeCamposItens = function(widget, idMedidaFixa){
        var isVisible      = widget.currentRow.IDTIPO === "M";
        var isVisibleAgrup = (widget.currentRow.IDTIPO !== "A" && widget.currentRow.IDTIPO !== "M");
        
        widget.getField("IDMULTMEDVRTOT").isVisible     = isVisible;
        widget.getField("NMDOCITEMORCAMENTO").isVisible = isVisibleAgrup;
        widget.getField("IDIMPRESSAOORCA").isVisible    = isVisibleAgrup;
        widget.getField("TXFORMULAITORCA").isVisible    = isVisibleAgrup;
        widget.getField("NMTIPOORCAMENTO").isVisible    = isVisibleAgrup;
        widget.getField("VRUNITITEMORCA").showOnView    = isVisible;
        widget.getField("VRUNITITEMORCA").showOnForm    = isVisible;
        widget.getField("DSTPMEDIDASERV").showOnView    = isVisible; 
        widget.getField("DSTPMEDIDASERV").showOnForm    = isVisible;
        
        if(widget.getField("VRUNITITEMORCATBLPRECO")) {
            widget.getField("VRUNITITEMORCATBLPRECO").showOnView = isVisible;
            widget.getField("VRUNITITEMORCATBLPRECO").showOnForm = isVisible;
        }
        
        /*widget.getField("IDTPDESCORCA").isVisible          = isVisible; 
        widget.getField("VRITEMSEMDESC").isVisible         = isVisible;
        widget.getField("NMGRUPOFATURA").isVisible         = isVisibleAgrup;
        widget.getField("DTINICFATORC").isVisible          = isVisibleAgrup;
        widget.getField("DSMODPGTO").isVisible             = isVisibleAgrup;
        widget.getField("IDFATURAORCA").isVisible          = isVisibleAgrup;
        widget.getField("QTPARCELASFATU").isVisible        = isVisibleAgrup;
        widget.getField("NRDIAVENCPARCELASFATU").isVisible = isVisibleAgrup;
        
        
        widget.getField("VRPERCDESCORCA").showOnView = isVisible;
        widget.getField("VRPERCDESCORCA").showOnForm = isVisible;
        widget.getField("VRDESCORCA").showOnView = isVisible;
        widget.getField("VRDESCORCA").showOnForm = isVisible;

        widget.getField("VRPERCDESCORCA").readOnly  = (!widget.currentRow.IDTPDESCORCA || widget.currentRow.IDTPDESCORCA === 'N');
        this.controlarCampoValorDoAjuste(widget);*/
    };
    
    /* ================ FUNÇÕES DA WIDGET EVOLUIR CRM ================ */
    
    /**
     * Abre a widget de Evoluir CRM.
     */
    this.abrirWdgEvoluirCRM = function(rowProposta) {
        ScreenService.closePopup();
        
        var widgetEvoluirCRM = templateManager.container.getWidget("wgdEvoluirCRM");

        widgetEvoluirCRM.isVisible = true;
        
        ScreenService.openSwipe(widgetEvoluirCRM).then(function() {
            widgetEvoluirCRM.edit();
            widgetEvoluirCRM.newRow();
            this.setDadosWdgEvoluirCRM(rowProposta, widgetEvoluirCRM);
        }.bind(this));
    };
    
    /**
    * preenche a widget evolução CRM com os dados necessários
    * @params{Object} widget evoluir CRM
    **/
    this.setDadosWdgEvoluirCRM = function(rowProposta, widgetEvoluirCRM){
        widgetEvoluirCRM.currentRow.DTEVOLUCAO  = moment().format('DD/MM/YYYY HH:mm');
        widgetEvoluirCRM.currentRow.DTREALFECHA = moment().format('DD/MM/YYYY HH:mm');
        
        widgetEvoluirCRM.currentRow.NMOPERADOR = nomeOperador;
        widgetEvoluirCRM.currentRow.CDOPEREVOL = opLogado;

        var fieldInterlocutor = widgetEvoluirCRM.getField("NMINTERLOCUTOR");
        fieldInterlocutor.dataSourceFilter[0].value = rowProposta.NRPARCNEGOCIO;
        fieldInterlocutor.reload();
        
        
        if(tpOperador === null)
            tpOperador = "E";

        widgetEvoluirCRM.getField("NMPARCRESP").dataSourceFilter[0].value = tpOperador;
        widgetEvoluirCRM.getField("NMPARCRESP").reload();
        
        widgetEvoluirCRM.getField("NMOPERADORREAL").dataSourceFilter[0].value = tpOperador;
        widgetEvoluirCRM.getField("NMOPERADORREAL").reload();
        
        widgetEvoluirCRM.currentRow.VRPREVOPORT       = rowProposta.VRORCAMENTO;
        widgetEvoluirCRM.currentRow.VROPORT           = rowProposta.VRORCAMENTO;
        widgetEvoluirCRM.currentRow.NRSEQOPORT        = rowProposta.NRSEQOPORT;
        widgetEvoluirCRM.currentRow.NRORG             = rowProposta.NRORG;
        widgetEvoluirCRM.currentRow.CDOPERADORLOGADO  = opLogado;
        widgetEvoluirCRM.currentRow.NRSEQCONTATO      = rowProposta.NRSEQCONTATO;
        widgetEvoluirCRM.currentRow.NRPARCNEGOCIO     = rowProposta.NRPARCNEGOCIO;
        widgetEvoluirCRM.currentRow.NRORCAMENTOMODELO = rowProposta.NRORCAMENTOMODELO;
        widgetEvoluirCRM.currentRow.NRORCAMENTOMODELO = rowProposta.NRORCAMENTOMODELO;

    };
    
    /**
     * Verifica se há proxima tarefa
     * @params{Object} widget Evoluir CRM
    **/
    this.verificaProxTarefa = function(widgetEvoluirCRM){
        if(widgetEvoluirCRM.currentRow.IDGERPROXATIV === "S"){
            this.habilitaProxTarefa(widgetEvoluirCRM, false);
            widgetEvoluirCRM.getField("DTPLANFILACONTA").validations = { "required": true };
        } else {
            this.habilitaProxTarefa(widgetEvoluirCRM, true);
            widgetEvoluirCRM.getField("DTPLANFILACONTA").validations = {};
        }    
    };
    
    /**
     * Habilita/Desabilita o fieldGroup de próxima tarefa
     * @params{Object} widget
     * @params{boolean} permissão
    **/
    this.habilitaProxTarefa = function(widget, isLocked) {

        var dataProxtf = widget.getField("DTPLANFILACONTA");
        var operadorProxTf = widget.getField("NMOPERADORREAL");
        var descricaoProxtf = widget.getField("DSFILACONTATO");

        dataProxtf.readOnly = isLocked;
        operadorProxTf.readOnly = isLocked;
        descricaoProxtf.readOnly = isLocked;
        
        if(isLocked){
            widget.currentRow.DTPLANFILACONTA = "";
            widget.currentRow.NMOPERADORREAL  = "";
            widget.currentRow.DSFILACONTATO   = "";
        }

    };
    
    /**
     * Verifica se há fechamento na fase de nedociação
     * @params{Object} widget Evoluir CRM
    **/
    this.verificaFechamento = function(widgetEvoluirCRM){
        if(widgetEvoluirCRM.currentRow.IDPERMITEFECHA === "S"){
            this.habilitaFechamento(widgetEvoluirCRM, false);
        } else {
            this.habilitaFechamento(widgetEvoluirCRM, true);
        }    
    };
    
    /**
     * Habilita/Desabilita o fieldGroup de fechamento
     * @params{Object} widget
     * @params{boolean} permissão
    **/
    this.habilitaFechamento = function(widget, isLocked) {

        var resultadoFecha      = widget.getField("DESCRESULEVOL");
        var operadorFecha       = widget.getField("NMRAZSOCCONC");
        var valorOpFecha        = widget.getField("VROPORT");
        var dataFecha           = widget.getField("DTREALFECHA");

        resultadoFecha.readOnly = isLocked;
        operadorFecha.readOnly  = isLocked;
        valorOpFecha.readOnly   = isLocked;
        dataFecha.readOnly      = isLocked;
        
        if(isLocked){
            widget.currentRow.DESCRESULEVOL = "";
            widget.currentRow.NMRAZSOCCONC  = "";
            widget.currentRow.VROPORT       = "";
            widget.currentRow.DTREALFECHA   = "";
            
            resultadoFecha.validations = {};
            valorOpFecha.validations   = {};
        } else {
            widget.currentRow.DTREALFECHA = moment().format('DD/MM/YYYY HH:mm');
            resultadoFecha.validations = { "required": true };
            valorOpFecha.validations   = { "required": true };
        }
    };
    
    /**
     * Fecha a tela de Evoluir CRM
     * 
     * @param {object} widget
     */
    this.fecharWdgEvoluirCRM = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.isVisible = false;
        });
    };
    
    /**
    * Salvar evolução de oportunidade CRM
    * @params{Object} widget evoluir CRM
    **/
    this.salvarEvolucaoOportunidade = function(widget){
        if(widget.isValid()){
            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            General.callRestEngine("product#/enviarElementIntegracaoCRM", "Row", widget.currentRow).then(function(response){
                ScreenService.showMessage("Evolução gerada com sucesso.");
                ScreenService.closeSwipe();
                wdgPropostaGrid.reload();
            }.bind(this))
            .catch(function(error){
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        } else {
           ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } 
    };
    
    /** Valida o campo de código da proposta
     * @params{Object} widget 
    **/
    this.validaCodPropostas = function(widget, action){
        if (widget.isValid()) {
            if (widget.currentRow.NRORCAMENTO) {
                var rota = "product#/crud/fct_orcamento";
                Utilities.validarCamposDuplicados(widget, ["NRSEQORCAMENTO"], ["NRORG", "NRORCAMENTO"], rota).then(function(response){
                    if (response) {
                        this.verificaAction(widget, action);
                    } else {
                        ScreenService.showMessage("Operação bloqueada.<br/>Já existe uma proposta com o código informado.");
                    } 
                }.bind(this));
            } else {
                this.verificaAction(widget, action);
            }
        }
    };
    
    /*this.removerEdicaoGrid = function(element) {
        if (element.field && element.field.name == "QTMEDIDAORCA"){
            if (element.context.children[1]) {
                element.context.children[1].remove();
            }
        }
    };*/
    
    this.actionEvolutionNewRowProposta = function(){
        this.closeActionsPopup().then(function(){
            var widget = templateManager.container.getWidget("evolucao");
            widget.activate().then(function(){
                setTimeout(function(){
                    widget.reload().then(function(){
                        this.evolucaoNewRow();
                    }.bind(this));
                }.bind(this), 300);
            }.bind(this));
        }.bind(this));
    };
   
    
    this.closeActionsPopup = function() {
        var deferred = ZHPromise.defer();
        ScreenService.closePopup();
        deferred.resolve();
        return deferred;
    };
    
     this.activateWidget = function(widget){
        var deferred = ZHPromise.defer();
        widget.activate();
        deferred.resolve();
        return deferred;
    };
    
     /**
     * Inicia o mvc para realizar um insert de tarefas
     * na widget de Evolução
    */
    this.evolucaoNewRow = function() {
        var widget      = templateManager.container.getWidget("evolucao");
        var wdgProposta = templateManager.container.getWidget('wdgProposta');
        widget.getField('NMRESULTEVOL').readOnly = false; 
        General.widgetNewRow(widget);

        widget.currentRow.DTHREVOLCONT = (widget.currentRow.DTHREVOLCONT) ? widget.currentRow.DTHREVOLCONT : moment(new Date()).format('DD/MM/YYYY HH:mm');
        widget.currentRow.NMOPERADOR   = nomeOperador;
        widget.currentRow.CDOPEREVOL   = opLogado;
        widget.currentRow.NRORG        = nrOrgOper;
        widget.currentRow.DTINCLUSAO   = self.momentNow();
        widget.currentRow.NRSEQCONTATO = wdgProposta.currentRow.NRSEQCONTATO;
        widget.currentRow.NRCONTATO    = wdgProposta.currentRow.NRSEQCONTATO;
        widget.currentRow.NRORG        = nrOrgOper;
        widget.currentRow.NRSEQEVOLCONT = "";
    };
    
    this.momentNow = function(){
        return moment(new Date()).format('DD/MM/YYYY HH:mm:ss');
    };
    
    /*this.setContactReasonFilter = function(field){
        var fieldFilter = field.dataSourceFilter;
        var nrorgFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "P_NRORG");
        //var cdoperadorFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "P_CDOPERADOR");
        
        fieldFilter[nrorgFilterIndex].value = nrOrgOper;
        //fieldFilter[cdoperadorFilterIndex].value = opLogado;
        field.reload();
    };
    
     
    this.setResultFilter = function(field){
    
        var fieldFilter = field.dataSourceFilter;
        
        var nrorgFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "P_NRORG");
        //var cdoperadorFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "P_CDOPERADOR");
        
        fieldFilter[nrorgFilterIndex].value = nrOrgOper;
        //fieldFilter[cdoperadorFilterIndex].value = opLogado;
        
        
        field.reload();
    };*/

     /**
     * Organiza os campos para salvar a evolução
     * e verifica tarefa já aberta.
     * 
     * @param {object} widget widget de evolução
    */
    this.saveEvolucao = function(widget) {
        if (widget.isValid()) {
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            var nrseqcontato;
            
            widget.currentRow.CDOPEREVOL      = opLogado;
            widget.currentRow.IDGERAFATURA    = 'NAO';
            widget.currentRow.IDTPEVOLCONT    = 'P';
            widget.currentRow.IDANEXOEVOLCONT = 'N';
            widget.currentRow.IDSENEVOCONT    = "S";
            widget.currentRow.CDFILIAL        = "";
            widget.currentRow.NRPARCNEGOCIO   = wdgProposta.currentRow.NRPARCNEGOCIO;
            
            widget.getField('NMRESULTEVOL').readOnly = true;
            
            if(!widget.currentRow.NRRELACIONAPARC)
                widget.currentRow.NRRELACIONAPARC = null;
            
            if(widget.currentRow.__is_new && !wdgProposta.currentRow.NRSEQCONTATO) {
                this.geraContatoProposta(wdgProposta.currentRow).then(function(response){
                    nrseqcontato                        = response.dataset.NRCONTATO[0];
                    wdgProposta.currentRow.NRSEQCONTATO = response.dataset.NRCONTATO[0];
                    widget.dataSourceFilter[0].value    = nrseqcontato; 
                    widget.currentRow.NRSEQCONTATO      = nrseqcontato;
                    widget.currentRow.NRCONTATO         = nrseqcontato;
                    
                    this.efetuaRequisicaoSaveEvolucao(widget);
                }.bind(this));
            } else {
                widget.currentRow.NRSEQCONTATO  = wdgProposta.currentRow.NRSEQCONTATO;
                widget.currentRow.NRCONTATO     = wdgProposta.currentRow.NRSEQCONTATO;
                this.efetuaRequisicaoSaveEvolucao(widget);
            }
            
            if(widget.currentRow.__is_new)
                ScreenService.closeSwipe();
            else 
                widget.view();
        }
    };
    
    /**
     * Chama a rota de salvar a evolução
     * @params{Object} widget widget de evolução
    **/
    this.efetuaRequisicaoSaveEvolucao = function(widget){
        var deferred = ZHPromise.defer();
        General.callRestEngine("product#/evolcont_parcnegocio/save", "DataSet",  widget.dataSource.data).then(function(response) {
        if (response.dataset && response.dataset.DADOS.NRSEQEVOLCONT)
            widget.currentRow.NRSEQEVOLCONT  = response.dataset.DADOS.NRSEQEVOLCONT;
            
            widget.reload();
            deferred.resolve();
        }.bind(this)).catch(function(error){
            console.log(error);
            deferred.reject();
            if(error === "Column NRSEQCONTATO cannot be null.")
                ScreenService.showMessage(ScreenService.i18n("Não é possível fazer a evolução em propostas que não estão ligadas a uma Oportunidade."));
            else 
                Utilities.showErrorMessage(error);
        });
        
        return deferred.promise;
    };
    
    /**
     * Salva uma evolução após enviar a proposta por email
     * @params{Object} widget widget evolucao
     * @params{String} idSituacao Situação do orcamento
    **/
    this.saveEvolucaoAposEmail = function(widget, idSituacao) {
        var wdgProposta    = templateManager.container.getWidget("wdgProposta");
        var wdgEnviarEmail = templateManager.container.getWidget("wdgEnviarEmail");
        var nrseqcontato;
        
        widget.currentRow.CDOPEREVOL      = opLogado;
        widget.currentRow.IDGERAFATURA    = 'NAO';
        widget.currentRow.IDTPEVOLCONT    = 'P';
        widget.currentRow.IDANEXOEVOLCONT = 'N';
        widget.currentRow.IDSENEVOCONT    = "S";
        widget.currentRow.CDFILIAL        = "";
        
        if(!widget.currentRow.NRPARCNEGOCIO)
            widget.currentRow.NRPARCNEGOCIO = wdgProposta.currentRow.NRPARCNEGOCIO;
        
        if(!widget.currentRow.NRRELACIONAPARC)
            widget.currentRow.NRRELACIONAPARC = null;

        if(widget.currentRow.__is_new && !wdgProposta.currentRow.NRSEQCONTATO) {
            this.geraContatoProposta(wdgProposta.currentRow).then(function(response){
                nrseqcontato                        = response.dataset.NRCONTATO[0];
                wdgProposta.currentRow.NRSEQCONTATO = response.dataset.NRCONTATO[0];
                widget.dataSourceFilter[0].value    = nrseqcontato; 
                widget.currentRow.NRSEQCONTATO      = nrseqcontato;
                widget.currentRow.NRCONTATO         = nrseqcontato;
                
                this.efetuaRequisicaoSaveEvolucao(widget).then(function(){
                    this.anexarDocumnetosdoEmail(idSituacao, nrseqcontato,  widget.currentRow.NRSEQEVOLCONT);
                }.bind(this));
                
            }.bind(this));
        } else {
            var nrseqevolcont = widget.currentRow.NRSEQEVOLCONT; 
            widget.currentRow.NRSEQCONTATO  = wdgProposta.currentRow.NRSEQCONTATO;
            widget.currentRow.NRCONTATO     = wdgProposta.currentRow.NRSEQCONTATO;
            this.efetuaRequisicaoSaveEvolucao(widget).then(function(){
                this.anexarDocumnetosdoEmail(idSituacao, nrseqcontato,  nrseqevolcont);
            }.bind(this));
        }
        
        wdgEnviarEmail.getField("NMTIPOCONT").validations     = {};
        wdgEnviarEmail.getField("NMMOTIEVOCON").validations   = {};
        wdgEnviarEmail.getField("NMSEQRESULEVOL").validations = {};
        
        widget.list();
        wdgProposta.activate();
    };
    /**
     * Inicia o mvc para realizar select de CONTATO
     * 
     * @param {object} widget widget para passar a currentRow
    */
    this.desobrigaCampos = function(widget){
        widget.fields.forEach(function(field){
            field.validations = {};
        }.bind(this));
    };   
    
    this.selectProspectContato = function(widget){        
        return ger00004500_Prospect_Service.callForSelectProspect(widget.currentRow);
    };
    /**
     * Seta o operador para preparar a proxima tarefa
     * da widget de evolução
     * 
     * @param {object} currentRow pega a currentRow
     * da evolução
    */
    this.getParentField = function(widget, nrseqcontato){
        var widgetPropostaGrid = getWidgetPropostaGrid();
        widget.currentRow.NRPARCNEGOCIO = widgetPropostaGrid.currentRow.NRPARCNEGOCIO;
        widget.currentRow.NRSEQCONTATO  = nrseqcontato;
        widget.currentRow.NRCONTATO     = nrseqcontato;
        
    };
    
     this.setEvolutionWidgetFilter = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        widget.dataSourceFilter[0].value = wdgProposta.currentRow.NRSEQCONTATO;
        widget.reload();
    };
    
     /**
     * Função para editar uma evolução.
     * 
     * @param {object} widget widget ativa, no
     * modo de visualização, com as informações
     * da evolução a ser editada.
    */
    this.editEvolution = function(widget){
        General.widgetEdit(widget);
        widget.getField('NMRESULTEVOL').readOnly = true;
        var fieldDTPLANFILACONTA = widget.getField('DTPLANFILACONTA');
        if (fieldDTPLANFILACONTA)
            widget.getField('DTPLANFILACONTA').validations = {};
        //ABEvolucaoController.setEvolDateChangePermission(widget);
    };
    
     /**
     * Função para fazer o download de uma arquivo
     * salvo no servidor.
     * 
     * @param {object} widget widget que contém
     * as informações do arquivo a ser baixado 
     * em sua currentRow.
    */    
    this.downloadFile = function(widget){
        var nmarquivo = widget.currentRow.NMARQUIVO;
        var dscaminho = widget.currentRow.DSCAMINHO;
        var type      = widget.currentRow.DSTIPO;
        
        Utilities.downloadFile(nmarquivo, dscaminho, type);   
    };
    
     /**
     * Função para abrir a widget de seleção
     * de arquivos.
    */
    this.openAnnexSwipe = function(){
        
        var fileUploadWidget = templateManager.container.getWidget('WDGT_SELECAO_ARQUIVOS');
        
        fileUploadWidget.isVisible = true;
        ScreenService.openSwipe(fileUploadWidget);
            
    };
    
     /**
     * Função para salvar os anexos selecionados
     * ao caso em questão.
     * 
     * @param {object} widget widget onde é
     * feito a seleção dos anexos.
    */
    this.uploadEvolutionFiles = function(widget){

        var files = widget.currentRow.DSCAMINHO;
        var widgetAnexos = getWidgetAnexos();
        var rows = [];
        
        files.forEach(function(file, fileIndex) {
            rows.push(this.setValuesForFileUpload(widget, fileIndex));
        }.bind(this));
        
        General.callRestEngine("product#/crm_anexocontato/save", "DataSet", rows).then(function() {
            this.closeAnnexSwipe(widget);
            widgetAnexos.reload();
        }.bind(this));
    };
    
     /**
     * Função para fechar o swipe de upload de imagens.
     * 
     * @param {object} widget widget de upload de imagens.
    */
    this.closeAnnexSwipe = function(widget){
        var widgetEvolucao = getWidgetEvolucao();
        ScreenService.closeSwipe().then(function(){
            
            widget.isVisible = false;
            widgetEvolucao.list();
            widgetEvolucao.activate();
           
        });
    };
    
     /**
     * Função para retornar uma row configurada
     * com as informações do anexo referente
     * ao ID, recebido como parâmetro, no array
     * de anexos na currentRow.
     * 
     * @param {object} widget widget onde foi selecionado
     * os anexos.Array
     * 
     * @param {int} fileIndex index do anexo no array de 
     * anexos presente na currentRow.
     * 
     * @return {object} currentRow row configurada com as informações
     * do anexo referente ao ID recebido.
    */
    this.setValuesForFileUpload = function(widget, fileIndex){
        var currentRow     = angular.copy(widget.currentRow);
        var widgetEvolucao = getWidgetEvolucao();

        currentRow.NRSEQCONTATO = widgetEvolucao.currentRow.NRSEQCONTATO;
        currentRow.NRORIGEM     = widgetEvolucao.currentRow.NRSEQEVOLCONT;
        currentRow.NMARQUIVO    = currentRow.DSCAMINHO[fileIndex].name;
        currentRow.DSTIPO       = this.getFileExtension(currentRow.DSCAMINHO[fileIndex].name);
        currentRow.VRTAMANHO    = currentRow.DSCAMINHO[fileIndex].size;
        currentRow.CDOPERADOR   = nrparcnegocioOperLogado;
        currentRow.NRORG        = nrOrgOper;
        currentRow.IDATIVO      = "S";
        currentRow.IDORIGEM     = "EVOLUCAO";
        
        return currentRow;
    };
     /**
     * Função para retornar o nome do arquivo sem
     * sua extenção.
     * 
     * @param {string} fileName nome completo do arquivo.
    */
    this.getFileNameWithoutExtension = function(fileName){
        return fileName.replace(/^(.*)\.(.*)$/, "$1");
    };
    
     /**
     * Função para retornar a extenção de um arquivo
     * extraida do seu nome.
     * 
     * @param {string} fileName nome completo do arquivo.
    */
    this.getFileExtension = function(fileName){
        return fileName.replace(/^(.*)\.(.*)$/, "$2");
    };
    
    this.tornaCamposObrigatorios = function(widget){
       if(qtdPrenchido.length >= 4)
            qtdPrenchido = [];
        var nmtipocont = widget.currentRow.NMTIPOCONT;
        var nmmotievocon = widget.currentRow.NMMOTIEVOCON;
        var nmseqresulevol = widget.currentRow.NMSEQRESULEVOL;
        
        if(nmtipocont)
            qtdPrenchido.push(1);
        
        if(nmmotievocon)
            qtdPrenchido.push(1);
        
        if(nmseqresulevol)
            qtdPrenchido.push(1);
        
        if(qtdPrenchido.length == 3)
        qtdPrenchido.push(1);
        
        if(qtdPrenchido.length == 1){
            if(!trancado)
                this.aplicaRegra(widget);
        }else
        if(qtdPrenchido.length == 0){
            if(!nmtipocont && !nmmotievocon && !nmseqresulevol)
            this.aplicaRegra(widget);
        }
        if(!nmtipocont && !nmmotievocon && !nmseqresulevol)
            qtdPrenchido = [];
    };
    
     this.aplicaRegra = function(widget){
        if(!is_required){
            is_required = this.alternaCampo(widget, true);
        } else
            is_required = this.alternaCampo(widget, false);
     };
     
     this.alternaCampo = function(widget, setObrg){
         var ocupado = false;
         if(widget.getField("NMTIPOCONT").value() || widget.getField("NMMOTIEVOCON").value() || widget.getField("NMSEQRESULEVOL").value())
            ocupado = true;
         
         if(setObrg){
            if(ocupado) {
                widget.getField("NMTIPOCONT").validations.required     = setObrg;
                widget.getField("NMMOTIEVOCON").validations.required   = setObrg;
                widget.getField("NMSEQRESULEVOL").validations.required = setObrg;
            }
         } else {
            widget.getField("NMTIPOCONT").validations = {};
            widget.getField("NMMOTIEVOCON").validations = {};
            widget.getField("NMSEQRESULEVOL").validations = {};
         }
         
        qtdPrenchido.pop();
        return setObrg;
    };
    
    /**
     * Abre o popup de edição do operador responsável da proposta
     * @params{Object} widget widget proposta grid
    **/
    this.abrirPopupAlterarOper = function(widget){
        var wdgPopup = templateManager.container.getWidget("wdgPopupAlteraOper");
        wdgPopup.isVisible = true;
        wdgPopup.getField("CDOPERRESPATUAL").readOnly = false;
        
        ScreenService.openPopup(wdgPopup).then(function(){
            wdgPopup.edit();
            wdgPopup.newRow();
            wdgPopup.activate();
            wdgPopup.getField("NRSEQORCAMENTO").dataSourceFilter = [];
            this.verificaHierarquia({}, wdgPopup);
        }.bind(this));
    };
    
    /**
     * filtra as fields de operador atual e propostas do popup de alteração dos
     * operadores responsáveis
     * @params{Object} widget widget popup altera operador responsável
     * @params{Object} field fieldAlterada nome da field de alteraçao
    **/
    this.filtraOperadoresEPropostas = function(widget, fieldAlterada){
        var fieldProposta  = widget.getField("NRSEQORCAMENTO");
        var fieldOperAtual = widget.getField("CDOPERRESPATUAL");

        if(widget.currentRow.CDOPERRESPATUAL && !widget.currentRow.NRSEQORCAMENTO || (widget.currentRow.NRSEQORCAMENTO && widget.currentRow.CDOPERRESPATUAL && fieldAlterada === "CDOPERRESPATUAL")){
            fieldProposta.clearValue();
            
            if(widget.currentRow.CDOPERRESPATUAL && widget.currentRow.CDOPERRESPATUAL.length > 0){
                fieldProposta.dataSourceFilter[0] = {
                    "name": "CDOPERRESPONSAVEL",
                    "operator": "IN",
                    "value": widget.currentRow.CDOPERRESPATUAL
                };
                
                fieldOperAtual.readOnly = false;  
            } else {
                if(fieldProposta.dataSourceFilter.length > 0){
                    fieldProposta.dataSourceFilter.pop();
                }
            }
            
            fieldProposta.reload();
            
        } else {
            widget.currentRow.CDOPERRESPONSAVEL = [];
            widget.getField("NRSEQORCAMENTO").dataSource.getCheckedRows().forEach(function(proposta){
                widget.currentRow.CDOPERRESPONSAVEL.push(proposta.CDOPERRESPONSAVEL);
            });
            
            if((!widget.currentRow.CDOPERRESPATUAL && widget.currentRow.NRSEQORCAMENTO) || (widget.currentRow.NRSEQORCAMENTO && widget.currentRow.CDOPERRESPATUAL && fieldAlterada === "NRSEQORCAMENTO")){
                if(widget.currentRow.CDOPERRESPONSAVEL &&  widget.currentRow.CDOPERRESPONSAVEL.length > 0){
                    fieldOperAtual.clearValue();
                    widget.currentRow.CDOPERRESPATUAL = [];
                    fieldOperAtual.dataSourceFilter[0] = {
                        "name": "CDOPERADOR",
                        "operator": "IN",
                        "value": widget.currentRow.CDOPERRESPONSAVEL
                    };
                    
                    fieldOperAtual.readOnly = true;
                    widget.currentRow.CDOPERRESPONSAVEL.forEach(function(operador){
                        if(widget.currentRow.CDOPERRESPATUAL.indexOf(operador) === -1){
                            widget.currentRow.CDOPERRESPATUAL.push(operador);
                        }
                    });
                    
                } else {
                    if(fieldOperAtual.dataSourceFilter.length > 0){
                        fieldOperAtual.dataSourceFilter.pop();
                        fieldOperAtual.readOnly = false; 
                    }
                }
                
                fieldOperAtual.reload();
            }
        }
    };
    
    /**
     * Cancela a edição do operador responsável da proposta
     * @params{Object} widget widget popup alterar operador responsavel
    **/
    this.cancelarAlteracaoOper = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgPropostaGrid");
        var fieldProposta  = widget.getField("NRSEQORCAMENTO");
        var fieldOperAtual = widget.getField("CDOPERRESPATUAL");
                
        fieldOperAtual.readOnly = false;
        fieldOperAtual.dataSource.clearCheckedRows();
        if(fieldOperAtual.dataSourceFilter.length > 0){
            fieldOperAtual.dataSourceFilter.pop();
        }
        
        fieldProposta.dataSource.clearCheckedRows();
        if(fieldProposta.dataSourceFilter.length > 1){
            fieldProposta.dataSourceFilter.pop();
        }
        
        widget.dataSource.data = [];
        widget.isVisible = false;
        ScreenService.closePopup();
        wdgProposta.reload();
        wdgProposta.activate();
    };
    
    /**
     * Salva as alterações do operador responsável
    **/
    this.salvaAteracoesOperResp = function(widget){
        if(widget.currentRow.NRSEQORCAMENTO.length === 0 && widget.currentRow.CDOPERRESPATUAL.length !== 0){
            widget.currentRow.NRSEQORCAMENTO  = [];
            widget.getField("NRSEQORCAMENTO").dataSource.data.forEach(function(data){
                widget.currentRow.NRSEQORCAMENTO.push(data.NRSEQORCAMENTO);
            });
        }
        
        if(widget.isValid()){
            var params = {
                "NRSEQORCAMENTO"   : widget.currentRow.NRSEQORCAMENTO,
                "CDOPERRESPONSAVEL": widget.currentRow.CDNOVOOPERRESP,
                "ALTERACONTRATO"   : widget.currentRow.ALTERACONTRATO
            };
            
            General.callRestEngine("product#/fct_orcamento_widget/salvaAlteracaoMassaOper", "Row", params).then(function(){
                ScreenService.showMessage("Propostas atualizados com sucesso.");
                this.cancelarAlteracaoOper(widget);
            }.bind(this));
        }
    };

    /**
     * Abre a widget para a adição de um novo interlocutor
     * durante a criação/edição da proposta 
     * @params{Object} widget widget prosposta view
    **/ 
   this.abreWidgetAddNewInterloc = function(widget){
       if(!widget.currentRow.NRPARCNEGOCIO){
           ScreenService.showMessage(ScreenService.i18n("Operação bloqueada.<br>Selecione o parceiro de negócio ligado a proposta."));
       } else {
            ScreenService.closePopup();
            var wdgPopup = templateManager.container.getWidget("wdgNewInterlocutor");
            wdgPopup.isVisible = true;
            ScreenService.openPopup(wdgPopup).then(function(){
                wdgPopup.activate();
                wdgPopup.getField("NMINTERLOCUTOR").readOnly = false;
                wdgPopup.label = "Novo Interlocutor";
                wdgPopup.dataSource.data = [];
                wdgPopup.newRow();
                wdgPopup.currentRow.NRPARCNEGOCIO       = widget.currentRow.NRPARCNEGOCIO;
                wdgPopup.currentRow.DSEMAIL             = widget.currentRow.EMAILINTERLOC;
                wdgPopup.currentRow.NRTELEFONE          = widget.currentRow.TELEFONE1;
                wdgPopup.currentRow.IDNOMEVINDODAAPI    = "N";
                wdgPopup.currentRow.DTNASCIMENTO  = "";
                // wdgPopup.currentRow.NRCPFINTERLOC = widget.currentRow.NRCPFINTERLOC;
                //widget.currentRow.IDTPSIGNATARIO = "N";
                this.mudaTipoSignatario(wdgPopup);
                this.gerarEventosCPF(wdgPopup);
                templateManager.updateTemplate().then(function(){
                    Utilities.clearCalendarField("DTNASCIMENTO");
                });
            }.bind(this));
       }
   };
   
   /**
    * Esse procedimento é responsável por gerar o evento do OnChange do campo de CPF da popup dos interlocutores.
    * @params {Widget} widget da popup do interlocutor
   */
    this.gerarEventosCPF = function(popUp) {
        var cpf = popUp.getField("NRCPFINTERLOC");
        var data = popUp.getField("DTNASCIMENTO");
        cpf.events = [{
            "id": "30185",
            "name": "FieldOnChange",
            "code": "Sm00001000_LancamentoProposta_Controller.fazerConsultaETrocarNome(args.owner.field.widget);"
        }];
        data.events = [{
            "id": "30185543",
            "name": "FieldOnChange",
            "code": "Sm00001000_LancamentoProposta_Controller.fazerConsultaETrocarNome(args.owner.field.widget);"
        }];
    };
    
    /**
    * Procedimento responsável por substituir os dados da popup do Interlocutor com uma row. 
    * @params{Object} - popupWdg widget da popup dos interlocutores 
    * @params{Object} - row onde os dados estao para substituirem a popup 
    */
    this.substituiRowDaPopupDoInterlocutor = function(popupWdg, row) {
        popupWdg.currentRow.NRPARCNEGOCIO = row.NRPARCNEGOCIO;  
        popupWdg.currentRow.NMINTERLOCUTOR = row.NMINTERLOCUTOR;  
        popupWdg.currentRow.NRCPFINTERLOC = row.NRCPFINTERLOC;  
        popupWdg.currentRow.DSEMAIL = row.DSEMAIL;  
        popupWdg.currentRow.EMAILINTERLOC = row.EMAILINTERLOC;
        popupWdg.currentRow.NRSEQINTERLOCUTOR = row.NRSEQINTERLOCUTOR;
        popupWdg.currentRow.IDTPSIGNATARIO = (row.IDTPSIGNATARIO) ? row.IDTPSIGNATARIO :  "N";  
        popupWdg.currentRow.DTNASCIMENTO = row.DTNASCIMENTO;  
        popupWdg.currentRow.IDNOMEVINDODAAPI = (row.IDNOMEVINDODAAPI) ? row.IDNOMEVINDODAAPI :  "N";  
        popupWdg.currentRow.__is_new = false;  
    };
    
   
    /**
    * Função responsável por retornar uma row partindo de um data de um datasource. Usada ao clicar no botão de edição da popup interlocutor.
    * @params{Object} - datasource.data dados que devem ser extraídos
    * @params{boolean} - isNew valor do isnew da row, padrão é falso 
    * @returns {row} - row a ser retornada.
    */
    this.extraiInterlocutorRetornandoRow = function(datasourceData, isNew){
        if (isNew == undefined) {
            isNew = false;
        }

        var row = {
            "NMINTERLOCUTOR": datasouceData.NMINTERLOCUTOR,
            "DSEMAIL" :datasouceData.EMAILINTERLOCUTOR,
            "IDTPSIGNATARIO": datasouceData.IDTPSIGNATARIO,
            "NRPARCNEGOCIO" :datasouceData.NRPARCNEGOCIO,
            "NRCPFINTERLOC" :datasouceData.NRCPFINTERLOC,
            "NRTELEFONE": datasouceData.NRTELEFONE1,
            "IDNOMEVINDODAAPI": datasouceData.IDNOMEVINDODAAPI,
            "__is_new": isNew
        };   
        console.assert( (row) , "Não foi possível extrair uma row do datasource.data");
        return row ;
    };
    
    this.abrirPopupInterlocutorModoEdicao = function(widget){
        if(!widget.currentRow.NRPARCNEGOCIO){
           ScreenService.showMessage(ScreenService.i18n("Operação bloqueada.<br>Selecione o parceiro de negócio ligado a proposta."));
       } else {
            ScreenService.closePopup();
            var wdgPopup = templateManager.container.getWidget("wdgNewInterlocutor");
            wdgPopup.isVisible = true;
            ScreenService.openPopup(wdgPopup).then(function(){
                wdgPopup.activate();
                wdgPopup.dataSource.data = [];
                wdgPopup.newRow();
                this.substituiRowDaPopupDoInterlocutor(wdgPopup, widget.currentRow);
                wdgPopup.getField("NMINTERLOCUTOR").readOnly = (wdgPopup.currentRow.IDNOMEVINDODAAPI != "N");
                wdgPopup.label = "Editar Interlocutor";
                this.gerarEventosCPF(wdgPopup);
                this.mudaTipoSignatario();
                templateManager.updateTemplate();
            }.bind(this));
       }
    };
    
    
    /**
     * Função responsável por criar uma action de editar Interlocutor
     * @return {Object} action - a ação de editar
     */
    this.criarActionEditarInterlocutor = function(){
        return {
            "icon": "pencil",
            "showAsAction": "grid-right",
            "tooltip": "Editar",
            "isVisible": true,
            "condition": true,
            "events": [
                {
                    "name": "ActionEvent",
                    "code": "Sm00001000_LancamentoProposta_Controller.abrirPopupInterlocutorModoEdicao(args.owner.widget)"
                }
            ]
        };
    };
    
    /**
     * Evento chamado para acionar a ação de edit do interlocutor
    */
    this.criarAcaoEditInterlocutor = function(field){
        SelectUtil.construirWidgetSelect(field, this.criarActionEditarInterlocutor());
    };
    
    /**
     * Esse procedimento é responsável por fazer toda a funcionalidade de substituir o nome do interlocutor baseado no CPF
     * @params{Widget} widget da popup do interlocutor
    */
    this.fazerConsultaETrocarNome = function(widget){
        //aplicar a validaçãqo do general
        widget.getField("NMINTERLOCUTOR").readOnly = false;
        widget.currentRow.IDNOMEVINDODAAPI = "N";
        var required = (widget.getField("NRCPFINTERLOC").validations) ? widget.getField("NRCPFINTERLOC").validations.required : false;
        var cpfValido = General.validateCPF(widget.currentRow.NRCPFINTERLOC);
        var data = widget.currentRow.DTNASCIMENTO; 
        var cpf = widget.currentRow.NRCPFINTERLOC; 
        if(cpfValido || required ){
            this.pegarValidacaoCPF().then(function(response){
                if(response.dataset.resulted.IDVALIDACPFRF != 'N'){
                    this.efetuarConsulta(widget, cpfValido);
                }else{
                    if(data && cpf){
                        ScreenService.confirmMessage("Deseja buscar o nome da pessoa de acordo com o CPF e Data de Nascimento?", "question", function(){
                            this.efetuarConsulta(widget, cpfValido);
                        }.bind(this));
                    }
                }
            }.bind(this));
        }else{
           if(required ) 
                ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br>O valor preenchido no CPF não representa um valor válido."))
        }
    };
    
    /**
     * Função responsável por fazer a consulta do CPF e alterar o nome do campo da popup
    */
    this.efetuarConsulta = function(popUp, cpfValido){
        if (cpfValido == undefined) {
            cpfValido = false;
        }

        var data = popUp.currentRow.DTNASCIMENTO; 
        var cpf = popUp.currentRow.NRCPFINTERLOC; 
        if (data && cpf) {
            if (cpfValido) {
                this.validaNaAPIAntes(data,cpf); 
            }
            else {
                ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br><br>O CPF informado é inválido."));
            }
        }
        
        /*if(cpfValido && data && cpf)
            this.validaNaAPIAntes(data,cpf); 
        else{
            ScreenService.showMessage(ScreenService.i18n(`Operação bloqueada! <br><br> ${!cpfValido 
            ? "O valor do campo do CPF é inválido, favor inserir um valor válido para o CPF."
            : "O campo de data deve possuir algum valor, favor inserir um valor no campo Data de Nascimento para efetuar a consulta."
            }`));
        }*/
    };
    
    
    /**
     * Essa função vê o estado da consulta automática do CPF parametrizada na tela de Parametrizações SM
     * @returns {Promise} Promise da consulta à tela de Parametrizações
    */
    this.pegarValidacaoCPF = function(){
        return General.callRestEngine("product#/fct_parametrogeral_widget/pegarValidacaoCPF", "Row", {});
    };
    
    /**
     * Esse procedimento efetua a validação e retornar a resposta
     * @params {string} data de nascimento
     * @params {string} número de cpf
    */
    this.validaNaAPIAntes = function(data, cpf){
        var params = {
            "DTNASCIMENTO" : data,
            "NRCPFINTERLOC" : cpf
        };
        General.callRestEngine("product#/consultaDadosCPF", "Row", params).then(function(response){
            var obj = response.dataset.CONSULTA;
            if(obj.Status){
               this.alterarNomeInterlocutor(obj.Nome);
            }else{
                ScreenService.showMessage(ScreenService.i18n('Operação bloqueada! <br><br>Falha na integração com a API de validação do CPF: <br>' + obj.Mensagem));
            }
        }.bind(this)).catch(function(err){
            ScreenService.showMessage(ScreenService.i18n(err));
        });
    };
    
    /**
     * Procedimento responsável por substituir o campo de nome e transformar ele em readonly, chamada somente em uma requisição bem sucedida
     * @params {string} nome a ser substituido
     * @params {Widget} widget da popup do interlocutor, pode ser opcional
    */
    this.alterarNomeInterlocutor = function(nome, wdg) {
        if (wdg == undefined) {
            wdg = null;
        }

        if(wdg == null)
            wdg  = templateManager.container.getWidget("wdgNewInterlocutor");
        wdg.currentRow.NMINTERLOCUTOR = nome;
        wdg.getField("NMINTERLOCUTOR").readOnly = true;
        wdg.currentRow.IDNOMEVINDODAAPI = "S";
    };

   /**
     * Salvar um novo interlocutor 
     * @params{Object} widget popup adicionar interlocutor
    **/
    this.salvarNovoInterlocutor = function(widget){
        var interlocutor = widget.currentRow.NMINTERLOCUTOR;
        var email        = widget.currentRow.EMAILINTERLOC;
        var telefone     = widget.currentRow.NRTELEFONE1;
        var wdgProposta  = templateManager.container.getWidget("wdgProposta");
        
        var podeSalvar = true;
        var isNew = widget.currentRow.__is_new;
        
        if (widget.isValid()) {
            //chamar o valida email e cpf
            var cpf = widget.currentRow.NRCPFINTERLOC;
            if (!Util.isEmptyOrBlank(cpf) && !General.validateCPF(cpf)) {
                podeSalvar = false;
                Utilities.showErrorMessage("Operação bloqueada! <br><br>CPF inválido, favor inserir um valor válido para o CPF.");
            }
            
            /* if(!Utilities.verificaEmail(email)){
                podeSalvar = false;
                Utilities.showErrorMessage("Operação bloqueada! <br>E-mail inválido.");
            }*/
                
            if (podeSalvar) {
                General.widgetSave(widget).then(function(response){
                    if(response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0){
                        wdgProposta.currentRow.NMPARCINTERLOCUTOR = interlocutor;
                        wdgProposta.currentRow.DSEMAIL            = email;
                        wdgProposta.currentRow.NRTELEFONE         = telefone;
                        wdgProposta.currentRow.NRSEQINTERLOCUTOR  = response.dataset.columnsToReturn[0].NRSEQINTERLOCUTOR ; 
                    }
                    widget.isVisible = false;
                    widget.template = widget.templateToForm;
                    
                    ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
                    ScreenService.closePopup();
                    wdgProposta.activate();
                    if(!isNew){
                        var field = wdgProposta.getField("NMPARCINTERLOCUTOR");
                        ScreenService.openPopup(field.selectWidget);
                        field.selectWidget.reload();
                    }
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }
        }
    };
    
    /**
     * Volta para o grid de itens
    **/
    this.voltarItensGrid = function(widget){
        if(["V", "P"].includes(widget.currentRow.IDTPDESCORCA) && parseFloat(widget.currentRow.VRDESCORCA) > 0){
            widget.currentRow.VRDESCORCA = parseFloat(widget.currentRow.VRDESCORCA) * -1;
            widget.currentRow.VRDESCORCA = widget.currentRow.VRDESCORCA.toFixed(2).replace(".", ",");
        }
        ScreenService.closeSwipe();
        widget.reload();
    };
    
    this.wdgParametrosFaturamentoItemNewRow = function(wdgParametrosFaturamentoItem) {
        General.widgetNewRow(wdgParametrosFaturamentoItem);
        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(wdgParametrosFaturamentoItem);

    };
    
    this.wdgParametrosFaturamentoItemEdit = function(wdgParametrosFaturamentoItem) {
        General.widgetEdit(wdgParametrosFaturamentoItem);
        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(wdgParametrosFaturamentoItem);
    };
    
    this.wdgParametrosFaturamentoItemSave = function(wdgParametrosFaturamentoItem) {
        if (wdgParametrosFaturamentoItem.isValid()) {
            Utilities.validarCamposDuplicados(wdgParametrosFaturamentoItem, ["NRPARAMFATITPROP"], ["NRSEQORCAMENTO", "NRITEMORCAMENTO"])
            .then(function(validacao) {
                if (!validacao) {
                    ScreenService.showMessage("Operação bloqueada. <br/>Já existe uma parametrização de faturamento cadastrada para este item.");
                } else {
                    Utilities.widgetSaveAndSetColumnsToReturn(wdgParametrosFaturamentoItem);
                }
            });
        }
    };
    /**
     * valida se o valor informado para a quandidade de medida do item se encontra 
     * na faixa de preco estabelecida na tabela de variação
    **/
    this.validarVariacaoPreco = function(nrseqorcamento, cdservorca, qtmedidaorca, cdtpmedida, idSituacao, nrtipoorcamento){
        var params = {
            "NRSEQORCAMENTO"     : nrseqorcamento,
            "CDSERVORCA"         : cdservorca,
            "QTMEDIDAORCA"       : qtmedidaorca,
            "CDTPMEDIDASERV"     : cdtpmedida,
            "IDSITUACAOORCAMENTO": idSituacao,
            "NRTIPOORCAMENTO"    : nrtipoorcamento
        };
        
        return General.callRestEngine("product#/fct_orcamento_widget/validarVariacaoPreco", "Row", params);
    };
    /**
     * Verifica qual widget está chamando a validação e capta os dados necessários para 
     * a validação do valor da quandidade de medida
     * @params {Object} widget - widget de itens ou de tipo de medida
    **/    
    this.buscaDadosParaValidacaoQtMedida = function(widget, nomeCampoMedida){
        var alteracaoQtMedida = this.validarAlteracaoQtMedida(widget.currentRow, nomeCampoMedida);
        if (alteracaoQtMedida) {
            if(this.validarQtMedidaPropInserindo(widget.currentRow, "QTMEDIDAVARIAVEL_inline")){
                var deferred    = ZHPromise.defer();
                var wdgProposta = getWidgetProposta();
        
                var nrseqorcamento  = wdgProposta.currentRow.NRSEQORCAMENTO;
                var idSituacao      = wdgProposta.currentRow.IDSITUACAOORCAMENTO;
                var nrtipoorcamento = wdgProposta.currentRow.NRTIPOORCAMENTO;
                var cdservorca      = widget.currentRow.CDSERVORCA;
                var qtmedidaorca;
                var cdtpmedida;
                
                if(widget.name === "wdgItensPropostaTree"){
                    qtmedidaorca = widget.currentRow.QTMEDIDAORCA;
                    cdtpmedida   = widget.currentRow.CDTPMEDIDASERV;
                } else {
                    if(widget.name == "wdgItensPropostaInserindo") {
                        qtmedidaorca = widget.currentRow.QTMEDIDASERV;
                        cdtpmedida   = widget.currentRow.CDTPMEDIDASERV;
                    } else {
                        cdservorca   = [];
                        qtmedidaorca = widget.currentRow.QTMEDIDAVARIAVEL_inline;
                        cdtpmedida   = widget.currentRow.CDTPMEDIDAVARIAVEL;
                    }
                }
                
                this.validarVariacaoPreco(nrseqorcamento, cdservorca, qtmedidaorca, cdtpmedida, idSituacao, nrtipoorcamento).then(function(response){
                    if(widget.name === "wdgItensPropostaTree"){
                        var newVrUnitItem = false;
                        if(response.dataset && response.dataset.fct_orcamento_widget && response.dataset.fct_orcamento_widget.NEWVRUNITITEM){
                            newVrUnitItem = response.dataset.fct_orcamento_widget.VRPRECOVARIACAO;
                            if(widget.currentRow.VRUNITITEMORCA !== newVrUnitItem){
                                widget.currentRow.VRUNITITEMORCA = newVrUnitItem;
                            }
                        }
                    }
                    this.calcularValorTotalItem(widget);
                    
                    deferred.resolve();
                }.bind(this)).catch(function(error){
                    deferred.reject();
                    Utilities.showErrorMessage(error);
                    if(widget.name === "wdgTipoMedidaPropostaEditando"){
                        widget.currentRow.QTMEDIDAVARIAVEL_inline = widget.currentRow.QTMEDIDAVARIAVEL;
                    } else {
                        widget.currentRow.QTMEDIDAORCA = widget.backupRow.QTMEDIDAORCA;
                        this.calcularValorTotalItem(widget);
                    }
                }.bind(this));
                return deferred.promise;
            }
        }
        else {
            this.calcularValorTotalItem(widget);
        }
    };
    /**
     * Adiciona o filtro de situação e carrega os dados na tela 
     * @params{Object} widget - widget itens tree
    **/
    this.carregarOnEnterTabItens = function(widget){
        if(!widget.dataSourceFilter[3].value || widget.dataSourceFilter[3].value !== getWidgetProposta().currentRow.IDSITUACAOORCAMENTO)
            widget.dataSourceFilter[3].value = getWidgetProposta().currentRow.IDSITUACAOORCAMENTO;
        Utilities.carregarWidgetOnEnterTab(widget);
    };
    /**
     * Valida se a quantidade de medida informada durante a inserção da proposta
     * é maior ou igual ao valor da franquia
     * @params{Object} row - row do inline edit
    **/
    this.validarQtMedidaPropInserindo = function(rowTpMedida, widget, nameQtMedida){
        if(rowTpMedida.QTFRANQUIA && rowTpMedida.QTFRANQUIA > parseFloat(rowTpMedida[nameQtMedida])){
            ScreenService.showMessage("Operação bloqueada!<br>O valor da franquia não pode ser menor que o valor da quantidade.");
            widget.currentRow.QUANTIDADE_inline = rowTpMedida.QTFRANQUIA;
            return false;
        }
        return true;
    };

    /**
     * Valida se o valor unitário informado no campo é menor que 
     * o valor unitário calculado da tabela de preços.
     * @param {object} widget 
     */
    this.validarValorUnitarioItem = function(widget) {
        this.validarPermissaoAlteracaoValorUnitario(widget).then(null).catch(null).finally(function() {
            widget.name == 'wdgItensProposta' ? this.calcularValoresItemOrcamento(widget) : this.calcularValorTotalItem(widget);
        }.bind(this));
    };
    
    this.validarPermissaoAlteracaoValorUnitario = function(widget) {
        var deferred = ZHPromise.defer();
        
        var vrunititemorca         = Utilities.tirarMascaraValores(widget.currentRow.VRUNITITEMORCA);
        var vrunititemorcatblpreco = Utilities.tirarMascaraValores(widget.currentRow.VRUNITITEMORCATBLPRECO);
     
        var params = {
            "NRTIPOORCAMENTO" : widget.currentRow.NRTIPOORCAMENTO
        };

        General.callRestEngine("product#/fct_orcamento_widget/validaBloqAlteraValor", "Row", params).then(function(response){

            var idaltvrtabrpecprop = response.dataset.retorno.idaltvrtabrpecprop;
            
            if (idaltvrtabrpecprop == '2' && vrunititemorca < vrunititemorcatblpreco) {
                ScreenService.showMessage("Operação bloqueada.<br/>O campo \"Valor Unitário\" não pode ser menor que o campo \"Valor Unitário da Tabela de Preços\".");
                widget.currentRow.VRUNITITEMORCA = vrunititemorcatblpreco;
                deferred.reject();
            }
            else if (idaltvrtabrpecprop == '3' && vrunititemorca > vrunititemorcatblpreco) {
                ScreenService.showMessage("Operação bloqueada.<br/>O campo \"Valor Unitário\" não pode ser maior que o campo \"Valor Unitário da Tabela de Preços\".");
                widget.currentRow.VRUNITITEMORCA = vrunititemorcatblpreco;
                deferred.reject();
            }
            else if (idaltvrtabrpecprop == '4' && vrunititemorca != vrunititemorcatblpreco) {
                ScreenService.showMessage("Operação bloqueada.<br/>O campo \"Valor Unitário\" não pode ter um valor diferente do valor do o campo \"Valor Unitário da Tabela de Preços\".");
                widget.currentRow.VRUNITITEMORCA = vrunititemorcatblpreco;
                deferred.reject();
            }
            else {
                deferred.resolve();
            }
        })
        .catch(function(error) {
            Utilities.showErrorMessage(error);
            deferred.reject();
        });
        
        return deferred.promise;
    };

    this.calcularValoresItemOrcamento = function(widget) {
        var qtde          = widget.currentRow.QTITEMORCAMENTO;
        var qtdeMedida    = widget.currentRow.QTMEDIDAORCA;
        var valorUnitario = widget.currentRow.VRUNITITEMORCA;
        
        /*var tipoAjuste    = widget.currentRow.IDTPDESCORCA;
        var valorDoAjuste = widget.currentRow.VRPERCDESCORCA;
        if (typeof valorDoAjuste === "string") {
            valorDoAjuste = (valorDoAjuste.length == 0) ? 0 : parseFloat(widget.currentRow.VRPERCDESCORCA.replace(",", "."));
        }*/

        if (qtdeMedida) {
            qtdeMedida = parseFloat(qtdeMedida);
        }
        else {
            qtdeMedida = 1;
        }
        
        if (valorUnitario && qtde) {
            valorUnitario = parseFloat(valorUnitario.toString().replace(",", "."));
            qtde = parseFloat(qtde.toString().replace(",", "."));
            var valorSemAjuste = valorUnitario * qtde * qtdeMedida;
            widget.currentRow.VRITEMSEMDESC = valorSemAjuste;

            var valorTotal = null;
        
            /*if (tipoAjuste == "N") {
                valorTotal = valorSemAjuste;
            }
            else if (tipoAjuste == "V") { //desconto valor
                valorTotal = valorSemAjuste - valorDoAjuste;
            }
            else if (tipoAjuste == "P") { //desconto porcentagem
                valorTotal = valorSemAjuste - (valorSemAjuste * valorDoAjuste / 100);
            }
            else if (tipoAjuste == "X") { //acrescimo valor
                valorTotal = valorSemAjuste + valorDoAjuste;
            }
            else if (tipoAjuste == "Y") { //acrescimo porcentagem
                valorTotal = valorSemAjuste + (valorSemAjuste * valorDoAjuste / 100);
            }*/
            
            valorTotal = valorSemAjuste;
            widget.currentRow.VRTOTALITEMORCA = valorTotal;
        }
    };
    
    /**
     * Controla a acessibiliada das abas da widget de tabbed principal
     * @params{string} mode   - modo da widget de tabbed 
     * @params{bool} visible  - visibilidade  
     * @params{Object} widget - widget que efetua a solucitação
    **/
    this.controlarAcessibilidadeTabbed = function(mode, visible, widget){
        var wdgTabbedPrincipal  = templateManager.container.getWidget("wdgTabbedPrincipal");
        if(mode == "edit")
            wdgTabbedPrincipal.edit();
        else 
            wdgTabbedPrincipal.view();
        var widgets = wdgTabbedPrincipal.widgets.filter(function(wid) {
            if(widget.parent && wid.name === widget.parent.name)
                widget.parent.mode = mode;
            return wid.name != widget.name && (widget.parent && wid.name !== widget.parent.name);
        });

        General.changeWidgetAcessibility(widgets, visible);
    };
    /**
     * Abre a widget de nome contratual durante a inserção da proposta
     * @params{Array} dataSource - dataSource da tela
    **/
    this.habilitarWidgetNomeContratual = function(dataSource){
        var widget     = templateManager.container.getWidget("wdgNomeContratual");
        var idarvore   = null;
        var nmInserido = [];
        
        widget.dataSource.data = [];
        dataSource.forEach(function(item) {
            if(item.IDTIPO === 'M') {
                idarvore = item.IDARVORE.split("_");
                idarvore = idarvore[0] + "_" + idarvore[1];
                if(nmInserido.indexOf(idarvore) === -1) {
                    widget.dataSource.data.push({
                        "NRITEMORCAMENTO"      : null,
                        "NMITEMORCAMENTO"      : item.NMSERVORCA,
                        "NMCONTRATUALITEMORCA" : item.NMSERVORCA,
                        "IDELEMENTO"           : idarvore
                    });
                    nmInserido.push(idarvore);
                }
            }
        });
        
        widget.enabled   = true;
        widget.isVisible = true;
        widget.activate().then(function(){
            widget.edit();
        });
    };

    /**
     * Avançar para a widget de itens inserindo
     * @params{Object} widget - widget de nome contratual
    **/
    this.avancarItensInserindo = function(widget){
        var wdgItens   = templateManager.container.getWidget("wdgItensPropostaInserindo");
        var idelemento = null;
        widget.enabled = false;
        
        wdgItens.enabled   = true;
        wdgItens.isVisible = true;
        widget.dataSource.data.forEach(function(itemNmCont){
            wdgItens.dataSource.data.forEach(function(item){
                idelemento = item.IDARVORE.split("_");
                idelemento = idelemento[0] + "_" + idelemento[1];
                if(itemNmCont.IDELEMENTO === idelemento)
                    item.NMCONTRATUALITEMORCA = itemNmCont.NMCONTRATUALITEMORCA;
            });
        });
        
        wdgItens.activate().then(function() {
            wdgItens.list();
            wdgItens.mode = "edit";
            //this.alterarVisibilidadeCamposItens(wdgItens, widget.currentRow.IDMEDIDAFIXA);
        }.bind(this));
    };

    /**
     * Volta para a widget de nome contratual
     * @params{Object} widget - widget de itens da proposta inserindo
    **/
    this.voltarWdgNomeContratual = function(widget){
        var alteracaoItens = widget.dataSource.data.filter(function(item){
            return item.changed;
        });
        
        var wdgNmContratual = templateManager.container.getWidget("wdgNomeContratual");
        if(alteracaoItens.length == 0) {
            widget.enabled = false;
            wdgNmContratual.enabled = true;
            wdgNmContratual.activate();
        } else {
            ScreenService.confirmMessage("Atenção!<br>Voltar para as etapas anteriores pode resultar na perda das alterações realizadas.<br>Deseja continuar?", "question", function(){
                widget.enabled = false;
                wdgNmContratual.enabled = true;
                wdgNmContratual.activate();
            });
        }
    };

    /**
     * Desabilita as fields de "Cancelar" e "Salvar Alterações"
     * da widget de nome contratual quando estiver em modo inserção
     * @params{Object} widget - widget de nome contratual
    **/
    this.desabilitarFieldsInline = function(widget){
        var isVisible = (getWidgetProposta().mode !== "view");
        widget.getAction('cancel').isVisible       = !isVisible;
        widget.getAction('save').isVisible         = !isVisible;
        widget.getAction('back').isVisible         = isVisible;
        widget.getAction('avancarItens').isVisible = isVisible;
        templateManager.updateTemplate();
    };

    /**
     * Valida se a widget de nome contratual está sendo editada para 
     * carregar os dados
     * @params{Object} widget - widget nome contratual
    **/
    this.validarOnEnterWdgNmContratual = function(widget){
        if(getWidgetProposta().mode != "view"){
            widget.floatingControl = false;
            widget.getField('NMCONTRATUALITEMORCA').inlineEditable = 'text';
        } else { 
            Utilities.carregarWidgetOnEnterTab(widget);
            widget.floatingControl = {
                customizationAction: {
                    filter: false,
                    layout: false,
                    view: false
                }
            };
        }
    };

    /**
     * Manda salvar um nome contratual
     * @params{Object} widget - widget nome contratual
    **/
    this.saveNomeContratual = function(widget){
        var rowsAlteradas = widget.dataSource.data.filter(function(row){
            return row.changed;
        });
        
        General.callRestEngine("product#/fct_itemorcamento_widget/salvarNomeContratual", "DataSet", rowsAlteradas).then(function(){
            widget.list();
            widget.reload().then(function(){
                ScreenService.showMessage("Processo realizado com sucesso!");
            });
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    

    this.wdgNegociacaoValoresAfterMoveRow = function(wdgNegociacaoValores) {
        var idtpfatura               = wdgNegociacaoValores.currentRow.IDTPFATURA;
        var wdgParcelas              = templateManager.container.getWidget("wdgParcelas");
        var wdgDescontoPorPeriodo    = templateManager.container.getWidget("wdgDescontoPorPeriodo");
        var wdgCompetenciaNegociacao = templateManager.container.getWidget("wdgCompetenciaNegociacao");

        if (idtpfatura == "P") {
            wdgParcelas.isVisible = true;
            wdgDescontoPorPeriodo.isVisible = false;
            wdgCompetenciaNegociacao.isVisible = false;
        } else {
            wdgParcelas.isVisible = false;
            wdgDescontoPorPeriodo.isVisible = true;
            wdgCompetenciaNegociacao.isVisible = true;
        }
    };
    
    this.wdgDescontoPorPeriodoOnEnterTab = function(wdgDescontoPorPeriodo) {
        if (getWidgetNegociacaoDeValores().currentRow.IDTPFATURA != 'X') {
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").readOnly = true;
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").hideEmptyValue = false;
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").defaultValue = null;
        }
        else {
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").readOnly = false;
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").hideEmptyValue = true;
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").defaultValue = "P";
        }
        Utilities.carregarWidgetOnEnterTab(wdgDescontoPorPeriodo);
    };
    
    /**
     * Habilita a widget de desconto por período para o recebimento de um novo registro
     * @params{Object} wdgDescontoPorPeriodo - widget de desconto por período
    **/
    this.wdgDescontoPorPeriodoNewRow = function(wdgDescontoPorPeriodo) {
        Utilities.widgetNewRow(wdgDescontoPorPeriodo);
        this.controlarCamposDescontos(wdgDescontoPorPeriodo);
        
        var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
        wdgDescontoPorPeriodo.currentRow.VRTOTNEGPROP = wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC;
    };

    this.atualizarValorItensNegociacaoValores = function(row) {
        var params = {
            "NRSEQORCAMENTO": row.NRSEQORCAMENTO,
            "NRITEMORCAMENTO": row.NRITEMORCAMENTO,
            "NRITEMORCAMENTOPAI": row.NRITEMORCAMENTOPAI,
            "VRTOTALITEMORCA": row.VRTOTALITEMORCA
        };

        return General.callRestEngine("product#/fct_itemorcamento_widget/atualizarValorItensNegociacaoValores", "Row", params);
    };

    this.salvarParcelaNegociacao = function(wdgParcelas) {
        if (wdgParcelas.isValid()) {
            Utilities.validarCamposDuplicados(wdgParcelas, ["NRPARCELANEGOPROP"], ["QTDIASFATURAMENTO", "NRNEGOCIACAOPROPOSTA"]).then(function(response) {
                if (response) {
                    
                    var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores"); 
                    
                    wdgParcelas.currentRow.NRNEGOCIACAOPROPOSTA = wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA;
                    
                    Utilities.widgetSaveAndSetColumnsToReturn(wdgParcelas).then(function(){
                        
                        var params = {
                            "NRNEGOCIACAOPROPOSTA" : wdgParcelas.currentRow.NRNEGOCIACAOPROPOSTA
                        };
                        
                        General.callRestEngine("product#/fct_orcamento_widget/atualizarNrParcelas", "Row", params).then(function(){
                
                            wdgParcelas.reload();
                        }).catch(function(error) {
                            Utilities.showErrorMessage(error);
                        }); 
                    });
                }
                else {
                    ScreenService.showMessage('Operação bloqueada. <br/>Não é possível inserir 2 registros com o mesmo valor para o campo "Quantidade de Dias para Faturamento".');
                }
            });
        }
    };
    /**
     * Salvar um registro de desconto por período
    **/
    this.salvarDescontoPorPeriodo = function(wdgDescontoPorPeriodo) {
        if (wdgDescontoPorPeriodo.isValid()) {
            this.validarNegociacaoValorDescontoOperadorEPerfil(wdgDescontoPorPeriodo.currentRow).then(function(continuar){ //Pode continuar o save?
                if(continuar){
                    Utilities.validarCamposDuplicados(wdgDescontoPorPeriodo, ["NRDESCPERIODONEGOPROP"], ["NRORDEMAPLICADESC", "NRNEGOCIACAOPROPOSTA"]).then(function(response) {
                        if(!response) {
                            ScreenService.showMessage('Operação bloqueada.<br/>Não é possível inserir 2 registros com o mesmo valor para o campo "Ordem de Aplicação".');
                        } else {
                            if(wdgDescontoPorPeriodo.currentRow.VRPERCDESCITEM != 100) {
                                wdgDescontoPorPeriodo.currentRow.VRPERCDESCITEM   = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRPERCDESCITEM);
                                wdgDescontoPorPeriodo.currentRow.VRDESCITEM       = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRDESCITEM);
                                wdgDescontoPorPeriodo.currentRow.VRTOTITEMCOMDESC = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRTOTITEMCOMDESC);
                                Utilities.widgetSaveAndSetColumnsToReturn(wdgDescontoPorPeriodo);
                            } else {
                                Utilities.validarCamposDuplicados(wdgDescontoPorPeriodo, ["NRDESCPERIODONEGOPROP"], ["VRPERCDESCITEM", "NRNEGOCIACAOPROPOSTA"]).then(function(response) {
                                    if(!response) {
                                        ScreenService.showMessage('Operação bloqueada.<br/>Não é possível inserir 2 registros com 100% de desconto.');
                                    } else {
                                        wdgDescontoPorPeriodo.currentRow.VRPERCDESCITEM   = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRPERCDESCITEM);
                                        wdgDescontoPorPeriodo.currentRow.VRDESCITEM       = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRDESCITEM);
                                        wdgDescontoPorPeriodo.currentRow.VRTOTITEMCOMDESC = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRTOTITEMCOMDESC);
                                        Utilities.widgetSaveAndSetColumnsToReturn(wdgDescontoPorPeriodo);
                                    }
                                });
                            }
                        }
                    });
                }
                // else{
                //   ScreenService.showMessage(ScreenService.i18n("O registro não pode ser salvo pois o perfil ou operador atual não pode efetuar mais descontos nesta faixa.")) 
                // }
            });
        }
    };

    this.abrirWdgParcelarNegociacaoPorIgual = function() {
        var wdgParcelarNegociacaoPorIgual = templateManager.container.getWidget("wdgParcelarNegociacaoPorIgual");
        wdgParcelarNegociacaoPorIgual.isVisible = true;
        ScreenService.openPopup(wdgParcelarNegociacaoPorIgual);
        wdgParcelarNegociacaoPorIgual.dataSource.data = [];
        wdgParcelarNegociacaoPorIgual.newRow();
    };

    this.parcelarNegociacaoPorIgual = function(wdgParcelarNegociacaoPorIgual) {
        if (wdgParcelarNegociacaoPorIgual.isValid()) {
            this.validarQuantidadeMaxParcelas(wdgParcelarNegociacaoPorIgual, "PARCELAPORIGUAL").then(function() {
                var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
                var params = {
                    "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA,
                    "QTPARCELAS"          : wdgParcelarNegociacaoPorIgual.currentRow.QTPARCELAS,
                    "QTDIASINTERVALO"     : wdgParcelarNegociacaoPorIgual.currentRow.QTDIASINTERVALO,
                    "NRDIAVENCIMENTO"     : wdgParcelarNegociacaoPorIgual.currentRow.NRDIAVENCIMENTO,
                    "QTCARENCIADIAS"      : wdgParcelarNegociacaoPorIgual.currentRow.QTCARENCIADIAS,
                    "VRTOTITEMCOMDESC"    : wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC
                };
    
                General.callRestEngine("product#/fct_orcamento_widget/parcelarNegociacaoPorIgual", "Row", params).then(function(response) {
                    ScreenService.closePopup();
                    ScreenService.showMessage("Operação realizada com sucesso!");
                    templateManager.container.getWidget("wdgParcelas").reload();
                })
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };

    this.controlarAcoesDeOperacaoParcelasNegociacao = function(situacao) {
        var wdgParcelas = templateManager.container.getWidget("wdgParcelas");
        var actParcelarPorIgual = wdgParcelas.getAction("parcelarPorIgual");
        //Controla a visibilidade do Parcelar por Igual
        actParcelarPorIgual.isVisible = (situacao == "ELABORACAO" || (situacao == 'CONCLUIDO' && idAlteraPropConc == 'S' && visibilidadeAcoes));
    };
    
    this.pegarStatusCarenciaMaxima = function(){
        var deferred = ZHPromise.defer();
        var params = {};
        General.callRestEngine('product#/fct_parametrogeral_widget/pegarIdControleCarencia', "Row", params).then(function(response) {
            var resposta =  ( response.dataset.request.resulted !== "N");
            deferred.resolve(resposta);
        }).catch(function(e){
            Utilities.showErrorMessage(e);
            deferred.reject(e);
        });
        return deferred.promise;
    };
    
    this.validarNegociacaoValorDescontoOperadorEPerfil = function(row){
        var deferred = ZHPromise.defer();
        var edit = !row.__is_new;
        this.pegarStatusCarenciaMaxima().then(function(statusCarencia){
            if(statusCarencia){
                var params = {
                  "NRNEGOCIACAOPROPOSTA" : row.NRNEGOCIACAOPROPOSTA  
                };
                return General.callRestEngine("product#/fct_itemorcamento_widget/validarNegDescOperadorPerfil", "Row", params);
            }
        }).then(function(response){
            var carencia =  response ?  response.dataset.request.resulted : response;
            var permitir = true;
            if(carencia){ //* Carencia está ativada?
                // #region SETUP
                var parcelas = parseInt(row.QTMESESDESC); 
                var percentual = parseFloat(row.VRPERCDESCITEM);
                // SETAR OPERADOR
                var  operador = (!carencia.CARENCIAMAXOPERADOR[0]) ? null : {
                    "PARCELAS" : carencia.CARENCIAMAXOPERADOR[0].NRPARCELASMAX,
                    "PERCENTUAL": carencia.CARENCIAMAXOPERADOR[0].VRPERCMAXATE
                };
                // SETAR FAIXA OPERADOR
                var faixaOperador=null;
                var qtdMaximaOperador; // maximo de parcelas
                if(carencia.CARENCIA){
                    var faixaOp = carencia.CARENCIA.filter(function(fop){
                        return fop.IDENTIFICADOR === "OPERADOR"; 
                    });
                   qtdMaximaOperador = (!faixaOp[0]) ? 0 : faixaOp[0].MAXIMO;
                    faixaOperador = (!faixaOp) ? null : this.extrairFaixaDeDescontoPorPeriodo(faixaOp, parcelas, percentual);
                }
                // SETAR PERFIL ACESSO
                var perfilAcesso = (!carencia.CARENCIAMAXPERFIL[0]) ? null : {
                    "PARCELAS" : carencia.CARENCIAMAXPERFIL[0].NRPARCELASMAX,
                    "PERCENTUAL": carencia.CARENCIAMAXPERFIL[0].VRPERCMAXATE
                } ; 
                // SETAR FAIXA PERFIL ACESSO
                var faixaPerfil= null;
                var qtdMaximaPerfil; // quantidade maxima por perfil
                if(carencia.CARENCIA){
                    var faixaPa = carencia.CARENCIA.filter(function(fpa){
                        return fpa.IDENTIFICADOR === "PERFIL"; 
                    });
                    qtdMaximaPerfil = (!faixaPa[0]) ? 0 : faixaPa[0].MAXIMO;
                    faixaPerfil = (!faixaPa) ? null : this.extrairFaixaDeDescontoPorPeriodo(faixaPa, parcelas, percentual);
                }
                // APLICAÇÂO DAS REGRAS
                var permitido = this.validarRegrasDesconto(faixaOperador,operador,faixaPerfil,perfilAcesso, parcelas, percentual);
                if(permitido.status){
                    //validar a quantidade de descontos dadas por este operador ou perfil
                    if(permitido.identificador === "operador" && carencia.OPERADOR[0]){
                        var qtdParcelasMax = (edit) ? parseInt(carencia.OPERADOR[0].QTPARCELAS - templateManager.container.getWidget("wdgDescontoPorPeriodo").backupRow.QTMESESDESC) : parseInt(carencia.OPERADOR[0].QTPARCELAS) ;
                        if(edit){
                            if(permitido.faixa){
                               permitido = this.validarQuantidadePermitidaFaixaOperador(qtdParcelasMax, parcelas, faixaOperador, edit);
                            } else if(!permitido.faixa){
                              permitido =  this.validarQuantidadePermitidaOperador(qtdParcelasMax, parcelas, operador, edit);
                            }
                        }else{ 
                            if(permitido.faixa){
                               permitido = this.validarQuantidadePermitidaFaixaOperador(qtdParcelasMax, parcelas, faixaOperador);
                            } else if(!permitido.faixa){
                              permitido =  this.validarQuantidadePermitidaOperador(qtdParcelasMax, parcelas, operador);
                            }
                        }
                    }else if(permitido.identificador === "perfil" && carencia.PERFIL[0]){
                        var qtdParcelasMax = (edit) ? parseInt(carencia.PERFIL[0].QTPARCELAS - templateManager.container.getWidget("wdgDescontoPorPeriodo").backupRow.QTMESESDESC) : parseInt(carencia.PERFIL[0].QTPARCELAS) ;
                        if(edit){
                            if(permitido.faixa){
                               permitido = this.validarQuantidadePermitidaFaixaPerfil(qtdParcelasMax, parcelas, faixaPerfil, edit);
                            } else if(!permitido.faixa){
                              permitido =  this.validarQuantidadePermitidaPerfil(qtdParcelasMax, parcelas, perfilAcesso, edit);
                            }
                        }else{
                            if(permitido.faixa){
                               permitido = this.validarQuantidadePermitidaFaixaPerfil(qtdParcelasMax, parcelas, faixaPerfil);
                            } else if(!permitido.faixa){
                              permitido =  this.validarQuantidadePermitidaPerfil(qtdParcelasMax, parcelas, perfilAcesso);
                            }
                        }
                    }
                }
                
                permitir = permitido.status;
                if(!permitido.status) 
                    ScreenService.showMessage(ScreenService.i18n('Operação Bloqueada! <br/><br/>' + permitido.mensagem));
            }
                deferred.resolve(permitir); 
        }.bind(this)).catch(function(e){
            Utilities.showErrorMessage(e);
            deferred.reject(e); 
        });
        return deferred.promise;
    };
    
    this.validarObj = function(obj, parcelas,percentual){
        var primeira = false;
        var permitir = obj.PERCENTUAL >= percentual ;
        if(!permitir)
            primeira = true;
        else
            permitir = obj.PARCELAS >= parcelas;
        return {"STATUS":permitir,"PRIMEIRA":primeira}
    };
    
    this.validarQuantidadePermitidaOperador = function(qtParcelas,parcelas,operador) {
        var permitir = (qtParcelas + parcelas) <= operador.PARCELAS;
        var permitido ={
            "status" : permitir,
            "mensagem" : (!permitir)? 'O operador logado não tem permissão para dar desconto com quantidade maior que ' + operador.PARCELAS + ' parcelas.': "",
            "identificador":"operador",
            "faixa":false
        };
        return permitido;
    };
    
    this.validarQuantidadePermitidaFaixaOperador = function(qtParcelas,parcelas,faixaOperador) {
        var permitir = (qtParcelas + parcelas) <= faixaOperador.PARCELAS;
        var permitido ={
            "status" : permitir,
            "mensagem" : (!permitir)? 'O operador logado não tem permissão para dar desconto com quantidade maior que ' + faixaOperador.PARCELAS + ' parcelas.': "",
            "identificador":"operador",
            "faixa":false
        };
        return permitido;
    };
    
    this.validarQuantidadePermitidaPerfil = function(qtParcelas,parcelas,perfil){
        var permitir = (qtParcelas+ parcelas) <= perfil.PARCELAS;
        var permitido ={
            "status" : permitir,
            "mensagem" : (!permitir)? 'O perfil de acesso do operador logado não tem permissão para dar desconto com quantidade maior que ' + perfil.PARCELAS + ' parcelas.': "",
            "identificador":"perfil",
            "faixa":false
        };
        return permitido;
    };
    
    this.validarQuantidadePermitidaFaixaPerfil = function(qtParcelas,parcelas,faixaPerfil){
        var permitir = (qtParcelas + parcelas) <= faixaPerfil.PARCELAS;
        var permitido ={
            "status" : permitir,
            "mensagem" : (!permitir)? 'O perfil de acesso do operador logado não tem permissão para dar desconto com quantidade maior que ' + faixaPerfil.PARCELAS + ' parcelas.': "",
            "identificador":"perfil",
            "faixa":true
        };
        return permitido;
    };
    
    this.pegarRecebidoNulo = function(){
        return  {
            "status" : false,
            "mensagem" : "",
            "identificador":"",
            "faixa":false
        };
    };
    
    this.validarRegrasDesconto = function(faixaOperador,operador,faixaPerfil,perfilAcesso, parcelas, percentual){
        /*REGRAS
            Na presença de um OPERADOR DEVE IGNORAR O PERFIL DE ACESSO
            Na presença de uma FAIXA DEVE IGNORAR O MAXIMO PARAMETRIZADO ( PERFIL && OPERADOR)
            1. FAIXA OPERADOR
            	1. ANALISAR SOMENTE A FAIXA
            2. OPERADOR 
            	1. ANALISAR SOMENTE O OPERADOR
            3. FAIXA PERFIL ACESSO 
            	1. ANALISAR FAIXA
            	2. ANALISAR MAXIMO PARAMETRIZADO
            4. MAXIMO PARAMETRIZADO
            	1. ANALISAR MAXIMO PARAMETRIZADO
            5.  Ao inserir deve se pegar a quantidade da soma que existe na tabela(operador e perfil)
            	1.  o da faixa é o valor maximo de parcelas da faixa
            6.  Ao editar deve se pegar todas menos a que esta sendo editada
            	1.  o valor da faixa é o valor maximo de parcelas da faixa
        */
        var permitido =this.pegarRecebidoNulo();
        if(faixaOperador){
            permitido = this.validarFaixaOperador(faixaOperador,parcelas,percentual);
        }else if(operador){
            permitido = this.validarOperador(operador,parcelas,percentual);
        }else if(faixaPerfil){
            permitido = this.validarFaixaPerfilAcesso(faixaPerfil,parcelas,percentual)|| this.validarPerfilAcesso(perfilAcesso,parcelas,percentual);
        }else if(perfilAcesso) {
            permitido =  this.validarPerfilAcesso(perfilAcesso,parcelas,percentual);
        }
        return permitido;
    };
    
    this.validarFaixaOperador = function(faixaOperador,parcelas, percentual){
        var permitir = this.validarObj(faixaOperador,parcelas,percentual);
        return {
            "status":permitir.STATUS,
            "mensagem" : (!permitir.STATUS) ? (!permitir.PRIMEIRA) ? 'O operador logado não tem permissão para dar desconto com quantidade maior que ' + faixaOperador.PARCELAS + 'parcelas.' : 'O operador logado não tem permissão para dar desconto maior que ' + faixaOperador.PERCENTUAL + '%' : "",
            "identificador":"operador",
            "faixa":true
        }
    };
    
    this.validarFaixaPerfilAcesso = function(faixaPerfil,parcelas, percentual){
        var permitir = this.validarObj(faixaPerfil,parcelas,percentual);
        return {
            "status":permitir.STATUS,
            "mensagem" : (!permitir.STATUS) ? (!permitir.PRIMEIRA) ? 'O perfil de acesso do operador logado não tem permissão para dar desconto com quantidade maior que ' + faixaPerfil.PARCELAS + 'parcelas.' : 'O perfil de acesso do operador logado não tem permissão para dar desconto maior que ' + faixaPerfil.PERCENTUAL + '%' : "",
            "identificador":"perfil",
            "faixa":true
        }
    };
    
    this.validarOperador = function(operador,parcelas, percentual){
        var permitir = this.validarObj(operador,parcelas,percentual);
        return {
            "status":permitir.STATUS,
            "mensagem" : (!permitir.STATUS) ? (!permitir.PRIMEIRA) ? 'O operador logado não tem permissão para dar desconto com quantidade maior que ' + operador.PARCELAS +  'parcelas.' : 'O operador logado não tem permissão para dar desconto maior que '+ operador.PERCENTUAL + '%' : "",
            "identificador":"operador",
            "faixa":false
        }
    };
    
    this.validarPerfilAcesso = function(perfil,parcelas, percentual){
        var permitir = this.validarObj(perfil,parcelas,percentual);
        return {
            "status":permitir.STATUS,
            "mensagem" : (!permitir.STATUS) ? (!permitir.PRIMEIRA) ? 'O perfil de acesso do operador logado não tem permissão para dar desconto com quantidade maior que '+ perfil.PARCELAS + 'parcelas.' : 'O perfil de acesso do operador logado não tem permissão para dar desconto maior que ' + perfil.PERCENTUAL + '%' : "",
            "identificador":"perfil",
            "faixa":false
        }
    };
    
    this.extrairFaixaDeDescontoPorPeriodo = function(dados, parcela, valor){
        var obj = null;
        if(dados.length > 0){
           dados.forEach(function(el){
            if(parseFloat(el.VRPERCDESCATE) >= valor && valor >= parseFloat(el.VRPERCDESCDE) && parseInt(el.NRPARCELASMAX) > parcela)
                obj = {
                    "PARCELAS": parseInt(el.NRPARCELASMAX),
                    "PERCENTUAL": parseFloat(el.VRPERCDESCATE)
                };
           });
        }
        return obj;
    };
    
    this.validarDadosCarencia = function(carencia){
        var perfilOperador  = [];    // carencia.PERFIL
        var dadosOperador   = {};    // carencia.OPERADOR
        var dadosPerfil     = [];    // carencia.PERFIS
        var dadosCarencia   = [];    // carencia.CARENCIA
        dadosOperador = carencia.OPERADOR[0];
        this.preencherArray(perfilOperador, carencia.PERFIL);
        this.preencherArray(dadosPerfil, carencia.PERFIS);
        this.preencherArray(dadosCarencia, carencia.CARENCIA);
        return false;
    };
    
    this.preencherArray = function(array, conjunto){
        conjunto.forEach(function(c){
            array.push(c);  
        });
    };

    this.salvarWdgNegociacaoValores = function(wdgNegociacaoValores) {
        if (wdgNegociacaoValores.isValid()) {
            var vrpercdescitem = wdgNegociacaoValores.currentRow.VRPERCDESCITEM;
            if (!vrpercdescitem) vrpercdescitem = 0;
            vrpercdescitem = Utilities.tirarMascaraValores(vrpercdescitem);
            if (vrpercdescitem > 0) {
                var params = {
                    "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA
                };
                General.callRestEngine("product#/fct_itemorcamento_widget/verificarSeExisteDescontoItensNegociacaoProposta", "Row", params)
                .then(function(success) {
                    if (success.dataset.retorno.existe) {
                        ScreenService.showMessage("Operação bloqueada. <br/>Não é possível dar desconto na negociação pois já existe desconto específico informado para seus itens.");
                    }
                    else {
                        General.callRestEngine("product#/fct_itemorcamento_widget/getServicoPercentualMinimoDescontoPermitidoNegociacaoProposta", "Row", params)
                        .then(function(response) {
                            var resultado = response.dataset.retorno.servico;
                            if (resultado && vrpercdescitem > resultado.VRPERCDESC) {
                                var contexto = (resultado.TIPO == 1) ? "Este": "O perfil de acesso deste";
                                ScreenService.showMessage("Operação bloqueada.<br/>"+contexto+" operador não tem autorização para oferecer desconto superior a " + resultado.VRPERCDESC + "% para o serviço \"" + resultado.NMSERVORCA + "\".");
                            }
                            else {
                                salvarNegociacaoValores(wdgNegociacaoValores);
                            }
                        })
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }
                })
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
            else {
                salvarNegociacaoValores(wdgNegociacaoValores);
            }
        }
    };

    var salvarNegociacaoValores = function(wdgNegociacaoValores) {
        var deletarParcelas = (wdgNegociacaoValores.backupRow && wdgNegociacaoValores.backupRow.VRTOTITEMCOMDESC !== wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC);
        
        Utilities.widgetSaveAndSetColumnsToReturn(wdgNegociacaoValores).then(function() {
            setDataSourceTipoDesconto(wdgNegociacaoValores, false);
            
            var params = {
                "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA,
                "VRTOTITEMCOMDESC"    : wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC
            };
            General.callRestEngine("product#/fct_orcamento_widget/atualizarValoresDescontoPorPeriodoDaNegociacao", "Row", params)
            .then(function() {
                var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                this.atualizarValorProposta(wdgPropostaGrid);
                
                if(deletarParcelas) {
                    params = { "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA };
                    General.callRestEngine("product#/fct_orcamento_widget/excluirParcelasNegociacao", "Row", params)
                    .then().catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
                
               // this.atualizarValorRateioItens(wdgNegociacaoValores); por hora, não está sendo utilizado rateio
                if (wdgNegociacaoValores.currentRow.IDTPFATURA == 'P') {
                    var mensagem = "Processo realizado com sucesso! <br><br>Atenção!<br>O valor total da soma das parcelas deve ser igual ao valor do item.";
                    ScreenService.showMessage(mensagem);
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }.bind(this));
    }.bind(this);
    
    this.cancelarSaveNegociacaoValores = function(widget) {
        setDataSourceTipoDesconto(widget, false);
        General.widgetCancelForm(widget);
    };

    this.pegarDadosDeFaturamentoGroup = function(widget){
        return widget.fieldGroups.filter(function(wid){
            return (wid.name == "fdgProdutosFaturamento")
        });
    };

    this.wdgItensNegociacaoAfterMoveRow = function(wdgItensNegociacao) {
        var campoParceiro      = wdgItensNegociacao.getField("NMPRINCIPALPARC");
        var campoValorItem     = wdgItensNegociacao.getField("VRTOTITEM");
        var campoPercDesconto  = wdgItensNegociacao.getField("VRPERCDESCITEM");
        var campoValorDesconto = wdgItensNegociacao.getField("VRDESCITEM");
        var campoTipoDesconto  = wdgItensNegociacao.getField("IDTPDESCONTO");
        var campoTipoMedida    = wdgItensNegociacao.getField("DSTPMEDIDASERV");

        var wdgProposta       = getWidgetProposta();
        var utilizaComposicao = wdgProposta.currentRow.QTITENSTRABSERV > 0;

        var dadosFaturamento =  this.pegarDadosDeFaturamentoGroup(wdgItensNegociacao)[0];
        
        if (wdgItensNegociacao.currentRow.NRITEMORCAMENTOPAI == null && utilizaComposicao) {
            campoValorItem.showOnView     = false;
            campoValorItem.showOnForm     = false;
            campoPercDesconto.showOnView  = false;
            campoPercDesconto.showOnForm  = false;
            campoValorDesconto.showOnView = false;
            campoValorDesconto.showOnForm = false;
            campoTipoDesconto.isVisible   = false;
            campoTipoMedida.showOnView    = false;
            campoTipoMedida.showOnForm    = false;
        } else {
            campoValorItem.showOnView     = true;
            campoValorItem.showOnForm     = true;
            campoPercDesconto.showOnView  = true;
            campoPercDesconto.showOnForm  = true;
            campoValorDesconto.showOnView = true;
            campoValorDesconto.showOnForm = true;
            campoTipoDesconto.isVisible   = true;
            campoTipoMedida.showOnView    = true;
            campoTipoMedida.showOnForm    = true;
        }
        
        dadosFaturamento.isVisible = (wdgItensNegociacao.currentRow._parent == null);
    };
    

    this.salvarWdgItensNegociacao = function(wdgItensNegociacao) {
        if (wdgItensNegociacao.isValid()) {
            var params               = {};
            var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
            var deletarParcelas      = (wdgItensNegociacao.backupRow && wdgItensNegociacao.backupRow.VRTOTITEMCOMDESC !== wdgItensNegociacao.currentRow.VRTOTITEMCOMDESC);
            
            if (!wdgItensNegociacao.currentRow._parent) {
                var mudouParceiro = false;
               
                if (wdgItensNegociacao.currentRow.NRPARCFATURA != wdgItensNegociacao.backupRow.NRPARCFATURA)
                    mudouParceiro = true;

                if (mudouParceiro) {
                    params = { "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA };
                    
                    General.callRestEngine("product#/fct_orcamento_widget/getQuantidadeItensNegociacaoProposta", "Row", params).then(function(response) {
                        var qtdeItens = response.dataset.retorno.quantidade;
                        if (qtdeItens > 1) {
                            ScreenService.confirmMessage("Atenção: <br/>Ao mudar o Parceiro de Negócio deste item, o item será movido deste grupo de Negociação de Valores para um novo grupo. Deseja continuar?", "question", function() {
                                General.widgetSave(wdgItensNegociacao).then(function() {
                                    
                                    setDataSourceTipoDesconto(wdgItensNegociacao, false);
                                    
                                    params = {
                                        "ITEM"      : wdgItensNegociacao.currentRow,
                                        "NEGOCIACAO": wdgNegociacaoValores.currentRow
                                    };
                                    
                                    General.callRestEngine("product#/fct_orcamento_widget/desagruparItemNegociacao", "Row", params).then(function() {
                                        if(deletarParcelas) {
                                            var params = { "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA };
                                            General.callRestEngine("product#/fct_orcamento_widget/excluirParcelasNegociacao", "Row", params)
                                            .then().catch(function(error) {
                                                Utilities.showErrorMessage(error);
                                            });
                                        }
                                        
                                        ScreenService.closeSwipe().then(function() {
                                            ScreenService.closeSwipe();
                                            wdgNegociacaoValores.activate();
                                            wdgNegociacaoValores.reload();
                                        });
                                    }).catch(function(error) {
                                        Utilities.showErrorMessage(error);
                                    });
                                });
                            });
                        }
                        else {
                            General.widgetSave(wdgItensNegociacao);
                            setDataSourceTipoDesconto(wdgItensNegociacao, false);
                        }
                    }).catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
                else {
                    General.widgetSave(wdgItensNegociacao).then(function() {
                        params = {
                            "NRNEGOCIACAOPROPOSTA": wdgItensNegociacao.currentRow.NRNEGOCIACAOPROPOSTA
                        };
                        
                        General.callRestEngine("product#/fct_orcamento_widget/atualizarValorNegociacaoComSomaDosItens", "Row", params)
                        .then(function(response) {
                            setDataSourceTipoDesconto(wdgItensNegociacao, false);
                            var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
                            
                            if(deletarParcelas) {
                                params = { "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA };
                                General.callRestEngine("product#/fct_orcamento_widget/excluirParcelasNegociacao", "Row", params)
                                .then().catch(function(error) {
                                    Utilities.showErrorMessage(error);
                                });
                            }
                            
                            var valorNegociacao = response.dataset.retorno.valorNegociacao;
                            wdgNegociacaoValores.currentRow.VRTOTITEM = valorNegociacao;
                            wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC = valorNegociacao;
                            Utilities.calcularValoresDesconto(wdgNegociacaoValores, 'VRTOTITEM', 'VRPERCDESCITEM', 'VRDESCITEM', 'VRTOTITEMCOMDESC', 'VRPERCDESCITEM');

                            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                            this.atualizarValorProposta(wdgPropostaGrid);
                            //this.atualizarValorRateioItens(wdgItensNegociacao);  Por hora, não será utilizado rateio
                        }.bind(this))
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }.bind(this))
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            }
            else {
                params = {
                    "NRNEGOCIACAOPROPOSTA": wdgItensNegociacao.currentRow.NRNEGOCIACAOPROPOSTA
                };
                
                General.callRestEngine("product#/fct_itemorcamento_widget/verificarSeExisteDescontoNegociacaoProposta", "Row", params)
                .then(function(success) {
                    var percDescontoAtual = wdgItensNegociacao.currentRow.VRPERCDESCITEM;
                    if (!percDescontoAtual) percDescontoAtual = 0;
                    percDescontoAtual = Utilities.tirarMascaraValores(percDescontoAtual);

                    if (success.dataset.retorno.existe && percDescontoAtual > 0) {
                        ScreenService.showMessage("Operação bloqueada. <br/>Não é possível dar desconto diretamente no item pois já existe desconto para o grupo inteiro da negociação de valores.");
                    } else {
                        var nritempai            = wdgItensNegociacao.currentRow.NRITEMORCAMENTOPAI;
                        var valorDescontoServico = 0;
                        var valorServico         = 0;
                        
                        wdgItensNegociacao.dataSource.data.filter(function(row) {
                            return row.NRITEMORCAMENTOPAI == nritempai;
                        }).forEach(function(row) {
                            valorDescontoServico += Utilities.tirarMascaraValores(row.VRDESCITEM);
                            valorServico         += Utilities.tirarMascaraValores(row.VRTOTITEM);
                        });
        
                        params = {
                            "CDSERVORCA": wdgItensNegociacao.currentRow.CDSERVORCA,
                            "IDTIPO"    : "D"
                        };
                        
                        General.callRestEngine("product#/fct_itemorcamento_widget/validaPorcentualDescontoAcrescimoSave", "Row", params).then(function(response) {
                            var resultado = response.dataset.RESULTADO.VRPERCDESC;
                            var percDescontoServico = valorDescontoServico / valorServico * 100;
                            if (resultado && percDescontoServico > resultado.VRPERCDESC) {
                                var contexto = (resultado.TIPO == 1) ? "Este": "O perfil de acesso deste";
                                ScreenService.showMessage("Operação bloqueada.<br/>"+contexto+" operador não tem autorização para oferecer desconto superior a " + resultado.VRPERCDESC + "% para o serviço \"" + resultado.NMSERVORCA + "\".");
                            }
                            else {
                                var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
                                General.widgetSave(wdgItensNegociacao).then(function() {
                                    setDataSourceTipoDesconto(wdgItensNegociacao, false);
                                    if(deletarParcelas) {
                                        params = { "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA };
                                        General.callRestEngine("product#/fct_orcamento_widget/excluirParcelasNegociacao", "Row", params)
                                        .then().catch(function(error) {
                                            Utilities.showErrorMessage(error);
                                        });
                                    }
                                    
                                    params = { "NRSEQORCAMENTO": wdgNegociacaoValores.currentRow.NRSEQORCAMENTO };
                                    if (wdgItensNegociacao.currentRow._parent) {
                                        params = {
                                            "NRNEGOCIACAOPROPOSTA": wdgItensNegociacao.currentRow.NRNEGOCIACAOPROPOSTA,
                                            "NRITEMORCAMENTOPAI"  : wdgItensNegociacao.currentRow.NRITEMORCAMENTOPAI
                                        };
                                        var idParcelamento = wdgNegociacaoValores.currentRow.IDTPFATURA == 'P';
                                        General.callRestEngine("product#/fct_orcamento_widget/atualizarValoresNegociacaoAoEditarElemento", "Row", params).then(function(response2) {
                                            var mensagem = "Processo realizado com sucesso!" + ((idParcelamento) ? "<br><br>Atenção!<br>O valor total da soma das parcelas deve ser igual ao valor do item." : "");
                                            ScreenService.showMessage(mensagem);
                                            
                                            ScreenService.closeSwipe();
                                            wdgItensNegociacao.reload();
                                            
                                            var valorNegociacao      = response2.dataset.retorno.valorNegociacao;
                                            
                                            wdgNegociacaoValores.currentRow.VRTOTITEM = valorNegociacao;
                                            wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC = valorNegociacao;
                                            Utilities.calcularValoresDesconto(wdgNegociacaoValores, 'VRTOTITEM', 'VRPERCDESCITEM', 'VRDESCITEM', 'VRTOTITEMCOMDESC', 'VRPERCDESCITEM');

                                            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                                            this.atualizarValorProposta(wdgPropostaGrid);
                                        }.bind(this))
                                        .catch(function(error) {
                                            Utilities.showErrorMessage(error);
                                        });
                                    }
                                }.bind(this))
                                .catch(function(error) {
                                    Utilities.showErrorMessage(error);
                                });
                            }
                        }.bind(this));
                    }
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }            
        }
    };

    /**
     * Valida as conposições de serviço antes de realizar a renegociação
    **/
    this.validarRenegociacao = function(widget) {
        var params = { "NRSEQORCAMENTO" : widget.currentRow.NRSEQORCAMENTO };
        General.callRestEngine("product#/fct_itemorcamento_widget/validarAtivacaoComposicao", "Row", params).then(function(response){
            this.abrePopupDsJustCancReneg(widget, 'versionar_proposta');
        }.bind(this))
        .catch(function(error){
            if (error.data && error.data.error.indexOf('POSSUI COMPOSICAO INATIVA') !== -1) 
                ScreenService.showMessage("Operação bloqueada!<br>Não é possível renegociar propostas ligadas a composições de serviço inativas.<br>Ative-as e tente novamente!");
            else 
                Utilities.showErrorMessage(error);
        });
    };
    /**
     * Coloca a widget de itens em mode de edição
     * durante a inserção da proposta
     * @params{Object} widget - widget itens proposta inserindo
    **/
    this.controlarCamposItensPropInserindo = function(widget, isEdit){
        if(isEdit) {
            var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
            widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
            widget.edit();
            widget.__is_new = false;
            this.alterarVisibilidadeAcoesItensInserindo(widget, false);
        }
        
        var isVisible   = widget.currentRow.IDTIPO === "M";
        var qtMdVisivel = (widget.currentRow.IDTIPO === "M" && widget.currentRow.IDMEDIDAFIXA === 'N' && widget.currentRow.IDMULTIPVARIACAO === "S");

        widget.getField("QTMEDIDASERV").isVisible  = isVisible;
        widget.getField("QTMEDIDASERV").readOnly   = qtMdVisivel;
        widget.getField("VRUNITITEMORCA").readOnly = (widget.currentRow.CDTPMEDVARESCOPO != null);
        widget.getField("VRUNITITEMORCA").help     = (!widget.currentRow.CDTPMEDVARESCOPO) ? null : "O campo foi desabilitado porque não é possível inserir acréscimos em itens vinculados a serviços faturados por medição.";
        this.alterarVisibilidadeCamposItens(widget, widget.currentRow.IDMEDIDAFIXA);
    };
    /**
     * Altera as actions da tela de itens durante a edição dos mesmo
     * @params{Object} widget  - widget de itens inserindo
     * @params{Bool} isVisible - visibilidade das actions que aparecem no grid 
    **/
    this.alterarVisibilidadeAcoesItensInserindo = function(widget, isVisible){
        widget.getAction("voltar").isVisible       = isVisible;
        widget.getAction("inserir_tudo").isVisible = isVisible;
        widget.getAction("act_cancel").isVisible   = !isVisible;
        widget.getAction("act_salvar").isVisible   = !isVisible;
    };
    /**
     * Volta para o grid de itens durante a insersão de uma proposta de 
     * composição de serviço
     * @params{Object} widget - widget de itens 
    **/
    this.voltarGridItensPropInserindo = function(widget){
        widget.getField("NMTIPOORCAMENTO").isVisible = true;
        widget.getField("VRUNITITEMORCA").isVisible  = true;
        widget.getField("QTMEDIDASERV").isVisible    = true;
        widget.getField("DSTPMEDIDASERV").isVisible  = true;
        
        ScreenService.closeSwipe();
        widget.mode = "edit";
        widget.activate();
        
        this.alterarVisibilidadeAcoesItensInserindo(widget, true);
    };
    /**
     * Salva as alterações realizadas nos itens durante a inserção 
     * de uma proposta de composição de serviço
    **/
    this.salvarAlteracoesItensPropInserindo = function(widget){
        this.replicarVrTotalItemPropInserindo(widget);
        this.alterarVisibilidadeAcoesItensInserindo(widget, true);
        widget.view();
    };
    /**
     * Replica o novo valor do item para seus itens pais na
     * arvore de itens durante a inserção da proposta
     * @params{Object} widget - widget de itens
    **/
    this.replicarVrTotalItemPropInserindo = function(widget){
        if(widget.currentRow.IDTIPO == "M") {
            widget.currentRow.VRUNITITEMORCA_GRID = Utilities.formatarNumero(widget.currentRow.VRUNITITEMORCA);
            widget.currentRow.QTMEDIDASERV_GRID   = Utilities.formatarNumero(widget.currentRow.QTMEDIDASERV);
                
            this.calcularValorTotalItem(widget);
            var cdservorca      = widget.currentRow.CDSERVORCA;
            var indexElSuperior = null;
            var totalServico    = 0;
            widget.dataSource.data.forEach(function(item, index){
                if(cdservorca == item.CDSERVORCA) {
                    if(!item.IDARVOREPAI)
                        indexElSuperior = index;
                    else 
                        totalServico += item.VRTOTALITEMORCA;
                }
            });
            
            if(indexElSuperior !== null) {
                widget.dataSource.data[indexElSuperior].VRTOTALITEMORCA = totalServico;
                widget.dataSource.data[indexElSuperior].VRUNITITEMORCA  = totalServico;
            }
        }
    };
    /**
     * Abre a Widget de negociação de valores para a edição
    **/
    /*this.abrirWidgetNegociacaoValores = function(currentRow) {
        var wdgPropostaGrid    = getWidgetPropostaGrid();
        var wdgProposta        = getWidgetProposta();
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        var wdgNegociacao      = templateManager.container.getWidget("wdgNegociacaoValores");
        
        wdgPropostaGrid.currentRow = {
            "IDSITUACAOORCAMENTO": "ELABORACAO",
            "IDUTILIZACOMP"      : currentRow.IDUTILIZACOMP,
            "IDUTILIZAVARPRECO"  : currentRow.IDUTILIZAVARPRECO,
            "NRSEQORCAMENTO"     : currentRow.NRSEQORCAMENTO,
            "NRORG"              : currentRow.NRORG
        };
        
        setLocalVar("PROPOSTA_NRSEQORCAMENTO", currentRow.NRSEQORCAMENTO);
        setLocalVar("PROPOSTA_NRORG", currentRow.NRORG);
        
        wdgProposta.list();
        wdgProposta.reload().then(function() {
            wdgProposta.view();
            ["NRORCAMENTO", "NMPRINCIPALPARC", "NMOPERRESPONSAVEL"].forEach(function(nameCampo) {
                if(wdgProposta.currentRow[nameCampo]) 
                    wdgProposta.descriptionOrder += ' - ' + wdgProposta.currentRow[nameCampo];
            });
        });
        
        this.validacoesGeraisProposta(wdgPropostaGrid);
        ScreenService.openSwipe(wdgTabbedPrincipal).then(function() {
            wdgTabbedPrincipal.openTab("wdgPropostaTabbed");
            wdgNegociacao.list();
            wdgNegociacao.reload().then(function() {
                wdgNegociacao.activate();
            });
        });
    };*/
    
    /**
     * Cancela a edição dos itens durante a inserção de uma proposta
     * @params{Object} widget - widget Itens proposta inserindo
    **/
    this.cancelarEdicaoItensPropostaInserindo = function(widget) {
        Utilities.widgetCancelForm(widget); 
        this.alterarVisibilidadeAcoesItensInserindo(widget, true);
    };
    /**
     * Volta para o grid de itens 
     * @params{Object} widget - widget de itens tree
    **/
    this.voltarGridItensPropEditando = function(widget){
        widget.getField("NMTIPOORCAMENTO").isVisible = true;
        ScreenService.closeSwipe();
        widget.reload();
    };
    /**
     * Abre o pop up de data do aceite 
    **/
    this.abrirWdgPopupDataDoAceite = function(widget) {
        ScreenService.closePopup();
        var wdgPopup = templateManager.container.getWidget("wdgPopupDataDoAceite");
        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function(){
            wdgPopup.activate();
            wdgPopup.newRow();
            wdgPopup.currentRow.NRSEQORCAMENTO = widget.currentRow.NRSEQORCAMENTO;
            wdgPopup.currentRow.NRORCAMENTO    = widget.currentRow.NRORCAMENTO;
        });
    };
    
    /**
     * valida a quantidade máxima de parcelas da negociação 
     * @params{Object} widget - widget de parcelas
     * @params{String} typeValidacao - tipo de parcela (PARCELAUNICA ou PARCELAPORIGUAL)
    **/
    this.validarQuantidadeMaxParcelas = function(widget, typeValidacao) {
        var deferred      = ZHPromise.defer();
        var rowNegociacao = templateManager.container.getWidget("wdgNegociacaoValores").currentRow;
        var params = {
            "NRNEGOCIACAOPROPOSTA" : rowNegociacao.NRNEGOCIACAOPROPOSTA, 
            "NRTIPOORCAMENTO"      : rowNegociacao.NRTIPOORCAMENTO,
            "IDTPVALIDACAO"        : typeValidacao
        };
        
        General.callRestEngine("product#/fct_parcelanegoprop/validarQuantidadeMaxParcelas", "Row", params).then(function(response) {
            if(response.dataset && response.dataset.fct_orcamento_widget) {
                var maximoparcelas = response.dataset.fct_orcamento_widget.NRMAXPARCELAS;
                
                if (typeValidacao == "PARCELAUNICA") {
                    if (maximoparcelas.indexOf("LIMITE_PARC_INSERIDO") == -1) {
                        deferred.resolve();
                    }
                    else {
                        deferred.reject("Operação bloqueada!<br>Quantidade máxima de parcelas já cadastrada no sistema.<br>Valor máximo permitido: " + maximoparcelas.split(" ")[1]);
                    }
                }
                else if (widget.currentRow.QTPARCELAS > maximoparcelas) {
                    widget.currentRow.QTPARCELAS = "";
                    deferred.reject("Operação bloqueada!<br>A quantidade de parcelas informada excede a quantidade máxima de parcelas da negociação.<br>Valor máximo permitido: " + maximoparcelas);
                }
                else {
                    deferred.resolve();
                }
            }
        }.bind(this));

        return deferred.promise;
    };

    this.novaParcela = function(wdgParcelas) {
        this.validarQuantidadeMaxParcelas(wdgParcelas, 'PARCELAUNICA').then(function() {
            General.widgetNewRow(wdgParcelas);
        }).catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    
    /**
     * valida se a proposta possui rateio inserido e se haverá alterações nele
    **/
    this.validarDadosRateioProposta = function(widget) {
        var deferred = ZHPromise.defer();
        var params   = { 
            "NRSEQORCAMENTO" : getWidgetProposta().currentRow.NRSEQORCAMENTO,
            "NRTIPOORCAMENTO": getWidgetProposta().currentRow.NRTIPOORCAMENTO,
            "NRESTRUTURAM"   : widget.currentRow.NRESTRUTURAM
        };
        
        General.callRestEngine("product#/fct_relacestruturaprop_widget/validarDadosRateioProposta", "Row", params).then(function(response) {
            var rateiosInseridos = false;
            if(response.dataset && response.dataset.fct_relacestruturaprop_widget && response.dataset.fct_relacestruturaprop_widget.length > 0) 
                rateiosInseridos = response.dataset.fct_relacestruturaprop_widget[0];
            return deferred.resolve(rateiosInseridos);
        }.bind(this))
        .catch(function(error) {
            ScreenService.showMessage("Falha ao validar os rateios da proposta");
            return deferred.resolve();
        });
        
        return deferred.promise;
    };
    /**
     * valida se o item de formação que está sendo inserido pode iterferir no 
     * rateio da proposta
     * @params{Object} widget - widget de itens de formação de preço
     * @params{bool} soma - se os itens serão somados ou não
    **/
    this.validarDadosRateioFormacaoPreco = function(widget, soma) {
        var deferred = ZHPromise.defer();
        this.validarDadosRateioProposta(widget).then(function(rateiosInseridos) {
            if(!rateiosInseridos) {
                this.insereItemFormPrec(widget, soma).then(function(resolve) {
                    deferred.resolve(resolve);
                });
            } else {
                var mensagem = (rateiosInseridos == "SEM_CENTROCUST") 
                    ? "Atenção!<br><br>Os locais de atendimento das formações de preço selecionadas, não está ligados a um centro de custo. Desse modo, não serão gerados rateios para os itens dessa proposta.<br>Deseja continuar?"  
                    : "Atenção!<br><br>O item selecionado não possui local de atendimento ligado a ele. Sua inserção acarretará na exclusão dos rateios criados para essa proposta.<br><br>Deseja continuar?";
                
                ScreenService.confirmMessage(mensagem, "question", function() {
                    this.insereItemFormPrec(widget, soma).then(function(resolve) {
                        deferred.resolve(resolve);
                    });
                }.bind(this));
            }
        }.bind(this));
        return deferred.promise;
    };
    /**
     * valida se o item de formação que está sendo inserido pode iterferir no 
     * rateio da proposta
     * @params{Object} widget - widget de itens de formação de preço
     * @params{bool} soma - se os itens serão somados ou não
    **/
    this.validarDadosRateioItens = function(widget) {
        widget.currentRow.NRESTRUTURAM = [null];
        this.validarDadosRateioProposta(widget).then(function(rateiosInseridos) {
            if(!rateiosInseridos) {
                this.salvarItens(widget);
            } else {
                ScreenService.confirmMessage("Atenção!<br><br>A inserção deste item impedirá a geração de rateios para os faturamentos ligados aos contratos dessa proposta.<br><br>Deseja continuar?", "question", function() {
                    this.salvarItens(widget);
                }.bind(this));
            }
        }.bind(this));
    };
    
    /**
     * Verifica se a proposta utiliza rateio e atualiza os valor deles
     * @params{Object} widget - widget de rateio 
    **/
    this.atualizarValorRateioItens = function(widget) {
        var idutilizacomp = getWidgetProposta().currentRow.IDUTILIZACOMP;
        if(!idutilizacomp || idutilizacomp == 'N') {
            var nritemorcamento = widget.currentRow.NRITEMORCAMENTO ? widget.currentRow.NRITEMORCAMENTO : null;
            var params = {
                "NRITEMORCAMENTO"      : nritemorcamento,
                "NRNEGOCIACAOPROPOSTA" : widget.currentRow.NRNEGOCIACAOPROPOSTA,
                "NRSEQORCAMENTO"       : getWidgetProposta().currentRow.NRSEQORCAMENTO
            };
            
            General.callRestEngine("product#/fct_itemorcamento_widget/atualizarValorRateioItens", "Row", params).then()
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    this.servicoOnChange = function(widget) {
        this.getValorUnitarioTblPrecoItem(widget);
        UtilitiesPropostaContrato.setarMedidaVariavelEscopo(widget, "P");
    };

    /**
     * Busca o valor do serviço na tabela de preço do escopo
     * @params{Object} widget - widget de itens (editando)
    **/
    this.getValorUnitarioTblPrecoItem = function(widget) {
        if (widget.currentRow.CDPRODSERV && widget.currentRow.TIPOPRODSERV == "S") {
            var wdgProposta = getWidgetProposta();
            var params = {
                "NRTIPOORCAMENTO"    : widget.currentRow.NRTIPOORCAMENTO,
                "CDSERVORCA"         : widget.currentRow.CDPRODSERV,
                "QTMEDIDASERV"       : widget.currentRow.QTMEDIDAORCA ? widget.currentRow.QTMEDIDAORCA : null,
                "CDTPMEDIDAVARIAVEL" : widget.currentRow.CDTPMEDIDAVARIAVEL ? widget.currentRow.CDTPMEDIDAVARIAVEL : null,
                "NRMOEDA"            : wdgProposta.currentRow.NRMOEDA
            };
            
            General.callRestEngine("product#/fct_itemorcamento_widget/getValorUnitarioTblPrecoItemProposta", "Row", params).then(function(response) {
                var quantidade = widget.currentRow.QTITEMORCAMENTO ? widget.currentRow.QTITEMORCAMENTO : 1;
                var qtdMedida = widget.currentRow.QTMEDIDAORCA ? widget.currentRow.QTMEDIDAORCA : 1;
                if (!response.dataset || !response.dataset.fct_itemorcamento_widget) {
                    this.calcularValoresItemOrcamento(widget);
                }
                else {
                    if (widget.currentRow.VRUNITITEMORCATBLPRECO != response.dataset.fct_itemorcamento_widget.VRPRECO) {
                        widget.currentRow.VRUNITITEMORCA = response.dataset.fct_itemorcamento_widget.VRPRECO;
                    }
                    widget.currentRow.VRUNITITEMORCATBLPRECO = response.dataset.fct_itemorcamento_widget.VRPRECO;
                    widget.currentRow.VRTOTALITEMORCA = widget.currentRow.VRUNITITEMORCA * quantidade * qtdMedida;
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
        else {
            this.calcularValoresItemOrcamento(widget);
            widget.currentRow.VRUNITITEMORCATBLPRECO = "";
        }
    };
    
    /**
     * Valida o tipo de tratamento que será dado aos acréscimos e descontos 
     * após a alteração dos elementos e medidas
    **/
    this.validarTpTratamentoDescAcres = function(widget, tpAtualizacao) {
        var params = { "NRSEQORCAMENTO" : getWidgetProposta().currentRow.NRSEQORCAMENTO };
        General.callRestEngine("product#/fct_orcamento_widget/validarExistenciaDescontoAcrescimoProposta", "Row", params).then(function(response) {
            if(!response.dataset || !response.dataset.fct_orcamento_widget || !response.dataset.fct_orcamento_widget[0]) {
                if(tpAtualizacao == 'ELEMENTOS') this.atualizarComposicao(widget, 'S');
                else this.atualizarTipoDeMedidaProposta(widget, 'S');
            } else {
                ScreenService.showCustomDialog("A atualização que deseja fazer resultará na redefinição dos valores dos itens e das negociações."
                    + "<br><br>O que deseja fazer com os acréscimos e descontos desta proposta?", 'CONFIRMATION', null, 
                    [
                        {
                            "label": "Descartá-los",
                            "code": 
                                function() {
                                    if(tpAtualizacao == 'ELEMENTOS') this.atualizarComposicao(widget, 'N');
                                    else this.atualizarTipoDeMedidaProposta(widget, 'N');
                                }.bind(this)
                        },
                        {
                            "label": "Mantê-los",
                            "code": 
                                function() {
                                    if(tpAtualizacao == 'ELEMENTOS') this.atualizarComposicao(widget, 'S');
                                    else this.atualizarTipoDeMedidaProposta(widget, 'S');
                                }.bind(this)
                        },
                        {
                            "label": "Cancelar",
                            "code": function() {}
                        }
                    ]
                );
            }
        }.bind(this))
        .catch(function(error) {
            if(tpAtualizacao == 'ELEMENTOS') this.atualizarComposicao(widget, 'N');
            else this.atualizarTipoDeMedidaProposta(widget, 'N');
        });
    };
    
    /**
     * Valida e habilita a edição da tela de negociação de valores
     * @param {Object} widget - widget de negociação de valores
    **/
    this.editarDescontoPorPeriodo = function(widget) {
        this.controlarCamposDescontos(widget);
        Utilities.widgetEdit(widget);
    };
    
    /**
     * controla as fields de desontonto de acordo com o parametro de tipo de desconto
     * @param {Object} widget
    **/
    this.controlarCamposDescontos = function(widget) {
        var tpDesconto = widget.currentRow.IDTPDESCONTO;
        if (!tpDesconto) {
            if (getWidgetNegociacaoDeValores().currentRow.IDTPFATURA != 'X') {
                widget.getField("VRDESCITEM").readOnly     = false;
                widget.getField("VRPERCDESCITEM").readOnly = false;
            }
            else if (widget.getField("IDTPDESCONTO").isVisible) {
                widget.getField("VRDESCITEM").readOnly     = true;
                widget.getField("VRPERCDESCITEM").readOnly = true;

                widget.getField("VRDESCITEM").value(0);
                widget.getField("VRPERCDESCITEM").value(0);
                Utilities.calcularValoresDesconto(widget, "VRTOTITEM", "VRPERCDESCITEM", "VRDESCITEM", "VRTOTITEMCOMDESC", "VRPERCDESCITEM");
            }
            
            widget.getField("VRDESCITEM").validations     = {};
            widget.getField("VRPERCDESCITEM").validations = {};
        }
        else {
            widget.getField("VRDESCITEM").readOnly     = false;
            widget.getField("VRPERCDESCITEM").readOnly = false;
            
            widget.getField("VRDESCITEM").validations     = {"required": true};
            widget.getField("VRPERCDESCITEM").validations = {"required": true};
        }
        
        templateManager.updateTemplate();
    };
    
    /**
     * Coloca a widget de negociacao ou itens da negociação em modo de edição
     * @params{Object} widget - Widget de negociação ou de itens da negociação
    **/
    this.editarNegociacaoValores = function(widget) {
        this.controlarCamposDescontos(widget);
        var wdgNegociacaoValores = getWidgetNegociacaoDeValores();
        var fieldTipoDesconto = widget.getField("IDTPDESCONTO");
        
        if (wdgNegociacaoValores.currentRow.IDTPFATURA != 'X') {
            fieldTipoDesconto.readOnly = true;
            General.widgetEdit(widget);
        }
        else {
            var params = { "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA };
            var rota = (widget.name == 'wdgNegociacaoValores') ? "verificarSeExisteDescontoItensNegociacaoProposta" : "verificarSeExisteDescontoNegociacaoProposta";
            
            General.callRestEngine("product#/fct_itemorcamento_widget/" + rota, "Row", params).then(function(success) {
                var existeOutrosDescontos = success.dataset.retorno.existe;
                fieldTipoDesconto.readOnly = existeOutrosDescontos;
                
                return self.validarMedicaoServico(widget.currentRow.NRNEGOCIACAOPROPOSTA, wdgNegociacaoValores.currentRow.NRTIPOORCAMENTO);
            })
            .then(function(medicaoDeServico) {
                if (medicaoDeServico)
                    setDataSourceTipoDesconto(widget, true);
                
                General.widgetEdit(widget);
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    this.validarMedicaoServico = function(nrnegociacaoproposta, nrtipoorcamento) {
        var deferred = ZHPromise.defer();
        var params = {
            "NRNEGOCIACAOPROPOSTA": nrnegociacaoproposta,
            "NRTIPOORCAMENTO"     : nrtipoorcamento
        };
        General.callRestEngine("product#/fct_orcamento_widget/validarMedicaoServico", "Row", params).then(function(response) {
            var medicaoDeServico = response.dataset.retorno.medicaoServico;
            deferred.resolve(medicaoDeServico);
        })
        .catch(function(error) {
            deferred.reject(error);
        });
        
        return deferred.promise;
    };
    
    /**
     * Filtra a field de serviço/produto para trazer apenas os tipos
     * permitidos pelo escopo.
     * 
     * @params{Object} widget - widget de itens manuais da proposta
    **/
    this.filtrarTipoDeItensManuaisDoEscopo = function(widget) {
        var idtipoorca    = widget.currentRow.IDTIPOORCA;
        var fieldServProd = widget.getField("NMITEMORCAMENTO");
        var posFiltroTipo = undefined;

        if(!fieldServProd.dataSourceFilter) fieldServProd.dataSourceFilter = [];
        
        if(!idtipoorca) {
            // if(fieldServProd.dataSourceFilter && fieldServProd.dataSourceFilter.length)
            //     fieldServProd.dataSourceFilter.pop();
            posFiltroTipo = Utilities.getFilterPosition(fieldServProd.dataSourceFilter, "TIPO");
            if (posFiltroTipo >= 0) {
                delete fieldServProd.dataSourceFilter[posFiltroTipo];
            }
        } else {
            idtipoorca = (idtipoorca == 'T') ? ['S', 'V'] : [idtipoorca];

            posFiltroTipo = Utilities.getFilterPosition(fieldServProd.dataSourceFilter, "TIPO");
            if (posFiltroTipo >= 0) {
                fieldServProd.dataSourceFilter[posFiltroTipo].value = idtipoorca;
            }
            else {
                fieldServProd.dataSourceFilter.push({
                    name     : "TIPO",
                    operator : "IN",
                    value    : idtipoorca
                });
            }
        }
    };
    
    this.validarVisibilidadeWidgetTpMedida = function(wdgTpMedida) {
        var wdgProposta = getWidgetPropostaGrid();
        var params = [
            {
                name: "NRSEQORCAMENTO",
                operator: "=",
                value: wdgProposta.currentRow.NRSEQORCAMENTO
            }/*,
            {
                name: "NRTIPOORCAMENTO",
                operator: "IN",
                value: wdgProposta.currentRow.NRTIPOORCAMENTO
            }*/
        ];
        
        General.callRestEngine("product#/crud/fct_relacmedidaproposta_widget", "FilterData", params).then(function(response) {
            if(response.dataset && response.dataset.fct_relacmedidaproposta_widget && response.dataset.fct_relacmedidaproposta_widget.length)
                wdgTpMedida.isVisible = true;
            else 
                wdgTpMedida.isVisible = false;
        });
    };
    
    /**
     * Busca nas parametrizações se o modelo de proposta é obrigatório
    **/
    this.validarObrigatoriedadeModelo = function() {
        var wdgProposta = getWidgetProposta();
        if(idObrigaModProp != null) {
            wdgProposta.getField("NMORCAMENTOMODELO").validations = (!idObrigaModProp) ? {} : { "required": true };
        } else {
            idObrigaModProp = false;
            General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", []).then(function(response) {
                if(response.dataset && response.dataset.fct_parametrogeral && response.dataset.fct_parametrogeral.length > 0) {
                    var parametrosService = response.dataset.fct_parametrogeral;
                    idObrigaModProp = (parametrosService.IDOBRIGMODORC == 'S');
                    wdgProposta.getField("NMORCAMENTOMODELO").validations = (idObrigaModProp == "N") ? {} : { "required": true };
                }
            }.bind(this));
        }
    };
    
    this.updateDsAlteracoesContr = function(widget) {
        
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        
        widget.currentRow.NRSEQORCAMENTO = wdgProposta.currentRow.NRSEQORCAMENTO;
        
        var params = { 
            "NRSEQORCAMENTO"      : widget.currentRow.NRSEQORCAMENTO,
            "DSALTERACONTRATUAIS" : widget.currentRow.DSALTERACONTRATUAIS
        };
        
        General.callRestEngine("product#/fct_orcamento_widget/updateDsAlteracoesContr", "Row", params).then(function(response) {
            
            ScreenService.showMessage("Atenção! <br><br> A alteração realizada será considerada para todos os escopos. Caso queira definir Alterações Contratuais específicas para cada escopo, favor alterar na aba 'Escopos de Faturamento'.");
            General.widgetCancelForm(widget);
            widget.view();
            widget.reload();
                
        });
    };
    
    this.getAlteracaoContratual = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
         
        widget.currentRow.NRSEQORCAMENTO = wdgProposta.currentRow.NRSEQORCAMENTO;
            
        var params = { 
            "NRSEQORCAMENTO" : widget.currentRow.NRSEQORCAMENTO
        };
    
        General.callRestEngine("product#/fct_orcamento_widget/getAlteracaoContratual", "Row", params).then(function(response) {
            
            widget.currentRow.DSALTERACONTRATUAIS = response.dataset.getAlteracaoContratual.resultado;
            
        }); 
    };

    /**
     * Valida se o operador tem acesso à página "Alterações Contratuais"
    **/
    this.validaAlteracoesContratuais = function(widget){
        
        var wdgAlteracoesContratuais = templateManager.container.getWidget("wdgAlteracoesContratuais");
        
        var params = {};
        
        General.callRestEngine("product#/fct_orcamento_widget/validaAlteracoesContratuais", "Row", params).then(function(response) {
            if (response.dataset.validaAlteracoesContratuais.resultado == 'HABILITADO') {
            
                wdgAlteracoesContratuais.isVisible = true;
            }
            else{
                wdgAlteracoesContratuais.isVisible = false; 
            }
        });
    };
    
    /**
     * insere as assinaturas da proposta 
    **/
    this.saveAssinaturaOrca = function(widget) {
        if (widget.currentRow.__is_new){
            
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            
            General.callRestEngine('product#/fct_orcamento_widget/validaAssinaturaOrca', "Row", widget.currentRow).then(function(response) {
                
                var retorno = response.dataset.validaAssinaturaOrca.valida;
                
                /* Recebe os envolvidos para notificação*/
                var envolvidos = null;
                var arrayEnvolvidos = widget.currentRow.DSEMAILNOTSIGNENV_INS;
                
                if (arrayEnvolvidos != null && arrayEnvolvidos != ""){
                    envolvidos = arrayEnvolvidos.join('_');
                }

                widget.currentRow.DSEMAILNOTSIGNENV = envolvidos; 
                
                /* Recebe os operdores para notificação */
                var arrayOperadores = widget.currentRow.OPERADORES;
                
                if(retorno == '0') {
                            
                    var params = {
                        "DADOS"           : widget.currentRow,
                        "NRTIPOORCAMENTO" : wdgProposta.currentRow.NRTIPOORCAMENTO,
                        "CDIDIOM"         : wdgProposta.currentRow.CDIDIOM
                    };
                    
                    General.callRestEngine('product#/fct_orcamento_widget/inserirSignatario', "Row", params).then(function(response) {
            
                        var resultado = response.dataset.inserirSignatario.resultado;
                        
                        if (resultado['STATUS'] == 'OK') {
                            
                            if (Array.isArray(arrayOperadores) && arrayOperadores.length > 0) {
                                var params = {
                                    "NRASSDIGITALORCA" : resultado['NRASSDIGITALORCA'],
                                    "OPERADORES"       : arrayOperadores
                                };
            
                                General.callRestEngine('product#/fct_orcamento_widget/insereOperadoresNotifica', "Row", params).then(function() {
                                    widget.widgets.forEach(function(wid) {
                                        wid.enabled = true;
                                    });
                                    ScreenService.closeSwipe();
                                    widget.reload();
                                })
                                .catch(function(error) {
                                    Utilities.showErrorMessage(error);
                                });
                            }
                            else {
                                ScreenService.closeSwipe();
                                widget.reload();
                            }
                        }
                        else {
                            ScreenService.showMessage(resultado);   
                        }
                        
                    }, function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
                else{
                    ScreenService.showMessage("Operação Bloqueada! <br><br> O signatário informado já foi inserido.");
                }
            });
        }
        else{
            
            /* Recebe os envolvidos para notificação*/
            var envolvidos = null;
            var arrayEnvolvidos = widget.currentRow.DSEMAILNOTSIGNENV_INS;
            
            if (arrayEnvolvidos != null){
                envolvidos = arrayEnvolvidos.join('_');
            }
            
            widget.currentRow.DSEMAILNOTSIGNENV = envolvidos;
            
            /* Recebe os operdores para notificação */
            var operadoresIguais = false;
            
            var bkpOperadores = widget.backupRow.OPERADORES;
            var arrayOperadores = widget.currentRow.OPERADORES;
            
            if (Utilities.arraysSaoIguais(bkpOperadores, arrayOperadores)) {
                operadoresIguais = true;
            }
            
            General.widgetSave(widget).then(function(response){
                
                if (operadoresIguais == false) {
                    var params = {
                        "NRASSDIGITALORCA" : widget.currentRow.NRASSDIGITALORCA,
                        "OPERADORES"       : arrayOperadores
                    };

                    General.callRestEngine('product#/fct_orcamento_widget/insereOperadoresNotifica', "Row", params).then(function() {
                        widget.reload().then(function() {
                            this.carregaDadosOperadores(widget);
                        }.bind(this));
                    }.bind(this))
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
                else {
                    widget.reload();
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    /**
     * insere as assinaturas da proposta através do Modelo de Proposta
    **/
    this.inserirAssinaturasOrca = function(widget){
        
        General.callRestEngine("product#/fct_orcamento_widget/inserirAssinaturasOrca", "Row", widget.currentRow).then(function() {

        }, function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    
    /**
     * Valida se o token da certsign está preenchido
     * Tela: Lançamento de Proposta
     */
    this.validaIdCertsign = function() {
        var wdgAssinaturasOrca = templateManager.container.getWidget("assinaturasOrca");
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
        
        var params = {
            "NRSEQORCAMENTO" : wdgPropostaGrid.currentRow.NRSEQORCAMENTO
        };
        General.callRestEngine('product#/fct_orcamento_widget/validaIdCertsign', "Row", params).then(function(response) {
            
            var idTokenCertsign   = response.dataset.validaIdCertsign.DSTOKENCERTSIGN;
            var idAssEletrDigital = response.dataset.validaIdCertsign.IDASSELETRDIGITAL;
            var iddocumcertsign   = response.dataset.validaIdCertsign.IDDOCUMCERTSIGN;
            
            
            //OTIMIZACAO : Diminuir o numero de acessos ao mesmo campo 
            var campoAssinatura = wdgProposta.getField('IDASSELETRDIGITAL');
            
            if (idTokenCertsign == null){
                
                /* Bloqueia a página "Signatários Padrões" */
                wdgAssinaturasOrca.enabled = false;
                
                /* Bloqueia o campo "Utiliza Assinatura Eletrônica ou Digital" */
                campoAssinatura.readOnly = true;
                campoAssinatura.validations = {};
                if (wdgProposta.currentRow.changed == true){
                    campoAssinatura.value('N');
                }
                this.contolaActionAssEletrDigital(wdgProposta, wdgPropostaGrid,  false, iddocumcertsign);
            }
            else{
                if (idAssEletrDigital == 'N'){
                    /* Bloqueia a página "Signatários Padrões" */
                    wdgAssinaturasOrca.enabled = false;
                    this.contolaActionAssEletrDigital(wdgProposta, wdgPropostaGrid, false, iddocumcertsign);
                }
                else {
                    /* Libera a página "Signatários Padrões" */
                    if (wdgProposta.currentRow.changed != true){
                        wdgAssinaturasOrca.enabled = true;
                        this.contolaActionAssEletrDigital(wdgProposta, wdgPropostaGrid, true, iddocumcertsign);
                    }
                }

                /* Libera o campo "Utiliza Assinatura Eletrônica ou Digital" */
                campoAssinatura.readOnly = false;
                campoAssinatura.validations = {"required" : true};    
            }
        }.bind(this));
    };

    /**
     * Habilita o campo Revenda, caso o campo 'Utiliza Revenda?' da tela de Modelo de Proposta esteja preenchida com 'Sim'". 
     * @param {object} widget widget de 
     * Laçamento de Proposta
    */
    
    this.habilitaCampoRevenda = function(){

        var wdgPropostaForm = templateManager.container.getWidget("wdgProposta");
        
        if(wdgPropostaForm.currentRow.IDUTILIZAREVENDA == 'N'){
            wdgPropostaForm.getField("NMREVENDA").readOnly = true;
        }else {
            wdgPropostaForm.getField("NMREVENDA").readOnly = false; 
        }
       
    };
    
    this.mudaTipoSignatario = function(wdgInterlocutor) {
        if (wdgInterlocutor === undefined) {
            wdgInterlocutor = null;
        }

        if(!wdgInterlocutor)
            wdgInterlocutor = templateManager.container.getWidget("wdgNewInterlocutor");
            
        var tipoSignatario = wdgInterlocutor.currentRow.IDTPSIGNATARIO;
        var obrigatorio = tipoSignatario != 'N';
        wdgInterlocutor.getField("NRCPFINTERLOC").validations = (obrigatorio)?{"required":true}:{}; 
        wdgInterlocutor.getField("DTNASCIMENTO").validations =  (obrigatorio)?{"required":true}:{}; 
        wdgInterlocutor.getField("EMAILINTERLOC").validations =  (obrigatorio)?{"required":true}:{}; 
    };
    
    /**
     * Envia novamente o e-mail de assinatura para os signatários que ainda não realziaram a assinatura
     */
    this.solicitarAssinatura = function() {
        
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        
        var params = {
            "NRSEQORCAMENTO"  : wdgProposta.currentRow.NRSEQORCAMENTO,
            "NRTIPOORCAMENTO" : wdgProposta.currentRow.NRTIPOORCAMENTO,
        };
        
        General.callRestEngine('product#/fct_orcamento_widget/solicitarAssinatura', "Row", params).then(function(response) {  
            
            var resultado = response.dataset.solicitarAssinatura.resultado;
            
            if (resultado == 'OK'){
                
                ScreenService.showMessage("Lembrete de assinatura enviado com sucesso!");
            }
            else if (resultado == 'OK_N_DOC'){
                
                ScreenService.showMessage("Lembrete de assinatura enviado com sucesso! <br><br> Obs: Alguns escopos de faturamento ainda não tiveram os seus documentos enviados para assinatura eletrônica/digital. Estes escopos foram desconsiderados.");
            }
            else if (resultado == 'N_DOC'){
                
                ScreenService.showMessage("Operação bloqueada! <br><br> Não foram enviados os documentos para assinatura eletrônica / digital.");
            }
            else{

                ScreenService.showMessage("Ocorreram erros no lembrete de assinatura. <br><br>Favor entrar em contato com o administrador do sistema. <br/>Erro: ".resultado);  
            }
        });
    };

        /**
     * Verifica se foi realizada a assinatura
     */

        this.verificarAssinatura = function() {
        
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            var wdgassinaturasOrca = templateManager.container.getWidget("assinaturasOrca");
            
            var params = {
                "NRSEQORCAMENTO"  : wdgProposta.currentRow.NRSEQORCAMENTO,
                "NRTIPOORCAMENTO" : wdgProposta.currentRow.NRTIPOORCAMENTO,
            };
            
            General.callRestEngine('product#/fct_orcamento_widget/verificarAssinatura', "Row", params).then(function(response) {  
                
                var resultado = response.dataset.verificarAssinatura.resultado;
            
                if (resultado == 'OK'){
    
                    ScreenService.showMessage("Assinatura verificada com sucesso!");
                    wdgassinaturasOrca.reload();
                    ScreenService.closePopup();
                }
                else{
                    ScreenService.showMessage("Ocorreram erros na verificação de assinatura. <br><br>Favor entrar em contato com o administrador do sistema. <br/>Erro: ".resultado);  
                    ScreenService.closePopup();
                }
            });
        };    
    
    /**
     * Exclui o signatário do Certsign e altera a situação para 'Assinado Manualmente'.
     */
    this.assinadoManualmente = function(widget, checked) {
        
        ScreenService.confirmMessage("Atenção!<br><br>Ao confirmar a assinatura manual, todos os documentos vinculados à este signatário serão atualizados para a situação 'Assinado Manualmente'. <br><br>Caso não deseje assinar manualmente todos os documentos, a rotina deve ser realizada individualmente para cada documento na aba 'Documentos'.<br><br>Deseja continuar?", "question", function() {

            var signatario = null;
            var situacao = null;
            
            if (checked == true) {    
                signatario = widget.getCheckedRows();
            }
            else{
                signatario = widget.currentRow;
                situacao = widget.currentRow.IDSITUACAO;
            }
            
            if (((checked  == false) && (situacao == 'P')) || (checked  == true)) {
                
                var params = {
                    "SIGNATARIO" : signatario,
                    "CHECKED" : checked
                };
                
                General.callRestEngine('product#/fct_orcamento_widget/assinadoManualmente', "Row", params).then(function(response) {
                    
                    var resultado = response.dataset.assinadoManualmente.resultado;
                    
                    if (resultado == 'OK') {
                        
                        if (checked == true) {
                            ScreenService.closePopup();
                            ScreenService.showMessage("Situação dos signatários atualizada com sucesso!");
                            widget.dataSource.clearCheckedRows();
                            widget.reload();
                        }
                        else {
                            ScreenService.showMessage("Situação do signatário atualizada com sucesso!");
                            ScreenService.closeSwipe().then(function() {
                                widget.reload();
                            });
                        }
                    }
                    else if (resultado == 'OK_N_DOC'){
                        if (checked == true) {
                            ScreenService.closePopup();
                            ScreenService.showMessage("Situação dos signatários atualizada com sucesso! <br><br> Obs: Alguns escopos de faturamento ainda não tiveram os seus documentos enviados para assinatura eletrônica/digital. Estes escopos foram desconsiderados.");
                            widget.dataSource.clearCheckedRows();
                            widget.reload();
                        }
                        else {
                            ScreenService.showMessage("Situação do signatário atualizada com sucesso! <br><br> Obs: Alguns escopos de faturamento ainda não tiveram os seus documentos enviados para assinatura eletrônica/digital. Estes escopos foram desconsiderados.");
                            ScreenService.closeSwipe().then(function() {
                                widget.reload();
                            });
                        }
                    }
                    else if (resultado == 'N_DOC'){
                        
                        ScreenService.showMessage("Operação bloqueada! <br><br> Não foram enviados os documentos para assinatura eletrônica / digital.");
                    }
                    else{
                        ScreenService.closePopup();
                        ScreenService.showMessage("Ocorreram erros na atualização da situação dos signatários. Favor entrar em contato com o administrador do sistema. <br/>Erro: " . resultado);
                    }
                })
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
            else {
                
                ScreenService.showMessage("Operação Bloqueada! <br><br> Só é permitida a assinatura manual para signatários que estejam com a situação 'Pendente'."); 
            }
        }.bind(this));
    };
    
    /**
     * Exclui o signatário do Certsign e altera a situação para 'Assinado Manualmente'.
     */
    this.assinadoManualmenteDoc = function(widget, checked) {
        
        var signatario = null;
        var situacao = null;
        
        if (checked == true) {    
            signatario = widget.getCheckedRows();
        }
        else{
            signatario = widget.currentRow;
            situacao = widget.currentRow.IDSITUACAO;
        }
        
        if (((checked  == false) && (situacao == 'P')) || (checked  == true)) {
            
            var params = {
                "SIGNATARIO" : signatario,
                "CHECKED" : checked
            };
            
            General.callRestEngine('product#/fct_orcamento_widget/assinadoManualmenteDoc', "Row", params).then(function(response) {
                
                var resultado = response.dataset.assinadoManualmenteDoc.resultado;
                
                if (resultado == 'OK') {
                    
                    if (checked == true) {
                        ScreenService.closePopup();
                        ScreenService.showMessage("Situação dos signatários atualizada com sucesso!");
                        widget.dataSource.clearCheckedRows();
                        widget.reload();
                    }
                    else {
                        ScreenService.showMessage("Situação do signatário atualizada com sucesso!");
                        ScreenService.closeSwipe().then(function() {
                            widget.reload();
                        });
                    }
                }
                else if (resultado == 'OK_N_DOC'){
                    if (checked == true) {
                        ScreenService.closePopup();
                        ScreenService.showMessage("Situação dos signatários atualizada com sucesso! <br><br> Obs: Alguns escopos de faturamento ainda não tiveram os seus documentos enviados para assinatura eletrônica/digital. Estes escopos foram desconsiderados.");
                        widget.dataSource.clearCheckedRows();
                        widget.reload();
                    }
                    else {
                        ScreenService.showMessage("Situação do signatário atualizada com sucesso! <br><br> Obs: Alguns escopos de faturamento ainda não tiveram os seus documentos enviados para assinatura eletrônica/digital. Estes escopos foram desconsiderados.");
                        ScreenService.closeSwipe().then(function() {
                            widget.reload();
                        });
                    }
                }
                else if (resultado == 'N_DOC'){
                    
                    ScreenService.showMessage("Operação bloqueada! <br><br> Não foram enviados os documentos para assinatura eletrônica / digital.");
                }
                else{
                    ScreenService.closePopup();
                    ScreenService.showMessage("Ocorreram erros na atualização da situação dos signatários. Favor entrar em contato com o administrador do sistema. <br/>Erro: " . resultado);
                }
            });
        }
        else {
            
            ScreenService.showMessage("Operação Bloqueada! <br><br> Só é permitida a assinatura manual para signatários que estejam com a situação 'Pendente'."); 
        }
    };
    
    /**
     * exclui os signatários da proposta e do ambiente Certsign 
    **/
    this.excluirSignatariosCertsign = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        
        var nrseqorcamento   = widget.currentRow.NRSEQORCAMENTO;
        var idsignatcertsign = widget.currentRow.IDSIGNATCERTSIGN;
        var nrcpfsignatario  = widget.currentRow.NRCPFSIGNATARIO;
        var nrassdigitalorca = widget.currentRow.NRASSDIGITALORCA;
        
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
            var params         = {
                    "NRSEQORCAMENTO"   : nrseqorcamento,
                    "NRASSDIGITALORCA" : nrassdigitalorca,
                    "NRTIPOORCAMENTO"  : wdgProposta.currentRow.NRTIPOORCAMENTO,
                    "IDSIGNATCERTSIGN" : idsignatcertsign,
                    "NRCPFSIGNATARIO"  : nrcpfsignatario
                };
                
            General.callRestEngine("product#/fct_orcamento_widget/excluirSignatariosCertsign", "Row", params).then(function(response) {
                
                var resultado = response.dataset.excluirSignatariosCertsign.resultado;
                
                if (resultado == 'OK') {
                    widget.removeRow().then(function () {
                        
                        ScreenService.closeSwipe().then(function () {
        					widget.activate();
                            widget.reload();
        				});
                    },
                    function (error) {
        				if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
        					ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
        				} else {
        					ScreenService.showMessage(error.data.error);
        				}
        			});
                } else {
                    
                    ScreenService.showMessage(resultado);
                }
            },
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        });
    };
    
    this.wdgAssinaturasOnEnterTab = function(widget) {
        Utilities.carregarWidgetOnEnterTab(widget);
        this.filtraParcnegocioAssinat();
        this.controlaEdicaoAssinat(widget);
        this.controlaAcoesAssinat(widget);
    };
    
    this.wdgAssinaturasAfterMoveRow = function(widget) {
        this.controlaAcoesAssinat(widget);
        this.carregaDadosOperadores(widget);
    };
    
    this.wdgAssinaturasVoltarView = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.currentRow.changed = false;
        });
    };
    
    this.wdgAssinaturasCancelar = function(widget) {
        var isNew = widget.currentRow.__is_new;
        General.widgetCancelForm(widget);
        if (!isNew)
            widget.currentRow.changed = false;
            
        this.carregaDadosOperadores(widget);
    };
    
    /**
     * filtra parcnegocio para datasouce do signatário
    **/
    this.filtraParcnegocioAssinat = function(){
        
        var wdgAssinaturasOrca = templateManager.container.getWidget("assinaturasOrca");
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
   
        wdgAssinaturasOrca.getField("NMPRINCIPALPARC").dataSourceFilter[0].value = wdgProposta.currentRow.NRPARCNEGOCIO;
    };
    
    /**
     * A função "contolaActionAssEletrDigital" controla a visibilidade da action de 
     * geração de documento para a assinatura eletrônica e digital.
     * Tela: Lançamento de Proposta
    **/
    this.contolaActionAssEletrDigital = function(wdgProposta, wdgPropostaGrid, visible, iddocumcertsign){
        var situacao = wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO;

        var assEletrDigital     = wdgProposta.getAction('assinatura_eletronica_digital');
        var downDocEletrDigital = wdgProposta.getAction('download_Doc_Eletr_Digital');
        var actHabAssEletrDig   = wdgProposta.getAction("habilitar_assinatura_eletronica_digital");
        
        if (situacao == 'CONCLUIDO') {
            if (iddocumcertsign == false) {
                downDocEletrDigital.isVisible = false;
                assEletrDigital.isVisible = visible;
                actHabAssEletrDig.isVisible  = true;
            }
            else {
                downDocEletrDigital.isVisible = visible;
                assEletrDigital.isVisible = false;
                actHabAssEletrDig.isVisible = false;
            }
        }
        else if (situacao == 'ELABORACAO' || situacao == 'CANCELADO') {
            downDocEletrDigital.isVisible = false;
            assEletrDigital.isVisible = false;
            actHabAssEletrDig.isVisible  = false;
        }
        /* ACEITO */
        else {
            downDocEletrDigital.isVisible = visible;
            assEletrDigital.isVisible = false;
            actHabAssEletrDig.isVisible = false;
        } 
    };
    
    /**
     * realiza o download do documento do ambiente Certsign 
    **/
    this.downloadDocEletrDigital = function(widget){
        
        var nrseqorcamento  = widget.currentRow.NRSEQORCAMENTO;
        var nrtipoorcamento = widget.currentRow.NRTIPOORCAMENTO;
        nrtipoorcamento.forEach(function(codEscopo) {
            window.open(window.backendIndexUrl + "/fct_orcamento_widget/downloadDocEletrDigital?requestType=DataSet&dataset[0][NRSEQORCAMENTO]=" + nrseqorcamento + "&dataset[0][NRTIPOORCAMENTO]=" + codEscopo);
        });
    };
    
    this.controlaEdicaoAssinat = function(widget){
        
        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
        
        var situacao = wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO;
        
        var edit = widget.getAction('EDIT');
        var actAssinaManual = widget.getAction('assinadoManualmente');
        var actAssinaManualChk = widget.getAction('assinadoManualmenteChk');
        var actSolicitaAss = widget.getAction('solicitarAssinatura'); 
        var actAlteraSituAss = widget.getAction('AlteraSituAss');
        var actAlteraSituAssChk = widget.getAction('AlteraSituAssChk');
        
        if (situacao == 'ELABORACAO') {
        
            edit.isVisible = true;
            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = false;
            actAlteraSituAss.isVisible = false;
            actAlteraSituAssChk.isVisible = false;
        }
        else if ((situacao == 'CONCLUIDO') || (situacao == 'ACEITO')){

            edit.isVisible = false;
            actAssinaManual.isVisible = true;
            actAssinaManualChk.isVisible = true;
            actSolicitaAss.isVisible = true;
            actAlteraSituAss.isVisible = true;
            actAlteraSituAssChk.isVisible = true;
        } 
        else /*CANCELADO*/ {
            edit.isVisible = false;
            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = false;
            actAlteraSituAss.isVisible = false;
            actAlteraSituAssChk.isVisible = false;
        }
        if(!edit.isVisible && situacao !== "CONCLUIDO" && situacao !== "CANCELADO"){
            var params = {
                "NRSEQORCAMENTO" : widget.currentRow.NRSEQORCAMENTO
            };
            General.callRestEngine('product#/fct_orcamento_widget/validaIdCertsign', "Row", params).then(function(response) {  
                edit.isVisible = response.dataset.validaIdCertsign.DSTOKENCERTSIGN;
                
            });
        }    
    };
    
    this.controlaAcoesAssinat = function(widget){
        var actAssinaManual = widget.getAction('assinadoManualmente');
        var actAssinaManualChk = widget.getAction('assinadoManualmenteChk');
        var actSolicitaAss = widget.getAction('solicitarAssinatura'); 
        var actAlteraSituAss = widget.getAction('AlteraSituAss');
        var actAlteraSituAssChk = widget.getAction('AlteraSituAssChk');
        
        var situacao = widget.currentRow.IDSITUACAO;
        var excluir = widget.getAction('EXCLUIR');
        
        if (situacao == 'P') {
            this.controlaEdicaoAssinat(widget);
            excluir.isVisible = true;
        }
        else {
            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = true;
            excluir.isVisible = false;
            actAlteraSituAss.isVisible = false;
            actAlteraSituAssChk.isVisible = false;
        }
    };
    
    
     this.controlaEdicaoAssinatDoc = function(widget){
        
        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
        
        var situacao = wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO;
        
        var actAssinaManual = widget.getAction('assinadoManualmenteDoc');
        var actAssinaManualChk = widget.getAction('assinadoManualmenteDocChk');
        var actSolicitaAss = widget.getAction('solicitarAssinaturaDoc'); 
        
        if (situacao == 'ELABORACAO') {
        
            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = false;
        }
        else if ((situacao == 'CONCLUIDO') || (situacao == 'ACEITO')){

            actAssinaManual.isVisible = true;
            actAssinaManualChk.isVisible = true;
            actSolicitaAss.isVisible = true;
        } 
        else /*CANCELADO*/ {
            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = false;
        }
        
    };
    
    this.controlaAcoesAssinatDoc = function(widget){
        var actAssinaManual = widget.getAction('assinadoManualmenteDoc');
        var actSolicitaAss = widget.getAction('solicitarAssinaturaDoc');
        var actAssinaManualChk = widget.getAction('assinadoManualmenteDocChk');
        var situacao = widget.currentRow.IDSITUACAO;
        
        if (situacao == 'P') {
            this.controlaEdicaoAssinatDoc(widget);
        }
        else {
            actAssinaManual.isVisible = false;
            actSolicitaAss.isVisible = false;
            actAssinaManualChk.isVisible = false;
        }
    };
    
    /**
     * A função "assinaturaEletrDigital" realiza a geração do documento de contrato no Certsign
    **/
    this.assinaturaEletrDigital = function(widget){
        
       var params = {
            "NRSEQORCAMENTO" : widget.currentRow.NRSEQORCAMENTO
       };
           
       General.callRestEngine('product#/fct_orcamento_widget/validaIdCertsign', "Row", params).then(function(response) {  
            var idTokenCertsign = response.dataset.validaIdCertsign.DSTOKENCERTSIGN;
            var idAssEletrDigital = response.dataset.validaIdCertsign.IDASSELETRDIGITAL;
            
            if ((idTokenCertsign != false ) && (idAssEletrDigital == 'S')){
                
                var wdgProposta = templateManager.container.getWidget("wdgProposta");
                
                var params = {
                    "NRSEQORCAMENTO"     : widget.currentRow.NRSEQORCAMENTO,
                    "NRTIPOORCAMENTO"    : wdgProposta.currentRow.NRTIPOORCAMENTO,
                    "IDSITUACAOORCAMENTO": wdgProposta.currentRow.IDSITUACAOORCAMENTO,
                    "NRORCAMENTO"        : wdgProposta.currentRow.NRORCAMENTO,
                    "CDIDIOM"           : wdgProposta.currentRow.CDIDIOM
                };
                
                var timeout = 5 * 60000; //5 min
                Utilities.callRestEngineWithTimeout("product#/fct_orcamento_widget/gerarDocumentoCertsign", "Row", params, timeout)
                .then(function(response) {
                    
                    var resultado = response.dataset.gerarDocumentoCertsign.resultado;
                    
                    if (resultado == 'OK') {
                        ScreenService.closePopup();
                        ScreenService.showMessage('Documentos enviados para assinatura eletrônica e digital com sucesso!');
                    } 
                    else {
                       
                       ScreenService.showMessage(resultado);
                    }
                this.validaIdCertsign();
                }.bind(this), function(error) {
                    Utilities.showErrorMessage(error);
                }.bind(this));
            }
       }.bind(this));
    };
    
    this.onEnterTabParcelasNegociacao = function(widget){
        
        var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores"); 
        
        widget.dataSourceFilter[0].value = wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA;
        
        Utilities.carregarWidgetOnEnterTab(widget);
        
    };
    
    this.deleteParcelasNegociacao = function(widget){
        
        General.deleteCrud(widget).then(function(){
            
            var params = {
                "NRNEGOCIACAOPROPOSTA" : widget.currentRow.NRNEGOCIACAOPROPOSTA
            };
            
            General.callRestEngine("product#/fct_orcamento_widget/atualizarNrParcelas", "Row", params).then(function(){
                
                widget.reload();
            }).catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        });
   
    };
    
    this.habilitarAssinaturaEletrDigital = function(widget){
        
        var idasseletrdigital = widget.currentRow.IDASSELETRDIGITAL;
        
        General.callRestEngine("product#/fct_orcamento_widget/habilitarAssinaturaEletrDigital", "Row", widget.currentRow).then(function(response){
                
            var resultado = response.dataset.habilitaAssinatura.resultado;
                    
            if (resultado == 'OK') {
                ScreenService.closePopup();
                widget.reload().then(function() {
                    this.validaIdCertsign();
                }.bind(this));
                
                if (idasseletrdigital == 'N'){
                    ScreenService.showMessage('A proposta foi habilitada para assinatura eletrônica e digital.');
                }
                else {
                    ScreenService.showMessage('A proposta foi desabilitada para assinatura eletrônica e digital.');
                }
            } 
            else {
               
               ScreenService.showMessage(resultado);
            }
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
        
    };
    
    
    this.carregaFiltrosServico = function(widget){
        
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        
        var params = {
            "NRTIPOORCAMENTO" : wdgProposta.currentRow.NRTIPOORCAMENTO
        };
        
        General.callRestEngine("product#/fct_orcamento_widget/validaFiltroServico", "Row", params).then(function(response){
                
            var resultado = response.dataset.validaFiltroServico.resultado;
            
            if (widget.name == 'itemFormacaoPreco') {
                
                widget.getField("NRFORMPRECOSERVICO").dataSourceFilter[1].value = resultado;
                widget.getField("NRFORMPRECOSERVICO").dataSourceFilter[2].value = wdgProposta.currentRow.NRTIPOORCAMENTO;
                
                widget.getField("NMSERVORCA").dataSourceFilter[0].value = resultado;
                widget.getField("NMSERVORCA").dataSourceFilter[1].value = wdgProposta.currentRow.NRTIPOORCAMENTO;
                
            }
            else {
            
                widget.getField("NMITEMORCAMENTO").dataSourceFilter[0].value = resultado;
                widget.getField("NMITEMORCAMENTO").dataSourceFilter[1].value = wdgProposta.currentRow.NRTIPOORCAMENTO;
            }
                    
            
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
        
    };
    
    this.controlaActionEnviarEmail = function(widget){
      
        var actEnviarPorEmail = widget.getAction("enviar_documento");
        
        var params = {
            "NRSEQORCAMENTO" : widget.currentRow.NRSEQORCAMENTO
        };
           
        General.callRestEngine('product#/fct_orcamento_widget/validaIdCertsign', "Row", params).then(function(response) {  
            var idTokenCertsign = response.dataset.validaIdCertsign.DSTOKENCERTSIGN;
            var idAssEletrDigital = response.dataset.validaIdCertsign.IDASSELETRDIGITAL;
            
            if ((idTokenCertsign != false ) && (idAssEletrDigital == 'S')){
                
                General.callRestEngine('product#/fct_orcamento_widget/validaStatusAssinaturas', "Row", params).then(function(response) { 
                    
                    var valida = response.dataset.validaStatusAssinaturas.resultado;
                    
                    if (valida == 'N_Pendente') {
                        
                        actEnviarPorEmail.isVisible = false;
                        
                    }
                    else {
                        
                        actEnviarPorEmail.isVisible = true; 
                    }
                })
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
            else {
                
                actEnviarPorEmail.isVisible = true;
            }
        })
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    

    this.recebeOrdenacaoENotificacoes = function(widget){
      
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        
        var orcamentoModelo = wdgProposta.currentRow.NRORCAMENTOMODELO;
        var nrseqinterlocutor = widget.currentRow.NRSEQINTERLOCUTOR;
        var idtestemunha = widget.currentRow.IDTESTEMUNHA;
        var nomeSignatario = widget.currentRow.NMPRINCIPALPARC;
        var tipoSignat = null;
        
        if (!Util.isEmptyOrBlank(nomeSignatario)) {
            
            if (nrseqinterlocutor != null) {
            
                if (idtestemunha == 'S'){
                    tipoSignat = 'TC';  
                }
                else {
                    tipoSignat = 'SC';  
                }
            }
            else {
             
                if (idtestemunha == 'S'){
                    tipoSignat = 'TP';  
                }
                else {
                    tipoSignat = 'SP';  
                }   
            }
          
            var params = {
                "NRORCAMENTOMODELO" : orcamentoModelo,
                "IDTPSIGNATARIO" : tipoSignat
            };
               
            General.callRestEngine('product#/fct_orcamento_widget/recebeOrdenacaoENotificacoes', "Row", params).then(function(response) {
                var dados = response.dataset.recebeOrdenacaoENotificacoes.resultado;
                
                if (dados['DADOS'] == 'OK'){
                    widget.getField("NRORDENACAO").readOnly = true;
                    widget.currentRow.NRORDENACAO = dados['NRORDENACAO'];
                    
                    var dsemailnotsignenvAux = dados['DSEMAILNOTSIGNENV'];
                    var dsemailnotsignenv = null;
                    if (!Util.isEmptyOrBlank(dsemailnotsignenvAux)) {
                        dsemailnotsignenv = dsemailnotsignenvAux.split('_');
                    }
                    
                    widget.currentRow.DSEMAILNOTSIGNENV_INS = dsemailnotsignenv;
                    widget.currentRow.DSEMAILNOTSIGNENV     = dsemailnotsignenv;
                    widget.currentRow.OPERADORES            = dados['OPERADORES'];
                    widget.currentRow.CDOPERADOR            = dados['OPERADORES'];
                    widget.currentRow.DSEMAILNOTIFOUTROS    = dados['DSEMAILNOTIFOUTROS'];
                }
                else {
                    widget.getField("NRORDENACAO").readOnly = false;
                    widget.currentRow.NRORDENACAO           = 1;
                    widget.currentRow.DSEMAILNOTSIGNENV_INS = null;
                    widget.currentRow.DSEMAILNOTSIGNENV     = null;
                    widget.currentRow.OPERADORES            = null;
                    widget.currentRow.CDOPERADOR            = null;
                    widget.currentRow.DSEMAILNOTIFOUTROS    = null;
                }
                
                this.carregaDadosOperadores(widget);
    
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    this.carregaDadosOperadores = function(widget){
       var operadores = widget.currentRow.OPERADORES;
       if (operadores && operadores.length > 0) {
            widget.getField("OPERADORES").dataSourceFilter[2] = {
                "name": "CDOPERADOR",
                "operator": "IN",
                "value": operadores
            };
                                                                
            widget.getField("OPERADORES").reload();
            widget.getField("OPERADORES").dataSourceFilter.splice(2);
            templateManager.updateTemplate();
        }
    };
    
    this.editarSignatario = function(widget) {
        General.widgetEdit(widget);
        widget.getField("NRORDENACAO").readOnly = true;
    };
    
    this.assinaturaEletronicaOnChange = function(widget) {
        var idasseletrdigital = widget.currentRow.IDASSELETRDIGITAL;
        var fieldEnviaDocOperResp = widget.getField("IDENVIADOCOPERRESPCONCASS");
        if (idasseletrdigital == "N") {
            fieldEnviaDocOperResp.readOnly = true;
            fieldEnviaDocOperResp.value("N");
        }
        else {
            fieldEnviaDocOperResp.readOnly = false;
        }
    };
    
    this.calcularValorUnitario = function(widget){
        var row = widget.currentRow;
        row.VRUNITITEMORCA = this.pegarValorUnitario(row);
        
        this.validarPermissaoAlteracaoValorUnitario(widget).then(null).catch(function() {
            widget.name == 'wdgItensProposta' ? this.calcularValoresItemOrcamento(widget) : this.calcularValorTotalItem(widget);
        }.bind(this));
    };
    
    this.pegarValorUnitario = function(row){
        var valor = Utilities.tirarMascaraValores(row.VRTOTALITEMORCA) / parseFloat(row.QTITEMORCAMENTO);
        if (row.IDMULTMEDVRTOT == "S")
            valor = valor / parseFloat(row.QTMEDIDAORCA);
        
        return parseFloat(valor.toFixed(8));
    };
    
    this.abrirPopupAlteraSituAss = function(widget, check) {
        ScreenService.closePopup();
        var wdgPopup = templateManager.container.getWidget("WdgPopupAlteraSituAssina");
        wdgPopup.isVisible = true;
        wdgPopup.dataSource.data = [];
        wdgPopup.currentRow = {};
        ScreenService.openPopup(wdgPopup).then(function() {
            wdgPopup.activate();
            wdgPopup.newRow();
            wdgPopup.currentRow.IDALTERAEMMASSA = check;
            
            if (check == true){
            
                assinaturas = [];
                rows = widget.getCheckedRows();
                
                rows.forEach(function(row) {
                   
                    assinaturas.push(row['NRASSDIGITALORCA']) ;
                });

            }
            else {
                assinaturas = widget.currentRow.NRASSDIGITALORCA;
                
            }
            
            wdgPopup.getField("DOCUMENTOS").dataSourceFilter[1] = {
                                                                        "name": "NRASSDIGITALORCA",
                                                                        "operator": "IN",
                                                                        "value": assinaturas
                                                                    };
            wdgPopup.getField("DOCUMENTOS").reload();
                
        }.bind(this));
    };
    
    this.validaSituAlteraAss = function(wdgPopup){
        
        if (wdgPopup.currentRow.IDSITUACAO == 'R') {
            
            wdgPopup.getField("DSJUSTIFICARECUSA").readOnly = false;
            wdgPopup.getField("DSJUSTIFICARECUSA").validations = { "required": true };
        }
        else {
         
            wdgPopup.getField("DSJUSTIFICARECUSA").readOnly = true;
            wdgPopup.getField("DSJUSTIFICARECUSA").validations = {};   
        }
        wdgPopup.reload();
    }
    
    this.salvarAlteraSituAssina = function(widget){
        
        if (widget.isValid()) {
            
            var widAssina = templateManager.container.getWidget('assinaturasOrca');
            var idalteraemmassa = widget.currentRow.IDALTERAEMMASSA;
            
            if (idalteraemmassa == true){
            
                var params = {
                    "WIDASSINATURA" : widAssina.getCheckedRows(),
                    "WIDSITUACAO" : widget.currentRow
                };
                    
            }
            else {
                
                var params = {
                    "WIDASSINATURA" : widAssina.currentRow,
                    "WIDSITUACAO" : widget.currentRow
                };
                
            }
            
            General.callRestEngine("product#/fct_orcamento_widget/salvarAlteraSituAssina", "Row", params).then(function(response){
                    
                var resultado = response.dataset.salvarAlteraSituAssina.resultado;
                        
                if (resultado == 'OK') {
                    ScreenService.closePopup();
                    widAssina.activate().then(function() {
                        widAssina.reload().then(function() {
                            this.controlaAcoesAssinat(widAssina);
                        }.bind(this));
                        if (idalteraemmassa == true){
                            ScreenService.showMessage('Situação da(s) assinatura(s) alterada com sucesso.');
                        }
                        else {
                            ScreenService.showMessage('Situação da assinatura alterada com sucesso.');
                        }
                    }.bind(this));
                    
                } 
                else {
                   Utilities.showErrorMessage(resultado);
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
        
    };

    this.moedaOnChange = function(widget) {
        if ((!widget.currentRow.__is_new && !mudouMoeda) &&
            (widget.backupRow && widget.backupRow.NRMOEDA != widget.currentRow.NRMOEDA) ) {
            mudouMoeda = true;
            ScreenService.showMessage(ScreenService.i18n("Atenção! <br><br>Mudar a moeda de uma proposta não fará com que o valor dos itens e dos descontos sejam recalculados automaticamente. Será necessário ajustar o valor dos itens manualmente para a nova moeda."));
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00001000_LancamentoProposta_Controller", Sm00001000_LancamentoProposta_Controller);
});



// FILE: ./assets/js/controllers/Sm00001000_LancamentoPropostaAlimenticia_Controller.js
/**
 * Controller JS para a tela de lançamento de propostas alimentícias
**/
function Sm00001000_LancamentoPropostaAlimenticia_Controller(Operador_Repository, ZHPromise, General, Utilities, templateManager,ScreenService, Sm00001000_LancamentoProposta_Service, UtilitiesPropostaContrato, Sm00001000_PropostaAlimenticia_Repository){
    
    var opLogado                = null;
    var nomeOperador            = null;
    var idUtilizaHierOper       = null;
    var idOperMaster            = null;
    var idAlteraPropConc        = null;
    var onClickMobra            = null;
    var onClickMobraItens       = null;
    var nrparcnegocioOperLogado = null;
    var tpOperador              = null;
    var nrOrgOper               = null;
    var codeRelacFormacao       = null;
    var countMObra              = -1;
    var idItensMObraAlterado    = false;
    /**
     * Coloca na currentRow os parametrôs de novo código
     * para a inserção de novos registros
     * @params{Object} widget - widget que irá inserir um novo registro
    **/
    this.criarParametrosNewCode = function(widget){
        if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
            widget.currentRow.__using_unit      = widget.paramsNewCode.__using_unit || false;
            widget.currentRow.__new_code_pk     = widget.paramsNewCode.__new_code_pk;
            widget.currentRow.__table_pk        = widget.paramsNewCode.__table_pk;
            widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
            widget.currentRow.__cast_number     = widget.paramsNewCode.__cast_number || false;
            if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
        }
    };
    /**
     * Busca as informações do operador logado e 
     * verifica se o sistema utiliza hierarquia de operadores 
    **/
    this.buscaOperadorLogado = function(){
        var deferred = ZHPromise.defer();
        Operador_Repository.getOperadorLogado().then(function(success) {
            opLogado                = success[0].CDOPERADOR;
            nomeOperador            = success[0].NMOPERADOR;
            nrparcnegocioOperLogado = success[0].NRPARCNEGOCIO;
            tpOperador              = success[0].IDTPOPERADOR;
            nrOrgOper               = success[0].NRORG;
            this.buscaInformacaoHierarquia();
            deferred.resolve(success);
        }.bind(this));
        return deferred.promise;
    };
    /**
     * Verifica se o sistema utiliza hierarquia de operadores e 
     * caso use busca sua posição
     * @params{Object} widgetPropostaGrid - widget grid de propostas 
     * @params{Object} widget - dados da proposta
     **/
    this.verificaHierarquia = function(widgetPropostaGrid, widget) {
        if(idOperMaster === null && idUtilizaHierOper === null){
            this.buscaInformacaoHierarquia();
        }
        widget.currentRow.CDOPERRESPONSAVEL = opLogado;
        widget.currentRow.CDNOVOOPERRESP    = opLogado;
        widget.currentRow.NMOPERRESP        = nomeOperador;
    };
    /**
     * busca as informações da hierarquia de operadores 
     * ligada ao usuário logado.
    **/
    this.buscaInformacaoHierarquia = function() {
        General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", {}).then(function(success) {
            if (success.dataset.fct_parametrogeral && success.dataset.fct_parametrogeral.length > 0) {
                var paramsFilter = [
                    {
                        name: "CDOPERADOR",
                        operator: "=",
                        value: opLogado
                    }
                ];
                idUtilizaHierOper = success.dataset.fct_parametrogeral[0].IDUTILIZAHIEROPER;
                General.callRestEngine("product#/crud/crm_hierarquiaoper", "FilterData", paramsFilter).then(function(success) {
                    if (success.dataset.crm_hierarquiaoper && success.dataset.crm_hierarquiaoper.length > 0) 
                        idOperMaster = success.dataset.crm_hierarquiaoper[0].IDOPERMASTER;
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Abre a widget de Dados da Proposta (view/form)
     **/
    this.abrirSwipeProposta = function() {
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        ScreenService.openSwipe(wdgTabbedPrincipal).then(function() {
            wdgTabbedPrincipal.openTab("wdgPropostaTabbed");
            wdgProposta.list();
            wdgProposta.reload().then(function() {
                wdgTabbedPrincipal.view();
                wdgProposta.view();
                wdgProposta.activate();
                this.controlarEscopoProposta(wdgProposta, 'S');
                //if(wdgProposta.getField("NMTIPOORCAMENTO").isVisible && wdgProposta.currentRow.NMTIPOORCAMENTO instanceof Array)
                //    wdgProposta.currentRow.NMTIPOORCAMENTO = wdgProposta.currentRow.NMTIPOORCAMENTO[0];
            }.bind(this));
        }.bind(this));
    };
    /**
     * verifica se o sistema permite a alteração de propostas já concluídas
     * @params{Object} widget - widget de proposta grid
    **/
    this.verificaParamAlteraPropConc = function(widget){
        var deferred = ZHPromise.defer();
        if(idAlteraPropConc){
            deferred.resolve();
        } else {
            var paramsGeral = [
                {
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("PROPOSTA_NRORG")
                }
            ];
            General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", paramsGeral).then(function(success) {
                if(success.dataset.fct_parametrogeral.length !== 0)
                    idAlteraPropConc = success.dataset.fct_parametrogeral[0].IDALTERAPROPCONC;
                deferred.resolve();
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
                deferred.reject();
            });
        }
        return deferred.promise;
    };
    /**
     * Faz o controle geral da tela de Proposta de acordo com o registro selecionado.
     * @param {object} widget - Widget de Proposta (grid)
     */
    this.validacoesGeraisProposta = function(wdgPropostaGrid) {
        var situacao    = wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO;
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        this.controlarAcoesDeCrud(wdgProposta, situacao);
        this.controlarAcoesDeOperacao(wdgProposta, situacao);
    };
    /**
     * Verifica a situação da Proposta para permitir/bloquear sua edição e/ou exclusão.
     * @param {object} wdgProposta - Widget de Dados da Proposta 
     * @param {String} situacao - Situação da Proposta
     */
    this.controlarAcoesDeCrud = function(wdgProposta, situacao) {
        var actDelete = wdgProposta.getAction("delete");
        var actEdit = wdgProposta.getAction("edit");
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        if(situacao == "CONCLUIDO"){
            wdgProposta.getAction("edit").isVisible = (idAlteraPropConc === "S") ? true : false;
            actDelete.isVisible = false;
            this.controlarAcoesDeCrudAux(wdgTabbedPrincipal, false, situacao);
        } else {
            if (situacao == "ELABORACAO") {
                actDelete.isVisible = true;
                actEdit.isVisible   = true;
                this.controlarAcoesDeCrudAux(wdgTabbedPrincipal, true, situacao);
            } else {
                actEdit.isVisible   = false;
                actDelete.isVisible = (situacao == "CANCELADO");
                this.controlarAcoesDeCrudAux(wdgTabbedPrincipal, false, situacao);
            }
        }
    };
    /**
     * Muda a visibilidade das ações de crud das widgets filhas de proposta
     * @params {Object} widget - widget tabbed principal
     * @params {boolean} visivel - estado em que a visibilidade das actions devem ficar
     * @params {String} situacao - situação da proposta
     **/
    this.controlarAcoesDeCrudAux = function(widget, visivel, situacao) {
        var widgetsIgnoradas = ["wdgProposta", "wdgPropostaTabbed"];
        widget.widgets.filter(function(wid) {
                return widgetsIgnoradas.indexOf(wid.name) == -1;
            }).forEach(function(wid) {
                this.setVisibilidadeDasAcoesDeCrud(wid, visivel, situacao);
                if (wid.widgets.length > 0)
                    this.controlarAcoesDeCrudAux(wid, visivel, situacao);
            }.bind(this));
    };
    /**
     * Ativa/desativa todas as actions novo, editar e excluir de uma widget.
     * @param {object} widget - widget que terá as ações alteradas
     * @param {boolean} visivel - estado em que a visibilidade das actions devem ficar
     * @params {String} situacao - situação da proposta
     */
    this.setVisibilidadeDasAcoesDeCrud = function(widget, visivel, situacao) {
        var actions = ["add", "edit", "delete"];
        if(situacao === "CONCLUIDO"){
            actions.forEach(function(act){
                if(widget.getAction(act))
                    widget.getAction(act).isVisible = (idAlteraPropConc === "S") ? true : false;   
            });
            actions = [];
        }
        
        widget.actions.forEach(function(action) {
            if(actions.includes(action.name)) {
                action.isVisible = visivel;
            }
        });
    };
    /**
     * Executa validações da proposta ao trocar de linha.
     * @param {object} wdgPropostaGrid - Widget de Proposta (grid)
     */
    this.wdgPropostaGridAfterMoveRow = function(wdgPropostaGrid) {
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        wdgTabbedPrincipal.descriptionOrder = wdgPropostaGrid.descriptionOrder;
        setLocalVar("PROPOSTA_NRSEQORCAMENTO", wdgPropostaGrid.currentRow.NRSEQORCAMENTO);
        setLocalVar("PROPOSTA_NRORG", wdgPropostaGrid.currentRow.NRORG);
        this.abrirSwipeProposta();
        this.verificaParamAlteraPropConc(wdgPropostaGrid).then(function(){
            this.validacoesGeraisProposta(wdgPropostaGrid);
        }.bind(this));
        this.desobrigaCamposEvolucaoEmail();
    };
    /**
     * Controla as ações do canto direito da tela de Proposta.
     * @param {object} widget - Widget de Proposta
     * @param string situacao - Situação da Proposta
     */
    this.controlarAcoesDeOperacao = function(widget, situacao) {
        var actConcluir       = widget.getAction("concluir_proposta");
        var actAceitar        = widget.getAction("aceitar_proposta");
        var actCopiar         = widget.getAction("copiar_proposta");
        var actVersionar      = widget.getAction("versionar_proposta");
        var actCancelar       = widget.getAction("cancelar_proposta");
        switch (situacao) {
            case "ELABORACAO": 
                actConcluir.isVisible       = true;
                actAceitar.isVisible        = false;
                actCopiar.isVisible         = true;
                actVersionar.isVisible      = true;
                actCancelar.isVisible       = true;
            break;
            case "CANCELADO":
               actConcluir.isVisible       = false;
               actAceitar.isVisible        = false;
               actCopiar.isVisible         = true;
               actVersionar.isVisible      = true;
               actCancelar.isVisible       = false;
            break;
            case "ACEITO":
                actConcluir.isVisible       = false;
                actAceitar.isVisible        = false;
                actCopiar.isVisible         = true;
                actVersionar.isVisible      = true;
                actCancelar.isVisible       = true;
            break;
            default: //CONCLUIDO ou ENVIADO
                actConcluir.isVisible       = false;
                actAceitar.isVisible        = true;
                actCopiar.isVisible         = true;
                actVersionar.isVisible      = true;
                actCancelar.isVisible       = true;
            break;
        }
    };
    this.desobrigaCamposEvolucaoEmail = function() {
        var wdgEnviarEmail = templateManager.container.getWidget("wdgEnviarEmail");
        var is_required    = this.alternaCampo(wdgEnviarEmail, false);
        wdgEnviarEmail.reload();
    };
    /**
     * Alterna o requerimento dos campos de evolução da tela de enviar e-mail
     * de acordo com o preenchimento
     * @params{Object} widget  - widget Enviar E-mail
     * @params{boolean} setObg - status da obrigatoriedade dos campos
    **/
    this.alternaCampo = function(widget, setObrg){
         var campoPreenchido = false;
         if(widget.getField("NMTIPOCONT").value() || widget.getField("NMMOTIEVOCON").value() || widget.getField("NMSEQRESULEVOL").value())
            campoPreenchido = true;
         if(!setObrg){
            widget.getField("NMTIPOCONT").validations = {};
            widget.getField("NMMOTIEVOCON").validations = {};
            widget.getField("NMSEQRESULEVOL").validations = {};
         } else {
            if(campoPreenchido){
                widget.getField("NMTIPOCONT").validations.required     = setObrg;
                widget.getField("NMMOTIEVOCON").validations.required   = setObrg;
                widget.getField("NMSEQRESULEVOL").validations.required = setObrg;
            }
        }
        return setObrg;
    };
    /**
     * Habilita a tela para a inserção de uma nova proposta
     **/
    this.novaProposta = function() {
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        setLocalVar("PROPOSTA_NRSEQORCAMENTO", "");
        ScreenService.openSwipe(wdgTabbedPrincipal).then(function() {
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            this.alterarAcessibilidadeAbasProposta(false);
            
            wdgTabbedPrincipal.edit();
            wdgProposta.getAction("avancar").isVisible = true;
            wdgProposta.getAction("save").isVisible    = false;
            wdgProposta.newRow();
            wdgProposta.template = "widget/form.html";
            
            if(!wdgProposta.currentRow.IDPROPALIMENTICIA)
                wdgProposta.currentRow.IDPROPALIMENTICIA = "S";
                
            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            this.verificaHierarquia(wdgPropostaGrid, wdgProposta);
            wdgPropostaGrid.currentRow = {};
            
            var wdgItens = templateManager.container.getWidget("wdgTabbedPrincipalItens");
            wdgItens.isVisible = false;
            
            var wdgEscopo = templateManager.container.getWidget("wdgEscoposProposta");
            wdgEscopo.isVisible = false;
            
            this.validacoesGeraisProposta(wdgPropostaGrid);
            this.controlaCodProposta(wdgProposta);
            this.removerDataLocalStorageLancamento();
        }.bind(this));
    };
    
    /**
     * Controla a acessibilidade das abas filhas de Proposta, desabilitando-as em 
     * modo de inserção e habilitando-as em modo de edição.
     * @param {boolean} enabled - status da acessibilidade das abas
     */
    this.alterarAcessibilidadeAbasProposta = function(enabled) {
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        wdgTabbedPrincipal.widgets.filter(function(wid) {
            return wid.name != "wdgPropostaTabbed";
        }).forEach(function(widget) {
            widget.enabled = enabled;
        });
        
        templateManager.updateTemplate();
    };
    /**
     * Altera a visibilidade das widgets de inserção
    **/
    this.alterarVisibilidadesWidgets = function(){
        var wdgCusto       = templateManager.container.getWidget("wdgCustoProposta");
        var wdgMObra       = templateManager.container.getWidget("wdgMaoObraGridInserindo");
        var wdgDespesa     = templateManager.container.getWidget("wdgDespesas");
        var wdgInvest      = templateManager.container.getWidget("wdgInvestimentosInserindo");
        var wdgTaxas       = templateManager.container.getWidget("wdgTaxasInserindo");
        var wdgServico     = templateManager.container.getWidget("wdgTabbedServico");
        var wdgItens       = templateManager.container.getWidget("wdgTabbedPrincipalItens");
        var wdgMObraTabbed = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
        var wdgCustoFinal  = templateManager.container.getWidget("wdgTabbedPrincipalResumoProp");
        var wdgEditarServ  = templateManager.container.getWidget("wdgEditarServico");
        var tabbedPrinc    = templateManager.container.getWidget("wdgTabbedPrincipal");
        var Propostatabbed = templateManager.container.getWidget("wdgPropostaTabbed");
        var wdgImposto     = templateManager.container.getWidget("wdgImpostoProposta");
        var wdgEscopo      = templateManager.container.getWidget("wdgEscoposProposta");

        wdgServico.isVisible          = false;
        wdgEditarServ.label           = "";
        wdgEditarServ.currentRow      = {};
        wdgEditarServ.dataSource.data = [];
        wdgEditarServ.fields.forEach(function(field){
            field.readOnly = true;
        });

        wdgCusto.isVisible = false;
        
        wdgMObraTabbed.isVisible  = false;
        wdgMObra.dataSource.data  = [];
        wdgMObraTabbed.currentRow = {};

        wdgDespesa.isVisible = false;
        wdgDespesa.dataSource.clearCheckedRows();

        wdgInvest.isVisible = false;
        wdgInvest.dataSource.clearCheckedRows();

        wdgTaxas.isVisible = false;
        wdgTaxas.dataSource.clearCheckedRows();
        
        wdgImposto.isVisible = false;
        wdgImposto.dataSource.clearCheckedRows();

        wdgItens.isVisible     = true;
        wdgEscopo.isVisible    = false;
        tabbedPrinc.enabled    = true;
        Propostatabbed.enabled = true;
        tabbedPrinc.view();
        
        wdgCustoFinal.mode = "list";
    };
    /**
     * Controla o campo de Código na aba de dados da proposta
     * @params{Object} widget - widget dados da proposta
     **/
    this.controlaCodProposta = function(widget) {
        var fieldCodigo = widget.getField("NRORCAMENTO");
        if (widget.currentRow.NMORCAMENTOMODELO && widget.currentRow.__is_new) {
            fieldCodigo.readOnly = true;
            widget.currentRow.NRORCAMENTO = "";
            fieldCodigo.validations = {};
        } else {
            fieldCodigo.readOnly = false;
            fieldCodigo.validations = { "required": true } ;
        }
    };
    /**
     * Seta a currentRow com o registro retornado do banco de dados.
     * @param {object} wdgProposta - Widget de Dados da Proposta (view/form)
     */
    this.setDadosProposta = function(wdgProposta) {
        wdgProposta.currentRow = wdgProposta.dataSource.data[0];
    };
    /**
     * Volta para o grid da tela de proposta.
     **/
    this.voltarParaPropostaGrid = function() {
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var wdgItens    = templateManager.container.getWidget("wdgTabbedPrincipalItens");
        wdgProposta.dataSource.data = [];
        wdgProposta.currentRow      = {};
        
        this.removerDataLocalStorageLancamento();
        ScreenService.closeSwipe().then(function() {
            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            wdgPropostaGrid.activate();
            wdgItens.enabled   = true;
            wdgItens.isVisible = true;
            wdgPropostaGrid.refreshWidget();
        });
    };
    /**
     * Coloca a tela de Proposta em modo de edição.
     * @param {object} wdgProposta - Widget de Proposta
     **/
    this.prepararEdicaoProposta = function(wdgProposta) {
        var wdgTabbed = templateManager.container.getWidget("wdgTabbedPrincipal");
        var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
        wdgTabbed.edit();
        
        wdgProposta.backupRow = Util.clone(wdgProposta.currentRow, ignoredKeys);
        wdgProposta.getAction("avancar").isVisible    = false;
        wdgProposta.getAction("save").isVisible       = true;
        wdgProposta.getField("VRPERCMARKUP").readOnly = true;
        wdgProposta.edit();
        wdgProposta.template = "widget/form.html";
        
        this.alterarAcessibilidadeAbasProposta(false);
        this.controlaCodProposta(wdgProposta);
        this.controlarEscopoProposta(wdgProposta, 'S');
    };
    /**
     * Exclui a Proposta.
     * 
     * @param {object} wdgProposta - Widget de Dados da Proposta (view/form)
     */
    this.excluirProposta = function(wdgProposta) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function() {
            Utilities.validarCamposDuplicados(wdgProposta, null, ["NRSEQCONTATO", "NRORG"], "product#/evolcont_parcnegocio").then(function(validacaoCampo){
                if(!validacaoCampo){
                    ScreenService.confirmMessage("Não é possível excluir a proposta porque ela possui registros filhos. Deseja inativar a proposta?", "question", function() {
                        General.callRestEngine("product#/fct_orcamento_widget/inativaPropContato", "Row", wdgProposta.currentRow)
                        .then().catch(function(error){
                            console.error(error);
                            ScreenService.showMessage("Não foi possível tornar a proposta inativa.");
                        });
                    });
                } else {
                    wdgProposta.removeRow().then(function() {
                        this.voltarParaPropostaGrid();
                    }.bind(this),
                    function(error) {
                        if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                            ScreenService.confirmMessage("Não é possível excluir a proposta porque ela possui registros filhos. Deseja inativar a proposta?", "question", function() {
                                this.inativarProposta(wdgProposta);
                            }.bind(this), null);
                        } else {
                            Utilities.showErrorMessage(error);
                        }
                    }.bind(this));
                }
            }.bind(this));
        }.bind(this));
    };
    /**
     * Inativa proposta
    **/
    this.inativarProposta = function(wdgProposta) {
        wdgProposta.currentRow.changed = true;
        wdgProposta.currentRow.IDATIVO = "N";
        General.widgetSave(wdgProposta).then(function() {
            this.voltarParaPropostaGrid();
        }.bind(this),
        function(error) {
            console.error(error);
            if (error.data)
                ScreenService.showMessage(error.data.error);
            else
                ScreenService.showMessage(error);
        });
    };
    /**
     * Seta o Modelo e os Escopos de Proposta para o filtro da Composição.
     * @param {object} widget - Widget de Serviço (inserindo)
     */
    this.setFiltroComposicao = function(widget) {
        var wdgProposta       = templateManager.container.getWidget("wdgProposta");
        var nmorcamentomodelo = widget.dataSourceFilter[0].value;
        var escopo            = widget.dataSourceFilter[1].value;
        var oldQuantidade     = getLocalVar("QTDIASVALIDADE_PROPOSTA");
        
        if(wdgProposta.currentRow.NRORCAMENTOMODELO !== nmorcamentomodelo || wdgProposta.currentRow.NRTIPOORCAMENTO !== escopo) {
            widget.dataSourceFilter[0].value = wdgProposta.currentRow.NRORCAMENTOMODELO;
            widget.dataSourceFilter[1].value = wdgProposta.currentRow.NRTIPOORCAMENTO;
            widget.reload();
        } else {
            Sm00001000_PropostaAlimenticia_Repository.getDataLocalStorageEngine("wdgServicosInserindo").then(function(response) {
                if(response && JSON.parse(response).length > 0) { 
                    var wdgServico = templateManager.container.getWidget("wdgServicosInserindo");
                    wdgServico.dataSource.data = JSON.parse(response);
                } else {
                    widget.dataSourceFilter[0].value = wdgProposta.currentRow.NRORCAMENTOMODELO;
                    widget.dataSourceFilter[1].value = wdgProposta.currentRow.NRTIPOORCAMENTO;
                    widget.reload();
                }
            }.bind(this));
        } 
    };
    /**
     * Refaz o cálculo da quantidade de serviços por período quando 
     * a quantidade de dias da proposta é alterado
     * @params{Object} widget - widget de serviço
    **/
    /*this.atualizarQuantidadePeriodoServico = function(widget) {
        var rowsAlteradas = widget.dataSource.data.filter(function(servico) {
            return servico._childCount == 0 && servico.QTMENSAL > 0;
        });
        
        var wdgEdicaoServico = templateManager.container.getWidget("wdgEditarServico");
        rowsAlteradas.forEach(function(row) {
            wdgEdicaoServico.currentRow = row;
            this.salvarEdicaoServico(wdgEdicaoServico);
        }.bind(this));
    };*/
    /**
     * Avançar para a inserção dos serviços na proposta
     * @params{Object} widget - widget dados da proposta
    **/
    this.avancarServico = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgPropostaTabbed     = templateManager.container.getWidget("wdgPropostaTabbed");
            wdgPropostaTabbed.enabled = false;

            var wdgServico = templateManager.container.getWidget("wdgServicosInserindo");
            var wdgTabbed  = templateManager.container.getWidget("wdgTabbedServico");
            wdgTabbed.isVisible = true;
            wdgTabbed.enabled   = true;
            wdgTabbed.isVisible = true;
            wdgTabbed.mode      = "edit";
            wdgTabbed.activate().then(function() {
                this.setFiltroComposicao(wdgServico);
            }.bind(this));
        }
    };
    /** 
     * Valida o campo de código da proposta
     * @params{Object} widget - dados da proposta
    **/
    this.validaCodPropostas = function(widget, nameAction){
        if(!widget.currentRow.NRORCAMENTO){
            this.avancarServico(widget);
        } else {
            Utilities.validarCamposDuplicados(widget, ["NRSEQORCAMENTO"], ["NRORG", "NRORCAMENTO"], "product#/crud/fct_orcamento").then(function(response){
                if(response)
                    if(nameAction === "save")
                        this.validarEdicaoProposta(widget);
                    else 
                        this.avancarServico(widget);
                else 
                    ScreenService.showMessage("Operação bloqueada.<br/>Já existe uma proposta com o código informado.");
            }.bind(this));
        }
    };
    /**
     * Verifica se o percentual de desconto oferecido permitido 
     * ao operador logado
     * @params{Object} widget - widget dados da proposta
     **/
    this.validaCarenciaOperOuPerf = function(widget, nameAction) {
        if(widget.currentRow.NRDIASCARENCIA === null){
            this.validaCodPropostas(widget, nameAction);
        } else {
            var totalMaxCarencia = null;
            var tipoCarencia     = null;
            var contexto         = "";
            General.callRestEngine("product#/fct_carenciaoper_parcnegocio/validaCarenciaOperOuPerf", "Row", {}).then(function (success) {
                if (!success.dataset.VALIDAOPERPERF.VALIDA) {
                    this.validaCodPropostas(widget, nameAction);
                } else {
                    totalMaxCarencia = success.dataset.VALIDAOPERPERF.VALIDA.NRDIAMAXIMO;
                    tipoCarencia = success.dataset.VALIDAOPERPERF.VALIDA.TIPO;
                    if(totalMaxCarencia >= widget.currentRow.NRDIASCARENCIA){
                        this.validaCodPropostas(widget, nameAction);
                    } else {
                        if(tipoCarencia == 1)
                            contexto = "operador logado.";
                        if(tipoCarencia == 2)
                            contexto = "perfil de acesso deste operador logado.";
                        ScreenService.showMessage("Quantidade de dias de carência informado está acima do permitido para o "+ contexto +" (Carência máxima permitida: "+ totalMaxCarencia +" dias).");
                    }
                }
            }.bind(this));
        }
    };
    /**
     * Cancela o salvamento da Proposta.
     * @param {object} wdgProposta - Widget de Proposta
     */
    this.cancelarSalvamentoProposta = function(wdgProposta) {
        var wdgTabbed = templateManager.container.getWidget("wdgTabbedPrincipal");
        var wdgPropostaTabbed = templateManager.container.getWidget("wdgPropostaTabbed");
        wdgPropostaTabbed.enabled = true;
        wdgTabbed.view();
        wdgProposta.currentRow.changed = false;
        this.alterarAcessibilidadeAbasProposta(true);
        this.alterarVisibilidadesWidgets();
        this.controlarEscopoProposta(wdgProposta, 'S');
        wdgProposta.template = "widget/form-view.html";
        wdgProposta.getAction("save").isVisible       = false;
        wdgProposta.getAction("avancar").isVisible    = true;
        wdgProposta.getField("VRPERCMARKUP").readOnly = false;
        
        if (wdgProposta.currentRow.__is_new) {
            if (wdgProposta.dataSource.data.indexOf(wdgProposta.currentRow) !== -1)
                wdgProposta.dataSource.data.pop();
                
            codeRelacFormacao = null;
            wdgProposta.list();
            this.voltarParaPropostaGrid();
        } else { 
            //caso a widget estiver em modo de edição, recupera o backup e entra no modo de visualização
            Object.keys(wdgProposta.backupRow).forEach(function(key) {
                wdgProposta.currentRow[key] = wdgProposta.backupRow[key];
            }); //copia dados coluna por coluna para não perder referência do objeto da currentRow
            wdgProposta.view();
        }
        
    };
    
    /**
     * Função disparada ao modificar o campo Modelo de Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.modeloOnChange = function(widget) {
        this.verificarModeloProposta(widget);
        this.calcularDataValidadeProposta(widget);
        this.escoposOnChange(widget);
        this.controlaCodProposta(widget);
        this.controlaEscopoModelo(widget);
    };
    /**
     * Ao carregar um modelo, é verificado se seus dados não poderão ser alterados e se será
     * obrigatório informar uma oportunidade de negócio.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.verificarModeloProposta = function(widget) {
        var idbloqalteradados = widget.currentRow.IDBLOQALTERADADOS;
        var readOnly = (idbloqalteradados == "SIM") ? true : false;
        //this.manipularCamposModeloProposta(widget, readOnly);

        var utilizaComposicao = (widget.currentRow.IDUTILIZACOMP == "S");
        this.controlarEscopoProposta(widget, utilizaComposicao);
    };
    /**
     * Controla a field de escopo de acordo com o modelo de orcamento
     * escolhido. 
     * @param {Object} widget widget propostas
    **/
    this.controlaEscopoModelo = function(widget){
        var params = [
            {
                name     : "NRORCAMENTOMODELO",
                operator : "=",
                value    : widget.currentRow.NRORCAMENTOMODELO
            }
        ];
        
        General.callRestEngine("product#/crud/fct_relacescopomodelo_widget", "FilterData", params).then(function(response){
            if(response.dataset && response.dataset.fct_relacescopomodelo_widget){
                var fieldEscopo  = (widget.currentRow.IDUTILIZACOMP == "S") ? widget.getField("NRTIPOORCAMENTO"): widget.getField("NMTIPOORCAMENTO");
                var fieldIsArray = fieldEscopo.template === "field/select-multiple.html"; //(widget.currentRow[fieldEscopo.name] instanceof Array);
                if(response.dataset.fct_relacescopomodelo_widget.length == 1){
                    fieldEscopo.readOnly = true;
                    widget.currentRow.NRTIPOORCAMENTO = fieldIsArray ? [response.dataset.fct_relacescopomodelo_widget[0].NRTIPOORCAMENTO] : response.dataset.fct_relacescopomodelo_widget[0].NRTIPOORCAMENTO;
                    widget.currentRow.NMTIPOORCAMENTO = fieldIsArray ? [response.dataset.fct_relacescopomodelo_widget[0].NMTIPOORCAMENTO] : response.dataset.fct_relacescopomodelo_widget[0].NMTIPOORCAMENTO;
                } else {
                    fieldEscopo.readOnly = false;
                    widget.currentRow.NRTIPOORCAMENTO = fieldIsArray ? [] : "";
                    widget.currentRow.NMTIPOORCAMENTO = fieldIsArray ? [] : "";
                }
            }
        }.bind(this));
    };
    
     /**
     * Calcula a data de validade da proposta a partir dos dias.
     * @param {object} widget - Widget de Proposta
     */
    this.calcularDataValidadeProposta = function(widget) {
        var qtDias = widget.currentRow.QTDIASVALIDADE;
        if (qtDias != null && qtDias >= 0) {
            var dataAtual = moment();
            widget.currentRow.DTVALIDADE = dataAtual.add(qtDias, "day").format("DD/MM/YYYY");
        } else {
            widget.currentRow.DTVALIDADE = "";
            widget.currentRow.QTDIASVALIDADE = "";
        }
    };
    
    /**
     * Seta o campo IDUTILIZAVARPRECO de acordo com os escopos selecionados.
     * @param {object} widget - Widget de Proposta
     */
    this.escoposOnChange = function(widget) {
        var fieldEscopo = widget.getField("NRTIPOORCAMENTO");
        var nrtipoorcamento = fieldEscopo.value();
        var escopos = fieldEscopo.dataSource.data;

        var idutilizavarpreco = [];
        escopos.forEach(function(row) {
            if (nrtipoorcamento.indexOf(row.NRTIPOORCAMENTO) > -1) {
                idutilizavarpreco.push(row.IDUTILIZAVARPRECO);
            }
        });

        if (idutilizavarpreco.length == 0)
            idutilizavarpreco = "";

        widget.currentRow.IDUTILIZAVARPRECO = idutilizavarpreco;
    };
    
    /**
     * Controla o campo de Escopo de Proposta de acordo com o Modelo (se utiliza composição de serviços ou não).
     * 
     * @param {object} widget - Widget de Proposta
     * @param {boolean} utilizaComposicao
     */
    this.controlarEscopoProposta = function(widget, utilizaComposicao, limparCampos) {
        if (limparCampos == undefined) {
            limparCampos = true;
        }

        var fieldCodEscopo    = widget.getField("NRTIPOORCAMENTO");
        var fieldNomeEscopo   = widget.getField("NMTIPOORCAMENTO");
        var fieldUtilVariacao = widget.getField("IDUTILIZAVARPRECO");
        fieldNomeEscopo.readOnly   = false;
        fieldCodEscopo.readOnly    = false;
        if (utilizaComposicao) {
            this.filtroEscopo(widget);
            fieldNomeEscopo.isVisible  = false;
            fieldNomeEscopo.showOnForm = false;
            fieldNomeEscopo.template   = "field/text-edit.html";
            fieldCodEscopo.template    = "field/select-multiple.html";
            fieldCodEscopo.validations = {"required": true};
            fieldCodEscopo.isVisible   = true;
            fieldCodEscopo.showOnForm  = true;
            fieldCodEscopo.reload().then(function() {
                if (fieldCodEscopo.dataSource.data && fieldCodEscopo.dataSource.data.length == 1) {
                    fieldCodEscopo.value([fieldCodEscopo.dataSource.data[0].NRTIPOORCAMENTO]);
                    fieldNomeEscopo.value([fieldCodEscopo.dataSource.data[0].NMTIPOORCAMENTO]);
                    fieldUtilVariacao.value([fieldCodEscopo.dataSource.data[0].IDUTILIZAVARPRECO]);
                }
            });
        } else {
            fieldCodEscopo.isVisible   = false;
            fieldCodEscopo.showOnForm  = false;
            fieldCodEscopo.template    = "field/text-edit.html";
            fieldCodEscopo.validations = {};
            fieldNomeEscopo.template   = "zh-select-autocomplete#field/select-autocomplete.html";
            fieldNomeEscopo.isVisible  = true;
            fieldNomeEscopo.showOnForm = true;
            Utilities.iniciarAutocomplete(fieldNomeEscopo);
        }

        if (limparCampos) {
            fieldNomeEscopo.value("");
            fieldCodEscopo.value("");
        }
    };
    /**
     * filtra o campo Escopo de Proposta quando tem Modelo.
     * @param {object} widget - Widget de Proposta
     */
    this.filtroEscopo = function(widget) {
        var fieldEscopo       = widget.getField("NRTIPOORCAMENTO");
        var nrorcamentomodelo = widget.currentRow.NRORCAMENTOMODELO;
        fieldEscopo.dataSourceFilter[1].value = nrorcamentomodelo;
        fieldEscopo.reload();
    };
    /**
     * Recarrega o field caso o dataSource tenha sido alterado 
     * @params{Object} fieldNmOp - field Nome Operador
     **/
    this.recarregaDadosField = function(fieldNmOp) {
        var widgetAtiva = templateManager.container.getActiveWidget();
        var fieldParc  =  widgetAtiva.getField("NRPARCNEGOCIO");
        if (widgetAtiva.name === "filterPopup") {
            fieldParc.dataSourceFilter[1] = {
                name: "P_CDOPERADOR",
                operator: "=",
                value: opLogado
            };
        }
        fieldNmOp.reload();
    };
    /**
     * Manda recalcular a arvore de formação
     * @params{Object} widget widget formação de preço
     * @params{int} nrformprecoservico nr da formação de preço
    **/
    this.recalculaFormacaoItens = function(widget, nrformprecoservico, nritemorcamento){
        var deferred = ZHPromise.defer();
        var params = {
            "NRFORMPRECOSERVICO" : nrformprecoservico,
            "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO",
            "NRITEMORCAMENTO"    : nritemorcamento
        };
        
        General.callRestEngine("product#/fct_calcformprecserv_widget/recalcularArvore", "Row", params).then(function() {
            Utilities.abreArvoreElementos(widget).then(function(){
                ScreenService.showMessage("Processo realizado com sucesso!");
                deferred.resolve();
            });
        })
        .catch(function(error) {
            console.log(error);
            deferred.reject();
        });
        
        return deferred.promise;
    };
    /**
     * Recalcula a árvore de elementos
     * @params{Object} widget widget editar itens de formação de preço
     * .parent substituido pela widget da grid principal
    **/
    this.recalcularArvore = function(widget) {
        var nrformprecoservico;
        if(widget.name === "editaCalculoItensFormPreco"){
            var wdgMObraIserindo = templateManager.container.getWidget("wdgMaoObraGridInserindo");
            nrformprecoservico   = wdgMObraIserindo.currentRow.NRFORMPRECOSERVICO;
            this.recalculaFormacaoItens(widget, nrformprecoservico, wdgMObraIserindo.currentRow.NRRELACFORMACAOPROP);
        } else {
            var wdgMObra = templateManager.container.getWidget("wdgItensMaoObra");
            nrformprecoservico = wdgMObra.currentRow.NRFORMPRECOSERVICO;
            this.recalculaFormacaoItens(widget, nrformprecoservico, wdgMObra.currentRow.NRRELACFORMACAOPROP).then(function(){
                Utilities.abreArvoreElementos(widget);
                this.atualizarValorItem();
            }.bind(this));
        }
    };
    /**
     * Volta para a aba de Proposta.
     * @param {object} widget - Widget de Serviço (inserindo)
     */
    this.voltarComposicaoProposta = function(widget) {
        widget.enabled = false;
        Sm00001000_PropostaAlimenticia_Repository.saveDataLocalStorageEngine("wdgServicosInserindo", templateManager.container.getWidget("wdgServicosInserindo").dataSource.data);
        setLocalVar("QTDIASVALIDADE_PROPOSTA", templateManager.container.getWidget("wdgProposta").currentRow.QTDIASVALIDADE);
        
        var wdgPropostaTabbed     = templateManager.container.getWidget("wdgPropostaTabbed");
        wdgPropostaTabbed.enabled = true;
        wdgPropostaTabbed.activate();
    };
    /**
     * habilita a edição das quantidades relacionadas aos serviços
     * @params{widget} widget servico grid
     * @params{String} event - nome do local em que a row estará
    **/
    this.editarQuantidadeServicos = function(widget, event){
        var wdgEditServ = templateManager.container.getWidget("wdgEditarServico");
        var wdgTabbed   = templateManager.container.getWidget("wdgTabbedServico");
        
        wdgEditServ.currentRow = Util.clone(((event === 'checkedRow') ? widget.getLastSelectedRow() : widget.currentRow), []);
        if(wdgEditServ.currentRow._childCount !== 0){
            wdgEditServ.fields.forEach(function(field){
                field.readOnly = true;
                field.value(null);
            }); 
        } else {
            wdgEditServ.fields.forEach(function(field){
                field.readOnly = false;
            }); 
    
            var label = "";
            if(wdgEditServ.currentRow._parent){
                label = this.getArvoreServico(widget, wdgEditServ.currentRow, [], false).filter(function(servico) {
                    return !servico._parent;
                });
                label = (label.length == 0) ? "" : label[0]["NMELEMENTO"] + " -> ";
            }
            
            wdgEditServ.label = label + wdgEditServ.currentRow.NMELEMENTO;    
            templateManager.updateTemplate();
        }
        
        wdgTabbed.activate();
    };
    /**
     * Avança para a widget de custo do serviço escolhido
     * @params{Object} widget - widget imposto inserindo
    **/
    this.avancarCusto = function(widget){
        if (!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            widget.enabled = false;
            Sm00001000_PropostaAlimenticia_Repository.saveDataLocalStorageEngine("wdgImpostoProposta", widget.dataSource.data);
            var wdgCusto = templateManager.container.getWidget("wdgCustoProposta");
            wdgCusto.enabled   = true;
            wdgCusto.isVisible = true;
            wdgCusto.mode = "edit";
            wdgCusto.activate().then(function() {
                this.setFiltroCustoServico(wdgCusto);
            }.bind(this));
        }
    };
    /**
     * Aplica um filtro widget de custo durante a inserção da proposta
     * @params{Object} widget - widget serviços inserindo
    **/
    this.setFiltroCustoServico = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var wdgServico  = templateManager.container.getWidget("wdgServicosInserindo");
        var composicoes = [];
        wdgServico.dataSource.getCheckedRows().forEach(function(serv) {
            composicoes.push(serv.NRCOMPSERVICO);
        });
        var params = {
            "NRTIPOORCAMENTO": wdgProposta.currentRow.NRTIPOORCAMENTO,
            "NRCOMPSERVICO": composicoes
        };
        General.callRestEngine("/fct_orcamentoaliment_widget/getComposicoesServicos", "Row", params).then(function(response) {
            widget.dataSource.data = response.dataset.fct_custocomposicao;
            widget.dataSource.data.forEach(function(custo){
                custo.NRCOMPSERVICO = this.retirarValoresDuplicados(custo.NRCOMPSERVICO.split("_"));
            }.bind(this));
            
            widget.getAction("save").isVisible   = false;
            widget.getAction("cancel").isVisible = false;
            this.setQuantidadesServicos();
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Volta para a aba de serviços.
     * @param {object} widget - Widget de custo (inserindo)
     */
    this.voltarServicoProposta = function(widget) {
        widget.enabled  = false;
        var wdgServicos = templateManager.container.getWidget("wdgTabbedServico");
        Sm00001000_PropostaAlimenticia_Repository.getDataLocalStorageEngine("wdgServicosInserindo").then(function(response) {
            if(response) {
                var wdgServicoGrid = templateManager.container.getWidget("wdgServicosInserindo");
                wdgServicoGrid.dataSource.data = JSON.parse(response);
            }
            
            wdgServicos.enabled = true;
            wdgServicos.activate();
        }.bind(this));
    };
    /**
     * Avança para a widget de Mão de obra durante a inserção
     * @params{Object} widget - widget custo inserindo
    **/
    this.avancarMaoObra = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            widget.enabled = false;
            Sm00001000_PropostaAlimenticia_Repository.saveDataLocalStorageEngine("wdgCustoProposta", widget.dataSource.data);
            var wdgMObraProposta = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
            var wdgMObraGrid     = templateManager.container.getWidget("wdgMaoObraGridInserindo");
            wdgMObraProposta.enabled   = true;
            wdgMObraProposta.isVisible = true;
            wdgMObraGrid.mode          = "edit";
            wdgMObraProposta.activate().then(function(){
                var params = {
                    "NRSEQORCAMENTO" : 0,
                    "NRITEMORCAMENTO": -1
                };
                
                General.callRestEngine("product#/fct_relacestruturaprop_widget/deletaAlteracoesInutilizadas", "Row", params).then(function(){
                    wdgMObraProposta.getField('NRCALCFORMPRECSERV').readOnly = false;
                });
            });
        }
    };
    /**
     * filtra o campo "Itens da Formação de Preço" de acordo com as 
     * formações de preço selecionadas
     * @params{Object} widget - widget mão de obra
     **/
    this.filtrarFieldItemFormPreco = function(widget) {
        var formPreco = widget.currentRow.NRFORMPRECOSERVICO;
        widget.getField('NRCALCFORMPRECSERV').dataSourceFilter[0].value = formPreco;
        widget.getField('NRCALCFORMPRECSERV').reload();
    };
    /***
     * retorna um serviço para cada formação
    **/
    this.getDataFormacaoServico = function(widget){
        var fieldFormacao = widget.getField("NRFORMPRECOSERVICO");
        var fieldServico  = widget.getField("CDSERVORCA");
        var data          = [];
        fieldServico.dataSource.getCheckedRows().forEach(function(servico){
            fieldFormacao.dataSource.getCheckedRows().forEach(function(formacao){
                data.push({
                    "NRFORMPRECOSERVICO": formacao.NRFORMPRECOSERVICO,
                    "NMSERVORCA"        : servico.NMSERVORCA,
                    "CDSERVORCA"        : servico.CDSERVORCA
                });
            });
        });
        
        if(data.length <= 0)
            data = fieldFormacao.dataSource.getCheckedRows();
            
        return data;
    };
    /**
     * Adiciona no grid as mão de obras selecionadas 
     * @params{Object} widget- widget mão de obra 
    **/
    this.addMaoObraProposta = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var mObraGrid    = templateManager.container.getWidget("wdgMaoObraGridInserindo");
            var tabbedMObra  = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
            var dataFormacao  = this.getDataFormacaoServico(widget);
            var fieldItemForm = widget.getField("NRCALCFORMPRECSERV");
            var newData       = [];
            var itemPai       = null;
            
            this.salvarAlteracoesCalculo(widget, 0, dataFormacao).then(function(sucess){
                widget.currentRow.QTFORMACAO = "0";
                dataFormacao.forEach(function(formacao) {
                    var idformacao      = 0;
                    var valorTotal      = 0;
                    var nrrelacformacao = null;
                    sucess.filter(function(idForm){
                        if(idForm.NRFORMPRECOSERVICO === formacao.NRFORMPRECOSERVICO && idForm.CDSERVORCA === formacao.CDSERVORCA){
                            idformacao      = idForm.IDFORMACAO;
                            nrrelacformacao = idForm.NRRELACFORMACAOPROP;
                        }
                    });
                    fieldItemForm.dataSource.getCheckedRows().forEach(function(item){
                        if(item.NRFORMPRECOSERVICO === formacao.NRFORMPRECOSERVICO && !item._parent){
                            valorTotal += item.VRTOTAL;
                            itemPai     = item.NRCALCFORMPRECSERV;
                        }
                    });
                    newData.push({
                        "NMFORMPRECOSERVICO"    : formacao.NMSERVORCA,
                        "NRFORMPRECOSERVICO"    : formacao.NRFORMPRECOSERVICO,
                        "VRUNITARIO"            : valorTotal,
                        "VRTOTAL"               : valorTotal * parseInt(widget.currentRow.QTFORMACAO),
                        "QTFORMACAO"            : parseInt(widget.currentRow.QTFORMACAO),
                        "IDPOSICAOFORM"         : formacao.NRFORMPRECOSERVICO + "_" + idformacao,
                        "IDALTERADO"            : "N",
                        "NRCALCFORMPRECSERV"    : itemPai,
                        "IDFORMACAO"            : idformacao,
                        "NRRELACFORMACAOPROP"   : nrrelacformacao,
                        "CDSERVORCA"            : formacao.CDSERVORCA
                    });
                });
                mObraGrid.dataSource.data = mObraGrid.dataSource.data.concat(newData);
                fieldItemForm.dataSourceFilter[0].value = [];
                templateManager.updateTemplate();
        
                widget.currentRow.NRFORMPRECOSERVICO = [];
                widget.currentRow.NRCALCFORMPRECSERV = [];
                widget.currentRow.CDSERVORCA         = [];
    
                fieldItemForm.readOnly = false;
                fieldItemForm.reload();
                tabbedMObra.activate().then(function(){
                    setTimeout(function() {
                        mObraGrid.getAction("save").isVisible = false;
                        mObraGrid.getAction("cancel").isVisible = false;
                    }, 100);
                });
            }.bind(this));   
        }
    };
    /**
     * Recupera o valor total de uma árvore de elementos
     * @params{Object} widget - widget mão de obra 
    *
    this.recuperarValorMaoObra = function(widget){
        var fieldItens = widget.getField("NRCALCFORMPRECSERV");
        var itensSelecionados = fieldItens.dataSource.getCheckedRows();
        fieldItens.dataSource.data.forEach(function(item){
           
        });
    };
    */
    /**
     * Volta para a widget de custo durante a inserção da proposta
     * @params{Object} widget - widget mão de obra
    **/
    this.voltarCustoServico = function(){
        var wdgTabbed     = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
        wdgTabbed.enabled = false;
        wdgTabbed.getField("NRCALCFORMPRECSERV").dataSourceFilter[0].value = [];

        var wdgCusto = templateManager.container.getWidget("wdgCustoProposta");
        Sm00001000_PropostaAlimenticia_Repository.getDataLocalStorageEngine("wdgCustoProposta").then(function(response) {
            if(response)
                wdgCusto.dataSource.data = JSON.parse(response);
                
            wdgCusto.enabled = true;
            wdgCusto.activate();
        }.bind(this));
    };
    /**
     * Deleta a mão de obra escolhida durante a inserção da proposta
     * @params{Object} widget - widget mão de obra
    **/
    this.deletarMaoObraEscolhida = function(widget){
        ScreenService.confirmMessage("Retirar Mão de Obra da Proposta?", "question", function(){
            var params = {
                "NRSEQORCAMENTO" : 0,
                "NRITEMORCAMENTO": -1,
                "NRCALCFORMPRECSERV": widget.currentRow.NRCALCFORMPRECSERV, 
                "NRFORMPRECOSERVICO": widget.currentRow.NRFORMPRECOSERVICO
            };
            
            General.callRestEngine("product#/fct_relacestruturaprop_widget/deletarArvoreFormPrecoItPro", "Row", params).then(function(){
                var fieldFormacao = templateManager.container.getWidget("wdgTabbedPrincipalMObra").getField("NRFORMPRECOSERVICO");
                var index         = widget.dataSource.data.indexOf(widget.currentRow);
                widget.dataSource.data.splice(index, 1);
                
                if(fieldFormacao.dataSourceFilter.length > 0) {
                    index = fieldFormacao.dataSourceFilter[0].value.indexOf(widget.currentRow.NRFORMPRECOSERVICO);
                    fieldFormacao.dataSourceFilter[0].value.splice(index, 1);
                    fieldFormacao.reload();
                }
            }.bind(this));  
        });
    };
    /**
     * Abre o grid de edição das formações de preço 
     * durante a inserção da proposta
     * @params{Object} widget - widget mão de obra
    **/
    this.abrirWdgEditarFormacaoPreco = function(widget){
        if(!onClickMobra || onClickMobra !== widget.currentRow.NRFORMPRECOSERVICO){
            onClickMobra = widget.currentRow.NRFORMPRECOSERVICO;
        } else{
            var tabbedFormacao = templateManager.container.getWidget("wdgTabbedPrincipalFormPreco");
            var itensCalculo   = templateManager.container.getWidget("editaCalculoItensFormPreco");
            var wdgTabbedMObra = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
            var maoObraGrid    = templateManager.container.getWidget("wdgMaoObraGridInserindo");
            var ignoredKeys    = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
            
            tabbedFormacao.mode      = 'edit';
            tabbedFormacao.isVisible = true;
            itensCalculo.isVisible   = true;
            widget.enabled           = false;
            wdgTabbedMObra.enabled   = false;
            idItensMObraAlterado     = false;
            
            widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
            widget.currentRow.VRUNITARIO = 0;
            widget.currentRow.VRTOTAL    = 0;
        
            if(!widget.currentRow.IDALTERADO || widget.currentRow.IDALTERADO === 'N'){
                if(itensCalculo.dataSourceFilter.length === 5)
                    itensCalculo.dataSourceFilter.pop();
            } else {
                itensCalculo.dataSourceFilter[4] = {
                    name     : "NRCALCFORMPRECSERV",
                    operator : "IN",
                    value    : maoObraGrid.currentRow.NRCALCFORMPRECSERVS
                }; 
            }

            ScreenService.openSwipe(tabbedFormacao).then(function() {
                itensCalculo.dataSourceFilter[0].value = maoObraGrid.currentRow.NRFORMPRECOSERVICO;
                itensCalculo.dataSourceFilter[3].value = maoObraGrid.currentRow.IDFORMACAO;

                Utilities.abreArvoreElementos(itensCalculo);
                onClickMobra = 0;
            }.bind(this));    
        }
    };
    /**
     * Vusca o valor do último código inserido na tabela fct_relacformacaoprop 
    **/
    this.getCodeRelacFormacaoProp = function(){
        var deferred = ZHPromise.defer();
        if(codeRelacFormacao){
            codeRelacFormacao++;
            deferred.resolve();
        } else {
            General.callRestEngine("product#/fct_orcamentoaliment_widget/getCodeRelacFormacaoProp", "Row", {}).then(function(response) {
                if(response.dataset && response.dataset.fct_relacformacaoprop && response.dataset.fct_relacformacaoprop.NRULTRELACFORM)
                    codeRelacFormacao = response.dataset.fct_relacformacaoprop.NRULTRELACFORM;
                deferred.resolve();
            }).catch(function(error) {
                deferred.reject();
            });
        }
        
        return deferred.promise;
    };
    /**
     * Salva as alterações feitas nos elementos do cálculo
     * @params{Object} widget - widget editar itens formação de preço
     * @params{boolean} isNew - currentRow.__is_new
     * @params{Array} nrItensProp - array de itens da proposta
    **/
    this.salvarAlteracoesCalculo = function(widget, nrseqorcamento, dataFormacao) {
        var deferred  = ZHPromise.defer();
        var formprecoservico = [];
        var idFormacoes      = [];
        this.getCodeRelacFormacaoProp().then(function(){
            dataFormacao.forEach(function(formacao){
                codeRelacFormacao++;
                idFormacoes.push({
                    "NRFORMPRECOSERVICO" : formacao.NRFORMPRECOSERVICO,
                    "IDFORMACAO"         : codeRelacFormacao,
                    "NRRELACFORMACAOPROP": codeRelacFormacao,
                    "CDSERVORCA"         : formacao.CDSERVORCA
                });
                widget.getField("NRCALCFORMPRECSERV").dataSource.getCheckedRows().forEach(function(elemento){
                    if(formacao.NRFORMPRECOSERVICO === elemento.NRFORMPRECOSERVICO){
                        formprecoservico.push({
                            "NRFORMPRECOSERVICO": formacao.NRFORMPRECOSERVICO,
                            "NRCALCFORMPRECSERV": "_" + elemento.NRCALCFORMPRECSERV + "_",
                            "IDFORMACAO"        : codeRelacFormacao
                        });
                    }
                });
            });
        
            var params = {
                "FORMPRECOSERVICOS" : formprecoservico,
                "NRSEQORCAMENTO"    : nrseqorcamento,
                "NRITEMORCAMENTO"   : -1
            };
            
            General.callRestEngine("product#/fct_itemorcamentoaliment_widget/insereMaoObraFormacao", "Row", params).then(function(){
                deferred.resolve(idFormacoes);
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
                deferred.reject(error);
            });
        }.bind(this));
        return deferred.promise;
    };
    /**
     * habilita / desabilita o campo "Expressão"
     * @param {object} widget - widget editar item formação
     **/
    this.habilitaExpressao = function(widget){
        var isNew = widget.currentRow.__is_new;
        widget.currentRow.IDTIPOCALCULO = (widget.currentRow.IDTIPOCALCULO) ? widget.currentRow.IDTIPOCALCULO : "V";
        if (widget.currentRow.IDTIPOCALCULO == 'E') {
            widget.getField("DSEXPRESSAOCALC").readOnly    = false;
            widget.getField("DSEXPRESSAOCALC").validations = {'required':true};
            widget.getField("VRPERCCALCULO").readOnly      = false;
            widget.getField("VRPERCCALCULO").validations   = {};
            widget.getField("VRUNITARIO").readOnly         = true;
            widget.getField("VRUNITARIO").validations      = {};
            widget.getField("VRTOTAL").validations         = {};
            
            if(isNew) {
                widget.currentRow.VRUNITARIO = null;
                widget.currentRow.VRTOTAL    = null;
            }
        } else if (widget.currentRow.IDTIPOCALCULO == 'S'){
            widget.getField("DSEXPRESSAOCALC").readOnly    = true;
            widget.getField("DSEXPRESSAOCALC").validations = {};
            widget.currentRow.DSEXPRESSAOCALC              = null;
            widget.getField("VRPERCCALCULO").readOnly      = true;
            widget.getField("VRPERCCALCULO").validations   = {};
            widget.currentRow.VRPERCCALCULO                = null;
            widget.getField("VRUNITARIO").readOnly         = true;
            widget.getField("VRUNITARIO").validations      = {};
            widget.getField("VRTOTAL").validations         = {};
            
            if(isNew) {
                widget.currentRow.VRUNITARIO = null;
                widget.currentRow.VRTOTAL    = null;
            }
        } else {
            widget.getField("DSEXPRESSAOCALC").readOnly    = true;
            widget.getField("VRPERCCALCULO").readOnly      = true;
            widget.getField("DSEXPRESSAOCALC").validations = {};
            widget.getField("VRPERCCALCULO").validations   = {};
            widget.currentRow.DSEXPRESSAOCALC              = null;
            widget.currentRow.VRPERCCALCULO                = null;
            widget.getField("VRUNITARIO").readOnly         = false;
            widget.getField("VRUNITARIO").validations      = {'required':true};
            widget.getField("VRTOTAL").validations         = {'required':true};
        }
    };

     /**
     * Realiza o controle dos campos de "Medida" de acordo 
     * com o field "Utiliza Medida?"
     * @param {object} widget - widget editar item formação
     **/
    this.bloquearDadosMedida = function(widget){
        var usamedida = widget.getField('IDUSAMEDIDA').value();
        if (usamedida == 'N') {
            widget.getField('DSTPMEDIDASERV').readOnly    = true;
            widget.getField('DSTPMEDIDASERV').validations = {};
            widget.getField('CDTPMEDIDASERV').validations = {};
            widget.getField('QTMEDIDACALC').readOnly      = true;
            widget.getField('QTMEDIDACALC').validations   = {};
            widget.getField('DSTPMEDIDASERV').setValue(null);
            widget.getField('CDTPMEDIDASERV').setValue(null);
            widget.getField('QTMEDIDACALC').setValue(null);
            
        } else {
            widget.getField('DSTPMEDIDASERV').readOnly    = false;
            widget.getField('DSTPMEDIDASERV').validations = {"required": true};
            widget.getField('QTMEDIDACALC').readOnly      = false;
            widget.getField('QTMEDIDACALC').validations   = {"required": true};
        }
    };
    /**
     * Volta para a widget de mão de obra durante a inserção da proposta
     * @params{Object} widget - widget editar itens formação de preço
    **/
    this.voltarWdgMObraProposta = function(widget) {
        var rowsAlteradas = widget.dataSource.data.filter(function(data) {
            return data.changed;
        });
        
        if(rowsAlteradas.length == 0 && !idItensMObraAlterado) {
            this.utilizarItensAlteradosFormPreco(widget);
        } else {
            ScreenService.confirmMessage("Deseja salvar as alterações realizadas?", "question", function(){
                this.utilizarItensAlteradosFormPreco(widget);
            }.bind(this), function() {
                var wdgTabbed    = templateManager.container.getWidget("wdgTabbedPrincipalFormPreco");
                var wdgGridMObra = templateManager.container.getWidget("wdgMaoObraGridInserindo");
                
                wdgGridMObra.currentRow.VRTOTAL    = wdgGridMObra.backupRow.VRTOTAL;
                wdgGridMObra.currentRow.VRUNITARIO = wdgGridMObra.backupRow.VRUNITARIO;
                
                wdgTabbed.isVisible = false;
                this.voltarBackupItensMaoObra(widget);
            }.bind(this));
        }
    };
    /**
     * Volta no banco de dados os valores antigos da formação de preço
     * @params{Object} widget - widget formação de preco
    **/
    this.voltarBackupItensMaoObra  = function(widget) {
        var wdgGridMObra = templateManager.container.getWidget("wdgMaoObraGridInserindo");
        var wdgTabbed    = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
        var params = {
            "NRRELACFORMACAOPROP": wdgGridMObra.currentRow.NRRELACFORMACAOPROP, 
            "NRFORMPRECOSERVICO" : wdgGridMObra.currentRow.NRFORMPRECOSERVICO,
            "NRCALCFORMPRECSERV" : "_" + wdgGridMObra.currentRow.NRCALCFORMPRECSERV + "_",
            "IDFORMACAO"         : wdgGridMObra.currentRow.IDFORMACAO
        };
        
        General.callRestEngine("product#/fct_orcamentoaliment_widget/voltarBackupItensMaoObra", "Row", params);
        ScreenService.closeSwipe().then(function(){
            widget.dataSource.data = [];
            widget.dataSource.clearCheckedRows();
            
            wdgTabbed.enabled    = true;
            wdgGridMObra.enabled = true;
            wdgTabbed.activate();
        });
        
        templateManager.updateTemplate();
    };
    /**
     * Habilita a tela para receber um novo item de formação de preço
     * e seta o número de ordenação na tela
     * @params{Object} widget widget editar formação de preço
    **/
    this.adicionarItensFormacao = function(widget){
        if(widget.mode !== "view"){
            widget.newRow();
            this.bloquearDadosMedida(widget);
        } else {
            var nmElementoSup = widget.currentRow.DSREFECALCULO;
            var nrCalcServPai = widget.currentRow.NRCALCFORMPRECSERV;
            widget.newRow();
            this.bloquearDadosMedida(widget);
            widget.currentRow.DSREFECALCULOPAI      = nmElementoSup;
            widget.currentRow.NRCALCFORMPRECSERVPAI = nrCalcServPai;
        }
        
        var widgetParent = null;
        if(widget.name === "wdgItensFormacaoPrecoProposta"){
            widgetParent = templateManager.container.getWidget("wdgItensMaoObra");
        } else {
            widgetParent = templateManager.container.getWidget("wdgMaoObraGridInserindo");
            widget.template = "widget/form.html";
        }
        
        var nritemorcamento    = widgetParent.currentRow.NRRELACFORMACAOPROP;
        var nrformprecoservico = widgetParent.currentRow.NRFORMPRECOSERVICO;
        var params = {
            "NRFORMPRECOSERVICO" : nrformprecoservico,
            "NRITEMORCAMENTO"    : nritemorcamento,
            "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO"
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/recebeOrdenacao', "Row", params).then(function(response){
            if(!response.dataset.ORDENACAO){
                widget.currentRow.IDORDENACAO = 10;
            } else {
                var ordenacao = response.dataset.ORDENACAO.IDORDENACAO;
                ordenacao = ordenacao ? (ordenacao + 10) : 10;
                widget.currentRow.IDORDENACAO        = ordenacao;
                widget.currentRow.NRFORMPRECOSERVICO = nrformprecoservico;
                widget.currentRow.NRITEMORCAMENTO    = nritemorcamento;
            }
        }, function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Filtra a field de elemento superior para aperecer apenas os elementos
     * já inseridos na árvore
     * @params{Object} widget - widget editar formaçao de preço
    **/
    this.filtraElementoSuperiorItensFormacao = function(widget){
        if(!widget.currentRow.DSREFECALCULOPAI){
            var nrformprecoservico = null;
            widget.currentRow.DSREFECALCULOPAI = "";
            nrformprecoservico = widget.dataSource.data[0].NRFORMPRECOSERVICO;
            if(widget.currentRow.NMELEMENTO){
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[0].value = nrformprecoservico;
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[1].value = (widget.currentRow.NRCALCFORMPRECSERV) ? widget.currentRow.NRCALCFORMPRECSERV : -1;
                
                if(widget.name === "wdgItensFormacaoPrecoProposta")
                    widget.getField("DSREFECALCULOPAI").dataSourceFilter[2].value = templateManager.container.getWidget("wdgProposta").currentRow.NRSEQORCAMENTO;
                else 
                    widget.getField("DSREFECALCULOPAI").dataSourceFilter[4].value = templateManager.container.getWidget('wdgMaoObraGridInserindo').currentRow.NRRELACFORMACAOPROP;
                
                widget.getField("DSREFECALCULOPAI").reload();
            }
        }
    };
    /**
     * Realiza a validação  do campo "identificador". 
     * Não podem haver dois identificadores iguais em uma mesma composição.
     * @param {object} widget widget
     **/
    this.validaIdentificador = function(widget){
        var widgetParent = null;
        if(widget.name === "wdgItensFormacaoPrecoProposta")
            widgetParent = templateManager.container.getWidget("wdgItensMaoObra");
        else 
            widgetParent = templateManager.container.getWidget("wdgMaoObraGridInserindo");
        var nritemorcamento    = widgetParent.currentRow.NRRELACFORMACAOPROP;
        var nrformprecoservico = widgetParent.currentRow.NRFORMPRECOSERVICO;
        var IdItemCalculo      = widget.currentRow.IDITEMCALCULO;
        var nrCalcFormPrecServ = widget.currentRow.NRCALCFORMPRECSERV;
        var params = {
            "NRFORMPRECOSERVICO"   : nrformprecoservico,
            "NRITEMORCAMENTO"      : nritemorcamento,
            "IDITEMCALCULO"        : IdItemCalculo,
            "NRCALCFORMPRECSERV"   : nrCalcFormPrecServ,
            "TABELA"               : "FCT_CALCFORMPRECSERV_ITPRO"
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/validaIdentificador', "Row", params).then(function(response){
            var identificador = response.dataset.IDENTIFICADOR.VALIDA;
            if (identificador != 0){
                widget.currentRow.NMELEMENTO = "",
                widget.currentRow.IDITEMCALCULO = null;
                ScreenService.showMessage('O identificador "'+IdItemCalculo+'" já está sendo utilizado para outro elemento.'); 
            }
        }.bind(this), 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });    
    };
    /**
     * Realiza o controle de Percentual do field "VRPERCCALCULO"
     * @param {object} widget - widget mão de obra
    */
    this.Validapercentual = function(widget){
        var vrdesconto = widget.getField('VRPERCCALCULO').value();
        if (vrdesconto < 0 || vrdesconto > 100) 
            widget.currentRow.VRPERCCALCULO = null;
    };
    /**
     * Realiza o cálculo do Valor Total
     * @param {object} widget - widget mão de obra
     **/
    this.calculaValorTotal = function(widget){
        var vrUnit = widget.currentRow.VRUNITARIO;
        if (!vrUnit) {
            vrUnit = 0;
        } else {
            vrUnit = vrUnit.toString();
            var primeiraPos = vrUnit.indexOf(".");
            var ultimaPos   = vrUnit.lastIndexOf(".");
            if (primeiraPos == ultimaPos) {
                if (vrUnit.includes(",")) 
                    vrUnit = parseFloat(vrUnit.replace(".", "").replace(",", "."));
                else 
                    vrUnit = parseFloat(vrUnit.replace(",", "."));
            } else {
                var parteInicial = vrUnit.substr(0, primeiraPos + 1);
                var parteFinal   = vrUnit.substr(primeiraPos + 1);
                vrUnit = parseFloat(parteInicial.replace(".", "") + parteFinal);
            }
        }
        
        var qtCalc    = widget.currentRow.QTCALCULO;
        var usamedida = widget.currentRow.IDUSAMEDIDA;
        var qtmedida  = widget.currentRow.QTMEDIDACALC;
        var vrTot     = 0;
        if (usamedida == 'S')
            vrTot = (vrUnit * qtCalc) * qtmedida ; 
        else 
            vrTot = vrUnit * qtCalc;
        widget.currentRow.VRTOTAL = vrTot;
    };
    /**
     * Salva as alterações realizadas na tela de edição dos 
     * itens de formação de preço
     * @params{Object} widget - widget editar itens de formação (mão de obra)
    **/
    this.salvarAlteracoesItemCalculo = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            idItensMObraAlterado = true;
            
            var wdgParent;
            var isnew       = widget.currentRow.__is_new;
            var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
            widget.currentRow.NRSEQORCAMENTO  = (widget.currentRow.NRSEQORCAMENTO) ? widget.currentRow.NRSEQORCAMENTO : 0;
            widget.currentRow.NRITEMORCAMENTO = (widget.name === "wdgItensFormacaoPrecoProposta") ? 0 : -1;
            if(!widget.currentRow.DSREFECALCULO)
                widget.currentRow.DSREFECALCULO = widget.currentRow.IDITEMCALCULO + " - " + widget.currentRow.NMELEMENTO;
            
            if(widget.name === "editaCalculoItensFormPreco" || isnew){
                wdgParent = templateManager.container.getWidget("wdgMaoObraGridInserindo");
                widget.currentRow.NRSEQORCAMENTO = 0;
            } else {
                widget.dataSource.name = "product#/crud/fct_calcformprecserv_itpro_widget";
                wdgParent       = templateManager.container.getWidget("wdgItensMaoObra");
                var wdgProposta = templateManager.container.getWidget("wdgProposta");
                widget.currentRow.NRSEQORCAMENTO = wdgProposta.currentRow.NRSEQORCAMENTO;
            }
                
            widget.currentRow.NRRELACFORMACAOPROP = wdgParent.currentRow.NRRELACFORMACAOPROP;
            this.criarParametrosNewCode(widget);
            widget.saveRow().then(function(response) {
                if(response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0)
                    widget.currentRow.NRCALCFORMPRECSERV = response.dataset.columnsToReturn[0].NRCALCFORMPRECSERV;
                
                if (tipoCalculo == 'S'){
                    var params = {
                        "NRCALCFORMPRECSERV" : widget.currentRow.NRCALCFORMPRECSERV,
                        "NRITEMORCAMENTO"    : widget.currentRow.NRITEMORCAMENTO,
                        "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO"
                    };
                    
                    General.callRestEngine('product#/fct_calcformprecserv_widget/calcSomatorioFilhos', "Row", params);  
                }
                
                ScreenService.confirmMessage("Deseja recalcular a Formação de Preço?", 'question', function (success) {
                    if(widget.name === "wdgItensFormacaoPrecoProposta"){
                        widget.dataSource.name = "product#/crud/fct_calcformprecservtelaitensprop_widget";
                        ScreenService.closeSwipe();
                    } else {
                        widget.template = "widget/tree/grid.html";
                    }
                    
                    widget.list();
                    this.recalcularArvore(widget);
                }.bind(this), function(){
                    if(widget.name === "wdgItensFormacaoPrecoProposta"){
                        widget.dataSource.name = "product#/crud/fct_calcformprecservtelaitensprop_widget";
                        if(!isnew){
                            widget.view();
                        } else { 
                            ScreenService.closeSwipe();
                            widget.list();
                        }
                    } else {
                        if(!isnew){
                            UtilitiesPropostaContrato.habilitarViewItensFormacao(widget);
                        } else {
                            widget.template = "widget/tree/grid.html";
                            widget.list();
                        }
                    }
                });    
            }.bind(this))
            .catch(function(error){
                if(widget.name === "wdgItensFormacaoPrecoProposta")
                    widget.dataSource.name = "product#/crud/fct_calcformprecservtelaitensprop_widget";
                
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Adiciona na tela de mão de obra os itens selecionado
     * @params{Object} widget - widget editar itens formação de preço (mão de obra)
    **/
    this.utilizarItensAlteradosFormPreco = function(widget){
        ScreenService.closePopup();
        var wdgTabbed     = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
        var wdgMobraGrid  = templateManager.container.getWidget("wdgMaoObraGridInserindo");
        var rows          = widget.getCheckedRows();
        if(widget.dataSource.data.length !== rows.length){
            if(rows.length == 0)
                rows = widget.dataSource.data;
            else 
                rows = UtilitiesPropostaContrato.buscaArvoreFormacaoCompleta(widget, rows);
        }
        this.criarParametrosNewCode(widget);
        rows.forEach(function(row){row.NRITEMORCAMENTO = -1;});
        General.callRestEngine("product#/fct_orcamentoaliment_widget/utilizarItensFormacaoProp", "DataSet", rows).then(function(){
            rows.forEach(function(row) {
                wdgMobraGrid.dataSource.data.forEach(function(item){
                    if(row.NRFORMPRECOSERVICO === item.NRFORMPRECOSERVICO && wdgMobraGrid.currentRow.NRRELACFORMACAOPROP === item.NRRELACFORMACAOPROP){
                        if(!item.NRCALCFORMPRECSERVS)
                            item.NRCALCFORMPRECSERVS = [];
                            
                        item.NRCALCFORMPRECSERVS.push(row.NRCALCFORMPRECSERV);
                        if(row._parent == null){
                            item.IDALTERADO = "S";
                            item.VRUNITARIO += row.VRTOTAL;
                            item.VRTOTAL    += parseFloat(row.VRTOTAL) * parseFloat(item.QTFORMACAO);
                        }
                    }
                });
            });
            
            ScreenService.closeSwipe().then(function(){
                widget.dataSource.data = [];
                widget.dataSource.clearCheckedRows();
                
                wdgTabbed.enabled    = true;
                wdgMobraGrid.enabled = true;
                wdgTabbed.activate();
            });
            
            templateManager.updateTemplate();
        });
    };
    /**
     * Deletas os itens de árvores da formação de preço 
     * não selecionadas 
     * @params{Object} widget - widget editar formação de preço
    **/
    this.deletarElementosNaoUtilizados = function(widget){
        var itensNSelects = widget.dataSource.data.filter(function(item){
            return !item.__isSelected && !item._parent;
        });
        
        UtilitiesPropostaContrato.buscaArvoreFormacaoCompleta(widget, itensNSelects).forEach(function(itemNSelect){
            widget.currentRow = itemNSelect;
            widget.removeRow().then(function() {
                widget.template = "widget/tree/grid.html";
                widget.list();
                var params = {
                    "NRFORMPRECOSERVICO" : widget.currentRow.NRFORMPRECOSERVICO,
                    "NRITEMORCAMENTO"    : widget.currentRow.NRRELACFORMACAOPROP,
                    "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO",
                };
                    
                General.callRestEngine("product#/fct_calcformprecserv_widget/recalcularArvore", "Row", params).then(function(){
                    Utilities.abreArvoreElementos(widget);
                }.bind(this))
                .catch(function(){
                    ScreenService.showMessage("Falha ao recalcular o valor da Formação de Preço.");
                });
            });
        });
    };
    /**
     * Avançar para widget de despesas
     * @params{Object} widget - widget despesas propostas
    **/
    this.avancarDespesas = function(widget){
        templateManager.container.getWidget("wdgTabbedPrincipalMObra").enabled = false;
        var wdgDespesa = templateManager.container.getWidget("wdgDespesas");
        wdgDespesa.enabled   = true;
        wdgDespesa.isVisible = true;
        wdgDespesa.mode = "edit";
        wdgDespesa.activate().then(function() {
            this.setFiltroDespesas(wdgDespesa);
        }.bind(this));
    };
    /**
     * Adiciona à tela de despesas o filtro de formação de preço e servico
     * @params{Object} widget    - widget despesas
     * @params{Object} wdgFilter - widget de filtro (servico ou formação)
     * @params{String} nameCampo - nome do campo que será filtrado 
    **/
    this.setFiltroDespesas = function(widget){
        var servicos  = [];
        var formacoes = [];
        templateManager.container.getWidget("wdgServicosInserindo").dataSource.getCheckedRows().forEach(function(serv){
            servicos.push(serv.NRCOMPSERVICO);
        });
        templateManager.container.getWidget("wdgMaoObraGridInserindo").dataSource.data.forEach(function(form){
            formacoes.push(form.NRFORMPRECOSERVICO);
        });
        servicos  = this.retirarValoresDuplicados(servicos);
        formacoes = this.retirarValoresDuplicados(formacoes);
        
        widget.dataSourceFilter[0].value = servicos;
        widget.dataSourceFilter[1].value = formacoes;
        widget.reload().then(function(){
            setTimeout(function(){
                widget.dataSource.data.forEach(function(despesa){
                    despesa.__isSelected = true;
                    widget.dataSource.addCheckedRows(despesa);
                }.bind(this));
                
                widget.getAction("save").isVisible   = false;
                widget.getAction("cancel").isVisible = false;
                templateManager.updateTemplate();
                this.calcularDespesaPorPeriodo(widget);
            }.bind(this), 300);
        }.bind(this));
    };
    /**
     * Verifica o período da despesa e cálcula o seu valor mensal
     * @params{Object} widget - widget despesas proposta inserindo
    **/
    this.validarECalcularPeriodo = function(rowDespesa, nameVrDesp, nameCampoVrTotal, nameCampoPeriodo){
        var vrDespesa = this.tirarMascaraValores(rowDespesa[nameVrDesp]);
        switch(rowDespesa[nameCampoPeriodo]){
                case '1': //Diária
                    rowDespesa[nameCampoVrTotal] = vrDespesa * 30;
                break;
                case '2': //Semanalmente
                    rowDespesa[nameCampoVrTotal] = vrDespesa * (30/7);
                break;
                case '3': //Quinzenalmente
                    rowDespesa[nameCampoVrTotal] = vrDespesa * 2;
                break;
                case '4': //Mensalmente
                    rowDespesa[nameCampoVrTotal] = vrDespesa;
                break;
                case '5': //Trimestralmente
                    rowDespesa[nameCampoVrTotal] = vrDespesa / 3;
                break;
                case '6': //Semestralmente
                    rowDespesa[nameCampoVrTotal] = vrDespesa / 6;
                break;
                case '7': //Anualmente
                    rowDespesa[nameCampoVrTotal] = vrDespesa / 12;
                break;
            }
    };
    /**
     * Verifica qual widget está mandando calcular a despesa e envia a row certa
     * @params{Object} widget - widget despesas (item ou inserindo)
    **/
    this.calcularDespesaPorPeriodo = function(widget){
        var nameVrTotal = (widget.name === "wdgItensDespesasProposta") ? "VRDESPESAORCA" : "VRTOTDESPPERIODO";
        var namePeriodo = (widget.name === "wdgItensDespesasProposta") ? "CDPERIODODESPPROP" : "CDPERIODO";
        var nameVrDesp  = (widget.name === "wdgItensDespesasProposta") ? "VRUNITDESPESA" : "VRDESPESA";
        if(widget.name === "wdgItensDespesasProposta" && widget.mode === "edit"){
            this.validarECalcularPeriodo(widget.currentRow, "VRUNITDESPESA", "VRDESPESAORCA", "CDPERIODODESPPROP");
        } else {
            widget.dataSource.data.forEach(function(despesa){
                this.validarECalcularPeriodo(despesa, nameVrDesp, nameVrTotal, namePeriodo);
            }.bind(this));
        }
    };
    /**
     * Retira os valores repetidos de um array
     * @params{Array} arrayData - array de dados
    **/
    this.retirarValoresDuplicados = function(arrayData){
        return arrayData.filter(function(value, index){
            return arrayData.indexOf(value) === index;
        });
    };
    /**
     * Volta para a widget de mão de obra
     * durante a inserção da proposta
     * @params{Object} widget - widget despesas
    **/
    this.voltarDespesaMObra = function(widget){
        widget.enabled = false;
        var wdgMObra     = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
        wdgMObra.enabled = true;
        wdgMObra.mode    = "edit";
        wdgMObra.activate();
    };
    /**
     * Abre um popup que permitirá a inserção de uma nova despesa
    **/
    this.adicionarNovaDespesa = function(widget){
        var despesas = [];
        var popupDespesa = templateManager.container.getWidget("wdgPopupInsereDesp");
        widget.dataSource.data.forEach(function(despesa){
            if(despesa.CDDESPESA)
                despesas.push(despesa.CDDESPESA);
        });
        
        popupDespesa.isVisible = true;
        ScreenService.openPopup(popupDespesa).then(function(){
            popupDespesa.getField("CDDESPESA").dataSourceFilter[0].value = despesas.length > 0 ? despesas : [-1];
            popupDespesa.getField("CDDESPESA").reload();
            popupDespesa.edit();
            popupDespesa.newRow();
            popupDespesa.activate();
        });
    };
    /**
     * Cancela a inserção de uma nova despesa
     * na tela de despesas durante a criação de uma proposta
    **/
    this.cancelarInsercaoNovaDespesa = function(){
        ScreenService.closePopup();
        var wdgDespesa  = templateManager.container.getWidget("wdgDespesas");
        wdgDespesa.enabled = true;
        wdgDespesa.mode    = "edit";
        wdgDespesa.activate().then(function(){
            setTimeout(function() {
                wdgDespesa.getAction("save").isVisible   = false;
                wdgDespesa.getAction("cancel").isVisible = false;
            }, 300);
        });
    };
    /**
     * Salva uma nova despesa criada durante a inserção da proposta
     * @params{Object} widget - widget popup despesas
    **/
    this.salvarNovaDespesa = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgDespesa = templateManager.container.getWidget("wdgDespesas");
            widget.currentRow.CDDESPESA.forEach(function(despesa){
                widget.getField("CDDESPESA").dataSource.data.forEach(function(valueDespesa){
                    if(valueDespesa.CDDESPESA === despesa){
                        var row = {
                            "DSDESPESA"       : valueDespesa.DSDESPESA,
                            "CDDESPESA"       : despesa,
                            "VRDESPESA"       : valueDespesa.VRDESPESA ? valueDespesa.VRDESPESA : "0",
                            "CDPERIODO"       : valueDespesa.CDPERIODO,
                            "VRTOTDESPPERIODO": 0,
                            "NRCOMPSERVICO"   : "",
                            "__is_new"        : false,
                            "__isSelected"    : true
                        };
                        wdgDespesa.dataSource.data.push(row);
                        wdgDespesa.dataSource.addCheckedRows(row);
                    }
                });
            });
            this.calcularDespesaPorPeriodo(wdgDespesa);
            this.cancelarInsercaoNovaDespesa();
        }
    };
    /**
     * Avançar para widget de investimento
     * @params{Object} widget - widget despesas propostas
    **/
    this.avancarInvestimento = function(widget){
        this.calcularWidgetDespesaInserindo(widget);
        Sm00001000_PropostaAlimenticia_Repository.getDataLocalStorageEngine("wdgDespesas", widget.dataSource.data);
        
        widget.enabled = false;
        var wdgInvest  = templateManager.container.getWidget("wdgInvestimentosInserindo");
        wdgInvest.enabled   = true;
        wdgInvest.isVisible = true;
        wdgInvest.mode = "edit";
        wdgInvest.activate().then(function() {
            this.setFiltroInvestimento(wdgInvest);
        }.bind(this));
    };
    /**
     * Aplica um filtro de servico e formação na widget de investimento
     * além de calcular o valor total
     * @params{Object} widget - widget investimento
    **/
    this.setFiltroInvestimento = function(widget){
        var servicos  = [];
        var formacoes = [];
        templateManager.container.getWidget("wdgServicosInserindo").dataSource.getCheckedRows().forEach(function(serv){
            servicos.push(serv.NRCOMPSERVICO);
        });
        templateManager.container.getWidget("wdgMaoObraGridInserindo").dataSource.data.forEach(function(form){
            formacoes.push(form.NRFORMPRECOSERVICO);
        });
        
        servicos  = this.retirarValoresDuplicados(servicos);
        formacoes = this.retirarValoresDuplicados(formacoes);
        widget.dataSourceFilter[0].value = servicos;
        widget.dataSourceFilter[1].value = formacoes;
        widget.reload().then(function(){
            widget.dataSource.data.forEach(function(investimento){
                investimento.VRTOTINVESTIMENTO = investimento.VRINVESTIMENTO;
                investimento.QTINVESTIMENTO    = "1",
                investimento.__isSelected      = true;
                widget.dataSource.addCheckedRows(investimento);
            });
            widget.getAction("save").isVisible   = false;
            widget.getAction("cancel").isVisible = false;
        }.bind(this));
    };
    /**
     * Calcula o valor total do investimento
     * @params{Object} widget - widget investimento
    **/
    this.calcularVrTotInvestimento = function(rowInvestimento, vrunitario, vrTotal, qtInvestimento){
        var qtAmortizacao = (rowInvestimento["QTMESARMOTIZACAO_inline"])  ? rowInvestimento["QTMESARMOTIZACAO_inline"]  : rowInvestimento["QTMESARMOTIZACAO"];
        vrunitario     = (rowInvestimento[vrunitario + "_inline"])     ? rowInvestimento[vrunitario + "_inline"]     : rowInvestimento[vrunitario];
        qtInvestimento = (rowInvestimento[qtInvestimento + "_inline"]) ? rowInvestimento[qtInvestimento + "_inline"] : rowInvestimento[qtInvestimento];
        qtInvestimento = qtInvestimento ? qtInvestimento : 1;
            
        rowInvestimento[qtInvestimento] = (rowInvestimento[qtInvestimento] && rowInvestimento[qtInvestimento] !== 0) ? parseFloat(rowInvestimento[qtInvestimento]) : 1;
        rowInvestimento[vrTotal]        = parseFloat(qtInvestimento) * parseFloat(vrunitario);
        rowInvestimento.VRPARCELA       = parseFloat(rowInvestimento[vrTotal])/parseFloat(qtAmortizacao);
    };
    /**
     * Valida se o que está sendo trabalhado é o datasource da tela ou 
     * a currentRow
     * @params{Object} widget - widget investimento (itens ou inserindo)
    **/
    this.validarParaCalculoInvestimento = function(widget){
        var vrUnitario    = (widget.name = "wdgItensInvestimentosProp") ? "VRUNITINVESTORCA" : "VRINVESTIMENTO"; 
        var vrTotal       = (widget.name = "wdgItensInvestimentosProp") ? "VRTOTINVETORCA" : "VRTOTINVESTIMENTO"; 
        var quantidade    = (widget.name = "wdgItensInvestimentosProp") ? "QTINVESTORCA" : "QTINVESTIMENTO";

        if(widget.name === "wdgItensInvestimentosProp" && widget.mode !== "list"){
            this.calcularVrTotInvestimento(widget.currentRow, "VRUNITINVESTORCA", "VRTOTINVETORCA", "QTINVESTORCA", "QTMESARMOTIZACAO");
        } else {
            widget.dataSource.data.forEach(function(investimento){
                this.calcularVrTotInvestimento(widget.currentRow, vrUnitario, vrTotal, quantidade);
            });
        }
    };
    /**
     * Volta para a aba de despesas durante a inserção da proposta
     * @params{Object} widget - widget investimento proposta
    **/
    this.voltarInvestDepesas = function(widget){
        widget.enabled = false;
        var wdgDespesas = templateManager.container.getWidget("wdgDespesas");
        Sm00001000_PropostaAlimenticia_Repository.getDataLocalStorageEngine("wdgDespesas").then(function(response) {
            if(response) 
                wdgDespesas.dataSource.data = JSON.parse(response);
    
            wdgDespesas.enabled = true;
            wdgDespesas.mode    = "edit";
            wdgDespesas.activate();
        }.bind(this));
    };
    /**
     * Avançar para widget de taxas
     * @params{Object} widget - widget investimento
    **/
    this.avancarTaxas = function(widget){
        this.calcularWidgetInvetimentoInserindo(widget);
        Sm00001000_PropostaAlimenticia_Repository.saveDataLocalStorageEngine("wdgInvestimentosInserindo", widget.dataSource.data);
        widget.enabled = false;
        
        var wdgTaxas = templateManager.container.getWidget("wdgTaxasInserindo");
        wdgTaxas.enabled   = true;
        wdgTaxas.isVisible = true;
        wdgTaxas.mode = "edit";
        wdgTaxas.activate().then(function() {
            wdgTaxas.reload().then(function(){
                setTimeout(function() {
                    wdgTaxas.getAction("save").isVisible   = false;
                    wdgTaxas.getAction("cancel").isVisible = false;
                }, 300);
            });
        }.bind(this));
    };
    /**
     * Volta para a widget de investimento durante a inserção da proposta
     * @params{Object} widget - widget taxas proposta
    **/
    this.voltarTaxasInvest = function(widget){
        widget.enabled = false;
        var wdgInvest = templateManager.container.getWidget("wdgInvestimentosInserindo");
        Sm00001000_PropostaAlimenticia_Repository.getDataLocalStorageEngine("wdgInvestimentosInserindo").then(function(response) {
            if(response)
                wdgInvest.dataSource.data = JSON.parse(response);
                
            wdgInvest.enabled = true;
            wdgInvest.mode    = "edit";
            wdgInvest.activate();
        }.bind(this));
    };
    /**
     * Avança para a aba de custo final durante a inserção da proposta
     * @params{Object} widget - widget taxas proposta
    **/
    this.avancarCustoFinal = function(widget){
        widget.enabled    = false;
        var wdgCustoFinal = templateManager.container.getWidget("wdgTabbedPrincipalResumoProp");
        wdgCustoFinal.enabled   = true;
        wdgCustoFinal.isVisible = true;
        wdgCustoFinal.mode = "edit";
        wdgCustoFinal.activate().then(function() {
            this.setDadosCustoFinal();
        }.bind(this));
    };
    /**
     * Voltar para a widget de taxa durante a inserção da proposta
     * @params{Object} widget - widget curto final
    **/
    this.voltarCFinalTaxa = function(widget){
        widget.enabled = false;
        var wdgTaxas   = templateManager.container.getWidget("wdgTaxasInserindo");
        wdgTaxas.enabled = true;
        wdgTaxas.mode    = "edit";
        wdgTaxas.activate();
    };
    /**
     * Coloca a lista de custos financeiros em negrito
     * @params{Object} widget - widget grid financeiro 
    **/
    this.negritarListaCusto = function(element) {
        if (element.field && (["VRFINANCEIRO", "DSFINANCEIRO"].includes(element.field.name))) {
            var style = element.context.children[0].getAttribute("style");
            if (!style) {
                element.context.children[0].setAttribute("style", " font-weight: bold;");
            } else if (style.indexOf("font-weight: bold;") === -1) {
                element.context.children[0].setAttribute("style", style + " font-weight: bold;");
            }
        }
    };
    /**
     * Efetua os cálculos da widget de custo final
    **/
    this.setDadosCustoFinal = function(){
        var wdgFinanceiro = templateManager.container.getWidget("wdgResumoGrid");
        var wdgRServico   = templateManager.container.getWidget("wdgResumoServico");
        this.calcularValoresServico(wdgRServico);
        this.calcularFinanceiro(wdgFinanceiro, true);
    };
    /**
     * Retorna todos os serviços relacionados a árvore de serviço a partir de um 
     * elemento selecionado
     * @params{Object} rowElemento - elemento selecionado
    **/
    this.getServicosRelacionados = function(rowElemento){
        var wdgCusto      = templateManager.container.getWidget("wdgCustoProposta");
        var wdgServico    = templateManager.container.getWidget("wdgServicosInserindo");
        var cdcompservico = [];
        var cdServRelac   = [];
        
        this.getArvoreServico(wdgServico, rowElemento, [], false).forEach(function(servico) {
            cdcompservico.push(servico.NRCOMPSERVICO);
        });
        cdcompservico = this.retirarValoresDuplicados(cdcompservico);
        wdgCusto.dataSource.data.forEach(function(custo){
            custo.NRCOMPSERVICO.forEach(function(compCusto){
                if(!cdServRelac.includes(custo.CDSERVORCARELAC) && custo.CDSERVORCARELAC)
                    cdServRelac.push(custo.CDSERVORCARELAC);
            });
        });
        
        return cdServRelac;
    };
    /**
     * Retorna o valor total do custo de um serviço
     * @params{Objects} wdgCusto - widget de custo inserindo
     * @params{Objects} wdgServico - widget de servico inserindo 
     * @params{Objects} rowServico - row do elemento que terá o valor calculado
    **/
    this.getCustoServicoTotal = function(wdgCusto, wdgServico, rowServico){
        var vrCustoTot    = 0;
        var cdcomposicoes = [];
        var eleSomados    = [];
        var elementoRaiz  = [];
        var nrCompCusto   = [];
        var elementoAtual = null;
        
        this.getArvoreServico(wdgServico, rowServico, [], false).forEach(function(eleArvore){
            elementoRaiz.push(eleArvore.IDELEMENTO);
            cdcomposicoes.push(eleArvore.NRCOMPSERVICO);
		});
		
		elementoRaiz  = this.retirarValoresDuplicados(elementoRaiz);
		cdcomposicoes = this.retirarValoresDuplicados(cdcomposicoes);
		wdgCusto.dataSource.data.forEach(function(custo){
            elementoAtual = custo.IDELEMENTO; 
            if(elementoAtual.split("_").length < 3)
	            elementoAtual += "_" + custo.CDSERVORCARELAC;
	            
	        if(!eleSomados.includes(elementoAtual) && custo.CDSERVORCARELAC && elementoRaiz.includes(custo.IDELEMENTORAIZ)){
        		nrCompCusto = custo.NRCOMPSERVICO.filter(function (element, index) {
                    if(cdcomposicoes.indexOf(element) == -1)
                        return element;
                });
                if(nrCompCusto && nrCompCusto.length){
                    vrCustoTot += parseFloat(custo.VRCUSTOSERV);
                    eleSomados.push(elementoAtual);
                }
	        }
        });
        return vrCustoTot;
    };
    /**
     * Calcula os valores dos serviços da proposta
     * @params{Object} widget - widget de resumo do serviço
    **/
    this.calcularValoresServico = function(widget){
        widget.dataSource.data = [];
        var wdgCusto    = templateManager.container.getWidget("wdgCustoProposta");
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var wdgServico  = templateManager.container.getWidget("wdgServicosInserindo");
        var wdgTaxas    = templateManager.container.getWidget("wdgTaxasInserindo");
        var elCalculado = [];
        var qtServicos  = 0;
        var nmElemento  = "";
        
        var totalOrcamento = this.calcularValorPropSemTaxa();
        totalOrcamento *= (1 + (this.getSomatorioDataSource(wdgTaxas, "VRPERCTAXA", true)/100));
        totalOrcamento *= (1 + (wdgProposta.currentRow.VRPERCMARKUP ? (wdgProposta.currentRow.VRPERCMARKUP/100) : 0));

        var elementoRaiz = wdgServico.dataSource.getCheckedRows().filter(function(serv) {
            if(serv.TIPO === "C" && !serv.IDARVOREPAI)    
                qtServicos += (serv.QTMENSAL <= 0) ? 1 : parseFloat(serv.QTMENSAL);
            return serv.TIPO === "C";
        });
        
        qtServicos = qtServicos <= 0 ? 1 : qtServicos;
        wdgCusto.dataSource.data.forEach(function(custoServ){
            custoServ.NRCOMPSERVICO.forEach(function(nrcompCusto){
                elementoRaiz.forEach(function(servico){
                    if(custoServ.NRCOMPSERVICO.includes(servico.NRCOMPSERVICO.toString()) && !elCalculado.includes(custoServ.IDELEMENTORAIZ)){
                        var vrPercServico  = this.getVrPercentualServico(null, servico);
                        var vrcusto        = this.getCustoServicoTotal(wdgCusto, wdgServico, servico);
                        var qtElemento = this.getArvoreServico(wdgServico, servico, [], false).filter(function(serv){
                            if(!serv.IDARVOREPAI)
                                nmElemento = serv.NMELEMENTO;
                            return !serv.IDARVOREPAI;
                        })[0]["QTMENSAL"];
                        var vrTotalServico = this.distribuiValorServico(vrPercServico/100, qtElemento, vrcusto);
                        widget.dataSource.data.push({
                            "NRCOMPSERVICO"       : servico.NRCOMPSERVICO,
                            "IDELEMENTO"          : custoServ.IDELEMENTORAIZ,
                            "CDSERVORCA"          : this.getServicosRelacionados(servico),
                            "NMELEMENTO"          : nmElemento,
                            "VRCUSTO"             : vrcusto,
                            "VRPERCSERVICO"       : vrPercServico,
                            "VRPRECOIDEAL"        : vrTotalServico / qtElemento,
                            "VRUNITITEMORCA"      : vrTotalServico / qtElemento,
                            "VRUNITITEMORCA_GRID" : vrTotalServico / qtElemento,
                            "VRTOTALITEMORCA"     : vrTotalServico,
                            "VRTOTIDEALSERV"      : vrTotalServico,
                            "QTITEMORCAMENTO"     : qtElemento,
                            "NRTIPOORCAMENTO"     : custoServ.NRTIPOORCAMENTO
                        });
                        elCalculado.push(custoServ.IDELEMENTORAIZ);
                    }
                }.bind(this));
            }.bind(this));
        }.bind(this));
    };
    /**
     * Distribui o valor dos demais componentes do serviço
     * de acordo com a porcentagem do custo alimentar de cada um
     * @params{float} vrpercservico - valor do percentual do servico
    **/
    this.distribuiValorServico = function(vrpercservico, qtElemento, vrcusto){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var wdgTaxas    = templateManager.container.getWidget("wdgTaxasInserindo");
        var totalCustoAliment = this.getVrTotalServico(null);
        var qtMesesProposta   = wdgProposta.currentRow.QTDIASVALIDADE/30;
        
        var totalOrcamento = this.calcularValorPropSemTaxa();
        totalOrcamento *= (1 + (this.getSomatorioDataSource(wdgTaxas, "VRPERCTAXA", true)/100));
        totalOrcamento *= (1 + (wdgProposta.currentRow.VRPERCMARKUP ? (wdgProposta.currentRow.VRPERCMARKUP/100) : 0));
        
        var valorSCustoAlimentar = totalOrcamento - totalCustoAliment;
        var totalServico = (qtElemento * vrcusto * qtMesesProposta) + (valorSCustoAlimentar * vrpercservico);
        return totalServico;
    };
    /**
     * Cálculo financeiro ligados a proposta
     * @params{Object} widget - widget de resumo financeiro
     * @params{Boolean} calcularPrecoIdeal - necessidade do calculo do preço ideal 
    **/
    this.calcularFinanceiro = function(widget, calcularPrecoIdeal){
        var wdgMObra        = templateManager.container.getWidget("wdgMaoObraGridInserindo");
        var wdgDespesas     = templateManager.container.getWidget("wdgDespesas");
        var wdgInvestimento = templateManager.container.getWidget("wdgInvestimentosInserindo");
        var wdgTaxas        = templateManager.container.getWidget("wdgTaxasInserindo");
        var wdgProposta     = templateManager.container.getWidget("wdgProposta");
        var wdgImpostos     = templateManager.container.getWidget("wdgImpostoProposta");

        var totalOrcaBruto = this.calcularValorPropSemTaxa();
        var totalTaxa      = totalOrcaBruto * (this.getSomatorioDataSource(wdgTaxas, "VRPERCTAXA", true)/100);
        totalOrcaBruto     += totalTaxa;
        
        var totalCmTaxa    = totalOrcaBruto;
        var totalMarkup    = totalOrcaBruto * (wdgProposta.currentRow.VRPERCMARKUP ? (wdgProposta.currentRow.VRPERCMARKUP/100) : 0);
        totalOrcaBruto     += totalMarkup;
        
        var totalImposto   = totalOrcaBruto * (this.getSomatorioDataSource(wdgImpostos, "VRPRECIMPOSTO", true)/100);
        var totalOrcamento = totalOrcaBruto - totalImposto;
        
        var totalMObra     = this.getSomatorioDataSource(wdgMObra, "VRTOTAL", false);
        var totalDespesa   = this.getSomatorioDataSource(wdgDespesas, "VRTOTDESPPERIODO", true);
        var totalInvest    = this.getSomatorioDataSource(wdgInvestimento, "VRPARCELA", true);
        var totalServico   = this.getVrTotalServico(null);
        var totalSTaxa     = this.calcularValorPropSemTaxa();
        
        this.setValoresWdgRFinanceiro(widget, totalOrcaBruto, totalImposto, totalOrcamento, totalServico, totalCmTaxa, totalMObra, totalDespesa, totalInvest, totalTaxa, totalMarkup, totalSTaxa, totalOrcaBruto, totalImposto, totalOrcamento, totalMarkup);
    };
    /**
     * Retorna o percentual de um servico relacionado 
     * @params{int} cdServRelac - servico que terá o percentual calculado
     * @params{Object} rowServico - row do elemento que terá o percential calculado
    **/
    this.getVrPercentualServico = function(cdServRelac, rowServico){
        var wdgServico = templateManager.container.getWidget("wdgServicosInserindo");
        var wdgCusto   = templateManager.container.getWidget("wdgCustoProposta");
        var totalServico  = 0;
        var totalSrvRelac = 0;
        wdgServico.dataSource.getCheckedRows().filter(function(serv) {
            return serv.TIPO === "C" && !serv.IDARVOREPAI;
        }).forEach(function(elementoRaiz){
            totalServico  += this.getCustoServicoTotal(wdgCusto, wdgServico, elementoRaiz);
        }.bind(this));
        if(cdServRelac)
            totalSrvRelac = this.getVrTotalServico(cdServRelac);
        else
            totalSrvRelac = this.getCustoServicoTotal(wdgCusto, wdgServico, rowServico);
      
        return totalSrvRelac * 100 / totalServico;
    };
    /**
     * Retorna o cálculo do valor do serviço da proposta
     * @param{int} cdServRelac - caso não seja nulo irá retornar o valor do servico para um determinado servico
    **/
    this.getVrTotalServico = function(cdServRelac){
        var totalServico = 0;
        var wdgCusto     = templateManager.container.getWidget("wdgCustoProposta");
        var wdgServico   = templateManager.container.getWidget("wdgServicosInserindo");
        var wdgProposta  = templateManager.container.getWidget("wdgProposta");
        var qtMeses      = wdgProposta.currentRow.QTDIASVALIDADE ? wdgProposta.currentRow.QTDIASVALIDADE/30 : 1;
        var cdcomposicoes = [];

        wdgServico.dataSource.getCheckedRows().filter(function(servico){
            return servico._childCount === 0 && servico.QTMENSAL;    
        }).forEach(function(servicoCusto){
            cdcomposicoes = []; 
            this.getArvoreServico(wdgServico, servicoCusto, [], false).forEach(function(servSelect){
                cdcomposicoes.push(servSelect.NRCOMPSERVICO);
            });
            cdcomposicoes = this.retirarValoresDuplicados(cdcomposicoes);
            wdgCusto.dataSource.data.forEach(function(custo){
                custo.NRCOMPSERVICO.forEach(function(nrcomposicao){
                    if(cdcomposicoes.includes(parseInt(nrcomposicao))){
                        if((!cdServRelac && custo.CDSERVORCARELAC)){
                            totalServico += (parseFloat(servicoCusto.QTMENSAL) * parseFloat(custo.VRCUSTOSERV) * qtMeses); 
                        } else {
                            if(cdServRelac && custo.CDSERVORCARELAC === cdServRelac){
                                totalServico += (parseFloat(custo.VRCUSTOSERV) * qtMeses);
                            }
                        }
                    }
                }.bind(this));
            }.bind(this));
        }.bind(this));
        
        return totalServico;
    };
    /**
     * Retorna o cálculo do valor bruto da proposta
    **/
    this.calcularValorPropSemTaxa = function(){
        var wdgMObra        = templateManager.container.getWidget("wdgMaoObraGridInserindo");
        var wdgDespesas     = templateManager.container.getWidget("wdgDespesas");
        var wdgInvestimento = templateManager.container.getWidget("wdgInvestimentosInserindo");
        
        var total = 0;
        total += this.getVrTotalServico(null);
        total += this.getSomatorioDataSource(wdgMObra, "VRTOTAL", false);
        total += this.getSomatorioDataSource(wdgDespesas, "VRTOTDESPPERIODO", true);
        total += this.getSomatorioDataSource(wdgInvestimento, "VRPARCELA", true);
        
        return total;
    };
    /**
     * Retorna o somatório dos valores desejados em uma widget
     * @params{Object} widget - widget que terá os dados somados
     * @params{string} nmCampoVr - nome do campo de valor
     * @params{boolean} checked - se os dados precisam estar selecionados ou não
    **/
    this.getSomatorioDataSource = function(widget, nmCampoVr, checked){
        var total = 0;
        var dataset = !checked ? widget.dataSource.data : widget.dataSource.getCheckedRows(); 
        dataset.forEach(function(data){
            total += this.tirarMascaraValores(data[nmCampoVr]);
        }.bind(this));
        return total;
    };
    /**
     * Retira a mascara de formatação dos valores 
     * e o inicializa com 0 caso seja null
     * @params{string} vrUnitario - valor a ser formatado 
    **/
    this.tirarMascaraValores = function(vrUnitario){
        if (!vrUnitario) {
            vrUnitario = 0;
        } else {
            vrUnitario      = vrUnitario.toString();
            var primeiraPos = vrUnitario.indexOf(".");
            var ultimaPos   = vrUnitario.lastIndexOf(".");
            if (primeiraPos == ultimaPos) {
                if (vrUnitario.includes(",")) 
                    vrUnitario = parseFloat(vrUnitario.replace(".", "").replace(",", "."));
                else 
                    vrUnitario = parseFloat(vrUnitario.replace(",", "."));
            } else {
                var parteInicial = vrUnitario.substr(0, primeiraPos + 1);
                var parteFinal   = vrUnitario.substr(primeiraPos + 1);
                vrUnitario       = parseFloat(parteInicial.replace(".", "") + parteFinal);
            }
        }
        return parseFloat(vrUnitario.toFixed(2));
    };
    /**
     * Retorna a árvore de serviço a partir de um elemento base
     * @params{Object} widget - widget de serviços grid
     * @params{Object} rowElemento - row do elemento base
    **/
    this.getArvoreServico = function(widget, rowElemento, subArvoreServico, subArvore){
        var elementos   = [];
        var idElementos = [];
        subArvoreServico.push(rowElemento);
        if(rowElemento.IDARVOREPAI){
            widget.dataSource.data.forEach(function(servico) {
                if(servico.IDARVORE === rowElemento.IDARVOREPAI)
                    elementos = this.getArvoreServico(widget, servico, subArvoreServico, subArvore);
            }.bind(this));
        } else {
            idElementos.push(rowElemento.IDARVORE);
            elementos.push(rowElemento);
            widget.dataSource.data.forEach(function(servico) {
                if(servico.IDARVOREPAI === rowElemento.IDARVORE || idElementos.includes(servico.IDARVOREPAI)) {
                    elementos.push(servico);
                    idElementos.push(servico.IDARVORE);
                }
            });
        }
        
        return subArvore ? subArvoreServico : elementos;
    };
    /**
     * Retorna a árvore de serviço a partir de um elemento base
     * @params{Object} widget - widget de serviços grid
     * @params{Object} rowElemento - row do elemento base
    **/
    this.getArvoreInvertidaServico = function(widget, rowElemento, subArvoreServico, subArvore){
        var elementos   = [];
        var idElementos = [];
        
        subArvoreServico.push(rowElemento);
        if(rowElemento._childCount > 0){
            widget.dataSource.data.forEach(function(servico) {
                if(servico.IDARVOREPAI === rowElemento.IDARVORE)
                    elementos = this.getArvoreInvertidaServico(widget, servico, subArvoreServico, subArvore);
            }.bind(this));
        } else {
            idElementos.push(rowElemento.IDARVORE);
            elementos.push(rowElemento);
            widget.dataSource.data.forEach(function(servico) {
                if(servico.IDARVORE === rowElemento.IDARVOREPAI || idElementos.includes(servico.IDARVOREPAI)) {
                    elementos.push(servico);
                    idElementos.push(servico.IDARVORE);
                }
            });
        }
        
        return subArvore ? subArvoreServico : elementos;
    };
    /**
     * Retorna quantas vezes o dia da semana passado como parâmetro aparece no período
    **/
    this.getTotalDiaDaSemanaPeriodo = function() {
        var wdgProposta  = templateManager.container.getWidget("wdgProposta");
        var dataInicial  = moment(wdgProposta.currentRow.DTREFORCAMENTO, 'DD/MM/YYYY');
        var dia          = null;
        var totalDias    = {
            "QTSEGUNDA" : 0,
            "QTTERCA"   : 0,
            "QTQUARTA"  : 0,
            "QTQUINTA"  : 0,
            "QTSEXTA"   : 0,
            "QTSABADO"  : 0, 
            "QTDOMINGO" : 0
        };
        
        var diasDaSemana = {
            "Segunda-feira" : "QTSEGUNDA",
            "Terça-feira"   : "QTTERCA",
            "Quarta-feira"  : "QTQUARTA",
            "Quinta-feira"  : "QTQUINTA",
            "Sexta-feira"   : "QTSEXTA",
            "Sábado"        : "QTSABADO", 
            "Domingo"       : "QTDOMINGO"
        }; 
        
        for(var i = 0; i < wdgProposta.currentRow.QTDIASVALIDADE; i++) {
            dia = dataInicial.add(1, 'day').format("dddd");
            totalDias[diasDaSemana[dia]]++;
        }
        
        return totalDias;
    };
    /**
     * Salva a edição da quantidade de elementos 
    **/
    this.salvarEdicaoServico = function(widget){
        var wdgServico  = templateManager.container.getWidget("wdgServicosInserindo");
        var wdgTabbed   = templateManager.container.getWidget("wdgTabbedServico");
        var wdgProposta = templateManager.container.getWidget("wdgProposta");

        var periodoProposta = parseInt(wdgProposta.currentRow.QTDIASVALIDADE/30);
        var rowQMensal      = Utilities.tirarMascaraValores(widget.currentRow.QTMENSAL);
        var rowQSabado      = Utilities.tirarMascaraValores(widget.currentRow.QTSABADO);
        var rowQdomingo     = Utilities.tirarMascaraValores(widget.currentRow.QTDOMINGO);
        
        var totalFimSemana   = 0;
        var totalDiasSemana  = 0;
        var somatorioPeriodo = 0;
        var totalDiaPeriodo  = this.getTotalDiaDaSemanaPeriodo();
        
        totalFimSemana = (rowQdomingo * totalDiaPeriodo["QTDOMINGO"]) + (rowQSabado * totalDiaPeriodo["QTSABADO"]);
        ["QTSEGUNDA", "QTTERCA", "QTQUARTA", "QTQUINTA", "QTSEXTA"].forEach(function(dia){
            somatorioPeriodo = (!widget.currentRow[dia] ? 0 : Utilities.tirarMascaraValores(widget.currentRow[dia]));
            somatorioPeriodo *= totalDiaPeriodo[dia];
            totalDiasSemana += somatorioPeriodo;
        });
        
        if(rowQMensal && rowQMensal > 0 && rowQMensal < (totalFimSemana + totalDiasSemana)) {
            ScreenService.showMessage("Operação Bloqueada!<br>O valor total mensal não pode ser menor que o somatório das quantidades referentes aos dias da semana.<br>Valor total do período: " + (totalDiasSemana + totalFimSemana));
        } else {
            if(!rowQMensal || rowQMensal == 0) rowQMensal = totalDiasSemana + totalFimSemana;
            if((totalDiasSemana + totalFimSemana) == 0) rowQMensal *= periodoProposta; 
            wdgServico.dataSource.data.forEach(function(row) {
                if(row.IDELEMENTO === widget.currentRow.IDELEMENTO){
                    row.QTMENSAL    = rowQMensal;
                    row.QTDIAUTIL   = totalDiasSemana;
                    row.QTFIMSEMANA = totalFimSemana;
                    row.HRENTRADA   = widget.currentRow.HRENTRADA;
                    row.HRSAIDA     = widget.currentRow.HRSAIDA;
                    row.QTSEGUNDA   = widget.currentRow.QTSEGUNDA;
                    row.QTTERCA     = widget.currentRow.QTTERCA;
                    row.QTQUARTA    = widget.currentRow.QTQUARTA;
                    row.QTQUINTA    = widget.currentRow.QTQUINTA;
                    row.QTSEXTA     = widget.currentRow.QTSEXTA;
                    row.QTSABADO    = widget.currentRow.QTSABADO;
                    row.QTDOMINGO   = widget.currentRow.QTDOMINGO;
                }
            });
            
            this.atualizarSomatorioQuantidadeServico(widget);
            widget.currentRow      = {};
            widget.dataSource.data = [];
            widget.label           = "";
            widget.fields.forEach(function(field){
                field.readOnly = true;
                field.value(null);
            });
            
            wdgTabbed.activate();
            templateManager.updateTemplate();
        }
    };
    /**
     * Atualiza o valor da quantidade dos elementos pai no grid se serviços 
     * @params{Object} widget - widget que contém a row do elemento inicial da contagem
    **/
    this.atualizarSomatorioQuantidadeServico = function(widget){
        var wdgServico          = templateManager.container.getWidget("wdgServicosInserindo");
        var idQuantidades       = ["QTMENSAL", "QTDIAUTIL", "QTFIMSEMANA", "QTSEGUNDA", "QTTERCA", "QTQUARTA", "QTQUINTA", "QTSEXTA", "QTSABADO", "QTDOMINGO"];
        var arvoreServInvertida = [];
        var cdServico           = [];

        this.getArvoreServico(wdgServico, widget.currentRow, [], true).forEach(function(servico){
            cdServico.push(servico.IDARVORE);
        });
        wdgServico.dataSource.data.forEach(function(servico) {
            if(cdServico.includes(servico.IDARVORE)) {
                servico.__isSelected = true;
                wdgServico.dataSource.addCheckedRows(servico);
                    
                if(servico._childCount > 0) {
                    idQuantidades.forEach(function(idQuantidade) {
                        servico[idQuantidade] = 0;
                    });
                    
                    arvoreServInvertida = this.getArvoreInvertidaServico(wdgServico, servico, [], true);
                    arvoreServInvertida.forEach(function(servFilho) {
                        if(servFilho._childCount == 0) {
                            idQuantidades.forEach(function(idQuantidade) {
                                servico[idQuantidade] += servFilho[idQuantidade];
                            });
                        }
                    });
                }
            }
        }.bind(this));
    };
    /**
     * Salva a widget de proposta
     * @params{Object} widget - widget proposta form-view
    **/
    this.salvarWidgetProposta = function(widget){
        var deferred = ZHPromise.defer();
        this.criarParametrosNewCode(widget);
        General.callRestEngine("product#/fct_orcamentoaliment_widget/save", "DataSet", widget.dataSource.data).then(function(response){
            deferred.resolve(response);
        }.bind(this))
        .catch(function(error){
            deferred.reject(error);
            Utilities.showErrorMessage(error);
        });
        return deferred.promise;
    };
    /**
     * Clona o dataSource da tela ignorando os campos determinados 
     * @params{Object} widget - widget que será clonada
     * @params{Array} ignoredKeys - array de chaves que devem ser ignoradas 
     * @params{boolean} check - somente dados selecionados ou não
    **/
    this.clonarData = function(widget, ignoredKeys, check){
        var data = [];
        var dataSet = check ? widget.dataSource.getCheckedRows() : widget.dataSource.data;
        dataSet.forEach(function(row, index){
            data[index] = Util.clone(row, ignoredKeys);
        });
        
        return data;
    };
    /**
     * Retorna um arrai de servicos relacionados da proposta
     * @params{Object} wdgItens - widget de resumo do servico
    **/
    this.getArrayPercServicoRelac = function(wdgItens){
        var wdgCusto = templateManager.container.getWidget("wdgCustoProposta");
        var qtMeses  = templateManager.container.getWidget("wdgProposta").currentRow.QTDIASVALIDADE;
        var arrServicoRelac = [];
        var cdservicos      = [];
        wdgCusto.dataSource.data.forEach(function(custo){
            if(custo.CDSERVORCARELAC)
                cdservicos.push(custo.CDSERVORCARELAC);
        }.bind(this));
        
        qtMeses = qtMeses ? qtMeses/30 : 1;
        cdservicos = this.retirarValoresDuplicados(cdservicos);
        cdservicos.forEach(function(serviRelac){
            if(serviRelac){
                var totalPerElemento = 0;
                totalPerElemento += this.getVrPercentualServico(serviRelac)/qtMeses;
                if(totalPerElemento == 0 && wdgCusto.dataSource.data.length > 0)
                    totalPerElemento = 100 / wdgItens.dataSource.data.length;
                arrServicoRelac.push({
                    "CDSERVORCARELAC": serviRelac,
                    "VRPERCSERVICO": totalPerElemento
                });
            }
        }.bind(this));
        
        return arrServicoRelac;
    };
    /**
     * Coloca nos dados do custo as quantidades de cada elemento
    **/
    this.setQuantidadesServicos = function(){
        var wdgServico = templateManager.container.getWidget("wdgServicosInserindo");
        var wdgCusto   = templateManager.container.getWidget("wdgCustoProposta");
        
        wdgCusto.dataSource.data.forEach(function(custo){
            wdgServico.dataSource.getCheckedRows().forEach(function(servico){
                if(custo.NRCOMPSERVICO === servico.NRCOMPSERVICO){
                    custo.QTMENSAL  = servico.QTMENSAL;
                    custo.QTSEGUNDA = servico.QTSEGUNDA;
                    custo.QTTERCA   = servico.QTTERCA;
                    custo.QTQUARTA  = servico.QTQUARTA;
                    custo.QTQUINTA  = servico.QTQUINTA;
                    custo.QTSEXTA   = servico.QTSEXTA;
                    custo.QTSABADO  = servico.QTSABADO;
                    custo.QTDOMINGO = servico.QTDOMINGO;
                }
            });
        });
    };
    /**
     * Salva todos as widgets ligadas a proposta inserida
     * @params{Object} widget - widget custo final
    **/
    this.salvarProposta = function(widget){
        var wdgProposta    = templateManager.container.getWidget("wdgProposta");
        var wdgCusto       = templateManager.container.getWidget("wdgCustoProposta");
        var wdgMObra       = templateManager.container.getWidget("wdgMaoObraGridInserindo");
        var wdgDespesa     = templateManager.container.getWidget("wdgDespesas");
        var wdgInvest      = templateManager.container.getWidget("wdgInvestimentosInserindo");
        var wdgTaxas       = templateManager.container.getWidget("wdgTaxasInserindo");
        var wdgRServico    = templateManager.container.getWidget("wdgResumoServico");
        var wdgGridServ    = templateManager.container.getWidget("wdgServicosInserindo");
        var wdgRFinaceiro  = templateManager.container.getWidget("wdgResumoGrid");
        var wdgImposto     = templateManager.container.getWidget("wdgImpostoProposta");

        var rowProp = Util.clone(wdgProposta.currentRow, []);
        var nrtipoorcamento = wdgProposta.currentRow.NRTIPOORCAMENTO;
        wdgProposta.currentRow.VRORCAMENTO          = wdgRFinaceiro.dataSource.data[0].VRFINANCEIRO;
        wdgProposta.currentRow.VRPERCMARKUPAPLICADO = wdgRFinaceiro.dataSource.data[8].VRPERCFINANCEIRO;
        this.salvarWidgetProposta(wdgProposta).then(function(response){
            var nrseqorcamento = response.dataset.columnsToReturn[0].NRSEQORCAMENTO; 
            var ignoredKeys = ["changed","changed_VRUNITITEMORCA","__index","__isSelected","__leftActions","__notSelectable","__rightActions","__proto__"];
            var params = {
                "NRTIPOORCAMENTO": nrtipoorcamento,
                "NRSEQORCAMENTO" : nrseqorcamento,
                "SERVICOS"       : wdgGridServ.dataSource.getCheckedRows(),
                "CUSTOS"         : wdgCusto.dataSource.data,
                "MOBRAS"         : this.clonarData(wdgMObra, ignoredKeys, false),
                "INVESTIMENTOS"  : wdgInvest.dataSource.getCheckedRows(),
                "DESPESAS"       : wdgDespesa.dataSource.getCheckedRows(),
                "TAXAS"          : wdgTaxas.dataSource.getCheckedRows(),
                "ITENS"          : this.clonarData(wdgRServico, ignoredKeys, false),
                "IMPOSTO"        : this.clonarData(wdgImposto, ignoredKeys, true),
                "SERVICORELAC"   : this.getArrayPercServicoRelac(wdgRServico)
            };
            General.callRestEngine("product#/fct_orcamentoaliment_widget/inserirTudoProposta", "Row", params).then(function(response) {
                wdgProposta.view();
                wdgProposta.template = "widget/form-view.html";
                this.alterarAcessibilidadeAbasProposta(true);
                this.alterarVisibilidadesWidgets();
                ScreenService.showMessage("Proposta salva com sucesso!");
                codeRelacFormacao = null;
            }.bind(this))
            .catch(function(error){
                wdgProposta.currentRow.NRSEQORCAMENTO = nrseqorcamento;
                wdgProposta.removeRow().then(function(){
                    wdgProposta.currentRow = rowProp;
                    Utilities.showErrorMessage(error);
                });
            }.bind(this))
            .finally(function() {
                this.voltarParaPropostaGrid();
            }.bind(this));
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Muda o field de valor informado e a action do grid 
     * para a edição dos valores
    
    this.editarPrecoResumoServico = function(widget, args, element){
        var wdgCustoFinal  = templateManager.container.getWidget("wdgTabbedPrincipalResumoProp");
        widget.getField("VRUNITITEMORCA").inlineEditable = "number";
        widget.getAction("editar_vrservico").icon = "checked no-border";
        widget.getAction("editar_vrservico").events[0].code = "Sm00001000_LancamentoPropostaAlimenticia_Controller.salvarNewPrecoInformado(args.owner.widget);";
        widget.redraw();
        wdgCustoFinal.activate().then(function() {
            setTimeout(function(){
                if(widget.getAction("save") && widget.getAction("cancel")){
                    widget.getAction("save").isVisible   = false;
                    widget.getAction("cancel").isVisible = false;
                }
            }, 200);
        });
    }*/
    /**
     * Guarda a alterção realizada no valor do serviço completo 
     * durante o resumo final da proposta
    **/
    this.salvarNewPrecoInformado = function(widget){
        var wdgRServico   = templateManager.container.getWidget("wdgResumoServico");
        var wdgCustoFinal = templateManager.container.getWidget("wdgTabbedPrincipalResumoProp");
        var vrInformado   = widget.currentRow.VRUNITITEMORCA_GRID;
        
        if(widget.currentRow.VRPRECOIDEAL.toFixed(2) == vrInformado)
            vrInformado = widget.currentRow.VRPRECOIDEAL;
        
        widget.currentRow.VRUNITITEMORCA = vrInformado;
        this.calcularValoresServico_inline(wdgRServico);
        templateManager.updateTemplate();
        wdgCustoFinal.activate();
    };
    /**
     * Valida em qual momento a widget de resultado está sendo alterada
    **/
    this.validarCalculoAlteracaoServico = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        if(wdgProposta.currentRow.NRSEQORCAMENTO){
            this.calcularNewVrOrcamento(widget, wdgProposta);
        } else {
            this.salvarNewPrecoInformado(widget);
        }
    };
    /**
     * Efetua os calculos da widget de resumo do servico 
     * quando o preço informado é alterado 
    **/
    this.calcularValoresServico_inline = function(widget){
        var wdgFinanceiro = templateManager.container.getWidget("wdgResumoGrid");
        var wdgImpostos   = templateManager.container.getWidget("wdgImpostoProposta");
        var wdgTaxas      = templateManager.container.getWidget("wdgTaxasInserindo");
        var totalServico  = (parseFloat(widget.currentRow.VRTOTIDEALSERV) * parseFloat(widget.currentRow.VRUNITITEMORCA))/ parseFloat(widget.currentRow.VRPRECOIDEAL);
        var vrOrcaBruto   = 0;
    
        var totalSemMarkup = this.calcularValorPropSemTaxa();
        totalSemMarkup *= (1 + (this.getSomatorioDataSource(wdgTaxas, "VRPERCTAXA", true)/100));

        widget.currentRow.VRTOTALITEMORCA = totalServico;
        widget.dataSource.data.forEach(function(servico){
            vrOrcaBruto += parseFloat(servico.VRTOTALITEMORCA);
        });
        
        widget.dataSource.data.forEach(function(servico){
            servico.VRPERCFINANCEIRO = (servico.VRTOTALITEMORCA * 100/vrOrcaBruto);
        });
        
        var vrImposto  = vrOrcaBruto * (this.getSomatorioDataSource(wdgImpostos, "VRPRECIMPOSTO", true)/100);
        var vrNewTotal = vrOrcaBruto - vrImposto;
        wdgFinanceiro.dataSource.data[0].VRFINANCEIRO = vrOrcaBruto;
        wdgFinanceiro.dataSource.data[1].VRFINANCEIRO = vrImposto;
        wdgFinanceiro.dataSource.data[2].VRFINANCEIRO = vrNewTotal;
        wdgFinanceiro.dataSource.data[8].VRFINANCEIRO = totalSemMarkup * ((vrOrcaBruto/totalSemMarkup) - 1);
        wdgFinanceiro.dataSource.data[8].VRPERCFINANCEIRO = ((vrOrcaBruto/totalSemMarkup) - 1) * 100;
        widget.redraw();
        wdgFinanceiro.redraw();
    };
    /**
     * Avança para a widget de Imposto
     * @params{Object} widget - widget servico proposta inserindo 
    **/
    this.avancarImposto = function(widget){
        Sm00001000_LancamentoProposta_Service.validarObrigatoriedadeItensComposicao(widget.dataSource.data).then(function(mensagemErro) {
            if (mensagemErro && mensagemErro !== "Operação bloqueada.<br/><br/>Nenhum elemento foi selecionado.") {
                ScreenService.showMessage(mensagemErro);
            } else {
                var dataServico = templateManager.container.getWidget("wdgServicosInserindo").dataSource.data;
                widget.enabled  = false;
                Sm00001000_PropostaAlimenticia_Repository.saveDataLocalStorageEngine("wdgServicosInserindo", dataServico);
                var wdgImposto = templateManager.container.getWidget("wdgImpostoProposta");
                wdgImposto.enabled   = true;
                wdgImposto.isVisible = true;
                wdgImposto.mode = "edit";
                wdgImposto.activate().then(function() {
                    this.setFiltroImpostos(wdgImposto);
                }.bind(this));
            }
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
     /**
     * Aplica um filtro na widget de imposto durante a inserção da proposta
     * @params{Object} widget - widget imposto inserindo
    **/
    this.setFiltroImpostos = function(widget){
        var wdgServico  = templateManager.container.getWidget("wdgServicosInserindo");
        var composicoes = [];
        wdgServico.dataSource.getCheckedRows().forEach(function(serv) {
            composicoes.push(serv.NRCOMPSERVICO);
        });
        composicoes = this.retirarValoresDuplicados(composicoes);
        widget.dataSourceFilter[0].value = composicoes;
        widget.reload().then(function(){
            widget.getAction("save").isVisible   = false;
            widget.getAction("cancel").isVisible = false;
            widget.dataSource.selectAll();
        });
    };
    /**
     * Volta para a widget de imposto durante a inserção da proposta
     * @params{Object} widget - widget custo inserindo
    **/
    this.voltarImpostosProposta = function(widget){
        widget.enabled = false;
        var wdgImposto = templateManager.container.getWidget("wdgImpostoProposta");
        wdgImposto.enabled = true;
        wdgImposto.mode    = "edit";
        wdgImposto.activate();
    };
    /**
     * Abre um popup para a seleção de um novo imposto
     * @params{Object} widget - widget impostos inserindo
    **/
    this.adicionarImpostos = function(widget){
        var wdgPopup = templateManager.container.getWidget("wdgPopupImpostos");
        var cdimpostos = [];
        widget.dataSource.data.filter(function(imposto){
            if(imposto.CDIMPOSTO)
                cdimpostos.push(imposto.CDIMPOSTO);
        });
        
        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function(){
            wdgPopup.edit();
            wdgPopup.newRow();
            wdgPopup.activate().then(function(){
                wdgPopup.getField("CDIMPOSTO").dataSourceFilter[0].value = cdimpostos.length > 0 ? cdimpostos : [-1];
                wdgPopup.getField("CDIMPOSTO").reload();
            });
            
            if(widget.name === "wdgItensImpostoProposta"){
                wdgPopup.getAction("save").events[0].code   = "Sm00001000_LancamentoPropostaAlimenticia_Controller.salvarNewImpostoItemOrca(args.owner.widget);";
                wdgPopup.getAction("cancel").events[0].code = "Sm00001000_LancamentoPropostaAlimenticia_Controller.cancelarNewImpostoItemOrca();";
            } else { 
                wdgPopup.getAction("save").events[0].code   = "Sm00001000_LancamentoPropostaAlimenticia_Controller.salvarNovoImposto(args.owner.widget);";
                wdgPopup.getAction("cancel").events[0].code = "Sm00001000_LancamentoPropostaAlimenticia_Controller.cancelarInsercaoImposto(args.owner.widget);";
            }
            
        }.bind(this));
    };
    /**
     * Cancela a inserção de um novo imposto durante a inserção da proposta
     * @params{Object} widget - widget popup de inserção do imposto
    **/
    this.cancelarInsercaoImposto = function(widget){
        ScreenService.closePopup();
        var wdgImposto = templateManager.container.getWidget("wdgImpostoProposta");
        wdgImposto.enabled = true;
        wdgImposto.mode    = "edit";
        wdgImposto.activate().then(function(){
            setTimeout(function() {
                wdgImposto.getAction("save").isVisible   = false;
                wdgImposto.getAction("cancel").isVisible = false;
            }, 300);
        });
    };
    /**
     * Adiciona ao dataSource da tela de impostos um novo imposto
     * durante a inserção da proposta
     * @param{Object} widget - widget popup impostos
    **/
    this.salvarNovoImposto = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgImpostoPro = templateManager.container.getWidget("wdgImpostoProposta");
            widget.currentRow.CDIMPOSTO.forEach(function(imposto){
                var nmimposto;
                widget.getField("CDIMPOSTO").dataSource.data.forEach(function(valueImposto){
                    if(valueImposto.CDIMPOSTO === imposto)
                        nmimposto = valueImposto.NMIMPOSTO;
                });
                var row = {
                    "NMIMPOSTO": nmimposto,
                    "CDIMPOSTO": imposto,
                    "VRPRECIMPOSTO": widget.currentRow.VRPRECIMPOSTO ? widget.currentRow.VRPRECIMPOSTO : "0",
                    "NRCOMPSERVICO": "",
                    "__is_new": false,
                    "__isSelected": true
                };
                wdgImpostoPro.dataSource.data.push(row);
                wdgImpostoPro.dataSource.addCheckedRows(row);
            });
            
            ScreenService.closePopup();
            wdgImpostoPro.mode = "edit";
            wdgImpostoPro.activate().then(function(){
                setTimeout(function() {
                    wdgImpostoPro.getAction("save").isVisible   = false;
                    wdgImpostoPro.getAction("cancel").isVisible = false;
                }, 300);
            });
        } 
    };
    /**
     * Abre um popup para adicionar um novo investimento
     * @params{Object} widget - widget investimento inserindo
    **/
    this.adicionarNovoInvestimento = function(widget){
        var wdgPopup = templateManager.container.getWidget("wdgPopupInvestimento");
        var investimentos = [];
        widget.dataSource.data.filter(function(invest){
            if(invest.NRINVESTIMENTO)
                investimentos.push(invest.NRINVESTIMENTO);
        });
        
        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function(){
            wdgPopup.edit();
            wdgPopup.newRow();
            wdgPopup.activate().then(function(){
                wdgPopup.getField("NRINVESTIMENTO").dataSourceFilter[0].value = investimentos.length > 0 ? investimentos : ["-1"];
                wdgPopup.getField("NRINVESTIMENTO").reload();
            });
        }.bind(this));
    };
    /**
     * Cancela a inserção de um novo investimento durante a inserção da proposta
     * @params{Object} widget - widget popup de inserção do investimento
    **/
    this.cancelarInsercaoInvestimento = function(widget){
        ScreenService.closePopup();
        var wdgIvest = templateManager.container.getWidget("wdgInvestimentosInserindo");
        wdgIvest.enabled = true;
        wdgIvest.mode    = "edit";
        wdgIvest.activate().then(function(){
            setTimeout(function() {
                wdgIvest.getAction("save").isVisible   = false;
                wdgIvest.getAction("cancel").isVisible = false;
            }, 300);
        });
    };
    /**
     * Adiciona ao dataSource da tela de impostos um novo imposto
     * durante a inserção da proposta
     * @param{Object} widget - widget popup impostos
    **/
    this.salvarNovoInvestimento = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgInvest = templateManager.container.getWidget("wdgInvestimentosInserindo");
            widget.currentRow.NRINVESTIMENTO.forEach(function(investimento){
                widget.getField("NRINVESTIMENTO").dataSource.data.forEach(function(valueIvest){
                    if(valueIvest.NRINVESTIMENTO === investimento){
                        var row = {
                            "DSINVESTIMENTO"    : valueIvest.DSINVESTIMENTO,
                            "NRINVESTIMENTO"    : investimento,
                            "VRINVESTIMENTO"    : valueIvest.VRINVESTIMENTO   ? valueIvest.VRINVESTIMENTO : "0",
                            "QTMESARMOTIZACAO"  : valueIvest.QTMESARMOTIZACAO ? valueIvest.QTMESARMOTIZACAO : "0",
                            "TPAMORTIZACAO"     : valueIvest.TPAMORTIZACAO,
                            "NMCATINVEST"       : valueIvest.NMCATINVEST,
                            "VRTOTINVESTIMENTO" : valueIvest.VRINVESTIMENTO ? valueIvest.VRINVESTIMENTO : "0",
                            "VRPARCELA"         : valueIvest.VRINVESTIMENTO/(valueIvest.QTMESARMOTIZACAO ? valueIvest.QTMESARMOTIZACAO : 1),
                            "QTINVESTIMENTO"    : "1",
                            "NRCOMPSERVICO"     : "",
                            "NRFORMPRECOSERVICO": "",
                            "__is_new"          : false,
                            "__isSelected"      : true
                        };
                        wdgInvest.dataSource.data.push(row);
                        wdgInvest.dataSource.addCheckedRows(row);
                    }
                });
            });
            this.cancelarInsercaoInvestimento(widget);
        } 
    };
    /**
     * Efetua o cálculo do valor total da formação de preço 
     * no grid de mao de obra
     * @params{Object} widget - widget mão de obra inserindo
    **/ 
    this.calcularValorTotalMObra = function(widget){
        var qtMObra = (widget.currentRow.QTFORMACAO_inline) ? widget.currentRow.QTFORMACAO_inline : widget.currentRow.QTFORMACAO;
        widget.currentRow.VRTOTAL = parseFloat(qtMObra) * parseFloat(widget.currentRow.VRUNITARIO);
    };
    /**
     * Abre o swipe de visualização das informções dos itens da proposta
    **/
    this.abrirSwipeItensProposta = function(widget){
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipalItens");
        var wdgItens           = templateManager.container.getWidget("wdgItensProposta");
        ScreenService.openSwipe(wdgTabbedPrincipal).then(function() {
            setLocalVar("PROPOSTA_NRITEMORCAMENTO", widget.currentRow.NRITEMORCAMENTO);
            wdgItens.list();
            wdgItens.reload().then(function() {
                wdgItens.view();
                wdgItens.activate();
            });
        });
    };
    /**
     * Cancela a edição da quantidade dos itens no campo de inline edit
     * @params{Object} widget - widget itens grid
    **/
    this.cancelarItens = function(widget) {
        widget.list();
        widget.reload();
    };
    /**
     * Prepara a widget para a inserção de um novo item (serviço)
     * @params{Object} widget - widget de serviço
    **/
    this.itensNovoRegistro = function(widget){
        General.widgetNewRow(widget);
        this.controlarCampoValorDoAjuste(widget);
    };
    /**
     * Sintetiza as funções em uma mesma função
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.funcoesDoCampoAjuste = function(widget) {
        this.controlarCampoValorDoAjuste(widget);
        this.calcularValorTotalItem(widget);
        if (widget.currentRow.VRPERCDESCORCA) {
            this.adequaAjusteAoCampo(widget);
        }
    };
    /**
     * Previni possiveis erros com a troca sequida no select native do campo Ajuste
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.adequaAjusteAoCampo = function(widget) {
        var quantidade        = widget.currentRow.QTITEMORCAMENTO;
        var valorUnitario     = widget.currentRow.VRUNITITEMORCA;
        var tipoAjuste        = widget.currentRow.IDTPDESCORCA;
        var ajuste            = widget.currentRow.VRPERCDESCORCA;

        var valorSemajuste = quantidade * valorUnitario;
        var valorTotal = valorSemajuste;
        if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));
        if (tipoAjuste == "P" && ajuste > 100) {
            widget.currentRow.VRPERCDESCORCA = null;
        }
        if (tipoAjuste == "V" && ajuste > valorTotal) {
            widget.currentRow.VRPERCDESCORCA = null;
        }
    };
    /**
     * Controla o campo de desconto de acordo com o tipo.
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.controlarCampoValorDoAjuste = function(widget) {
        var tipoAjuste = widget.currentRow.IDTPDESCORCA;
        var fieldAjuste = widget.getField("VRPERCDESCORCA");
        widget.getField('IDTPDESCORCA').readOnly = false;
        if (tipoAjuste == "N") {
            fieldAjuste.readOnly = true;
            fieldAjuste.value("");
        } else {
            fieldAjuste.readOnly = false;
        }

        fieldAjuste.readOnly = (tipoAjuste == "N") ? true : false;
    };
        /**
     * Calcula o valor total do item da proposta.
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.calcularValorTotalItem = function(widget) {
        if (widget.mode != "list" && widget.mode != "inline_edit") {
            var quantidade    = widget.currentRow.QTITEMORCAMENTO;
            var valorUnitario = widget.currentRow.VRUNITITEMORCA;
            /*var tipoAjuste = widget.currentRow.IDTPDESCORCA;
            var ajuste = widget.currentRow.VRPERCDESCORCA;*/
            var qtMedVariavel = widget.currentRow.QTMEDIDAVARIAVEL ? parseFloat(widget.currentRow.QTMEDIDAVARIAVEL.toString().replace(",", ".")) : 1;
    
            if (quantidade) quantidade = parseFloat(quantidade.toString().replace(",", "."));
            if (valorUnitario) valorUnitario = parseFloat(valorUnitario.toString().replace(",", "."));
            //if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));
            
            var validacao = true;
            if (validacao) {
                var valorSemAjuste = quantidade * valorUnitario ;

                if (widget.currentRow.IDMULTIPVARIACAO == "S") 
                    valorSemAjuste *= qtMedVariavel ;
        
                var valorTotal = valorSemAjuste;
                /*if(!widget.currentRow.IDTIPO || widget.currentRow.IDTIPO !== "I"){
                    if (tipoAjuste == "V" && ajuste <= valorTotal && ajuste >= 0) {
                        valorTotal -= ajuste;
                    }
                    else if (tipoAjuste == "P" && ajuste <= 100 && ajuste >= 0) {
                        valorTotal -= valorTotal * ajuste / 100;
                    }
                    else if (tipoAjuste == "X" && ajuste >= 0) {
                        valorTotal += ajuste;
                    }
                    else if (tipoAjuste == "Y" && ajuste >= 0) {
                        valorTotal += valorTotal * ajuste / 100;
                    }
                }*/
        
                //widget.currentRow.VRITEMSEMDESC = valorSemAjuste;
                widget.currentRow.VRTOTALITEMORCA = valorTotal;
                if(widget.currentRow.TIPO === "M")
                    widget.currentRow.VRCUSTOALIMENTAR = quantidade * valorUnitario;
            }
        }
    };
    /**
     * Abre a widget de itens de formação de preço da proposta 
     * @params{widget} widget - widget mão de obra (tela itens)
    **/
    this.exibirItensFormPrecProposta = function(widget){
        if(!onClickMobraItens || onClickMobraItens !== (widget.currentRow.NRFORMPRECOSERVICO + "_" + widget.currentRow.NRRELACFORMACAOPROP)){
            onClickMobraItens = widget.currentRow.NRFORMPRECOSERVICO + "_" + widget.currentRow.NRRELACFORMACAOPROP;
        } else {
            var wdgTabbed     = templateManager.container.getWidget("wdgTabbedItensMObraForm");
            var wdgItensForm  = templateManager.container.getWidget("wdgItensFormacaoPrecoProposta");
            wdgItensForm.isVisible = true;
            wdgItensForm.enabled   = true;
            ScreenService.openSwipe(wdgTabbed).then(function() {
                wdgItensForm.list();
                wdgItensForm.activate().then(function(){
                    wdgItensForm.dataSourceFilter[2].value = widget.currentRow.NRFORMPRECOSERVICO;
                    wdgItensForm.dataSourceFilter[4].value = widget.currentRow.NRRELACFORMACAOPROP;
                    Utilities.abreArvoreElementos(wdgItensForm);
                });
                onClickMobraItens = null;
            });
        }
    };
    /**
     * Verifica se a widget de resultado está sendo acessada para
     * visualização de dados já inseridos e manda buscar as informações
     * @params{Object} widget - widget Resultado dashboard
    **/
    this.buscarInformacoesResultadoProp = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        if(widget.mode === "list" && !wdgProposta.currentRow.__is_new){
            var wdgServico    = templateManager.container.getWidget("wdgResumoServico");
            var wdgFinanceiro = templateManager.container.getWidget("wdgResumoGrid");
            var params        = {"NRSEQORCAMENTO": getLocalVar("PROPOSTA_NRSEQORCAMENTO")};
            General.callRestEngine("product#/fct_orcamentoaliment_widget/getInfoResultado", "Row", params).then(function(response){
                if(response.dataset && response.dataset.getInfoResultado){
                    wdgServico.dataSource.data = response.dataset.getInfoResultado.SERVICOS;
                    var vrFin = response.dataset.getInfoResultado.FINANCEIRO;
                    this.setValoresWdgRFinanceiro(wdgFinanceiro, vrFin["TOTALBRUTO"], vrFin["IMPOSTO"], vrFin["TOTALLIQ"], vrFin["CALIMENTAR"], vrFin["TOTALCTAXA"], vrFin["MOBRA"], vrFin["DESPESAS"], vrFin["INVESTIMENTOS"], vrFin["TAXAS"], vrFin["MARKUP"], vrFin["TOTALSTAXA"], vrFin["TOTALIDEAL"], vrFin["IMPOSTOIDEAL"], vrFin["TOTALLIQIDEAL"], vrFin["MARKUPIDEAL"]);
                }
                wdgServico.redraw();
                wdgFinanceiro.redraw();
            }.bind(this));
        }
    };
    /**
     * Adiciona na lista fixa da widget de resultados os valores 
     * de cada informação da proposta 
     * @params{Object} widget - widget resumo Financeiro
     * @params{float} (demais parametrôs) - totais de cada elemento
    **/
    this.setValoresWdgRFinanceiro = function(widget, totalOrcaBruto, totalImposto, totalOrcamento, totalServico, totalCmTaxa, totalMObra, totalDespesa, totalInvest, totalTaxa, totalMarkup, totalSTaxa, totalOrcaIdeal, totalImpostoIdeal, totalOrcamentoIdeal, totalMarkupIdeal){
        widget.dataSource.data[0].VRFINIDEAL       = totalOrcaIdeal;
        widget.dataSource.data[0].VRFINANCEIRO     = totalOrcaBruto;
        widget.dataSource.data[0].VRPERCFINIDEAL   = (totalOrcaIdeal * 100/totalOrcaIdeal);
        widget.dataSource.data[0].VRPERCFINANCEIRO = (totalOrcaBruto * 100/totalOrcaBruto);
        
        widget.dataSource.data[1].VRFINANCEIRO     = totalImposto;
        widget.dataSource.data[1].VRFINIDEAL       = totalImpostoIdeal;
        widget.dataSource.data[1].VRPERCFINANCEIRO = (totalImposto * 100/totalOrcaBruto);
        widget.dataSource.data[1].VRPERCFINIDEAL   = (totalImpostoIdeal * 100/totalOrcaIdeal);
        
        widget.dataSource.data[2].VRFINANCEIRO     = totalOrcamento;
        widget.dataSource.data[2].VRFINIDEAL       = totalOrcamentoIdeal;
        widget.dataSource.data[2].VRPERCFINANCEIRO = (totalOrcamento * 100/totalOrcaBruto);
        widget.dataSource.data[2].VRPERCFINIDEAL   = (totalOrcamentoIdeal * 100/totalOrcaIdeal);
        
        widget.dataSource.data[3].VRFINANCEIRO     = totalServico;
        widget.dataSource.data[3].VRFINIDEAL       = totalServico;
        widget.dataSource.data[3].VRPERCFINANCEIRO = (totalServico * 100/totalCmTaxa);
        widget.dataSource.data[3].VRPERCFINIDEAL   = (totalServico * 100/totalCmTaxa);
        
        widget.dataSource.data[4].VRFINANCEIRO     = totalMObra;
        widget.dataSource.data[4].VRFINIDEAL       = totalMObra;
        widget.dataSource.data[4].VRPERCFINANCEIRO = (totalMObra * 100/totalCmTaxa);
        widget.dataSource.data[4].VRPERCFINIDEAL   = (totalMObra * 100/totalCmTaxa);
        
        widget.dataSource.data[5].VRFINANCEIRO     = totalDespesa;
        widget.dataSource.data[5].VRFINIDEAL       = totalDespesa;
        widget.dataSource.data[5].VRPERCFINANCEIRO = (totalDespesa * 100/totalCmTaxa);
        widget.dataSource.data[5].VRPERCFINIDEAL   = (totalDespesa * 100/totalCmTaxa);
        
        widget.dataSource.data[6].VRFINANCEIRO     = totalInvest;
        widget.dataSource.data[6].VRFINIDEAL       = totalInvest;
        widget.dataSource.data[6].VRPERCFINANCEIRO = (totalInvest * 100/totalCmTaxa);
        widget.dataSource.data[6].VRPERCFINIDEAL   = (totalInvest * 100/totalCmTaxa);
        
        widget.dataSource.data[7].VRFINANCEIRO     = totalTaxa;
        widget.dataSource.data[7].VRFINIDEAL       = totalTaxa;
        widget.dataSource.data[7].VRPERCFINANCEIRO = (totalTaxa * 100/totalSTaxa);
        widget.dataSource.data[7].VRPERCFINIDEAL   = (totalTaxa * 100/totalSTaxa);
        
        widget.dataSource.data[8].VRFINANCEIRO     = totalMarkup;
        widget.dataSource.data[8].VRFINIDEAL       = totalMarkupIdeal;
        widget.dataSource.data[8].VRPERCFINANCEIRO = (totalMarkup * 100/ totalCmTaxa);
        widget.dataSource.data[8].VRPERCFINIDEAL   = (totalMarkupIdeal * 100/ totalCmTaxa);
    };
    /**
     * Volta a widget de proposta para o modo de visualização
     * @params{Object} widget - widget de proposta
    **/
    this.voltarParaPropostaView = function(widget){
        var wdgPropostaTabbed     = templateManager.container.getWidget("wdgPropostaTabbed");
        var wdgTabbedPrincipal    = templateManager.container.getWidget("wdgTabbedPrincipal");
        wdgPropostaTabbed.enabled = true;
        this.alterarAcessibilidadeAbasProposta(true);
        
        widget.template = "widget/form-view.html";
        widget.currentRow.changed = false;
        widget.getAction("save").isVisible       = false;
        widget.getAction("avancar").isVisible    = true;
        widget.getField("VRPERCMARKUP").readOnly = false;
        
        wdgPropostaTabbed.mode  = "view";
        wdgTabbedPrincipal.mode = "view";
        widget.view();
    };
    /**
     * Valida a edição da proposta
     * @params{Object} widget - widget proposta view
    **/
    this.validarEdicaoProposta = function(widget){
        this.salvarWidgetProposta(widget).then(function(){
            this.voltarParaPropostaView(widget);
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /* Aplica o filtro de situação da proposta na tela de escopo 
    * para aparecer apenas o documento da situação atual
    * @params{Object} widget escopo proposta
    */
    this.setFilterSituacaoEscopo = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var edititem    = templateManager.container.getWidget("wdgEscoposProposta").getAction('edit');
        widget.dataSourceFilter[2].value = wdgProposta.currentRow.IDSITUACAOORCAMENTO;
        widget.reload();

        if(wdgProposta.currentRow.IDSITUACAOORCAMENTO == "CONCLUIDO")
            edititem.isVisible = true;
    };
    /**
     * Salva os escopos da proposta.
     * 
     * @param {widget} widget - Widget de Escopos da Proposta
     */
    this.salvarEscopoProposta = function(widget) {
        widget.currentRow.__columns_to_return = ["NRRELACESCOPOPROPOSTA", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        General.widgetSave(widget).then(function(success) {
            if (success.dataset && success.dataset.columnsToReturn[0]) {
                var columnsToReturn = success.dataset.columnsToReturn[0];
                widget.currentRow.NRRELACESCOPOPROPOSTA = columnsToReturn.NRRELACESCOPOPROPOSTA;
                widget.currentRow.NRORG                 = columnsToReturn.NRORG;
                widget.currentRow.DTINCLUSAO            = columnsToReturn.DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO        = columnsToReturn.CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO         = columnsToReturn.NRORGINCLUSAO;
            }
        }).catch(function(error) {
            ScreenService.showMessage(error);
        });
    };
    /**
     * Conclui a Proposta.
     * 
     * @param {object} wdgProposta - Widget de Proposta
     */
    this.concluirProposta = function(wdgProposta) {
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente concluir a proposta?"), 'question', function(){
            var nrseqorcamento = wdgProposta.currentRow.NRSEQORCAMENTO;
            var nrorg          = wdgProposta.currentRow.NRORG;
            
            wdgProposta.currentRow.changed = true;
            wdgProposta.currentRow.IDSITUACAOORCAMENTO = "CONCLUIDO";
            Sm00001000_LancamentoProposta_Service.validarItensPropostaParaConclusao(nrseqorcamento, nrorg).then(function() {
                General.callRestEngine("product#/fct_orcamentoaliment_widget/save", "DataSet", wdgProposta.dataSource.data).then(function() {
                    ScreenService.showMessage("Processo realizado com sucesso!");
                    wdgProposta.currentRow.changed = false;
                    ScreenService.closePopup();
                    var  params = {
                        "NRSEQORCAMENTO"     : wdgProposta.currentRow.NRSEQORCAMENTO,
                        "IDSITUACAOORCAMENTO": "CONCLUIDO"
                    };
                    
                    var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                    wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO = "CONCLUIDO";
                    this.validacoesGeraisProposta(wdgPropostaGrid);
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }.bind(this),
            function(error) {
                Utilities.showErrorMessage(error);
            });
        }.bind(this));
    };
    /**
     * Aceita a Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.aceitarProposta = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente aceitar a proposta?"), 'question', function(){
            var params = {
                "NRSEQORCAMENTO": widget.currentRow.NRSEQORCAMENTO,
                "NRORG"         : widget.currentRow.NRORG,
                "IDUTILIZACOMP" : widget.currentRow.IDUTILIZACOMP,
                "NRPARCNEGOCIO" : widget.currentRow.NRPARCNEGOCIO
            };
    
            General.callRestEngine("product#/fct_orcamentoaliment_widget/aceitarProposta", "Row", params).then(function() {
                    ScreenService.showMessage("Processo realizado com sucesso!");
                    ScreenService.closePopup();
                    widget.reload().then(function() {
                        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                        wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO = "ACEITO";
                        wdgPropostaGrid.currentRow.DTACEITEORCA = widget.currentRow.DTACEITEORCA;
                        this.validacoesGeraisProposta(wdgPropostaGrid);
                    }.bind(this));
                }.bind(this),
                function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
        }.bind(this));
    };
    /**
     * Copia ou versiona a Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     * @param {string} tipo - 'C' para Copiar, 'V' para Versionar
     */
    this.copiarVersionarProposta = function(widget, tipo) {
        var row = widget.currentRow;
        var params = {
            "IDSITUACAOORCAMENTO": widget.currentRow.IDSITUACAOORCAMENTO,
            "NRSEQORCAMENTO"     : widget.currentRow.NRSEQORCAMENTO,
            "NRORG"              : widget.currentRow.NRORG,
            "TIPO"               : tipo
        };

        General.callRestEngine("product#/fct_orcamentoaliment_widget/copiarVersionarProposta", "Row", params).then(function(response) {
            if(!widget.currentRow.NRCONTATO){
                row.NRSEQORCAMENTO = response.dataset.NRSEORCGERADO[0];
                //this.geraContatoProposta(row);
            }
            
            ScreenService.showMessage("Processo realizado com sucesso!");
            ScreenService.closePopup();
            this.voltarParaPropostaGrid();
        }.bind(this),
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    /**
     * Ação que cancela a Proposta.
     * 
     * @param {object} wdgProposta - Widget de Prposta
     */
    this.cancelarProposta = function(wdgProposta) {
        var params = {
            "NRSEQORCAMENTO": wdgProposta.currentRow.NRSEQORCAMENTO,
            "NRORG": wdgProposta.currentRow.NRORG
        };

        General.callRestEngine("product#/fct_orcamentoaliment_widget/cancelarProposta", "Row", params).then(function(success) {
            ScreenService.showMessage("Processo realizado com sucesso!");
            ScreenService.closePopup();
            wdgProposta.currentRow.IDSITUACAOORCAMENTO = "CANCELADO";

            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO = "CANCELADO";
            this.validacoesGeraisProposta(wdgPropostaGrid);
        }.bind(this),
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    /**
     * Verifica o tipo do item para habilitar as ações de edição e exclusão
     * @params{Object} widget - widget de itens 
    **/
    this.validarItens = function(widget){
        var wdProposta = templateManager.container.getWidget("wdgProposta");
        if(["CONCLUIDO", "ELABORACAO"].includes(wdProposta.currentRow.IDSITUACAOORCAMENTO)){
            var tpItens = (widget.currentRow.IDTIPO  && widget.currentRow.IDTIPO !== "I");
            widget.fields.forEach(function(field){
                if(field.fieldGroup === "fdgDadosAdicionais")
                    field.isVisible = !tpItens;
                    
                if(field.fieldGroup === "fdgDadosServico")
                    field.isVisible = tpItens;
            });
            
            widget.getField("VRUNITITEMORCA").isVisible  = tpItens;
            widget.getField("VRUNITITEMORCA").readOnly   = !tpItens;
            widget.getField("QTITEMORCAMENTO").readOnly  = !tpItens;
            widget.getField("NMTIPOORCAMENTO").isVisible = !tpItens;
        }
    };
    /**
     * Coloca a tela de Itens ou de Adicionais em modo de edição.
     * 
     * @param {object} widget - Widget de Itens ou de Adicionais
     */
    this.prepararEdicaoItens = function(widget) {
        General.widgetEdit(widget);
        this.controlarCampoValorDoAjuste(widget);
        this.validarItens(widget);
    };
    /**
     * Apaga um item do orcamento
     * @params{Object} widget - widget itens 
     */
    this.excluirItem = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
            General.callRestEngine("product#/fct_itemorcamentoaliment_widget/deletarItemOrcamento", "Row", widget.currentRow).then(function(){
                ScreenService.closeSwipe().then(function () {
                    widget.activate();
                    widget.reload();
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }.bind(this));
    };
    /**
     * Converte o valor de ajuste para valor percentual
     * 
     * @param {object} widget widget de 
     * Item de Proposta para salvar o novo registro.
     */
    this.converteParaPercentual = function(widget) {
        var percentual        = widget.currentRow.VRPERCDESCORCA;
        var tipoAjuste        = widget.currentRow.IDTPDESCORCA;
        var quantidade        = widget.currentRow.QTITEMORCAMENTO;
        var valorUnitario     = widget.currentRow.VRUNITITEMORCA;
        var valorSemAjuste    = quantidade * valorUnitario;
        var valorTotal        = valorSemAjuste;
        if (tipoAjuste == "V" || tipoAjuste == "X")
            return (percentual / valorTotal) * 100;
        else
            return percentual;
    };
    /**
     * Atualiza o ajuste dos elementos da arvore de composição de serviço
     * @params{Object} widget widget itens
    **/
    this.salvarNovasQuantidadesElemento = function(widget){
        var paramsElemento = {
            "NRSEQORCAMENTO" : widget.currentRow.NRSEQORCAMENTO,
            "CDSERVORCA"     : widget.currentRow.CDSERVORCA,
            "IDELEMENTO"     : widget.currentRow.IDELEMENTO,
            "VRCUSTOSERV"    : this.tirarMascaraValores(widget.currentRow.VRUNITITEMORCA),
            "QTSEGUNDA"      : widget.currentRow.QTSEGUNDA,
            "QTTERCA"        : widget.currentRow.QTTERCA,
            "QTQUARTA"       : widget.currentRow.QTQUARTA,
            "QTQUINTA"       : widget.currentRow.QTQUINTA,
            "QTSEXTA"        : widget.currentRow.QTSEXTA,
            "QTSABADO"       : widget.currentRow.QTSABADO,
            "QTDOMINGO"      : widget.currentRow.QTDOMINGO,
            "QTMENSAL"       : widget.currentRow.QTITEMORCAMENTO,
            "HRENTRADA"      : widget.currentRow.HRENTRADA,
            "HRSAIDA"        : widget.currentRow.HRSAIDA
        };
        General.callRestEngine("product#/fct_itemorcamentoaliment_widget/salvarNovasQuantidadesElemento", "Row", paramsElemento)
        .then(function() {
            this.atualizarValorItem();
        }.bind(this)).catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Salva um item ou um adicional.
     * 
     * @param {object} widget - Widget de Itens ou de Adicionais
     */
    this.salvarItens = function(widget){
        var deferred = ZHPromise.defer();  
        var isNew    = widget.currentRow.__is_new;
        var tipo     = widget.currentRow.IDTIPO;
        
        widget.currentRow.__columns_to_return = ["NRITEMORCAMENTO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];
        if(widget.currentRow.NMDOCITEMORCAMENTO && widget.currentRow.NMDOCITEMORCAMENTO !== widget.currentRow.NMITEMORCAMENTO){
            widget.currentRow.NMITEMORCAMENTO = widget.currentRow.NMDOCITEMORCAMENTO;
        }
        
        if(!widget.currentRow.VRUNITITEMORCA && tipo == 'I')
            widget.currentRow.VRUNITITEMORCA = widget.currentRow.VRUNITITEMORCA;
            
        General.callRestEngine("product#/fct_itemorcamentoaliment_widget/save", "DataSet", [widget.currentRow]).then(function(success) {
            if (success.dataset && success.dataset.columnsToReturn[0]) {
                widget.currentRow.NRITEMORCAMENTO = success.dataset.columnsToReturn[0].NRITEMORCAMENTO;
                widget.currentRow.NRORG           = success.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO      = success.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO  = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
            if(tipo == 'M')
                this.salvarNovasQuantidadesElemento(widget);
            else 
                this.atualizarValorItem();
            if(isNew){
                ScreenService.closeSwipe().then(function(){
                    widget.list();
                    widget.reload();
                });
            } else {
                widget.view();
            }
            
            deferred.resolve();
        }.bind(this)).catch(function(error) {
            Utilities.showErrorMessage(error);
            deferred.reject();
        });
        
        return deferred.promise;
    };
    /**
     * Realiza a s validaçoes antes de salvar um registro
     * 
     * @param {object} widget widget de 
     * Item de Proposta para salvar o novo registro.
     */
    this.validaPercentualSaveItens = function(widget) {
        var tipoAjuste = widget.currentRow.IDTPDESCORCA;
        var percentual = this.converteParaPercentual(widget);
        var contexto   = "";
        var params;

        if (widget.currentRow.TIPOPRODSERV == 'S')
            widget.currentRow.CDSERVORCA = widget.currentRow.CDPRODSERV;
        else 
            widget.currentRow.CDPRODUTO = widget.currentRow.CDPRODSERV;
            
        if (!widget.currentRow.CDSERVORCA || widget.currentRow.IDTIPO !== "M") {
            this.salvarItens(widget, true);
        } else {
            if (tipoAjuste === "N"){
                this.salvarItens(widget);
            } else {
                params = {
                    "CDSERVORCA": widget.currentRow.CDSERVORCA,
                    "IDTIPO"    : (tipoAjuste == "P" || tipoAjuste == "V") ? "D" : "A"
                };
                
                General.callRestEngine("product#/fct_itemorcamento_widget/validaPorcentualDescontoAcrescimoSave", "Row", params).then(function(response) {
                    var resultado = response.dataset.RESULTADO.VRPERCDESC;
                    if (resultado) {
                        if (percentual <= resultado.VRPERCDESC) {
                            this.salvarItens(widget);
                        } else {
                            if(resultado.TIPO == 1)
                               contexto = "Este";
                            else
                                contexto = "O perfil de acesso deste";
                            var tipo = (tipoAjuste == "P" || tipoAjuste == "V") ? "oferecer desconto" : "propor acréscimo";
                            ScreenService.showMessage(contexto+" operador nao têm autorização para "+ tipo +" superior a " + resultado.VRPERCDESC + "% do valor sem ajuste");
                        }
                    } else {
                        this.salvarItens(widget);
                    }
                }.bind(this)).catch(function(error) {
                    console.error(error);
                    if (error.data)
                        ScreenService.showMessage(error.data.error);
                    else
                        ScreenService.showMessage(error);
                });
            }
        }
    };
    /**
     * Atualiza o valor total da proposta de acordo com a soma dos itens.
     * 
     * @param {object} wdgPropostaGrid - Widget de Proposta (grid)
     */
    this.atualizarValorProposta = function(wdgPropostaGrid) {
        var params = {
            "NRSEQORCAMENTO": wdgPropostaGrid.currentRow.NRSEQORCAMENTO,
            "NRORG": wdgPropostaGrid.currentRow.NRORG
        };

        General.callRestEngine("product#/fct_orcamentoaliment_widget/atualizarValorProposta", "Row", params).then(function(success) {
            var valorProposta = success.dataset.FCT_ORCAMENTO.VRORCAMENTO;
            var wdgProposta = templateManager.container.getWidget("wdgProposta");

            wdgPropostaGrid.currentRow.VRORCAMENTO = valorProposta;
            wdgProposta.currentRow.VRORCAMENTO = valorProposta;
        },
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    /**
     * Valida se o valor semanal do item é menor que o valor mensal
     * @params{Object} widget - widget de itens
    **/
    this.validarQuantidadesItens = function(widget, isMensal){
        var totalQuantidade = 0;
        var totalPeriodos   = this.getTotalDiaDaSemanaPeriodo();
        var arrDias         = ["QTSEGUNDA", "QTTERCA", "QTQUARTA", "QTQUINTA", "QTSEXTA", "QTSABADO", "QTDOMINGO"];
        arrDias.forEach(function(dia) {
            totalQuantidade += (parseInt(widget.currentRow[dia]) * totalPeriodos[dia]);
        });
        
        if(!isMensal) {
            widget.currentRow.QTITEMORCAMENTO = totalQuantidade;
            this.calcularValorTotalItem(widget);
        } else {
            if(totalQuantidade > widget.currentRow.QTITEMORCAMENTO){
                ScreenService.showMessage("Operação Bloqueada!<br>A quantidade mensal não pode ser menor que a quantidade semanal");
                widget.currentRow.QTITEMORCAMENTO = 0;
                this.calcularValorTotalItem(widget);
            }
        }
    };
    /**
     * Insere o imposto num orcamento já inserido
     * @params{Object} widget - widget imposto na tela de itens 
    **/
    this.salvarNewImpostoItemOrca = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgImposto  = templateManager.container.getWidget("wdgItensImpostoProposta");
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            if(widget.name !== "wdgItensImpostoProposta"){
                wdgImposto.currentRow.CDIMPOSTO      = widget.currentRow.CDIMPOSTO;
                wdgImposto.currentRow.VRPERCIMPOSTO  = widget.currentRow.VRPRECIMPOSTO;
                wdgImposto.currentRow.NRSEQORCAMENTO = wdgProposta.currentRow.NRSEQORCAMENTO;
                wdgImposto.currentRow.__is_new       = true;
                Utilities.inserirMultiplosComNovoCodigo(wdgImposto, "CDIMPOSTO").then(function(){
                    this.cancelarNewImpostoItemOrca();
                    this.atualizarValorItem();
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            } else {
                wdgImposto.saveRow().then(function(){
                    this.cancelarNewImpostoItemOrca();
                    this.atualizarValorItem();
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }
        }
    };
    /**
     * Cancela a inserção de um novo imposto
     * @params{Object} widget - widget imposto popup
    **/
    this.cancelarNewImpostoItemOrca = function(){
        var widget = templateManager.container.getWidget("wdgItensImpostoProposta");
        ScreenService.closePopup();
        widget.list();
        widget.activate();
        widget.reload();
    };
    /**
     * Atualiza o valor do item da proposta
    **/
    this.atualizarValorItem = function(){
        var wdgProposta    = templateManager.container.getWidget("wdgProposta");
        var nrseqorcamento = wdgProposta.currentRow.NRSEQORCAMENTO;
        var params = {"NRSEQORCAMENTO" : nrseqorcamento};
        General.callRestEngine("product#/fct_itemorcamentoalimenticio_widget/atualizarValorItem", "Row", params).then(function(response) {
            if(response.dataset && response.dataset.fct_orcamentoaliment_widget && response.dataset.fct_orcamentoaliment_widget.VRORCAMENTO){
                wdgProposta.currentRow.VRORCAMENTO = response.dataset.fct_orcamentoaliment_widget.VRORCAMENTO;
            }
        });
    };
    /**
     * Exclui os impostos de um imposto
    **/
    this.excluirImpostoItensOrca = function(widget){
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function() {
            widget.dataSource.getCheckedRows().forEach(function(imposto){
                widget.currentRow = imposto;
                widget.removeRow();
            });
            this.cancelarNewImpostoItemOrca();
            setTimeout(function() {
                this.atualizarValorItem();
            }.bind(this), 100);
        }.bind(this));
    };
    /**
     * Permite ao usuário adicionar uma nova formação de preço
     * @params{Object} widget - widget mão de obra itens
    **/
    this.adicionarItensNovaFormacao = function(widget){
        var wdgItensForm = templateManager.container.getWidget("wdgItensFormacao");
        widget.enabled   = false;
        wdgItensForm.isVisible = true;
        ScreenService.openSwipe(wdgItensForm).then(function() {
            wdgItensForm.enabled = true;
            wdgItensForm.activate();
            wdgItensForm.edit();
        });
    };
    /**
     * Cancela a inserção de uma nova formação de preço no orcamento inserido
     * @params{Object} widget - widget itens de formação de preço  
    **/
    this.voltarWidgetMObraItens = function(widget){
        ScreenService.closePopup();
        widget.isVisible = false;
        widget.enabled   = false;
        
        var wdgMObra = templateManager.container.getWidget("wdgItensMaoObra");
        ScreenService.closeSwipe().then(function(){
            wdgMObra.enabled = true;
            wdgMObra.list();
            wdgMObra.activate().then(function() {
                wdgMObra.reload();
            });
        });
    };
    /**
     * Salva uma nova formação de preço na proposta
     * @params{Object} widget - widget itens formação de preço
    **/
    this.salvaItemFormacaoPreco = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgProposta    = templateManager.container.getWidget("wdgProposta");
            var nrseqorcamento = wdgProposta.currentRow.NRSEQORCAMENTO;
            var formprecoservico = [];
            this.getDataFormacaoServico(widget).forEach(function(newFormacao){
                widget.getField("NRCALCFORMPRECSERV").dataSource.getCheckedRows().forEach(function(elemento){
                    if(newFormacao.NRFORMPRECOSERVICO === elemento.NRFORMPRECOSERVICO){
                        formprecoservico.push({
                            "NRFORMPRECOSERVICO": newFormacao.NRFORMPRECOSERVICO ,
                            "NRCALCFORMPRECSERV": "_" + elemento.NRCALCFORMPRECSERV + "_",
                            "NRSEQORCAMENTO"    : nrseqorcamento,
                            "CDSERVORCA"        : newFormacao.CDSERVORCA,
                            "NRITEMORCAMENTO"   : 0,
                            "QTFORMACAOPROP"    : 0,
                            "__is_new"          : true
                        });
                    }
                });
            });
            General.callRestEngine("product#/fct_itemorcamentoaliment_widget/atualizarQuantidadeMObraItens", "DataSet", formprecoservico).then(function(response){
                ScreenService.showMessage("Processo realizado com sucesso!");
                this.voltarWidgetMObraItens(widget, nrseqorcamento);
                this.atualizarValorItem();
            }.bind(this));
        }
    };
    /**
     * Recalcula o valor total da formação de preço na proposta após a alteração 
     * da quantidade 
     * @params{Object} widget - widget mão de obra (itens)
    **/
    this.calcularVrMObraItens = function(widget){
        var qtFormacao = parseFloat(widget.currentRow.QTFORMACAOPROP_inline);
        var vrunitario = parseFloat(widget.currentRow.VRUNITARIO);
        widget.currentRow.VRTOTAL = qtFormacao * vrunitario;
    };
    /**
     * Edita a quantidade da mão de obra na proposta e manda salvar
     * @params{Object} widget - widget mão de obra (itens)
    **/
    this.atualizarQuantidadeMObraItens = function(widget){
        var nrseqorcamento  = templateManager.container.getWidget("wdgProposta").currentRow.NRSEQORCAMENTO;
        var formModificadas = [];
        widget.dataSource.data.forEach(function(row){
            if(row.QTFORMACAOPROP_inline && row.QTFORMACAOPROP_inline != row.changed_QTFORMACAOPROP){
                row.NRSEQORCAMENTO = nrseqorcamento;
                row.__is_new = false;
                row.changed  = true;
                formModificadas.push(row);
            }
        }.bind(this));
        General.callRestEngine("product#/fct_itemorcamentoaliment_widget/atualizarQuantidadeMObraItens", "DataSet", formModificadas).then(function(){
            widget.list();
            widget.reload();
            this.atualizarValorItem();
            ScreenService.showMessage("Processo realizado com sucesso!");
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * remove formações de preço da proposta
     * @params{Object} widget - widget mão de obra (itens)
    **/
    this.removerMaoObraItens = function(widget){
        var rows = widget.dataSource.getCheckedRows();
        var nrseqorcamento = templateManager.container.getWidget("wdgProposta").currentRow.NRSEQORCAMENTO;
        rows.forEach(function(row){
            row.NRSEQORCAMENTO = nrseqorcamento;
        });
        
        General.callRestEngine("product#/fct_itemorcamentoaliment_widget/deletarMaoObraItens", "DataSet", rows).then(function(){
            widget.list();
            widget.reload();
            ScreenService.showMessage("Processo realizado com sucesso!");
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Exclui um elemento da formçaõ de preço auxiliar e manda
     * editar o valor do item
     * @params{Object} widget widget editar itens formação de preço
    **/
    this.excluirFormacaoItens = function(widget){
        var mensagem = "";
        var params = [
            {
                name: "NRCALCFORMPRECSERVPAI",
                operator: "=",
                value: widget.currentRow.NRCALCFORMPRECSERV
            },
            {
                name: "NRSEQORCAMENTO",
                operator: "=",
                value: widget.currentRow.NRSEQORCAMENTO
            },
            {
                name: "NRFORMPRECOSERVICO",
                operator: "=",
                value: widget.currentRow.NRFORMPRECOSERVICO
            },
            {
                name: "NRRELACFORMACAOPROP",
                operator: "=",
                value: widget.currentRow.NRRELACFORMACAOPROP
            }
        ];
        
        General.callRestEngine("product#/crud/fct_calcformprecserv_itpro_widget", "FilterData", params).then(function(response){
            if(response.dataset && response.dataset.fct_calcformprecserv_itpro_widget && response.dataset.fct_calcformprecserv_itpro_widget.length > 0)
                mensagem = "Apagar este elemento levará a exclusão dos elementos inferiores a ele.<br>Confirma exclusão?";
            else 
                mensagem = "Delete Register?";

            ScreenService.confirmMessage(ScreenService.i18n(mensagem), "question", function(){
                params = {
                    "NRCALCFORMPRECSERV" : widget.currentRow.NRCALCFORMPRECSERV,
                    "NRSEQORCAMENTO"     : widget.currentRow.NRSEQORCAMENTO,
                    "NRRELACFORMACAOPROP": widget.currentRow.NRRELACFORMACAOPROP,
                    "NRFORMPRECOSERVICO" : widget.currentRow.NRFORMPRECOSERVICO
                };

                General.callRestEngine("product#/fct_itemorcamentoaliment_widget/deletarElementosFormacaoProp", "Row", params).then(function(){
                    ScreenService.confirmMessage(ScreenService.i18n("Deseja recalcular a Formação de Preço?"), 'question', function (success) {
                        this.recalcularArvore(widget);
                    }.bind(this));
                    ScreenService.closeSwipe().then(function () {
                        widget.activate();
                        this.atualizarValorItem();
                    }.bind(this));
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }.bind(this));
        }.bind(this));
    };
    /**
     * Salva uma niva despesa inserida na tela de itens 
     * @params{Object} widget - widget despesa(tela itens)
    **/
    this.salvarItemDespesa = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            widget.currentRow.NRSEQORCAMENTO = wdgProposta.currentRow.NRSEQORCAMENTO;
            General.widgetSave(widget).then(function(response){
                if(response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0){
                    widget.currentRow.NRDESPPROP = response.dataset.columnsToReturn[0].NRDESPPROP;
                    widget.currentRow.NRORG      = response.dataset.columnsToReturn[0].NRORG    ;
                }
                this.atualizarValorItem();
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Salva um novo investimento em uma proposta já inclusa
     * @params{Object} widget - widget investimento (aba itens)
     **/
    this.salvarItemInvestimento = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            widget.currentRow.NRSEQORCAMENTO = wdgProposta.currentRow.NRSEQORCAMENTO;
            General.widgetSave(widget).then(function(response){
                if(response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0){
                    widget.currentRow.NRINVESTPROP = response.dataset.columnsToReturn[0].NRINVESTPROP;
                    widget.currentRow.NRORG        = response.dataset.columnsToReturn[0].NRORG;
                }
                this.atualizarValorItem();
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Cancela o popup de inserção de uma taxa em um orcamento já inserido
    **/
    this.cancelarPopupItensTaxa = function(){
        var widget = templateManager.container.getWidget("wdgItensTaxasProposta");
        ScreenService.closePopup();
        widget.list();
        widget.activate().then(function(){
            widget.reload();
        });
    };
    /**
     * Salva novas taxas em um orcamento
     * @params{Object} widget - widget taxas (itens)
    **/
    this.salvarItensTaxas = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            if(!widget.currentRow.__is_new){
                widget.saveRow().then(function(){
                    widget.list();
                    widget.reload();
                });
            } else { 
                var wdgProposta = templateManager.container.getWidget("wdgProposta");
                widget.currentRow.NRSEQORCAMENTO = wdgProposta.currentRow.NRSEQORCAMENTO;
                Utilities.inserirMultiplosComNovoCodigo(widget, "CDTAXA").then(function(){
                    this.cancelarPopupItensTaxa();
                    this.atualizarValorItem();
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }
        }
    };
    /**
     * Deleta taxas de um orcamento já inserido
     * @params{Object} widget - widget taxas (itens)
    **/
    this.deletarItensTaxas = function(widget){
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function() {
            widget.dataSource.getCheckedRows().forEach(function(imposto){
                widget.currentRow = imposto;
                widget.removeRow();
            });
            this.cancelarPopupItensTaxa();
            setTimeout(function() {
                this.atualizarValorItem();
            }.bind(this), 100);
        }.bind(this));
    };
    /**
     * Abre um popup para a inserção de uma nova taxa
    **/
    this.adicionarNewTaxaItens = function(widget){
        var cdTaxas  = [];
        var wdgPopup = templateManager.container.getWidget("wdgPopupTaxasItens");
        widget.dataSource.data.forEach(function(cdtaxa){
            cdTaxas.push(cdtaxa.CDTAXA);
        });
        
        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function(){
            wdgPopup.edit();
            wdgPopup.newRow();
            wdgPopup.activate().then(function() {
                wdgPopup.getField("CDTAXA").dataSourceFilter[0].value = cdTaxas;
                wdgPopup.getField("CDTAXA").reload();
            });
        }.bind(this));
    };
    /**
     * Exibe mensagem de confirmação para copiar a Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.confirmarRenegociarCopiarProposta = function(widget, tipo) {
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente copiar a proposta?"), 'question', function(){   
            this.copiarVersionarProposta(widget, tipo);
        }.bind(this));
    };
    /**
     * Verifica se a proposta escolhida está aceita
     * Caso esteja abre um popup pedindo a justificativa 
     * @params{Object} widget widget proposta
     * @params{String} action nome da action que acionou a função
    **/
    this.validaSituacaoPropCancReneg = function(widget, action){
        var msgConfirmacao = null;
        if((widget.currentRow.IDSITUACAOORCAMENTO == "CANCELADO" && action === "versionar_proposta") || action === "cancelar_proposta")
            msgConfirmacao = "Deseja realmente "+ (action === "cancelar_proposta" ? "cancelar" : "renegociar") +" a proposta?";
        else 
            msgConfirmacao = "Ao renegociar esta proposta, sua versão atual ficará como cancelada. Deseja continuar?";
        
        if(widget.currentRow.IDSITUACAOORCAMENTO === "ACEITO"){
            var params = {
                "NRSEQORCAMENTO": widget.currentRow.NRSEQORCAMENTO
            };
            General.callRestEngine("product#/fct_orcamento_widget/getContratosDifPendentes", "Row", params)
            .then(function(response){
                if(response.dataset && response.dataset.fct_orcamento_widget && response.dataset.fct_orcamento_widget.CONTRATOS){
                    ScreenService.showMessage("Operação bloqueada.<br> Não é permitido "+ (action === "cancelar_proposta" ? "o cancelamento" : "a renegociação") +" de propostas que possuem contratos já aprovados");
                } else {
                    ScreenService.confirmMessage(ScreenService.i18n(msgConfirmacao), 'question', function(){
                        this.abrePopupDsJustCancReneg(widget, action);
                    }.bind(this));
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        } else {
            ScreenService.confirmMessage(ScreenService.i18n(msgConfirmacao), 'question', function(){
                if(action === "cancelar_proposta")
                    this.cancelarProposta(widget);
                else 
                    this.copiarVersionarProposta(widget, 'V');
            }.bind(this));
        }
    };
    /**
     * Abre o popup de edição do operador responsável da proposta
     * @params{Object} widget widget proposta grid
    **/
    this.abrirPopupAlterarOper = function(widget){
        var wdgPopup = templateManager.container.getWidget("wdgPopupAlteraOper");
        wdgPopup.isVisible = true;
        wdgPopup.getField("CDOPERRESPATUAL").readOnly = false;
        
        ScreenService.openPopup(wdgPopup).then(function(){
            wdgPopup.newRow();
            wdgPopup.activate().then(function(){
                this.verificaHierarquia({}, wdgPopup);
                wdgPopup.getField("NRSEQORCAMENTO").dataSourceFilter = [wdgPopup.getField("NRSEQORCAMENTO").dataSourceFilter[0]];
            }.bind(this));
        }.bind(this));
    };
    
    /**
     * filtra as fields de operador atual e propostas do popup de alteração dos
     * operadores responsáveis
     * @params{Object} widget widget popup altera operador responsável
     * @params{Object} field fieldAlterada nome da field de alteraçao
    **/
    this.filtraOperadoresEPropostas = function(widget, fieldAlterada){
        var fieldProposta  = widget.getField("NRSEQORCAMENTO");
        var fieldOperAtual = widget.getField("CDOPERRESPATUAL");

        if(widget.currentRow.CDOPERRESPATUAL && !widget.currentRow.NRSEQORCAMENTO || (widget.currentRow.NRSEQORCAMENTO && widget.currentRow.CDOPERRESPATUAL && fieldAlterada === "CDOPERRESPATUAL")){
            fieldProposta.clearValue();
            if(widget.currentRow.CDOPERRESPATUAL && widget.currentRow.CDOPERRESPATUAL.length > 0){
                fieldProposta.dataSourceFilter[1] = {
                    "name": "CDOPERRESPONSAVEL",
                    "operator": "IN",
                    "value": widget.currentRow.CDOPERRESPATUAL
                };
                fieldOperAtual.readOnly = false;  
            } else {
                if(fieldProposta.dataSourceFilter.length > 1)
                    fieldProposta.dataSourceFilter.pop();
            }
            
            fieldProposta.reload();
        } else {
            widget.currentRow.CDOPERRESPONSAVEL = [];
            widget.getField("NRSEQORCAMENTO").dataSource.getCheckedRows().forEach(function(proposta){
                widget.currentRow.CDOPERRESPONSAVEL.push(proposta.CDOPERRESPONSAVEL);
            });
            
            if((!widget.currentRow.CDOPERRESPATUAL && widget.currentRow.NRSEQORCAMENTO) || (widget.currentRow.NRSEQORCAMENTO && widget.currentRow.CDOPERRESPATUAL && fieldAlterada === "NRSEQORCAMENTO")){
                if(widget.currentRow.CDOPERRESPONSAVEL &&  widget.currentRow.CDOPERRESPONSAVEL.length > 0){
                    fieldOperAtual.clearValue();
                    widget.currentRow.CDOPERRESPATUAL = [];
                    fieldOperAtual.dataSourceFilter[0] = {
                        "name": "CDOPERADOR",
                        "operator": "IN",
                        "value": widget.currentRow.CDOPERRESPONSAVEL
                    };
                    
                    fieldOperAtual.readOnly = true;
                    widget.currentRow.CDOPERRESPONSAVEL.forEach(function(operador){
                        if(widget.currentRow.CDOPERRESPATUAL.indexOf(operador) === -1)
                            widget.currentRow.CDOPERRESPATUAL.push(operador);
                    });
                } else {
                    if(fieldOperAtual.dataSourceFilter.length > 0){
                        fieldOperAtual.dataSourceFilter.pop();
                        fieldOperAtual.readOnly = false; 
                    }
                }
                
                fieldOperAtual.reload();
            }
        }
    };
    
    /**
     * Cancela a edição do operador responsável da proposta
     * @params{Object} widget widget popup alterar operador responsavel
    **/
    this.cancelarAlteracaoOper = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgPropostaGrid");
        var fieldProposta  = widget.getField("NRSEQORCAMENTO");
        var fieldOperAtual = widget.getField("CDOPERRESPATUAL");
        fieldOperAtual.readOnly = false;
        fieldOperAtual.dataSource.clearCheckedRows();
        if(fieldOperAtual.dataSourceFilter.length > 0){
            fieldOperAtual.dataSourceFilter.pop();
        }
        fieldProposta.dataSource.clearCheckedRows();
        if(fieldProposta.dataSourceFilter.length > 1){
            fieldProposta.dataSourceFilter.pop();
        }
        widget.dataSource.data = [];
        widget.isVisible = false;
        ScreenService.closePopup();
        wdgProposta.activate().then(function() {
            wdgProposta.reload();
        });
    };
    
    /**
     * Salva as alterações do operador responsável
    **/
    this.salvaAteracoesOperResp = function(widget){
        if(widget.currentRow.NRSEQORCAMENTO.length === 0 && widget.currentRow.CDOPERRESPATUAL.length !== 0){
            widget.currentRow.NRSEQORCAMENTO  = [];
            widget.getField("NRSEQORCAMENTO").dataSource.data.forEach(function(data){
                widget.currentRow.NRSEQORCAMENTO.push(data.NRSEQORCAMENTO);
            });
        }
        
        if(widget.isValid()){
            var params = {
                "NRSEQORCAMENTO"   : widget.currentRow.NRSEQORCAMENTO,
                "CDOPERRESPONSAVEL": widget.currentRow.CDNOVOOPERRESP,
                "ALTERACONTRATO"   : widget.currentRow.ALTERACONTRATO
            };
            General.callRestEngine("product#/fct_orcamento_widget/salvaAlteracaoMassaOper", "Row", params).then(function(){
                ScreenService.showMessage("Propostas atualizados com sucesso.");
                this.cancelarAlteracaoOper(widget);
            }.bind(this));
        }
    };
    /**
     * Verirfica se o elemento que está sendo editado pode receber um preço
     * @params{Object} rowCusto - row que está sendo altera no inlin edit
    **/
    this.validarValorCusto = function(rowCusto){
        if(!rowCusto.CDSERVORCARELAC && rowCusto.VRCUSTOSERV){
            ScreenService.showMessage("Operação bloqueada!<br>Não é possível associar um preço a elementos que não possuem serviços relacionados.");
            rowCusto.VRCUSTOSERV = "";
        }
    };
    /**
     * Cancela o popup e ativa a widget que contem o nome do paramêtro
     * @params{String} widgetName - nome da widget que será ativada
    **/
    this.cancelarPopupDsJustificativa = function(widgetName){
        ScreenService.closePopup();
        templateManager.container.getWidget(widgetName).activate();
    };
    /**
     * Salva a justificativa de cancelamento ou renegociação da proposta
     * @params{Object} widget - widget popup de justificativa de cancelamento ou renegociação
    **/
    this.salvarJustificativaCancReneg = function(widget){
        if(widget.isValid()){
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            wdgProposta.currentRow.DSJUSTIFCANCRENEG = widget.currentRow.DSJUSTIFCANCRENEG;
            wdgProposta.currentRow.DTACEITEORCA      = null;
            wdgProposta.saveRow().then(function(){
                ScreenService.closePopup();
                widget.isVisible = false;
                wdgProposta.activate();
                if(widget.currentRow.ACTION === "cancelar_proposta")
                    this.cancelarProposta(wdgProposta);
                else 
                    this.copiarVersionarProposta(wdgProposta, 'V');
            }.bind(this))
            .catch(function(error){ 
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Recalcula toda a widget de investimento durante a inserção da proposta
     * @params{Oject} widget - widget investimento inserindo
    **/
    this.calcularWidgetInvetimentoInserindo = function(widget){
        widget.dataSource.data.forEach(function(investimento){
            var qtMeses = this.tirarMascaraValores(investimento.QTMESARMOTIZACAO);
            var vrtotal = this.tirarMascaraValores(investimento.VRINVESTIMENTO);
            var quantidade = this.tirarMascaraValores(investimento.QTINVESTIMENTO);
            investimento.VRTOTINVESTIMENTO = vrtotal * quantidade;
            investimento.VRPARCELA = (vrtotal * quantidade)/qtMeses;
        }.bind(this));
    };
    /**
     * Recalcula toda a widget de investimento durante a inserção da proposta
     * @params{Oject} widget - widget investimento inserindo
    **/
    this.calcularWidgetDespesaInserindo = function(widget){
        widget.dataSource.data.forEach(function(despesa){
            despesa.VRDESPESA = this.tirarMascaraValores(despesa.VRDESPESA);
            this.validarECalcularPeriodo(despesa, "VRDESPESA", "VRTOTDESPPERIODO", "CDPERIODO");
        }.bind(this));
    };
    /**
     * Verifica se a fields de inline edit podem ser alteradas 
     * @params{Object} row - row que está sendo alterada
     * @params{Object} widget - widget da alteracao
    **/
    this.validarSituacaoProposta = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta"); 
        if(wdgProposta.currentRow.IDSITUACAOORCAMENTO !== "ELABORACAO"){
            if(wdgProposta.currentRow.IDSITUACAOORCAMENTO !== "CONCLUIDO" || idAlteraPropConc === "N"){
                ScreenService.showMessage("Operação bloqueada!<br>Não é permitido fazer alterações em propostas com situações diferente de Elaboração.");
                widget.list();
                widget.reload();
                return false;
            }
        }
        return true;
    };
    /**
     * Calcula e salva o valor da proposta apos a alteração do valor de um servico
     * @params{Object} widget - widget de resumo do servico (proposta já inserida)
     * @params{Object} wdgProposta - widget de proposta
    **/
    this.calcularNewVrOrcamento = function(widget, wdgProposta){
        var wdgFinanceiro = templateManager.container.getWidget("wdgResumoGrid");
        var wdgTabbed     = templateManager.container.getWidget("wdgTabbedPrincipalResumoProp");
        if(this.validarSituacaoProposta){
            var params = {
                "NEWVRSERVICO"   : widget.currentRow.VRUNITITEMORCA_GRID,
                "QTITEMORCAMENTO": widget.currentRow.QTITEMORCAMENTO,
                "NRCOMPSERVICO"  : widget.currentRow.NRCOMPSERVICO,
                "NRSEQORCAMENTO" : wdgProposta.currentRow.NRSEQORCAMENTO
            };
            General.callRestEngine("product#/fct_orcamentoaliment_widget/alterarValorServicoTotalItem", "Row", params).then(function(response){
                if(response.dataset && response.dataset.getInfoResultado){
                    wdgProposta.currentRow.VRORCAMENTO = response.dataset.getInfoResultado.VRORCAMENTO;
                    widget.dataSource.data = response.dataset.getInfoResultado.SERVICOS;
                    var vrFin              = response.dataset.getInfoResultado.FINANCEIRO;
                    this.setValoresWdgRFinanceiro(wdgFinanceiro, vrFin["TOTALBRUTO"], vrFin["IMPOSTO"], vrFin["TOTALLIQ"], vrFin["CALIMENTAR"], vrFin["TOTALCTAXA"], vrFin["MOBRA"], vrFin["DESPESAS"], vrFin["INVESTIMENTOS"], vrFin["TAXAS"], vrFin["MARKUP"], vrFin["TOTALSTAXA"], vrFin["TOTALIDEAL"], vrFin["IMPOSTOIDEAL"], vrFin["TOTALLIQIDEAL"], vrFin["MARKUPIDEAL"]);
                }
                widget.redraw();
                wdgFinanceiro.redraw();
                wdgTabbed.list();
            }.bind(this));
        }
    };
    /**
     * Fecha a tela de Enviar E-mail.
     * 
     * @param {object} widget
     */
    this.fecharWdgEnviarEmail = function(widget) {
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        this.desobrigaCamposEvolucaoEmail();
        ScreenService.closeSwipe().then(function() {
            widget.isVisible = false;
            wdgProposta.view();
            wdgProposta.activate();
        });
    };
    /**
     * Exclui um item do cálculo da mão de obra
     * @params{Object} widget - widget itens de formação de preço inserindo
    **/
    this.excluiItensMObraArvore = function(widget) {
        idItensMObraAlterado = true;
        UtilitiesPropostaContrato.excluiItensArvore(widget);
    };
    /**
     * Confirma se o lançamento de proposta será cancelado
    **/
    this.confirmarCancelamentoDoLancamento = function() {
        ScreenService.confirmMessage("Cancelar o lançamento da proposta acarretará na perda das informações descritas até o momento.<br>Deseja continuar?", "question", function() {
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            this.cancelarSalvamentoProposta(wdgProposta);
        }.bind(this));
    };
    /**
     * Limpa todos os dados de lancamentos de proposta do Local Storage
    **/
    this.removerDataLocalStorageLancamento = function() {
        Sm00001000_PropostaAlimenticia_Repository.removeDataLocalStorageEngine("wdgImpostoProposta");
        Sm00001000_PropostaAlimenticia_Repository.removeDataLocalStorageEngine("wdgServicosInserindo");
        Sm00001000_PropostaAlimenticia_Repository.removeDataLocalStorageEngine("wdgInvestimentosInserindo");
        Sm00001000_PropostaAlimenticia_Repository.removeDataLocalStorageEngine("wdgDespesas");
        Sm00001000_PropostaAlimenticia_Repository.removeDataLocalStorageEngine("wdgMaoObraGridInserindo");
        Sm00001000_PropostaAlimenticia_Repository.removeDataLocalStorageEngine("wdgCustoProposta");
    };
    /**
     * Volta para o grid de serviços na tela de itens
     * @params{Object} widget - widget de itens 
    **/
    this.voltarGridItensProposta = function(widget) {
        ScreenService.closeSwipe(); 
        widget.reload();
        widget.getField("VRUNITITEMORCA").isVisible  = true;
        widget.getField("QTITEMORCAMENTO").readOnly  = true;
        widget.getField("NMTIPOORCAMENTO").isVisible = true;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00001000_LancamentoPropostaAlimenticia_Controller", Sm00001000_LancamentoPropostaAlimenticia_Controller);  
});

// FILE: ./assets/js/controllers/Sm00001001_Documento_Proposta_Controller.js
function Sm00001001_Documento_Proposta_Controller(General, ScreenService, Utilities) {
    
/* --- FUNÇÕES DA PÁGINA DOCUMENTO --- */

    var self = this;
    
    /**
     * Salva o registro da página "Documento"
     * 
     * @param {Object} widget widget de documento
     */
    this.salvarDocumento = function(widget) {
        widget.currentRow.__columns_to_return = ["NRSEQORCADOC", "NRORG", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        widget.currentRow.MUDOU_SITUACAO = false;
        if (widget.currentRow.__is_new || 
            (widget.backupRow && widget.backupRow.IDSITUACAOORCAMENTO.toString() != widget.currentRow.IDSITUACAOORCAMENTO.toString())
            ) {
            widget.currentRow.MUDOU_SITUACAO = true;
        }

        Utilities.widgetSaveAndSetColumnsToReturn(widget);

        /*var nrtipoorcamento = widget.currentRow.NRTIPOORCAMENTO;
        
        if (nrtipoorcamento instanceof Object) {
            nrtipoorcamento = 0;
        }
        
        var isNew = widget.currentRow.__is_new;
        var dscaminhoarq = widget.currentRow.DSCAMINHOARQ;
        
        if (!isNew && !(dscaminhoarq instanceof Array)) {
            widget.currentRow.DSCAMINHOARQ = [{
                "name": widget.currentRow.NMARQUIVO,
                "path": dscaminhoarq,
                "size": widget.currentRow.VRTAMANHO * 1024,
                "type": widget.currentRow.DSTIPO
            }];
        }
        
        General.widgetSave(widget).then(function(response) {
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                widget.currentRow.NRSEQORCADOC   = response.dataset.columnsToReturn[0].NRSEQORCADOC;
                widget.currentRow.NRORG          = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO  = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
            }
            
            widget.reload().then(function() {
                this.habilitaFieldSituacao(widget, "save");
                this.verificarSeExisteAnexo(widget);
            }.bind(this));
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });*/
    };
    
    this.setFilterSituacaoEscopo = function(widgetDocumento){
        var fieldEscopo = widgetDocumento.getField("NMTIPOORCAMENTO");
        
        if(widgetDocumento.currentRow.NRSEQORCAMENTO !== ""){
            if(widgetDocumento.currentRow.__is_new){
                widgetDocumento.currentRow.NRTIPOORCAMENTO = "";
            }
            
            fieldEscopo.dataSourceFilter[0].operador = "IN"; 
            fieldEscopo.dataSourceFilter[0].value  = widgetDocumento.currentRow.NRSEQORCAMENTO;
            fieldEscopo.reload();
        } else {
            fieldEscopo.dataSourceFilter[0].operador = "IS_NOT_NULL"; 
        }
    };
    
    /**
     * Atualiza o documento de acordo com o modelo encontrado.
     * 
     * @param {Object} widget
     */
    this.atualizaDocumento = function(widget){
        var params = {
            "NRSEQMODDOCFCT" : widget.currentRow.NRSEQMODDOCFCT,
            "NRSEQORCADOC"   : '_' + widget.currentRow.NRSEQORCADOC + "_"
        };
        
        General.callRestEngine("product#/fct_orcamentodoc_widget/atualizaDocumento", "Row", params).then(function() {
            ScreenService.closePopup();
            ScreenService.showMessage("Documento atualizado com sucesso!");
            widget.reload().then(function() {
                this.verificarSeExisteAnexo(widget);
            }.bind(this));
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    /**
     * Realiza o controle de visibilidade das actions.
     * Se a situação for diferente de "Elaboração" as  
     * actions de edit, Atualizar e delete são blqueadas.
     */
    /*this.controlaVisibilidadeDoc = function(widget) {
        this.verificarSeExisteAnexo(widget);
        
        var situacao = widget.currentRow.IDSITUACAOORCAMENTO;
        
        var actAtualizarDocumento = widget.getAction("atualizarDocumento");
        //var actDocumento          = widget.getAction("documento_proposta");
        var actExcluirAnexo       = widget.getAction("excluirAnexo");
        var actEdit               = widget.getAction("edit");
        var actDelete             = widget.getAction("delete");
        
        if (situacao == "ELABORACAO") {
            actAtualizarDocumento.isVisible = true;
            actEdit.isVisible               = true;
            //actDocumento.isVisible          = true;
            actDelete.isVisible             = true;
        }
        else if (situacao == "CANCELADO") {
            actAtualizarDocumento.isVisible = false;
            actEdit.isVisible               = false;
            //actDocumento.isVisible          = false;
            actExcluirAnexo.isVisible       = false;
            actDelete.isVisible             = false;
        }
        else if (situacao == "CONCLUIDO" || situacao == "ACEITO" || situacao == "ENVIADO") {
            actAtualizarDocumento.isVisible = false;
            actEdit.isVisible               = false;
            //actDocumento.isVisible          = true;
            actExcluirAnexo.isVisible       = false;
            actDelete.isVisible             = false;
        }
    };
    */
    
    
    /**
     * Gera o relatório do documento da proposta.
     */
    /*this.gerarRelatorioDocumentoProposta = function(widget) {
        var nrorg = widget.currentRow.NRORG;
        var nrseqorcamento = widget.currentRow.NRSEQORCAMENTO;
        var nrtipoorcamento = (widget.currentRow.NRTIPOORCAMENTO != null) ? widget.currentRow.NRTIPOORCAMENTO : 0;
        
         var params = {
            "NRSEQORCAMENTO" : nrseqorcamento,
            "NRTIPOORCAMENTO": nrtipoorcamento,
            "NRORG"          : nrorg
        };
        
        General.callRestEngine("product#/fct_orcamentodoc_widget/gerarDocumentoProposta", "Row", params).then(function() {
            
            var reportName = "documento_proposta";
            Utilities.openReport("product#/geraBirtReport", reportName, params);
            ScreenService.closePopup();
        },
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };*/
    
    this.verificarSeExisteAnexo = function(wdgDocumento) {
        var actDownload     = wdgDocumento.getAction("download");
        var actExcluirAnexo = wdgDocumento.getAction("excluirAnexo");
        var fieldAnexo      = wdgDocumento.getField("DSCAMINHOARQ");

        var existe = (wdgDocumento.currentRow.NMARQUIVO) ? true : false;
        actDownload.isVisible     = existe;
        actExcluirAnexo.isVisible = existe;
        fieldAnexo.isVisible      = !existe;
    }; 
    
    /**
     * Abre uma nova janela para fazer o download do documento.
     */
    /*this.downloadDocumento = function(wdgDocumento) {
        ScreenService.closePopup();
        var params = { "NRSEQORCADOC": wdgDocumento.currentRow.NRSEQORCADOC };
        General.callRestEngine("product#/fct_orcamentodoc_widget/downloadDocFileServer", "Row", params).then(function(response) {
            if(response.dataset && response.dataset.URLDOCUMENTO && response.dataset.URLDOCUMENTO.length > 0) {
                var urlDocumento = window.backendIndexUrl + response.dataset.URLDOCUMENTO[0]; 
                window.open(urlDocumento);
            }
        }).catch(function(error){
            window.open(error);
        });
    };
    
    this.excluirAnexo = function(wdgDocumento) {
        var params = {
            "NRSEQORCADOC": wdgDocumento.currentRow.NRSEQORCADOC,
            "DSCAMINHOARQ": wdgDocumento.currentRow.DSCAMINHOARQ,
            "NRORG"       : wdgDocumento.currentRow.NRORG
        };
        
        General.callRestEngine("product#/fct_orcamentodoc_widget/excluirAnexo", "Row", params).then(function() {
            ScreenService.showMessage("Anexo excluído com sucesso!");
            ScreenService.closePopup();
            wdgDocumento.currentRow.NMARQUIVO    = null;
            wdgDocumento.currentRow.DSCAMINHOARQ = null;
            wdgDocumento.currentRow.VRTAMANHO    = null;
            this.verificarSeExisteAnexo(wdgDocumento);
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };*/

    /**
     * Coloca a tela em modo de edição 
     * @params{Object} widget - widget documento de proposta
    **/
    this.editarDocumentoProposta = function(widget) {
        this.habilitaFieldSituacao(widget, "edit");
        General.widgetEdit(widget);
    };
    /**
    * Altera a visibilidade dos dados na field de situacao
    * @params{Object} widget documento da proposta
    **/
    this.habilitaFieldSituacao = function(widget, actionName){
        var idSituacao = widget.currentRow.IDSITUACAOORCAMENTO;
        if(idSituacao) {
            if(actionName == "edit" && !Array.isArray(idSituacao)) 
                widget.currentRow.IDSITUACAOORCAMENTO = idSituacao.split(", ");
            else 
                if(actionName != "edit" && Array.isArray(idSituacao))
                    widget.currentRow.IDSITUACAOORCAMENTO = idSituacao.join(", ");
        }
    };
    /**
     * Deleta um documento ligado a uma proposta
     * @params{Object} widget - widget docuementos proposta
    **/
    this.deletarDocumentoProposta = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function () {
            General.callRestEngine("product#/fct_orcamentodoc_widget/deletarDocumentoProposta", "DataSet", [widget.currentRow]).then(function(){
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
            }).catch(function(error){
                Utilities.showErrorMessage(error);
            });
        });
    };

    this.adicionarAnexoIdioma = function(widget) {
        General.widgetNewRow(widget);
    };
    
    this.editarAnexoIdioma = function(widget) {
        General.widgetEdit(widget);
    };
    
    this.excluirAnexoIdioma = function(widget) {
        General.deleteCrud(widget);
    };
    
    this.salvarAnexoIdioma = function(widget) {
        if (widget.isValid()) {
            Utilities.validarCamposDuplicados(widget, ["NRORCAMENTODOCIDIOMA"], ["CDIDIOM", "NRSEQORCADOC"]).then(function(podeSalvar) {
                if (podeSalvar) {
                    var isNew = widget.currentRow.__is_new;
                    var dscaminhoarq = widget.currentRow.DSCAMINHOARQ;

                    if (!isNew && !(dscaminhoarq instanceof Array)) {
                        widget.currentRow.DSCAMINHOARQ = [{
                            "name": widget.currentRow.NMARQUIVO,
                            "path": dscaminhoarq,
                            "size": widget.currentRow.VRTAMANHO * 1024,
                            "type": widget.currentRow.DSTIPO
                        }];
                    }

                    Utilities.widgetSaveAndSetColumnsToReturn(widget).then(function() {
                        widget.reload();
                    });
                }
                else {
                    ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br><br>Já existe um anexo para o idioma informado."));
                }
            });
        }
    };

    this.cancelarAnexoIdioma = function(widget) {
        General.widgetCancelForm(widget);
    };

    this.downloadDocumento = function(widget) {
        ScreenService.closePopup();
        var params = { "NRORCAMENTODOCIDIOMA": widget.currentRow.NRORCAMENTODOCIDIOMA };
        General.callRestEngine("product#/fct_orcamentodocidioma_completo/downloadDocFileServer", "Row", params).then(function(response) {
            if (response.dataset && response.dataset.URLDOCUMENTO && !Util.isEmptyOrBlank(response.dataset.URLDOCUMENTO[0])) {
                var urlDocumento = window.backendIndexUrl + response.dataset.URLDOCUMENTO[0]; 
                window.open(urlDocumento);
            }
            else {
                ScreenService.showMessage(ScreenService.i18n("Documento não encontrado!"));
            }
        }).catch(function(error) {
            window.open(error);
        });
    };

    this.excluirAnexo = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente excluir o anexo?"), 'question', function() {
            var params = {
                "NRORCAMENTODOCIDIOMA": widget.currentRow.NRORCAMENTODOCIDIOMA,
                "NRSEQORCADOC"        : widget.currentRow.NRSEQORCADOC,
                "DSCAMINHOARQ"        : widget.currentRow.DSCAMINHOARQ,
                "IDPADRAO"            : widget.currentRow.IDPADRAO
            };
            
            General.callRestEngine("product#/fct_orcamentodocidioma_completo/excluirAnexo", "Row", params).then(function() {
                ScreenService.showMessage(ScreenService.i18n("Anexo excluído com sucesso!"));
                ScreenService.closePopup();
                widget.currentRow.NMARQUIVO    = null;
                widget.currentRow.DSCAMINHOARQ = null;
                widget.currentRow.DSTIPO       = null;
                widget.currentRow.VRTAMANHO    = null;
                self.verificarSeExisteAnexo(widget);
            }).catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        });
    };

    this.atualizarDocumentoIdioma = function(wdgAnexosIdioma) {
        ScreenService.confirmMessage("Deseja realmente sobrescrever o documento em anexo para o mesmo documento do Modelo de Documento e Idioma informados?", "question", function() {
            var params = {
                "NRORCAMENTODOCIDIOMA" : wdgAnexosIdioma.currentRow.NRORCAMENTODOCIDIOMA,
                "NRSEQORCADOC"         : wdgAnexosIdioma.currentRow.NRSEQORCADOC,
                "IDPADRAO"             : wdgAnexosIdioma.currentRow.IDPADRAO
            };
            
            General.callRestEngine("product#/fct_orcamentodoc_widget/atualizarDocumentoIdioma", "Row", params).then(function() {
                ScreenService.closePopup();
                ScreenService.showMessage("Documento atualizado com sucesso!");
                return wdgAnexosIdioma.reload();
            }).then(function() {
                self.verificarSeExisteAnexo(wdgAnexosIdioma);
            }).catch(function(error) {
                if (error.data.error.indexOf("SEM_MODELO") === 0) {
                    ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br><br>Não foi encontrado um anexo de acordo com o Modelo de Documento e Idioma informado."));
                }
                else {
                    Utilities.showErrorMessage(error);
                }
            });
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00001001_Documento_Proposta_Controller", Sm00001001_Documento_Proposta_Controller);
});

// FILE: ./assets/js/controllers/Sm00001002_HistoricoAlteracoesProposta_Controller.js
function Sm00001002_HistoricoAlteracoesProposta_Controller(templateManager) {
    
    /**
     * Filtra todas as widgets pelas propostas selecionadas.
     * 
     * @param {object} wdgTabbedPrincipal
     */
    this.filtrarPropostas = function(wdgTabbedPrincipal) {
        var propostas = wdgTabbedPrincipal.getField("NRSEQORCAMENTO").value();
        
        var wdgDadosAlteracao = templateManager.container.getWidget("wdgDadosAteracaoProp");
        wdgDadosAlteracao.dataSourceFilter[0].value = propostas;
        wdgDadosAlteracao.reload();
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00001002_HistoricoAlteracoesProposta_Controller", Sm00001002_HistoricoAlteracoesProposta_Controller);
});

// FILE: ./assets/js/controllers/Sm00001101_RelatorioCustomizavelProposta_Controller.js
function Sm00001101_RelatorioCustomizavelProposta_Controller(templateManager, ScreenService, Sm00001000_LancamentoProposta_Controller, General, ZhFilterPreferences) {
    
    /**
    * Abre a tela de proposta filtrando pelo orcaamento
    * 
    * @param {object} widget widget de dados da proposta  
    **/
    this.abrirProposta = function(widget) {
       ScreenService.openWindow('product#sm_00001_100_lancamento_proposta').then(function() {
            var wdgPropostaGrid = templateManager.container.getWidget('wdgPropostaGrid');
            
            wdgPropostaGrid.reload().then(function() {
                ScreenService.closePopup();
                wdgPropostaGrid.currentRow = {
                    "IDSITUACAOORCAMENTO" : widget.currentRow.IDSITUACAOORCAMENTO,
                    "NRSEQORCAMENTO"      : widget.currentRow.NRSEQORCAMENTO,
                    "IDUTILIZAVARPRECO"   : widget.currentRow.IDUTILIZAVARPRECO,
                    "IDUTILIZACOMP"       : widget.currentRow.IDUTILIZACOMP,
                    "QTITENSTRABSERV"     : widget.currentRow.QTITENSTRABSERV,
                    "NRORG"               : widget.currentRow.NRORG
                };
                Sm00001000_LancamentoProposta_Controller.wdgPropostaGridAfterMoveRow(wdgPropostaGrid);
            });
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00001101_RelatorioCustomizavelProposta_Controller", Sm00001101_RelatorioCustomizavelProposta_Controller);
});

// FILE: ./assets/js/controllers/Sm00002000_Modelo_Proposta_Controller.js
function Sm00002000_Modelo_Proposta_Controller(General, ScreenService, templateManager, Sm00002000_Modelo_Proposta_Service, Utilities, ZhFilterPreferences) {

    var tokenCertisign = null;
    
    var getWidgetPerfilAcesso = function () {
        return templateManager.container.getWidget('ParamPerfilAcesso');
    };
    //field
    var getFieldNrperfilacessoFromWidgetPerfilAcesso = function(){
        return getWidgetPerfilAcesso().getField('NRPERFILACESSO');
    };
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Modelo de proposta para salvar 
     * o novo registro.
    */
    this.saveModeloProposta = function(widget){
        
        widget.currentRow.IDTRANSPORTADORA = null;
        widget.currentRow.IDUTILIZAHIERSERV = null;
        widget.currentRow.NRTIPOORCAMENTO = null; 
        widget.currentRow.__columns_to_return = ["NRORCAMENTOMODELO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "DTULTATU", "CDOPERULTATU"];
        
        General.widgetSave(widget).then(function(response){
            this.bloqueiaItensOuComposicao(widget);
            this.insereSignatariosPadroesOrca(widget);
            
            if (response.dataset){
                widget.currentRow.NRORCAMENTOMODELO = response.dataset.columnsToReturn[0].NRORCAMENTOMODELO;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.DTULTATU = response.dataset.columnsToReturn[0].DTULTATU;
                widget.currentRow.CDOPERULTATU = response.dataset.columnsToReturn[0].CDOPERULTATU;
                
            }
            
        }.bind(this));
        
    };
    
    /**
    * Edita um modelo de documento
    * @params{Object} widget modelo de documento
    **/
    this.documentoEdit = function(widgetModeloDocumento){
        this.habilitaFieldSituacao(widgetModeloDocumento, "edit");
        General.widgetEdit(widgetModeloDocumento);
    };

    this.WdgOnEnterOnce = function(widget){
        ZhFilterPreferences.openFilterWidget(widget).then(function(){
        var widget = templateManager.container.getActiveWidget();
            widget.currentRow.IDATIVO = 'S';
        });
    };

    this.fixSelectMultipleBug = function() {
        var field = getFieldNrperfilacessoFromWidgetPerfilAcesso();
        field.reload();
    };
    this.salvarPerfilAcesso = function(widget){

    var parametro = {
        "NRORCAMENTOMODELO"     : widget.currentRow.NRORCAMENTOMODELO,
        "NRORG"   : widget.currentRow.NRORG,
        "PROPOUCONT" : "P"
    };
    General.callRestEngine('product#/procuraPerfilParametrizado', "Row", parametro).then(function (perfilusuario) {
        if(perfilusuario.dataset.validacao){
            var perfisPermitidos = [];
            widget.currentRow.NRPERFILACESSO.forEach(function(perfil){
                if(!perfilusuario.dataset.validacao.includes(perfil)){
                    perfisPermitidos.push(perfil);
                }
            });
            widget.currentRow.NRPERFILACESSO = perfisPermitidos;
        }

            Utilities.inserirMultiplosComNovoCodigo(widget, "NRPERFILACESSO").then(function() {
                ScreenService.closeSwipe().then(function(){
                    getWidgetPerfilAcesso().reload();
                });

          }.bind(this));
        var field = getFieldNrperfilacessoFromWidgetPerfilAcesso();
        field.dataSource.data = [];
        })
    };
    /**
     * Exclui a Composição relacionada e seus registros filhos.
     * 
     * @param {object} widget
     */
    this.excluirComposicao = function(widget) {
        General.deleteCrud(widget).then(function() {
            widget.reload();
        },
        function(error) {
            console.error(error);
            ScreenService.showMessage(error);
        });
    };
    
    /**
     * Valida se o modelo utiliza composição para bloquear a inserção 
     * de itens que não sejam item espelho.
     * 
     * @param {object} widget 
    */
    
    this.validaComposicao = function(widget){
    
      var wdgModelo = templateManager.container.getWidget("gridModeloProposta");    
        
      var UsaComposicao = wdgModelo.currentRow.IDUTILIZACOMP;
      var UsaItemEspelho = widget.getField("IDITEMESPELHO").value();
      
      if (UsaComposicao == 'S') {
          
          if (UsaItemEspelho == 'NAO'){
              return true;
          }
          else{
             ScreenService.showMessage("Operação Bloqueada.<br><br>Quando o modelo de proposta utiliza composição de serviços, não é permitido vincular um item espelho."); 
             return false; 
          }
      }
      else{
          
        return true;
      }
    };
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Item para salvar o novo registro.
    */
    this.salvarItem = function(widget){
        var wdgModelo = templateManager.container.getWidget("gridModeloProposta");   
        var validaComposicao = this.validaComposicao(widget);
        
        if (validaComposicao){
            widget.currentRow.NRORCAMENTOMODELO = wdgModelo.currentRow.NRORCAMENTOMODELO;
            widget.currentRow.IDADICCOBRITEMORC = 'N';
            this.setFields(widget);
            
            if(widget.currentRow.TIPO == 'S')
                widget.currentRow.CDSERVORCA = widget.currentRow.CODIGO;
            else            
                widget.currentRow.CDPRODUTO = widget.currentRow.CODIGO;
            
            widget.currentRow.__columns_to_return = ["NRITEMORCAMODELO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "DTULTATU", "CDOPERULTATU"];
            General.widgetSave(widget).then(function(response){
                if (response.dataset)
                    widget.currentRow.NRITEMORCAMODELO = response.dataset.columnsToReturn[0].NRITEMORCAMODELO;
                    widget.currentRow.NRORG            = response.dataset.columnsToReturn[0].NRORG;
                    widget.currentRow.DTINCLUSAO       = response.dataset.columnsToReturn[0].DTINCLUSAO;
                    widget.currentRow.CDOPERINCLUSAO   = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                    widget.currentRow.DTULTATU         = response.dataset.columnsToReturn[0].DTULTATU;
                    widget.currentRow.CDOPERULTATU     = response.dataset.columnsToReturn[0].CDOPERULTATU;
                    
                    this.bloqueiaAdicionais(widget);
                    widget.reload();
                    
            }.bind(this));
        }
    };
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Adicionais para salvar o novo registro.
    */
    this.salvarAdicItem= function(widget){
        var wdgModelo = templateManager.container.getWidget("wdgItens");   
        widget.currentRow.NRORCAMENTOMODELO = wdgModelo.currentRow.NRORCAMENTOMODELO;
        widget.currentRow.NRITEMORCAMODELOPAI = wdgModelo.currentRow.NRITEMORCAMODELO;
        this.setFields(widget);
        
        if(widget.currentRow.TIPO == 'S')
            widget.currentRow.CDSERVORCA = widget.currentRow.CODDIGO;
        else
            widget.currentRow.CDPRODUTO = widget.currentRow.CODIGO;
        
        widget.currentRow.__columns_to_return = ["NRITEMORCAMODELO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "DTULTATU", "CDOPERULTATU"];
        General.widgetSave(widget).then(function(response){
            if (response.dataset)
                widget.currentRow.NRITEMORCAMODELO = response.dataset.columnsToReturn[0].NRITEMORCAMODELO;
                widget.currentRow.NRORG            = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO       = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO   = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.DTULTATU         = response.dataset.columnsToReturn[0].DTULTATU;
                widget.currentRow.CDOPERULTATU     = response.dataset.columnsToReturn[0].CDOPERULTATU;
                widget.reload();
            
        }.bind(this));
    };
    
    /**
    * A função "validaEscopoProposta" verifica se o escopo 
     * que está sendo relacionado já está relacionado ao modelo 
     * e salva o escopo no modelo.
     * 
     * * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do escopo relacioando.
     */
    this.validaEscopoProposta = function(widget){
        if (widget.isValid()) {
            var wdgModelo = templateManager.container.getWidget("gridModeloProposta"); 
            var insert = widget.currentRow.__is_new;
          
            if (insert == true){
                widget.currentRow.NRORCAMENTOMODELO = wdgModelo.currentRow.NRORCAMENTOMODELO;
                
                var arrayEscopoModelo = this.criaArrayEscoposModelo(widget);
        
                var tipoOrcamento = arrayEscopoModelo.listaValida;
                var orcaModelo = wdgModelo.currentRow.NRORCAMENTOMODELO;
            
               
                Sm00002000_Modelo_Proposta_Service.validaEscopoProposta(tipoOrcamento, orcaModelo).then(function(response){
                    if (response.dataset.VALIDAESCOPO.VALIDA == false) {
                        
                        General.callRestEngine("product#/fct_relacescopomodelo_widget/saveInsert", "DataSet", arrayEscopoModelo.rows).then(function(){
                        
                            ScreenService.closeSwipe().then(function(){
                                widget.reload();
                                widget.widgets.forEach(function(widget){
                                    widget.enabled = true;
                                });
                            });
                        }.bind(this));
                    }
                    else{
                          ScreenService.showMessage('Operação Bloqueada. Alguns dos escopos de faturamento selecionados já estão inseridos.'); 
                    }
        
               }.bind(this)).catch(function(response){
                   ScreenService.showMessage(response.data.error);
               });
            }
            else{
                General.widgetSave(widget);
            }
        }
    };
    
    
    /**
     * Função para criar o array dos escopos que foram 
     * pegos na função "recebeEscoposModelo";
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do escopo relacioando.
     * 
    */
    
    this.criaArrayEscoposModelo = function(widget){
        
        var listaEscoposModelo = widget.currentRow.NRTIPOORCAMENTO;
        var rows = [];
        var listaValida = [];
        var row;
        
        listaEscoposModelo.forEach(function(escopoModelo, indice){
            row = this.recebeEscoposModelo(widget, indice);
            rows.push(row);
            listaValida.push(row.NRTIPOORCAMENTO);
        }.bind(this)); 
        
        return {
            "rows": rows,
            "listaValida": listaValida
        };
    };
    
    /**
     * Função para pegar as informações do array 
     * de escopos na currentRow, e tratá-las em 
     * uma cópia da currentRow para realizar o save.
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do escopo relacioando.
     * 
     * @param {int} indice index do escopo no array de 
     * escopos presente na currentRow.
    */
    this.recebeEscoposModelo = function(widget, indice){
        
        var wdgModelo = templateManager.container.getWidget("gridModeloProposta"); 
        var currentRow = angular.copy(widget.currentRow);
        
        currentRow.__is_new = true;

        currentRow.NRTIPOORCAMENTO = currentRow.NRTIPOORCAMENTO[indice];
        currentRow.NRORCAMENTOMODELO = wdgModelo.currentRow.NRORCAMENTOMODELO;
        
        return currentRow;
    };
   
   
     /**
     * A função "salvarCompProposta" salva as 
     * composicoes para o modelo.
     * 
     * * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro da composição relacionada.
     */
    this.salvarCompProposta = function(widget) {
        if(widget.isValid()){
            var arrayComposicaoModelo = this.criaArrayComposicaoModelo(widget);
            
            General.callRestEngine("product#/fct_relorcamodcomposicao_widget/save", "DataSet", arrayComposicaoModelo.rows).then(function() {
                ScreenService.closeSwipe().then(function(){
                    widget.reload();
                });
            }).catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    
    /**
     * Função para criar o array das composições que foram 
     * pegos na função "recebeComposicaoModelo";
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro da composição relacionada.
     * 
    */
    
    this.criaArrayComposicaoModelo = function(widget){
        
        var listaComposicaoModelo = widget.currentRow.NRCOMPSERVICO;
        var rows = [];
        var listaValida = [];
        var row;
        
        listaComposicaoModelo.forEach(function(composicaoModelo, indice){
            row = this.recebeComposicaoModelo(widget, indice);
            rows.push(row);
            listaValida.push(row.NRCOMPSERVICO);
        }.bind(this)); 
        
        return {
            "rows": rows,
            "listaValida": listaValida
        };
    };
    
    /**
     * Função para pegar as informações do array 
     * de composições na currentRow, e tratá-las em 
     * uma cópia da currentRow para realizar o save.
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do escopo relacioando.
     * 
     * @param {int} indice index do escopo no array de 
     * escopos presente na currentRow.
    */
    this.recebeComposicaoModelo = function(widget, indice){
        var wdgModelo = templateManager.container.getWidget("gridModeloProposta"); 
        var currentRow = angular.copy(widget.currentRow);
        
        currentRow.__is_new = true;
        
        currentRow.NRCOMPSERVICO = currentRow.NRCOMPSERVICO[indice];
        currentRow.NRORCAMENTOMODELO = wdgModelo.currentRow.NRORCAMENTOMODELO;
        
        return currentRow;
        
    };
    
    /**
     * Seta os campos da widget de itens 
     * que não aparecem na tela para o 
     * usuário
     * 
     * @param {object} widget widget de 
     * Itens
    */
    this.setFields = function(widget){
        widget.currentRow.SGUNIDADE = null;
        widget.currentRow.SGUNIDADEADICIONAL = null;
        widget.currentRow.CDPRECBASESERV = null;
        widget.currentRow.NRPARCTRANSPORTADORA = null;
        widget.currentRow.NRPARCFORNECEDOR = null;
        widget.currentRow.NMREDUITEMORCA = null;
        widget.currentRow.NMMNEMOITEMORCA = null;
        widget.currentRow.DSQTITEMORCA = null;
        widget.currentRow.DSQTITEMORCAADIC = null;
        widget.currentRow.QTITEMORCAADIC = null;
        widget.currentRow.DSAGRUPAITEMORCA = null;
        widget.currentRow.TXFORMULAITORCA = null;
        widget.currentRow.QTDIASCONTRATO = null;
        widget.currentRow.NRTIPOHIERARQSERV = null;
        widget.currentRow.IDSOMASUBITENS = 'N';
        widget.currentRow.IDSOMANOITEMPAI = 'S';
        widget.currentRow.IDSOMANOTOTALORCA = 'S';
        widget.currentRow.IDTRANSPORTADORA = 'TERCEIRO';
        widget.currentRow.IDUTILIZAHIERSERV = 'NAO';
    };
    
    /**
     * Bloqueia ou libera o campo de
     * modalidade de pagamento
     * 
     * @param {object} widget widget de 
     * Itens
    */
    this.blockFieldModPg = function(widget){
        
        var campoNomeModPgto = widget.getField("DSMODPGTO");
        var campoQtdParcelas = widget.getField("QTPARCELASFATU");
        var campoDiaVencParcelaFatu = widget.getField("NRDIAVENCPARCELASFATU");
        
        var tipofaturamento = widget.currentRow.IDFATURACONTR;
        
        if (tipofaturamento == "O") {
            
            campoNomeModPgto.readOnly = false;
            campoNomeModPgto.validations = {"required": true};
            
            campoQtdParcelas.readOnly = true;
            campoQtdParcelas.value(null);
            campoQtdParcelas.validations = {};
            
            campoDiaVencParcelaFatu.validations = {};
        } 
        else {
            campoNomeModPgto.readOnly = true;
            campoNomeModPgto.value(null);
            campoNomeModPgto.validations = {};
            widget.currentRow.CDMODPGTO = null;
            
            if (tipofaturamento == "P") {
                campoQtdParcelas.readOnly = false;
                campoQtdParcelas.validations = {"required": true};
                
                campoDiaVencParcelaFatu.validations = {"required": true};
            }
            else {
                campoQtdParcelas.readOnly = true;
                campoQtdParcelas.value(null);
                campoQtdParcelas.validations = {};
                
                campoDiaVencParcelaFatu.validations = {};
            }
        }
    };
    
    this.blockFieldItModPg = function(widget){
        
        var campoNomeModPgto = widget.getField("DSMODPGTO");
        var campoQtdParcelas = widget.getField("QTPARCELASFATU");
        var campoDiaVencParcelaFatu = widget.getField("NRDIAVENCPARCELASFATU");
        
        var tipofaturamento = widget.currentRow.IDFATURAORCA;
        
        if (tipofaturamento == "O") {
            
            campoNomeModPgto.readOnly = false;
            campoNomeModPgto.validations = {"required": true};
            
            campoQtdParcelas.readOnly = true;
            campoQtdParcelas.value(null);
            campoQtdParcelas.validations = {};
            
            campoDiaVencParcelaFatu.validations = {};
        } 
        else {
            campoNomeModPgto.readOnly = true;
            campoNomeModPgto.value(null);
            campoNomeModPgto.validations = {};
            widget.currentRow.CDMODPGTO = null;
            
            if (tipofaturamento == "P") {
                campoQtdParcelas.readOnly = false;
                campoQtdParcelas.validations = {"required": true};
                
                campoDiaVencParcelaFatu.validations = {"required": true};
            }
            else {
                campoQtdParcelas.readOnly = true;
                campoQtdParcelas.value(null);
                campoQtdParcelas.validations = {};
                
                campoDiaVencParcelaFatu.validations = {};
            }
        }
    };
    
    /**
     * Bloqueia ou libera o campo de
     * Serviço/Produto da widget de Itens
     * 
     * @param {object} widget widget de 
     * Itens
    */
    this.blockServProd = function(widget, trocarLabel) {
        if (trocarLabel === undefined) {
            trocarLabel = true;
        }
        var Itemespelho =  widget.currentRow.IDITEMESPELHO;
        
        if (Itemespelho == "SIM") {
            widget.getField("NMITEMORCAMENTO").readOnly = true;
            if (trocarLabel) widget.currentRow.NMITEMORCAMENTO = 'Item Espelho';
            widget.getField('NMITEMORCAMENTO').validations = {};
        }
        else {
            widget.getField("NMITEMORCAMENTO").readOnly = false;
            if (trocarLabel) widget.currentRow.NMITEMORCAMENTO = null;
            widget.getField('NMITEMORCAMENTO').validations = {"required": true};
        }
    };
    
    /**
     * Bloqueia ou libera os campos
     * 'Adicional para cobrança?'  e 
     * 'Item Pai' da widget de Itens
     * 
     * @param {object} widget widget de 
     * Itens
    */
    this.blockAdItmPai = function(widget) {
        
        var itempai = widget.currentRow.IDADICITEMORC;
        
        if (itempai == "S") {
            
            widget.getField("IDADICCOBRITEMORC").readOnly = false;
            widget.getField("NRITEMORCAMODELOPAI").readOnly = false;
        }
        else {
            widget.getField("IDADICCOBRITEMORC").readOnly = true;
            widget.currentRow.IDADICCOBRITEMORC = 'N';
            
            widget.getField("NRITEMORCAMODELOPAI").readOnly = true;
            widget.currentRow.NRITEMORCAMODELOPAI = null;
        }
    };
    
    /**
     * Realiza o controle dos campos de "Medida" de acordo 
     * com o field "Utiliza Medida?"
     * 
     * @param {object} widget widget de 
     * Itens para pegar o field "IDMULTMEDVRTOT".
    */
    this.bloquearDadosMedida = function(widget){
        
        var usamedida = widget.getField('IDMULTMEDVRTOT').value();
        
        /* Bloqueia a Medida*/
        if (usamedida == 'N') {
            
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = true;
            widget.getField('DSTPMEDIDASERV').value(null);
            widget.getField('CDTPMEDIDASERV').value(null);
            widget.getField('DSTPMEDIDASERV').validations = {};
            
            /* Quantidade de Medida */
            widget.getField('QTMEDIDAORCA').readOnly = true;
            widget.getField('QTMEDIDAORCA').value(null);
            widget.getField('QTMEDIDAORCA').validations = {};
            
        } 
        /* Desbloqueia a Medida*/
        else {
            
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = false;
            widget.getField('DSTPMEDIDASERV').validations = {"required": true};
            
            /* Quantidade de Medida */
            widget.getField('QTMEDIDAORCA').readOnly = false;
            widget.getField('QTMEDIDAORCA').validations = {"required": true};
        }

    };
    
    /**
     * Realiza o cálculo do valor total do item
     * 
     * @param {object} widget widget de 
     * Itens para pegar os fields para cálculo
    */
    
    this.calculaValorTotItem = function(widget){
        var quantidade        = widget.currentRow.QTITEMORCAMENTO;
        var utilizaQtdeMedida = widget.currentRow.IDMULTMEDVRTOT;
        var qtdeMedida        = widget.currentRow.QTMEDIDAORCA;
        var valorUnitario     = widget.currentRow.VRUNITITEMORCA;
        
        if (quantidade) quantidade = parseFloat(quantidade.toString().replace(",", "."));
        if (qtdeMedida) qtdeMedida = parseFloat(qtdeMedida.toString().replace(",", "."));
        if (valorUnitario) valorUnitario = parseFloat(valorUnitario.toString().replace(",", "."));
        
        var valorSemDesconto = quantidade * valorUnitario;
        if (utilizaQtdeMedida == "S") {
            valorSemDesconto *= qtdeMedida;
        }
        
        var valorTotal = valorSemDesconto;
        
        //widget.currentRow.VRITEMSEMDESC   = valorSemDesconto;
        widget.currentRow.VRTOTALITEMORCA = valorTotal;
    };
    
    /**
     * Consolida as chamadas de funções das 
     * Widgets "Itens" e "Adicionais"
     * 
     * @param {object} widget - widget de Itens
     */
    this.funcoesItens = function(widget) {
        this.blockAdItmPai(widget);
        this.blockFieldItModPg(widget);
        this.bloquearDadosMedida(widget);
        this.blockServProd(widget, false);
    };
    
    /**
     * Bloqueia / Desbloqueia as widgets de "Composição de Serviços" e "Adicionais" 
     * ao selecionar um registro.
     * 
     * @param {object} widget 
     */
    this.bloqueiaItensOuComposicao = function(widget) {
        var usaComposicao = widget.currentRow.IDUTILIZACOMP;
        
        if (usaComposicao) {
            if (usaComposicao == 'S') {
                templateManager.container.getWidget("wdgComposicoes").enabled = true;
                templateManager.container.getWidget("wdgItens").enabled = false;
            }
            else {
                templateManager.container.getWidget("wdgComposicoes").enabled = false;
                templateManager.container.getWidget("wdgItens").enabled = true;
            }
        }
    };
    
    /**
     * Bloqueia / Desbloqueia a widget  "Adicionais" ao selecionar ao salvar um item.
     * 
     * @param {object} widget 
     */
    this.bloqueiaAdicionais = function(widget) {
        var wdgModelo = templateManager.container.getWidget("gridModeloProposta");
        
        var UsaComposicao = wdgModelo.currentRow.IDUTILIZACOMP;
      
        if (UsaComposicao == 'S') {
          
            templateManager.container.getWidget("wdgAdicionais").enabled = false;
        }
        else {
          
            templateManager.container.getWidget("wdgAdicionais").enabled = true;
        }
      
    };
    
    /**
     * Bloqueia / Desbloqueiaos campos "Item Obrigatório" e "Item Espelho".
     * 
     * @param {object} widget 
     */
    /*this.bloqCamposComposicao = function(widget) {
      
        var UsaComposicao = widget.parent.currentRow.IDUTILIZACOMP;
      
        var insert = widget.currentRow.__is_new;
     
        if (UsaComposicao == 'S') {
          
            //Item Espelho
            widget.getField('IDITEMESPELHO').readOnly = true;
            widget.currentRow.IDITEMESPELHO = 'NAO';
          
            //Faturamento do Item
            widget.getField('IDFATURAORCA').readOnly = true;
            widget.currentRow.IDFATURAORCA = 'A';
          
            //Grupo de Faturamento
            widget.getField('NMGRUPOFATURA').readOnly = true;
          
            //Dias para início do Faturamento
            widget.getField('QTDIASINICFAT').readOnly = true;
          
            //Item Obrigatório
            widget.getField('IDITEMOBRIGORCA').readOnly = true;
            widget.currentRow.IDITEMOBRIGORCA = 'N';
          
            //Bloqueia alteração
            widget.getField('IDBLOQALTORC').readOnly = true;
            widget.currentRow.IDBLOQALTORC = 'N';
          
            //Exibe no Documento
            widget.getField('IDIMPRESSAOORCA').readOnly = true;
            widget.currentRow.IDIMPRESSAOORCA = 'S';
          
        }
        else {
          
            //Item Espelho
            widget.getField('IDITEMESPELHO').readOnly = false;
          
            //Faturamento do Item
            widget.getField('IDFATURAORCA').readOnly = false;
          
            //Grupo de Faturamento
            widget.getField('NMGRUPOFATURA').readOnly = false;
          
            //Dias para início do Faturamento
            widget.getField('QTDIASINICFAT').readOnly = false;
          
            //Item Obrigatório
            widget.getField('IDITEMOBRIGORCA').readOnly = false;
          
            //Bloqueia alteração
            widget.getField('IDBLOQALTORC').readOnly = false;
          
            //Exibe no Documento
            widget.getField('IDIMPRESSAOORCA').readOnly = false;
          
            if (insert) {
              
                //Item Espelho
                widget.currentRow.IDITEMESPELHO = 'NAO'; 
             
                //Faturamento do Item
                widget.currentRow.IDFATURAORCA = 'A';
             
                //Item Obrigatório*
                widget.currentRow.IDITEMOBRIGORCA = 'N';
             
                //Bloqueia alteração
                widget.currentRow.IDBLOQALTORC = 'N';
             
                //Exibe no Documento
                widget.currentRow.IDIMPRESSAOORCA = 'S';
            }
        }
    };*/
    
     
    /**
     * Salva o novo registro da página "Documento"
     * 
     * @param {object} widget widget de 
     * Documento para salvar o 
     * novo registro.
    */
    this.salvarDocumento = function(widget){
        var wdgModelo = templateManager.container.getWidget("gridModeloProposta");
        
        widget.currentRow.NRORCAMENTOMODELO =wdgModelo.currentRow.NRORCAMENTOMODELO;
        
        widget.currentRow.__columns_to_return = ["NRSEQMODDOCORCA",  "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset)
                widget.currentRow.NRSEQMODDOCORCA = response.dataset.columnsToReturn[0].NRSEQMODDOCORCA;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO  = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            
        }.bind(this));
    };
    
    
    
  /**
     * bloqueia o campo de escopo na edição da página escopo de proposta.
     * 
     * @param {object} widget 
    */
    
    this.preparaCampoEscopo = function(widget){
      
        var insert = widget.currentRow.__is_new;
      
        if (insert == false){
         
            widget.getField('NRTIPOORCAMENTO').showOnForm = false;
            widget.getField('NMTIPOORCAMENTO').showOnForm = true;

        }
        else{
              
            widget.getField('NRTIPOORCAMENTO').showOnForm = true;
            widget.getField('NMTIPOORCAMENTO').showOnForm = false;
        }
    };
    
    this.novoModelo = function(widget) {
        General.widgetNewRow(widget);
        this.blockFieldModPg(widget);
    //    this.validaIdCertsign();
        this.assinaturaEletronicaOnChange(widget);
    };
    
    this.novoItem = function(widget) {
        General.widgetNewRow(widget);
        this.funcoesItens(widget);
    };
    
    /* Salva um modelo de documento */
    this.salvaModeloDocumento = function(widgetModeloDocumento){
        if(!widgetModeloDocumento.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios a serem preenchidos.");
        } else {
            var isNew          = widgetModeloDocumento.currentRow.__is_new;
            var idSituacao     = widgetModeloDocumento.currentRow.IDSITUACAOORCAMENTO;
            var idSituacaoBckp = widgetModeloDocumento.backupRow ? widgetModeloDocumento.backupRow.IDSITUACAOORCAMENTO : null;
            widgetModeloDocumento.currentRow.__columns_to_return  = ["NRRELACESCOPOMODELO", "NRORCAESCOPODOC", "NRORG"];
            widgetModeloDocumento.currentRow.IDSITUACAOADICIONADA = (isNew || !idSituacaoBckp) ? idSituacao : idSituacao.filter(function (situacao) {
                return widgetModeloDocumento.backupRow.IDSITUACAOORCAMENTO.indexOf(situacao) === -1;
            });
            General.callRestEngine("product#/fct_orcaescopodoc_widget/validarExistenciaIdsituacao", "Row", widgetModeloDocumento.currentRow).then(function(response){
                if(isNew && idSituacao !== ""){
                    Utilities.inserirMultiplosComNovoCodigo(widgetModeloDocumento, "IDSITUACAOORCAMENTO", null, "product#/crud/fct_orcaescopodoc/save").then(function(response){
                        if(response.dataset){
                            widgetModeloDocumento.currentRow.NRRELACESCOPOMODELO = response.dataset.columnsToReturn[0].NRRELACESCOPOMODELO;
                            widgetModeloDocumento.currentRow.NRORG               = response.dataset.columnsToReturn[0].NRORG;
                            widgetModeloDocumento.currentRow.NRORCAESCOPODOC     = response.dataset.columnsToReturn[0].NRORCAESCOPODOC;
                        }
                        
                        ScreenService.closeSwipe();
                        widgetModeloDocumento.reload();
                    }.bind(this))
                    .catch(function(error){
                        Utilities.showErrorMessage(error);
                    });
                } else {
                    widgetModeloDocumento.currentRow.IDSITUACAORETIRADA = (isNew || !idSituacaoBckp) ? [] : widgetModeloDocumento.backupRow.IDSITUACAOORCAMENTO.filter(function (situacao) {
                        return idSituacao.indexOf(situacao) === -1;
                    });
                    
                    General.callRestEngine("product#/fct_orcaescopodoc_widget/editarDocumentosEscopoModelo", "Row", widgetModeloDocumento.currentRow).then(function(response){
                        if(!isNew) {
                            widgetModeloDocumento.view();
                        } else {
                            ScreenService.closeSwipe();
                            widgetModeloDocumento.list();
                        }
                        this.habilitaFieldSituacao(widgetModeloDocumento, "save");
                        widgetModeloDocumento.reload();
                    }.bind(this))
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            }.bind(this))
            .catch(function(error){
                if (error.data && error.data.error.indexOf("SITUACAO_JA_CADASTRADA") !== -1) 
                    ScreenService.showMessage("Operação bloqueada!<br>Este modelo de documento já está vinculado a(s) situação(ões) de: " + error.data.error.split("::")[1]);
                else 
                    Utilities.showErrorMessage(error);
            });
        }
    };
    
    /**
    * Habilita fields de situação
    * @params{Object} widget documento
    **/
    this.habilitaFieldSituacao = function(widgetDocumento, actionName){
        var idSituacao = widgetDocumento.currentRow.IDSITUACAOORCAMENTO;
        if(idSituacao) {
            if(actionName == "edit" && !Array.isArray(idSituacao)) 
                widgetDocumento.currentRow.IDSITUACAOORCAMENTO = idSituacao.split(", ");
            else 
                if(actionName != "edit" && Array.isArray(idSituacao))
                    widgetDocumento.currentRow.IDSITUACAOORCAMENTO = idSituacao.join(", ");
        }
    };
    
    /**
    * Cancela o formulário da widget de documento
    * @params{Object} widget
    **/
    this.cancelaFormDocumento = function(widgetDocumento){
        this.habilitaFieldSituacao(widgetDocumento, "cancel");
        General.widgetCancelForm(widgetDocumento);
    };
    
    this.wdgPerfilAcessoCancelar = function() {
        var field = getFieldNrperfilacessoFromWidgetPerfilAcesso();
        field.dataSource.data = [];
    };
    /**
     * Deleta os modelos de documentos ligados a um escopo
     * @params{Object} widget - widget de modelo de documento
    **/
    this.deletarDocumentosEscopo = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), "question", function(){
            General.callRestEngine("product#/fct_orcaescopodoc/deletarDocumentos", "DataSet", [widget.currentRow]).then(function() {
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
            }, function(error) {
                Utilities.showErrorMessage(error);
            });
        });
    };
    
    this.saveSignatPadr = function(widget) {
        if (widget.isValid() ) {
            if (widget.currentRow.__is_new) {
                
                    
                General.callRestEngine('product#/fct_relacescopomodelo_widget/validaSignatPadrOrca', "Row", widget.currentRow).then(function(response) {  
                    
                    var retorno = response.dataset.validaSignatPadrOrca.valida;
                    
                    /* Recebe os envolvidos para notificação*/
                    var envolvidos = null;
                    var arrayEnvolvidos = widget.currentRow.DSEMAILNOTSIGNENV_INS;
                    
                    if (arrayEnvolvidos != null && arrayEnvolvidos != ""){
                        envolvidos = arrayEnvolvidos.join('_');
                    }

                    widget.currentRow.DSEMAILNOTSIGNENV = envolvidos; 
                    
                    /* Recebe os operdores para notificação */
                    var operadores = null;
                    var arrayOperadores = widget.currentRow.CDOPERADORES;
                    
                    // if (arrayOperadores != null && arrayOperadores != ""){
                    //     operadores = arrayOperadores.join('_');
                    // }
                    
                    
                    if(retorno == '0') {
                        General.widgetSave(widget).then(function(response){
                            if(response.dataset && response.dataset.columnsToReturn[0]){
                                widget.currentRow.NRORG             = response.dataset.columnsToReturn[0].NRORG;
                                widget.currentRow.NRSIGNATPADRORCA  = response.dataset.columnsToReturn[0].NRSIGNATPADRORCA;
                                widget.currentRow.NRORCAMENTOMODELO = response.dataset.columnsToReturn[0].NRORCAMENTOMODELO;
                                
                                if (operadores != null) {
                                   var params = { 
                                        "NRSIGNATPADRORCA"  : widget.currentRow.NRSIGNATPADRORCA,
                                        "OPERADORES"        : arrayOperadores
                                   };
                
                                    // if(widget.currentRow.CDOPERADORES.length > 0){
                                       General.callRestEngine('product#/fct_relacescopomodelo_widget/insereOperadoresNotifica', "Row", params).then(function() {
                                       
                                            widget.reload();
                                            
                                       })
                                       .catch(function(response) {
                                           console.error(response);
                                           if (response instanceof Object)
                                               ScreenService.showMessage(response.data.error);
                                           else
                                               ScreenService.showMessage(response);
                                       });
                                    // }
                               }
                               else{
                                   
                                  widget.reload(); 
                               }
                            }
                        }.bind(this))
                        .catch(function(response) {
                            console.error(response);
                            if (response instanceof Object)
                                ScreenService.showMessage(response.data.error);
                            else
                                ScreenService.showMessage(response);
                        });
                    }
                    else{
                        ScreenService.showMessage("Operação Bloqueada! <br><br> O signatário informado já foi inserido.");
                    }
                });
            }
            else{
              
                /* Recebe os envolvidos para notificação*/
                var envolvidos = null;
                var arrayEnvolvidos = widget.currentRow.DSEMAILNOTSIGNENV_INS;
                
                if (arrayEnvolvidos != null){
                    envolvidos = arrayEnvolvidos.join('_');
                }
                
                widget.currentRow.DSEMAILNOTSIGNENV = envolvidos;
                
                /* Recebe os operdores para notificação */
                var operadoresIguais = false;
                
                var bkpOperadores = widget.backupRow.CDOPERADORES;
                var arrayOperadores = widget.currentRow.CDOPERADORES;
                
                if (Utilities.arraysSaoIguais(bkpOperadores, arrayOperadores)) {
                    operadoresIguais = true;
                }
                
                General.widgetSave(widget).then(function(response){
                    if (response.dataset && response.dataset.columnsToReturn[0]) {
                        widget.currentRow.NRORG             = response.dataset.columnsToReturn[0].NRORG;
                        widget.currentRow.NRSIGNATPADRORCA  = response.dataset.columnsToReturn[0].NRSIGNATPADRORCA;
                        widget.currentRow.NRORCAMENTOMODELO = response.dataset.columnsToReturn[0].NRORCAMENTOMODELO;
                    }
                    
                    if (operadoresIguais == false) {
                        // var operadores = arrayOperadores.join('_');
                        
                        var params = { 
                            "NRSIGNATPADRORCA" : widget.currentRow.NRSIGNATPADRORCA,
                            "OPERADORES"       : arrayOperadores
                        };
        
                        // if(widget.currentRow.CDOPERADORES.length > 0){
                            General.callRestEngine('product#/fct_relacescopomodelo_widget/insereOperadoresNotifica', "Row", params).then(function() {
                                widget.reload().then(function() {
                                    this.carregaDadosOperadores(widget);
                                }.bind(this));
                            }.bind(this))
                            .catch(function(error) {
                                Utilities.showErrorMessage(error);
                            });
                        // }
                    }
                    else {
                        this.carregaDadosOperadores(widget);
                    }
                }.bind(this))
                .catch(function(response) {
                    console.error(response);
                    if (response instanceof Object)
                        ScreenService.showMessage(response.data.error);
                    else
                        ScreenService.showMessage(response);
                }); 
            }
        }
    };
    
    /**
     * insere os signatários padrões de modelo de proposta que foram buscados na parametrização SM 
    **/
    this.insereSignatariosPadroesOrca = function(widget){
        if (widget.currentRow.IDASSELETRDIGITAL == "S") {
            General.callRestEngine("product#/fct_relacescopomodelo_widget/insereSignatariosPadroesOrca", "Row", widget.currentRow).then(null).catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    /**
     * Abre a widget de Cadastramento Automático.
     */
    this.abrirCadAuto = function() {
        var wdgSignatPadrCadAuto = templateManager.container.getWidget("signatariosPadroesCadAuto");
        var wdgGridModeloProposta = templateManager.container.getWidget("gridModeloProposta");
        
        wdgSignatPadrCadAuto.isVisible = true;
        
        var row = wdgSignatPadrCadAuto.currentRow;
        row.IDTPSIGNATARIO = "SE";
        row.DSEMAILNOTIFOUTROS = "";
        row.DSEMAILNOTSIGNENV = [];
        row.CDOPERADOR = [];
        row.CDOPERADORES = [];
        wdgSignatPadrCadAuto.edit();
        
        wdgSignatPadrCadAuto.getField("NRORCAMENTOMODELO").reload().then(function() {
            row.NRORCAMENTOMODELO = [wdgGridModeloProposta.currentRow.NRORCAMENTOMODELO];
        });
        
        row.NRPARCNEGOCIO     = [];
        row.IDTIPOASSINATURA  = 'E';
        row.IDTESTEMUNHA      = 'N';
        row.NRORDENACAO       = null;
        
        ScreenService.openSwipe(wdgSignatPadrCadAuto);
        this.controlaSignatarioCadastroAutomatico(wdgSignatPadrCadAuto);
        templateManager.updateTemplate();
    };
    
    /**
     * Fecha a widget de Cadastramento Automático.
     */
    this.cancelaCadAutoSignatPadr = function() {
        
        var wdgSignatPadrCadAuto = templateManager.container.getWidget("signatariosPadroesCadAuto");

        ScreenService.closeSwipe().then(function() {
            wdgSignatPadrCadAuto.isVisible = false;
        });
    };
    
    /**
     * Coloca a widget "signatariosPadroesCadAuto" invisível
     */
    this.CadAutoInvisivel = function() {
        
        var wdgSignatPadrCadAuto = templateManager.container.getWidget("signatariosPadroesCadAuto");

        wdgSignatPadrCadAuto.isVisible = false;
    };
    
    /**
     * Salva os signatários padrões em cadastramento automático  if (wdgProposta.isValid()) {
     */
    this.saveSignatPadrCadAutoOrca = function(widget) {
        
        if (widget.isValid()) {
            
            var wdgSignatariosPadroes = templateManager.container.getWidget("signatariosPadroes");
            
            General.callRestEngine('product#/fct_relacescopomodelo_widget/cadAutoSignatPadrOrca', "Row", widget.currentRow).then(function(response) {  
                var retorno = response.dataset.cadauto.retorno;
                if(retorno == 'OK') {
                    ScreenService.closeSwipe().then(function() {
                        widget.isVisible = false;
                        wdgSignatariosPadroes.reload();
                        ScreenService.showMessage("Cadastramento automático realizado com sucesso!");
                    });
                } else {
                    ScreenService.showMessage("Ocorreram erros durante o processo de cadastramento automático. Favor entrar em contato o administrador do sistema. <br><br> Erros: <br><br>"+retorno);
                }
            });
        }
    };
    
    /**
     * Valida se o token da certsign está preenchido
     */
    this.validaIdCertsign = function(buscarToken) {
        if (buscarToken === undefined) {
            buscarToken = false;
        }
        if (buscarToken) {
            General.callRestEngine('product#/fct_relacescopomodelo_widget/validaIdCertsign', "Row", {}).then(function(response) {  
                tokenCertisign = response.dataset.validaIdCertsign.retorno;
                this.controlarCamposCertsign();
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
        else {
            this.controlarCamposCertsign();
        }
    };
    
    this.controlarCamposCertsign = function() {
        var wdgSignatariosPadroes = templateManager.container.getWidget("signatariosPadroes");
        var wdgGridModeloProposta = templateManager.container.getWidget("gridModeloProposta");
        
        if (tokenCertisign == null) {
            /* Bloqueia a página "Signatários Padrões" */
            wdgSignatariosPadroes.enabled = false;
            
            /* Bloqueia o campo "Utiliza Assinatura Eletrônica ou Digital" */
            wdgGridModeloProposta.getField('IDASSELETRDIGITAL').readOnly = true;
            wdgGridModeloProposta.getField('IDASSELETRDIGITAL').validations = {};
            if (wdgGridModeloProposta.mode == "edit") {
               wdgGridModeloProposta.getField('IDASSELETRDIGITAL').value('N');
            }
        }
        else {
            /* Libera o campo "Utiliza Assinatura Eletrônica ou Digital" */
            wdgGridModeloProposta.getField('IDASSELETRDIGITAL').readOnly = false;
            wdgGridModeloProposta.getField('IDASSELETRDIGITAL').validations = {"required" : true};
            
            if (wdgGridModeloProposta.mode != "edit") {
                /* Libera a página "Signatários Padrões" */
                wdgSignatariosPadroes.enabled = true;
            }
        }
    };
    
    this.adicionarSignatarioPadrao = function(widget) {
        General.widgetNewRow(widget);
        this.controlaSignatario(widget);
    };
    
    this.editarSignatarioPadrao = function(widget) {
        General.widgetEdit(widget);
        this.controlaSignatario(widget);
        this.carregaDadosOperadores(widget);
    };
    
    this.controlaSignatario = function (widget){
        var campoNomePrincipal = widget.getField("NMPRINCIPALPARC");
        var campoIdTestemunha =  widget.getField("IDTIPOASSINATURA");
        var campoTipoAssinatura =  widget.getField("IDTESTEMUNHA");
        if (widget.currentRow.IDTPSIGNATARIO == 'SE') {
            campoNomePrincipal.readOnly = false;
            campoNomePrincipal.validations = {"required": true};
            campoIdTestemunha.readOnly = false;
            campoTipoAssinatura.readOnly = false;
        }
        else {
            campoNomePrincipal.readOnly = true;
            campoNomePrincipal.validations = {};
            campoTipoAssinatura.readOnly = true;
            campoIdTestemunha.readOnly = true;
            widget.currentRow.IDTIPOASSINATURA = "E";
            if (["SC", "SP"].includes(widget.currentRow.IDTPSIGNATARIO)) {
                widget.currentRow.IDTESTEMUNHA = "N";
            }
            else if (["TC", "TP"].includes(widget.currentRow.IDTPSIGNATARIO)) {
                widget.currentRow.IDTESTEMUNHA = "S";
            }
            //Limpando os campos readonly
            widget.currentRow.NRINSCRICAOPARC = widget.currentRow.NMPRINCIPALPARC = "";
        }
    };
    
    this.controlaSignatarioCadastroAutomatico = function(widget){
        var campoNomePrincipal = widget.getField("NRPARCNEGOCIO");
        var campoIdTestemunha =  widget.getField("IDTIPOASSINATURA");
        var campoTipoAssinatura =  widget.getField("IDTESTEMUNHA");
        if (widget.currentRow.IDTPSIGNATARIO == 'SE') {
            campoNomePrincipal.readOnly = false;
            campoNomePrincipal.validations = {"required": true};
            campoIdTestemunha.readOnly = false;
            campoTipoAssinatura.readOnly = false;
        }
        else {
            campoNomePrincipal.readOnly = true;
            campoNomePrincipal.validations = {};
            campoTipoAssinatura.readOnly = true;
            campoIdTestemunha.readOnly = true;
            widget.currentRow.IDTIPOASSINATURA = "E";
            if (["SC", "SP"].includes(widget.currentRow.IDTPSIGNATARIO)) {
                widget.currentRow.IDTESTEMUNHA = "N";
            }
            else if (["TC", "TP"].includes(widget.currentRow.IDTPSIGNATARIO)) {
                widget.currentRow.IDTESTEMUNHA = "S";
            }
        }
        templateManager.updateTemplate();
    };    
    
    this.carregaDadosOperadores = function(widget){
       var operadores = widget.currentRow.CDOPERADORES;
       if (operadores && operadores.length > 0) {
            widget.getField("CDOPERADORES").dataSourceFilter[2] = {
                "name": "CDOPERADOR",
                "operator": "IN",
                "value": operadores
            };
                                                                
            widget.getField("CDOPERADORES").reload();
            widget.getField("CDOPERADORES").dataSourceFilter.splice(2);
            templateManager.updateTemplate();
        }
    };
    
    this.controlaCampoEnviaDocConclusao = function (widget){
        
        if(widget.currentRow.IDASSELETRDIGITAL == 'S') {
             widget.getField("IDENVIADOCCONCLUSAO").readOnly = false;
        }
        else {
             widget.getField("IDENVIADOCCONCLUSAO").readOnly  = true;
        }
    };
    
    this.wdgSignatariosPadroesVoltarView = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.currentRow.changed = false;
        });
    };
    
    this.wdgSignatariosPadroesCancelar = function(widget) {
        var isNew = widget.currentRow.__is_new;
        General.widgetCancelForm(widget);
        if (!isNew)
            widget.currentRow.changed = false;
            
        this.carregaDadosOperadores(widget);
    };
    
    this.assinaturaEletronicaOnChange = function(widget) {
        var idasseletrdigital = widget.currentRow.IDASSELETRDIGITAL;
        var fieldEnviaDocOperResp = widget.getField("IDENVIADOCOPERRESPCONCASS");
        var fieldTipoDocumento = widget.getField("NMTIPODOCCERTSIGN");
        
        if (idasseletrdigital == "N") {
            fieldEnviaDocOperResp.readOnly = true;
            fieldEnviaDocOperResp.value("N");
            fieldTipoDocumento.readOnly = true;
            widget.currentRow.NRTIPODOCCERTSIGN = null;
            widget.currentRow.NMTIPODOCCERTSIGN = null;
        }
        else {
            fieldEnviaDocOperResp.readOnly = false;
            fieldTipoDocumento.readOnly = false;
        }
    };
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00002000_Modelo_Proposta_Controller', Sm00002000_Modelo_Proposta_Controller);
});

// FILE: ./assets/js/controllers/Sm00003000_Escopo_Proposta_Controller.js
function Sm00003000_Escopo_Proposta_Controller(General, ScreenService, Sm00003000_Escopo_Proposta_Service, ValidacoesParametrosFaturamentoUtil, Utilities, templateManager) {
    
    /**
     * Salva o novo registro
     * e seta IDATIVO
     * 
     * @param {object} widget widget de 
     * Escopo de proposta para salvar o 
     * novo registro.
    */
    this.saveEscopoProposta = function(widget){
        this.setNull(widget.currentRow);
        
        widget.currentRow.__columns_to_return = ["NRTIPOORCAMENTO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        
        General.widgetSave(widget).then(function(response) {
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                widget.currentRow.NRTIPOORCAMENTO = response.dataset.columnsToReturn[0].NRTIPOORCAMENTO;
                widget.currentRow.NRORG           = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO      = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO  = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO   = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
            }
        }).catch(function(error){
            
            console.error(error);
            if (error.data){
              ScreenService.showMessage(error.data.error);
            }
            else{
              ScreenService.showMessage(error);
            }
        });
    };
    
    
    /**
     * Seta campos não utilizados
     * com null
     * 
     * @param {object} currentRow 
     * currentRow de Escopo de proposta 
    */
    this.setNull = function(currentRow){
        
        currentRow.IDFATURAORCA = null ;
        currentRow.IDAPURAFATUR = null ;
        currentRow.NRGRUPFATADIC = null;
    };
    
    /**
     * A função "validaServicoEscopo" verifica se o serviço 
     * que está sendo relacionado já está relacionado ao escopo 
     * e salva o serviço no escopo.
     * 
     * * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do serviço relacioando.
     */
     
    this.validaServicoEscopo = function(widget){
        
        if (widget.isValid()) {
            var isNew = widget.currentRow.__is_new;
            if (isNew) {
                var arrayServTabela = this.criaArrayServicosEscopo(widget);

                var servOrca = arrayServTabela.listaValida;
                var escopoOrca = widget.currentRow.NRTIPOORCAMENTO;
               
                Sm00003000_Escopo_Proposta_Service.validaServicoEscopo(servOrca, escopoOrca).then(function(response){
                    if (response.dataset.VALIDASERV.VALIDA == false) {
                        
                        General.callRestEngine("product#/fct_itescopoproposta_widget/save", "DataSet", arrayServTabela.rows).then(function() {
                            General.changeWidgetAcessibility(widget.widgets, true);
                            ScreenService.closeSwipe().then(function() {
                                widget.reload();
                            });
                        }.bind(this))
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }
                    else{
                        ScreenService.showMessage('Operação Bloqueada. Alguns dos serviços selecionados já estão inseridos.'); 
                    }
        
               }.bind(this)).catch(function(response){
                   ScreenService.showMessage(response.data.error);
               });
            }
            else {
                General.widgetSave(widget).then(null).catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
        }
    };
    
    
    /**
     * Função para criar o array dos serviços que foram 
     * pegos na função "recebeServicosEscopo";
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do serviço.
     * 
    */
    
    this.criaArrayServicosEscopo = function(widget){
        
        var listaServicosEscopo = widget.currentRow.CDSERVORCA;
        var rows = [];
        var listaValida = [];
        var row;
        
        listaServicosEscopo.forEach(function(servicoRelac , indice){
            row = this.recebeServicosEscopo(widget, indice);
            rows.push(row);
            listaValida.push(row.CDSERVORCA);
        }.bind(this)); 
        
        return {
            "rows": rows,
            "listaValida": listaValida
        };
    };
    
    /**
     * Função para pegar as informações do  array 
     * de serviços na currentRow, e tratá-las em 
     * uma cópia da currentRow para realizar o save.
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do serviço.
     * 
     * @param {int} indice index do serviço no array de 
     * serviços presente na currentRow.
    */
    this.recebeServicosEscopo = function(widget, indice){
        
        var currentRow = angular.copy(widget.currentRow);
        
        currentRow.__is_new = true;

        currentRow.CDSERVORCA = currentRow.CDSERVORCA[indice];
        
        return currentRow;
        
    };
    
    /**
     * Bloqueia ou libera o campo de
     * Modelo de Contrato
     * 
     * @param {object} widget widget de de Escopo de Proposta
    */
    this.idBloqueiaModContrato = function(widget){
        if(widget.getField("NMMODCONTRATO")) {
            var currentRow = widget.currentRow;
            
            if(currentRow.IDGERACONTRATO == "S"){
                widget.getField("NMMODCONTRATO").readOnly = false;
                widget.getField("NMMODCONTRATO").validations = {"required": true};
            } else {
                widget.getField("NMMODCONTRATO").readOnly = true;
                widget.getField("NMMODCONTRATO").validations = {};
                currentRow.NMMODCONTRATO = null;
                currentRow.NRSEQMODCONTRATO = null;
            }
        }
    };
    
    /**
     * Bloqueia ou libera os campos de Tabela de Preços (Produto) ou
     * Tabela de Preços (Serviço) de acordo com o "Tipo de Escopo"
     * 
     * @param {object} widget widget de Escopo de Proposta
    */
    this.idBloqueiaTabPreco = function(widget){
        if(widget.getField("IDTIPOORCA")){        
            var currentRow = widget.currentRow;
            if(currentRow.IDTIPOORCA == "S") {
                widget.getField("DSPRECBASESERV").readOnly = false;
                widget.getField("DSPRECOBASE").readOnly    = true;
                currentRow.DSPRECOBASE    = null;
                currentRow.CDPRECBASEVEND = null;
            }
            else if (currentRow.IDTIPOORCA == "V") {
                widget.getField("DSPRECOBASE").readOnly    = false;
                widget.getField("DSPRECBASESERV").readOnly = true;
                currentRow.DSPRECBASESERV = null;
                currentRow.CDPRECBASESERV = null;
            }
            else { //IDTIPOORCA = "T"
                widget.getField("DSPRECOBASE").readOnly    = false;
                widget.getField("DSPRECBASESERV").readOnly = false;
            }
        }
    };

    this.atualizaNotaFiscal = function(widget, nomeFieldSerie) {
        if (nomeFieldSerie === undefined) {
            nomeFieldSerie = "DSSERIE";
        }

        var fieldSerie = widget.getField(nomeFieldSerie);
        fieldSerie.clearValue();
        fieldSerie.reload().then(function() {
            ScreenService.alertNotification('Atenção: campo "Série da Nota Fiscal" foi recarregado', 5000);
        });
    }
    
      /**
     * Prepara campos para edição
     * 
     * @param {object} widget widget de Escopo de Proposta
    */
    this.PreparaEdicao = function(widget){
        this.idBloqueiaModContrato(widget);
        this.idBloqueiaTabPreco(widget);
        General.widgetEdit(widget);
    };
    
    this.adicionarEscopo = function(widget) {
        General.widgetNewRow(widget);
        this.idBloqueiaModContrato(widget);
        this.idBloqueiaTabPreco(widget);
    };
    
    /**
     * Ao entrar na widget, pega registro, que deve ser único
     * 
     * @param {object} widget widget de  registro único de Parametrizações
    */
    this.WidgetAfterLoadDataSource = function(widget){
        var actAdd = widget.getAction("add");
        var actEdit = widget.getAction("edit");
        
        if (widget.dataSource.data.length > 0) {
            widget.currentRow = widget.dataSource.data[0];
            actAdd.isVisible = false;
            actEdit.isVisible = true;
        }
        else {
            widget.currentRow = {};
            actAdd.isVisible = true;
            actEdit.isVisible = false;
        }
    };
    
    this.novoParametro = function(widget) {
        widget.newRow();
	    widget.template = 'widget/form.html';
	    
	    this.ControlaCamposParamFat(widget);
    };
    
    this.novoParametroServicos = function(widget) {
        widget.newRow();
	    widget.template = 'widget/form.html';
	    
	    this.bloquearFieldsVencimentoItem(widget);
    };
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget - Widget de registro único de Parametrizações
    */
    this.saveParameterFatura = function(widget, campoPk) {
    
        widget.currentRow.__columns_to_return = [campoPk, "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        
        General.widgetSave(widget).then(function(success){
            if (success.dataset && success.dataset.columnsToReturn[0]) {
                widget.currentRow.NRSEQFATUESCOPO = success.dataset.columnsToReturn[0].NRSEQFATUESCOPO;
                widget.currentRow.NRORG           = success.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO      = success.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO  = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO   = success.dataset.columnsToReturn[0].NRORGINCLUSAO;
            }
            
            widget.view();
            widget.template = "widget/form-view.html";
            this.WidgetAfterLoadDataSource(widget);
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            if (error.data)
                ScreenService.showMessage(error.data.error);
            else
                ScreenService.showMessage(error);
        });
    };
    
    /**
     * Coloca o registro das páginas de parametrização em edição e cria o backup
     * 
     * @param {object} widget widget de registro único de Parametrizações
    */
    
    this.editWidget = function(widget) {
        widget.backupRow = Util.clone(widget.currentRow);
		widget.edit();
		
		widget.template = "widget/form.html";
		
	    this.ControlaCamposParamFat(widget);
    };
    
    this.editarParamFatServicos = function(widget) {
        widget.backupRow = Util.clone(widget.currentRow);
		widget.edit();
		
		widget.template = "widget/form.html";
		
        this.bloquearFieldsVencimentoItem(widget);
    };

    /**
     * Cancela a edição do registro e volta o registro de Backup
     * 
     * @param {object} widget widget de registro único de Parametrizações
    */
    this.cancelEdit = function(widget) {
    	if (widget.backupRow) {
        	Object.keys(widget.backupRow).forEach(function (key) {
    			widget.currentRow[key] = widget.backupRow[key];
    		});
    	}
    	else {
    	    widget.currentRow = {};
    	}
    	
		widget.view();
		widget.reload();
		widget.template = "widget/form-view.html";
    };
    
    /**
     * Realiza o controle dos campos de vencimento de acordo com o "Tipo de Faturamento"
     * 
     * @param {object} widget widget de Contrato para pegar o field "IDFATURACONTR".
    */
    
    this.bloquearFieldsVencimento = function(widget){
        var campoNomeModPgto        = widget.getField("DSMODPGTO");
        var campoQtdParcelas        = widget.getField("QTPARCELASFATU");
        var campoDiaVencParcelaFatu = widget.getField("NRDIAVENCPARCELASFATU");
        var campoTpCompetencia      = widget.getField("IDQTDIASCOMPCON");
        var campoTpFchCompetencia   = widget.getField("IDTPCOMPETCONT");
        var campoTpEmissaoNF        = widget.getField("IDTPDTEMISSNF");
        var campoTpVencNF           = widget.getField("IDTPDTVENC");
        var campoTpPeriodoInc       = widget.getField("IDTPFATUINCOMP");
        var campoFatItem            = widget.getField("IDSOMAITGRUPFAT");
        var campoTpFat              = widget.getField("IDSOMAFATUCONT");
        
        var tipofaturamento = widget.currentRow.IDFATURACONTR;
        
        campoFatItem.defaultValue           = "N";
        widget.currentRow.IDSOMAITGRUPFAT   = "N";
        campoTpFat.defaultValue             = "N";
        widget.currentRow.IDSOMAFATUCONT    = "N";
        
        /*widget.currentRow.NRQTDIASCOMPCON           = null;
        widget.getField("NRQTDIASCOMPCON").value(null);
        widget.getField("NRQTDIASCOMPCON").readOnly = true;*/
        this.bloquearFieldDiasComp(widget);
        
        /*widget.currentRow.NRDIACOMPETCONT           = null;
        widget.getField("NRDIACOMPETCONT").value(null);
        widget.getField("NRDIACOMPETCONT").readOnly = true;*/
        this.bloquearFieldDiasFecha(widget);
        
        /*widget.currentRow.NRDIADTEMISSNF            = null;
        widget.getField("NRDIADTEMISSNF").value(null);
        widget.getField("NRDIADTEMISSNF").readOnly  = true;*/
        this.bloquearFieldDiasEmissao(widget);
        
        /*widget.currentRow.NRDIATPDTVENC           = null;
        widget.getField("NRDIATPDTVENC").value(null);
        widget.getField("NRDIATPDTVENC").readOnly = true;*/
        this.bloquearFieldDiasVenc(widget);
        

        if (tipofaturamento == "A") {
            
            campoNomeModPgto.readOnly           = true;
            campoNomeModPgto.validations        = {};
            widget.currentRow.CDMODPGTO         = null;
            campoNomeModPgto.value(null);
            
            campoQtdParcelas.readOnly           = true;
            campoQtdParcelas.validations        = {};
            campoQtdParcelas.value(null);
            
            campoDiaVencParcelaFatu.readOnly    = true;
            campoDiaVencParcelaFatu.value(null);
            campoDiaVencParcelaFatu.validations = {};
            
            campoTpCompetencia.readOnly         = false;
            campoTpCompetencia.validations      = { "required": true };
            
            campoTpFchCompetencia.readOnly      = false;
            campoTpFchCompetencia.validations   = { "required": true };
            
            campoTpEmissaoNF.readOnly           = false;
            campoTpEmissaoNF.validations        = { "required": true };
            
            campoTpVencNF.readOnly              = false;
            campoTpVencNF.validations           = { "required": true };
            
            campoTpPeriodoInc.readOnly          = false;
            campoTpPeriodoInc.validations       = { "required": true };
            
        } else {
            if(tipofaturamento === "O"){
                
                campoNomeModPgto.readOnly           = false;
                campoNomeModPgto.validations        = { "required": true };
                
                campoQtdParcelas.readOnly           = true;
                campoQtdParcelas.validations        = {};
                campoQtdParcelas.value(null);
                
                campoDiaVencParcelaFatu.readOnly    = true;
                campoDiaVencParcelaFatu.validations = {};
                campoDiaVencParcelaFatu.value(null);
                
                campoTpCompetencia.readOnly         = true;
                campoTpCompetencia.validations      = {};
                campoTpCompetencia.value(null);
                
                campoTpFchCompetencia.readOnly      = true;
                campoTpFchCompetencia.validations   = {};
                campoTpFchCompetencia.value(null);
                
                campoTpEmissaoNF.readOnly           = true;
                campoTpEmissaoNF.validations        = {};
                campoTpEmissaoNF.value(null);
                
                campoTpVencNF.readOnly              = true;
                campoTpVencNF.validations           = {};
                campoTpVencNF.value(null);
                
                campoTpPeriodoInc.readOnly          = true;
                campoTpPeriodoInc.validations       = {};
                campoTpPeriodoInc.value(null);
                
            } else {
                if(tipofaturamento === "X"){
                    
                    campoNomeModPgto.readOnly           = true;
                    campoNomeModPgto.validations        = {};
                    widget.currentRow.CDMODPGTO         = null;
                    campoNomeModPgto.value(null);
                    
                    campoQtdParcelas.readOnly           = true;
                    campoQtdParcelas.validations        = {};
                    campoQtdParcelas.value(null);
                    
                    campoDiaVencParcelaFatu.readOnly    = true;
                    campoDiaVencParcelaFatu.validations = {};
                    campoDiaVencParcelaFatu.value(null);
                    
                    campoTpCompetencia.readOnly         = false;
                    campoTpCompetencia.validations      = { "required": true };
                    
                    campoTpFchCompetencia.readOnly      = false;
                    campoTpFchCompetencia.validations   = { "required": true };
                    
                    campoTpEmissaoNF.readOnly           = false;
                    campoTpEmissaoNF.validations        = { "required": true };
                    
                    campoTpVencNF.readOnly              = false;
                    campoTpVencNF.validations           = { "required": true };
                    
                    campoTpPeriodoInc.readOnly          = false;
                    campoTpPeriodoInc.validations       = { "required": true };
                        
                    campoFatItem.defaultValue            = "S";
                    widget.currentRow.IDSOMAITGRUPFAT   = "S";
                    campoTpFat.defaultValue              = "S";
                    widget.currentRow.IDSOMAFATUCONT    = "S";
                    
                } else {
                    if(tipofaturamento === "P"){
                        campoNomeModPgto.readOnly           = true;
                        campoNomeModPgto.validations        = {};
                        widget.currentRow.CDMODPGTO         = null;
                        campoNomeModPgto.value(null);
                        
                        campoQtdParcelas.readOnly           = false;
                        campoQtdParcelas.validations        = { "required": true};
                        
                        campoDiaVencParcelaFatu.readOnly    = false;
                        campoDiaVencParcelaFatu.validations = { "required": true};
                        
                        campoTpCompetencia.readOnly         = true;
                        campoTpCompetencia.validations      = {};
                        campoTpCompetencia.value(null);
                        
                        campoTpFchCompetencia.readOnly      = true;
                        campoTpFchCompetencia.validations   = {};
                        campoTpFchCompetencia.value(null);
                        
                        campoTpEmissaoNF.readOnly           = true;
                        campoTpEmissaoNF.validations        = {};
                        campoTpEmissaoNF.value(null);
                        
                        campoTpVencNF.readOnly              = true;
                        campoTpVencNF.validations           = {};
                        campoTpVencNF.value(null);
                        
                        campoTpPeriodoInc.readOnly          = true;
                        campoTpPeriodoInc.validations       = {};
                        campoTpPeriodoInc.value(null);
                    }
                }
            }
        }
    };
    
    /**
     * Realiza o controle dos campos de vencimento de acordo com o "Tipo de Faturamento"
     * 
     * @param {object} widget widget de Contrato para pegar o field "IDFATURACONTR".
     */
    
    this.bloquearFieldsVencimentoItem = function(widget){
        
        var tipofaturamento = widget.currentRow.IDFATURACONTR;
        
        // Não Periódico: Desbloqueia a Modalidade de Pagamento
        if (tipofaturamento == 'O') {
            
            widget.getField('DSMODPGTO').readOnly = false;
            widget.getField('DSMODPGTO').validations = {"required": true};

        }
        // Periódico / Apuração: Bloqueia a Modalidade de Pagamento
        else{
    
            widget.getField('DSMODPGTO').readOnly = true;
            widget.getField('DSMODPGTO').value(null);
            widget.getField('CDMODPGTO').value(null);
            widget.getField('CDMODPGTO').validations = {};
        }
    };
    
    
     /**
     * Realiza o controle do campo "Dia Para Competência do Contrato" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDQTDIASCOMPCON".
    */
    
    this.bloquearFieldDiasComp = function(widget){
        
        var tipoCompetencia = widget.getField('IDQTDIASCOMPCON').value();
        
        /* Desbloqueia o campo "Dia Para Competência do Contrato" */
        if (tipoCompetencia == "2") {
            
            widget.getField('NRQTDIASCOMPCON').readOnly = false;
            widget.getField('NRQTDIASCOMPCON').validations = {"required": true};
        } 
        /* Bloqueia o campo "Dia Para Competência do Contrato" */
        else {

            widget.getField('NRQTDIASCOMPCON').readOnly = true;
            widget.getField('NRQTDIASCOMPCON').value(null);
            widget.getField('NRQTDIASCOMPCON').validations = {};
        }
    };
   
   
    /**
     * Realiza o controle do campo "Dia Para Competência do Contrato" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDQTDIASCOMPCON".
    */
    
    this.ValidaFieldDiasComp = function(widget){
        
        var dia = widget.getField('NRQTDIASCOMPCON').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRQTDIASCOMPCON = null;
        }

    };
   
        /**
     * Realiza o controle do campo "Dia Para Fechamento da Competência" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPCOMPETCONT".
    */
    
    this.bloquearFieldDiasFecha = function(widget){
        
        var fechaCompetencia = widget.getField('IDTPCOMPETCONT').value();
        
        /* Bloqueia o campo "Dia Para Fechamento da Competência" */
        if (!fechaCompetencia || fechaCompetencia == "2") {
            
            widget.getField('NRDIACOMPETCONT').readOnly = true;
            widget.getField('NRDIACOMPETCONT').value(null);
            widget.getField('NRDIACOMPETCONT').validations = {};
        } 
        /* Desbloqueia o campo "Dia Para Fechamento da Competência" */
        else {

            widget.getField('NRDIACOMPETCONT').readOnly = false;
            widget.getField('NRDIACOMPETCONT').validations = {"required": true};
        }
    };
   
   
   /**
     * Realiza o controle do campo "Dia Para Fechamento da Competência" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPCOMPETCONT".
    */
    
    this.ValidaFieldDiasFecha = function(widget){
        
        var dia = widget.getField('NRDIACOMPETCONT').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRDIACOMPETCONT = null;
        }

    };
   
    /**
     * Realiza o controle do campo "Dia Para Emissão da NF" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTEMISSNF".
    */
    
    this.bloquearFieldDiasEmissao = function(widget){
        
        var emissao = widget.getField('IDTPDTEMISSNF').value();
        
        /* Bloqueia o campo "Dia Para Emissão da NF" */
        if (!emissao || emissao == "9") {
            
            widget.getField('NRDIADTEMISSNF').readOnly = true;
            widget.getField('NRDIADTEMISSNF').value(null);
            widget.getField('NRDIADTEMISSNF').validations = {};

        } 
        /* Desbloqueia o campo "Dia Para Emissão da NF" */
        else {

            widget.getField('NRDIADTEMISSNF').readOnly = false;
            widget.getField('NRDIADTEMISSNF').validations = {"required": true};
        }
    };
    
    /**
     * Realiza o controle do campo "Dia Para Emissão da NF" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTEMISSNF".
    */
    
    this.ValidaFielDiasEmissao = function(widget){
        
        var dia = widget.getField('NRDIADTEMISSNF').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRDIADTEMISSNF = null;
        }

    };
    
    /**
     * Realiza o controle do campo "Dia Para Vencimento da Parcela" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTVENC".
    */
    
    this.bloquearFieldDiasVenc = function(widget){
        
        var vencimento = widget.getField('IDTPDTVENC').value();
        
        /* Bloqueia o campo "Dia Para Vencimento da Parcela" */
        if (!vencimento || vencimento == "5") {
            
            widget.getField('NRDIATPDTVENC').readOnly = true;
            widget.getField('NRDIATPDTVENC').value(null);
            widget.getField('NRDIATPDTVENC').validations = {};
            
        } 
        /* Desbloqueia o campo "Dia Para Vencimento da Parcela" */
        else {

            widget.getField('NRDIATPDTVENC').readOnly = false;
            widget.getField('NRDIATPDTVENC').validations = {"required": true};
        }
    };
   
    /**
     * Realiza o controle do campo "Dia Para Vencimento da Parcela" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTVENC".
    */
    
    this.ValidaFielDiasVenc = function(widget){
        
        var dia = widget.getField('NRDIATPDTVENC').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRDIATPDTVENC = null;
        }

    };
    
    this.ControlaCamposParamFat = function(widget){
        this.bloquearFieldsVencimento(widget);
        /*this.bloquearFieldDiasComp(widget);
        this.bloquearFieldDiasFecha(widget);
        this.bloquearFieldDiasEmissao(widget);
        this.bloquearFieldDiasVenc(widget);*/
    };
    
    this.parametrosOnEnterTab = function(widget) {
        widget.list();
        widget.reload().then(function() {
            widget.view();
        });
    };
    
    this.wdgParametrosFaturamentoItemNewRow = function(wdgParametrosFaturamentoItem) {
        Utilities.formWidgetNewRow(wdgParametrosFaturamentoItem);
        General.changeWidgetAcessibility(getWidgetsParaControlarAcessibilidadeFaturamentoItem(wdgParametrosFaturamentoItem), false);

        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(wdgParametrosFaturamentoItem, 'wdgCompetenciaServico');
    };

    this.wdgParametrosFaturamentoItemEdit = function(wdgParametrosFaturamentoItem) {
        Utilities.formWidgetEdit(wdgParametrosFaturamentoItem);
        General.changeWidgetAcessibility(getWidgetsParaControlarAcessibilidadeFaturamentoItem(wdgParametrosFaturamentoItem), false);

        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(wdgParametrosFaturamentoItem, 'wdgCompetenciaServico');
    };

    this.wdgParametrosFaturamentoItemCancel = function(wdgParametrosFaturamentoItem) {
        Utilities.formWidgetCancel(wdgParametrosFaturamentoItem);
        General.changeWidgetAcessibility(getWidgetsParaControlarAcessibilidadeFaturamentoItem(wdgParametrosFaturamentoItem), true);
        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(wdgParametrosFaturamentoItem, 'wdgCompetenciaServico');
    };

    var getWidgetsParaControlarAcessibilidadeFaturamentoItem = function(widget) {
        var widgetsToDeactivate = widget.parent.widgets;
        
        return widgetsToDeactivate.filter(function(subwidget) {
            return subwidget.name != widget.name;
        }).concat(widget.parent);
    };

    this.wdgParametrosFaturamentoItemSave = function(wdgParametrosFaturamentoItem) {
        if (wdgParametrosFaturamentoItem.isValid()) {
            Utilities.validarCamposDuplicados(wdgParametrosFaturamentoItem, ["NRPARAMFATITESC"], ["NRTIPOORCAMENTO", "NRITESCOPOPROPOSTA"])
            .then(function(validacao) {
                if (!validacao) {
                    ScreenService.showMessage("Operação bloqueada. <br/>Já existe uma parametrização de faturamento cadastrada para este item.");
                }
                else {
                    Utilities.formWidgetSaveAndSetColumnsToReturn(wdgParametrosFaturamentoItem).then(function() {
                        ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoOnEnter(wdgParametrosFaturamentoItem, "wdgCompetenciaServico");
                    });
                }
            });
        }
    };

    this.wdgTabelaPrecosSave = function(wdgTabelaPrecosItem) {
        if (wdgTabelaPrecosItem.isValid()) {
            Utilities.validarCamposDuplicados(wdgTabelaPrecosItem,["NRTABPRECESCOPOFAT"],["NRTIPOORCAMENTO", "NRMOEDA"])
            .then(function(podeSalvar) {
                var idPadraoOld = null;
                if (wdgTabelaPrecosItem.backupRow) {
                    idPadraoOld = wdgTabelaPrecosItem.backupRow.IDPADRAO;
                }
                var wdgEscopo = templateManager.container.getWidget("grid_escopo");
                if(!podeSalvar || wdgTabelaPrecosItem.currentRow.IDPADRAO == 'S' && wdgEscopo.currentRow.CDPRECBASESERV != null && idPadraoOld != "S"){
                    ScreenService.showMessage("Operação bloqueada. <br/><br/>Já existe uma tabela de preço cadastrada para a moeda informada.");
                } else {
                    if(wdgTabelaPrecosItem.currentRow.NRTABPRECESCOPOFAT == null) {
                        wdgTabelaPrecosItem.currentRow.__is_new = true;
                    }
                    
                    Utilities.widgetSaveAndSetColumnsToReturn(wdgTabelaPrecosItem);
                    
                   
                    if (wdgTabelaPrecosItem.currentRow.IDPADRAO == "N" && wdgTabelaPrecosItem.backupRow && wdgTabelaPrecosItem.backupRow.IDPADRAO == "S") {
                        wdgEscopo.currentRow.CDPRECBASESERV = null;
                        wdgEscopo.currentRow.CDPRECBASEVEND = null;
                        wdgEscopo.currentRow.changed = true;
                        wdgEscopo.saveRow();
                    } 
                    
                    if (wdgTabelaPrecosItem.currentRow.IDPADRAO == "S") {
                        wdgEscopo.currentRow.CDPRECBASESERV = wdgTabelaPrecosItem.currentRow.CDPRECBASESERV;
                        wdgEscopo.currentRow.CDPRECBASEVEND = wdgTabelaPrecosItem.currentRow.CDPRECBASEVEND;
                        wdgEscopo.currentRow.changed = true;
                        wdgEscopo.saveRow();
                    }    
                  
                }
            }).catch(function(error){
                Utilities.showErrorMessage(error);
            });
        };
    };

    this.wdgTabelaPrecosDelete = function(wdgTabelaPrecosItem) {
        var idpadrao = wdgTabelaPrecosItem.currentRow.IDPADRAO;
        General.deleteCrud(wdgTabelaPrecosItem).then(function() {
            if (idpadrao == 'S') {
                var wdgEscopo = templateManager.container.getWidget("grid_escopo");
                wdgEscopo.currentRow.CDPRECBASESERV = null;
                wdgEscopo.currentRow.CDPRECBASEVEND = null;
                wdgEscopo.currentRow.changed = true;
                wdgEscopo.saveRow().then(function() {
                    wdgTabelaPrecosItem.reload();
                });
            }
        });
    }

    this.wdgAgrupamentoSave = function(wdgAgrupamentos) {
        if (wdgAgrupamentos.isValid()) {
            Utilities.widgetSaveAndSetColumnsToReturn(wdgAgrupamentos).then(function() {;    
                wdgAgrupamentos.reload();  
            });    
        };
    };

    this.wdgAgrupamentoDelete = function(wdgAgrupamentos) {
        General.deleteCrud(wdgAgrupamentos).then(function() {
            wdgAgrupamentos.reload();
        });
    }
    

    this.salvarDescricaoEscopo = function(wdgDescricaoEscopo) {
        if (wdgDescricaoEscopo.isValid()) {
            var camposValidacao = [
                "NRTIPOORCAMENTO",
                "IDTPFATURA",
                "CDFILIALFATURA",
                "NRPARCFATURA",
                "CDTIPOOPERFATURA",
                "CDSERIENF",
                "CDEMPRESA",
                "CDINSCESTA",
                "CDINSCMUNI",
                "IDPREVCOBRFUT"
            ];

            Utilities.validarCamposDuplicados(wdgDescricaoEscopo, ["NRDESCAGRUPESCOPO"], camposValidacao)
            .then(function(podeSalvar) {
                if (!podeSalvar) {
                    ScreenService.showMessage("Operação bloqueada. <br/><br/>Já existe uma descrição cadastrada para estas parametrizações.");
                }
                else {
                    Utilities.widgetSaveAndSetColumnsToReturn(wdgDescricaoEscopo);
                }
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
        else {
            ScreenService.showMessage("Preencha todos os campos corretamente!");
        }
    };
    
    /**
     * Preenchee o campo de percentual com o valor restante
     * 
     * @params{Object} widget - widget de índice
    **/    
    this.completaPercentual = function(widget){
        General.widgetNewRow(widget);
        var params = {
           "NRTIPOORCAMENTO" : widget.currentRow.NRTIPOORCAMENTO
        };
        
        General.callRestEngine('product#/fct_paramfatescopo_widget/completaPercentual', "Row", params).then(function(response){  
            var percentual = response.dataset.completaPercentual.PERCENTUAL;
            widget.currentRow.NRPERCENTINDICE = percentual;
        });
    };
    
    /**
     * Valida o percentual dos índices financeiros
     * 
     * @params{Object} widget - widget de índices financeiros
    **/
    this.validaSaveIndFinanceiro = function(widget) {
        if(!widget.currentRow.__is_new) {
            this.salvarIndiceFinanceiro(widget);
        } else {
            Utilities.validarCamposDuplicados(widget, null, ["CDINDICE", "NRTIPOORCAMENTO"]).then(function(response) {
                if(!response)
                    ScreenService.showMessage("Operação bloqueada!<br><br>O índice selecionado já está inserido.");
                else
                    this.salvarIndiceFinanceiro(widget);
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    /**
     * Salva um índice
     * 
     * @params{object} widget - widget de índice financeiro 
    **/
    this.salvarIndiceFinanceiro = function(widget) {
        var params = {
           "NRTIPOORCAMENTO" : widget.currentRow.NRTIPOORCAMENTO,
           "NRPERCENTINDICE" : widget.currentRow.NRPERCENTINDICE,
           "CDINDICE"        : (widget.currentRow.__is_new) ? null : widget.currentRow.CDINDICE
        };
        
        General.callRestEngine('product#/fct_paramfatescopo_widget/validaPercentual', "Row", params).then(function(response) {  
            var percentual = response.dataset.validaPercentual.PERCENTUAL;
            if (percentual > 100) {
                ScreenService.showMessage("Operação Bloqueada!<br><br>A soma dos percentuais de utilização dos índices financeiros ultrapassou o total de 100%.");
            } else {
                General.widgetSave(widget).then(function(response){
                    widget.reload();
                    
                    if(percentual < 100)
                        ScreenService.showMessage("Atenção! <br><br> A soma dos percentuais de utilização dos índices financeiros não está totalizando os 100% necessários para realização do reajuste dos contratos.");
                });
            }
        }.bind(this));
        
    };
    
    /**
     * Chama as funções necessárias para o cancelamento da edição e insersão 
     * da widget de parametros de faturamento do escopo
     * 
     * @params{Object} widget - widget de parâmetros de faturamento do escopo
    **/
    this.cancelarEdicaoParametrosDeFaturamento = function(widget) {
        ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoCancel(widget, 'grid_escopo', 'wdgParametrosFaturamentoTabbed'); 
        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(widget, 'wdgParametrosFaturamento');  
        
        this.habilitarCamposDeCalculoDtSaida(widget);
        templateManager.container.getWidget('wdgParametrosFaturamentoTabbed').activate(); 
    };
    
    /**
     * Valida o tipo de cálculo da data de saída e habilita o campo 
     * do parametro de cálculo
     * 
     * @params{Object} widget - widget parametros do escopo
    **/
    this.habilitarCamposDeCalculoDtSaida = function(widget) {
        var fieldNrCalculo = widget.getField("NRCALCULODTSAIDANF");
        var idtpcalculo    = widget.currentRow.IDTPCALCDTSAIDANF;
        
        if(!idtpcalculo || idtpcalculo == 1) {
            fieldNrCalculo.readOnly              = true;
            fieldNrCalculo.label                 = "Dia fixo / Quantidade de dias para data de saída";
            fieldNrCalculo.validations           = {};
            widget.currentRow.NRCALCULODTSAIDANF = null;
        } else {
            var nmlabel = ([2, 3].indexOf(idtpcalculo) != -1) ? "Dia fixo para data de saída" : "Quantidade de dias para data de saída";
            
            fieldNrCalculo.readOnly    = false;
            fieldNrCalculo.label       = nmlabel;
            fieldNrCalculo.validations = { "required": true };
        }
        
        templateManager.updateTemplate();
    };
    
    /**
     * Valida se o valor digitado se aplica ao tipo de calculo escolhido
     * 
     * @params{Object} widget - widget parametros de faturamento 
    **/
    this.validarParametrosCalculoDtSaida = function(widget) {
        if([2, 3].indexOf(widget.currentRow.IDTPCALCDTSAIDANF) != -1) {
            if(widget.currentRow.NRCALCULODTSAIDANF > 31) {
                ScreenService.showMessage("Operação bloqueada!<br><br>Não é possível atribuir um dia fixo maior que 31 para o cálculo da data de saída da nota fiscal.");
                widget.currentRow.NRCALCULODTSAIDANF = 0;
            }
        }
    };
    
    /**
     * Valida se o escopo participa de um agupamento e bloqueia a alteração
     * dos parametros de faturamento.
     * 
     * @params{Object} widget - widget de parametros de faturamento do contrato
    **/
    this.salvarDadosGeraisParamFaturamento = function(widget) {
        var escopoAgrupado = templateManager.container.getWidget("grid_escopo").currentRow.ESCOPOAGRUPADO;
        if(escopoAgrupado != "S") {
            ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoSave(widget, 'grid_escopo', 'wdgParametrosFaturamentoTabbed', 'wdgParametrosFaturamento');
        } else {
            var camposFatura = ["CDTIPOOPERFATURA", "CDSERIENF", "CDEMPRESA", "CDINSCESTA", "CDINSCMUNI", "CDFILIALFATURA", "IDTPCALCDTSAIDANF"];
            var possuiAlteracaoParams = false;
            if (widget.backupRow) {
                camposFatura.forEach(function(campo) {
                    if(widget.currentRow[campo] != widget.backupRow[campo]) {
                        widget.currentRow[campo] = widget.backupRow[campo];
                        possuiAlteracaoParams    = true;
                    }
                });
            }
            
            if(possuiAlteracaoParams)
                ScreenService.showMessage("Operação bloqueada!<br><br>Não é possível alterar as informações de Tipo de Operação, Série da Nota Fiscal, Unidade para Faturamento e Tipo de Cálculo para Data de Saída dos escopos que participam de agrupamentos.");
            else 
                ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoSave(widget, 'grid_escopo', 'wdgParametrosFaturamentoTabbed', 'wdgParametrosFaturamento');
        }
    };
    
    /**
     * Valida se o escopo participa de um agupamento e bloqueia a alteração
     * da competencia.
     * 
     * @params{Object} widget - widget de competencia
    **/
    this.salvarParamsCompetenciaEscopo = function(widget) {
        var escopoAgrupado = templateManager.container.getWidget("grid_escopo").currentRow.ESCOPOAGRUPADO;
        if(escopoAgrupado != "S") {
            ValidacoesParametrosFaturamentoUtil.wdgParametrosFaturamentoSave(widget, 'NRTIPOORCAMENTO');
        } else {
            if(widget.currentRow.__is_new) {
                ScreenService.showMessage("Operação bloqueada!<br><br>Não é possível adicionar informações de competência para escopos que participam de agrupamentos.");
            } else {
                var camposAlterados = false;
                Object.keys(widget.backupRow).forEach(function (key) {
    				if(widget.backupRow[key] && widget.currentRow[key] != widget.backupRow[key]) {
    				    widget.currentRow[key] = widget.backupRow[key];
    				    camposAlterados        = true;
    				}
    			});
    			
    			if(camposAlterados)
                    ScreenService.showMessage("Operação bloqueada!<br><br>Não é possível alterar as informações de competência dos escopos que participam de agrupamentos.");
                else 
                    ValidacoesParametrosFaturamentoUtil.wdgParametrosFaturamentoSave(widget, 'NRTIPOORCAMENTO');
            }
        }
    };
    
    /**
     * Valida se o escopo participa de um agupamento e bloqueia a exclusao
     * da competência
     * 
     * @params{Object} widget - widget de competencia
    **/
    this.excluirCompetenciaEscopo = function(widget) {
        if(widget.currentRow.ESCOPOAGRUPADO != "S")
            General.deleteCrud(widget);
        else 
            ScreenService.showMessage("Operação bloqueada!<br><br>Não é possível deletar as informações de competência dos escopos que participam de agrupamentos.");
    };
    
    /**
     * Valida se o escopo participa de um agupamento e bloqueia a exclusao
     * dos seus parâmetros de faturamento.
     * 
     * @params{Object} widget - widget de parametros de faturamento do contrato
    **/
    this.excluirDadosGeraisParamsFatEscopo = function(widget) {
        if(widget.currentRow.ESCOPOAGRUPADO != "S")
            ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoDelete(widget, 'wdgParametrosFaturamento');
        else 
            ScreenService.showMessage("Operação bloqueada!<br><br>Não é possível deletar os parâmetros de faturamento dos escopos que participam de agrupamentos.");
    };
    
    this.adicionarServicoEscopo = function(wdgServicoEscopo) {
        var fieldCodigoServ = wdgServicoEscopo.getField("CDSERVORCA");
        var fieldNomeServ = wdgServicoEscopo.getField("NMSERVORCA");
        fieldCodigoServ.showOnForm = true;
        fieldCodigoServ.validations = {"required": true};
        fieldNomeServ.showOnForm = false;
        General.widgetNewRow(wdgServicoEscopo);
    };
    
    this.editarServicoEscopo = function(wdgServicoEscopo) {
        var fieldCodigoServ = wdgServicoEscopo.getField("CDSERVORCA");
        var fieldNomeServ = wdgServicoEscopo.getField("NMSERVORCA");
        fieldCodigoServ.showOnForm = false;
        fieldCodigoServ.validations = {};
        fieldNomeServ.showOnForm = true;
        General.widgetEdit(wdgServicoEscopo);
    };
    
    this.alterarVisibilidadeNaturezaOperacao = function(widget) {
        if (widget.currentRow.IDTIPONATUOPER == 'F') {
            widget.getField("IDNATUOPER").readOnly = false;
            widget.getField("IDNATUOPER").validations = {"required": true};
        }
        else {
            widget.getField("IDNATUOPER").readOnly = true;
            widget.getField("IDNATUOPER").validations = {};
            widget.getField("IDNATUOPER").value(null);
        }
    };
    
    this.editarParamFatEscopo = function(wdgCapaParametrosFaturamento) {
        ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoEdit(wdgCapaParametrosFaturamento, 'grid_escopo', 'wdgParametrosFaturamentoTabbed', 'wdgParametrosFaturamento');
        this.habilitarCamposDeCalculoDtSaida(wdgCapaParametrosFaturamento);
        this.alterarVisibilidadeNaturezaOperacao(wdgCapaParametrosFaturamento);
    };
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00003000_Escopo_Proposta_Controller', Sm00003000_Escopo_Proposta_Controller);
});

// FILE: ./assets/js/controllers/Sm00004000_Contrato_Controller-old.js


// FILE: ./assets/js/controllers/Sm00004000_Contrato_Controller.js
function Sm00004000_Contrato_Controller(General, ScreenService, templateManager, Sm00004000_Contrato_Service, Utilities, localStorageEngine, Operador_Repository, ZHPromise, UtilitiesPropostaContrato, Sm00001000_LancamentoProposta_Service, ValidacoesParametrosFaturamentoUtil, ZhFilterPreferences, eventAggregator) {
    var self = this;
    /* ---- FUNÇÕES DA PÁGINA CONTRATO --- */

    var opLogado = null;
    var nomeOperador = null;
    var idOperMaster = null;
    var idUtilizaHierOper = null;
    var idAlteraContAprv = null;
    var nrorgOperador = null;
    var nrParceiroOperador = null;
    var permitirAlteracao = null;
    var possuiAgrup = null;
    var visibilidadeAcoes = null; //{bool} o usuário esta dentro do perfil que pode editar contrato aprovado?

    var cdidiom_padrao = null;
    var nmidioma_padrao = null;
    var nrmoeda_padrao = null;
    var nmmoeda_padrao = null;

    var mudouMoeda = false;

    // Armazenamento das Widgets em variaveis globais
    var getWidgetGridContrato = function () {
        return templateManager.container.getWidget('gridContrato');
    };
    var getWidgetFaturamentoContrato = function () {
        return templateManager.container.getWidget('faturamentocontrato');
    };
    var getWidgetItemContrato = function () {
        return templateManager.container.getWidget('itemcontrato');
    };
    var getWidgetAdicItemContrato = function () {
        return templateManager.container.getWidget('adicitemcontrato');
    };
    var getWidgetContratoAnexos = function () {
        return templateManager.container.getWidget('contratoAnexos');
    };
    var getWidgetLocalAtendContrato = function () {
        return templateManager.container.getWidget('localAtendContrato');
    };
    var getWidgetFaturamentoContrato = function () {
        return templateManager.container.getWidget('faturamentocontrato');
    };
    var getWidgetNegociacaoDeValores = function () {
        return templateManager.container.getWidget('wdgNegociacaoValores');
    };

    var getWidgetGridLembreteContrato = function () {
        return templateManager.container.getWidget('lembretesContrato');
    };

    var getWidgetGridRateioCentCust = function () {
        return templateManager.container.getWidget('rateioCentCust');
    };

    var getActionEditWidgetProposta = function () {
        return getWidgetGridContrato().getAction('edit');
    };
    var getActionEditWidgetItemContrato = function () {
        return getWidgetItemContrato().getAction('edit');
    };
    var getActionAdicItemContrato = function () {
        return getWidgetItemContrato().getAction('add');
    };
    var getActionAddWidgetAdicItemContrato = function () {
        return getWidgetAdicItemContrato().getAction('add');
    };
    var getActionAddWidgetEditItemContrato = function () {
        return getWidgetAdicItemContrato().getAction('edit');
    };
    var getActionAdicLocalAtendimento = function () {
        return getWidgetLocalAtendContrato().getAction('add');
    };
    var getActionAdicFaturamento = function () {
        return getWidgetFaturamentoContrato().getAction('add_fat');
    };
    var getActionEditFaturamento = function () {
        return getWidgetFaturamentoContrato().getAction('edit_fat');
    };
    var getActionAddAnexo = function () {
        return getWidgetContratoAnexos().getAction('add');
    };
    // get field
    var getFieldModeloDeContratoWidgetGridContrato = function () {
        return getWidgetGridContrato().getField('NMMODCONTRATO');
    };

    var getDataSourceTipoDesconto = function (somenteP) {
        var data = [
            {
                "VALUE": "P",
                "NAME": "Na quantidade medida total (vendida + adicional)"
            }
        ];
        if (!somenteP) {
            data.push({
                "VALUE": "V",
                "NAME": "Apenas na quantidade vendida"
            });
        }

        return data;
    };

    var setDataSourceTipoDesconto = function (widget, somenteP) {
        var fieldTipoDesconto = widget.getField("IDTPDESCONTO");
        fieldTipoDesconto.dataSource.data = getDataSourceTipoDesconto(somenteP);
    };

    this.gridContratoOnEnterOnce = function (wdgContratroGrid) {
        this.buscaInformacoesOperadorLogado();
        Utilities.setRowFilterWidgetSemDefaultValue(wdgContratroGrid, ['IDATIVO', 'IDSITUACAOCONTRATO'], ['S', ['A', 'P', 'D']]).then(function () {
            eventAggregator.publish("abriuFiltroContrato");
        });
    };

    /**
     * Busca as informações do operador logado e seu
     * perfil de acesso para validações na tela
     **/
    this.buscaInformacoesOperadorLogado = function () {
        this.buscaOperadorLogado().then(function () {
            this.validarAlteracaoDeContratosAprovados();
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };

    //função era chamada no widgetOnEnter
    /*this.setAggregator = function() {
        ZhFilterPreferences.openFilterWidget(templateManager.container.getActiveWidget()).then(function(){
            Operador_Repository.getOperadorLogado().then(function(success) {
                eventAggregator.publish("abriuFiltroContrato");
            }.bind(this));
        });
    };*/

    /* Busca o operador logado a sessão atual 
     * @params{Object} widget de contato grid
     */
    this.buscaOperadorLogado = function () {
        var deferred = ZHPromise.defer();
        Operador_Repository.getOperadorLogado().then(function (success) {
            opLogado = success[0].CDOPERADOR;
            nomeOperador = success[0].NMOPERADOR;
            nrorgOperador = success[0].NRORG;
            nrParceiroOperador = success[0].NRPARCNEGOCIO;
            this.buscaInformacaoHierarquia();
            deferred.resolve(success);
        }.bind(this));

        return deferred.promise;
    };

    /**
     * Coloca na currentRow os parametrôs de novo código
     * para a inserção de novos registros
     * @params{Object} widget - widget que irá inserir um novo registro
     **/
    this.criarParametrosNewCode = function (widget) {
        if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
            widget.currentRow.__using_unit = widget.paramsNewCode.__using_unit || false;
            widget.currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
            widget.currentRow.__table_pk = widget.paramsNewCode.__table_pk;
            widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
            widget.currentRow.__cast_number = widget.paramsNewCode.__cast_number || false;
            if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
        }
    };
    /* habilita a widget para receber uma nova row
     * @param{Object} widget contrato
     */
    this.habilitaNewRow = function (widgetContrato) {
        Operador_Repository.getOperadorLogado().then(function (success) {
            var parametro = {
                "NRPARCNEGOCIO": success[0].NRPARCNEGOCIO,
                "NRORG": success[0].NRORG,
                "PROPOUCONT": "C"
            };

            General.callRestEngine('product#/validarPerfilAcesso', "Row", parametro).then(function (perfilusuario) {
                if (perfilusuario.dataset.validarperfil.PERFILACESSO) {
                    var modelosDeContrato = [];
                    var field = getFieldModeloDeContratoWidgetGridContrato();
                    perfilusuario.dataset.validarperfil.PERFILACESSO.forEach(function (modeloContrato) {
                        modelosDeContrato.push(modeloContrato.MODELO);
                    }.bind(this));
                    field.dataSourceFilter = [{
                        "name": "NRSEQMODCONTRA",
                        "operator": "NOT_IN",
                        "value": modelosDeContrato
                    }];
                }

                if (widgetContrato.getField("NRTIPOORCAMENTO")) widgetContrato.getField("NRTIPOORCAMENTO").isVisible = true;
                if (widgetContrato.getField("NMORCAMENTOMODELO")) widgetContrato.getField("NMORCAMENTOMODELO").isVisible = true;
                General.widgetNewRow(widgetContrato);
                widgetContrato.currentRow.CDOPERRESPONSAVEL = opLogado;
                widgetContrato.currentRow.NMOPERRESP = nomeOperador;

                var params = {
                    "OPERADOR": opLogado
                };

                General.callRestEngine("product#/fct_contrato_widget/buscaFilialHierarquia", "Row", params).then(function (response) {
                    if (response.dataset.DATA != 0) {
                        var filial = response.dataset.DATA[0].CDFILIAL;
                        var nmfilial = response.dataset.DATA[0].NMFILIAL;

                        widgetContrato.currentRow.CDFILIALORCA = filial;
                        widgetContrato.currentRow.NMFILIAL = nmfilial;
                    }
                },
                    function (error) {
                        console.log(error);
                        ScreenService.showMessage(error.data.error);
                    });
            });
        }.bind(this));
    };

    /* Verifica se o usuário tem hierarquia de operadores e caso tenha
     * busca sua posição
     */
    this.verificaHierarquia = function (widget) {
        if (idOperMaster === null && idUtilizaHierOper === null) {
            this.buscaInformacaoHierarquia();
        }

        if (opLogado) {
            widget.currentRow.CDOPERRESPONSAVEL = opLogado;
            widget.currentRow.CDNOVOOPERRESP = opLogado;
            widget.currentRow.NMOPERRESP = nomeOperador;
        }
        else {
            this.buscaOperadorLogado().then(function () {
                widget.currentRow.CDOPERRESPONSAVEL = opLogado;
                widget.currentRow.CDNOVOOPERRESP = opLogado;
                widget.currentRow.NMOPERRESP = nomeOperador;
            });
        }
    };

    /* busca os dados de hierarquia do usuário logado
     * @ params{Object} widget  
     */
    this.buscaInformacaoHierarquia = function () {
        General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", {}).then(function (success) {
            if (success.dataset.fct_parametrogeral && success.dataset.fct_parametrogeral.length > 0) {
                var paramsFilter = [{
                    name: "CDOPERADOR",
                    operator: "=",
                    value: opLogado
                }];

                idUtilizaHierOper = success.dataset.fct_parametrogeral[0].IDUTILIZAHIEROPER;
                General.callRestEngine("product#/crud/crm_hierarquiaoper", "FilterData", paramsFilter).then(function (success) {
                    if (success.dataset.crm_hierarquiaoper && success.dataset.crm_hierarquiaoper.length > 0)
                        idOperMaster = success.dataset.crm_hierarquiaoper[0].IDOPERMASTER;
                }.bind(this))
                    .catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
            }
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };


    /* Recarrega o field casa o dataSource tenha sido alterado 
     * @params{Object} field que será recarregada
     */
    this.recarregaDadosField = function (field) {
        var widgetAtiva = templateManager.container.getActiveWidget();
        var fieldParc = widgetAtiva.getField("NRPARCNEGOCIO");

        if (widgetAtiva.name === "filterPopup") {
            fieldParc.dataSourceFilter[1] = {
                name: "P_CDOPERADOR",
                operator: "=",
                value: opLogado
            };
        }

        Utilities.iniciarAutocomplete(field);
    };

    this.recarregaDadosFieldFat = function (field) {
        var widgetAtiva = templateManager.container.getActiveWidget();
        var fieldParc = widgetAtiva.getField("NRPARCNEGOCIOFAT");

        if (widgetAtiva.name === "filterPopup") {
            fieldParc.dataSourceFilter[1] = {
                name: "P_CDOPERADOR",
                operator: "=",
                value: opLogado
            };
        }

        Utilities.iniciarAutocomplete(field);
    };

    this.recarregaDadosparceiro = function() {
        /* Valida se a coluna IDGERACONTFOR da tabela fct_modelocontrato está como S ou N.
            Caso S, permite com que o parceiro de negócio seja do tipo Fornecedor.
        */

        var widget = getWidgetGridContrato();
        var idGeraContFor = widget.currentRow.IDGERACONTFOR;
        var fieldParc = widget.getField("NMPRINCIPALPARC");


        if (idGeraContFor == 'N') {
            fieldParc.dataSourceFilter[0] = {
                name: "CDTIPOPARCEIRO",
                operator: "IN",
                value:  ["CLIENTE", "PROSPECT"]
            };
        } else {
            fieldParc.dataSourceFilter[0] = {
                name: "CDTIPOPARCEIRO",
                operator: "IN",
                value: ["CLIENTE", "PROSPECT", "FORNECEDOR"]
            };
        }
        fieldParc.reload();

    };
    
    /**
     * Verifica se o código do contrato já está inserido
     * @params{Object} widget
     **/
    this.validaCodigo = function (widget) {
        if (widget.isValid()) {
            if (widget.currentRow.NRCONTRATOFCT && widget.currentRow.__is_new) {
                Utilities.validarCamposDuplicados(widget, ["NRSEQCONTRATO"], ["NRORG", "NRCONTRATOFCT"]).then(function (response) {
                    if (response) {
                        this.salvarContrato(widget);
                    }
                    else {
                        ScreenService.showMessage("Operação bloqueada.<br/>Já existe um contrato com o código informado.");
                    }
                }.bind(this));
            }
            else {
                this.salvarContrato(widget);
            }
        }
        else {
            ScreenService.showMessage("Preencha todos os campos corretamente!");
        }
    };

    /**
     * Salva o novo registro do Contrato e salva 
     * também os "Parâmetros para Faturamento" do Contrato.
     * 
     * @param {object} widget widget de 
     * Contrato para salvar o novo registro.
     */
    this.salvarContrato = function (widget) {
        var isNew = widget.currentRow.__is_new;
        var widgetTabbed = templateManager.container.getWidget("wdgTabbedPrincipal");

        widget.currentRow.__columns_to_return = ["NRSEQCONTRATO", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO", "NRCONTRATOFCT", "CDOPERULTATU", "DTULTATU", "NRORGULTATU"];
        if (!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
        else {
            widget.currentRow.NRTIPOORCAMENTO = widget.currentRow.NRTIPOORCAMENTO_CONT;

            var mudouParceiro = false;
            if (widget.backupRow && widget.backupRow.NRPARCNEGOCIOFAT !== widget.currentRow.NRPARCNEGOCIOFAT) {
                mudouParceiro = true;
            }

            var municipiofat = widget.currentRow.IDMUNIPRESTSERV;
            var mudouMunicipioFatura = false;
            if (widget.backupRow && widget.backupRow.IDMUNIPRESTSERV != municipiofat) {
                mudouMunicipioFatura = true;
            }

            var idagrupparc = widget.currentRow.IDAGRUPPARC;
            var mudouAgrupamentoParcelas = false;
            if (widget.backupRow && widget.backupRow.IDAGRUPPARC != idagrupparc) {
                mudouAgrupamentoParcelas = true;
            }
            var escopo = widget.currentRow.NRTIPOORCAMENTO;
            var mudouEscopoFatura = false;
            if (widget.backupRow && widget.backupRow.NRTIPOORCAMENTO != escopo) {
                mudouEscopoFatura = true;
            }

            var agrupaEscalaTrabHCM = widget.currentRow.IDAGRUPAESCALAHCM;
            var mudouagrupaEscalaTrabHCM = false;
            if (widget.backupRow && widget.backupRow.IDAGRUPAESCALAHCM != agrupaEscalaTrabHCM) {
                mudouagrupaEscalaTrabHCM = true;
            }

            if (!isNew) {
                this.validarAgrupamentoContratos(widget.currentRow.NRSEQCONTRATO).then(function (response) {
                    if (response.dataset && response.dataset.fct_relacagrupamentocont_widget && response.dataset.fct_relacagrupamentocont_widget.length > 0) {
                        var nrmoeda = widget.currentRow.NRMOEDA;
                        if (widget.backupRow && widget.backupRow.NRMOEDA != nrmoeda) {
                            ScreenService.showMessage(ScreenService.i18n('Operação bloqueada! <br><br>O valor do campo "Moeda" não pode ser alterado porque o contrato está ligado a um agrupamento de contratos.'));
                        }
                    }
                }.bind(this))
            }

            var params = {
                "CONTRATO": widget.currentRow.NRSEQCONTRATO,
                "USALOTEAGRUP": widget.currentRow.IDUTILIZALOTE
            };

            General.callRestEngine("product#/fct_contrato_widget/validaItensLoteFatura", "Row", params).then(function (response) {

            var resultado = response.dataset.validaItensLoteFatura.resultado;

            if (resultado != 'OK') {
                ScreenService.showMessage(ScreenService.i18n(resultado));
            }
            else { 
                this.validarAgrupamentoParcelasParaContratoAgrupado(widget.currentRow.NRSEQCONTRATO, idagrupparc, isNew).then(function (response) {
                    if (!response) {
                        ScreenService.showMessage(ScreenService.i18n('Operação bloqueada! <br><br>O valor do campo "Agrupamento de Parcelas" não pode ser diferente de "Agrupar tudo" porque o contrato está ligado a um agrupamento de contratos.'));
                    }
                    else {
    
                        var params = {
                            "CONTRATO": widget.currentRow.NRSEQCONTRATO,
                            "NRTIPOORCAMENTO": escopo,
                            "NRSEQMODCONTRA": widget.currentRow.NRSEQMODCONTRA,
                            "ALT_ESCOPO": mudouEscopoFatura,
                            "USALOTEAGRUP": widget.currentRow.IDUTILIZALOTE
                        };
    
                        General.callRestEngine("product#/fct_contrato_widget/validaAlteracaoEscopo", "Row", params).then(function (response) {
    
                            var resultado = response.dataset.validaAlteracaoEscopo.resultado;
    
                            if (resultado != 'OK') {
    
                                ScreenService.showMessage(ScreenService.i18n(resultado));
                            }
                            else {
                                this.criarParametrosNewCode(widget);
    
                                if (isNew) {
                                    if (Util.isEmptyOrBlank(widget.currentRow.DTINIVIGENCONT))
                                        widget.currentRow.DTINIVIGENCONT = widget.currentRow.DTINICIOCONTRATO;
    
                                    if (Util.isEmptyOrBlank(widget.currentRow.DTFINVIGENCONT))
                                        widget.currentRow.DTFINVIGENCONT = widget.currentRow.DTFIMCONTRATO;
    
                                    if (Util.isEmptyOrBlank(widget.currentRow.NRDIASPRORGCONT))
                                        widget.currentRow.NRDIASPRORGCONT = widget.currentRow.NRDIASPRAZOCONT;
                                }
    
                                widget.saveRow().then(function (response) {
                                    if (response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0) {
                                        widget.currentRow.NRSEQCONTRATO = response.dataset.columnsToReturn[0].NRSEQCONTRATO;
                                        widget.currentRow.NRCONTRATOFCT = response.dataset.columnsToReturn[0].NRCONTRATOFCT;
                                        widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                                        widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                                        widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                                        widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                                        widget.currentRow.DTULTATU = response.dataset.columnsToReturn[0].DTULTATU;
                                        widget.currentRow.CDOPERULTATU = response.dataset.columnsToReturn[0].CDOPERULTATU;
                                        widget.currentRow.NRORGULTATU = response.dataset.columnsToReturn[0].NRORGULTATU;
                                    }
                                    widgetTabbed.view();
                                    widget.view();
                                    if (isNew) {
                                        this.salvaInformacoesContrato(widget);
                                        this.insereIndicesPadroes(widget);
                                        this.inserirAssinaturasContr(widget);
                                        setTimeout(function () {
                                            this.voltarParaContratoGrid();
                                        }.bind(this), 200);
                                    }
                                    else {
                                        if (mudouParceiro) {
                                            var params = {
                                                "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                                                "NRPARCNEGOCIO": widget.currentRow.NRPARCNEGOCIOFAT
                                            };
                                            General.callRestEngine("product#/fct_contrato_widget/atualizarParceiroNegociacoesContrato", "Row", params)
                                                .then(null)
                                                .catch(function (error) {
                                                    Utilities.showErrorMessage(error);
                                                });
                                        }
                                        widget.currentRow.changed = false;
                                        delete widget.backupRow;
                                    }
    
                                    this.controlarAbasContrato(true, widget.name);
    
                                    if (mudouAgrupamentoParcelas) {
                                        var params = {
                                            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                                            "NRTIPOORCAMENTO": getWidgetGridContrato().currentRow.NRTIPOORCAMENTO,
                                            "IDAGRUPAESCALAHCM" : widget.currentRow.IDAGRUPAESCALAHCM
                                        };
                                        General.callRestEngine("product#/fct_itemcontrato_widget/regerarNegociacaoValores", "Row", params).then(function () {
                                            ScreenService.showMessage(ScreenService.i18n("Atenção! <br/><br/>Os registros da aba \"Negociação de Valores\" foram regerados devido a alteração do parâmetro \"Agrupamento de Parcelas\"."));
                                        })
                                            .catch(function (error) {
                                                Utilities.showErrorMessage(error);
                                            });
                                    }
    
                                    if (mudouEscopoFatura) {
                                        var params = {
                                            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                                            "NRTIPOORCAMENTO": escopo,
                                            "IDAGRUPAESCALAHCM" : widget.currentRow.IDAGRUPAESCALAHCM
                                        };
                                        General.callRestEngine("product#/fct_itemcontrato_widget/regerarNegociacaoValores", "Row", params).then(function () {
                                            ScreenService.showMessage(ScreenService.i18n("Atenção! <br/><br/>Os registros da aba \"Negociação de Valores\" foram regerados devido a alteração do parâmetro \"Escopo de Faturamento\"."));
                                        })
                                            .catch(function (error) {
                                                Utilities.showErrorMessage(error);
                                            });
                                    }
    
                                    else if (mudouMunicipioFatura) {
                                        var params = {
                                            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                                            "IDMUNIPRESTSERV": widget.currentRow.IDMUNIPRESTSERV
                                        };
                                        General.callRestEngine("product#/fct_itemcontrato_widget/atualizaMunicipioNegociacaoValores", "Row", params).then(function () {
                                            ScreenService.showMessage(ScreenService.i18n("Atenção! <br/><br/>Os registros da aba \"Negociação de Valores\" tiveram o campo \"Município para Faturamento\" atualizado devido a alteração do parâmetro \"Município de Prestação do Serviço\"."));
                                        })
                                            .catch(function (error) {
                                                Utilities.showErrorMessage(error);
                                            });
                                    }
                                    
                                    if (mudouagrupaEscalaTrabHCM) {

                                        var params = {
                                            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                                            "NRTIPOORCAMENTO": getWidgetGridContrato().currentRow.NRTIPOORCAMENTO,
                                            "IDAGRUPAESCALAHCM" : widget.currentRow.IDAGRUPAESCALAHCM
                                        };
                                        General.callRestEngine("product#/fct_itemcontrato_widget/regerarNegociacaoValores", "Row", params).then(function () {
                                            ScreenService.showMessage(ScreenService.i18n("Atenção! <br/><br/>Os registros da aba \"Negociação de Valores\" foram regerados devido a alteração do parâmetro \"Considera Escala de Trabalho no Agrupamento?\"."));
                                        })
                                            .catch(function (error) {
                                                Utilities.showErrorMessage(error);
                                            });
                                    }

                                }.bind(this))
                                    .catch(function (error) {
                                        Utilities.showErrorMessage(error);
                                    });
                            }
                        }.bind(this))
                            .catch(function (error) {
                                Utilities.showErrorMessage(error);
                            });
                    }
                }.bind(this));

            }
            }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * salva os demais dados do contrato
     * @params{Object} widget - widget de dados do contrato
     **/
    this.salvaInformacoesContrato = function (widget) {
        var seqContrato = widget.currentRow.NRSEQCONTRATO;
        var seqModContrato = widget.getField("NRSEQMODCONTRA").value();

        if (seqModContrato) {
            /* Salva parametrizações à partir do Modelo do Contrato */
            self.salvaDadosModContrato(widget, seqModContrato, seqContrato);
        }
        else {
            /* Salva os Parâmetro de Faturamento */
            var row = {
                "__using_unit": false,
                "__new_code_pk": "NRSEQFATUCONTR",
                "__table_pk": "NRSEQFATUCONTR",
                "__cast_number": true,
                "__is_new": true,
                "NRSEQFATUCONTR": "",
                "NRSEQCONTRATO": seqContrato,
                "CDPRODFATURA": "",
                "CDFILIALFATURA": "",
                "NRGRUPOFATURA": "",
                "NRPARCFATURA": "",
                "IDSOMAFATUCONT": "N",
                "IDSOMAITGRUPFAT": "N",
                "IDGERANFPARC": "N",
                "IDBLOQVRMEDIC": "N",
                "IDGERARATEIO": "N",
                "QTFRANQEVOLCONTR": "",
                "IDAPURAAPONTAMENTO": "N"
            };

            General.callRestEngine("product#/fct_paramfatcont_widget/save", "Row", row).then(null,
                function (error) {
                    console.log(error);
                    ScreenService.showMessage(error.data.error);
                });

            widget.reload();
        }
    }

    /**
     * Retorna o valor da filial para filtrar o campo "Centro de Custo"
     * 
     * @param {object} widget widget de 
     * Contrato para pegar o field "CDFILIALORCA".
     */

    this.filtraFilial = function (widget) {

        var cdfilial = widget.currentRow.CDFILIALORCA;

        widget.getField('NMCENTCUST').dataSourceFilter[0].value = cdfilial;

    };

    /**
     * Realiza o controle dos campos de vencimento de acordo com o "Tipo de Faturamento"
     * 
     * @param {object} widget widget de 
     * Contrato para pegar o field "IDFATURACONTR".
     */

    this.bloquearFieldsVencimento = function (widget) {
        var campoNomeModPgto = widget.getField("DSMODPGTO");
        var campoQtdParcelas = widget.getField("QTPARCELASFATU");
        var campoDiaVencParcelaFatu = widget.getField("NRDIAVENCPARCELASFATU");
        var tipofaturamento = widget.currentRow.IDFATURACONTR;

        if (tipofaturamento == "O") {
            campoNomeModPgto.readOnly = false;
            campoNomeModPgto.validations = {
                "required": true
            };

            campoQtdParcelas.readOnly = true;
            campoQtdParcelas.value(null);
            campoQtdParcelas.validations = {};

            campoDiaVencParcelaFatu.validations = {};
        }
        else {
            campoNomeModPgto.readOnly = true;
            campoNomeModPgto.value(null);
            campoNomeModPgto.validations = {};
            widget.currentRow.CDMODPGTO = null;

            if (tipofaturamento == "P") {
                campoQtdParcelas.readOnly = false;
                campoQtdParcelas.validations = {
                    "required": true
                };

                campoDiaVencParcelaFatu.validations = {
                    "required": true
                };
            }
            else {
                campoQtdParcelas.readOnly = true;
                campoQtdParcelas.value(null);
                campoQtdParcelas.validations = {};

                campoDiaVencParcelaFatu.validations = {};
            }
        }
    };

    /**
     * Realiza o cálculo do prazo do contrato
     * 
     * @param {object} widget widget de 
     * Itens para pegar os fields "IDTPDESCONTO" 
     * e "VRDESCONTO".
     */
    this.aprovaReprovaContrato = function (widget, idAprovaReprova) {
        if (idAprovaReprova == "A") {
            ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente aprovar o contrato?"), 'question', function (success) {
                this.insereAprovacao(widget);
            }.bind(this));
        }
        else {
            ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente reprovar o contrato?"), 'question', function (success) {
                this.insereReprovacao(widget);
            }.bind(this));
        }

        widget.reload();
    };

    /**
     * Insere os itens na tabela de agendamento pendente
     **/
    this.insereAgendamentoPendente = function (nrseqcontrato, nritemocontrato, quantidade, situacao) {
        var params = {
            "NRSEQCONTRATO": nrseqcontrato,
            "NRITEMOCONTRATO": nritemocontrato,
            "QUANTIDADE": quantidade,
            "IDSITUACAOCONTRATO": situacao,
            "ROTINA": "CONTRATO"
        };

        General.callRestEngine("product#/fct_agendrecursocont_widget/salvarAgendamentosPendentes", "Row", params).then(null).catch(function (error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    }

    /**
     * Realiza o cálculo do prazo do contrato
     * 
     * @param {object} widget widget de 
     * Itens para pegar os fields "IDTPDESCONTO" 
     * e "VRDESCONTO".
     */
    this.insereAprovacao = function (widget) {
        // Cria Registro Aprovação
        var nrSeqContrato = widget.currentRow.NRSEQCONTRATO;
        // Gera Parcelas do contrato
        Sm00004000_Contrato_Service.geraParcelasContrato(nrSeqContrato).then(function () {
            var dataAtual = moment().format("DD/MM/YYYY");
            var row = {
                "__using_unit": false,
                "__new_code_pk": "FCT_APROVACONTRATO",
                "__table_pk": "NRSEQAPROVCONT",
                "__new_code_length": 10,
                "__cast_number": true,
                "__is_new": true,
                "NRSEQAPROVCONT": "",
                "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                "DTAPROVCONTR": dataAtual,
                "IDAPROCCONTR": "A",
                "CDINTERLOCONTR": "",
                "NRCONTATOCONTR": "",
                "CDFILCONTCONTR": "",
                "DSAPROVCONTR": ""
            };

            var eventParams = {
                title: 'Renovação de Contrato: ' + widget.currentRow.NRCONTRATOFCT + " - " + widget.currentRow.DSREFERENCIACONT.substring(0,40),
                description: 'Hoje termina a validade contrato ' + widget.currentRow.NRCONTRATOFCT + ' - ' +  widget.currentRow.DSREFERENCIACONT + '  do cliente ' + widget.currentRow.NMSECUNDARIPARC,
                endDate : widget.currentRow.DTFINVIGENCONT
            };

            General.callRestEngine("product#/crud/fct_aprovacontrato_widget/save", "Row", row).then(function () {
                // updata contrato
                widget.currentRow.IDSITUACAOCONTRATO = "A";
                widget.currentRow.changed = true;
                var params;
                widget.saveRow().then(function () {
                    params = {
                        "NRSEQCONTRATO": nrSeqContrato,
                        "IDSITUACAOCONTRATO": "A"
                    };
                    General.callRestEngine("product#/fct_contrato_widget/atualizaDocumentoSituacao", "Row", params).then(function () {
                        this.gerarContratoFaturamento(widget.currentRow).then(function (response) {
                            ScreenService.showMessage("O contrato foi aprovado.").then(function(){
                                Utilities.SaveOnGoogleAgenda(eventParams)
                            });;                            
                            ScreenService.closePopup();
                            ScreenService.closeSwipe().then(function () {
                                widget.reload();
                            });
                        }.bind(this))
                            .catch(function (error) {
                                Utilities.showErrorMessage(error);
                            });
                    }.bind(this))
                        .catch(function (error) {
                            Utilities.showErrorMessage(error);
                        });

                    params = {
                        "NRSEQCONTRATO": nrSeqContrato
                    };
                    General.callRestEngine("product#/fct_contrato_widget/atualizaAgendamentosPend", "Row", params);

                }.bind(this))
                    .catch(function (error) {
                        console.error(error);
                        ScreenService.showMessage(error.data.error);
                    });
            }.bind(this),
                function (error) {
                    console.error(error);
                    if (error.data.error.indexOf("ORA-20001") !== -1)
                        ScreenService.showMessage("O contrato está sem parametrização de faturamento.");
                    else
                        ScreenService.showMessage(error.data.error);
                });
        }.bind(this))
            .catch(function (error) {
                console.error(error);
                if (error.data.error.indexOf("ORA-20001") !== -1) {
                    ScreenService.showMessage("O contrato está sem parametrização de faturamento.");
                }
                else if (error.data.error.indexOf("ORA-20002") !== -1) {
                    var msg = error.data.error;
                    var inicio = msg.indexOf("@@") + 2;
                    var msgInicial = msg.substr(inicio);
                    var fim = msgInicial.indexOf("@@");
                    var nomeLocalAtendimento = msgInicial.substring(0, fim);
                    ScreenService.showMessage("Operação Bloqueada.<br/><br/>O Local de Atendimento \"" + nomeLocalAtendimento + "\" não possui Centro de Custo parametrizado, impossibilitando a geração dos rateios.");
                }
                else {
                    ScreenService.showMessage(error.data.error);
                }
            });

    };

    this.validarMoedaItensContrato = function (nrseqcontrato) {

        var params = {
            "NRSEQCONTRATO": nrseqcontrato
        };

        return General.callRestEngine("product#/fct_contrato_widget/getQuantidadeItensMoedaDiferenteContrato", "Row", params);

    };

    this.aprovarContrato = function (widget, nrAgrupamento) {
        var eventParams = {
            title: 'Renovação de Contrato: ' + widget.currentRow.NRCONTRATOFCT + " - " + widget.currentRow.DSREFERENCIACONT.substring(0,40),
            description: 'Hoje termina a validade contrato ' + widget.currentRow.NRCONTRATOFCT + ' - ' +  widget.currentRow.DSREFERENCIACONT + '  do cliente ' + widget.currentRow.NMSECUNDARIPARC,
            endDate : widget.currentRow.DTFINVIGENCONT
        };
        
        if (widget.currentRow.CDTIPOPARCPRINCIPAL =='FORNECEDOR') {
            widget.currentRow.IDSITUACAOCONTRATO = "A";
            widget.currentRow.changed = true;
            widget.saveRow().then(function () {
                ScreenService.showMessage("O contrato foi aprovado.").then(function(){
                    Utilities.SaveOnGoogleAgenda(eventParams)
                });
                ScreenService.closePopup();
                this.voltarParaContratoGrid();
            }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
        }else {
            this.validarMoedaItensContrato(widget.currentRow.NRSEQCONTRATO).then(function (response) {
                if (response.dataset.retorno.qtd > 0) {
                    ScreenService.showMessage("Operação bloqueada! <br><br>Os itens do contrato foram inseridos com uma moeda diferente da moeda atual do contrato. Favor atualizar o valor dos itens.");
                } else {
                    var params = {
                        "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO
                    };
                    General.callRestEngine("product#/fct_contrato_widget/validarNegociacaoValores", "Row", params).then(function (response) {
                        if (response.dataset.validacao.resultado) {

                            var params = {
                                "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO
                            };

                            General.callRestEngine("product#/fct_contrato_widget/validarProdutoFaturamento", "Row", params).then(function (quantidadeNulos) {
                                if (!quantidadeNulos.dataset.validarProdutoFaturamento.resultado) { //se não tiver campos nulos
                                    General.callRestEngine("product#/fct_contrato_widget/validaAprovacaoCertsign", "Row", params).then(function (resultado) {
                                        if (resultado.dataset.validaAprovacaoCertsign.resultado != 'OK') {
                                            ScreenService.showMessage("Operação bloqueada!<br><br>Existem assinaturas eletrônicas ou digitais pendentes ou rejeitadas.");
                                        }
                                        else {

                                            var params = {
                                                "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO
                                            };

                                            General.callRestEngine("product#/fct_contrato_widget/validaRateioCentcust", "Row", params).then(function (resultado) {
                                            if (resultado.dataset.validaRateioCentcust.resultado != 'OK') {
                                                ScreenService.showMessage("Operação bloqueada!<br><br>" + resultado.dataset.validaRateioCentcust.resultado);
                                            } else {


                                                General.callRestEngine("product#/IntegracaoFM/integraLocalAtend", "Row", params).then(function (resultado) {
                                                    if (resultado.dataset.integraLocalAtend.resultado != 'OK') {
                                                        ScreenService.showMessage("Operação bloqueada!<br><br>Ocorreu uma falha na integração dos locais de atendimento do módulo 'SM' para o módulo 'FM'.<br><br> Motivo: " + resultado.dataset.integraLocalAtend.resultado);
                                                    }
                                                    else {
                                                        General.callRestEngine("product#/IntegracaoTecfood/integraServicosTecfood", "Row", params).then(function (resultado) {
                                                            if (resultado.dataset.integraServicosTecfood.resultado != 'OK') {
                                                                ScreenService.showMessage("Operação bloqueada!<br><br>" + resultado.dataset.integraServicosTecfood.resultado);
                                                            }
                                                            else {
                                                                var paramsHCM = {
                                                                    "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                                                                    "IDAPROVACAO": 'S'
                                                                };
                                                        
                                                                General.callRestEngine("product#/IntegracaoHcm", "Row", paramsHCM).then(function (resultado) {
                                                                    if (resultado.dataset.integracaoHcm.resultado != 'OK') {
                                                                        ScreenService.showMessage("Operação bloqueada!<br><br>" + resultado.dataset.integracaoHcm.resultado);
                                                                    } else {
                                                                        
                                                                        this.insereAprovacaoFatNovo(widget, nrAgrupamento);
            
                                                                        // this.enviarEmailPosAssinaturaDadosContrato(widget);
                                                                    }
                                                                }.bind(this))
                                                            }
                                                        }.bind(this))
                                                    }
                                                }.bind(this))
                                                .catch(function (error) {
                                                    Utilities.showErrorMessage(error);
                                                })
                                            
                                            }
                                        }.bind(this))      
                                        }
                                    }.bind(this))
                                    .catch(function (error) {
                                        Utilities.showErrorMessage(error);
                                    })
                                }
                                else { //se tiver itens de negociação sem produto para faturamento preenchido
                                    ScreenService.showMessage("Operação bloqueada.<br/><br/>Não é possível aprovar o contrato pois exitem Itens da Negociação sem o campo \"Produto para Faturamento\" preenchido. Favor preencher e tentar novamente.");
                                }
                            }.bind(this))
                        }
                        else {
                            ScreenService.showMessage('Operação bloqueada.<br/><br/>Os itens da aba "Negociação de Valores" que são de parcelamento não estão com as parcelas criadas corretamente: o valor total da soma das parcelas deve ser igual ao valor do item.');
                        }
                    }.bind(this))
                        .catch(function (error) {
                            Utilities.showErrorMessage(error);
                        })
                }
            }.bind(this));
        }
    };

    /**
     * Realiza o cálculo do prazo do contrato
     * 
     * @param {object} widget widget de 
     * Itens para pegar os fields "IDTPDESCONTO" 
     * e "VRDESCONTO".
     */
    this.insereAprovacaoFatNovo = function (widget, nrAgrupamento) {
        var eventParams = {
            title: 'Renovação de Contrato: ' + widget.currentRow.NRCONTRATOFCT + " - " + widget.currentRow.DSREFERENCIACONT.substring(0,40),
            description: 'Hoje termina a validade contrato ' + widget.currentRow.NRCONTRATOFCT + ' - ' +  widget.currentRow.DSREFERENCIACONT + '  do cliente ' + widget.currentRow.NMSECUNDARIPARC,
            endDate : widget.currentRow.DTFINVIGENCONT
        };

        var params = { "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO };
        General.callRestEngine("product#/fct_contrato_widget/validarParametrosFaturamentoCont", "Row", params).then(function () {
            if (widget.currentRow.IDPROPALIMENTICIA == 'S') {
                widget.currentRow.IDSITUACAOCONTRATO = "A";
                widget.currentRow.changed = true;
                widget.saveRow().then(function () {
                    ScreenService.showMessage("O contrato foi aprovado.").then(function(){
                        Utilities.SaveOnGoogleAgenda(eventParams)
                    });
                    ScreenService.closePopup();
                    this.voltarParaContratoGrid();
                }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
            }
            else {
                if (widget.currentRow.NRVERSAOCONTRATO > 1 && widget.currentRow.NRSEQCONTRATOPAI) {
                    this.verificarSeContratoPossuiNegociacao(widget.currentRow.NRSEQCONTRATO, widget.currentRow.NRSEQCONTRATOPAI).then(function (idReplicarNegociacao) {
                        this.aprovaContratoAux(widget, idReplicarNegociacao, nrAgrupamento);
                    }.bind(this));
                }
                else {
                    
                    /* 1 - Verifica se já existe um agrupamento para o cliente e escopo de faturamento do contrato 
                    2 - Pergunta se o usuário deseja incluir o contrato no agrupamento.
                    2.1 - Se houver apenas um agrupamento, vincula automaticamente.
                    2.2 - Se houver mais de um agrupamento, abre a popup para o usuário escolher o agrupamento que deseja vincular. */
                    
                    this.aprovaContratoAux(widget, "N", nrAgrupamento);
                }
            }
            
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };

    this.aprovaContratoAux = function (widget, idReplicarNegociacao, nrAgrupamento) {
        var widgetContrato = getWidgetGridContrato();
        // Cria Registro Aprovação
        var widGoogle = widget.currentRow;
        var nrSeqContrato = widget.currentRow.NRSEQCONTRATO;
        var params = {
            "NRSEQCONTRATO": nrSeqContrato,
            "NRMOEDA": widget.currentRow.NRMOEDA
        };
        General.callRestEngine("product#/fct_contrato_widget/salvarDadosMedidaContrato", "Row", params).then(function () {
            var params = {
                "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                "IDUTILIZACOMP": widgetContrato.currentRow.IDUTILIZACOMP,
                "IDREPLICARNEGOCIACAO": idReplicarNegociacao,
                "NRSEQCONTRATOPAI": widget.currentRow.NRSEQCONTRATOPAI,
                "NRPARCNEGOCIO": widget.currentRow.NRPARCNEGOCIO,
                "IDAPROVACAO": "S"
            };

            var eventParams = {
                title: 'Renovação de Contrato: ' + widget.currentRow.NRCONTRATOFCT + " - " + widget.currentRow.DSREFERENCIACONT.substring(0,40),
                description: 'Hoje termina a validade contrato ' + widget.currentRow.NRCONTRATOFCT + ' - ' +  widget.currentRow.DSREFERENCIACONT + '  do cliente ' + widget.currentRow.NMSECUNDARIPARC,
                endDate : widget.currentRow.DTFINVIGENCONT
            };

            // Gera Parcelas do contrato
            var timeout = 8 * 60000; //8 min
            Utilities.callRestEngineWithTimeout('product#/fct_contrato_widget/geraParcelasContratoNovoFat', "Row", params, timeout).then(function () {
                var dataAtual = moment().format("DD/MM/YYYY");
                var row = {
                    "__using_unit": false,
                    "__new_code_pk": "FCT_APROVACONTRATO",
                    "__table_pk": "NRSEQAPROVCONT",
                    "__new_code_length": 10,
                    "__cast_number": true,
                    "__is_new": true,
                    "NRSEQAPROVCONT": "",
                    "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                    "DTAPROVCONTR": dataAtual,
                    "IDAPROCCONTR": "A",
                    "CDINTERLOCONTR": "",
                    "NRCONTATOCONTR": "",
                    "CDFILCONTCONTR": "",
                    "DSAPROVCONTR": ""
                };

                General.callRestEngine("product#/crud/fct_aprovacontrato_widget/save", "Row", row).then(function () {
                    // updata contrato
                    widget.currentRow.IDSITUACAOCONTRATO = "A";
                    widget.currentRow.changed = true;
                    widget.saveRow().then(function () {
                        ScreenService.showMessage("O contrato foi aprovado.").then(function(){
                            Utilities.SaveOnGoogleAgenda(eventParams)
                        });
                        ScreenService.closePopup();
                        this.voltarParaContratoGrid();
                        var params = { "NRSEQCONTRATO": nrSeqContrato };
                        General.callRestEngine("product#/fct_contrato_widget/atualizaAgendamentosPend", "Row", params);

                        if (nrAgrupamento != null){
                            var params = { 
                                "NRSEQCONTRATO": nrSeqContrato, 
                                "NRAGRUPAMENTO": nrAgrupamento
                            };

                            General.callRestEngine("product#/fct_contrato_widget/vinculaContratoNoAgrupamento", "Row", params);

                            var params = { "NRSEQCONTRATO": nrSeqContrato};
                            General.callRestEngine('product#/fct_contrato_widget/geraParcelasContratoNovoFat', "Row", params);
                        }

                        this.criaAgendaGoogle(widGoogle, false);

                    }.bind(this))
                        .catch(function (error) {
                            Utilities.showErrorMessage(error);
                        });
                }.bind(this),
                    function (error) {
                        console.error(error);
                        if (error.data.error.indexOf("ORA-20001") !== -1)
                            ScreenService.showMessage("O contrato está sem parametrização de faturamento.");
                        else
                            ScreenService.showMessage(error.data.error);
                    });
            }.bind(this))
                .catch(function (error) {
                    console.error(error);
                    var msg = error.data.error;
                    var nomeItem = null;

                    if (msg.indexOf("@@") !== -1) {
                        var inicio = msg.indexOf("@@") + 2;
                        var msgInicial = msg.substr(inicio);
                        var fim = msgInicial.indexOf("@@");
                        nomeItem = msgInicial.substring(0, fim);
                    }

                    if (msg.indexOf("CONTRATO_SEM_PROPOSTA") !== -1) {
                        ScreenService.showMessage("Operação bloqueada. <br/>O contrato não foi gerado a partir de uma proposta.");
                    }
                    else if (msg.indexOf("SEM_TIPO_FATURAMENTO") !== -1) {
                        ScreenService.showMessage('Operação bloqueada. <br/>Não foi encontrado o parâmetro "Tipo de Faturamento" para o item "' + nomeItem + '".');
                    }
                    else if (msg.indexOf("SEM_GRUPO_FATURAMENTO") !== -1) {
                        ScreenService.showMessage('Operação bloqueada. <br/>Não foi encontrado o parâmetro "Grupo de Faturamento" para o item "' + nomeItem + '".');
                    }
                    else if (msg.indexOf("SEM_PRODUTO_FATURAMENTO") !== -1) {
                        ScreenService.showMessage('Operação bloqueada. <br/>Não foi encontrado o parâmetro "Produto para Faturamento" para o item "' + nomeItem + '".');
                    }
                    else if (msg.indexOf("SEM_FILIAL_FATURAMENTO") !== -1) {
                        ScreenService.showMessage('Operação bloqueada. <br/>Não foi encontrado o parâmetro "Unidade para Faturamento" para o item "' + nomeItem + '".');
                    }
                    else {
                        ScreenService.showMessage(msg);
                    }
                });
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };

    /**
     * Realiza o cálculo do prazo do contrato
     * 
     * @param {object} widget widget de 
     * Itens para pegar os fields "IDTPDESCONTO" 
     * e "VRDESCONTO".
     */
    this.insereReprovacao = function (widget) {

        var nrSeqContrato = widget.currentRow.NRSEQCONTRATO;
        var dataAtual = moment().format("DD/MM/YYYY");

        var row = {
            "__using_unit": false,
            "__new_code_pk": "FCT_APROVACONTRATO",
            "__table_pk": "NRSEQAPROVCONT",
            "__new_code_length": 10,
            "__cast_number": true,
            "__is_new": true,
            "NRSEQAPROVCONT": "",
            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
            "DTAPROVCONTR": dataAtual,
            "IDAPROCCONTR": "R",
            "CDINTERLOCONTR": "",
            "NRCONTATOCONTR": "",
            "CDFILCONTCONTR": "",
            "DSAPROVCONTR": ""
        };

        General.callRestEngine('product#/crud/fct_aprovacontrato_widget/save', "Row", row).then(function () {
            /* updata contrato */
            widget.currentRow.IDSITUACAOCONTRATO = 'R';
            widget.currentRow.changed = true;
            widget.saveRow().then(function () {
                var params = {
                    "NRSEQCONTRATO": nrSeqContrato,
                    "IDSITUACAOCONTRATO": "R"
                };
                General.callRestEngine("product#/fct_contrato_widget/atualizaDocumentoSituacao", "Row", params).then(function () {
                    /* Cancela Parcelas do contrato */
                    Sm00004000_Contrato_Service.cancelaParcelasContrato(nrSeqContrato).then(function () {
                        var wdgTabbed = templateManager.container.getWidget("wdgTabbedPrincipal");
                        ScreenService.closePopup();
                        ScreenService.showMessage("O contrato foi reprovado.");
                        if (wdgTabbed)
                            this.voltarParaContratoGrid();
                        else
                            ScreenService.closeSwipe().then(function () {
                                widget.reload();
                            });
                    }.bind(this))
                        .catch(function (error) {
                            Utilities.showErrorMessage(error);
                        });
                }.bind(this));
            }.bind(this));
        }.bind(this),
            function (error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });
    };

    this.calcularPrazoVigente = function (wdgContrato) {
        var dtini = wdgContrato.currentRow.DTINIVIGENCONT;
        var dtfin = wdgContrato.currentRow.DTFINVIGENCONT;
        var prazo = Utilities.calculaDiferencaDatasEmDias(dtini, dtfin);

        if (prazo < 0) { //tratamento para não permitir data final menor que inicial
            ScreenService.showMessage('A Data Final não pode ser menor que a inicial!');
            wdgContrato.currentRow.DTFINVIGENCONT = dtini;
            prazo = 0;
        }

        wdgContrato.currentRow.NRDIASPRORGCONT = prazo ? prazo + 1 : 1;
    };

    this.calcularPrazoInicial = function (wdgContrato) {
        var dtini = wdgContrato.currentRow.DTINICIOCONTRATO;
        var dtfin = wdgContrato.currentRow.DTFIMCONTRATO;
        var prazo = Utilities.calculaDiferencaDatasEmDias(dtini, dtfin);

        wdgContrato.currentRow.NRDIASPRAZOCONT = prazo ? prazo + 1 : 1;
    };

    this.verificarSeContratoPossuiNegociacao = function (nrseqcontrato, nrseqcontratopai) {
        var deferred = ZHPromise.defer();

        var params = {
            "NRSEQCONTRATO": nrseqcontrato,
            "NRSEQCONTRATOPAI": nrseqcontratopai
        };
        General.callRestEngine("product#/fct_contrato_widget/verificarSeContratoPossuiNegociacao", "Row", params).then(function (response) {
            if (response.dataset.retorno.possuiNegociacao) {
                ScreenService.confirmMessage("Atenção! <br/><br/>O contrato anterior possuía negociações cadastradas para seus itens. Deseja manter essas negociações para esta nova versão do contrato?", "question", function () {
                    deferred.resolve('S');
                },
                    function () {
                        deferred.resolve('N');
                    });
            }
            else {
                deferred.resolve('N');
            }
        })
            .catch(function (error) {
                deferred.reject(error);
            });

        return deferred.promise;
    };

    /**
     * A função "copiaVersionaContrato" executa a procedure
     * "FCT_COPIA_VERSIONA_CONTRATO".
     * @params{String} adicionaAgrupamento - define se o novo contrato será ou não adicionado ao agrupamento de contratos
     */

    this.copiaVersionaContrato = function (widget, idCopiaVersiona, adicionaAgrupamento, idParcelaConsiderada, dataRenegociacao, dtFimContrato) {
        if (adicionaAgrupamento === undefined) {
            adicionaAgrupamento = "N"
        }

        var nrSeqContrato = widget.currentRow.NRSEQCONTRATO;
        var wdgTabbed = templateManager.container.getWidget("wdgTabbedPrincipal");
        if (idCopiaVersiona == 'V') { // renegociacao do Contrato

            Sm00004000_Contrato_Service.copiaVersionaContrato(nrSeqContrato, idCopiaVersiona, adicionaAgrupamento, idParcelaConsiderada, "N", dataRenegociacao, dtFimContrato).then(function () {

                var params = {
                    "NRSEQCONTRATO": nrSeqContrato
                };
                General.callRestEngine("product#/fct_contrato_widget/verificarParcelasContReneg", "Row", params).then(function (response) {

                    var resultado = response.dataset.verificarParcelasContReneg.resultado;

                    if (resultado == 'OK') {

                        ScreenService.closePopup();
                        if (wdgTabbed) {
                            this.voltarParaContratoGrid();
                        }
                        else {
                            ScreenService.closeSwipe();
                            widget.reload();
                        }

                        ScreenService.showMessage("Renegociação do contrato realizada com sucesso.");
                    }
                    else {

                        ScreenService.closePopup();
                        if (wdgTabbed) {
                            this.voltarParaContratoGrid();
                        }
                        else {
                            ScreenService.closeSwipe();
                            widget.reload();
                        }

                        ScreenService.showMessage(resultado);
                    }

                }.bind(this))
                    .catch(function (error) {
                        Utilities.showErrorMessage(error);
                    }.bind(this));


            }.bind(this),
                function (error) {
                    console.log(error);
                    ScreenService.showMessage(error.data.error);
                });
        }
        else {
            ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente copiar o contrato?"), 'question', function (success) {
                ScreenService.closePopup();
                Sm00004000_Contrato_Service.copiaVersionaContrato(nrSeqContrato, idCopiaVersiona, adicionaAgrupamento).then(function () {
                    ScreenService.showMessage("O contrato foi copiado.");
                    if (wdgTabbed) {
                        this.voltarParaContratoGrid();
                    }
                    else {
                        ScreenService.closeSwipe();
                        widget.reload();
                    }
                }.bind(this), function (error) {
                    console.log(error);
                    ScreenService.showMessage(error.data.error);
                });
            }.bind(this));
        }
    };


    this.validaIdDocDocx = function (widget) {
        var deferred = ZHPromise.defer();
        var nrtiporescisao = widget.currentRow.NRTIPORESCISAO;

        /* Encerramento singular */
        var wdgContrato = templateManager.container.getWidget("gridContrato");

        /* Encerramento em massa */
        var wdgContratroGrid = templateManager.container.getWidget("wdgContratroGrid");
        checkedRows = wdgContratroGrid.getCheckedRows();

        var params = {
            "NRTIPORESCISAO": nrtiporescisao
        };

        General.callRestEngine("product#/fct_contrato_widget/getParametroExtensaoDocEncerra", "Row", params).then(function (response) {

            if (response.dataset.IDPERMITEDOCX[0].NRSEQMODDOCFCT != null) {

                if (response.dataset.IDPERMITEDOCX[0].IDPERMITEDOCX == 'S') {

                    ScreenService.showCustomDialog("O documento de rescisão do contrato pode ser gerado como PDF ou como DOCX.<br>Como deseja gerá-lo?", 'CONFIRMATION', null, [{
                        "label": "Gerar PDF",
                        "code": function () {
                            if (widget.currentRow.IDRESCICAOEMMASSA == 'N') {
                                this.encerraContrato(wdgContrato, 'P', widget).then(function () {
                                    deferred.resolve();
                                })
                                    .catch(function (error) {
                                        deferred.reject(error);
                                    });
                            }
                            else {
                                this.encerraContratoEmMassa(widget, checkedRows, 'P').then(function () {
                                    deferred.resolve();
                                })
                                    .catch(function (error) {
                                        deferred.reject(error);
                                    });
                            }
                        }.bind(this)
                    }, {
                        "label": "Gerar DOCX",
                        "code": function () {
                            if (widget.currentRow.IDRESCICAOEMMASSA == 'N') {
                                this.encerraContrato(wdgContrato, 'D', widget).then(function () {
                                    deferred.resolve();
                                })
                                    .catch(function (error) {
                                        deferred.reject(error);
                                    });
                            }
                            else {
                                this.encerraContratoEmMassa(widget, checkedRows, 'D').then(function () {
                                    deferred.resolve();
                                })
                                    .catch(function (error) {
                                        deferred.reject(error);
                                    });
                            }
                        }.bind(this)
                    }
                    ]);
                }
                else {
                    if (widget.currentRow.IDRESCICAOEMMASSA == 'N') {
                        this.encerraContrato(wdgContrato, 'P', widget).then(function () {
                            deferred.resolve();
                        })
                            .catch(function (error) {
                                deferred.reject(error);
                            });
                    }
                    else {
                        this.encerraContratoEmMassa(widget, checkedRows, 'P').then(function () {
                            deferred.resolve();
                        })
                            .catch(function (error) {
                                deferred.reject(error);
                            });
                    }
                }
            }
            else {

                if (widget.currentRow.IDRESCICAOEMMASSA == 'N') {
                    this.encerraContrato(wdgContrato, null, widget).then(function () {
                        deferred.resolve();
                    })
                        .catch(function (error) {
                            deferred.reject(error);
                        });
                }
                else {

                    this.encerraContratoEmMassa(widget, checkedRows, null).then(function () {
                        deferred.resolve();
                    })
                        .catch(function (error) {
                            deferred.reject(error);
                        });
                }
            }
        }.bind(this))
            .catch(function (error) {
                deferred.reject(error);
            }.bind(this))

        return deferred.promise;
    }

    /**
     * A função "encerraContrato" executa a procedure
     * "FCT_ENCERRA_CONTRATO".
     */
    this.encerraContrato = function (widget, idDocX, wdgPopupEncerra) {
        var deferred = ZHPromise.defer();
        var situacao = widget.currentRow.IDSITUACAOCONTRATO;
        ScreenService.closePopup();

        if (situacao == 'E') {
            ScreenService.showMessage("O contrato já está encerrado.");
            deferred.resolve();
        }
        else {
            var nrparcnegocio = widget.currentRow.NRPARCNEGOCIO;
            var nrSeqContrato = widget.currentRow.NRSEQCONTRATO;
            var idSituacao = widget.currentRow.IDSITUACAOCONTRATO;
            var idtpexclusaoparcela = wdgPopupEncerra.currentRow.IDTPEXCLUSAOPARCELA;
            var dtrecisao = wdgPopupEncerra.currentRow.DTRECISAOCONTR;
            var nrtiporescisao = wdgPopupEncerra.currentRow.NRTIPORESCISAO;

            var dtpedrescisaocontr = wdgPopupEncerra.currentRow.DTPEDRESCISAOCONTR;
            var nrmotivorescisao = wdgPopupEncerra.currentRow.NRMOTIVORESCISAO;
            var dsjustificaencerra = wdgPopupEncerra.currentRow.DSJUSTIFICAENCERRA;
            var vrpendenciafin = wdgPopupEncerra.currentRow.VRPENDENCIAFIN;

            Sm00004000_Contrato_Service.encerraContrato(nrSeqContrato, idSituacao, dtrecisao, idtpexclusaoparcela, nrtiporescisao, idDocX, dtpedrescisaocontr, nrmotivorescisao, dsjustificaencerra, vrpendenciafin).then(function () {
                var params = {
                    "NRSEQCONTRATO": nrSeqContrato,
                    "IDSITUACAOCONTRATO": "E"
                };

                General.callRestEngine("product#/fct_contrato_widget/atualizaDocumentoSituacao", "Row", params).then(function () {

                    if (idDocX != null) {
                        var params = {
                            "CONTRATOS": null,
                            "NRPARCNEGOCIO": nrparcnegocio,
                            "NRSEQCONTRATO": nrSeqContrato,
                            "NRTIPORESCISAO": nrtiporescisao,
                            "IDDOCX": idDocX
                        };

                        var timeout = 5 * 60000; //5 min
                        Utilities.callRestEngineWithTimeout("product#/fct_contrato_widget/gerarDocumentoContratoRescisao", "Row", params, timeout).then(function (success) {
                            var links = success.dataset.documentos.links;
                            links.forEach(function (link) {
                                if (link)
                                    window.open(window.backendIndexUrl + link);
                            });

                            ScreenService.showMessage("O contrato foi encerrado.");
                            ScreenService.closePopup();
                        })
                            .catch(function (error) {
                                console.error(error);
                                var msg = "Timeout";
                                if (error.status !== 0 && error.data)
                                    msg = error.data.error;

                                ScreenService.showMessage(msg);
                            });
                    }
                    else {
                        ScreenService.showMessage("O contrato foi encerrado.");
                        ScreenService.closePopup();
                    }

                    deferred.resolve();

                }.bind(this))
                    .catch(function (error) {
                        deferred.reject(error);
                    });
            }.bind(this))
                .catch(function (error) {
                    deferred.reject(error);
                })
        }

        return deferred.promise;
    };

    this.validaPerfilNovoItem = function () {
        var wdgContrato = getWidgetGridContrato();
        var newitem = getActionAdicItemContrato();
        if (wdgContrato.currentRow.IDSITUACAOCONTRATO == "A") {
            newitem.isVisible = visibilidadeAcoes;
        }
    };
    /**
     * Valida qual das telas de contrato acionou o evento AfterMoveRow
     * e chama a função das validações da tela
     * @params{Object} widget - widget contrato grid
     **/
    this.validarWidgetContratoGrid = function (widget, nameAction) {
        setLocalVar("SEQ_CONTRATO", widget.currentRow.NRSEQCONTRATO);
        setLocalVar("NRORG_CONTRATO", widget.currentRow.NRORG);

        var wdgTabbedPrincipal = templateManager.container.getWidget('wdgTabbedPrincipal');
        var gridContrato = templateManager.container.getWidget('gridContrato');

        ScreenService.openSwipe(wdgTabbedPrincipal).then(function () {
            wdgTabbedPrincipal.openTab("wdgTabbedContrato");
            gridContrato.list();
            gridContrato.reload().then(function () {
                gridContrato.view();
                gridContrato.activate();
                wdgTabbedPrincipal.view();
                gridContrato.currentRow = gridContrato.dataSource.data[0];
                this.controlarAbasContrato(true, gridContrato.name);
                this.gridContratoAfterMoveRow(gridContrato);
            }.bind(this));
        }.bind(this));
    };

    /**
     * Habilita a inserção de um novo registro na widget de tabbed do novo faturamento
     */
    this.habilitaNewRowTabbed = function () {
        if (!opLogado) {
            this.buscaOperadorLogado().then(function () {
                this.habilitaNewRowTabbed();
            }.bind(this));
        }
        else {
            var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
            var gridContrato = templateManager.container.getWidget("gridContrato");

            var parametro = {
                "NRPARCNEGOCIO": nrParceiroOperador,
                "NRORG": nrorgOperador,
                "PROPOUCONT": "C"
            };

            General.callRestEngine('product#/validarPerfilAcesso', "Row", parametro).then(function (perfilusuario) {
                if (perfilusuario.dataset.validarperfil.PERFILACESSO) {
                    var modelosDeContrato = [];
                    var field = getFieldModeloDeContratoWidgetGridContrato();

                    perfilusuario.dataset.validarperfil.PERFILACESSO.forEach(function (modeloContrato) {
                        modelosDeContrato.push(modeloContrato.MODELO);
                    });

                    field.dataSourceFilter = [{
                        "name": "NRSEQMODCONTRA",
                        "operator": "NOT_IN",
                        "value": modelosDeContrato
                    }];
                }

                setLocalVar("SEQ_CONTRATO", "");
                setLocalVar("NRORG_CONTRATO", "");

                return ScreenService.openSwipe(wdgTabbedPrincipal);
            }).then(function () {
                gridContrato.template = "widget/form.html";
                gridContrato.newRow();

                this.controlaPaginas(wdgTabbedPrincipal);
                this.controlarAbasContrato(false, gridContrato.name);
                wdgTabbedPrincipal.edit();

                if (gridContrato.getField("NRTIPOORCAMENTO")) gridContrato.getField("NRTIPOORCAMENTO").isVisible = true;
                if (gridContrato.getField("NMORCAMENTOMODELO")) gridContrato.getField("NMORCAMENTOMODELO").isVisible = true;

                gridContrato.getField("NMTIPOORCAMENTO_CONT").readOnly = false;
                gridContrato.getField("NMTIPOORCAMENTO_CONT").validations = { "required": true };

                gridContrato.getAction("SAVE").isVisible = true;
                gridContrato.getAction("Avancar").isVisible = false;

                this.habilitaCampoVingenciaAtual(gridContrato, true);
                this.verificaHierarquia(gridContrato);
                this.validaIdCertsign();

                gridContrato.getField("IDAGRUPPARC").readOnly = false;

                var params = { "OPERADOR": opLogado };
                return General.callRestEngine("product#/fct_contrato_widget/buscaFilialHierarquia", "Row", params);
            }.bind(this)).then(function (response) {
                if (response.dataset.DATA > 0) {
                    gridContrato.currentRow.CDFILIALORCA = response.dataset.DATA[0].CDFILIAL;
                    gridContrato.currentRow.NMFILIAL = response.dataset.DATA[0].NMFILIAL;
                }
                gridContrato.IDMUNIPRESTSERV = "";

                //setar idioma e moeda padrao
                return General.callRestEngine("product#/configuracao_regional/getIdiomaMoedaPadrao", "Row", {});
            }).then(function (response) {
                if (response.dataset.retorno.CDIDIOM) {
                    cdidiom_padrao = gridContrato.currentRow.CDIDIOM = response.dataset.retorno.CDIDIOM;
                    nmidioma_padrao = gridContrato.currentRow.NMIDIOMA = response.dataset.retorno.NMIDIOMA;
                    nrmoeda_padrao = gridContrato.currentRow.NRMOEDA = response.dataset.retorno.NRMOEDA;
                    nmmoeda_padrao = gridContrato.currentRow.NMMOEDA = response.dataset.retorno.NMMOEDA;
                }
            }).catch(function (error) {
                Utilities.showErrorMessage(error);
            });
        }
    };

    /**
     * Efetua as validações necessárias ao abrir o view do contrato
     * @params{Object} widget - widget de contrato grid
     **/
    this.gridContratoAfterMoveRow = function (wdgContrato) {
        if (wdgContrato.currentRow.IDSITUACAOCONTRATO != "A") {
            this.controlaPaginas(wdgContrato);
            this.controlaVisibilidade(wdgContrato);
            this.controlaCodContrato(wdgContrato);
        }
        else {
            var edit = getActionEditWidgetProposta();
            if (idOperMaster === null && idUtilizaHierOper === null)
                this.buscaInformacaoHierarquia();

            var permiteEdicaoAlteracao = visibilidadeAcoes && idAlteraContAprv == 'S';
            var nrseqcontratopai = wdgContrato.currentRow.NRSEQCONTRATOPAI != null;
            var idgeradopor = wdgContrato.currentRow.IDGERADOPOR == 'CONTRATO';

            if (visibilidadeAcoes != null || idAlteraContAprv != null) {
                edit.isVisible = (permiteEdicaoAlteracao && (nrseqcontratopai || idgeradopor));
                this.controlaPaginas(wdgContrato);
                this.controlaVisibilidade(wdgContrato);
                this.controlaCodContrato(wdgContrato);
            }
            else {
                this.validarAlteracaoDeContratosAprovados().then(function () {
                    permiteEdicaoAlteracao = visibilidadeAcoes && idAlteraContAprv == 'S';
                    edit.isVisible = (permiteEdicaoAlteracao && (nrseqcontratopai || idgeradopor));

                    this.controlaPaginas(wdgContrato);
                    this.controlaVisibilidade(wdgContrato);
                    this.controlaCodContrato(wdgContrato);
                });
            }
        }
    };
    /**
     * Altera a visibilidade das widgets de elementos, itens de composição,
     * tipo de medida e nome contratual
     * @params{Object} widget - widget gridContrato
     **/
    this.controlaPaginas = function (widget) {
        var wdgElementosProposta = templateManager.container.getWidget("wdgElementosProposta");
        var wdgTipoMedidaPropostaEditando = templateManager.container.getWidget("wdgTipoMedidaPropostaEditando");
        var wdgItensPropostaTree = templateManager.container.getWidget("wdgItensPropostaTree");
        var wdgNmContrat = templateManager.container.getWidget("wdgNomeContratual");
        var wdgItFaturaRateio = templateManager.container.getWidget("RateioItFaturamento");
        var wdgFaturaRateio = templateManager.container.getWidget("Rateiofaturamento");
        var itemcontrato = templateManager.container.getWidget("itemcontrato");
        var nrseqorcamento = widget.currentRow.NRSEQORCAMENTO;
        var nrorg = widget.currentRow.NRORG;

        if (nrseqorcamento != null) {
            var params = {
                "NRSEQORCAMENTO": nrseqorcamento,
                "NRORG": nrorg
            };

            General.callRestEngine('product#/fct_contrato_widget/validaCompServico', "Row", params).then(function (response) {
                var compServico = response.dataset.COMPSERVICO[0].EXISTE;
                if (compServico === 'S') {
                    setLocalVar("PROPOSTA_NRSEQORCAMENTO", nrseqorcamento);
                    setLocalVar("PROPOSTA_NRORG", nrorg);

                    wdgElementosProposta.isVisible = true;
                    wdgItensPropostaTree.isVisible = true;
                    wdgElementosProposta.enabled = true;
                    wdgItensPropostaTree.enabled = true;
                    itemcontrato.isVisible = false;
                    wdgTipoMedidaPropostaEditando.enabled = true;
                    wdgItFaturaRateio.isVisible = false;
                    wdgFaturaRateio.isVisible = false;

                    if (getWidgetGridContrato().currentRow.IDUTILIZAVARPRECO === "S")
                        this.validarVisibilidadeWidgetTpMedida(wdgTipoMedidaPropostaEditando, widget.currentRow.NRTIPOORCAMENTO, widget.currentRow.NRSEQCONTRATO);
                    else wdgTipoMedidaPropostaEditando.isVisible = false;

                    if (wdgNmContrat) {
                        wdgNmContrat.isVisible = true;
                        wdgNmContrat.enabled = true;
                    }
                }
                else {
                    wdgElementosProposta.isVisible = false;
                    wdgTipoMedidaPropostaEditando.isVisible = false;
                    wdgItensPropostaTree.isVisible = false;
                    itemcontrato.isVisible = true;
                    itemcontrato.enabled = true;
                    wdgItFaturaRateio.isVisible = true;
                    wdgFaturaRateio.isVisible = true;
                    if (wdgNmContrat) {
                        wdgNmContrat.isVisible = false;
                        wdgNmContrat.enabled = false;
                    }
                }
            }.bind(this), function (error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });
        }
        else {
            wdgElementosProposta.isVisible = false;
            wdgTipoMedidaPropostaEditando.isVisible = false;
            wdgItensPropostaTree.isVisible = false;
            itemcontrato.isVisible = true;
            wdgItFaturaRateio.isVisible = true;
            wdgFaturaRateio.isVisible = true;

            if (wdgNmContrat) wdgNmContrat.isVisible = false;
        }
    };

    /**
     * Realiza o controle de visibilidade das actions.
     * Se a situação for "Aprovado" ou "Encerrado" as 
     * actions de edit , insert e delete são blqueadas. 
     * 
     * @param {Object} widget
     * @tags : Action, Ações, ações, actions,Contrato, lançamento de contrato
     */
    this.controlaVisibilidade = function (widget) {
        var situacao = widget.currentRow.IDSITUACAOCONTRATO;

        var permiteEdicao = true;
        if (widget.currentRow.IDGERADOPOR != 'CONTRATO' && !widget.currentRow.NRSEQCONTRATOPAI)
            permiteEdicao = false;

        var actionEdit = widget.getAction('edit');
        var actionDel = widget.getAction('delete');

        var actionAprova = widget.getAction('aprovarcontrato');
        var actionReprova = widget.getAction('reprovarContrato');
        var actionEncerra = widget.getAction('encerrarContrato');
        //var actionVigencia    = widget.getAction('vigenciaContrato');
        var actionVersiona = widget.getAction('versionarContrato');
        var actionRegerarFat = widget.getAction('regerarFaturamento');
        var paralisarFatura = widget.getAction('paralisarContrato');
        var actionDocumento = widget.getAction('documento_contrato');
        var enviarIntegraGoogle = widget.getAction('enviarIntegraGoogle');

        var actionAtualizaHCM = widget.getAction('atualizaHCM');

        var visivel = false;
        /* Aprovado */
        if (situacao == 'A') {

            actionDel.isVisible = false;
            actionEdit.isVisible = (idAlteraContAprv == 'S' && visibilidadeAcoes);

            actionAprova.isVisible = false;
            actionReprova.isVisible = false;
            actionEncerra.isVisible = true;
            //actionVigencia.isVisible    = true;
            actionVersiona.isVisible = true;
            actionRegerarFat.isVisible = true;
            paralisarFatura.isVisible = true;
            //actionDocumento.isVisible   = true;
            enviarIntegraGoogle.isVisible = true;
            actionAtualizaHCM.isVisible = true;


        } /* Documento Enviado */
        else if (situacao == 'D') {

            actionDel.isVisible = false;
            actionEdit.isVisible = false;

            actionAprova.isVisible = true;
            actionReprova.isVisible = true;
            actionEncerra.isVisible = true;
            //actionVigencia.isVisible    = true;
            actionVersiona.isVisible = true;
            actionRegerarFat.isVisible = false;
            paralisarFatura.isVisible = false;
            //actionDocumento.isVisible   = true;
            enviarIntegraGoogle.isVisible = false;
            actionAtualizaHCM.isVisible = false;

        }
        /* Reprovado */
        else if (situacao == 'R') {

            actionDel.isVisible = false;
            actionEdit.isVisible = false;

            actionAprova.isVisible = false;
            actionReprova.isVisible = false;
            actionEncerra.isVisible = true;
            //actionVigencia.isVisible    = false;
            actionVersiona.isVisible = true;
            actionRegerarFat.isVisible = false;
            paralisarFatura.isVisible = false;
            //actionDocumento.isVisible   = false;
            enviarIntegraGoogle.isVisible = false;
            actionAtualizaHCM.isVisible = false;

        } /* Encerrado */
        else if (situacao == 'E') {

            actionDel.isVisible = false;
            actionEdit.isVisible = false;

            actionAprova.isVisible = false;
            actionReprova.isVisible = false;
            actionEncerra.isVisible = false;
            //actionVigencia.isVisible    = false;
            actionVersiona.isVisible = true;
            actionRegerarFat.isVisible = false;
            paralisarFatura.isVisible = false;
            //actionDocumento.isVisible   = false;
            enviarIntegraGoogle.isVisible = false;
            actionAtualizaHCM.isVisible = false;

        } // Suspenso
        else if (situacao == 'S') {

            actionDel.isVisible = false;
            actionEdit.isVisible = false;

            actionAprova.isVisible = false;
            actionReprova.isVisible = false;
            actionEncerra.isVisible = true;
            //actionVigencia.isVisible    = false;
            actionVersiona.isVisible = false;
            actionRegerarFat.isVisible = false;
            paralisarFatura.isVisible = false;
            actionDocumento.isVisible = false;
            enviarIntegraGoogle.isVisible = false;
            actionAtualizaHCM.isVisible = false;

        } /* Pendente */
        else {

            actionDel.isVisible = permiteEdicao;
            actionEdit.isVisible = true;

            actionAprova.isVisible = true;
            actionReprova.isVisible = true;
            actionEncerra.isVisible = true;
            //actionVigencia.isVisible    = false;
            actionVersiona.isVisible = false;
            actionRegerarFat.isVisible = false;
            paralisarFatura.isVisible = false;
            //actionDocumento.isVisible   = false;
            enviarIntegraGoogle.isVisible = false;
            actionAtualizaHCM.isVisible = false;

            visivel = true;
        }

        //visibilidade ações relacionadas a rescisão do contrato
        var visivelResc = widget.currentRow.IDSITUACAORESCISAO === 'S';
        widget.getAction('encerrarContrato').isVisible = (situacao == 'A' && !visivelResc);
        widget.getAction('gerarDocumentoRescisao').isVisible = visivelResc;
        widget.getAction('cancelarRescisao').isVisible = visivelResc;
        widget.getAction('confirmarRescisao').isVisible = visivelResc;

        visivel = (visivel && permiteEdicao);
        widget.getAction("copiarContrato").isVisible = (!getWidgetGridContrato().currentRow.NRSEQORCAMENTO);
        this.validacoesGeraisWidgetsContrato(visivel);
    };

    /**
     * Efetua as validações e altera a visibilidade das actions 
     * de cada widget
     * @params{bool} widget - se as actions de negociação ficarão ou não visíveis
     **/
    this.validacoesGeraisWidgetsContrato = function (visivel) {
        var wdgItens = templateManager.container.getWidget("itemcontrato");
        var wdgAdicionais = templateManager.container.getWidget("adicitemcontrato");
        var wdglocalAtend = templateManager.container.getWidget("localAtendContrato");
        var wdgfaturamento = templateManager.container.getWidget("faturamentocontrato");

        this.setVisibilidadeDasActionsItens(wdgItens, visivel);
        this.setVisibilidadeDasActionsItens(wdgAdicionais, visivel);
        this.controlaVisibilidadeLocalAt(wdglocalAtend);
        this.controlaVisibilidadeFat(wdgfaturamento);

        //var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
        var wdgParcelas = templateManager.container.getWidget("wdgParcelas");
        var wdgDescontoPorPeriodo = templateManager.container.getWidget("wdgDescontoPorPeriodo");
        this.setVisibilidadeDasActionsExcetoVoltar([ /*wdgNegociacaoValores, */ wdgParcelas, wdgDescontoPorPeriodo], visivel);
        this.controlarVisibilidadeParalisacao();
        this.controlarVisibilidadeInlineEdit();
    };

    /**
     * Controla a visibilidade da widget de histórico e 
     * da ação de retomada do faturamento
     **/
    this.controlarVisibilidadeParalisacao = function () {
        if (getWidgetGridContrato().currentRow.IDSITUACAOCONTRATO == "P") {
            templateManager.container.getWidget("wdgHistoricoParalisacao").isVisible = false;
            getWidgetGridContrato().getAction("retomarFaturamento").isVisible = false;
        }
        else {
            var params = [{
                name: "NRSEQCONTRATO",
                operator: "=",
                value: getWidgetGridContrato().currentRow.NRSEQCONTRATO
            }];

            General.callRestEngine("product#/crud/fct_paramparalisacaocont_widget", "FilterData", params).then(function (response) {
                var possuiParalisacao = (response.dataset && response.dataset.fct_paramparalisacaocont_widget && response.dataset.fct_paramparalisacaocont_widget.length > 0);
                templateManager.container.getWidget("wdgHistoricoParalisacao").isVisible = possuiParalisacao;

                if (!possuiParalisacao) {
                    getWidgetGridContrato().getAction("retomarFaturamento").isVisible = false;
                }
                else {
                    var paramParalisacao = response.dataset.fct_paramparalisacaocont_widget;
                    var dataAtual = moment();

                    var paralisacaoAtiva = paramParalisacao.filter(function (paralisacao) {
                        var dtRetomada = moment(paralisacao.DTRETOMADAFATURA, 'DD/MM/YYYY');
                        return dataAtual.isBefore(dtRetomada) || dataAtual.format("DD/MM/YYYY") == dtRetomada.format("DD/MM/YYYY");
                    });

                    getWidgetGridContrato().getAction("retomarFaturamento").isVisible = (paralisacaoAtiva.length > 0);
                }

            }).catch(function (error) {
                templateManager.container.getWidget("wdgHistoricoParalisacao").isVisible = false;
                getWidgetGridContrato().getAction("retomarFaturamento").isVisible = false;
            });
        }
    };
    /**
     * Valida se o perfil de acesso pode alterar as informações dos 
     * constratos
     **/
    this.validarPerfilDeAcesso = function () {
        var nrparcnegocioOperLogado = [];
        nrparcnegocioOperLogado[0] = nrParceiroOperador;
        nrparcnegocioOperLogado[1] = nrorgOperador;
        nrparcnegocioOperLogado[2] = 'C';

        General.callRestEngine('product#/fct_parametrogeral_widget/validarperfilacesso', "Row", nrparcnegocioOperLogado).then(function (perfilusuario) {
            visibilidadeAcoes = perfilusuario.dataset.fct_parametrogeral_widget[0];
        }.bind(this))
            .catch(function () {
                Utilities.showErrorMessage("Falha ao validar o perfil de acesso!");
            });
    };
    /**
     * Verifica se o contrato pode ser editado após a sua aprovação
     * @params{Object} widget widget lançamento de Contrato
     **/
    this.validarAlteracaoDeContratosAprovados = function () {
        var deferred = ZHPromise.defer();
        if (idAlteraContAprv !== null) {
            if (visibilidadeAcoes == null) {
                if (!nrParceiroOperador) {
                    this.validarPerfilDeAcesso();
                }
                else {
                    Operador_Repository.getOperadorLogado().then(function (success) {
                        nrParceiroOperador = success[0].NRPARCNEGOCIO;
                        this.validarPerfilDeAcesso();
                    }.bind(this));
                }
            }
            deferred.resolve();
        }
        else {
            General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", []).then(function (response) {
                if (response.dataset.fct_parametrogeral.length > 0)
                    idAlteraContAprv = response.dataset.fct_parametrogeral[0].IDALTERACONTAPROV;

                if (!nrParceiroOperador) {
                    this.validarPerfilDeAcesso();
                }
                else {
                    Operador_Repository.getOperadorLogado().then(function (success) {
                        nrParceiroOperador = success[0].NRPARCNEGOCIO;
                        this.validarPerfilDeAcesso();
                    }.bind(this));
                }
            }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                    deferred.reject;
                });
        }

        return deferred.promise;
    };

    /**
     * Este procedimento é utilizado para atualizar a data de reajuste baseado na alteração do campo Data Inicial
     * Este método é atrelado ao evento OnFieldChange do campo Data Inicial. 
     * @param {Widget} É a widget relativa aos dados do Contrato
    */
    this.atualizarDataDeReajusteOnChange = function (widget) {
        var dias = (widget.currentRow.NRDIASREAJAUTOMATICO == undefined || widget.currentRow.NRDIASREAJAUTOMATICO == null) ? widget.currentRow.NRDIASREAJAUTOMATICO : 1;
        widget.currentRow.DTPADREAJCONT = moment(widget.currentRow.DTFIMCONTRATO, "DD/MM/YYYY").add(dias, "days").format("DD/MM/YYYY");
    };

    /**
     * A função "preencheDadosModContrato" importa os dados do modelo de contrato 
     * para o contrato.
     */
    this.preencheDadosModContrato = function (widget) {
        /* -- Dados do Contrato -- */
        /* Obs: Os dados de contrato foram passados através do outdata do campo de Modelo de contrato.

         Portanto, aqui só será calculado a data do contrato.*/


        var modContrato = widget.currentRow.NRSEQMODCONTRA;
        var fieldCodigo = widget.getField("NRCONTRATOFCT");

        widget.getField("NMTIPOORCAMENTO_CONT").reload();

        if (modContrato) {
            var dataInicial = moment();
            var prazo = widget.currentRow.NRDIASVIGCONT;

            var dataInicialString = dataInicial.format("DD/MM/YYYY");
            widget.currentRow.DTINICIOCONTRATO = dataInicialString;

            if (prazo) {
                var dataFinal = dataInicial.add(prazo - 1, "day");
                var dataFinalString = dataFinal.format("DD/MM/YYYY");

                widget.currentRow.DTFIMCONTRATO = dataFinalString;
                widget.currentRow.NRDIASPRAZOCONT = prazo;

                this.atualizarDataDeReajusteOnChange(widget);
            }

            fieldCodigo.readOnly = true;
            fieldCodigo.validations = {};

            self.bloquearFieldsVencimento(widget);
        }
        else {
            fieldCodigo.readOnly = false;
            fieldCodigo.validations = {
                "required": true
            };
        }

        this.habilitarFieldsModEscProposta(widget);
        this.integracaoFMOnChange(widget);
        this.recarregaDadosparceiro();
        this.integracaoTecFoodOnChange();
        this.integracaoHCMOnChange(widget);
        this.validaIdCertsign();
    };
    /**
     * habilita as fields de escopo e modelo da proposta
     * @params{Object} widget - widget contrato
     **/
    this.habilitarFieldsModEscProposta = function (widget) {
        var geraProposta = (widget.currentRow.NRSEQMODCONTRA && widget.currentRow.IDGERAPROPOSTA && widget.currentRow.IDGERAPROPOSTA === 'S');
        var fieldModelo = widget.getField("NMORCAMENTOMODELO");
        var fieldEscopo = widget.getField("NRTIPOORCAMENTO");

        if (fieldModelo) {
            widget.currentRow.NMORCAMENTOMODELO = "";
            widget.currentRow.NMTIPOORCAMENTO = [];
            widget.currentRow.NRTIPOORCAMENTO = [];
            fieldEscopo.dataSource.clearCheckedRows();
            fieldModelo.dataSource.clearCheckedRows();

            if (!geraProposta) {
                fieldModelo.readOnly = true;
                fieldEscopo.readOnly = true;

                widget.getAction("Avancar").isVisible = false;
                widget.getAction("SAVE").isVisible = true;
            }
            else {
                if (!opLogado) {
                    this.buscaOperadorLogado().then(function () {
                        this.habilitarFieldsModEscProposta();
                    }.bind(this));
                }
                else {
                    fieldModelo.dataSourceFilter[1].value = widget.currentRow.NRSEQMODCONTRA;
                    var parametro = {
                        "NRPARCNEGOCIO": nrParceiroOperador,
                        "NRORG": nrorgOperador,
                        "PROPOUCONT": "P"
                    };

                    General.callRestEngine('product#/validarPerfilAcesso', "Row", parametro).then(function (perfilusuario) {
                        if (perfilusuario.dataset.validarperfil.PERFILACESSO) {
                            var modelosDeProposta = [];

                            perfilusuario.dataset.validarperfil.PERFILACESSO.forEach(function (modeloProposta) {
                                modelosDeProposta.push(modeloProposta.MODELO);
                            }.bind(this));

                            fieldModelo.dataSourceFilter[2] = {
                                "name": "NRORCAMENTOMODELO",
                                "operator": "NOT_IN",
                                "value": modelosDeProposta
                            };
                        }

                        fieldModelo.reload().then(function () {
                            fieldModelo.readOnly = !geraProposta;
                        });
                    });
                }
            }

            this.fieldModeloPropostaOnChange(fieldModelo);
        }
    };

    this.fieldModeloPropostaOnChange = function (field) {
        var widget = field.widget;
        if (field.value()) {
            widget.getField("NMTIPOORCAMENTO_CONT").readOnly = true;
            widget.getField("NMTIPOORCAMENTO_CONT").validations = {};
            widget.currentRow.NMTIPOORCAMENTO_CONT = "";
            widget.currentRow.NRTIPOORCAMENTO_CONT = "";

            widget.currentRow.IDAGRUPPARC = "1";
            widget.getField("IDAGRUPPARC").readOnly = true;
        }
        else {
            widget.getField("NMTIPOORCAMENTO_CONT").readOnly = false;
            widget.getField("NMTIPOORCAMENTO_CONT").validations = { "required": true };

            widget.getField("IDAGRUPPARC").readOnly = false;
        }

        this.filtrarEscopos(widget);
    };

    /**
     * Filtra os escopos de acordo com o modelo de proposta selecionado 
     * @params{Object} widget - widget contrato grid
     **/
    this.filtrarEscopos = function (widget) {
        var fieldEscopo = widget.getField("NRTIPOORCAMENTO");
        fieldEscopo.dataSourceFilter[1].value = widget.currentRow.NRORCAMENTOMODELO;
        fieldEscopo.readOnly = (widget.currentRow.NRORCAMENTOMODELO) ? false : true;

        widget.getAction("Avancar").isVisible = widget.currentRow.NRORCAMENTOMODELO ? true : false;
        widget.getAction("SAVE").isVisible = widget.currentRow.NRORCAMENTOMODELO ? false : true;

        fieldEscopo.reload().then(function () {
            if (fieldEscopo.dataSource.data.length === 1) {
                widget.currentRow.NMTIPOORCAMENTO = [fieldEscopo.dataSource.data[0].NMTIPOORCAMENTO];
                widget.currentRow.NRTIPOORCAMENTO = [fieldEscopo.dataSource.data[0].NRTIPOORCAMENTO];
                widget.currentRow.IDUTILIZAVARPRECO = [fieldEscopo.dataSource.data[0].IDUTILIZAVARPRECO];
                fieldEscopo.dataSource.addCheckedRows(fieldEscopo.dataSource.data[0]);
                fieldEscopo.dataSource.data[0].__isSelected = true;
                fieldEscopo.readOnly = true;
                UtilitiesPropostaContrato.validarParamsFaturamentoEscopo(widget, "NRTIPOORCAMENTO", "ORC");
            }
            else {
                widget.currentRow.NMTIPOORCAMENTO = [];
                widget.currentRow.NRTIPOORCAMENTO = [];
                fieldEscopo.dataSource.clearCheckedRows();
            }
        });
    };
    /**
     * A função "controlaCodContrato" controla campo "Código" da página "Contrato".
     */
    this.controlaCodContrato = function (widget) {
        var modContrato = widget.currentRow.NRSEQMODCONTRA;
        var fieldCodigo = widget.getField("NRCONTRATOFCT");

        if (modContrato && widget.currentRow.__is_new) {
            fieldCodigo.readOnly = true;
            fieldCodigo.validations = {};
        }
        else {
            fieldCodigo.readOnly = false;
            fieldCodigo.validations = {
                "required": true
            };
        }
    };

    /**
     * A função "salvaDadosModContrato" salva os dados da tela 
     * "Parâmetrização de Contrato"  à partir do Modelo de Contrato
     */
    this.salvaDadosModContrato = function (widget, seqModContrato, seqContrato) {

        // Dados de Faturamento
        //this.InsereDadosFaturaParamCont(seqModContrato, seqContrato);

        // Dados de Reajuste
        this.InsereDadosReajsuteParamCont(seqModContrato, seqContrato);

        // Dados de Documento
        this.InsereDadosDocumentoCont(seqModContrato, seqContrato);

        // Altera código do Contrato --
        this.alteraCodigoContr(widget, seqContrato);
    };

    /**
     * A função "InsereDadosFaturaParamCont" insere os dados de Faturamento 
     * da tela "Parâmetrização de Contrato"  à partir do Modelo de Contrato
     */
    this.InsereDadosFaturaParamCont = function (seqModContrato, seqContrato) {

        Sm00004000_Contrato_Service.recebeParamFatModCont(seqModContrato).then(function (response) {

            var nrGrupoFatura = response.dataset.PARAMFATMODCONT[0].NRGRUPOFATURA;
            var idSomaFatuCont = response.dataset.PARAMFATMODCONT[0].IDSOMAFATUCONT;
            var idSomaItGrupFat = response.dataset.PARAMFATMODCONT[0].IDSOMAITGRUPFAT;
            var idGeraNfParc = response.dataset.PARAMFATMODCONT[0].IDGERANFPARC;
            //var idBloqSemMedic = response.dataset.PARAMFATMODCONT[0].IDBLOQSEMMEDIC;
            var idBloqVrMedic = response.dataset.PARAMFATMODCONT[0].IDBLOQVRMEDIC;
            var idGeraRateio = response.dataset.PARAMFATMODCONT[0].IDGERARATEIO;

            var row = {
                "__using_unit": false,
                "__new_code_pk": "NRSEQFATUCONTR",
                "__table_pk": "NRSEQFATUCONTR",
                "__new_code_length": 10,
                "__cast_number": true,
                "__is_new": true,
                "NRSEQFATUCONTR": "",
                "NRSEQCONTRATO": seqContrato,
                "CDPRODFATURA": "",
                "CDFILIALFATURA": "",
                "NRGRUPOFATURA": nrGrupoFatura,
                "NRPARCFATURA": "",
                "IDSOMAFATUCONT": idSomaFatuCont,
                "IDSOMAITGRUPFAT": idSomaItGrupFat,
                "IDGERANFPARC": idGeraNfParc,
                "IDBLOQVRMEDIC": idBloqVrMedic,
                "IDGERARATEIO": idGeraRateio,
                "QTFRANQEVOLCONTR": "",
                "IDAPURAAPONTAMENTO": "N"
            };

            General.callRestEngine('product#/fct_paramfatcont_widget/save', "Row", row).then(function () {

            },
                function (error) {
                    console.log(error);
                    ScreenService.showMessage(error.data.error);
                });
        })
            .catch(function (response) {
                ScreenService.showMessage(response.data.error);
            });
    };

    /**
     * A função "InsereDadosReajsuteParamCont" insere os dados de Reajuste 
     * da tela "Parâmetrização de Contrato"  à partir do Modelo de Contrato
     */
    this.InsereDadosReajsuteParamCont = function (seqModContrato, seqContrato) {

        Sm00004000_Contrato_Service.recebeModReajustContr(seqModContrato).then(function (response) {

            var modeloReajuste = response.dataset.MODREAJUSTCONTR;

            modeloReajuste.forEach(function (modeloReajus) {

                var nrSeqReajusMod = modeloReajus.NRSEQMODREAJUS;
                var nmReajusContr = modeloReajus.NMMODREAJUSCONTR;
                var idTpReajuste = modeloReajus.IDTPREAJUSTE;
                var idTpMesReajuste = modeloReajus.IDTPMESREAJUSTE;
                var nrQtdReajuste = modeloReajus.NRQTDREAJUSTE;

                var row = {
                    "__using_unit": false,
                    "__new_code_pk": "NRSEQREAJUS",
                    "__table_pk": "NRSEQREAJUS",
                    "__new_code_length": 10,
                    "__cast_number": true,
                    "__is_new": true,
                    "__columns_to_return": ["NRSEQREAJUS"],
                    "NRSEQREAJUS": "",
                    "NMREAJUSCONTR": nmReajusContr,
                    "NRSEQCONTRATO": seqContrato,
                    "IDTPREAJUSTE": idTpReajuste,
                    "IDTPMESREAJUSTE": idTpMesReajuste,
                    "NRQTDREAJUSTE": nrQtdReajuste

                };

                General.callRestEngine('product#/crud/fct_reajuscontr_operador/save', "Row", row).then(function (response) {

                    var nrSeqReajusNew = response.dataset.columnsToReturn[0].NRSEQREAJUS;
                    this.InsereDadosIndiceParamCont(seqModContrato, nrSeqReajusMod, nrSeqReajusNew, seqContrato);

                }.bind(this),
                    function (error) {
                        console.log(error);
                        ScreenService.showMessage(error.data.error);
                    });

            }.bind(this));

        }.bind(this)).catch(function (response) {
            ScreenService.showMessage(response.data.error);
        });
    };

    /**
     * A função "InsereDadosIndiceParamCont" insere os dados de Índice de Reajuste 
     * da tela "Parametrização de Contrato" à partir do Modelo de Contrato
     */

    this.InsereDadosIndiceParamCont = function (seqModContrato, nrSeqReajusMod, nrSeqReajusNew, seqContrato) {

        Sm00004000_Contrato_Service.recebeModIndContr(seqModContrato, nrSeqReajusMod).then(function (response) {

            var indiceContrato = response.dataset.MODINDCONTR;

            indiceContrato.forEach(function (indiceContr) {

                var cdTpIndice = indiceContr.CDTPINDICE;
                var idPerValor = indiceContr.IDPERVALOR;
                var cdIndice = indiceContr.CDINDICE;
                var vrPercIndContr = indiceContr.VRPERCINDCONTR;

                var row = {
                    "__using_unit": false,
                    "__new_code_pk": "NRSEQINDCONTR",
                    "__table_pk": "NRSEQINDCONTR",
                    "__new_code_length": 10,
                    "__cast_number": true,
                    "__is_new": true,
                    "NRSEQINDCONTR": "",
                    "NRSEQCONTRATO": seqContrato,
                    "NRSEQREAJUS": nrSeqReajusNew,
                    "CDTPINDICE": cdTpIndice,
                    "IDPERVALOR": idPerValor,
                    "CDINDICE": cdIndice,
                    "VRPERCINDCONTR": vrPercIndContr

                };

                General.callRestEngine('product#/crud/fct_indicecontr_widget/save', "Row", row).then(function () {

                },
                    function (error) {
                        console.log(error);
                        ScreenService.showMessage(error.data.error);
                    });

            });

        }).catch(function (response) {
            ScreenService.showMessage(response.data.error);
        });
    };

    /**
     * A função "InsereDadosFaturaCont" insere os dados de Documento 
     * da tela "Contrato"  à partir do Modelo de Contrato
     */
    this.InsereDadosDocumentoCont = function (seqModContrato, seqContrato) {
        var params = {
            'NRSEQMODCONTRA': seqModContrato,
            'NRSEQCONTRATO': seqContrato
        }
        General.callRestEngine('product#/fct_contrato_widget/inserirDocumentos', "Row", params).then(null)
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };


    /**
     * A função "alteraCodigoContr" insere os dados de Documento 
     * da tela "Contrato"  à partir do Modelo de Contrato
     */

    this.alteraCodigoContr = function (widget, seqContrato) {

        /* -- 1 - Gera código do contrato -- */
        Sm00004000_Contrato_Service.geraCodigoContrato(seqContrato).then(function (response) {

            widget.currentRow.NRCONTRATOFCT = response.dataset.CODIGOCONTR.CODIGO;
            //widget.reload();

        }).catch(function (response) {
            ScreenService.showMessage(response.data.error);
        });
    };

    /* --- FUNÇÕES DA PÁGINA ITENS --- */

    /**
     * Realiza a s validaçoes antes de salvar um registro
     * 
     * @param {object} widget widget de 
     * Item de Contrato para salvar o novo registro.
     */
    this.validaPercentualSaveItens = function (widget) {
        if (widget.isValid()) {
            if (widget.currentRow.IDPRODSERV == 'S')
                widget.currentRow.CDSERVORCA = widget.currentRow.CDPRODSERV;
            else
                widget.currentRow.CDPRODUTO = widget.currentRow.CDPRODSERV;

            if (widget.currentRow.NMDOCITEMCONT && widget.currentRow.NMDOCITEMCONT !== widget.currentRow.NMITEMCONTRATO)
                widget.currentRow.NMITEMCONTRATO = widget.currentRow.NMDOCITEMCONT;

            this.salvarItemContrato(widget);
            /*this.validarDadosRateioContrato(widget, "manual").then(function(response) {
                var gerarRateio = (widget.currentRow.__is_new && widget.currentRow.NRESTRUTURAM);
                widget.currentRow.GERARRATEIO = gerarRateio;

                if (!response) {
                    this.salvarItemContrato(widget);
                }
                else {
                    ScreenService.confirmMessage("Atenção!<br><br>Inserir este item sem vincular um local de atendimento a ele, acarretará na perda dos rateios durante a geração das parcelas.<br><br>Deseja continuar?", "question", function() {
                        this.salvarItemContrato(widget);
                    }.bind(this));
                }
            }.bind(this));*/
        }
        else {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
    };
    /**
     * Converte o valor de ajuste para valor percentual
     * 
     * @param {object} widget widget de 
     * Item de Contrato para salvar o novo registro.
     */
    this.converteParaPercentual = function (widget) {
        var percentual = widget.currentRow.VRDESCONTO;
        var tipoAjuste = widget.currentRow.IDTPDESCONTO;
        var quantidade = widget.currentRow.QTITEMCONTRATO;
        var valorUnitario = widget.currentRow.VRUNITITEMCONT;
        var utilizaQtdeMedida = widget.currentRow.IDMULTMEDVRTOT;
        var qtdeMedida = widget.currentRow.QTMEDIDACONTR;
        var valorSemAjuste = quantidade * valorUnitario;
        if (utilizaQtdeMedida == "S") {
            valorSemAjuste *= qtdeMedida;
        }

        var valorTotal = valorSemAjuste;

        if (tipoAjuste == "V" || tipoAjuste == "X") {
            return (percentual / valorTotal) * 100;
        }
        else
            return percentual;
    };
    /**
     * Verifica se o item é de uma composição de serviço
     * e monta a row para salvá-lo
     * @params{Object} widget - widget itens
     **/
    this.getRowItensContrato = function (widget) {
        var deferred = ZHPromise.defer();
        var rowContrato = getWidgetGridContrato().currentRow;
        if (widget.name !== "wdgItensPropostaTree" || !rowContrato.NRSEQORCAMENTO) {
            this.criarParametrosNewCode(widget);
            widget.currentRow.EDITAITEMPROPOSTA = false;
            deferred.resolve(widget.currentRow);
        }
        else {
            var params = [{
                name: "NRITEMOCONTRATO",
                operator: "=",
                value: widget.currentRow.NRITEMOCONTRATO
            },
            {
                name: "NRSEQCONTRATO",
                operator: "=",
                value: rowContrato.NRSEQCONTRATO
            }
            ];
            General.callRestEngine("product#/crud/fct_itemcontrato", "FilterData", params).then(function (response) {
                var row = response.dataset.fct_itemcontrato[0];
                row["EDITAITEMPROPOSTA"] = rowContrato.IDGERADOPOR == 'CONTRATO';
                row["NRSEQCONTRATO"] = rowContrato.NRSEQCONTRATO;
                row["QTITEMCONTRATO"] = widget.currentRow.QTITEMORCAMENTO;
                row["NRITEMOCONTRATO"] = widget.currentRow.NRITEMOCONTRATO;
                row["NRSEQORCAMENTO"] = widget.currentRow.NRSEQORCAMENTO;
                row["NRITEMORCAMENTO"] = widget.currentRow.NRITEMORCAMENTO;
                row["QTMEDIDAORCA"] = widget.currentRow.QTMEDIDAORCA;
                row["IDTPDESCORCA"] = widget.currentRow.IDTPDESCORCA;
                row["IDFATURAITEM"] = widget.currentRow.IDFATURAITEM;
                row["DTINICFATCONTR"] = widget.currentRow.DTINICFATCONTR;
                row["IDFATURACONTR"] = widget.currentRow.IDFATURACONTR;
                row["QTPARCELASFATU"] = widget.currentRow.QTPARCELASFATU;
                row["NMDOCITEMCONT"] = widget.currentRow.NMDOCITEMCONT;
                row["NRESTRUTURAM"] = widget.currentRow.NRESTRUTURAM;
                row["VRUNITITEMORCA"] = Utilities.tirarMascaraValores(widget.currentRow.VRUNITITEMORCA);
                row["VRUNITITEMCONT"] = Utilities.tirarMascaraValores(widget.currentRow.VRUNITITEMORCA);
                row["VRTOTALITEMORCA"] = Utilities.tirarMascaraValores(widget.currentRow.VRTOTALITEMORCA);
                row["VRTOTITEMCONT"] = Utilities.tirarMascaraValores(widget.currentRow.VRTOTALITEMORCA);
                row["VRPERCDESCORCA"] = Utilities.tirarMascaraValores(widget.currentRow.VRPERCDESCORCA);
                row["VRDESCORCA"] = Utilities.tirarMascaraValores(widget.currentRow.VRDESCORCA);
                row["VRITEMSEMDESC"] = Utilities.tirarMascaraValores(widget.currentRow.VRITEMSEMDESC);
                row["__is_new"] = false;

                widget.dataSource.data.forEach(function (item) {
                    if (item.CDSERVORCA === widget.currentRow.CDSERVORCA && item.IDTIPO == "M" && item.NRITEMORCAMENTO !== widget.currentRow.NRITEMORCAMENTO && item.IDELEMENTO && item.IDELEMENTO !== widget.currentRow.IDELEMENTO) {
                        row["VRTOTITEMCONT"] += Utilities.tirarMascaraValores(item.VRTOTALITEMORCA);
                        row["VRUNITITEMCONT"] += Utilities.tirarMascaraValores(item.VRUNITITEMORCA);
                    }
                });

                widget.backupRow.QTITEMCONTRATO = widget.backupRow.QTITEMORCAMENTO;
                deferred.resolve(row);
            }.bind(this));
        }
        return deferred.promise;
    };
    /**
     * Salva um item ou um adicional.
     * 
     * @param {object} widget - Widget de Itens ou de Adicionais
     */
    this.salvarItemContrato = function (widget) {
        var deferred = ZHPromise.defer();
        if (!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
            deferred.reject();
        }
        else {
            if (!widget.currentRow.GERARRATEIO) widget.currentRow.GERARRATEIO = false;
            if (!widget.currentRow.QTMEDIDACALCULADA) widget.currentRow.QTMEDIDACALCULADA = widget.currentRow.QTMEDIDACONTR;
            if (!widget.currentRow.NRTIPOORCAMENTO)
                widget.currentRow.NRTIPOORCAMENTO = getWidgetGridContrato().currentRow.NRTIPOORCAMENTO;

            var nritempai = widget.currentRow.NRITEMCONTRATOPAI;
            var percAcrescimoServico = 0;
            widget.dataSource.data.filter(function (row) {
                return row.NRITEMCONTRATOPAI == nritempai;
            }).forEach(function (row) {
                var percentual = (Utilities.tirarMascaraValores(row.VRUNITITEMORCA) - Utilities.tirarMascaraValores(row.VRUNITITEMORCATBLPRECO)) / Utilities.tirarMascaraValores(row.VRUNITITEMORCATBLPRECO) * 100;
                percAcrescimoServico += percentual;
            });

            var params = {
                "CDSERVORCA": widget.currentRow.CDSERVORCA,
                "IDTIPO": "A"
            };

            General.callRestEngine("product#/fct_itemcontrato_widget/validaPorcentualDescontoAcrescimoSave", "Row", params).then(function (response) {
                var resultado = response.dataset.RESULTADO.VRPERCDESC;
                if (resultado && percAcrescimoServico > resultado.VRPERCDESC) {
                    var contexto = (resultado.TIPO == 1) ? "Este" : "O perfil de acesso deste";
                    ScreenService.showMessage("Operação bloqueada.<br/>" + contexto + " operador não têm autorização para propor acréscimo superior a " + resultado.VRPERCDESC + "%.");
                }
                else {
                    var isNew = widget.currentRow.__is_new;

                    var quantidade = widget.currentRow.QTITEMCONTRATO;
                    var quantidadeAntiga = quantidade;
                    var localAtendNovo = widget.currentRow.NRESTRUTURAM;
                    var localAtendAntigo = localAtendNovo;
                    var dsLoteFaturaNovo = widget.currentRow.DSLOTEFATURAMENTO;
                    var dsLoteFaturaAntigo = dsLoteFaturaNovo;
                    var nrEscalaTrabHCM = widget.currentRow.NRESCALATRABALHOHCM;
                    var nrEscalaTrabHCMAntigo = nrEscalaTrabHCM;
                    var agrupaEscalaTrabHCM = getWidgetGridContrato().currentRow.IDAGRUPAESCALAHCM;

                    if (widget.backupRow) {
                        quantidadeAntiga = widget.backupRow.QTITEMCONTRATO;
                        localAtendAntigo = widget.backupRow.NRESTRUTURAM;
                        dsLoteFaturaAntigo = widget.backupRow.DSLOTEFATURAMENTO;
                        nrEscalaTrabHCMAntigo = widget.backupRow.NRESCALATRABALHOHCM;
                    }

                    widget.currentRow.NRSEQCONTRATO = getWidgetGridContrato().currentRow.NRSEQCONTRATO;

                    var cdprodserv = widget.currentRow.CDPRODSERV;
                    if (widget.name !== "wdgItensPropostaTree") {
                        if (widget.currentRow.IDPRODSERV == 'S') {
                            widget.currentRow.CDSERVORCA = cdprodserv;
                            widget.currentRow.CDPRODUTO = null;
                        }
                        else {
                            widget.currentRow.CDSERVORCA = null;
                            widget.currentRow.CDPRODUTO = cdprodserv;
                        }
                    }

                    widget.currentRow.ITEMCOMPOSICAO = (widget.name == "wdgItensPropostaTree");
                    widget.currentRow["__columns_to_return"] = ["NRITEMOCONTRATO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];

                    widget.currentRow.VRTOTITEMCONT = Utilities.tirarMascaraValores(widget.currentRow.VRTOTITEMCONT, 8);
                    widget.currentRow.VRUNITITEMCONT = Utilities.tirarMascaraValores(widget.currentRow.VRUNITITEMCONT, 8);
                    if (widget.currentRow.IDMULTMEDVRTOT == "S")
                        widget.currentRow.QTMEDIDACONTR = Utilities.tirarMascaraValores(widget.currentRow.QTMEDIDACONTR);

                    if (!widget.currentRow.__is_new) {
                        if (widget.backupRow)
                            widget.currentRow.QTDIFMEDIDACONTR = widget.currentRow.QTMEDIDACONTR - Utilities.tirarMascaraValores(widget.backupRow.QTMEDIDACONTR);
                        else
                            widget.currentRow.QTDIFMEDIDACONTR = 0;
                    }
                    var nrEstruturam = widget.currentRow.NRESTRUTURAM;
                    if (Array.isArray(nrEstruturam) && nrEstruturam.length > 0) {
                        Utilities.inserirMultiplos(widget, "NRESTRUTURAM", null, "product#/fct_itemcontrato_widget/save", "DataSet", [widget.currentRow]).then(function (response) {
                            deferred.resolve();
                            this.continuacaoAposSalvarItem(widget, response, quantidade, isNew, quantidadeAntiga, localAtendNovo, localAtendAntigo, dsLoteFaturaNovo, dsLoteFaturaAntigo, nrEscalaTrabHCM, nrEscalaTrabHCMAntigo, agrupaEscalaTrabHCM);
                        }.bind(this))
                            .catch(function (error) {
                                Utilities.showErrorMessage(error);
                                deferred.reject(error);
                            });
                    }
                    else {
                        General.callRestEngine("product#/fct_itemcontrato_widget/save", "DataSet", [widget.currentRow]).then(function (response) {
                            deferred.resolve();
                            this.continuacaoAposSalvarItem(widget, response, quantidade, isNew, quantidadeAntiga, localAtendNovo, localAtendAntigo, dsLoteFaturaNovo, dsLoteFaturaAntigo, nrEscalaTrabHCM, nrEscalaTrabHCMAntigo, agrupaEscalaTrabHCM);
                        }.bind(this))
                            .catch(function (error) {
                                Utilities.showErrorMessage(error);
                                deferred.reject(error);
                            });
                    }
                }
            }.bind(this)).catch(function (error) {
                Utilities.showErrorMessage(error);
            });
        }

        return deferred.promise;
    };

    this.continuacaoAposSalvarItem = function (widget, response, quantidade, isNew, quantidadeAntiga, localAtendNovo, localAtendAntigo, dsLoteFaturaNovo, dsLoteFaturaAntigo, nrEscalaTrabHCM, nrEscalaTrabHCMAntigo, agrupaEscalaTrabHCM) {
        if (response.dataset && response.dataset.columnsToReturn[0]) {
            widget.currentRow.NRITEMOCONTRATO = response.dataset.columnsToReturn[0].NRITEMOCONTRATO;
            widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
            widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
            widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
            widget.currentRow.changed = false;

            delete widget.backupRow;

            if (widget.currentRow.__is_new) {
                ScreenService.closeSwipe();
                widget.list();
            }
            else widget.view();

            General.changeWidgetAcessibility(widget.widgets, true);
            if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;
            if (isNew) {
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
                this.insereAgendamentoPendente(widget.currentRow.NRSEQCONTRATO, widget.currentRow.NRITEMOCONTRATO, quantidade, getWidgetGridContrato().currentRow.IDSITUACAOCONTRATO);
            } else {
                widget.view();
                if (quantidadeAntiga !== quantidade) {
                    var params = {
                        "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                        "NRITEMOCONTRATO": widget.currentRow.NRITEMOCONTRATO,
                        "QTPENDENCIA": quantidade,
                        "SITUACAO": getWidgetGridContrato().currentRow.IDSITUACAOCONTRATO
                    };
                    General.callRestEngine("product#/fct_itemcontrato_widget/atualizaQuantidadeItensAgendPendente", "Row", params).then(null)
                        .catch(function (error) {
                            Utilities.showErrorMessage(error);
                        });
                }
                if (getWidgetGridContrato().currentRow.QTITENSTRABSERV == 0) {
                    this.atualizarValoresNegociacaoAoEditarItemSemComposicao(widget.currentRow.NRITEMOCONTRATO, widget.currentRow.VRTOTITEMCONT, localAtendNovo, localAtendAntigo, dsLoteFaturaNovo, dsLoteFaturaAntigo, nrEscalaTrabHCM, nrEscalaTrabHCMAntigo, agrupaEscalaTrabHCM);
                }
            }
            getWidgetGridContrato().mode = 'list';
            widget.getField("NMESTRUTURAM").isVisible = true;
            widget.reload();
            getWidgetGridContrato().reload();
            getWidgetGridContrato().mode = 'view';
        }
    };

    this.atualizarValoresNegociacaoAoEditarItemSemComposicao = function (nritemocontrato, vrtotitemcont, localAtendNovo, localAtendAntigo, dsLoteFaturaNovo, dsLoteFaturaAntigo, nrEscalaTrabHCM, nrEscalaTrabHCMAntigo, agrupaEscalaTrabHCM) {
        var params = {
            "NRITEMOCONTRATO": nritemocontrato,
            "VRTOTITEMCONT": vrtotitemcont,
            "NRESTRUTURAM_NEW": localAtendNovo,
            "NRESTRUTURAM_OLD": localAtendAntigo,
            "DSLOTEFATURAMENTO_NEW": dsLoteFaturaNovo,
            "DSLOTEFATURAMENTO_OLD": dsLoteFaturaAntigo,
            "NRESCALATRABALHOHCM_NEW": nrEscalaTrabHCM,
            "NRESCALATRABALHOHCM_OLD": nrEscalaTrabHCMAntigo,
            "AGRUPAESCALATRABHCM": agrupaEscalaTrabHCM
        };

        General.callRestEngine("product#/fct_contrato_widget/atualizarValoresNegociacaoAoEditarItemSemComposicao", "Row", params)
            .then(null).catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };

    /**
     * Deleta itens do contrato
     * @params{Object} widget itens contrato
     **/
    this.deletarItensContrato = function (widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function () {
            widget.currentRow.NRTIPOORCAMENTO = getWidgetGridContrato().currentRow.NRTIPOORCAMENTO;
            General.callRestEngine("product#/fct_itemcontrato_widget/DeletaItemContratoAprovado", "Row", widget.currentRow)
                .then(function () {
                    var params = { "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO };
                    return General.callRestEngine("product#/fct_contrato_widget/atualizarValorTodasNegociacoesComSomaDosItens", "Row", params);
                }).then(function () {
                    ScreenService.closeSwipe().then(function () {
                        widget.activate();
                        widget.reload();
                    });
                }).catch(function (error) {
                    ScreenService.showMessage(error.data.error);
                });
        }.bind(this));
    };

    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Adicionais de Item de Contrato
     * para salvar o novo registro.
     */
    this.salvarAdicItemContrato = function (widget) {
        var widgetItemContrato = getWidgetItemContrato();
        var cdprodserv = widget.currentRow.CDPRODSERV;
        var isNew = widget.currentRow.__is_new;
        var nrseqcontrato = widget.currentRow.NRSEQCONTRATO;
        var quantidade = widget.currentRow.QTITEMCONTRATO;
        var idSituacao = templateManager.container.getWidget("gridContrato").currentRow.IDSITUACAOCONTRATO;
        widget.currentRow.NRSEQCONTRATO = widgetItemContrato.currentRow.NRSEQCONTRATO;
        widget.currentRow.NRITEMCONTRATOPAI = widgetItemContrato.currentRow.NRITEMOCONTRATO;


        if (widget.currentRow.IDPRODSERV == 'S') {
            widget.currentRow.CDSERVORCA = cdprodserv;
            widget.currentRow.CDPRODUTO = null;
        }
        else {
            widget.currentRow.CDSERVORCA = null;
            widget.currentRow.CDPRODUTO = cdprodserv;
        }

        widget.currentRow.__columns_to_return = ["NRITEMOCONTRATO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        General.widgetSave(widget).then(function (response) {
            if (response.dataset)
                widget.currentRow.NRITEMOCONTRATO = response.dataset.columnsToReturn[0].NRITEMOCONTRATO;
            widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
            widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
            widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;

            if (idSituacao === "A" && isNew) {
                this.insereAgendamentoPendente(nrseqcontrato, widget.currentRow.NRITEMOCONTRATO, quantidade, idSituacao);
            }
            else {
                if (!isNew) {
                    var params = {
                        "NRSEQCONTRATO": nrseqcontrato,
                        "NRITEMOCONTRATO": widget.currentRow.NRITEMOCONTRATO,
                        "QTPENDENCIA": quantidade,
                        "SITUACAO": idSituacao
                    };

                    General.callRestEngine("product#/fct_itemcontrato_widget/atualizaQuantidadeItensAgendPendente", "Row", params)
                        .then(null)
                        .catch(function (error) {
                            ScreenService.showMessage(error.data.error);
                        });
                }
            }

        }.bind(this));
    };

    /**
     * Realiza o controle dos campos de "Medida" de acordo 
     * com o field "Utiliza Medida?"
     * 
     * @param {object} widget widget de 
     * Itens para pegar o field "IDMULTMEDVRTOT".
     */

    this.bloquearDadosMedida = function (widget) {
        var usamedida = widget.getField('IDMULTMEDVRTOT').value();

        /* Bloqueia a Modalidade de pagamento*/
        if (usamedida == 'N') {
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = true;
            widget.getField('DSTPMEDIDASERV').setValue(null);
            widget.getField('DSTPMEDIDASERV').validations = {};
            widget.currentRow.DSTPMEDIDASERV = "";

            widget.getField('CDTPMEDIDASERV').setValue(null);
            widget.getField('CDTPMEDIDASERV').validations = {};
            widget.currentRow.CDTPMEDIDASERV = "";

            /* Quantidade de Medida */
            widget.getField('QTMEDIDACONTR').readOnly = true;
            widget.getField('QTMEDIDACONTR').setValue(null);
            widget.getField('QTMEDIDACONTR').validations = {};
            widget.currentRow.QTMEDIDACONTR = "";

        }
        /* Desbloqueia a Medida*/
        else {

            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = false;
            widget.getField('DSTPMEDIDASERV').validations = {
                "required": true
            };

            /* Quantidade de Medida */
            widget.getField('QTMEDIDACONTR').readOnly = false;
            widget.getField('QTMEDIDACONTR').validations = {
                "required": true
            };
        }

    };

    /**
     * Realiza o controle de Percentual de desconto do field "Desconto"
     * 
     * @param {object} widget widget de 
     * Itens para pegar os fields "IDTPDESCONTO" 
     * e "VRDESCONTO".
     */
    this.ValidaDescontoPerc = function (widget) {

        var desconto = widget.getField('IDTPDESCONTO').value();
        var vrdesconto = widget.getField('VRDESCONTO').value();



        /* Aplica Max e Min Value*/
        if (desconto == 'P') {

            if (vrdesconto < 0 || vrdesconto > 100) {
                /* Desconto */
                widget.currentRow.VRDESCONTO = null;
            }
        }
    };
    /**
     * Realiza as validaçoes no campo conforme o tipo de ajuste selecionado.
     * nos itens de contratos com composição de serviço
     * @param {object} widget - Widget de Itens da Contrato ou de Adicionais
     */
    this.validarPorcentagemValorContComposicao = function (widget) {
        var tipoAjuste = widget.currentRow.IDTPDESCORCA;
        var vrpercajuste = widget.currentRow.VRPERCDESCORCA;
        var ajuste = widget.currentRow.VRDESCORCA;

        if (ajuste) ajuste = parseFloat(Utilities.tirarMascaraValores(ajuste));
        if (vrpercajuste) vrpercajuste = parseFloat(Utilities.tirarMascaraValores(vrpercajuste));
        if (['P', 'Y'].includes(tipoAjuste))
            ajuste = vrpercajuste;

        var valorSemAjuste = widget.currentRow.VRITEMSEMDESC;
        if (tipoAjuste == "P") {
            if (vrpercajuste < 0 || vrpercajuste > 100) {
                ScreenService.showMessage("O valor percentual do desconto deve ser maior que 0 (zero) e inferior a 100%");
                widget.currentRow.VRPERCDESCORCA = null;
                widget.currentRow.VRDESCORCA = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            }
        }
        else if (tipoAjuste == "Y") {
            if (vrpercajuste < 0) {
                ScreenService.showMessage("O valor percentual do acréscimo não pode ser inferior a 0 (zero)");
                widget.currentRow.VRPERCDESCORCA = null;
                widget.currentRow.VRDESCORCA = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            }
        }
        else if (tipoAjuste == "V") {
            if (ajuste < 0) {
                ScreenService.showMessage("O valor do ajuste não pode ser inferior a 0 (zero)");
                widget.currentRow.VRPERCDESCORCA = null;
                widget.currentRow.VRDESCORCA = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            }
            else if (ajuste > valorSemAjuste) {
                ScreenService.showMessage("O valor do ajuste não pode ser superior ao valor total");
                widget.currentRow.VRPERCDESCORCA = null;
                widget.currentRow.VRDESCORCA = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            }
        }
        else if (tipoAjuste == "X") {
            if (ajuste < 0) {
                ScreenService.showMessage("O valor do ajuste não pode ser inferior a 0 (zero)");
                widget.currentRow.VRPERCDESCORCA = null;
                widget.currentRow.VRDESCORCA = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            }
        }

        this.calcularValorTotalItem(widget);
    };
    /**
     * Realiza as validaçoes no campo conforme o tipo de ajuste selecionado.
     * dos itens do contrato
     * @param {object} widget - Widget de Itens da Contrato ou de Adicionais
     */
    this.validarCampoPorcentagemValorContrato = function (widget) {
        var quantidade = widget.currentRow.QTITEMCONTRATO;
        var utilizaQtdeMedida = widget.currentRow.IDMULTMEDVRTOT;
        var tipoAjuste = widget.getField('IDTPDESCONTO').value();
        var ajuste = widget.getField('VRDESCONTO').value();
        var valorUnitario = widget.currentRow.VRUNITITEMCONT;
        var qtdeMedida = widget.currentRow.QTMEDIDACONTR;

        if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));
        var valorSemajuste = quantidade * valorUnitario;
        if (utilizaQtdeMedida == "S") {
            valorSemajuste *= qtdeMedida;
        }

        if (tipoAjuste == "P") {
            if (ajuste < 0) {
                ScreenService.showMessage("O valor percentual não pode ser inferior a 0 (zero)");
                widget.currentRow.VRDESCONTO = null;
            }
            else if (ajuste > 100) {
                ScreenService.showMessage("O valor percentual não pode ser superior a 100%");
                widget.currentRow.VRDESCONTO = null;
            }
        }
        else
            if (tipoAjuste == "Y") {
                if (ajuste < 0) {
                    ScreenService.showMessage("O valor percentual não pode ser inferior a 0 (zero)");
                    widget.currentRow.VRDESCONTO = null;
                }
            }
            else
                if (tipoAjuste == "V") {
                    if (ajuste < 0) {
                        ScreenService.showMessage("O valor do ajuste não pode ser inferior a 0 (zero)");
                        widget.currentRow.VRDESCONTO = null;
                    }
                    else if (ajuste > valorSemajuste) {
                        ScreenService.showMessage("O valor do ajuste não pode ser superior ao valor total");
                        widget.currentRow.VRDESCONTO = null;
                    }
                }
                else
                    if (tipoAjuste == "X") {
                        if (ajuste < 0) {
                            ScreenService.showMessage("O valor do ajuste não pode ser inferior a 0 (zero)");
                            widget.currentRow.VRDESCONTO = null;
                        }
                    }
    };

    /**
     * Ativa/desativa todas as actions de uma widget, menos as do tipo "back".
     * 
     * @param {Object} widget
     * @param {boolean} visivel
     */
    this.setVisibilidadeDasActionsItens = function (widget, visivel) {
        var idSituacao = getWidgetGridContrato().currentRow.IDSITUACAOCONTRATO;
        var nrseqcontratopai = getWidgetGridContrato().currentRow.NRSEQCONTRATOPAI;
        var idGeradoPor = getWidgetGridContrato().currentRow.IDGERADOPOR;
        var tipos = ["add", "edit", "delete"];

        if (idSituacao === "A" && (nrseqcontratopai || idGeradoPor == "CONTRATO")) {
            widget.getAction("edit").isVisible = (idAlteraContAprv === "S") ? true : false;
            widget.getAction("add").isVisible = (idAlteraContAprv === "S") ? true : false;
            widget.getAction("delete").isVisible = (idAlteraContAprv === "S") ? true : false;
        }
        else {
            widget.actions.forEach(function (action) {
                if (tipos.includes(action.name)) {
                    action.isVisible = visivel;
                }
            });
        }
    };


    /**
     * Consolida as chamadas de funções das 
     * Widgets "Itens" e "Adicionais"
     * 
     * @param {object} widget widget de 
     * Itens
     */
    this.funcoesItens = function (widget) {
        this.bloquearDadosMedida(widget);
        this.filtrarTipoDeItensManuaisDoEscopo(widget);
        this.bloquearDadosMedida(widget);
        this.integracaoTecFoodOnChange();
        this.integracaoHCMItensOnChange();
        //widget.getField("NMESTRUTURAM").readOnly = (getWidgetGridContrato().currentRow.IDSITUACAOCONTRATO != 'P');
    };



    this.mostrarLocalUnicoEsconderMultiplo = function (widget) {
        this.mostrarMultiploLocalDeAtendimento(widget, false);
        this.mostrarUnicoLocalDeAtendimento(widget, true);
    };

    this.mostrarLocalMultiploEsconderUnico = function (widget) {
        this.mostrarMultiploLocalDeAtendimento(widget, true);
        this.mostrarUnicoLocalDeAtendimento(widget, false);
    };

    this.getUnicoLocalDeAtendimento = function (widget) {
        return widget.getField("NMESTRUTURAM");
    };

    this.getMultiploLocalDeAtendimento = function (widget) {
        return widget.getField("NRESTRUTURAM");
    };

    this.mostrarMultiploLocalDeAtendimento = function (widget, inverter) {
        this.getMultiploLocalDeAtendimento(widget).isVisible = inverter;
    };

    this.mostrarUnicoLocalDeAtendimento = function (widget, inverter) {
        this.getUnicoLocalDeAtendimento(widget).isVisible = inverter;
    };

    this.editarItemContrato = function (widget) {
        this.funcoesItens(widget);
        General.widgetEdit(widget);

        var nrtipoorcamento = widget.currentRow.NRTIPOORCAMENTO ? widget.currentRow.NRTIPOORCAMENTO : getWidgetGridContrato().currentRow.NRTIPOORCAMENTO;
        var tipoprodserv = widget.currentRow.IDPRODSERV ? widget.currentRow.IDPRODSERV : 'S';
        var cdprodserv = widget.currentRow.CDPRODSERV ? widget.currentRow.CDPRODSERV : widget.currentRow.CDSERVORCA;

        if (nrtipoorcamento && tipoprodserv == "S" && cdprodserv) {
            var params = {
                "NRTIPOORCAMENTO": nrtipoorcamento,
                "CDSERVORCA": cdprodserv
            };
            General.callRestEngine("product#/fct_itemorcamento_widget/getMedidaVariavelEscopo", "Row", params).then(function (response) {
                if (response.dataset && response.dataset.retorno) {
                    widget.getField("DSTPMEDIDASERV").readOnly = true;
                    widget.currentRow.INSERIR_MEDIDA = true;

                    if (widget.name == "wdgItensPropostaTree" || widget.currentRow.IDARVOREPAI) {
                        widget.getField("VRUNITITEMCONT").readOnly = true;
                        widget.getField("VRUNITITEMCONT").help = "O campo foi desabilitado porque não é possível inserir acréscimos em itens vinculados a serviços faturados por medição.";
                    }
                    else {
                        widget.getField("VRUNITITEMCONT").readOnly = false;
                        widget.getField("VRUNITITEMCONT").help = null;
                    }
                }
                else {
                    widget.getField("DSTPMEDIDASERV").readOnly = false;
                    widget.currentRow.INSERIR_MEDIDA = false;

                    widget.getField("VRUNITITEMCONT").readOnly = false;
                    widget.getField("VRUNITITEMCONT").help = null;
                }
            });
        }
    };

    /**
     * Realiza o cálculo do valor total do item
     * 
     * @param {object} widget widget de 
     * Itens para pegar os fields para cálculo
     */
    this.calculaValorTotItem = function (widget) {
        var quantidade = widget.currentRow.QTITEMCONTRATO;
        var utilizaQtdeMedida = widget.currentRow.IDMULTMEDVRTOT;
        var qtdeMedida = widget.currentRow.QTMEDIDACONTR;
        var valorUnitario = widget.currentRow.VRUNITITEMCONT;
        var tipoajuste = widget.currentRow.IDTPDESCONTO;
        var ajuste = widget.currentRow.VRDESCONTO;

        if (quantidade) quantidade = parseFloat(quantidade.toString().replace(",", "."));
        if (qtdeMedida) qtdeMedida = parseFloat(qtdeMedida.toString().replace(",", "."));
        if (valorUnitario) valorUnitario = parseFloat(valorUnitario.toString().replace(",", "."));
        if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));

        var valorSemajuste = quantidade * valorUnitario;
        if (utilizaQtdeMedida == "S") {
            valorSemajuste *= qtdeMedida;
        }

        var valorTotal = valorSemajuste;

        if (tipoajuste == "V" && ajuste <= valorTotal && ajuste >= 0) {
            valorTotal -= ajuste;
        }
        else if (tipoajuste == "P" && ajuste <= 100 && ajuste >= 0) {
            valorTotal -= valorTotal * ajuste / 100;
        }
        else if (tipoajuste == "X" && ajuste >= 0) {
            valorTotal += ajuste;
        }
        else if (tipoajuste == "Y" && ajuste >= 0) {
            valorTotal += valorTotal * ajuste / 100;
        }

        //widget.currentRow.VRITEMSEMDESC = valorSemajuste;
        widget.currentRow.VRTOTITEMCONT = valorTotal;
    };

    /**
     * Novo Registro da aba de Itens do contrato ou de Adicionais.
     * 
     * @param {object} widget - Widget de Itens do contrato ou de Adicionais
     */
    this.itensNovoRegistro = function (widget) {
        ScreenService.showCustomDialog("Como deseja realizar a inclusão dos itens no contrato?", 'CONFIRMATION', null, [{
            "label": "Manualmente",
            "code": function () {
                General.widgetNewRow(widget);
                widget.currentRow.NRMOEDA = getWidgetGridContrato().currentRow.NRMOEDA;
                this.funcoesItens(widget);
            }.bind(this)
        }, {
            "label": "Formação de Preços",
            "code": function () {
                this.abrirWdgFormacaoPrec();
                this.integracaoTecFoodOnChange();
                this.integracaoHCMItensOnChange();
            }.bind(this)
        }, {
            "label": "Cancelar",
            "code": function () { }
        }]);
    };

    /**
     * Verifica se a formação de preço selecionada está habilitada para 
     * permitir a inserção um serviço
     * @params{Object} widget widget item formação de preço
     **/
    this.verificaAcessibilidadeFieldServ = function (widget) {
        if (widget.currentRow.NRFORMPRECOSERVICO && widget.currentRow.NRFORMPRECOSERVICO.length === 1) {
            widget.getField("NMSERVORCA").readOnly = false;
        }
        else {
            widget.getField("NMSERVORCA").readOnly = true;
            widget.currentRow.NMSERVORCA = "";
            widget.currentRow.CDSERVORCA = "";
        }
    };

    /**
     * Abre a widget que permite a edição do cálculo da formação de preço
     * @params{Object} widget widget Item Formação de preço
     **/
    this.abreWidgetEdicaoCalculoItens = function (widget) {
        var tabbedFormacao = templateManager.container.getWidget("wdgTabbedPrincipalFormPreco");
        var itensCalculo = templateManager.container.getWidget("editaCalculoItensFormCont");
        var widgetContrato = templateManager.container.getWidget("gridContrato");

        ScreenService.closePopup();
        if (!widget.currentRow.NMFORMPRECOSERVICO || widget.currentRow.NMFORMPRECOSERVICO.length === 0) {
            ScreenService.showMessage("Não é possível editar os itens do cálculo sem indicar uma Formação de Preço.<br> Para continuar selecione uma Formação de Preço <br>");
        }
        else {
            if (widget.currentRow.NMFORMPRECOSERVICO && widget.currentRow.NMFORMPRECOSERVICO.length > 1) {
                ScreenService.showMessage("A edição do Cálculo de Formação de Preço só é permitida quando apenas um tipo de Formação de Preço é selecionado. <br>Para continuar escolha apenas uma Formação de Preço.<br>");
            }
            else {
                if (widget.currentRow.NRCALCFORMPRECSERV && widget.currentRow.NRCALCFORMPRECSERV.length > 0) {
                    ScreenService.showMessage("Não é possível editar os itens do cálculo após selecionar os Itens de Formação de Preço.<br>Para continuar desfaça a seleção de itens atual.<br>");
                }
                else {
                    var params = {
                        "NRSEQCONTRATO": widgetContrato.currentRow.NRSEQCONTRATO,
                        "NRITEMOCONTRATO": null
                    };

                    General.callRestEngine("product#/fct_itemcontrato_widget/deletaAlteracoesInutiFormCont", "Row", params).then(function () {
                        this.salvarAlteracoesCalculo(itensCalculo, true, null).then(function () {
                            tabbedFormacao.isVisible = true;
                            itensCalculo.isVisible = true;

                            ScreenService.openSwipe(tabbedFormacao).then(function () {
                                tabbedFormacao.openTab("editaCalculoItensFormCont");

                                itensCalculo.dataSourceFilter[0].value = widget.currentRow.NRFORMPRECOSERVICO[0];
                                itensCalculo.dataSourceFilter[1].value = templateManager.container.getWidget('gridContrato').currentRow.NRSEQCONTRATO;
                                Utilities.abreArvoreElementos(itensCalculo);

                                UtilitiesPropostaContrato.habilitaEdicaoWidgetItemFormacao(widget, true);
                            });
                        }.bind(this));
                    }.bind(this));
                }
            }
        }
    };

    /**
     * Realiza o controle dos campos de "Medida" de acordo 
     * com o field "Utiliza Medida?"
     * 
     * @param {object} widget widget de 
     * Itens para pegar o field "IDUSAMEDIDA".
     */

    this.bloquearDadosMedidaEdicao = function (widget) {
        var usamedida = widget.getField('IDUSAMEDIDA').value();

        /* Bloqueia a Modalidade de pagamento*/
        if (usamedida == 'N') {
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = true;
            widget.getField('DSTPMEDIDASERV').setValue(null);
            widget.getField('DSTPMEDIDASERV').validations = {};
            widget.getField('CDTPMEDIDASERV').setValue(null);
            widget.getField('CDTPMEDIDASERV').validations = {};
            /* Quantidade de Medida */
            widget.getField('QTMEDIDACALC').readOnly = true;
            widget.getField('QTMEDIDACALC').setValue(null);
            widget.getField('QTMEDIDACALC').validations = {};

        }
        else {
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = false;
            widget.getField('DSTPMEDIDASERV').validations = { "required": true };
            /* Quantidade de Medida */
            widget.getField('QTMEDIDACALC').readOnly = false;
            widget.getField('QTMEDIDACALC').validations = { "required": true };
        }
    };

    /**
     * Habilita a tela para receber um novo item de formação de preço
     * e seta o número de ordenação na tela
     * @params{Object} widget widget editar formação de preço
     **/
    this.adicionarItensFormacao = function (widget) {
        var nritemocontrato = null;
        var nrformprecoservico = null;
        var widgetItemContratro = getWidgetItemContrato();

        if (widget.mode === "view") {
            var nmElementoSup = widget.currentRow.DSREFECALCULO;
            var nrCalcServPai = widget.currentRow.NRCALCFORMPRECSERV;
            widget.newRow();
            this.bloquearDadosMedidaEdicao(widget);

            widget.currentRow.DSREFECALCULOPAI = nmElementoSup;
            widget.currentRow.NRCALCFORMPRECSERVPAI = nrCalcServPai;
        }
        else {
            widget.newRow();
            this.bloquearDadosMedidaEdicao(widget);
        }

        if (widget.name === "editaItensFormPrecoCont") {
            nritemocontrato = widgetItemContratro.currentRow.NRITEMOCONTRATO;
            nrformprecoservico = widget.dataSource.data[0].NRFORMPRECOSERVICO;
        }
        else {
            nrformprecoservico = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
            widget.template = "widget/form.html";
        }

        var params = {
            "NRFORMPRECOSERVICO": nrformprecoservico,
            "NRITEMOCONTRATO": nritemocontrato,
            "TABELA": (widget.dataSource.name === "product#/fct_calcformprecserv_widget") ? null : "FCT_CALCFORMPRECSERV_ITCONT"
        };

        General.callRestEngine('product#/fct_calcformprecserv_widget/recebeOrdenacao', "Row", params).then(function (response) {
            if (response.dataset.ORDENACAO) {
                var ordenacao = response.dataset.ORDENACAO.IDORDENACAO;
                ordenacao = ordenacao ? (ordenacao + 10) : 10;
                widget.currentRow.IDORDENACAO = ordenacao;
                widget.currentRow.NRFORMPRECOSERVICO = nrformprecoservico;
                widget.currentRow.NRITEMOCONTRATO = nritemocontrato;
            }
            else {
                widget.currentRow.IDORDENACAO = 10;
            }
        },
            function (error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });
    };

    /**
     * Realiza a validação  do campo "identificador". Não podem haver dois identificadores iguais em uma mesma composição.
     * @param {object} widget widget
     */

    this.validaIdentificador = function (widget) {
        var nrformprecoservico = null;
        var nritemocontrato = null;
        var IdItemCalculo = widget.currentRow.IDITEMCALCULO;
        var nrCalcFormPrecServ = widget.currentRow.NRCALCFORMPRECSERV;
        var widgetItemContrato = getWidgetItemContrato();
        /* === Ordenação === */

        if (widget.name === "editaItensFormPrecoCont") {
            nritemocontrato = widgetItemContrato.currentRow.NRITEMOCONTRATO;
            nrformprecoservico = widget.dataSource.data[0].NRFORMPRECOSERVICO;
        }
        else {
            nrformprecoservico = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
        }

        var params = {
            "NRFORMPRECOSERVICO": nrformprecoservico,
            "NRITEMOCONTRATO": nritemocontrato,
            "TABELA": (widget.dataSource.name === "product#/fct_calcformprecserv_widget") ? null : "FCT_CALCFORMPRECSERV_ITCONT",
            "IDITEMCALCULO": IdItemCalculo,
            "NRCALCFORMPRECSERV": nrCalcFormPrecServ
        };

        General.callRestEngine('product#/fct_calcformprecserv_widget/validaIdentificador', "Row", params).then(function (response) {

            var identificador = response.dataset.IDENTIFICADOR.VALIDA;
            if (identificador != 0) {
                widget.currentRow.NMELEMENTO = "",
                    widget.currentRow.IDITEMCALCULO = null;
                ScreenService.showMessage('O identificador "' + IdItemCalculo + '" já está sendo utilizado para outro elemento.');
            }

        }.bind(this),
            function (error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });
    };

    /**
     * Seta o número do item gerado durante a inserção do item no contrato
     * @params{Object} widget widget editar formaçao de preço
     * @params{Array} nrItensProp array de nr de itens gerados
     **/
    this.setNrItemFormacaoAux = function (widget, nrItensProp) {
        var rows = [];

        if (nrItensProp === null) {
            widget.dataSource.data.forEach(function (dado) {
                dado.NRITEMOCONTRATO = null;
            });
        }
        else {
            if (nrItensProp.length === 1) {
                widget.dataSource.data.forEach(function (dado) {
                    dado.NRITEMOCONTRATO = nrItensProp[0].NRITEMOCONTRATO;
                });
            }
            else {
                nrItensProp.forEach(function (item) {
                    widget.dataSource.data.filter(function (data) {
                        if (data.IDELEMENTO === item.IDELEMENTO) {
                            data.NRITEMOCONTRATO = item.NRITEMOCONTRATO;
                            rows.push(data);
                        }
                    });
                });

                widget.dataSource.data.forEach(function (data) {
                    rows.filter(function (row) {
                        if (data._parent && row === data._parent) {
                            if (rows.indexOf(data) === -1) {
                                data.NRITEMOCONTRATO = data._parent.NRITEMOCONTRATO;
                                rows.push(data);
                            }
                        }
                    });
                });
            }
        }
    };

    /**
     * Salva as alterações feitas nos elementos do cálculo
     * @params{Object} widget widget editar itens formação de preço
     **/
    this.salvarAlteracoesCalculo = function (widget, isNew, nrItensProp) {
        var deferred = ZHPromise.defer();
        if (isNew) {
            var params = {
                "NRFORMPRECOSERVICO": templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0],
                "NRSEQCONTRATO": templateManager.container.getWidget("gridContrato").currentRow.NRSEQCONTRATO,
                "NRITEMOCONTRATO": null
            };

            General.callRestEngine("product#/fct_itemcontrato_widget/insereAlteracaoFormPrecCont", "Row", params).then(function () {
                deferred.resolve();
            }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                    deferred.reject();
                });
        }
        else {
            var paramsEdit = [];
            try {
                this.setNrItemFormacaoAux(widget, nrItensProp);
                nrItensProp.forEach(function (item) {
                    var idItemCalculo = [];
                    var dadosItem = widget.dataSource.data.filter(function (dado) {
                        return dado.NRITEMOCONTRATO === item.NRITEMOCONTRATO;
                    });

                    dadosItem.forEach(function (dadoItem) {
                        idItemCalculo.push(dadoItem.IDITEMCALCULO);
                    });

                    var param = {
                        "NRFORMPRECOSERVICO": widget.dataSource.data[0].NRFORMPRECOSERVICO,
                        "NRSEQCONTRATO": templateManager.container.getWidget("gridContrato").currentRow.NRSEQCONTRATO,
                        "IDITEMCALCULO": idItemCalculo,
                        "NRITEMOCONTRATO": item.NRITEMOCONTRATO
                    };

                    paramsEdit.push(param);
                });
            }
            catch (error) {
                deferred.reject();
            }

            General.callRestEngine("product#/fct_itemcontrato_widget/editaItensFormacaoAuxCont", "DataSet", paramsEdit).then(function () {
                deferred.resolve();
            }).catch(function (error) {
                deferred.reject();
            });
        }

        return deferred.promise;
    };

    /**
     * Salva no grid as alterações feitas no item de formação de preco
     * @params{Object} widget widget editar calculo de formação de preço
     */
    this.salvarLocalmenteAlteracoesFormPreco = function (widget) {
        if (widget.isValid()) {
            var isnew = widget.currentRow.__is_new;
            var tipoCalculo = widget.currentRow.IDTIPOCALCULO;

            widget.currentRow.NRSEQCONTRATO = templateManager.container.getWidget("gridContrato").currentRow.NRSEQCONTRATO;
            widget.currentRow.NRFORMPRECOSERVICO = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
            widget.currentRow.NRITEMOCONTRATO = null;

            if (!widget.currentRow.DSREFECALCULO)
                widget.currentRow.DSREFECALCULO = widget.currentRow.IDITEMCALCULO + " - " + widget.currentRow.NMELEMENTO;

            UtilitiesPropostaContrato.calculaItem(widget, "FCT_CALCFORMPRECSERV_ITCONT", null, widget.currentRow.NRITEMOCONTRATO).then(function () {
                if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
                    widget.currentRow.__using_unit = widget.paramsNewCode.__using_unit || false;
                    widget.currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
                    widget.currentRow.__table_pk = widget.paramsNewCode.__table_pk;
                    widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
                    widget.currentRow.__cast_number = widget.paramsNewCode.__cast_number || false;
                    if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                        widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
                }

                widget.saveRow().then(function (response) {
                    if (response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0)
                        widget.currentRow.NRCALCFORMPRECSERV = response.dataset.columnsToReturn[0].NRCALCFORMPRECSERV;

                    if (tipoCalculo == 'S') {
                        var params = {
                            "NRCALCFORMPRECSERV": widget.currentRow.NRCALCFORMPRECSERV,
                            "NRITEMOCONTRATO": widget.currentRow.NRITEMOCONTRATO,
                            "TABELA": "FCT_CALCFORMPRECSERV_ITCONT"
                        };
                        General.callRestEngine('product#/fct_calcformprecserv_widget/calcSomatorioFilhos', "Row", params);
                    }

                    ScreenService.confirmMessage("Deseja recalcular a Formação de Preço?", 'question',
                        function (success) {
                            widget.template = "widget/tree/grid.html";
                            widget.list();
                            this.recalcularArvore(widget);
                        }.bind(this),
                        function () {
                            if (!isnew) {
                                widget.currentRow.changed = false;
                                UtilitiesPropostaContrato.habilitarViewItensFormacao(widget);
                            }
                            else {
                                widget.list();
                                widget.template = "widget/tree/grid.html";
                                Utilities.abreArvoreElementos(widget);
                            }
                        }
                    );
                }.bind(this));
            }.bind(this));
        }
    };

    /**
     * Manda recalcular a arvore de formação
     * @params{Object} widget widget formação de preço
     * @params{int} nrformprecoservico nr da formação de preço
     **/
    this.recalculaFormacaoItens = function (widget, nrformprecoservico, nritemocontrato) {
        var deferred = ZHPromise.defer();
        var params = {
            "NRFORMPRECOSERVICO": nrformprecoservico,
            "TABELA": "FCT_CALCFORMPRECSERV_ITCONT",
            "NRITEMOCONTRATO": nritemocontrato
        };

        General.callRestEngine("product#/fct_calcformprecserv_widget/recalcularArvore", "Row", params).then(function () {
            Utilities.abreArvoreElementos(widget).then(function () {
                ScreenService.showMessage("Processo realizado com sucesso!");
                deferred.resolve();
            });
        })
            .catch(function (error) {
                console.log(error);
                deferred.reject();
            });

        return deferred.promise;
    };

    /**
     * Recalcula a árvore de elementos
     * @params{Object} widget widget editar itens de formação de preço
     **/
    this.recalcularArvore = function (widget) {
        var widgetItemContrato = getWidgetItemContrato();
        if (widget.name === "editaItensFormPrecoCont") {
            this.recalculaFormacaoItens(widget, widget.dataSource.data[0].NRFORMPRECOSERVICO, widgetItemContrato.currentRow.NRITEMOCONTRATO).then(function () {
                this.recalculaVrItem(widget, null, null);
                widget.activate();
            }.bind(this))
                .catch(function (error) {
                    console.error(error);
                });
        }
        else {
            var nrformprecoservico = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
            this.recalculaFormacaoItens(widget, nrformprecoservico, null);
        }
    };

    /**
     * Salva os itens selecionados pelo usuário com as novas formaçoes de preço
     * @params{Object} widget widget editar itens formação de preço
     **/
    this.utilizarItensAlteradosFormPrec = function (widget) {
        var widgetItensFomacao = templateManager.container.getWidget("itemFormacaoPreco");
        var widgetContrato = templateManager.container.getWidget("gridContrato");
        var widgetItensCont = templateManager.container.getWidget("itemcontrato");
        var rows = widget.getCheckedRows();
        var itensSelecionados = [];
        var nrformprecoservico = widgetItensFomacao.currentRow.NRFORMPRECOSERVICO;
        var servico = null;

        if (widgetItensFomacao.currentRow.CDSERVORCA)
            servico = widgetItensFomacao.currentRow.CDSERVORCA;
        else
            servico = widgetItensFomacao.currentRow.CDSERVORCAFORM;

        if (widget.dataSource.data.length !== rows.length) {
            rows = UtilitiesPropostaContrato.buscaArvoreFormacaoCompleta(widget, rows);
        }

        rows.forEach(function (row) {
            rows.NRSEQCONTRATO = widgetContrato.currentRow.NRSEQCONTRATO;
            if (rows.indexOf(row._parent) == -1) {
                widgetItensFomacao.getField("NRCALCFORMPRECSERV").dataSource.addCheckedRows(row);
                itensSelecionados.push(row.NRCALCFORMPRECSERV);
            }
        });

        widgetItensFomacao.currentRow.NRCALCFORMPRECSERV = itensSelecionados;
        widget.dataSource.data = rows;

        UtilitiesPropostaContrato.habilitaEdicaoWidgetItemFormacao(widgetItensFomacao, false);
        this.salvaItemformacaopreco(widgetItensFomacao).then(function (response) {
            var nrItensProp = null;
            if (response)
                nrItensProp = response;
            widgetItensCont.getField("NMESTRUTURAM").isVisible = true;
            this.salvarAlteracoesCalculo(widget, false, nrItensProp).then(function () {
                widget.dataSource.clearCheckedRows();
                ScreenService.closePopup();
                ScreenService.closeSwipe().then(function () {
                    ScreenService.closeSwipe().then(function () {
                        widgetItensCont.activate();
                        widgetItensCont.reload();
                        widgetItensCont.getField("NMESTRUTURAM").isVisible = true;
                    });
                });
            }.bind(this))
                .catch(function () {
                    if (nrItensProp && nrItensProp.length > 0) {
                        var itens = [];
                        nrItensProp.forEach(function (item) {
                            itens.push(item.NRITEMOCONTRATO);
                        });

                        var params = {
                            "NRSEQCONTRATO": widgetContrato.currentRow.NRSEQCONTRATO,
                            "NRITEMOCONTRATO": itens
                        };

                        General.callRestEngine("product#/fct_itemcontrato_widget/deletarItensFormacao", "Row", params).then(function () {
                            widgetItensFomacao.getField("NRFORMPRECOSERVICO").dataSource.addCheckedRows(nrformprecoservico);
                            widgetItensFomacao.currentRow.NRFORMPRECOSERVICO = nrformprecoservico;
                            widgetItensFomacao.currentRow.CDSERVORCAFORM = servico;
                            ScreenService.closePopup();
                            ScreenService.showMessage("Não foi possivel utilizar o(s) item(ns) selecionados.<br/>Erro durante a inserção do(s) mesmo(s) no contrato.");
                        }.bind(this));
                    }
                }.bind(this));
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };

    /**
     * Filtra a field de elemento superior para aperecer apenas os elementos
     * já inseridos na árvore
     * @params{Object} widget widget editar formaçao de preço
     **/
    this.filtraElementoSuperiorItensFormacao = function (widget) {
        var widgetItemContrato = getWidgetItemContrato();
        if (!widget.currentRow.DSREFECALCULOPAI) {
            var nrformprecoservico = null;
            widget.currentRow.DSREFECALCULOPAI = "";
            if (widget.name === "editaItensFormPrecoCont") {
                nrformprecoservico = widget.dataSource.data[0].NRFORMPRECOSERVICO;
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[3].value = widgetItemContrato.currentRow.NRITEMOCONTRATO;
            }
            else {
                nrformprecoservico = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
            }

            if (widget.currentRow.NMELEMENTO) {
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[0].value = nrformprecoservico;
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[1].value = (widget.currentRow.NRCALCFORMPRECSERV) ? widget.currentRow.NRCALCFORMPRECSERV : -1;
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[2].value = templateManager.container.getWidget('gridContrato').currentRow.NRSEQCONTRATO;
                widget.getField("DSREFECALCULOPAI").reload();
            }
        }
    };

    /**
     * Verifica se o item do contrato faz parte de uma formação de preço 
     * auxiliar para habilita a widget de edição de formações auxiliares
     * @params{Object} widget widget item orcamento
     **/
    this.validaWidgetEditaFormAux = function (widget) {
        var params = [{
            name: "NRITEMOCONTRATO",
            operator: "=",
            value: widget.currentRow.NRITEMOCONTRATO
        }

        ];

        General.callRestEngine("product#/crud/fct_calcformprecserv_itcont_widget", "FilterData", params).then(function (response) {
            var wdgFormacao = templateManager.container.getWidget("editaItensFormPrecoCont");
            var situacao = templateManager.container.getWidget("gridContrato").currentRow.IDSITUACAOCONTRATO;
            if (!response.dataset || response.dataset.fct_calcformprecserv_itcont_widget.length == 0) {
                wdgFormacao.isVisible = false;
            }
            else {
                wdgFormacao.isVisible = true;
                if (situacao === "P" || (idAlteraContAprv === "S" && situacao === "A"))
                    wdgFormacao.actions.forEach(function (action) {
                        action.isVisible = true;
                    });
                else
                    wdgFormacao.actions.forEach(function (action) {
                        if (action.name !== "VoltarWdgFormPrec")
                            action.isVisible = false;
                    });
            }
        }.bind(this));

        var wdgContrato = getWidgetGridContrato();
        var edititem = getActionEditWidgetItemContrato();
        if (wdgContrato.currentRow.IDSITUACAOCONTRATO == "A") {
            edititem.isVisible = visibilidadeAcoes;
        }
    };

    this.validarPerfiladicionaritens = function () {
        var wdgContrato = getWidgetGridContrato();
        var newitem = getActionAddWidgetAdicItemContrato();
        var editItem = getActionAddWidgetEditItemContrato();
        if (wdgContrato.currentRow.IDSITUACAOCONTRATO == "A") {
            newitem.isVisible = visibilidadeAcoes;
            editItem.isVisible = visibilidadeAcoes;
        }
    };
    this.validaPerfilAdicAnexo = function () {
        var wdgContrato = getWidgetGridContrato();
        var newitem = getActionAddAnexo();
        if (wdgContrato.currentRow.IDSITUACAOCONTRATO == "A") {
            newitem.isVisible = visibilidadeAcoes;
        }
    };
    /**
     * Valida se o perfil de acesso pode adicionar ou alterar 
     * as informações do faturamento 
     **/
    this.validaPerfilAdicFaturamento = function () {
        var rowContrato = getWidgetGridContrato().currentRow;
        var newitem = getActionAdicFaturamento();
        if (!rowContrato.NRSEQCONTRATOPAI && rowContrato.IDGERADOPOR != 'CONTRATO') {
            newitem.isVisible = false;
        }
        else {
            if (rowContrato.IDSITUACAOCONTRATO == "A")
                newitem.isVisible = (visibilidadeAcoes && idAlteraContAprv == 'S');
        }
    };

    this.validaPerfilEditFaturamento = function () {
        var wdgContrato = getWidgetGridContrato();
        var newitem = getActionEditFaturamento();
        if (!wdgContrato.currentRow.NRSEQCONTRATOPAI && wdgContrato.currentRow.IDGERADOPOR != 'CONTRATO') {
            newitem.isVisible = false;
        }
        else {
            if (wdgContrato.currentRow.IDSITUACAOCONTRATO == "A")
                newitem.isVisible = (visibilidadeAcoes && idAlteraContAprv == 'S');
        }
    };

    this.ValidaPerfilAdicLocal = function () {
        var rowContrato = getWidgetGridContrato().currentRow;
        var newitem = getActionAdicLocalAtendimento();
        if (!rowContrato.NRSEQCONTRATOPAI && rowContrato.IDGERADOPOR != 'CONTRATO') {
            newitem.isVisible = false;
        }
        else {
            if (rowContrato.IDSITUACAOCONTRATO == "A")
                newitem.isVisible = (visibilidadeAcoes && idAlteraContAprv == 'S');
        }
    };
    /**
     * Edita o valor do item quando a formação de preço 
     * é alterada.
     * @params{Object} widget widget editar itens formação de preco
     **/
    this.recalculaVrItem = function (widget, elementos, acao) {
        var widgetItemContrato = getWidgetItemContrato();
        var valor = 0;
        if (!elementos) {
            elementos = widget.dataSource.data.filter(function (dado) {
                return dado.NRCALCFORMPRECSERVPAI === null;
            });
        }

        elementos.forEach(function (elemento) {
            valor = valor + elemento.VRTOTAL;
        });

        if (acao === "DELETE")
            valor = widgetItemContrato.currentRow.VRUNITITEMCONT - elementos[0].VRTOTAL;
        else
            if (acao === "INSERT")
                valor = widgetItemContrato.currentRow.VRUNITITEMCONT + elementos[0].VRTOTAL;

        if (widgetItemContrato.currentRow.VRUNITITEMCONT !== valor) {
            widgetItemContrato.currentRow.VRUNITITEMCONT = valor;
            widgetItemContrato.currentRow.VRTOTITEMCONT = valor;
            this.calcularValorTotalItem(widgetItemContrato);
            this.salvarItemContrato(widgetItemContrato).then(function () {
                widget.activate();
                Utilities.abreArvoreElementos(widget);
            });
        }
    };

    /**
     * Exclui um elemento da formçaõ de preço auxiliar e manda
     * editar o valor do item
     * @params{Object} widget widget editar itens formação de preço
     **/
    this.excluirFormacaoItens = function (widget) {
        var dado = [widget.currentRow];
        var params = [{
            name: "NRCALCFORMPRECSERVPAI",
            operator: "=",
            value: widget.currentRow.NRCALCFORMPRECSERV
        },
        {
            name: "NRSEQCONTRATO",
            operator: "=",
            value: widget.currentRow.NRSEQCONTRATO
        },
        {
            name: "NRITEMOCONTRATO",
            operator: "=",
            value: widget.currentRow.NRITEMOCONTRATO
        },
        {
            name: "NRFORMPRECOSERVICO",
            operator: "=",
            value: widget.currentRow.NRFORMPRECOSERVICO
        }
        ]

        General.callRestEngine("product#/crud/fct_calcformprecserv_itcont_widget", "FilterData", params).then(function (response) {
            if (response.dataset && response.dataset.fct_calcformprecserv_itcont_widget && response.dataset.fct_calcformprecserv_itcont_widget.length > 0)
                mensagem = "Apagar este elemento levará a exclusão dos elementos inferiores a ele.<br>Confirma exclusão?";
            else
                mensagem = "Delete Register?";

            ScreenService.confirmMessage(ScreenService.i18n(mensagem), "question", function () {
                params = {
                    "NRCALCFORMPRECSERV": widget.currentRow.NRCALCFORMPRECSERV,
                    "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                    "NRITEMOCONTRATO": widget.currentRow.NRITEMOCONTRATO,
                    "NRFORMPRECOSERVICO": widget.currentRow.NRFORMPRECOSERVICO
                };

                General.callRestEngine("product#/fct_itemcontrato_widget/deletarArvoreFormPrecoItCont", "Row", params).then(function () {
                    ScreenService.confirmMessage(ScreenService.i18n("Deseja recalcular a Formação de Preço?"), 'question', function (success) {
                        this.recalcularArvore(widget);
                    }.bind(this));
                    ScreenService.closeSwipe().then(function () {
                        widget.activate();
                        this.recalculaVrItem(widget, dado, "DELETE");
                    }.bind(this));
                }.bind(this))
                    .catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
            }.bind(this));
        }.bind(this));
    };

    /**
     * salva as edições feitas nas formações auxiliares dos itens
     * @params{Object} widget widget editar itens formação na aba itens
     **/
    this.salvarEdicaoItFormacao = function (widget) {
        var isNew = widget.currentRow.__is_new;
        var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
        if (!widget.currentRow.DSREFECALCULO)
            widget.currentRow.DSREFECALCULO = widget.currentRow.IDITEMCALCULO + " - " + widget.currentRow.NMELEMENTO;
        UtilitiesPropostaContrato.calculaItem(widget, "FCT_CALCFORMPRECSERV_ITCONT", null, widget.currentRow.NRITEMOCONTRATO).then(function () {
            General.widgetSave(widget).then(function (response) {
                if (response.dataset && response.dataset.columnsToReturn.length > 0) {
                    widget.currentRow.NRSEQCONTRATO = response.dataset.columnsToReturn[0].NRSEQCONTRATO;
                    widget.currentRow.VRTOTAL = response.dataset.columnsToReturn[0].VRTOTAL;
                    widget.currentRow.NRCALCFORMPRECSERV = response.dataset.columnsToReturn[0].NRCALCFORMPRECSERV;
                    widget.currentRow.NRFORMPRECOSERVICO = response.dataset.columnsToReturn[0].NRFORMPRECOSERVICO;
                    widget.currentRow.NRITEMOCONTRATO = response.dataset.columnsToReturn[0].NRITEMOCONTRATO;
                    widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                }
                if (tipoCalculo == 'S') {
                    var params = {
                        "NRCALCFORMPRECSERV": widget.currentRow.NRCALCFORMPRECSERV,
                        "NRITEMOCONTRATO": widget.currentRow.NRITEMOCONTRATO,
                        "TABELA": "FCT_CALCFORMPRECSERV_ITCONT"
                    };
                    General.callRestEngine('product#/fct_calcformprecserv_widget/calcSomatorioFilhos', "Row", params);
                }
                ScreenService.confirmMessage("Deseja recalcular a Formação de Preço?", 'question', function (success) {
                    if (!isNew)
                        ScreenService.closeSwipe();

                    widget.list();
                    this.recalcularArvore(widget, null);
                }.bind(this),
                    function () {
                        this.recalculaVrItem(widget, [widget.currentRow], "INSERT");
                        (!isNew) ? widget.view() : widget.list();
                    }.bind(this));

            }.bind(this))
                .catch(function (error) {
                    console.error(error);
                });
        }.bind(this))
    };

    /* --- FUNÇÕES DA PÁGINA LOCAL DE ATENDIMENTO --- */
    this.validaCnpj = function (widget) {
        var cnpj = widget.currentRow.NRINSCRICAOPARC;
        var testaCnpj = General.validateCNPJ(cnpj);

        if (testaCnpj)
            widget.currentRow.CDTIPOINSCRICAO = 'CNPJ';
        else
            widget.currentRow.CDTIPOINSCRICAO = 'LIVRE';
    };

    /**
     * Realiza à rotina de inserção de todas as informações em suas respectivas tabelas.
     * 
     * @param {object} widget widget 
     */
    this.inserirNvLocalAtend = function (widget) {

        this.validaCnpj(widget);
        var params = {
            "NMESTRUTURAM": widget.currentRow.NMESTRUTURAMNV,
            "NRESTRUTURAMSUP": widget.currentRow.NRESTRUTURAMSUP,
            "NRTIPOESTRUTURA": widget.currentRow.NRTIPOESTRUTURA,
            "NRINSCRICAOPARC": widget.currentRow.NRINSCRICAOPARC.replace(/[\D]+/g, ''),
            "CDCENTCUST": widget.currentRow.CDCENTCUST,
            "CDTIPOINSCRICAO": widget.currentRow.CDTIPOINSCRICAO,
            "DSENDERECO": widget.currentRow.DSENDERECO,
            "NRIMOVELENDERECO": widget.currentRow.NRIMOVELENDERECO,
            "DSCOMPLEENDERECO": widget.currentRow.DSCOMPLEENDERECO,
            "CDPAIS": widget.currentRow.CDPAIS,
            "SGESTADO": widget.currentRow.SGESTADO,
            "CDMUNICIPIO": widget.currentRow.CDMUNICIPIO,
            "CDBAIRRO": widget.currentRow.CDBAIRRO,
            "NMBAIRROENDERECO": widget.currentRow.NMBAIRROENDERECO,
            "NRCEPENDERECO": widget.currentRow.NRCEPENDERECO,
            "DSREFERENCIAENDE": widget.currentRow.DSREFERENCIAENDE
        };

        General.callRestEngine('product#/estruturam_widget/inserirTudo', "Row", params).then(function (response) {
            widget.currentRow.NRESTRUTURAM = response.dataset.estruturam_widget[0];
            General.widgetSave(widget).then(function () {
                ScreenService.closeSwipe().then(function () {
                    widget.currentRow.NOVOLOCALATEND = "N";
                    this.habilitaNovoLocalAtend(widget);
                    widget.reload();
                }.bind(this));
            }.bind(this));
        }.bind(this), function (error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Salva o novo registro da página "Local de Atendimento"
     * 
     * @param {object} widget widget de local de atendimento 
     * para salvar o novo registro.
     */
    this.salvarLocalAtend = function (widget) {
        if (widget.isValid()) {
            if (widget.currentRow.__is_new) {
                if (widget.currentRow.NOVOLOCALATEND === 'S') {
                    this.inserirNvLocalAtend(widget);
                }
                else {
                    Utilities.inserirMultiplos(widget, "NRESTRUTURAM").then(function () {
                        ScreenService.closeSwipe().then(function () {
                            widget.currentRow.NOVOLOCALATEND = "N";
                            this.habilitaNovoLocalAtend(widget);
                            widget.reload();
                        }.bind(this));
                    }.bind(this),
                        function (error) {
                            Utilities.showErrorMessage(error);
                        });
                }
            }
            else {

        
                if (widget.backupRow && widget.backupRow.IDCONSENDELOCALATNF != widget.currentRow.IDCONSENDELOCALATNF) {
                    var mudouIdConsEndeLocalAtNF = 'S';
                }
                else {
                    var mudouIdConsEndeLocalAtNF = 'N';
                }
                
                General.widgetSave(widget);

                if (mudouIdConsEndeLocalAtNF == 'S') {
             
                    var params = {
                        "NRSEQCONTRATO": getWidgetGridContrato().currentRow.NRSEQCONTRATO,
                        "NRTIPOORCAMENTO": getWidgetGridContrato().currentRow.NRTIPOORCAMENTO,
                        "IDAGRUPAESCALAHCM" : getWidgetGridContrato().currentRow.IDAGRUPAESCALAHCM
                    };
                    General.callRestEngine("product#/fct_itemcontrato_widget/regerarNegociacaoValores", "Row", params).then(function () {
                        ScreenService.showMessage(ScreenService.i18n("Atenção! <br/><br/>Os registros da aba \"Negociação de Valores\" foram regerados devido a alteração do parâmetro \"Agrupamento de Parcelas\"."));
                    })
                    .catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            }
        }
    };

    /* Filtra a field de estrutura superior para aperecer apenas locais de atendimento 
     * do contrato selecionado
     * @params{Object} field
     */
    this.filtraEstrutSuperior = function (fieldEstSuperior) {
        fieldEstSuperior.dataSourceFilter[0].value = fieldEstSuperior.widget.currentRow.NRSEQCONTRATO;
        fieldEstSuperior.dataSourceFilter[1].value = fieldEstSuperior.widget.currentRow.NRORG;
        fieldEstSuperior.reload();
    };

    this.adicionarLocalDeAtendimento = function (wdgLocalAtend) {
        General.widgetNewRow(wdgLocalAtend);
        wdgLocalAtend.getField("NRESTRUTURAM").showOnForm = true;
        wdgLocalAtend.getField("NRESTRUTURAM").validations = { "required": true };
        wdgLocalAtend.getField("NMESTRUTURAM").showOnForm = false;

        var wdgContrato = getWidgetGridContrato();
        wdgLocalAtend.getField("IDLOCALTOMADORNF").readOnly = (wdgContrato.currentRow.IDAGRUPPARC != "2");
        wdgLocalAtend.getField("IDCONSENDELOCALATNF").readOnly = (wdgContrato.currentRow.IDAGRUPPARC != "2");

        //var pos = Utilities.getFieldGroupPosition(wdgLocalAtend.fieldGroups, "fdg_novo_local_atendimento");
        //wdgLocalAtend.fieldGroups[pos].isVisible = true;
    };

    this.editarLocalDeAtendimento = function (wdgLocalAtend) {
        General.widgetEdit(wdgLocalAtend);
        wdgLocalAtend.getField("NRESTRUTURAM").showOnForm = false;
        wdgLocalAtend.getField("NRESTRUTURAM").validations = {};
        wdgLocalAtend.getField("NMESTRUTURAM").showOnForm = true;

        var wdgContrato = getWidgetGridContrato();
        wdgLocalAtend.getField("IDLOCALTOMADORNF").readOnly = (wdgContrato.currentRow.IDAGRUPPARC != "2" || wdgLocalAtend.currentRow.IDCONVERTIDOCLIENTE == "N");
        wdgLocalAtend.getField("IDCONSENDELOCALATNF").readOnly = (wdgContrato.currentRow.IDAGRUPPARC != "2");

        //var pos = Utilities.getFieldGroupPosition(wdgLocalAtend.fieldGroups, "fdg_novo_local_atendimento");
        //wdgLocalAtend.fieldGroups[pos].isVisible = false;
    };

    this.controlaIdConsEndeLocalAtNf = function (wdgLocalAtend) {

        if (wdgLocalAtend.currentRow.IDLOCALTOMADORNF == 'S'){
            wdgLocalAtend.currentRow.IDCONSENDELOCALATNF = 'S';
            wdgLocalAtend.getField("IDCONSENDELOCALATNF").readOnly = true;
        }
        else {
            wdgLocalAtend.getField("IDCONSENDELOCALATNF").readOnly = false;
        }
    }
    

    this.deletarLocalAtend = function (widget) {
        var estruturas = [widget.currentRow.NRESTRUTURAM];
        var possuiFilhos = false;
        if (widget.dataSource.data) {
            widget.dataSource.data.forEach(function (dado) {
                if ((dado.NRESTRUTURAMSUP != null && dado.NRESTRUTURAMSUP === widget.currentRow.NRESTRUTURAM) || estruturas.includes(dado.NRESTRUTURAMSUP)) {
                    possuiFilhos = true;
                    estruturas.push(dado.NRESTRUTURAM);
                }
            });
        }

        var params = [
            {
                name: "NRSEQCONTRATO",
                operator: "=",
                value: widget.currentRow.NRSEQCONTRATO
            },
            {
                name: "NRESTRUTURAM",
                operator: "IN",
                value: estruturas
            }
        ];

        General.callRestEngine("product#/crud/fct_itemcontrato_widget", "FilterData", params).then(function (response) {
            if (response.dataset.fct_itemcontrato_widget && response.dataset.fct_itemcontrato_widget.length > 0) {
                ScreenService.showMessage("Operação bloqueada!<br><br> Não é possível excluir os locais de atendimento porque eles estão relacionados aos itens do contrato.");
            }
            else {
                if (possuiFilhos) {
                    this.deletarPaieFilhos(widget);
                }
                else {
                    General.deleteCrud(widget).then(function () {
                        widget.reload();
                    })
                        .catch(function (error) {
                            Utilities.showErrorMessage(error);
                        });
                }
            }
        }.bind(this));
    };


    /**
     * Exclui os local de atandimento e os filhos.
     * 
     * @param {object} widget widget
     */

    this.deletarPaieFilhos = function (widget) {
        var nrEstruturam = widget.currentRow.NRESTRUTURAM;
        var nrSeqContrato = widget.currentRow.NRSEQCONTRATO;

        ScreenService.confirmMessage("A exclusão deste local de atendimento também irá excluir os locais de atendimento inferiores a ele. Confirma exclusão?", "question", function () {
            var params = {
                "NRESTRUTURAM": nrEstruturam,
                "NRSEQCONTRATO": nrSeqContrato
            };

            General.callRestEngine('product#/fct_relacestruturacont_widget/deletarPaieFilhos', "Row", params).then(function () {
                ScreenService.showMessage('Exclusão realizada com sucesso.');
                ScreenService.closeSwipe();
                widget.reload();
            },
                function (error) {
                    console.log(error);
                    ScreenService.showMessage(error.data.error);
                });
        }.bind(this));
    };

    /**
     * Realiza o controle de visibilidade das actions.
     * Se a situação for "Aprovado" ou "Encerrado" as 
     * actions de edit , insert e delete são blqueadas. 
     * 
     * @param {Object} widget
     * @param {boolean} visivel
     */
    this.controlaVisibilidadeLocalAt = function (widget) {
        var widgetContratoGrid = getWidgetGridContrato();
        var situacao = widgetContratoGrid.currentRow.IDSITUACAOCONTRATO;
        if (situacao == 'E' || situacao == 'R') {
            this.setVisibilidadeDasActionsLocalAt(widget, false, situacao);
        }
        else {
            this.setVisibilidadeDasActionsLocalAt(widget, true, situacao);
        }
    };

    /**
     * Ativa/desativa todas as actions de uma widget, menos as do tipo "back".
     * 
     * @param {Object} widget
     * @param {boolean} visivel
     */
    this.setVisibilidadeDasActionsLocalAt = function (widget, visivel, situacao) {
        var tipos = ["add", "edit", "delete", "never_view"];
        if (situacao === "A") {
            widget.getAction("add").isVisible = (idAlteraContAprv === "S") ? true : false;
            widget.getAction("delete").isVisible = (idAlteraContAprv === "S") ? true : false;
            tipos = ["edit", "never_view"];
        }
        else {
            if (situacao == 'E' || situacao == 'R')
                tipos = ["add", "edit", "delete"];
        }

        widget.actions.forEach(function (action) {
            if (tipos.includes(action.name)) {
                action.isVisible = visivel;
            }
        });
    };


    /* --- FUNÇÕES DA PÁGINA FATURAMENTO --- */


    /**
     * Realiza a regra de mudança do campo "Alterado pelo Usuário?"
     * da página "Faturamento" para "Sim", caso sejam alterados os 
     * campos citados abaixo.
     * 
     * @param {object} widget widget de 
     * Itens para pegar os fields para cálculo
     */
    this.validaAltUsuarioFatura = function (widget) {
        var currentRow = widget.currentRow;
        var backupRow = widget.backupRow;

        if (currentRow.DTINICOMPET != backupRow.DTINICOMPET || currentRow.DTFINCOMPET != backupRow.DTFINCOMPET ||
            currentRow.DTEMISSAO != backupRow.DTEMISSAO || currentRow.DTVENCIMENTO != backupRow.DTVENCIMENTO ||
            currentRow.VRVENCIMENTO != backupRow.VRVENCIMENTO) {
            widget.currentRow.IDALTUSUARIO = 'S';
        }
    };


    /**
     * Realiza o controle de visibilidade das actions.
     * Se a situação for "Aprovado" ou "Encerrado" as 
     * actions de edit , insert e delete são blqueadas. 
     * 
     * @param {Object} widget
     * @param {boolean} visivel
     */
    this.controlaVisibilidadeFat = function (widget) {
        var widgetFaturamentoContrato = getWidgetFaturamentoContrato();
        var situacao = widgetFaturamentoContrato.currentRow.IDSITUVENCPREV;

        if (widget.name == 'faturamentocontrato') {
            situacao = widget.currentRow.IDSITUVENCPREV;
        }

        if (situacao != 'P') {
            this.setVisibilidadeDasActionsFat(widget, false);
        }
        else {
            this.setVisibilidadeDasActionsFat(widget, true);
        }
    };

    /**
     * Ativa/desativa todas as actions de uma widget, menos as do tipo "back".
     * 
     * @param {Object} widget
     * @param {boolean} visivel
     */
    this.setVisibilidadeDasActionsFat = function (widget, visivel) {
        var tipos = ["add", "edit", "delete"];
        widget.actions.forEach(function (action) {
            if (tipos.includes(action.name)) {
                action.isVisible = visivel;
            }
        });
    };


    /**
     * Realiza o controle de Percentual de rateio do field "VRPERCENTRATEIO"
     * 
     * @param {object} widget widget de 
     * Rateio para pegar os field "VRPERCENTRATEIO".
     */
    this.ValidaDescontoRatFatura = function (widget) {

        var vrdesconto = widget.getField('VRPERCENTRATEIO').value();

        /* Aplica Max e Min Value*/

        if (vrdesconto < 0 || vrdesconto > 100) {
            /* Desconto */
            widget.currentRow.VRDESCONTO = null;
        }
    };



    /* --- FUNÇÕES DA PÁGINA  ANEXOS --- */

    /**
     * Abre a janela para a escolha dos arquivos a serem anexados 
     * 
     * @param {object} widget widget de anexos.
     */
    this.downloadArquivo = function (widget) {
        var nmarquivo = widget.currentRow.NMARQUIVO;
        var dscaminho = widget.currentRow.DSCAMINHO;
        var type = widget.currentRow.DSTIPO;

        Utilities.downloadFile(nmarquivo, dscaminho, type);
    };

    /**
     * Abre o Swipe para realizar o Anexo dos arquivos
     * 
     * @param {object} widget widget de anexos
     */

    this.abrirAnexoSwipe = function (widget) {

        widget.isVisible = true;
        ScreenService.openSwipe(widget);
    };

    /**
     * Realiza o Anexo dos arquivos
     * 
     * @param {object} widget widget de anexos
     */
    this.uploadArquivo = function (widget) {
        var currentRow = widget.currentRow;
        var widgetContratoAnexos = getWidgetContratoAnexos()

        var files = currentRow.DSCAMINHO;

        if (currentRow.DSCAMINHO && currentRow.DSCAMINHO.length !== 0) {
            for (var i = 0; i < files.length; i++) {
                this.setValoresAnexos(widget, files, i);

                General.widgetSave(widget).then(function () {

                    this.fecharAnexoSwipe(widget);
                    widgetContratoAnexos.reload();

                }.bind(this));
            }
        }
        else {
            ScreenService.showMessage("Selecione um arquivo para anexar ao contrato.");
        }
    };

    /**
     * Fecha o Swipe para realizar o Anexo dos arquivos
     * 
     * @param {object} widget widget de anexos
     */
    this.fecharAnexoSwipe = function (widget) {
        var contratoAnexos = templateManager.container.getWidget("contratoAnexos");
        ScreenService.closeSwipe().then(function () {
            widget.isVisible = false;
            contratoAnexos.list();
            contratoAnexos.activate();
        });
    };

    /**
     * Recebe os valores dos campos a serem salvos no registro de anexo.
     * 
     * @param {object} widget widget de anexos
     */
    this.setValoresAnexos = function (widget, dscaminho, indexFile) {
        var widgetGridContrato = getWidgetGridContrato();

        widget.currentRow.NRSEQCONTRATO = widgetGridContrato.currentRow.NRSEQCONTRATO;
        widget.currentRow.NMARQUIVO = dscaminho[indexFile].name;
        widget.currentRow.DSTIPO = dscaminho[indexFile].name.replace(/^(.*)\.(.*)$/, "$2");
        widget.currentRow.VRTAMANHO = dscaminho[indexFile].size;
        widget.currentRow.DSCAMINHO = [dscaminho[indexFile]];
        widget.currentRow.NRORG = widgetGridContrato.currentRow.NRORG;

    };

    /**
     * Realiza o controle de visibilidade das actions.
     * Se a situação for "Aprovado" ou "Encerrado" as 
     * actions de edit , insert e delete são blqueadas. 
     * 
     * @param {Object}  widget
     * @param {boolean} visivel
     */
    this.controlaVisibilidadeAnexos = function (widget) {
        var widgetGridContrato = getWidgetGridContrato();
        var situacao = widgetGridContrato.currentRow.IDSITUACAOCONTRATO;

        if (situacao == 'E' || situacao == 'R') {
            this.setVisibilidadeDasActionsAnexos(widget, false);
        }
        else {
            this.setVisibilidadeDasActionsAnexos(widget, true);
        }
    };

    /**
     * Ativa/desativa todas as actions de uma widget, menos as do tipo "back".
     * 
     * @param {Object} widget
     * @param {boolean} visivel
     */
    this.setVisibilidadeDasActionsAnexos = function (widget, visivel) {

        var tipos = ["add", "edit", "delete"];

        widget.actions.forEach(function (action) {
            if (tipos.includes(action.name)) {
                action.isVisible = visivel;
            }
        });
    };

    /* --- FUNÇÕES DA PÁGINA ALTERAR PERIODO VIGENTE --- */

    /**
     * abre a tela "Alterar Vigencia do Contrato"
     */

    this.abrirWdgAlteraVig = function () {
        ScreenService.closePopup();
        var alterarVigencia = templateManager.container.getWidget("alterarVigencia");
        alterarVigencia.isVisible = true;
        ScreenService.openSwipe(alterarVigencia).then(function () {
            alterarVigencia.edit();
            this.setDadosPadraoAlterarVigencia(alterarVigencia);
        }.bind(this));
    };

    /**
     * Seta os dados padrões de alteração de vigência.
     * 
     * @param {object} widget widget "Alterar Vigencia do Contrato"
     */
    this.setDadosPadraoAlterarVigencia = function (widget) {
        widget.currentRow.__is_new = true;
        var contrato = templateManager.container.getWidget('gridContrato');

        /* Data inicial Contrato */
        var dtinicialcont = contrato.currentRow.DTINICIOCONTRATO;
        /* Data Final Contrato */
        var dtfinalcont = contrato.currentRow.DTFIMCONTRATO;
        /* Data inicial Vigencia Contrato */
        var dtinivigcont = contrato.currentRow.DTINIVIGENCONT;
        /* Data final vigencia Contrato */
        var dtfinvigcont = contrato.currentRow.DTFINVIGENCONT;

        if (dtinivigcont != null) {
            widget.currentRow.DTNVDATAINICIAL = dtinivigcont;
        }
        else {
            widget.currentRow.DTNVDATAINICIAL = dtinicialcont;
        }

        if (dtfinvigcont != null) {
            widget.currentRow.DTNVDATAFINAL = dtfinvigcont;
        }
        else {
            widget.currentRow.DTNVDATAFINAL = dtfinalcont;
        }

        widget.currentRow.NRSEQPRORRCONT = null;
        widget.currentRow.DSPRORRCONT = null;
        widget.currentRow.IDPRORRCONT = null;
        widget.currentRow.NRSEQCONTRATO = contrato.currentRow.NRSEQCONTRATO;
        widget.currentRow.IDDTATIVO = 'S';

        this.validaIdProrrCont(widget);
    };

    /**
     * Calcula Valor do field IDPRORRCONT;
     * 
     * @param {object} widget widget "Alterar Vigencia do Contrato"
     */
    this.validaIdProrrCont = function (widget) {
        /* Data inicial Contrato */
        var dtinicialcont = templateManager.container.getWidget("gridContrato").currentRow.DTINICIOCONTRATO;

        /* Data Final Contrato */
        var dtfinalcont = templateManager.container.getWidget("gridContrato").currentRow.DTFIMCONTRATO;

        /* Data inicial Vigencia Contrato */
        var dtinivigcont = templateManager.container.getWidget("gridContrato").currentRow.DTINIVIGENCONT;

        /* Data final vigencia Contrato */
        var dtfinvigcont = templateManager.container.getWidget("gridContrato").currentRow.DTFINVIGENCONT;

        /* Nova Data inicial */
        var dtinicialnova = widget.getField('DTNVDATAINICIAL').value();
        dtinicialnova = moment(dtinicialnova, "DD/MM/YYYY");

        /* Nova Data final */
        var dtfinalnova = widget.getField('DTNVDATAFINAL').value();
        dtfinalnova = moment(dtfinalnova, "DD/MM/YYYY");

        var retorno = null;

        /*Se a data de vigencia estiver preenchida, o teste é realizado com ela*/
        if (dtinivigcont != null && dtfinvigcont != null) {
            dtinivigcont = moment(dtinivigcont, "DD/MM/YYYY");
            var diffDtIni = dtinivigcont.diff(dtinicialnova);

            /* Se a Data inicial de vigência for menor que a nova data => Prorrogação */
            if (diffDtIni < 0) {
                retorno = 'P';
            }
            /* Se a Data inicial de vigência for maior que a nova data => Antecipação */
            else if (diffDtIni > 0) {
                retorno = 'A';
            }
            else if (diffDtIni == 0) {
                dtfinvigcont = moment(dtfinvigcont, "DD/MM/YYYY");
                var diffDtFin = dtfinvigcont.diff(dtfinalnova);

                /* Se a Data Final de vigência for menor que a nova data => Prorrogação */
                if (diffDtFin < 0) {
                    retorno = 'P';
                }
                /* Se a Data Final de vigência for maior que a nova data => Antecipação */
                else if (diffDtFin > 0) {
                    retorno = 'A';
                }
            }
        }
        else {
            dtinicialcont = moment(dtinicialcont, "DD/MM/YYYY");
            var diffDtIni = dtinicialcont.diff(dtinicialnova);

            /* Se a Data inicial for menor que a nova data => Prorrogação */
            if (diffDtIni < 0) {
                retorno = 'P';
            }
            /* Se a Data inicial for maior que a nova data => Antecipação */
            else if (diffDtIni > 0) {
                retorno = 'A';
            }
            else if (diffDtIni == 0) {
                dtfinalcont = moment(dtfinalcont, "DD/MM/YYYY");
                var diffDtFin = dtfinalcont.diff(dtfinalnova);

                /* Se a Data Final for menor que a nova data => Prorrogação */
                if (diffDtFin < 0) {
                    retorno = 'P';
                }
                /* Se a Data Final for maior que a nova data => Antecipação */
                else if (diffDtFin > 0) {
                    retorno = 'A';
                }
            }
        }

        widget.currentRow.IDPRORRCONT = retorno;

    };


    /**
     * Realiza a validação das novas datas de vigência do contrato
     * 
     * @param {object} widget widget "Alterar Vigencia do Contrato"
     */
    this.validaDatas = function (widget) {

        /* Nova Data inicial */
        var dtinicialnova = widget.getField('DTNVDATAINICIAL').value();
        dtinicialnova = moment(dtinicialnova, "DD/MM/YYYY");

        /* Nova Data final */
        var dtfinalnova = widget.getField('DTNVDATAFINAL').value();
        dtfinalnova = moment(dtfinalnova, "DD/MM/YYYY");

        var retorno = null;

        var diffDatas = dtinicialnova.diff(dtfinalnova);

        if (diffDatas > 0) {

            retorno = false;
        }
        else {
            retorno = true;
        }

        return retorno;

    };

    /**
     * Fecha a tela "Alterar Vigencia do Contrato"
     * 
     * @param {object} widget
     */

    this.fecharWdgAlterarVigencia = function (widget) {
        ScreenService.closeSwipe().then(function () {
            widget.isVisible = false;
        });
        templateManager.container.getWidget("gridContrato").activate();
    };

    /**
     * Salva a nova vigência do contrato e atualiza as parcelas do contrato;
     * 
     * @param {object} widget widget "Alterar Vigencia do Contrato"
     */
    this.salvaPeriodoVigente = function (widget) {
        var datasValidas = this.validaDatas(widget);

        if (datasValidas == true) {

            //this.setDadosPadraoAlterarVigencia(widget);

            this.validaIdProrrCont(widget);

            if (widget.currentRow.IDPRORRCONT != null) {

                ScreenService.confirmMessage(ScreenService.i18n("Ao alterar o período de vigência do contrato, as parcelas e medições de faturamento serão regeradas. <br><br>Deseja realmente alterar o período de vigência do contrato?"), 'question', function (success) {

                    widget.currentRow.__columns_to_return = ["NRSEQPRORRCONT", "DTNVDATAINICIAL", "DTNVDATAFINAL", "NRSEQCONTRATO", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];

                    General.widgetSave(widget).then(function (response) {
                        if (response.dataset) {
                            widget.currentRow.NRSEQPRORRCONT = response.dataset.columnsToReturn[0].NRSEQPRORRCONT;
                            widget.currentRow.DTNVDATAINICIAL = response.dataset.columnsToReturn[0].DTNVDATAINICIAL;
                            widget.currentRow.DTNVDATAFINAL = response.dataset.columnsToReturn[0].DTNVDATAFINAL;
                            widget.currentRow.NRSEQCONTRATO = response.dataset.columnsToReturn[0].NRSEQCONTRATO;
                            widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                            widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                            widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                            widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;

                            /* Altera dados do Contrato */
                            this.alteraVigenciaContrato(widget);

                            /* Gera Parcelas do contrato */
                            Sm00004000_Contrato_Service.geraParcelasContrato(widget.currentRow.NRSEQCONTRATO);

                            this.fecharWdgAlterarVigencia(widget);

                            var widContrato = templateManager.container.getWidget("gridContrato");

                            widContrato.reload();

                            ScreenService.showMessage("Período de vigência do contrato alterado com sucesso.");

                        }

                    }.bind(this));
                }.bind(this));
            }
            else {
                ScreenService.showMessage("Operação bloqueada!<br><br> Não houve alteração na vigência do contrato.");
            }
        }
        else {

            ScreenService.showMessage("Operação bloqueada!<br><br> Nova data inicial não pode ser maior que a nova data final.");

        }
    };


    /* Valida se foi realizado o reajuste dos valores antes de alterar o periodo e regerar das parcelas de faturamento */
    this.validaReajusteAlteraVig = function (widget) {

        var params = {
            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
            "DTINICIO": widget.currentRow.DTNVDATAINICIAL,
            "DTFINAL": widget.currentRow.DTNVDATAFINAL
        };

        General.callRestEngine('product#/fct_contrato_widget/validaReajuste', "Row", params).then(function (response) {

            /* Se buscarem dados, significa que a validação de reajsute está ativa. Neste caso, deve ser verificado se o contrato já teve reajuste realizado. */
            if (response.dataset.retorno.valida.length > 0) {

                if (response.dataset.retorno.valida[0].VALIDA > 0) {
                    this.salvaPeriodoVigenteNovoFat(widget);
                }
                else {

                    ScreenService.showMessage("Operação bloqueada!<br><br> Não foi realizado o reajuste dos valores para este contrato dentro do período que está sendo alterado. <br><br> Deve ser realizado o reajuste dos valores do contrato antes de ser alterado o período de vigência do contrato.");
                    ScreenService.closePopup();

                }
            }
            /* Se não buscarem dados, significa que o contrato ainda não teve as datas alteradas ou a validação de reakjuste está desabilitada. Neste caso, a rotina segue normalmente. */
            else {

                this.salvaPeriodoVigenteNovoFat(widget);

            }

        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });

    }


    /**
     * Salva a nova vigência do contrato e atualiza as parcelas do contrato;
     * 
     * @param {object} widget widget "Alterar Vigencia do Contrato"
     */
    this.salvaPeriodoVigenteNovoFat = function (widget) {
        var datasValidas = this.validaDatas(widget);

        if (datasValidas == true) {

            //this.setDadosPadraoAlterarVigencia(widget);

            this.validaIdProrrCont(widget);

            if (widget.currentRow.IDPRORRCONT != null) {
                ScreenService.confirmMessage(ScreenService.i18n("Ao alterar o período de vigência do contrato, as parcelas e medições de faturamento serão regeradas. <br><br>Deseja realmente alterar o período de vigência do contrato?"), 'question', function () {

                    widget.currentRow.__columns_to_return = ["NRSEQPRORRCONT", "DTNVDATAINICIAL", "DTNVDATAFINAL", "NRSEQCONTRATO", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];

                    General.widgetSave(widget).then(function (response) {
                        if (response.dataset) {
                            widget.currentRow.NRSEQPRORRCONT = response.dataset.columnsToReturn[0].NRSEQPRORRCONT;
                            widget.currentRow.DTNVDATAINICIAL = response.dataset.columnsToReturn[0].DTNVDATAINICIAL;
                            widget.currentRow.DTNVDATAFINAL = response.dataset.columnsToReturn[0].DTNVDATAFINAL;
                            widget.currentRow.NRSEQCONTRATO = response.dataset.columnsToReturn[0].NRSEQCONTRATO;
                            widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                            widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                            widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                            widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;

                            /* Altera dados do Contrato */
                            this.alteraVigenciaContratoNovo(widget).then(function () {
                                /* Gera Parcelas do contrato */

                                var params = {
                                    "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO
                                };

                                General.callRestEngine('product#/fct_contrato_widget/geraParcelasContratoNovoFat', "Row", params).then(function () {

                                    this.fecharWdgAlterarVigencia(widget);

                                    var widContrato = templateManager.container.getWidget("gridContrato");

                                    widContrato.mode = 'list';
                                    widContrato.reload();
                                    widContrato.mode = 'view';
                                    ScreenService.showMessage("Período de vigência do contrato alterado com sucesso.");
                                }.bind(this))
                                    .catch(function (error) {
                                        Utilities.showErrorMessage(error);
                                    });

                            }.bind(this));
                        }
                    }.bind(this));
                }.bind(this));
            }
            else {
                ScreenService.showMessage("Operação bloqueada!<br><br> Não houve alteração na vigência do contrato.");
            }
        }
        else {
            ScreenService.showMessage("Operação bloqueada!<br><br> Nova data inicial não pode ser maior que a nova data final.");
        }
    };



    /**
     * A função "alteraDadosProrrog" altera dados das vigencias de contrato anteritores 
     * 
     * @param {object} widget widget "Alterar Vigencia do Contrato"
     */

    this.alteraDadosProrrog = function (widget) {

        var seqContrato = widget.currentRow.NRSEQCONTRATO;

        Sm00004000_Contrato_Service.validaDadosProrrog(seqContrato).then(function (response) {
            var seqprorrog = response.dataset.NRSEQPRORRCONT.SEQPRORRCONT;

            Sm00004000_Contrato_Service.alteraDadosProrrog(seqContrato, seqprorrog).then(null).catch(function (response) {
                ScreenService.showMessage(response.data.error);
            });
        })
            .catch(function (response) {
                ScreenService.showMessage(response.data.error);
            });

    };


    /**
     * A função "alteraVigenciaContrato" altera os dados de vigência do contrato
     * 
     * @param {object} widget widget "Alterar Vigencia do Contrato"
     */
    this.alteraVigenciaContrato = function (widget) {
        var widgetContrato = getWidgetGridContrato();

        var params = {
            "DTINICVIGCONTR": widget.currentRow.DTNVDATAINICIAL,
            "DTFINVIGCONTR": widget.currentRow.DTNVDATAFINAL,
            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO
        };

        General.callRestEngine("product#/fct_prorrogcontrato_widget/alteraVigenciaContrato", "Row", params).then(function (response) {

            /* Altera dados das vigencias de contrato anteriores */
            this.alteraDadosProrrog(widget);

        }.bind(this)).catch(function (response) {
            ScreenService.showMessage(response.data.error);
        });
    };



    /**
     * A função "alteraVigenciaContrato" altera os dados de vigência do contrato
     * 
     * @param {object} widget widget "Alterar Vigencia do Contrato"
     */
    this.alteraVigenciaContratoNovo = function (widget) {
        var deferred = ZHPromise.defer();
        var widgetContrato = getWidgetGridContrato();

        var params = {
            "NRORG": widget.currentRow.NRORG,
            "IDUTILIZACOMP": widgetContrato.currentRow.IDUTILIZACOMP,
            "DTINICVIGCONTR": widget.currentRow.DTNVDATAINICIAL,
            "DTFINVIGCONTR": widget.currentRow.DTNVDATAFINAL,
            "PERIODOEXTENDIDO": true,
            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO
        };

        General.callRestEngine("product#/fct_prorrogcontrato_widget/alteraVigenciaContratoNovo", "Row", params).then(function (response) {
            /* Altera dados das vigencias de contrato anteriores */
            this.alteraDadosProrrog(widget);
            deferred.resolve();
        }.bind(this)).catch(function (response) {
            ScreenService.showMessage(response.data.error);
            deferred.reject();
        });

        return deferred.promise;
    };


    /* ---=========  FUNÇÕES DA PÁGINA FORMAÇÃO PREÇO =========--- */


    /**
     * A função "abrirWdgFormacaoPrec" abre a tela "Itens de Formação de Preço"
     */

    this.abrirWdgFormacaoPrec = function () {
        ScreenService.closePopup();
        var formacaoPreco = templateManager.container.getWidget("itemFormacaoPreco");
        formacaoPreco.getField("NRCALCFORMPRECSERV").dataSource.clearCheckedRows();
        formacaoPreco.isVisible = true;
        ScreenService.openSwipe(formacaoPreco).then(function () {
            formacaoPreco.edit();

            formacaoPreco.currentRow.NRESTRUTURAM = null;
            formacaoPreco.currentRow.NMESTRUTURAM = null;

            this.filtraFormPreco(formacaoPreco);
            this.verificaAcessibilidadeFieldServ(formacaoPreco);

            var wdgProposta = getWidgetGridContrato();
            var codEscopo = wdgProposta.currentRow.NRTIPOORCAMENTO;
            var nomeEscopo = wdgProposta.currentRow.NMTIPOORCAMENTO;

            if (codEscopo) {
                formacaoPreco.currentRow.NRTIPOORCAMENTO = codEscopo;
                formacaoPreco.currentRow.NMTIPOORCAMENTO = nomeEscopo;
            }
            else {
                var fieldNomeEscopo = formacaoPreco.getField("NMTIPOORCAMENTO");
                fieldNomeEscopo.readOnly = false;
            }
        }.bind(this));
    };

    /**
     *   A função "fecharWdgFormacaoPrec" fecha a tela "Itens de Formação de Preço"
     */

    this.fecharWdgFormacaoPrec = function (widget) {
        ScreenService.closeSwipe().then(function () {
            widget.isVisible = false;

            /*Limpa os campos*/
            this.limpaCamposFormPrec(widget);

            templateManager.container.getWidget("itemcontrato").activate();

        }.bind(this));

    };


    /**
     * A função "filtraItFormPreco" filtra o campo "Itens da Formação de Preço" 
     */

    this.filtraItFormPreco = function (widget) {
        var formPreco = widget.currentRow.NRFORMPRECOSERVICO;
        widget.getField('NRCALCFORMPRECSERV').dataSourceFilter[0].value = formPreco;
        widget.getField('NRCALCFORMPRECSERV').reload();

    };


    /**
     * A Função "filtraFormPreco" filtra o campo "Formação de Preço" 
     */

    this.filtraFormPreco = function (widget) {

        var contrato = templateManager.container.getWidget("gridContrato").currentRow;
        var nrseqcontrato = contrato.NRSEQCONTRATO;
        var nrparcnegocio = contrato.NRPARCNEGOCIO;
        var nrorg = contrato.NRORG;

        var filter = [{
            "name": "NRSEQCONTRATO",
            "operator": "=",
            "value": nrseqcontrato
        }, {
            "name": "NRORG",
            "operator": "=",
            "value": nrorg
        }];

        localStorageEngine.download("product#/fct_relacestruturacont_widget", filter).then(function (response) {
            var localAtend = response;

            /*Recebe Parceiro de Negócio*/
            var parcNeg = [];
            parcNeg.push(nrparcnegocio);
            parcNeg.push(null);

            /*Recebe Local Atendimento*/
            var locAtend = [];
            locAtend.push(null);

            localAtend.forEach(function (localAt) {

                locAtend.push(localAt.NRESTRUTURAM);
            });

            widget.getField('NRFORMPRECOSERVICO').dataSourceFilter[0].value = parcNeg;
            widget.getField('NRFORMPRECOSERVICO').dataSourceFilter[1].value = locAtend;
            widget.getField('NRFORMPRECOSERVICO').reload();

        });
    };


    /**
     *  A função "verificaServicosRepetidos" verifica entre as formações que foram marcadas, 
     *  se exitem serviços repetidos.
     */
    this.verificaServicosRepetidos = function (widget) {
        var servRepetidos = [];
        var servicos = widget.currentRow.CDSERVORCAFORM;
        var servAnte = 0;

        servicos.sort();
        servicos.forEach(function (servAux) {
            if (servAux == servAnte) {
                if (!servRepetidos.includes(servAux)) {
                    servRepetidos.push(servAux);
                }
            }
            servAnte = servAux;
        });

        return servRepetidos;
    };


    /**
     *  A função "salvaItemformacaopreco" centraliza o salvamento dos 
     *  itens de formação de preço na proposta.
     */

    this.salvaItemformacaopreco = function (widget) {
        if (widget.isValid()) {
            var servRepetidos = this.verificaServicosRepetidos(widget);
            if (servRepetidos.length != 0) {
                ScreenService.showCustomDialog("Foram selecionados diferentes formações de preço que possuem o mesmo serviço. O que você deseja fazer com os serviços repetidos?", 'CONFIRMATION', null, [{
                    "label": "Somar os Valores",
                    "code": function () {
                        this.validarDadosRateioItensFormacao(widget, true);
                    }.bind(this)
                },
                {
                    "label": "Incluir Separadamente",
                    "code": function () {
                        this.validarDadosRateioItensFormacao(widget, false);
                    }.bind(this)
                },
                {
                    "label": "Selecionar Novamente",
                    "code": function () { }
                }
                ]);
            }
            else {
                var deferred = ZHPromise.defer();
                this.validarDadosRateioItensFormacao(widget, false).then(function (response) {
                    var widgetItensCont = templateManager.container.getWidget("itemcontrato");
                    widgetItensCont.getField("NMESTRUTURAM").isVisible = true;
                    deferred.resolve(response);
                }).catch(function (error) {
                    deferred.reject(error);
                });

                return deferred.promise;
            }
        }
    };

    /**
     *  A função "insereItemFormPrec" insere os serviços selecionados.
     */
    this.insereItemFormPrec = function (widget, soma) {
        var deferred = ZHPromise.defer();
        var rowContrato = templateManager.container.getWidget("gridContrato").currentRow;
        var servicos = widget.getField("NRCALCFORMPRECSERV").dataSource.getCheckedRows();
        if (widget.currentRow.NMSERVORCA) {
            // pega o serviço da field serviço da tela e substitui o serviço da formação de preço
            servicos.forEach(function (servico) {
                servico.CDSERVORCA = widget.currentRow.CDSERVORCA;
                servico.NMSERVORCA = widget.currentRow.NMSERVORCA;
            });
        }

        var params = {
            "SERVICOS": servicos,
            "SOMA": soma,
            "CONTRATO": rowContrato.NRSEQCONTRATO,
            "NRTIPOORCAMENTO": rowContrato.NRTIPOORCAMENTO,
            "NRESTRUTURAM": widget.currentRow.NRESTRUTURAM
        };
        General.callRestEngine("product#/fct_relacestruturacont_widget/insereItemFormPrec", "Row", params).then(function (response) {
            this.limpaCamposFormPrec(widget);

            var widgetAtiva = templateManager.container.getActiveWidget();
            if (widgetAtiva.name === "itemFormacaoPreco") {
                ScreenService.closeSwipe();
                templateManager.container.getWidget("itemcontrato").activate();
                templateManager.container.getWidget("itemcontrato").reload();
            }


            var itens = response.dataset.fct_relacestruturacont_widget;
            itens.forEach(function (item) {
                this.insereAgendamentoPendente(rowContrato.NRSEQCONTRATO, item.NRITEMOCONTRATO, 1, rowContrato.IDSITUACAOCONTRATO);
            }.bind(this));
            widget.getField("NMESTRUTURAM").isVisible = true;
            getWidgetGridContrato().reload();
            deferred.resolve(itens);
        }.bind(this),
            function (error) {
                Utilities.showErrorMessage(error);
                deferred.reject();
            });

        return deferred.promise;
    };
    /**
     * Sintetiza as funções em uma mesma função
     * 
     * @param {object} widget - Widget de Itens da Contrato ou de Adicionais
     */
    this.funcoesAjuste = function (widget) {
        this.calcularValorTotalItem(widget);
        if (widget.currentRow.VRDESCONTO) {
            this.adequaAjusteAoCampo(widget);
        }

    };

    /**
     * Previni possiveis erros com a troca sequida no select native do campo Ajuste
     * 
     * @param {object} widget - Widget de Itens da Contrato ou de Adicionais
     */
    this.adequaAjusteAoCampo = function (widget) {
        var quantidade = widget.currentRow.QTITEMCONTRATO;
        var valorUnitario = widget.currentRow.VRUNITITEMCONT;
        var tipoajuste = widget.currentRow.IDTPDESCONTO;
        var ajuste = widget.currentRow.VRDESCONTO;
        var utilizaQtdeMedida = widget.currentRow.IDMULTMEDVRTOT;
        var qtdeMedida = widget.currentRow.QTMEDIDACONTR;
        var valorSemajuste = quantidade * valorUnitario;
        if (utilizaQtdeMedida == "S") {
            valorSemajuste *= qtdeMedida;
        }
        var valorTotal = valorSemajuste;
        if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));
        if (tipoajuste == "P" && ajuste > 100) {
            widget.currentRow.VRDESCONTO = null;
        }
        if (tipoajuste == "V" && ajuste > valorTotal) {
            widget.currentRow.VRDESCONTO = null;
        }
    };

    /**
     *  A função "controlaVisibItFormPrec" controla a visibilidade da
     *  action "Itens de Formação de preço".
     */
    this.controlaVisibItFormPrec = function (widget) {
        var situacao = templateManager.container.getWidget("gridContrato").currentRow.IDSITUACAOCONTRATO;

        if (situacao == 'P') {
            widget.getAction("formacaoPreco").isVisible = true;
        }
        else {
            if (situacao == 'A') {
                widget.getAction("formacaoPreco").isVisible = (idAlteraContAprv === "S") ? true : false;
            }
            else {
                widget.getAction("formacaoPreco").isVisible = false;
            }
        }
    };


    /**
     *  A função "limpaCamposFormPrec" Limpa os campos após o fechamento do swipe
     */
    this.limpaCamposFormPrec = function (widget) {
        widget.currentRow.NRFORMPRECOSERVICO = null;
        widget.currentRow.NMFORMPRECOSERVICO = null;
        widget.currentRow.CDSERVORCAFORM = null;

        widget.currentRow.NRCALCFORMPRECSERV = null;
        widget.currentRow.CDSERVORCA = null;
        widget.currentRow.NMSERVORCA = null;
        widget.currentRow.NRESTRUTURAM = null;
        widget.currentRow.CDHABILITSERV = null;
        widget.currentRow.NMELEMENTO = null;
        widget.currentRow.VRUNITARIO = null;
        widget.currentRow.QTCALCULO = null;
        widget.currentRow.IDUSAMEDIDA = null;
        widget.currentRow.CDTPMEDIDASERV = null;
        widget.currentRow.QTMEDIDACALC = null;
        widget.currentRow.VRTOTAl = null;
    };

    /**
     * Regera as parcelas de faturamento
     */
    this.regerarFaturamento = function (widget) {
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente regerar o faturamento do contrato?"), 'question', function (success) {
            var nrseqcontrato = widget.currentRow.NRSEQCONTRATO;

            Sm00004000_Contrato_Service.cancelaParcelasContrato(nrseqcontrato).then(function () {
                Sm00004000_Contrato_Service.geraParcelasContrato(nrseqcontrato).then(function () {
                    ScreenService.showMessage("Processo realizado com sucesso!");
                    ScreenService.closePopup();
                });
            });
        }.bind(this));
    };

    /* Valida se foi realizado o reajuste dos valores dentro do período de vigência do contrato antes de regerar das parcelas de faturamento */
    this.validareajusteRegFat = function (widget) {

        var params = {
            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
            "DTINICIO": null,
            "DTFINAL": null
        };

        General.callRestEngine('product#/fct_contrato_widget/validaReajuste', "Row", params).then(function (response) {

            /* Se buscarem dados, significa que a validação de reajsute está ativa. Neste caso, deve ser verificado se o contrato já teve reajuste realizado. */
            if (response.dataset.retorno.valida.length > 0) {

                if (response.dataset.retorno.valida[0].VALIDA > 0) {
                    this.regerarFaturamentoNovo(widget);
                }
                else {

                    ScreenService.showMessage("Operação bloqueada!<br><br> O período de vigência deste contrato foi alterado, mas não foi realizado o reajuste dos valores do mesmo.<br><br> Deve ser realizado o reajuste dos valores do contrato antes de serem regeradas as parcelas para faturamento.");
                    ScreenService.closePopup();

                }
            }
            /* Se não buscarem dados, significa que o contrato ainda não teve as datas alteradas ou a validação dereajsute está desativada. Neste caso, a rotina segue normalmente. */
            else {

                this.regerarFaturamentoNovo(widget);

            }

        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });

    }


    /**
     * Regera as parcelas de faturamento
     */
    this.regerarFaturamentoNovo = function (widget) {

        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente regerar o faturamento do contrato?"), 'question', function (success) {

            var params = {
                "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO
            };

            General.callRestEngine('product#/fct_contrato_widget/geraParcelasContratoNovoFat', "Row", params).then(function () {
                ScreenService.showMessage("Processo realizado com sucesso!");
                ScreenService.closePopup();
            })
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
        });

    };

    /**
     * Valida se os documentos podem ser gerados em DOCX
     * e pergunta com qual extensão o usuário deseja baixar
     **/
    this.validaParametrosDoc = function (widget) {

        General.callRestEngine("product#/fct_contrato_widget/getParametroExtensaoDoc", "Row", { "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO }).then(function (response) {
            if (!response.dataset || !response.dataset.IDPERMITEDOCX) {
                ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente gerar o documento do contrato?"), 'question', function () {
                    this.gerarDocumentoContrato(widget, 'P');
                }.bind(this));
            }
            else {
                response.dataset.IDPERMITEDOCX.forEach(function (idpermite) {
                    if (!idpermite.IDPERMITEDOCX || idpermite.IDPERMITEDOCX == 'N') {
                        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente gerar o documento do contrato?"), 'question', function () {
                            this.gerarDocumentoContrato(widget, 'P');
                        }.bind(this));
                    }
                    else {
                        ScreenService.showCustomDialog("O documento pode ser gerados em DOCX.<br>Como deseja gerá-lo?", 'CONFIRMATION', null, [{
                            "label": "Gerar PDF",
                            "code": function () {
                                this.gerarDocumentoContrato(widget, 'P');
                            }.bind(this)
                        }, {
                            "label": "Gerar DOCX",
                            "code": function () {
                                this.gerarDocumentoContrato(widget, 'D');
                            }.bind(this)
                        }, {
                            "label": "Cancelar",
                            "code": function () { }
                        }]);
                    }
                }.bind(this));
            }
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    }

    this.gerarDocumentoContrato = function (wdgContrato, idExtensao) {
        var nrseqcontrato = wdgContrato.currentRow.NRSEQCONTRATO;
        var nrorg = wdgContrato.currentRow.NRORG;
        var idSituacao = wdgContrato.currentRow.IDSITUACAOCONTRATO;

        var params = {
            "NRSEQCONTRATO": nrseqcontrato,
            "NRORG": nrorg,
            "IDSITUACAOCONTRATO": idSituacao,
            "IDPERMITEDOCX": idExtensao
        };

        var timeout = 5 * 60000; //5 min
        Utilities.callRestEngineWithTimeout("product#/fct_contrato_widget/gerarDocumentoContrato", "Row", params, timeout).then(function (success) {
            var links = success.dataset.documentos.links;
            links.forEach(function (link) {
                if (link)
                    window.open(window.backendIndexUrl + link);
            });

            if (success && success.messages && success.messages.length > 0 && success.messages[0].message)
                ScreenService.showMessage(success.messages[0].message);

            ScreenService.closePopup();
        })
            .catch(function (error) {
                console.error(error);
                var msg = "Timeout";
                if (error.status !== 0 && error.data)
                    msg = error.data.error;

                ScreenService.showMessage(msg);
            });
    };

    /* Habilita os campos para a inserção de um novo local de atendimento
     * @params{Object} widget local de atendimento
     */
    this.habilitaNovoLocalAtend = function (widgetLocalAtend) {
        var fields = ["NMESTRUTURAMNV", "NRINSCRICAOPARC", "NRTIPOESTRUTURA", "NMTIPOESTRUTURA", "NRESTRUTURAMSUP", "NMESTRUTURAMSUP", "NMCENTCUST", "NRCEPENDERECO", "NMPAIS", "SGESTADO", "NMESTADO", "NMMUNICIPIO", "DSENDERECO", "NRIMOVELENDERECO", "NMBAIRROENDERECO", "DSCOMPLEENDERECO", "DSREFERENCIAENDE"];

        if (widgetLocalAtend.currentRow.NOVOLOCALATEND === 'S') {
            widgetLocalAtend.getField('NRESTRUTURAM').readOnly = true;
            widgetLocalAtend.getField('NRESTRUTURAM').validations = {};
            fields.forEach(function (field) {
                widgetLocalAtend.getField(field).readOnly = false;
            });
            widgetLocalAtend.getField("NMESTRUTURAMNV").validations = { "required": true };
            widgetLocalAtend.getField("NMTIPOESTRUTURA").validations = { "required": true };
            this.recuperaEnderecoParcNegocio(widgetLocalAtend);
        }
        else {
            widgetLocalAtend.getField('NRESTRUTURAM').validations = { "required": true };
            widgetLocalAtend.getField("NMESTRUTURAMNV").validations = {};
            widgetLocalAtend.getField("NMTIPOESTRUTURA").validations = {};
            fields.forEach(function (field) {
                widgetLocalAtend.getField(field).readOnly = true;
                widgetLocalAtend.currentRow[field.name] = "";
            });

            widgetLocalAtend.getField('NRESTRUTURAM').readOnly = false;

        }
    };

    /**
     * Busca o endereço através do CEP digitado.
     * 
     * @param {object} widget
     */

    this.buscarEndereco = function (widget) {
        var fields = ["CDPAIS", "NMPAIS", "SGESTADO", "NMESTADO", "CDMUNICIPIO", "NMMUNICIPIO", "CDBAIRRO", "NMBAIRROENDERECO", "CDLOGRADOURO", "DSLOGRADOURO", "DSENDERECO"];

        if (widget.currentRow.NRCEPENDERECO) {
            Utilities.buscarCep(widget.currentRow.NRCEPENDERECO).then(function (response) {
                var endereco = response.dataset.ENDERECO;
                fields.forEach(function (field) {
                    widget.currentRow[field] = (endereco[field]) ? endereco[field] : "";
                });

                if (response.messages) {
                    var msg = ScreenService.i18n(response.messages[0].message);
                    ScreenService.showMessage(msg);
                }

                angular.element('#footer').click();
            },
                function (error) {
                    console.error(error);
                    if (error.data) {
                        var mensagem = error.data.error;
                        if (error.data.error === "Erro de configuração de proxy!")
                            mensagem = "Endereço não encotrado.";

                        ScreenService.showMessage(mensagem);
                    }
                    else {
                        ScreenService.showMessage("Falha na conexão.");
                    }
                });
        }
        else {
            fields.forEach(function (field) {
                widget.currentRow[field] = "";
            });
        }

    };

    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o País.
     * 
     * @param {object} widget
     */
    this.controlarCamposEnderecoPorPais = function (widget) {
        widget.currentRow.SGESTADO = "";
        widget.currentRow.NMESTADO = "";
        widget.currentRow.CDMUNICIPIO = "";
        widget.currentRow.NMMUNICIPIO = "";
        widget.currentRow.CDBAIRRO = "";
        widget.currentRow.NMBAIRROENDERECO = "";

        var cdpais = widget.currentRow.CDPAIS;
        var campoEstado = widget.getField("NMESTADO");
        var campoMunicipio = widget.getField("NMMUNICIPIO");
        var campoBairro = widget.getField("NMBAIRROENDERECO");
        campoEstado.dataSourceFilter[0].value = cdpais;
        campoEstado.reload();

        campoMunicipio.dataSourceFilter[0].value = cdpais;
        campoBairro.dataSourceFilter[0].value = cdpais;
    };


    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o Estado.
     * 
     * @param {object} widget
     */
    this.controlarCamposEnderecoPorEstado = function (widget) {
        widget.currentRow.CDMUNICIPIO = "";
        widget.currentRow.NMMUNICIPIO = "";
        widget.currentRow.CDBAIRRO = "";
        widget.currentRow.NMBAIRROENDERECO = "";

        var sgestado = widget.currentRow.SGESTADO;
        var campoMunicipio = widget.getField("NMMUNICIPIO");
        var campoBairro = widget.getField("NMBAIRROENDERECO");
        campoMunicipio.dataSourceFilter[1].value = sgestado;
        campoMunicipio.reload();

        campoBairro.dataSourceFilter[1].value = sgestado;
    };

    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o Município.
     * 
     * @param {object} widget
     */
    this.controlarCamposEnderecoPorMunicipio = function (widget) {
        widget.currentRow.CDBAIRRO = "";
        widget.currentRow.NMBAIRROENDERECO = "";

        var cdmunicipio = widget.currentRow.CDMUNICIPIO;
        var campoBairro = widget.getField("NMBAIRROENDERECO");
        campoBairro.dataSourceFilter[2].value = cdmunicipio;
        campoBairro.reload();
    };

    /**
     * Recupera o endereço do parceiro de negócio do contrato e preenche 
     * os dados de novo local de atendimento
     * @params{Object} local de atendimento do contrato
     **/
    this.recuperaEnderecoParcNegocio = function (wdgNvLocalAtend) {
        var widgetLancContrato = templateManager.container.getWidget("gridContrato");
        var paramsFilter = [{
            name: "NRPARCNEGOCIO",
            operator: "=",
            value: widgetLancContrato.currentRow.NRPARCNEGOCIO
        }];

        General.callRestEngine("product#/crud/enderecoparc", "FilterData", paramsFilter).then(function (success) {
            if (success.dataset && success.dataset.enderecoparc.length > 0) {
                success.dataset.enderecoparc.every(function (dado) {
                    if (dado.CDTIPOENDERECO === "PRINCIPAL") {
                        wdgNvLocalAtend.currentRow.NRCEPENDERECO = dado.NRCEPENDERECO;
                        this.buscarEndereco(wdgNvLocalAtend);
                    }
                }.bind(this));

                if (!wdgNvLocalAtend.currentRow.NRCEPENDERECO) {
                    wdgNvLocalAtend.currentRow.NRCEPENDERECO = success.dataset.enderecoparc[0].NRCEPENDERECO;
                    this.buscarEndereco(wdgNvLocalAtend);
                }
            }
        }.bind(this));
    };

    this.selecionarItensPai = function (select_multiple_widget) {
        select_multiple_widget.dataSource.unSelectAll();
        var itensRaiz = select_multiple_widget.dataSource.data.filter(function (row) {
            return row.NRCALCFORMPRECSERVPAI == null;
        });

        select_multiple_widget.dataSource.selectAll(itensRaiz);
        templateManager.updateTemplate();
    };

    this.abrirWidgetSubstituicaoRecurso = function () {
        if (validarRecursos()) {
            var wdgSubstituicaoRecurso = templateManager.container.getWidget("wdgSubstituicaoRecurso");
            ScreenService.closePopup();
            ScreenService.openPopup(wdgSubstituicaoRecurso).then(function () {
                wdgSubstituicaoRecurso.dataSource.data = [];
                wdgSubstituicaoRecurso.newRow();
            });
        }
    };

    this.abrirWidgetCoberturaRecurso = function () {
        if (validarRecursos()) {
            var wdgCoberturaRecurso = templateManager.container.getWidget("wdgCoberturaRecurso");
            ScreenService.closePopup();
            ScreenService.openPopup(wdgCoberturaRecurso).then(function () {
                wdgCoberturaRecurso.dataSource.data = [];
                wdgCoberturaRecurso.newRow();
            });
        }
    };

    var validarRecursos = function () {
        var wdgRecursos = templateManager.container.getWidget("wdgRecursos");
        var validacao = wdgRecursos.getCheckedRows().every(function (row) {
            row.IDALTERACAOEQUIPE == null || row.IDALTERACAOEQUIPE == "";
        });

        if (validacao) {
            return true;
        }
        else {
            ScreenService.showMessage("Operação bloqueada.<br/>Um ou mais recursos selecionados já possuem solicitação de substituição ou cobertura.");
            return false;
        }
    };

    this.gerarSolicitacaoSubstituicaoRecurso = function (wdgSubstituicaoRecurso) {
        if (wdgSubstituicaoRecurso.isValid()) {
            var wdgRecursos = templateManager.container.getWidget("wdgRecursos");
            var justificativa = wdgSubstituicaoRecurso.currentRow.JUSTIFICATIVA;

            var rows = [];
            wdgRecursos.getCheckedRows().forEach(function (row) {
                rows.push({
                    "NRAGENDRECURSOCONT": row.NRAGENDRECURSOCONT,
                    "NRRECURSOSAGENDCONT": row.NRRECURSOSAGENDCONT
                });
            });

            var params = {
                "RECURSOS": rows,
                "JUSTIFICATIVA": justificativa
            };

            General.callRestEngine("product#/fct_agendrecursocont_widget/solicitarSubstituicaoRecurso", "Row", params).then(function () {
                ScreenService.showMessage("Processo realizado com sucesso!");
                ScreenService.closePopup();
            })
                .catch(function (error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
        }
    };

    this.gerarSolicitacaoCoberturaRecurso = function (wdgCoberturaRecurso) {
        if (wdgCoberturaRecurso.isValid()) {
            var wdgRecursos = templateManager.container.getWidget("wdgRecursos");
            var justificativa = wdgCoberturaRecurso.currentRow.JUSTIFICATIVA;

            var rows = [];
            wdgRecursos.getCheckedRows().forEach(function (row) {
                rows.push({
                    "NRAGENDRECURSOCONT": row.NRAGENDRECURSOCONT,
                    "NRRECURSOSAGENDCONT": row.NRRECURSOSAGENDCONT
                });
            });

            var params = {
                "RECURSOS": rows,
                "JUSTIFICATIVA": justificativa
            };

            General.callRestEngine("product#/fct_agendrecursocont_widget/solicitarCoberturaRecurso", "Row", params).then(function () {
                ScreenService.showMessage("Processo realizado com sucesso!");
                ScreenService.closePopup();
            })
                .catch(function (error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
        }
    };

    this.excluirContrato = function (wdgContrato) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function () {
            General.callRestEngine("product#/fct_contrato_widget/remove", 'Row', wdgContrato.currentRow).then(function () {
                var wdgTabbed = templateManager.container.getWidget("wdgTabbedPrincipal");
                if (wdgTabbed) {
                    this.voltarParaContratoGrid();
                }
                else {
                    ScreenService.closeSwipe();
                    wdgContrato.reload();
                }
            }.bind(this),
                function (error) {
                    if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                        //ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
                        ScreenService.confirmMessage("Não é possível excluir o contrato porque ele possui registros filhos. Deseja inativar o contrato?", "question",
                            function () {
                                this.inativarContrato(wdgContrato);
                            }.bind(this),
                            null);
                    }
                    else {
                        console.error(error);
                        ScreenService.showMessage(error.data.error);
                    }
                }.bind(this));
        }.bind(this));
    };

    this.inativarContrato = function (wdgContrato) {
        wdgContrato.currentRow.changed = true;
        wdgContrato.currentRow.IDATIVO = "N";
        General.widgetSave(wdgContrato).then(function () {
            var wdgTabbed = templateManager.container.getWidget("wdgTabbedPrincipal");
            if (wdgTabbed) {
                this.voltarParaContratoGrid();
            }
            else {
                ScreenService.closeSwipe();
                wdgContrato.reload();
            }
        }.bind(this),
            function (error) {
                console.error(error);
                if (error.data)
                    ScreenService.showMessage(error.data.error);
                else
                    ScreenService.showMessage(error);
            });
    };

    /**
     * Abre o popup de edição em massa de operadores responsáveis
     * @params{Object} widget widget contrato grid
     **/
    this.abrirWdgAlteraOpersResp = function (widget) {
        var wdgPopup = templateManager.container.getWidget("WdgPopupAlteraOpers");
        wdgPopup.getField("CDOPERRESPATUAL").readOnly = false;
        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function () {
            wdgPopup.edit();
            wdgPopup.newRow();
            wdgPopup.activate();
            wdgPopup.getField("NRSEQCONTRATO").dataSourceFilter = [];
            this.verificaHierarquia(wdgPopup);
        }.bind(this));
    };

    /**
     * Cancela a alteração de operadores responsáveis
     * @params{Object} widget widget popup alteração de operadores responsáveis 
     **/
    this.cancelarWdgAlteracaoOpers = function (widget) {
        var fieldCont = widget.getField("NRSEQCONTRATO");
        var fieldOper = widget.getField("CDOPERRESPATUAL");

        fieldOper.readOnly = false;
        fieldOper.dataSource.clearCheckedRows();
        if (fieldOper.dataSourceFilter.length > 0) {
            fieldOper.dataSourceFilter.pop();
        }

        fieldCont.dataSource.clearCheckedRows();
        if (fieldCont.dataSourceFilter.length > 1) {
            fieldCont.dataSourceFilter.pop();
        }

        widget.dataSource.data = [];
        widget.isVisible = false;

        var widContrato = templateManager.container.getWidget("wdgContratroGrid");
        if (!widContrato)
            widContrato = templateManager.container.getWidget("gridContrato");

        ScreenService.closePopup();
        widContrato.reload();
        widContrato.activate();
    };

    /**
     * Filtra as fields de contrato e operador atual
     * @params{Object} widget widget popup altera operadores responsáveis
     * @params{String} fieldAlterada field contrato
     **/
    this.filtraContratosEOperadores = function (widget, fieldAlterada) {
        var fieldCont = widget.getField("NRSEQCONTRATO");
        var fieldOper = widget.getField("CDOPERRESPATUAL");

        if (widget.currentRow.CDOPERRESPATUAL && !widget.currentRow.NRSEQCONTRATO || (widget.currentRow.NRSEQCONTRATO && widget.currentRow.CDOPERRESPATUAL && fieldAlterada === "CDOPERRESPATUAL")) {
            if (widget.currentRow.CDOPERRESPATUAL && widget.currentRow.CDOPERRESPATUAL.length > 0) {
                fieldCont.clearValue();
                fieldCont.dataSourceFilter[0] = {
                    "name": "CDOPERRESPONSAVEL",
                    "operator": "IN",
                    "value": widget.currentRow.CDOPERRESPATUAL
                };

                fieldOper.readOnly = false;
            }
            else {
                if (fieldCont.dataSourceFilter.length > 0) {
                    fieldCont.dataSourceFilter.pop();
                }
            }

            fieldCont.reload();

        }
        else {
            widget.currentRow.CDOPERRESPONSAVEL = [];
            widget.getField("NRSEQCONTRATO").dataSource.getCheckedRows().forEach(function (contrato) {
                widget.currentRow.CDOPERRESPONSAVEL.push(contrato.CDOPERRESPONSAVEL);
            });

            if ((!widget.currentRow.CDOPERRESPATUAL && widget.currentRow.NRSEQCONTRATO) || (widget.currentRow.NRSEQCONTRATO && widget.currentRow.CDOPERRESPATUAL && fieldAlterada === "NRSEQCONTRATO")) {
                if (widget.currentRow.CDOPERRESPONSAVEL && widget.currentRow.CDOPERRESPONSAVEL.length > 0) {
                    fieldOper.clearValue();
                    widget.currentRow.CDOPERRESPATUAL = [];
                    fieldOper.dataSourceFilter[0] = {
                        "name": "CDOPERADOR",
                        "operator": "IN",
                        "value": widget.currentRow.CDOPERRESPONSAVEL
                    };

                    fieldOper.readOnly = true;
                    if (widget.currentRow.CDOPERRESPONSAVEL) {
                        widget.currentRow.CDOPERRESPONSAVEL.forEach(function (operador) {
                            if (widget.currentRow.CDOPERRESPATUAL.indexOf(operador) === -1) {
                                widget.currentRow.CDOPERRESPATUAL.push(operador);
                            }
                        });
                    }
                }
                else {
                    if (fieldOper.dataSourceFilter.length > 0) {
                        fieldOper.dataSourceFilter.pop();
                        fieldOper.readOnly = false;
                    }
                }

                fieldOper.reload();
            }
        }
    };

    /**
     * Salva a alteração do operador responsável nos contratos selecionados
     * @partams{Object} widget widget popup de alteração dos operadores responsáveis
     **/
    this.salvarAlteracoesOpers = function (widget) {
        if (widget.currentRow.NRSEQCONTRATO.length === 0 && widget.currentRow.CDOPERRESPATUAL.length !== 0) {
            widget.currentRow.NRSEQCONTRATO = [];
            widget.getField("NRSEQCONTRATO").dataSource.data.forEach(function (data) {
                widget.currentRow.NRSEQCONTRATO.push(data.NRSEQCONTRATO);
            });
        }

        if (widget.isValid()) {
            var params = {
                "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                "CDOPERRESPONSAVEL": widget.currentRow.CDNOVOOPERRESP,
                "ALTERAPROPOSTA": widget.currentRow.ALTERAPROPOSTA
            };

            General.callRestEngine("product#/fct_contrato_widget/salvaAlteracaoOperContrato", "Row", params).then(function () {
                ScreenService.showMessage("Contratos atualizados com sucesso.");
                this.cancelarWdgAlteracaoOpers(widget);
            }.bind(this));
        }
    };

    /**
     * Aplica o filtro na situação do contrato
     * @params{Object} args
     **/
    this.antesDeFiltrarContrato = function (args) {
        var rowFiltro = args.e.preference.FILTER_ROW;
        var posicaoFiltroSituacao = Utilities.getFilterPosition(args.owner.dataSourceFilter, "IDSITUACAOCONTRATO");

        if (Array.isArray(rowFiltro.IDSITUACAOCONTRATO) && rowFiltro.IDSITUACAOCONTRATO.length > 0) {
            args.owner.dataSourceFilter[posicaoFiltroSituacao].value = rowFiltro.IDSITUACAOCONTRATO;
        }
        else {
            posicaoFiltroSituacao = posicaoFiltroSituacao ? posicaoFiltroSituacao : 0;
            args.owner.dataSourceFilter[posicaoFiltroSituacao] = {
                "name": "IDSITUACAOCONTRATO",
                "operator": "<>",
                "value": ""
            };

            if (Array.isArray(rowFiltro.NRSEQORCAMENTO) && rowFiltro.NRSEQORCAMENTO.length > 0)
                args.owner.dataSourceFilter[posicaoFiltroSituacao].value = "-1";
            else
                args.owner.dataSourceFilter[posicaoFiltroSituacao].value = "E";
        }

        args.owner.reload();
    };

    /**
     * Salva as alterações da parcela do item na aba de faturamento
     * @param {Object} widget widget itens faturamento
     **/
    this.saveItensFaturamento = function (widget) {
        var wdgFaturamento = templateManager.container.getWidget("faturamentocontrato");
        var dataVenc = widget.currentRow.DTVENCIMENTO;
        var valorParc = widget.currentRow.VRVENCIMENTO;
        var dataInicial = widget.currentRow.DTINICOMPET;
        var dataFinal = widget.currentRow.DTFINCOMPET;
        var idAltUsuario = widget.currentRow.IDALTUSUARIO;
        var dataEmissao = widget.currentRow.DTEMISSAO;

        General.widgetSave(widget).then(function (response) {
            if (idAltUsuario == "S") {
                wdgFaturamento.currentRow.DTVENCIMENTO = dataVenc;
                wdgFaturamento.currentRow.VRVENCIMENTO = valorParc;
                wdgFaturamento.currentRow.DTINICOMPET = dataInicial;
                wdgFaturamento.currentRow.DTFINCOMPET = dataFinal;
                wdgFaturamento.currentRow.IDALTUSUARIO = idAltUsuario;
                wdgFaturamento.currentRow.DTEMISSAO = dataEmissao;
                wdgFaturamento.saveRow();
            }

            ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso"));
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            })
    };

    /**
     * Controla as ações da aba de faturamento
     * de acordo com a situação da parcela
     * @param {Object} widget widget faturamento
     **/
    this.controlaAcessActionsFat = function (widget) {
        if (widget.currentRow.IDSITUVENCPREV === 'A' || widget.currentRow.IDSITUVENCPREV === 'F')
            widget.getAction("edit_fat").isVisible = false;
        else
            widget.getAction("edit_fat").isVisible = true;

        if (widget.currentRow.IDSITUVENCPREV === 'P' && widget.currentRow.IDALTUSUARIO === 'S')
            widget.getAction("delete_fat").isVisible = true;
        else
            widget.getAction("delete_fat").isVisible = false;
    };

    /**
     * Cancela o formulário da aba de faturamento
     * @param {Object} widget widget faturamento
     **/
    this.cancelFormFaturamento = function (widget) {
        widget.getField("NMITEMCONTRATO").isVisible = false;
        widget.getField("VRTOTITEMCONT").isVisible = false;
        General.widgetCancelForm(widget);
    };

    /**
     * Habilita a aba de faturamento para receber os dados de 
     * uma nova parcela
     * @param {Object} widget widget faturamento
     **/
    this.faturamentoNewRow = function (widget) {
        var wdgContrato = templateManager.container.getWidget("gridContrato");

        var nrparcnegocio = null;
        var nmparcnegocio = null;
        var nrprupofatura = null;
        var nmgrupofatura = null;

        General.widgetNewRow(widget);

        var params = {
            "NRSEQCONTRATO": wdgContrato.currentRow.NRSEQCONTRATO
        };

        General.callRestEngine("product#/fct_contrato_widget/getParamFatCont", "Row", params).then(function (response) {

            if (response.dataset.PARAM != 0) {
                nrparcnegocio = response.dataset.PARAM[0].NRPARCFATURA;
                nmparcnegocio = response.dataset.PARAM[0].NMPRINCIPALPARC;
                nrprupofatura = response.dataset.PARAM[0].NRGRUPOFATURA;
                nmgrupofatura = response.dataset.PARAM[0].NMGRUPOFATURA;
            }

            widget.currentRow.NRPARCNEGOCIO = (nrparcnegocio != null) ? nrparcnegocio : wdgContrato.currentRow.NRPARCNEGOCIO;
            widget.currentRow.NMPRINCIPALPARC = (nmparcnegocio != null) ? nmparcnegocio : wdgContrato.currentRow.NMPRINCIPALPARC;

            widget.currentRow.NRGRUPOFATURA = nrprupofatura;
            widget.currentRow.NMGRUPOFATURA = nmgrupofatura;
        })
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });

        widget.currentRow.IDSITUVENCPREV = "P";

        widget.currentRow.NRPARCNEGOCIOCONT = wdgContrato.currentRow.NRPARCNEGOCIO;
        widget.currentRow.NMPRINCIPALPARCCONT = wdgContrato.currentRow.NMPRINCIPALPARC;

        widget.currentRow.NRPARCNEGOCIO = (nrparcnegocio != null) ? nrparcnegocio : wdgContrato.currentRow.NRPARCNEGOCIO;
        widget.currentRow.NMPRINCIPALPARC = (nmparcnegocio != null) ? nmparcnegocio : wdgContrato.currentRow.NMPRINCIPALPARC;

        widget.currentRow.NRGRUPOFATURA = nrprupofatura;
        widget.currentRow.NMGRUPOFATURA = nmgrupofatura;

        widget.currentRow.NRSEQCONTRATO = wdgContrato.currentRow.NRSEQCONTRATO;

        widget.currentRow.IDFATURACONTR = wdgContrato.currentRow.IDFATURACONTR;
        widget.currentRow.CDMODPGTO = wdgContrato.currentRow.CDMODPGTO;
        widget.currentRow.DSMODPGTO = wdgContrato.currentRow.DSMODPGTO;

        widget.currentRow.IDALTUSUARIO = "S";

        widget.getField("NMITEMCONTRATO").dataSourceFilter[0].value = wdgContrato.currentRow.NRSEQCONTRATO;
        widget.getField("NMITEMCONTRATO").reload();

        widget.getField("NMITEMCONTRATO").isVisible = true;
        widget.getField("VRTOTITEMCONT").isVisible = true;
    };

    /**
     * Atribui ao valor da parcela o valor do item escolhido
     * @param {Object} widget widget faturamento 
     **/
    this.modificarValorTotalParcela = function (widget) {
        if (widget.currentRow.NRITEMOCONTRATO) {

            var vrItem = widget.currentRow.VRTOTITEMCONT;

            if (vrItem) {
                vrItem = vrItem.toString();
                var primeiraPos = vrItem.indexOf(".");
                var ultimaPos = vrItem.lastIndexOf(".");
                if (primeiraPos == ultimaPos) {
                    if (vrItem.includes(",")) {
                        vrItem = parseFloat(vrItem.replace(".", "").replace(",", "."));
                    }
                    else {
                        vrItem = parseFloat(vrItem.replace(",", "."));
                    }
                }
                else {
                    var parteInicial = vrItem.substr(0, primeiraPos + 1);
                    var parteFinal = vrItem.substr(primeiraPos + 1);
                    vrItem = parseFloat(parteInicial.replace(".", "") + parteFinal);
                }
            }
            else {
                vrItem = 0;
            }

            widget.currentRow.VRVENCIMENTO = vrItem;
            widget.getField("VRVENCIMENTO").readOnly = true;
            widget.getField("VRTOTITEMCONT").readOnly = false;

        }
        else {

            widget.currentRow.VRTOTITEMCONT = "";
            widget.currentRow.NMITEMCONTRATO = "";
            widget.currentRow.NRITEMOCONTRATO = "";
            widget.currentRow.QTITEMCONTRATO = "";
            widget.currentRow.CDTPMEDIDASERV = "";
            widget.currentRow.QTMEDIDACONTR = "";
            widget.currentRow.VRUNITITEMCONT = "";
            widget.getField("VRVENCIMENTO").readOnly = false;
            widget.getField("VRTOTITEMCONT").readOnly = true;
        }
    };

    /**
     * Salva as parcelas do faturamento
     * @param {Object} widget widget faturamento
     **/
    this.saveFaturamentoContrato = function (widget) {
        var isNew = widget.currentRow.__is_new;
        var item = widget.currentRow.NRITEMOCONTRATO;
        var wdgItensFat = templateManager.container.getWidget("Itensfaturamento");
        var wdgContrato = templateManager.container.getWidget("gridContrato");

        if (!isNew) {
            this.validaAltUsuarioFatura(widget);
        }
        else {
            widget.currentRow.NRAPURACAO = null;
            widget.currentRow.QTDIASVENCIMENTO = moment(widget.currentRow.DTFINCOMPET, "DD/MM/YYYY").diff(moment(widget.currentRow.DTINICOMPET, "DD/MM/YYYY"), "days");
            widget.currentRow.VRFATURADO = 0;
            widget.currentRow.VRFALTAFATURAR = widget.currentRow.VRVENCIMENTO;

            if (widget.currentRow.NRITEMOCONTRATO) {
                wdgItensFat.currentRow.NRSEQCONTRATO = wdgContrato.currentRow.NRSEQCONTRATO;
                wdgItensFat.currentRow.NRITEMOCONTRATO = widget.currentRow.NRITEMOCONTRATO;
                wdgItensFat.currentRow.IDFATURACONTR = widget.currentRow.IDFATURACONTR;
                wdgItensFat.currentRow.CDMODPGTO = widget.currentRow.CDMODPGTO;
                wdgItensFat.currentRow.QTDIASVENCIMENTO = moment(widget.currentRow.DTFINCOMPET, "DD/MM/YYYY").diff(moment(widget.currentRow.DTINICOMPET, "DD/MM/YYYY"), "days");
                wdgItensFat.currentRow.DTINICOMPET = widget.currentRow.DTINICOMPET;
                wdgItensFat.currentRow.DTFINCOMPET = widget.currentRow.DTFINCOMPET;
                wdgItensFat.currentRow.DTVENCIMENTO = widget.currentRow.DTVENCIMENTO;
                wdgItensFat.currentRow.DTEMISSAO = widget.currentRow.DTEMISSAO;
                wdgItensFat.currentRow.VRVENCIMENTO = widget.currentRow.VRVENCIMENTO;
                wdgItensFat.currentRow.QTITEMCONTRATO = widget.currentRow.QTITEMCONTRATO;
                wdgItensFat.currentRow.CDTPMEDIDASERV = widget.currentRow.CDTPMEDIDASERV;
                wdgItensFat.currentRow.QTMEDIDACONTR = widget.currentRow.QTMEDIDACONTR;
                wdgItensFat.currentRow.VRUNITITEMCONT = widget.currentRow.VRUNITITEMCONT;
                wdgItensFat.currentRow.VRTOTITEMCONT = widget.currentRow.VRTOTITEMCONT;
                wdgItensFat.currentRow.IDALTUSUARIO = widget.currentRow.IDALTUSUARIO;
                wdgItensFat.currentRow.__is_new = true;
                wdgItensFat.currentRow.changed = true;

                if (wdgItensFat.paramsNewCode && wdgItensFat.paramsNewCode.__is_new_code) {
                    wdgItensFat.currentRow.__using_unit = wdgItensFat.paramsNewCode.__using_unit || false;
                    wdgItensFat.currentRow.__new_code_pk = wdgItensFat.paramsNewCode.__new_code_pk;
                    wdgItensFat.currentRow.__table_pk = wdgItensFat.paramsNewCode.__table_pk;
                    wdgItensFat.currentRow.__new_code_length = wdgItensFat.paramsNewCode.__new_code_length || 5;
                    wdgItensFat.currentRow.__cast_number = wdgItensFat.paramsNewCode.__cast_number || false;
                    if (!wdgItensFat.currentRow.__columns_to_return && wdgItensFat.paramsNewCode && wdgItensFat.paramsNewCode.__columns_to_return)
                        wdgItensFat.currentRow.__columns_to_return = wdgItensFat.paramsNewCode.__columns_to_return;
                }
            }
        }

        General.widgetSave(widget).then(function (response) {
            if (response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0) {
                widget.currentRow.NRVENCPREVCONT = response.dataset.columnsToReturn[0].NRVENCPREVCONT;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
            }

            if (isNew && item) {
                wdgItensFat.currentRow.NRVENCPREVCONT = widget.currentRow.NRVENCPREVCONT;
                wdgItensFat.saveRow().then(function (response) {
                    widget.getField("NMITEMCONTRATO").isVisible = false;
                    widget.getField("VRTOTITEMCONT").isVisible = false;
                    ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso."));
                }.bind(this))
                    .catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
            }
            else {
                widget.getField("NMITEMCONTRATO").isVisible = false;
                widget.getField("VRTOTITEMCONT").isVisible = false;
                ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso."));
            }
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };

    /**
     * Valida as datas informadas na tela de faturamento
     * @param {Object} widget widget faturamento
     * @param {String} nameCampo nome do campo de data
     **/
    this.validaDatasFaturamento = function (widget, nameCampo) {
        var wdgCont = templateManager.container.getWidget("gridContrato");
        var dataIniCont = moment(wdgCont.currentRow.DTINICIOCONTRATO, "DD/MM/YYYY");
        var dataFimCont = moment(wdgCont.currentRow.DTFIMCONTRATO, "DD/MM/YYYY");
        var data = moment(widget.currentRow[nameCampo], "DD/MM/YYYY");

        if (data < dataIniCont || data > dataFimCont) {
            ScreenService.showMessage(ScreenService.i18n("Operação bloqueada.<br>A data informada não está presente no período de validade do contrato.<br>Período permitido: de " + dataIniCont.format("DD/MM/YYY") + " à " + dataFimCont.format("DD/MM/YYYY"))).then(function () {
                widget.getField(nameCampo).value(null);
            });
        }
        else {
            if (widget.currentRow.DTINICOMPET && nameCampo !== "DTINICOMPET") {
                var dataIniFat = moment(widget.currentRow.DTINICOMPET, "DD/MM/YYYY");
                if (data < dataIniFat) {
                    ScreenService.showMessage(ScreenService.i18n("Operação bloqueada.<br>A data informada necessita ser maior ou igual a data inicial")).then(function () {
                        widget.getField(nameCampo).value(null);
                    });
                }
            }
        }
    };


    /**
     * Preenche o filtro das tabs de Faturamento
     **/
    this.entraTabFaturamento = function (widget, nameCampo) {
        var wdgCont = templateManager.container.getWidget("gridContrato");

        var wdgFaturamento = templateManager.container.getWidget("faturamentocontrato");

        wdgFaturamento.dataSourceFilter[0].value = wdgCont.currentRow.NRORG;
        wdgFaturamento.dataSourceFilter[1].value = wdgCont.currentRow.NRSEQCONTRATO;
        wdgFaturamento.reload();

        var wdgMedicao = templateManager.container.getWidget("medicaoContrato");

        wdgMedicao.dataSourceFilter[0].value = wdgCont.currentRow.NRORG;
        wdgMedicao.dataSourceFilter[1].value = wdgCont.currentRow.NRSEQCONTRATO;
        wdgMedicao.reload();


    };

    /**
     * Preenche o filtro da tab de itens da Parcela
     **/
    this.entraTabItemFaturamento = function (widget, nameCampo) {
        var wdgFaturamento = templateManager.container.getWidget("faturamentocontrato");

        var item = templateManager.container.getWidget("Itensfaturamento");

        item.dataSourceFilter[0].value = wdgFaturamento.currentRow.NRORG;
        item.dataSourceFilter[1].value = wdgFaturamento.currentRow.NRSEQCONTRATO;

        if (wdgFaturamento.currentRow.NRVENCPREVCONT == null) {
            item.dataSourceFilter[2].name = "NRFATURACONT";
            item.dataSourceFilter[2].value = wdgFaturamento.currentRow.NRFATURACONT;
        }
        else {
            item.dataSourceFilter[2].name = "NRVENCPREVCONT";
            item.dataSourceFilter[2].value = wdgFaturamento.currentRow.NRVENCPREVCONT;
        }

        item.reload();
    };

    /**
     * Preenche o filtro da tab de itens da Medição
     **/
    this.entraTabItemMedicao = function (widget, nameCampo) {
        var wdgMedicao = templateManager.container.getWidget("medicaoContrato");

        var item = templateManager.container.getWidget("ItensMedicao");

        item.dataSourceFilter[0].value = wdgMedicao.currentRow.NRORG;
        item.dataSourceFilter[1].value = wdgMedicao.currentRow.NRSEQCONTRATO;
        item.dataSourceFilter[2].value = wdgMedicao.currentRow.NRMEDICAOFATURA;
        item.reload();
    };


    /* -------------------------------------------
       | -----  Funções faturamento Novo  ----- |
     -------------------------------------------*/

    /**
     * Controla as ações da aba de faturamento
     * de acordo com a situação da parcela
     * @param {Object} widget widget faturamento
    **/
    this.controlaAcessActionsFatNovo = function (widget) {
        if (widget.currentRow.IDSITUACAO === '1') {
            widget.getAction("edit_fat").isVisible = true;
            widget.getAction("delete_fat").isVisible = true;
        }
        else {
            widget.getAction("edit_fat").isVisible = false;
            widget.getAction("delete_fat").isVisible = false;
        }

    };

    /**
     * Atualiza dados da Parcela ao clicar na aba Parcelas do Menu Faturamento da tela Lançamento de Contrato.
    */
    this.abrirParcelasTab = function () {
        var wdgFaturamento = templateManager.container.getWidget("faturamentocontrato");
        var params = {
            "NRFATURACONT": wdgFaturamento.currentRow.NRFATURACONT
        };

        General.callRestEngine("product#/fct_contrato_widget/getDadosUmaParcela", "Row", params).then(function (response) {
            var parcela = response.dataset.PARCELA[0];
            wdgFaturamento.currentRow.VRVENCIMENTO = parcela.VRVENCIMENTO;
            wdgFaturamento.currentRow.VRFALTAFATURAR = parcela.VRFALTAFATURAR;
        }).catch(function (error) {
            Utilities.showErrorMessage(error);
        });
    };

    this.controlaVisibilidadeFatNovo = function (widget) {
        var wdgFaturamento = templateManager.container.getWidget("faturamentocontrato");
        var situacao = wdgFaturamento.currentRow.IDSITUACAO;

        if (widget.name == 'Itensfaturamento') {
            var nritfaturacont = widget.currentRow.NRITFATURACONT;
            if (nritfaturacont == null) {
                this.setVisibilidadeDasActionsFatNovo(widget, false);
            }
            else {
                if (situacao != '1') {
                    this.setVisibilidadeDasActionsFatNovo(widget, false);
                }
                else {
                    this.setVisibilidadeDasActionsFatNovo(widget, true);
                }
            }
        }
        else {
            if (situacao != '1') {
                this.setVisibilidadeDasActionsFatNovo(widget, false);
            }
            else {
                this.setVisibilidadeDasActionsFatNovo(widget, true);
            }
        }
    };

    /**
     * Ativa/desativa todas as actions de uma widget, menos as do tipo "back".
     * 
     * @param {Object} widget
     * @param {boolean} visivel
     */
    this.setVisibilidadeDasActionsFatNovo = function (widget, visivel) {
        var tipos = ["add", "edit", "delete"];
        widget.actions.forEach(function (action) {
            if (tipos.includes(action.name)) {
                action.isVisible = visivel;
            }
        });
    };

    /**
     * Atribui ao valor da parcela o valor do item escolhido
     * @param {Object} widget widget faturamento 
     **/
    this.modificarValorTotalParcelaNew = function (widget) {
        if (widget.currentRow.NRITEMOCONTRATO) {
            widget.currentRow.VRVENCIMENTO = widget.currentRow.VRTOTITEMCONT;
            widget.currentRow.VRFATURADO = 0;
            widget.currentRow.VRFALTAFATURAR = widget.currentRow.VRTOTITEMCONT;

            widget.getField("VRVENCIMENTO").readOnly = true;
            widget.getField("VRTOTITEMCONT").readOnly = false;
        }
        else {
            widget.currentRow.VRFATURADO = 0;
            widget.currentRow.VRFALTAFATURAR = widget.currentRow.VRVENCIMENTO;

            widget.currentRow.VRTOTITEMCONT = "";
            widget.currentRow.NMITEMCONTRATO = "";
            widget.currentRow.NRITEMOCONTRATO = "";
            widget.currentRow.QTITEMCONTRATO = "";
            widget.currentRow.CDTPMEDIDASERV = "";
            widget.currentRow.QTMEDIDACONTR = "";
            widget.currentRow.VRUNITITEMCONT = "";
            widget.getField("VRVENCIMENTO").readOnly = false;
            widget.getField("VRTOTITEMCONT").readOnly = true;
        }
    };

    /**
     * Atribui ao valor da parcela o valor do item escolhido
     * @param {Object} widget widget faturamento 
     **/

    this.modificarValorTotalItemNew = function (widget) {

        widget.currentRow.VRFATURADO = 0;
        //transformar o valor à faturar em um float entendido pelo DB
        var cache = widget.currentRow.VRVENCIMENTO;
        if (cache.includes('.')) {
            cache = cache.replace(".", "");
        }
        widget.currentRow.VRFALTAFATURAR = cache.replace(",", ".");
    };

    /**
     * Habilita a aba de faturamento para receber os dados de 
     * uma nova parcela
     * @param {Object} widget widget faturamento
     **/
    this.faturamentoNewRowNovo = function (widget) {
        var wdgContrato = templateManager.container.getWidget("gridContrato");
        General.widgetNewRow(widget);
        widget.currentRow.IDSITUACAO = "1";
        widget.currentRow.CDFILIALFATURA = wdgContrato.currentRow.CDFILIALORCA;
        widget.currentRow.NRPARCFATURA = wdgContrato.currentRow.NRPARCNEGOCIO;
        widget.currentRow.NMPRINCIPALPARC = wdgContrato.currentRow.NMPRINCIPALPARC;
        widget.currentRow.NRSEQCONTRATO = wdgContrato.currentRow.NRSEQCONTRATO;
        widget.currentRow.IDALTUSUARIO = "S";

        widget.getField("NMITEMCONTRATO").dataSourceFilter[0].value = wdgContrato.currentRow.NRSEQCONTRATO;
        widget.getField("NMITEMCONTRATO").reload();

        widget.getField("NMITEMCONTRATO").isVisible = true;
        widget.getField("VRTOTITEMCONT").isVisible = true;
    };

    /**
     * Salva as parcelas do faturamento
     * @param {Object} widget widget faturamento
     **/
    this.saveFaturamentoContratoNovo = function (widget) {
        var isNew = widget.currentRow.__is_new;
        var item = widget.currentRow.NRITEMOCONTRATO;
        var serv = widget.currentRow.CDSERVROCA;
        var prod = widget.currentRow.CDPRODUTO;
        var wdgItensFat = templateManager.container.getWidget("Itensfaturamento");
        var wdgContrato = templateManager.container.getWidget("gridContrato");

        if (!isNew) {
            this.validaAltUsuarioFaturaNovo(widget);
        }
        else {
            var nrseqmodcontra = wdgContrato.currentRow.NRSEQMODCONTRA;

            var params = {
                "NRSEQMODCONTRA": nrseqmodcontra
            };

            General.callRestEngine("product#/fct_contrato_widget/getParamVencimentos", "Row", params).then(function (response) {

                var qtDiasApuracao = response.dataset.PARAM[0].QTDIASAPURACAO;
                var idTpCalcVenc = response.dataset.PARAM[0].IDTPCALCVENC;
                var nrDiaCalcVenc = response.dataset.PARAM[0].NRDIACALCVENC;
                var idMesCalcVenc = response.dataset.PARAM[0].IDMESCALCVENC;
                var QtDiasFatuVenc = response.dataset.PARAM[0].QTDIASFATUVENC;

                /* limite para apuração */
                var dtLimiteApu = widget.currentRow.DTFINCOMPET;
                dtLimiteApu = moment(dtLimiteApu, "DD/MM/YYYY").add(qtDiasApuracao, 'days');

                dtLimiteApu = moment(dtLimiteApu).format("DD/MM/YYYY");

                widget.currentRow.DTLIMITEAPU = dtLimiteApu;
                /*vencimento*/

                if (idTpCalcVenc == '1') {

                    var dtReferencia = widget.currentRow.DTFINCOMPET;
                    var dtVencimento = null;
                    nrDiaCalcVenc = nrDiaCalcVenc.toString();

                    /*mes anterior*/
                    if (idMesCalcVenc == 1) {

                        dtReferencia = moment(dtReferencia, "MM/YYYY").subtract(1, 'months');
                        dtReferencia = dtReferencia.toString();

                    }
                    /*mês atual*/
                    else if (idTpCalcVenc == '2') {

                        dtReferencia = moment(dtReferencia, "MM/YYYY");
                        dtReferencia = dtReferencia.toString();

                    }
                    /*proximo mês*/
                    else {

                        dtReferencia = moment(dtReferencia, "MM/YYYY").add(1, 'months');
                        dtReferencia = dtReferencia.toString();

                    }

                    dtVencimento = nrDiaCalcVenc + '/' + dtReferencia;
                    dtVencimento = moment(dtVencimento).format("DD/MM/YYYY");

                    widget.currentRow.DTVENCIMENTO = dtVencimento;
                    widget.currentRow.NRDIASVENCIMENTO = nrDiaCalcVenc;
                    widget.currentRow.QTDIASFATUVENC = QtDiasFatuVenc;

                }
                else {
                    widget.currentRow.DTVENCIMENTO = null;
                    widget.currentRow.NRDIASVENCIMENTO = nrDiaCalcVenc;
                    widget.currentRow.QTDIASFATUVENC = QtDiasFatuVenc;
                }


                if (widget.currentRow.NRITEMOCONTRATO) {


                    wdgItensFat.currentRow.NRSEQCONTRATO = wdgContrato.currentRow.NRSEQCONTRATO;
                    wdgItensFat.currentRow.NRITEMOCONTRATO = widget.currentRow.NRITEMOCONTRATO;
                    if (prod) {
                        wdgItensFat.currentRow.CDPRODFATURA = prod;
                    }
                    else {
                        if (serv) {
                            prod = this.retornaProduto(serv);
                            wdgItensFat.currentRow.CDPRODFATURA = prod;

                        }
                        wdgItensFat.currentRow.CDPRODFATURA = null;
                    }
                    wdgItensFat.currentRow.IDSITUACAO = "1";
                    wdgItensFat.currentRow.VRVENCIMENTO = widget.currentRow.VRVENCIMENTO;
                    wdgItensFat.currentRow.VRFATURADO = widget.currentRow.VRFATURADO;
                    wdgItensFat.currentRow.VRFALTAFATURAR = widget.currentRow.VRFALTAFATURAR;
                    wdgItensFat.currentRow.IDFORMAFATURA = null;
                    wdgItensFat.currentRow.DSJUSTIFICATIVA = widget.currentRow.DSJUSTIFICATIVA;
                    wdgItensFat.currentRow.NRAPURACAO = null;
                    wdgItensFat.currentRow.__is_new = true;
                    wdgItensFat.currentRow.changed = true;


                    if (wdgItensFat.paramsNewCode && wdgItensFat.paramsNewCode.__is_new_code) {
                        wdgItensFat.currentRow.__using_unit = wdgItensFat.paramsNewCode.__using_unit || false;
                        wdgItensFat.currentRow.__new_code_pk = wdgItensFat.paramsNewCode.__new_code_pk;
                        wdgItensFat.currentRow.__table_pk = wdgItensFat.paramsNewCode.__table_pk;
                        wdgItensFat.currentRow.__new_code_length = wdgItensFat.paramsNewCode.__new_code_length || 5;
                        wdgItensFat.currentRow.__cast_number = wdgItensFat.paramsNewCode.__cast_number || false;
                        if (!wdgItensFat.currentRow.__columns_to_return && wdgItensFat.paramsNewCode && wdgItensFat.paramsNewCode.__columns_to_return)
                            wdgItensFat.currentRow.__columns_to_return = wdgItensFat.paramsNewCode.__columns_to_return;
                    }
                }

                General.widgetSave(widget).then(function (response) {
                    if (response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0) {
                        widget.currentRow.NRFATURACONT = response.dataset.columnsToReturn[0].NRFATURACONT;
                        widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                        widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                        widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                        widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                    }

                    if (isNew && item) {
                        wdgItensFat.currentRow.NRFATURACONT = widget.currentRow.NRFATURACONT;
                        wdgItensFat.currentRow.NRORG = widget.currentRow.NRORG;
                        wdgItensFat.currentRow.NRORGINCLUSAO = widget.currentRow.NRORGINCLUSAO;
                        wdgItensFat.currentRow.DTINCLUSAO = widget.currentRow.DTINCLUSAO;
                        wdgItensFat.currentRow.CDOPERINCLUSAO = widget.currentRow.CDOPERINCLUSAO;

                        wdgItensFat.saveRow().then(function (response) {
                            widget.getField("NMITEMCONTRATO").isVisible = false;
                            widget.getField("VRTOTITEMCONT").isVisible = false;
                            wdgItensFat.reload();
                            widget.reload();
                            ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso."));
                        })
                            .catch(function (error) {
                                Utilities.showErrorMessage(error);
                            });
                    }
                    else {
                        widget.getField("NMITEMCONTRATO").isVisible = false;
                        widget.getField("VRTOTITEMCONT").isVisible = false;
                        wdgItensFat.reload();
                        widget.reload();
                        ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso."));
                    }
                })
                    .catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
            }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
        }
    };

    /**
     * Realiza a regra de mudança do campo "Alterado pelo Usuário?"
     * da página "Faturamento" para "Sim", caso sejam alterados os 
     * campos citados abaixo.
     * 
     * @param {object} widget widget de 
     * Itens para pegar os fields para cálculo
     */
    this.validaAltUsuarioFaturaNovo = function (widget) {
        var currentRow = widget.currentRow;
        var backupRow = widget.backupRow;

        if (currentRow.DTINICOMPET != backupRow.DTINICOMPET || currentRow.DTFINCOMPET != backupRow.DTFINCOMPET ||
            currentRow.DTLIMITEAPU != backupRow.DTLIMITEAPU || currentRow.DTVENCIMENTO != backupRow.DTVENCIMENTO ||
            currentRow.VRVENCIMENTO != backupRow.VRVENCIMENTO) {
            widget.currentRow.IDALTUSUARIO = 'S';
        }
    };


    this.retornaProduto - function (serv) {

        var params = {
            "CDSERVORCA": serv
        };

        General.callRestEngine("product#/fct_contrato_widget/getProdServico", "Row", params).then(function (response) {

            var servico = response.dataset.SERV.CDPRODFATSERV;

            return servico;

        })
            .catch(function (error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
    };

    /**
     * Salva as alterações da parcela do item na aba de faturamento
     * @param {Object} widget widget itens faturamento
     **/
    this.saveItensFaturamentoNovo = function (widget) {
        General.widgetSave(widget).then(function (response) {

            var params = {
                "NRFATURACONT": widget.currentRow.NRFATURACONT,
                "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO
            };

            return General.callRestEngine("product#/fct_contrato_widget/AtulizaValorFaturamento", "Row", params);
        })
            .then(function () {
                ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
                this.abrirParcelasTab();
            }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };
    /**
     * Validação para que o swipe do view não seja aberto quando for selecionada
     * uma linha correspondente a uma medida variável, e não um item de proposta.
     */
    this.validarAbreSwipeItens = function (widget) {
        var isVisible = widget.currentRow.IDTIPO === "M";
        var isVisibleAgrup = (widget.currentRow.IDTIPO !== "A" && widget.currentRow.IDTIPO !== "M");
        if (widget.name == "wdgItensPropostaTree") {
            widget.getField("VRUNITITEMCONT").help = null;

            widget.getField("QTMEDIDACONTR").isVisible = isVisible;
            widget.getField("NMESTRUTURAM").isVisible = isVisibleAgrup;
            widget.getField("NMDOCITEMCONT").isVisible = isVisibleAgrup;
            widget.getField("VRUNITITEMCONTTBLPRECO").showOnView = isVisible;
            widget.getField("VRUNITITEMCONTTBLPRECO").showOnForm = isVisible;
            widget.getField("VRUNITITEMCONT").showOnView = isVisible;
            widget.getField("VRUNITITEMCONT").showOnForm = isVisible;

            if (widget.currentRow.IDTIPO !== "A") this.validarGeradorContrato(widget);
            else widget.getAction("edit").isVisible = (widget.currentRow.IDTIPO !== "A");
        }
        else {
            widget.getField("VRUNITITEMORCA").help = null;

            widget.getField("VRUNITITEMORCATBLPRECO").showOnView = isVisible;
            widget.getField("VRUNITITEMORCATBLPRECO").showOnForm = isVisible;
            widget.getField("VRUNITITEMORCA").showOnView = isVisible;
            widget.getField("VRUNITITEMORCA").showOnForm = isVisible;

            widget.getAction("act_edit").isVisible = (widget.currentRow.IDTIPO !== "A");
        }

        widget.getField("IDMULTMEDVRTOT").isVisible = isVisible;
        widget.getField("DSTPMEDIDASERV").showOnView = isVisible;
        widget.getField("DSTPMEDIDASERV").showOnForm = isVisible;
    };
    /**
     * Busca o parametro de idgeradopor
     * @params{Object} widget - widget que está solicitando a validação
     **/
    /*this.buscarIdGeradoPor = function(widget){
        var deferred    = ZHPromise.defer();
        var rowContrato = getWidgetGridContrato().currentRow;
        var params      = [
            {
                name: "NRSEQORCAMENTO",
                operator: '=',
                value: rowContrato.NRSEQORCAMENTO,
            }, 
            {
                name: "NRSEQCONTRATO",
                operator: "=",
                value: rowContrato.NRSEQCONTRATO
            }
        ];
        General.callRestEngine("product#/crud/fct_refeorcacont", "FilterData", params).then(function(response){
            if(response.dataset && response.dataset.fct_refeorcacont[0]){
                idGeradoPor = {
                    "NRSEQCONTRATO": rowContrato.NRSEQCONTRATO,
                    "IDGERADOPOR"  : response.dataset.fct_refeorcacont[0].IDGERADOPOR
                };
            }
            deferred.resolve();
        }.bind(this))
        .catch(function(error) {
            deferred.reject();
        });
        
        return deferred.promise;
    };*/
    /**
     * Valida de o contrato foi gerado pela proposta ou não para habilitar as 
     * actions de edição, inserção e exclusão
     * @params{Object} widget - widget que contém as actions a serem validadas 
     **/
    this.validarGeradorContrato = function (widget) {
        var rowContrato = getWidgetGridContrato().currentRow;
        var situacao = rowContrato.IDSITUACAOCONTRATO;
        if (situacao === 'P' || (situacao == 'A' && visibilidadeAcoes && idAlteraContAprv == 'S'))
            widget.getAction("edit").isVisible = (rowContrato.IDGERADOPOR == 'CONTRATO' || rowContrato.NRSEQCONTRATOPAI) ? true : false;
        else
            widget.getAction("edit").isVisible = false;
    };
    /**
     * Valida o tipo de desconto para voltar ao grid de itens
     * @params{Object} widget - widget itens controato tree
     **/
    this.voltarGridItensProposta = function (widget) {
        if (['P', 'V'].includes(widget.currentRow.IDTPDESCORCA)) {
            if (parseFloat(widget.currentRow.VRDESCORCA) > 0) {
                widget.currentRow.VRDESCORCA = parseFloat(widget.currentRow.VRDESCORCA.toString().replace(",", ".")) * -1;
                widget.currentRow.VRDESCORCA = widget.currentRow.VRDESCORCA.toFixed(2).replace(".", ",");
            }
        }
        ScreenService.closeSwipe();
        widget.reload();
    };

    /**
     * Abre o popup para a justificativa de uma renegociação ou um cancelamento
     * de uma porposta já aceita
     * @params{Object} widget widget 
     **/

    this.abrePopupDsJustEncReneg = function (widget, action, nameWdg) {
        if (nameWdg === undefined) {
            nameWdg = null;
        }

        ScreenService.closePopup();

        if (!nameWdg)
            nameWdg = (action === "encerrarContrato") ? "PopupDsJustificaEncerra" : "PopupDsJustCancReneg";
        var wdgPopup = templateManager.container.getWidget(nameWdg);

        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function () {
            wdgPopup.dataSource.data = [];
            wdgPopup.newRow();

            //Popup Justificativa de Renegociação do Contrato
            wdgPopup.currentRow.NRSEQCONTRATO = widget.currentRow.NRSEQCONTRATO;
            wdgPopup.currentRow.NRCONTRATOFCT = widget.currentRow.NRCONTRATOFCT;
            wdgPopup.currentRow.DSREFERENCIACONT = widget.currentRow.DSREFERENCIACONT;
            wdgPopup.currentRow.DTFIMCONTRATO = widget.currentRow.DTFINVIGENCONT;
            wdgPopup.currentRow.NRDIASPRAZOCONT = 0;

            if (action === "encerrarContrato") {
                wdgPopup.currentRow.DTPEDRESCISAOCONTR = moment().format("DD/MM/YYYY");
                wdgPopup.currentRow.DTRECISAOCONTR = moment().format("DD/MM/YYYY");
                wdgPopup.currentRow.IDRESCICAOEMMASSA = 'N';
            }

            wdgPopup.currentRow.ACTION = action;
            wdgPopup.redraw();
        }.bind(this));
    };

    /**
     * Validar o prazo popup  
     * @params {Widget} Widget widget popup de justificativa de cancelamento ou renegociação
    */
    this.validarPrazoPopupJustificativa = function (widget) {
        //pegar o valor da data de renegociação
        var renegociacaoData = widget.currentRow.DTRENEGOCIACONT;
        //pegar o valor da data de final 
        var finalData = widget.currentRow.DTFIMCONTRATO;
        //Subtrair a renegociacao - a final tendo como resultante o prazo
        var diffDatas = moment(finalData, "DD/MM/YYYY").diff(moment(renegociacaoData, "DD/MM/YYYY"), 'days');
        if (diffDatas > 0) {
            widget.currentRow.NRDIASPRAZOCONT = diffDatas + 1;
        } else if (renegociacaoData != "" && finalData != "") {
            widget.currentRow.NRDIASPRAZOCONT = "prazo inválido";
        }

    };

    /**
     * Salva a justificativa de cancelamento ou renegociação
     * @params{Object} widget widget popup de justificativa de cancelamento ou renegociação
    **/
    this.salvarJustificativaCancReneg = function (widget) {
        if (widget.currentRow.NRDIASPRAZOCONT > 0) {
            if (widget.isValid()) {
                var wdgContrato = templateManager.container.getWidget("gridContrato");

                if (wdgContrato.currentRow.IDSITUACAOCONTRATO == 'A') {

                    var nrtipoorcamento = wdgContrato.currentRow.NRTIPOORCAMENTO;
                    this.getIdParcelaConsiderada(nrtipoorcamento).then(function (idParcelaConsiderada) {
                        this.salvarRenegociacaoContrato(widget, wdgContrato, idParcelaConsiderada);
                    }.bind(this))
                        .catch(function (error) {
                            Utilities.showErrorMessage(error);
                        });
                }
                else {
                    this.salvarRenegociacaoContrato(widget, wdgContrato, null);
                }
            } else {
                ScreenService.showMessage("Operação bloqueada! <br/> Favor preencher todos os campos obrigatórios.");
            }
        } else {
            ScreenService.showMessage("Operação bloqueada! <br/> A nova data de renegociação deve ser maior que a nova data final.");
        }
    };

    this.salvarRenegociacaoContrato = function (widgetPopup, wdgContrato, idParcelaConsiderada) {
        var dataRenegociacao = widgetPopup.currentRow.DTRENEGOCIACONT;
        wdgContrato.currentRow.DSJUSTIFCANCRENEG = widgetPopup.currentRow.DSJUSTIFCANCRENEG;
        var novaDataFim = widgetPopup.currentRow.DTFIMCONTRATO; //Nova data final do contrato

        wdgContrato.saveRow().then(function () {
            ScreenService.closePopup();
            widgetPopup.isVisible = false;
            wdgContrato.activate();
            this.validarContratoAgrupamento(wdgContrato, idParcelaConsiderada, dataRenegociacao, novaDataFim);
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };

    /**
     * Salva a justificativa de cancelamento ou renegociação
     * @params{Object} widget widget popup de justificativa de cancelamento ou renegociação
     **/
    this.salvarJustificativaEncerra = function (widget) {
        if (widget.isValid()) {
            //usar a row criada e chamar a parte que solicita a rescisão
            if (widget.currentRow.IDRESCICAOEMMASSA == 'N') {
                this.solicitarRescisao(widget);
            } else {
                this.solicitarRescisaoMultiplos(widget);
            }
            /*if (widget.currentRow.IDRESCICAOEMMASSA == 'N'){
            
                var wdgContrato = templateManager.container.getWidget("gridContrato");
                wdgContrato.currentRow.IDTPEXCLUSAOPARCELA = widget.currentRow.IDTPEXCLUSAOPARCELA;
                wdgContrato.currentRow.DTRECISAOCONTR      = widget.currentRow.DTRECISAOCONTR;
                
                wdgContrato.currentRow.NRTIPORESCISAO = widget.currentRow.NRTIPORESCISAO;
                wdgContrato.currentRow.VRPENDENCIAFIN = widget.currentRow.VRPENDENCIAFIN;
                
                this.validaIdDocDocx(widget).then(function() {
    
                    ScreenService.closePopup();
                    widget.isVisible = false;
                    wdgContrato.activate();
                    
                    this.voltarParaContratoGrid();
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
            else {
                this.validaIdDocDocx(widget).then(function() {
                    this.voltarParaContratoGrid();
                    
                    ScreenService.closePopup();
                    widget.isVisible = false;

                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }*/
        }
    };

    /**
     * Valida se o contrato faz parte de um agrupamento
     * @param { Widget } Widget que contém os dados do Contrato 
     * @param { Number } Número do id da parcela
     * @param { any    } Nova data de renegociação do Contrato
     * @param { any    } Nova data final do Contrato
     * @param { number } Novo prazo do Contrato
    **/
    this.validarContratoAgrupamento = function (widget, idParcelaConsiderada, dataRenegociacao, dataFim) {
        var params = [
            {
                name: "NRSEQCONTRATO",
                operator: "=",
                value: widget.currentRow.NRSEQCONTRATO
            }
        ];
        General.callRestEngine("product#/crud/fct_relacagrupamentocont_widget", "FilterData", params).then(function (response) {
            if (response.dataset && response.dataset.fct_relacagrupamentocont_widget && response.dataset.fct_relacagrupamentocont_widget.length > 0) {
                ScreenService.confirmMessage(ScreenService.i18n("Atenção!<br>O contrato que está sendo renegociado faz parte de um agrupamento de contratos para faturamento.<br>Deseja adicionar o novo contrato ao mesmo?"), 'question', function () {
                    this.copiaVersionaContrato(widget, 'V', 'S', idParcelaConsiderada, dataRenegociacao, dataFim);
                }.bind(this), function () {
                    this.copiaVersionaContrato(widget, 'V', 'N', idParcelaConsiderada, dataRenegociacao, dataFim);
                }.bind(this));
            }
            else {
                this.copiaVersionaContrato(widget, 'V', 'N', idParcelaConsiderada, dataRenegociacao, dataFim);
            }
        }.bind(this));
    };

    /* Valida se o contrato faz parte de um agrupamento
     * @param { Widget } Widget que contém os dados do Contrato 
     * @param { Number } Número do id da parcela
     * @param { any    } Nova data de renegociação do Contrato
     * @param { any    } Nova data final do Contrato
     * @param { number } Novo prazo do Contrato
    **/
    this.validarContratoAgrupamento = function (widget, idParcelaConsiderada, dataRenegociacao, dataFim) {
        var params = [
            {
                name: "NRSEQCONTRATO",
                operator: "=",
                value: widget.currentRow.NRSEQCONTRATO
            }
        ];
        General.callRestEngine("product#/crud/fct_relacagrupamentocont_widget", "FilterData", params).then(function (response) {
            if (response.dataset && response.dataset.fct_relacagrupamentocont_widget && response.dataset.fct_relacagrupamentocont_widget.length > 0) {
                ScreenService.confirmMessage(ScreenService.i18n("Atenção!<br>O contrato que está sendo renegociado faz parte de um agrupamento de contratos para faturamento.<br>Deseja adicionar o novo contrato ao mesmo?"), 'question', function () {
                    this.copiaVersionaContrato(widget, 'V', 'S', idParcelaConsiderada, dataRenegociacao, dataFim);
                }.bind(this), function () {
                    this.copiaVersionaContrato(widget, 'V', 'N', idParcelaConsiderada, dataRenegociacao, dataFim);
                }.bind(this));
            }
            else {
                this.copiaVersionaContrato(widget, 'V', 'N', idParcelaConsiderada, dataRenegociacao, dataFim);
            }
        }.bind(this));
    };

    /**
     * Cancela o popup e ativa a widget que contem o nome do paramêtro
     * @params{String} widgetName nome da widget que será ativada
     **/
    this.cancelaPopup = function (widgetAtiva, widgetName) {
        var name = (widgetAtiva.currentRow.IDRESCICAOEMMASSA == 'S') ? 'wdgContratroGrid' : 'gridContrato';
        ScreenService.closePopup();
        if (widgetAtiva.currentRow) {
            widgetAtiva.currentRow = {};
            widgetAtiva.dataSource.data = [];
        }
        templateManager.container.getWidget(name).activate();
    };

    /**
     * Verifica se a proposta escolhida está aceita
     * Caso esteja abre um popup pedindo a justificativa 
     * @params{Object} widget widget proposta
     * @params{String} action nome da action que acionou a função
     **/
    this.validaSituacaoContEncReneg = function (widget, action) {
        var msgConfirmacao = null;
        if (widget.currentRow) {
            if (widget.currentRow.IDSITUACAOCONTRATO != 'A') {
                msgConfirmacao = ("Deseja realmente " + ((action === "encerrarContrato") ? "solicitar rescisão d" : "a renegociação ") + "o contrato?");
            }
            else {
                var contratos = templateManager.container.getWidget("wdgContratroGrid").getCheckedRows();
                if (action === "encerrarContrato")
                    msgConfirmacao = ("Deseja realmente " + ((action === "encerrarContrato") ? "solicitar rescisão d" : "a renegociação ") + "o contrato?");
                else
                    msgConfirmacao = "Ao renegociar este contrato serão realizadas as seguintes operações:<br><br> - A situação do contrato atual ficará como 'Encerrada'.<br><br> - Todas as parcelas de faturamento com situação 'Pendente' serão excluídas.<br><br>Deseja realmente renegociar o contrato?";
            }
        }
        try {
            ScreenService.confirmMessage(ScreenService.i18n(msgConfirmacao), 'question', function () {
                this.abrePopupDsJustEncReneg(widget, action);
            }.bind(this));
        } catch (err) {
            Utilities.showErrorMessage(err);
        }
    };

    /**
     * Retira a visibilidade das fields de escopo e modelo da proposta
     * @params{Object} widget - widget de contrato
     **/
    this.editarContrato = function (widget) {
        widget.getField("NRTIPOORCAMENTO").isVisible = false;
        widget.getField("NMORCAMENTOMODELO").isVisible = false;
        General.widgetEdit(widget);
    };
    /**
     * Volta para a aba de Contrato.
     * 
     * @param {object} widget - Widget de Elementos (inserindo)
     */
    this.voltarComposicao = function (widget) {
        widget.enabled = false;

        var gridContrato = templateManager.container.getWidget("gridContrato");
        var wdgTabbed = templateManager.container.getWidget("wdgTabbedContrato");

        wdgTabbed.enabled = true;
        gridContrato.enabled = true;
        wdgTabbed.activate();
    };
    /**
     * Avança para a aba de elemento caso o modelo utiliza a composição de serviço
     * @params{Object} widget - widget contrato
     **/
    this.avancarComposicao = function (widget) {
        if (!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
        else {
            var gridContrato = templateManager.container.getWidget("wdgTabbedContrato");
            gridContrato.enabled = false;
            var wdgElementos = templateManager.container.getWidget("wdgElementosContInserindo");
            wdgElementos.enabled = true;
            wdgElementos.isVisible = true;
            wdgElementos.mode = "edit";
            wdgElementos.activate().then(function () {
                this.setFiltroComposicao(wdgElementos);
            }.bind(this));
        }
    };
    /**
     * Seta o Modelo e os Escopos para o filtro da Composição.
     * 
     * @param {object} widget - Widget de Elementos (inserindo)
     */
    this.setFiltroComposicao = function (widget) {
        var gridContrato = templateManager.container.getWidget("gridContrato");
        var codModelo = gridContrato.currentRow.NRORCAMENTOMODELO;
        var escopos = gridContrato.currentRow.NRTIPOORCAMENTO;

        widget.dataSourceFilter[0].value = codModelo;
        widget.dataSourceFilter[1].value = escopos;
        widget.reload();
    };
    /**
     * Avança para a aba de Tipo de Medida Variável (inserindo).
     * 
     * @param {object} widget - Widget de Elementos (inserindo)
     */
    this.avancarTipoMedida = function (widget) {
        Sm00001000_LancamentoProposta_Service.validarObrigatoriedadeItensComposicao(widget.dataSource.data)
            .then(function (msg) {
                if (msg) {
                    ScreenService.showMessage(msg);
                }
                else {
                    widget.enabled = false;
                    var gridContrato = templateManager.container.getWidget("gridContrato");
                    var idutilizavarpreco = gridContrato.currentRow.IDUTILIZAVARPRECO;

                    if ((idutilizavarpreco) && (idutilizavarpreco.indexOf("S") > -1)) {
                        var wdgMedidaVariavel = templateManager.container.getWidget("wdgTipoMedidaContInserindo");
                        wdgMedidaVariavel.enabled = true;
                        wdgMedidaVariavel.isVisible = true;
                        wdgMedidaVariavel.mode = "edit";
                        wdgMedidaVariavel.activate().then(function () {
                            this.setFiltroTipoMedida(wdgMedidaVariavel);
                        }.bind(this));
                    }
                    else {
                        this.avancarItensContrato();
                    }
                }
            }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };
    /**
     * Volta para a aba de Elementos (inserindo).
     * 
     * @param {object} widget - Widget de Tipo de Medida (inserindo)
     */
    this.voltarTipoMedida = function (widget) {
        widget.enabled = false;
        var wdgElementos = templateManager.container.getWidget("wdgElementosContInserindo");
        wdgElementos.enabled = true;
        wdgElementos.mode = "edit";
        wdgElementos.activate().then(function () {
            setTimeout(function () {
                angular.element("label.zh-tab-cell")[1].click();
            }, 100);
        });
    };
    /**
     * Volta para a aba de tipo de medida (inserindo).
     * 
     * @param {object} widget - Widget de itens (inserindo)
     */
    this.voltarItensProposta = function (widget) {
        widget.enabled = false;
        var wdgMedida = templateManager.container.getWidget("wdgTipoMedidaContInserindo");
        wdgMedida.enabled = true;
        wdgMedida.mode = "edit";
        wdgMedida.activate().then(function () {
            setTimeout(function () {
                angular.element("label.zh-tab-cell")[2].click();
            }, 100);
        });
    };
    /**
     * Seta o filtro da widget de Tipo de Medida.
     * 
     * @param {object} widget - Widget de Tipo de Medida (inserindo)
     */
    this.setFiltroTipoMedida = function (widget) {
        var wdgItensComp = templateManager.container.getWidget("wdgElementosContInserindo");
        var idElementos = [];
        var idElementosRaiz = [];
        var elementosRaiz = [];
        wdgItensComp.dataSource.data.forEach(function (row) {
            if (row.TIPO == "E" && row.__isSelected && row._parent.TIPO == "C") {
                if (!idElementosRaiz.includes(row.IDELEMENTO)) {
                    idElementosRaiz.push(row.IDELEMENTO);
                    elementosRaiz.push(row);
                }
            }
        });

        wdgItensComp.dataSource.data.forEach(function (row) {
            if (row.TIPO == "E" && row.__isSelected) {
                idElementos.push(row.IDELEMENTO);
            }
        });

        idElementos = idElementos.filter(function (elem, i) {
            return idElementos.indexOf(elem) == i;
        });

        var wdgContrato = templateManager.container.getWidget("gridContrato");
        var escopos = wdgContrato.currentRow.NRTIPOORCAMENTO;

        var params = {
            "ID_ELEMENTOS": idElementos,
            "ELEMENTOS_RAIZ": elementosRaiz,
            "ESCOPOS": escopos
        };

        General.callRestEngine("product#/fct_orcamento_widget/getTiposMedidaVariaveisParaInserir", "Row", params).then(function (success) {
            widget.dataSource.data = success.dataset.tipos_medida_variavel;
            widget.edit();
            widget.getAction("save").isVisible = false;
            widget.getAction("cancel").isVisible = false;
        }.bind(this),
            function (error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
    };
    /**
     * Avança para a aba de Itens (inserindo).
     */
    this.avancarItensContrato = function () {
        var wdgMedida = templateManager.container.getWidget("wdgTipoMedidaContInserindo");
        var wdgItens = templateManager.container.getWidget("itemcontrato");
        wdgItens.isVisible = false;
        var gridContrato = templateManager.container.getWidget("gridContrato");
        var wdgComposicao = templateManager.container.getWidget("wdgElementosContInserindo");
        var wdgMedidas = templateManager.container.getWidget("wdgTipoMedidaContInserindo");
        var elementosEscolhidos = [];
        var composicoes = [];

        wdgComposicao.dataSource.data.forEach(function (row) {
            if (row.__isSelected && row.TIPO == "E") {
                elementosEscolhidos.push({
                    "IDELEMENTO": row.IDELEMENTO,
                    "NRCOMPSERVICO": row.NRCOMPSERVICO
                });
            }
            else if (row.TIPO == "C") {
                composicoes.push({
                    "NRCOMPSERVICO": row.CDELEMENTO,
                    "NMCOMPSERVICO": row.NMELEMENTO,
                    "NRCOMPSERVICOPAI": row.IDARVOREPAI,
                    "IDCOMPOBRIGATORIA": row.IDSERVOBRIGCOMP,
                    "IDESCOLHIDO": row.__isSelected ? "S" : "N"
                });
            }
        });

        var params = {
            "PROPOSTA": { "NRSEQORCAMENTO": null, "NRTIPOORCAMENTO": gridContrato.currentRow.NRTIPOORCAMENTO },
            "ELEMENTOS_ESCOLHIDOS": elementosEscolhidos,
            "COMPOSICOES": composicoes,
            "MEDIDAS": wdgMedidas.dataSource.data
        };
        General.callRestEngine("product#/fct_orcamento_widget/getItensPropostaParaInserir", "Row", params).then(function (success) {
            var wdgItensComposicao = templateManager.container.getWidget("wdgItensContratoInserindo");
            wdgMedida.enabled = false;

            wdgItensComposicao.dataSource.data = success.dataset.fct_itemorcamento;
            this.habilitarWidgetNomeContratual(success.dataset.fct_itemorcamento);
        }.bind(this),
            function (error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
    };
    /**
     * Valida a quantidade de escopos selecionados
     * @params{Object} widget - widget de contrato
     **/
    this.validarQuantidadeEscopos = function (widget) {
        var gridContrato = templateManager.container.getWidget("gridContrato");
        if (!gridContrato.currentRow.NRTIPOORCAMENTO || gridContrato.currentRow.NRTIPOORCAMENTO.length == 1) {
            this.avancarComposicao(widget);
        }
        else {
            ScreenService.confirmMessage("Atenção!<br>A seleção de mais de um escopo resultará na criação de mais de um contrato.<br>Deseja continuar?", "question", function () {
                this.avancarComposicao(widget);
            }.bind(this));
        }
    };
    /**
     * Manda salvar a proposta e os contratos com a composição de serviço 
     **/
    this.salvarContratoCompMedida = function () {
        var gridContrato = templateManager.container.getWidget("gridContrato");
        var wdgElementos = templateManager.container.getWidget("wdgElementosContInserindo");
        var wdgTipoMedida = templateManager.container.getWidget("wdgTipoMedidaContInserindo");
        var wdgItens = templateManager.container.getWidget("wdgItensContratoInserindo");
        var wdgNomeCont = templateManager.container.getWidget("wdgNomeContratual");
        var wdgTabbed = templateManager.container.getWidget("wdgTabbedPrincipal");
        var wdgTabbedContrato = templateManager.container.getWidget("wdgTabbedContrato");
        var countEscopos = gridContrato.currentRow.NRTIPOORCAMENTO.length;

        var params = {
            "CONTRATO": gridContrato.currentRow,
            "MEDIDAS": wdgTipoMedida.dataSource.data,
            "ELEMENTOS": wdgElementos.dataSource.getCheckedRows(),
            "ITENS": wdgItens.dataSource.data
        };

        General.callRestEngine("product#/fct_contrato_widget/salvarContratoCompMedida", "Row", params).then(function (response) {
            if (countEscopos != 1) {
                ScreenService.showMessage("Contratos salvo com sucesso!");
            }
            else {
                ScreenService.confirmMessage("Contrato salvo com sucesso!<br><br>Deseja editar a Negociação de Valores?", "question", function () {
                    var currentRow = {
                        "NRSEQCONTATO": response.dataset.columnsToReturn[0].NRSEQCONTRATO,
                        "NRORG": response.dataset.columnsToReturn[0].NRORG
                    };
                    this.abrirWidgetNegociacaoValores(currentRow);
                }.bind(this));
            }
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            }).finally(function () {
                wdgElementos.isVisible = false;
                wdgTipoMedida.isVisible = false;
                wdgItens.isVisible = false;

                wdgNomeCont.isVisible = false;
                wdgNomeCont.list();

                wdgTabbedContrato.enabled = true;
                this.voltarParaContratoGrid();
                this.controlarAbasContrato(true, gridContrato.name);
                wdgTabbed.view();
            }.bind(this));
    };
    /**
     * Edita os elementos do contrato
     * @params{Object} widget - widget de elementos
     **/
    this.editarElementosContrato = function (widget) {
        widget.templateManager = "widget/master_detail/tree/grid.html";

        widget.edit();
        widget.showCheckbox = true;
        widget.selectAll();

       // var eleProposta = Util.clone(widget.dataSource.data, []);
        //this.marcarElementosEscolhidos(widget, eleProposta);

        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        var widgets = wdgTabbedPrincipal.widgets.filter(function (wid) {
            return wid.name != widget.name;
        });

        General.changeWidgetAcessibility(widgets, false);

        /*
        var gridContrato = getWidgetGridContrato();
        var eleProposta = Util.clone(widget.dataSource.data, []);
        var params = [{
            "name": "NRORCAMENTOMODELO",
            "operator": "=",
            "value": gridContrato.currentRow.NRORCAMENTOMODELO
        },
        {
            "name": "NRTIPOORCAMENTO",
            "operator": "IN",
            "value": gridContrato.currentRow.NRTIPOORCAMENTO
        }
        ];

         var nomeDataSourceFind = "composicao_elemento_proposta";

        if (gridContrato.currentRow.NRSEQCONTRATOPAI && gridContrato.currentRow.NRVERSAOCONTRATO > 1) {
            params.push({
                "name": "NRSEQCONTRATO",
                "operator": "=",
                "value": gridContrato.currentRow.NRSEQCONTRATO
            });

            nomeDataSourceFind = "composicao_elemento_contrato_renegociado";
        }

        General.callRestEngine("product#/crud/" + nomeDataSourceFind, "FilterData", params).then(function (response) {
            widget.dataSource.data = response.dataset[nomeDataSourceFind];
            widget.edit();
            widget.showCheckbox = true;
            this.marcarElementosEscolhidos(widget, eleProposta);

            var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
            var widgets = wdgTabbedPrincipal.widgets.filter(function (wid) {
                return wid.name != widget.name;
            });

            General.changeWidgetAcessibility(widgets, false);
        }.bind(this));
        */
    };

    /**
     * Marca os elementos que já foram escolhidos ao clicar em editar.
     * 
     * @param {object} widget
     */
    this.marcarElementosEscolhidos = function (widget, elementos) {
        widget.dataSource.data.forEach(function (row) {
            if (row.TIPO == 'C') row.OLDCOMPPROPOSTA = 'N';
            elementos.forEach(function (elemento) {
                if (elemento["IDELEMENTO"] == row["IDELEMENTO"] && elemento["NRCOMPSERVICO"] == row["NRCOMPSERVICO"]) {
                    if (row.TIPO == 'C') {
                        row.OLDCOMPPROPOSTA = 'S';
                    }
                    row.__isSelected = true;
                    widget.dataSource.addCheckedRows(row);
                    row.CDCENTCUST = elemento["CDCENTCUST"];
                }
            });
        });
    };
    /**
     * Ação de cancelar dos Elementos da Proposta (editando).
     * 
     * @param {object} widget - Widget de Elementos da Proposta (editando)
     */
    this.cancelarElementosContrato = function (widget) {
        widget.reload().then(function () {
            widget.list();
            widget.showCheckbox = false;
            widget.unSelectAll();
            var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
            var widgets = wdgTabbedPrincipal.widgets.filter(function (wid) {
                return wid.name != widget.name;
            });
            General.changeWidgetAcessibility(widgets, true);
        });
    };
    /**
     * Atualiza os Elementos da Proposta.
     * 
     * @param {object} widget - Widget de Elementos da Proposta (editando)
     */
    this.atualizarComposicao = function (widget, manterDescAcres) {
        var params = [{
            name: "NRSEQORCAMENTO",
            operator: "=",
            value: getWidgetGridContrato().currentRow.NRSEQORCAMENTO
        }];
        General.callRestEngine("product#/crud/fct_refeorcacont_widget", "FilterData", params).then(function (response) {
            if (response.dataset && response.dataset.fct_refeorcacont_widget.length > 0) {
                var escopos = [];
                response.dataset.fct_refeorcacont_widget.forEach(function (referencia) {
                    escopos.push(referencia.NRTIPOORCAMENTO);
                });
            }
            var rowContrato = getWidgetGridContrato().currentRow;
            var infoProposta = {
                "IDSITUACAOORCAMENTO": 'ACEITO',
                "NRTIPOORCAMENTO": escopos,
                "NRSEQORCAMENTO": rowContrato.NRSEQORCAMENTO,
                "NRORCAMENTOMODELO": rowContrato.NRORCAMENTOMODELO
            };

            var compElementos = this.getElementosParaAtualizacao(widget);
            var elementos = compElementos["ELEMENTOS"];
            var composicoes = compElementos["COMPOSICOES"];
            var params = {
                "PROPOSTA": infoProposta,
                "ELEMENTOS": elementos,
                "COMPOSICOES": composicoes,
                "TPALTERACAO": "ELEMENTOS",
                "CONTRATO": getWidgetGridContrato().currentRow,
                "TPTRATAMENTO": manterDescAcres
            };

            Sm00001000_LancamentoProposta_Service.validarObrigatoriedadeItensComposicao(widget.dataSource.data).then(function (mensagem) {
                if (mensagem) {
                    ScreenService.showMessage(mensagem);
                }
                else {
                    General.callRestEngine("product#/fct_contrato_widget/atualizarItensContratoProposta", "Row", params).then(function (response) {
                        this.cancelarElementosContrato(widget);
                    }.bind(this),
                        function (error) {
                            Utilities.showErrorMessage(error);
                        });
                }
            }.bind(this));
        }.bind(this));
    };
    /**
     * Retorna o pai da lista de elementos
     * @params{Object} row - row inicial
     **/
    this.getElementoPaiServico = function (row) {
        if (row.__isSelected && row._parent && row._parent.TIPO == 'C')
            return null;
        return row.IDARVORE;
    };

    /**
     * Atualiza os tipos de medida variáveis da proposta.
     * 
     * @param {object} widget - Widget de Tipo de Medida da Proposta (editando)
     */
    this.atualizarTipoDeMedidaOrcaCont = function (widget, manterDescAcres) {
        var gridContrato = getWidgetGridContrato();
        var params = [{
            name: "NRSEQORCAMENTO",
            operator: "=",
            value: gridContrato.currentRow.NRSEQORCAMENTO
        }];
        General.callRestEngine("product#/crud/fct_refeorcacont_widget", "FilterData", params).then(function (response) {
            if (response.dataset && response.dataset.fct_refeorcacont_widget.length > 0) {
                var escopos = [];
                response.dataset.fct_refeorcacont_widget.forEach(function (referencia) {
                    escopos.push(referencia.NRTIPOORCAMENTO);
                });
            }

            var params = {
                "PROPOSTA": {
                    "IDSITUACAOORCAMENTO": "ACEITO",
                    "NRSEQORCAMENTO": gridContrato.currentRow.NRSEQORCAMENTO,
                    "NRTIPOORCAMENTO": escopos,
                    "NRORCAMENTOMODELO": gridContrato.currentRow.NRORCAMENTOMODELO,
                },
                "TPALTERACAO": "TPMEDIDAS",
                "CONTRATO": getWidgetGridContrato().currentRow,
                "MEDIDA": widget.currentRow,
                "TPTRATAMENTO": manterDescAcres
            };
            General.callRestEngine("product#/fct_contrato_widget/atualizarItensContratoProposta", "Row", params).then(function () {
                widget.list();
                widget.reload();
            }.bind(this),
                function (error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
        }.bind(this),
            function (error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
    };
    /**
     * Calcula o valor total do item do contrato.
     * 
     * @param {object} widget - Widget de Itens do contrato
     */
    this.calcularValorTotalItem = function (widget) {
        if (widget.mode != "list" && widget.mode != "inline_edit") {
            var isEdit = (widget.name == "wdgItensPropostaTree");
            var utilizaQtdeMedida = widget.currentRow.IDMULTMEDVRTOT;
            var quantidade = widget.currentRow[((isEdit) ? "QTITEMCONTRATO" : "QTITEMORCAMENTO")];
            var valorUnitario = widget.currentRow[((isEdit) ? "VRUNITITEMCONT" : "VRUNITITEMORCA")];
            var qtdeMedida = widget.currentRow[((isEdit) ? "QTMEDIDACONTR" : "QTMEDIDASERV")];
            var tipoAjuste = widget.currentRow[((isEdit) ? "IDTPDESCONTO" : "IDTPDESCORCA")];

            if (quantidade) quantidade = parseFloat(Utilities.tirarMascaraValores(quantidade));
            if (qtdeMedida) qtdeMedida = parseFloat(Utilities.tirarMascaraValores(qtdeMedida));
            if (valorUnitario) valorUnitario = parseFloat(Utilities.tirarMascaraValores(valorUnitario));

            var valorSemAjuste = quantidade * valorUnitario;
            if (utilizaQtdeMedida == "S")
                valorSemAjuste *= qtdeMedida;
            var valorTotal = valorSemAjuste;

            if (tipoAjuste && tipoAjuste != 'S' && tipoAjuste != 'N') {
                var ajuste = widget.currentRow.VRDESCORCA;
                var vrperdescorca = widget.currentRow.VRPERCDESCORCA;

                if (ajuste) ajuste = parseFloat(Utilities.tirarMascaraValores(ajuste));
                if (vrperdescorca) vrperdescorca = parseFloat(Utilities.tirarMascaraValores(vrperdescorca));
                if (!widget.currentRow.__is_new && ["P", "Y"].includes(tipoAjuste))
                    ajuste = valorTotal * vrperdescorca / 100;
                else
                    vrperdescorca = ajuste * 100 / valorSemAjuste;

                if (!widget.currentRow.IDTIPO || widget.currentRow.IDTIPO !== "I") {
                    if (["V", "P"].includes(tipoAjuste) && (ajuste <= valorTotal || ajuste > 0)) {
                        if (ajuste < 0)
                            ajuste *= -1;

                        valorTotal -= ajuste;
                    }
                    else if (["X", "Y"].includes(tipoAjuste)) {
                        valorTotal += ajuste;
                    }
                }

                widget.currentRow.VRDESCORCA = ajuste;
                widget.currentRow.VRPERCDESCORCA = vrperdescorca;
                widget.currentRow.VRITEMSEMDESC = valorSemAjuste;
            }

            widget.currentRow[(isEdit) ? "VRTOTITEMCONT" : "VRTOTALITEMORCA"] = valorTotal;
            templateManager.updateTemplate();
        }
    };
    /**
     * Valida se a alteração na quantidade de medida é permitida
     * @params{Object} row - currentRow itens
     **/
    this.validarAlteracaoQtMedida = function (row, nomeCampoMedida) {
        var deferred = ZHPromise.defer();
        UtilitiesPropostaContrato.validarPermissaoAlteracaoQtMedida().then(function (permiteAlteracao) {
            var qtmedida = row[nomeCampoMedida];
            if (!qtmedida) qtmedida = row.QTMEDIDACONTR;

            qtmedida = Utilities.tirarMascaraValores(qtmedida);
            var qtmedidacalc = Utilities.tirarMascaraValores(row.QTMEDIDACALCULADA);
            var qtmedidabackup = Utilities.tirarMascaraValores(row.QTMEDIDACONTR_GRID);

            if (qtmedida !== qtmedidacalc && row.IDTIPO == 'M') {
                if (row.IDALTQTDMEDIDAPROP == "4") { //nao altera
                    ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido alterar a Quantidade de Medida deste item.");
                    row[nomeCampoMedida] = qtmedidabackup;
                    deferred.reject();
                }
                else if (row.IDALTQTDMEDIDAPROP == "3" && qtmedida > qtmedidacalc) { //somente reduzir
                    ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido aumentar a Quantidade de Medida deste item. O campo foi atribuído com o valor máximo (" + qtmedidacalc.formatFloat(2) + ").");
                    row[nomeCampoMedida] = qtmedidabackup;
                    deferred.reject();
                }
                if (qtmedida < qtmedidacalc) {
                    if (row.IDALTQTDMEDIDAPROP != "2") {
                        if (!permiteAlteracao) {
                            ScreenService.showMessage("Operação bloqueada.<br/>O operador logado não tem permissão para diminuir a quantidade de medida deste item.");
                            row[nomeCampoMedida] = qtmedidabackup;
                            deferred.reject();
                        }
                    }
                    else {
                        if (permiteAlteracao)
                            ScreenService.showMessage("Operação bloqueada.<br/>O escopo de faturamento deste item não permite a diminuição de sua quantidade de medida.<br>Contate um administrador e tente novamente.");
                        else
                            ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido reduzir a Quantidade de Medida deste item. O campo foi atribuído com o valor mínimo (" + qtmedidacalc.formatFloat(2) + ").");

                        row[nomeCampoMedida] = qtmedidabackup;
                        deferred.reject();
                    }
                }
            }
            deferred.resolve();
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });

        return deferred.promise;
    };
    /**
     * Verifica o Tipo de Faturamento para bloquear/desbloquear o campo Modalidade de Pagamento.
     * 
     * @param {object} widget
     */
    this.bloquearModalidadePagamento = function (widget) {
        var idfaturaorca = widget.currentRow.IDFATURAORCA;
        var campoCodModalidadePgto = widget.getField("CDMODPGTO");
        var campoNomeModalidadePgto = widget.getField("DSMODPGTO");
        var campoQuantidadeParcelas = widget.getField("QTPARCELASFATU");
        var campoDiaVencimentoParcela = widget.getField("NRDIAVENCPARCELASFATU");

        if (idfaturaorca == "O") { //modalidade de pagamento
            campoNomeModalidadePgto.readOnly = false;
            campoNomeModalidadePgto.validations = {
                "required": true
            };

            campoQuantidadeParcelas.readOnly = true;
            campoQuantidadeParcelas.value(null);
            campoQuantidadeParcelas.validations = {};

            campoDiaVencimentoParcela.validations = {};
        }
        else {
            campoCodModalidadePgto.value(null);
            campoNomeModalidadePgto.readOnly = true;
            campoNomeModalidadePgto.value(null);
            campoNomeModalidadePgto.validations = {};

            if (idfaturaorca == "P") { //parcelamento
                campoQuantidadeParcelas.readOnly = false;
                campoQuantidadeParcelas.validations = {
                    "required": true
                };

                campoDiaVencimentoParcela.validations = {
                    "required": true
                };
            }
            else { //periodico
                campoQuantidadeParcelas.readOnly = true;
                campoQuantidadeParcelas.value(null);
                campoQuantidadeParcelas.validations = {};

                campoDiaVencimentoParcela.validations = {};
            }
        }
    };
    /**
     * Sintetiza as funções em uma mesma função
     * 
     * @param {object} widget - Widget de Itens do contrato
     */
    this.funcoesDoCampoAjuste = function (widget) {
        this.controlarCampoValorDoAjuste(widget);
        this.calcularValorTotalItem(widget);
        if (widget.currentRow.VRPERCDESCORCA) {
            this.adequaAjusteAoCampo(widget);
        }
    };
    /**
     * Controla o campo de desconto de acordo com o tipo.
     * 
     * @param {object} widget - Widget de Itens do contrato
     */
    this.controlarCampoValorDoAjuste = function (widget) {
        var tipoAjuste = widget.currentRow.IDTPDESCORCA ? widget.currentRow.IDTPDESCORCA : "N";
        var fieldAjuste = widget.getField("VRDESCORCA");
        var fieldPercAjuste = widget.getField("VRPERCDESCORCA");
        var vrItemSAjuste = widget.currentRow.VRITEMSEMDESC;
        if (vrItemSAjuste) vrItemSAjuste = parseFloat(vrItemSAjuste.toString().replace(",", "."));

        widget.getField('IDTPDESCORCA').readOnly = false;
        if (tipoAjuste == "N") {
            fieldAjuste.readOnly = (tipoAjuste == "N");
            fieldPercAjuste.readOnly = (tipoAjuste == "N");
            fieldAjuste.value("");
            fieldPercAjuste.value("");
        }
        else {
            var readOnly = ["P", "Y"].includes(tipoAjuste);
            fieldPercAjuste.readOnly = !readOnly;
            fieldAjuste.readOnly = readOnly;

            var ajuste = fieldAjuste.value();
            var vrpercajuste = fieldPercAjuste.value();
            if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));
            if (vrpercajuste) vrpercajuste = parseFloat(vrpercajuste.toString().replace(",", "."));

            if (!readOnly) {
                fieldPercAjuste.value("");
                if (ajuste) fieldPercAjuste.value(parseFloat(((ajuste * 100) / vrItemSAjuste).toFixed(2)));
            }
            else {
                fieldAjuste.value("");
                if (vrpercajuste) fieldAjuste.value((vrItemSAjuste * (vrpercajuste / 100)).toFixed(2).replace(".", ","));
            }
        }
    };

    /**
     * Valida se o valor unitário informado no campo é menor que 
     * o valor unitário calculado da tabela de preços.
     * @param {object} widget 
     */
    this.validarValorUnitarioItem = function (widget) {
        var isEdit = (widget.name == "wdgItensPropostaTree");

        var nomeCampoValorUnitario = "VRUNITITEMCONT";
        var nomeCampoValorTblPreco = "VRUNITITEMCONTTBLPRECO";
        if (!isEdit) {
            nomeCampoValorUnitario = "VRUNITITEMORCA";
            nomeCampoValorTblPreco = "VRUNITITEMORCATBLPRECO";
        }

        var vrUnitarioItem = Utilities.tirarMascaraValores(widget.currentRow[nomeCampoValorUnitario]);
        var vrunititemtblpreco = Utilities.tirarMascaraValores(widget.currentRow[nomeCampoValorTblPreco]);

        var wdgContrato = getWidgetGridContrato();
        var renegociando = (wdgContrato.currentRow.NRVERSAOCONTRATO > 1 && wdgContrato.currentRow.NRSEQCONTRATOPAI);

        var params = {
            "NRTIPOORCAMENTO": wdgContrato.currentRow.NRTIPOORCAMENTO
        };

        General.callRestEngine("product#/fct_orcamento_widget/validaBloqAlteraValor", "Row", params).then(function (response) {

            var idaltvrtabrpecprop = response.dataset.retorno.idaltvrtabrpecprop;

            if (idaltvrtabrpecprop == '2' && vrUnitarioItem < vrunititemtblpreco) {
                if (renegociando) {
                    ScreenService.showMessage("Atenção! <br/><br/>O campo \"Valor Unitário\" foi alterado para um valor menor que o campo \"Valor Unitário da Tabela de Preços\".");
                }
                else {
                    ScreenService.showMessage("Operação bloqueada.<br/><br/>O campo \"Valor Unitário\" não pode ser menor que o campo \"Valor Unitário da Tabela de Preços\".");
                    widget.currentRow[nomeCampoValorUnitario] = vrunititemtblpreco;
                }
            }
            else if (idaltvrtabrpecprop == '3' && vrUnitarioItem > vrunititemtblpreco) {
                if (renegociando) {
                    ScreenService.showMessage("Atenção! <br/><br/>O campo \"Valor Unitário\" foi alterado para um valor maior que o campo \"Valor Unitário da Tabela de Preços\".");
                }
                else {
                    ScreenService.showMessage("Operação bloqueada.<br/><br/>O campo \"Valor Unitário\" não pode ser maior que o campo \"Valor Unitário da Tabela de Preços\".");
                    widget.currentRow[nomeCampoValorUnitario] = vrunititemtblpreco;
                }
            }
            else if (idaltvrtabrpecprop == '4' && vrUnitarioItem != vrunititemtblpreco) {
                if (renegociando) {
                    ScreenService.showMessage("Atenção! <br/><br/>O campo \"Valor Unitário\" foi alterado para um valor diferente do campo \"Valor Unitário da Tabela de Preços\".");
                }
                else {
                    ScreenService.showMessage("Operação bloqueada.<br/><br/>O campo \"Valor Unitário\" não pode ter um valor diferente do valor do o campo \"Valor Unitário da Tabela de Preços\".");
                    widget.currentRow[nomeCampoValorUnitario] = vrunititemtblpreco;
                }
            }

            this.calcularValorTotalItem(widget);

        }.bind(this));
    };

    /**
     * Confirma a edição dos elementos e medidas dos itens de composição de serviço
     * @params{Object} widget - widget que solicitou a confirmação 
     **/
    this.confirmarEdicaoContratoComp = function (widget) {
        //desabilitar os outros elementos do cancelar e salvar
        var elementosProposta = templateManager.container.getWidget('wdgElementosProposta');
        var saveEdit = elementosProposta.getAction('save_comp_serv');
        var cancelarEdit = elementosProposta.getAction('cancel_comp_serv');
        cancelarEdit.isVisible = saveEdit.isVisible = false;
        var wdgContrato = getWidgetGridContrato();
        if (wdgContrato.currentRow.NRSEQCONTRATOPAI) {
            this.editarElementosContrato(widget);
        }
        else {
            var params = [{
                name: "NRSEQORCAMENTO",
                operator: "=",
                value: getWidgetGridContrato().currentRow.NRSEQORCAMENTO
            },
            {
                name: "IDATIVO",
                operator: "=",
                value: 'S'
            }
            ];

            General.callRestEngine("product#/crud/fct_refeorcacont_widget", "FilterData", params).then(function (response) {
                if (!response.dataset || response.dataset.fct_refeorcacont_widget.length <= 1) {
                    this.editarElementosContrato(widget);
                }
                else {
                    var contratos = [];
                    response.dataset.fct_refeorcacont_widget.forEach(function (referencia) {
                        if (referencia.NRSEQCONTRATO !== getWidgetGridContrato().currentRow.NRSEQCONTRATO)
                            contratos.push(referencia.NRCONTRATOFCT + " - " + referencia.DSREFERENCIACONT);
                    });
                    contratos = contratos.join(",<br>");
                    ScreenService.confirmMessage("Atenção!<br>A alteração dos elementos e medidas desse contrato acarretará em mudanças no(s) contrato(s): <br>" + contratos + "<br><br>Deseja continuar?", "question", function () {
                        this.editarElementosContrato(widget);
                    }.bind(this));
                }
            }.bind(this));
        }
    };
    /**
     * Cancela a adição e inserção dos parametros de faturamento 
     * @params{Object} widget - widget parametro de faturmanento itens
     **/
    this.cancelWdgParamFaturamentoItens = function (widget) {
        Utilities.formWidgetCancel(widget);
        var wdgItens = templateManager.container.getWidget("wdgItensPropostaTree");
        wdgItens.widgets.forEach(function (widget) {
            widget.enabled = true;
        });

        wdgItens.enabled = true;
        ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoOnEnter(widget, "wdgCompetenciaItem");
    };
    /**
     * Salva um parametro de faturamento do item
     * @params {Object} widget - widget de faturmanto itens
     **/
    this.saveParamFaturamentoItens = function (widget) {
        var wdgItens = templateManager.container.getWidget("wdgItensPropostaTree");
        Utilities.formWidgetSaveAndSetColumnsToReturn(widget).then(function () {
            ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoOnEnter(widget, "wdgCompetenciaItem");
            wdgItens.widgets.forEach(function (widget) {
                widget.enabled = true;
            });

            wdgItens.enabled = true;
        });
    };
    /**
     * Coloca a widget de parametros fo faturamento do item em modo edição
     * @params {Object} widget - widget parametros de faturamento do item
     **/
    this.editarParamFaturamentoItens = function (widget) {
        var wdgItens = templateManager.container.getWidget("wdgItensPropostaTree");
        General.widgetEdit(widget);
        wdgItens.widgets.forEach(function (wdg) {
            if (wdg.name !== widget.name)
                wdg.enabled = false;
        });
        wdgItens.enabled = false;
        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(widget, "wdgCompetenciaItem");
    };
    /**
     * Coloca a widget de parametros fo faturamento do item em modo inserção
     * @params {Object} widget - widget parametros de faturamento do item
     **/
    this.adicionarParamFaturamentoItem = function (widget) {
        var wdgItens = templateManager.container.getWidget("wdgItensPropostaTree");
        General.widgetNewRow(widget);
        wdgItens.widgets.forEach(function (wdg) {
            if (wdg.name !== widget.name)
                wdg.enabled = false;
        });
        wdgItens.enabled = false;
        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(widget, "wdgCompetenciaItem");
    };
    /** Abre a widget de nome contratual durante a inserção da proposta
     * @params{Array} dataSource - dataSource da tela
     **/
    this.habilitarWidgetNomeContratual = function (dataSource) {
        var widget = templateManager.container.getWidget("wdgNomeContratual");
        var idarvore = null;
        var nmInserido = [];

        widget.dataSource.data = [];
        dataSource.forEach(function (item) {
            if (item.IDTIPO === 'M') {
                idarvore = item.IDARVORE.split("_");
                idarvore = idarvore[0] + "_" + idarvore[1];
                if (nmInserido.indexOf(idarvore) === -1) {
                    widget.dataSource.data.push({
                        "NRITEMOCONTRATO": null,
                        "NMITEMCONTRATO": item.NMSERVORCA,
                        "NMCONTRATUALITEMCONT": item.NMSERVORCA,
                        "IDELEMENTO": idarvore
                    });
                    nmInserido.push(idarvore);
                }
            }
        });

        widget.enabled = true;
        widget.isVisible = true;
        widget.activate().then(function () {
            widget.edit();
            widget.getAction("back").isVisible = true;
            widget.getAction("avancarItens").isVisible = true;
        });
    };
    /**
     * Avançar para a widget de itens inserindo
     * @params{Object} widget - widget de nome contratual
     **/
    this.avancarItensInserindo = function (widget) {
        var wdgItens = templateManager.container.getWidget("wdgItensContratoInserindo");
        var idelemento = null;
        widget.enabled = false;

        wdgItens.enabled = true;
        wdgItens.isVisible = true;
        widget.dataSource.data.forEach(function (itemNmCont) {
            wdgItens.dataSource.data.forEach(function (item) {
                idelemento = item.IDARVORE.split("_");
                idelemento = idelemento[0] + "_" + idelemento[1];
                if (itemNmCont.IDELEMENTO === idelemento)
                    item.NMCONTRATUALITEMORCA = itemNmCont.NMCONTRATUALITEMCONT;
            });
        });

        wdgItens.activate().then(function () {
            wdgItens.list();
            wdgItens.mode = "edit";
        }.bind(this));
    };
    /**
     * Volta para a widget de nome contratual
     * @params{Object} widget - widget de itens da proposta inserindo
     **/
    this.voltarWdgNomeContratual = function (widget) {
        var alteracaoItens = widget.dataSource.data.filter(function (item) {
            return item.changed;
        });

        var wdgNmContratual = templateManager.container.getWidget("wdgNomeContratual");
        if (alteracaoItens.length == 0) {
            widget.enabled = false;
            wdgNmContratual.enabled = true;
            wdgNmContratual.activate();
        }
        else {
            ScreenService.confirmMessage("Atenção!<br>Voltar para a etapa de Nome Contratual resultará na perca das alterações realizadas nos itens.<br>Deseja continuar?", "question", function () {
                widget.enabled = false;
                wdgNmContratual.enabled = true;
                wdgNmContratual.activate();
            });
        }
    };
    /**
     * Desabilita as fields de "Cancelar" e "Salvar Alterações"
     * da widget de nome contratual quando estiver em modo inserção
     * @params{Object} widget - widget de nome contratual
     **/
    this.desabilitarFieldsInline = function (widget) {
        var isVisible = (getWidgetGridContrato().mode !== "view");
        widget.getAction('cancel').isVisible = !isVisible;
        widget.getAction('save').isVisible = !isVisible;
        widget.getAction('back').isVisible = isVisible;
        widget.getAction('avancarItens').isVisible = isVisible;
        templateManager.updateTemplate();
    };
    /**
     * Valida se a widget de nome contratual está sendo editada para 
     * carregar os dados
     * @params{Object} widget - widget nome contratual
     **/
    this.validarOnEnterWdgNmContratual = function (widget) {
        if (getWidgetGridContrato().mode != "view") {
            widget.floatingControl = false;
        }
        else {
            Utilities.carregarWidgetOnEnterTab(widget);
            widget.floatingControl = {
                customizationAction: {
                    filter: false,
                    layout: false,
                    view: false
                }
            };
        }
    };
    /**
     * Cancela as edições na widget de contrato
     * @params{Object} widget - widget de contrato
     **/
    this.cancelarWidgetContrato = function (widget) {
        var wdgTabbed = templateManager.container.getWidget("wdgTabbedPrincipal");
        General.widgetCancelForm(widget);
        widget.currentRow.changed = false;
        delete widget.backupRow;

        if (!widget.currentRow.__is_new && wdgTabbed) {
            wdgTabbed.view();
            this.controlarAbasContrato(true, widget.name);
        }
        else {
            var wdgElementos = templateManager.container.getWidget("wdgElementosContInserindo");
            var wdgTipoMedida = templateManager.container.getWidget("wdgTipoMedidaContInserindo");
            var wdgItens = templateManager.container.getWidget("wdgItensContratoInserindo");
            var wdgNomeCont = templateManager.container.getWidget("wdgNomeContratual");
            wdgElementos.isVisible = false;
            wdgTipoMedida.isVisible = false;
            wdgItens.isVisible = false;
            wdgNomeCont.isVisible = false;
        }
    };
    /**
     * Manda salvar um nome contratual
     * @params{Object} widget - widget nome contratual
     **/
    this.saveNomeContratual = function (widget) {
        var rowContrato = getWidgetGridContrato().currentRow;
        var idReplicaOrca = (rowContrato.IDGERADOPOR == "CONTRATO" && rowContrato.NRVERSAOCONTRATO == 1 && rowContrato.NRSEQORCAMENTO != null);
        var rowsAlteradas = widget.dataSource.data.filter(function (row) {
            return row.changed;
        });
        var params = {
            "CONTRATO": rowContrato.NRSEQCONTRATO,
            "IDREPLICARORCA": idReplicaOrca,
            "ROWSALTERADAS": rowsAlteradas
        };
        General.callRestEngine("product#/fct_itemcontrato_widget/salvarNomeContratualContrato", "Row", params).then(function () {
            widget.list();
            widget.reload().then(function () {
                ScreenService.showMessage("Processo realizado com sucesso!");
            });
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };
    /**
     * Volta para o modo list da tela de contrato.
     * 
     * @param {object} widget - Widget de Contrato form
     */
    this.voltarParaContratoGrid = function () {
        var wdgContrato = templateManager.container.getWidget("gridContrato");
        wdgContrato.dataSource.data = [];
        wdgContrato.currentRow = {};

        ScreenService.closeSwipe().then(function () {
            var wdgContratroGrid = templateManager.container.getWidget("wdgContratroGrid");
            wdgContratroGrid.activate();
            wdgContratroGrid.refreshWidget();
        });
    };
    /**
     * Manipula a acessibilidade das demais abas e coloca a widget 
     * de dados do contrato em modo de edição
     * @params{Object} widget - widget de dados do contrato
     **/
    this.editarDadosContrato = function (widget) {
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        widget.currentRow.changed = true;

        this.validaBloqueiaDataInicialContReneg(widget);

        if (!wdgTabbedPrincipal) {
            General.widgetEdit(widget);
        }
        else {
            var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];

            widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
            widget.edit();
            wdgTabbedPrincipal.edit();

            widget.getAction("Avancar").isVisible = false;
            widget.getAction("SAVE").isVisible = true;

            this.controlarAbasContrato(false, widget.name);
            this.habilitaCampoVingenciaAtual(widget, false);
            this.assinaturaEletronicaOnChange(widget);
            this.integracaoFMOnChange(widget);
            this.integracaoTecFoodOnChange();
            this.integracaoHCMOnChange(widget);
            this.alterarVisibilidadeAgrupamento(widget);

            widget.getField("NMTIPOORCAMENTO_CONT").readOnly = (widget.currentRow.IDSITUACAOCONTRATO != "P");
        }

        mudouMoeda = false;
    };
    /**
     * Manipula a acessibilidade das abas
     * @params{Object} widget - widget que será colocada ou retirada do modo edição
     * @params{Bool} enabled - situação das demais widget
     **/
    this.controlarAbasContrato = function (enabled, nameWidget) {
        var wdgTabbedContrato = templateManager.container.getWidget("wdgTabbedContrato");
        wdgTabbedContrato.widgets.filter(function (wid) {
            return wid.name != nameWidget;
        }).forEach(function (widget) {
            widget.enabled = enabled;
        });

        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        wdgTabbedPrincipal.widgets.filter(function (wid) {
            return wid.name != "wdgTabbedContrato";
        }).forEach(function (widget) {
            widget.enabled = enabled;
        });

        this.controlaVisibilidadeLocalAt(templateManager.container.getWidget("localAtendContrato"));
        templateManager.updateTemplate();
    };
    /**
     * Coloca a widget de dados gerais em modo de edição
     * @params{Object} widget - widget dados gerais (parametros de faturamento)
     **/
    this.colocarWdgParaFaturamentoEdit = function (widget, actionName) {
        if (actionName == "add")
            ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoNewRow(widget, 'gridContrato', 'wdgParametrosFaturamentoTabbed', 'wdgParamFaturamentoCompetCont');
        else
            ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoEdit(widget, 'gridContrato', 'wdgParametrosFaturamentoTabbed', 'wdgParamFaturamentoCompetCont');
        var wdgTabbed = templateManager.container.getWidget("wdgTabbedPrincipal");
        wdgTabbed.edit();
        this.controlarAbasContrato(false, widget.name);
        templateManager.container.getWidget("wdgParametrosFaturamentoTabbed").enabled = true;
    };

    /**
     * Coloca a widget de dados gerais em modo de visualização
     * @params{Object} widget - widget dados gerais (parametros de faturamento)
     **/
    this.voltarWdgParamFaturamentoView = function (widget, actionName) {
        if (actionName != 'save') {
            ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoCancel(widget, 'gridContrato', 'wdgParametrosFaturamentoTabbed', 'wdgParamFaturamentoCompetCont');
            this.alterarAcessibilidadeParamFat(widget);
        }
        else {
            var regerarParcelas = widget.currentRow.IDTPFATURA == widget.backupRow.IDTPFATURA;
            var wdgContrato = templateManager.container.getWidget("gridContrato");

            if (regerarParcelas || wdgContrato.currentRow.IDSITUACAOCONTRATO != "A" || !possuiAgrup) {
                ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoSave(widget, 'gridContrato', 'wdgParametrosFaturamentoTabbed');
                this.alterarAcessibilidadeParamFat(widget);
            }
            else {
                ScreenService.confirmMessage("Atenção!<br>Alterações no tipo de faturamento de contratos ligados a agrupamentos resultará na regeração de suas parcelas.<br>Deseja continuar?", "question", function () {
                    ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoSave(widget, 'gridContrato', 'wdgParametrosFaturamentoTabbed');
                    var params = { "NRSEQCONTRATO": wdgContrato.currentRow.NRSEQCONTRATO };
                    General.callRestEngine('product#/fct_contrato_widget/geraParcelasContratoNovoFat', "Row", params).then(function () {
                        this.alterarAcessibilidadeParamFat(widget);
                    }.bind(this));
                }.bind(this));
            }
        }
    };
    /**
     * Altera a acessibilidade da widget de parametros de faturamento do contrato
     * @params{Object} widget - widget parametros de faturamento do contrato
     **/
    this.alterarAcessibilidadeParamFat = function (widget) {
        var wdgTabbed = templateManager.container.getWidget("wdgTabbedPrincipal");
        wdgTabbed.view();
        this.controlarAbasContrato(true, widget.name);
        templateManager.container.getWidget("wdgParametrosFaturamentoTabbed").enabled = true;
    };

    this.wdgNegociacaoValoresOnEnterTab = function (wdgNegociacaoValores) {
        Utilities.carregarWidgetOnEnterTab(wdgNegociacaoValores);
        this.controlarEdicaoNegociacaoValores(wdgNegociacaoValores);
    };

    this.controlarEdicaoNegociacaoValores = function (wdgNegociacaoValores) {
        var wdgContrato = templateManager.container.getWidget("gridContrato");
        var idGeradoPor = wdgContrato.currentRow.IDGERADOPOR;
        var situacao = wdgContrato.currentRow.IDSITUACAOCONTRATO;

        var wdgItensNegociacao = templateManager.container.getWidget("wdgItensNegociacao");
        var wdgDescontoPorPeriodo = templateManager.container.getWidget("wdgDescontoPorPeriodo");
        var wdgParcelas = templateManager.container.getWidget("wdgParcelas");
        var wdgCompetenciaNegociacao = templateManager.container.getWidget("wdgCompetenciaNegociacao");
        var widgetsDescontoParcela = [wdgDescontoPorPeriodo, wdgParcelas, wdgCompetenciaNegociacao];
        var visibilidade;

       /* if (idGeradoPor == "PROPOSTA") {
            Verificar na rotina de permitir alterar quando regerar a negociação

            if (situacao == 'P' && wdgContrato.currentRow.NRSEQCONTRATOPAI)
                visibilidade = true;
            else
                visibilidade = false;

            wdgNegociacaoValores.getField("IDTPFATURA").readOnly = !visibilidade;
            wdgNegociacaoValores.getField("VRPERCDESCITEM").readOnly = !visibilidade;
            wdgNegociacaoValores.getField("VRDESCITEM").readOnly = !visibilidade;
            wdgItensNegociacao.getField("VRPERCDESCITEM").readOnly = !visibilidade;
            wdgItensNegociacao.getField("VRDESCITEM").readOnly = !visibilidade;
            wdgItensNegociacao.getField("NRPARCFATURA").readOnly = !visibilidade;

            this.setVisibilidadeDasActionsExcetoVoltar([wdgDescontoPorPeriodo, wdgParcelas], visibilidade);
            this.setVisibilidadeDasActionsExcetoVoltar([wdgCompetenciaNegociacao], (situacao == 'P'));
        }
        else {*/
            if (situacao != "P") {
                if (situacao == "A")
                    visibilidade = (idAlteraContAprv == 'S');
                else
                    visibilidade = true;

                wdgNegociacaoValores.getField("VRPERCDESCITEM").readOnly = !(visibilidadeAcoes && visibilidade);
                wdgNegociacaoValores.getField("VRDESCITEM").readOnly = !(visibilidadeAcoes && visibilidade);
                wdgItensNegociacao.getField("VRPERCDESCITEM").readOnly = !(visibilidadeAcoes && visibilidade);
                wdgItensNegociacao.getField("VRDESCITEM").readOnly = !(visibilidadeAcoes && visibilidade);
                wdgNegociacaoValores.getField("IDTPFATURA").readOnly = false;
                wdgNegociacaoValores.getField("NRPARCFATURA").readOnly = false;

                this.setVisibilidadeDasActionsExcetoVoltar(widgetsDescontoParcela, (visibilidadeAcoes && visibilidade));
            }
            else {
                wdgNegociacaoValores.getField("VRPERCDESCITEM").readOnly = false;
                wdgNegociacaoValores.getField("VRDESCITEM").readOnly = false;
                wdgItensNegociacao.getField("VRPERCDESCITEM").readOnly = false;
                wdgItensNegociacao.getField("VRDESCITEM").readOnly = false;
                wdgNegociacaoValores.getField("IDTPFATURA").readOnly = false;
                wdgItensNegociacao.getField("NRPARCFATURA").readOnly = false;

                this.setVisibilidadeDasActionsExcetoVoltar(widgetsDescontoParcela, true);
            }
       /* } */
    };

    this.wdgNegociacaoValoresAfterMoveRow = function (wdgNegociacaoValores) {
        var idtpfatura = wdgNegociacaoValores.currentRow.IDTPFATURA;
        var wdgParcelas = templateManager.container.getWidget("wdgParcelas");
        var wdgDescontoPorPeriodo = templateManager.container.getWidget("wdgDescontoPorPeriodo");
        var wdgCompetenciaNegociacao = templateManager.container.getWidget("wdgCompetenciaNegociacao");

        if (idtpfatura == "P") {
            wdgParcelas.isVisible = true;
            wdgDescontoPorPeriodo.isVisible = false;
            wdgCompetenciaNegociacao.isVisible = false;
        }
        else {
            wdgParcelas.isVisible = false;
            wdgDescontoPorPeriodo.isVisible = true;
            wdgCompetenciaNegociacao.isVisible = true;
        }
    };

    this.wdgDescontoPorPeriodoOnEnterTab = function (wdgDescontoPorPeriodo) {
        if (getWidgetNegociacaoDeValores().currentRow.IDTPFATURA != 'X') {
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").readOnly = true;
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").hideEmptyValue = false;
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").defaultValue = null;
        }
        else {
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").readOnly = false;
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").hideEmptyValue = true;
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").defaultValue = "P";
        }
        Utilities.carregarWidgetOnEnterTab(wdgDescontoPorPeriodo);
    };

    this.wdgDescontoPorPeriodoNewRow = function (wdgDescontoPorPeriodo) {
        General.widgetNewRow(wdgDescontoPorPeriodo);
        this.controlarCamposDesconto(wdgDescontoPorPeriodo);

        var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
        wdgDescontoPorPeriodo.currentRow.VRTOTNEGCONT = wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC;
    };

    /*this.atualizarValorItensNegociacaoValores = function(row) {
        var nritemorcamento = row.NRITEMORCAMENTO;
        if (row._parent)
            nritemorcamento = row._parent.NRITEMORCAMENTO;

        var params = {
            "NRSEQORCAMENTO": row.NRSEQORCAMENTO,
            "NRITEMORCAMENTO": nritemorcamento,
            "VRTOTALITEMORCA": row.VRTOTALITEMORCA
        };

        General.callRestEngine("product#/fct_itemcontrato_widget/atualizarValorItensNegociacaoValores", "Row", params).then(null).catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };*/

    this.salvarParcelaNegociacao = function (wdgParcelas) {
        if (wdgParcelas.isValid()) {
            Utilities.validarCamposDuplicados(wdgParcelas, ["NRPARCELANEGOCONT"], ["QTDIASFATURAMENTO", "NRNEGOCIACAOCONTRATO"]).then(function (response) {
                if (!response) {
                    ScreenService.showMessage('Operação bloqueada. <br/>Não é possível inserir 2 registros com o mesmo valor para o campo "Quantidade de Dias para Faturamento".');
                }
                else {
                    var rowContrato = getWidgetGridContrato().currentRow;
                    var is_new = wdgParcelas.currentRow.__is_new;
                    wdgParcelas.currentRow.IDREPLICARORCA = (rowContrato.IDGERADOPOR == "CONTRATO" && rowContrato.NRVERSAOCONTRATO == 1 && rowContrato.NRSEQORCAMENTO != null);
                    wdgParcelas.currentRow.NRNEGOCIACAOPROPOSTA = templateManager.container.getWidget("wdgNegociacaoValores").currentRow.NRNEGOCIACAOPROPOSTA;

                    General.callRestEngine("product#/fct_contrato_widget/salvarParcelasNegociacao", "Row", wdgParcelas.currentRow).then(function () {

                        var params = {
                            "NRNEGOCIACAOCONTRATO": wdgParcelas.currentRow.NRNEGOCIACAOCONTRATO
                        };

                        General.callRestEngine("product#/fct_contrato_widget/atualizarNrParcelas", "Row", params).then(function () {
                            if (!is_new) {
                                wdgParcelas.view();
                                wdgParcelas.reload();
                            }
                            else {
                                ScreenService.closeSwipe();
                                wdgParcelas.list();
                                wdgParcelas.reload();
                            }
                        }).catch(function (error) {
                            Utilities.showErrorMessage(error);
                        });
                    }).catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            });
        }
    };

    this.salvarDescontoPorPeriodo = function (wdgDescontoPorPeriodo) {
        if (wdgDescontoPorPeriodo.isValid()) {
            this.validarNegociacaoValorDescontoOperadorEPerfil(wdgDescontoPorPeriodo.currentRow).then(function (continuar) {
                if (continuar) {
                    Utilities.validarCamposDuplicados(wdgDescontoPorPeriodo, ["NRDESCPERIODONEGOCONT"], ["NRORDEMAPLICADESC", "NRNEGOCIACAOCONTRATO"]).then(function (response) {
                        if (response) {
                            Utilities.validarCamposDuplicados(wdgDescontoPorPeriodo, ["NRDESCPERIODONEGOCONT"], ["VRPERCDESCITEM", "NRNEGOCIACAOCONTRATO"]).then(function (response2) {
                                if (response2) {
                                    var rowContrato = getWidgetGridContrato().currentRow;
                                    var wdgNegociacao = templateManager.container.getWidget("wdgNegociacaoValores");
                                    var is_new = wdgDescontoPorPeriodo.currentRow.__is_new;
                                    if (rowContrato.IDGERADOPOR == 'CONTRATO' && !rowContrato.NRSEQCONTRATOPAI && rowContrato.NRSEQORCAMENTO)
                                        wdgDescontoPorPeriodo.currentRow.IDREPLICARORCA = true;
                                    else
                                        wdgDescontoPorPeriodo.currentRow.IDREPLICARORCA = false;
                                    wdgDescontoPorPeriodo.currentRow.NRNEGOCIACAOPROPOSTA = wdgNegociacao.currentRow.NRNEGOCIACAOPROPOSTA;
                                    wdgDescontoPorPeriodo.currentRow.VRPERCDESCITEM = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRPERCDESCITEM);
                                    wdgDescontoPorPeriodo.currentRow.VRDESCITEM = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRDESCITEM);
                                    wdgDescontoPorPeriodo.currentRow.VRTOTITEMCOMDESC = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRTOTITEMCOMDESC);
                                    General.callRestEngine("product#/fct_contrato_widget/salvarDescontoPorPeriodo", "DataSet", [wdgDescontoPorPeriodo.currentRow]).then(function () {
                                        var params = {
                                            "NRNEGOCIACAOCONTRATO": wdgDescontoPorPeriodo.currentRow.NRNEGOCIACAOCONTRATO,
                                            "NRDESCPERIODONEGOCONT": wdgDescontoPorPeriodo.currentRow.NRDESCPERIODONEGOCONT,
                                            "QTMESESDESC": wdgDescontoPorPeriodo.currentRow.QTMESESDESC,
                                            "DTINIVIGENCONT": rowContrato.DTINIVIGENCONT
                                        };
                                        General.callRestEngine("product#/fct_contrato_widget/salvarDataFinalValidadeNegociacao", "Row", params).then(function (response) {
                                            var dataValidade = response.dataset.retorno.dataValidade;
                                            wdgDescontoPorPeriodo.currentRow.DTFINALVALIDADEDESC = dataValidade;
                                        }).catch(function (error) {
                                            Utilities.showErrorMessage(error);
                                        }).finally(function () {
                                            if (!is_new) {
                                                wdgDescontoPorPeriodo.view();
                                            }
                                            else {
                                                ScreenService.closeSwipe();
                                                wdgDescontoPorPeriodo.list();
                                                wdgDescontoPorPeriodo.reload();
                                            }
                                        });
                                    }).catch(function (error) {
                                        Utilities.showErrorMessage(error);
                                    });
                                }
                                else {
                                    ScreenService.showMessage("Operação bloqueada. <br/>Já existe um registro com esse percentual de desconto.");
                                }
                            })
                                .catch(function (error) {
                                    Utilities.showErrorMessage(error);
                                });
                        }
                        else {
                            ScreenService.showMessage('Operação bloqueada. <br/>Não é possível inserir 2 registros com o mesmo valor para o campo "Ordem de Aplicação".');
                        }
                    }.bind(this)).catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            }).catch(function (error) {
                Utilities.showErrorMessage(error);
            });
        }
    };

    this.abrirWdgParcelarNegociacaoPorIgual = function () {
        var wdgParcelarNegociacaoPorIgual = templateManager.container.getWidget("wdgParcelarNegociacaoPorIgual");
        wdgParcelarNegociacaoPorIgual.isVisible = true;
        ScreenService.openPopup(wdgParcelarNegociacaoPorIgual);
        wdgParcelarNegociacaoPorIgual.dataSource.data = [];
        wdgParcelarNegociacaoPorIgual.newRow();
    };

    this.parcelarNegociacaoPorIgual = function (wdgParcelarNegociacaoPorIgual) {
        if (wdgParcelarNegociacaoPorIgual.isValid()) {
            this.validarQuantidadeMaxParcelas(wdgParcelarNegociacaoPorIgual, "PARCELAPORIGUAL").then(function () {
                var rowContrato = getWidgetGridContrato().currentRow;
                var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
                var idReplicaOrca = (rowContrato.IDGERADOPOR == "CONTRATO" && rowContrato.NRVERSAOCONTRATO == 1 && rowContrato.NRSEQORCAMENTO != null);

                var params = {
                    "NRNEGOCIACAOCONTRATO": wdgNegociacaoValores.currentRow.NRNEGOCIACAOCONTRATO,
                    "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA,
                    "QTPARCELAS": wdgParcelarNegociacaoPorIgual.currentRow.QTPARCELAS,
                    "QTDIASINTERVALO": wdgParcelarNegociacaoPorIgual.currentRow.QTDIASINTERVALO,
                    "NRDIAVENCIMENTO": wdgParcelarNegociacaoPorIgual.currentRow.NRDIAVENCIMENTO,
                    "QTCARENCIADIAS": wdgParcelarNegociacaoPorIgual.currentRow.QTCARENCIADIAS,
                    "VRTOTITEMCOMDESC": wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC,
                    "IDREPLICARORCA": idReplicaOrca
                };

                General.callRestEngine("product#/fct_contrato_widget/parcelarNegociacaoPorIgual", "Row", params).then(function (response) {
                    ScreenService.closePopup();
                    ScreenService.showMessage("Operação realizada com sucesso!");
                    templateManager.container.getWidget("wdgParcelas").reload();
                }).catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
            }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
        }
    };

    this.setVisibilidadeDasActionsExcetoVoltar = function (widgets, visivel) {
        if (!Array.isArray(widgets)) {
            widgets = [widgets];
        }

        widgets.forEach(function (wid) {
            wid.actions.filter(function (act) {
                if (!act.showAsAction.startsWith("back")) {
                    act.isVisible = visivel;
                }
            });
        });
    };

    /*this.setDataFinalValidadeDescontoPorPeriodo = function(wdgDescontoPorPeriodo) {
        var qtdMeses = wdgDescontoPorPeriodo.currentRow.QTMESESDESC;
        if (qtdMeses) {
            var wdgContrato = templateManager.container.getWidget("gridContrato");
            var dtInicioContrato = wdgContrato.currentRow.DTINIVIGENCONT;
            if (!dtInicioContrato) {
                dtInicioContrato = wdgContrato.currentRow.DTINICIOCONTRATO;
            }
            
            dtInicioContrato = dtInicioContrato.split(" ")[0];
            dtInicioContrato = moment(dtInicioContrato, "DD/MM/YYYY");
            
            wdgDescontoPorPeriodo.currentRow.DTFINALVALIDADEDESC = dtInicioContrato.add(qtdMeses, "months").calendar();
        }
    };*/

    this.salvarWdgNegociacaoValores = function (wdgNegociacaoValores) {
        if (wdgNegociacaoValores.isValid()) {
            var vrpercdescitem = wdgNegociacaoValores.currentRow.VRPERCDESCITEM;
            if (!vrpercdescitem) vrpercdescitem = 0;

            vrpercdescitem = Utilities.tirarMascaraValores(vrpercdescitem);

            if (vrpercdescitem > 0) {
                var params = {
                    "NRNEGOCIACAOCONTRATO": wdgNegociacaoValores.currentRow.NRNEGOCIACAOCONTRATO
                };

                General.callRestEngine("product#/fct_itemcontrato_widget/verificarSeExisteDescontoItensNegociacaoContrato", "Row", params)
                    .then(function (success) {
                        if (success.dataset.retorno.existe) {
                            ScreenService.showMessage("Operação bloqueada. <br/>Não é possível dar desconto na negociação pois já existe desconto específico informado para seus itens.");
                        }
                        else {
                            General.callRestEngine("product#/fct_itemcontrato_widget/getServicoPercentualMinimoDescontoPermitidoNegociacaoContrato", "Row", params)
                                .then(function (response) {
                                    var resultado = response.dataset.retorno.servico;
                                    if (resultado && vrpercdescitem > resultado.VRPERCDESC) {
                                        var contexto = (resultado.TIPO == 1) ? "Este" : "O perfil de acesso deste";
                                        ScreenService.showMessage("Operação bloqueada.<br/>" + contexto + " operador não têm autorização para oferecer desconto superior a " + resultado.VRPERCDESC + "% para o serviço \"" + resultado.NMSERVORCA + "\".");
                                    }
                                    else {
                                        salvarNegociacaoValores(wdgNegociacaoValores);
                                    }
                                })
                                .catch(function (error) {
                                    Utilities.showErrorMessage(error);
                                });
                        }
                    })
                    .catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
            }
            else {
                salvarNegociacaoValores(wdgNegociacaoValores);
            }
        }
    };

    var salvarNegociacaoValores = function (wdgNegociacaoValores) {
        var deletarParcelas = (wdgNegociacaoValores.backupRow && wdgNegociacaoValores.backupRow.VRTOTITEMCOMDESC !== wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC);
        Utilities.widgetSaveAndSetColumnsToReturn(wdgNegociacaoValores).then(function () {
            var wdgContrato = templateManager.container.getWidget("gridContrato");
            if (wdgContrato.currentRow.IDGERADOPOR == "CONTRATO" && wdgContrato.currentRow.NRSEQORCAMENTO) {
                var params = {
                    "NEGOCIACAO": wdgNegociacaoValores.currentRow
                };
                General.callRestEngine("product#/fct_contrato_widget/atualizarNegociacaoProposta", "Row", params).then(function () {
                    if (deletarParcelas) {
                        params = { "NRNEGOCIACAOCONTRATO": wdgNegociacaoValores.currentRow.NRNEGOCIACAOCONTRATO };
                        General.callRestEngine("product#/fct_contrato_widget/excluirParcelasNegociacaoContrato", "Row", params)
                            .then().catch(function (error) {
                                Utilities.showErrorMessage(error);
                            });
                    }

                    this.atualizarValorRateioItens(wdgNegociacaoValores);
                    if (wdgNegociacaoValores.currentRow.IDTPFATURA == 'P') {
                        var mensagem = "Processo realizado com sucesso! <br><br>Atenção!<br>O valor total da soma das parcelas deve ser igual ao valor do item.";
                        ScreenService.showMessage(mensagem);
                    }
                }.bind(this))
                    .catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
            }
            else {
                var params = {
                    "NRNEGOCIACAOCONTRATO": wdgNegociacaoValores.currentRow.NRNEGOCIACAOCONTRATO,
                    "VRTOTITEMCOMDESC": wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC
                };
                General.callRestEngine("product#/fct_contrato_widget/atualizarValoresDescontoPorPeriodoDaNegociacao", "Row", params).then(function () {
                    this.atualizarValorRateioItens(wdgNegociacaoValores);
                }.bind(this))
                    .catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
            }
        }.bind(this));
    }.bind(this);

    this.cancelarSaveNegociacaoValores = function (widget) {
        setDataSourceTipoDesconto(widget, false);
        General.widgetCancelForm(widget);
    };

    /**
     * Valida se o contrato participa de um agrupamento para permitir a edição dos mesmo
     * @param{Object} widget - widgets de faturamento
     **/
    this.validarAlteracaoFaturamento = function (widget) {
        var params = [{
            name: "NRSEQCONTRATO",
            operator: "=",
            value: templateManager.container.getWidget("gridContrato").currentRow.NRSEQCONTRATO
        }];
        General.callRestEngine("product#/crud/fct_relacagrupamentocont_widget", "FilterData", params).then(function (response) {
            var wdgCometencia = templateManager.container.getWidget("wdgParamFaturamentoCompetCont");
            possuiAgrup = (response.dataset && response.dataset.fct_relacagrupamentocont_widget && response.dataset.fct_relacagrupamentocont_widget.length > 0);

            if (wdgCometencia.isVisible) {
                ["add", "edit", "delete"].forEach(function (action) {
                    wdgCometencia.getAction(action).isVisible = !possuiAgrup;
                });
            }

            widget.getField("NMGRUPOFATURA").readOnly = possuiAgrup;
            widget.getField("NMPRINCIPALPARC").readOnly = possuiAgrup;
            widget.getField("NMFILIAL").readOnly = possuiAgrup;
            ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoOnEnter(widget, 'wdgParamFaturamentoCompetCont');
        }.bind(this));
    };
    /**
     * Valida a composição de serviço antes de copiar o contrato
     * @params{Object} widget - widget de contrato
     **/
    this.validarCopiaContrato = function (widget, actionName) {
        if (widget.currentRow.IDUTILIZACOMP == "N") {
            actionName == 'copiarContrato' ? this.copiaVersionaContrato(widget, 'C') : this.validaSituacaoContEncReneg(widget, actionName);
        }
        else {
            var params = {
                "NRSEQORCAMENTO": widget.currentRow.NRSEQORCAMENTO,
                "ACAO": (actionName == 'copiarContrato') ? "copiar" : "renegociar"
            };

            General.callRestEngine("product#/fct_itemorcamento_widget/validarAtivacaoComposicao", "Row", params).then(function (response) {
                actionName == 'copiarContrato' ? this.copiaVersionaContrato(widget, 'C') : this.validaSituacaoContEncReneg(widget, actionName);
            }.bind(this))
                .catch(function (error) {
                    if (error.data && error.data.error.indexOf('POSSUI COMPOSICAO INATIVA') !== -1)
                        ScreenService.showMessage("Operação bloqueada!<br>Não é possível " + (actionName == 'copiarContrato' ? "copiar" : "renegociar") + " contratos ligados a composições de serviço inativas.<br>Ative-as e tente novamente!");
                    else
                        Utilities.showErrorMessage(error);
                });
        }
    };

    this.wdgItensNegociacaoAfterMoveRow = function (wdgItensNegociacao) {
        var wdgContrato = getWidgetGridContrato();

        var rowContrato = wdgContrato.currentRow;
        var campoParceiro = wdgItensNegociacao.getField("NMPRINCIPALPARC");
        var campoValorItem = wdgItensNegociacao.getField("VRTOTITEM");
        var campoPercDesconto = wdgItensNegociacao.getField("VRPERCDESCITEM");
        var campoValorDesconto = wdgItensNegociacao.getField("VRDESCITEM");
        var campoTipoDesconto = wdgItensNegociacao.getField("IDTPDESCONTO");
        var campoTipoMedida = wdgItensNegociacao.getField("DSTPMEDIDASERV");
        var situacao = rowContrato.IDSITUACAOCONTRATO;
        var visibilidade;

        var utilizaComposicao = wdgContrato.currentRow.IDUTILIZACOMP;

        /* Verificar na rotina de permitir alterar quando regerar a negociação */   
        wdgItensNegociacao.getAction("edit").isVisible = (wdgItensNegociacao.currentRow._parent == null ||
            /*(rowContrato.IDGERADOPOR == 'CONTRATO' || rowContrato.NRSEQCONTRATOPAI) &&*/
            (situacao == 'P' || (situacao == 'A' && visibilidadeAcoes && idAlteraContAprv == 'S')));

        var dadosFaturamento = this.pegarDadosDeFaturamentoGroup(wdgItensNegociacao)[0];

        if (wdgItensNegociacao.currentRow.NRITEMCONTRATOPAI == null && utilizaComposicao == "S") {
            dadosFaturamento.isVisible = true;
            campoValorItem.showOnView = false;
            campoValorItem.showOnForm = false;
            campoPercDesconto.showOnView = false;
            campoPercDesconto.showOnForm = false;
            campoValorDesconto.showOnView = false;
            campoValorDesconto.showOnForm = false;
            campoTipoDesconto.isVisible = false;
            campoTipoMedida.showOnView = false;
            campoTipoMedida.showOnForm = false;
        }
        else {
            campoValorItem.showOnView = true;
            campoValorItem.showOnForm = true;
            campoPercDesconto.showOnView = true;
            campoPercDesconto.showOnForm = true;
            campoValorDesconto.showOnView = true;
            campoValorDesconto.showOnForm = true;
            campoTipoDesconto.isVisible = true;
            campoTipoMedida.showOnView = true;
            campoTipoMedida.showOnForm = true;
        }

        dadosFaturamento.isVisible = (wdgItensNegociacao.currentRow._parent == null);
    };

    this.pegarDadosDeFaturamentoGroup = function (widget) {
        return widget.fieldGroups.filter(function (wid) {
            return (wid.name == "fdgProdutosFaturamento")
        });
    };

    /**
     * Alterar todos os códigos de fatura dos itens filhos em um contrato.
     * { Widget } @param widget objeto representando a widget
     * { string } @param codigo código que deve ser atualizado
    */

    this.alterarTodosOsCodigosDosItens = function (widget, codigo) {
        widget.dataSource.data.filter(function (w) {

            return w.NRITEMCONTRATOPAI == codigo
        }).forEach(function (row) {
            row.CDPRODFATURA = widget.currentRow.CDPRODFATURA;
            row.changed = true;
        })

    };

    this.salvarWdgItensNegociacao = function (wdgItensNegociacao) {
        if (!wdgItensNegociacao.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
        else {
            var deletarParcelas = (wdgItensNegociacao.backupRow && wdgItensNegociacao.backupRow.VRTOTITEMCOMDESC !== wdgItensNegociacao.currentRow.VRTOTITEMCOMDESC);
            var wdgContrato = templateManager.container.getWidget("gridContrato");
            var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");

            if (!wdgItensNegociacao.currentRow._parent) { //PREENCHIMENTO DO PAI

                var mudouParceiro = (wdgItensNegociacao.currentRow.NRPARCFATURA != wdgItensNegociacao.backupRow.NRPARCFATURA);
                var mudouProduto = (wdgItensNegociacao.currentRow.CDPRODFATURA != wdgItensNegociacao.backupRow.CDPRODFATURA);

                if (!mudouParceiro) {

                    if (mudouProduto)
                        this.alterarTodosOsCodigosDosItens(wdgItensNegociacao, wdgItensNegociacao.currentRow.NRITEMOCONTRATO);

                    General.widgetSave(wdgItensNegociacao).then(function () {


                        params = {
                            "NRNEGOCIACAOCONTRATO": wdgItensNegociacao.currentRow.NRNEGOCIACAOCONTRATO
                        };

                        General.callRestEngine("product#/fct_contrato_widget/atualizarValorNegociacaoComSomaDosItens", "Row", params)
                            .then(function (response) {
                                if (deletarParcelas) {
                                    General.callRestEngine("product#/fct_contrato_widget/excluirParcelasNegociacaoContrato", "Row", params)
                                        .then().catch(function (error) {
                                            Utilities.showErrorMessage(error);
                                        });
                                }


                                this.atualizarValorRateioItens(wdgItensNegociacao);
                                var valorNegociacao = response.dataset.retorno.valorNegociacao;
                                var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");

                                wdgNegociacaoValores.currentRow.VRTOTITEM = valorNegociacao;
                                wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC = valorNegociacao;
                                Utilities.calcularValoresDesconto(wdgNegociacaoValores, 'VRTOTITEM', 'VRPERCDESCITEM', 'VRDESCITEM', 'VRTOTITEMCOMDESC', 'VRPERCDESCITEM');
                            }.bind(this))
                            .catch(function (error) {
                                Utilities.showErrorMessage(error);
                            });
                    }.bind(this))
                        .catch(function (error) {
                            Utilities.showErrorMessage(error);
                        });
                }
                else {
                    var params = {
                        "NRNEGOCIACAOCONTRATO": wdgNegociacaoValores.currentRow.NRNEGOCIACAOCONTRATO
                    };

                    General.callRestEngine("product#/fct_contrato_widget/getQuantidadeItensNegociacaoContrato", "Row", params).then(function (response) {
                        var qtdeItens = response.dataset.retorno.quantidade;
                        if (qtdeItens == 1) {

                            if (mudouProduto)
                                this.alterarTodosOsCodigosDosItens(wdgItensNegociacao, wdgItensNegociacao.currentRow.NRITEMOCONTRATO);

                            General.widgetSave(wdgItensNegociacao).then(function () {


                                if (wdgContrato.currentRow.IDGERADOPOR == "CONTRATO" && !wdgContrato.currentRow.NRSEQCONTRATOPAI && wdgContrato.currentRow.NRSEQORCAMENTO) {
                                    if (deletarParcelas) {
                                        var params = { "NRNEGOCIACAOCONTRATO": wdgNegociacaoValores.currentRow.NRNEGOCIACAOCONTRATO };
                                        General.callRestEngine("product#/fct_contrato_widget/excluirParcelasNegociacaoContrato", "Row", params)
                                            .then().catch(function (error) {
                                                Utilities.showErrorMessage(error);
                                            }.bind(this));
                                    }

                                    this.atualizarInformacoesNegociacaoProposta(wdgItensNegociacao, wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA);
                                }
                            }.bind(this));
                        }
                        else {
                            ScreenService.confirmMessage("Atenção: <br/>Ao mudar o Parceiro de Negócio deste item, o item será movido deste grupo de Negociação de Valores para um novo grupo. Deseja continuar?", "question", function () {

                                if (mudouProduto)
                                    this.alterarTodosOsCodigosDosItens(wdgItensNegociacao, wdgItensNegociacao.currentRow.NRITEMOCONTRATO);

                                General.widgetSave(wdgItensNegociacao).then(function () {


                                    if (deletarParcelas) {
                                        var params = { "NRNEGOCIACAOCONTRATO": wdgNegociacaoValores.currentRow.NRNEGOCIACAOCONTRATO };
                                        General.callRestEngine("product#/fct_contrato_widget/excluirParcelasNegociacaoContrato", "Row", params)
                                            .then().catch(function (error) {
                                                Utilities.showErrorMessage(error);
                                            });
                                    }

                                    this.atualizarValorRateioItens(wdgItensNegociacao);
                                    var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
                                    params = {
                                        "ITEM": wdgItensNegociacao.currentRow,
                                        "NEGOCIACAO": wdgNegociacaoValores.currentRow,
                                        "NRSEQORCAMENTO": wdgContrato.currentRow.NRSEQORCAMENTO,
                                        "NRSEQCONTRATOPAI": wdgContrato.currentRow.NRSEQCONTRATOPAI,
                                        "IDGERADOPOR": wdgContrato.currentRow.IDGERADOPOR
                                    };
                                    General.callRestEngine("product#/fct_contrato_widget/desagruparItemNegociacao", "Row", params).then(function () {
                                        ScreenService.closeSwipe().then(function () {
                                            ScreenService.closeSwipe();
                                            wdgNegociacaoValores.activate();
                                            wdgNegociacaoValores.reload();
                                        });
                                    }.bind(this)).catch(function (error) {
                                        Utilities.showErrorMessage(error);
                                    });
                                }.bind(this));
                            });
                        }
                    }.bind(this)).catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            }
            else {
                var params = {
                    "NRNEGOCIACAOCONTRATO": wdgItensNegociacao.currentRow.NRNEGOCIACAOCONTRATO
                };

                General.callRestEngine("product#/fct_itemcontrato_widget/verificarSeExisteDescontoNegociacaoContrato", "Row", params)
                    .then(function (success) {
                        var percDescontoAtual = Utilities.tirarMascaraValores(wdgItensNegociacao.currentRow.VRPERCDESCITEM);

                        if (success.dataset.retorno.existe && percDescontoAtual > 0) {
                            ScreenService.showMessage("Operação bloqueada. <br/>Não é possível dar desconto diretamente no item pois já existe desconto para o grupo inteiro da negociação de valores.");
                        }
                        else {
                            var nritempai = wdgItensNegociacao.currentRow.NRITEMCONTRATOPAI;
                            var valorDescontoServico = 0;
                            var valorServico = 0;
                            wdgItensNegociacao.dataSource.data.filter(function (row) {
                                return row.NRITEMCONTRATOPAI = nritempai;
                            }).forEach(function (row) {
                                valorDescontoServico += Utilities.tirarMascaraValores(row.VRDESCITEM);
                                valorServico += Utilities.tirarMascaraValores(row.VRTOTITEM);
                            });

                            params = {
                                "CDSERVORCA": wdgItensNegociacao.currentRow.CDSERVORCA,
                                "IDTIPO": "D"
                            };

                            General.callRestEngine("product#/fct_itemcontrato_widget/validaPorcentualDescontoAcrescimoSave", "Row", params)
                                .then(function (response) {
                                    var resultado = response.dataset.RESULTADO.VRPERCDESC;
                                    var percDescontoServico = valorDescontoServico / valorServico * 100;
                                    if (resultado && percDescontoServico > resultado.VRPERCDESC) {
                                        var contexto = (resultado.TIPO == 1) ? "Este" : "O perfil de acesso deste";
                                        ScreenService.showMessage("Operação bloqueada.<br/>" + contexto + " operador não têm autorização para oferecer desconto superior a " + resultado.VRPERCDESC + "% para o serviço \"" + resultado.NMSERVORCA + "\".");
                                    }
                                    else {
                                        General.widgetSave(wdgItensNegociacao).then(function () {

                                            if (deletarParcelas) {
                                                var params = { "NRNEGOCIACAOCONTRATO": wdgItensNegociacao.currentRow.NRNEGOCIACAOCONTRATO };
                                                General.callRestEngine("product#/fct_contrato_widget/excluirParcelasNegociacaoContrato", "Row", params)
                                                    .then().catch(function (error) {
                                                        Utilities.showErrorMessage(error);
                                                    });
                                            }

                                            var mensagem = "Processo realizado com sucesso!" + ((wdgNegociacaoValores.currentRow.IDTPFATURA == 'P') ? "<br><br>Atenção!<br>O valor total da soma das parcelas deve ser igual ao valor do item." : "");
                                            ScreenService.showMessage(mensagem);

                                            if (wdgContrato.currentRow.IDGERADOPOR == "CONTRATO" && !wdgContrato.currentRow.NRSEQCONTRATOPAI && wdgContrato.currentRow.NRSEQORCAMENTO) {
                                                this.atualizarInformacoesNegociacaoProposta(wdgItensNegociacao, wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA);
                                            }
                                            else {
                                                params = {
                                                    "NRNEGOCIACAOCONTRATO": wdgItensNegociacao.currentRow.NRNEGOCIACAOCONTRATO,
                                                    "NRITEMCONTRATOPAI": wdgItensNegociacao.currentRow.NRITEMCONTRATOPAI
                                                };
                                                General.callRestEngine("product#/fct_contrato_widget/atualizarValoresNegociacaoAoEditarElemento", "Row", params).then(function (response) {
                                                    voltarParaAbaNegociacao(response.dataset.retorno.valorNegociacao, wdgItensNegociacao);
                                                }.bind(this)).catch(function (error) {
                                                    Utilities.showErrorMessage(error);
                                                });
                                            }
                                        }.bind(this)).catch(function (error) {
                                            Utilities.showErrorMessage(error);
                                        });
                                    }
                                }.bind(this)).catch(function (error) {
                                    Utilities.showErrorMessage(error);
                                });
                        }
                    }.bind(this)).catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
            }
        }
    };
    /**
     * Atualiza as informações da negociação de valores na proposta
     * Params{Object} wdgItensNegociacao - widget de itens da negociação 
     * Params{int} nrnegociacaoproposta  - código da negociação na proposta
     **/
    this.atualizarInformacoesNegociacaoProposta = function (wdgItensNegociacao, nrnegociacaoproposta) {
        var params = {
            "ITNEGOCIACAO": wdgItensNegociacao.currentRow,
            "NRNEGOCIACAOPROPOSTA": nrnegociacaoproposta
        };
        General.callRestEngine("product#/fct_contrato_widget/atualizarItemNegociacaoProposta", "Row", params).then(function (response) {
            voltarParaAbaNegociacao(response.dataset.retorno.valorNegociacao, wdgItensNegociacao);
        }).catch(function (error) {
            Utilities.showErrorMessage(error);
        });
    };

    var voltarParaAbaNegociacao = function (valorNegociacao, wdgItensNegociacao) {
        ScreenService.closeSwipe();
        wdgItensNegociacao.reload();
        var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
        wdgNegociacaoValores.currentRow.VRTOTITEM = valorNegociacao;
        wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC = valorNegociacao;
        Utilities.calcularValoresDesconto(wdgNegociacaoValores, 'VRTOTITEM', 'VRPERCDESCITEM', 'VRDESCITEM', 'VRTOTITEMCOMDESC', 'VRPERCDESCITEM');

    };

    /**
     * Abre o popup de paralisação do contrato
     * @params{Object} widget - widget de dados do contrato
     **/

    this.abrirWdgParalisacaoContrato = function (widget) {
        ScreenService.closePopup();
        var wdgPopup = templateManager.container.getWidget("wdgPopupParalisacaoCont");
        wdgPopup.isVisible = true;
        wdgPopup.dataSource.data = [];
        wdgPopup.currentRow = {};
        ScreenService.openPopup(wdgPopup).then(function () {
            wdgPopup.activate();
            wdgPopup.newRow();

            wdgPopup.currentRow.NRSEQCONTRATO = widget.currentRow.NRSEQCONTRATO;
            wdgPopup.currentRow.CDOPERSOLICITANTE = opLogado;
            wdgPopup.currentRow.NMOPERSOLICITANTE = nomeOperador;
        }.bind(this));
    };
    /**
     * Calcula a data final da paralisação do faturamento 
     * @params{Object} widget - popup de paralisação do contrato
     **/
    this.calcularDataFinalParalisacao = function (widget, nomeCampoAlterado) {
        var dtIniParalisacao = widget.currentRow.DTINIPARALISACAO;
        var qtDiasParalisacao = widget.currentRow.QTDIASPARALISACAO;
        var dtRetomadaFatura = widget.currentRow.DTRETOMADAFATURA;

        var dataInicial = null;
        var dataFinal = null;

        if (nomeCampoAlterado == "DTINIPARALISACAO" && dtIniParalisacao) {
            if (dtRetomadaFatura) {
                dataInicial = moment(dtIniParalisacao, "DD/MM/YYYY");
                dataFinal = moment(dtRetomadaFatura, "DD/MM/YYYY");
                widget.currentRow.QTDIASPARALISACAO = dataFinal.diff(dataInicial, 'days') + 1;
            }
            else if (qtDiasParalisacao) {
                dataFinal = moment(dtIniParalisacao, "DD/MM/YYYY").add(qtDiasParalisacao, 'days');
                widget.currentRow.DTRETOMADAFATURA = dataFinal.format('DD/MM/YYYY');
            }
        }
        else if (nomeCampoAlterado == "QTDIASPARALISACAO" && qtDiasParalisacao) {
            if (dtIniParalisacao) {
                dataFinal = moment(dtIniParalisacao, "DD/MM/YYYY").add(qtDiasParalisacao - 1, 'days');
                widget.currentRow.DTRETOMADAFATURA = dataFinal.format('DD/MM/YYYY');
            }
            else if (dtRetomadaFatura) {
                dataInicial = moment(dtRetomadaFatura, "DD/MM/YYYY").subtract(qtDiasParalisacao + 1, 'days');
                widget.currentRow.DTINIPARALISACAO = dataInicial.format('DD/MM/YYYY');
            }
        }
        else if (dtRetomadaFatura) { //nomeCampoAlterado = DTRETOMADAFATURA
            if (dtIniParalisacao) {
                dataInicial = moment(dtIniParalisacao, "DD/MM/YYYY");
                dataFinal = moment(dtRetomadaFatura, "DD/MM/YYYY");
                widget.currentRow.QTDIASPARALISACAO = dataFinal.diff(dataInicial, 'days') + 1;
            }
            else if (qtDiasParalisacao) {
                dataInicial = moment(dtRetomadaFatura, "DD/MM/YYYY").subtract(qtDiasParalisacao + 1, 'days');
                widget.currentRow.DTINIPARALISACAO = dataInicial.format('DD/MM/YYYY');
            }
        }

        dtIniParalisacao = moment(widget.currentRow.DTINIPARALISACAO, "DD/MM/YYYY");
        dtRetomadaFatura = moment(widget.currentRow.DTRETOMADAFATURA, "DD/MM/YYYY");
        var dtinivigencont = moment(getWidgetGridContrato().currentRow.DTINIVIGENCONT.split(" ")[0], "DD/MM/YYYY");
        var dtfinvigencont = moment(getWidgetGridContrato().currentRow.DTFINVIGENCONT.split(" ")[0], "DD/MM/YYYY");

        if (dtinivigencont.isAfter(dtIniParalisacao) || dtRetomadaFatura.isAfter(dtfinvigencont)) {
            ScreenService.showMessage("Operação bloqueada!<br>O período de paralisação deve estar dentro do período de vigência do contrato.<br>Período vigente: " + dtinivigencont.format("DD/MM/YYYY") + " a " + dtfinvigencont.format("DD/MM/YYYY") + ".");
        }
        else if (dtIniParalisacao.isAfter(dtRetomadaFatura)) {
            ScreenService.showMessage("Operação bloqueada!<br>A Data Final deve ser maior que a Data Inicial.");
        }
    };
    /**
     * Coloca a widget de itens em mode de edição
     * durante a inserção de um contrato com composição de serviço
     * @params{Object} widget - widget itens contrato inserindo
     **/
    this.controlarCamposItensContratoInserindo = function (widget, isEdit) {
        if (isEdit) {
            var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
            widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
            widget.edit();
            widget.__is_new = false;
            this.alterarVisibilidadeAcoesItensInserindo(widget, false);
        }

        var isVisible = widget.currentRow.IDTIPO === "M";
        var qtMdVisivel = (widget.currentRow.IDTIPO === "M" && widget.currentRow.IDMEDIDAFIXA === 'N' && widget.currentRow.IDMULTIPVARIACAO === "S");
        var isVisibleAgrup = (widget.currentRow.IDTIPO !== "A" && widget.currentRow.IDTIPO !== "M");

        widget.getField("NMDOCITEMORCAMENTO").isVisible = isVisibleAgrup;
        widget.getField("TXFORMULAITORCA").isVisible = isVisibleAgrup;
        widget.getField("QTMEDIDASERV").isVisible = isVisible;
        widget.getField("QTMEDIDASERV").readOnly = qtMdVisivel;

        this.validarAbreSwipeItens(widget);
        widget.getField("VRUNITITEMORCA").readOnly = (widget.currentRow.CDTPMEDVARESCOPO != null);
        widget.getField("VRUNITITEMORCA").help = (!widget.currentRow.CDTPMEDVARESCOPO) ? null : "O campo foi desabilitado porque não é possível inserir acréscimos em itens vinculados a serviços faturados por medição.";
    };
    /**
     * Altera as actions da tela de itens durante a edição dos mesmo
     * @params{Object} widget  - widget de itens inserindo
     * @params{Bool} isVisible - visibilidade das actions que aparecem no grid 
     **/
    this.alterarVisibilidadeAcoesItensInserindo = function (widget, isVisible) {
        widget.getAction("voltar").isVisible = isVisible;
        widget.getAction("inserir_tudo").isVisible = isVisible;
        widget.getAction("act_cancel").isVisible = !isVisible;
        widget.getAction("act_salvar").isVisible = !isVisible;
    };
    /**
     * Volta para o grid de itens durante a insersão de um contrato com 
     * composição de serviço
     * @params{Object} widget - widget de itens 
     **/
    this.voltarGridItensContratoInserindo = function (widget) {
        ScreenService.closeSwipe();
        widget.mode = "edit";
        widget.activate();
        widget.getField("NMTIPOORCAMENTO").isVisible = true;
        this.alterarVisibilidadeAcoesItensInserindo(widget, true);
    };
    /**
     * Salva as alterações realizadas nos itens durante a inserção 
     * de uma contrato de composição de serviço
     * @params{Object} widget - widget itens de composição inserindo
     **/
    this.salvarAlteracoesItensContratoInserindo = function (widget) {
        this.replicarVrTotalItemContratoInserindo(widget);
        this.alterarVisibilidadeAcoesItensInserindo(widget, true);
        widget.view();
    };
    /**
     * Replica o novo valor do item para seus itens pais na
     * arvore de itens durante a inserção da proposta
     * @params{Object} widget - widget de itens
     **/
    this.replicarVrTotalItemContratoInserindo = function (widget) {
        if (widget.currentRow.IDTIPO == "M") {
            this.calcularValorTotalItem(widget);
            widget.currentRow.VRUNITITEMORCA_GRID = Utilities.formatarNumero(widget.currentRow.VRUNITITEMORCA);
            widget.currentRow.QTMEDIDASERV_GRID = Utilities.formatarNumero(widget.currentRow.QTMEDIDASERV);

            var cdservorca = widget.currentRow.CDSERVORCA;
            var indexElSuperior = null;
            var totalServico = 0;
            widget.dataSource.data.forEach(function (item, index) {
                if (cdservorca == item.CDSERVORCA) {
                    if (!item.IDARVOREPAI)
                        indexElSuperior = index;
                    else
                        totalServico += item.VRTOTALITEMORCA;
                }
            });

            if (indexElSuperior !== null) {
                widget.dataSource.data[indexElSuperior].VRTOTALITEMORCA = totalServico;
                widget.dataSource.data[indexElSuperior].VRUNITITEMORCA = totalServico;
            }
        }
    };
    /**
     * Salva a paralisação do faturamento de um contrato
     * @param{Object} widget - widget de paralisação do faturamentof
     **/
    this.salvarParalisacaoContrato = function (widget) {
        if (!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
        else {
            var dtIniParalisacao = moment(widget.currentRow.DTINIPARALISACAO, "DD/MM/YYYY");
            var dtRetomadaFatura = moment(widget.currentRow.DTRETOMADAFATURA, "DD/MM/YYYY");
            var dtinivigencont = moment(getWidgetGridContrato().currentRow.DTINIVIGENCONT.split(" ")[0], "DD/MM/YYYY");
            var dtfinvigencont = moment(getWidgetGridContrato().currentRow.DTFINVIGENCONT.split(" ")[0], "DD/MM/YYYY");

            if (dtinivigencont.isAfter(dtIniParalisacao) || dtRetomadaFatura.isAfter(dtfinvigencont)) {
                ScreenService.showMessage("Operação bloqueada!<br>O período de paralisação deve estar dentro do período de vigência do contrato.<br>Período vigente: " + dtinivigencont.format("DD/MM/YYYY") + " a " + dtfinvigencont.format("DD/MM/YYYY") + ".");
            }
            else if (dtIniParalisacao.isAfter(dtRetomadaFatura)) {
                ScreenService.showMessage("Operação bloqueada!<br>A Data Final deve ser maior que a Data Inicial.");
            }
            else {
                widget.currentRow.DSMOTIVORETOMADA = null;
                this.criarParametrosNewCode(widget);
                widget.saveRow().then(function (response) {
                    if (response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0) {
                        widget.currentRow.NRPARAMPARALISAO = response.dataset.columnsToReturn[0].NRPARAMPARALISAO;
                        widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                    }

                    var params = {
                        "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                        "DTINIPARALISACAO": widget.currentRow.DTINIPARALISACAO,
                        "DTRETOMADAFATURA": widget.currentRow.DTRETOMADAFATURA
                    };

                    General.callRestEngine("product#/fct_contrato_widget/paralisarFaturamentoContrato", "Row", params).then(function () {
                        ScreenService.showMessage("Processo realizado com sucesso!");
                        templateManager.container.getWidget('wdgHistoricoParalisacao').isVisible = true;
                        this.cancelaPopup(widget, 'gridContrato');
                        this.gridContratoAfterMoveRow(getWidgetGridContrato());
                    }.bind(this))
                        .catch(function (error) {
                            if (error.data && error.data.error.indexOf("SEM_MEDICAO_PARCELA") !== -1) {
                                ScreenService.showMessage("Operação bloqueada! <br/><br/>Não há nenhuma medição ou parcela a ser paralisada para o período informado.");
                            }
                            else {
                                Utilities.showErrorMessage(error);
                            }

                            widget.removeRow();
                        });
                }.bind(this))
                    .catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
            }
        }
    };
    /**
     * Abre o popup de retomada do faturamento
     * @params{Object} widget - widget grid contrato
     **/
    this.abrirWdgRetomarFaturamentoContrato = function (widget) {
        ScreenService.closePopup();
        var wdgPopup = templateManager.container.getWidget("wdgPopupRetormarFatura");
        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function () {
            wdgPopup.activate();
            wdgPopup.newRow();
            wdgPopup.currentRow.NRSEQCONTRATO = widget.currentRow.NRSEQCONTRATO;
        }.bind(this));
    };
    /**
     * Retoma o faturmaneto do contrato
     * @params{Object} widget - widget de contrato 
     **/
    this.retormarFaturmanentoContrato = function (widget) {
        var params = {
            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
            "DSMOTIVORETOMADA": widget.currentRow.DSMOTIVORETOMADA,
            "NRPARAMPARALISAO": widget.currentRow.NRPARAMPARALISAO
        };

        General.callRestEngine("product#/fct_contrato_widget/retormarFaturamentoContrato", "Row", params).then(function (response) {
            ScreenService.showMessage("Processo realizado com sucesso!");
            this.cancelaPopup(widget, 'gridContrato');
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };

    this.editarCompetenciaNegociacao = function (wdgCompetenciaNegociacao) {
        General.widgetEdit(wdgCompetenciaNegociacao);
        ValidacoesParametrosFaturamentoUtil.controlarTipoCalculoVencimento(wdgCompetenciaNegociacao);
    };
    /**
     * Valida se o contrato faz parte de um agrupamento de contratos 
     * antes de habilitar a edição da negociação
     * @params{Object} widget - widgets da negociação
     * @params{String} actionName - nome da ação que está sendo executado
     **/
    this.validarInclusaoContratoNoAgrupamento = function (widget, actionName) {
        var params = [{
            name: "NRSEQCONTRATO",
            operator: "=",
            value: getWidgetGridContrato().currentRow.NRSEQCONTRATO
        }];

        General.callRestEngine("product#/crud/fct_relacagrupamentocont_widget", "FilterData", params).then(function (response) {
            var agrupamentos = (response.dataset && response.dataset.fct_relacagrupamentocont_widget && response.dataset.fct_relacagrupamentocont_widget.length > 0);
            var permitirEdicao = this.validarEdicaoParametrosAgrupamento(widget, agrupamentos);

            if (permitirEdicao) {
                if (widget.name == 'wdgNegociacaoValores' || widget.name == 'wdgItensNegociacao') {
                    this.editarDescontoNegociacao(widget);
                }
                else {
                    if (actionName == 'edit')
                        this.editarCompetenciaNegociacao(widget);
                    else if (actionName == 'delete')
                        this.deletarCompetenciaNegociacao(widget);
                    else
                        ValidacoesParametrosFaturamentoUtil.wdgParametrosFaturamentoNewRow(widget);
                }
            }
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };
    /**
     * valida se o valor informado para a quandidade de medida do item se encontra 
     * na faixa de preco estabelecida na tabela de variação
     **/
    this.validarVariacaoPreco = function (nrseqorcamento, cdservorca, qtmedidaorca, cdtpmedida, idSituacao, nrtipoorcamento) {
        var params = {
            "NRSEQORCAMENTO": nrseqorcamento,
            "CDSERVORCA": cdservorca,
            "QTMEDIDAORCA": qtmedidaorca,
            "CDTPMEDIDASERV": cdtpmedida,
            "IDSITUACAOORCAMENTO": idSituacao,
            "NRTIPOORCAMENTO": nrtipoorcamento
        };

        return General.callRestEngine("product#/fct_orcamento_widget/validarVariacaoPreco", "Row", params);
    };
    /**
     * Verifica qual widget está chamando a validação e capta os dados necessários para 
     * a validação do valor da quandidade de medida
     * @params {Object} widget - widget de itens ou de tipo de medida
     **/
    this.buscaDadosParaValidacaoQtMedida = function (widget) {
        var deferred = ZHPromise.defer();
        var isEdit = (widget.name == "wdgItensPropostaTree");
        var nameFieldQt = (isEdit) ? "QTMEDIDACONTR" : "QTMEDIDASERV";

        this.validarAlteracaoQtMedida(widget.currentRow, nameFieldQt).then(function () {
            if (!isEdit) {
                widget.currentRow.QTMEDIDACONTR = widget.currentRow.QTMEDIDASERV;
                widget.currentRow.CDTPMEDIDASERV_GRID = widget.currentRow.CDTPMEDIDAVARIAVEL;
            }

            General.callRestEngine("product#/fct_itemcontrato_widget/validarQuantidadeMedidaItens", "Row", widget.currentRow).then(function (response) {
                if (response.dataset && response.dataset.fct_itemcontrato_widget && response.dataset.fct_itemcontrato_widget.NEWVRUNITITEM) {
                    var newVrUnitItem = response.dataset.fct_itemcontrato_widget.VRPRECOVARIACAO;
                    var nameFieldVr = ((isEdit) ? "VRUNITITEMORCA" : "VRUNITITEMCONT");

                    if (widget.currentRow[nameFieldVr] !== newVrUnitItem)
                        widget.currentRow[nameFieldVr] = newVrUnitItem;
                }

                this.calcularValorTotalItem(widget);
                deferred.resolve();
            }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                    widget.currentRow[nameFieldQt] = widget.backupRow[nameFieldQt];
                    this.calcularValorTotalItem(widget);

                    deferred.reject();
                }.bind(this));
        }.bind(this))
            .catch(function (error) {
                widget.currentRow[nameFieldQt] = widget.backupRow[nameFieldQt];
                this.calcularValorTotalItem(widget);

                deferred.reject();
            }.bind(this));

        return deferred.promise;
    };
    /**
     * Valida se a quantidade de medida informada durante a inserção do contrato
     * é maior ou igual ao valor da franquia
     * @params{Object} row - row do inline edit
     **/
    this.validarQtMedidaContratoInserindo = function (rowTpMedida, widget, nameQtMedida) {
        if (rowTpMedida.QTFRANQUIA && rowTpMedida.QTFRANQUIA > parseFloat(rowTpMedida[nameQtMedida])) {
            ScreenService.showMessage("Operação bloqueada!<br>O valor da franquia não pode ser menor que o valor da quantidade.");
            widget.currentRow.QUANTIDADE_inline = rowTpMedida.QTFRANQUIA;
            return false;
        }
        return true;
    };
    /**
     * Abre a Widget de negociação de valores para a edição
     **/
    this.abrirWidgetNegociacaoValores = function (currentRow) {
        var gridContrato = getWidgetGridContrato();
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        var wdgNegociacao = templateManager.container.getWidget("wdgNegociacaoValores");

        setLocalVar("SEQ_CONTRATO", currentRow.NRSEQCONTATO);
        setLocalVar("NRORG_CONTRATO", currentRow.NRORG);

        wdgTabbedPrincipal.activate();
        gridContrato.list();
        gridContrato.reload().then(function () {
            gridContrato.view();
            ["NRCONTRATOFCT", "NMPRINCIPALPARC", "NMOPERRESPONSAVEL"].forEach(function (nameCampo) {
                if (gridContrato.currentRow[nameCampo])
                    gridContrato.descriptionOrder += ' - ' + gridContrato.currentRow[nameCampo];
            });

            gridContrato.currentRow = gridContrato.dataSource.data[0];
            this.gridContratoAfterMoveRow(gridContrato);
            this.controlarAbasContrato(true, gridContrato.name);
            ScreenService.openSwipe(wdgTabbedPrincipal).then(function () {
                wdgTabbedPrincipal.openTab("wdgTabbedContrato");
                wdgNegociacao.list();
                wdgNegociacao.reload().then(function () {
                    wdgNegociacao.activate();
                });
            });
        }.bind(this));
    };
    /**aqui
     * Valida o idGerador da proposta e chama a função 
     * respectiva de cada um
     **/
    this.validarChamadasFuncoesDeAtualizacaoItens = function (widget, idMantemDescAcres) {

        var rowContrato = getWidgetGridContrato().currentRow;
        if (rowContrato.IDGERADOPOR == 'CONTRATO' && !rowContrato.NRSEQCONTRATOPAI && rowContrato.NRSEQORCAMENTO)
            this.atualizarComposicao(widget, idMantemDescAcres);
        else
            this.atualizarItensContrato(widget, idMantemDescAcres);
        
    };
    /**
     * Retorna um array de elementos para a atualização dos itens 
     * do contrato
     * @params {Object} widget - widget de elementos (editando)
     **/
    this.getElementosParaAtualizacao = function (widget) {
        var elementos = [];
        var composicoes = [];
        var rowContrato = getWidgetGridContrato().currentRow;
        widget.dataSource.data.forEach(function (row) {
            if (row.__isSelected) {
                if (row.TIPO == "E") {
                    elementos.push({
                        "NRSEQCONTRATO": rowContrato.NRSEQCONTRATO,
                        "NRCOMPSERVICO": row.NRCOMPSERVICO,
                        "NRITCOMPSERVICO": row.NRITCOMPSERVICO,
                        "CDSERVORCA": row.CDSERVORCA,
                        "IDELEMENTO": row.IDELEMENTO,
                        "IDELEMENTOPAI": this.getElementoPaiServico(row),
                        "IDESCOLHIDO": (row.__isSelected) ? "S" : "N",
                        "NRORG": row.NRORG,
                        "IDAPARECEPROPOSTA": row.IDAPARECEPROPOSTA,
                        "CDCENTCUST": row.CDCENTCUST
                    });
                }
                else {
                    composicoes.push({
                        "NRSEQCONTRATO": rowContrato.NRSEQCONTRATO,
                        "NRCOMPSERVICO": row.CDELEMENTO,
                        "NMCOMPSERVICO": row.NMELEMENTO,
                        "NRCOMPSERVICOPAI": row.IDARVOREPAI,
                        "IDCOMPOBRIGATORIA": row.IDSERVOBRIGCOMP,
                        "IDESCOLHIDO": (row.__isSelected) ? "S" : "N",
                        "NRORG": row.NRORG,
                        "IDAPARECEPROPOSTA": row.IDAPARECEPROPOSTA,
                        "CDCENTCUST": row.CDCENTCUST
                    });
                }
            }
        }.bind(this));
        var arrays = {
            "ELEMENTOS": elementos,
            "COMPOSICOES": composicoes
        };

        return arrays;
    };
    /**
     * Chama a rota de atualização dos itens de um contrato quando há alterações
     * no tipo de medida ou nos elementos
     * @params {Object} widget - widget de elementos
     **/
    this.atualizarItensContrato = function (widget, mantemDescAcres) {
        var compElementos = this.getElementosParaAtualizacao(widget);
        var elementos = compElementos["ELEMENTOS"];
        var composicoes = compElementos["COMPOSICOES"];
        var params = {
            "CONTRATO": getWidgetGridContrato().currentRow,
            "ELEMENTOS": elementos,
            "COMPOSICOES": composicoes,
            "TPTRATAMENTO": mantemDescAcres
        };
        var timeout = 5 * 60000; //5 min
        Utilities.callRestEngineWithTimeout("product#/fct_contrato_widget/atualizarItensContrato", "Row", params, timeout).then(function (response) {
            if (response.messages[0].message != null)
                ScreenService.showMessage(response.messages[0].message);
            this.cancelarElementosContrato(widget);
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };
    /**
     * Efetua as validações necessárias ao entrar na tela de tipo de medida 
     * @params {Object} widget - widget tipo de medida variável (editando)
     **/
    this.validarChamadasFuncoesSaveTpMedida = function (widget, idMantemDescAcres) {
        var rowContrato = getWidgetGridContrato().currentRow;
        if (rowContrato.IDGERADOPOR == 'CONTRATO' && !rowContrato.NRSEQCONTRATOPAI && rowContrato.NRSEQORCAMENTO)
            this.atualizarTipoDeMedidaOrcaCont(widget, idMantemDescAcres);
        else
            this.atualizarTipoDeMedidaContrato(widget, idMantemDescAcres);
    };
    /**
     * Atualiza o tipo de medida variável ligado apenas ao contrato
     * @params{Object} widget - widget tipo de medida variavel (editando)
     **/
    this.atualizarTipoDeMedidaContrato = function (widget, idMantemDescAcres) {
        widget.saveRow().then(function () {
            var params = {
                "CONTRATO": getWidgetGridContrato().currentRow,
                "TPTRATAMENTO": idMantemDescAcres
            };
            var timeout = 5 * 60000; //5 min
            Utilities.callRestEngineWithTimeout("product#/fct_contrato_widget/atualizarItensContratoPeloTiposDeMedida", "Row", params, timeout).then(function (response) {
                widget.list();
                widget.reload();
            }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };
    /**
     * Controla a propriedade inlineEdit nas widgets
     * @params{Object} widget - widget que será alterada
     **/
    this.controlarVisibilidadeInlineEdit = function () {
        var wdgTipoMedida = templateManager.container.getWidget("wdgTipoMedidaPropostaEditando");
        //var wdgNomeContratual = templateManager.container.getWidget("wdgNomeContratual");

        var fieldsInline = [];
        var widgets = [wdgTipoMedida, /*wdgNomeContratual*/];

        widgets.forEach(function (widget) {
            fieldsInline = fieldsInline.concat(widget.fields.filter(function (field) {
                return field.editavelGrid == true;
            }));
        });

        var situacao = getWidgetGridContrato().currentRow.IDSITUACAOCONTRATO;
        var nrseqcontratopai = getWidgetGridContrato().currentRow.NRSEQCONTRATOPAI;
        var idgeradopor = getWidgetGridContrato().currentRow.IDGERADOPOR;

        if ((nrseqcontratopai || idgeradopor == 'CONTRATO') && (situacao == "P" || (situacao == 'A' && idAlteraContAprv == 'S' && visibilidadeAcoes))) {
            fieldsInline.forEach(function (field) {
                field.inlineEditable = (field.name == 'NMCONTRATUALITEMCONT') ? "text" : "number";
            });
        }
        else {
            fieldsInline.forEach(function (field) {
                delete field.inlineEditable;
            });
        }
    };
    /**
     * Exclui uma parametrização de desconto por período do contrato
     * @params{Object} widget - widget desconto por periodo
     **/
    this.deletarDescontoPorPeriodo = function (widget) {
        var rowContrato = getWidgetGridContrato().currentRow;
        if (rowContrato.IDGERADOPOR != 'CONTRATO' || rowContrato.NRVERSAOCONTRATO < 1 || !rowContrato.NRSEQORCAMENTO) {
            General.deleteCrud(widget);
        }
        else {
            ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function () {
                General.callRestEngine("product#/fct_contrato_widget/deletarDescontoPorPeriodo", "Row", widget.currentRow).then(function () {
                    ScreenService.closeSwipe();
                    widget.list();
                    widget.reload();
                });
            });
        }
    };
    /**
     * Salva os dados da competência na negociação de valores
     * @params{Object} widget - widget competência
     **/
    this.salvarCompetenciaNegociacao = function (widget) {
        var rowContrato = getWidgetGridContrato().currentRow;
        if (!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
        else {
            var nomeTabela = widget.paramsNewCode.__new_code_pk;
            var nomeCampoPK = widget.paramsNewCode.__table_pk;
            var is_new = widget.currentRow.__is_new;

            widget.currentRow.IDREPLICARORCA = (rowContrato.IDGERADOPOR == 'CONTRATO' && rowContrato.NRVERSAOCONTRATO == 1 && rowContrato.NRSEQORCAMENTO != null);
            widget.currentRow.NRNEGOCIACAOPROPOSTA = templateManager.container.getWidget("wdgNegociacaoValores").currentRow.NRNEGOCIACAOPROPOSTA;

            ValidacoesParametrosFaturamentoUtil.validarSePeriodoDeCompetenciaExiste(widget, nomeTabela, nomeCampoPK, "NRNEGOCIACAOCONTRATO").then(function (existe) {
                if (existe !== false) {
                    Utilities.showErrorMessage("Operação bloqueada. <br/>Já existe um registro para o período de competência informado.");
                }
                else {
                    General.callRestEngine("product#/fct_contrato_widget/salvarCompetenciaNegociacao", "Row", widget.currentRow).then(function (response) {
                        if (!is_new) {
                            widget.view();
                        }
                        else {
                            ScreenService.closeSwipe();
                            widget.list();
                            widget.reload();
                        }
                    }).catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            }).catch(function (error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Deleta um registro de competencia do contrato
     * @params{Object} widget - widget competencia
     **/
    this.deletarCompetenciaNegociacao = function (widget) {
        var rowContrato = getWidgetGridContrato().currentRow;
        if (rowContrato.IDGERADOPOR != 'CONTRATO' || rowContrato.NRVERSAOCONTRATO > 1 || !rowContrato.NRSEQORCAMENTO) {
            General.deleteCrud(widget);
        }
        else {
            General.callRestEngine("product#/fct_contrato_widget/deletarCompetenciaNegociacao", "Row", widget.currentRow).then(function (response) {
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
            }).catch(function (error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Deleta as parcelas da negociação de valores
     * @params{Object} widget - widget parcelas
     **/
    this.deletarParcelasNegociacao = function (widget) {
        var rowContrato = getWidgetGridContrato().currentRow;
        if (rowContrato.IDGERADOPOR != "CONTRATO" || rowContrato.NRVERSAOCONTRATO > 1 || !rowContrato.NRSEQORCAMENTO) {
            General.deleteCrud(widget);
        }
        else {
            General.callRestEngine("product#/fct_contrato_widget/deletarParcelasNegociacao", "Row", widget.currentRow).then(function () {
                var params = {
                    "NRNEGOCIACAOCONTRATO": widget.currentRow.NRNEGOCIACAOCONTRATO
                };

                General.callRestEngine("product#/fct_contrato_widget/atualizarNrParcelas", "Row", params).then(function () {
                    ScreenService.closeSwipe();
                    widget.list();
                    widget.reload();
                }).catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
            }).catch(function (error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * valida a quantidade máxima de parcelas da negociação 
     * @params{Object} widget - widget de parcelas
     * @params{String} typeValidacao - tipo de parcela (PARCELAUNICA ou PARCELAPORIGUAL)
     **/
    this.validarQuantidadeMaxParcelas = function (widget, typeValidacao) {
        var deferred = ZHPromise.defer();
        var rowNegociacao = templateManager.container.getWidget("wdgNegociacaoValores").currentRow;
        var params = {
            "NRNEGOCIACAOCONTRATO": rowNegociacao.NRNEGOCIACAOCONTRATO,
            "NRTIPOORCAMENTO": rowNegociacao.NRTIPOORCAMENTO,
            "IDTPVALIDACAO": typeValidacao
        };

        General.callRestEngine("product#/fct_parcelanegocont/validarQuantidadeMaxParcelas", "Row", params).then(function (response) {
            if (response.dataset && response.dataset.fct_contrato_widget) {
                var maximoparcelas = response.dataset.fct_contrato_widget.NRMAXPARCELAS;

                if (typeValidacao == "PARCELAPORIGUAL") {
                    if (widget.currentRow.QTPARCELAS > maximoparcelas) {
                        widget.currentRow.QTPARCELAS = "";
                        deferred.reject("Operação bloqueada!<br>A quantidade de parcelas informada excede a quantidade máxima de parcelas da negociação.<br>Valor máximo permitido: " + maximoparcelas);
                    }
                    else {
                        deferred.resolve();
                    }
                }
                else if (maximoparcelas.indexOf("LIMITE_PARC_INSERIDO") != -1) {
                    deferred.reject("Operação bloqueada!<br>Quantidade máxima de parcelas já cadastrada no sistema.<br>Valor máximo permitido: " + maximoparcelas.split(" ")[1]);
                }
                else {
                    deferred.resolve();
                }

            }
        }.bind(this))
            .catch(function (error) {
                deferred.reject(error);
            });

        return deferred.promise;
    };

    this.novaParcela = function (wdgParcelas) {
        this.validarQuantidadeMaxParcelas(wdgParcelas, 'PARCELAUNICA').then(function () {
            General.widgetNewRow(wdgParcelas);
        })
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };
    /**
     * Faz a validação dos rateios de itens de formação de preço
     * @params{Object} widget - widget de itens
     * @params {bool} soma - se os itens de formção serão somados 
     **/
    this.validarDadosRateioItensFormacao = function (widget, soma) {
        var deferred = ZHPromise.defer();
        var nrformprecoservico = widget.currentRow.NRFORMPRECOSERVICO[0];
        var nrcalcformprecserv = widget.currentRow.NRCALCFORMPRECSERV[0];

        this.insereItemFormPrec(widget, soma).then(function (response) {

            deferred.resolve(response);
            /*var nritemocontrato = response[0].NRITEMOCONTRATO;
            
            var wdgContrato = templateManager.container.getWidget("gridContrato");
            var params = {
                "NRFORMPRECOSERVICO": nrformprecoservico,
                "NRSEQCONTRATO": wdgContrato.currentRow.NRSEQCONTRATO,
                "NRITEMOCONTRATO": nritemocontrato,
                "NRCALCFORMPRECSERV": nrcalcformprecserv
            };

            General.callRestEngine("product#/fct_itemcontrato_widget/insereAlteracaoFormPrecCont", "Row", params).then(function() {
                deferred.resolve(response);
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
                deferred.reject();
            });*/
        });
        /*this.validarDadosRateioContrato(widget).then(function(rateiosInseridos) {
            if (!rateiosInseridos) {
                this.insereItemFormPrec(widget, soma).then(function(resolve) {
                    deferred.resolve(resolve);
                });
            }
            else {
                var mensagem = (rateiosInseridos == "SEM_CENTROCUST") ?
                    "Atenção!<br>Os locais de atendimento das formações de preço selecionadas, não está ligados a um centro de custo. Desse modo, não serão gerados rateios para os itens desse contrato.<br>Deseja continuar?" :
                    "Atenção!<br><br>O item selecionado não possui local de atendimento ligado a ele. Sua inserção acarretará na exclusão dos rateios criados para esse contrato.<br><br>Deseja continuar?";

                ScreenService.confirmMessage(mensagem, "question", function() {
                    this.insereItemFormPrec(widget, soma).then(function(resolve) {
                        deferred.resolve(resolve);
                    });
                }.bind(this));
            }
        }.bind(this));*/
        return deferred.promise;
    };
    /**
     * valida se o contrato possui rateio inserido e se haverá alterações nele
     * @params{Object} widget - widget de itens
     **/
    this.validarDadosRateioContrato = function (widget, dsTipoRateio) {
        if (dsTipoRateio === undefined) {
            dsTipoRateio = "automatico";
        }

        var deferred = ZHPromise.defer();
        var params = {
            "NRSEQCONTRATO": getWidgetGridContrato().currentRow.NRSEQCONTRATO,
            "NRTIPOORCAMENTO": getWidgetGridContrato().currentRow.NRTIPOORCAMENTO,
            "NRESTRUTURAM": widget.currentRow.NRESTRUTURAM
        };

        General.callRestEngine("product#/fct_relacestruturacont_widget/validarDadosRateioContrato", "Row", params).then(function (response) {
            var rateiosInseridos = false;
            if (response.dataset && response.dataset.fct_relacestruturacont_widget && response.dataset.fct_relacestruturacont_widget.length > 0)
                rateiosInseridos = response.dataset.fct_relacestruturacont_widget[0];

            return deferred.resolve(rateiosInseridos);
        }.bind(this))
            .catch(function (error) {
                ScreenService.showMessage("Falha ao validar os rateios do contrato");
                return deferred.resolve();
            });

        return deferred.promise;
    };

    /**
     * Atualiza a tabela de rateio quando há alteração no valor dos itens da negociação
     * @params{Object} widget - widget itens da negociacao
     **/
    this.atualizarValorRateioItens = function (widget) {
        var idutilizacomp = getWidgetGridContrato().currentRow.IDUTILIZACOMP;
        if (!idutilizacomp || idutilizacomp == 'N') {
            var $nritemocontrato = widget.currentRow.NRITEMOCONTRATO ? widget.currentRow.NRITEMOCONTRATO : null;
            var params = {
                "NRITEMOCONTRATO": $nritemocontrato,
                "NRNEGOCIACAOCONTRATO": widget.currentRow.NRNEGOCIACAOCONTRATO,
                "NRSEQCONTRATO": getWidgetGridContrato().currentRow.NRSEQCONTRATO
            };

            General.callRestEngine("product#/fct_itemcontrato_widget/atualizarValorRateiosContrato", "Row", params).then()
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
        }
    };

    this.servicoOnChange = function (widget) {
        this.getValorUnitarioTblPrecoItem(widget);
        UtilitiesPropostaContrato.setarMedidaVariavelEscopo(widget, "C");
        this.getServicoIntegraHCM(widget);
    };

    /**
     * Busca o valor do serviço na tabela de preço do escopo
     * @param {Object} widget - widget de itens (editando)
     */
    this.getValorUnitarioTblPrecoItem = function (widget) {
        widget.currentRow.VRUNITITEMORCA = "";
        if (!widget.currentRow.CDPRODSERV) {
            this.calculaValorTotItem(widget);
        }
        else {
            var wdgContrato = getWidgetGridContrato();
            var params = {
                "NRTIPOORCAMENTO": wdgContrato.currentRow.NRTIPOORCAMENTO_CONT,
                "CDSERVORCA": widget.currentRow.CDPRODSERV,
                "QTMEDIDASERV": widget.currentRow.QTMEDIDACONTR ? widget.currentRow.QTMEDIDACONTR : null,
                "CDTPMEDIDAVARIAVEL": widget.currentRow.CDTPMEDIDAVARIAVEL ? widget.currentRow.CDTPMEDIDAVARIAVEL : null,
                "NRMOEDA": wdgContrato.currentRow.NRMOEDA
            };

            General.callRestEngine("product#/fct_itemorcamento_widget/getValorUnitarioTblPrecoItemProposta", "Row", params).then(function (response) {
                var quantidade = widget.currentRow.QTITEMCONTRATO ? widget.currentRow.QTITEMCONTRATO : 1;
                var qtdMedida = widget.currentRow.QTMEDIDACONTR ? widget.currentRow.QTMEDIDACONTR : 1;
                if (!response.dataset || !response.dataset.fct_itemorcamento_widget) {
                    this.calculaValorTotItem(widget);
                }
                else {
                    if (widget.currentRow.VRUNITITEMCONT == "") {
                        widget.currentRow.VRUNITITEMCONT = response.dataset.fct_itemorcamento_widget.VRPRECO;
                    }
                    widget.currentRow.VRTOTITEMCONT = widget.currentRow.VRUNITITEMCONT * quantidade * qtdMedida;
                }
            }.bind(this));
        }
    };

    /**
     * Valida o tipo de tratamento que será dado aos acréscimos e descontos 
     * após a alteração dos elementos e medidas
     **/
    this.validarAcrescimosDescontosContrato = function (widget, tpAtualizacao) {
        var wdgContrato = getWidgetGridContrato();
        if (wdgContrato.currentRow.NRVERSAOCONTRATO > 1 && wdgContrato.currentRow.NRSEQCONTRATOPAI) {
            ScreenService.showCustomDialog("Atenção! <br/><br/>A alteração que está fazendo pode resultar na alteração do valor do contrato. <br/>Deseja manter o valor unitário dos itens e os descontos que já estão definidos ou recalcular os valores de acordo com a tabela de preço vigente?", null, "question", [{
                "label": "Manter valores",
                "code": function () {
                    if (tpAtualizacao == 'EXC_ELEM'){
                        //this.validarChamadasFuncoesDeAtualizacaoItens(widget, 'S');
                        this.excluirElementoContr(widget, 'S');
                    }
                    else {
                        this.confirmInsElementoContr(widget, 'S');
                    }
                    
                }.bind(this)
            },
            {
                "label": "Recalcular",
                "code": function () {
                    if (tpAtualizacao == 'EXC_ELEM'){
                     //   this.validarChamadasFuncoesDeAtualizacaoItens(widget, 'N');
                        this.excluirElementoContr(widget, 'N');
                    }
                    else {
                        this.confirmInsElementoContr(widget, 'N');
                    }
                }.bind(this)
            },
            {
                "label": "Cancelar",
                "code": function () { }
            }
            ]);
        }
        else {
            var params = {
                "NRSEQCONTRATO": wdgContrato.currentRow.NRSEQCONTRATO
            };

            General.callRestEngine("product#/fct_contrato_widget/validarExistenciaDescontoAcrescimoContrato", "Row", params).then(function (response) {
                if (!response.dataset || !response.dataset.fct_contrato_widget || !response.dataset.fct_contrato_widget[0]) {
                    if (tpAtualizacao == 'EXC_ELEM'){
                       // this.validarChamadasFuncoesDeAtualizacaoItens(widget, 'N');
                        this.excluirElementoContr(widget, 'N');
                    }
                    else {
                        this.confirmInsElementoContr(widget, 'N');
                    }
                    
                }
                else {
                    ScreenService.showCustomDialog("A atualização que deseja fazer resultará na redefinição dos valores dos itens e das negociações." +
                        "<br><br>O que deseja fazer com os acréscimos e descontos deste contrato?", 'CONFIRMATION', null, [{
                            "label": "Descartá-los",
                            "code": function () {
                                if (tpAtualizacao == 'EXC_ELEM'){
                                 //   this.validarChamadasFuncoesDeAtualizacaoItens(widget, 'N');
                                    this.excluirElementoContr(widget, 'N');
                                }
                                else {
                                    this.confirmInsElementoContr(widget, 'N');
                                }
                                
                            }.bind(this)
                        },
                        {
                            "label": "Mantê-los",
                            "code": function () {
                                if (tpAtualizacao == 'EXC_ELEM'){
                                 //   this.validarChamadasFuncoesDeAtualizacaoItens(widget, 'S');
                                    this.excluirElementoContr(widget, 'S');
                                }
                                else {
                                    this.confirmInsElementoContr(widget, 'S');
                                }
                                
                            }.bind(this)
                        },
                        {
                            "label": "Cancelar",
                            "code": function () { }
                        }
                    ]
                    );
                }
            }.bind(this))
                .catch(function (error) {
                    if (tpAtualizacao == 'EXC_ELEM'){
                        //this.validarChamadasFuncoesDeAtualizacaoItens(widget, 'N');
                        this.excluirElementoContr(widget, 'N');
                    }
                    else {
                        this.confirmInsElementoContr(widget, 'N');
                    }
                });
        }
    };

    /**
     * Coloca a widget de desconto por período em modo de edição
     * @params{Object} widget - widget de desconto por periodo
     **/
    this.editarDescontoPorPeriodo = function (widget) {
        this.controlarCamposDesconto(widget);
        General.widgetEdit(widget);
    };

    /**
     * habilita o campo de desconto de acordo com o tipo selecionado
     * @params{Object} widget
     **/
    this.controlarCamposDesconto = function (widget) {
        var wdgContrato = getWidgetGridContrato();
        var situacao = wdgContrato.currentRow.IDSITUACAOCONTRATO;
        var idGeradoPor = wdgContrato.currentRow.IDGERADOPOR;
       /*  Verificar na rotina de permitir alterar quando regerar a negociação
       
          var readOnly = false;

       /* if (idGeradoPor == "PROPOSTA")
            readOnly = (situacao != 'P' || !wdgContrato.currentRow.NRSEQCONTRATOPAI);
        else */
            readOnly = (situacao != "A") ? false : (idAlteraContAprv != 'S');
        

        

        var tpDesconto = widget.currentRow.IDTPDESCONTO;
        if (!tpDesconto) {
            if (getWidgetNegociacaoDeValores().currentRow.IDTPFATURA != 'X') {
                widget.getField("VRDESCITEM").readOnly = readOnly;
                widget.getField("VRPERCDESCITEM").readOnly = readOnly;
            }
            else if (widget.getField("IDTPDESCONTO").isVisible) {
                widget.getField("VRDESCITEM").readOnly = true;
                widget.getField("VRPERCDESCITEM").readOnly = true;

                widget.getField("VRDESCITEM").value(0);
                widget.getField("VRPERCDESCITEM").value(0);
                Utilities.calcularValoresDesconto(widget, "VRTOTITEM", "VRPERCDESCITEM", "VRDESCITEM", "VRTOTITEMCOMDESC", "VRPERCDESCITEM");
            }

            widget.getField("VRDESCITEM").validations = {};
            widget.getField("VRPERCDESCITEM").validations = {};

        }
        else {
            widget.getField("VRDESCITEM").readOnly = readOnly;
            widget.getField("VRPERCDESCITEM").readOnly = readOnly;

            widget.getField("VRDESCITEM").validations = readOnly ? {} : { "required": true };
            widget.getField("VRPERCDESCITEM").validations = readOnly ? {} : { "required": true };
        }

        templateManager.updateTemplate();
    };

    /**
     * Coloca a widget de negociacao em modo de edição
     * @params{Object} widget
     **/
    this.editarDescontoNegociacao = function (widget) {
        this.controlarCamposDesconto(widget);
        var wdgNegociacaoValores = getWidgetNegociacaoDeValores();
        var fieldTipoDesconto = widget.getField("IDTPDESCONTO");

        if (wdgNegociacaoValores.currentRow.IDTPFATURA != 'X') {
            fieldTipoDesconto.readOnly = true;
            General.widgetEdit(widget);
        }
        else {
            var params = { "NRNEGOCIACAOCONTRATO": wdgNegociacaoValores.currentRow.NRNEGOCIACAOCONTRATO };
            var rota = (widget.name == 'wdgNegociacaoValores') ? "verificarSeExisteDescontoItensNegociacaoContrato" : "verificarSeExisteDescontoNegociacaoContrato";

            General.callRestEngine("product#/fct_itemcontrato_widget/" + rota, "Row", params).then(function (success) {
                var existeOutrosDescontos = success.dataset.retorno.existe;

                fieldTipoDesconto.readOnly = existeOutrosDescontos;
                return self.validarMedicaoServico(widget.currentRow.NRNEGOCIACAOCONTRATO, wdgNegociacaoValores.currentRow.NRTIPOORCAMENTO);
            }.bind(this))
                .then(function (medicaoDeServico) {
                    if (medicaoDeServico)
                        setDataSourceTipoDesconto(widget, true);

                    General.widgetEdit(widget);
                })
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
        }
    };

    this.validarMedicaoServico = function (nrnegociacaocontrato, nrtipoorcamento) {
        var deferred = ZHPromise.defer();
        var params = {
            "NRNEGOCIACAOCONTRATO": nrnegociacaocontrato,
            "NRTIPOORCAMENTO": nrtipoorcamento
        };
        General.callRestEngine("product#/fct_contrato_widget/validarMedicaoServico", "Row", params).then(function (response) {
            var medicaoDeServico = response.dataset.retorno.medicaoServico;
            deferred.resolve(medicaoDeServico);
        })
            .catch(function (error) {
                deferred.reject(error);
            });

        return deferred.promise;
    };

    this.completaPercentual = function (widget) {
        General.widgetNewRow(widget);
        widget.currentRow.NRSEQCONTRATO = getWidgetGridContrato().currentRow.NRSEQCONTRATO;

        var params = { "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO };
        General.callRestEngine('product#/fct_contrato_widget/completaPercentual', "Row", params).then(function (response) {
            var percentual = response.dataset.completaPercentual.PERCENTUAL;
            widget.currentRow.NRPERCENTINDICE = percentual;
        });
    };

    /**
     * Valida o percentual dos índices financeiros
     * 
     * @params{Object} widget - widget de índices financeiros
     **/
    this.validaSaveIndFinanceiro = function (widget) {
        if (!widget.currentRow.__is_new) {
            this.salvarIndiceFinanceiro(widget);
        }
        else {
            Utilities.validarCamposDuplicados(widget, null, ["CDINDICE", "NRSEQCONTRATO"]).then(function (response) {
                if (!response)
                    ScreenService.showMessage("Operação bloqueada!<br><br>O índice selecionado já está inserido.");
                else
                    this.salvarIndiceFinanceiro(widget);
            }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
        }
    };

    /**
     * Valida o salvamento de um indice
     * 
     * @params{Object} widget - widget de indices
     **/
    this.salvarIndiceFinanceiro = function (widget) {
        widget.currentRow.NRSEQCONTATO = getWidgetGridContrato().currentRow.NRSEQCONTRATO;

        var params = {
            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
            "NRPERCENTINDICE": widget.currentRow.NRPERCENTINDICE,
            "CDINDICE": (widget.currentRow.__is_new) ? null : widget.currentRow.CDINDICE
        };

        General.callRestEngine('product#/fct_contrato_widget/validaPercentual', "Row", params).then(function (response) {
            var percentual = response.dataset.validaPercentual.PERCENTUAL;
            if (percentual > 100) {
                ScreenService.showMessage("Operação Bloqueada!<br><br>A soma dos percentuais de utilização dos índices financeiros ultrapassou o total de 100%.");
            }
            else {
                General.widgetSave(widget).then(function (response) {
                    widget.reload();

                    if (percentual < 100)
                        ScreenService.showMessage("Atenção!<br><br>A soma dos percentuais de utilização dos índices financeiros não está totalizando os 100% necessários para realização do reajuste dos contratos.");
                });
            }

        }.bind(this));

    };

    /**
     * Valida se o contrato faz parte de um agrupamento e
     * controla a visibilidade dos parametros
     * @params{Object} widget    - widget que solicitou a edição 
     * @params{bool} agrupamento - define se o contrato está ligado há agrupamentos  
     **/
    this.validarEdicaoParametrosAgrupamento = function (widget, agrupamentos) {
        if (widget.name == 'wdgCompetenciaNegociacao') {
            if (agrupamentos) {
                ScreenService.showMessage("Operação bloqueada!<br>Este contrato está ligado a um agrupamento de contratos para faturamento. Separe-o e tente novamente.");
                return false;
            }
        }
        else {
            var fieldFilial = widget.getField("NMFILIALFATURA");
            var fieldSerie = widget.getField("DSSERIE");
            var fieldTipooper = widget.getField("NMTIPOOPER");
            var fieldParceiro = widget.getField("NMPRINCIPALPARC");

            if (fieldFilial) fieldFilial.readOnly = agrupamentos;
            if (fieldSerie) fieldSerie.readOnly = agrupamentos;
            if (fieldTipooper) fieldTipooper.readOnly = agrupamentos;
            if (fieldParceiro) fieldParceiro.readOnly = agrupamentos;
        }

        return true;
    };

    /**
     * A função "insereIndicesPadroes" insere os indices padrões que já estiverem preenchidos para o Parceiro de negócio ou para o escopo de faturamento, com prioridade para o parceiro de negócio.
     */

    this.insereIndicesPadroes = function (widget) {
        var params = {
            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
            "NRTIPOORCAMENTO": widget.currentRow.NRTIPOORCAMENTO,
            "NRPARCNEGOCIO": widget.currentRow.NRPARCNEGOCIO
        };

        General.callRestEngine('product#/fct_contrato_widget/insereIndicesPadroes', "Row", params);
    };

    /**
     * Habilita o campo de vigência atual para os contratos pendentes
     * @params{Object} widget - widget de contratos
     * @params{bool} readOnly - se o campo será habilitado ou não
     **/
    this.habilitaCampoVingenciaAtual = function (widget, readOnly) {
        if (!widget.currentRow.IDSITUACAOCONTRATO || widget.currentRow.IDSITUACAOCONTRATO == 'P') {
            widget.getField("DTINICIOCONTRATO").readOnly = !readOnly;
            widget.getField("DTFIMCONTRATO").readOnly = !readOnly;
            widget.getField("DTINIVIGENCONT").readOnly = readOnly;
            widget.getField("DTFINVIGENCONT").readOnly = readOnly;
        }
        else {
            widget.getField("DTINICIOCONTRATO").readOnly = true;
            widget.getField("DTFIMCONTRATO").readOnly = true;
            widget.getField("DTINIVIGENCONT").readOnly = true;
            widget.getField("DTFINVIGENCONT").readOnly = true;
        }
        //edição do campo Data de Reajuste
        // widget.getField("DTPADREAJCONT").readOnly = !( visibilidadeAcoes && idAlteraContAprv == 'S');
    };

    /**
     * Filtra a field de serviço/produto para trazer apenas os tipos
     * permitidos pelo escopo.
     * 
     * @params{Object} widget - widget de itens manuais do contrato
     **/
    this.filtrarTipoDeItensManuaisDoEscopo = function (widget) {
        var idtipoorca = getWidgetGridContrato().currentRow.IDTIPOORCA;
        var fieldServProd = widget.getField("NMITEMCONTRATO");
        var posFiltroTipo = undefined;

        if (!fieldServProd.dataSourceFilter) fieldServProd.dataSourceFilter = [];

        if (!idtipoorca) {
            // if(fieldServProd.dataSourceFilter && fieldServProd.dataSourceFilter.length) 
            //     fieldServProd.dataSourceFilter.pop();
            posFiltroTipo = Utilities.getFilterPosition(fieldServProd.dataSourceFilter, "TIPO");
            if (posFiltroTipo >= 0) {
                delete fieldServProd.dataSourceFilter[posFiltroTipo];
            }
        }
        else {
            idtipoorca = (idtipoorca == 'T') ? ['S', 'V'] : [idtipoorca];

            posFiltroTipo = Utilities.getFilterPosition(fieldServProd.dataSourceFilter, "TIPO");
            if (posFiltroTipo >= 0) {
                fieldServProd.dataSourceFilter[posFiltroTipo].value = idtipoorca;
            }
            else {
                fieldServProd.dataSourceFilter.push({
                    name: "TIPO",
                    operator: "IN",
                    value: idtipoorca
                });
            }
        }
    };

    this.validarVisibilidadeWidgetTpMedida = function (wdgTpMedida, nrtipoorcamento, nrseqcontrato) {
        var params = [{
            name: "NRSEQCONTRATO",
            operator: "=",
            value: nrseqcontrato
        }
            /*,
                        {
                            name: "NRTIPOORCAMENTO",
                            operator: "=",
                            value: nrtipoorcamento
                        }*/
        ];

        General.callRestEngine("product#/crud/fct_relacmedidacontrato_widget", "FilterData", params).then(function (response) {
            if (response.dataset && response.dataset.fct_relacmedidacontrato_widget && response.dataset.fct_relacmedidacontrato_widget.length)
                wdgTpMedida.isVisible = true;
            else
                wdgTpMedida.isVisible = false;
        });
    };

    /**
     * Valida se houve renovações automáticas ou se há contratos proximos 
     * ao vencimento sem renovação definida.
     **/
    /*this.validarExistenciaRenovacaoAutomatica = function() {
        var dataAtual = moment().subtract(1, 'days').format("DD/MM/YYYY");
        var params = { "DTREFERENCIA": dataAtual };

        General.callRestEngine("product#/fct_contrato_widget/validarExistenciaRenovacaoAutomatica", "Row", params).then(function(response) {
            if (response.dataset.fct_contrato_widget && response.dataset.fct_contrato_widget.POSSUICONTRATOSVENC) {
                ScreenService.confirmMessage("Atenção!<br><br>Os contratos com renovação automática que venceram na data de ontem foram reiterados pelo sistema. Os demais estão aguardando a renovação manual.<br><br>Deseja ir para a tela 'Renovação de Contratos'?", 'question', function() {
                    ScreenService.openWindow('product#sm_00004_108_renovacao_contrato').then(function() {
                        var widgetRenovacao = templateManager.container.getWidget('wdgGridContratosRenovados');
                        Utilities.buildCriteria(widgetRenovacao);

                        var filter = {
                            "DTFINVIGENCONT": {
                                "start": dataAtual,
                                "end": moment().format("DD/MM/YYYY")
                            }
                        };

                        ZhFilterPreferences.setOriginalFilter(widgetRenovacao.id, filter);
                        ZhFilterPreferences.applyOriginalFilter(widgetRenovacao);
                    });
                });
            }
        });
    };*/

    this.updateDsAlteracoesContr = function (widget) {

        var wdgGridContrato = templateManager.container.getWidget("gridContrato");

        widget.currentRow.NRSEQCONTRATO = wdgGridContrato.currentRow.NRSEQCONTRATO;

        var params = {
            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
            "DSALTERACONTRATUAIS": widget.currentRow.DSALTERACONTRATUAIS
        };

        General.callRestEngine("product#/fct_contrato_widget/updateDsAlteracoesContr", "Row", params).then(function (response) {

            General.widgetCancelForm(widget);
            widget.view();
            widget.reload();
        });
    };


    this.getAlteracaoContratual = function (widget) {
        var wdgGridContrato = templateManager.container.getWidget("gridContrato");

        widget.currentRow.NRSEQCONTRATO = wdgGridContrato.currentRow.NRSEQCONTRATO;

        var params = {
            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO
        };

        General.callRestEngine("product#/fct_contrato_widget/getAlteracaoContratual", "Row", params).then(function (response) {

            widget.currentRow.DSALTERACONTRATUAIS = response.dataset.getAlteracaoContratual.resultado;

        });
    };

    /**
     * Valida se o operador tem acesso à página "Alterações Contratuais"
     **/

    this.validaAlteracoesContratuais = function (widget) {

        var wdgAlteracoesContratuais = templateManager.container.getWidget("wdgAlteracoesContratuais");

        var params = {};

        General.callRestEngine("product#/fct_contrato_widget/validaAlteracoesContratuais", "Row", params).then(function (response) {
            if (response.dataset.validaAlteracoesContratuais.resultado == 'HABILITADO') {

                wdgAlteracoesContratuais.isVisible = true;
            }
            else {
                wdgAlteracoesContratuais.isVisible = false;
            }
        });
    };

    this.saveAssinaturaCont = function (widget) {
        if (widget.isValid()) {
            var isNew = widget.currentRow.__is_new;

            General.callRestEngine('product#/fct_contrato_widget/validaAssinaturaCont', "Row", widget.currentRow).then(function (response) {

                var retorno = response.dataset.validaAssinaturaCont.valida;

                /* Recebe os envolvidos para notificação*/
                var envolvidos = null;
                var arrayEnvolvidos = widget.currentRow.DSEMAILNOTSIGNENV_INS;

                if (arrayEnvolvidos != null && arrayEnvolvidos != "") {
                    envolvidos = arrayEnvolvidos.join('_');
                }

                widget.currentRow.DSEMAILNOTSIGNENV = envolvidos;

                /* Recebe os operdores para notificação */
                var arrayOperadores = widget.currentRow.OPERADORES;

                var operadoresIguais = false;

                if (widget.backupRow) {
                    var bkpOperadores = widget.backupRow.OPERADORES;

                    if (Utilities.arraysSaoIguais(bkpOperadores, arrayOperadores)) {
                        operadoresIguais = true;
                    }
                }

                if (retorno == '0') {

                    General.widgetSave(widget).then(function (response) {
                        if (response.dataset && response.dataset.columnsToReturn[0]) {
                            widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                            widget.currentRow.NRASSDIGITALCONTR = response.dataset.columnsToReturn[0].NRASSDIGITALCONTR;
                        }

                        if (isNew) {
                            operadoresIguais = false;
                            General.callRestEngine('product#/fct_contrato_widget/inserirSignatario', "Row", widget.currentRow).then(null).catch(function (error) {
                                Utilities.showErrorMessage(error);
                            });
                        }

                        if (operadoresIguais == false) {
                            var params = {
                                "NRASSDIGITALCONTR": widget.currentRow.NRASSDIGITALCONTR,
                                "OPERADORES": arrayOperadores
                            };

                            General.callRestEngine('product#/fct_contrato_widget/insereOperadoresNotifica', "Row", params).then(function () {
                                if (!isNew) {
                                    widget.reload().then(function () {
                                        this.carregaDadosOperadores(widget);
                                    }.bind(this));
                                }
                            }.bind(this))
                                .catch(function (error) {
                                    Utilities.showErrorMessage(error);
                                });
                        }
                        else {
                            this.carregaDadosOperadores(widget);
                        }

                    }.bind(this))
                        .catch(function (response) {
                            console.error(response);
                            if (response instanceof Object)
                                ScreenService.showMessage(response.data.error);
                            else
                                ScreenService.showMessage(response);
                        });
                }
                else {
                    ScreenService.showMessage("Operação Bloqueada! <br><br> O signatário informado já foi inserido.");
                }
            }.bind(this));
        }
        else {
            ScreenService.showMessage("Preencha todos os campos corretamente!");
        }
    };

    /**
     * insere as assinaturas do contrato 
     **/
    this.inserirAssinaturasContr = function (widget) {

        General.callRestEngine("product#/fct_contrato_widget/inserirAssinaturasContr", "Row", widget.currentRow).then(function () {

        }, function (error) {
            Utilities.showErrorMessage(error);
        });
    };

    /**
     * Valida se o token da certsign está preenchido
     */
    this.validaIdCertsign = function () {
        var wdgassinaturasCont = templateManager.container.getWidget("assinaturasCont");
        var wdgGridContrato = templateManager.container.getWidget("gridContrato");
        var wdgContratroGrid = templateManager.container.getWidget("wdgContratroGrid");

        var params = null;

        if (wdgGridContrato.currentRow.__is_new) {
            params = {
                "NRSEQCONTRATO": null
            };
        }
        else if (wdgGridContrato.currentRow.NRSEQCONTRATO) {
            params = {
                "NRSEQCONTRATO": wdgGridContrato.currentRow.NRSEQCONTRATO
            };
        };

        if (params) {
            General.callRestEngine('product#/fct_contrato_widget/validaIdCertsign', "Row", params).then(function (response) {
                var idTokenCertsign = response.dataset.validaIdCertsign.DSTOKENCERTSIGN;
                var idAssEletrDigital = response.dataset.validaIdCertsign.IDASSELETRDIGITAL;

                if (idTokenCertsign == null) {

                    /* Bloqueia a página "Signatários Padrões" */
                    wdgassinaturasCont.enabled = false;

                    /* Bloqueia o campo "Utiliza Assinatura Eletrônica ou Digital" */
                    wdgGridContrato.getField('IDASSELETRDIGITAL').readOnly = true;
                    wdgGridContrato.getField('IDASSELETRDIGITAL').validations = {};
                    if (wdgGridContrato.currentRow.changed == true) {
                        wdgGridContrato.getField('IDASSELETRDIGITAL').value('N');
                    }
                    this.contolaActionAssEletrDigital(wdgGridContrato, false);
                }
                else {

                    if (idAssEletrDigital == 'N') {
                        /* Bloqueia a página "Signatários Padrões" */
                        wdgassinaturasCont.enabled = false;
                        this.contolaActionAssEletrDigital(wdgGridContrato, false);
                    }
                    else {
                        /* Linera a página "Signatários Padrões" */
                        if (wdgGridContrato.currentRow.changed != true) {
                            wdgassinaturasCont.enabled = true;
                            this.contolaActionAssEletrDigital(wdgGridContrato, true);
                        }
                    }

                    /* Libera o campo "Utiliza Assinatura Eletrônica ou Digital" */
                    wdgGridContrato.getField('IDASSELETRDIGITAL').readOnly = false;
                    wdgGridContrato.getField('IDASSELETRDIGITAL').validations = { "required": true };
                }
            }.bind(this));
        }
    };

    /**
     * Envia novamente o e-mail de assinatura para os signatários que ainda não realizaram a assinatura
     */
    this.solicitarAssinatura = function (widget) {

        var wdgGridContrato = templateManager.container.getWidget("gridContrato");

        var params = {
            "NRSEQCONTRATO": wdgGridContrato.currentRow.NRSEQCONTRATO,
        };

        General.callRestEngine('product#/fct_contrato_widget/solicitarAssinatura', "Row", params).then(function (response) {

            var resultado = response.dataset.solicitarAssinatura.resultado;

            if (resultado == 'OK') {

                ScreenService.showMessage("Solicitação de assinatura enviada com sucesso!");

            }
            else {

                ScreenService.showMessage("Ocorreram erros na solicitação de assinatura. <br><br>Favor entrar em contato com o administrador do sistema.");
            }
        });
    };

    /**
     * Exclui o signatário do Certsign e altera a situação para 'Assinado Manualmente'.
     */
    this.assinadoManualmente = function (widget, checked) {

        ScreenService.confirmMessage("Atenção!<br><br>Ao confirmar a assinatura manual do signatário, todos os documentos vinculados à este signatário serão atualizados para a situação 'Assinado Manualmente'. <br><br> Caso não deseja assinar manualmente todos os documentos, a rotina deve ser realizada diretamente para os documentos.<br><br>Deseja continuar?", "question", function () {

            var signatario = null;
            var situacao = null;

            if (checked == true) {
                signatario = widget.getCheckedRows();
            }
            else {
                signatario = widget.currentRow;
                situacao = widget.currentRow.IDSITUACAO;
            }

            if (((checked == false) && (situacao == 'P')) || (checked == true)) {

                var params = {
                    "SIGNATARIO": signatario,
                    "CHECKED": checked
                };

                General.callRestEngine('product#/fct_contrato_widget/assinadoManualmente', "Row", params).then(function (response) {

                    var resultado = response.dataset.assinadoManualmente.resultado;

                    if (resultado == 'OK') {

                        if (checked == true) {
                            ScreenService.closePopup();
                            ScreenService.showMessage("Situação dos signatários atualizada com sucesso!");
                            widget.dataSource.clearCheckedRows();
                            widget.reload();
                        }
                        else {
                            ScreenService.showMessage("Situação do signatário atualizada com sucesso!");
                            ScreenService.closeSwipe().then(function () {
                                widget.reload();
                            });
                        }
                    }
                    else {
                        ScreenService.closePopup();
                        ScreenService.showMessage("Ocorreram erros na atualização da situação dos signatários. Favor entrar em contato com o administrador do sistema. <br/>Erro: ".resultado);
                    }
                });
            }
            else {

                ScreenService.showMessage("Operação Bloqueada! <br><br> Só é permitida a assinatura manual para signatários que estejam com a situação 'Pendente'.");
            }
        }.bind(this));
    };


    /**
     * filtra parcnegocio para datasouce do signatário
     **/
    this.filtraParcnegocioAssinat = function () {

        var wdgassinaturasCont = templateManager.container.getWidget("assinaturasCont");
        var wdgGridContrato = templateManager.container.getWidget("gridContrato");

        wdgassinaturasCont.getField("NMPRINCIPALPARC").dataSourceFilter[0].value = wdgGridContrato.currentRow.NRPARCNEGOCIO;
    };

    /**
     * A função "contolaActionAssEletrDigital" controla a visibilidade da action de 
     * geração de documento para a assinatura eletrônica e digital.
     **/
    this.contolaActionAssEletrDigital = function (widget, visible) {
        var situacao = widget.currentRow.IDSITUACAOCONTRATO;

        var assEletrDigital = widget.getAction('assinatura_eletronica_digital');
        var downDocEletrDigital = widget.getAction('download_Doc_Eletr_Digital');

        /* Aprovado */
        if (situacao == 'A') {

            assEletrDigital.isVisible = false;
            downDocEletrDigital.isVisible = visible;
        } /* Reprovado */
        else if (situacao == 'R') {

            assEletrDigital.isVisible = false;
            downDocEletrDigital.isVisible = false;
        } /* Encerrado */
        else if (situacao == 'E') {

            assEletrDigital.isVisible = false;
            downDocEletrDigital.isVisible = false;
        } /* Suspenso */
        else if (situacao == 'S') {

            assEletrDigital.isVisible = false;
            downDocEletrDigital.isVisible = false;
        } /* Documento Enviado*/
        else if (situacao == 'D') {

            assEletrDigital.isVisible = false;
            downDocEletrDigital.isVisible = visible;
        } /* Pendente */
        else {

            assEletrDigital.isVisible = visible;
            downDocEletrDigital.isVisible = false;
        }

    };

    /**
     * A função "assinaturaEletrDigital" realiza a geração do documento de contrato no Certsign
     **/
    this.assinaturaEletrDigital = function (widget) {

        var params = {
            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO
        };

        General.callRestEngine('product#/fct_contrato_widget/validaIdCertsign', "Row", params).then(function (response) {
            var idTokenCertsign = response.dataset.validaIdCertsign.DSTOKENCERTSIGN;
            var idAssEletrDigital = response.dataset.validaIdCertsign.IDASSELETRDIGITAL;

            if ((idTokenCertsign != false) && (idAssEletrDigital == 'S')) {

                ScreenService.confirmMessage(ScreenService.i18n("Ao enviar os documentos para assinatura eletrônica e digital, o contrato será bloqueado e não poderá receber novas alterações.<br><br>Deseja realmente enviar os documentos para assinatura eletrônica e digital?"), 'question', function (success) {

                    var params = {
                        "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                        "IDSITUACAOCONTRATO": widget.currentRow.IDSITUACAOCONTRATO,
                        "NRCONTRATOFCT": widget.currentRow.NRCONTRATOFCT
                    };

                    General.callRestEngine("product#/fct_contrato_widget/gerarDocumentoCertsign", "Row", params).then(function (response) {

                        var resultado = response.dataset.gerarDocumentoCertsign.resultado;

                        if (resultado == 'OK') {
                            ScreenService.closePopup();
                            this.voltarParaContratoGrid();
                            ScreenService.showMessage('Documentos enviados para assinatura eletrônica e digital com sucesso.');
                        }
                        else {

                            ScreenService.showMessage(resultado);
                        }

                    }.bind(this), function (error) {
                        Utilities.showErrorMessage(error);
                    }.bind(this));
                }.bind(this));
            }
        }.bind(this));
    };

    /**
     * exclui os signatários da proposta e do ambiente Certsign 
     **/
    this.excluirSignatariosCertsign = function (widget) {

        var nrseqcontrato = widget.currentRow.NRSEQCONTRATO;
        var idsignatcertsign = widget.currentRow.IDSIGNATCERTSIGN;
        var nrcpfsignatario = widget.currentRow.NRCPFSIGNATARIO;
        var nrassdigitalcontr = widget.currentRow.NRASSDIGITALCONTR;

        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
            var params = {
                "NRSEQCONTRATO": nrseqcontrato,
                "NRASSDIGITALCONTR": nrassdigitalcontr,
                "IDSIGNATCERTSIGN": idsignatcertsign,
                "NRCPFSIGNATARIO": nrcpfsignatario
            };

            General.callRestEngine("product#/fct_contrato_widget/excluirSignatariosCertsign", "Row", params).then(function () {

                widget.removeRow().then(function () {

                    ScreenService.closeSwipe().then(function () {
                        widget.activate();
                        widget.reload();
                    });
                },
                    function (error) {
                        if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                            ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
                        }
                        else {
                            ScreenService.showMessage(error.data.error);
                        }
                    });
            },
                function (error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
        });
    };


    /**
     * realiza o download do documento do ambiente Certsign 
     **/
    this.downloadDocEletrDigital = function (widget) {

        var nrseqcontrato = widget.currentRow.NRSEQCONTRATO;

        window.open(serviceUrl + "/fct_contrato_widget/downloadDocEletrDigital?requestType=DataSet&dataset[0][NRSEQCONTRATO]=" + nrseqcontrato);

    };

    this.controlaEdicaoAssinat = function (widget) {

        var wdgGridContrato = templateManager.container.getWidget("gridContrato");

        var situacao = wdgGridContrato.currentRow.IDSITUACAOCONTRATO;

        var edit = widget.getAction('EDIT');
        var actAssinaManual = widget.getAction('assinadoManualmente');
        var actAssinaManualChk = widget.getAction('assinadoManualmenteChk');
        var actSolicitaAss = widget.getAction('solicitarAssinatura');
        var actAlteraSituAss = widget.getAction('AlteraSituAss');
        var actAlteraSituAssChk = widget.getAction('AlteraSituAssChk');

        if (situacao == 'P') {

            edit.isVisible = true;
            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = false;
            actAlteraSituAss.isVisible = false;
            actAlteraSituAssChk.isVisible = false;
        }
        else if ((situacao == 'A') || (situacao == 'D')) {
            edit.isVisible = false;
            actAssinaManual.isVisible = true;
            actAssinaManualChk.isVisible = true;
            actSolicitaAss.isVisible = true;
            actAlteraSituAss.isVisible = true;
            actAlteraSituAssChk.isVisible = true;
        }
        else {

            edit.isVisible = false;
            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = false;
            actAlteraSituAss.isVisible = false;
            actAlteraSituAssChk.isVisible = false;
        }
    };

    this.controlaAcoesAssinat = function (widget) {

        var actAssinaManual = widget.getAction('assinadoManualmente');
        var actAssinaManualChk = widget.getAction('assinadoManualmenteChk');
        var actSolicitaAss = widget.getAction('solicitarAssinatura');
        var actAlteraSituAss = widget.getAction('AlteraSituAss');
        var actAlteraSituAssChk = widget.getAction('AlteraSituAssChk');

        var excluir = widget.getAction('EXCLUIR');

        var situacao = widget.currentRow.IDSITUACAO;

        if (situacao == 'P') {
            this.controlaEdicaoAssinat(widget);
            excluir.isVisible = true;

        }
        else {

            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = false;
            excluir.isVisible = false;
            actAlteraSituAss.isVisible = false;
            actAlteraSituAssChk.isVisible = false;
        }
    }

    this.controlaEdicaoAssinatDoc = function (widget) {

        var wdgGridContrato = templateManager.container.getWidget("gridContrato");

        var situacao = wdgGridContrato.currentRow.IDSITUACAOCONTRATO;

        var actAssinaManual = widget.getAction('assinadoManualmenteDoc');
        var actAssinaManualChk = widget.getAction('assinadoManualmenteDocChk');
        var actSolicitaAss = widget.getAction('solicitarAssinaturaDoc');

        if (situacao == 'P') {

            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = false;
        }
        else if ((situacao == 'A') || (situacao == 'D')) {

            actAssinaManual.isVisible = true;
            actAssinaManualChk.isVisible = true;
            actSolicitaAss.isVisible = true;
        }
        else /*CANCELADO*/ {
            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = false;
        }

    };

    this.controlaAcoesAssinatDoc = function (widget) {
        var actAssinaManual = widget.getAction('assinadoManualmenteDoc');
        var situacao = widget.currentRow.IDSITUACAO;

        if (situacao == 'P') {
            this.controlaEdicaoAssinatDoc(widget);
        }
        else {
            actAssinaManual.isVisible = false;
        }
    };

    /**
     * Exclui o signatário do Certsign e altera a situação para 'Assinado Manualmente'.
     */
    this.assinadoManualmenteDoc = function (widget, checked) {

        var signatario = null;
        var situacao = null;

        if (checked == true) {
            signatario = widget.getCheckedRows();
        }
        else {
            signatario = widget.currentRow;
            situacao = widget.currentRow.IDSITUACAO;
        }

        if (((checked == false) && (situacao == 'P')) || (checked == true)) {

            var params = {
                "SIGNATARIO": signatario,
                "CHECKED": checked
            };

            General.callRestEngine('product#/fct_contrato_widget/assinadoManualmenteDoc', "Row", params).then(function (response) {

                var resultado = response.dataset.assinadoManualmenteDoc.resultado;

                if (resultado == 'OK') {

                    if (checked == true) {
                        ScreenService.closePopup();
                        ScreenService.showMessage("Situação dos signatários atualizada com sucesso!");
                        widget.dataSource.clearCheckedRows();
                        widget.reload();
                    }
                    else {
                        ScreenService.showMessage("Situação do signatário atualizada com sucesso!");
                        ScreenService.closeSwipe().then(function () {
                            widget.reload();
                        });
                    }
                }
                else {
                    ScreenService.closePopup();
                    ScreenService.showMessage("Ocorreram erros na atualização da situação dos signatários. Favor entrar em contato com o administrador do sistema. <br/>Erro: ".resultado);
                }
            });
        }
        else {

            ScreenService.showMessage("Operação Bloqueada! <br><br> Só é permitida a assinatura manual para signatários que estejam com a situação 'Pendente'.");
        }
    };

    this.getIdParcelaConsiderada = function (nrtipoorcamento) {
        var deferred = ZHPromise.defer();
        var params = {
            "NRTIPOORCAMENTO": nrtipoorcamento
        };

        General.callRestEngine("product#/fct_paramfatescopo_widget/getTipoCalculoPeriodosIncompletos", "Row", params).then(function (retorno) {
            var idtpcalcparcinc = null;
            if (retorno.dataset && retorno.dataset.retorno.IDTPCALCPARCINC) {
                idtpcalcparcinc = retorno.dataset.retorno.IDTPCALCPARCINC;
            }

            if (idtpcalcparcinc == '2') {
                ScreenService.showCustomDialog("Atenção! <br><br>Ao renegociar o contrato, os valores da parcelas podem sofrer alterações.<br><br> Quanto a parcela da competência atual, o sistema pode considerar as seguintes opções: <br><br> - Manter no contrato vigente com o valor que está sendo praticado atualmente <br><br> - Gerar no contrato de nova versão já com os valores renegociados.<br><br> Qual opção o sistema deve considerar?", "CONFIRMATION", null, [{
                    "label": "Manter no contrato vigente",
                    "code": function () {
                        deferred.resolve('2');
                    }
                },
                {
                    "label": "Gerar no contrato renegociado",
                    "code": function () {
                        deferred.resolve('1');
                    }
                },
                {
                    "label": "Cancelar",
                    "code": function () { }
                }
                ]);
            }
            else {
                deferred.resolve(null);
            }
        })
            .catch(function (error) {
                deferred.reject(error);
            });

        return deferred.promise;
    };

    this.onEnterTabParcelasNegociacao = function (widget) {

        var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");

        widget.dataSourceFilter[0].value = wdgNegociacaoValores.currentRow.NRNEGOCIACAOCONTRATO;

        Utilities.carregarWidgetOnEnterTab(widget);

    };

    this.suspenderContratoDialog = function (widget) {
        ScreenService.closePopup();
        var wdgPopup = templateManager.container.getWidget("wdgPopupSuspensaoCont");
        wdgPopup.isVisible = true;
        wdgPopup.dataSource.data = [];
        wdgPopup.currentRow = {};
        ScreenService.openPopup(wdgPopup).then(function () {
            wdgPopup.activate();
            wdgPopup.newRow();
        }.bind(this));
    };

    this.confirmaSuspensao = function (widget) {
        if (widget.isValid()) {
            ScreenService.showCustomDialog("Ao suspender o contrato, todas as parcelas de faturamento com situação 'Pendente' à partir da data da suspensão e todas as medições de faturamento que ainda não foram faturadas serão excluídas. <br><br> As parcelas de faturamento com situação 'Pendente' que forem anteriores a data de suspensão podem ser mantidas para continuidade do faturamento até a data de suspensão do contrato ou podem ser excluídas. <br><br> O que deve ser feito com as parcelas de situação 'Pendente' anteriores à suspensão do contrato?", 'CONFIRMATION', null, [{
                "label": "Manter as Parcelas",
                "code": function () {
                    this.suspenderContrato(widget, 'MANTER');
                }.bind(this)
            }, {
                "label": "Excluir as Parcelas",
                "code": function () {
                    this.suspenderContrato(widget, 'EXCLUIR');
                }.bind(this)
            }, {
                "label": "Cancelar",
                "code": function () { }
            }]);
        }
    }

    /**
     * suspende o contrato para pausar o faturamento e as alterações no contrato.
     **/
    this.suspenderContrato = function (widget, idParcelasAnteriores) {

        var wdgContratroGrid = templateManager.container.getWidget('wdgContratroGrid');
        var gridContrato = templateManager.container.getWidget('gridContrato');


        var params = {
            "NRSEQCONTRATO": gridContrato.currentRow.NRSEQCONTRATO,
            "PARCELASANTERIORES": idParcelasAnteriores,
            "CDOPERSOLSUSPENSAO": widget.currentRow.CDOPERSOLSUSPENSAO,
            "DSJUSTIFCASUSPENSAO": widget.currentRow.DSJUSTIFCASUSPENSAO

        };

        General.callRestEngine("product#/fct_contrato_widget/suspenderContrato", "Row", params).then(function () {

            ScreenService.showMessage("Suspensão de contrato realizada com sucesso!");
            ScreenService.closeSwipe().then(function () {
                ScreenService.closePopup();
                wdgContratroGrid.activate();
                wdgContratroGrid.reload();
            });

        }, function (error) {
            Utilities.showErrorMessage(error);
        });
    };

    /**
     * Cancela a suspensão do contrato
     **/
    this.cancelarSuspensaoContrato = function (widget) {
        ScreenService.confirmMessage("Deseja realmente cancelar a suspensão do contrato?", 'question', function () {
            var wdgContratroGrid = templateManager.container.getWidget('wdgContratroGrid');
            var params = {
                "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO
            };

            General.callRestEngine("product#/fct_contrato_widget/cancelarSuspensaoContrato", "Row", params).then(function () {
                ScreenService.showMessage("Suspensão de contrato cancelada com sucesso!");
                ScreenService.closeSwipe().then(function () {
                    ScreenService.closePopup();
                    wdgContratroGrid.reload();
                });

            }, function (error) {
                Utilities.showErrorMessage(error);
            });
        });
    };


    this.rescisaoMassa = function (widget) {

        var resposta = this.validarSolicitacaoMultiplosContratos(widget.getCheckedRows());
        var wdgPopup = templateManager.container.getWidget("PopupDsJustificaEncerra");
        if (resposta.STATUS) {
            try {
                ScreenService.confirmMessage(ScreenService.i18n(resposta.MENSAGEM), 'question', function () {
                    wdgPopup.isVisible = true;
                    ScreenService.closePopup();
                    ScreenService.openPopup(wdgPopup).then(function () {
                        wdgPopup.dataSource.data = [];
                        wdgPopup.newRow();

                        wdgPopup.currentRow.IDRESCICAOEMMASSA = 'S';
                        wdgPopup.currentRow.DTPEDRESCISAOCONTR = moment().format("DD/MM/YYYY");
                        wdgPopup.currentRow.DTRECISAOCONTR = moment().format("DD/MM/YYYY");

                        wdgPopup.currentRow.ACTION = "encerrarContrato";
                        wdgPopup.redraw();
                    }.bind(this));
                }.bind(this));
            }
            catch (err) {
                Utilities.showErrorMessage(err);
            }
        }
        else {
            ScreenService.showMessage(ScreenService.i18n(resposta.MENSAGEM));
        }
    };

    /**
     * A função "encerraContrato" executa a procedure
     * "FCT_ENCERRA_CONTRATO".
     */
    this.encerraContratoEmMassa = function (widget, checkedRows, idDocX) {
        var deferred = ZHPromise.defer();

        var wdgContratroGrid = templateManager.container.getWidget("wdgContratroGrid");

        var nrtiporescisao = widget.currentRow.NRTIPORESCISAO;

        var params = {
            "DADOSRESCISAO": widget.currentRow,
            "CONTRATOS": checkedRows,
            "IDDOCX": idDocX
        };

        General.callRestEngine("product#/fct_contrato_widget/encerrarContratoEmMassa", "Row", params).then(function (response) {

            var resultado = response.dataset.encerrarContratoEmMassa.resultado;

            if (idDocX != null) {

                var params = {
                    "CONTRATOS": resultado,
                    "NRPARCNEGOCIO": null,
                    "NRSEQCONTRATO": null,
                    "NRTIPORESCISAO": nrtiporescisao,
                    "IDDOCX": idDocX
                };

                var timeout = 5 * 60000; //5 min
                Utilities.callRestEngineWithTimeout("product#/fct_contrato_widget/gerarDocumentoContratoRescisao", "Row", params, timeout).then(function (success) {
                    var links = success.dataset.documentos.links;
                    links.forEach(function (link) {
                        if (link)
                            window.open(window.backendIndexUrl + link);
                    });

                    ScreenService.showMessage("Os contratos foram encerrados.");
                    ScreenService.closePopup();

                    wdgContratroGrid.dataSource.clearCheckedRows();
                    wdgContratroGrid.reload();
                })
                    .catch(function (error) {
                        console.error(error);
                        var msg = "Timeout";
                        if (error.status !== 0 && error.data)
                            msg = error.data.error;

                        ScreenService.showMessage(msg);
                    });

            }
            else {

                ScreenService.showMessage("Os contratos foram encerrados.");
                ScreenService.closePopup();

                wdgContratroGrid.dataSource.clearCheckedRows();
                wdgContratroGrid.reload();
            }

            deferred.resolve();

        }.bind(this))
            .catch(function (error) {
                deferred.reject(error);
            });

        return deferred.promise;
    };


    this.carregaFiltrosServico = function (widget) {

        var gridContrato = templateManager.container.getWidget("gridContrato");

        var params = {
            "NRTIPOORCAMENTO": gridContrato.currentRow.NRTIPOORCAMENTO
        };

        General.callRestEngine("product#/fct_contrato_widget/validaFiltroServico", "Row", params).then(function (response) {

            var resultado = response.dataset.validaFiltroServico.resultado;

            if (widget.name == 'itemFormacaoPreco') {

                widget.getField("NRFORMPRECOSERVICO").dataSourceFilter[2].value = resultado;
                widget.getField("NRFORMPRECOSERVICO").dataSourceFilter[3].value = gridContrato.currentRow.NRTIPOORCAMENTO;

                widget.getField("NMSERVORCA").dataSourceFilter[0].value = resultado;
                widget.getField("NMSERVORCA").dataSourceFilter[1].value = gridContrato.currentRow.NRTIPOORCAMENTO;

            }
            else {

                widget.getField("NMITEMCONTRATO").dataSourceFilter[0].value = resultado;
                widget.getField("NMITEMCONTRATO").dataSourceFilter[1].value = gridContrato.currentRow.NRTIPOORCAMENTO;
            }


        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });

    };

    this.aprovarContrMassa = function (validarAgrupamentoContratos) {
        ScreenService.confirmMessage("Deseja realmente aprovar os contratos selecionados?", 'question', function () {
            var wdgContratroGrid = templateManager.container.getWidget('wdgContratroGrid');

            var checkedRows = wdgContratroGrid.getCheckedRows();

            var timeout = 8 * 60000; //8 minutos

            Utilities.callRestEngineWithTimeout("product#/fct_contrato_widget/aprovarContrMassa", "Row", checkedRows, timeout).then(function (response) {

                var resultado = response.dataset.aprovarContrMassa.resultado;

                if (resultado.QTERROS > 0) {
                    ScreenService.closePopup();
                    wdgContratroGrid.reload();
                    wdgContratroGrid.dataSource.clearCheckedRows();
                    this.popupAprovarContrMassa(resultado.ERROS);
                }
                else {

                    ScreenService.showMessage("Contratos aprovados com sucesso!");
                    ScreenService.closePopup();
                    wdgContratroGrid.reload();
                    wdgContratroGrid.dataSource.clearCheckedRows();
                }

            }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
        }.bind(this));
    };

    /**
     * Abre a widget wdgErroAprovaContrato 
     **/

    this.popupAprovarContrMassa = function (dados) {

        var wdgErroAprovaContrato = templateManager.container.getWidget('wdgErroAprovaContrato');
        wdgErroAprovaContrato.dataSource.data = dados;
        wdgErroAprovaContrato.isVisible = true;

        ScreenService.openPopup(wdgErroAprovaContrato).then(function () {
            $('.zh-new-grid-wdgErroAprovaContrato').parent().parent().parent().parent().parent().parent().addClass('cp-high-modal');
        });

    };

    /**
     * Fecha a widget wdgErroAprovaContrato 
     **/
    this.fecharAprovarContrMassa = function () {
        $('.zh-new-grid-wdgErroAprovaContrato').parent().parent().parent().parent().parent().parent().removeClass('cp-high-modal');

        var wdgErroAprovaContrato = templateManager.container.getWidget('wdgErroAprovaContrato');
        wdgErroAprovaContrato.isVisible = true;
        ScreenService.closePopup();

    };

    this.regerarFaturaMassa = function (widget) {
        ScreenService.confirmMessage("Deseja realmente regerar o faturamento dos contratos selecionados?", 'question', function () {
            var wdgContratroGrid = templateManager.container.getWidget('wdgContratroGrid');

            var checkedRows = wdgContratroGrid.getCheckedRows();

            General.callRestEngine("product#/fct_contrato_widget/regerarFaturaMassa", "Row", checkedRows).then(function (response) {

                var resultado = response.dataset.regerarFaturaMassa.resultado;

                if (resultado.QTERROS > 0) {
                    ScreenService.closePopup();
                    wdgContratroGrid.reload();
                    wdgContratroGrid.dataSource.clearCheckedRows();
                    this.popupRegerarFaturaMassa(resultado.ERROS);

                }
                else {

                    ScreenService.showMessage("Processo realizado com sucesso!");
                    ScreenService.closePopup();
                    wdgContratroGrid.reload();
                    wdgContratroGrid.dataSource.clearCheckedRows();
                }

            }.bind(this), function (error) {
                Utilities.showErrorMessage(error);
            }.bind(this));
        }.bind(this));
    };

    /**
     * Abre a widget wdgErroRegerarFatura 
     **/
    this.popupRegerarFaturaMassa = function (dados) {

        var wdgErroRegerarFatura = templateManager.container.getWidget('wdgErroRegerarFatura');
        wdgErroRegerarFatura.dataSource.data = dados;
        wdgErroRegerarFatura.isVisible = true;

        ScreenService.openPopup(wdgErroRegerarFatura).then(function () {
            $('.zh-new-grid-wdgErroRegerarFatura').parent().parent().parent().parent().parent().parent().addClass('cp-high-modal');
        });

    };

    /**
     * Fecha a widget wdgErroRegerarFatura 
     **/
    this.fecharRegerarFaturaMassa = function () {
        $('.zh-new-grid-wdgErroRegerarFatura').parent().parent().parent().parent().parent().parent().removeClass('cp-high-modal');

        var wdgErroRegerarFatura = templateManager.container.getWidget('wdgErroRegerarFatura');
        wdgErroRegerarFatura.isVisible = true;
        ScreenService.closePopup();

    };

    this.carregaDadosOperadores = function (widget) {
        var operadores = widget.currentRow.OPERADORES;
        if (operadores && operadores.length > 0) {
            widget.getField("OPERADORES").dataSourceFilter[2] = {
                "name": "CDOPERADOR",
                "operator": "IN",
                "value": operadores
            };

            widget.getField("OPERADORES").reload();
            widget.getField("OPERADORES").dataSourceFilter.splice(2);
            templateManager.updateTemplate();
        }
    };

    this.habilitarDadosDeFaturamento = function (widget) {
        console.log("chamou habilitar Dados de Faturamento", widget);
    };

    this.recebeOrdenacaoENotificacoes = function (widget) {
        var wdgContrato = getWidgetGridContrato();

        var modeloContrato = wdgContrato.currentRow.NRSEQMODCONTRA;
        var nrseqinterlocutor = widget.currentRow.NRSEQINTERLOCUTOR;
        var idtestemunha = widget.currentRow.IDTESTEMUNHA;
        var tipoSignat = null;

        if (nrseqinterlocutor != null) {
            if (idtestemunha == 'S') {
                tipoSignat = 'TC';
            }
            else {
                tipoSignat = 'SC';
            }
        }
        else {
            if (idtestemunha == 'S') {
                tipoSignat = 'TP';
            }
            else {
                tipoSignat = 'SP';
            }
        }

        var params = {
            "NRSEQMODCONTRA": modeloContrato,
            "IDTPSIGNATARIO": tipoSignat
        };

        General.callRestEngine('product#/fct_contrato_widget/recebeOrdenacaoENotificacoes', "Row", params).then(function (response) {
            var dados = response.dataset.recebeOrdenacaoENotificacoes.resultado;

            if (dados['DADOS'] == 'OK') {
                widget.currentRow.NRORDENACAO = dados['NRORDENACAO'];

                var dsemailnotsignenvAux = dados['DSEMAILNOTSIGNENV'];
                var dsemailnotsignenv = dsemailnotsignenvAux.split('_');

                widget.currentRow.DSEMAILNOTSIGNENV_INS = dsemailnotsignenv;
                widget.currentRow.DSEMAILNOTSIGNENV = dsemailnotsignenv;
                widget.currentRow.OPERADORES = dados['OPERADORES'];
                widget.currentRow.DSEMAILNOTIFOUTROS = dados['DSEMAILNOTIFOUTROS'];
            }
            else {
                widget.currentRow.NRORDENACAO = 1;
                widget.currentRow.DSEMAILNOTSIGNENV_INS = null;
                widget.currentRow.DSEMAILNOTSIGNENV = null;
                widget.currentRow.OPERADORES = null;
                widget.currentRow.DSEMAILNOTIFOUTROS = null;
            }

            this.carregaDadosOperadores(widget);

        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };
    this.saveFaturamentoJustificativa = function (widget) {
        widget.currentRow.IDALTUSUARIO = "S";
        var wdgPopup = templateManager.container.getWidget("wdgPopupDsJustificaFat");
        wdgPopup.isVisible = true;
        wdgPopup.dataSource.data = [];
        wdgPopup.currentRow = {};
        ScreenService.openPopup(wdgPopup).then(function () {
            wdgPopup.activate();
            wdgPopup.newRow();
        }.bind(this));
    };
    this.ConfirmFaturamentoJustificativa = function (widget) {
        var faturamentocontrato = templateManager.container.getWidget("faturamentocontrato");

        if (faturamentocontrato.currentRow.DSJUSTIFICATIVA != null) {
            faturamentocontrato.currentRow.DSJUSTIFICATIVA = faturamentocontrato.currentRow.DSJUSTIFICATIVA + "\n" + moment().format("DD/MM/YYYY HH:mm:ss") + " - " + widget.currentRow.DSJUSTIFICATIVA;
        }
        else {
            faturamentocontrato.currentRow.DSJUSTIFICATIVA = moment().format("DD/MM/YYYY HH:mm:ss") + " - " + widget.currentRow.DSJUSTIFICATIVA;
        }
        General.widgetSave(faturamentocontrato);
        ScreenService.closePopup();
        faturamentocontrato.activate();
        this.controlaAcessActionsFatNovo(faturamentocontrato);
    };

    this.abreWidgetAddNewTipoAnexo = function (widget) {
        ScreenService.closePopup();
        var wdgPopup = templateManager.container.getWidget("wdgNewITipoAnexo");
        wdgPopup.isVisible = true;
        wdgPopup.dataSource.data = [];
        ScreenService.openPopup(wdgPopup).then(function () {
            wdgPopup.activate();
            Utilities.formWidgetNewRow(wdgPopup);
            wdgPopup.nmwdganterior = widget.name;
        });
    };

    this.cancelaWidgetTipoAnexo = function (widget) {
        var wdgAnterior = templateManager.container.getWidget(widget.nmwdganterior);
        ScreenService.closePopup();
        wdgAnterior.activate();
    };

    this.salvarNovoTipoAnexo = function (widget) {
        var wdgAnterior = templateManager.container.getWidget(widget.nmwdganterior);

        General.widgetSave(widget).then(function (response) {
            wdgAnterior.currentRow.NRTIPOANEXO = response.dataset.columnsToReturn[0].NRTIPOANEXO;
            wdgAnterior.currentRow.NMTIPOANEXO = widget.currentRow.NMTIPOANEXO;
            widget.isVisible = false;
            ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
            ScreenService.closePopup();
            wdgAnterior.activate();
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };

    this.validaBloqueiaDataInicialContReneg = function (widget) {
        var params = {
            "NRSEQMODCONTRA": widget.currentRow.NRSEQMODCONTRA,
            "NRORG": widget.currentRow.NRORG
        };
        General.callRestEngine("product#/fct_contrato_widget/validaBloqueiaDataInicialContReneg", "Row", params).then(function (response) {
            if (response.dataset.validaBloqueiaDataInicialContReneg.resultado.IDBLOQALTDTINICONTRENEG == 'S' && widget.currentRow.NRVERSAOCONTRATO != 1) {
                widget.getField("DTINIVIGENCONT").readOnly = true;
            }
        }.bind(this))
    };

    this.assinaturaEletronicaOnChange = function (widget) {
        var idasseletrdigital = widget.currentRow.IDASSELETRDIGITAL;
        var fieldEnviaDocOperResp = widget.getField("IDENVIADOCOPERRESPCONCASS");
        if (idasseletrdigital == "N") {
            fieldEnviaDocOperResp.readOnly = true;
            fieldEnviaDocOperResp.value("N");
        }
        else {
            fieldEnviaDocOperResp.readOnly = false;
        }
    };

    this.enterTabSeguro = function (widget) {
        var wdgContrato = templateManager.container.getWidget("gridContrato");
        var situacao = wdgContrato.currentRow.IDSITUACAOCONTRATO;
        var adicionaSeguro = widget.getAction('add');
        var editaSeguro = widget.getAction('edit');
        var deletaSeguro = widget.getAction('delete');

        Utilities.formWidgetOnEnterTab(widget, true).then(function () {
            if (!['A', 'P'].includes(situacao)) {
                adicionaSeguro.isVisible = editaSeguro.isVisible = deletaSeguro.isVisible = false;
            }
        });
    };

    this.abreWidgetRenovaSeguro = function (widget) {
        var wdgPopup = templateManager.container.getWidget("wdgPopupRenovarSeguro");
        var wdgAnterior = templateManager.container.getWidget("wdgSeguroVigente");
        var fieldDataIniSeguro = wdgPopup.getField("DTINISEGURO");
        var fieldDataFinSeguro = wdgPopup.getField("DTFINSEGURO");
        var fieldTaxaSeguro = wdgPopup.getField("VRTAXASEGURO");
        var fieldValorApolice = wdgPopup.getField("VRAPOLICE");
        var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];

        var prazo = widget.currentRow.NRDIASVIGENCIA;
        var dataAux = moment(widget.currentRow.DTFINSEGURO, "DD/MM/YYYY");

        fieldDataIniSeguro.readOnly = false;
        fieldDataFinSeguro.readOnly = false;
        fieldTaxaSeguro.readOnly = false;
        fieldValorApolice.readOnly = false;

        wdgPopup.isVisible = true;
        wdgPopup.dataSource.data = [];
        ScreenService.openPopup(wdgPopup).then(function () {
            wdgPopup.activate();
            wdgPopup.newRow();
            wdgPopup.currentRow = Util.clone(wdgAnterior.currentRow, ignoredKeys);
            wdgPopup.currentRow.NRSEGUROCONTRATO = null;
            wdgPopup.currentRow.__is_new = true;

            dataAux.add(1, "days");
            wdgPopup.currentRow.DTINISEGURO = dataAux.format("DD/MM/YYYY");
            dataAux.add(prazo - 1, "days");
            wdgPopup.currentRow.DTFINSEGURO = dataAux.format("DD/MM/YYYY");
            wdgPopup.currentRow.NRDIASVIGENCIA = prazo;
        }.bind(this));
    };

    this.habilitarEdicaoAbaElementos = function (widget) {
        var contratoCurrentRow = getWidgetGridContrato().currentRow;
        var editComposicaoServicos = widget.getAction('edit_comp_serv');
        var insElementos = widget.getAction('insert_elemento_contrato');
        var save = widget.getAction('save');
        var cancelar = widget.getAction('cancel');
        cancelar.isVisible = save.isVisible = this.habilitarAcaoElementos(contratoCurrentRow);
        editComposicaoServicos.isVisible = this.habilitarAcaoElementosEComposicao(contratoCurrentRow);
        insElementos.isVisible = this.habilitarAcaoElementosEComposicao(contratoCurrentRow);
    };

    this.habilitarEdicaoAbaElementosNew = function (widget) {
        var contratoCurrentRow = getWidgetGridContrato().currentRow;
        var situacao = contratoCurrentRow.IDSITUACAOCONTRATO;

        var save = widget.getAction('save');
        var cancelar = widget.getAction('cancel');
        var editar = widget.getAction('edit');
        var insElementos = widget.getAction('insert_elemento_contrato');
        var editComposicaoServicos = widget.getAction('edit_comp_serv');
        var saveEdit = widget.getAction('save_comp_serv');
        var cancelarEdit = widget.getAction('cancel_comp_serv');
        
        if (situacao == 'A') {
            save.isVisible = false;
            cancelar.isVisible = false;
            editar.isVisible = false;
            insElementos.isVisible = false;
            editComposicaoServicos.isVisible = false;
            saveEdit.isVisible = false;
            cancelarEdit.isVisible = false;
        } /* Documento Enviado */
        else if (situacao == 'D') {
            save.isVisible = false;
            cancelar.isVisible = false;
            editar.isVisible = false;
            insElementos.isVisible = false;
            editComposicaoServicos.isVisible = false;
            saveEdit.isVisible = false;
            cancelarEdit.isVisible = false;
        }
        /* Reprovado */
        else if (situacao == 'R') {
            save.isVisible = false;
            cancelar.isVisible = false;
            editar.isVisible = false;
            insElementos.isVisible = false;
            editComposicaoServicos.isVisible = false;
            saveEdit.isVisible = false;
            cancelarEdit.isVisible = false;
        } /* Encerrado */
        else if (situacao == 'E') {
            save.isVisible = false;
            cancelar.isVisible = false;
            editar.isVisible = false;
            insElementos.isVisible = false;
            editComposicaoServicos.isVisible = false;
            saveEdit.isVisible = false;
            cancelarEdit.isVisible = false;
        } // Suspenso
        else if (situacao == 'S') {
            save.isVisible = false;
            cancelar.isVisible = false;
            editar.isVisible = false;
            insElementos.isVisible = false;
            editComposicaoServicos.isVisible = false;
            saveEdit.isVisible = false;
            cancelarEdit.isVisible = false;
        } /* Pendente */
        else {

            save.isVisible = true;
            cancelar.isVisible = true;
            editar.isVisible = true;
            insElementos.isVisible = true;
            editComposicaoServicos.isVisible = false;
            saveEdit.isVisible = false;
            cancelarEdit.isVisible = false;
        }
    };

    this.controlaAcoesInsElementoContr = function (widget, tipo){
        
        var confirmar = widget.getAction('confirmInsElementoContr');
        var voltar = widget.getAction('VOLTAR');
        if (tipo == 'abre'){
            confirmar.isVisible = true;
            voltar.isVisible = true;
        }
        else {
            confirmar.isVisible = false;
            voltar.isVisible = false;
        }
    }

    this.habilitarAcaoElementos = function (row) {
        return this.contratoEmSituacaoPendente(row) && row.NRVERSAOCONTRATO > 1;
    };

    this.habilitarAcaoElementosEComposicao = function (row) {
        return this.contratoEmSituacaoPendente(row) || (row.IDSITUACAOCONTRATO == 'A' && visibilidadeAcoes && idAlteraContAprv == 'S');
    };

    this.editarDadosComposicaoElementos = function (widget) {
        var campoCentroDeCusto = widget.getField('NMCENTCUST');
        campoCentroDeCusto.readOnly = false;
        General.widgetEdit(widget);
        var save = widget.getAction('save');
        var cancelar = widget.getAction('cancel');
        var saveEdit = widget.getAction('save_comp_serv');
        var cancelarEdit = widget.getAction('cancel_comp_serv');
        try {
            cancelarEdit.isVisible = saveEdit.isVisible = true;
            cancelar.isVisible = save.isVisible = false;
        } catch (e) {
            console.log(e);
        }
    };

    this.cancelarAbaElementos = function (widget) {
        var save = widget.getAction('save');
        var cancelar = widget.getAction('cancel');
        cancelar.isVisible = save.isVisible = true;
        General.widgetCancelForm(widget);
    };

    this.salvarAbaElementos = function (widget) {
        try {
            var row = widget.currentRow;
            var params = {
                "CDCENTCUST": row.CDCENTCUST,
                "NRCOMPSERVICO": row.NRCOMPSERVICO,
                "NRSEQCONTRATO": row.NRSEQCONTRATO,
                "IDELEMENTO": row.IDELEMENTO,
                "TIPO": row.TIPO
            };
            General.callRestEngine('product#/fct_contrato_widget/salvarCentroDeCusto', "Row", params).then(function (resposta) {
                widget.view();
                ScreenService.closeSwipe();
                widget.reload();

                var save = widget.getAction('save');
                var cancelar = widget.getAction('cancel');
                cancelar.isVisible = save.isVisible = true;
            }.bind(this));
        } catch (e) {
            console.log(e);
        }
    };

    //#region UTIL DO CONTRATO

    /**
     * A situacao do contrato é pendente?
     * @params {row - Object}  currentRow do contrato da aba Contrato da tela Lançamento de Contrato
     * return bool
    */
    this.contratoEmSituacaoPendente = function (row) {
        return row.IDSITUACAOCONTRATO === "P";
    };

    //#endregion

    this.editaSeguroVig = function (widget) {
        var wdgContrato = templateManager.container.getWidget("gridContrato");
        var wdgTabbedContrato = templateManager.container.getWidget("wdgTabbedContrato");
        var situacao = wdgContrato.currentRow.IDSITUACAOCONTRATO;
        var dataInicial = widget.getField('DTINISEGURO');
        var dataFinal = widget.getField('DTFINSEGURO');

        /*
        if (situacao == 'A'){
            dataInicial.readOnlyOnEdit = true;
            dataFinal.readOnlyOnEdit = true;
        }*/
        Utilities.formWidgetEdit(widget);
        wdgTabbedContrato.edit();
        this.controlarVisibilidadeWidgetsAoEditarSeguro(true);
    };

    this.adicionaSeguroVig = function (widget) {
        Utilities.formWidgetNewRow(widget);
        var wdgTabbedContrato = templateManager.container.getWidget("wdgTabbedContrato");
        var wdgContrato = templateManager.container.getWidget("gridContrato");
        widget.currentRow.DTINISEGURO = wdgContrato.currentRow.DTINIVIGENCONT;
        widget.currentRow.DTFINSEGURO = wdgContrato.currentRow.DTFINVIGENCONT;
        wdgTabbedContrato.edit();
        this.controlarVisibilidadeWidgetsAoEditarSeguro(false);
        this.calcularPrazoVigSeguro(widget);
        this.calcularDiasVencer(widget);
    };

    this.cancelaSeguroVig = function (widget) {
        var wdgTabbedContrato = templateManager.container.getWidget("wdgTabbedContrato");
        wdgTabbedContrato.list();
        this.controlarVisibilidadeWidgetsAoEditarSeguro(true);
        this.validaIdCertsign();
        Utilities.formWidgetCancel(widget);
    };

    this.salvaSeguroVig = function (widget) {
        if (widget.isValid()) {
            var dataIniSeguro = widget.currentRow.DTINISEGURO;
            var dataFinSeguro = widget.currentRow.DTFINSEGURO;
            var prazo = Utilities.getDiasEntreDatas(dataIniSeguro, dataFinSeguro);

            var nrseguro = widget.currentRow.NRSEGUROCONTRATO;

            if (prazo > 0) {
                if (widget.currentRow.NRSEGUROCONTRATO !== "") {
                    widget.backupRow.DTINCLUSAO = widget.currentRow.DTULTATU;
                    widget.backupRow.CDOPERINCLUSAO = widget.currentRow.CDOPERINCLUSAO;
                    this.saveHistoricoContrato(widget);
                } // EDIT
                var wdgTabbedContrato = templateManager.container.getWidget("wdgTabbedContrato");
                var novoRegistro = widget.currentRow.__is_new;
                widget.currentRow.IDVIGENTE = 'S';

                widget.currentRow.CDOPERINCLUSAO = opLogado;
                Utilities.formWidgetSaveAndSetColumnsToReturn(widget).then(function (resposta) {
                    wdgTabbedContrato.list();
                    this.controlarVisibilidadeWidgetsAoEditarSeguro(true);
                    this.validaIdCertsign();
                    if (novoRegistro) this.enterTabSeguro(widget);
                    widget.activate();
                }.bind(this));
            }
            else {
                ScreenService.showMessage("A data final da vigência do seguro deve ser maior que a data inicial.");
            }
        }
        else {
            ScreenService.showMessage("Preencha todos os campos corretamente!");
        }
    };

    this.saveSeguro = function (widget, prazo) {
        var wdgTabbedContrato = templateManager.container.getWidget("wdgTabbedContrato");
        var novoRegistro = true;
        widget.currentRow.IDVIGENTE = 'S';
        widget.currentRow.NRSEGUROCONTRATO = "";
        Utilities.formWidgetSaveAndSetColumnsToReturn(widget).then(function (resposta) {
            wdgTabbedContrato.list();
            this.controlarVisibilidadeWidgetsAoEditarSeguro(true);
            this.validaIdCertsign();
            if (novoRegistro) this.enterTabSeguro(widget);
            widget.activate();
        }.bind(this));
    };

    this.excluirSeguroVigente = function (widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
            this.saveHistoricoContrato(widget, widget.currentRow);
            widget.removeRow().then(function () {
                this.enterTabSeguro(widget);
            }.bind(this),
                function (error) {
                    if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                        ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
                    }
                    else {
                        Utilities.showErrorMessage(error);
                    }
                });
        }.bind(this));
    };

    this.controlarVisibilidadeWidgetsAoEditarSeguro = function (visivel) {
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        var wdgTabbedContrato = templateManager.container.getWidget("wdgTabbedContrato");
        var wdgTabbedSeguro = templateManager.container.getWidget("wdgTabbedSeguro");

        wdgTabbedPrincipal.widgets.forEach(function (w) { if (w.name != "wdgTabbedContrato") w.enabled = visivel });
        wdgTabbedContrato.widgets.forEach(function (w) { if (w.name != "wdgTabbedSeguro") w.enabled = visivel });
        wdgTabbedSeguro.widgets.forEach(function (w) { if (w.name != "wdgSeguroVigente") w.enabled = visivel });
    };

    this.cancelaWdgPopupRenovarSeguro = function (widget) {
        var wdgAnterior = templateManager.container.getWidget("wdgSeguroVigente");
        ScreenService.closePopup();
        wdgAnterior.activate();
    };

    this.salvaRenovarSeguro = function (wdgPopup) {
        if (wdgPopup.isValid()) {

            var dataIniSeguro = wdgPopup.currentRow.DTINISEGURO;
            var dataFinSeguro = wdgPopup.currentRow.DTFINSEGURO;
            var prazo = Utilities.getDiasEntreDatas(dataIniSeguro, dataFinSeguro);

            if (prazo > 0) {
                var wdgSeguroVig = templateManager.container.getWidget("wdgSeguroVigente");

                wdgSeguroVig.currentRow.changed = true;
                wdgSeguroVig.currentRow.IDVIGENTE = 'N';
                wdgSeguroVig.saveRow()
                    .then(function () {
                        wdgPopup.currentRow.__is_new_code = true;
                        wdgPopup.currentRow.__cast_number = true;
                        wdgPopup.currentRow.__columns_to_return = ["NRSEGUROCONTRATO", "NRORG"];
                        wdgPopup.currentRow.__new_code_pk = "FCT_SEGUROCONTRATO";
                        wdgPopup.currentRow.__table_pk = "NRSEGUROCONTRATO";

                        wdgPopup.currentRow.IDVIGENTE = 'S';
                        return wdgPopup.saveRow();
                    })
                    .then(function () {
                        wdgPopup.isVisible = false;
                        ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
                        ScreenService.closePopup();
                        wdgSeguroVig.activate();
                        this.enterTabSeguro(wdgSeguroVig);
                    }.bind(this))
                    .catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
            }
            else {
                ScreenService.showMessage("A data final da vigência do seguro deve ser maior que a data inicial.");
            }
        }
    };

    this.calcularPrazoVigSeguro = function (widget) {
        var dataIniSeguro = widget.currentRow.DTINISEGURO;
        var dataFinSeguro = widget.currentRow.DTFINSEGURO;

        var prazo = Utilities.getDiasEntreDatas(dataIniSeguro, dataFinSeguro);
        if (prazo === null) {
            widget.currentRow.NRDIASVIGENCIA = prazo;
        }
        else if (prazo >= 0) {
            widget.currentRow.NRDIASVIGENCIA = prazo + 1;
        }
        else {
            ScreenService.showMessage("A data final da vigência do seguro deve ser maior que a data inicial.");
            widget.currentRow.NRDIASVIGENCIA = "prazo inválido";
        }
    };

    this.calcularDiasVencer = function (widget) {
        var diasVencer = null;
        var dataFinSeguro = widget.currentRow.DTFINSEGURO;
        if (dataFinSeguro.length > 0) {
            dataFinSeguro = moment(dataFinSeguro, "DD/MM/YYYY");
            var dataAtual = moment().startOf("day");
            diasVencer = dataFinSeguro.diff(dataAtual, "days");
        }
        widget.currentRow.NRDIASVENCSEGURO = diasVencer;
    };

    this.calcularValorUnitario = function (widget) {
        var row = widget.currentRow;
        row.VRUNITITEMCONT = this.pegarValorUnitario(row);
    };

    this.pegarValorUnitario = function (row) {
        var valor = Utilities.tirarMascaraValores(row.VRTOTITEMCONT) / parseFloat(row.QTITEMCONTRATO);
        if (row.IDMULTMEDVRTOT == "S")
            valor = valor / parseFloat(row.QTMEDIDACONTR);

        return parseFloat(valor.toFixed(8));
    };

    /**
     * Valida se pode editar o campo IDAGRUPPARC do contrato (não pode editar se o contrato estiver em um agrupamento).
     * retorna uma promise.
     */
    this.validarAgrupamentoParcelasParaContratoAgrupado = function (nrseqcontrato, idagrupparc, isNew) {
        var deferred = ZHPromise.defer();
        if (isNew || idagrupparc == '1') {
            deferred.resolve(true);
        }
        else {
            var params = [
                {
                    name: "NRSEQCONTRATO",
                    operator: "=",
                    value: nrseqcontrato
                }
            ];
            General.callRestEngine("product#/crud/fct_relacagrupamentocont_widget", "FilterData", params).then(function (response) {
                if (response.dataset && response.dataset.fct_relacagrupamentocont_widget && response.dataset.fct_relacagrupamentocont_widget.length > 0) {
                    deferred.resolve(false);
                }
                else {
                    deferred.resolve(true);
                }
            })
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                    deferred.reject(error);
                });
        }

        return deferred.promise;
    }

    this.validarAgrupamentoContratos = function (nrseqcontrato) {
        var params = [
            {
                name: "NRSEQCONTRATO",
                operator: "=",
                value: nrseqcontrato
            }
        ];
        return General.callRestEngine("product#/crud/fct_relacagrupamentocont_widget", "FilterData", params);
    }

    this.pegarStatusCarenciaMaxima = function () {
        var deferred = ZHPromise.defer();
        var params = {};

        General.callRestEngine('product#/fct_parametrogeral_widget/pegarIdControleCarencia', "Row", params).then(function (response) {
            var resposta = (response.dataset.request.resulted !== "N");
            deferred.resolve(resposta);
        }).catch(function (e) {
            Utilities.showErrorMessage(e);
            deferred.reject(e);
        });
        return deferred.promise;
    };

    this.validarNegociacaoValorDescontoOperadorEPerfil = function (row) {
        var deferred = ZHPromise.defer();
        this.pegarStatusCarenciaMaxima().then(function (statusCarencia) {
            if (statusCarencia) {
                var params = {
                    "NRNEGOCIACAOCONTRATO": row.NRNEGOCIACAOCONTRATO
                };
                return General.callRestEngine("product#/fct_itemorcamento_widget/validarNegDescOperadorPerfil", "Row", params);
            }
        }).then(function (response) {
            var carencia = response ? response.dataset.request.resulted : response;
            var permitir = true;
            if (carencia) { //* Carencia está ativada?
                // #region SETUP
                var parcelas = parseInt(row.QTMESESDESC);
                var percentual = parseFloat(row.VRPERCDESCITEM);
                // SETAR OPERADOR
                var operador = (!carencia.CARENCIAMAXOPERADOR[0]) ? null : {
                    "PARCELAS": carencia.CARENCIAMAXOPERADOR[0].NRPARCELASMAX,
                    "PERCENTUAL": carencia.CARENCIAMAXOPERADOR[0].VRPERCMAXATE
                };
                // SETAR FAIXA OPERADOR
                var faixaOperador = null;
                var qtdMaximaOperador; // maximo de parcelas
                if (carencia.CARENCIA) {
                    var faixaOp = carencia.CARENCIA.filter(function (fop) {
                        return fop.IDENTIFICADOR === "OPERADOR";
                    });
                    qtdMaximaOperador = (!faixaOp[0]) ? 0 : faixaOp[0].MAXIMO;
                    faixaOperador = (!faixaOp) ? null : this.extrairFaixaDeDescontoPorPeriodo(faixaOp, parcelas, percentual);
                }
                // SETAR PERFIL ACESSO
                var perfilAcesso = (!carencia.CARENCIAMAXPERFIL[0]) ? null : {
                    "PARCELAS": carencia.CARENCIAMAXPERFIL[0].NRPARCELASMAX,
                    "PERCENTUAL": carencia.CARENCIAMAXPERFIL[0].VRPERCMAXATE
                };
                // SETAR FAIXA PERFIL ACESSO
                var faixaPerfil = null;
                var qtdMaximaPerfil; // quantidade maxima por perfil
                if (carencia.CARENCIA) {
                    var faixaPa = carencia.CARENCIA.filter(function (fpa) {
                        return fpa.IDENTIFICADOR === "PERFIL";
                    });
                    qtdMaximaPerfil = (!faixaPa[0]) ? 0 : faixaPa[0].MAXIMO;
                    faixaPerfil = (!faixaPa) ? null : this.extrairFaixaDeDescontoPorPeriodo(faixaPa, parcelas, percentual);
                }
                // APLICAÇÂO DAS REGRAS
                var permitido = this.validarRegrasDesconto(faixaOperador, operador, faixaPerfil, perfilAcesso, parcelas, percentual);
                if (permitido.status) {
                    //validar a quantidade de descontos dadas por este operador ou perfil
                    if (permitido.identificador === "operador" && carencia.OPERADOR[0]) {
                        var qtdParcelasMax = (edit) ? parseInt(carencia.OPERADOR[0].QTPARCELAS - templateManager.container.getWidget("wdgDescontoPorPeriodo").backupRow.QTMESESDESC) : parseInt(carencia.OPERADOR[0].QTPARCELAS);
                        if (edit) {
                            if (permitido.faixa) {
                                permitido = this.validarQuantidadePermitidaFaixaOperador(qtdParcelasMax, parcelas, faixaOperador, edit);
                            } else if (!permitido.faixa) {
                                permitido = this.validarQuantidadePermitidaOperador(qtdParcelasMax, parcelas, operador, edit);
                            }
                        } else {
                            if (permitido.faixa) {
                                permitido = this.validarQuantidadePermitidaFaixaOperador(qtdParcelasMax, parcelas, faixaOperador);
                            } else if (!permitido.faixa) {
                                permitido = this.validarQuantidadePermitidaOperador(qtdParcelasMax, parcelas, operador);
                            }
                        }
                    } else if (permitido.identificador === "perfil" && carencia.PERFIL[0]) {
                        var qtdParcelasMax = (edit) ? parseInt(carencia.PERFIL[0].QTPARCELAS - templateManager.container.getWidget("wdgDescontoPorPeriodo").backupRow.QTMESESDESC) : parseInt(carencia.PERFIL[0].QTPARCELAS);
                        if (edit) {
                            if (permitido.faixa) {
                                permitido = this.validarQuantidadePermitidaFaixaPerfil(qtdParcelasMax, parcelas, faixaPerfil, edit);
                            } else if (!permitido.faixa) {
                                permitido = this.validarQuantidadePermitidaPerfil(qtdParcelasMax, parcelas, perfilAcesso, edit);
                            }
                        } else {
                            if (permitido.faixa) {
                                permitido = this.validarQuantidadePermitidaFaixaPerfil(qtdParcelasMax, parcelas, faixaPerfil);
                            } else if (!permitido.faixa) {
                                permitido = this.validarQuantidadePermitidaPerfil(qtdParcelasMax, parcelas, perfilAcesso);
                            }
                        }
                    }
                }

                permitir = permitido.status;

                if (!permitido.status)
                    ScreenService.showMessage(ScreenService.i18n('Operação Bloqueada! <br/><br/> ' + permitido.mensagem));
            }
            deferred.resolve(permitir);
        }.bind(this)).catch(function (e) {
            Utilities.showErrorMessage(e);
            deferred.reject(e);
        });
        return deferred.promise;
    };

    this.validarObj = function (obj, parcelas, percentual) {
        var primeira = false;
        var permitir = obj.PERCENTUAL >= percentual;
        if (!permitir)
            primeira = true;
        else
            permitir = obj.PARCELAS >= parcelas;
        return { "STATUS": permitir, "PRIMEIRA": primeira }
    };

    this.validarQuantidadePermitidaOperador = function (qtParcelas, parcelas, operador) {
        var permitir = (qtParcelas + parcelas) <= operador.PARCELAS;
        var permitido = {
            "status": permitir,
            "mensagem": (!permitir) ? 'O operador logado não tem permissão para dar desconto com quantidade maior que ' + operador.PARCELAS + 'parcelas.' : "",
            "identificador": "operador",
            "faixa": false
        };
        return permitido;
    };

    this.validarQuantidadePermitidaFaixaOperador = function (qtParcelas, parcelas, faixaOperador) {
        var permitir = (qtParcelas + parcelas) <= faixaOperador.PARCELAS;
        var permitido = {
            "status": permitir,
            "mensagem": (!permitir) ? 'O operador logado não tem permissão para dar desconto com quantidade maior que ' + faixaOperador.PARCELAS + 'parcelas.' : "",
            "identificador": "operador",
            "faixa": false
        };
        return permitido;
    };

    this.validarQuantidadePermitidaPerfil = function (qtParcelas, parcelas, perfil) {
        var permitir = (qtParcelas + parcelas) <= perfil.PARCELAS;
        var permitido = {
            "status": permitir,
            "mensagem": (!permitir) ? 'O perfil de acesso do operador logado não tem permissão para dar desconto com quantidade maior que ' + perfil.PARCELAS + 'parcelas.' : "",
            "identificador": "perfil",
            "faixa": false
        };
        return permitido;
    };

    this.validarQuantidadePermitidaFaixaPerfil = function (qtParcelas, parcelas, faixaPerfil) {
        var permitir = (qtParcelas + parcelas) <= faixaPerfil.PARCELAS;
        var permitido = {
            "status": permitir,
            "mensagem": (!permitir) ? 'O perfil de acesso do operador logado não tem permissão para dar desconto com quantidade maior que ' + faixaPerfil.PARCELAS + 'parcelas.' : "",
            "identificador": "perfil",
            "faixa": true
        };
        return permitido;
    };

    this.pegarRecebidoNulo = function () {
        return {
            "status": false,
            "mensagem": "",
            "identificador": "",
            "faixa": false
        };
    };

    this.validarRegrasDesconto = function (faixaOperador, operador, faixaPerfil, perfilAcesso, parcelas, percentual) {
        /*REGRAS
            Na presença de um OPERADOR DEVE IGNORAR O PERFIL DE ACESSO
            Na presença de uma FAIXA DEVE IGNORAR O MAXIMO PARAMETRIZADO ( PERFIL && OPERADOR)
            1. FAIXA OPERADOR
                1. ANALISAR SOMENTE A FAIXA
            2. OPERADOR 
                1. ANALISAR SOMENTE O OPERADOR
            3. FAIXA PERFIL ACESSO 
                1. ANALISAR FAIXA
                2. ANALISAR MAXIMO PARAMETRIZADO
            4. MAXIMO PARAMETRIZADO
                1. ANALISAR MAXIMO PARAMETRIZADO
            5.  Ao inserir deve se pegar a quantidade da soma que existe na tabela(operador e perfil)
                1.  o da faixa é o valor maximo de parcelas da faixa
            6.  Ao editar deve se pegar todas menos a que esta sendo editada
                1.  o valor da faixa é o valor maximo de parcelas da faixa
        */
        var permitido = this.pegarRecebidoNulo();
        if (faixaOperador) {
            permitido = this.validarFaixaOperador(faixaOperador, parcelas, percentual);
        } else if (operador) {
            permitido = this.validarOperador(operador, parcelas, percentual);
        } else if (faixaPerfil) {
            permitido = this.validarFaixaPerfilAcesso(faixaPerfil, parcelas, percentual) || this.validarPerfilAcesso(perfilAcesso, parcelas, percentual);
        } else if (perfilAcesso) {
            permitido = this.validarPerfilAcesso(perfilAcesso, parcelas, percentual);
        }
        return permitido;
    };

    this.validarFaixaOperador = function (faixaOperador, parcelas, percentual) {
        var permitir = this.validarObj(faixaOperador, parcelas, percentual);
        return {
            "status": permitir.STATUS,
            "mensagem": (!permitir.STATUS) ? (!permitir.PRIMEIRA) ? 'O operador logado não tem permissão para dar desconto com quantidade maior que ' + faixaOperador.PARCELAS + 'parcelas.' : 'O operador logado não tem permissão para dar desconto maior que ' + faixaOperador.PERCENTUAL + '%' : "",
            "identificador": "operador",
            "faixa": true
        }
    };

    this.validarFaixaPerfilAcesso = function (faixaPerfil, parcelas, percentual) {
        var permitir = this.validarObj(faixaPerfil, parcelas, percentual);
        return {
            "status": permitir.STATUS,
            "mensagem": (!permitir.STATUS) ? (!permitir.PRIMEIRA) ? 'O perfil de acesso do operador logado não tem permissão para dar desconto com quantidade maior que ' + faixaPerfil.PARCELAS + 'parcelas.' : 'O perfil de acesso do operador logado não tem permissão para dar desconto maior que ' + faixaPerfil.PERCENTUAL + '%' : "",
            "identificador": "perfil",
            "faixa": true
        }
    };

    this.validarOperador = function (operador, parcelas, percentual) {
        var permitir = this.validarObj(operador, parcelas, percentual);
        return {
            "status": permitir.STATUS,
            "mensagem": (!permitir.STATUS) ? (!permitir.PRIMEIRA) ? 'O operador logado não tem permissão para dar desconto com quantidade maior que ' + operador.PARCELAS + 'parcelas.' : 'O operador logado não tem permissão para dar desconto maior que ' + operador.PERCENTUAL + '%' : "",
            "identificador": "operador",
            "faixa": false
        }
    };

    this.validarPerfilAcesso = function (perfil, parcelas, percentual) {
        var permitir = this.validarObj(perfil, parcelas, percentual);
        return {
            "status": permitir.STATUS,
            "mensagem": (!permitir.STATUS) ? (!permitir.PRIMEIRA) ? 'O perfil de acesso do operador logado não tem permissão para dar desconto com quantidade maior que ' + perfil.PARCELAS + 'parcelas.' : 'O perfil de acesso do operador logado não tem permissão para dar desconto maior que ' + perfil.PERCENTUAL + '%' : "",
            "identificador": "perfil",
            "faixa": false
        }
    };

    this.extrairFaixaDeDescontoPorPeriodo = function (dados, parcela, valor) {
        var obj = null;
        if (dados.length > 0) {
            dados.forEach(function (el) {
                if (parseFloat(el.VRPERCDESCATE) >= valor && valor >= parseFloat(el.VRPERCDESCDE) && parseInt(el.NRPARCELASMAX) > parcela)
                    obj = {
                        "PARCELAS": parseInt(el.NRPARCELASMAX),
                        "PERCENTUAL": parseFloat(el.VRPERCDESCATE)
                    };
            });
        }
        return obj;
    };

    /**
     * 
     * Chamado somente na ação de aprovar um único contrato.
     * Essa função está sendo depreciada pois foi incorporada a rota gerarParcelasContratoNovoFat
     * @deprecated
     * @params {row} - Row dos dados do contrato
     * @returns {Promise} - chamada para a rota que faz a validação dos dados
    */
    this.gerarContratoFaturamento = function (row) {
        var params = {
            "NRPARCNEGOCIO": row.NRPARCNEGOCIO,
            "NRSEQCONTRATO": row.NRSEQCONTRATO
            //colocar aqui os params para a chamada da função no backend
        };
        return General.callRestEngine("product#/fct_contrato_widget/gerarContratoFaturamento", "Row", params);
    };

    this.saveHistoricoContrato = function (widget, row) {
        if (row === undefined) {
            row = null;
        }
        this.inserirSeguroContrato((row) ? row : widget.backupRow, 'N');
    };

    this.saveSeguro = function (widget) {
        this.inserirSeguroContrato(widget.currentRow, 'S');
    };

    this.inserirSeguroContrato = function (row, idVigente) {
        if (idVigente === undefined) {
            idVigente = 'S';
        }

        var params = {
            "NRSEQCONTRATO": row.NRSEQCONTRATO,
            "NRAPOLICE": row.NRAPOLICE,
            "DSMODALIDADE": row.DSMODALIDADE,
            "DTINISEGURO": row.DTINISEGURO,
            "DTFINSEGURO": row.DTFINSEGURO,
            "VRTAXASEGURO": row.VRTAXASEGURO,
            "VRAPOLICE": row.VRAPOLICE,
            "NRPARCCORRETORA": row.NRPARCCORRETORA,
            "NRPARCSEGURADORA": row.NRPARCSEGURADORA,
            "IDVIGENTE": idVigente,
            "DTINCLUSAO": row.DTINCLUSAO,
            "CDOPERINCLUSAO": row.CDOPERINCLUSAO
        };
        General.callRestEngine("product#/fct_contrato_widget/inserirSeguroContrato", "Row", params).then(function (response) {
            ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };

    this.gerarDocumentoRescisao = function (widget) {
        if (widget.currentRow.IDSITUACAOCONTRATO == 'A' && widget.currentRow.IDSITUACAORESCISAO == 'S') {
            var params = {
                "NRTIPORESCISAO": widget.currentRow.NRTIPORESCISAO
            };
            General.callRestEngine("product#/fct_contrato_widget/getParametroExtensaoDocEncerra", "Row", params).then(function (response) {
                if (response.dataset.IDPERMITEDOCX[0].NRSEQMODDOCFCT != null) {
                    if (response.dataset.IDPERMITEDOCX[0].IDPERMITEDOCX == 'S') {
                        ScreenService.showCustomDialog("O documento de rescisão do contrato pode ser gerado como PDF ou como DOCX.<br>Como deseja gerá-lo?", 'CONFIRMATION', null, [
                            {
                                "label": "Gerar PDF",
                                "code": function () {
                                    this.gerarDocumentoParaUmContrato(widget, 'P');
                                }.bind(this)
                            },
                            {
                                "label": "Gerar DOCX",
                                "code": function () {
                                    this.gerarDocumentoParaUmContrato(widget, 'D');
                                }.bind(this)
                            },
                            {
                                "label": "Cancelar",
                                "code": function () {

                                }.bind(this)
                            }
                        ]);
                    }
                    else {
                        this.gerarDocumentoParaUmContrato(widget, 'P');
                    }
                } else {
                    ScreenService.showMessage(ScreenService.i18n('Operação bloqueada! <br><br>Não há um documento relacionado ao Tipo de Rescisão informado neste contrato.'));
                }
            }.bind(this))
                .catch(function (error) {
                    deferred.reject(error);
                }.bind(this));
        } else {
            ScreenService.showMessage(ScreenService.i18n("Este contrato não pode ter seu documento gerado. Somente contratos com a situação \"Aprovada\" e rescisão solicitada podem ter seu documento gerado."));
        }
    };

    this.confirmarRescisao = function (widget) {
        ScreenService.confirmMessage(ScreenService.i18n("Deseja deseja realmente confirmar a rescisão deste contrato?"), "question", function () {
            var contrato = templateManager.container.getWidget("gridContrato");
            if (widget.currentRow.IDSITUACAOCONTRATO == 'A' && widget.currentRow.IDSITUACAORESCISAO == 'S') {
                var row = widget.currentRow;
                var params = {
                    "NRSEQCONTRATO": row.NRSEQCONTRATO,
                    "IDSITUACAOCONTRATO": row.IDSITUACAOCONTRATO,
                    "DTRECISAOCONTR": row.DTRECISAOCONTR,
                    "IDTPEXCLUSAOPARCELA": (row.IDTPEXCLUSAOPARCELA) ? (row.IDTPEXCLUSAOPARCELA) : null,
                    "NRTIPORESCISAO": row.NRTIPORESCISAO,
                    "IDDOCX": null,
                    "DTPEDRESCISAOCONTR": row.DTPEDRESCISAOCONTR,
                    "NRMOTIVORESCISAO": row.NRMOTIVORESCISAO,
                    "DSJUSTIFICAENCERRA": row.DSJUSTIFICAENCERRA,
                    "VRPENDENCIAFIN": row.VRPENDENCIAFIN
                };
                var timeout = 5 * 60000; //5 min
                Utilities.callRestEngineWithTimeout("product#/fct_contrato_widget/encerraContrato", "Row", params, timeout).then(function (response) {
                    this.atualizarContratoEFecharPopup(contrato);
                    ScreenService.showMessage("Operação realizada com sucesso.");
                }.bind(this))
                    .catch(function (e) {
                        Utilities.showErrorMessage(e);
                    });
            }
            else {
                ScreenService.showMessage(ScreenService.i18n("Este contrato não pode ter sua rescisão confirmada. Somente contratos com a situação \"Aprovada\" e rescisão solicitada podem ter sua rescisão confirmada."));
            }
        }.bind(this));
    };

    this.gerarTodosOsDocumentos = function (params, mensagem) {
        if (mensagem === undefined) {
            mensagem = "MENSAGEM NÃO FOI PASSADA";
        }
        var timeout = 5 * 60000;
        Utilities.callRestEngineWithTimeout("product#/fct_contrato_widget/gerarDocumentoContratoRescisao", "Row", params, timeout).then(function (success) {
            var links = success.dataset.documentos.links;
            links.forEach(function (link) {
                if (link)
                    window.open(window.backendIndexUrl + link);
            });
            ScreenService.showMessage(mensagem);
            // ScreenService.closePopup();
            var wdgContratroGrid = templateManager.container.getWidget('wdgContratroGrid');
            //wdgContratroGrid.dataSource.clearCheckedRows();
            //wdgContratroGrid.reload();
        })
            .catch(function (error) {
                console.error(error);
                var msg = 'Timeout <br><br> ' + (error.status !== 0 && error.data) ? error.data.error : "";
                ScreenService.showMessage(msg);
            });
    };

    this.gerarDocumentosParaMultiplosContratos = function (widget, idDocX, contratos) {
        if (contratos === undefined) {
            contratos = null;
        }

        if (!contratos)
            contratos = widget.getCheckedRows();
        var params = {
            "CONTRATOS": contratos,
            "NRPARCNEGOCIO": null,
            "NRSEQCONTRATO": null,
            "NRTIPORESCISAO": contratos[0].NRTIPORESCISAO,
            "IDDOCX": idDocX
        };
        this.gerarTodosOsDocumentos(params, 'Os documentos dos contratos foram gerados.');
    };

    this.gerarDocumentoParaUmContrato = function (widget, idDocX) {
        var row = widget.currentRow;
        var params = {
            "CONTRATOS": null,
            "NRPARCNEGOCIO": row.NRPARCNEGOCIO,
            "NRSEQCONTRATO": row.NRSEQCONTRATO,
            "NRTIPORESCISAO": row.NRTIPORESCISAO,
            "IDDOCX": idDocX
        };
        this.gerarTodosOsDocumentos(params, "O documento do contrato foi gerado.");
    };

    this.validarGeracaoDocumentosRescisaoMultiplos = function (contratos) {
        var resposta = {};
        resposta.CONTRATOS = contratos.filter(function (c) {
            return c.IDSITUACAOCONTRATO == 'A' && c.IDSITUACAORESCISAO == 'S';
        });

        resposta.STATUS = resposta.CONTRATOS.length > 0;
        if (!resposta.STATUS) {
            resposta.MENSAGEM = "Operação bloqueada! <br><br>Somente contratos com rescisão solicitada podem ter seus documentos gerados.";
        }
        else {
            resposta.MENSAGEM = "Deseja realmente gerar documento de rescisão dos contratos?";
            var obs1 = false;
            if (resposta.CONTRATOS.length != contratos.length) {
                resposta.MENSAGEM += "<br><br>OBS: Alguns dos contratos selecionados não estão com a situação \"Aprovado\" ou situação \"Encerrado\" e com a rescisão solicitada. Estes contratos serão desconsiderados.";
                obs1 = true;
            }

            var contratosPorCliente = {};
            resposta.CONTRATOS.forEach(function (c) {
                if (contratosPorCliente.hasOwnProperty(c.NRPARCNEGOCIO))
                    contratosPorCliente[c.NRPARCNEGOCIO].push(c);
                else
                    contratosPorCliente[c.NRPARCNEGOCIO] = [c];
            });

            var contratosTipoDiferenteParceiros = [];
            var contratosConsideradosParceiros = [];

            Object.keys(contratosPorCliente).forEach(function (parc) {
                var tipoDiferente = contratosPorCliente[parc].filter(function (c) {
                    return (c.NRTIPORESCISAO != contratosPorCliente[parc][0].NRTIPORESCISAO ||
                        c.DTPEDRESCISAOCONTR != contratosPorCliente[parc][0].DTPEDRESCISAOCONTR ||
                        c.DTRECISAOCONTR != contratosPorCliente[parc][0].DTRECISAOCONTR ||
                        c.NRMOTIVORESCISAO != contratosPorCliente[parc][0].NRMOTIVORESCISAO ||
                        c.VRPENDENCIAFIN != contratosPorCliente[parc][0].VRPENDENCIAFIN ||
                        c.DSJUSTIFICAENCERRA != contratosPorCliente[parc][0].DSJUSTIFICAENCERRA);
                });

                var contratosConsiderados = contratosPorCliente[parc].filter(function (c) {
                    return (c.NRTIPORESCISAO == contratosPorCliente[parc][0].NRTIPORESCISAO &&
                        c.DTPEDRESCISAOCONTR == contratosPorCliente[parc][0].DTPEDRESCISAOCONTR &&
                        c.DTRECISAOCONTR == contratosPorCliente[parc][0].DTRECISAOCONTR &&
                        c.NRMOTIVORESCISAO == contratosPorCliente[parc][0].NRMOTIVORESCISAO &&
                        c.VRPENDENCIAFIN == contratosPorCliente[parc][0].VRPENDENCIAFIN &&
                        c.DSJUSTIFICAENCERRA == contratosPorCliente[parc][0].DSJUSTIFICAENCERRA);
                });

                contratosTipoDiferenteParceiros = contratosTipoDiferenteParceiros.concat(tipoDiferente);
                contratosConsideradosParceiros = contratosConsideradosParceiros.concat(contratosConsiderados);
            });

            if (contratosTipoDiferenteParceiros.length > 0) {
                var msgObs2 = obs1 ? "<br><br>OBS 2: " : "<br><br>OBS: ";
                resposta.MENSAGEM += msgObs2 + "Só é possível gerar documento agrupado para vários contratos de um mesmo cliente se eles possuem os mesmos dados de rescisão. Os contratos abaixo serão desconsiderados pois possuem dados de rescisão diferentes:<br>" + contratosTipoDiferenteParceiros.map(function (c) {
                    return c.NRCONTRATOFCT;
                }).join("<br>");

                resposta.CONTRATOS = contratosConsideradosParceiros;
            }
        }

        return resposta;
    };

    this.gerarDocumentoRescisaoMultiplos = function (widget) {
        var resposta = this.validarGeracaoDocumentosRescisaoMultiplos(widget.getCheckedRows());
        if (resposta.STATUS) {

            ScreenService.confirmMessage(ScreenService.i18n(resposta.MENSAGEM), 'question', function (success) {
                // se os tipos diferente aparecer uma mensagem confirmando o usuario
                var params = {
                    "NRTIPORESCISAO": resposta.CONTRATOS[0].NRTIPORESCISAO
                };
                General.callRestEngine("product#/fct_contrato_widget/getParametroExtensaoDocEncerra", "Row", params).then(function (response) {
                    if (response.dataset.IDPERMITEDOCX[0].NRSEQMODDOCFCT != null) {
                        if (response.dataset.IDPERMITEDOCX[0].IDPERMITEDOCX == 'S') {
                            ScreenService.showCustomDialog("Os documentos de resciões dos contratos podem ser gerados como PDF ou como DOCX.<br>Como deseja gerá-los?", 'CONFIRMATION', null, [
                                {
                                    "label": "Gerar PDF",
                                    "code": function () {
                                        this.gerarDocumentosParaMultiplosContratos(widget, 'P', resposta.CONTRATOS);
                                    }.bind(this)
                                },
                                {
                                    "label": "Gerar DOCX",
                                    "code": function () {
                                        this.gerarDocumentosParaMultiplosContratos(widget, 'D', resposta.CONTRATOS);
                                    }.bind(this)
                                },
                                {
                                    "label": "Cancelar",
                                    "code": function () {
                                    }.bind(this)
                                }
                            ]);
                        }
                        else {
                            this.gerarDocumentosParaMultiplosContratos(widget, 'P', resposta.CONTRATOS);
                        }
                    } else {
                        ScreenService.showMessage(ScreenService.i18n('Operação bloqueada! <br><br>Não há um documento relacionado ao Tipo de Rescisão informado neste contrato.'));
                    }

                }.bind(this))
                    .catch(function (e) {
                        Utilities.showErrorMessage(e);
                    });
            }.bind(this));
        } else {
            ScreenService.showMessage(ScreenService.i18n(resposta.MENSAGEM));
        }
    };

    this.validarConfirmacaoRescisaoMultiplos = function (contratos) {
        var resposta = {};
        resposta.STATUS = true;

        resposta.CONTRATOS = contratos.filter(function (c) {
            return c.IDSITUACAOCONTRATO == 'A' && c.IDSITUACAORESCISAO == 'S';
        });

        var msg = 'Deseja realmente confirmar a rescisão do(s) contrato(s)?';

        if (contratos.length == resposta.CONTRATOS.length) {
            resposta.MENSAGEM = msg;
        }
        else if (resposta.CONTRATOS.length > 0) {
            resposta.MENSAGEM = msg + '<br><br>OBS: Alguns dos contratos selecionados não estão com a rescisão solicitada. Estes contratos serão desconsiderados.';
        }
        else {
            resposta.STATUS = false;
            resposta.MENSAGEM = 'Operação bloqueada! <br><br>Nenhum dos contratos selecionados está com a rescisão solicitada.';
        }

        return resposta;
    };

    this.confirmarRescisaoMultiplos = function (widget) {
        var resposta = this.validarConfirmacaoRescisaoMultiplos(widget.getCheckedRows());
        if (resposta.STATUS) {
            ScreenService.confirmMessage(ScreenService.i18n(resposta.MENSAGEM), 'question', function (success) {
                var contratos = resposta.CONTRATOS;
                var params = {
                    "DADOSRESCISAO": contratos[0],
                    "CONTRATOS": contratos,
                    "IDDOCX": null
                };
                var timeout = 5 * 60000; //5 min
                Utilities.callRestEngineWithTimeout("product#/fct_contrato_widget/encerrarContratoEmMassa", "Row", params, timeout).then(function (response) {
                    ScreenService.showMessage(ScreenService.i18n('Operação realizada com sucesso.'));
                    ScreenService.closePopup();
                    var wdgContratroGrid = templateManager.container.getWidget("wdgContratroGrid");
                    wdgContratroGrid.dataSource.clearCheckedRows();
                    wdgContratroGrid.reload();
                }.bind(this))
                    .catch(function (e) {
                        Utilities.showErrorMessage(e);
                    });
            }.bind(this));
        } else {
            ScreenService.showMessage(ScreenService.i18n(resposta.MENSAGEM));
        }
    };

    this.cancelarRescisao = function (widget) {
        var contrato = templateManager.container.getWidget("gridContrato");

        if (widget.currentRow.IDSITUACAOCONTRATO == 'A' && widget.currentRow.IDSITUACAORESCISAO == 'S') {
            var params = {
                "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO
            };
            General.callRestEngine('product#/fct_contrato/removerRescisao', "Row", params).then(function (response) {
                ScreenService.showMessage(ScreenService.i18n('Operação realizada com sucesso.'));
                self.atualizarContratoEFecharPopup(contrato);
            }).catch(function (e) {
                Utilities.showErrorMessage(e);
            });
        } else {
            ScreenService.showMessage(ScreenService.i18n("Este contrato não pode ter o cancelamento de sua rescisão efetivada. Somente contratos com situação \"Aprovada\" e rescisão solicitada podem ser cancelados."));
        }
    };

    this.validarCancelamentoDeSolicitaoRescisaoMultiplos = function (contratos) {
        var resposta = {};
        resposta.STATUS = true;
        resposta.CONTRATOS = contratos.filter(function (c) {
            return c.IDSITUACAOCONTRATO == 'A' && c.IDSITUACAORESCISAO == 'S';
        });

        var msg = 'Deseja realmente cancelar a solicitação da rescisão do(s) contrato(s)?';

        if (contratos.length == resposta.CONTRATOS.length) {
            resposta.MENSAGEM = msg;
        }
        else if (resposta.CONTRATOS.length > 0) {
            resposta.MENSAGEM = msg + '<br><br>OBS: Alguns dos contratos selecionados não estão com a situação "Aprovado" ou com a rescisão solicitada. Estes contratos serão desconsiderados.';
        }
        else {
            resposta.STATUS = false;
            resposta.MENSAGEM = 'Operação bloqueada! <br><br>Nenhum dos contratos selecionados está com a rescisão solicitada.';
        }

        return resposta;
    };

    this.cancelarSolicitacaoRescisaoMultiplos = function (widget) {
        var resposta = this.validarCancelamentoDeSolicitaoRescisaoMultiplos(widget.getCheckedRows());
        if (resposta.STATUS) {
            ScreenService.confirmMessage(ScreenService.i18n(resposta.MENSAGEM), 'question', function (success) {
                var params = {
                    "CONTRATOS": this.pegarNumeroDeMultiplosContratos(null, resposta.CONTRATOS)
                };
                General.callRestEngine('product#/fct_contrato/removerRescisaoMultiplos', "Row", params).then(function (response) {
                    ScreenService.closePopup();
                    ScreenService.showMessage(ScreenService.i18n('Operação realizada com sucesso.'));

                    var wdgContratroGrid = templateManager.container.getWidget("wdgContratroGrid");
                    wdgContratroGrid.reload();
                    templateManager.updateTemplate();
                }).catch(function (e) {
                    Utilities.showErrorMessage(e);
                });
            }.bind(this));
        } else {
            ScreenService.showMessage(ScreenService.i18n(resposta.MENSAGEM));
        }
    };

    this.solicitarRescisaoAmbas = function (widget) {
        if (widget.currentRow.IDRESCICAOEMMASSA == 'S')
            this.solicitacaoRescisaoMultiplos(widget);
        else
            this.solicitarRescisao(widget);
    };

    this.solicitarRescisao = function (widget) {
        var contrato = templateManager.container.getWidget("gridContrato");
        if (widget.isValid() && contrato.currentRow.IDSITUACAOCONTRATO == 'A' && contrato.currentRow.IDSITUACAORESCISAO != 'S') {
            var idtpexclusaoparcela = widget.currentRow.IDTPEXCLUSAOPARCELA;
            var dtrecisao = widget.currentRow.DTRECISAOCONTR;
            var nrtiporescisao = widget.currentRow.NRTIPORESCISAO;
            var dtpedrescisaocontr = widget.currentRow.DTPEDRESCISAOCONTR;
            var nrmotivorescisao = widget.currentRow.NRMOTIVORESCISAO;
            var dsjustificaencerra = widget.currentRow.DSJUSTIFICAENCERRA;
            var vrpendenciafin = widget.currentRow.VRPENDENCIAFIN;
            var idsituacao = contrato.currentRow.IDSITUACAOCONTRATO;
            var params = {
                "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                "IDSITUACAOCONTRATO": idsituacao,
                "IDDOCX": null,
                "DTRECISAOCONTR": dtrecisao,
                "NRTIPORESCISAO": nrtiporescisao,
                "IDTPEXCLUSAOPARCELA": idtpexclusaoparcela,
                "DTPEDRESCISAOCONTR": dtpedrescisaocontr,
                "NRMOTIVORESCISAO": nrmotivorescisao,
                "DSJUSTIFICAENCERRA": dsjustificaencerra,
                "VRPENDENCIAFIN": vrpendenciafin
            };

            General.callRestEngine('product#/fct_contrato/solicitarRescisao', "Row", params).then(function (response) {
                ScreenService.showMessage(ScreenService.i18n('Operação realizada com sucesso.'));
                this.atualizarContratoEFecharPopup(contrato);
            }.bind(this)).catch(function (e) {
                Utilities.showErrorMessage(e);
            });
        }
    };

    this.atualizarContratoEFecharPopup = function (contrato) {
        contrato.list();
        contrato.reload().then(function () {
            contrato.view();
            contrato.currentRow = contrato.dataSource.data[0];
            this.controlaVisibilidade(contrato);
            ScreenService.closePopup();
        }.bind(this)).catch(function (e) {
            Utilities.showErrorMessage(e);
        });
    };

    this.validarSolicitacaoMultiplosContratos = function (contratos) {
        var resposta = {};
        resposta.STATUS = true;

        resposta.CONTRATOS = contratos.filter(function (c) {
            return c.IDSITUACAOCONTRATO == 'A' && Util.isEmptyOrBlank(c.IDSITUACAORESCISAO);
        });

        var msg = 'Deseja realmente solicitar a rescisão do(s) contrato(s)?';

        if (contratos.length == resposta.CONTRATOS.length) {
            resposta.MENSAGEM = msg;
        }
        else if (resposta.CONTRATOS.length > 0) {
            resposta.MENSAGEM = msg + '<br><br>OBS: Alguns dos contratos selecionados não estão com a situação "Aprovado" ou já tiveram sua rescisão solicitada. Estes contratos serão desconsiderados.';
        }
        else {
            resposta.STATUS = false;
            resposta.MENSAGEM = 'Operação bloqueada! <br><br>Os contratos selecionados ainda não foram aprovados ou já estão com a rescisão solicitada.';
        }

        return resposta;
    }

    this.solicitacaoRescisaoMultiplos = function (widget) {
        var idtpexclusaoparcela = widget.currentRow.IDTPEXCLUSAOPARCELA;
        var dtrecisao = widget.currentRow.DTRECISAOCONTR;
        var nrtiporescisao = widget.currentRow.NRTIPORESCISAO;
        var dtpedrescisaocontr = widget.currentRow.DTPEDRESCISAOCONTR;
        var nrmotivorescisao = widget.currentRow.NRMOTIVORESCISAO;
        var dsjustificaencerra = widget.currentRow.DSJUSTIFICAENCERRA;
        var vrpendenciafin = widget.currentRow.VRPENDENCIAFIN;
        var wdgContratos = templateManager.container.getWidget("wdgContratroGrid");
        var contratos = wdgContratos.getCheckedRows().filter(function (c) {
            return c.IDSITUACAORESCISAO != 'S' && c.IDSITUACAOCONTRATO == 'A';
        });
        if (contratos.length > 0) {
            var params = {
                "NRSEQCONTRATO": contratos[0].NRSEQCONTRATO,
                "IDSITUACAOCONTRATO": 'A',
                "IDDOCX": null,
                "DTRECISAOCONTR": dtrecisao,
                "NRTIPORESCISAO": nrtiporescisao,
                "IDTPEXCLUSAOPARCELA": idtpexclusaoparcela,
                "DTPEDRESCISAOCONTR": dtpedrescisaocontr,
                "NRMOTIVORESCISAO": nrmotivorescisao,
                "DSJUSTIFICAENCERRA": dsjustificaencerra,
                "VRPENDENCIAFIN": vrpendenciafin,
                "CONTRATOS": this.pegarNumeroDeMultiplosContratos(wdgContratos, contratos)
            };

            General.callRestEngine('product#/fct_contrato/solicitarRescisaoMultiplos', "Row", params).then(function (response) {
                wdgContratos.reload();
                ScreenService.closePopup();
                ScreenService.showMessage(ScreenService.i18n('Operação realizada com sucesso.'));
            }).catch(function (e) {
                Utilities.showErrorMessage(e);
            });
        } else {
            ScreenService.showMessage(ScreenService.i18n('Nenhum dos contratos selecionados podem ter sua rescisão solicitada.'));
        }
    };


    /**
     * Pega os numeros de um monte de contratos selecionados e os retorna da seguinte forma, (6337,6338).
     * Prontos para ser utilizado em uma consulta que usa IN.
     * @params widget - Widget selecionada para efetuar alguma ação em Massa.
    */

    this.pegarNumeroDeMultiplosContratos = function (widget, contratos) {
        if (contratos === undefined) {
            contratos = null;
        }

        if (!contratos)
            contratos = widget.getCheckedRows();
        var numeros = "";
        if (contratos.length > 1) {
            contratos.forEach(function (el, indx) {
                numeros += el.NRSEQCONTRATO + ",";
            });
            return numeros.slice(0, -1);
        } else {
            numeros = contratos[0].NRSEQCONTRATO;
            return numeros;
        }
    };

    this.salvarHistoricoSeguro = function (row) {
        var params = {
            "NRSEGUROCONTRATO": row.NRSEGUROCONTRATO
        };
        return General.callRestEngine("product#/fct_contrato_widget/salvarHistoricoSeguro", "Row", params);
    };

    this.groupContratosBy = function (contratos, chave) {
        var auxiliar = contratos.reduce(function (row, el) {
            (row[el[chave]] = row[el[chave]] || []).push(el);
            return row;
        }, []);
        return auxiliar;
    };

    this.alterarVisibilidadeAgrupamento = function (widget) {
        if (widget.currentRow.QTITENSTRABSERV > 0) {
            widget.getField('IDAGRUPPARC').readOnly = true;
        }
        else {
            if (widget.currentRow.IDMUNIPRESTSERV == 'U') {
                widget.getField('IDAGRUPPARC').readOnly = true;
                widget.currentRow.IDAGRUPPARC = '1';
            }
            else {
                widget.getField('IDAGRUPPARC').readOnly = false;
            }
        }
    };

    this.abrirPopupAlteraSituAss = function (widget, check) {
        ScreenService.closePopup();
        var wdgPopup = templateManager.container.getWidget("WdgPopupAlteraSituAssina");
        wdgPopup.isVisible = true;
        wdgPopup.dataSource.data = [];
        wdgPopup.currentRow = {};
        ScreenService.openPopup(wdgPopup).then(function () {
            wdgPopup.activate();
            wdgPopup.newRow();
            wdgPopup.currentRow.IDALTERAEMMASSA = check;
        }.bind(this));
    };

    this.validaSituAlteraAss = function (wdgPopup) {

        if (wdgPopup.currentRow.IDSITUACAO == 'R') {

            wdgPopup.getField("DSJUSTIFICARECUSA").readOnly = false;
            wdgPopup.getField("DSJUSTIFICARECUSA").validations = { "required": true };
        }
        else {

            wdgPopup.getField("DSJUSTIFICARECUSA").readOnly = true;
            wdgPopup.getField("DSJUSTIFICARECUSA").validations = {};
        }
        wdgPopup.reload();
    }

    this.salvarAlteraSituAssina = function (widget) {

        if (widget.isValid()) {

            var widAssina = templateManager.container.getWidget('assinaturasCont');
            var idalteraemmassa = widget.currentRow.IDALTERAEMMASSA;

            if (idalteraemmassa == true) {

                var params = {
                    "WIDASSINATURA": widAssina.getCheckedRows(),
                    "WIDSITUACAO": widget.currentRow
                };

            }
            else {

                var params = {
                    "WIDASSINATURA": widAssina.currentRow,
                    "WIDSITUACAO": widget.currentRow
                };

            }

            General.callRestEngine("product#/fct_contrato_widget/salvarAlteraSituAssina", "Row", params).then(function (response) {

                var resultado = response.dataset.salvarAlteraSituAssina.resultado;

                if (resultado == 'OK') {

                    ScreenService.closePopup();
                    widAssina.activate().then(function () {
                        widAssina.reload();
                        if (idalteraemmassa == true) {
                            ScreenService.showMessage('Situação da(s) assinatura(s) alterada com sucesso.');
                        }
                        else {
                            ScreenService.showMessage('Situação da assinatura alterada com sucesso.');
                        }
                    }.bind(this));
                }
                else {

                    ScreenService.showMessage(resultado);
                }
            }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
        }

    };


    /**
     * Abre o popup de edição em massa de Escopo de Faturamento
     * @params{Object} widget widget contrato grid
     **/
    this.abrirWdgAlteraEscopoFat = function (widget) {
        var wdgPopup = templateManager.container.getWidget("WdgPopupAlteraEscopoFat");
        ScreenService.closePopup();
        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function () {
            //wdgPopup.edit();
            wdgPopup.activate();
            wdgPopup.dataSource.data = [];
            wdgPopup.newRow();
        }.bind(this));
    };

    /**
     * Cancela a alteração de Escopo de Faturamento
     * @params{Object} widget widget popup alteração de Escopo de Faturamento 
     **/
    this.cancelarWdgAlteracaoEscopoFat = function (widget) {
        ScreenService.closePopup();
        widget.isVisible = false;
        var widContrato = templateManager.container.getWidget("wdgContratroGrid");
        widContrato.activate();
    };

    /**
     * Salva a alteração do escopo de faturamento nos contratos selecionados
     * @partams{Object} widget widget popup de alteração dos escopos de faturamento
     **/
    this.salvarAlteracoesEscopoFat = function (widget) {

        if (widget.isValid()) {

            var widContrato = templateManager.container.getWidget("wdgContratroGrid");

            var params = {
                "CONTRATOS": widContrato.getCheckedRows(),
                "NRTIPOORCAMENTO": widget.currentRow.NRTIPOORCAMENTO
            };

            General.callRestEngine("product#/fct_itemcontrato_widget/salvarAlteracaoEscopoMassa", "Row", params).then(function (response) {

                var resultado = response.dataset.salvarAlteracaoEscopoMassa.resultado;

                if (resultado == 'OK') {
                    ScreenService.showMessage("Contratos atualizados com sucesso!");
                    this.cancelarWdgAlteracaoEscopoFat(widget);
                    widContrato.reload();
                }
                else {
                    this.cancelarWdgAlteracaoEscopoFat(widget);
                    this.popupAlterarEscopoMassa(resultado);
                }
            }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
        }
    };

    this.popupAlterarEscopoMassa = function (dados) {

        var wdgErroAlterarEscopoMassa = templateManager.container.getWidget('wdgErroAlterarEscopoMassa');
        wdgErroAlterarEscopoMassa.dataSource.data = dados;
        wdgErroAlterarEscopoMassa.isVisible = true;

        ScreenService.openPopup(wdgErroAlterarEscopoMassa).then(function () {
            $('.zh-new-grid-wdgErroAlterarEscopoMassa').parent().parent().parent().parent().parent().parent().addClass('cp-high-modal');
        });

    };

    /**
     * Fecha a widget wdgErroAlterarEscopoMassa 
     **/
    this.fecharAlterarEscopoMassa = function () {
        $('.zh-new-grid-wdgErroAlterarEscopoMassa').parent().parent().parent().parent().parent().parent().removeClass('cp-high-modal');

        var wdgErroAlterarEscopoMassa = templateManager.container.getWidget('wdgErroAlterarEscopoMassa');
        wdgErroAlterarEscopoMassa.isVisible = true;
        ScreenService.closePopup();

    };

    /**
     * Função para controlar os campos referente à integração do FM
     */
    this.integracaoFMOnChange = function (widget) {
        var idIntegracaoFM = widget.currentRow.IDINTEGRACAOFM;
        var idIntegraLocaisFM = widget.getField("IDINTEGRALOCAISFM");
        var idIntegraMedicaoFM = widget.getField("IDINTEGRAMEDICAOFM");

        if (idIntegracaoFM == "N") {
            idIntegraLocaisFM.readOnly = true;
            idIntegraLocaisFM.validations = {};
            idIntegraLocaisFM.value("N");
            idIntegraMedicaoFM.readOnly = true;
            idIntegraMedicaoFM.validations = {};
            idIntegraMedicaoFM.value("N");
        }
        else {
            idIntegraLocaisFM.readOnly = false;
            idIntegraLocaisFM.validations = { "required": true };
            idIntegraMedicaoFM.readOnly = false;
            idIntegraMedicaoFM.validations = { "required": true };
        }
    };

    this.atualizaNotaFiscal = function (widget) {
        var fieldSerie = widget.getField("DSSERIE");
        fieldSerie.clearValue();
        fieldSerie.reload();
    };

    this.moedaOnChange = function (widget) {
        if ((!widget.currentRow.__is_new && !mudouMoeda) &&
            (widget.backupRow && widget.backupRow.NRMOEDA != widget.currentRow.NRMOEDA)) {
            mudouMoeda = true;
            ScreenService.showMessage(ScreenService.i18n("Atenção! <br><br>Mudar a moeda de um contrato não fará com que o valor dos itens e dos descontos sejam recalculados automaticamente. Será necessário ajustar o valor dos itens manualmente para a nova moeda."));
        }
    };

    this.parceiroOnChange = function (wdgContrato) {
        if (wdgContrato.currentRow.__is_new) {
            if (wdgContrato.currentRow.CDIDIOM_PARC) {
                wdgContrato.currentRow.CDIDIOM = wdgContrato.currentRow.CDIDIOM_PARC;
                wdgContrato.currentRow.NMIDIOMA = wdgContrato.currentRow.NMIDIOMA_PARC;
            }
            else if (cdidiom_padrao) {
                wdgContrato.currentRow.CDIDIOM = cdidiom_padrao;
                wdgContrato.currentRow.NMIDIOMA = nmidioma_padrao;
            }

            if (wdgContrato.currentRow.NRMOEDA_PARC) {
                wdgContrato.currentRow.NRMOEDA = wdgContrato.currentRow.NRMOEDA_PARC;
                wdgContrato.currentRow.NMMOEDA = wdgContrato.currentRow.NMMOEDA_PARC;
                this.moedaOnChange(wdgContrato);
            }
            else if (nrmoeda_padrao) {
                wdgContrato.currentRow.NRMOEDA = nrmoeda_padrao;
                wdgContrato.currentRow.NMMOEDA = nmmoeda_padrao;
                this.moedaOnChange(wdgContrato);
            }
        }
    };

    /**
     * Função para controlar os campos referente à integração do Tecfood
     */
    this.integracaoTecFoodOnChange = function () {

        var wdgContrato = templateManager.container.getWidget('gridContrato');
        var wdgItens = templateManager.container.getWidget('itemcontrato');

        var idIntegracaoTecFood = wdgContrato.currentRow.IDINTEGRACAOTECFOOD;
        var idTipoServicoTecFood = wdgContrato.getField("IDTIPOSERVINTTECFOOD");

        var cdGrupServIntTecfood = wdgItens.getField("CDGRUPSERVINTTECFOOD");
        var hrIniPrepServIntTecfood = wdgItens.getField("HRINIPREPSERVINTTECFOOD");


        if (idIntegracaoTecFood == "N") {
            idTipoServicoTecFood.readOnly = true;
            idTipoServicoTecFood.validations = {};
            idTipoServicoTecFood.value(null);

            cdGrupServIntTecfood.readOnly = true;
            cdGrupServIntTecfood.value(null);

            hrIniPrepServIntTecfood.readOnly = true;
            hrIniPrepServIntTecfood.value(null);
        }
        else {
            idTipoServicoTecFood.readOnly = false;
            idTipoServicoTecFood.validations = { "required": true };

            cdGrupServIntTecfood.readOnly = false;

            hrIniPrepServIntTecfood.readOnly = false;
        }
    };

    this.validaUsaLoteFaturamentoItem = function (widget) {

        var gridContrato = templateManager.container.getWidget("gridContrato");
        var usaLoteFaturamento = gridContrato.currentRow.IDUTILIZALOTE;
        var dsLoteFatura = widget.getField("DSLOTEFATURAMENTO");

        if (usaLoteFaturamento == 'S'){
            dsLoteFatura.readOnly = false;
        }
        else {
            dsLoteFatura.readOnly = true;
        }
    };

    /**
     * Abre o popup de inserção de elementos no contrato
     **/
    this.abrirPopUpInsElemento = function () {

        var gridContrato = templateManager.container.getWidget("gridContrato");
        var wdgInserirElementosContr = templateManager.container.getWidget('wdgInserirElementosContr');

        wdgInserirElementosContr.dataSourceFilter[0].value = gridContrato.currentRow.NRORCAMENTOMODELO;
        wdgInserirElementosContr.reload();
        this.controlaAcoesInsElementoContr(wdgInserirElementosContr, 'abre');
        wdgInserirElementosContr.isVisible = true;

        ScreenService.openPopup(wdgInserirElementosContr).then(function () {
            $('.zh-new-grid-wdgInserirElementosContr').parent().parent().parent().parent().parent().parent().addClass('cp-high-modal');
        });

    };

    /**
     * Fecha a widget wdgInserirElementosContr 
     **/
    this.fecharPopUpInsElemento = function () {
        $('.zh-new-grid-wdgInserirElementosContr').parent().parent().parent().parent().parent().parent().removeClass('cp-high-modal');

        var wdgInserirElementosContr = templateManager.container.getWidget('wdgInserirElementosContr');
        var wdgElementosProposta = templateManager.container.getWidget('wdgElementosProposta');
        wdgInserirElementosContr.isVisible = false;
        this.controlaAcoesInsElementoContr(wdgInserirElementosContr, 'fecha');
        this.habilitarEdicaoAbaElementosNew(wdgElementosProposta);
        wdgElementosProposta.reload();
        ScreenService.closePopup();

    };

    this.confirmInsElementoContr = function (widget, idMantemDescAcres) {
        var widContrato = templateManager.container.getWidget("wdgContratroGrid");

        var params = {
            "CONTRATO": widContrato.currentRow.NRSEQCONTRATO,
            "ESCOPO": widContrato.currentRow.NRTIPOORCAMENTO,
            "MODELO": widContrato.currentRow.NRSEQMODCONTRA,
            "MOEDA": widContrato.currentRow.NRMOEDA,
            "MANTER_DESC": idMantemDescAcres,
            "DADOS": widget.getCheckedRows()
        };

        var timeout = 10 * 60000; //10 min
        Utilities.callRestEngineWithTimeout("product#/fct_contrato_widget/confirmInsElementoContr", "Row", params, timeout).then(function (response) {

            var resultado = response.dataset.confirmInsElementoContr.resultado;

            if (resultado == 'OK') {
                ScreenService.showMessage("Operação realizada com sucesso!"); 
                widget.reload();
                this.fecharPopUpInsElemento();
            }
            else {
                ScreenService.showMessage(resultado);
            }
        }.bind(this))
        .catch(function (error) {
            Utilities.showErrorMessage(error);
        });
    };

    this.excluirElementoContr = function (widget, idMantemDescAcres) {
        var widContrato = templateManager.container.getWidget("wdgContratroGrid");

        var params = {
            "CONTRATO": widContrato.currentRow.NRSEQCONTRATO,
            "ESCOPO": widContrato.currentRow.NRTIPOORCAMENTO,
            "MODELO": widContrato.currentRow.NRSEQMODCONTRA,
            "MOEDA": widContrato.currentRow.NRMOEDA,
            "MANTER_DESC": idMantemDescAcres,
            "DADOS": widget.dataSource.data,
            "DADOS_MARCADOS": widget.getCheckedRows()
        };

        var timeout = 10 * 60000; //10 min
        Utilities.callRestEngineWithTimeout("product#/fct_contrato_widget/excluirElementoContr", "Row", params, timeout).then(function (response) {

            var resultado = response.dataset.excluirElementoContr.resultado;

            if (resultado == 'OK') {
                ScreenService.showMessage("Operação realizada com sucesso!"); 
                this.cancelarElementosContrato(widget);
                widget.reload();
            }
            else if (resultado == 'OK_PARC') {
                ScreenService.showMessage("Operação realizada com sucesso! <br><br> Foram recalculados os dados de parcelamento da negociação de valores."); 
                this.cancelarElementosContrato(widget);
                widget.reload();;
            }
            else {
                ScreenService.showMessage(resultado);
            }
        }.bind(this))
        .catch(function (error) {
            Utilities.showErrorMessage(error);
        });

    };
    
    this.editarTpMedidaElementoContr = function (widget) {
        var widContrato = templateManager.container.getWidget("wdgContratroGrid");

        var params = {
            "CONTRATO": widContrato.currentRow.NRSEQCONTRATO,
            "DADOS_TPMEDIDA": widget.dataSource.data
        };

        var timeout = 10 * 60000; //10 min
        Utilities.callRestEngineWithTimeout("product#/fct_contrato_widget/editarTpMedidaElementoContr", "Row", params, timeout).then(function (response) {

            var resultado = response.dataset.editarTpMedidaElementoContr.resultado;

            if (resultado == 'OK') {
                ScreenService.showMessage("Operação realizada com sucesso!"); 
                widget.list();
                widget.reload();
            }
            else if (resultado == 'OK_PARC') {
                ScreenService.showMessage("Operação realizada com sucesso! <br><br> Foram recalculados os dados de parcelamento da negociação de valores."); 
                widget.list();
                widget.reload();
            }
            else {
                ScreenService.showMessage(resultado);
            }
        }.bind(this))
        .catch(function (error) {
            Utilities.showErrorMessage(error);
        });

    };

    this.validaAgrupaAutomatico = function (widget) {                  

        var params = {
            "NRPARCFATURA": widget.currentRow.NRPARCNEGOCIO,
            "NRTIPOORCAMENTO": widget.currentRow.NRTIPOORCAMENTO
        };

        General.callRestEngine("product#/fct_contrato_widget/validaAgrupamento", "Row", params).then(function (response) {

            var resultado = response.dataset.validaAgrupamento.resultado;

            if (resultado['QUANT'] > 0) {
                ScreenService.showCustomDialog("O parceiro de negócio vinculado ao contrato possui agrupamento de contratos para faturamento cadastrados.<br><br>Deseja vincular este contrato a um agrupamento de contratos para faturamento?", 'CONFIRMATION', null, [{
                    "label": "Sim",
                    "code": function () {
                         if (resultado['NRAGRUPAMENTO'] != null){
                            this.aprovarContrato(widget, resultado['NRAGRUPAMENTO']);
                         }
                         else {
                            this.abrirPopupAgrupaAutomatico(widget);   
                         }
                    }.bind(this)
                }, {
                    "label": "Não",
                    "code": function () {
                        this.aprovarContrato(widget, null);
                    }.bind(this)
                }, {
                    "label": "Cancelar",
                    "code": function () { }
                }]);
            }
            else {
                this.aprovarContrato(widget, null);    
            }

        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
        });
    }

    /**
     * Abre o popup de agrupamento automático
     **/
    this.abrirPopupAgrupaAutomatico = function (widget) {
        var wdgPopup = templateManager.container.getWidget("popupAgrupaAutomatico");
        ScreenService.closePopup();

        wdgPopup.currentRow.NRSEQCONTRATO = widget.currentRow.NRSEQCONTRATO;
        wdgPopup.currentRow.NRPARCFATURA = widget.currentRow.NRPARCNEGOCIO;

        wdgPopup.getField("NMAGRUPAMENTO").dataSourceFilter[1] = {
            "name": "NRPARCFATURA",
            "operator": "=",
            "value": widget.currentRow.NRPARCNEGOCIO
        };

        wdgPopup.getField("NMAGRUPAMENTO").reload();

        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function () {
            wdgPopup.edit();
            wdgPopup.activate();
        }.bind(this));
    };

    /**
     * Salva o contrato automaticamente no agrupamento 
     **/
    this.salvarAgrupaAutomatico = function (widget) {
        if (widget.isValid()) {

            var gridContrato = templateManager.container.getWidget("gridContrato");

            this.aprovarContrato(gridContrato, widget.currentRow.NRAGRUPAMENTO);
        }
    };

    /**
     * Cria um evento na Agenda do Google
     **/
    this.criaAgendaGoogle = function (widget, action) {

        if (action == true) {
            wid = widget.currentRow;
        }
        else {
            wid = widget;
        }

        General.callRestEngine("product#/fct_contrato_widget/validaParamIntegraGoogle", "Row", wid).then(function (response) {

            var resultado = response.dataset.validaParamIntegraGoogle.resultado;

            if (resultado == 'PARAM_OK') {
                ScreenService.showCustomDialog("Deseja criar um evento no Google Agenda?", 'CONFIRMATION', null, [{
                    "label": "Sim",
                    "code": function () {
                        General.callRestEngine("product#/fct_contrato_widget/integraGoogleAgenda", "Row", wid).then(function (response) {
                            var integra = response.dataset.integraGoogleAgenda.resultado;

                            if (integra == 'OK') {
                                ScreenService.showMessage("Operação realizada com sucesso!"); 
                            }
                        })
                        .catch(function (error) {
                            Utilities.showErrorMessage(error);
                        })
                    }.bind(this)
                }, {
                    "label": "Não",
                    "code": function () {}
                }]);
            }
            else if  (resultado == 'PARAM_N_OK') {
                ScreenService.showMessage("O contrato está marcado para realizar integração com Google Agenda, porém, a parametrização não está preenchida. <br><br> Caso deseja realizar a integração, favor preencher a parametrização na tela 'Parametrização SM' e realizar a integração novamente através da ação 'Criar Evento (Google Agenda)'. ");    
            }

        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
        });
    };

    this.atualizaQuadroHCM = function (widget) {
        var params = {
            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
            "IDAPROVACAO": 'N'
        };

        General.callRestEngine("product#/IntegracaoHcm", "Row", params).then(function (resultado) {
            if (resultado.dataset.integracaoHcm.resultado != 'OK') {
                ScreenService.showMessage("Operação bloqueada!<br><br>" + resultado.dataset.integracaoHcm.resultado);
            } else {
                ScreenService.showMessage("Operação realizada com sucesso!");
            }
        })
    };

    this.atualizaQuadroHCMMassa = function (){
        ScreenService.confirmMessage("Deseja realmente atualizar o quadro de vagas do HCM para os contratos selecionados?", 'question', function () {
            
            var wdgContratroGrid = templateManager.container.getWidget('wdgContratroGrid');

            var checkedRows = wdgContratroGrid.getCheckedRows();

            General.callRestEngine("product#/integracaoHcmMassa", "Row", checkedRows).then(function (resultado) {
                if (resultado.dataset.integracaoHcmMassa.resultado != 'OK') {
                    ScreenService.closePopup();
                    wdgContratroGrid.reload();
                    wdgContratroGrid.dataSource.clearCheckedRows();
                    this.popupAtualizaQuadroHCMMassa(resultado.dataset.integracaoHcmMassa.resultado);

                } else {
                    ScreenService.closePopup();
                    ScreenService.showMessage("Operação realizada com sucesso!");
                }
            }.bind(this))

        }.bind(this));
    };

     /**
     * Abre a widget wdgErroAtualizaHCM 
     **/

     this.popupAtualizaQuadroHCMMassa = function (dados) {

        var wdgErroAtualizaHCM = templateManager.container.getWidget('wdgErroAtualizaHCM');
        wdgErroAtualizaHCM.dataSource.data = dados;
        wdgErroAtualizaHCM.isVisible = true;

        ScreenService.openPopup(wdgErroAtualizaHCM).then(function () {
            $('.zh-new-grid-wdgErroAtualizaHCM').parent().parent().parent().parent().parent().parent().addClass('cp-high-modal');
        });

    };

    /**
     * Fecha a widget wdgErroAtualizaHCM 
     **/
    this.fecharAtualizaQuadroHCMMassa = function () {
        $('.zh-new-grid-wdgErroAtualizaHCM').parent().parent().parent().parent().parent().parent().removeClass('cp-high-modal');

        var wdgErroAtualizaHCM = templateManager.container.getWidget('wdgErroAtualizaHCM');
        wdgErroAtualizaHCM.isVisible = true;
        ScreenService.closePopup();

    };

    this.atualizaModDocFisc = function (widget) {
        var fieldModDocFisc = widget.getField("DSMODDOCFISC");
        fieldModDocFisc.clearValue();
        fieldModDocFisc.reload();
    };

     /**
     * Função para controlar os campos referente à integração do FM
     */
     this.integracaoHCMOnChange = function (widget) {
        var idIntegracaoHCM = widget.currentRow.IDINTEGRACAOHCM;
        var integraEscalaHCMValue = widget.currentRow.IDINTEGRAESCALAHCM;
        var tipoEstruturaHCM = widget.getField("TPESTRUTINTEGRAHCM");
        var integraEscalaHCM = widget.getField("IDINTEGRAESCALAHCM");
        var agrupaEscalaHCM = widget.getField("IDAGRUPAESCALAHCM");

        if (idIntegracaoHCM == "N") {
            tipoEstruturaHCM.readOnly = true;
            tipoEstruturaHCM.validations = {};
            tipoEstruturaHCM.value("T");
            integraEscalaHCM.readOnly = true;
            integraEscalaHCM.validations = {};
            integraEscalaHCM.value("N");
            agrupaEscalaHCM.readOnly = true;
            agrupaEscalaHCM.validations = {};
            agrupaEscalaHCM.value("N");
        }
        else {
            tipoEstruturaHCM.readOnly = false;
            tipoEstruturaHCM.validations = { "required": true };
            integraEscalaHCM.readOnly = false;
            integraEscalaHCM.validations = { "required": true };

            if (integraEscalaHCMValue == "N") {
                agrupaEscalaHCM.readOnly = true;
                agrupaEscalaHCM.validations = {};
                agrupaEscalaHCM.value("N");
            }
            else {
                agrupaEscalaHCM.readOnly = false;
                agrupaEscalaHCM.validations = { "required": true };
            }
        }
    };

    this.integracaoHCMItensOnChange = function () {

        var wdgContrato = templateManager.container.getWidget('gridContrato');
        var wdgItens = templateManager.container.getWidget('itemcontrato');

        var idIntegracaoHCM = wdgContrato.currentRow.IDINTEGRACAOHCM;
        var integraEscalaHCM = wdgContrato.currentRow.IDINTEGRAESCALAHCM;
        var agrupaEscalaHCM = wdgContrato.currentRow.IDAGRUPAESCALAHCM;

        var nrEscalaTrabHCM = wdgItens.getField("NRESCALATRABALHOHCM");
        var idServIntegraHCM = wdgItens.getField("IDSERVINTEGRAHCM");

        var idServIntegraHCM_Val = wdgItens.currentRow.IDSERVINTEGRAHCM;

        if (idIntegracaoHCM == "N") {
            nrEscalaTrabHCM.readOnly = true;
            nrEscalaTrabHCM.value(null);

            idServIntegraHCM.readOnly = true;
            idServIntegraHCM.value(null);
            idServIntegraHCM.validations = {};
        }
        else {
            
            if ((integraEscalaHCM == "S") || (agrupaEscalaHCM == "S"))  {
               nrEscalaTrabHCM.readOnly = false;
            }
            else {
                nrEscalaTrabHCM.readOnly = true;
                nrEscalaTrabHCM.value(null);  
            }

            idServIntegraHCM.readOnly = false;
            idServIntegraHCM.validations = { "required": true };

            if (idServIntegraHCM_Val == null) {
                this.getServicoIntegraHCM(wdgItens);
            };
        }
    };

    this.carregaDadosEscalaintegracaoHCMItens = function () {

        var wdgContrato = templateManager.container.getWidget('gridContrato');

        var idIntegracaoHCM = wdgContrato.currentRow.IDINTEGRACAOHCM;
        var integraEscalaHCM = wdgContrato.currentRow.IDINTEGRAESCALAHCM;

        if ((idIntegracaoHCM == "S") && (integraEscalaHCM == "S")){

            var params = {
                "NRSEQCONTRATO": wdgContrato.currentRow.NRSEQCONTRATO,
                "IDINTEGRACAOHCM": idIntegracaoHCM,
                "IDINTEGRAESCALAHCM": integraEscalaHCM,
                "IDAPROVACAO": 'N'
            };

            General.callRestEngine("product#/carregaDadosEscalaintegracaoHCMItens", "Row", params).then(function () {
            }.bind(this))

        }
    };

    this.limpaValoresEscalaintegracaoHCMItens = function () {

        var wdgContrato = templateManager.container.getWidget('gridContrato');

        var integraEscalaHCM = wdgContrato.currentRow.IDINTEGRAESCALAHCM;

        if (integraEscalaHCM == "N") {

            var params = {
                "NRSEQCONTRATO": wdgContrato.currentRow.NRSEQCONTRATO
            };

            General.callRestEngine("product#/limpaValoresEscalaintegracaoHCMItens", "Row", params).then(function () {
            }.bind(this))

        }
    };

    /**
     * Busca se o serviço está na lista de exceções da integtração com o HCM
     * @param {Object} widget - widget de itens (editando)
     */
    this.getServicoIntegraHCM = function (widget) {
       
        var wdgContrato = getWidgetGridContrato();
        var params = {
            "NRSEQMODCONTRA": wdgContrato.currentRow.NRSEQMODCONTRA,
            "CDSERVORCA": widget.currentRow.CDPRODSERV
        };

        General.callRestEngine("product#/fct_itemcontrato_widget/getServicoIntegraHCM", "Row", params).then(function (response) {
            if (!response.dataset || !response.dataset.getServicoIntegraHCM) {
                widget.currentRow.IDSERVINTEGRAHCM = 'S';
            }
            else {

                widget.currentRow.IDSERVINTEGRAHCM = response.dataset.getServicoIntegraHCM.resultado;
            }
        });
    };
    
    this.enviarEmailPosAssinaturaGrid = function () {

        ScreenService.confirmMessage("Deseja realmente enviar o email de confirmação de aprovação para os contratos selecionados?", 'question', function () {
            var wdgContratroGrid = templateManager.container.getWidget('wdgContratroGrid');

            var checkedRows = wdgContratroGrid.getCheckedRows();

            General.callRestEngine("product#/fct_contrato_widget/validarEnvioEmailPosAssinatura", "Row", checkedRows).then(function (response) {

                var resultado = response.dataset.validarEnvioEmailPosAssinatura.resultado;

                if (resultado.STATUS === 'OK') {
                    var contratos = resultado.CONTRATOS;
                    console.log(contratos);
                    var timeout = 5 * 60000; //5 min
                    
                    Utilities.callRestEngineWithTimeout("product#/fct_contrato_widget/enviaEmailPosAssinatura", "Row", contratos, timeout).then(function() {
                        ScreenService.showMessage(resultado.MENSAGEM).then(function(){
                    }.bind(this),
                    function(error) {
                        console.error(error);
                        var msg = "Timeout";
                        if (error.status !== 0 && error.data)
                            msg = error.data.error;
                        ScreenService.showMessage(msg);
                    });
                    }.bind(this))
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                } else {
                    ScreenService.showMessage(resultado.MENSAGEM);
                }

            }.bind(this), function (error) {
                Utilities.showErrorMessage(error);
            }.bind(this));
        }.bind(this));
    }

    this.enviarEmailPosAssinaturaDadosContrato = function (widget) {
        var contrato = [];

        var params = {
            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
            "NRTIPOORCAMENTO": widget.currentRow.NRTIPOORCAMENTO,
            "NRSEQMODCONTRA": widget.currentRow.NRSEQMODCONTRA,
            "NRCONTRATOFCT": widget.currentRow.NRCONTRATOFCT, 
        };

        contrato.push(params);

        ScreenService.confirmMessage("Deseja realmente enviar o email de confirmação de aprovação do contrato?", 'question', function () {

            General.callRestEngine("product#/fct_contrato_widget/validarEnvioEmailPosAssinatura", "Row", contrato).then(function (response) {

                var resultado = response.dataset.validarEnvioEmailPosAssinatura.resultado;

                if (resultado.STATUS === 'OK') {
                    var contratos = resultado.CONTRATOS;
                    var timeout = 5 * 60000; //5 min
                    
                    Utilities.callRestEngineWithTimeout("product#/fct_contrato_widget/enviaEmailPosAssinatura", "Row", contratos, timeout).then(function() {
                        ScreenService.showMessage(resultado.MENSAGEM).then(function(){
                    }.bind(this),
                    function(error) {
                        console.error(error);
                        var msg = "Timeout";
                        if (error.status !== 0 && error.data)
                            msg = error.data.error;
                        ScreenService.showMessage(msg);
                    });
                    }.bind(this))
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                } else {
                    ScreenService.showMessage(resultado.CONTRATOS[0].LOG_STATUS);
                }

            }.bind(this), function (error) {
                Utilities.showErrorMessage(error);
            }.bind(this));
        }.bind(this));
    }

    this.createGoogleEvent = function (widget) {
        var eventParams = {
            title: 'Renovação de Contrato: ' + widget.currentRow.NRCONTRATOFCT + " - " + widget.currentRow.DSREFERENCIACONT.substring(0,40),
            description: 'Hoje termina a validade contrato ' + widget.currentRow.NRCONTRATOFCT + ' - ' +  widget.currentRow.DSREFERENCIACONT + '  do cliente ' + widget.currentRow.NMSECUNDARIPARC,
            endDate : widget.currentRow.DTFINVIGENCONT
        };

        Utilities.SaveOnGoogleAgenda(eventParams);  
    }


    /**
     * Volta para o modo list da tela de lembretes de contrato.
     * 
     * @param {object} widget - Widget de Lembretes
     */
    this.voltarParaLembreteGrid = function () {
        var wdgLembrete = getWidgetGridLembreteContrato();
        wdgLembrete.dataSource.data = [];
        wdgLembrete.currentRow = {};

        ScreenService.closeSwipe().then(function () {
            var wdgLembreteGrid = templateManager.container.getWidget("lembretesContrato");
            wdgLembreteGrid.activate();
            wdgLembreteGrid.refreshWidget();
        });
    };

    /**
     * Salvar Lembrentes do Contrato
     * 
     * @params{Object} widget - widget de Lembretes
     **/
    this.saveLembretesContrato = function(widget) {
        if (widget.isValid()) {

            if (((widget.currentRow.DSEMAILLEMBRETE != '') && (widget.currentRow.DSEMAILLEMBRETE != null)) || 
                ((widget.currentRow.PARTICIPANTES != '') && (widget.currentRow.PARTICIPANTES != null))) {
                var wdgContrato = templateManager.container.getWidget('gridContrato');

                var params = {
                    "NRSEQCONTRATO": wdgContrato.currentRow.NRSEQCONTRATO,
                    "LEMBRETE": widget.currentRow
                };
                
                General.callRestEngine('product#/fct_contrato_widget/salvarLembreteContrato', "Row", params).then(function (response) {
                    if (response.dataset.request.resulted === 'OK') {
                        ScreenService.closeSwipe();
                        ScreenService.showMessage("Operação realizada com sucesso!");
                    } else {
                        ScreenService.closeSwipe();
                        ScreenService.showMessage(response.dataset.request.resulted);
                    }
                }.bind(this));
            }
            else {
                ScreenService.showMessage("Operação Bloqueada! <br><br> Deve ser preenchido ao menos um participante ou e-mail para o lembrete.");
            }
        }
    };

    /**
     * Adiciona novo Lembrente
     * 
     * @params{Object} widget - widget de Lembretes
     **/
    this.novoLembrete = function (widget) {
        widget.newRow();
    };

   /**
     * Adiciona novo Lembrente
     * 
     * @params{Object} widget - widget de Lembretes
     **/
    this.editLembrete = function (widget) {
        widget.edit();
    };

   /**
     * Altera a vizualização dos campos de acordo com o tipo de notificação selecionada.
     * 
     * @params{Object} widget - widget de Lembretes
     **/
    this.tipoNotificacaoOnChange = function (widget) {
        var fieldTipoNotificacao = widget.currentRow.IDTPNOTIFICACAOLEMBRCONT;
        var dtFinalRecorrencia = widget.getField("DTFINALRECORRENCIA");
        var fieldQuantidadeEvento = widget.getField("QTEVENTOLEMBRETE");
        var idPeriodoEvent = widget.getField("IDPERIODOEVENTLEMBRCONT");
        var idTpEvento = widget.getField("IDTPEVENTLEMBRCONT");
        var qtDiasRecorencia = widget.getField("QTDIASRECORRENCIA");
        var idDiasRecorrencia = widget.getField("IDDIASRECLEMBRCONT");

        if (fieldTipoNotificacao == "E") {

            fieldQuantidadeEvento.readOnly = false;
            fieldQuantidadeEvento.validations = {
                "required": true
            };

            idPeriodoEvent.readOnly = false;
            idPeriodoEvent.validations = {
                "required": true
            };

            idTpEvento.readOnly = false;
            idTpEvento.validations = {
                "required": true
            };

            dtFinalRecorrencia.value(null);
            dtFinalRecorrencia.readOnly = true;

            qtDiasRecorencia.value(null);
            qtDiasRecorencia.readOnly = true;
            qtDiasRecorencia.validations = {};

            idDiasRecorrencia.value(null);
            idDiasRecorrencia.readOnly = true;
            idDiasRecorrencia.validations = {};

        } else if (fieldTipoNotificacao == "R") {

            fieldQuantidadeEvento.value(null);
            fieldQuantidadeEvento.readOnly = true;
            fieldQuantidadeEvento.validations = {};

            idPeriodoEvent.value(null);
            idPeriodoEvent.readOnly = true;
            idPeriodoEvent.validations = {};

            idTpEvento.value(null);
            idTpEvento.readOnly = true;
            idTpEvento.validations = {};

            dtFinalRecorrencia.readOnly = false;
            
            qtDiasRecorencia.readOnly = false;
            qtDiasRecorencia.validations = {
                "required": true
            };

            idDiasRecorrencia.readOnly = false;
            idDiasRecorrencia.validations = {
                "required": true
            };

        }
    };

    /* Recarrega o field caso o dataSource tenha sido alterado 
     * @params{Object} field Nome Operador
     */
    this.carregaDadosParticipantes = function(widget) {
        var participantes = widget.currentRow.PARTICIPANTES;
        var arrayParticipantes = [];
        
        if(participantes) {
            arrayParticipantes = participantes.split(",");
        }

        if (arrayParticipantes && arrayParticipantes.length > 0) {
            widget.getField("PARTICIPANTES").dataSourceFilter[0] = {
                "name": "CDOPERADOR",
                "operator": "IN",
                "value": arrayParticipantes
            };

            widget.getField("PARTICIPANTES").reload();
        }
    };

    /**
     * Carregar informações ao selecionar um item do grid de Lembrentes
     * 
     * @params{Object} widget - widget de Lembretes
     **/
    
    this.loadInfoLembrete = function (widget) {
        var wdglembrete = getWidgetGridLembreteContrato();
        var lembretefield = wdglembrete.getField("PARTICIPANTES");
        
        participantes = wdglembrete.currentRow.PARTICIPANTES;
        participantesaux = participantes.split(",");
        lembretefield.value(participantesaux);
    };

    /**
     * Busca se o serviço está na lista de exceções da integtração com o HCM
     * @param {Object} widget - widget de itens (editando)
     */
    this.getServicoIntegraHCM = function (widget) {
       
        var wdgContrato = getWidgetGridContrato();
        var params = {
            "NRSEQMODCONTRA": wdgContrato.currentRow.NRSEQMODCONTRA,
            "CDSERVORCA": widget.currentRow.CDPRODSERV
        };

        General.callRestEngine("product#/fct_itemcontrato_widget/getServicoIntegraHCM", "Row", params).then(function (response) {
            if (!response.dataset || !response.dataset.getServicoIntegraHCM) {
                widget.currentRow.IDSERVINTEGRAHCM = 'S';
            }
            else {

                widget.currentRow.IDSERVINTEGRAHCM = response.dataset.getServicoIntegraHCM.resultado;
            }
        });
    };

    /**
     * Adiciona novo Lembrente
     * 
     * @params{Object} widget - widget de Lembretes
     **/
    this.insertNotificaLembrete = function (widget) {
        widget.newRow();
    };

   /**
     * Adiciona novo Lembrente
     * 
     * @params{Object} widget - widget de Lembretes
     **/
    this.editNotificaLembrete = function (widget) {
        widget.edit();
    };

    this.validaDataInicialLembrete = function (widget) {
        var wdglembrete = getWidgetGridLembreteContrato();
        var dataInicio = moment(wdglembrete.currentRow.DTHRLEMBRETE, "DD/MM/YYYY");
        var dataFim = moment(widget.currentRow.DTFINALRECORRENCIA, "DD/MM/YYYY");

        if (dataFim < dataInicio) {
            ScreenService.showMessage("Operação Bloqueada! <br><br> Por favor, insira uma data posterior à do lembrete");
        }
    };

    this.saveNotificaLembrete = function (widget) {
        if (widget.isValid()) {

            if ((widget.currentRow.IDEXIBICAOLEMBRCONT != '') && (widget.currentRow.IDEXIBICAOLEMBRCONT != null)) {
                var params = {
                    "DADOS": widget.currentRow
                };

                General.callRestEngine("product#/fct_contrato_widget/saveNotificaLembrete", "Row", params).then(function (response) {

                    var resultado = response.dataset.saveNotificaLembrete.resultado;

                    if (resultado == 'OK') {
                        ScreenService.showMessage("Operação realizada com sucesso!"); 
                        ScreenService.closeSwipe();
                        widget.reload();
                    }
                    else {
                        ScreenService.showMessage(resultado);
                    }
                }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
            }
            else {
                ScreenService.showMessage("Operação Bloqueada! <br><br> Deve ser preenchido ao menos uma forma de lembrete para a notificação.");
            }
        }
    };

    /**
     * Realiza o controle de visibilidade das actions.
     * Se a situação for "Aprovado" ou "Encerrado" as 
     * actions de edit , insert e delete são blqueadas. 
     * 
     * @param {Object}  widget
     * @param {boolean} visivel
     */
    this.controlaVisibilidadeRateioCentCust = function (widget) {
        var widgetGridContrato = getWidgetGridContrato();
        var situacao = widgetGridContrato.currentRow.IDSITUACAOCONTRATO;

        if (situacao == 'E' || situacao == 'R') {
            this.setVisibilidadeDasActionsRateioCentCust(widget, false);
        }
        else {
            this.setVisibilidadeDasActionsRateioCentCust(widget, true);
        }
    };

    /**
     * Ativa/desativa todas as actions de uma widget, menos as do tipo "back".
     * 
     * @param {Object} widget
     * @param {boolean} visivel
     */
    this.setVisibilidadeDasActionsRateioCentCust = function (widget, visivel) {

        var tipos = ["add", "delete"];

        widget.actions.forEach(function (action) {
            if (tipos.includes(action.name)) {
                action.isVisible = visivel;
            }
        });
    };

    this.validaPerfilAdicRateioCentCust = function () {
        var wdgContrato = getWidgetGridContrato();
        var newitem = getWidgetGridRateioCentCust().getAction('add');
        if (wdgContrato.currentRow.IDSITUACAOCONTRATO == "A") {
            newitem.isVisible = visibilidadeAcoes;
        }
    };

    this.newRowRateioCentCust = function(widget){
        var wdgContrato = getWidgetGridContrato();

        General.widgetNewRow(widget);
        widget.currentRow.NRSEQCONTRATO = wdgContrato.currentRow.NRSEQCONTRATO;
        widget.currentRow.NRORG = wdgContrato.currentRow.NRORG;
    }
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00004000_Contrato_Controller", Sm00004000_Contrato_Controller);
});


// FILE: ./assets/js/controllers/Sm00004001_HistoricoAlteracoesContrato_Controller.js
function Sm00004001_HistoricoAlteracoesContrato_Controller(General, templateManager, ScreenService, Utilities) {
    
    /**
     * Varíavel que serve para determinar se uma renovação possui um reasjute atrelado a ela.
    */
    this.renovacao = false;
    
    /**
     * Variável que serve para determinar se um reajuste está atrelado a esse contrato
    */
    this.reajuste = false;
    
    this.wdgAlteracaoPeriodoVigenteOnEnterTab = function(widget) {
        Utilities.carregarWidgetOnEnterTab(widget);
        this.chamarRequisicoesUltimaRenovacaoEReajuste(widget);
    };
    
    this.wdgReajustesOnEnterTab = function(widget) {
        Utilities.carregarWidgetOnEnterTab(widget);
        this.chamarRequisicoesUltimaRenovacaoEReajuste(widget);
    };
    
    this.chamarRequisicoesUltimaRenovacaoEReajuste = function(widget) {
        this.controlAcoesTabPeriodoVigente(widget);
        this.controlAcoesTabReajuste(widget);
    };
    
    /**
     * Filtra todas as widgets pelos contratos selecionados.
     * 
     * @param {object} wdgTabbedPrincipal
     */
    this.filtrarContratos = function(wdgTabbedPrincipal) {
        var contratos = wdgTabbedPrincipal.getField("NRSEQCONTRATO").value();
        
        wdgTabbedPrincipal.widgets.forEach(function(wid) {
            wid.dataSourceFilter[0].value = contratos;
            wid.reload();
        });
    };
    
    this.abrirWdgHistorico = function(){
        var wdg = templateManager.container.getWidget("tabbedHistorico");
        ScreenService.openSwipe(wdg);
    };
    
    this.controlAcoesTabPeriodoVigente = function(widget) {
        var wdgDadosContrato = templateManager.container.getWidget("wdgDadosAteracaoCont");
        var params = {
            "NRSEQCONTRATO": wdgDadosContrato.currentRow.NRSEQCONTRATO,
        };
        this.renovacao = null;
        
        General.callRestEngine('product#/fct_contrato_widget/possuiDadosRenovacao', "Row", params).then(function(response) {
            templateManager.container.getWidget("wdgAlteracaoPeriodoVigente").getAction("Desfazer_Renovacao").isVisible = response.dataset.Desfeito.Possui;
            this.renovacao = response.dataset.Desfeito.Possui;
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    
    this.controlAcoesTabReajuste = function(widget) {
        var wdgDadosContrato = templateManager.container.getWidget("wdgDadosAteracaoCont");
        var params = {
            "NRSEQCONTRATO": wdgDadosContrato.currentRow.NRSEQCONTRATO
        };
        this.reajuste = null;
        
        General.callRestEngine('product#/fct_contrato_widget/possuiDadosReajuste', "Row", params).then(function(response) {
            templateManager.container.getWidget( "wdgReajustes" ).getAction("Desfazer_Reajuste").isVisible = response.dataset.Desfeito.Possui;
            this.reajuste = response.dataset.Desfeito.Possui;
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    
    this.validarMensagemReajuste = function(){
        try{
            var mensagem={};
            if(this.reajuste != null){
                mensagem = {
                    mensagem: "Deseja realmente desfazer o último reajuste deste contrato?",
                    status:true,
                    junto:false
                };
            }
            //Barrar
            if( this.reajuste == null || this.renovacao == null || (this.renovacao.NRSEQPRORRCONT != this.reajuste.NRSEQPRORRCONT  ||  this.renovacao.NRCONTREAJUSTE != this.reajuste.NRCONTREAJUSTE )){
                //so barra se o reajuste for menor que a renovacao
                if(this.reajuste != null && this.renovacao != null && this.reajuste.NRSEQPRORRCONT < this.renovacao.NRSEQPRORRCONT){
                    mensagem = {
                        mensagem: "Operação Bloqueada! <br><br> Este último reajuste não está ligada a última operação, favor desfazer a última renovação primeiro.",
                        status: false
                    };
                }
            }
            //permitir ambas
            else if(this.renovacao.NRSEQPRORRCONT == this.reajuste.NRSEQPRORRCONT && this.renovacao.NRCONTREAJUSTE == this.reajuste.NRCONTREAJUSTE ){
                mensagem = {
                    mensagem: "Atenção!<br><br> O último reajuste foi feito a partir de uma renovação do período do contrato. Desfazer esse reajuste também irá desfazer essa renovação. Deseja continuar?",
                    status: true,
                    junto:true
                };
            }
            return mensagem;
        }catch(err){
            console.log(err);
            console.log("RENOVACAO:" , this.renovacao);
            console.log("REAJUSTE:" , this.reajuste);
        }
    };
    
    this.DesfazerUltimoReajuste = function(widget){
        var mensagem = this.validarMensagemReajuste();
        try{
            if(mensagem.status){
                ScreenService.confirmMessage(ScreenService.i18n(mensagem.mensagem ), 'question', function(success) {
                        
                    var params = {
                        "NRSEQCONTRATO": widget.parent.parent.currentRow.NRSEQCONTRATO,
                        "NRSEQPRORRCONT": (this.renovacao != null) ? this.renovacao.NRSEQPRORRCONT : null,
                        "JUNTO" : mensagem.junto //reajuste foi feito juntamente com uma renovacao
                    };
                    General.callRestEngine('product#/fct_contrato_widget/desfazerReajusteContrato', "Row", params).then(function(response) {
                        widget.reload();
                        widget.parent.parent.reload();
                        ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
                        this.chamarRequisicoesUltimaRenovacaoEReajuste(widget);
                    }.bind(this))
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }.bind(this));
            }else{
                ScreenService.showMessage(ScreenService.i18n(mensagem.mensagem));
            }
        }catch(err){
            console.log(mensagem);
        }
    };
    
    
    this.validarMensagemRenovacao = function(){
        var mensagem = {};
        try{
            if (this.renovacao != null) {
                var mensagem = {
                    mensagem: "Deseja realmente desfazer a última renovação deste contrato?",
                    status: true,
                    junto:false
                };
            }
            //Barrar
            if (this.renovacao == null || this.reajuste == null ||( this.renovacao.NRSEQPRORRCONT != this.reajuste.NRSEQPRORRCONT ||  this.renovacao.NRCONTREAJUSTE != this.reajuste.NRCONTREAJUSTE )) {
                //barrar somente se a renovacao for menor que o reajuste
                if(this.reajuste != null && this.renovacao != null && this.renovacao.NRSEQPRORRCONT < this.reajuste.NRSEQPRORRCONT){
                    mensagem = {
                        mensagem: "Operação Bloqueada! <br><br> Esta última renovação não está ligada a última operação, favor desfazer o último reajuste primeiro.",
                        status: false
                    };
                }
            }
            //permitir ambas
            else if (this.renovacao.NRSEQPRORRCONT == this.reajuste.NRSEQPRORRCONT && this.renovacao.NRCONTREAJUSTE == this.reajuste.NRCONTREAJUSTE) {
                mensagem = {
                    mensagem: "Atenção!<br><br> Essa última renovação foi feita em união com um reajuste. Desfazer essa renovação também irá desfazer o último reajuste. Deseja continuar?",
                    status: true,
                    junto:true
                };
            }
            return mensagem;
        } catch (err) {
            console.log("RENOVACAO:", this.renovacao);
            console.log("REAJUSTE:", this.reajuste);
        }
    };
    
    
    this.DesfazerUltimaRenovacao = function(widget){
        var resposta = this.validarMensagemRenovacao();
        try{
            if(resposta.status){
                ScreenService.confirmMessage(ScreenService.i18n(resposta.mensagem), 'question', function(success) { 
                        var params = {
                            "NRSEQCONTRATO": widget.parent.parent.currentRow.NRSEQCONTRATO,
                            "NRSEQPRORRCONT": (this.renovacao != null) ? this.renovacao.NRSEQPRORRCONT : null,
                            "JUNTO": resposta.junto
                        };
                        General.callRestEngine('product#/fct_contrato_widget/desfazerRenovacaoContrato', "Row", params).then(function(response) {
                            widget.reload();
                            widget.parent.parent.reload();
                            this.chamarRequisicoesUltimaRenovacaoEReajuste(widget);
                            ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
                        }.bind(this))
                        .catch(function(error) {
                            if(error.data.error.indexOf("SEM_DATA_ANTIGA") !== -1){
                                ScreenService.showMessage(ScreenService.i18n("Operação Bloqueada!</br></br>Só é permitido desfazer a renovação que foram realizadas a partir da versão 3.14."));
                            }else{
                                Utilities.showErrorMessage(error);
                            }
                        });
                }.bind(this));
            }else{
                ScreenService.showMessage(ScreenService.i18n(resposta.mensagem));
            }
        }catch(err){
           console.log(resposta); 
        }
    };


    this.abrePopUpDesfazerUltimaRenovacaoReajusteMassa = function () {
       
        var wdgPopup = templateManager.container.getWidget('PopupCancelaRenovacaoReajusteMassa');

        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function () {
            wdgPopup.dataSource.data = [];
            wdgPopup.newRow();

        }.bind(this));
    };

    this.salvarDesfazerRenovaReajusMassa = function (widget) {

        var wdgPrincipal = templateManager.container.getWidget('wdgDadosAteracaoCont');

        var params = {
            "CONTRATOS": wdgPrincipal.getCheckedRows(),
            "DATA_ESPECIFICA": widget.currentRow.DTCANCELAPROCESSO
        };
        General.callRestEngine('product#/fct_contrato_widget/salvarDesfazerRenovaReajusMassa', "Row", params).then(function(response) {
            ScreenService.closePopup();
            wdgPrincipal.reload();
            ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });

    }
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00004001_HistoricoAlteracoesContrato_Controller", Sm00004001_HistoricoAlteracoesContrato_Controller);
});

// FILE: ./assets/js/controllers/Sm00004003_Reajuste_Contrato_Controller.js
function Sm00004003_Reajuste_Contrato_Controller(General, ScreenService, templateManager, Utilities, ZHPromise) {

    /* A variável global "bloqReajuste" será utilziada para bloquear as ações da tela caso existam indices financeiros
    sem aliquota cadastrada. O valor dela é atribuída na função "validaReajusteContr" */
    var bloqReajuste = 'N';

    var utilizaComposicao = false;

    /* --- FUNÇÕES DA PÁGINA REAJUSTAR VALORES DO CONTRATO --- */

    /**
     * abre a tela "Reajustar Valores do Contrato"
    */

    this.abrirWdgReajustContr = function (ck_rows) {
        ScreenService.closePopup();
        var ReajustContr = templateManager.container.getWidget("reajustarContrato");
        ReajustContr.isVisible = true;
        ScreenService.openSwipe(ReajustContr).then(function () {
            ReajustContr.edit();
            this.setDadosPadraoReajustContr(ReajustContr, ck_rows);
        }.bind(this));
    };

    /**
     * Seta dados padrões do reajuste de contrato
     * 
     * @param {object} widget widget "Reajustar Valores do Contrato"
    */
    this.setDadosPadraoReajustContr = function (widget, ck_rows) {

        var contrato = templateManager.container.getWidget('gridContrato');

        /* Sequencial Contrato */
        var SeqContrato = contrato.currentRow.NRSEQCONTRATO;
        /* Código Contrato */
        var CodContrato = contrato.currentRow.NRCONTRATOFCT;
        /* Referência Contrato */
        var RefeContrato = contrato.currentRow.DSREFERENCIACONT;
        /*Nome Contrato*/
        var NmContrato = CodContrato + ' - ' + RefeContrato;

        if (ck_rows) {
            widget.getField('CONTRATO').isVisible = false;

            widget.currentRow.NRSEQCONTRATO = null;
            widget.currentRow.CONTRATO = null;
            widget.currentRow.CKROWS = 'S';
        }
        else {

            widget.getField('CONTRATO').isVisible = true;

            widget.currentRow.NRSEQCONTRATO = SeqContrato;
            widget.currentRow.CONTRATO = NmContrato;
            widget.currentRow.CKROWS = 'N';
        }

        widget.currentRow.IDREAJUSTE = 'P';
        widget.currentRow.NRULTIMOREAJUSTE = contrato.currentRow.NRULTIMOREAJUSTE;
        widget.currentRow.DTPADREAJCONT = contrato.currentRow.DTPADREAJCONT;
        this.controlaTipoReajuste(widget);

    };

    /**
    * Salva o reajuste para os contratos.
    * 
    * @param {object} widget 
   */
    this.saveReajusteContr = function (widget) {
        if (widget.isValid()) {
            var contrato = templateManager.container.getWidget('gridContrato');

            if (widget.currentRow.CKROWS == 'S') {
                /* Realiza o reajuste quando são marcados vários contratos */
                var arrContratos = contrato.getCheckedRows();
                var rows = [];
                arrContratos.forEach(function (contReajuste) {
                    rows.push(this.geraObjetoDeParametroParaSalvarReajusteDoContrato(widget));
                });

                General.callRestEngine("product#/fct_contratoreajuste_widget/reajustarMultiplosContratos", "DataSet", rows)
                .then(function () {
                    this.aoSalvarReajusteDoContratoComSucesso(widget, contrato);
                }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
            }
            else {
                /* Realiza o reajuste quando para apenas um contrato */
                var params = this.geraObjetoDeParametroParaSalvarReajusteDoContrato(widget);

                General.callRestEngine('product#/fct_contratoreajuste_widget/reajustaContrato', "Row", params)
                .then(function () {
                    this.aoSalvarReajusteDoContratoComSucesso(widget, contrato);
                }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
            }
        }
    };

    /**
     * Procedimento responsável por retornar mensagem de sucesso ao salvar o reajuste de um contrato
     * @params {Widget} widget - Widget que foi salva
     * @params {Widget} contrato - Widget do grid de contratos 
    */
    this.aoSalvarReajusteDoContratoComSucesso = function (widget, contrato) {
        ScreenService.showMessage('O Reajuste foi realizado com sucesso!');
        ScreenService.closeSwipe();
        this.limparCamposWdgReajuste(widget);
        contrato.activate();
    };

    /**
     * Função responsável por retornar um objeto que é utilizado como parametro da operação de save do reajuste de um contrato.
     * Retorna nulo caso nao tenha sido possível criar um objeto.
     * @params {Widget} widget - Widget que contem o current row necessário para pegar os dados da tabela FCT_CONTRATOREAJUSTE 
    */
    this.geraObjetoDeParametroParaSalvarReajusteDoContrato = function (widget) {
        var params = {
            "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
            "NRITEMOCONTRATO": null,
            "IDREAJUSTE": widget.currentRow.IDREAJUSTE,
            "PERCENTUAL": widget.currentRow.VRPERCREAJUSTE,
            "CDTPINDICE": widget.currentRow.CDTPINDICE,
            "IDPERVALOR": widget.currentRow.IDPERVALOR,
            "CDINDICE": widget.currentRow.CDINDICE,
            "PERCINDICE": widget.currentRow.VRPERCINDICE,
            "VALORFIXO": null,
            "VRPERCVARIACAOIT": null,
            "NRULTIMOREAJUSTE": widget.currentRow.NRULTIMOREAJUSTE,
            "DTPADREAJCONT": widget.currentRow.DTPADREAJCONT,
            "DSJUSTIFICATIVA": widget.currentRow.DSJUSTIFICATIVA
        };
        return (params != null) ? params : null;
    };

    /**
     * Fecha a tela "Reajustar Valores do Contrato"
     * 
     * @param {object} widget widget "Reajustar Valores do Contrato"
     */
    this.fecharWdgReajustContr = function (widget) {
        ScreenService.closeSwipe().then(function () {
            this.limparCamposWdgReajuste(widget);
            templateManager.container.getWidget("gridContrato").activate();
        }.bind(this));
    };

    this.limparCamposWdgReajuste = function (widget) {
        widget.currentRow.VRPERCREAJUSTE = null;

        /* -- Tipo do índice -- */
        widget.currentRow.CDTPINDICE = null;

        /* -- Identificador Percentual -- */

        widget.currentRow.IDPERVALOR = null;

        /* -- Código do índice -- */
        widget.currentRow.CDINDICE = null;

        /* -- Índice Financeiro -- */
        widget.currentRow.NMINDICE = null;

        /* -- Percentual do Índice -- */
        widget.currentRow.VRPERCINDICE = null;

        /* -- valor Antigo -- */

        widget.currentRow.VRTOTITANT = null;

        /* -- valor Novo -- */
        widget.currentRow.VRTOTITNOVO = null;

        /* -- Percentua de Diferença -- */
        widget.currentRow.VRPERCVARIACAOIT = null;

        /* -- Justificativa -- */
        widget.currentRow.DSJUSTIFICATIVA = null;

        widget.isVisible = false;
    };

    /**
    * abre a tela "Reajustar Valores do Item"
   */
    this.abrirWdgReajustItemContr = function (ck_rows, nomeWidgetItens) {
        ScreenService.closePopup();
        var ReajustItemContr = templateManager.container.getWidget("reajustarItemContrato");
        ReajustItemContr.isVisible = true;
        ScreenService.openSwipe(ReajustItemContr).then(function () {
            ReajustItemContr.edit();
            this.setDadosPadraoReajustItemContr(ReajustItemContr, ck_rows, nomeWidgetItens);
        }.bind(this));
    };

    /**
     * Coloca a tela em modo de inserção
     * 
     * @param {object} widget widget "Reajustar Valores do Item"
    */
    this.setDadosPadraoReajustItemContr = function (widget, ck_rows, nomeWidgetItens) {

        var itens = templateManager.container.getWidget(nomeWidgetItens);

        /* Sequencial Contrato */
        var SeqContrato = itens.currentRow.NRSEQCONTRATO;
        /*Nome Contrato*/
        var NmContrato = null;
        /* Sequencial Item */
        var SeqItContrato = itens.currentRow.NRITEMOCONTRATO;
        /*Nome Item*/
        var NmItContrato = itens.currentRow.NMITEMCONTRATO;

        if (ck_rows) {
            widget.getField('NMITEMCONTRATO').isVisible = false;

            widget.currentRow.NRSEQCONTRATO = SeqContrato;
            widget.currentRow.CONTRATO = NmContrato;
            widget.currentRow.NRITEMOCONTRATO = null;
            widget.currentRow.NMITEMCONTRATO = null;
            widget.currentRow.CKROWS = 'S';

            widget.getField("VRTOTITANT").isVisible = false;
            widget.getField("VRPERCVARIACAOIT").isVisible = false;
        }
        else {
            widget.getField('NMITEMCONTRATO').isVisible = true;

            widget.currentRow.NRSEQCONTRATO = SeqContrato;
            widget.currentRow.CONTRATO = NmContrato;
            widget.currentRow.NRITEMOCONTRATO = SeqItContrato;
            widget.currentRow.NMITEMCONTRATO = NmItContrato;
            widget.currentRow.CKROWS = 'N';

            widget.getField("VRTOTITANT").isVisible = true;
            widget.getField("VRPERCVARIACAOIT").isVisible = true;
        }

        widget.currentRow.IDREAJUSTE = 'P';
        widget.currentRow.WDGITENS = nomeWidgetItens;
        this.controlaTipoReajuste(widget, nomeWidgetItens);
    };

    /**
     * Salva o reajuste para os itens do contrato.
     * 
     * @param {object} widget 
    */
    this.saveReajusteItContr = function (widget) {
        if (widget.isValid()) {
            var wdgContrato = templateManager.container.getWidget("gridContrato");
            var wdgItens = templateManager.container.getWidget(widget.currentRow.WDGITENS);

            var rows = [];
            if (widget.currentRow.CKROWS == 'S') {
                wdgItens.getCheckedRows().forEach(function (item) {
                    rows.push({
                        "NRITEMOCONTRATO": item.NRITEMOCONTRATO,
                        "NRITEMCONTRATOPAI": item.NRITEMCONTRATOPAI,
                        "VRUNITITEMCONT": item.VRUNITITEMCONT,
                        "QTMEDIDACONTR": item.QTMEDIDACONTR,
                        "QTITEMCONTRATO": item.QTITEMCONTRATO,
                        "VRTOTITEMCONT": item.VRTOTITEMCONT,
                        "NRNEGOCIACAOCONTRATO": item.NRNEGOCIACAOCONTRATO,
                        "NRITNEGOCIACAOCONTRATO": item.NRITNEGOCIACAOCONTRATO,
                    });
                });
            }
            else {
                rows.push({
                    "NRITEMOCONTRATO": wdgItens.currentRow.NRITEMOCONTRATO,
                    "NRITEMCONTRATOPAI": wdgItens.currentRow.NRITEMCONTRATOPAI,
                    "VRUNITITEMCONT": wdgItens.currentRow.VRUNITITEMCONT,
                    "QTMEDIDACONTR": wdgItens.currentRow.QTMEDIDACONTR,
                    "QTITEMCONTRATO": wdgItens.currentRow.QTITEMCONTRATO,
                    "VRTOTITEMCONT": wdgItens.currentRow.VRTOTITEMCONT,
                    "NRNEGOCIACAOCONTRATO": wdgItens.currentRow.NRNEGOCIACAOCONTRATO,
                    "NRITNEGOCIACAOCONTRATO": wdgItens.currentRow.NRITNEGOCIACAOCONTRATO
                });
            }

            var params = {
                "NRSEQCONTRATO": wdgContrato.currentRow.NRSEQCONTRATO,
                "IDREAJUSTE": widget.currentRow.IDREAJUSTE,
                "PERCENTUAL": widget.currentRow.VRPERCREAJUSTE,
                "CDTPINDICE": widget.currentRow.CDTPINDICE,
                "IDPERVALOR": widget.currentRow.IDPERVALOR,
                "CDINDICE": widget.currentRow.CDINDICE,
                "PERCINDICE": widget.currentRow.VRPERCINDICE,
                "DSJUSTIFICATIVA": widget.currentRow.DSJUSTIFICATIVA,
                "VALORFIXO": widget.currentRow.VRTOTITNOVO,
                "VRPERCVARIACAOIT": widget.currentRow.VRPERCVARIACAOIT,
                "UTILIZACOMPOSICAO": (wdgContrato.currentRow.QTITENSTRABSERV > 0) ? "S" : "N",
                "ITENS": rows
            };

            General.callRestEngine("product#/fct_contratoreajuste_widget/reajustarItensContrato", "Row", params)
            .then(function () {
                ScreenService.showMessage('O Reajuste foi realizado com sucesso!');
                ScreenService.closeSwipe();
                this.limparCamposWdgReajuste(widget);
                wdgItens.activate();
                wdgItens.unSelectAll();
                wdgItens.reload();
            }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
        }
    };

    /**
    * Fecha a tela "Reajustar Valores do Item"
    * 
     * @param {object} widget widget "Reajustar Valores do Item"
    */
    this.fecharWdgReajustItemContr = function (widget) {
        ScreenService.closeSwipe().then(function () {
            this.limparCamposWdgReajuste(widget);

            var wdgItens = templateManager.container.getWidget(widget.currentRow.WDGITENS);
            wdgItens.activate();
        }.bind(this));
    };

    /**
     * Realiza o controle de Percentual de reajuste do field "VRPERCREAJUSTE"
     * 
     * @param {object} widget widget de 
     * reajsutar valores do contrato para pegar o field "VRPERCREAJUSTE".
    */

    this.validaPercentReajust = function (widget) {

        var vrdesconto = widget.getField('VRPERCREAJUSTE').value();

        /* Aplica Max e Min Value*/

        if (vrdesconto < -100 || vrdesconto > 100) {
            /* Desconto */
            widget.currentRow.VRPERCREAJUSTE = null;
        }
    };

    /**
     * Realiza o controle de Percentual de reajuste do field "VRPERCINDICE"
     * 
     * @param {object} widget widget de 
     * reajsutar valores do contrato para pegar o field "VRPERCINDICE".
    */

    this.validaPercentReajustIndFin = function (widget) {

        var vrdesconto = widget.getField('VRPERCINDICE').value();

        /* Aplica Max e Min Value*/

        if (vrdesconto < -100 || vrdesconto > 100) {
            /* Desconto */
            widget.currentRow.VRPERCINDICE = null;
        }
    };

    /**
     * Realiza o controle dos campos de "Percentual" e "índice Financeiro" 
     * do Reajuste.
     * 
     * @param {object} widget widget de 
     * reajsutar valores do contrato para pegar o field "IDREAJUSTE".
    */

    this.controlaTipoReajuste = function (widget, nomeWidgetItens) {

        var tiporeajuste = widget.getField('IDREAJUSTE').value();

        if (tiporeajuste == 'P') {

            /* -- Percentual para reajuste -- */
            widget.getField('VRPERCREAJUSTE').readOnly = false;
            widget.getField('VRPERCREAJUSTE').validations = { "required": true };

            /* -- Tipo do índice -- */
            widget.getField('CDTPINDICE').readOnly = true;
            widget.getField('CDTPINDICE').validations = {};
            widget.getField('CDTPINDICE').value(null);

            /* -- Identificador Percentual -- */
            widget.getField('IDPERVALOR').readOnly = true;
            widget.getField('IDPERVALOR').validations = {};
            widget.getField('IDPERVALOR').value(null);

            /* -- Código do índice -- */
            widget.getField('CDINDICE').readOnly = true;
            widget.getField('CDINDICE').validations = {};
            widget.getField('CDINDICE').value(null);

            /* -- Índice Financeiro -- */
            widget.getField('NMINDICE').readOnly = true;
            widget.getField('NMINDICE').validations = {};
            widget.getField('NMINDICE').value(null);

            /* -- Percentual do Índice -- */
            widget.getField('VRPERCINDICE').readOnly = true;
            widget.getField('VRPERCINDICE').validations = {};
            widget.getField('VRPERCINDICE').value(null);

            /* -- valor Antigo -- */
            widget.getField('VRTOTITANT').readOnly = true;
            widget.getField('VRTOTITANT').validations = {};
            widget.getField('VRTOTITANT').value(null);

            /* -- valor Novo -- */
            widget.getField('VRTOTITNOVO').readOnly = true;
            widget.getField('VRTOTITNOVO').validations = {};
            widget.getField('VRTOTITNOVO').value(null);

            /* -- Percentua de Diferença -- */
            widget.getField('VRPERCVARIACAOIT').readOnly = true;
            widget.getField('VRPERCVARIACAOIT').validations = {};
            widget.getField('VRPERCVARIACAOIT').value(null);
        }
        else if (tiporeajuste == 'I') {

            /* -- Percentual para reajuste -- */
            widget.getField('VRPERCREAJUSTE').readOnly = true;
            widget.getField('VRPERCREAJUSTE').validations = {};
            widget.getField('VRPERCREAJUSTE').value(null);

            /* -- Tipo do índice -- */
            widget.getField('CDTPINDICE').readOnly = true;
            widget.getField('CDTPINDICE').validations = { "required": true };

            /* -- Identificador Percentual -- */
            widget.getField('IDPERVALOR').readOnly = true;
            widget.getField('IDPERVALOR').validations = { "required": true };

            /* -- Código do índice -- */
            widget.getField('CDINDICE').readOnly = true;
            widget.getField('CDINDICE').validations = { "required": true };

            /* -- Índice Financeiro -- */
            widget.getField('NMINDICE').readOnly = false;
            widget.getField('NMINDICE').validations = { "required": true };

            /* -- Percentual do Índice -- */
            widget.getField('VRPERCINDICE').readOnly = true;
            widget.getField('VRPERCINDICE').validations = { "required": true };

            /* -- valor Antigo -- */
            widget.getField('VRTOTITANT').readOnly = true;
            widget.getField('VRTOTITANT').validations = {};
            widget.getField('VRTOTITANT').value(null);

            /* -- valor Novo -- */
            widget.getField('VRTOTITNOVO').readOnly = true;
            widget.getField('VRTOTITNOVO').validations = {};
            widget.getField('VRTOTITNOVO').value(null);

            /* -- Percentua de Diferença -- */
            widget.getField('VRPERCVARIACAOIT').readOnly = true;
            widget.getField('VRPERCVARIACAOIT').validations = {};
            widget.getField('VRPERCVARIACAOIT').value(null);

        }
        else {
            var itemContrato = templateManager.container.getWidget(nomeWidgetItens);

            /* -- Percentual para reajuste -- */
            widget.getField('VRPERCREAJUSTE').readOnly = true;
            widget.getField('VRPERCREAJUSTE').validations = {};
            widget.getField('VRPERCREAJUSTE').value(null);

            /* -- Tipo do índice -- */
            widget.getField('CDTPINDICE').readOnly = true;
            widget.getField('CDTPINDICE').validations = {};
            widget.getField('CDTPINDICE').value(null);

            /* -- Identificador Percentual -- */
            widget.getField('IDPERVALOR').readOnly = true;
            widget.getField('IDPERVALOR').validations = {};
            widget.getField('IDPERVALOR').value(null);

            /* -- Código do índice -- */
            widget.getField('CDINDICE').readOnly = true;
            widget.getField('CDINDICE').validations = {};
            widget.getField('CDINDICE').value(null);

            /* -- Índice Financeiro -- */
            widget.getField('NMINDICE').readOnly = true;
            widget.getField('NMINDICE').validations = {};
            widget.getField('NMINDICE').value(null);

            /* -- Percentual do Índice -- */
            widget.getField('VRPERCINDICE').readOnly = true;
            widget.getField('VRPERCINDICE').validations = {};
            widget.getField('VRPERCINDICE').value(null);

            /* -- valor Antigo -- */
            widget.getField('VRTOTITANT').readOnly = true;
            //widget.getField('VRTOTITANT').validations = {"required": true};
            widget.currentRow.VRTOTITANT = itemContrato.currentRow.VRUNITITEMCONT;

            /* -- valor Novo -- */
            widget.getField('VRTOTITNOVO').readOnly = false;
            widget.getField('VRTOTITNOVO').validations = { "required": true };

            /* -- Percentua de Diferença -- */
            widget.getField('VRPERCVARIACAOIT').readOnly = true;
            //widget.getField('VRPERCVARIACAOIT').validations = {"required": true};

        }
    };

    /**
    * Calcula o percentual de Diferença do item.
    * 
    * @param {object} widget 
   */

    this.calculaPercentualDiffItem = function (widget) {
        var itemContrato = templateManager.container.getWidget(widget.currentRow.WDGITENS);

        var itAntigo = itemContrato.currentRow.VRUNITITEMCONT;
        var itnovo = widget.currentRow.VRTOTITNOVO;

        var vrDiff = (itnovo * 100) / itAntigo;

        vrDiff = vrDiff - 100;

        widget.currentRow.VRPERCVARIACAOIT = vrDiff;

    };

    this.validarAbreSwipeItens = function (widget) {
        var isVisible = widget.currentRow.IDTIPO === "M";
        //var isVisibleAgrup = (widget.currentRow.IDTIPO !== "A" && widget.currentRow.IDTIPO !== "M");

        widget.getField("QTMEDIDACONTR").isVisible = isVisible;
        //widget.getField("NMESTRUTURAM").isVisible            = isVisibleAgrup;
        //widget.getField("NMDOCITEMCONT").isVisible           = isVisibleAgrup;
        widget.getField("VRUNITITEMCONTTBLPRECO").showOnView = isVisible;
        widget.getField("VRUNITITEMCONTTBLPRECO").showOnForm = isVisible;
        widget.getField("VRUNITITEMCONT").showOnView = isVisible;
        widget.getField("VRUNITITEMCONT").showOnForm = isVisible;

        widget.getField("IDMULTMEDVRTOT").isVisible = isVisible;
        widget.getField("DSTPMEDIDASERV").showOnView = isVisible;
        widget.getField("DSTPMEDIDASERV").showOnForm = isVisible;

        if (bloqReajuste == 'S') {
            widget.getAction("reajustar_item").isVisible = false;
        }
        else {

            widget.getAction("reajustar_item").isVisible = widget.currentRow.NRITEMCONTRATOPAI != null;
        }


    };

    this.validarAbaItens = function (wdgContrato) {
        var wdgItens = templateManager.container.getWidget("itemcontrato");
        var wdgItensTree = templateManager.container.getWidget("wdgItensContratoTree");

        var params = {
            "NRSEQCONTRATO": wdgContrato.currentRow.NRSEQCONTRATO
        };

        General.callRestEngine("product#/fct_contrato_widget/getUtilizaComposicaoServico", "Row", params).then(function (response) {
            utilizaComposicao = response.dataset.retorno.utilizaComposicao;
            wdgItens.isVisible = !utilizaComposicao;
            wdgItensTree.isVisible = utilizaComposicao;
        })
        .catch(function (error) {
            Utilities.showErrorMessage(error);
        });

    };


    this.validaReajusteContr = function (widget) {
        var widItens = templateManager.container.getWidget('itemcontrato');
        var widItensTree = templateManager.container.getWidget('wdgItensContratoTree');
        var params = [];

        General.callRestEngine("product#/fct_contratoreajuste_widget/validaBloqReajusteContrato", "Row", params).then(function (response) {
            if (response.dataset.retorno.valida == 'BLOQUEAR') {
                bloqReajuste = 'S';

                widget.getAction("reajustarContrato").isVisible = false;
                widget.getAction("reajustarContrato_ck_rows").isVisible = false;

                widItens.getAction("reajustarItemContrato").isVisible = false;
                widItens.getAction("reajustarItemContrato_ck_rows").isVisible = false;

                widItensTree.getAction("reajustar_itens").isVisible = false;
                widItensTree.getAction("reajustar_item").isVisible = false;


                ScreenService.showMessage("Atenção!<br><br>Não será possível realizar o reajuste dos contratos.<br><br>Existem índices financeiros com cadastro de alíquota pendentes. Estes cadastros devem ser regularizados antes da realização dos reajustes dos contratos.");
            } else {
                bloqReajuste = 'N';

                widget.getAction("reajustarContrato").isVisible = true;
                widget.getAction("reajustarContrato_ck_rows").isVisible = true;

                widItens.getAction("reajustarItemContrato").isVisible = true;
                widItens.getAction("reajustarItemContrato_ck_rows").isVisible = true;

                widItensTree.getAction("reajustar_itens").isVisible = true;
                widItensTree.getAction("reajustar_item").isVisible = true;

            }
        }.bind(this))
        .catch(function (error) {
            Utilities.showErrorMessage(error);
        });
    };

    this.validaReajusteContrIndividual = function () {
        var promiseBloqReajInd = ZHPromise.defer();
        var wdgContrato = templateManager.container.getWidget('gridContrato');

        var params = {
            "NRSEQCONTRATO": wdgContrato.currentRow.NRSEQCONTRATO,
            "NRTIPOORCAMENTO": wdgContrato.currentRow.NRTIPOORCAMENTO
        };

        General.callRestEngine("product#/fct_contratoreajuste_widget/validaBloqReajusteContrIndividual", "Row", params).then(function (response) {

            if (response.dataset.retorno.valida == 'BLOQUEAR') {

                promiseBloqReajInd.resolve('S');

                ScreenService.showMessage("Atenção!<br><br>Não será possível realizar o reajuste do contrato. <br><br>Existem índices financeiros com cadastro de alíquota pendentes. Estes cadastros devem ser regularizados antes da realização do reajuste do contrato.");
            }
            else if (response.dataset.retorno.valida == 'BLOQ_PERC_CONTR') {

                promiseBloqReajInd.resolve('S');

                ScreenService.showMessage("Atenção!<br><br>Não será possível realizar o reajuste do contrato. <br><br> A soma dos percentuais a serem considerados para os índices de reajuste do contrato parametrizados na página 'Índices Financeiros Padrões' da tela 'Lançamento de contrato' para o contrato selecionado não está totalizando 100%. Estes cadastros devem ser regularizados antes da realização do reajuste do contrato.");

            }
            else if (response.dataset.retorno.valida == 'BLOQ_PERC_ESCOPO') {

                promiseBloqReajInd.resolve('S');

                ScreenService.showMessage("Atenção!<br><br>Não será possível realizar o reajuste do contrato. <br><br> A soma dos percentuais a serem considerados para os índices de reajuste do contrato parametrizados na página 'Índices Financeiros Padrões' da tela 'Escopo de Faturamento' para o escopo vinculado ao contrato selecionado não está totalizando 100%. Estes cadastros devem ser regularizados antes da realização do reajuste do contrato.");

            }
            else if (response.dataset.retorno.valida == 'BLOQ_PERC_PARAM') {

                promiseBloqReajInd.resolve('S');

                ScreenService.showMessage("Atenção!<br><br>Não será possível realizar o reajuste do contrato. <br><br> A soma dos percentuais a serem considerados para os índices de reajuste do contrato parametrizados na página 'Índices Financeiros Padrões' da tela 'Parametrização SM' não está totalizando 100%. Estes cadastros devem ser regularizados antes da realização do reajuste do contrato.");

            }
            else if (response.dataset.retorno.valida == 'BLOQ_SEM_PARAM') {

                promiseBloqReajInd.resolve('S');

                ScreenService.showMessage("Atenção!<br><br>Não será possível realizar o reajuste do contrato. <br><br>Não foi identificado o cadastro dos índices financeiros padrões para a realização do reajuste. Estes cadastros devem ser regularizados antes da realização do reajuste do contrato.");

            }
            else {
                promiseBloqReajInd.resolve('N');
            }

        }.bind(this))
        .catch(function (error) {
            Utilities.showErrorMessage(error);
        });

        return promiseBloqReajInd.promise;

    };

    this.abrirPopupTipoReajusteEmMassa = function () {
        var wdgTipoReajusteEmMassa = templateManager.container.getWidget("wdgTipoReajusteEmMassa");
        wdgTipoReajusteEmMassa.dataSource.data = [];
        //wdgTipoReajusteEmMassa.currentRow = {};
        wdgTipoReajusteEmMassa.isVisible = true;
        wdgTipoReajusteEmMassa.newRow();
        ScreenService.openPopup(wdgTipoReajusteEmMassa).then(function () {
            this.popupTipoReajusteOnChange(wdgTipoReajusteEmMassa);
        }.bind(this));
    };

    this.confirmarTipoReajusteEmMassa = function (wdgTipoReajusteEmMassa) {
        if (wdgTipoReajusteEmMassa.isValid()) {
            var tipoReajuste = wdgTipoReajusteEmMassa.currentRow.TIPOREAJUSTE;
            var vrPercentual = null;
            var dsjustificativa = wdgTipoReajusteEmMassa.currentRow.DSJUSTIFICATIVA;

            if (tipoReajuste == "P") {
                vrPercentual = wdgTipoReajusteEmMassa.currentRow.VRPERCENTUAL;
            }

            var gridContrato = templateManager.container.getWidget('gridContrato');
            var checkedRows = gridContrato.getCheckedRows();

            if (vrPercentual === null) {
                var params = {
                    "CONTRATOS": checkedRows
                };
                General.callRestEngine("product#/fct_contratoreajuste_widget/validaBloqReajusteContrato", "Row", params).then(function (response) {
                    if (response.dataset.retorno.valida == 'BLOQUEAR') {
                        ScreenService.showMessage("Atenção!<br><br>Não será possível realizar o reajuste dos contratos.<br><br>Existem índices financeiros com cadastro de alíquota pendentes. Estes cadastros devem ser regularizados antes da realização dos reajustes dos contratos.");
                    }
                    else if (response.dataset.retorno.valida == 'INDIVIDUAL') {
                        ScreenService.showMessage("Atenção!<br><br>Não será possível realizar o reajuste dos contratos.<br><br> Alguns dos contratos selecionados possuem índices financeiros com cadastro de alíquota pendentes. Estes cadastros devem ser regularizados antes da realização dos reajustes dos contratos.");
                    }
                    else if (response.dataset.retorno.valida == 'BLOQ_SEM_PARAM') {
                        ScreenService.showMessage("Atenção!<br><br>Não será possível realizar o reajuste dos contratos.<br><br> Não há cadastro de índices financeiros para reajustar estes contratos.");
                    }
                    else {
                        ScreenService.closePopup();
                        this.reajusteContrMassa(checkedRows, vrPercentual, dsjustificativa);
                    }
                }.bind(this))
                .catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
            }
            else {
                ScreenService.closePopup();
                this.reajusteContrMassa(checkedRows, vrPercentual, dsjustificativa);
            }
        }
    };

    this.reajusteContrMassa = function (checkedRows, vrPercentual, dsjustificativa) {
        if (dsjustificativa === undefined) {
            dsjustificativa = null;
        }
        
        var nrorg = '';
        var contratos = '_';

        checkedRows.forEach(function (contrato) {
            contratos = contratos + contrato.NRSEQCONTRATO + '_';
            nrorg = contrato.NRORG;
        });

        var wdgReajContrMassa = templateManager.container.getWidget('wdgReajContrMassa');
        wdgReajContrMassa.isVisible = true;
        wdgReajContrMassa.dataSourceFilter[0].value = contratos;
        wdgReajContrMassa.dataSourceFilter[1].value = nrorg;
        wdgReajContrMassa.dataSourceFilter[3].value = dsjustificativa;

        if (vrPercentual !== null || vrPercentual !== undefined) {
            wdgReajContrMassa.dataSourceFilter[2].operator = "=";
            wdgReajContrMassa.dataSourceFilter[2].value = vrPercentual;
        }
        else {
            wdgReajContrMassa.dataSourceFilter[2].operator = "IS_NULL";
            delete wdgReajContrMassa.dataSourceFilter[2].value;
        }

        ScreenService.openPopup(wdgReajContrMassa).then(function () {
            $('.zh-new-grid-wdgReajContrMassa').parent().parent().parent().parent().parent().parent().addClass('cp-high-modal');
        });

        wdgReajContrMassa.reload();
    };

    /**
     * Fecha a widget wdgReajContrMassa 
    **/
    this.fecharReajusteContrMassa = function () {
        $('.zh-new-grid-wdgReajContrMassa').parent().parent().parent().parent().parent().parent().removeClass('cp-high-modal');

        var wdgReajContrMassa = templateManager.container.getWidget('wdgReajContrMassa');
        wdgReajContrMassa.isVisible = true;
        ScreenService.closePopup();

    };

    /**
     * Reajusta os contratos selecionados
     * 
     * @params{Object} widget - widget de reajustar contratos em massa
    **/
    this.ConfirmarReajusteContrMassa = function (widget) {
        this.validarPercentualReajuste(widget).then(function () {
            var dados = [widget.dataSource.data];
            this.ConfirmarReajusteContrMassaAux(dados);
        }.bind(this));
    };

    this.ConfirmarReajusteContrMassaAux = function (dados) {

        if (dados.length > 0) {
            var timeout = 30 * 60000; //30 min
            var dadosAux = dados.splice(0, 10);
            Utilities.callRestEngineWithTimeout("product#/fct_contratoreajuste_widget/reajusteContrMassa", "Row", dadosAux, timeout).then(function () {
                this.ConfirmarReajusteContrMassaAux(dados);
            }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
        }
        else {
            ScreenService.showMessage('O Reajuste foi realizado com sucesso!');
            this.fecharReajusteContrMassa();
        }
    };

    /**
     * Valida se todos os contratos possuem a soma do percetual de 
     * utilização dos indices igual a 100%
     * 
     * @params{Object} widget - widget de reajuste em massa
    **/
    this.validarPercentualReajuste = function (widget) {
        var deferred = ZHPromise.defer();
        var contratosValidados = [];
        var contratosInvalidos = [];

        widget.dataSource.data.forEach(function (contrato, index) {
            if (!contratosValidados.includes(contrato.NRSEQCONTRATO)) {
                var percentualcont = contrato.NRPERCENTINDICE;
                widget.dataSource.data.forEach(function (data) {
                    if (contrato.NRSEQCONTRATO == data.NRSEQCONTRATO && contrato.CDINDICE != data.CDINDICE)
                        percentualcont += data.NRPERCENTINDICE;
                });

                contratosValidados.push(contrato.NRSEQCONTRATO);
                if (percentualcont != 100) {
                    contratosInvalidos.push({
                        "NRSEQCONTRATO": contrato.NRSEQCONTRATO,
                        "index": index
                    });
                }
            }
        });

        if (contratosInvalidos.length == 0) {
            deferred.resolve();
        } else {
            ScreenService.confirmMessage("Atenção!<br><br>Os contratos selecionados que não possuem somatório de 100% no percentual de utilização dos índices não serão reajustados. Deseja continuar?", "question", function () {
                contratosInvalidos.forEach(function (contrato) {
                    widget.dataSource.data.splice(contrato.index, 1);
                });

                deferred.resolve();
            }, function () {
                deferred.reject();
            });
        }

        return deferred.promise;
    };

    this.popupTipoReajusteOnChange = function (wdgTipoReajusteEmMassa) {
        var tipoReajuste = wdgTipoReajusteEmMassa.currentRow.TIPOREAJUSTE;
        if (tipoReajuste == "P") {
            wdgTipoReajusteEmMassa.getField("VRPERCENTUAL").readOnly = false;
            wdgTipoReajusteEmMassa.getField("VRPERCENTUAL").validations = { "required": true };
        }
        else {
            wdgTipoReajusteEmMassa.currentRow.VRPERCENTUAL = "";
            wdgTipoReajusteEmMassa.getField("VRPERCENTUAL").readOnly = true;
            wdgTipoReajusteEmMassa.getField("VRPERCENTUAL").validations = {};
        }
        templateManager.updateTemplate();
    };

    this.wdgNegociacaoValoresAfterMoveRow = function (wdgNegociacaoValores) {
        var idtpfatura = wdgNegociacaoValores.currentRow.IDTPFATURA;
        var wdgParcelas = templateManager.container.getWidget("wdgParcelas");
        var wdgDescontoPorPeriodo = templateManager.container.getWidget("wdgDescontoPorPeriodo");
        var wdgCompetenciaNegociacao = templateManager.container.getWidget("wdgCompetenciaNegociacao");

        if (idtpfatura == "P") {
            wdgParcelas.isVisible = true;
            wdgDescontoPorPeriodo.isVisible = false;
            wdgCompetenciaNegociacao.isVisible = false;
        }
        else {
            wdgParcelas.isVisible = false;
            wdgDescontoPorPeriodo.isVisible = true;
            wdgCompetenciaNegociacao.isVisible = true;
        }
    };

    this.wdgItensNegociacaoAfterMoveRow = function (wdgItensNegociacao) {
        var campoValorItem = wdgItensNegociacao.getField("VRTOTITEM");
        var campoPercDesconto = wdgItensNegociacao.getField("VRPERCDESCITEM");
        var campoValorDesconto = wdgItensNegociacao.getField("VRDESCITEM");
        var campoTipoDesconto = wdgItensNegociacao.getField("IDTPDESCONTO");
        var campoTipoMedida = wdgItensNegociacao.getField("DSTPMEDIDASERV");

        var dadosFaturamento = this.pegarDadosDeFaturamentoGroup(wdgItensNegociacao)[0];

        if (wdgItensNegociacao.currentRow.NRITEMCONTRATOPAI == null && utilizaComposicao == true) {
            dadosFaturamento.isVisible = true;
            campoValorItem.showOnView = false;
            campoValorItem.showOnForm = false;
            campoPercDesconto.showOnView = false;
            campoPercDesconto.showOnForm = false;
            campoValorDesconto.showOnView = false;
            campoValorDesconto.showOnForm = false;
            campoTipoDesconto.isVisible = false;
            campoTipoMedida.showOnView = false;
            campoTipoMedida.showOnForm = false;
        }
        else {
            campoValorItem.showOnView = true;
            campoValorItem.showOnForm = true;
            campoPercDesconto.showOnView = true;
            campoPercDesconto.showOnForm = true;
            campoValorDesconto.showOnView = true;
            campoValorDesconto.showOnForm = true;
            campoTipoDesconto.isVisible = true;
            campoTipoMedida.showOnView = true;
            campoTipoMedida.showOnForm = true;
        }

        dadosFaturamento.isVisible = (wdgItensNegociacao.currentRow._parent == null);
    };

    this.pegarDadosDeFaturamentoGroup = function (widget) {
        return widget.fieldGroups.filter(function (wid) {
            return (wid.name == "fdgProdutosFaturamento");
        });
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00004003_Reajuste_Contrato_Controller", Sm00004003_Reajuste_Contrato_Controller);
});

// FILE: ./assets/js/controllers/Sm00004004_Reajuste_Contrato_Formacao_Preco_Controller.js
function Sm00004004_Reajuste_Contrato_Formacao_Preco_Controller(General, ScreenService, templateManager, ZHPromise ) {
    /**
     * A função "filtraItFormPreco" filtra o campo "Itens da Formação de Preço"
     * 
     * @param {object} widget widget de 
     * Reajusta Elemento
     */
    this.selecionaPorTipo = function(row, tipo){
        var widget = templateManager.container.getActiveWidget(); 
        if (row._parent && row._parent.__isSelected) {
            widget.dataSource.unSelectAll([row]);
        }
        else if (row._childCount > 0) {
            var filhosMarcados = widget.dataSource.data.filter(function(linha) {
                return linha.VALORPAI == row.VALORORIGINAL && linha.__isSelected;
            });
            
            if (filhosMarcados.length > 0) {
                widget.dataSource.unSelectAll([row]);
            }
        }
    };
     
    /**
     * A função "filtraItFormPreco" filtra o campo "Itens da Formação de Preço"
     * 
     * @param {object} widget widget de 
     * Reajusta Elemento
     */
    this.filtraItemFormPreco = function(widget){
        
        var formPrecoReaj = widget.currentRow.NRREAJUSCONTFORMPREC;
        var formPreco = widget.currentRow.NRFORMPRECOSERVICO;
       
        widget.getField('NRCALCFORMPRECSERV').dataSourceFilter[0].value = formPrecoReaj;
        widget.getField('NRCALCFORMPRECSERV').dataSourceFilter[1].value = formPreco;
      
        widget.getField('NRCALCFORMPRECSERV').reload();
        
    };
     /**
     * A função "filtraItemFormPrecoSunrch" filtra o campo "Itens da Formação de Preço do slect sunrch"
     * 
     * @param {object} widget widget de 
     * Reajusta Elemento surnch
     */
     
    this.filtraItemFormPrecoSurch = function(widget){
    
        this.buscarNumeroFormacoes().then(function(response) {
            var filtro = response.dataset.dados;
                widget.getField('VALORORIGINAL').dataSourceFilter[0].value = filtro;
        
                widget.getField('VALORORIGINAL').reload();
            });
        
    };
    /**
     * Centraliza metodos do Evento WidgetAfterUnSelectAll
     * 
     * @param {object} widget widget de 
     * Reajusta Elemento
     */
     

    this.MetodosUnchecks = function(widget){
    this.desativaAcao();
    this.uncheckFilhos(widget);
    }; 
   
     /**
     * Desmarca os checkedBox das Rows que são finhas de formações originais 
     * 
     * @param {object} widget widget de 
     * Reajusta Elemento
     */
     

    this.uncheckFilhos = function(widget){
    var rows = widget.getCheckedRows();
    if(rows)
    rows.forEach(function(row){
        if(row.VALORPAI != null){
            widget.unSelectAll([row]) }});

    };
   
    
    /**
     * A função "filtraItFormPreco" filtra o campo "Itens da Formação de Preço
     * 
     * @param {object} widget widget de 
     * Reajusta Elemento" 
     */
     

    this.bloqueiReajuste = function(widget){
     var contrato = templateManager.container.getWidget("gridReajuste");
     var row = contrato.getCheckedRows();
    var ultimoReajustes = Array();
    var string;
    var i = 1; 
    var completa="";
     if(row["VALORPAI"] == null){
     
      this.buscarUltimosReajustes().then(function(response) {
          ultimoReajustes = response.dataset.dados;    
          ultimoReajustes.forEach(function(ultimo){
            
            if(i!=ultimoReajustes.length){
                 string ="row.NRREAJUSCONTFORMPREC == "+ultimo+" || ";
            }else{
                 string ="row.NRREAJUSCONTFORMPREC == "+ultimo;
            }
        
            completa += string;
           i++;
          });
           var styles = [
              {
                "condition": "row.VALORPAI != null",
                "cssClass": "hideCheckBoxPai"
              },
              {
                  "condition": completa,
                  "cssClass": "showCheckBoxFilho"
              }
          ];
          contrato.styles = styles;
      });
         }
        
    };
    
    
   /**
    * Abri a ação de reajuste de Formação de preço
    * 
    * @param {bolean} da função chamadora 
    */
    this.abrirWdgReajustarFormacoesDePreco = function(abrir) {
        var gridReajuste = templateManager.container.getWidget("reajustarFormacoesDePreco");
       
            gridReajuste.isVisible = true;
            ScreenService.openPopup(gridReajuste).then(function() {
                    gridReajuste.newRow();
            }, function(){
            });
        
    };
    /**
    * Abri a ação de reajuste de Formação de preço automático
    * 
    * @param {bolean} da função chamadora  
    */
    this.abrirWdgReajusteAutomatico = function(abrir) {
        var gridReajusteAutomatico = templateManager.container.getWidget("WdgReajusteAutomatico");
       
            gridReajusteAutomatico.isVisible = true;
            ScreenService.openPopup(gridReajusteAutomatico).then(function() {
                    gridReajusteAutomatico.newRow();
            }, function(){
            });
        
    };
    /**
    * Desativa a ação padrão se o checkbox estiver selecionado e ativa outra ação
    * 
    * @param {nulo}  
    */
    this.desativaAcao = function() {
        var gridReajuste = templateManager.container.getWidget("gridReajuste");
        var checado = gridReajuste.getCheckedRows();
        if(checado.length == 0){
        gridReajuste.getAction("reajusteAutomatico").isVisible = true;
        }else{
        gridReajuste.getAction("reajusteAutomatico").isVisible = false;
        }
            
        
    };
    /**
    * Fehcar e cancela a ação de reajuste de preço
    * 
    * @param {bolean} da função chamadora  
    */
    this.fecharWdgReajustarFormacoesDePreco = function(fechar) {
        var gridReajuste = templateManager.container.getWidget("reajustarFormacoesDePreco");
        var wdgReajuste = templateManager.container.getWidget("gridReajuste");
       
        gridReajuste.isVisible = fechar;
        ScreenService.closePopup(gridReajuste);
        wdgReajuste.reload();
        
    };
    /**
    * Fehcar e cancela a ação de reajuste de preço automático
    * 
    * @param {bolean} da função chamadora   
    */
    this.fecharWdgReajustarAutomatico = function(fechar) {
        var gridReajusteAut = templateManager.container.getWidget("WdgReajusteAutomatico");
        var wdgReajuste = templateManager.container.getWidget("gridReajuste");
       
        gridReajusteAut.isVisible = fechar;
        ScreenService.closePopup(gridReajusteAut);
        wdgReajuste.reload();
    };
    /**
    * Fecha a ação de reajuste após conclusão da ação
    * 
    * @param {bolean} da função chamadora   
    */
    this.fecharWdgAposReajusteElemento = function(Desativar,tipoTela) {
        var gridReajuste = templateManager.container.getWidget("gridReajuste");
        var gridCalculo;
         if(tipoTela == "A"){
            gridCalculo = templateManager.container.getWidget("gridCalculo");
        }else
        if(tipoTela == "B"){
             gridCalculo = templateManager.container.getWidget("gridCalculoFilho");
            }
       
        gridCalculo.getAction("aplicar").isVisible = Desativar;
        gridCalculo.getAction("ADD").isVisible = !Desativar;
        gridCalculo.getAction("BACK_VIEW").isVisible = !Desativar;
         if(tipoTela == "A"){
            gridCalculo.dataSource.name = "product#/fct_calcformprecserv_widget";
        }else
        if(tipoTela == "B"){
             gridCalculo.dataSource.name = "product#/crud/fct_calcreajformprec_widget";
             gridCalculo.parentFilter.refkeys = [
                            "VALORORIGINAL",
                            "NRORG"
                        ];
            gridCalculo.parentFilter.parentkeys = [
                            "NRREAJUSCONTFORMPREC",
                            "NRORG"
                        ];
            }
        ScreenService.closeSwipe();
        gridReajuste.reload();
        
    };
    /**
    * Cancela e desfaz toda a operação de de reajuste por elemento
    * 
    * @param {object} widget widget de 
    * Reajusta Elemento
    */
    this.cancelarAplicacao = function(widget) {
        var WdgReajustarElemento = templateManager.container.getWidget("WdgReajustarElemento");
        var tipoTela = WdgReajustarElemento.currentRow.IDTIPOTELA;
         var wdgCalculo;
         if(tipoTela == "A"){
            wdgCalculo = templateManager.container.getWidget("gridCalculo");
        }else
        if(tipoTela == "B"){
             wdgCalculo = templateManager.container.getWidget("gridCalculoFilho");
            }
        var valor = "vazio";
        var params = {
            "VAZIO": valor
        };
        
         if(tipoTela == "A"){
            General.callRestEngine('product#/fct_calcformprecserv_widget/removerAjusteTemp', "Row",params).then(function(){
                
                wdgCalculo.dataSource.name = "product#/fct_calcformprecserv_widget";
                wdgCalculo.getAction("aplicar").isVisible = false;
               // wdgCalculo.getAction("recalcular_elemento").isVisible = true;
                wdgCalculo.getAction("ADD").isVisible = true;
                wdgCalculo.getAction("BACK_VIEW").isVisible = true;
                wdgCalculo.getAction("BACK_LIST").isVisible = false;
                wdgCalculo.reload();
            }.bind(this), 
            function(error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });
        }else
        if(tipoTela == "B"){
            General.callRestEngine('product#/fct_calcreajformprec_widget/removerAjusteTemp', "Row",params).then(function(){
                
                wdgCalculo.dataSource.name = "product#/crud/fct_calcreajformprec_widget";
                wdgCalculo.getAction("aplicar").isVisible = false;
              //  wdgCalculo.getAction("recalcular_elemento").isVisible = true;
                wdgCalculo.getAction("ADD").isVisible = true;
                wdgCalculo.getAction("BACK_VIEW").isVisible = true;
                wdgCalculo.getAction("BACK_LIST").isVisible = false;
                wdgCalculo.reload();
                wdgCalculo.parentFilter.refkeys = [
                                "VALORORIGINAL",
                                "NRORG"
                            ];
                wdgCalculo.parentFilter.parentkeys = [
                                "NRREAJUSCONTFORMPREC",
                                "NRORG"
                            ];
            }.bind(this), 
            function(error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });
        }   
                
        
    };
    /**
    * Ativa a ação de aplicar após realizar reajuste
    * 
    * @param {object} widget widget de 
    * Reajusta Elemento
    */
    this.ativaAcaoAplicar = function(ativar,tipoTela) {
        var WdgReajustarElemento = templateManager.container.getWidget("WdgReajustarElemento");
        var gridCalculo;
         if(tipoTela == "A"){
            gridCalculo = templateManager.container.getWidget("gridCalculo");
        }else
        if(tipoTela == "B"){
             gridCalculo = templateManager.container.getWidget("gridCalculoFilho");
            }
       
        gridCalculo.getAction("aplicar").isVisible = ativar;
       // gridCalculo.getAction("recalcular_elemento").isVisible = !ativar;
        gridCalculo.getAction("ADD").isVisible = !ativar;
        gridCalculo.getAction("BACK_VIEW").isVisible = !ativar;
        gridCalculo.getAction("BACK_LIST").isVisible = ativar;
        ScreenService.closePopup(WdgReajustarElemento);
         if(tipoTela == "A"){
            gridCalculo.dataSource.name = "product#/crud/tmp_fct_calcreajformprec_widget";
        }else
        if(tipoTela == "B"){
             gridCalculo.dataSource.name = "product#/crud/tmp_fct_calcreajformprec_filho_widget";
             gridCalculo.parentFilter.refkeys = [
                            "NRFORMPRECOSERVICO",
                            "NRORG"
                        ];
            gridCalculo.parentFilter.parentkeys = [
                            "NRFORMPRECOSERVICO",
                            "NRORG"
                        ];
            }
        gridCalculo.reload();
        
    };
    
    this.fecharWdgReajustarElemento = function(fechar) {
        var AcaoReajusteElemento = templateManager.container.getWidget("WdgReajustarElemento");
        var gridReajuste = templateManager.container.getWidget("gridReajuste");
       
        AcaoReajusteElemento.isVisible = fechar;
        ScreenService.closePopup(AcaoReajusteElemento);
        gridReajuste.reload();
        
    };
    /**
     * Realiza o cálculo descrito no campo "Expressão" e retorna para o campo "Valor Unitário"
     * 
     * @param {object} widget widget
       /**
     * Fecha a widget de Cadastramento Automático.
     * 
     * @param {object} widget
     */
    this.alternarWidget = function(widgetPai) {
        if(!widgetPai){
        var widget = templateManager.container.getActiveWidget();
        }else{
            widget = widgetPai;
        }
        if(widget.name == "gridReajuste"){
            var Ativo = widget.currentRow.VALORPAI;
            
            var wdgCalculo = templateManager.container.getWidget("gridCalculo");
            var wdgCalculoFilho = templateManager.container.getWidget("gridCalculoFilho");
            if(!Ativo){
                wdgCalculo.isVisible = true;
                wdgCalculoFilho.isVisible = false;
            }else{
                wdgCalculoFilho.isVisible = true;
                wdgCalculo.isVisible = false;
            }
        }else{
            var wdgReajuste = templateManager.container.getWidget("gridReajuste");
            var AtivoElse = wdgReajuste.currentRow.VALORPAI;
            var wdgCalculoElse = templateManager.container.getWidget("gridCalculo");
            var wdgCalculoFilhoElse = templateManager.container.getWidget("gridCalculoFilho");
            if(!AtivoElse){
                wdgCalculoElse.isVisible = true;
                wdgCalculoFilhoElse.isVisible = false;
            }else{
                wdgCalculoFilhoElse.isVisible = true;
                wdgCalculoElse.isVisible = false;
            }
        }
    }; 
   /**
     * Fecha a widget de Reajuste de Foramção de preço
     * 
     * @param {object} widget
     */
    this.cancelarReajusteDePreco = function(widget) {
        widget.isVisible = false;
        this.limpaCamposDeReajuste(widget);
        ScreenService.closePopup();
        templateManager.container.getWidget("gridReajuste").activate();
    };
    /*
     * Seta true para as rows com o tiṕo de calor fixo
     * 
     * @param {object} widget de Reajuste de calculo
     */
    this.checkedBoxFixos = function(widget) {
        var wdgCalculo = templateManager.container.getWidget("gridCalculo");
        var tipo = widget.currentRow.IDTIPOCALCULO;
        if(tipo == "V"){
            wdgCalculo.showCheckbox = true;
        }
    };
     /*
     * Ativa o checkedBox do ultimo reajuste do contrato
     * 
     * @param {nulo} 
     */
    this.checkUltimo = function() {
    var contrato = templateManager.container.getWidget("gridReajuste");
    var ultimoReajustes = Array();
    var string;
    var i = 1; 
    var completa="";
      this.buscarUltimosReajustes().then(function(response) {
          ultimoReajustes = response.dataset.dados;    
          ultimoReajustes.forEach(function(ultimo){
            
            if(i!=ultimoReajustes.length){
                 string ="row.NRREAJUSCONTFORMPREC == "+ultimo+" || ";
            }else{
                 string ="row.NRREAJUSCONTFORMPREC == "+ultimo;
            }
        
            completa += string;
           i++;
          });
           var styles = [
              {
                "condition": "row.VALORPAI != null",
                "cssClass": "hideCheckBoxPai"
              },
              {
                  "condition": completa,
                  "cssClass": "showCheckBoxFilho"
              }
          ];
          contrato.styles = styles;
      });
    };
     /*
     * Busca o ultimo reajuste de cada contrato
     * 
     * @param {nulo} 
     */
   this.buscarUltimosReajustes = function(){
    var params = {};
    return General.callRestEngine("product#/fct_calcformprecserv_widget/buscarUltimosReajustes", "Row", params);
   };  /*
     * Busca o ultimo reajuste de cada contrato
     * 
     * @param {nulo} 
     */
   this.buscarNumeroFormacoes = function(){
    var params = {};
    return General.callRestEngine("product#/fct_calcformprecserv_widget/buscarNumeroFormacoes", "Row", params);
   };
    /**
     * Salva o Reajuste de formação de preço.
     * 
     * @param {object} widget Reajustar Preço
     */
    this.saveReajuste = function(widget) {
        var contrato = templateManager.container.getWidget("gridReajuste");
        var rows = contrato.getCheckedRows();
       var params = {
            "IDTPAJUSTE" : widget.currentRow.IDTPAJUSTE,
            "VRPERCDESCAJUSTE": widget.currentRow.VRPERCDESCAJUSTE,
            "ROWS" : rows
        };
        General.callRestEngine('product#/fct_calcformprecserv_widget/saveReajuste', "Row",params).then(function(){
            this.fecharWdgReajustarElemento(false);
               contrato.unSelectAll(); 
            ScreenService.showMessage('Reajuste de Formação de Preço realizada com sucesso.').then(function(){
               contrato.reload();
                }); 
        }.bind(this), 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        }); 
    };
    /**
     * Salva o Reajuste de preço Automático 
     * 
     * @param {object} widget
     */
    this.ReajusteAutomatico = function(widget) {
       var contrato = templateManager.container.getWidget("gridReajuste");
        var formacoes = [];
               
       widget.getField("VALORORIGINAL").dataSource.data.filter(function(row){
            return row.__isSelected;
        }).forEach(function(row){
            
            var elemetos = widget.getField("NRCALCFORMPRECSERV").dataSource.data.filter(function(row2){
                   return row2.__isSelected}).filter(function(row3){
                       return (row3.NRREAJUSCONTFORMPREC == row.NRREAJUSCONTFORMPREC);
                       }
                    );
            
            
            var object = {
                "CODIGO": row.CODIGO,
                    "NRREAJUSCONTFORMPREC": row.NRREAJUSCONTFORMPREC,
                    "NRFORMPRECOSERVICO": row.NRFORMPRECOSERVICO,
                    "NRSEQCONTRATO": row.NRSEQCONTRATO,
                    "NRITEMOCONTRATO": row.NRITEMOCONTRATO,
                    "DADOS": elemetos};
            
            formacoes.push(object);
        });
        
   
       var params = {
            "FORMACOES": formacoes,
            "IDTPAJUSTE": widget.currentRow.IDTPAJUSTE,
            "VRPERCDESCAJUSTE": widget.currentRow.VRPERCDESCAJUSTE
        };
        
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/saveReajusteAutomatico', "Row",params).then(function(response){
                this.fecharWdgReajustarAutomatico(false);
                ScreenService.showMessage('Reajuste de Formação de Preço realizada com sucesso.').then(function(){
                    contrato.reload();
                });
                
        }.bind(this), 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        }); 
    };
     /**
     * Salva o Reajuste de elemento. Na tabela temporária
     * 
     * @param {object} widget Reajuste de Elemento
     */
    this.ReajusteElemento = function(widget) {
        var calculo;
        var tipoTelaCalculo = widget.currentRow.IDTIPOTELA;
        var contrato = templateManager.container.getWidget("gridReajuste");
        if(tipoTelaCalculo == "A"){
            calculo = templateManager.container.getWidget("gridCalculo");
        }else
        if(tipoTelaCalculo == "B"){
             calculo = templateManager.container.getWidget("gridCalculoFilho");
            }
        var rows = calculo.getCheckedRows();
        var escolhidos = Array();
        var tabela;
        rows.forEach(function(row){
            escolhidos.push(row.IDITEMCALCULO);
            tabela = row.TABELA;
        });
       var params = {
            "NRFORMPRECOSERVICO": contrato.currentRow.NRFORMPRECOSERVICO,
            "IDTPAJUSTEELEM" : widget.currentRow.IDTPAJUSTEELEM,
            "VRPERCDESCAJUSTEELEM": widget.currentRow.VRPERCDESCAJUSTEELEM,
            "NRSEQCONTRATO": contrato.currentRow.NRSEQCONTRATO,
            "NRITEMOCONTRATO": contrato.currentRow.NRITEMOCONTRATO,
            "ESCOLHIDOS" : escolhidos,
            "TABELA" :  tabela
        };
        General.callRestEngine('product#/fct_calcformprecserv_widget/saveReajusteElemento', "Row",params).then(function(response){
            this.ativaAcaoAplicar(true,tipoTelaCalculo);
            calculo.unSelectAll(); 
            calculo.reload();
        }.bind(this), 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        }); 
    };
     /**
     * Salva o Reajuste de elemento que estaba na tabela temporária e salva na tabela permanente
     * 
     * @param {bollean} widget Reajuste de Elemento
     */
    this.aplicarReajuste = function(aplica){
        var WdgReajustarElemento = templateManager.container.getWidget("WdgReajustarElemento");
        var tipoTela = WdgReajustarElemento.currentRow.IDTIPOTELA;
        var contrato = templateManager.container.getWidget("gridReajuste");
         var params = {
            "NRREAJUSCONTFORMPREC": contrato.currentRow.NRREAJUSCONTFORMPREC,
            "NRFORMPRECOSERVICO": contrato.currentRow.NRFORMPRECOSERVICO,
            "NRSEQCONTRATO": contrato.currentRow.NRSEQCONTRATO,
            "NRITEMOCONTRATO": contrato.currentRow.NRITEMOCONTRATO,
            "VALORPAI": contrato.currentRow.VALORPAI,
            "NRORG": contrato.currentRow.NRORG
        };
        if(aplica){
            General.callRestEngine('product#/fct_calcformprecserv_widget/aplicarReajuste', "Row",params).then(function(response){
                if (response.dataset.APLICADO) {
                    this.fecharWdgAposReajusteElemento(false,tipoTela);
                }  
            }.bind(this), 
            function(error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            }); 
        }
    };
    
    /**
     * Seta os campos Padrões para a página de cálculo da Formação de Preços.
     * 
     * @param {object} widget widget
    */ 
    
    this.setaCamposPadraoCalculo = function(widget,  elemSup, elemSupName){
        
        /* === Código da Formção === */
        
        var NrFormPrecoServico = widget.parent.currentRow.NRFORMPRECOSERVICO;
        
        widget.currentRow.NRFORMPRECOSERVICO = NrFormPrecoServico;
        
        if (elemSup != null) {
            widget.currentRow.NRCALCFORMPRECSERVPAI = elemSup;
        }
        
        if (elemSupName != null){
            widget.currentRow.DSREFECALCULOPAI = elemSupName;
        }
        
        /* === Ordenação === */
        
        var params = {
            "NRFORMPRECOSERVICO" : NrFormPrecoServico,
            "TABELA"             : null 
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/recebeOrdenacao', "Row", params).then(function(response){
          
            var ordenacao = response.dataset.ORDENACAO.IDORDENACAO;
              
            if (ordenacao == 0){
                ordenacao =  10;
            }
            else {
                ordenacao = ordenacao + 10;
            }
              
            widget.currentRow.IDORDENACAO = ordenacao;
        }, 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    /**
     * Salva o novo registro da página "Cálculo"
     * 
     * @param {object} widget widget
     */
    
    this.salvarCalculoPreco = function(widget){
        
        widget.currentRow.__columns_to_return = ["NRCALCFORMPRECSERV", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        widget.currentRow.NRFORMPRECOSERVICO = widget.parent.currentRow.NRFORMPRECOSERVICO;
        
        var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
        
        this.preencheReferencia(widget);
        this.calculaItem(widget).then(function(){
                
            General.widgetSave(widget).then(function(response){
                if (response.dataset && response.dataset.columnsToReturn[0]){
                    widget.currentRow.NRCALCFORMPRECSERV = response.dataset.columnsToReturn[0].NRCALCFORMPRECSERV;
                    widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                    widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                    widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                    widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                }
                
                /* == Somatório dos Elementos Filhos ==*/
                if (tipoCalculo == 'S'){
                    
                    var params = {
                        "NRCALCFORMPRECSERV" : widget.currentRow.NRCALCFORMPRECSERV,
                        "TABELA"             : null
                    };
                    
                    General.callRestEngine('product#/fct_calcformprecserv_widget/calcSomatorioFilhos', "Row", params);  
                }
                
                ScreenService.closeSwipe();
                widget.reload();
            });
        });
    };
    
    /**
     * Funções antes de inserir
     * 
     * @param {object} widget widget
     */

    this.novoCalculo = function(widget) {
        if (widget.mode == "view") {
            
            var elemSup = widget.currentRow.NRCALCFORMPRECSERV;
            var elemSupName = widget.currentRow.DSREFECALCULO;
            
            General.widgetNewRow(widget);
            this.habilitaExpressao(widget);
            this.setaCamposPadraoCalculo(widget, elemSup, elemSupName);
        }
        else {
            General.widgetNewRow(widget);
            this.habilitaExpressao(widget);
            this.setaCamposPadraoCalculo(widget, null, null);
        }
        
        this.bloquearDadosMedida(widget);
    };
    
    /**
     * Funções antes de Editar
     * 
     * @param {object} widget widget
     */
    
    this.EditarCalculo = function(widget) {
        General.widgetEdit(widget);
        this.habilitaExpressao(widget); 
    };
    
    
    
    /**
     * Realiza o controle de Percentual do field "VRPERCCALCULO"
     * 
     * @param {object} widget widget
    */
    this.Validapercentual = function(widget){
        
        var vrdesconto = widget.getField('VRPERCCALCULO').value();
        
        /* Aplica Max e Min Value*/
        
        if (vrdesconto < 0 || vrdesconto > 100) {    
            /* Desconto */
            widget.currentRow.VRPERCCALCULO = null;
        }
    };
    
    
    /**
     * habilita / desabilita o campo "Expressão"
     * 
     * @param {object} widget widget
     */
    
    this.habilitaExpressao = function(widget){
        
        var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
        
        if (tipoCalculo == 'E') {
            /* Expressão  */
            widget.getField("DSEXPRESSAOCALC").readOnly = false;
            widget.getField("DSEXPRESSAOCALC").validations = {'required':true};
            
            /* Percentual  */
            widget.getField("VRPERCCALCULO").readOnly = false;
            widget.getField("VRPERCCALCULO").validations = {};
            
            /* Valor Unitário  */
            widget.getField("VRUNITARIO").readOnly = true;
            widget.getField("VRUNITARIO").validations = {};
            widget.currentRow.VRUNITARIO = null;
        }
        else if (tipoCalculo == 'S'){
            /* Expressão  */
            widget.getField("DSEXPRESSAOCALC").readOnly = true;
            widget.getField("DSEXPRESSAOCALC").validations = {};
            widget.currentRow.DSEXPRESSAOCALC = null;
            
            /* Percentual  */
            widget.getField("VRPERCCALCULO").readOnly = true;
            widget.getField("VRPERCCALCULO").validations = {};
            widget.currentRow.VRPERCCALCULO = null;
            
            /* Valor Unitário  */
            widget.getField("VRUNITARIO").readOnly = true;
            widget.getField("VRUNITARIO").validations = {};
            widget.currentRow.VRUNITARIO = null;
        }
        else {
            /* Expressão  */
            widget.getField("DSEXPRESSAOCALC").readOnly = true;
            widget.getField("DSEXPRESSAOCALC").validations = {};
            widget.currentRow.DSEXPRESSAOCALC = null;
            
            /* Percentual  */
            widget.getField("VRPERCCALCULO").readOnly = true;
            widget.getField("VRPERCCALCULO").validations = {};
            widget.currentRow.VRPERCCALCULO = null;
            
            /* Valor Unitário  */
            widget.getField("VRUNITARIO").readOnly = false;
            widget.getField("VRUNITARIO").validations = {'required':true};
        }
    };
    
    
    
     /**
     * Realiza o controle dos campos de "Medida" de acordo 
     * com o field "Utiliza Medida?"
     * 
     * @param {object} widget widget de 
     * Itens para pegar o field "IDUSAMEDIDA".
    */
    
    this.bloquearDadosMedida = function(widget){
        
        var usamedida = widget.getField('IDUSAMEDIDA').value();
        
        /* Bloqueia a Modalidade de pagamento*/
        if (usamedida == 'N') {
            
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = true;
            widget.getField('DSTPMEDIDASERV').setValue(null);
            widget.getField('DSTPMEDIDASERV').validations = {};
            
            widget.getField('CDTPMEDIDASERV').setValue(null);
            widget.getField('CDTPMEDIDASERV').validations = {};
            
            /* Quantidade de Medida */
            widget.getField('QTMEDIDACALC').readOnly = true;
            widget.getField('QTMEDIDACALC').setValue(null);
            widget.getField('QTMEDIDACALC').validations = {};
            
        } 
        /* Desbloqueia a Medida*/
        else {
            
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = false;
            widget.getField('DSTPMEDIDASERV').validations = {"required": true};
            
            /* Quantidade de Medida */
            widget.getField('QTMEDIDACALC').readOnly = false;
            widget.getField('QTMEDIDACALC').validations = {"required": true};
        }

    };
    /**
     * Realiza o cálculo do campo "Valor Unitário"
     * 
     * @param {object} widget widget
     */

    this.calculaItem = function(widget){
        var defered = ZHPromise.defer();
        
        var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
        
        /* == Expressão ==*/
        if (tipoCalculo == 'E') {
            this.calculaExpressao(widget).then(function(){
                
                this.calculaValorTotal(widget);
                defered.resolve();
            }.bind(this));
        }
        /* == Somatório dos Elementos Filhos ==*/
        else if (tipoCalculo == 'S'){
            
            defered.resolve();
        }
        /* == Valor Fixo ==*/
        else{
            this.calculaValorTotal(widget);
            defered.resolve();
        }
        
        return defered;
    };
    
    
    this.calculaExpressao = function(widget){
        var defered = ZHPromise.defer();
        
        var NrFormPrecoServico = widget.parent.currentRow.NRFORMPRECOSERVICO;
        var expressao = widget.currentRow.DSEXPRESSAOCALC;
        var percentual = widget.currentRow.VRPERCCALCULO;
        var vrunit = 0;
        
        var params = {
            "NRFORMPRECOSERVICO" : NrFormPrecoServico
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/buscaIdentValores', "Row", params).then(function(response){
          
            var identVal = response.dataset.IDENTVAL;
            
            identVal.forEach(function(identfica){
                
                var identificador = '['+identfica.IDITEMCALCULO+']';
                var valor = identfica.VRTOTAL;

                expressao = expressao.split(identificador).join(valor);    
            });
            
            expressao = expressao.split("%").join("/100");
            
            if (percentual != null){
               
               vrunit = eval(expressao);
               vrunit = (vrunit*percentual)/100;
            }
            else{
                
                vrunit = eval(expressao);
            }
            
            widget.currentRow.VRUNITARIO =  vrunit;
            
            defered.resolve();
        }, 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
            defered.reject();
        });
        return defered;
    };
    
  
    /**
     * Realiza a validação  do campo "identificador". Não podem haver dois identificadores iguais em uma mesma composição.
     * 
     * @param {object} widget widget
     */

    this.validaIdentificador = function(widget){
        
        var NrFormPrecoServico = widget.parent.currentRow.NRFORMPRECOSERVICO;
        var IdItemCalculo = widget.currentRow.IDITEMCALCULO;
        var nrCalcFormPrecServ = widget.currentRow.NRCALCFORMPRECSERV;
        /* === Ordenação === */
        
        var params = {
            "NRFORMPRECOSERVICO" : NrFormPrecoServico,
            "IDITEMCALCULO" : IdItemCalculo,
            "NRCALCFORMPRECSERV" : nrCalcFormPrecServ
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/validaIdentificador', "Row", params).then(function(response){
        
            var identificador = response.dataset.IDENTIFICADOR.VALIDA;
              
            if (identificador != 0){
                widget.currentRow.IDITEMCALCULO = null;
                ScreenService.showMessage('O identificador "'+IdItemCalculo+'" já está sendo utilizado para outro elemento.'); 
            }
            else{
                this.salvarCalculoPreco(widget);
            }
        }.bind(this), 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });    
    };
    
    /**
     * Realiza o cálculo do Valor Total
     * 
     * @param {object} widget widget
     */
     
    this.calculaValorTotal = function(widget){
        
        var vrUnit = widget.currentRow.VRUNITARIO;
        var qtCalc = widget.currentRow.QTCALCULO;
        var usamedida = widget.currentRow.IDUSAMEDIDA;
        var qtmedida = widget.currentRow.QTMEDIDACALC;
        var vrTot =  0;
        
        if (usamedida == 'S'){
            vrTot = (vrUnit*qtCalc)*qtmedida ; 
        
        }
        else {
            vrTot = vrUnit*qtCalc;
        }
        
        widget.currentRow.VRTOTAL = vrTot;
    };
    
    /**
     * Recebe o valor do campo de código da Formação para filtro de Data Source.
     * 
     * @param {object} widget widget
     */
    
    this.recebeValorFormacao = function(widget){
        
        var NrFormPrecoServico = widget.parent.currentRow.NRFORMPRECOSERVICO;
        var nrCalcFormPrecServ = widget.currentRow.NRCALCFORMPRECSERV;
        
        widget.getField("DSREFECALCULOPAI").dataSourceFilter[0].value = NrFormPrecoServico;
        widget.getField("DSREFECALCULOPAI").dataSourceFilter[1].value = nrCalcFormPrecServ;
        widget.getField("DSREFECALCULOPAI").reload();
    };
    
    
     /**
     * Exclui os elementos e os filhos.
     * 
     * @param {object} widget widget
     */
    
    this.deletarPaieFilhos = function(widget){
         
         var nrCalcFormPrecServ = widget.currentRow.NRCALCFORMPRECSERV;
         
         ScreenService.confirmMessage("A exclusão deste elemento também irá excluir os elementos inferiores a ele. Confirma exclusão?", "question", function() {
                                
            var params = {
                "NRCALCFORMPRECSERV" : nrCalcFormPrecServ
            };
            
            General.callRestEngine('product#/fct_calcformprecserv_widget/deletarPaieFilhos', "Row", params).then(function(){
            
                ScreenService.showMessage('Exclusão realizada com sucesso.'); 
                ScreenService.closeSwipe();
                widget.reload();
              
            }, 
            function(error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });    
            
        }.bind(this));
    };
     /**
     * Preenche Referencias
     * 
     * @param {object} widget widget
     */
    this.preencheReferencia = function(widget){
        
        var referencia =  widget.currentRow.DSREFECALCULO;
        var identificador =  widget.currentRow.IDITEMCALCULO;
        var elemento =  widget.currentRow.NMELEMENTO;
        
        if (referencia == "") {
            
            if(identificador != ""){
                
                referencia = '['+identificador+'] - ';
            }
            
            if(elemento != ""){
                
                referencia = referencia+elemento;
            }
            
            widget.currentRow.DSREFECALCULO = referencia;
        }
        
    };
    
     /* Abre a widget de Reajuste de Elemento
     *
     * 
     * @param {object} widget
     */
    this.abrirWdgReajustarElemento = function(widget,tipoTela) {
        ScreenService.closePopup();
        var wdgReajElem = templateManager.container.getWidget("WdgReajustarElemento");
        wdgReajElem.isVisible = true;
        ScreenService.openPopup(wdgReajElem).then(function() {
            wdgReajElem.edit();
            wdgReajElem.currentRow.IDTIPOTELA = tipoTela;
        });
    };
    
    
     /* Abre a widget de Reajuste de Elemento
     *
     * 
     * @param {object} widget
     */
    this.desativarComandos = function(widget) {
       var recalcular_elemento = widget.getAction("recalcular_elemento");
       recalcular_elemento.isVisible = false;
    };
    
     /* Abre a widget de Reajuste de Elemento
     *
     * 
     * @param {object} widget
     */
     
    this.ativarComandos = function(widget) {
        var checks = widget.getCheckedRows();
       var recalcular_elemento = widget.getAction("recalcular_elemento");
       if(checks.length == 0){
        recalcular_elemento.isVisible = false;
       }
    };
    
    /**
     * A função "CopiarFormacaoPreco" executa a procedure
     * "FCT_COPIA_FORM_PRECO".
     */
    
    this.copiarFormacaoPreco = function(widget){
        
        var NrFormPrecoServico = widget.currentRow.NRFORMPRECOSERVICO;
         
        var params = {
            "NRFORMPRECOSERVICO" : NrFormPrecoServico
        }; 
        
        General.callRestEngine('product#/fct_formprecoservico_widget/executarProcedureFctCopiaFormPreco', "Row", params).then(function(){
            //ScreenService.closePopup();
            ScreenService.showMessage('Cópia da formação de preço realizada com sucesso.'); 
            ScreenService.closeSwipe();
            widget.reload();
          
        }, 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });    
    };
    
    
    this.recalculo = function (ind, recalc, widget){
    
        if (ind <= recalc.length-1){
            /* == Expressão ==*/
            if (recalc[ind].IDTIPOCALCULO == 'E') {
                this.recalculaExpressao(recalc[ind].NRFORMPRECOSERVICO, recalc[ind].DSEXPRESSAOCALC, recalc[ind].VRPERCCALCULO).then(function(response){
                    var vrunit = response;
                    var qtCalc = recalc[ind].QTCALCULO;
                    var usamedida = recalc[ind].IDUSAMEDIDA;
                    var qtmedida = recalc[ind].QTMEDIDACALC;
                    var vrTot = 0;
                    
                    if (usamedida == 'S'){
                        vrTot = (vrunit*qtCalc)*qtmedida ; 
                    
                    }
                    else {
                        vrTot = vrunit*qtCalc;
                    }
                              
                    var params = {
                        "NRCALCFORMPRECSERV" : recalc[ind].NRCALCFORMPRECSERV,
                        "VRUNITARIO" : vrunit,
                        "QTCALCULO" : qtCalc,
                        "QTMEDIDACALC" : qtmedida,
                        "VRTOTAL" : vrTot
                    };
                                    
                    General.callRestEngine('product#/fct_calcformprecserv_widget/salvarRecalculo', "Row", params).then(function() {
                        this.recalculo(ind+1, recalc, widget);
                    }.bind(this),
                    function(error) {
                        console.log(error);
                        ScreenService.showMessage(error.data.error);
                    });
                    
                }.bind(this));
            }
            /* == Somatório dos Elementos Filhos ==*/
            else if (recalc[ind].IDTIPOCALCULO == 'S'){
                this.recalculaSomatorio(recalc[ind].NRCALCFORMPRECSERV).then(function(response){
                    var vrunit = response;
                    var qtCalc = recalc[ind].QTCALCULO;
                    var usamedida = recalc[ind].IDUSAMEDIDA;
                    var qtmedida = recalc[ind].QTMEDIDACALC;
                    var vrTot = 0;
                    
                    if (usamedida == 'S'){
                        vrTot = (vrunit*qtCalc)*qtmedida ; 
                    
                    }
                    else {
                        vrTot = vrunit*qtCalc;
                    }
                    
                    var params = {
                        "NRCALCFORMPRECSERV" : recalc[ind].NRCALCFORMPRECSERV,
                        "VRUNITARIO" : vrunit,
                        "QTCALCULO" : qtCalc,
                        "QTMEDIDACALC" : qtmedida,
                        "VRTOTAL" : vrTot
                    };
                    
                    General.callRestEngine('product#/fct_calcformprecserv_widget/salvarRecalculo', "Row", params).then(function() {
                        this.recalculo(ind+1, recalc, widget);   
                    }.bind(this),
                    function(error) {
                        console.log(error);
                        ScreenService.showMessage(error.data.error);
                    });   
                    
                }.bind(this));
            }
            /* == Valor Fixo ==*/
            else {
                this.recalculo(ind+1, recalc, widget); 
            }
        }
        else{
            ScreenService.closeSwipe();
            widget.reload();
        }

    };
    
    



  /**
     * A função "recalcular" realiza o recalculo do valor dos elementos.
     * 
     * @param {object} widget widget
     */
    
    this.recalcular = function(widget){
        
         var nrcalcreajformpre = widget.currentRow.NRCALCREAJFORMPRE;
        
        var params = {
            "NRCALCREAJFORMPRE" : nrcalcreajformpre
        };
        
        General.callRestEngine('product#/fct_contformpreco_widget/recalculoElemento', "Row", params).then(function() {
            ScreenService.showMessage("Processo realizado com sucesso!");
            ScreenService.closeSwipe().then(function() {
                widget.reload();
            });
        })
        .catch(function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });
        
    };
    
    
     /**
     * Realiza o cálculo descrito no campo "Expressão" e retorna para o campo "Valor Unitário"
     * 
     * @param {object} widget widget
     */

    this.recalculaExpressao = function(NrFormPrecoServico, expressao, percentual){
        var defered = ZHPromise.defer();
        var vrunit = 0;
        
        var params = {
            "NRFORMPRECOSERVICO" : NrFormPrecoServico
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/buscaIdentValores', "Row", params).then(function(response){
          
            var identVal = response.dataset.IDENTVAL;
            
            identVal.forEach(function(identfica){
                
                var identificador = '['+identfica.IDITEMCALCULO+']';
                var valor = identfica.VRTOTAL;

                expressao = expressao.split(identificador).join(valor);    
            });
            
            expressao = expressao.split("%").join("/100");
            
            
            if (percentual != null){
               
               vrunit = eval(expressao);
               vrunit = (vrunit*percentual)/100;
            }
            else{
                
                vrunit = eval(expressao);
            }

            
            defered.resolve(vrunit);
        }, 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
            defered.reject();
        });
        return defered;
    };
    
    
    this.recalculaSomatorio = function(nrCalcFormPrecServ){
        var defered = ZHPromise.defer();
        
        var params = {
            "NRCALCFORMPRECSERV" : nrCalcFormPrecServ
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/somatorioRecalculo', "Row", params).then(function(response){
            
            var vrUnit = response.dataset.SOMATORIO.VRSOMA;
            
            defered.resolve(vrUnit);    
        },
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
            defered.reject();
        });
        return defered;  
    };
    
    this.limpaCamposDeReajuste = function(widget){
    
        widget.currentRow.IDTPAJUSTE = null;
        widget.currentRow.VRPERCDESCAJUSTE = null;
       
    };
    
    
    
    
    
    
    
    
    /*-------------------------------------------------------------------*/

    
    
    
    this.recalcularArvore = function(widget) {
        var nrreajuscontformprec = widget.parent.currentRow.NRREAJUSCONTFORMPREC;
        
        var params = {
            "NRREAJUSCONTFORMPREC" : nrreajuscontformprec
        };
        
        General.callRestEngine("product#/fct_contformpreco_widget/recalcularArvore", "Row", params).then(function() {
      
        ScreenService.showMessage("Processo realizado com sucesso!");
        widget.reload();

        })
        .catch(function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00004004_Reajuste_Contrato_Formacao_Preco_Controller', Sm00004004_Reajuste_Contrato_Formacao_Preco_Controller);
});

// FILE: ./assets/js/controllers/Sm00004005_Parametrizacao_Contrato_Controller.js
function Sm00004005_Parametrizacao_Contrato_Controller(General, Sm00004005_Parametrizacao_Contrato_Service, ScreenService, templateManager, Utilities, ValidacoesParametrosFaturamentoUtil) {
    
   /* Funções Faturamento*/
   var permitirAlteracao = null;
    /**
     * Realiza o controle dos campos de vencimento de acordo com o "Tipo de Faturamento"
     * 
     * @param {object} widget widget de 
     * Contrato para pegar o field "IDFATURACONTR".
     */
    this.bloquearFieldsParcela = function(widget){
        
        var tipofaturamento = widget.currentRow.IDFATURACONTR;
        
        /* Desbloqueia os campos de parametrização da parcela*/
        if (tipofaturamento == 'O') {
            
            /* Tipo de Competência do Contrato */
            widget.getField('IDQTDIASCOMPCON').readOnly = true;
            widget.getField('IDQTDIASCOMPCON').value(null);
            widget.getField('IDQTDIASCOMPCON').validations = {};
            
            /* Dia Para Competência do Contrato */
            widget.getField('NRQTDIASCOMPCON').readOnly = true;
            widget.getField('NRQTDIASCOMPCON').value(null);
            widget.getField('NRQTDIASCOMPCON').validations = {};
            
            /* Tipo de Fechamento da Competência */
            widget.getField('IDTPCOMPETCONT').readOnly = true;
            widget.getField('IDTPCOMPETCONT').value(null);
            widget.getField('IDTPCOMPETCONT').validations = {};
            
            /* Dia Para Fechamento da Competência */
            widget.getField('NRDIACOMPETCONT').readOnly = true;
            widget.getField('NRDIACOMPETCONT').value(null);
            widget.getField('NRDIACOMPETCONT').validations = {};
            
            /* Tipo de Faturamento Para Períodos Incompletos */
            widget.getField('IDTPFATUINCOMP').readOnly = true;
            widget.getField('IDTPFATUINCOMP').value(null);
            widget.getField('IDTPFATUINCOMP').validations = {};
            
            /* Tipo de Emissão da NF */
            widget.getField('IDTPDTEMISSNF').readOnly = true;
            widget.getField('IDTPDTEMISSNF').value(null);
            widget.getField('IDTPDTEMISSNF').validations = {};
            
            /* Dia Para Emissão da NF */
            widget.getField('NRDIADTEMISSNF').readOnly = true;
            widget.getField('NRDIADTEMISSNF').value(null);
            widget.getField('NRDIADTEMISSNF').validations = {};
            
            /* Tipo de Vencimento da Parcela */
            widget.getField('IDTPDTVENC').readOnly = true;
            widget.getField('IDTPDTVENC').value(null);
            widget.getField('IDTPDTVENC').validations = {};
            
            /* Dia Para Vencimento da Parcela */
            widget.getField('NRDIAVENPARCFAT').readOnly = true;
            widget.getField('NRDIAVENPARCFAT').value(null);
            widget.getField('NRDIAVENPARCFAT').validations = {};
            
        } 
        /* Bloqueia os campos de parametrização da parcela*/
        else {
            
            /* Tipo de Competência do Contrato */
            widget.getField('IDQTDIASCOMPCON').readOnly = false;
            widget.getField('IDQTDIASCOMPCON').validations = {"required": true};
            
            /* Dia Para Competência do Contrato */
            widget.getField('NRQTDIASCOMPCON').readOnly = false;
            widget.getField('NRQTDIASCOMPCON').validations = {"required": true};
            
            /* Tipo de Fechamento da Competência */
            widget.getField('IDTPCOMPETCONT').readOnly = false;
            widget.getField('IDTPCOMPETCONT').validations = {"required": true};
            
            /* Dia Para Fechamento da Competência */
            widget.getField('NRDIACOMPETCONT').readOnly = false;
            widget.getField('NRDIACOMPETCONT').validations = {"required": true};
            
            /* Tipo de Faturamento Para Períodos Incompletos */
            widget.getField('IDTPFATUINCOMP').readOnly = false;
            widget.getField('IDTPFATUINCOMP').validations = {"required": true};
            
            /* Tipo de Emissão da NF */
            widget.getField('IDTPDTEMISSNF').readOnly = false;
            widget.getField('IDTPDTEMISSNF').validations = {"required": true};
            
            /* Dia Para Emissão da NF */
            widget.getField('NRDIADTEMISSNF').readOnly = false;
            widget.getField('NRDIADTEMISSNF').validations = {"required": true};
            
            /* Tipo de Vencimento da Parcela */
            widget.getField('IDTPDTVENC').readOnly = false;
            widget.getField('IDTPDTVENC').validations = {"required": true};
            
            /* Dia Para Vencimento da Parcela */
            widget.getField('NRDIAVENPARCFAT').readOnly = false;
            widget.getField('NRDIAVENPARCFAT').validations = {"required": true};
        }
    };
   
   
     /**
     * Realiza o controle do campo "Dia Para Competência do Contrato" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDQTDIASCOMPCON".
    */
    
    this.bloquearFieldDiasComp = function(widget){
        
        var tipoCompetencia = widget.getField('IDQTDIASCOMPCON').value();
        
        /* Desbloqueia o campo "Dia Para Competência do Contrato" */
        if (tipoCompetencia == "2") {
            
            widget.getField('NRQTDIASCOMPCON').readOnly = false;
            widget.getField('NRQTDIASCOMPCON').validations = {"required": true};
        } 
        /* Bloqueia o campo "Dia Para Competência do Contrato" */
        else {

            widget.getField('NRQTDIASCOMPCON').readOnly = true;
            widget.getField('NRQTDIASCOMPCON').value(null);
            widget.getField('NRQTDIASCOMPCON').validations = {};
        }
    };
   
   
    /**
     * Realiza o controle do campo "Dia Para Competência do Contrato" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDQTDIASCOMPCON".
    */
    
    this.ValidaFieldDiasComp = function(widget){
        
        var dia = widget.getField('NRQTDIASCOMPCON').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRQTDIASCOMPCON = null;
        }

    };
   
        /**
     * Realiza o controle do campo "Dia Para Fechamento da Competência" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPCOMPETCONT".
    */
    
    this.bloquearFieldDiasFecha = function(widget){
        
        var fechaCompetencia = widget.getField('IDTPCOMPETCONT').value();
        
        /* Bloqueia o campo "Dia Para Fechamento da Competência" */
        if (fechaCompetencia == "2" || fechaCompetencia == null) {
            
            widget.getField('NRDIACOMPETCONT').readOnly = true;
            widget.getField('NRDIACOMPETCONT').value(null);
            widget.getField('NRDIACOMPETCONT').validations = {};
        } 
        /* Desbloqueia o campo "Dia Para Fechamento da Competência" */
        else {

            widget.getField('NRDIACOMPETCONT').readOnly = false;
            widget.getField('NRDIACOMPETCONT').validations = {"required": true};
        }
    };
   
   
   /**
     * Realiza o controle do campo "Dia Para Fechamento da Competência" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPCOMPETCONT".
    */
    
    this.ValidaFieldDiasFecha = function(widget){
        
        var dia = widget.getField('NRDIACOMPETCONT').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRDIACOMPETCONT = null;
        }

    };
   
    /**
     * Realiza o controle do campo "Dia Para Emissão da NF" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTEMISSNF".
    */
    
    this.bloquearFieldDiasEmissao = function(widget){
        
        var emissao = widget.getField('IDTPDTEMISSNF').value();
        
        /* Bloqueia o campo "Dia Para Emissão da NF" */
        if (emissao == "9" || emissao == null) {
            
            widget.getField('NRDIADTEMISSNF').readOnly = true;
            widget.getField('NRDIADTEMISSNF').value(null);
            widget.getField('NRDIADTEMISSNF').validations = {};

        } 
        /* Desbloqueia o campo "Dia Para Emissão da NF" */
        else {

            widget.getField('NRDIADTEMISSNF').readOnly = false;
            widget.getField('NRDIADTEMISSNF').validations = {"required": true};
        }
    };
    
    /**
     * Realiza o controle do campo "Dia Para Emissão da NF" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTEMISSNF".
    */
    
    this.ValidaFielDiasEmissao = function(widget){
        
        var dia = widget.getField('NRDIADTEMISSNF').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRDIADTEMISSNF = null;
        }

    };
    
    /**
     * Realiza o controle do campo "Dia Para Vencimento da Parcela" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTVENC".
    */
    
    this.bloquearFieldDiasVenc = function(widget){
        
        var vencimento = widget.getField('IDTPDTVENC').value();
        
        /* Bloqueia o campo "Dia Para Vencimento da Parcela" */
        if (vencimento == "5" || vencimento == null) {
            
            widget.getField('NRDIAVENPARCFAT').readOnly = true;
            widget.getField('NRDIAVENPARCFAT').value(null);
            widget.getField('NRDIAVENPARCFAT').validations = {};
            
        } 
        /* Desbloqueia o campo "Dia Para Vencimento da Parcela" */
        else {

            widget.getField('NRDIAVENPARCFAT').readOnly = false;
            widget.getField('NRDIAVENPARCFAT').validations = {"required": true};
        }
    };
   
   
    /**
     * Realiza o controle do campo "Dia Para Vencimento da Parcela" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTVENC".
    */
    
    this.ValidaFielDiasVenc = function(widget){
        
        var dia = widget.getField('NRDIAVENPARCFAT').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRDIAVENPARCFAT = null;
        }

    };
   
   /*
   * Agrupa os controles de campos do faturamento
   *
   * @param {object} widget widget de Parametrização de Faturamento
   */
   
    this.ControlaCamposParcela = function(widget){
       
       this.bloquearFieldsParcela(widget);
       this.bloquearFieldDiasComp(widget);
       this.bloquearFieldDiasFecha(widget);
       this.bloquearFieldDiasEmissao(widget);
       this.bloquearFieldDiasVenc(widget);
       
    };
    
    this.editarParamFat = function(wdgParametrosFat) {
        
        var wdgContrato = templateManager.container.getWidget("gridContrato");
        var wdgReajuste = templateManager.container.getWidget("Reajustecontrato");
        
        if(wdgParametrosFat.dataSource.data[0]){
            General.widgetEdit(wdgParametrosFat);
        }
        else{
            wdgParametrosFat.newRow();
		    wdgParametrosFat.template = 'widget/form.html';
		    wdgParametrosFat.currentRow.NRSEQCONTRATO =   wdgContrato.currentRow.NRSEQCONTRATO;  
		    
		    if (wdgContrato.currentRow.NRSEQMODCONTRA != null){
		        
    		    var params = {
                    "NRSEQMODCONTRA": wdgContrato.currentRow.NRSEQMODCONTRA,
                    "NRORG": wdgContrato.currentRow.NRORG
                };
                
                General.callRestEngine("product#/fct_paramfatcont_widget/getDadosModeloContrato", "Row", params).then(function(response) {
                    
                
                    wdgParametrosFat.currentRow.NRGRUPOFATURA         = response.dataset.MODELO[0].NRGRUPOFATURA;
                    wdgParametrosFat.currentRow.IDTPCOMPETCONT        = response.dataset.MODELO[0].IDTPCOMPETCONT;
                    wdgParametrosFat.currentRow.NRDIACOMPETCONT       = response.dataset.MODELO[0].NRDIACOMPETCONT;
                    wdgParametrosFat.currentRow.IDTPDTEMISSNF         = response.dataset.MODELO[0].IDTPDTEMISSNF;
                    wdgParametrosFat.currentRow.NRDIADTEMISSNF        = response.dataset.MODELO[0].NRDIADTEMISSNF;
                    wdgParametrosFat.currentRow.IDTPDTVENC            = response.dataset.MODELO[0].IDTPDTVENC;
                    wdgParametrosFat.currentRow.NRDIAVENPARCFAT       = response.dataset.MODELO[0].NRDIAVENPARCFAT;
                    wdgParametrosFat.currentRow.IDQTDIASCOMPCON       = response.dataset.MODELO[0].IDQTDIASCOMPCON;
                    wdgParametrosFat.currentRow.NRQTDIASCOMPCON       = response.dataset.MODELO[0].NRQTDIASCOMPCON;
                    wdgParametrosFat.currentRow.IDTPREAJUSTE          = response.dataset.MODELO[0].IDTPREAJUSTE;
                    wdgParametrosFat.currentRow.IDTPMESREAJUSTE       = response.dataset.MODELO[0].IDTPMESREAJUSTE;
                    wdgParametrosFat.currentRow.NRQTDREAJUSTE         = response.dataset.MODELO[0].NRQTDREAJUSTE;
                    wdgParametrosFat.currentRow.CDMODPGTO             = response.dataset.MODELO[0].CDMODPGTO;
                    wdgParametrosFat.currentRow.DTINICFATCONTR        = response.dataset.MODELO[0].DTINICFATCONTR;
                    wdgParametrosFat.currentRow.IDFATURACONTR         = response.dataset.MODELO[0].IDFATURACONTR;
                    wdgParametrosFat.currentRow.IDTPFATUINCOMP        = response.dataset.MODELO[0].IDTPFATUINCOMP;
                    wdgParametrosFat.currentRow.QTPARCELASFATU        = response.dataset.MODELO[0].QTPARCELASFATU;
                    wdgParametrosFat.currentRow.NRDIAVENCPARCELASFATU = response.dataset.MODELO[0].NRDIAVENCPARCELASFATU;
                    wdgParametrosFat.currentRow.IDSOMAFATUCONT        = response.dataset.MODELO[0].IDSOMAFATUCONT;
                    wdgParametrosFat.currentRow.IDSOMAITGRUPFAT       = response.dataset.MODELO[0].IDSOMAITGRUPFAT;
                    wdgParametrosFat.currentRow.IDGERANFPARC          = response.dataset.MODELO[0].IDGERANFPARC;
                    wdgParametrosFat.currentRow.IDBLOQVRMEDIC         = response.dataset.MODELO[0].IDBLOQVRMEDIC;
                    wdgParametrosFat.currentRow.IDGERARATEIO          = response.dataset.MODELO[0].IDGERARATEIO;
                    wdgParametrosFat.currentRow.IDAPURAAPONTAMENTO    = response.dataset.MODELO[0].IDAPURAAPONTAMENTO;
                    
                    
                    /* Bloqueia o campo "Dia Para Fechamento da Competência" */
                    if (wdgParametrosFat.currentRow.IDTPCOMPETCONT == "2" || wdgParametrosFat.currentRow.IDTPCOMPETCONT == null) {
                        
                        wdgParametrosFat.getField('NRDIACOMPETCONT').readOnly = true;
                        wdgParametrosFat.getField('NRDIACOMPETCONT').value(null);
                        wdgParametrosFat.getField('NRDIACOMPETCONT').validations = {};
                    } 
                    /* Desbloqueia o campo "Dia Para Fechamento da Competência" */
                    else {
            
                        wdgParametrosFat.getField('NRDIACOMPETCONT').readOnly = false;
                        wdgParametrosFat.getField('NRDIACOMPETCONT').validations = {"required": true};
                    }
                    
                    
                    /* Bloqueia o campo "Dia Para Emissão da NF" */
                    if (wdgParametrosFat.currentRow.IDTPDTEMISSNF == "9" || wdgParametrosFat.currentRow.IDTPDTEMISSNF == null) {
                        
                        wdgParametrosFat.getField('NRDIADTEMISSNF').readOnly = true;
                        wdgParametrosFat.getField('NRDIADTEMISSNF').value(null);
                        wdgParametrosFat.getField('NRDIADTEMISSNF').validations = {};
            
                    } 
                    /* Desbloqueia o campo "Dia Para Emissão da NF" */
                    else {
            
                        wdgParametrosFat.getField('NRDIADTEMISSNF').readOnly = false;
                        wdgParametrosFat.getField('NRDIADTEMISSNF').validations = {"required": true};
                    }
                    
                    /* Bloqueia o campo "Dia Para Vencimento da Parcela" */
                    if (wdgParametrosFat.currentRow.IDTPDTVENC == "5" || wdgParametrosFat.currentRow.IDTPDTVENC == null) {
                        
                        wdgParametrosFat.getField('NRDIAVENPARCFAT').readOnly = true;
                        wdgParametrosFat.getField('NRDIAVENPARCFAT').value(null);
                        wdgParametrosFat.getField('NRDIAVENPARCFAT').validations = {};
                        
                    } 
                    /* Desbloqueia o campo "Dia Para Vencimento da Parcela" */
                    else {
            
                        wdgParametrosFat.getField('NRDIAVENPARCFAT').readOnly = false;
                        wdgParametrosFat.getField('NRDIAVENPARCFAT').validations = {"required": true};
                    }
                    
                })
                .catch(function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
		    }
    		    
        }
        
        wdgContrato.enabled = false;
        wdgReajuste.enabled = false;
        
        this.ControlaCamposParcela(wdgParametrosFat);
        this.bloquearFieldModpgto(wdgParametrosFat);
    };
    
    
    
    this.cancelarParamFat = function(wdgParametrosFat) {
        General.widgetCancelForm(wdgParametrosFat);
        var wdgContrato = templateManager.container.getWidget("gridContrato");
        var wdgReajuste = templateManager.container.getWidget("Reajustecontrato");
        wdgContrato.enabled = true;
        wdgReajuste.enabled = true;
    };
    
    this.salvarParamFat = function(wdgParametrosFat) {
        this.insereDadosContrato(wdgParametrosFat).then(function() {
            return General.widgetSave(wdgParametrosFat);
        })
        .then(function() {
            var wdgContrato = templateManager.container.getWidget("gridContrato");
            var wdgReajuste = templateManager.container.getWidget("Reajustecontrato");
            wdgContrato.enabled = true;
            wdgReajuste.enabled = true;
        })
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
  
    /**
     * A função "insereDadosContrato" insere os dados de parametização 
     * de faturamento na tabela de Contrato (FCT_CONTRATO).
     * 
     * @param {object} widget widget de Parametros de Faturamento
     */
     
    this.insereDadosContrato = function(widget){
        var IdFaturaContr         = widget.currentRow.IDFATURACONTR; 
        var CdModPgto             = widget.currentRow.CDMODPGTO;
        var QtParcelasFatu        = widget.currentRow.QTPARCELASFATU;
        var NrDiaVencParcelasFatu = widget.currentRow.NRDIAVENCPARCELASFATU;
        
        var dtInicFatContr  = widget.currentRow.DTINICFATCONTR;
        var idQtDiasCompCon = widget.currentRow.IDQTDIASCOMPCON;
        var nrQtDiasCompCon = widget.currentRow.NRQTDIASCOMPCON;
        var idCompetCont    = widget.currentRow.IDTPCOMPETCONT;
        var nrDiaCompetCont = widget.currentRow.NRDIACOMPETCONT;
        var idTpDtEmissNf   = widget.currentRow.IDTPDTEMISSNF;
        var nrDiaDtEmissNf  = widget.currentRow.NRDIADTEMISSNF;
        var idTpDtVenc      = widget.currentRow.IDTPDTVENC;
        var nrDiaVenParcFat = widget.currentRow.NRDIAVENPARCFAT;
        var idTpFatuIncomp  = widget.currentRow.IDTPFATUINCOMP;
        var seqContrato     = widget.currentRow.NRSEQCONTRATO;
           
        return Sm00004005_Parametrizacao_Contrato_Service.insereDadosContrato(IdFaturaContr, CdModPgto, QtParcelasFatu, NrDiaVencParcelasFatu, dtInicFatContr, idQtDiasCompCon, nrQtDiasCompCon, idCompetCont, nrDiaCompetCont, idTpDtEmissNf, nrDiaDtEmissNf, idTpDtVenc, nrDiaVenParcFat, idTpFatuIncomp, seqContrato).then(null).catch(function(response) {
            ScreenService.showMessage(response.data.error);
        });
        
    };
    
    
     /**
     * A função "insereDadosContrato" insere os dados de parametização 
     * de fatiuramento na tabela de Contrato (FCT_CONTRATO).
     * 
     * @param {object} widget widget de Parametros de Faturamento
     */
     
    this.insereDadosItContrato = function(widget){
        var IdFaturaContr         = widget.currentRow.IDFATURACONTR; 
        var CdModPgto             = widget.currentRow.CDMODPGTO;
        var QtParcelasFatu        = widget.currentRow.QTPARCELASFATU;
        var NrDiaVencParcelasFatu = widget.currentRow.NRDIAVENCPARCELASFATU;

        var seqContrato   = widget.currentRow.NRSEQCONTRATO;
        var seqitContrato = widget.currentRow.NRITEMOCONTRATO;
           
        return Sm00004005_Parametrizacao_Contrato_Service.insereDadosItContrato(IdFaturaContr, CdModPgto, QtParcelasFatu, NrDiaVencParcelasFatu, seqContrato, seqitContrato).then(null).catch(function(response) {
            ScreenService.showMessage(response.data.error);
        });
        
    };
    
    this.salvarItemParamFatCont = function(wdgItParamFatCont) {
        if (wdgItParamFatCont.isValid()) {
            this.insereDadosItContrato(wdgItParamFatCont).then(function() {
                return General.widgetSave(wdgItParamFatCont);
            })
            .catch(function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        }
    };
    
    /**
     * Realiza o controle do campo "Modalidade de Pagamento" 
     * de acordo com o field "Tipo de Faturamento"
     * 
     * @param {object} widget widget de 
     * Itens para pegar o field "IDFATURAITEM".
    */
    
    this.bloquearFieldModpgto = function(widget){
        var campoNomeModPgto        = widget.getField("DSMODPGTO");
        var campoQtdParcelas        = widget.getField("QTPARCELASFATU");
        var campoDiaVencParcelaFatu = widget.getField("NRDIAVENCPARCELASFATU");
        var campoTpCompetencia      = widget.getField("IDQTDIASCOMPCON");
        var campoTpFchCompetencia   = widget.getField("IDTPCOMPETCONT");
        var campoTpEmissaoNF        = widget.getField("IDTPDTEMISSNF");
        var campoTpVencNF           = widget.getField("IDTPDTVENC");
        var campoTpPeriodoInc       = widget.getField("IDTPFATUINCOMP");
        var campoFatItem            = widget.getField("IDSOMAITGRUPFAT");
        var campoTpFat              = widget.getField("IDSOMAFATUCONT");
        
        var tipofaturamento = widget.currentRow.IDFATURACONTR;
        
        campoFatItem.defaultValue           = "N";
        campoTpFat.defaultValue             = "N";

        widget.currentRow.NRQTDIASCOMPCON           = null;
        widget.getField("NRQTDIASCOMPCON").value(null);
        widget.getField("NRQTDIASCOMPCON").readOnly = true;
        
        this.bloquearFieldDiasFecha(widget);
        this.bloquearFieldDiasEmissao(widget);
        this.bloquearFieldDiasVenc(widget);

        if (tipofaturamento == "A") {
            campoNomeModPgto.readOnly           = true;
            campoNomeModPgto.validations        = {};
            widget.currentRow.CDMODPGTO         = null;
            campoNomeModPgto.value(null);
            
            campoQtdParcelas.readOnly           = true;
            campoQtdParcelas.validations        = {};
            campoQtdParcelas.value(null);
            
            campoDiaVencParcelaFatu.readOnly    = true;
            campoDiaVencParcelaFatu.value(null);
            campoDiaVencParcelaFatu.validations = {};
            
            campoTpCompetencia.readOnly         = false;
            campoTpCompetencia.validations      = { "required": true };
            
            campoTpFchCompetencia.readOnly      = false;
            campoTpFchCompetencia.validations   = { "required": true };
            
            campoTpEmissaoNF.readOnly           = false;
            campoTpEmissaoNF.validations        = { "required": true };
            
            campoTpVencNF.readOnly              = false;
            campoTpVencNF.validations           = { "required": true };
            
            campoTpPeriodoInc.readOnly          = false;
            campoTpPeriodoInc.validations       = { "required": true };
            
        } else {
            if(tipofaturamento === "O"){
                
                campoNomeModPgto.readOnly           = false;
                campoNomeModPgto.validations        = { "required": true };
                
                campoQtdParcelas.readOnly           = true;
                campoQtdParcelas.validations        = {};
                campoQtdParcelas.value(null);
                
                campoDiaVencParcelaFatu.readOnly    = true;
                campoDiaVencParcelaFatu.validations = {};
                campoDiaVencParcelaFatu.value(null);
                
                campoTpCompetencia.readOnly         = true;
                campoTpCompetencia.validations      = {};
                campoTpCompetencia.value(null);
                
                campoTpFchCompetencia.readOnly      = true;
                campoTpFchCompetencia.validations   = {};
                campoTpFchCompetencia.value(null);
                
                campoTpEmissaoNF.readOnly           = true;
                campoTpEmissaoNF.validations        = {};
                campoTpEmissaoNF.value(null);
                
                campoTpVencNF.readOnly              = true;
                campoTpVencNF.validations           = {};
                campoTpVencNF.value(null);
                
                campoTpPeriodoInc.readOnly          = true;
                campoTpPeriodoInc.validations       = {};
                campoTpPeriodoInc.value(null);
                
            } else {
                if(tipofaturamento === "X"){
                    
                    campoNomeModPgto.readOnly           = true;
                    campoNomeModPgto.validations        = {};
                    widget.currentRow.CDMODPGTO         = null;
                    campoNomeModPgto.value(null);
                    
                    campoQtdParcelas.readOnly           = true;
                    campoQtdParcelas.validations        = {};
                    campoQtdParcelas.value(null);
                    
                    campoDiaVencParcelaFatu.readOnly    = true;
                    campoDiaVencParcelaFatu.validations = {};
                    campoDiaVencParcelaFatu.value(null);
                    
                    campoTpCompetencia.readOnly         = false;
                    campoTpCompetencia.validations      = { "required": true };
                    
                    campoTpFchCompetencia.readOnly      = false;
                    campoTpFchCompetencia.validations   = { "required": true };
                    
                    campoTpEmissaoNF.readOnly           = false;
                    campoTpEmissaoNF.validations        = { "required": true };
                    
                    campoTpVencNF.readOnly              = false;
                    campoTpVencNF.validations           = { "required": true };
                    
                    campoTpPeriodoInc.readOnly          = false;
                    campoTpPeriodoInc.validations       = { "required": true };
                        
                    campoFatItem.defaultValue            = "S";
                    widget.currentRow.IDSOMAITGRUPFAT   = "S";
                    campoTpFat.defaultValue              = "S";
                    widget.currentRow.IDSOMAFATUCONT    = "S";
                    
                } else {
                    if(tipofaturamento === "P"){
                        campoNomeModPgto.readOnly           = true;
                        campoNomeModPgto.validations        = {};
                        widget.currentRow.CDMODPGTO         = null;
                        campoNomeModPgto.value(null);
                        
                        campoQtdParcelas.readOnly           = false;
                        campoQtdParcelas.validations        = { "required": true};
                        
                        campoDiaVencParcelaFatu.readOnly    = false;
                        campoDiaVencParcelaFatu.validations = { "required": true};
                        
                        campoTpCompetencia.readOnly         = true;
                        campoTpCompetencia.validations      = {};
                        campoTpCompetencia.value(null);
                        
                        campoTpFchCompetencia.readOnly      = true;
                        campoTpFchCompetencia.validations   = {};
                        campoTpFchCompetencia.value(null);
                        
                        campoTpEmissaoNF.readOnly           = true;
                        campoTpEmissaoNF.validations        = {};
                        campoTpEmissaoNF.value(null);
                        
                        campoTpVencNF.readOnly              = true;
                        campoTpVencNF.validations           = {};
                        campoTpVencNF.value(null);
                        
                        campoTpPeriodoInc.readOnly          = true;
                        campoTpPeriodoInc.validations       = {};
                        campoTpPeriodoInc.value(null);
                    }
                }
            }
        }
    };
  
   
   /* Funções Reajustes */
   
    /**
     * Salva o novo registro da página "Parâmetros de Reajuste"
     * 
     * @param {object} widget widget de 
     * Parâmetros de Reajuste para salvar 
     * o novo registro.
    */
    this.salvarParamReajsute = function(widget){
        
        widget.currentRow.NRSEQCONTRATO = widget.parent.currentRow.NRSEQCONTRATO;
        widget.currentRow.__columns_to_return = ["NRSEQREAJUS"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset)
                widget.currentRow.NRSEQREAJUS = response.dataset.columnsToReturn[0].NRSEQREAJUS;
            
        }.bind(this));
    };


    /**
     * Salva o novo registro da página "índice de Reajuste"
     * 
     * @param {object} widget widget de 
     * índice de Reajuste para salvar 
     * o novo registro.
    */
    this.salvarIndReajsute = function(widget){
        
        widget.currentRow.NRSEQCONTRATO = widget.parent.currentRow.NRSEQCONTRATO;
        widget.currentRow.__columns_to_return = ["NRSEQINDCONTR"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset)
                widget.currentRow.NRSEQINDCONTR = response.dataset.columnsToReturn[0].NRSEQINDCONTR;
            
        }.bind(this));
    };
    
     /**
     * Realiza o controle de bloqueio e desbloqueio dos campos 
     * da página "Parâmetros de Reajuste"
     * 
     * @param {object} widget widget de 
     * Parâmetros de Reajuste para salvar 
     * o novo registro.
    */
    
    this.controlaCamposReajuste = function(widget){
        
        var tiporeajuste = widget.getField('IDTPREAJUSTE').value();
        
        /* 1 - Anual: Desbloqueia o campo "Mês para Reajsute" e 
         * bloqueia o campo "Número de Meses".
         */
         
        if (tiporeajuste == '1') {
            
            /* Mês para Reajuste */
            widget.getField('IDTPMESREAJUSTE').readOnly = false;
            widget.getField('IDTPMESREAJUSTE').validations = {"required": true};
            
            /* Número de Meses */
            widget.getField('NRQTDREAJUSTE').readOnly = true;
            widget.getField('NRQTDREAJUSTE').value(null);
            widget.getField('NRQTDREAJUSTE').validations = {};
            
        } 
        /* 2 - No ato de Prorrogação do Contrato: Bloqueia os campos 
         * "Mês para Reajsute" e "Número de Meses".
         */
        else if (tiporeajuste == '2') {
            
            /* Mês para Reajuste */
            widget.getField('IDTPMESREAJUSTE').readOnly = true;
            widget.getField('IDTPMESREAJUSTE').value(null);
            widget.getField('IDTPMESREAJUSTE').validations = {};
            
            /* Número de Meses */
            widget.getField('NRQTDREAJUSTE').readOnly = true;
            widget.getField('NRQTDREAJUSTE').value(null);
            widget.getField('NRQTDREAJUSTE').validations = {};
        }
        /* 3 - Número de Meses: Bloqueia o campo "Mês para Reajsute" e 
         * desbloqueia o campo "Número de Meses".
         */
        else{
           
            /* Mês para Reajuste */
            widget.getField('IDTPMESREAJUSTE').readOnly = true;
            widget.getField('IDTPMESREAJUSTE').value(null);
            widget.getField('IDTPMESREAJUSTE').validations = {};
            
            /* Número de Meses */
            widget.getField('NRQTDREAJUSTE').readOnly = false;
            widget.getField('NRQTDREAJUSTE').validations = {"required": true};
        }
    };
    
   /**
     * Realiza o controle de Percentual do field "Valor Percentual"
     * 
     * @param {object} widget widget de 
     * Itens para pegar o field "VRPERCINDCONTR"".
    */
    
    this.ValidaPercIndice = function(widget){
        
        var vrPercentual = widget.getField('VRPERCINDCONTR').value();
        
        /* Aplica Max e Min Value*/
        if (vrPercentual < 0 || vrPercentual > 100) {    

            widget.currentRow.VRPERCINDCONTR = null;
        }

    };
    
    this.carregarRegistroWidgetParametrosFaturamento = function(wdgParametrosFat) {
        wdgParametrosFat.list();
        wdgParametrosFat.reload().then(function() {
            if(wdgParametrosFat.dataSource.data[0]){
                    wdgParametrosFat.view();
                    wdgParametrosFat.currentRow = wdgParametrosFat.dataSource.data[0];
            }
            else {
                wdgParametrosFat.view();
            }
        });
    };
    
    this.setFiltroParametrosFaturamento = function(widget) {
        var nrseqcontrato = widget.currentRow.NRSEQCONTRATO;
        var nrorg = widget.currentRow.NRORG;
        
        var wdgParamFatCont = templateManager.container.getWidget("FaturaParamcontrato");
        var wdgItParamFatCont = templateManager.container.getWidget("Faturaitemcontrato");
        
        wdgParamFatCont.dataSourceFilter[0].value = nrseqcontrato;
        wdgItParamFatCont.dataSourceFilter[0].value = nrseqcontrato;
        
        wdgParamFatCont.dataSourceFilter[1].value = nrorg;
        wdgItParamFatCont.dataSourceFilter[1].value = nrorg;
    };
    
    this.editarItemParamFat = function(wdgItParamFatCont) {
        General.widgetEdit(wdgItParamFatCont);
        this.controlaCamposModalidadePgto(wdgItParamFatCont);
    };
    
    this.controlaCamposModalidadePgto = function(widget) {
        var campoModPgto = widget.getField("DSMODPGTO");
        var campoQtdParcelas = widget.getField("QTPARCELASFATU");
        
        var tipoFaturamento = widget.currentRow.IDFATURACONTR;
        if (tipoFaturamento == "O") {
            campoModPgto.readOnly = false;
            campoModPgto.validations = {"required": true};
            
            campoQtdParcelas.value(null);
            campoQtdParcelas.readOnly = true;
            campoQtdParcelas.validations = {};
        }
        else {
            campoModPgto.readOnly = true;
            campoModPgto.value(null);
            widget.currentRow.CDMODPGTO = null;
            campoModPgto.validations = {};
            
            if (tipoFaturamento == "P") {
                campoQtdParcelas.readOnly = false;
                campoQtdParcelas.validations = {"required": true};
            }
            else {
                campoQtdParcelas.readOnly = true;
                campoQtdParcelas.value(null);
                campoQtdParcelas.validations = {};
            }
        }
    };

    this.wdgParametrosFaturamentoItemNewRow = function(wdgParametrosFaturamentoItem) {
        General.widgetNewRow(wdgParametrosFaturamentoItem);
        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(wdgParametrosFaturamentoItem);
    };

    this.wdgParametrosFaturamentoItemEdit = function(wdgParametrosFaturamentoItem) {
        General.widgetEdit(wdgParametrosFaturamentoItem);
        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(wdgParametrosFaturamentoItem);
    };

    this.wdgParametrosFaturamentoItemSave = function(wdgParametrosFaturamentoItem) {
        if (wdgParametrosFaturamentoItem.isValid()) {
            Utilities.validarCamposDuplicados(wdgParametrosFaturamentoItem, ["NRPARAMFATITCON"], ["NRSEQCONTRATO", "NRITEMOCONTRATO"])
            .then(function(validacao) {
                if (!validacao) {
                    ScreenService.showMessage("Operação bloqueada. <br/>Já existe uma parametrização de faturamento cadastrada para este item.");
                }
                else {
                    Utilities.widgetSaveAndSetColumnsToReturn(wdgParametrosFaturamentoItem);
                }
            });
        }
    };
    /**
     * Verifica a widget que solicitou a validação e chama o método de edição dela
     * @params{Object} widget  - widgets de faturamento
     * @params{Bool} isVisible - visibilidade das actions de alteração
    **/
    this.editarInformacoesFatura = function(widget, isVisible){
        var wdgTabbed = templateManager.container.getWidget("wdgParametrosFaturamentoTabbed");
        var actions   = ['add', 'edit', 'delete'];
        wdgTabbed.widgets.forEach(function(widget){
            widget.actions.forEach(function(action){
                if(actions.includes(action.name))
                    action.isVisible = isVisible;
            });
        });
    };
    /**
     * Valida se o contrato participa de um agrupamento para permitir a edição dos mesmo
     * @param{Object} widget - widgets de faturamento
    **/
    this.validarAlteracaoFaturamento = function(widget){
        var params = [
            {
                name: "NRSEQCONTRATO",
                operator: "=",
                value: templateManager.container.getWidget("gridContrato").currentRow.NRSEQCONTRATO
            }
        ];
        General.callRestEngine("product#/crud/fct_relacagrupamentocont_widget", "FilterData", params).then(function(response){
            var possuiAgrup   = (!response.dataset || !response.dataset.fct_relacagrupamentocont_widget || response.dataset.fct_relacagrupamentocont_widget.length == 0);
            permitirAlteracao = possuiAgrup;
            this.editarInformacoesFatura(widget, possuiAgrup);
        }.bind(this));
    };
    /**
     * Controla se a widget deve habilitar actions de edição e inserção
     * @params{Object} widget - widget de dados do faturamento 
    **/
    this.controlarActions = function(widget){
        if(permitirAlteracao)
            ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoOnEnter(widget);
    }
}

Configuration(function(ContextRegister)
{
    ContextRegister.register("Sm00004005_Parametrizacao_Contrato_Controller", Sm00004005_Parametrizacao_Contrato_Controller);
});

// FILE: ./assets/js/controllers/Sm00004006_Documento_Contrato_Controller.js
function Sm00004006_Documento_Contrato_Controller(General, ScreenService, templateManager, Utilities) {
    
    this.verificarSeExisteAnexo = function(wdgDocumento) {
        var actDownload     = wdgDocumento.getAction("download");
        var actExcluirAnexo = wdgDocumento.getAction("excluirAnexo");
        var fieldAnexo      = wdgDocumento.getField("DSCAMINHOARQ");

        var existe = (wdgDocumento.currentRow.NMARQUIVO) ? true : false;
        actDownload.isVisible     = existe;
        actExcluirAnexo.isVisible = existe;
        fieldAnexo.isVisible      = !existe;
    };
    
    /**
     * Salva o novo registro da página "Documento"
     * 
     * @param {object} widget widget de 
     * Documento para salvar o 
     * novo registro.
     */
    this.salvarDocumento = function(widget){
        
        widget.currentRow.__columns_to_return = ["NRSEQCONTDOC", "NRORG", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        
        var isNew = widget.currentRow.__is_new;
        var dscaminhoarq = widget.currentRow.DSCAMINHOARQ;
        
        if (!isNew && !(dscaminhoarq instanceof Array)) {
            widget.currentRow.DSCAMINHOARQ = [{
                "name": widget.currentRow.NMARQUIVO,
                "path": dscaminhoarq,
                "size": widget.currentRow.VRTAMANHO * 1024,
                "type": widget.currentRow.DSTIPO
            }];
        }

        General.widgetSave(widget).then(function(response){
        
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                widget.currentRow.NRSEQCONTDOC   = response.dataset.columnsToReturn[0].NRSEQCONTDOC;
                widget.currentRow.NRORG          = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.DTINCLUSAO     = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO  = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
            }
            widget.reload().then(function() {
                this.habilitaFieldSituacao(widget, "save");
                this.verificarSeExisteAnexo(widget);
            }.bind(this));
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    
    /**
     * A função "atualizaDocumento" Atualiza o documento de acordo com o modelo encontrado.
     * 
     * @param {Object} widget
     */
    this.atualizaDocumento = function(widget){
        var params = {
            "NRSEQMODDOCFCT": widget.currentRow.NRSEQMODDOCFCT,
            "NRSEQCONTDOC"  : widget.currentRow.NRSEQCONTDOC
        };
        
        General.callRestEngine('product#/fct_contratodoc_widget/atualizaDocumento', "Row",params).then(function() {
            ScreenService.showMessage("Documento atualizado com sucesso.");
            //ScreenService.closePopup();
            widget.reload();
        })
        .catch(function(response){
            ScreenService.showMessage(response.data.error);
        });
    };
    
    this.editarDocumentoContrato = function(widget) {
        this.habilitaFieldSituacao(widget, "edit");
        General.widgetEdit(widget);
    };
    
    this.habilitaFieldSituacao = function(widget, actionName){
        var idSituacao = widget.currentRow.IDSITUACAOCONTRATO;
        if(idSituacao) {
            if(actionName == "edit" && !Array.isArray(idSituacao)) 
                widget.currentRow.IDSITUACAOCONTRATO = idSituacao.split(", ");
            else 
                if(actionName != "edit" && Array.isArray(idSituacao))
                    widget.currentRow.IDSITUACAOCONTRATO = idSituacao.join(", ");
        }
    };
    
    /**
     * Abre uma nova janela para fazer o download do documento.
     */
    this.downloadDocumento = function(wdgDocumento) {
        ScreenService.closePopup();
        var params = { "NRSEQCONTDOC": wdgDocumento.currentRow.NRSEQCONTDOC };
        General.callRestEngine("product#/fct_contratodoc_widget/downloadDocFileServer", "Row", params).then(function(response) {
            if(response.dataset && response.dataset.URLDOCUMENTO && response.dataset.URLDOCUMENTO.length > 0) {
                var urlDocumento = window.backendIndexUrl + response.dataset.URLDOCUMENTO[0]; 
                window.open(urlDocumento);
            }
        }).catch(function(error){
            window.open(error);
        });
    };
    
    this.excluirAnexo = function(wdgDocumento) {
        var params = {
            "NRSEQCONTDOC": wdgDocumento.currentRow.NRSEQCONTDOC,
            "DSCAMINHOARQ": wdgDocumento.currentRow.DSCAMINHOARQ,
            "NRORG"       : wdgDocumento.currentRow.NRORG
        };
        
        General.callRestEngine("product#/fct_contratodoc_widget/excluirAnexo", "Row", params).then(function() {
            ScreenService.showMessage("Anexo excluído com sucesso!");
            ScreenService.closePopup();
            wdgDocumento.currentRow.NMARQUIVO    = null;
            wdgDocumento.currentRow.DSCAMINHOARQ = null;
            wdgDocumento.currentRow.VRTAMANHO    = null;
            this.verificarSeExisteAnexo(wdgDocumento);
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    this.verificarSeExisteAnexo = function(wdgDocumento) {
        var actDownload = wdgDocumento.getAction("download");
        var actExcluirAnexo = wdgDocumento.getAction("excluirAnexo");
        var fieldAnexo = wdgDocumento.getField("DSCAMINHOARQ");
        
        var existe = (wdgDocumento.currentRow.NMARQUIVO) ? true : false;
        actDownload.isVisible     = existe;
        actExcluirAnexo.isVisible = existe;
        fieldAnexo.isVisible      = !existe;
    };
    
    this.deletarDocumentoContrato = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function () {
            General.callRestEngine("product#/fct_contratodoc_widget/deletarDocumentoContrato", "DataSet", [widget.currentRow]).then(function(){
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
            }).catch(function(error){
                Utilities.showErrorMessage(error);
            });
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00004006_Documento_Contrato_Controller", Sm00004006_Documento_Contrato_Controller);
});

// FILE: ./assets/js/controllers/Sm00004007_Produto_Servico_AdquiridoController.js
function Sm00004007_Produto_Servico_AdquiridoController(Utilities, templateManager, ScreenService, General, ZHPromise) {
    var getWidgetGrid = function () {
        return templateManager.container.getWidget('gridProdutosAdquiridos');
    };
    var getWidgetElementoGrid = function () {
        return templateManager.container.getWidget('gridElemento');
    };
    var getTabbedWidgetGrid = function () {
        return templateManager.container.getWidget('TabbedProdutosAdquiridos');
    };

    var getFieldParcnegocio = function() {
        return getTabbedWidgetGrid().getField('NMPRINCIPALPARC');
    };

    this.habilitarGrid = function() {
        var widget = getWidgetGrid();
        var elementos = getWidgetElementoGrid();

        var field = getFieldParcnegocio().value();
        var tabWidget = getTabbedWidgetGrid();
        var parcnegocio = tabWidget.currentRow.NRPARCNEGOCIO;
        if(field) {
            widget.dataSourceFilter = [
                {
                    name: "NRPARCNEGOCIO",
                    operator: "=",
                    value: parcnegocio
                }
            ];
            elementos.dataSourceFilter = [
                {
                    name: "NRPARCNEGOCIO",
                    operator: "=",
                    value: parcnegocio
                }
            ];
        } else {
            widget.dataSourceFilter = [
                {
                    name: "NRPARCNEGOCIO",
                    operator: "=",
                    value: 0
                }
            ];
            elementos.dataSourceFilter = [
                {
                    name: "NRPARCNEGOCIO",
                    operator: "=",
                    value: 0
                }
            ];
        }
        widget.reload();
        elementos.reload();
    };
}
Configuration(function(ContextRegister) {
    ContextRegister.register('Sm00004007_Produto_Servico_AdquiridoController', Sm00004007_Produto_Servico_AdquiridoController);
});


// FILE: ./assets/js/controllers/Sm00004101_RelatorioCustomizavelContrato_Controller.js
function Sm00004101_RelatorioCustomizavelContrato_Controller(templateManager, ScreenService, Sm00004000_Contrato_Controller, General, ZhFilterPreferences, Utilities) {
    
    /**
    * Abre a tela de contrato filtrando pelo contrato selecionado
    * 
    * @param {object} widget widget de dados do contrato  
    **/
    this.abrirContrato = function(widget) {
       ScreenService.openWindow('product#sm_00004_100_contrato').then(function() {
            var wdgContratroGrid = templateManager.container.getWidget('wdgContratroGrid');
            
            wdgContratroGrid.reload().then(function() {
                ScreenService.closePopup();
                wdgContratroGrid.currentRow = {
                    "NRSEQCONTRATO"      : widget.currentRow.NRSEQCONTRATO, 
                    "NRSEQCONTRATOPAI"   : widget.currentRow.NRSEQCONTRATOPAI, 
                    "IDGERADOPOR"        : widget.currentRow.IDGERADOPOR,
                    "NRSEQORCAMENTO"     : widget.currentRow.NRSEQORCAMENTO, 
                    "NRORG"              : widget.currentRow.NRORG, 
                    "IDSITUACAOCONTRATO" : widget.currentRow.IDSITUACAOCONTRATO, 
                    "NRSEQMODCONTRA"     : widget.currentRow.NRSEQMODCONTRA
                };
                Sm00004000_Contrato_Controller.validarWidgetContratoGrid(wdgContratroGrid);
            });
        });
    };
    
    this.downloadArquivo = function(widget) {
        var nmarquivo = widget.currentRow.NMARQUIVO;
        var dscaminho = widget.currentRow.DSCAMINHO;
        var type = widget.currentRow.DSTIPO;

        Utilities.downloadFile(nmarquivo, dscaminho, type);
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00004101_RelatorioCustomizavelContrato_Controller", Sm00004101_RelatorioCustomizavelContrato_Controller);
});

// FILE: ./assets/js/controllers/Sm00004108_RenovacaoContratoController.js
function Sm00004108_RenovacaoContratoController(ScreenService, templateManager, General, Utilities, ZHPromise, localStorageEngine) {
    
    /**
     * Valida se a ação 'Renovar Contrato' pode ser habilitada
     * 
     * @params{Object} widget - widget de renovação de contratos
    **/
    this.validarDataFinalContrato = function(widget) {
        //var actionEditar    = widget.getAction("edit");
        var actionRenovar   = widget.getAction("renovar_contrato");
        var contratoVencido = (widget.currentRow.IDCONTVENCIDO == "S");
        
        //actionEditar.isVisible  = !contratoVencido;
        actionRenovar.isVisible = contratoVencido;
    };
    
    /**
     * Valida se o usuário pode determinar os parametros de renovação automatica
     * 
     * @params{Object} widget - widget de renovação automática
    **/
    this.validarParametrosRenovacao = function(widget) {
        var fieldReajuste       = widget.getField("IDREAJUSTEAUTOMATICO");
        var fieldProrrogacao    = widget.getField("NRDIASPRORGCONTRENOV");
        var renovacaoautomatica = (widget.currentRow.IDRENOVACAOAUTOMATICA != 'S');
        
        if (renovacaoautomatica)
            fieldReajuste.value("N");
        
        fieldReajuste.readOnly = renovacaoautomatica;
        
        if ((widget.currentRow.NRDIASPRORGCONTRENOV != null) || (widget.currentRow.NRDIASPRORGCONTRENOV != 0)){
            fieldProrrogacao.value(widget.currentRow.NRDIASPRORGCONTRENOV);
            var qtDiasProrrog = widget.currentRow.NRDIASPRORGCONTRENOV;
        }
        else{
            fieldProrrogacao.value(widget.currentRow.NRDIASPRORGCONT);
            var qtDiasProrrog = widget.currentRow.NRDIASPRORGCONT;
        }
        
        fieldProrrogacao.readOnly = renovacaoautomatica;
        
        var dataInicio = moment(widget.currentRow.DTINIVIGENCONTRENOV, "DD/MM/YYYY");
        var DataFinal = dataInicio.add(qtDiasProrrog, "days").format("DD/MM/YYYY");
        
        widget.currentRow.DTFINVIGENCONTRENOV = DataFinal;

    };
    
    /**
     * Abre o popup para a definição da nova data de vigência do contrato
     * 
     * @params{Object} widget - widget de renovação de contratos
    **/
    this.abrirPopupPeriodoVigenciaRenovacaoManual = function(widget) {
        var wdgPopupRenovacao = templateManager.container.getWidget("wdgPopupDataInicialVigente");
        var dataFinalVigenteAtual = moment(widget.currentRow.DTFINVIGENCONT.split(" ")[0], "DD/MM/YYYY");
        var dataInicialVigenteNova = dataFinalVigenteAtual.add(1, "days");
        var pos = Utilities.getFieldGroupPosition(wdgPopupRenovacao.fieldGroups, "fdg_seguro");
        var filter = [
                {
                    "name": "NRSEQCONTRATO",
                    "operator": "=",
                    "value": widget.currentRow.NRSEQCONTRATO
                },
                {
                    "name": "IDVIGENTE",
                    "operator": "=",
                    "value": "S"
                }
        ];
        localStorageEngine.download("product#/crud/fct_segurocontrato_widget", filter).then(function(dataset) {
            if (dataset.length > 0) {
                var row = dataset[0];
                wdgPopupRenovacao.seguro = row;
                wdgPopupRenovacao.fieldGroups[pos].isVisible = true;
                this.calcularPrazoVigenciaSeguro(wdgPopupRenovacao);
            }
            else {
                wdgPopupRenovacao.fieldGroups[pos].isVisible = false;
            }
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
        
        ScreenService.openPopup(wdgPopupRenovacao).then(function() {
            wdgPopupRenovacao.newRow();
            
            this.habilitaFieldsSeguro(wdgPopupRenovacao);
            
            wdgPopupRenovacao.currentRow.NRSEQCONTRATO   = widget.currentRow.NRSEQCONTRATO;
            wdgPopupRenovacao.currentRow.NRTIPOORCAMENTO = widget.currentRow.NRTIPOORCAMENTO;
            wdgPopupRenovacao.currentRow.NRDIASPRORGCONT = widget.currentRow.NRDIASPRORGCONTRENOV;
            wdgPopupRenovacao.currentRow.DTINIVIGENCONT  = dataInicialVigenteNova.format("DD/MM/YYYY");
            wdgPopupRenovacao.currentRow.NRDIASREAJAUTOMATICO = widget.currentRow.NRDIASREAJAUTOMATICO;
            wdgPopupRenovacao.currentRow.DTPADREAJCONT = widget.currentRow.DTPADREAJCONT;
            this.calcularNovaDataReajuste(wdgPopupRenovacao);
            
            var diasProrrogacao = parseInt(widget.currentRow.NRDIASPRORGCONTRENOV, 10);
            wdgPopupRenovacao.currentRow.DTFINVIGENCONT = dataInicialVigenteNova.add(diasProrrogacao-1, "days").format("DD/MM/YYYY");
            
            var params = {
                "NRSEQCONTRATO" : widget.currentRow.NRSEQCONTRATO
            };
            
            General.callRestEngine("product#/fct_contrato_renovacao_widget/getDadosIndFinanc", "Row", params).then(function(response) {    
                
                if ((response.dataset) && (response.dataset.dados != 0)) {
                    wdgPopupRenovacao.currentRow.IDPERVALOR      = response.dataset.dados[0]["IDPERVALOR"]; 
                    wdgPopupRenovacao.currentRow.CDINDICE        = response.dataset.dados[0]["CDINDICE"]; 
                    wdgPopupRenovacao.currentRow.NMINDICE        = response.dataset.dados[0]["NMINDICE"]; 
                    wdgPopupRenovacao.currentRow.VRINDICE        = response.dataset.dados[0]["VRINDICE"]; 
                    wdgPopupRenovacao.currentRow.NRPERCENTINDICE = response.dataset.dados[0]["NRPERCENTINDICE"]; 
                }

                if (wdgPopupRenovacao.currentRow.NRPERCENTINDICE != 100) {
                    ScreenService.showMessage("Operação bloqueada!<br/><br/> A soma dos percentuais de utilização dos índices financeiros não está totalizando os 100% necessários para realização do reajuste deste contrato.");
                    ScreenService.closePopup();
                }
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });

        }.bind(this));
    };
    
    /**
     * Calcula uma previsão da data inicial e final da nova vigência
     * 
     * @params{Object} widget - popup de renovação manual do contrato
    **/
    this.calcularPeriodoVigenciaContrato = function(widget) {
        if(widget.currentRow.DTINIVIGENCONT) {
            var wdgRenovacao     = templateManager.container.getWidget("wdgContratosRenovados");
            var oldDataFinVigent = moment(wdgRenovacao.currentRow.DTFINVIGENCONT, "DD/MM/YYYY");
            var newDataIniVigent = moment(widget.currentRow.DTINIVIGENCONT, "DD/MM/YYYY");
            
            var diasProrrogacao = parseInt(widget.currentRow.NRDIASPRORGCONT, 10);
            if(oldDataFinVigent <= newDataIniVigent) {
                widget.currentRow.DTFINVIGENCONT = newDataIniVigent.add(diasProrrogacao-1, "days").format("DD/MM/YYYY");
                widget.currentRow.DTPADREAJCONT = newDataIniVigent.add(1, "days").format("DD/MM/YYYY");
            } else {
                ScreenService.showMessage("A data inicial da nova vigência deve ser maior que a atual data final do contrato.");
                widget.currentRow.DTINIVIGENCONT = moment().format("DD/MM/YYYY");
                widget.currentRow.DTFINVIGENCONT = newDataIniVigent.add(diasProrrogacao-1, "days").format("DD/MM/YYYY");
                widget.currentRow.DTPADREAJCONT = newDataIniVigent.add(1, "days").format("DD/MM/YYYY");
            }
        }
    };
    
    this.calcularNovaDataReajuste = function(widget) {
        widget.currentRow.DTPADREAJCONT = moment(widget.currentRow.DTINIVIGENCONT,"DD/MM/YYYY").add(widget.currentRow.NRDIASREAJAUTOMATICO,"days").format("DD/MM/YYYY");
    };
    
    this.calcularNovaDataReajusteContratosMassa = function(widget){
        widget.currentRow.DTPADREAJCONT = moment(widget.currentRow.DTINIRENOVACAO,"DD/MM/YYYY").add(widget.currentRow.NRDIASREAJAUTOMATICO,"days").format("DD/MM/YYYY");
    };
    
    /**
     * Realiza a renovação manual dos contratos
     * 
     * @params{Object} widget - widget popup de data de reajuste
    **/
    this.renovarContrato = function(widget) {
        if (!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
        else {
            var dataIniSeguro = widget.currentRow.DTINISEGURO;
            var dataFinSeguro = widget.currentRow.DTFINSEGURO;
            var prazo = Utilities.getDiasEntreDatas(dataIniSeguro, dataFinSeguro);
            
            if (widget.currentRow.IDRENOVASEGURO == 'N' || (widget.currentRow.IDRENOVASEGURO == 'Y' && prazo > 0)) {
                var params = {
                    "NRSEQCONTRATO"        : widget.currentRow.NRSEQCONTRATO,
                    "NRTIPOORCAMENTO"      : widget.currentRow.NRTIPOORCAMENTO,
                    "DTINIVIGENCONT"       : widget.currentRow.DTINIVIGENCONT,
                    "NRDIASPRORGCONT"      : widget.currentRow.NRDIASPRORGCONT,
                    //"DTFINVIGENCONT"       : widget.currentRow.DTINIVIGENCONT,
                    "IDREAJUSTEAUTOMATICO" : widget.currentRow.IDREAJUSTEAUTOMATICO == 'Y',
                    "DTPADREAJCONT"        : widget.currentRow.DTPADREAJCONT,
                    "NRDIASREAJAUTOMATICO" : widget.currentRow.NRDIASREAJAUTOMATICO
                };
                
                General.callRestEngine("product#/fct_contrato_renovacao_widget/renovarContrato", "Row", params).then(function(response) {
                    var wdgRenovacao = templateManager.container.getWidget("wdgContratosRenovados");
                    ScreenService.closePopup();
                    
                    wdgRenovacao.currentRow.NRDIASPRORGCONT = widget.currentRow.NRDIASPRORGCONT;
                    wdgRenovacao.currentRow.DTINIVIGENCONT  = widget.currentRow.DTINIVIGENCONT;
                    wdgRenovacao.currentRow.DTFINVIGENCONT  = widget.currentRow.DTFINVIGENCONT;
                    
                    var dtiniproxrenov = moment(widget.currentRow.DTINIVIGENCONT, "DD/MM/YYYY").add(widget.currentRow.NRDIASPRORGCONT, "days");
                    wdgRenovacao.currentRow.NRDIASPRORGCONTRENOV = widget.currentRow.NRDIASPRORGCONT;
                    wdgRenovacao.currentRow.QTDIASATEVENCIMENTO  = widget.currentRow.NRDIASPRORGCONT;
                    wdgRenovacao.currentRow.DTINIVIGENCONTRENOV  = dtiniproxrenov.format("DD/MM/YYYY");
                    wdgRenovacao.currentRow.DTFINVIGENCONTRENOV  = dtiniproxrenov.add(widget.currentRow.NRDIASPRORGCONT, "days").format("DD/MM/YYYY");
                    
                    if (widget.currentRow.IDRENOVASEGURO == 'Y') {
                        var row = widget.seguro;
                        row.changed = true;
                        row.IDVIGENTE = 'N';
        
                        General.callRestEngine("product#/crud/fct_segurocontrato_widget/save", "DataSet", [row]).then(function() {
                            row.__is_new = true;
                            row.NRSEGUROCONTRATO = null;
                            row.IDVIGENTE = 'S';
                            row.DTINISEGURO = widget.currentRow.DTINISEGURO;
                            row.DTFINSEGURO = widget.currentRow.DTFINSEGURO;
                
                            row.__is_new_code = true;
                            row.__new_code_pk = "FCT_SEGUROCONTRATO";
                            row.__table_pk = "NRSEGUROCONTRATO";
                            row.__cast_number = true;
                
                            return General.callRestEngine("product#/crud/fct_segurocontrato_widget/save", "DataSet", [row]);
                        })
                        .then(function() {
                            widget.isVisible = false;
                            ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
                            ScreenService.closePopup();
                            //wdgSeguroVig.activate();
                        })
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }
                    else {
                        ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
                    }
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
            else {
                ScreenService.showMessage("A data final da vigência do seguro deve ser maior que a data inicial.");
            }
        }
    };
    
    
    /**
     * Calcula uma previsão da data inicial e final da nova vigência
     * 
    **/
    this.calcularPeriodoVigenciaRenov = function(wdgRenovacao) {
        
        var nrDiasProrrogContRenov = wdgRenovacao.currentRow.NRDIASPRORGCONTRENOV;
        var dataInicio = moment(wdgRenovacao.currentRow.DTINIVIGENCONTRENOV, "DD/MM/YYYY");
        var DataFinal = dataInicio.add(nrDiasProrrogContRenov, "days").format("DD/MM/YYYY");
        
        wdgRenovacao.currentRow.DTFINVIGENCONTRENOV = DataFinal;
        
    };
    
    this.renovarContrMassa = function() {
        var wdgContratosRenovados = templateManager.container.getWidget('wdgContratosRenovados');
        var checkedRows = wdgContratosRenovados.getCheckedRows();

        var nrorg        = '';
        var contratos    = '_';
        
        checkedRows.forEach(function(contrato) {
            contratos = contratos + contrato.NRSEQCONTRATO + '_';
            nrorg     = contrato.NRORG;
        });
        
        var wdgRenovContrMassa = templateManager.container.getWidget('wdgRenovContrMassa');
        wdgRenovContrMassa.isVisible                 = true;
        wdgRenovContrMassa.dataSourceFilter[0].value = contratos;
        wdgRenovContrMassa.dataSourceFilter[1].value = nrorg;
        ScreenService.openPopup(wdgRenovContrMassa).then(function() {
            $('.zh-new-grid-wdgRenovContrMassa').parent().parent().parent().parent().parent().parent().addClass('cp-high-modal');
            var acoes = ["voltar", "confirmar_reajuste"];
            wdgRenovContrMassa.actions.forEach(function(act) {
                if (acoes.includes(act.name) === false) {
                    act.isVisible = false;
                }
            });
        });
         
        wdgRenovContrMassa.reload();
    };
    
    /**
     * Fecha a widget wdgRenovContrMassa 
    **/
    this.fecharRenovarContrMassa = function(){
        $('.zh-new-grid-wdgRenovContrMassa').parent().parent().parent().parent().parent().parent().removeClass('cp-high-modal');
        
        var wdgRenovContrMassa = templateManager.container.getWidget('wdgRenovContrMassa');
        wdgRenovContrMassa.isVisible = true;
        ScreenService.closePopup();
    };
    
    /**
     * Renova e reajusta os contratos selecionados
    **/
    this.confirmarRenovarContrMassa = function(widget) {
        var wdgContratosRenovados = templateManager.container.getWidget('wdgContratosRenovados');
        
        var dataSource = widget.dataSource.data;
        var params = {
            "CONTRATOS": dataSource
        };

        General.callRestEngine("product#/fct_contratoreajuste_widget/validaBloqReajusteContrato", "Row", params).then(function(response) {
            if (response.dataset.retorno.valida == 'BLOQUEAR') {
                ScreenService.showMessage("Atenção!<br><br>Não será possível realizar o reajuste dos contratos.<br><br>Existem índices financeiros com cadastro de alíquota pendentes. Estes cadastros devem ser regularizados antes da realização dos reajustes dos contratos.");
            }
            else if (response.dataset.retorno.valida == 'INDIVIDUAL') {
                ScreenService.showMessage("Atenção!<br><br>Não será possível realizar o reajuste dos contratos.<br><br> Alguns dos contratos selecionados possuem índices financeiros com cadastro de alíquota pendentes. Estes cadastros devem ser regularizados antes da realização dos reajustes dos contratos.");
            }
            else if (response.dataset.retorno.valida == 'BLOQ_SEM_PARAM') {
                ScreenService.showMessage("Atenção!<br><br>Não será possível realizar o reajuste dos contratos.<br><br> Não há cadastro de índices financeiros para reajustar estes contratos.");
            }
            else {
        
                this.validarPercentualReajuste(widget).then(function() {
                    var dados = widget.dataSource.data;
                    General.callRestEngine("product#/fct_contrato_renovacao_widget/renovContrMassa", "Row", dados)
                    .then(function() {
                        ScreenService.showMessage('Os contratos foram renovados e reajustados com sucesso!');
                        this.fecharRenovarContrMassa();
                        ScreenService.closePopup();
                        wdgContratosRenovados.unSelectAll();
                        wdgContratosRenovados.reload();
                        
                    }.bind(this))
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }.bind(this));
            }
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    
    
    this.renovReajAuto = function() {
        
        var wdgPopupRenovReajAuto = templateManager.container.getWidget('wdgPopupRenovReajAuto');
        ScreenService.openPopup(wdgPopupRenovReajAuto).then(function() {
            wdgPopupRenovReajAuto.dataSource.data = [];
            wdgPopupRenovReajAuto.newRow();
        });
    };
    
    /**
     * Altera os parâmetros "IDRENOVACAOAUTOMATICA" e "IDREAJUSTEAUTOMATICO" para os contratos selecionados
    **/
    this.salvarRenovReajAuto = function(widget) {
        if (widget.isValid()) {
            var wdgContratosRenovados = templateManager.container.getWidget('wdgContratosRenovados');
            var checkedRows = wdgContratosRenovados.getCheckedRows();

            var contratos = '';
            
            checkedRows.forEach(function(contrato) {
                if (contratos == '') {
                    
                    contratos = contrato.NRSEQCONTRATO;
                }
                else{
                    contratos = contratos + '_' + contrato.NRSEQCONTRATO;
                }
            });
            
            var params = {
                "NRSEQCONTRATO"         : contratos,
                "IDRENOVACAOAUTOMATICA" : widget.currentRow.IDRENOVACAOAUTOMATICA,
                "IDREAJUSTEAUTOMATICO"  : widget.currentRow.IDREAJUSTEAUTOMATICO
            };
            
            General.callRestEngine("product#/fct_contrato_renovacao_widget/salvarRenovReajAuto", "Row", params).then(function() {
                
                ScreenService.showMessage("Processo realizado com sucesso."); 
                ScreenService.closePopup();
                ScreenService.closePopup();
                wdgContratosRenovados.reload();
                
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    this.atualizarperiodoRenovacaoMassa = function(widget) {
        
        var nrDiasProrrogContRenov = widget.currentRow.NRDIASRENOVACAO;
        var dataInicio = moment(widget.currentRow.DTINIRENOVACAO, "DD/MM/YYYY");
        var dataFinal = dataInicio.add(nrDiasProrrogContRenov-1, "days").format("DD/MM/YYYY");
        var dataReajuste = dataInicio.add(1, "days").format("DD/MM/YYYY");
        
        widget.currentRow.DTFINRENOVACAO = dataFinal;
        widget.currentRow.DTPADREAJCONT = dataReajuste;
    };
    
    
    /**
     * Valida se todos os contratos possuem a soma do percetual de 
     * utilização dos indices igual a 100%
     * 
     * @params{Object} widget - widget de reajuste em massa
    **/
    this.validarPercentualReajuste = function(widget) {
        var deferred           = ZHPromise.defer();
        var contratosValidados = [];
        var contratosInvalidos = [];
        
        widget.dataSource.data.forEach(function(contrato, index) {
            if(!contratosValidados.includes(contrato.NRSEQCONTRATO)) {
                var percentualcont = contrato.NRPERCENTINDICE;
                widget.dataSource.data.forEach(function(data) {
                    if(contrato.NRSEQCONTRATO == data.NRSEQCONTRATO && contrato.CDINDICE != data.CDINDICE)
                        percentualcont += data.NRPERCENTINDICE;
                });
                
                contratosValidados.push(contrato.NRSEQCONTRATO);
                if(percentualcont != 100 && contrato.IDREAJUSTEAUTOMATICO == "Y") {
                    contratosInvalidos.push({
                        "NRSEQCONTRATO" : contrato.NRSEQCONTRATO,
                        "NRCONTRATOFCT" : contrato.NRCONTRATOFCT,
                        "index"         : index
                    });
                }
            }
        });
        
        if (contratosInvalidos.length == 0) {
            deferred.resolve();
        } else {
            var codContratos = "<br/>";
            contratosInvalidos.forEach(function(cont) {
                codContratos += cont.NRCONTRATOFCT + "<br/>";
            });

            ScreenService.confirmMessage("Atenção!<br><br>Os seguintes contratos não possuem somatório de 100% no percentual de utilização dos índices para o reajuste: <br/>" + codContratos + "<br/> Esses contratos serão desconsiderados. Deseja continuar?", "question", function() {
                contratosInvalidos.forEach(function(contrato) {
                    widget.dataSource.data.splice(contrato.index, 1);
                });
                
                deferred.resolve();
            },
            function() {
                deferred.reject();
            });
        }
        
        return deferred.promise;
    };
    
    this.habilitaFieldsSeguro = function(widget) {
        var dataInicial = widget.getField("DTINISEGURO");
        var dataFinal = widget.getField("DTFINSEGURO");
        var renovarSeguro = widget.currentRow.IDRENOVASEGURO == 'Y';
        
        dataInicial.readOnly = !renovarSeguro;
        dataFinal.readOnly = !renovarSeguro;
        
        dataInicial.validations = renovarSeguro ? {"required": true} : {};
        dataFinal.validations = renovarSeguro ? {"required": true} : {};
    };
    
    this.calcularPrazoVigenciaSeguro = function(widget) {
        var row = widget.seguro;
        var antigaDataInicial = moment(row.DTINISEGURO, "DD/MM/YYYY");
        var antigaDataFinal = moment(row.DTFINSEGURO, "DD/MM/YYYY");
        var prazo = antigaDataFinal.diff(antigaDataInicial, "days");
        antigaDataFinal.add(1, "days");
        widget.currentRow.DTINISEGURO = antigaDataFinal.format("DD/MM/YYYY");
        antigaDataFinal.add(prazo, "days");
        widget.currentRow.DTFINSEGURO = antigaDataFinal.format("DD/MM/YYYY");
        widget.currentRow.NRDIASVIGENCIA = prazo + 1;
    };
    
    this.calcularPrazoVigSeguro = function(widget) {
        var dataIniSeguro = widget.currentRow.DTINISEGURO;
        var dataFinSeguro = widget.currentRow.DTFINSEGURO;
        
        var prazo = Utilities.getDiasEntreDatas(dataIniSeguro, dataFinSeguro);
        if (prazo === null) {
            widget.currentRow.NRDIASVIGENCIA = prazo;
        }
        else if (prazo >= 0) {
            widget.currentRow.NRDIASVIGENCIA = prazo + 1;
        }
        else {
            ScreenService.showMessage("A data final da vigência do seguro deve ser maior que a data inicial.");
            widget.currentRow.NRDIASVIGENCIA = "prazo inválido";
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00004108_RenovacaoContratoController", Sm00004108_RenovacaoContratoController);
})

// FILE: ./assets/js/controllers/Sm00004109_Negociacao_Contrato_Controller.js
function Sm00004109_Negociacao_Contrato_Controller(General, ScreenService, templateManager, Utilities, Operador_Repository, ZhFilterPreferences) {
    
    
    /* Limpa os campos da tela e seta o filtro de Parceiro de negócio no agrupamento e no contrato*/
    this.setaFiltroParceiro = function(widget) {
         
         var nrparcnegocio = widget.currentRow.NRPARCNEGOCIO;
         
         /*-- Agrupamento --*/
         
         /* Limpa campos de Agrupamento */
         widget.currentRow.NRAGRUPAMENTO = null;
         widget.currentRow.NMAGRUPAMENTO = null;
         
         /* Limpa campos de Grupo de Faturamento */
         widget.currentRow.GRUPOFATURA      = null;
         widget.currentRow.CDFILIAL         = null;
         widget.currentRow.NMFILIAL         = null;
         widget.currentRow.IDTPFATURA       = null;
         widget.currentRow.DSTPFATURA       = null;
         widget.currentRow.NRPARCFATURA     = null;
         widget.currentRow.NMSECUNDARIPARC  = null;
         widget.currentRow.CDTIPOOPERFATURA = null;
         widget.currentRow.NMTIPOOPER       = null;
         widget.currentRow.CDSERIENF        = null;
         widget.currentRow.CDINSCESTA       = null;
         widget.currentRow.CDINSCMUNI       = null;
         widget.currentRow.CDEMPRESA        = null;
         widget.currentRow.DSSERIE          = null;
         
         /* Seta fitro no Agrupamento */
         if ((nrparcnegocio != "") && (nrparcnegocio != null)){
             widget.getField("NMAGRUPAMENTO").readOnly = false;
             widget.getField("NMAGRUPAMENTO").dataSourceFilter[0].value = nrparcnegocio;
         }
         else {
             widget.getField("NMAGRUPAMENTO").readOnly = true;
             widget.getField("NMAGRUPAMENTO").dataSourceFilter[0].value = "";
         }
         
         /*-- Contrato -- */
         
         /* Limpa campos de Contrato */
         widget.currentRow.NRSEQCONTRATO = null;
         widget.currentRow.NRCONTRATOFCT = null;
         
         /* Limpa campos do item do contrato */
         widget.currentRow.NRITEMOCONTRATO = null;
         widget.currentRow.NMITEMCONTRATO  = null;
         
         /* Limpa campos do Elemento */
         widget.currentRow.NRITEMELEMENTO = null;
         widget.currentRow.NMELEMENTO = null;
         
         /* Seta fitro no Contrato */
         if ((nrparcnegocio != "") && (nrparcnegocio != null)){
             widget.getField("NRCONTRATOFCT").readOnly = false;
             widget.getField("NRCONTRATOFCT").dataSourceFilter[0].value = "";
             widget.getField("NRCONTRATOFCT").dataSourceFilter[1].value = nrparcnegocio;
         }
         else {
             widget.getField("NRCONTRATOFCT").readOnly = true;
             widget.getField("NRCONTRATOFCT").dataSourceFilter[0].value = "";
             widget.getField("NRCONTRATOFCT").dataSourceFilter[1].value = "";
         }
         
         this.setaGrupoFaturamento(widget);
         this.setaItemContrato(widget);
         
     };
     
     
     /* Limpa os campos do Grupo de Faturamento e seta o filtro de agrupamento no Grupo de Faturamento*/
     this.setaGrupoFaturamento = function(widget) {
         
         var nragrupamento = widget.currentRow.NRAGRUPAMENTO;
         var idrealizarnegociacao = widget.currentRow.IDREALIZARNEGOCIACAO;
         
         /* Limpa campos de Grupo de Faturamento */
         widget.currentRow.GRUPOFATURA      = null;
         widget.currentRow.CDFILIAL         = null;
         widget.currentRow.NMFILIAL         = null;
         widget.currentRow.IDTPFATURA       = null;
         widget.currentRow.DSTPFATURA       = null;
         widget.currentRow.NRPARCFATURA     = null;
         widget.currentRow.NMSECUNDARIPARC  = null;
         widget.currentRow.CDTIPOOPERFATURA = null;
         widget.currentRow.NMTIPOOPER       = null;
         widget.currentRow.CDSERIENF        = null;
         widget.currentRow.CDINSCESTA       = null;
         widget.currentRow.CDINSCMUNI       = null;
         widget.currentRow.CDEMPRESA        = null;
         widget.currentRow.DSSERIE          = null;
         
         /* Limpa campos de Contrato */
         widget.currentRow.NRSEQCONTRATO = null;
         widget.currentRow.NRCONTRATOFCT = null;

         if (idrealizarnegociacao == 'M') {
             /* Seta fitro no Grupo de Faturamento */
             if ((nragrupamento != "") && (nragrupamento != null)){
                 /* Grupo Faturamento */
                 widget.getField("GRUPOFATURA").readOnly = false;
                 widget.getField("GRUPOFATURA").validations = {"required": true};
                 widget.getField("GRUPOFATURA").dataSourceFilter[0].value = nragrupamento;
                 
                 /* Contrato */
                 widget.getField("NRCONTRATOFCT").dataSourceFilter[0].value = nragrupamento;
             }
             else {
                 /*Grupo Faturamento*/
                 widget.getField("GRUPOFATURA").readOnly = true;
                 widget.getField("GRUPOFATURA").validations = {};
                 widget.getField("GRUPOFATURA").dataSourceFilter[0].value = "";
                 
                 /* Contrato */
                 widget.getField("NRCONTRATOFCT").dataSourceFilter[0].value = "";
             }
             
             this.setaItemContrato(widget);
         }
         else {
            if ((nragrupamento != "") && (nragrupamento != null)) {
               /*Grupo Faturamento*/
               widget.getField("GRUPOFATURA").readOnly = true;
               
               /*Contrato*/
               widget.getField("NRCONTRATOFCT").readOnly = true;
            }
            else {
                 /*Grupo Faturamento*/
                 widget.getField("GRUPOFATURA").readOnly = true;
                 widget.getField("GRUPOFATURA").validations = {};
                 widget.getField("GRUPOFATURA").dataSourceFilter[0].value = "";
                 
                 /* Contrato */
                 widget.getField("NRCONTRATOFCT").readOnly = false;
                 widget.getField("NRCONTRATOFCT").dataSourceFilter[0].value = "";
             }

        }
     };
     
     /* Limpa os campos de item e elemento do contrato e seta o filtro de contrato no item de contrato*/
     this.setaItemContrato = function(widget) {
         
         var nrseqcontrato = widget.currentRow.NRSEQCONTRATO;
         var nragrupamento = widget.currentRow.NRAGRUPAMENTO;
         var idrealizarnegociacao = widget.currentRow.IDREALIZARNEGOCIACAO;
         
         /* Limpa campos do item do contrato */
         widget.currentRow.NRITEMOCONTRATO = null;
         widget.currentRow.NMITEMCONTRATO  = null;
         
         /* Limpa campos do Elemento */
         widget.currentRow.NRITEMELEMENTO = null;
         widget.currentRow.NMELEMENTO = null;
         
         if (idrealizarnegociacao == 'M') {

             /* Seta fitro no Item */
             if ((nrseqcontrato != "") && (nrseqcontrato != null)){
                 widget.getField("NMITEMCONTRATO").readOnly = false;
                 widget.getField("NMITEMCONTRATO").validations = {"required": true};
                 widget.getField("NMITEMCONTRATO").dataSourceFilter[0].value = nrseqcontrato;
             }
             else {
                 widget.getField("NMITEMCONTRATO").readOnly = true;
                 widget.getField("NMITEMCONTRATO").validations = {};
                 widget.getField("NMITEMCONTRATO").dataSourceFilter[0].value = "";
             }
             
             this.setaElemento(widget);
         }
         else {

             /* Seta fitro no Item */
             if ((nrseqcontrato != "") && (nrseqcontrato != null)){
               
                 widget.getField("NMAGRUPAMENTO").readOnly = true; 
                 widget.getField("NMITEMCONTRATO").readOnly = true;
                 widget.getField("PAERCELASAPUR").dataSourceFilter[0].value = (nrseqcontrato == null) ? nragrupamento : nrseqcontrato;
             }
             else {
                 widget.getField("NMAGRUPAMENTO").readOnly = false; 
                 widget.getField("NMITEMCONTRATO").readOnly = true;
                 widget.getField("PAERCELASAPUR").dataSourceFilter[0].value = "";
             }

             this.setaElemento(widget);

         }
     };
     
     
     /* Limpa os campos do elemento de contrato a e seta o filtro de item no elemento */
     this.setaElemento = function(widget) {
         
         var nritemocontrato = widget.currentRow.NRITEMOCONTRATO;
         var idrealizarnegociacao = widget.currentRow.IDREALIZARNEGOCIACAO;
         
         /* Limpa campos do Elemento */
         widget.currentRow.NRITEMELEMENTO = null;
         widget.currentRow.NMELEMENTO = null;

         if (idrealizarnegociacao == 'M') {

             /* Seta fitro no Elemento */
             if ((nritemocontrato != "") && (nritemocontrato != null)){
                 widget.getField("NMELEMENTO").readOnly = false;
                 widget.getField("NMELEMENTO").dataSourceFilter[0].value = nritemocontrato;
             }
             else {
                 widget.getField("NMELEMENTO").readOnly = true;
                 widget.getField("NMELEMENTO").dataSourceFilter[0].value = "";
             }
         } else{

            widget.getField("NMELEMENTO").readOnly = true;
            widget.getField("NMELEMENTO").dataSourceFilter[0].value = "";

         }
    };
    
    this.setaIdMultiplicaQtde = function(widget){
         var nmElemento = widget.currentRow.NMELEMENTO;
         
         /* libera ou bloqueia o campo de multiplicar quantidade */
         if ((nmElemento != "") && (nmElemento != null)){
             widget.getField("IDMULTIPLICAQTMED").readOnly = false;
             widget.getField("IDMULTIPLICAQTMED").validations = {"required": true};

         }
         else {
             widget.getField("IDMULTIPLICAQTMED").readOnly = true;
             widget.getField("IDMULTIPLICAQTMED").validations = {};
             widget.currentRow.IDMULTIPLICAQTMED = "N";
         }
    };
     
    this.limparFiltrosNewRowNegociacao = function (widget){
        
        widget.getField("NMAGRUPAMENTO").dataSourceFilter[0].value = "";
        widget.getField("GRUPOFATURA").dataSourceFilter[0].value = "";
             
        widget.getField("NRCONTRATOFCT").dataSourceFilter[0].value = "";
        widget.getField("NMITEMCONTRATO").dataSourceFilter[0].value = "";
        widget.getField("NMELEMENTO").dataSourceFilter[0].value = "";
        
        General.widgetNewRow(widget);
    };
    
     
    this.preencheVariavelNegociacao = function(widget){
        
        var idrealizarnegociacao = widget.currentRow.IDREALIZARNEGOCIACAO;
        if (idrealizarnegociacao == 'M') {
            var varNegociacao = '';
            
            var nrItemelemento = widget.currentRow.NRITEMELEMENTO;
            var nmElemento = widget.currentRow.NMELEMENTO;
            
            var nrItemContrato = widget.currentRow.NRITEMOCONTRATO;
            var nmItemContrato = widget.currentRow.NMITEMCONTRATO;
            
            var nrAgrupamento = widget.currentRow.NRAGRUPAMENTO;
            var nmAgrupamento = widget.currentRow.NMAGRUPAMENTO;
            
            if (nrItemelemento){
                
                varNegociacao = 'Elemento - '+nmElemento;
                
            } 
            else if (nrItemContrato){
                
                varNegociacao = 'Serviço - '+nmItemContrato;
            }
            else if (nrAgrupamento){
                
                varNegociacao = 'Agrupamento - '+nmAgrupamento; 
            }
            
            widget.currentRow.DSVARIAVELNEGOCIA = varNegociacao;
        }
        else{
            varNegociacao = 'Apuração de Faturamento';
            widget.currentRow.DSVARIAVELNEGOCIA = varNegociacao;
        }
        
        return varNegociacao;
        
    };
     
     /**
     * insere as assinaturas da proposta 
    **/
    this.saveNegociacaoContrato = function(widget) {

       var wdgNegociacaoContr = templateManager.container.getWidget("wdgNegociacaoContr");
       
       var params = {
            "NEGOCIACAO" : wdgNegociacaoContr.currentRow,
            "PARCELAS": widget.dataSource.data
        };
        
        General.callRestEngine('product#/fct_negcontrato_widget/saveDadosNegContrato', "Row", params).then(function(response){
            var resultado = response.dataset.dados.existeParcelaApurada;
            
            ScreenService.closeSwipe().then(function(){
                this.habilitaWidgetParcelasNormal();
                
                if (resultado==false){
                    ScreenService.showMessage("Processo realizado com sucesso!");
                } else {
                    ScreenService.showMessage("Processo realizado com sucesso!<br><br>OBS: Algumas parcelas não foram regeradas porque já estão apuradas.");
                } 
                wdgNegociacaoContr.reload();
            }.bind(this));
        }.bind(this), 
        function(error) {
            Utilities.showErrorMessage(error);
        }.bind(this));
        
    };
    
    /** Busca o operador logado a sessão atual 
     * @params{Object} widget de proposta
     */
    this.wdgGridOnEnterOnce = function(widget) {
        //ZhFilterPreferences.openFilterWidget(templateManager.container.getActiveWidget());
        Utilities.setRowFilterWidgetSemDefaultValue(widget, ["IDATIVO"], ["S"]);
    };
    
    /**
     * salva as alterações das parcelas de negociação da proposta 
    **/
    this.editParcelasNegoContr = function(widget) {
        var wdgNegociacaoContr = templateManager.container.getWidget("wdgNegociacaoContr");
        
        widget.dataSource.data.forEach(function(row) {
            delete row.__rightActions;
            delete row.__leftActions;
        });
        
        var params = {
            "PARCELAS": widget.dataSource.data,
            "NEGOCIACAO": {
                "NRPARCNEGOCIO" : wdgNegociacaoContr.currentRow.NRPARCNEGOCIO,
                "NRSEQCONTRATO" : wdgNegociacaoContr.currentRow.NRSEQCONTRATO,
                "NRITEMELEMENTO": wdgNegociacaoContr.currentRow.NRITEMELEMENTO,
                "NRITEMCONTRATO": wdgNegociacaoContr.currentRow.NRITEMOCONTRATO,
                "NRAGRUPAMENTO" : wdgNegociacaoContr.currentRow.NRAGRUPAMENTO,
                "NRPARCNEGOCIO" : wdgNegociacaoContr.currentRow.NRPARCNEGOCIO
            }
        };
        
        General.callRestEngine('product#/fct_negcontrato_widget/editParcelasNegoContr', "Row", params).then(function(response) {
            var obs = "";
            
            var existeParcelaApurada = response.dataset.retorno.existeParcelaApurada;
            if (existeParcelaApurada) {
                obs = "<br/><br/>OBS: Algumas das parcelas não puderam ser alteradas porque estão sendo utilizadas em registros de Apuração de Faturamento que já foram apurados.";
            }
            
            ScreenService.showMessage("Processo realizado com sucesso!" + obs);
            widget.list();
            widget.reload();
        },
        function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    
    /** Busca o operador logado a sessão atual 
     * @params{Object} widget de proposta
     */
    this.gridParcelasActivate = function(widget) {
        
        var wdgNegociacaoContr = templateManager.container.getWidget("wdgNegociacaoContr");
        
        var idtiponegocia = wdgNegociacaoContr.currentRow.IDTIPONEGOCIA;
        widget.getField("VRPARCELANEGOCIA").label = getLabelTipoNegociacao(idtiponegocia);
        
        widget.list();
        widget.dataSourceFilter[0].value = wdgNegociacaoContr.currentRow.NRNEGCONTRATO;
        widget.reload().then(function(){
        
            var elemento = wdgNegociacaoContr.currentRow.NRITEMELEMENTO;
        
            /* libera ou bloqueia o campo de multiplicar quantidade */
            if ((elemento != "") && (elemento != null)){
                widget.getField("IDMULTIPLICAQTMED").inlineEditable = "checkbox";
            }
            else {
                widget.getField("IDMULTIPLICAQTMED").inlineEditable = "";
            } 
             
            var actSaveInline = widget.getAction('save');
            if (actSaveInline) actSaveInline.isVisible = false;
        });
        
    };
    
    var getLabelTipoNegociacao = function(idtiponegocia) {
        var tipos = {
            "VF": "Valor Fixo",
            "DP": "Desconto (Percentual)",
            "DV": "Desconto (Valor)",
            "AP": "Acréscimo (Percentual)",
            "AV": "Acréscimo (Valor)"
        };
        
        return idtiponegocia ? tipos[idtiponegocia] : "Valor";
    };
    
    this.adicionarParcelasOpen = function() {
        
        var wdgPopupAdicionarParcelas = templateManager.container.getWidget('wdgPopupAdicionarParcelas');
        
        wdgPopupAdicionarParcelas.currentRow.DTINICIO = null;
        wdgPopupAdicionarParcelas.currentRow.DTFINAL = null;
        wdgPopupAdicionarParcelas.currentRow.IDMULTIPLICAQTMED = null;
        wdgPopupAdicionarParcelas.currentRow.VRPARCELANEGOCIA = null;
        
        ScreenService.openPopup(wdgPopupAdicionarParcelas).then(function() {
            wdgPopupAdicionarParcelas.dataSource.data = [];
            wdgPopupAdicionarParcelas.newRow();
            
            var wdgNegociacaoContr = templateManager.container.getWidget('wdgNegociacaoContr');
            var nmElemento = wdgNegociacaoContr.currentRow.NMELEMENTO;
         
            /* libera ou bloqueia o campo de multiplicar quantidade */
            if ((nmElemento != "") && (nmElemento != null)){
                wdgPopupAdicionarParcelas.getField("IDMULTIPLICAQTMED").readOnly = false;
                wdgPopupAdicionarParcelas.getField("IDMULTIPLICAQTMED").validations = {"required": true};
            }
            else {
                wdgPopupAdicionarParcelas.getField("IDMULTIPLICAQTMED").readOnly = true;
                wdgPopupAdicionarParcelas.getField("IDMULTIPLICAQTMED").validations = {};
                wdgPopupAdicionarParcelas.currentRow.IDMULTIPLICAQTMED = "N";
            }
        });
    };
    
    this.validaDatasParcelasNegContr = function(widget) {
        
        if (widget.isValid()){
            var wdgNegociacaoContr = templateManager.container.getWidget("wdgNegociacaoContr");
            var wdgParcelasNegociacao = templateManager.container.getWidget("wdgParcelasNegociacao");
            
            var nrnegcontrato     = wdgNegociacaoContr.currentRow.NRNEGCONTRATO;
            var dtiniparcnegocia  = widget.currentRow.DTINICIO;
            var dtfimparcnegocia  = widget.currentRow.DTFINAL;
            var idmultiplicaqtmed = widget.currentRow.IDMULTIPLICAQTMED;
            var vrparcelanegocia  = widget.currentRow.VRPARCELANEGOCIA;
            
            var dtinicio = widget.currentRow.DTINICIO;
            var dtFinal  = widget.currentRow.DTFINAL;
            
            dtinicio = moment(dtinicio, "DD/MM/YYYY");
            dtFinal = moment(dtFinal, "DD/MM/YYYY");
            
            var testaDatas = dtFinal.diff(dtinicio);
                    
            if (testaDatas > 0) {
                
                
                dtinicio = dtinicio.format("DD/MM/YYYY");
                dtFinal = dtFinal.format("DD/MM/YYYY");
                
                var params = {
                    "NRNEGCONTRATO": nrnegcontrato,
                    "DTINICIO"     : dtinicio,
                    "DTFINAL"      : dtFinal
                };
                
                General.callRestEngine("product#/fct_negcontrato_widget/recebeMinMaxParcelas", "Row", params).then(function(response) {
                    
                    var parcelasMaxMin = response.dataset.dados.parcelas;
                    
                    var parcelaMin = parcelasMaxMin.DTMINPARCELAS;
                    var parcelaMax = parcelasMaxMin.DTMAXPARCELAS;
                    
                    if((parcelaMin != null) && (parcelaMax != null)){
                        
                        var idInsere = 'N'; /* Define se a inserção pode ser feita. */
                        var idIntervalodatas = 'N'; /* Verifica se o período já inserido está compreendido entre o novo período */
                        
                        dtiniparcnegocia = moment(dtiniparcnegocia, "DD/MM/YYYY");
                        dtfimparcnegocia = moment(dtfimparcnegocia, "DD/MM/YYYY");
                        parcelaMin       = moment(parcelaMin, "DD/MM/YYYY");
                        parcelaMax       = moment(parcelaMax, "DD/MM/YYYY");
                        
                        var dtAuxiniparcnegocia = dtiniparcnegocia;
                        var dtAuxfimparcnegocia = dtfimparcnegocia;
                        
                        dtAuxiniparcnegocia.add(-1, "day");
                        dtAuxfimparcnegocia.add(1, "day");
                        
                        var dtDiff_Ini = dtAuxfimparcnegocia.diff(parcelaMin); // Compara dias entre a data final que está sendo inserida e a data iniical ja inserida
                        var dtDiff_Fin = dtAuxiniparcnegocia.diff(parcelaMax); // Compara dias entre a data inicial que está sendo inserida e a data final ja inserida
                        
                        if (dtDiff_Ini == 0){
                            
                            idInsere = 'S';
                            
                        } else if (dtDiff_Fin == 0){
                            
                            idInsere = 'S';
                            
                        }
                        else if ((dtDiff_Ini >= 0) && (dtDiff_Fin <= 0)){
                            idInsere = 'S';
                            idIntervalodatas = 'S';
                        }
                        
                        if (idInsere == 'S') {
                            
                            if (idIntervalodatas == 'S'){
                                
                                ScreenService.showCustomDialog("Existem parcelas já criadas dentro do período solicitado para inclusão de novas parcelas de negociação. O que você deseja fazer com estas parcelas?", 'CONFIRMATION', null, 
                                [
                                    {
                                        "label": "Manter",
                                        "code": function() {
                        
                                            this.adicionarParcelasNegContr(nrnegcontrato, dtinicio, dtFinal, idmultiplicaqtmed, vrparcelanegocia, 'N');
                                        }.bind(this)
                                    },
                                    {
                                        "label": "Substituir",
                                        "code": function() {
                                            
                                            
                                            this.adicionarParcelasNegContr(nrnegcontrato, dtinicio, dtFinal, idmultiplicaqtmed, vrparcelanegocia, 'S');
                                        }.bind(this)
                                    },
                                    {
                                        "label": "Cancelar",
                                        "code": function() {}
                                    }
                                ]);
                            }
                            else{
                                
                                this.adicionarParcelasNegContr(nrnegcontrato, dtinicio, dtFinal, idmultiplicaqtmed, vrparcelanegocia, 'S');
                            }
                        }
                        else {
                            ScreenService.showMessage("Operação bloqueada!<br><br> O período a ser adicionado deve fazer sequência ao período já existente."); 
                        }
                    }
                    else{
                        /* Se não houver nenhuma parcela, ele cria. */
                        this.adicionarParcelasNegContr(nrnegcontrato, dtinicio, dtFinal, idmultiplicaqtmed, vrparcelanegocia, 'S');  
                    }
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
            else {
                
                 ScreenService.showMessage("Operação bloqueada!<br><br> Data final deve ser maior do que a data inicial."); 
            }
        }
    };
    
    this.adicionarParcelasNegContr = function(nrnegcontrato, dtiniparcnegocia, dtfimparcnegocia, idmultiplicaqtmed, vrparcelanegocia, idSubstitui) {
        var wdgParcelasNegociacao  = templateManager.container.getWidget("wdgParcelasNegociacao");
        var wdgNegociacaoContr  = templateManager.container.getWidget("wdgNegociacaoContr");
        
        var params = {
            "NRNEGCONTRATO"     : nrnegcontrato,
            "DTINIPARCNEGOCIA"  : dtiniparcnegocia,
            "DTFIMPARCNEGOCIA"  : dtfimparcnegocia,
            "IDMULTIPLICAQTMED" : idmultiplicaqtmed,
            "VRPARCELANEGOCIA"  : vrparcelanegocia,
            "IDSUBSTITUI"       : idSubstitui,
            "NRPARCNEGOCIO"     : wdgNegociacaoContr.currentRow.NRPARCNEGOCIO,
            "NRSEQCONTRATO"     : wdgNegociacaoContr.currentRow.NRSEQCONTRATO,
            "NRITEMELEMENTO"    : wdgNegociacaoContr.currentRow.NRITEMELEMENTO,
            "NRITEMCONTRATO"    : wdgNegociacaoContr.currentRow.NRITEMOCONTRATO,
            "NRAGRUPAMENTO"     : wdgNegociacaoContr.currentRow.NRAGRUPAMENTO,
            "NRPARCNEGOCIO"     : wdgNegociacaoContr.currentRow.NRPARCNEGOCIO
        };
            
        General.callRestEngine("product#/fct_negcontrato_widget/adicionarParcelasNegContr", "Row", params).then(function(response) {
            var resultado = response.dataset.dados.existeParcelaApurada;
            
            if (resultado==false){
                ScreenService.showMessage("Processo realizado com sucesso!");
            } else {
                ScreenService.showMessage("Processo realizado com sucesso!<br><br>OBS: Algumas parcelas não foram regeradas porque já estão apuradas.");
            } 
            
            ScreenService.closePopup();
            wdgParcelasNegociacao.reload();
            
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        }); 
    };
    
    this.excluirParcela = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
            var params = {
                "NRPARCELASNEGCONTR" : widget.currentRow.NRPARCELASNEGCONTR,
                "NRNEGCONTRATO"      : widget.currentRow.NRNEGCONTRATO
            };
                
            General.callRestEngine("product#/fct_negcontrato_widget/excluirParcela", "Row", params).then(function() {
                    
                ScreenService.showMessage("Processo realizado com sucesso!"); 
                widget.reload();
                
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            }); 
        });
    };
    
    this.avancarParcNegociacaoContrato = function(widget) {
        if (widget.isValid()) {
            
            var varNegociacao = this.preencheVariavelNegociacao(widget);
        
            if (varNegociacao != ""){
                
                var dtinicio = widget.currentRow.DTINICIO;
                var dtFinal  = widget.currentRow.DTFINAL;
                
                dtinicio = moment(dtinicio, "DD/MM/YYYY");
                dtFinal = moment(dtFinal, "DD/MM/YYYY");
                
                var testaDatas = dtFinal.diff(dtinicio);
                        
                if (testaDatas > 0) {
                
                    General.callRestEngine("product#/fct_negcontrato_widget/validarNegociacoesInseridas", "Row", widget.currentRow).then(function(response) {
                    
                        if (response.dataset.dados.valida == 'OK'){ 
                            
                            var wdgParcelasNegociacao = templateManager.container.getWidget('wdgParcelasNegociacao');
                            wdgParcelasNegociacao.isVisible = false;
                            
                            var wdgParcelasNegociacaoWiz = templateManager.container.getWidget('wdgParcelasNegociacaoWiz');
                            wdgParcelasNegociacaoWiz.isVisible = true;
                            
                            wdgParcelasNegociacaoWiz.enabled = true;
                            wdgParcelasNegociacaoWiz.isVisible = true;
                            
                            var idtiponegocia = widget.currentRow.IDTIPONEGOCIA;
                            wdgParcelasNegociacaoWiz.getField("VRPARCELANEGOCIA").label = getLabelTipoNegociacao(idtiponegocia);
                            
                            wdgParcelasNegociacaoWiz.activate().then(function() {
                                //wdgParcelasNegociacaoWiz.getAction("cancel").isVisible = false;
                                wdgParcelasNegociacaoWiz.actions.forEach(function(act, position) {
                                    if (act.name == "cancel" && act.showAsAction == "back_inline_edit") {
                                        delete wdgParcelasNegociacaoWiz.actions[position];
                                        return;
                                    }
                                });
                                
                                wdgParcelasNegociacaoWiz.inlineEdit();
                                this.carregarDadosParcelas(widget);
                            }.bind(this));
                        }
                        else{
                            ScreenService.showMessage("Operação bloqueada!<br><br>Já existe um registro de negociação com os dados que estão sendo inseridos."); 
                        }
                    
                    }.bind(this),
                    function(error) {
                        Utilities.showErrorMessage(error);
                    }); 
                }
                else {
                
                     ScreenService.showMessage("Operação bloqueada!<br><br> Data final deve ser maior do que a data inicial."); 
                }
            }
            else {
            
                ScreenService.showMessage("Operação bloqueada!<br><br> Não foi selecionado uma variável de negociação para alteração do valor.<br><br> Favor preencher uma das variáveis abaixo:<br><br> - Agrupamento <br> - Serviço / Produto <br> - Elemento");
            }
        }
        else {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
    };
    
    this.carregarDadosParcelas = function(widget) {
        
        var wdgParcelasNegociacaoWiz = templateManager.container.getWidget('wdgParcelasNegociacaoWiz');
        
        dtinicio = widget.currentRow.DTINICIO;
        dtFinal = widget.currentRow.DTFINAL;
        
        dtinicio = moment(dtinicio, "DD/MM/YYYY").format("DD/MM/YYYY");
        dtFinal = moment(dtFinal, "DD/MM/YYYY").format("DD/MM/YYYY");
        
        var params = {
            "DTINICIO"          : dtinicio, 
            "DTFINAL"           : dtFinal, 
            "IDMULTIPLICAQTMED" : widget.currentRow.IDMULTIPLICAQTMED, 
            "VRPARCELANEGOCIA"  : widget.currentRow.VRPARCELANEGOCIA
        };
            
        General.callRestEngine("product#/fct_negcontrato_widget/gerarParcelasNegContratoWiz", "Row", params).then(function(response) {
                
            wdgParcelasNegociacaoWiz.dataSource.data = response.dataset.dados.parcelas;
            wdgParcelasNegociacaoWiz.getAction('save').isVisible = false;
            wdgParcelasNegociacaoWiz.reload();
            
            var elemento = widget.currentRow.NRITEMELEMENTO;
        
            /* libera ou bloqueia o campo de multiplicar quantidade */
             if ((elemento != "") && (elemento != null)){
                 wdgParcelasNegociacaoWiz.getField("IDMULTIPLICAQTMED").inlineEditable = "checkbox";
    
             }
             else {
                 wdgParcelasNegociacaoWiz.getField("IDMULTIPLICAQTMED").inlineEditable = "";
             } 
            
        })
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        }); 
        
    };
    
    this.voltarNegociacao = function(widget) {
        widget.enabled = false;
        var wdgNegociacaoContr = templateManager.container.getWidget("wdgNegociacaoContr");
        
        var negociacao = wdgNegociacaoContr.currentRow;
        wdgNegociacaoContr.activate();
        wdgNegociacaoContr.moveToPrev();
        
        wdgNegociacaoContr.edit();
        
        wdgNegociacaoContr.currentRow = negociacao;
        
    };

    this.habilitaWidgetParcelasNormal = function(){
        
        var wdgParcelasNegociacaoWiz = templateManager.container.getWidget('wdgParcelasNegociacaoWiz');
        wdgParcelasNegociacaoWiz.isVisible = false;
        wdgParcelasNegociacaoWiz.enabled   = false;
        
        var wdgParcelasNegociacao = templateManager.container.getWidget('wdgParcelasNegociacao');
        wdgParcelasNegociacao.isVisible  = true;
        wdgParcelasNegociacao.enabled = true;
        
        var wdgNegociacaoContr = templateManager.container.getWidget('wdgNegociacaoContr');
        
        wdgNegociacaoContr.getAction("save").isVisible = false;
        wdgNegociacaoContr.getAction("Avancar").isVisible = true;
        
        wdgNegociacaoContr.getField("IDMULTIPLICAQTMED").isVisible = true;
        wdgNegociacaoContr.getField("VRPARCELANEGOCIA").isVisible = true;
        
        wdgNegociacaoContr.getField("IDMULTIPLICAQTMED").validations = {"required": true};
        wdgNegociacaoContr.getField("VRPARCELANEGOCIA").validations = {"required": true};
    
    };
    
    this.editarWidgetNeg = function(widget) {
        General.widgetEdit(widget);
        
        widget.getAction("save").isVisible = true;
        widget.getAction("Avancar").isVisible = false;
        
        widget.getField("IDMULTIPLICAQTMED").isVisible = false;
        widget.getField("VRPARCELANEGOCIA").isVisible = false;
        
        widget.getField("VRPARCELANEGOCIA").validations = {};
        widget.getField("IDMULTIPLICAQTMED").validations = {};

        widget.getField("PAERCELASAPUR").isVisible = false;
        widget.getField("PAERCELASAPURDESC").isVisible = true;


    };
    
    this.salvarWidgetNeg = function(widget) {
        if (widget.isValid()) {
            var idrealizarnegociacao = widget.currentRow.IDREALIZARNEGOCIACAO;

            if (idrealizarnegociacao == 'M') {
                General.widgetSave(widget);
            }
            else {
                this.preencheVariavelNegociacao(widget);
                this.saveNegociacaoContrato(widget);
            }
            
            widget.getAction("save").isVisible = false;
            widget.getAction("Avancar").isVisible = true;
            
            widget.getField("IDMULTIPLICAQTMED").isVisible = true;
            widget.getField("VRPARCELANEGOCIA").isVisible = true;
            
            widget.getField("VRPARCELANEGOCIA").validations = {"required": true};
        widget.getField("IDMULTIPLICAQTMED").validations = {"required": true};
        }
    };
    
    this.controlaRealizaNegociacao = function(newRow) {

         var wdgNegociacaoContr = templateManager.container.getWidget("wdgNegociacaoContr");
         if (newRow == true){
            var idrealizarnegociacao = 'M';
         }
         else {
            var idrealizarnegociacao = wdgNegociacaoContr.currentRow.IDREALIZARNEGOCIACAO;
         }

         if (idrealizarnegociacao == 'M') {

            wdgNegociacaoContr.getField("DTINICIO").readOnly = false;
            wdgNegociacaoContr.getField("DTINICIO").validations = {"required": true};
            wdgNegociacaoContr.getField("DTFINAL").readOnly = false;
            wdgNegociacaoContr.getField("DTFINAL").validations = {"required": true};
            wdgNegociacaoContr.getField("PAERCELASAPUR").readOnly = true;
            wdgNegociacaoContr.getField("PAERCELASAPUR").validations = {};

            /* IDMULTIPLICAQTMED */
            this.setaIdMultiplicaQtde(wdgNegociacaoContr);

            wdgNegociacaoContr.getAction('save').isVisible = false;
            wdgNegociacaoContr.getAction('Avancar').isVisible = true;

           this.habilitaWidgetParcelasNormal();

         }
         else {

            wdgNegociacaoContr.getField("DTINICIO").readOnly = true;
            wdgNegociacaoContr.getField("DTINICIO").validations = {};
            wdgNegociacaoContr.getField("DTFINAL").readOnly = true;
            wdgNegociacaoContr.getField("DTFINAL").validations = {};
            wdgNegociacaoContr.getField("PAERCELASAPUR").readOnly = false;
            wdgNegociacaoContr.getField("PAERCELASAPUR").validations = {"required": true};
            
            wdgNegociacaoContr.getField("IDMULTIPLICAQTMED").readOnly = true;
            wdgNegociacaoContr.getField("IDMULTIPLICAQTMED").validations = {};
            wdgNegociacaoContr.currentRow.IDMULTIPLICAQTMED = "N";

            wdgNegociacaoContr.getAction('save').isVisible = true;
            wdgNegociacaoContr.getAction('Avancar').isVisible = false;

            var wdgParcelasNegociacaoWiz = templateManager.container.getWidget('wdgParcelasNegociacaoWiz');
            wdgParcelasNegociacaoWiz.isVisible = false;
            wdgParcelasNegociacaoWiz.enabled   = false;
            
            var wdgParcelasNegociacao = templateManager.container.getWidget('wdgParcelasNegociacao');
            wdgParcelasNegociacao.isVisible  = false;
            wdgParcelasNegociacao.enabled = false;
         }
    };

    this.excluirWidgetNeg = function(widget) {
        var idrealizarnegociacao = widget.currentRow.IDREALIZARNEGOCIACAO;

        if (idrealizarnegociacao == 'M') {
            General.deleteCrud(widget);
        }
        else {
            ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
                
                General.callRestEngine("product#/fct_negcontrato_widget/excluirNegociacaoApura", "Row", widget.currentRow).then(function(resultado) {
                    
                    var resultado = resultado.dataset.excluirNegociacaoApura.resultado;

                    if (resultado == 'OK') {
                        ScreenService.showMessage("Processo realizado com sucesso!"); 
                    }
                    else {
                        ScreenService.showMessage("Algumas parcelas de apuração de faturamento já haviam sido apuradas. Por este motivo a negociação destas parcelas não foram excluídas.<br><br> As demais negociações de parcelas que estavam pendentes foram excluídas"); 
                    }
                    
                    ScreenService.closeSwipe();
                    widget.reload();
                })
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                }); 
            });
        }
        
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00004109_Negociacao_Contrato_Controller", Sm00004109_Negociacao_Contrato_Controller);
});

// FILE: ./assets/js/controllers/Sm00004110_Tipo_de_Rescisao_Controller.js
function Sm00004110_Tipo_de_Rescisao_Controller(General, Utilities, ScreenService, templateManager) {
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Tipo de Medida para salvar o novo registro
    */
    this.salvarTipoRescisao = function(widget){
        
        widget.currentRow.__columns_to_return = ["NRTIPORESCISAO", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
    
        General.widgetSave(widget).then(function(response){
            if(response.dataset && response.dataset.columnsToReturn[0]){
                widget.currentRow.NRTIPORESCISAO = response.dataset.columnsToReturn[0].NRTIPORESCISAO;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
        }.bind(this))
        .catch(function(response) {
            Utilities.showErrorMessage(response);
        });
    };
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00004110_Tipo_de_Rescisao_Controller', Sm00004110_Tipo_de_Rescisao_Controller);
});

// FILE: ./assets/js/controllers/Sm00005000_Parametrizacoes_Services_Controller.js
function Sm00005000_Parametrizacoes_Services_Controller(General, ScreenService, ZHPromise, templateManager, Utilities) {

    this.AfterLoadDataSorce = function(widget){
        
        if(widget.dataSource.data) {
            widget.currentRow = widget.dataSource.data[0];
            
            var wdgSignatariosPadroes = templateManager.container.getWidget("signatariosPadroes");
            var dstokencertsign = widget.currentRow.DSTOKENCERTSIGN;
            
            if (dstokencertsign == null) {
                wdgSignatariosPadroes.enabled = false;    
            } 
            else {
                wdgSignatariosPadroes.enabled = true;  
            }
        }
        
        widget.view();
    };

    this.onClick = function(widget) {
        if (widget.isValid()){
           this.saveParameterServico(widget);
           
           widget.getField("FCT_ALTPROPCONCPERF").reload();
           widget.getField("FCT_ALTCONTRAPROVPERF").reload();
        }
    };

    /**
     * Salva o novo registro
     * e seta IDATIVO
     * 
     * @param {object} widget widget de 
     * registro único de Parametrizações
     * que salva o mesmo.
    */
    this.saveParameterServico = function(widget){
        var params      = widget.currentRow;
        var validaEmail = this.validaEmailDesalocRec(widget);
        
        if(validaEmail){
                var parametrizacoesEmail = templateManager.container.getWidget("parametrizacoesEmail");
                this.setIdAtivo(widget);
                this.validaNovoCodigo(widget).then(function() {
                    General.callRestEngine('product#/fct_parametrogeral_widget/saveperfilacesso', "Row", params);
                    General.widgetSave(widget).then(function() {
                        widget.view();
                    }).catch(function(response) {
                        console.error(response);
                        if (response instanceof Object)
                            ScreenService.showMessage(response.data.error);
                        else
                            ScreenService.showMessage(response);
                    });
                }).catch(function(error) {
                    console.log(error);
                    ScreenService.showMessage(error);
                });
    
                parametrizacoesEmail.enabled = true;
        } else {
            ScreenService.showMessage("E-mail para Alerta de Desalocação de Recursos inválido<br>");
        }
    };
    
    /**
     * Valida os emails adicionados para rceber o alerta de 
     * desalocação de recursos
     * @params {widget} widget parametrização service
    **/
    this.validaEmailDesalocRec = function(widget){
        if(widget.currentRow.DSEMAILSDESALCREC)
            return this.validarEmailsParametrizacoes(widget, "DSEMAILSDESALCREC");
        else
            return true;
    };
    
    /**
     * Valida a filial do email que está sendo inserida.
     * @parms{Object} widget widget parametrização e-mail
    **/
    this.validaFilialEmail = function(widget){
        Utilities.validarCamposDuplicados(widget, ["NRPARAMEMAILFILIAL"], ["CDFILIAL"]).then(function(valida){
            if(valida){
                this.saveParameterEmail(widget);
            } else {
                ScreenService.showMessage("Filial selecionada já possui e-mail parametrizado. <br><br>");
            }
        }.bind(this))
        .catch(function(error){
            console.error(error);
        });
    };
    
     /**
     * Salva o registro completando o registro com os datos de paramentrição de e-mail
     * 
     * @param {object} widget widget de 
     * registro único de Parametrizações
     * que salva o mesmo.
    */
    this.saveParameterEmail = function(widget){
        var wdgParamService = templateManager.container.getWidget("parametrizacoesServices");

        General.widgetSave(widget).then(function(response){
            if(response.dataset && response.dataset.columnsToReturn[0]){
                widget.currentRow.NRORG              = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.NRPARAMEMAILFILIAL = response.dataset.columnsToReturn[0].NRPARAMEMAILFILIAL;
            }
        }.bind(this))
        .catch(function(response) {
            console.error(response);
            if (response instanceof Object)
                ScreenService.showMessage(response.data.error);
            else
                ScreenService.showMessage(response);
        });
        
        wdgParamService.enabled = true;
    };
    
    /**
     * Seta o campo IDATIVO para 
     * ser salvo no BD
     * 
     * @param {object} widget de 
     * registro único de Parametrizações 
     * para pegar a currentRow e setar 
     * o IDATIVO
    */
    this.setIdAtivo = function(widget) {
        widget.currentRow.IDATIVO = "S";
    };
    
    /**
     * Abre a edição de serviços controlando as tabbled nao ultilzada no momento
     * 
     * @param {object} widget de 
     * registro único de Parametrizações 
    */
    this.editWidgetServico = function(widget) {
        widget.parent.widgets.forEach(function(w,i){
            if(w.name !== "parametrizacoesServices"){
                w.enabled = false;
            }
        });
        
        widget.backupRow = Util.clone(widget.currentRow);
		
		widget.edit();
		widget.template = "widget/form.html";
		
		this.controlaUrlOM(widget);
		
		this.validarParametrosRenovacaoContrato(widget);
	    this.controlaCamposCertisign(widget);
    };
    
    this.cancelEditServico = function(widget) {
        
    	Object.keys(widget.backupRow).forEach(function (key) {
			widget.currentRow[key] = widget.backupRow[key];
		});
       widget.parent.widgets.forEach(function(w,i){
            if(w.name !== "parametrizacoesServices"){
                w.enabled = true;
            }
        });
		widget.view();
		widget.template = "widget/form-view.html";
    };

    /**
     * Controla o campo "URL para integração com o módulo OM"
     * 
     * @param {object} widget - Widget de parametrizaçãoServiço
     */
    this.controlaUrlOM = function(widget) {
        var utilizaOM = widget.currentRow.IDINTEGRACAOOM;
        
        if (utilizaOM == "S") {
            widget.getField("URLINTEGRACAOOM").readOnly    = false;
            widget.getField("URLINTEGRACAOOM").validations = {"required": true};
        } else {
            widget.getField("URLINTEGRACAOOM").readOnly    = true;
            widget.getField("URLINTEGRACAOOM").validations = {};
        }
    };
    
    /**
     * Controla o campo "URL para integração com o módulo OM"
     * 
     * @param {object} widget - Widget de parametrizaçãoServiço
     */
    this.controlaUrlCRM = function(widget) {
        var utilizaCRM = widget.currentRow.IDINTEGRACAOCRM;
        
        if (utilizaCRM == "S") {
            widget.getField("URLINTEGRACAOCRM").readOnly = false;
            widget.getField("URLINTEGRACAOCRM").validations = {"required": true};
        } else {
            widget.getField("URLINTEGRACAOCRM").readOnly = true;
            widget.getField("URLINTEGRACAOCRM").validations = {};
        }
    };
    
    /**
    * Atualiza o sequencial do novo código da proposta
    **/
    this.validaNovoCodigoProposta = function(widget){
        var defered = ZHPromise.defer();
        
        var velhoCodigo = widget.backupRow.NRINISEQCODORCA;
        var novoCodigo  = widget.currentRow.NRINISEQCODORCA;
        var cdContador  = 'FCT_ORCAMENTOCD';
        
        if (novoCodigo != velhoCodigo){
            var params = {
                "CDCONTADOR" : cdContador
            };
            
            General.callRestEngine('product#/fct_parametrogeral_widget/buscaNovoCodigo', "Row", params).then(function(response){  
                if (response.dataset.SEQUENCIAL.length > 0){
                    var sequencial = response.dataset.SEQUENCIAL[0].NRSEQUENCIAL;
                    if (sequencial < novoCodigo){
                        this.atualizaNovoCodigo(cdContador, novoCodigo);
                        defered.resolve();
                    } else {
                        defered.reject("Operação bloqueada.<br/><br/>Sequencial atual da Proposta é maior do que o sequencial informado.");
                    }
                } else {
                    this.insereNovoCodigo(cdContador, novoCodigo);
                    defered.resolve();
                }
            }.bind(this));
        } else {
            defered.resolve();
        }
        
        return defered;
    };
    
    /**
    * Atualiza o sequencial do novo código do contrato
    **/
    
    this.validaNovoCodigoContrato = function(widget){
        var defered = ZHPromise.defer();
        
        var velhoCodigo = widget.backupRow.NRINISEQCODCONT;
        var novoCodigo  = widget.currentRow.NRINISEQCODCONT;
        var cdContador  = 'FCT_CONTRATOCD';
        
        if (novoCodigo != velhoCodigo){
            var params = {
                "CDCONTADOR" : cdContador
            };
            
            General.callRestEngine('product#/fct_parametrogeral_widget/buscaNovoCodigo', "Row", params).then(function(response){  
                if (response.dataset.SEQUENCIAL.length > 0){
                    var sequencial = response.dataset.SEQUENCIAL[0].NRSEQUENCIAL;
                    if (sequencial < novoCodigo){
                        this.atualizaNovoCodigo(cdContador, novoCodigo);
                        defered.resolve();
                    } else {
                        defered.reject("Operação bloqueada.<br/><br/>Sequencial atual do contrato é maior do que o sequencial informado.");
                    }
                } else {
                    this.insereNovoCodigo(cdContador, novoCodigo);
                    defered.resolve();
                }
            }.bind(this));
        } else {
            defered.resolve();
        }
        
        return defered;
    };
    
    /**
     * A função "atualizaNovoCodigo" atualiza o novo codigo do contrato ou da proposta
     */
     
    this.atualizaNovoCodigo = function(cdContador, novoCodigo){
        var params = {
            "CDCONTADOR" : cdContador,
            "NRSEQUENCIAL" : novoCodigo
        };    
        
        General.callRestEngine('product#/fct_parametrogeral_widget/atualizaNovoCodigo', "Row", params);
    };
    
    /**
     * A função "insereNovoCodigo" insere o novo codigo do contrato ou da proposta
     */
    this.insereNovoCodigo = function(cdContador, novoCodigo){
        var params = {
            "CDCONTADOR" : cdContador,
            "NRSEQUENCIAL" : novoCodigo
        }; 
        
        General.callRestEngine('product#/fct_parametrogeral_widget/insereNovoCodigo', "Row", params);
    };
    
    this.validaNovoCodigo = function(widget){
        var defered = ZHPromise.defer();

        this.validaNovoCodigoProposta(widget).then(function() {
            return this.validaNovoCodigoContrato(widget);
        }.bind(this))
        .then(function(){
            defered.resolve();
        }).catch(function(error){
            defered.reject(error);
        });
       
        return defered;     
    };
    /**
     * A função que válida se o e-mail no campo rementete do email é válido se ele for preenchido
     * 
     * @param {object} widget - Widget de parametrizaçãoEmail
     */
    this.validaEmail = function(widget){
        var email  = widget.getField('DSREMETENTEEMAIL').value();
        var valido =  Utilities.verificaEmail(email);
        if(email){
            if(!valido){
                widget.currentRow.DSREMETENTEEMAIL = null;
                ScreenService.showMessage("E-mail inválido<br><br>");
            }
        }
    };

    this.validaParamAgendamento = function(field, widget){
        if(field.name === "QTDIASDESALOCREC"){
            if(widget.currentRow.QTDIASDESALOCREC !== null){
                widget.getField("DSEMAILSDESALCREC").validations = { "required": true };
            } else {
                widget.getField("DSEMAILSDESALCREC").validations = {};
            }
        } else {
            if(widget.currentRow.DSEMAILSDESALCREC !== ""){
                widget.getField("QTDIASDESALOCREC").validations = { "required": true };
            } else {
                widget.getField("QTDIASDESALOCREC").validations = {};
            }
        }
    };
    
    
    this.completaPercentual = function(widget){
        General.widgetNewRow(widget);
        var params = {};
        General.callRestEngine('product#/fct_parametrogeral_widget/completaPercentual', "Row", params).then(function(response){  
            var percentual = response.dataset.completaPercentual.PERCENTUAL;
            widget.currentRow.NRPERCENTINDICE = percentual;
        });
    };
    
    /**
     * Valida o percentual dos índices financeiros
     * 
     * @params{Object} widget - widget de índices financeiros
    **/
    this.validarIndiceFinanceiro = function(widget) {
        if(!widget.currentRow.__is_new) {
            this.salvarIndiceFinanceiro(widget);
        } else {
            Utilities.validarCamposDuplicados(widget, null, ["CDINDICE"]).then(function(response) {
                if(!response)
                    ScreenService.showMessage("Operação bloqueada!<br><br>O índice selecionado já está inserido.");
                else
                    this.salvarIndiceFinanceiro(widget);
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    /**
     * Salva um índice padrão
     * 
     * @params{Object} widget - widget de índices financeiros
    **/
    this.salvarIndiceFinanceiro = function(widget) {
        var params = { 
            "NRPERCENTINDICE" : widget.currentRow.NRPERCENTINDICE,
            "CDINDICE"        : (widget.currentRow.__is_new) ? null : widget.currentRow.CDINDICE
        };
        
        General.callRestEngine('product#/fct_parametrogeral_widget/validaPercentual', "Row", params).then(function(response) {  
            var percentual = response.dataset.validaPercentual.PERCENTUAL;
            if(percentual > 100) {
                ScreenService.showMessage("Operação Bloqueada!<br><br>A soma dos percentuais de utilização dos índices financeiros ultrapassou o total de 100%.");
            } else {
                General.widgetSave(widget).then(function(response) {
                    widget.reload();
                    
                    if(percentual < 100)
                        ScreenService.showMessage("Atenção!<br><br>A soma dos percentuais de utilização dos índices financeiros não está totalizando os 100% necessários para realização do reajuste dos contratos.");
                });
            }
        }.bind(this));
    };
    
    /**
     * Valida os e-mails, habilita ou desabilita o campo de periodicidade
     * @params{Object} widget - widget de parametrização SM
    **/
    this.validarParametrosRenovacaoContrato = function(widget) {
        var fieldPeriod = widget.getField("IDPERIODALERTRENOVCONT");
        if(!widget.currentRow.DSEMAILALERTRENOVCONT) {
            fieldPeriod.value("N");
            fieldPeriod.readOnly = true;
        } else {
            var emailsinvalidos = this.validarEmailsParametrizacoes(widget, "DSEMAILALERTRENOVCONT", true);
            if(emailsinvalidos.length == 0) {
                fieldPeriod.readOnly = false;
            } else {
                var stremailsinvalidos = "";
                emailsinvalidos.forEach(function(email, index) {
                    stremailsinvalidos += (email + (index != emailsinvalidos.length - 1 ? ", " : " "));
                });
                
                ScreenService.showMessage("O e-mails " + stremailsinvalidos + "estão inválidos.");
            }
        }
    };
    
    /**
     * Separa os e-mails do campo e manda validar se estão corretos
     * 
     * @params{Object} widget - widget que contém o campo de e-mail
     * @params{String} nmCampoEmails - nome do campo de e-mails
     * @params{bool} retornaEmailInvalido - retorna uma lista de e-mails inválidos
    **/
    this.validarEmailsParametrizacoes = function(widget, nmCampoEmails, retornaEmailInvalido) {
        if (retornaEmailInvalido === undefined) {
            retornaEmailInvalido = false;
        }

        var emails          = widget.currentRow[nmCampoEmails].split(" ");
        var emailsInvalidos = [];
        
        if(emails.length === 0) {
            emails = widget.currentRow[nmCampoEmails].split(",");
        } else {
            emails.forEach(function(email, index) {
                if(email.indexOf(",") !== -1)
                    emails[index] = email.split(",")[0];
            });
        }
        
        if(emails.length > 0) {
            emailsInvalidos = emails.filter(function(email) {
                return !Utilities.verificaEmail(email);
            });
        }
        
        return (retornaEmailInvalido) ? emailsInvalidos : emailsInvalidos.length == 0;
    };
    
    this.saveSignatPadr = function(widget) {
        
        if (widget.currentRow.__is_new) {
            General.callRestEngine('product#/fct_parametrogeral_widget/validaSignatPadr', "Row", widget.currentRow).then(function(response) {  
                
                var retorno = response.dataset.validaSignatPadr.valida;
                if(retorno == '0') {
                    
                    /* Recebe os envolvidos para notificação*/
                    var envolvidos = null;
                    var arrayEnvolvidos = widget.currentRow.DSEMAILNOTSIGNENV_INS;
                    
                    if (arrayEnvolvidos != null && arrayEnvolvidos != ""){
                        envolvidos = arrayEnvolvidos.join('_');
                    }

                    widget.currentRow.DSEMAILNOTSIGNENV = envolvidos; 
                    
                    /* Recebe os operdores para notificação */
                    var operadores = null;
                    var arrayOperadores = widget.currentRow.OPERADORES;
                    
                    if (arrayOperadores != null && arrayOperadores != ""){
                        operadores = arrayOperadores.join('_');
                    }
                    
                    General.widgetSave(widget).then(function(response){
                       
                        if(response.dataset && response.dataset.columnsToReturn[0]){
                           widget.currentRow.NRORG              = response.dataset.columnsToReturn[0].NRORG;
                           widget.currentRow.NRSIGNATARIOSPADR = response.dataset.columnsToReturn[0].NRSIGNATARIOSPADR;
                           
                           if (operadores != null) {
                               var params = { 
                                    "NRSIGNATARIOSPADR" : widget.currentRow.NRSIGNATARIOSPADR,
                                    "OPERADORES"        : operadores
                               };
            
                               General.callRestEngine('product#/fct_parametrogeral_widget/insereOperadoresNotifica', "Row", params).then(function() {
                               
                                    widget.reload();
                                    
                               })
                               .catch(function(response) {
                                   console.error(response);
                                   if (response instanceof Object)
                                       ScreenService.showMessage(response.data.error);
                                   else
                                       ScreenService.showMessage(response);
                               });
                           }
                           else{
                               
                              widget.reload(); 
                           }
                       }
                   })
                   .catch(function(response) {
                       console.error(response);
                       if (response instanceof Object)
                           ScreenService.showMessage(response.data.error);
                       else
                           ScreenService.showMessage(response);
                   });
                }
                else{
                    ScreenService.showMessage("Operação Bloqueada! <br><br> O signatário informado já foi inserido.");
                }
                
            });
        }
        else {
        
            /* Recebe os envolvidos para notificação*/
            var envolvidos = null;
            var arrayEnvolvidos = widget.currentRow.DSEMAILNOTSIGNENV_INS;
            
            if (arrayEnvolvidos != null){
                envolvidos = arrayEnvolvidos.join('_');
            }
            
            widget.currentRow.DSEMAILNOTSIGNENV = envolvidos;
            
            /* Recebe os operdores para notificação */
            var operadoresIguais = false;
            
            var bkpOperadores = widget.backupRow.OPERADORES;
            var arrayOperadores = widget.currentRow.OPERADORES;
            
            if (bkpOperadores == arrayOperadores) {
                
                operadoresIguais = true;
            }
                    
            General.widgetSave(widget).then(function(response){
                
               if(response.dataset && response.dataset.columnsToReturn[0]){
                   widget.currentRow.NRORG              = response.dataset.columnsToReturn[0].NRORG;
                   widget.currentRow.NRSIGNATARIOSPADR = response.dataset.columnsToReturn[0].NRSIGNATARIOSPADR;
                   
                   if (operadoresIguais == false){
                        
                        if (arrayOperadores != null) {
                            var operadores = arrayOperadores.join('_');
                            
                            var params = { 
                                    "NRSIGNATARIOSPADR" : widget.currentRow.NRSIGNATARIOSPADR,
                                    "OPERADORES"        : operadores
                               };
            
                               General.callRestEngine('product#/fct_parametrogeral_widget/insereOperadoresNotifica', "Row", params).then(function() {
                                    
                                 widget.reload();
                                
                               })
                               .catch(function(response) {
                                   console.error(response);
                                   if (response instanceof Object)
                                       ScreenService.showMessage(response.data.error);
                                   else
                                       ScreenService.showMessage(response);
                               });
                        }
                        else{
                            
                            widget.reload();  
                        }
                   }
                   else {
                       
                      widget.reload();
                   }
               }
           })
           .catch(function(response) {
               console.error(response);
               if (response instanceof Object)
                   ScreenService.showMessage(response.data.error);
               else
                   ScreenService.showMessage(response);
           });
        }
    };
    
    /**
     * Abre a widget de Cadastramento Automático.
     */
    this.abrirCadAuto = function() {
        
        var wdgSignatPadrCadAuto = templateManager.container.getWidget("signatariosPadroesCadAuto");

        wdgSignatPadrCadAuto.isVisible = true;
        wdgSignatPadrCadAuto.dataSource.data = [];
        wdgSignatPadrCadAuto.newRow();
        
        wdgSignatPadrCadAuto.currentRow.NRPARCNEGOCIO = null;
        wdgSignatPadrCadAuto.currentRow.IDTIPOASSINATURA = 'E';
        wdgSignatPadrCadAuto.currentRow.IDTESTEMUNHA     = 'N';
        wdgSignatPadrCadAuto.currentRow.NRORDENACAO = null;
        wdgSignatPadrCadAuto.currentRow.IDINSERCAOAUTO   = 'P';
        
        ScreenService.openSwipe(wdgSignatPadrCadAuto);
    };
    
    /**
     * Fecha a widget de Cadastramento Automático.
     */
    this.cancelaCadAutoSignatPadr = function() {
        
        var wdgSignatPadrCadAuto = templateManager.container.getWidget("signatariosPadroesCadAuto");

        ScreenService.closeSwipe().then(function() {
            wdgSignatPadrCadAuto.isVisible = false;
        });
    };
    
    /**
     * Coloca a widget "signatariosPadroesCadAuto" invisível
     */
    this.CadAutoInvisivel = function() {
        
        var wdgSignatPadrCadAuto = templateManager.container.getWidget("signatariosPadroesCadAuto");

        wdgSignatPadrCadAuto.isVisible = false;
    };
    
    /**
     * Salva os signatários padrões em cadastramento automático
     */
    this.saveSignatPadrCadAuto = function(widget) {
        
        if (widget.isValid()) {
        
            var wdgSignatariosPadroes = templateManager.container.getWidget("signatariosPadroes");
            
            General.callRestEngine('product#/fct_parametrogeral_widget/cadAutoSignatPadr', "Row", widget.currentRow).then(function(response) {  
                var retorno = response.dataset.cadauto.retorno;
                if(retorno == 'OK') {
                    ScreenService.closeSwipe().then(function() {
                        widget.isVisible = false;
                        wdgSignatariosPadroes.reload();
                        ScreenService.showMessage("Cadastramento automático realizado com sucesso!");
                    });
                } else {
                    ScreenService.showMessage("Ocorreram erros durante o processo de cadastramento automático. Favor entrar em contato o administrador do sistema. <br><br> Erros: <br><br>"+retorno);
                }
            });
        }
    };    
    
    /**
     * Fecha a widget de Cadastramento Automático.
     */
    this.controlaCamposCertisign = function(widget) {
        
        var tokenCertisign = widget.currentRow.DSTOKENCERTSIGN;
        
        if ((tokenCertisign != null) && (tokenCertisign != "")){
            
            widget.getField("NMSENDERCERTSIGN").validations      = {"required": true};
            widget.getField("DSEMAILSENDERCERTSIGN").validations = {"required": true, "mail": true};
            widget.getField("NRCPFSENDERCERTSIGN").validations   = {"required": true};
        }
        else {
            
            widget.getField("NMSENDERCERTSIGN").validations      = {};
            widget.getField("DSEMAILSENDERCERTSIGN").validations = {"mail": true};
            widget.getField("NRCPFSENDERCERTSIGN").validations   = {};
        }
    };
    
    this.validaEmailCertisign = function(widget){
        
        if(widget.currentRow.DSEMAILSENDERCERTSIGN)
            return this.validarEmailsParametrizacoes(widget, "DSEMAILSENDERCERTSIGN");
        else
            return true;
    };
    
    this.adicionarSignatarioPadrao = function(widget) {
        General.widgetNewRow(widget);
        this.controlaSignatario(widget);
    };
    
    this.controlaSignatario = function(widget){
        if (widget.currentRow.IDTPSIGNATARIO == 'SE') {
            widget.getField("NMPRINCIPALPARC").readOnly = false;
            widget.getField("NMPRINCIPALPARC").validations = {"required": true};
            widget.getField("IDTIPOASSINATURA").readOnly = false;
            widget.getField("IDTESTEMUNHA").readOnly = false;
        }
        else {
            widget.getField("NMPRINCIPALPARC").readOnly = true;
            widget.getField("NMPRINCIPALPARC").validations = {};
            widget.getField("IDTIPOASSINATURA").readOnly = true;
            widget.getField("IDTESTEMUNHA").readOnly = true;
            widget.currentRow.IDTIPOASSINATURA = "E";
            
            if (["SC", "SP"].includes(widget.currentRow.IDTPSIGNATARIO)) {
                widget.currentRow.IDTESTEMUNHA = "N";
            }
            else if (["TC", "TP"].includes(widget.currentRow.IDTPSIGNATARIO)) {
                widget.currentRow.IDTESTEMUNHA = "S";
            }
        }
    };
    
    this.carregaDadosNotifEnvolvidos = function(widget){
        
       var envolvidos = widget.currentRow.DSEMAILNOTSIGNENV;
       
       if (envolvidos != null) {
           widget.currentRow.DSEMAILNOTSIGNENV_INS = envolvidos.split('_');
       }
    };
    
    
    this.carregaDadosOperadores = function(widget){
       var operadores = widget.currentRow.OPERADORES;
       if (operadores && operadores.length > 0) {
            widget.getField("OPERADORES").dataSourceFilter[2] = {
                "name": "CDOPERADOR",
                "operator": "IN",
                "value": operadores
            };
                                                                
            widget.getField("OPERADORES").reload();
            widget.getField("OPERADORES").dataSourceFilter.splice(2);
            templateManager.updateTemplate();
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Sm00005000_Parametrizacoes_Services_Controller', Sm00005000_Parametrizacoes_Services_Controller);
});

// FILE: ./assets/js/controllers/Sm00006000_Composicao_Servicos_Controller.js
function Sm00006000_Composicao_Servicos_Controller(General, ScreenService, templateManager, Utilities) {
    
    /**
     * Executa um novo registro na widget de Composição.
     * 
     * @param {object} widget
     */
    this.novaComposicao = function(widget) {
        if (widget.mode == "view") {
            var nrcompservico = widget.currentRow.NRCOMPSERVICO;
            var nmcompservico = widget.currentRow.NMCOMPSERVICO;
            
            General.widgetNewRow(widget);
            
            widget.currentRow.NRCOMPSERVICOPAI = nrcompservico;
            widget.currentRow.NMCOMPSERVICOPAI = nmcompservico;
        }
        else {
            General.widgetNewRow(widget);
        }
    };
    
    /**
     * Salva o novo registro.
     * 
     * @param {object} widget - Widget de Composição de Serviços
     */
    this.saveComposicaoServico = function(widget) {
        var isNew           = widget.currentRow.__is_new;
        var replicarIdAtivo = (widget.backupRow && widget.backupRow.IDATIVO != widget.currentRow.IDATIVO);
        General.widgetSave(widget).then(function() {
            if (!isNew) {
                if(!replicarIdAtivo){
                    ScreenService.closeSwipe().then(function() {
                        widget.reload();
                    });
                } else {
                    var params = {
                        "NRCOMPSERVICO": widget.currentRow.NRCOMPSERVICO,
                        "IDATIVO": widget.currentRow.IDATIVO,
                        "CDCENTCUST": widget.currentRow.CDCENTCUST
                    };
                    
                    General.callRestEngine("product#/fct_compservico_widget/replicarAlteracaoIdAtivo", "Row", params).then(function() {
                        ScreenService.closeSwipe().then(function() {
                            widget.reload();
                        });
                    }).catch(function(error){
                        Utilities.showErrorMessage(error);
                    });
                }
            } else {
                widget.reload();
            }
        })
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error);
        });
    };
    
    /**
     * Exclui a Composição e seus registros filhos.
     * 
     * @param {object} widget
     */
    this.excluirComposicao = function(widget) {
        General.deleteCrud(widget).then(function() {
            widget.reload();
        },
        function(error) {
            console.error(error);
            ScreenService.showMessage(error);
        });
    };
    
    /**
     * Seta o filtro do campo Elemento de Serviço Superior.
     * 
     * @param {object} widget
     */
    this.setDataSourceFilter = function(widget) {
        var fieldElementoPai = widget.getField("NMSERVORCAPAI");
        var nritcompservico = widget.currentRow.NRITCOMPSERVICO;
        var wdgGridComposicaoServico = templateManager.container.getWidget("gridComposicaoServico");
        if (!nritcompservico) {
            nritcompservico = -1;
        }
        
        fieldElementoPai.dataSourceFilter[0].value = wdgGridComposicaoServico.currentRow.NRCOMPSERVICO;
        fieldElementoPai.dataSourceFilter[1].value = wdgGridComposicaoServico.currentRow.NRORG;
        fieldElementoPai.dataSourceFilter[2].value = nritcompservico;
        fieldElementoPai.reload();
    };
    
    /**
     * 
     * 
     * @param {object} widget
     */
    this.preparaSalvarServico = function(widget) {
        var idelementopai = widget.currentRow.IDELEMENTOPAI_VALIDACAO;
        
        if (idelementopai) {
            var params = {
                "NRCOMPSERVICO": widget.currentRow.NRCOMPSERVICO,
                "IDELEMENTO"   : idelementopai
            };
            
            General.callRestEngine("product#/fct_itcompservico_widget/verificarSeElementoExiste", "Row", params).then(function(response) {
                if (response.dataset.validacao.existe == true) {
                    if (idelementopai == widget.currentRow.IDELEMENTOPAI) {
                        this.salvarServico(widget);
                    }
                    else {
                        ScreenService.showMessage("Operação bloqueada. Um Elemento não pode ser inserido sem o seu Elemento Superior.");
                    }
                }
                else {
                    ScreenService.showMessage("Operação bloqueada. Um Elemento não pode ser inserido sem o seu Elemento Superior.");
                }
            }.bind(this),
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        }
        else {
            this.salvarServico(widget);
        }
    };
    
    /**
     * Salva o serviço da composição.
     * 
     * @param {object} widget
     */
    this.salvarServico = function(widget) {
        var replicarObrigatoriedade = false;
        var isNew = widget.currentRow.__is_new;
        
        if (widget.backupRow && (widget.currentRow.IDSERVOBRIGCOMP != widget.backupRow.IDSERVOBRIGCOMP)) {
            replicarObrigatoriedade = true;
        }
        
        General.widgetSave(widget).then(function() {
            if (replicarObrigatoriedade) {
                var params = {
                    "IDELEMENTO"     : widget.currentRow.IDELEMENTO,
                    "NRCOMPSERVICO"  : widget.currentRow.NRCOMPSERVICO,
                    "IDSERVOBRIGCOMP": widget.currentRow.IDSERVOBRIGCOMP,
                    "NRORG"          : widget.currentRow.NRORG
                };
                
                General.callRestEngine("product#/fct_itcompservico_widget/replicarObrigatoriedadeItensFilhos", "Row", params).then(function() {
                    ScreenService.closeSwipe().then(function() {
                        widget.reload();
                    });
                },
                function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
            }
            else {
                if (isNew) {
                    widget.reload();
                }
                else {
                    ScreenService.closeSwipe().then(function() {
                        widget.reload();
                    });
                }
            }
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error);
        });
    };
    
    /**
     * Exclui o Serviço da Composição e seus filhos.
     * 
     * @param {object} widget
     */
    this.excluirServico = function(widget) {
        General.deleteCrud(widget).then(function() {
            widget.reload();
        },
        function(error) {
            console.error(error);
            ScreenService.showMessage(error);
        });
    };
    
    /**
     * Valida se o Elemento de Serviço Superior escolhido não é o mesmo Elemento que se está editando.
     * 
     * @param {object} widget
     */
    this.validarElementoServicoSuperior = function(widget) {
        var idelemento = widget.currentRow.IDELEMENTO;
        var idelementopai = widget.currentRow.IDELEMENTOPAI;
        
        if ((idelementopai) && (idelemento == idelementopai)) {
            ScreenService.showMessage("Operação bloqueada. Um Elemento de Serviço não pode ser superior a ele mesmo.");
            widget.currentRow.IDELEMENTOPAI = null;
            widget.currentRow.NMSERVORCAPAI = null;
        }
    };
    
    /**
     * Executa um novo registro na widget de Serviços.
     * 
     * @param {object} widget
     */
    this.novoServico = function(widget) {
        if (widget.mode == "view") {
            var idelemento = widget.currentRow.IDELEMENTO;
            var nmservorca = widget.currentRow.NMSERVORCA;
            
            General.widgetNewRow(widget);
            
            widget.currentRow.IDELEMENTOPAI = idelemento;
            widget.currentRow.NMSERVORCAPAI = nmservorca;
        }
        else {
            General.widgetNewRow(widget);
        }
    };
    
    /**
     * Sincroniza os elementos da composição de acordo com a tela de elementos.
     * 
     * @param {object} widget - Widget de Itens da Composição
     */
    this.sincronizarComposicao = function(widget) {
        var wdgGridComposicaoServico = templateManager.container.getWidget("gridComposicaoServico");
        var params = {
            "NRCOMPSERVICO": wdgGridComposicaoServico.currentRow.NRCOMPSERVICO,
            "NRORG": wdgGridComposicaoServico.currentRow.NRORG
        };
        
        General.callRestEngine("product#/fct_compservico_widget/sincronizarComposicao", "Row", params).then(function(success) {
            ScreenService.showMessage("Processo realizado com sucesso!");
            widget.reload();
        },
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    /**
     * Abre a widget de Cadastramento Automático.
     * 
     * @param {object} widget
     */
    this.abrirWdgCadastramentoAutomatico = function(widget) {
        ScreenService.closePopup();
        var wdgCadAuto = templateManager.container.getWidget("wdgCadastramentoAutomatico");
        var wdgGridComposicaoServico = templateManager.container.getWidget("gridComposicaoServico");
        wdgCadAuto.isVisible = true;
        ScreenService.openSwipe(wdgCadAuto).then(function() {
            wdgCadAuto.edit();
            wdgCadAuto.currentRow.CDSERVORCA = [];
            wdgCadAuto.currentRow.IDELEMENTO = [];
            wdgCadAuto.currentRow.IDELEMENTOPAI = [];
            wdgCadAuto.currentRow.NMSERVORCAPAI = "";
            wdgCadAuto.currentRow.IDSOMAVALORSUP = "S";
            wdgCadAuto.currentRow.IDSERVOBRIGCOMP = "S";
            wdgCadAuto.currentRow.IDAPARECEPROPOSTA = "S";
            wdgCadAuto.currentRow.NRCOMPSERVICO = wdgGridComposicaoServico.currentRow.NRCOMPSERVICO;
            wdgCadAuto.currentRow.IDTIPOELEMENTO = "G";
            wdgCadAuto.currentRow.IDATIVO = "S";
            wdgCadAuto.currentRow.NRORG = "";
            wdgCadAuto.currentRow.DTINCLUSAO = "";
            wdgCadAuto.currentRow.DTULTATU = "";
            wdgCadAuto.currentRow.NRORGINCLUSAO = "";
            wdgCadAuto.currentRow.CDOPERINCLUSAO = "";
            wdgCadAuto.currentRow.NRORGULTATU = "";
            wdgCadAuto.currentRow.CDOPERULTATU = "";
        });
    };
    
    /**
     * Fecha a widget de Cadastramento Automático.
     * 
     * @param {object} widget
     */
    this.fecharWdgCadastramentoAutomatico = function(widget) {
        widget.isVisible = false;
        ScreenService.goBack();
        templateManager.container.getWidget("arvore_servicos_composicao").activate();
    };
    
    /**
     * Salva o Cadastramento Automático, inserindo múltiplos elementos de uma vez.
     * 
     * @param {object} widget
     */
    this.salvarCadastramentoAutomatico = function(widget) {
        var camposOutData = {
            "IDELEMENTO"    : "IDELEMENTO",
            "IDELEMENTOPAI" : "IDELEMENTOPAI",
            "CDSERVORCA"    : "CDSERVORCA",
            "IDTIPOELEMENTO": "IDTIPOELEMENTO"
        };
        
        widget.currentRow._cad_auto = true;
        
        Utilities.inserirMultiplos(widget, "IDELEMENTO", camposOutData).then(function() {
            this.fecharWdgCadastramentoAutomatico(widget);
            templateManager.container.getWidget("arvore_servicos_composicao").reload();
        }.bind(this),
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    /**
     * Valida se a Composição Pai escolhido não é a mesma Composição que se está editando.
     * 
     * @param {object} widget
     */
    this.validarComposicaoPai = function(widget) {
        var nrcompservico = widget.currentRow.NRCOMPSERVICO;
        var nrcompservicopai = widget.currentRow.NRCOMPSERVICOPAI;
        
        if ((nrcompservicopai) && (nrcompservico == nrcompservicopai)) {
            ScreenService.showMessage("Operação bloqueada. Uma Composição não pode ser superior a ela mesma.");
            widget.currentRow.NRCOMPSERVICOPAI = null;
            widget.currentRow.NMCOMPSERVICOPAI = null;
        }
    };
    /**
     * valida a composição que está sendo editada
     * @params{Object} widget - widget de serviços
    **/
    this.validarNovoServico = function(widget){
        var wdgComposicao = templateManager.container.getWidget("gridComposicaoServico");
        if(wdgComposicao.currentRow.IDATIVO == 'S'){
            this.novoServico(widget); 
            this.setDataSourceFilter(widget);
            //this.controlarCampoElemento(widget);
        } else {
            ScreenService.confirmMessage("Atenção!<br>Esta composição de serviço está inativa. Deseja continuar?", "question", function(){
                this.novoServico(widget); 
                this.setDataSourceFilter(widget);
                //this.controlarCampoElemento(widget);
            }.bind(this));
        }
    };
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00006000_Composicao_Servicos_Controller', Sm00006000_Composicao_Servicos_Controller);
});

// FILE: ./assets/js/controllers/Sm00006000_Elemento_Controller.js
function Sm00006000_Elemento_Controller(General,restEngine, ScreenService, templateManager, Utilities, Sm00006000_Elemento_Service) {
    
    /**
     * Executa um novo registro na widget de Elemento.
     * 
     * @param {object} widget
     */
    this.novoElemento = function(widget) {
        if (widget.mode == "view") {
            var idelemento = widget.currentRow.IDELEMENTO;
            var nmservorca = widget.currentRow.NMSERVORCA;
            var cdMedidaVariavel = widget.currentRow.CDTPMEDIDAVARIAVEL;
            var dsMedidaVariavel = widget.currentRow.DSTPMEDIDASERV;
            var idFormacaoPreco = widget.currentRow.IDFORMACAOPRECO;
            
            General.widgetNewRow(widget);
            
            widget.currentRow.IDELEMENTOPAI = idelemento;
            widget.currentRow.NMSERVORCAPAI = nmservorca;
            widget.currentRow.CDTPMEDIDAVARIAVEL = cdMedidaVariavel;
            widget.currentRow.DSTPMEDIDASERV = dsMedidaVariavel;
            widget.currentRow.IDFORMACAOPRECO = idFormacaoPreco;
        }
        else {
            General.widgetNewRow(widget);
        }
        
        this.habilitarCampo(widget);
        this.controlaVisuDadosAdc(widget);
    };
    
    /**
     * Salva o registro.
     * 
     * @param {object} widget - Widget de Elementos para salvar o novo registro.
     */
    this.saveElemento = function(widget) {
        if (widget.isValid()) {
            if (widget.__is_new == true) { 
                var idFormacaoPreco = widget.currentRow.IDFORMACAOPRECO;
                
                if (idFormacaoPreco == 'N'){
                    this.salvarElemento(widget);
                }
                else{
                    
                    var IdItemCalculo = widget.currentRow.IDITEMCALCULO;
                    
                    var params = {
                        "IDITEMCALCULO":  IdItemCalculo
                    };
                    
                    General.callRestEngine('product#/fct_servicoorca_tela_elemento/validaIdentificador', "Row", params).then(function(response){
                    
                        var identificador = response.dataset.IDENTIFICADOR.VALIDA;
                          
                        if (identificador != 0){
                            widget.currentRow.IDITEMCALCULO = null;
                            ScreenService.showMessage('O identificador "'+IdItemCalculo+'" já está sendo utilizado para outro elemento.'); 
                        }
                        else{
                            this.salvarElemento(widget);
                        }
                    }.bind(this), 
                    function(error) {
                        console.log(error);
                        ScreenService.showMessage(error.data.error);
                    });
                }
            }
            else {
                this.salvarElemento(widget);
            }
        }
    
    };
    
    
    this.salvarElemento = function(widget){
        var IdBloqueado = widget.currentRow.IDBLOQUEADO;
        var nomeNovo = null;
        var nomeAntigo = null;
        
        if(IdBloqueado == 'S' && widget.backupRow) {
            nomeNovo = widget.currentRow.NMSERVORCA;
            nomeAntigo = widget.backupRow.NMSERVORCA;
        }
        
        General.widgetSave(widget).then(function() {
            if (widget.mode == "view") {
                ScreenService.closeSwipe().then(function() {
                    widget.reload();
                });
            }
            else {
                widget.reload();
            }
            
            if (nomeNovo != nomeAntigo) {
                var params = {
                   'NMSERVORCA' : nomeNovo,
                   'IDELEMENTO' : widget.currentRow.IDELEMENTO
                };
                General.callRestEngine('product#/fct_servicoorca_tela_elemento/updateNomeOm', "Row", params).then(null).catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
        })
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });  
        
    };
    
    /**
     * Exclui o Elemento e seus filhos.
     * 
     * @param {object} widget - Widget de Elementos
     */
    this.excluirElemento = function(widget) {
        General.deleteCrud(widget).then(function() {
            widget.reload();
        });
    };
    
    /**
     * Valida se o Elemento Pai escolhido não é o mesmo Elemento que se está editando.
     * 
     * @param {object} widget - Widget de Elementos
     */
    this.validarElementoPai = function(widget) {
        var idelemento = widget.currentRow.IDELEMENTO;
        var idelementopai = widget.currentRow.IDELEMENTOPAI;
        
        if ((idelementopai) && (idelemento == idelementopai)) {
            ScreenService.showMessage("Operação bloqueada. Um Elemento não pode ser superior a ele mesmo.");
            widget.currentRow.IDELEMENTOPAI = null;
            widget.currentRow.NMSERVORCAPAI = null;
        }
    };
    
    /**
     * Verifica se o registro está bloqueado para edição ou exclusão.
     * 
     * @param {object} widget - Widget de Elementos
     */
    this.verificarBloqueioEdicao = function(widget) {
        var bloqueado = widget.currentRow.IDBLOQUEADO;
        var actAdicionar = widget.getAction("add");
        var actExcluir = widget.getAction("delete");
        
        if (bloqueado == "S") {
            actExcluir.isVisible = false;
            actAdicionar.isVisible = false;
            this.controlarCamposBloqueio(widget, true);
        }
        else {
            
            actExcluir.isVisible = true;
            actAdicionar.isVisible = true;
            this.controlarCamposBloqueio(widget, false);
        }
    };
    
    /**
     * Controla os campos de acordo com o bloqueio da widget.
     * 
     * @param {object} widget
     * @param {boolean} readOnly
     */
    this.controlarCamposBloqueio = function(widget, readOnly) {
        widget.fields.filter(function(field) {
            //return field.name != "DSTPMEDIDASERV" && field.name != "DSSERVORCA" && field.name != "NMSERVORCA";
            return !["DSTPMEDIDASERV", "DSSERVORCA", "NMSERVORCA"].includes(field.name);
        })
        .forEach(function(field) {
            if(["IDITEMCALCULO", "DSEXPRESSAOCALC", "IDTIPOCALCULO", "VRPERCCALCULO"].indexOf(field.name) === -1){
                field.readOnly = readOnly;
            }
        }.bind(this));
    };
     /**
     * Controla os campos de acordo com o bloqueio da widget.
     * 
     * @param {object} widget
     * @param {boolean} readOnly
     */
    this.habilitarCampo = function(widget) {
        var insert        = widget.currentRow.__is_new;
        var utiliza       = widget.currentRow.IDFORMACAOPRECO;
        var iditemcalculo = widget.currentRow.IDITEMCALCULO;
        if (utiliza == "S" && (insert || Util.isEmptyOrBlank(iditemcalculo))){
            widget.getField("IDITEMCALCULO").readOnly       = false;
            widget.getField("IDITEMCALCULO").validations    = { "required": true};
            widget.getField("IDORDENAELEMENTO").validations = { "required": true};
            widget.getField("IDTIPOCALCULO").readOnly       = false;
            
            widget.getField("DSTPMEDIDASERV").value("");
            widget.getField("DSTPMEDIDASERV").readOnly = true;
        } else {
            widget.getField("IDITEMCALCULO").readOnly       = true;
            widget.getField("IDITEMCALCULO").validations    = {};
            widget.getField("IDORDENAELEMENTO").validations = {};
            widget.getField("IDTIPOCALCULO").readOnly       = !(utiliza == "S");
            
            widget.getField("DSTPMEDIDASERV").readOnly = (utiliza == "S");
        }
        widget.getField("NMCENTCUST").readOnly = false;
    };
    
    this.editarElemento = function(widget) {
        General.widgetEdit(widget);
        this.habilitarCampo(widget);
        this.controlaVisuDadosAdc(widget);
    };
    
    /**
     * Volta para o modo list da tela de Proposta
     * 
     * @param {object} widget
     */
    this.voltarParaListProposta = function(widget) {
        ScreenService.closeSwipe();
        var actAdicionar = widget.getAction("add");
        actAdicionar.isVisible = true;
    };
    
    /**
     * Salva o relacionamento do Elemento com Servico.
     * 
     * @param {object} widget
     */
    this.salvarRelacionamentoServico = function(widget) {
        if (widget.isValid()) {
            var servicos       = widget.currentRow.CDSERVORCARELAC;
            var cdservorca     = widget.currentRow.CDSERVORCA;
            var cdtpmedidaserv = widget.currentRow.CDTPMEDIDASERV;
            var isNew          = widget.currentRow.__is_new;
            var params = {
                "IDMEDIDAFIXA"   : widget.currentRow.IDMEDIDAFIXA,
                "IDELEMENTO"     : this.getElementoPaiArvore(widget.currentRow.CDSERVORCA),
                "CDSERVORCARELAC": widget.currentRow.CDSERVORCARELAC
            };
            
            if (isNew){
                Sm00006000_Elemento_Service.validaServicoRelacionadoUnit(servicos, cdservorca, cdtpmedidaserv).then(function(success) {
                    if (success.dataset.VALIDASERV.VALIDA == false) {
                        
                        widget.currentRow.__columns_to_return = ["NRRELACSERVICO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];
                        General.widgetSave(widget).then(function(success) {
                            if (success.dataset) {
                                widget.currentRow.NRRELACSERVICO = success.dataset.columnsToReturn[0].NRRELACSERVICO;
                                widget.currentRow.NRORG          = success.dataset.columnsToReturn[0].NRORG;
                                widget.currentRow.DTINCLUSAO     = success.dataset.columnsToReturn[0].DTINCLUSAO;
                                widget.currentRow.CDOPERINCLUSAO = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                            }
                            
                            General.callRestEngine("product#/fct_relacservico_widget/replicarAlteracaoMedidaServ", "Row", params)
                            .then().catch(function(error){
                                Utilities.showErrorMessage(error);
                            });
                        }.bind(this)).catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    } else {
                        ScreenService.showMessage("Operação Bloqueada. Serviço selecionado já está relacionado ao tipo de medida informado.");
                    }
                }.bind(this));
            } else{
               General.widgetSave(widget).then(function(){
                   General.callRestEngine("product#/fct_relacservico_widget/replicarAlteracaoMedidaServ", "Row", params)
                    .then().catch(function(error){
                        Utilities.showErrorMessage(error);
                    });
               }); 
            }
        }
    };
    /**
     * Retorna o elemento pai da arvore de elementos
     * @params{String} cdSerorca - código do elemento que ira iniciar a busaca
    **/
    this.getElementoPaiArvore = function(cdSerorca){
        var wdgElemento = templateManager.container.getWidget("gridElemento");
        var rowElemento = wdgElemento.dataSource.data.filter(function(elemento){
            return elemento.CDSERVORCA === cdSerorca;
        })[0];
        
        if(!rowElemento.IDELEMENTOPAI){
            return rowElemento.IDELEMENTO;
        } else {
            return this.getElementoPaiArvore(rowElemento._parent.CDSERVORCA);
        }
    };
    /**
     * Abre a widget de Cadastramento Automático.
     */
    this.abrirWdgCadastramentoAutomatico = function() {
        var wdgCadAuto       = templateManager.container.getWidget("wdgCadastramentoAutomatico");
        wdgCadAuto.isVisible = true;
        ScreenService.openSwipe(wdgCadAuto).then(function() {
            wdgCadAuto.newRow();
            wdgCadAuto.currentRow.CDSERVORCARELAC = [];
            wdgCadAuto.currentRow.CDSERVORCA      = [];
            wdgCadAuto.currentRow.CDTPMEDIDASERV  = "";
            wdgCadAuto.currentRow.DSTPMEDIDASERV  = "";
            wdgCadAuto.currentRow.QTMEDIDASERV    = "";
        });
    };
    
    /**
     * Fecha a widget de Cadastramento Automático.
     * 
     * @param {object} widget
     */
    this.fecharWdgCadastramentoAutomatico = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.isVisible = false;
            templateManager.container.getWidget("wdgRelacionamentoServico").activate();
        });
    };
    
    /**
     * Salva o Cadastramento Automático, inserindo dataset de serviços de uma vez.
     * 
     * @param {object} widget
     */
     
    this.salvarCadastramentoAutomatico = function(widget) {
        if (!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            General.callRestEngine("product#/fct_relacservico_widget/SaveAutomatico","Row", widget.currentRow).then(function(response) {
                if(response.dataset && response.dataset.VALIDACAO && response.dataset.VALIDACAO.length > 0) 
                    ScreenService.showMessage("ATENÇÃO!<br><br>Os elementos listados a seguir já estão relacionados aos serviços selecionados.<br><br>" + response.dataset.VALIDACAO[0], 'INFORMATION');
                    
                this.fecharWdgCadastramentoAutomatico(widget);
                templateManager.container.getWidget("wdgRelacionamentoServico").reload();
            }.bind(this))
            .catch(function(error) {
                if (error.data.error == "SERVICO_JA_EXISTE") 
                    ScreenService.showMessage("Operação Bloqueada. Alguns dos serviços selecionados já estão relacionados ao tipo de medida informado.");
                else 
                    Utilities.showErrorMessage(error);
            });
        }
    };

    /**
     * Controla a visualização do campo de expressão padrão
     * @param {Object} widget widget elemento
     **/
    this.controlaVisuDadosAdc = function(widget){
        var readOnly = widget.currentRow.IDTIPOCALCULO === "E" ? true : false;
        widget.getField("DSEXPRESSAOCALC").readOnly    = !readOnly;
        widget.getField("DSEXPRESSAOCALC").validations = readOnly ? {"required": true} : {};
        widget.getField("VRPERCCALCULO").readOnly      = !readOnly;
    };
}
 
Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00006000_Elemento_Controller', Sm00006000_Elemento_Controller);
});


// FILE: ./assets/js/controllers/Sm00006000_Servicos_Controller.js
function Sm00006000_Servicos_Controller(General, ScreenService, Sm00006000_Servicos_Service, Utilities, templateManager) {
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Serviços para salvar o novo registro.
    */
    this.saveService = function(widget) {
        widget.currentRow.__columns_to_return = ["CDSERVORCA", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset) {
                widget.currentRow.CDSERVORCA = response.dataset.columnsToReturn[0].CDSERVORCA;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
        }.bind(this));
    };
    
    
    /**
     * A função "validaServicoRelacionado" verifica se o serviço 
     * que está sendo relacionado já existe está relacionado ao serviço
     * e salva o serviço relacionado.
     * 
     * * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do serviço relacioando.
     */
     
    this.validaServicoRelacionado = function(widget){
        
        var currentRow = widget.currentRow;
        
        if  (currentRow.__is_new == true){ 
            var arrayServRelac = this.criaArrayServicosRelacionados(widget);
            
            var servOrcaRelac = arrayServRelac.listaValida;
            var servOrca = widget.currentRow.CDSERVORCA;
            var tipoMedida = widget.currentRow.CDTPMEDIDASERV;
           
            Sm00006000_Servicos_Service.validaServicoRelacionado(servOrcaRelac, servOrca, tipoMedida).then(function(response){
                if (response.dataset.VALIDASERV.VALIDA == false) {
                    
                    General.callRestEngine("product#/fct_relacservico_widget/save", "DataSet", arrayServRelac.rows).then(function(response){
                            
                            ScreenService.closeSwipe().then(function(){
                                this.controlaFieldServico(widget, false);
                                widget.reload();
                        }.bind(this));
                    }.bind(this));
                }
                else{
                    ScreenService.showMessage('Operação Bloqueada. Alguns dos serviços selecionados já estão relacionados para a medida informada.'); 
                }
    
           }.bind(this)).catch(function(response){
               ScreenService.showMessage(response.data.error);
           });
        }
        else{
            
            General.widgetSave(widget).then(function(){
                ScreenService.closeSwipe().then(function(){
                    widget.reload();
                }.bind(this));
            }.bind(this));
        }
    };
    
    /**
     * Função para criar o array dos serviços relacionados
     * que foram pegos na função "recebeServicosRelac";
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do serviço relacioando.
     * 
    */
    
    this.criaArrayServicosRelacionados = function(widget){
        
        var listaServicosRelac = widget.currentRow.CDSERVORCARELAC;
        var rows = [];
        var listaValida = [];
        var row;
        
        listaServicosRelac.forEach(function(servicoRelac , indice){
            row = this.recebeServicosRelac(widget, indice);
            rows.push(row);
            listaValida.push(row.CDSERVORCARELAC);
        }.bind(this)); 
        
        return {
            "rows": rows,
            "listaValida": listaValida
        };
    };
    
    /**
     * Função para pegar as informações do 
     * array de serviços relacionados na currentRow, 
     * e tratá-las em uma cópia da currentRow para
     * realizar o save.
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do serviço relacioando.
     * 
     * @param {int} indice index do serviço no array de 
     * serviços relacionados presente na currentRow.
    */
    this.recebeServicosRelac = function(widget, indice){
        
        var currentRow = angular.copy(widget.currentRow);
        
        currentRow.__is_new = true;

        currentRow.CDSERVORCARELAC = currentRow.CDSERVORCARELAC[indice];
        
        return currentRow;
        
    };
    
    /**
     * Função para controlar a visualização do campo 
     * de serviço da página "serviços relacionados" 
     * nos modos de edição e inserção;
     * 
     * @param {object} widget - widget ativa do serviço relacioando.
     * 
     * @param {boolean} idadicionar - verifica se está em modo de adição
     * 
    */
    
    this.controlaFieldServico = function(widget, idadicionar){
        
        /* Em modo de Edição, deve mostrar apenas o nome do serviço. */
        if (idadicionar == false){
            
            widget.getField("CDSERVORCARELAC").isVisible = false;
            widget.getField("CDSERVORCARELAC").showOnForm = false;
            
            widget.getField("NMSERVORCARELAC").isVisible = true;
            widget.getField("NMSERVORCARELAC").showOnForm = true;
        }
        /* Em modo de Inserção, deve mostrar apenas o Código do serviço. */
        else {
            
            widget.getField("CDSERVORCARELAC").isVisible = true;
            widget.getField("CDSERVORCARELAC").showOnForm = true;
            
            widget.getField("NMSERVORCARELAC").isVisible = false;
            widget.getField("NMSERVORCARELAC").showOnForm = false;
        }
    };
    
    /**
     * Função que recebe o serviço pai e insere na filtro do field "CDSERVORCARELAC".
     * 
     * * @param {object} widget - widget ativa do serviço relacioando.
     * 
     */
    
    this.recebeServicoParent = function(widget){
        var cdServOrcaRelac = widget.getField("CDSERVORCARELAC");
        cdServOrcaRelac.dataSourceFilter[1].value = templateManager.container.getWidget("gridServico").currentRow.CDSERVORCA;
        cdServOrcaRelac.reload();
    };

    this.salvarHabilidades = function(widget){
        
        var parametros = {
            "CDSERVORCA": widget.currentRow.CDSERVORCA,
            "CDHABILITSERV": widget.currentRow.CDHABILITSERV
        };
       General.callRestEngine("product#/fct_relacservhabili_widget/validacao", "Row", parametros).then(function(sucesso){ 
           if(sucesso.dataset.validacao.resultado){
                Utilities.inserirMultiplos(widget, "CDHABILITSERV", null, "product#/fct_relacservhabili_widget/save").then(function(success){
                    ScreenService.closeSwipe();
                    widget.reload();
                })
                .catch(function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
            }
           else{
                ScreenService.showMessage("Operação bloqueada.<br>Uma ou mais habilidades selecionadas já estão cadastradas!");
            }
        }).catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00006000_Servicos_Controller', Sm00006000_Servicos_Controller);
});



// FILE: ./assets/js/controllers/Sm00006001_Tabela_Preco_Servico_Controller.js
function Sm00006001_Tabela_Preco_Servico_Controller(General, ScreenService, Sm00006001_Tabela_Preco_Servico_Service, templateManager) {
    /**
     Get Widget: criação de variaveis para armazenar os valores das widgets
    */
    var getWidgetServicos = function () {
        return templateManager.container.getWidget('servicos');
    };
    var getWidgetGridMewdidaVariavel = function () {
        return templateManager.container.getWidget('gridMewdidaVariavel');
    };
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Tabela de preços para salvar o 
     * novo registro.
    */
    this.saveTabelaPreco = function(widget){
        
        widget.currentRow.__columns_to_return = ["CDPRECBASESERV", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset.columnsToReturn)
                widget.currentRow.CDPRECBASESERV = response.dataset.columnsToReturn[0].CDPRECBASESERV;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
            
        }.bind(this));
    };
    

   /**
     * A função "validaServicoTabela" verifica se o serviço 
     * que está sendo relacionado já está relacionado na tabela
     * e salva os serviços da tabela de preço.
     * 
     * * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do serviço relacioando.
     */
     
    this.validaServicoTabela = function(widget){
        if (widget.isValid()) {
            var currentRow = widget.currentRow;
            
            if  (currentRow.__is_new == true){ 
            
                 var arrayServTabela = this.criaArrayServicosTabela(widget);
        
                var servOrca = arrayServTabela.listaValida;
                var precBaseServ = widget.currentRow.CDPRECBASESERV;
                var tipoMedida = widget.currentRow.CDTPMEDIDASERV;
               
                widget.currentRow.__columns_to_return = ["NRSEQITPRECBSERV", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
                
                Sm00006001_Tabela_Preco_Servico_Service.validaServicoTabela(servOrca, precBaseServ, tipoMedida).then(function(response){
                    if (response.dataset.VALIDASERV.VALIDA == false) {
                        
                        General.callRestEngine("product#/fct_itprecbaseserv_widget/save", "DataSet", arrayServTabela.rows).then(function(response){
                        
                            ScreenService.confirmMessage("Deseja gerar as faixas de variação de preço automaticamente?", "question", function() {
                                
                                
                                response.dataset.ITPRECBASESERV.forEach(function(itprecBase){
                                        this.sincronizarVariacaoPreco(widget, 'I', itprecBase);
                                }.bind(this)); 
                                                        
                                ScreenService.closeSwipe().then(function(){
                                    this.controlaFieldServico(widget, false);
                                    General.changeWidgetAcessibility(widget.widgets, true);
                                    widget.reload();
                                }.bind(this));    
                            
                                
                            }.bind(this),
                            function() {
                                
                                ScreenService.closeSwipe().then(function(){
                                    this.controlaFieldServico(widget, false);
                                    General.changeWidgetAcessibility(widget.widgets, true);
                                    widget.reload();
                                }.bind(this));
                                
                            }.bind(this));
                            
                        }.bind(this));
                    }
                    else{
                          ScreenService.showMessage('Operação Bloqueada. Alguns dos serviços selecionados já estão inseridos para a medida informada.'); 
                    }
        
               }.bind(this)).catch(function(response){
                   ScreenService.showMessage(response.data.error);
               });
            }
            else{
                
                General.widgetSave(widget);
                ScreenService.confirmMessage("Deseja gerar as faixas de variação de preço automaticamente?", "question", function() {
                                
                    this.sincronizarVariacaoPreco(widget, 'E'); 
                    
                }.bind(this));
            }
        }
        else {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
    };
    
    
    /**
     * Função para criar o array dos serviços que foram 
     * pegos na função "recebeServicosTabela";
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do serviço relacioando.
     * 
    */
    
    this.criaArrayServicosTabela = function(widget){
        
        var listaServicosTabela = widget.currentRow.CDSERVORCA;
        var rows = [];
        var listaValida = [];
        var row;
        
        listaServicosTabela.forEach(function(servicoRelac , indice){
            row = this.recebeServicosTabela(widget, indice);
            rows.push(row);
            listaValida.push(row.CDSERVORCA);
        }.bind(this)); 
        
        return {
            "rows": rows,
            "listaValida": listaValida
        };
    };
    
    /**
     * Função para pegar as informações do  array 
     * de serviços na currentRow, e tratá-las em 
     * uma cópia da currentRow para realizar o save.
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do serviço relacioando.
     * 
     * @param {int} indice index do serviço no array de 
     * serviços relacionados presente na currentRow.
    */
    this.recebeServicosTabela = function(widget, indice){
        
        var currentRow = angular.copy(widget.currentRow);
        
        currentRow.__is_new = true;

        currentRow.CDSERVORCA = currentRow.CDSERVORCA[indice];
        
        return currentRow;
        
    };
    
    /**
     * Função para controlar a visualização do campo 
     * de serviço da página "serviços" 
     * nos modos de edição e inserção;
     * 
     * @param {object} widget - widget ativa do serviço relacioando.
     * 
     * @param {boolean} idadicionar - verifica se está em modo de adição
     * 
    */
    
    this.controlaFieldServico = function(widget, idadicionar){
        
        /* Em modo de Edição, deve mostrar apenas o nome do serviço. */
        if (idadicionar == false){
            
            widget.getField("CDSERVORCA").isVisible = false;
            widget.getField("CDSERVORCA").showOnForm = false;
            
            widget.getField("NMSERVORCA").isVisible = true;
            widget.getField("NMSERVORCA").showOnForm = true;
        }
        /* Em modo de Inserção, deve mostrar apenas o Código do serviço. */
        else {
            
            widget.getField("CDSERVORCA").isVisible = true;
            widget.getField("CDSERVORCA").showOnForm = true;
            
            widget.getField("NMSERVORCA").isVisible = false;
            widget.getField("NMSERVORCA").showOnForm = false;
        }
        
    };
    
    this.trocaNomeCampo = function(widget){
        
        var help = widget.getField('DSTPMEDIDASERV').value();
        
        if (help == null) {
            
            help =  'Tipo de Medida';
        }
        
        widget.getField('VRPRECO').help = 'O preço informado neste campo se refere a 1 '+help+'.';
        
    };
    
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Tipo de Medida para salvar o novo registro
    */
    this.salvarVariacaoTipoDeMedida = function(widget){

       var gridMewdidaVariavel = getWidgetGridMewdidaVariavel();
        widget.currentRow.NRSEQITPRECBSERV = gridMewdidaVariavel.currentRow.NRSEQITPRECBSERV;
        widget.currentRow.CDTPMEDIDASERV = gridMewdidaVariavel.currentRow.CDTPMEDIDASERV;
        widget.currentRow.CDTPMEDIDAVARIAVEL = gridMewdidaVariavel.currentRow.CDTPMEDIDAVARIAVEL;
        widget.currentRow.IDMULTIPVARIACAO = gridMewdidaVariavel.currentRow.IDMULTIPVARIACAO;
        widget.currentRow.QTFRANQUIA = gridMewdidaVariavel.currentRow.QTFRANQUIA;

        
        widget.currentRow.__columns_to_return = ["NRVARIACAOPRECBASESERV", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                widget.currentRow.NRVARIACAOPRECBASESERV = response.dataset.columnsToReturn[0].NRVARIACAOPRECBASESERV;
                widget.currentRow.NRORG                  = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO             = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO          = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO         = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
        }.bind(this)).catch(function(error){
            ScreenService.showMessage(error);
        });
    };
    
    
     /**
     * A função "validaTipoMedidaVariavel" verifica se o tipo de medida variável
     * ja está criado para a faixa existente.
     * 
     * * @param {object} widget widget ativa 
     */
     
    this.validaTipoMedidaVariavel = function(widget){
        
        var currentRow = widget.currentRow;

        var widgetMewdidaVariavel = getWidgetGridMewdidaVariavel();

        var qtMedidaIniNew = currentRow.QTMEDIDAINIVARIACAO;
        var qtMedidaFinNew = currentRow.QTMEDIDAFIMVARIACAO;
        var nrSeqItPrecBServ = widgetMewdidaVariavel.currentRow.NRSEQITPRECBSERV;
        var cdTpMedidaServ = widgetMewdidaVariavel.currentRow.CDTPMEDIDASERV;
        var CdtpMedidaVariavel = widgetMewdidaVariavel.currentRow.CDTPMEDIDAVARIAVEL;

        var nrVaricacaoPrecBaseServ = widget.currentRow.NRVARIACAOPRECBASESERV;
        
        var inserir = 'S';
        Sm00006001_Tabela_Preco_Servico_Service.validaTipoMedidaVariavel(nrSeqItPrecBServ, cdTpMedidaServ, CdtpMedidaVariavel, nrVaricacaoPrecBaseServ).then(function(response){
            
            var arrayValidacao = response.dataset.VALIDACAO;
            
            arrayValidacao.forEach(function(arrValida){
                 
                var qtMedidaIniOld = arrValida.QTMEDIDAINIVARIACAO;
                var qtMedidaFinOld  = arrValida.QTMEDIDAFIMVARIACAO;

                /*A qt.medida inicial do registro a ser inserido deve ser 
                  menor que a qt.medida inicial ou maior do que a qt.medida final 
                  dos registros já inseridos na tabela.*/
                if (qtMedidaIniNew >= qtMedidaIniOld && qtMedidaIniNew <= qtMedidaFinOld){
                
                      inserir = 'N';
                }
                /*A qt.medida inicial do registro a ser inserido deve ser 
                  menor que a qt.medida inicial ou maior do que a qt.medida final 
                  dos registros já inseridos na tabela.*/
                else if (qtMedidaFinNew >= qtMedidaIniOld && qtMedidaFinNew <= qtMedidaFinOld){
                    
                    inserir = 'N';
                }
                else if (qtMedidaIniNew <= qtMedidaIniOld && qtMedidaFinNew >= qtMedidaFinOld){
                    
                    inserir = 'N';
                }
                
            });
            
            
            if (inserir == 'S') {
                
                this.salvarVariacaoTipoDeMedida(widget);
            } 
            else {
            
                ScreenService.showMessage('Operação Bloqueada. <br><br>Já existe uma variação cadastrada para o "Tipo de Medida para Variação" e para o intervalo de "Quantidade da Medida" informados.'); 
                
            }

       }.bind(this)).catch(function(response){
           ScreenService.showMessage(response.data.error);
       });
    };
    
    
    /**
     * A função "validaIntervaloQtMed" realiza a validação das quantidades de medida
     * da variação de tipo de medida.
     * 
     * * @param {object} widget widget ativa 
     */
    
    this.validaIntervaloQtMed = function(widget){
        if (widget.isValid()) {
            var qtMedidaIni = widget.currentRow.QTMEDIDAINIVARIACAO;
            var qtMedidaFin = widget.currentRow.QTMEDIDAFIMVARIACAO;
            
            if (qtMedidaIni > qtMedidaFin){
              
                ScreenService.showMessage('Operação Bloqueada. <br><br> "Quantidade da Medida - De" não pode ser maior que "Quantidade da Medida - Até". '); 
            }
            else{
                this.validaTipoMedidaVariavel(widget);
            }
        }
        else {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
    };
    
      /**
     * A função "sincronizarVariacaoPreco" executa a procedure "FCT_SINCRONIZAR_VARIACAO_PRECO".
     */
     
    this.sincronizarVariacaoPreco = function(widget, tipo, itprecBase) {
        if (itprecBase === undefined) {
            itprecBase = null;
        }

        var wdgServicos = templateManager.container.getWidget("servicos");
        var nrSeqItPrecBServ = null;
        if (tipo == 'S'){
            nrSeqItPrecBServ = wdgServicos.currentRow.NRSEQITPRECBSERV;
        }
        else if (tipo == 'I'){
            
            nrSeqItPrecBServ = itprecBase;
        }
        else
        {
            nrSeqItPrecBServ = widget.currentRow.NRSEQITPRECBSERV;
        }
        
        Sm00006001_Tabela_Preco_Servico_Service.sincronizarVariacaoPreco(nrSeqItPrecBServ,  tipo).then(function(response){
              
            ScreenService.showMessage("As faixas de variação de preço foram geradas com sucesso.");
            
            if (tipo == 'S'){
                widget.reload(); 
            }
            
        }.bind(this)).catch(function(response){
            
           ScreenService.showMessage(response.data.error);
           
        });
        
    };
    
    /* Sinccroniza as faixas de variação de preço de acordo com as variações por tipo de medida  */
    
    this.sincronizacaoVariacaoPreco = function(widget){
        
        ScreenService.confirmMessage("Ao realizar a sincronização, as variações de preço atuais serão excluídas.<br/><br/>Deseja realmente sincronizar as variações de preço?", "question", function() {
            
            this.sincronizarVariacaoPreco(widget, 'S');
        }.bind(this));
    };
    
    
    /** Salva a inserção e a Edição das variaçoes de preço das abas "Medida Variável"  
      e as faixas de variação da aba "Variação de preço" */
    
    this.salvaVariacao = function(widget){
        var qtFranquia      = widget.currentRow.QTFRANQUIA;
        var qtFranquiaAnte  = null;
        if (widget.currentRow.__is_new != true){
            qtFranquiaAnte  = widget.backupRow.QTFRANQUIA;
        }
        
        var usaFranquia = 'N';
        
        if (qtFranquia != qtFranquiaAnte){
        
            var widgetServicos = getWidgetServicos();
    
            var nrSeqItPrecBServ    = widgetServicos.currentRow.NRSEQITPRECBSERV;
            var cdTpMedidaServ      = widgetServicos.currentRow.CDTPMEDIDASERV;
            var CdtpMedidaVariavel  = widget.currentRow.CDTPMEDIDAVARIAVEL; 
            
            
            var params = {
                "NRSEQITPRECBSERV" : nrSeqItPrecBServ,
                "CDTPMEDIDASERV" : cdTpMedidaServ,
                "CDTPMEDIDAVARIAVEL" :CdtpMedidaVariavel,
                "QTFRANQUIA": qtFranquia
            };
            
            General.callRestEngine('product#/fct_variacaoprecbaseserv_widget/validaFranquiaFaixa', "Row", params).then(function(response){
                
                if  (response.dataset.VALIDACAO[0].FAIXAS == 0) {
                    
                    usaFranquia = 'S';
                    
                    /*ScreenService.confirmMessage("Ao definir um valor para o campo 'Franquia', todos os registros desta mesma medida variável nos demais serviços desta tabela de preço serão alterados para considerar o valor definido como franquia.<br><br>Deseja continuar?", "question", function() {
                                
                        usaFranquia = 'S';   
                        this.testaInserteEdit(widget, usaFranquia);
                            
                    }.bind(this));*/
                    
                    this.testaInserteEdit(widget, usaFranquia);
                }
                else{
    
                    ScreenService.showMessage("Operação bloqueada!<br><br>Não é permitido definir uma franquia caso existam faixas de variação de preço as quais:<br><br>- O intervalo das quantidades de medida ('De' e 'Até') seja menor que a franquia.<br><br>- A franquia esteja dentro do intervalo das quantidades de medida ('De' e 'Até').");
                    
                }
            }.bind(this),
            function(error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            }.bind(this)); 
        }
        else{
            
            this.testaInserteEdit(widget, usaFranquia);
            
        }
        
    };
    
    
    this.testaInserteEdit = function(widget, usaFranquia){

         if (widget.currentRow.__is_new == true){
                        
            ScreenService.confirmMessage("Deseja gerar as faixas de variação de preço automaticamente?", "question", function() {
                                
                var sincronizar = 'S'; /*Sim*/    
                this.inserevariacao(widget, sincronizar, usaFranquia);

                    
            }.bind(this), 
            function(){
                
                var sincronizar = 'N'; /*Não*/
                this.inserevariacao(widget, sincronizar, usaFranquia);
                
            }.bind(this));
            
            
        } else{
            
            this.editarVariacao(widget, usaFranquia);
        }
        
    };
    
    
    /** Insere as variaçoes de preço das abas "Medida Variável"  
      e as faixas de variação da aba "Variação de preço" */
    
    this.inserevariacao = function(widget, sincronizar, usaFranquia){
        var widgetServicos = getWidgetServicos();

        var nrSeqItPrecBServ    = widgetServicos.currentRow.NRSEQITPRECBSERV;
        var cdTpMedidaServ      = widgetServicos.currentRow.CDTPMEDIDASERV;
        var CdtpMedidaVariavel  = widget.currentRow.CDTPMEDIDAVARIAVEL; 
        var vrPrecoVariacao     = widgetServicos.currentRow.VRPRECO;

        var idMultipVariacao    = widget.currentRow.IDMULTIPVARIACAO; 
        var qtFranquia          = widget.currentRow.QTFRANQUIA;
        
        var params = {
            "NRSEQITPRECBSERV"   : nrSeqItPrecBServ,
            "CDTPMEDIDASERV"     : cdTpMedidaServ,
            "CDTPMEDIDAVARIAVEL" : CdtpMedidaVariavel,
            "IDMULTIPVARIACAO"   : idMultipVariacao,
            "QTFRANQUIA"         : qtFranquia,
            "VRPRECOVARIACAO"    : vrPrecoVariacao,
            "SINCRONIZAR"        : sincronizar,
            "USAFRANQUIA"        : usaFranquia
        };
        
        General.callRestEngine('product#/fct_variacaoprecbaseserv_widget/insereVariacaoPreco', "Row", params).then(function(){
        
            ScreenService.closeSwipe();
            widget.reload();
            General.changeWidgetAcessibility(widget.widgets, true);
          
        },
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });    
        
    };
    
    /** Altera as variaçoes de preço das abas "Medida Variável"  
      e as faixas de variação da aba "Variação de preço" */
    
    this.editarVariacao = function(widget, usaFranquia){

        var widgetServicos = getWidgetServicos();
        
        var nrSeqItPrecBServ    = widgetServicos.currentRow.NRSEQITPRECBSERV;
        var cdTpMedidaServ      = widgetServicos.currentRow.CDTPMEDIDASERV;

        var CdtpMedidaVariavel  = widget.currentRow.CDTPMEDIDAVARIAVEL; 
        var idMultipVariacao    = widget.currentRow.IDMULTIPVARIACAO; 
        var qtFranquia          = widget.currentRow.QTFRANQUIA;
        
         var params = {
            "NRSEQITPRECBSERV"   : nrSeqItPrecBServ,
            "CDTPMEDIDASERV"     : cdTpMedidaServ,
            "CDTPMEDIDAVARIAVEL" : CdtpMedidaVariavel,
            "IDMULTIPVARIACAO"   : idMultipVariacao,
            "QTFRANQUIA"         : qtFranquia,
            "USAFRANQUIA"        : usaFranquia
        };
        
        General.callRestEngine('product#/fct_variacaoprecbaseserv_widget/editaVariacaoPreco', "Row", params).then(function(){
            ScreenService.closeSwipe();
            widget.reload();
		    General.changeWidgetAcessibility(widget.widgets, true);
        },
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });    
        
    };
    
    
    /** Deleta as variaçoes de preço das abas "Medida Variável"  
      e as faixas de variação da aba "Variação de preço" */
    
    this.deletaVariacao = function(widget){
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
            
            var widgetServicos = getWidgetServicos();
            
            var nrSeqItPrecBServ    = widgetServicos.currentRow.NRSEQITPRECBSERV;
            var cdTpMedidaServ      = widgetServicos.currentRow.CDTPMEDIDASERV;
    
            var CdtpMedidaVariavel  = widget.currentRow.CDTPMEDIDAVARIAVEL; 
            
             var params = {
                "NRSEQITPRECBSERV"   : nrSeqItPrecBServ,
                "CDTPMEDIDASERV"     : cdTpMedidaServ,
                "CDTPMEDIDAVARIAVEL" : CdtpMedidaVariavel
            };
            
            General.callRestEngine('product#/fct_variacaoprecbaseserv_widget/deletaVariacaoPreco', "Row", params).then(function(){
            
                ScreenService.closeSwipe();
                widget.reload();
              
            },
            function(error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });    
        });
    };

    this.copiarOutraMoeda = function($tipo) {
        
        var wdgPopupCopiarMoeda = templateManager.container.getWidget('wdgPopupCopiarMoeda');
        ScreenService.openPopup(wdgPopupCopiarMoeda).then(function() {
            wdgPopupCopiarMoeda.dataSource.data = [];
            wdgPopupCopiarMoeda.newRow();
            wdgPopupCopiarMoeda.currentRow.TIPO = $tipo;
        });
    };

    this.confirmarCopiaOutraMoeda = function(widget) {
        if (widget.isValid()) {
            var wdgTabelaPreco = templateManager.container.getWidget('gridTabelaPreco');

            if (widget.currentRow.TIPO == 'check') {
                var checkedRowsTabelaPreco = wdgTabelaPreco.getCheckedRows();
            }
            else {
                var checkedRowsTabelaPreco = {
                    "0": {  "CDPRECBASESERV" : wdgTabelaPreco.currentRow.CDPRECBASESERV,
                            "DSPRECBASESERV" : wdgTabelaPreco.currentRow.DSPRECBASESERV }
                };
            }

            var params = {
                "CDPRECBASESERV"         : checkedRowsTabelaPreco,
                "fator_conversao"        : widget.currentRow.fator_conversao,
                "NRMOEDA"                : widget.currentRow.NRMOEDA,
                "NMMOEDA"                : widget.currentRow.NMMOEDA,
            };
            
            General.callRestEngine("product#/fct_precbaseserv_widget/salvarCopiaOutraMoeda", "Row", params).then(function() {
                
                ScreenService.showMessage("Processo realizado com sucesso."); 
                if (widget.currentRow.TIPO == 'check') {
                    ScreenService.closePopup();
                    ScreenService.closePopup();
                    wdgTabelaPreco.reload();
                }
                else {
                    ScreenService.closePopup();
                    ScreenService.closeSwipe();
                    wdgTabelaPreco.reload();
                }
                
            }.bind(this))
            .catch(function(response){
                ScreenService.showMessage(response.data.error);
            });
        }
    };
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00006001_Tabela_Preco_Servico_Controller', Sm00006001_Tabela_Preco_Servico_Controller);
});

// FILE: ./assets/js/controllers/Sm00006002_Tipo_de_Medida_Controller.js
function Sm00006002_Tipo_de_Medida_Controller(General, ScreenService, templateManager, Sm00006002_Tipo_de_Medida_Service) {
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Tipo de Medida para salvar o novo registro
    */
    this.salvarTipoDeMedida = function(widget){
        
        widget.currentRow.__columns_to_return = ["CDTPMEDIDASERV", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset && response.dataset.columnsToReturn[0]){
                widget.currentRow.CDTPMEDIDASERV = response.dataset.columnsToReturn[0].CDTPMEDIDASERV;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
            
            this.habilitaVariacao(widget);
            
        }.bind(this));
        
    };
    
    /**
     * Habilitar ou desabilitar a página "Variação do tipo de Medida". 
     * Só deve ficar habilitado quando o tipo de medida for do tipo "Fixo".
     * 
     * @param {object} widget widget de 
     * Tipo de Medida 
    */
    
    this.habilitaVariacao = function(widget){
    
        var tipo = widget.currentRow.IDTPMEDIDASERV;   
        
        var insert = widget.currentRow.__is_new;
      
        if (insert == false){
            if (tipo == 'F') {
                
                templateManager.container.getWidget("gridVariacaoMedida").enabled = true;

            }
            else{
                
                templateManager.container.getWidget("gridVariacaoMedida").enabled = false;
            }
      }
        
    };
    

    /**
     * A função "validaEditTpMedida" realiza validações na edição do Tipo de Medida
     * 
     * @param {object} widget widget de 
     * Tipo de Medida 
     * 
     */
     
    this.validaEditTpMedida = function(widget){
         
        var currentRow = widget.currentRow;
         
        if (currentRow.changed == true){
             
            var tipo = widget.currentRow.IDTPMEDIDASERV;   
            var cdTpMedidaServ = widget.currentRow.CDTPMEDIDASERV;
             
            if(tipo == 'V'){
                
                /** A função "validaExsiteMedidaVariavel" verifica se existem medidas variaveis para o tipo de medida atual.  
                  * Caso tenha, impede a edição de um tipo de medida "Fixo" para "Variavel" */ 
                Sm00006002_Tipo_de_Medida_Service.validaExsiteMedidaVariavel(cdTpMedidaServ).then(function(response){
                    if (response.dataset.MEDIDAVARIAVEL.VALIDA == false) {
                     
                        this.salvarTipoDeMedida(widget);
                     
                    }
                    else{
                        ScreenService.showMessage('Operação Bloqueada. <br><br>"Tipo" não pode ser alterado de "Fixo" para "Variável" quando houverem variações de tipo de medida cadastrados.'); 
                    }
                }.bind(this)).catch(function(response){
                    ScreenService.showMessage(response.data.error);
                }); 
                
            }
            else{
              
                /** A função "validaExsiteVariacao" verifica se o tipo de medida variavel esá sendo utilizado.
                  * Caso tenha impede a edição de um tipo de medida "Variavel" para "Fixo" */
                Sm00006002_Tipo_de_Medida_Service.validaExsiteVariacao(cdTpMedidaServ).then(function(response){
                    if (response.dataset.VARIACAO.VALIDA == false) {
                     
                        this.salvarTipoDeMedida(widget);
                     
                    }
                    else{
                        ScreenService.showMessage('Operação Bloqueada. <br><br>"Tipo" não pode ser alterado de "Variável" para "Fixo" quando estiver sendo utilizado em variações de tipo de medida.'); 
                    }
                }.bind(this)).catch(function(response){
                    ScreenService.showMessage(response.data.error);
                });    
            }
             
             
        } else{
           
            this.salvarTipoDeMedida(widget);
             
        }
         
    };
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Tipo de Medida para salvar o novo registro
    */
    this.salvarVariacaoTipoDeMedida = function(widget){
        

        widget.currentRow.CDTPMEDIDASERV = templateManager.container.getWidget('gridTipoMedida').currentRow.CDTPMEDIDASERV;
        
        widget.currentRow.__columns_to_return = ["NRVARIACAOTPMEDIDA", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset && response.dataset.columnsToReturn[0]){
                widget.currentRow.NRVARIACAOTPMEDIDA = response.dataset.columnsToReturn[0].NRVARIACAOTPMEDIDA;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
            
        }.bind(this));
    };
    
    
     /**
     * A função "validaTipoMedidaVariavel" verifica se o tipo de medida variável
     * ja está criado para a faixa existente.
     * 
     * * @param {object} widget widget ativa 
     */
     
    this.validaTipoMedidaVariavel = function(widget){
        
        var currentRow = widget.currentRow;
        
        var qtMedidaIniNew = currentRow.QTMEDIDAINIVARIACAO;
        var qtMedidaFinNew = currentRow.QTMEDIDAFIMVARIACAO;
        var cdTpMedidaServ = widget.currentRow.CDTPMEDIDASERV;
        var CdtpMedidaVariavel = widget.currentRow.CDTPMEDIDAVARIAVEL;
        var nrVaricacaoTpMedida = widget.currentRow.NRVARIACAOTPMEDIDA;
        
        var inserir = 'S';
        Sm00006002_Tipo_de_Medida_Service.validaTipoMedidaVariavel(cdTpMedidaServ, CdtpMedidaVariavel, nrVaricacaoTpMedida).then(function(response){
            
            var arrayValidacao = response.dataset.VALIDACAO;
            
            arrayValidacao.forEach(function(arrValida){
            
                var qtMedidaIniOld = arrValida.QTMEDIDAINIVARIACAO;
                var qtMedidaFinOld  = arrValida.QTMEDIDAFIMVARIACAO;

                /*A qt.medida inicial do registro a ser inserido deve ser 
                  menor que a qt.medida inicial ou maior do que a qt.medida final 
                  dos registros já inseridos na tabela.*/
                if (qtMedidaIniNew >= qtMedidaIniOld && qtMedidaIniNew <= qtMedidaFinOld){
                
                      inserir = 'N';
                }
                /*A qt.medida inicial do registro a ser inserido deve ser 
                      menor que a qt.medida inicial ou maior do que a qt.medida final 
                      dos registros já inseridos na tabela.*/
                else if (qtMedidaFinNew >= qtMedidaIniOld && qtMedidaFinNew <= qtMedidaFinOld){
                    
                    inserir = 'N';
                }
                else if (qtMedidaIniNew <= qtMedidaIniOld && qtMedidaFinNew >= qtMedidaFinOld){
                    
                    inserir = 'N';
                }
                
            });
            
            if (inserir == 'S') {
                
                this.salvarVariacaoTipoDeMedida(widget);
            } 
            else {
            
                ScreenService.showMessage('Operação Bloqueada. <br><br>Já existe uma variação cadastrada para o "Tipo de Medida para Variação" e para o intervalo de "Quantidade da Medida" informados.'); 
                
            }

       }.bind(this)).catch(function(response){
           ScreenService.showMessage(response.data.error);
       });
    };
    
    
    /**
     * A função "validaIntervaloQtMed" realiza a validação das quantidades de medida
     * da variação de tipo de medida.
     * 
     * * @param {object} widget widget ativa 
     */
    
    this.validaIntervaloQtMed = function(widget){
        
        var qtMedidaIni = widget.currentRow.QTMEDIDAINIVARIACAO;
        var qtMedidaFin = widget.currentRow.QTMEDIDAFIMVARIACAO;
        
        if (qtMedidaIni > qtMedidaFin){
          
            ScreenService.showMessage('Operação Bloqueada. <br><br> "Quantidade da Medida - De" não pode ser maior que "Quantidade da Medida - Até". '); 
            
        }
        else{
            
            this.validaTipoMedidaVariavel(widget);
            
        }
        
    };
    
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00006002_Tipo_de_Medida_Controller', Sm00006002_Tipo_de_Medida_Controller);
});

// FILE: ./assets/js/controllers/Sm00007000_Grupo_Faturamento_Controller.js
function Sm00007000_Grupo_Faturamento_Controller(General, ScreenService) {
    
    /**
     * Bloqueia ou libera o campo "Produto para Faturamento
     * 
     * @param {object} widget 
    */
    this.idBloqueiaProdutoFat = function(widget, idinsertEdit){
        
        var currentRow = widget.currentRow;
        
        if(currentRow.IDPRODUTOFATURA == "UNICO"){
            widget.getField("NMPRODUTOVEND").readOnly = false;
        }
        else{
            widget.getField("NMPRODUTOVEND").readOnly = true;
        }
        
        if (idinsertEdit == 'N'){
            currentRow.NMPRODUTOVEND = null;
        }
    };
}
Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00007000_Grupo_Faturamento_Controller', Sm00007000_Grupo_Faturamento_Controller);
});

// FILE: ./assets/js/controllers/Sm00008000_Habilidade_Pessoal_Controller.js
function Sm00008000_Habilidade_Pessoal_Controller(General, ScreenService, Utilities, templateManager) {
    
    /**
     * Salva o novo registro de Habilidade
     * 
     * @param {object} widget widget 
    */
    this.saveHabilidade = function(widget){
        
        widget.currentRow.__columns_to_return = ["CDHABILITSERV", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                widget.currentRow.CDHABILITSERV  = response.dataset.columnsToReturn[0].CDHABILITSERV;
                widget.currentRow.NRORG          = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO     = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO  = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
            }
            
        }.bind(this));
    };
    
    /**
     * Salva o novo registro de Pessoa
     * 
     * @param {object} widget widget 
    */
    
    this.salvarPessoa = function(widget){
        
        var parametros = {
            "CDHABILITSERV": widget.currentRow.CDHABILITSERV,
            "NRPARCNEGOCIO": widget.currentRow.NRPARCNEGOCIO
        };
        
        /* Valida se uma pessoa ja esta inserida*/
        General.callRestEngine("product#/fct_relachabilipessoa_widget/validacao", "Row", parametros).then(function(sucesso){ // chama uma requisição no backend
            if(sucesso.dataset.validacao.resultado){
                Utilities.inserirMultiplos(widget, "NRPARCNEGOCIO", null, "product#/fct_relachabilipessoa_widget/save").then(function(success) {
                    ScreenService.closeSwipe();
                    widget.reload();
                })
                .catch(function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
            }
            else{
                ScreenService.showMessage("Operação bloqueada.<br>Uma ou mais pessoas selecionadas já estão cadastradas!");
            }
            
        }).catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
        
    };
    
    /**
     * Salva o novo registro de Tarefa
     * 
     * @param {object} widget widget 
    */
    
    this.salvarTarefa = function(widget){
        widget.currentRow.__columns_to_return = ["NRTAREFAHABILI", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        var parametros = {
            "IDORDENACAO"   : widget.currentRow.IDORDENACAO,
            "NRTAREFAHABILI": widget.currentRow.NRTAREFAHABILI,
            "CDHABILITSERV" : widget.currentRow.CDHABILITSERV
        };
        /* Valida se já existe a ordenação da tarefa que está sendo inserida */
        General.callRestEngine("product#/fct_tarefahabili/validar","Row",parametros).then(function(TemSim) {
            if(TemSim.dataset.validar.resultado) {
                General.widgetSave(widget).then(function(response) {
                    if (response.dataset && response.dataset.columnsToReturn[0]) {
                        widget.currentRow.NRTAREFAHABILI = response.dataset.columnsToReturn[0].NRTAREFAHABILI;
                        widget.currentRow.NRORG          = response.dataset.columnsToReturn[0].NRORG;
                        widget.currentRow.DTINCLUSAO     = response.dataset.columnsToReturn[0].DTINCLUSAO;
                        widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                        widget.currentRow.NRORGINCLUSAO  = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                    }
                    
                    if (widget.mode == "list")
                        widget.reload();
                    
                }.bind(this));
            }
            else{
                ScreenService.showMessage("Operação bloqueada!<br/>Já existe uma tarefa cadastrada com a ordem informada.");
            }
        }).catch(function(error){
           console.error(error);
           ScreenService.showMessage(error.data.error);
        });
       
    };
    
    
     /**
     *  A função "ordenar" recebe a próxima a ordenação da tarefa 
     *  que está sendo inserida para a habilidade.
     */
    this.ordenar = function(widget){
        
        var gridTarefa = templateManager.container.getWidget("gridTarefa");
        var cdHabilitServ = gridTarefa.currentRow.CDHABILITSERV;
        var params = {
            "CDHABILITSERV" : cdHabilitServ
        };
        
        General.callRestEngine('product#/fct_tarefahabili_widget/recebeOrdenacao', "Row", params).then(function(response){
          
            var ordenacao = response.dataset.ORDENACAO.IDORDENACAO;
              
            if (ordenacao == 0){
                ordenacao =  10;
            }
            else {
                if (ordenacao % 10 == 0){
                ordenacao = ordenacao + 10;
                }
                else{
                    while((ordenacao % 10) != 0){
                        ordenacao = ordenacao + 1;
                    }
                }
            }
              
            widget.currentRow.IDORDENACAO = ordenacao;
        }, 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    /**
     *  A função "validarHora" realiza a validação da hora para que não 
     *  seja menor que "00:00" nem maior que "23:59" e que o segundo digito da hora 
     *  nao seja maior "6".
     * 
    */
    this.validarHora = function(widget){
        
        var hora = widget.getField('IDHORATAREFA').value();
        var numTempo='';
        var strHora="";
    
        for (var i = 0; i < hora.length; i++) {
            if(hora[i]!=':'){
                
            numTempo=hora[i];
            strHora=strHora+numTempo;
            }
        }
        numTempo = parseInt(strHora);
            if (numTempo < 0 || numTempo > 2359) {    
                widget.currentRow.IDHORATAREFA = null;
            }else{
                const Num = 0;
                if((Num = parseInt(strHora[2])>=6)){
                    widget.currentRow.IDHORATAREFA = null;
            }
        }
    }; 
    }
Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00008000_Habilidade_Pessoal_Controller', Sm00008000_Habilidade_Pessoal_Controller);
});

// FILE: ./assets/js/controllers/Sm00009000_Contrato_Modelo_Controller.js
function Sm00009000_Contrato_Modelo_Controller(General, Utilities, ScreenService, Sm00009000_Contrato_Modelo_Service, templateManager, ZHPromise) {

    var tokenCertisign = null;
    
    var getWidgetPerfilAcesso = function () {
        return templateManager.container.getWidget('ParamPerfilAcesso');
    };
    //field
    var getFieldNrperfilacessoFromWidgetPerfilAcesso = function(){
        return getWidgetPerfilAcesso().getField('NRPERFILACESSO');
    };

    var getWidgetGridContrato = function () {
        return templateManager.container.getWidget('gridModContrato');
    };

    var self = this;

    /**
     * Salva o novo registro do Contrato e salva 
     * também os "Parâmetros para Faturamento" do Contrato.
     * 
     * @param {object} widget widget de 
     * Contrato para salvar o 
     * novo registro.
     */

    this.salvarModeloContrato = function(widget) {
        this.validarParametrosRenovacaoAutomatica(widget).then(function() {
            
            widget.currentRow.__columns_to_return = ["NRSEQMODCONTRA", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
           
            General.widgetSave(widget).then(function(response){
                this.insereSignatariosPadroesContr(widget);
                
                if (response.dataset && response.dataset.columnsToReturn[0]) {
                    widget.currentRow.NRSEQMODCONTRA = response.dataset.columnsToReturn[0].NRSEQMODCONTRA;
                    widget.currentRow.NRORG          = response.dataset.columnsToReturn[0].NRORG;
                    widget.currentRow.DTINCLUSAO     = response.dataset.columnsToReturn[0].DTINCLUSAO;
                    widget.currentRow.NRORGINCLUSAO  = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                    widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                    
                }    
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            }.bind(this));
        }.bind(this));
    };
   
      /**
     * Realiza o controle dos campos de vencimento de acordo com o "Tipo de Faturamento"
     * 
     * @param {object} widget widget de 
     * Modelo de Contrato para pegar o field "IDFATURACONTR".
    */
    
    this.bloquearFieldsVencimento = function(widget){

        var campoNomeModPgto        = widget.getField("DSMODPGTO");
        var campoQtdParcelas        = widget.getField("QTPARCELASFATU");
        var campoDiaVencParcelaFatu = widget.getField("NRDIAVENCPARCELASFATU");
        
        
        var tipofaturamento = widget.currentRow.IDFATURACONTR;

        if (tipofaturamento == "A") {
            
            campoNomeModPgto.readOnly           = true;
            campoNomeModPgto.validations        = {};
            widget.currentRow.CDMODPGTO         = null;
            campoNomeModPgto.value(null);
            
            campoQtdParcelas.readOnly           = true;
            campoQtdParcelas.validations        = {};
            campoQtdParcelas.value(null);
            
            campoDiaVencParcelaFatu.readOnly    = true;
            campoDiaVencParcelaFatu.value(null);
            campoDiaVencParcelaFatu.validations = {};
            
        } else {
            if(tipofaturamento === "O"){
                
                campoNomeModPgto.readOnly           = false;
                campoNomeModPgto.validations        = { "required": true };
                
                campoQtdParcelas.readOnly           = true;
                campoQtdParcelas.validations        = {};
                campoQtdParcelas.value(null);
                
                campoDiaVencParcelaFatu.readOnly    = true;
                campoDiaVencParcelaFatu.validations = {};
                campoDiaVencParcelaFatu.value(null);
                
            } else {
                if(tipofaturamento === "X"){
                    
                    campoNomeModPgto.readOnly           = true;
                    campoNomeModPgto.validations        = {};
                    widget.currentRow.CDMODPGTO         = null;
                    campoNomeModPgto.value(null);
                    
                    campoQtdParcelas.readOnly           = true;
                    campoQtdParcelas.validations        = {};
                    campoQtdParcelas.value(null);
                    
                    campoDiaVencParcelaFatu.readOnly    = true;
                    campoDiaVencParcelaFatu.validations = {};
                    campoDiaVencParcelaFatu.value(null);

                } else {
                    if(tipofaturamento === "P"){
                        campoNomeModPgto.readOnly           = true;
                        campoNomeModPgto.validations        = {};
                        widget.currentRow.CDMODPGTO         = null;
                        campoNomeModPgto.value(null);
                        
                        campoQtdParcelas.readOnly           = false;
                        campoQtdParcelas.validations        = { "required": true};
                        
                        campoDiaVencParcelaFatu.readOnly    = false;
                        campoDiaVencParcelaFatu.validations = { "required": true};
                    }
                }
            }
        }
    };
    
    
    /* Funções Faturamento*/
   
    /**
     * Realiza o controle dos campos de vencimento de acordo com o "Tipo de Faturamento"
     * 
     * @param {object} widget widget de 
     * Contrato para pegar o field "IDFATURACONTR".
     */
    this.bloquearFieldsParcela = function(widget){
        
        var tipofaturamento = widget.parent.getField('IDFATURACONTR').value();
      
        var campoTpCompetencia      = widget.getField("IDQTDIASCOMPCON");
        var campoTpFchCompetencia   = widget.getField("IDTPCOMPETCONT");
        var campoTpEmissaoNF        = widget.getField("IDTPDTEMISSNF");
        var campoTpVencNF           = widget.getField("IDTPDTVENC");
        var campoTpPeriodoInc       = widget.getField("IDTPFATUINCOMP");
        var campoFatItem            = widget.getField("IDSOMAITGRUPFAT");
        var campoTpFat              = widget.getField("IDSOMAFATUCONT");
        
        campoFatItem.defaultValue            = "N";
        widget.currentRow.IDSOMAITGRUPFAT   = "N";
        campoTpFat.defaultValue              = "N";
        widget.currentRow.IDSOMAFATUCONT    = "N";

        if (tipofaturamento == "A") {
            
            campoTpCompetencia.readOnly         = false;
            campoTpCompetencia.validations      = { "required": true };
            
            campoTpFchCompetencia.readOnly      = false;
            campoTpFchCompetencia.validations   = { "required": true };
            
            campoTpEmissaoNF.readOnly           = false;
            campoTpEmissaoNF.validations        = { "required": true };
            
            campoTpVencNF.readOnly              = false;
            campoTpVencNF.validations           = { "required": true };
            
            campoTpPeriodoInc.readOnly          = false;
            campoTpPeriodoInc.validations       = { "required": true };
            
        } else {
            if(tipofaturamento === "O"){
                
                campoTpCompetencia.readOnly         = true;
                campoTpCompetencia.validations      = {};
                campoTpCompetencia.value(null);
                
                campoTpFchCompetencia.readOnly      = true;
                campoTpFchCompetencia.validations   = {};
                campoTpFchCompetencia.value(null);
                
                campoTpEmissaoNF.readOnly           = true;
                campoTpEmissaoNF.validations        = {};
                campoTpEmissaoNF.value(null);
                
                campoTpVencNF.readOnly              = true;
                campoTpVencNF.validations           = {};
                campoTpVencNF.value(null);
                
                campoTpPeriodoInc.readOnly          = true;
                campoTpPeriodoInc.validations       = {};
                campoTpPeriodoInc.value(null);
                
            } else {
                if(tipofaturamento === "X"){
                    
                    campoTpCompetencia.readOnly         = false;
                    campoTpCompetencia.validations      = { "required": true };
                    
                    campoTpFchCompetencia.readOnly      = false;
                    campoTpFchCompetencia.validations   = { "required": true };
                    
                    campoTpEmissaoNF.readOnly           = false;
                    campoTpEmissaoNF.validations        = { "required": true };
                    
                    campoTpVencNF.readOnly              = false;
                    campoTpVencNF.validations           = { "required": true };
                    
                    campoTpPeriodoInc.readOnly          = false;
                    campoTpPeriodoInc.validations       = { "required": true };
                        
                    campoFatItem.defaultValue            = "S";
                    widget.currentRow.IDSOMAITGRUPFAT   = "S";
                    campoTpFat.defaultValue              = "S";
                    widget.currentRow.IDSOMAFATUCONT    = "S";

                } else {
                    if(tipofaturamento === "P"){
                        
                        campoTpCompetencia.readOnly         = true;
                        campoTpCompetencia.validations      = {};
                        campoTpCompetencia.value(null);
                        
                        campoTpFchCompetencia.readOnly      = true;
                        campoTpFchCompetencia.validations   = {};
                        campoTpFchCompetencia.value(null);
                        
                        campoTpEmissaoNF.readOnly           = true;
                        campoTpEmissaoNF.validations        = {};
                        campoTpEmissaoNF.value(null);
                        
                        campoTpVencNF.readOnly              = true;
                        campoTpVencNF.validations           = {};
                        campoTpVencNF.value(null);
                        
                        campoTpPeriodoInc.readOnly          = true;
                        campoTpPeriodoInc.validations       = {};
                        campoTpPeriodoInc.value(null);
                    }
                }
            }
        }
    };
   
   
     /**
     * Realiza o controle do campo "Dia Para Competência do Contrato" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDQTDIASCOMPCON".
    */
    
    this.bloquearFieldDiasComp = function(widget){
        
        var tipoCompetencia = widget.getField('IDQTDIASCOMPCON').value();
        
        /* Desbloqueia o campo "Dia Para Competência do Contrato" */
        if (tipoCompetencia == "2") {
            
            widget.getField('NRQTDIASCOMPCON').readOnly = false;
            widget.getField('NRQTDIASCOMPCON').validations = {"required": true};
        } 
        /* Bloqueia o campo "Dia Para Competência do Contrato" */
        else {
            widget.getField('NRQTDIASCOMPCON').readOnly = true;
            widget.getField('NRQTDIASCOMPCON').setValue(null);
            widget.getField('NRQTDIASCOMPCON').validations = {};
        }
    };
   
   
     /**
     * Realiza o controle do campo "Dia Para Competência do Contrato" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDQTDIASCOMPCON".
    */
    
    this.ValidaFieldDiasComp = function(widget){
        
        var dia = widget.getField('NRQTDIASCOMPCON').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRQTDIASCOMPCON = null;
        }

    };
   
    /**
     * Realiza o controle do campo "Dia Para Fechamento da Competência" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPCOMPETCONT".
    */
    
    this.bloquearFieldDiasFecha = function(widget){
        
        var fechaCompetencia = widget.getField('IDTPCOMPETCONT').value();
        
        /* Bloqueia o campo "Dia Para Fechamento da Competência" */
        if (fechaCompetencia == "2" || fechaCompetencia == null) {
            
            widget.getField('NRDIACOMPETCONT').readOnly = true;
            widget.getField('NRDIACOMPETCONT').setValue(null);
            widget.getField('NRDIACOMPETCONT').validations = {};
        } 
        /* Desbloqueia o campo "Dia Para Fechamento da Competência" */
        else {

            widget.getField('NRDIACOMPETCONT').readOnly = false;
            widget.getField('NRDIACOMPETCONT').validations = {"required": true};
        }
    };
   
   
    /**
     * Realiza o controle do campo "Dia Para Fechamento da Competência" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPCOMPETCONT".
    */
    
    this.ValidaFieldDiasFecha = function(widget){
        
        var dia = widget.getField('NRDIACOMPETCONT').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRDIACOMPETCONT = null;
        }

    };
   
    /**
     * Realiza o controle do campo "Dia Para Emissão da NF" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTEMISSNF".
    */
    
    this.bloquearFieldDiasEmissao = function(widget){
        
        var emissao = widget.getField('IDTPDTEMISSNF').value();
        
        /* Bloqueia o campo "Dia Para Emissão da NF" */
        if (emissao == "9" || emissao == null) {
            
            widget.getField('NRDIADTEMISSNF').readOnly = true;
            widget.getField('NRDIADTEMISSNF').setValue(null);
            widget.getField('NRDIADTEMISSNF').validations = {};

        } 
        /* Desbloqueia o campo "Dia Para Emissão da NF" */
        else {

            widget.getField('NRDIADTEMISSNF').readOnly = false;
            widget.getField('NRDIADTEMISSNF').validations = {"required": true};
        }
    };
    
    /**
     * Realiza o controle do campo "Dia Para Emissão da NF" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTEMISSNF".
    */
    
    this.ValidaFielDiasEmissao = function(widget){
        
        var dia = widget.getField('NRDIADTEMISSNF').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRDIADTEMISSNF = null;
        }

    };
    
    /**
     * Realiza o controle do campo "Dia Para Vencimento da Parcela" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTVENC".
    */
    
    this.bloquearFieldDiasVenc = function(widget){
        
        var vencimento = widget.getField('IDTPDTVENC').value();
        
        /* Bloqueia o campo "Dia Para Vencimento da Parcela" */
        if (vencimento == "5" || vencimento == null) {
            
            widget.getField('NRDIAVENPARCFAT').readOnly = true;
            widget.getField('NRDIAVENPARCFAT').setValue(null);
            widget.getField('NRDIAVENPARCFAT').validations = {};
            
        } 
        /* Desbloqueia o campo "Dia Para Vencimento da Parcela" */
        else {

            widget.getField('NRDIAVENPARCFAT').readOnly = false;
            widget.getField('NRDIAVENPARCFAT').validations = {"required": true};
        }
    };
    
   /**
     * Realiza o controle do campo "Dia Para Vencimento da Parcela" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTVENC".
    */
    
    this.validaFielDiasVenc = function(widget){
        var dia = widget.currentRow.NRDIAVENPARCFAT;
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    
            widget.currentRow.NRDIAVENPARCFAT = null;
        }
    };
   
  /**
   * Agrupa os controles de campos do faturamento
   *
   * @param {object} widget widget de Parametrização de Faturamento
   */
   this.controlaCamposParcela = function(widget){
       this.bloquearFieldsParcela(widget);
       this.bloquearFieldDiasComp(widget);
       this.bloquearFieldDiasFecha(widget);
       this.bloquearFieldDiasEmissao(widget);
       this.bloquearFieldDiasVenc(widget);
   };
  
   /**
    * A função "insereDadosModContrato" insere os dados de parametização 
    * de fatiuramento na tabela de Modelo de Contrato (FCT_MODELOCONTRATO).
    * 
    * @param {object} widget widget de Parametros de Faturamento
    */
    this.insereDadosModContrato = function(widget){
        var dtInicFatContr  = widget.currentRow.DTINICFATCONTR;
        var idQtDiasCompCon = widget.currentRow.IDQTDIASCOMPCON;
        var nrQtDiasCompCon = widget.currentRow.NRQTDIASCOMPCON;
        var idCompetCont    = widget.currentRow.IDTPCOMPETCONT;
        var nrDiaCompetCont = widget.currentRow.NRDIACOMPETCONT;
        var idTpDtEmissNf   = widget.currentRow.IDTPDTEMISSNF;
        var nrDiaDtEmissNf  = widget.currentRow.NRDIADTEMISSNF;
        var idTpDtVenc      = widget.currentRow.IDTPDTVENC;
        var nrDiaVenParcFat = widget.currentRow.NRDIAVENPARCFAT;
        var idTpFatuIncomp  = widget.currentRow.IDTPFATUINCOMP;
        var seqModContrato  = widget.currentRow.NRSEQMODCONTRA;
           
        Sm00009000_Contrato_Modelo_Service.insereDadosModContrato(dtInicFatContr, idQtDiasCompCon, nrQtDiasCompCon, idCompetCont, nrDiaCompetCont, idTpDtEmissNf, nrDiaDtEmissNf, idTpDtVenc, nrDiaVenParcFat, idTpFatuIncomp, seqModContrato).then(null).catch(function(response){
            ScreenService.showMessage(response.data.error);
        });
        
    };
    
    
   
    
    /* Funções Reajustes */
   
    /**
     * Salva o novo registro da página "Parâmetros de Reajuste"
     * 
     * @param {object} widget widget de 
     * Parâmetros de Reajuste para salvar 
     * o novo registro.
    */
    this.salvarParamReajsute = function(widget){
        
        widget.currentRow.NRSEQMODCONTRA = templateManager.container.getWidget("gridModContrato").currentRow.NRSEQMODCONTRA;
        widget.currentRow.__columns_to_return = ["NRSEQMODREAJUS", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset)
                widget.currentRow.NRSEQMODREAJUS = response.dataset.columnsToReturn[0].NRSEQMODREAJUS;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            
        }.bind(this));
    };

    this.fixSelectMultipleBug = function() {
        var field = getFieldNrperfilacessoFromWidgetPerfilAcesso();
        field.reload();
    };


    this.salvarPerfilAcesso = function(widget){
        var parametro = {
            "NRSEQMODCONTRA"     : widget.currentRow.NRSEQMODCONTRA,
            "NRORG"   : widget.currentRow.NRORG,
            "PROPOUCONT" : "C"
        };
        General.callRestEngine('product#/procuraPerfilParametrizado', "Row", parametro).then(function (perfilusuario) {
            if(perfilusuario.dataset.validacao){
                var perfisPermitidos = [];
                widget.currentRow.NRPERFILACESSO.forEach(function(perfil){
                    if(!perfilusuario.dataset.validacao.includes(perfil)){
                        perfisPermitidos.push(perfil);
                    }
                });
                widget.currentRow.NRPERFILACESSO = perfisPermitidos;
            }
            Utilities.inserirMultiplosComNovoCodigo(widget, "NRPERFILACESSO").then(function () {
                ScreenService.closeSwipe().then(function () {
                    getWidgetPerfilAcesso().reload();
                });

            }.bind(this));
            var field = getFieldNrperfilacessoFromWidgetPerfilAcesso();
            field.dataSource.data = [];
        });
    };


    /**
     * Salva o novo registro da página "índice de Reajuste"
     * 
     * @param {object} widget widget de 
     * índice de Reajuste para salvar 
     * o novo registro.
    */
    this.salvarIndReajsute = function(widget){
        
        widget.currentRow.NRSEQMODCONTRA = widget.parent.currentRow.NRSEQMODCONTRA;
        widget.currentRow.NRSEQMODREAJUS = widget.parent.currentRow.NRSEQMODREAJUS;
        
        widget.currentRow.__columns_to_return = ["NRSEQINDCONTR",  "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset)
                widget.currentRow.NRSEQINDCONTR = response.dataset.columnsToReturn[0].NRSEQINDCONTR;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO  = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            
        }.bind(this));
    };
    
     /**
     * Realiza o controle de bloqueio e desbloqueio dos campos 
     * da página "Parâmetros de Reajuste"
     * 
     * @param {object} widget widget de 
     * Parâmetros de Reajuste para salvar 
     * o novo registro.
    */
    
    this.controlaCamposReajuste = function(widget){
        
        var tiporeajuste = widget.getField('IDTPREAJUSTE').value();
        
        /* 1 - Anual: Desbloqueia o campo "Mês para Reajsute" e 
         * bloqueia o campo "Número de Meses".
         */
         
        if (tiporeajuste == '1') {
            
            /* Mês para Reajuste */
            widget.getField('IDTPMESREAJUSTE').readOnly = false;
            widget.getField('IDTPMESREAJUSTE').validations = {"required": true};
            
            /* Número de Meses */
            widget.getField('NRQTDREAJUSTE').readOnly = true;
            widget.getField('NRQTDREAJUSTE').setValue(null);
            widget.getField('NRQTDREAJUSTE').validations = {};
            
        } 
        /* 2 - No ato de Prorrogação do Contrato: Bloqueia os campos 
         * "Mês para Reajsute" e "Número de Meses".
         */
        else if (tiporeajuste == '2') {
            
            /* Mês para Reajuste */
            widget.getField('IDTPMESREAJUSTE').readOnly = true;
            widget.getField('IDTPMESREAJUSTE').setValue(null);
            widget.getField('IDTPMESREAJUSTE').validations = {};
            
            /* Número de Meses */
            widget.getField('NRQTDREAJUSTE').readOnly = true;
            widget.getField('NRQTDREAJUSTE').setValue(null);
            widget.getField('NRQTDREAJUSTE').validations = {};
        }
        /* 3 - Número de Meses: Bloqueia o campo "Mês para Reajsute" e 
         * desbloqueia o campo "Número de Meses".
         */
        else{
           
            /* Mês para Reajuste */
            widget.getField('IDTPMESREAJUSTE').readOnly = true;
            widget.getField('IDTPMESREAJUSTE').setValue(null);
            widget.getField('IDTPMESREAJUSTE').validations = {};
            
            /* Número de Meses */
            widget.getField('NRQTDREAJUSTE').readOnly = false;
            widget.getField('NRQTDREAJUSTE').validations = {"required": true};
        }
    };
    
   /**
     * Realiza o controle de Percentual do field "Valor Percentual"
     * 
     * @param {object} widget widget de 
     * Itens para pegar o field "VRPERCINDCONTR"".
    */
    
    this.ValidaPercIndice = function(widget){
        
        var vrPercentual = widget.getField('VRPERCINDCONTR').value();
        
        /* Aplica Max e Min Value*/
        if (vrPercentual < 0 || vrPercentual > 100) {    

            widget.currentRow.VRPERCINDCONTR = null;
        }

    };
    
    
    
    /* Funções Documento */

    /**
     * Salva o novo registro da página "Documento"
     * 
     * @param {object} widget widget de 
     * Documento para salvar o 
     * novo registro.
    */
    this.salvarDocumento = function(widget){
        if (widget.isValid()) {
            widget.currentRow.NRSEQMODCONTRA = templateManager.container.getWidget("gridModContrato").currentRow.NRSEQMODCONTRA;
            widget.currentRow.__columns_to_return = ["NRSEQMODELOCCONTR",  "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
            Utilities.validarCamposDuplicados(widget, ["NRSEQMODELOCCONTR"], ["NRSEQMODCONTRA", "IDSITUACAOCONTRATO"]).then(function(validacao){
                if(!validacao){
                    ScreenService.showMessage("Já existe um modelo de documento vinculado a essa situação."); 
                }
                else {
                    var isNew = widget.currentRow.__is_new;
                    if(!isNew) {
                        this.requestDeletarDocumentosContrato(widget).then(function() {
                            return Utilities.inserirMultiplosComNovoCodigo(widget, "IDSITUACAOCONTRATO");
                        }.bind(this))
                        .then(function() {
                            ScreenService.closeSwipe();
                            widget.reload();
                        })
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }
                    else {
                        Utilities.inserirMultiplosComNovoCodigo(widget, "IDSITUACAOCONTRATO").then(function(response){
                            ScreenService.closeSwipe();
                            widget.reload();
                        });
                    }
                }
            }.bind(this));
        }
    };
    
    /**
    * Habilita fields de situação
    * @params{Object} widget documento
    **/
    this.habilitaFieldSituacao = function(widgetDocumento, actionName){
        var idSituacao = widgetDocumento.currentRow.IDSITUACAOCONTRATO;
        if(idSituacao) {
            if(actionName == "edit" && !Array.isArray(idSituacao)) 
                widgetDocumento.currentRow.IDSITUACAOCONTRATO = idSituacao.split(", ");
            else 
                if(actionName != "edit" && Array.isArray(idSituacao))
                    widgetDocumento.currentRow.IDSITUACAOCONTRATO = idSituacao.join(", ");
        }
        /*var fieldId      = widgetDocumento.getField("IDSITUACAOCONTRATOEDIT");
        var fieldIdMulti = widgetDocumento.getField("IDSITUACAOCONTRATO");
        
        if(habilita){
            fieldId.name           = "IDSITUACAOCONTRATO";
            fieldIdMulti.name      = "IDSITUACAOCONTRATOEDIT";
            fieldId.isVisible      = true;
            fieldIdMulti.isVisible = false;
        } else {
            if(!widgetDocumento.currentRow.__is_new){
                fieldIdMulti.isVisible = false;
                fieldId.isVisible      = true;
                fieldIdMulti.name      = "IDSITUACAOCONTRATOEDIT";
                fieldId.name           = "IDSITUACAOCONTRATO";   
            }
        }*/
    };
    
    /**
    * Edita um documento
    * @params{Object} widget modelo de documento
    **/
    this.documentoEdit = function(widgetDocumento){
        this.habilitaFieldSituacao(widgetDocumento, "edit");
        General.widgetEdit(widgetDocumento);
    };
    
    /**
    * Cancela o formulário da widget de documento
    * @params{Object} widget
    **/
    this.cancelaFormDocumento = function(widgetDocumento){
        this.habilitaFieldSituacao(widgetDocumento, "cancel");
        General.widgetCancelForm(widgetDocumento);
    };
    
    this.deletarDocumentosContrato = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), "question", function(){
            this.requestDeletarDocumentosContrato(widget).then(function() {
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }.bind(this));
    };
    
    this.requestDeletarDocumentosContrato = function(widget) {
        return General.callRestEngine("product#/fct_modelodoccontr_widget/deletarDocumentos", "DataSet", [widget.currentRow]);
    };
    
    this.setarCurrentRowWdgParamFaturamento = function(wdgParamFaturamento) {
        var row = wdgParamFaturamento.dataSource.data[0];
        wdgParamFaturamento.currentRow = (row) ? row : {};
    };
    
    this.wdgParamFaturamentoOnEnter = function(wdgParamFaturamento) {
        wdgParamFaturamento.list();
        wdgParamFaturamento.reload();
        wdgParamFaturamento.view();
    };

    this.wdgPerfilAcessoCancelar = function() {
        var field = getFieldNrperfilacessoFromWidgetPerfilAcesso();
        field.dataSource.data = [];
    };
    
    this.editarParamFat = function(widget) {
        widget.backupRow = Util.clone(widget.currentRow);
		widget.edit();
		widget.template = "widget/form.html";
		this.controlaCamposParcela(widget);
    };
    this.editarPerfil = function(widget) {
		widget.edit();
		widget.template = "widget/form.html";
    };
    
    this.cancelarParamFat = function(widget) {
        Object.keys(widget.backupRow).forEach(function (key) {
			widget.currentRow[key] = widget.backupRow[key];
		}); 
		widget.view();
		widget.template = "widget/form-view.html";
    };
    
    this.salvaWdgtParamFat = function(widget){
        General.widgetSave(widget).then(function(response){
            widget.view();
            widget.template = "widget/form-view.html";
        });
    };
    /**
     * Salva um escopo de faturamento
     * @params{Object} widget - widget de escopo do faturamento
    **/
    this.salvarEscopoFaturamento = function(widget){
        if(!widget.currentRow.__is_new){
            General.widgetSave(widget);
        } else {
            Utilities.validarCamposDuplicados(widget, null, ["NRSEQMODCONTRA", "NRTIPOORCAMENTO", "NRORG"]).then(function(response){
                if(!response){
                    ScreenService.showMessage("Operação bloqueada!<br>O escopo de faturamento selecionado já está inserido nesse modelo de contrato.");
                } else {
                    Utilities.inserirMultiplosComNovoCodigo(widget, "NRTIPOORCAMENTO").then(function(){
                        ScreenService.closeSwipe();
                        widget.list();
                        widget.reload();
                    }).catch(function(error){
                        Utilities.showErrorMessage(error);
                    });
                }
            });
        }
    };
    
    /**
     * Habilita e desabilita os campos ligados aos parametros de renovação dos contratos
     * 
     * @params{Object} widget - widget de modelo de contrato
    **/
    this.validarCamposRenovacaoAutomatica = function(widget) {
        var fieldDiasProrrogacao = widget.getField("NRDIASPRORGCONT");
        var fieldPermiteReajuste = widget.getField("IDREAJUSTEAUTOMATICO");
        var possuiRenovacao = (widget.currentRow.IDRENOVACAOAUTOMATICA == 'S');

        fieldDiasProrrogacao.readOnly = !possuiRenovacao;
        fieldPermiteReajuste.readOnly = !possuiRenovacao;
        if (!possuiRenovacao) {
            fieldPermiteReajuste.value("N");
        }
    };

    
    this.adicionarModelo = function(widget) {
        General.widgetNewRow(widget);
        this.validarCamposRenovacaoAutomatica(widget);
        this.assinaturaEletronicaOnChange(widget);
        this.integracaoFMOnChange(widget);
        this.integracaoTecFoodOnChange(widget);
    };

    this.editarModelo = function(widget) {
        General.widgetEdit(widget);
        this.validarCamposRenovacaoAutomatica(widget);
        this.validaIdCertsign(false);
        this.integracaoFMOnChange(widget);
        this.integracaoTecFoodOnChange(widget);
    };
    
    /**
     * Valida se as informações de dias para prorrogação estão coniventes com o parametro
     * de renovação automática
     * 
     * @params{Object} widget - widget de modelo de contrato
    **/
    this.validarParametrosRenovacaoAutomatica = function(widget) {
        var deferred = ZHPromise.defer();
        if(widget.currentRow.IDRENOVACAOAUTOMATICA != "S" || widget.currentRow.NRDIASPRORGCONT > 0) {
            deferred.resolve();
        } else {
            ScreenService.confirmMessage("Atenção!<br><br>A não definição da quantidade de dias para prorrogação do contrato não permitirá a realização da renovação automática dos contratos vinculados a esse modelo.<br>Deseja continuar?", 'question',
            function() {
                deferred.resolve();
            }, function() {
                deferred.reject();
            });
        }
        
        return deferred.promise;
    };
    
    this.saveSignatPadr = function(widget) {
        if (widget.currentRow.__is_new) {
            General.callRestEngine('product#/fct_paramfatmodcont_widget/validaSignatPadrCont', "Row", widget.currentRow).then(function(response) {  
                
                var retorno = response.dataset.validaSignatPadrCont.valida;
                
                /* Recebe os envolvidos para notificação*/
                var envolvidos = null;
                var arrayEnvolvidos = widget.currentRow.DSEMAILNOTSIGNENV_INS;
                
                if (arrayEnvolvidos != null && arrayEnvolvidos != ""){
                    envolvidos = arrayEnvolvidos.join('_');
                }

                widget.currentRow.DSEMAILNOTSIGNENV = envolvidos; 
                
                /* Recebe os operdores para notificação */
                var operadores = null;
                var arrayOperadores = widget.currentRow.CDOPERADORES;
                
                if (arrayOperadores != null && arrayOperadores != ""){
                    operadores = arrayOperadores.join('_');
                }
                
                if(retorno == '0') {
                    General.widgetSave(widget).then(function(response){
                        if(response.dataset && response.dataset.columnsToReturn[0]){
                            widget.currentRow.NRORG            = response.dataset.columnsToReturn[0].NRORG;
                            widget.currentRow.NRSIGNATPADRCONT = response.dataset.columnsToReturn[0].NRSIGNATPADRCONT;
                            widget.currentRow.NRSEQMODCONTRA   = response.dataset.columnsToReturn[0].NRSEQMODCONTRA;
                                            
                            if (operadores != null) {
                               var params = { 
                                    "NRSIGNATPADRCONT" : widget.currentRow.NRSIGNATPADRCONT,
                                    "OPERADORES"        : operadores
                               };
            
                               General.callRestEngine('product#/fct_paramfatmodcont_widget/insereOperadoresNotifica', "Row", params).then(function() {
                               
                                    widget.reload();
                                    
                               })
                               .catch(function(response) {
                                   console.error(response);
                                   if (response instanceof Object)
                                       ScreenService.showMessage(response.data.error);
                                   else
                                       ScreenService.showMessage(response);
                               });
                           }
                           else{
                               
                              widget.reload(); 
                           }
                        }
                    }.bind(this))
                    .catch(function(response) {
                        console.error(response);
                        if (response instanceof Object)
                            ScreenService.showMessage(response.data.error);
                        else
                            ScreenService.showMessage(response);
                    });
                }
                else{
                    ScreenService.showMessage("Operação Bloqueada! <br><br> O signatário informado já foi inserido.");
                }
            });
        }
        else{
            /* Recebe os envolvidos para notificação*/
            var envolvidos = null;
            var arrayEnvolvidos = widget.currentRow.DSEMAILNOTSIGNENV_INS;
            
            if (arrayEnvolvidos != null){
                envolvidos = arrayEnvolvidos.join('_');
            }
            
            widget.currentRow.DSEMAILNOTSIGNENV = envolvidos;
            
            /* Recebe os operdores para notificação */
            var operadoresIguais = false;
            
            var bkpOperadores = widget.backupRow.CDOPERADORES;
            var arrayOperadores = widget.currentRow.CDOPERADORES;
            
            if (Utilities.arraysSaoIguais(bkpOperadores, arrayOperadores)) {
                operadoresIguais = true;
            }
            
            General.widgetSave(widget).then(function(response) {
                if (response.dataset && response.dataset.columnsToReturn[0]) {
                    widget.currentRow.NRORG            = response.dataset.columnsToReturn[0].NRORG;
                    widget.currentRow.NRSIGNATPADRCONT = response.dataset.columnsToReturn[0].NRSIGNATPADRCONT;
                    widget.currentRow.NRSEQMODCONTRA   = response.dataset.columnsToReturn[0].NRSEQMODCONTRA;
                }
                
                if (operadoresIguais == false) {
                    var operadores = arrayOperadores.join('_');
                    
                    var params = { 
                        "NRSIGNATPADRCONT" : widget.currentRow.NRSIGNATPADRCONT,
                        "OPERADORES"       : operadores
                    };

                    General.callRestEngine('product#/fct_paramfatmodcont_widget/insereOperadoresNotifica', "Row", params).then(function() {
                        widget.reload().then(function() {
                            this.carregaDadosOperadores(widget);
                        }.bind(this));
                    }.bind(this))
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
                else {
                    this.carregaDadosOperadores(widget);
                }
            }.bind(this))
            .catch(function(response) {
                console.error(response);
                if (response instanceof Object)
                    ScreenService.showMessage(response.data.error);
                else
                    ScreenService.showMessage(response);
            });  
        }
    };
    
    /**
     * insere os signatários padrões de modelo de contrato que foram buscados na parametrização SM 
    **/
    this.insereSignatariosPadroesContr = function(widget){
        
        General.callRestEngine("product#/fct_paramfatmodcont_widget/insereSignatariosPadroesContr", "Row", widget.currentRow).then(function() {

        }, function(error) {
            Utilities.showErrorMessage(error);
        });
        
    };
    /**
     * Abre a widget de Cadastramento Automático.
     */
    this.abrirCadAuto = function() {
        
        var wdgSignatPadrCadAuto = templateManager.container.getWidget("signatariosPadroesCadAuto");
        var wdgGridModContrato = templateManager.container.getWidget("gridModContrato");
        
        wdgSignatPadrCadAuto.isVisible = true;
        var row = wdgSignatPadrCadAuto.currentRow;
        row.IDTPSIGNATARIO = "SE";
        row.DSEMAILNOTIFOUTROS = "";
        row.DSEMAILNOTSIGNENV = [];
        row.CDOPERADOR = [];
        row.CDOPERADORES = [];
        wdgSignatPadrCadAuto.edit();
        
        wdgSignatPadrCadAuto.getField("NRSEQMODCONTRA").reload().then(function() {
            wdgSignatPadrCadAuto.currentRow.NRSEQMODCONTRA = [wdgGridModContrato.currentRow.NRSEQMODCONTRA];
        });
        
        row.NRPARCNEGOCIO    = [];
        row.IDTIPOASSINATURA = 'E';
        row.IDTESTEMUNHA     = 'N';
        row.NRORDENACAO      = null;
        
        ScreenService.openSwipe(wdgSignatPadrCadAuto);
        this.controlaSignatarioCadastroAutomatico(wdgSignatPadrCadAuto);
        templateManager.updateTemplate();
    };
    
    /**
     * Fecha a widget de Cadastramento Automático.
     */
    this.cancelaCadAutoSignatPadr = function() {
        
        var wdgSignatPadrCadAuto = templateManager.container.getWidget("signatariosPadroesCadAuto");

        ScreenService.closeSwipe().then(function() {
            wdgSignatPadrCadAuto.isVisible = false;
        });
    };
    
    /**
     * Coloca a widget "signatariosPadroesCadAuto" invisível
     */
    this.CadAutoInvisivel = function() {
        
        var wdgSignatPadrCadAuto = templateManager.container.getWidget("signatariosPadroesCadAuto");

        wdgSignatPadrCadAuto.isVisible = false;
    };
    
    /**
     * Salva os signatários padrões em cadastramento automático
     */
    this.saveSignatPadrCadAutoCont = function(widget) {
        
        if (widget.isValid()) {
        
            var wdgSignatariosPadroes = templateManager.container.getWidget("signatariosPadroes");
            
            General.callRestEngine('product#/fct_paramfatmodcont_widget/cadAutoSignatPadrCont', "Row", widget.currentRow).then(function(response) {  
                var retorno = response.dataset.cadauto.retorno;
                if(retorno == 'OK') {
                    ScreenService.closeSwipe().then(function() {
                        widget.isVisible = false;
                        wdgSignatariosPadroes.reload();
                        ScreenService.showMessage("Cadastramento automático realizado com sucesso!");
                    });
                } else {
                    ScreenService.showMessage("Ocorreram erros durante o processo de cadastramento automático. Favor entrar em contato o administrador do sistema. <br><br> Erros: <br><br>"+retorno);
                }
            });
        }
    };
    
    /**
     * Valida se o token da certsign está preenchido
     */
    this.validaIdCertsign = function(buscarToken) {
        if (buscarToken === undefined) {
            buscarToken = false;
        }

        if (buscarToken) {
            General.callRestEngine('product#/fct_paramfatmodcont_widget/validaIdCertsign', "Row", {}).then(function(response) {
                tokenCertisign = response.dataset.validaIdCertsign.retorno;
                this.controlarCamposCertsign();
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
        else {
            this.controlarCamposCertsign();
        }
    };
    
    this.controlarCamposCertsign = function() {
        var wdgSignatariosPadroes = templateManager.container.getWidget("signatariosPadroes");
        var wdgGridModContrato = templateManager.container.getWidget("gridModContrato");
        
        if (tokenCertisign == null) {
            /* Bloqueia a página "Signatários Padrões" */
            wdgSignatariosPadroes.enabled = false;
            
            /* Bloqueia o campo "Utiliza Assinatura Eletrônica ou Digital" */
            wdgGridModContrato.getField('IDASSELETRDIGITAL').readOnly = true;
            wdgGridModContrato.getField('IDASSELETRDIGITAL').validations = {};
            if (wdgGridModContrato.mode == "edit") {
                wdgGridModContrato.getField('IDASSELETRDIGITAL').value('N');
            }
        }
        else{
            /* Libera o campo "Utiliza Assinatura Eletrônica ou Digital" */
            wdgGridModContrato.getField('IDASSELETRDIGITAL').readOnly = false;
            wdgGridModContrato.getField('IDASSELETRDIGITAL').validations = {"required" : true};
            
            if (wdgGridModContrato.mode != "edit") {
                /* Linera a página "Signatários Padrões" */
                wdgSignatariosPadroes.enabled = true;
            }
        }
    };
    
    this.adicionarSignatario = function(widget) {
        General.widgetNewRow(widget);
        this.controlaSignatario(widget);
    };
    
    this.controlaSignatario = function (widget){
        var campoNomePrincipal = widget.getField("NMPRINCIPALPARC");
        var campoIdTestemunha =  widget.getField("IDTIPOASSINATURA");
        var campoTipoAssinatura =  widget.getField("IDTESTEMUNHA");
        if (widget.currentRow.IDTPSIGNATARIO == 'SE') {
            campoNomePrincipal.readOnly = false;
            campoNomePrincipal.validations = {"required": true};
            campoIdTestemunha.readOnly = false;
            campoTipoAssinatura.readOnly = false;
        }
        else {
            campoNomePrincipal.readOnly = true;
            campoNomePrincipal.validations = {};
            campoTipoAssinatura.readOnly = true;
            campoIdTestemunha.readOnly = true;
            widget.currentRow.IDTIPOASSINATURA = "E";
            if (["SC", "SP"].includes(widget.currentRow.IDTPSIGNATARIO)) {
                widget.currentRow.IDTESTEMUNHA = "N";
            }
            else if (["TC", "TP"].includes(widget.currentRow.IDTPSIGNATARIO)) {
                widget.currentRow.IDTESTEMUNHA = "S";
            }
            //Limpando os campos readonly
            widget.currentRow.NRINSCRICAOPARC = widget.currentRow.NMPRINCIPALPARC = "";
        }
    };
    
    this.controlaSignatarioCadastroAutomatico = function (widget){
        var campoNomePrincipal = widget.getField("NRPARCNEGOCIO");
        var campoIdTestemunha =  widget.getField("IDTIPOASSINATURA");
        var campoTipoAssinatura =  widget.getField("IDTESTEMUNHA");
        if (widget.currentRow.IDTPSIGNATARIO == 'SE') {
            campoNomePrincipal.readOnly = false;
            campoNomePrincipal.validations = {"required": true};
            campoIdTestemunha.readOnly = false;
            campoTipoAssinatura.readOnly = false;
        }
        else {
            campoNomePrincipal.readOnly = true;
            campoNomePrincipal.validations = {};
            campoTipoAssinatura.readOnly = true;
            campoIdTestemunha.readOnly = true;
            widget.currentRow.IDTIPOASSINATURA = "E";
            if (["SC", "SP"].includes(widget.currentRow.IDTPSIGNATARIO)) {
                widget.currentRow.IDTESTEMUNHA = "N";
            }
            else if (["TC", "TP"].includes(widget.currentRow.IDTPSIGNATARIO)) {
                widget.currentRow.IDTESTEMUNHA = "S";
            }
        }
        templateManager.updateTemplate();
    };
    
    this.carregaDadosOperadores = function(widget) {
       var operadores = widget.currentRow.CDOPERADORES;
       if (operadores && operadores.length > 0) {
            widget.getField("CDOPERADORES").dataSourceFilter[2] = {
                "name": "CDOPERADOR",
                "operator": "IN",
                "value": operadores
            };
            
            widget.getField("CDOPERADORES").reload();
            widget.getField("CDOPERADORES").dataSourceFilter.splice(2);
            templateManager.updateTemplate();
        }
    };
    
    this.editarSignatarioPadrao = function(widget) {
        General.widgetEdit(widget);
        this.controlaSignatario(widget);
        this.carregaDadosOperadores(widget);
    };
    
    this.assinaturaEletronicaOnChange = function(widget) {
        var idasseletrdigital = widget.currentRow.IDASSELETRDIGITAL;
        var fieldEnviaDocOperResp = widget.getField("IDENVIADOCOPERRESPCONCASS");
        var fieldTipoDocumento = widget.getField("NMTIPODOCCERTSIGN");
        
        if (idasseletrdigital == "N") {
            fieldEnviaDocOperResp.readOnly = true;
            fieldEnviaDocOperResp.value("N");
            fieldTipoDocumento.readOnly = true;
            widget.currentRow.NRTIPODOCCERTSIGN = null;
            widget.currentRow.NMTIPODOCCERTSIGN = null;
        }
        else {
            fieldEnviaDocOperResp.readOnly = false;
            fieldTipoDocumento.readOnly = false;
        }
    };
    
    /**
     * Função para controlar os campos referente à integração do FM
     */
    this.integracaoFMOnChange = function(widget) {
        var idIntegracaoFM = widget.currentRow.IDINTEGRACAOFM;
        var idIntegraLocaisFM = widget.getField("IDINTEGRALOCAISFM");
        var idIntegraMedicaoFM = widget.getField("IDINTEGRAMEDICAOFM");
        
        if (idIntegracaoFM == "N") {
            idIntegraLocaisFM.readOnly = true;
            idIntegraLocaisFM.validations = {};
            idIntegraLocaisFM.value("N");
            idIntegraMedicaoFM.readOnly = true;
            idIntegraMedicaoFM.validations = {};
            idIntegraMedicaoFM.value("N");
        }
        else {
            idIntegraLocaisFM.readOnly = false;
            idIntegraLocaisFM.validations = { "required": true };
            idIntegraMedicaoFM.readOnly = false;
            idIntegraMedicaoFM.validations = { "required": true };
        }
    };

     /**
     * Função para controlar os campos referente à integração do Tecfood
     */
    this.integracaoTecFoodOnChange = function(widget) {
        var idIntegracaoTecFood = widget.currentRow.IDINTEGRACAOTECFOOD;
        var idTipoServicoTecFood= widget.getField("IDTIPOSERVINTTECFOOD");

        
        if (idIntegracaoTecFood == "N") {
            idTipoServicoTecFood.readOnly = true;
            idTipoServicoTecFood.validations = {};
            idTipoServicoTecFood.value(null);
        }
        else {
            idTipoServicoTecFood.readOnly = false;
            idTipoServicoTecFood.validations = { "required": true };
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00009000_Contrato_Modelo_Controller", Sm00009000_Contrato_Modelo_Controller);
});

// FILE: ./assets/js/controllers/Sm00010000_Faturamento_Apuracao_Controller.js
function Sm00010000_Faturamento_Apuracao_Controller(General, ScreenService, templateManager, ZHPromise, Sm00004000_Contrato_Service, Utilities) {
    
    var contratos_apuracao = null;
    var parceiros_apuracao = null;
    var dtInicial_apuracao = null;
    var dtFinal_apuracao = null;
    var vencimentos_apuracao = null;
    var horas_apuracao = null;
    
    /**
     * Se a parcela já estiver faturada, as ações de apurar da aba de Itens fica invisível.
     */

    // armazenamento das widgets em variaveis globais
    var getWidgetGridFaturamento = function () {
        return templateManager.container.getWidget('grid_faturamento');
    };

    this.verificarSituacao = function(wdgFaturamento) {
        var situacao             = wdgFaturamento.currentRow.IDSITUVENCPREV;
        var idapuraapontamento   = wdgFaturamento.currentRow.IDAPURAAPONTAMENTO;
        var idFaturaContr        = wdgFaturamento.currentRow.IDFATURACONTR;
        
        var actFaturar           = wdgFaturamento.getAction("faturar");
        var actCancelarApuracao  = wdgFaturamento.getAction("cancelarApuracao");
        var gerarEspelho         = wdgFaturamento.getAction("faturarHora");
        var mostraEspelho        = wdgFaturamento.getAction("relatEspelhoFat");
        var emitirNotaAntecipada = wdgFaturamento.getAction("emitirNotaAntecipada");
        
        gerarEspelho.isVisible  = (idFaturaContr == "X" && idapuraapontamento == "S" && situacao !== "C");
        mostraEspelho.isVisible = (idFaturaContr == "X" && idapuraapontamento == "S" && situacao !== "C");
        
        /* --- Faturado --- */
        if (situacao == "F") {
            actFaturar.isVisible           = false;
            actCancelarApuracao.isVisible  = true;
            emitirNotaAntecipada.isVisible = false;
        } else if (situacao == "I") {
        /* --- Faturado Parcialmente --- */
            actFaturar.isVisible = true;
            actCancelarApuracao.isVisible = true;
            emitirNotaAntecipada.isVisible = false;
        } else {
        /* --- Cancelado ou Pendente --- */
            actCancelarApuracao.isVisible = false;
            var vrvencimento          = wdgFaturamento.currentRow.VRVENCIMENTO;
            var nrvencprevcontantecip = wdgFaturamento.currentRow.NRVENCPREVCONTANTECIP;
            
            if ((idFaturaContr == "X") && (idapuraapontamento == "S")) {
                /* --- Verifica NF antecipada --- */
                if (nrvencprevcontantecip){
                    emitirNotaAntecipada.isVisible = false;
                } else {
                    emitirNotaAntecipada.isVisible = true;
                }
                
                /* --- Verifica valor do vencimento --- */
                if (vrvencimento > 0) {
                    actFaturar.isVisible = true;
                } else {
                    actFaturar.isVisible = false;
                }
            } else {
                emitirNotaAntecipada.isVisible = false;
                actFaturar.isVisible = true;
            }
        }
    };
    
    this.verificarSituacaoItem = function() {
        
        var wdgFaturamento = templateManager.container.getWidget("grid_faturamento");
        var wdgItemFatura = templateManager.container.getWidget("wdgItemFatura");
        
        if (wdgFaturamento.currentRow.IDFATURACONTR != 'X'){
            if (wdgFaturamento.currentRow.IDSITUVENCPREV == 'P') {
                
                wdgItemFatura.getAction("editItem").isVisible = true;
            }
            else{
                
                wdgItemFatura.getAction("editItem").isVisible = false;
            }
        }
        else{
           wdgItemFatura.getAction("editItem").isVisible = false; 
        }
    };
    
    this.verificarSituacaoRateio = function(wdgRateio) {
        var faturado = (wdgRateio.currentRow.IDSITUVENCRATEIO == "F");
        wdgRateio.getAction("apurarFaturamentoRateio").isVisible = !faturado;
        wdgRateio.getAction("cancelarApuracaoRateio").isVisible = faturado;
    };
    
    /**
     * Prepara os parâmetros para apurar o faturamento do rateio selecionado.
     */
    this.apurarFaturamentoRateio = function(wdgRateio) {
        var widgetGridFaturamento = getWidgetGridFaturamento();
        wdgRateio.currentRow.NRSEQCONTRATO = wdgRateio.currentRow.NRSEQCONTRATO;
        wdgRateio.currentRow.NRPARCNEGOCIO = widgetGridFaturamento.currentRow.NRPARCNEGOCIO;
        wdgRateio.currentRow.DTINICOMPET = widgetGridFaturamento.currentRow.DTINICOMPET;
        wdgRateio.currentRow.DTFINCOMPET = widgetGridFaturamento.currentRow.DTFINCOMPET;
        
        this.prepararFaturaParcela(wdgRateio);
    };
    
    this.apurarFaturamentoRateioMarcados = function(wdgRateio) {
        var rateiosMarcados = wdgRateio.getCheckedRows();
        var widgetGridFaturamento = getWidgetGridFaturamento();
        
        var todosRateiosPendentes = rateiosMarcados.every(function(row) {
            return ["P", "C"].includes(row.IDSITUVENCRATEIO);
        });
        
        if (todosRateiosPendentes) {
            var contratos   = "_" + widgetGridFaturamento.currentRow.NRSEQCONTRATO + "_";
            var parceiros   = "_" + widgetGridFaturamento.currentRow.NRPARCNEGOCIO + "_";
            var dtInicial   = widgetGridFaturamento.currentRow.DTINICOMPET;
            var dtFinal     = widgetGridFaturamento.currentRow.DTFINCOMPET;
            var vencimentos = "_" + widgetGridFaturamento.currentRow.NRVENCPREVCONT + "_";
            var rateios     = "_";
            
            rateiosMarcados.forEach(function(row) {
                rateios += row.NRRATEIOITEMVENC + "_";
            });
            
            this.faturarParcelas(contratos, parceiros, dtInicial, dtFinal, vencimentos, null, rateios).then(function(success) {
                ScreenService.showMessage(success);
                ScreenService.closePopup();
                wdgRateio.unSelectAll();
                wdgRateio.reload();
                var wdgFaturamento = getWidgetGridFaturamento();
                wdgFaturamento.reload().then(function() {
                    this.verificarSituacao(wdgFaturamento);
                }.bind(this));
            }.bind(this));
        }
        else {
            ScreenService.showMessage("Operação bloqueada.<br/>Um ou mais rateios selecionados já estão faturados.");
        }
    };
    
    this.gerarEspelhoFaturamento = function(widget) {
        ScreenService.closePopup();
        
        // Dados para geração da Parcela de Faturamento
        /*var contratos   = '_' + widget.currentRow.NRSEQCONTRATO + '_';
        var parceiros   = '_' + widget.currentRow.NRPARCNEGOCIO + '_';
        var dtInicial   = widget.currentRow.DTINICOMPET;
        var dtFinal     = widget.currentRow.DTFINCOMPET;
        var vencimentos = '_' + widget.currentRow.NRVENCPREVCONT + '_';*/
        
        var data = widget.currentRow.DTINICOMPET;
        data = data.substring(3,10);
        var params = {
            "NRSEQCONTRATO" : widget.currentRow.NRSEQCONTRATO,
            "NRPARCNEGOCIO" : widget.currentRow.NRPARCNEGOCIO,
            "NRVENCPREVCONT": widget.currentRow.NRVENCPREVCONT,
            "DTINICOMPET"   : widget.currentRow.DTINICOMPET,
            "DTFINCOMPET"   : widget.currentRow.DTFINCOMPET,
            "MESCOMPET"     : data
        };
        
        General.callRestEngine("product#/fct_vencprevcont_widget/gerarEspelhoFaturamento", "Row", params).then(function(response) {
            if (response.messages) {
                window.open(response.messages[0].message);
                widget.reload().then(function() {
                    this.verificarSituacao(widget);
                }.bind(this));
            }
            
            /*this.faturarParcelas(contratos, parceiros, dtInicial, dtFinal, vencimentos, null, null).then(function(success) {
                // CHAMAR FCT_VENCPREV_CONT PARA GERAR A PROXIMA PARCELA
                Sm00004000_Contrato_Service.geraParcelasContrato(widget.currentRow.NRSEQCONTRATO).then(function() {
                    ScreenService.closePopup();
                    ScreenService.closeSwipe();
                    ScreenService.showMessage(success);
                    var wdgFaturamento = templateManager.container.getWidget("grid_faturamento");
                    wdgFaturamento.reload();
 
                })
                .catch(function(error) {
                    console.error(error);
                    if (error.data.error.indexOf("ORA-20001") !== -1)
                        ScreenService.showMessage("O contrato está sem parametrização de faturamento.");
                    else
                        ScreenService.showMessage(error.data.error);
                });
            })
            .catch(function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });*/
            
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    /**
     * Prepara os parâmetros para faturar o vencimento selecionado.
     * 
     * @param {object} widget 
     */
    this.prepararFaturaParcela = function(widget) {
        ScreenService.closePopup();
        
        var situacao = widget.currentRow.IDSITUVENCPREV;
        var params = {};
        
        var contratos   = '_' + widget.currentRow.NRSEQCONTRATO + '_';
        var parceiros   = '_' + widget.currentRow.NRPARCNEGOCIO + '_';
        var dtInicial   = widget.currentRow.DTINICOMPET;
        var dtFinal     = widget.currentRow.DTFINCOMPET;
        var vencimentos = '_' + widget.currentRow.NRVENCPREVCONT + '_';
        var rateios = widget.currentRow.NRRATEIOITEMVENC ? '_' + widget.currentRow.NRRATEIOITEMVENC + '_' : null;
        
        if (situacao == "F") {
            ScreenService.showMessage("O vencimento já está faturado.");
        }
        else if (situacao == "A") {
            
            var idapuraapontamento = widget.currentRow.IDAPURAAPONTAMENTO;
            if (idapuraapontamento == "S") {
                
                //GERA NF
                this.faturarParcelas(contratos, parceiros, dtInicial, dtFinal, vencimentos, null, rateios).then(function(success) {
                
                    ScreenService.showMessage(success);
                    ScreenService.closeSwipe().then(function() {
                        widget.reload();
                    });
                    
                    params = {
                        "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                        "NRVENCPREVCONT": widget.currentRow.NRVENCPREVCONT
                    };
                    
                    //EXCLUI TABELAS TEMPORARIAS DO FATURAMENTO
                    General.callRestEngine("product#/fct_vencprevcont_widget/limparTabelasTemporariasFaturamento", "Row", params)
                    .then(null).catch(function(error) {
                        console.error(error);
                        ScreenService.showMessage(error.data.error);
                    });
                })
                .catch(function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
            }
            else {
                params = {
                    "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                    "DTINICOMPET"  : widget.currentRow.DTINICOMPET,
                    "DTFINCOMPET"  : widget.currentRow.DTFINCOMPET
                };
                
                General.callRestEngine("product#/fct_vencprevcont_widget/getMedidasVariaveisDaPropostaDoContrato", "Row", params).then(function(success) {
                    
                    if (success.dataset) {
                        var medidas = success.dataset.medidas;
                        
                        if (medidas.length > 0) {
                            contratos_apuracao = contratos;
                            parceiros_apuracao = parceiros;
                            dtInicial_apuracao = dtInicial;
                            dtFinal_apuracao = dtFinal;
                            vencimentos_apuracao = vencimentos;
                            horas_apuracao = null;
                            
                            medidas.map(function(row) {
                                row.TIPO = "M";
                            });
                            
                            var wdgTiposMedidaVariaveis = templateManager.container.getWidget("wdgTiposMedidaVariaveis");
                            wdgTiposMedidaVariaveis.dataSource.data = medidas;
                            
                            var wdgTiposMedidaVariaveisTabbed = templateManager.container.getWidget("wdgTiposMedidaVariaveisTabbed");
                            wdgTiposMedidaVariaveisTabbed.isVisible = true;
                            ScreenService.openSwipe(wdgTiposMedidaVariaveisTabbed);
                        }
                        else {
                            this.faturarParcelas(contratos, parceiros, dtInicial, dtFinal, vencimentos, null, rateios).then(function(success) {
                                ScreenService.showMessage(success);
                                ScreenService.closeSwipe().then(function() {
                                    widget.reload();
                                });
                            });
                        }
                    }
                }.bind(this),
                function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
            }
            
            /*params = {
                "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                "DTINICOMPET"  : widget.currentRow.DTINICOMPET,
                "DTFINCOMPET"  : widget.currentRow.DTFINCOMPET
            };
            
            General.callRestEngine("product#/fct_vencprevcont_widget/getMedidasVariaveisDaPropostaDoContrato", "Row", params).then(function(success) {
                if (success.dataset) {
                    var medidas = success.dataset.medidas;
                    var horas = success.dataset.horas;
                    
                    if (medidas.length > 0 && horas !== undefined && horas.agendadas !== null) {
                        contratos_apuracao = contratos;
                        parceiros_apuracao = parceiros;
                        dtInicial_apuracao = dtInicial;
                        dtFinal_apuracao = dtFinal;
                        vencimentos_apuracao = vencimentos;
                        horas_apuracao = horas;
                        
                        medidas.map(function(row) {
                            row.TIPO = "M";
                        });
                        
                        var datasetHoras = this.montarDatasetHoras(horas, widget);
                        
                        var wdgTiposMedidaVariaveis = templateManager.container.getWidget("wdgTiposMedidaVariaveis");
                        wdgTiposMedidaVariaveis.dataSource.data = medidas.concat(datasetHoras);
                        
                        var wdgTiposMedidaVariaveisTabbed = templateManager.container.getWidget("wdgTiposMedidaVariaveisTabbed");
                        wdgTiposMedidaVariaveisTabbed.isVisible = true;
                        ScreenService.openSwipe(wdgTiposMedidaVariaveisTabbed);
                    }
                    else if (medidas.length > 0) {
                        contratos_apuracao = contratos;
                        parceiros_apuracao = parceiros;
                        dtInicial_apuracao = dtInicial;
                        dtFinal_apuracao = dtFinal;
                        vencimentos_apuracao = vencimentos;
                        horas_apuracao = null;
                        
                        medidas.map(function(row) {
                            row.TIPO = "M";
                        });
                        
                        var wdgTiposMedidaVariaveis = templateManager.container.getWidget("wdgTiposMedidaVariaveis");
                        wdgTiposMedidaVariaveis.dataSource.data = medidas;
                        
                        var wdgTiposMedidaVariaveisTabbed = templateManager.container.getWidget("wdgTiposMedidaVariaveisTabbed");
                        wdgTiposMedidaVariaveisTabbed.isVisible = true;
                        ScreenService.openSwipe(wdgTiposMedidaVariaveisTabbed);
                    }
                    else if (horas !== undefined && horas.agendadas !== null) {
                        contratos_apuracao = contratos;
                        parceiros_apuracao = parceiros;
                        dtInicial_apuracao = dtInicial;
                        dtFinal_apuracao = dtFinal;
                        vencimentos_apuracao = vencimentos;
                        horas_apuracao = horas;
                        
                        var datasetHoras = this.montarDatasetHoras(horas, widget);
                        
                        var wdgTiposMedidaVariaveis = templateManager.container.getWidget("wdgTiposMedidaVariaveis");
                        wdgTiposMedidaVariaveis.dataSource.data = datasetHoras;
                        
                        var wdgTiposMedidaVariaveisTabbed = templateManager.container.getWidget("wdgTiposMedidaVariaveisTabbed");
                        wdgTiposMedidaVariaveisTabbed.isVisible = true;
                        ScreenService.openSwipe(wdgTiposMedidaVariaveisTabbed);
                    }
                    else {
                        this.faturarParcelas(contratos, parceiros, dtInicial, dtFinal, vencimentos, null, rateios).then(function(success) {
                            ScreenService.showMessage(success);
                            ScreenService.closeSwipe().then(function() {
                                widget.reload();
                            });
                        });
                    }
                }
            }.bind(this),
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });*/
        }
        else {
            this.faturarParcelas(contratos, parceiros, dtInicial, dtFinal, vencimentos, null, rateios).then(function(success) {
                ScreenService.closePopup();
                ScreenService.showMessage(success);
                ScreenService.closeSwipe().then(function() {
                    widget.reload();
                    if (rateios) {
                        var wdgFaturamento = templateManager.container.getWidget("grid_faturamento");
                        wdgFaturamento.reload().then(function() {
                            this.verificarSituacao(wdgFaturamento);
                        }.bind(this));
                    }
                }.bind(this));
            }.bind(this));
        }
    };
    
    /**
     * Prepara os parâmetros para faturar o(s) vencimento(s) marcado(s).
     * 
     * @param {object} widget 
     */
    this.prepararFaturaParcelasMarcadas = function(widget) {
        var arrContratos = widget.getCheckedRows();
        
        var contratos = '_';
        var parceiros = '_';
        var dtInicial = null;
        var dtFinal = null;
        var vencimentos = '_';
        
        var vencimentosPendentes = arrContratos.filter(function(row) {
            return ["P", "C", "I"].includes(row.IDSITUVENCPREV);
        });
        
        if (vencimentosPendentes.length == 0) {
            ScreenService.showMessage("Operação bloqueada.<br/>Um ou mais vencimentos selecionados já estão faturados.");
        }
        else {
            vencimentosPendentes.forEach(function(contFatura){
                
                /* -- Concatena os Contratos -- */
                if (contFatura.NRSEQCONTRATO != null){
                    contratos = contratos + contFatura.NRSEQCONTRATO + '_';
                }
                
                /* -- Concatena os Parceiros -- */
                if (contFatura.NRPARCNEGOCIO != null){
                    parceiros = parceiros + contFatura.NRPARCNEGOCIO + '_';
                }
                
                /* -- Busca menor data inicial  -- */
                if (contFatura.DTINICOMPET != null){
                    var dtIniAux =  moment(contFatura.DTINICOMPET,"DD/MM/YYYY");  
                    if (dtInicial != null){
                        
                        var diffIni = dtIniAux.diff(dtInicial);
                
                        if (diffIni < 0){
        
                            dtInicial = dtIniAux;
                            
                        }
                    } 
                    else {
                        
                        dtInicial = dtIniAux;    
                    }
                }
                
                /* -- Busca maior data final  -- */
                
                if (contFatura.DTFINCOMPET != null){
                    var dtFinAux = moment(contFatura.DTFINCOMPET,"DD/MM/YYYY");  
                    if (dtFinal != null){
                    
                        var diffFin = dtFinAux.diff(dtFinal);
                        
                        if (diffFin > 0){
        
                            dtFinal = dtFinAux;
                            
                        }
                    } 
                    else {
                        
                        dtFinal = dtFinAux;    
                    }
                }
                
                /* -- Concatena os Vencimentos -- */
                if (contFatura.NRVENCPREVCONT != null){
                    vencimentos  = vencimentos+contFatura.NRVENCPREVCONT+'_';
                }
                
            });
        
            if (contratos == '_') {
                contratos = null;
            }
            
            if (parceiros == '_') {
                parceiros = null;
            }
            
            if (vencimentos == '_') {
                vencimentos = null;
            }
            
            dtInicial = dtInicial.format("DD/MM/YYYY");
            dtFinal = dtFinal.format("DD/MM/YYYY");
            
            this.faturarParcelas(contratos, parceiros, dtInicial, dtFinal, vencimentos, null, null).then(function(success) {
                ScreenService.showMessage(success);
                ScreenService.closePopup();
                widget.unSelectAll();
                widget.reload();
            });
        }
    };
    
    /**
     * Fatura os vencimentos.
     */
    this.faturarParcelas = function(nrseqcontrato, nrparcnegocio, dataini, datafin, nrvencprevcont, nrgrupofatura, nrrateioitemvenc) {
        var deferred = ZHPromise.defer();
        
        var params = {
            "P_NRSEQCONTRATO"   : nrseqcontrato,
            "P_NRPARCNEGOCIO"   : nrparcnegocio,
            "P_DATAINI"         : dataini,
            "P_DATAFIN"         : datafin,
            "P_NRVENCPREVCONT"  : nrvencprevcont,
            "P_NRGRUPOFATURA"   : nrgrupofatura,
            "P_NRRATEIOITEMVENC": nrrateioitemvenc
        };
    
        General.callRestEngine("product#/fct_vencprevcont_widget/faturarParcelas", "Row", params).then(function() {
            
            var wdgFaturamento = templateManager.container.getWidget("grid_faturamento");
            var tipoFaturamento = wdgFaturamento.currentRow.IDFATURACONTR;
            var contrato = wdgFaturamento.currentRow.NRSEQCONTRATO;
            
            if(tipoFaturamento === 'X') {
            
                // CHAMAR FCT_VENCPREV_CONT PARA GERAR A PROXIMA PARCELA
                Sm00004000_Contrato_Service.geraParcelasContrato(contrato).then(function() {
                    ScreenService.closeSwipe().then(function() {
                        wdgFaturamento.reload();
                        deferred.resolve("Apuração de faturamento realizada com sucesso!");
                    });
                })
                .catch(function(error) {
                    console.error(error);
                    if (error.data.error.indexOf("ORA-20001") !== -1)
                        //ScreenService.showMessage("O contrato está sem parametrização de faturamento.");
                        deferred.reject("O contrato está sem parametrização de faturamento.");
                    else
                        //ScreenService.showMessage(error.data.error);
                        deferred.reject(error.data.error);
                });
            }
            else{
                
                deferred.resolve("Apuração de faturamento realizada com sucesso!");
                
            }
            
        },
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
            deferred.reject();
        });
        
        return deferred.promise;
    };
    
    /**
     * Cancela a apuração do faturamento na tela de Tipos de Meidda Variáveis.
     */
    this.cancelarApuracaoMedidasVariaveis = function(wdgTiposMedidaVariaveis) {
        wdgTiposMedidaVariaveis.list();
        ScreenService.closeSwipe().then(function() {
            var wdgFaturamento = templateManager.container.getWidget("grid_faturamento");
            wdgFaturamento.activate();
        });
    };
    
    /**
     * Coloca a widget de Tipos de Medida Variáveis em modo de edição.
     */
    this.wdgTiposMedidaVariaveisOnEnter = function(wdgTiposMedidaVariaveis) {
        wdgTiposMedidaVariaveis.inlineEdit();
        var actSalvarAlteracoes = wdgTiposMedidaVariaveis.getAction("save");
        if (actSalvarAlteracoes) actSalvarAlteracoes.isVisible = false;
    };
    
    this.gerarFaturamentoTiposMedidaVariaveis = function(wdgTiposMedidaVariaveis) {
        var medidasVariaveis = wdgTiposMedidaVariaveis.dataSource.data;
        
        var wdgFaturamento = templateManager.container.getWidget("grid_faturamento");
        var nrseqcontrato = wdgFaturamento.currentRow.NRSEQCONTRATO;
        var nrvencprevcont = wdgFaturamento.currentRow.NRVENCPREVCONT;
        
        var params = {
            "NRVENCPREVCONT"        : nrvencprevcont,
            "NRSEQCONTRATO"         : nrseqcontrato,
            "TIPOS_MEDIDA_VARIAVEIS": medidasVariaveis,
            "HORAS"                 : horas_apuracao
        };
        
        General.callRestEngine("product#/fct_vencprevcont_widget/atualizarValoresVencimento", "Row", params).then(function() {
            this.faturarParcelas(contratos_apuracao, parceiros_apuracao, dtInicial_apuracao, dtFinal_apuracao, vencimentos_apuracao, null, null).then(function(success) {
                // CHAMAR FCT_VENCPREV_CONT PARA GERAR A PROXIMA PARCELA
                Sm00004000_Contrato_Service.geraParcelasContrato(nrseqcontrato).then(function() {
                    ScreenService.showMessage(success);
                    ScreenService.closeSwipe().then(function() {
                        ScreenService.closeSwipe().then(function() {
                            var wdgFaturamento = templateManager.container.getWidget("grid_faturamento");
                            wdgFaturamento.reload();
                        });
                    });
                })
                .catch(function(error) {
                    console.error(error);
                    if (error.data.error.indexOf("ORA-20001") !== -1)
                        ScreenService.showMessage("O contrato está sem parametrização de faturamento.");
                    else
                        ScreenService.showMessage(error.data.error);
                });
            });
        }.bind(this),
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    /**
     * Prepara os parâmertos para chamar o cancelamento dos vencimentos.
     * 
     * @param {object} wdgFaturamento
     */
    this.prepararCancelarFaturamentoVencimentosMarcados = function(wdgFaturamento) {
        var contratos   = "_";
        var dtInicial   = null;
        var dtFinal     = null;
        var vencimentos = "_";
        
        var vencimentosMarcados = wdgFaturamento.getCheckedRows();
            
        var todosVencimentosFaturados = vencimentosMarcados.every(function(row) {
            return row.IDSITUVENCPREV == "F" || row.IDSITUVENCPREV == "I";
        });
        
        var existeVencimentoTipoApuracao = vencimentosMarcados.some(function(row) {
            return row.IDFATURACONTR == "X";
        });
        
        if (todosVencimentosFaturados == false) {
            ScreenService.showMessage("Operação bloqueada.<br/>Um ou mais vencimentos selecionados não estão faturados.");
        }
        else if (existeVencimentoTipoApuracao) {
            ScreenService.showMessage("Operação bloqueada.<br/>Vencimentos com tipo de faturamento \"Apuração\" são cancelados separadamente.");
        }
        else {
            vencimentosMarcados.forEach(function(rowVencimento) {
                
                // -- Concatena os Contratos --
                if (rowVencimento.NRSEQCONTRATO != null) {
                    contratos = contratos + rowVencimento.NRSEQCONTRATO + "_";
                }
                
                // -- Busca menor data inicial --
                if (rowVencimento.DTINICOMPET != null) {
                    var dtIniAux = moment(rowVencimento.DTINICOMPET, "DD/MM/YYYY");
                    if (dtInicial != null) {
                        var diffIni = dtIniAux.diff(dtInicial);
                        if (diffIni < 0) {
                            dtInicial = dtIniAux;
                        }
                    } 
                    else {
                        dtInicial = dtIniAux;
                    }
                }
                
                // -- Busca maior data final --
                if (rowVencimento.DTFINCOMPET != null) {
                    var dtFinAux = moment(rowVencimento.DTFINCOMPET,"DD/MM/YYYY");
                    if (dtFinal != null) {
                    
                        var diffFin = dtFinAux.diff(dtFinal);
                        if (diffFin > 0) {
                            dtFinal = dtFinAux;
                        }
                    }
                    else {
                        dtFinal = dtFinAux;    
                    }
                }
                
                // -- Concatena os Vencimentos --
                if (rowVencimento.NRVENCPREVCONT != null) {
                    vencimentos = vencimentos+rowVencimento.NRVENCPREVCONT + "_";
                }
            });
            
            if (contratos == "_") {
                contratos = null;
            }

            if (vencimentos == "_") {
                vencimentos = null;
            }
            
            dtInicial = dtInicial.format("DD/MM/YYYY");
            dtFinal = dtFinal.format("DD/MM/YYYY");
            
            this.cancelarFaturamentoVencimentos(contratos, dtInicial, dtFinal, vencimentos, null).then(function() {
                ScreenService.showMessage("Apuração de faturamento cancelada com sucesso!");
                ScreenService.closePopup();
                wdgFaturamento.unSelectAll();
                wdgFaturamento.reload();
            })
            .catch(function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        }
    };
    
    /**
     * Prepara os parâmetros para cancelar o faturamento do vencimento selecionado.
     */
    this.prepararCancelarFaturamentoVencimento = function(wdgFaturamento) {
        var situacao = wdgFaturamento.currentRow.IDSITUVENCPREV;
        if (situacao != "F" && situacao != "I") {
            ScreenService.showMessage("Operação bloqueada.<br/>O vencimento não está faturado.");
        }
        else {
            var nrseqcontrato = wdgFaturamento.currentRow.NRSEQCONTRATO;
            var nrvencprevcont = wdgFaturamento.currentRow.NRVENCPREVCONT;
            
            var idfaturacontr = wdgFaturamento.currentRow.IDFATURACONTR;
            if (idfaturacontr == "X") {
                
                var nrapuracao = wdgFaturamento.currentRow.NRAPURACAO;
                this.cancelarFaturamentoVencimentoTipoApuracao(nrseqcontrato, nrvencprevcont, nrapuracao);
            }
            else {
                var contratos   = "_" + nrseqcontrato + "_";
                var dtInicial   = wdgFaturamento.currentRow.DTINICOMPET;
                var dtFinal     = wdgFaturamento.currentRow.DTFINCOMPET;
                var vencimentos = "_" + nrvencprevcont + "_";
            
                this.cancelarFaturamentoVencimentos(contratos, dtInicial, dtFinal, vencimentos, null).then(function() {
                    ScreenService.showMessage("Apuração de faturamento cancelada com sucesso!");
                    ScreenService.closeSwipe().then(function() {
                        wdgFaturamento.reload();
                        this.verificarSituacao(wdgFaturamento);
                    }.bind(this));
                }.bind(this))
                .catch(function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
            }
        }
    };
    
    /**
     * Cancela a apuração do faturamento para o tipo "Apuração".
     */
    this.cancelarFaturamentoVencimentoTipoApuracao = function(nrseqcontrato, nrvencprevcont, nrapuracao) {
        var params = {
            "NRSEQCONTRATO" : nrseqcontrato,
            "NRVENCPREVCONT": nrvencprevcont,
            "NRAPURACAO"    : nrapuracao
        };
        
        General.callRestEngine("product#/fct_vencprevcont_widget/cancelarFaturamentoVencimentoTipoApuracao", "Row", params).then(function() {
            ScreenService.showMessage("Apuração de faturamento cancelada com sucesso!");
            
            var wdgFaturamento = templateManager.container.getWidget("grid_faturamento");
            ScreenService.closeSwipe().then(function() {
                ScreenService.closePopup();
                wdgFaturamento.reload();
            });
        },
        function(error) {
            console.error(error);
            var msg = error.data.error;
            if (msg.indexOf("VENCIMENTO_NAO_ULTIMO") !== -1) {
                ScreenService.showMessage("Operação bloqueada.<br/>Para o tipo de faturamento \"Apuração\", só é possível cancelar o último vencimento apurado.");
            }
            else {
                ScreenService.showMessage(msg);
            }
        });
    };
    
    /**
     * Cancela a apuração do faturamento.
     */
    this.cancelarFaturamentoVencimentos = function(nrseqcontrato, dataini, datafin, nrvencprevcont, nrrateioitemvenc) {
        var params = {
            "P_NRSEQCONTRATO"   : nrseqcontrato,
            "P_DATAINI"         : dataini,
            "P_DATAFIN"         : datafin,
            "P_NRVENCPREVCONT"  : nrvencprevcont,
            "P_NRRATEIOITEMVENC": nrrateioitemvenc
        };
        
        return General.callRestEngine("product#/fct_vencprevcont_widget/cancelarFaturamentoVencimentos", "Row", params);
    };
    
    /**
     * Prepara os parâmetros para cancelar o faturamento do rateio selecionado.
     */
    this.prepararCancelarFaturamentoRateio = function(wdgRateio) {
        var widgetGridFaturamento = getWidgetGridFaturamento();
        var nrseqcontrato    = wdgRateio.currentRow.NRSEQCONTRATO;
        var nrvencprevcont   = wdgRateio.currentRow.NRVENCPREVCONT;
        var nrrateioitemvenc = wdgRateio.currentRow.NRRATEIOITEMVENC;
        
        var contratos   = "_" + nrseqcontrato + "_";
        var dtInicial   = widgetGridFaturamento.currentRow.DTINICOMPET;
        var dtFinal     = widgetGridFaturamento.currentRow.DTFINCOMPET;
        var vencimentos = "_" + nrvencprevcont + "_";
        var rateios     = "_" + nrrateioitemvenc + "_";
    
        this.cancelarFaturamentoVencimentos(contratos, dtInicial, dtFinal, vencimentos, rateios).then(function() {
            ScreenService.showMessage("Apuração de faturamento cancelada com sucesso!");
            ScreenService.closeSwipe().then(function() {
                wdgRateio.reload();
                var wdgFaturamento = templateManager.container.getWidget("grid_faturamento");
                wdgFaturamento.reload();
            });
        })
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    this.prepararCancelarFaturamentoRateioMarcados = function(wdgRateio) {
        var widgetGridFaturamento = getWidgetGridFaturamento();
        var rateiosMarcados = wdgRateio.getCheckedRows();
        
        var todosRateiosCancelados = rateiosMarcados.every(function(row) {
            return row.IDSITUVENCRATEIO == "F";
        });
        
        if (todosRateiosCancelados) {
            var contratos   = "_" + widgetGridFaturamento.currentRow.NRSEQCONTRATO + "_";
            var dtInicial   = widgetGridFaturamentoparent.currentRow.DTINICOMPET;
            var dtFinal     = widgetGridFaturamento.currentRow.DTFINCOMPET;
            var vencimentos = "_" + widgetGridFaturamento.currentRow.NRVENCPREVCONT + "_";
            var rateios     = "_";
            
            rateiosMarcados.forEach(function(row) {
                rateios += row.NRRATEIOITEMVENC + "_";
            });
        
            this.cancelarFaturamentoVencimentos(contratos, dtInicial, dtFinal, vencimentos, rateios).then(function() {
                ScreenService.showMessage("Apuração de faturamento cancelada com sucesso!");
                ScreenService.closePopup();
                wdgRateio.unSelectAll();
                wdgRateio.reload();
                var wdgFaturamento = getWidgetGridFaturamento();
                wdgFaturamento.reload().then(function() {
                    this.verificarSituacao(wdgFaturamento);
                }.bind(this));
            }.bind(this))
            .catch(function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        }
        else {
            ScreenService.showMessage("Operação bloqueada.<br/>Um ou mais rateios selecionados não estão faturados.");
        }
    };
    
    this.montarDatasetHoras = function(horas, wdgFaturamento) {
        var nrseqcontrato = wdgFaturamento.currentRow.NRSEQCONTRATO;
        var nrorg = wdgFaturamento.currentRow.NRORG;
        var tipo = "H";
        
        var dataset = [];
        
        dataset.push({
            "NRSEQORCAMENTO"       : null,
            "NRSEQCONTRATO"        : nrseqcontrato,
            "IDELEMENTORAIZ"       : null,
            "NRRELACMEDIDAPROPOSTA": null,
            "CDTPMEDIDAVARIAVEL"   : -1,
            "DSTPMEDIDAVARIAVEL"   : "Horas Agendadas",
            "QTMEDIDAVARIAVEL"     : horas.agendadas,
            "NRORG"                : nrorg,
            "TIPO"                 : tipo
        });
        
        dataset.push({
            "NRSEQORCAMENTO"       : null,
            "NRSEQCONTRATO"        : nrseqcontrato,
            "IDELEMENTORAIZ"       : null,
            "NRRELACMEDIDAPROPOSTA": null,
            "CDTPMEDIDAVARIAVEL"   : -2,
            "DSTPMEDIDAVARIAVEL"   : "Horas Apontadas",
            "QTMEDIDAVARIAVEL"     : horas.apontadas,
            "NRORG"                : nrorg,
            "TIPO"                 : tipo
        });
        
        return dataset;
    };
    
    this.removerEdicaoGrid = function(element) {
        if (element.field && element.field.name == "QTMEDIDAVARIAVEL"){
            if (element.context.children[1]) {
                element.context.children[1].remove();
            }
        }
    };
    
    this.editItens = function(widget){
        
        widget.currentRow.IDALTUSUARIO = 'S';
        var nrvencprevcont = widget.currentRow.NRVENCPREVCONT;
        var nrseqcontrato = widget.currentRow.NRSEQCONTRATO;
        
        General.widgetSave(widget).then(function() {
            
            var params = {
                "NRVENCPREVCONT": nrvencprevcont,
                "NRSEQCONTRATO": nrseqcontrato
            };
            
            General.callRestEngine("product#/fct_vencprevcont_widget/atualizarValorParcela", "Row", params).then(function(response){
                
                ScreenService.showMessage("Valores da parcela atualizados com sucesso.");
                var wdgFaturamento = templateManager.container.getWidget("grid_faturamento");
                ScreenService.closeSwipe().then(function() {
                    wdgFaturamento.reload();
                });
                 
            })
            .catch(function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        })
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    
    /**
     * Sintetiza as funções em uma mesma função
     * 
     * @param {object} widget - Widget de Itens da Contrato ou de Adicionais
     */
    this.funcoesAjuste = function(widget) {
        this.bloquearAjustes(widget);
        this.calculaValorTotItem(widget);
        if (widget.currentRow.VRDESCONTO) {
            this.adequaAjusteAoCampo(widget);
        }

    };
    
    
    /**
     * Realiza o controle dos campos de "Desconto" de acordo 
     * com o field "Tipo de Desconto"
     * 
     * @param {object} widget widget de 
     * Itens para pegar o field "IDTPDESCONTO".
     */

    this.bloquearAjustes = function(widget) {
        var quantidade = widget.currentRow.QTITEMCONTRATO;
        var valorUnitario = widget.currentRow.VRUNITITEMCONT;
        var desconto = widget.getField('IDTPDESCONTO').value();
        var valorSemajuste = quantidade * valorUnitario;

        /* Bloqueia o valor do desconto*/
        if (desconto == 'S') {

            widget.getField('VRDESCONTO').readOnly = true;
            widget.getField('VRDESCONTO').setValue(null);
        }
        /* Aplica Max e Min Value*/
        else if (desconto == 'V') {

            widget.getField('VRDESCONTO').readOnly = false;
            widget.getField('VRDESCONTO').minValue = 0;
            widget.getField('VRDESCONTO').maxValue = valorSemajuste;
        }
        else if (desconto == 'P') {

            widget.getField('VRDESCONTO').readOnly = false;
            widget.getField('VRDESCONTO').minValue = 0;
            widget.getField('VRDESCONTO').maxValue = 100;
        }
        else if (desconto == 'X') {

            widget.getField('VRDESCONTO').readOnly = false;
            widget.getField('VRDESCONTO').minValue = 0;
            widget.getField('VRDESCONTO').maxValue = null;
        }
        else {
            widget.getField('VRDESCONTO').readOnly = false;
            widget.getField('VRDESCONTO').minValue = 0;
            widget.getField('VRDESCONTO').maxValue = null;
        }
    };
    
    
    /**
     * Realiza o cálculo do valor total do item
     * 
     * @param {object} widget widget de 
     * Itens para pegar os fields para cálculo
     */
     
    this.calculaValorTotItem = function(widget) {
        var quantidade = widget.currentRow.QTITEMCONTRATO;
        var qtdeMedida = widget.currentRow.QTMEDIDACONTR;
        var valorUnitario = widget.currentRow.VRUNITITEMCONT;
        var tipoajuste = widget.currentRow.IDTPDESCONTO;
        var ajuste = widget.currentRow.VRDESCONTO;
        
        if (qtdeMedida == null || qtdeMedida == 0){
           qtdeMedida = 1; 
        }
        
        if (quantidade) quantidade = parseFloat(quantidade.toString().replace(",", "."));
        if (qtdeMedida) qtdeMedida = parseFloat(qtdeMedida.toString().replace(",", "."));
        if (valorUnitario) valorUnitario = parseFloat(valorUnitario.toString().replace(",", "."));
        if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));

        var valorTotal = (quantidade * valorUnitario) * qtdeMedida;

        if (tipoajuste == "V" && ajuste <= valorTotal && ajuste >= 0) {
            valorTotal -= ajuste;
        }
        else if (tipoajuste == "P" && ajuste <= 100 && ajuste >= 0) {
            valorTotal -= valorTotal * ajuste / 100;
        }
        else if (tipoajuste == "X" && ajuste >= 0) {
            valorTotal += ajuste;
        }
        else if (tipoajuste == "Y" && ajuste >= 0) {
            valorTotal += valorTotal * ajuste / 100;
        }

        widget.currentRow.VRTOTITEMCONT = valorTotal;
    };
    
    
     /**
      * Previni possiveis erros com a troca sequida no select native do campo Ajuste
      * 
      * @param {object} widget - Widget de Itens da Contrato ou de Adicionais
      */
    this.adequaAjusteAoCampo = function(widget) {
        var quantidade = widget.currentRow.QTITEMCONTRATO;
        var valorUnitario = widget.currentRow.VRUNITITEMCONT;
        var tipoajuste = widget.currentRow.IDTPDESCONTO;
        var ajuste = widget.currentRow.VRDESCONTO;
        var qtdeMedida = widget.currentRow.QTMEDIDACONTR;
        
        if (qtdeMedida == null || qtdeMedida == 0){
           qtdeMedida = 1; 
        }
        
        var valorTotal = (quantidade * valorUnitario) * qtdeMedida;
        
        if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));
        
        if (tipoajuste == "P" && ajuste > 100) {
            widget.currentRow.VRDESCONTO = null;
        }
        if (tipoajuste == "V" && ajuste > valorTotal) {
            widget.currentRow.VRDESCONTO = null;
        }
    };
    
   /**
    * Realiza as validaçoes no campo conforme o tipo de ajuste selecionado.
    * 
    * @par
    * am {object} widget - Widget de Itens da Contrato ou de Adicionais
    */
    this.validarCampoPorcentagemValor = function(widget) {
        var quantidade = widget.currentRow.QTITEMCONTRATO;
        var tipoAjuste = widget.getField('IDTPDESCONTO').value();
        var ajuste = widget.getField('VRDESCONTO').value();
        var valorUnitario = widget.currentRow.VRUNITITEMCONT;
        var qtdeMedida = widget.currentRow.QTMEDIDACONTR;

        if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));
        
        
        if (qtdeMedida == null || qtdeMedida == 0){
           qtdeMedida = 1; 
        }
        
        var valorSemajuste = (quantidade * valorUnitario) * qtdeMedida;

        if (tipoAjuste == "P") {
            if (ajuste < 0) {
                ScreenService.showMessage("O valor percentual não pode ser inferior a 0 (zero)");
                widget.currentRow.VRDESCONTO = null;
            }
            else if (ajuste > 100) {
                ScreenService.showMessage("O valor percentual não pode ser superior a 100%");
                widget.currentRow.VRDESCONTO = null;
            }
        }
        else
        if (tipoAjuste == "Y") {
            if (ajuste < 0) {
                ScreenService.showMessage("O valor percentual não pode ser inferior a 0 (zero)");
                widget.currentRow.VRDESCONTO = null;
            }
        }
        else
        if (tipoAjuste == "V") {
            if (ajuste < 0) {
                ScreenService.showMessage("O valor do ajuste não pode ser inferior a 0 (zero)");
                widget.currentRow.VRDESCONTO = null;
            }
            else if (ajuste > valorSemajuste) {
                ScreenService.showMessage("O valor do ajuste não pode ser superior ao valor total");
                widget.currentRow.VRDESCONTO = null;
            }
        }
        else
        if (tipoAjuste == "X") {
            if (ajuste < 0) {
                ScreenService.showMessage("O valor do ajuste não pode ser inferior a 0 (zero)");
                widget.currentRow.VRDESCONTO = null;
            }
        }
    };
    
    
   /**
    * Rotina de Geração de Nota Antecipada.
    * 
    * Gera uma parcela do tipo "Periódico" e gera a nota fical corresponente a essa NF
    */
    this.gerarNotaAntecipada = function(widget) {
        
        var wdgFaturamento = templateManager.container.getWidget("grid_faturamento");
        
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente gerar a nota fiscal antecipada para esta parcela?"), 'question', function(){
            
            
            var params = { 
                "NRVENCPREVCONT"   : wdgFaturamento.currentRow.NRVENCPREVCONT,
                "NRSEQCONTRATO"    : wdgFaturamento.currentRow.NRSEQCONTRATO,
                "NRGRUPOFATURA"    : wdgFaturamento.currentRow.NRGRUPOFATURA,
                "IDFATURACONTR"    : wdgFaturamento.currentRow.IDFATURACONTR,
                "CDMODPGTO"        : wdgFaturamento.currentRow.CDMODPGTO,
                "NRAPURACAO"       : wdgFaturamento.currentRow.NRAPURACAO,
                "QTDIASVENCIMENTO" : wdgFaturamento.currentRow.QTDIASVENCIMENTO,
                "DTINICOMPET"      : wdgFaturamento.currentRow.DTINICOMPET,
                "DTFINCOMPET"      : wdgFaturamento.currentRow.DTFINCOMPET,
                "DTVENCIMENTO"     : wdgFaturamento.currentRow.DTVENCIMENTO,
                "VRVENCIMENTO"     : wdgFaturamento.currentRow.VRVENCIMENTO,
                "DTEMISSAO"        : wdgFaturamento.currentRow.DTEMISSAO,
                "IDALTUSUARIO"     : wdgFaturamento.currentRow.IDALTUSUARIO,
                "IDSITUVENCPREV"   : wdgFaturamento.currentRow.IDSITUVENCPREV,
                "NRAGRUPFATUR"     : wdgFaturamento.currentRow.NRAGRUPFATUR,
                "NRPARCNEGOCIO"    : wdgFaturamento.currentRow.NRPARCNEGOCIO,
                "VRFATURADO"       : wdgFaturamento.currentRow.VRFATURADO,
                "VRFALTAFATURAR"   : wdgFaturamento.currentRow.VRFALTAFATURAR,
                "VRNOTAFISCANTE"   : widget.currentRow.VRNOTAFISCANTE
                
            };
            
            General.callRestEngine("product#/fct_vencprevcont_widget/gerarNotaAntecipada", "Row", params).then(function(response){
    
                ScreenService.showMessage("Nota fiscal antecipada gerada com sucesso.");
                ScreenService.closeSwipe().then(function() {
                    ScreenService.closePopup();
                    wdgFaturamento.reload();
                });
    
            })
            .catch(function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        });
        
    };
    
    this.abrirWidgetNotaAntecipada = function() {
        var wdgNotaAntecipada = templateManager.container.getWidget("wdgNotaAntecipada");
        ScreenService.closePopup();
        ScreenService.openPopup(wdgNotaAntecipada).then(function() {
            wdgNotaAntecipada.dataSource.data = [];
            wdgNotaAntecipada.newRow();
        });
    };
    
    /**
     * gera o relatório de espelho do faturamento
     * @params{Object} widget - widget de apuração 
    **/
    this.mostrarRelatorioEspelhoFaturamento = function(widget){
        var params = {
            "NRSEQCONTRATO" : widget.currentRow.NRSEQCONTRATO,
            "NRVENCPREVCONT": widget.currentRow.NRVENCPREVCONT
        };
        
        General.callRestEngine("product#/fct_vencprevcont_widget/mostrarRelatorioEspelhoFaturamento", "Row", params).then(function (response){
            if (response.messages) {
                window.open(response.messages[0].message);
                widget.reload().then(function() {
                    this.verificarSituacao(widget);
                }.bind(this));
            }
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
}

Configuration(function(ContextRegister)
{
    ContextRegister.register("Sm00010000_Faturamento_Apuracao_Controller", Sm00010000_Faturamento_Apuracao_Controller);
});

// FILE: ./assets/js/controllers/Sm00010001_Faturamento_Medicao_Controller.js
function Sm00010001_Faturamento_Medicao_Controller(General, ScreenService) {


  /**
     * Realiza o cálculo do valor total do item
     * 
     * @param {object} widget widget de 
     * Itens para pegar os fields para cálculo
    */
    
    this.calculaValorTotItem = function(widget){
        var quantidade        = widget.currentRow.QTITEMMED;
        var qtdeMedida        = widget.currentRow.QTMEDIDACONTRMED;
        var valorUnitario     = widget.currentRow.VRUNITITEMCONTMED;
      
        if (quantidade) quantidade = parseFloat(quantidade.toString().replace(",", "."));
        if (qtdeMedida) qtdeMedida = parseFloat(qtdeMedida.toString().replace(",", "."));
        if (valorUnitario) valorUnitario = parseFloat(valorUnitario.toString().replace(",", "."));
        
        var valorTotal = valorUnitario * quantidade;
        valorTotal = valorTotal * qtdeMedida;
 
        widget.currentRow.VRTOTITEMCONTMED = valorTotal;
    };  

    /**
     * Controla os fields referentes cà cada tipo de Medição
     * 
     * @param {object} widget widget de 
     * Itens para pegar os fields para cálculo
     */    
    this.bloquearFieldsTipoMedicao = function(widget, mudarValor) {
        var tipomedicao = widget.currentRow.IDTPMEDIAPU;
        var fieldParcelaFaturamento = widget.getField("NMVENCPREVCONT");
        var fieldGrupoFaturamento = widget.getField("NMGRUPOFATURA");
        var fieldProdutoServico = widget.getField("NMPRODSERV");
        
        // Prevista
        if (tipomedicao == 'P') {
            // Parcela de Faturamento
            fieldParcelaFaturamento.readOnly = false;
            fieldParcelaFaturamento.validations = {"required": true};
            
            // Grupo de Faturamento
            fieldGrupoFaturamento.readOnly = true;
            fieldGrupoFaturamento.validations = {};
            
            // Produto/Serviço
            fieldProdutoServico.readOnly = true;
            fieldProdutoServico.validations = {};
            
            if (mudarValor == true) {
                fieldGrupoFaturamento.value(null);
                fieldProdutoServico.value(null);
                this.setCodigoProdutoServico(widget);
            }
        } 
        // Extra
        else {
            // Parcela de Faturamento
            fieldParcelaFaturamento.readOnly = true;
            fieldParcelaFaturamento.validations = {};
            
            // Grupo de Faturamento
            fieldGrupoFaturamento.readOnly = false;
            fieldGrupoFaturamento.validations = {"required": true};
            
            // Produto/Serviço
            fieldProdutoServico.readOnly = false;
            fieldProdutoServico.validations = {"required": true};
            
            if (mudarValor == true) {
                fieldParcelaFaturamento.value(null);
                
                // Inicio Competencia
                widget.currentRow.DTINICOMPET = null;
                
                // Final Competencia
                widget.currentRow.DTFINCOMPET = null;
                
                // Vencimento
                widget.currentRow.DTVENCIMENTO = null;
                
                // Valor
                widget.currentRow.VRTOTITEMCONTMED_NI = null;
            }
        }
    };
    
    this.inserefiltroVencimento = function(widget){
        
        widget.getField('NMVENCPREVCONT').dataSourceFilter[0].value = widget.currentRow.NRSEQCONTRATO;
        widget.getField('NMVENCPREVCONT').reload();
        
        /*var tipomedicao = widget.currentRow.IDTPMEDIAPU;
        
        //  Prevista
        if (tipomedicao == 'P') {
            // Parcela de Faturamento
            widget.getField('NMVENCPREVCONT').dataSourceFilter[0].value = widget.currentRow.NRSEQCONTRATO;
            widget.getField('NMVENCPREVCONT').reload();
        } 
        // Extra
        else {
            // Parcela de Faturamento
            widget.getField('NMVENCPREVCONT').dataSourceFilter[0].value = null;
            widget.getField('NMVENCPREVCONT').reload();
        }*/
    };

    /**
     * Controla os fields referentes cà cada tipo de Medição
     * 
     * @param {object} widget widget de 
     * Itens para pegar os fields para cálculo
     */    
    this.bloquearFieldsMedicaoItem = function(widget){
        
        var itemContrato = widget.currentRow.NRITEMOCONTRATO;
        
        if (itemContrato) {
            
            // Valor
            widget.getField('VRTOTITEMCONTMED_NI').readOnly = true;
            widget.getField('VRTOTITEMCONTMED_NI').value(null);
            widget.getField('VRTOTITEMCONTMED_NI').validations = {};
            
            // Quantidade
            widget.getField('QTITEMMED').readOnly = false;
            widget.getField('QTITEMMED').validations = {"required": true};
            widget.getField('QTITEMMED').value(widget.currentRow.QTITEMCONTRATO); 
            
            // Valor Unitário
            widget.getField('VRUNITITEMCONTMED').readOnly = false;
            widget.getField('VRUNITITEMCONTMED').validations = {"required": true};
            widget.getField('VRUNITITEMCONTMED').value(widget.currentRow.VRUNITITEMCONT);
            
            // Medida
            widget.getField('DSTPMEDIDASERVMED').readOnly = false;
            widget.getField('DSTPMEDIDASERVMED').validations = {"required": true};
            widget.getField('CDTPMEDIDASERVMED').value(widget.currentRow.CDTPMEDIDASERV);
            widget.getField('DSTPMEDIDASERVMED').value(widget.currentRow.DSTPMEDIDASERV);
            
            // Qt. Medida
            widget.getField('QTMEDIDACONTRMED').readOnly = false;
            widget.getField('QTMEDIDACONTRMED').validations = {"required": true};
            widget.getField('QTMEDIDACONTRMED').value(widget.currentRow.QTMEDIDACONTR);
            
            // Valor Total
            //widget.getField('VRTOTITEMCONTMED').readOnly = false;
            widget.getField('VRTOTITEMCONTMED').validations = {"required": true};
            widget.getField('VRTOTITEMCONTMED').value(widget.currentRow.VRTOTITEMCONT);
          
        } 
        else {
  
            // Valor
            widget.getField('VRTOTITEMCONTMED_NI').readOnly = false;
            widget.getField('VRTOTITEMCONTMED_NI').value(null);
            widget.getField('VRTOTITEMCONTMED_NI').validations = {"required": true};
            
            // Quantidade
            widget.getField('QTITEMMED').readOnly = true;
            widget.getField('QTITEMMED').value(null);
            widget.getField('QTITEMMED').validations = {};
            
            // Valor Unitário
            widget.getField('VRUNITITEMCONTMED').readOnly = true;
            widget.getField('VRUNITITEMCONTMED').value(null);
            widget.getField('VRUNITITEMCONTMED').validations = {};
            
            // Medida
            widget.getField('DSTPMEDIDASERVMED').readOnly = true;
            widget.getField('DSTPMEDIDASERVMED').value(null);
            widget.getField('DSTPMEDIDASERVMED').validations = {};
            
            // Qt. Medida
            widget.getField('QTMEDIDACONTRMED').readOnly = true;
            widget.getField('QTMEDIDACONTRMED').value(null);
            widget.getField('QTMEDIDACONTRMED').validations = {};
            
            // Valor Total
            //widget.getField('VRTOTITEMCONTMED').readOnly = true;
            widget.getField('VRTOTITEMCONTMED').value(null);
            widget.getField('VRTOTITEMCONTMED').validations = {};
        }
    };
    
    this.setCodigoProdutoServico = function(widget) {
        var tipo   = widget.currentRow.IDPRODSERV;
        var codigo = widget.currentRow.CDPRODSERV;
        
        if (codigo) {
            if (tipo == "PRODUTO") {
                widget.currentRow.CDPRODUTO = codigo;
                widget.currentRow.CDSERVORCA = null;
            }
            else {
                widget.currentRow.CDPRODUTO = null;
                widget.currentRow.CDSERVORCA = codigo;
            }
        }
        else {
            widget.currentRow.CDPRODUTO = null;
            widget.currentRow.CDSERVORCA = null;
        }
    };
    
    this.inserefiltroitem = function(widget){
        widget.getField('NMITEMCONTRATO').dataSourceFilter[0].value = widget.currentRow.NRSEQCONTRATO;
        widget.getField('NMITEMCONTRATO').reload();
    };
    
    this.salvarMedicao = function(widget) {
        widget.currentRow.__columns_to_return = ["NRMEDICAOAPU", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        var valor = widget.currentRow.VRTOTITEMCONTMED_NI;
        if (valor) {
            widget.currentRow.VRTOTITEMCONTMED = valor;
        }
        
        General.widgetSave(widget).then(function(success) {
            if (success.dataset && success.dataset.columnsToReturn[0]) {
                widget.currentRow.NRMEDICAOAPU   = success.dataset.columnsToReturn[0].NRMEDICAOAPU;
                widget.currentRow.NRORG          = success.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO     = success.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO  = success.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
        },
        function(error) {
            console.error(error);
            if (error.data)
                ScreenService.showMessage(error.data.error);
            else
                ScreenService.showMessage(error);
        });
    };
}

Configuration(function(ContextRegister)
{
    ContextRegister.register("Sm00010001_Faturamento_Medicao_Controller", Sm00010001_Faturamento_Medicao_Controller);
});

// FILE: ./assets/js/controllers/Sm00011000_Apuracao_Faturamento_Controller.js
function Sm00011000_Apuracao_Faturamento_Controller(General, ScreenService, templateManager, ZHPromise, Utilities, localStorageEngine) {
    
    var self = this;

    /* A variável global "bloqReajuste" será utilziada para bloquear as ações da tela caso existam contratos de manutenção 
       ativos sem parcela gerada. No caso, necessitando de reajuste. O valor dela é atribuída na função "validaReajusteContr" */
    var bloqReajuste = 'N';
    
    /*
      Faz o controle da visibilidade das actions da tela
    */
    this.controlaVisibilidadeAct = function(widget) {
        var situacao       = widget.currentRow.IDSITUACAO;
        var dstipoapuracao = widget.currentRow.DSTIPOAPURACAO;
        var vrVencimento   = widget.currentRow.VRVENCIMENTO;
        var nrparcelasapurnegcontrato = widget.currentRow.NRPARCELASAPURNEGCONTRATO;
        var nrparcelasnegcontr = widget.currentRow.NRPARCELASNEGCONTR

        var actionApura     = widget.getAction('apurar_parcela');
        var actionCancela   = widget.getAction('cancelar_apuracao');
        var actionDesmembra = widget.getAction('desmembar_parcela');
        var actionFatManual = widget.getAction('faturamento_manual');
        var actionBloqueia  = widget.getAction('nao_faturar');
        var actionCancelarFaturamento = widget.getAction('cancelar_faturamento');
        var actionNegociarApura = widget.getAction('negociar_apura_faturamento');
        var actionVisuNegApura = widget.getAction('visualizar_negociacao');
        var actionExcluirNegApura  = widget.getAction('excluir_negociacao');
        
        /* Bloqueio de Reajuste */ 
        if (bloqReajuste == 'S') {
            actionApura.isVisible     = false;
            actionCancela.isVisible   = false;
            actionDesmembra.isVisible = false;
            actionFatManual.isVisible = false;
            actionBloqueia.isVisible  = false;
            actionCancelarFaturamento.isVisible = false;
            actionNegociarApura.isVisible = false;
            actionVisuNegApura.isVisible  = false;
            actionExcluirNegApura.isVisible = false;
        }
        else {
            /* Situacao Pendente */
            if (situacao == '1') {
                actionApura.isVisible     = true;
                actionCancela.isVisible   = false;
                actionDesmembra.isVisible = true;
                actionFatManual.isVisible = true;
                actionBloqueia.isVisible  = true;
                actionCancelarFaturamento.isVisible = false;

                if (nrparcelasapurnegcontrato != null){
                    actionNegociarApura.isVisible = false;
                    actionVisuNegApura.isVisible  = true;
                    actionExcluirNegApura.isVisible = true;
                }
                else {
                    if (nrparcelasnegcontr != null){
                        actionNegociarApura.isVisible = false;
                    }
                    else {
                        actionNegociarApura.isVisible = true;
                    }
                    actionVisuNegApura.isVisible  = false;
                    actionExcluirNegApura.isVisible = false;
                }

            }
            /* Parcialmente apurada */
            else if (situacao == '6'){
                actionApura.isVisible     = (dstipoapuracao == 'APU');
                actionDesmembra.isVisible = false;
                actionCancela.isVisible   = true;
                actionFatManual.isVisible = false;
                actionBloqueia.isVisible  = false;
                actionCancelarFaturamento.isVisible = false;

                if (nrparcelasapurnegcontrato != null){
                    actionNegociarApura.isVisible = false;
                    actionVisuNegApura.isVisible  = true;
                    actionExcluirNegApura.isVisible = false;
                }
                else {
                    actionNegociarApura.isVisible = false;
                    actionVisuNegApura.isVisible  = false;
                    actionExcluirNegApura.isVisible = false;
                }
            }
            /* Apurado */
            else if (situacao == '2') {
                actionApura.isVisible     = false;
                actionCancela.isVisible   = (dstipoapuracao == 'APU');
                actionDesmembra.isVisible = false;
                actionFatManual.isVisible = false;
                actionBloqueia.isVisible  = false;
                actionCancelarFaturamento.isVisible = false;

                if (nrparcelasapurnegcontrato != null){
                    actionNegociarApura.isVisible = false;
                    actionVisuNegApura.isVisible  = true;
                    actionExcluirNegApura.isVisible = false;
                }
                else {
                    actionNegociarApura.isVisible = false;
                    actionVisuNegApura.isVisible  = false;
                    actionExcluirNegApura.isVisible = false;
                }
            }
            /* Faturado com valor zero */
            else if (situacao == '3' && vrVencimento == 0) {
                actionApura.isVisible     = false;
                actionCancela.isVisible   = false;
                actionDesmembra.isVisible = false;
                actionFatManual.isVisible = false;
                actionBloqueia.isVisible  = false;
                actionCancelarFaturamento.isVisible = true;

                if (nrparcelasapurnegcontrato != null){
                    actionNegociarApura.isVisible = false;
                    actionVisuNegApura.isVisible  = true;
                    actionExcluirNegApura.isVisible = false;
                }
                else {
                    actionNegociarApura.isVisible = false;
                    actionVisuNegApura.isVisible  = false;
                    actionExcluirNegApura.isVisible = false;
                }
            }
            /* Faturado (3), Anulado (4) ou Paralisado (5) */
            else {
                actionApura.isVisible     = false;
                actionCancela.isVisible   = false;
                actionDesmembra.isVisible = false;
                actionFatManual.isVisible = false;
                actionBloqueia.isVisible  = false;
                actionCancelarFaturamento.isVisible = false;

                if (nrparcelasapurnegcontrato != null){
                    actionNegociarApura.isVisible = false;
                    actionVisuNegApura.isVisible  = true;
                    actionExcluirNegApura.isVisible = false;
                }
                else {
                    actionNegociarApura.isVisible = false;
                    actionVisuNegApura.isVisible  = false;
                    actionExcluirNegApura.isVisible = false;
                }
            }
        }
        
        templateManager.container.getWidget("wdgDadosAgrupamento").isVisible = (widget.currentRow.IDPARCAGRUP == "S");
        templateManager.container.getWidget("RateioItFaturamento").isVisible = (widget.currentRow.IDGERARATEIO == "S" || widget.currentRow.TOTALDERATEIOS > 0);
        templateManager.container.getWidget("Rateiofaturamento").isVisible   = (widget.currentRow.IDGERARATEIO == "S" || widget.currentRow.TOTALDERATEIOS > 0);
        actionDesmembra = (widget.currentRow.IDPARCAGRUP == "N");
    };

    /*
        Task Descrição NF no Contrato
    */
    this.abrirWdgEditarDescNf = function () {
        ScreenService.closePopup();
        var wdgEditarDescNf = templateManager.container.getWidget("wdgEditarDescNf");
        wdgEditarDescNf.dataSource.data = [];
        ScreenService.openPopup(wdgEditarDescNf);

        var wdgApuracaoFaturamento = templateManager.container.getWidget("wdgApuracaoFaturamento");
        wdgEditarDescNf.currentRow.NRSEQCONTRATO = wdgApuracaoFaturamento.currentRow.NRSEQCONTRATO;
        wdgEditarDescNf.currentRow.DSSERVORCANF = wdgApuracaoFaturamento.currentRow.DSSERVORCANF;
        wdgEditarDescNf.currentRow.DSSERVORCABOL = wdgApuracaoFaturamento.currentRow.DSSERVORCABOL;        
    };

    this.salvarWdgEditarDescNf = function(widget) {
        var params = {
            "NRSEQCONTRATO"   : widget.currentRow.NRSEQCONTRATO,
            "DSSERVORCANF"    : widget.currentRow.DSSERVORCANF,
            "DSSERVORCABOL"   : widget.currentRow.DSSERVORCABOL
        };

       General.callRestEngine("product#/fct_vencprevcont_widget/salvarWdgEditarDescNf", "Row", params).then(function() {
            ScreenService.showMessage("Descrição da Nota/Boleto atualizados com sucesso!");
            ScreenService.closePopup();
            var wdgApuracao = templateManager.container.getWidget("wdgApuracaoFaturamento");
            wdgApuracao.reload();
        },
        function(error) {
            Utilities.showErrorMessage(error);
        });

    };


    /*
      Abrir a widget de anulação / cancelamento de parcela  
    */
    this.abrirWdgAnularApuracao = function() {
        ScreenService.closePopup();
        var wdgAnularParcela = templateManager.container.getWidget("wdgAnularParcela");
        wdgAnularParcela.dataSource.data = [];
        ScreenService.openPopup(wdgAnularParcela);
    };

    /*
      anulação de parcela
    */
    this.anularApuracao = function(wdgAnularParcela) {
        if (wdgAnularParcela.isValid()) {

            var hora = moment().format("DD/MM/YYYY HH:mm:ss");
            var justificativa = hora+" -> A parcela foi anulada pelo seguinte motivo: " + wdgAnularParcela.currentRow.DSJUSTIFICATIVA;
            
            var wdgApuracao = templateManager.container.getWidget("wdgApuracaoFaturamento");
            
            var nrfaturacont = wdgApuracao.currentRow.NRFATURACONT;
            var nrseqcontrato = wdgApuracao.currentRow.NRSEQCONTRATO;
            var nrjustifcancelaapur = wdgAnularParcela.currentRow.NRJUSTIFCANCELAAPUR;
            
            var params = {
                "DSJUSTIFICATIVA"    : justificativa,
                "NRFATURACONT"       : nrfaturacont,
                "NRSEQCONTRATO"      : nrseqcontrato,
                "NRJUSTIFCANCELAAPUR": nrjustifcancelaapur
            };
        
            General.callRestEngine("product#/fct_vencprevcont_widget/anularApuracao", "Row", params).then(function() {
                //ScreenService.showMessage("Apuração de faturamento realizada com sucesso!");
                ScreenService.showMessage("Apuração de faturamento anulada com sucesso!");
                ScreenService.closePopup();
                wdgApuracao.reload();
            },
            function(error) {
                Utilities.showErrorMessage(error);
            });
            
        }
    };
    
    /*
      Salvar a anulação da parcela
    */
    this.salvarwdgAnularParcela = function(wdgAnularParcela) {
        
        ScreenService.confirmMessage("Deseja realmente Anular a apuração da parcela?", "question", function() {
            this.anularApuracao(wdgAnularParcela);
        }.bind(this), null);
    };

    /*
      Abrir a Widget de Faturamento manual
    */
    this.abrirWdgFatuManual = function() {
        ScreenService.closePopup();
        var wdgFatuManual = templateManager.container.getWidget("wdgFatuManual");
        wdgFatuManual.dataSource.data = [];
        ScreenService.openPopup(wdgFatuManual);
        wdgFatuManual.newRow();
         /*preenche o filtro dos itens a serem desmembrados*/
        var wdgApuracao = templateManager.container.getWidget("wdgApuracaoFaturamento");
        
        wdgFatuManual.fields[0].dataSourceFilter[0].value = wdgApuracao.currentRow.NRORG;
        wdgFatuManual.fields[0].dataSourceFilter[1].value = wdgApuracao.currentRow.NRPARCFATURA;
        wdgFatuManual.fields[0].reload();
    };

    var invoiceValidateHasAlreadyBeenUsed = function(nrLanctoNF,cdFilial) {
        var params = {
            NRLANCTONF: nrLanctoNF,
            CDFILIAL: cdFilial,
        };
        return General.callRestEngine("/invoiceValidateHasAlreadyBeenUsed", "Row", params).then(function (response) {
            return response.dataset.invoiceValidateHasAlreadyBeenUsed;
        });
    };
    
    /*
     Faturamento Manual de parcela
    */
    this.FatuManual = function(wdgFatuManual) {
        if (wdgFatuManual.isValid()) {

            var nrLanctoNF = wdgFatuManual.currentRow.NRLANCTONF;
            var cdFilial = wdgFatuManual.currentRow.CDFILIAL;

            invoiceValidateHasAlreadyBeenUsed(nrLanctoNF,cdFilial).then(function (validate) {
                var message = "Deseja realmente vincular um faturamento manual a parcela?";
                if(validate.length > 0){
                    var messageValidate = 'Atenção!<BR>Essa nota fiscal já esta vinculada a outra(s) parcela(s) :<BR><BR>';
                    validate.forEach(function (faturamento) {
                        messageValidate += faturamento.MESSAGE+'<BR>';
                    });
                    message = messageValidate + '<BR>' + message;
                }
                ScreenService.confirmMessage(message, "question", function() {

                    var wdgApuracao = templateManager.container.getWidget("wdgApuracaoFaturamento");

                    var nrfaturacont = wdgApuracao.currentRow.NRFATURACONT;
                    var nrseqcontrato = wdgApuracao.currentRow.NRSEQCONTRATO;
                    var nrParcFatura = wdgApuracao.currentRow.NRPARCFATURA;
                    var vrVencimento = wdgApuracao.currentRow.VRVENCIMENTO;

                    var hora = moment().format("DD/MM/YYYY HH:mm:ss");
                    var justificativa = hora+" -> A parcela foi vinculada a um faturamento manual pelo seguinte motivo: " + wdgFatuManual.currentRow.DSJUSTIFICATIVA;

                    var cdEmpresa = wdgFatuManual.currentRow.CDEMPRESA;
                    var vrNotaFisc = wdgFatuManual.currentRow.VRNOTAFISC;


                    var params = {
                        "NRFATURACONT"    : nrfaturacont,
                        "NRSEQCONTRATO"   : nrseqcontrato,
                        "NRPARCFATURA"    : nrParcFatura,
                        "VRVENCIMENTO"    : vrVencimento,
                        "DSJUSTIFICATIVA" : justificativa,
                        "NRLANCTONF"      : nrLanctoNF,
                        "CDFILIAL"        : cdFilial,
                        "CDEMPRESA"       : cdEmpresa,
                        "VRNOTAFISC"      : vrNotaFisc
                    };

                    General.callRestEngine("product#/fct_vencprevcont_widget/FatuManual", "Row", params).then(function() {
                            //ScreenService.showMessage("Apuração de faturamento realizada com sucesso!");
                            ScreenService.showMessage("Apuração de faturamento manual realizada com sucesso!");
                            ScreenService.closePopup();
                            wdgApuracao.reload();
                        },
                        function(error) {
                            Utilities.showErrorMessage(error);
                        });

                }, null);
            });


        }
    };
    
    /*
     Abrir a Widget de Desmembramento de parcela
    */
    this.abrirwdgDesmembrar = function() {
        ScreenService.closePopup();
        var wdgApuracao = templateManager.container.getWidget("wdgApuracaoFaturamento");
        
        var nrFaturaCont = wdgApuracao.currentRow.NRFATURACONT;
        var nrSeqContrato = wdgApuracao.currentRow.NRSEQCONTRATO;
        
        var params = {
            "NRFATURACONT"  : nrFaturaCont,
            "NRSEQCONTRATO" : nrSeqContrato     
        };

        General.callRestEngine("product#/fct_vencprevcont_widget/validaQtItens", "Row", params).then(function(response) {
            var valida = response.dataset.VALIDA.QUANTIDADE;
            
            if (valida > 1){
            
                var wdgDesmembrar = templateManager.container.getWidget("wdgDesmembrar");
                wdgDesmembrar.dataSource.data = [];
                ScreenService.openPopup(wdgDesmembrar);
                wdgDesmembrar.newRow();
                
                /*preenche o filtro dos itens a serem desmembrados*/
                
                wdgDesmembrar.fields[0].dataSourceFilter[0].value = wdgApuracao.currentRow.NRORG;
                wdgDesmembrar.fields[0].dataSourceFilter[1].value = nrFaturaCont;
                wdgDesmembrar.fields[0].reload();
            }
            else{
                
                ScreenService.closePopup();
                ScreenService.showMessage("Operação Bloqueada. <br/>Só é permitido desmembrar parcelas que possuem dois ou mais itens.");
            }
        },
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };

    /*
     Salvar o desmembramento da parcela
    */
    this.saveDesmebr = function(widget) {
        if (widget.isValid()) {
            ScreenService.confirmMessage("Deseja realmente desmembrar a parcela?", "question", function() {
                var itensDesmembr = widget.currentRow.ITDESMEMBRAMENTO;
                var formaDesmembr = widget.currentRow.IDFORMADESMEMBAMENTO;
                var hora = moment().format("DD/MM/YYYY HH:mm:ss");
                var justificativa = hora + " -> A parcela foi desmembrada pelo seguinte motivo: " + widget.currentRow.DSJUSTIFICATIVA;
                
                var wdgApuracao = templateManager.container.getWidget("wdgApuracaoFaturamento");
            
                var nrfaturacont = wdgApuracao.currentRow.NRFATURACONT;
                var nrseqcontrato = wdgApuracao.currentRow.NRSEQCONTRATO;
    
                
                var params = {
                    "NRFATURACONT"         : nrfaturacont,
                    "NRSEQCONTRATO"        : nrseqcontrato,
                    "ITDESMEMBRAMENTO"     : itensDesmembr,
                    "IDFORMADESMEMBAMENTO" : formaDesmembr,
                    "DSJUSTIFICATIVA"      : justificativa      
                };
            
                General.callRestEngine("product#/fct_vencprevcont_widget/desmembrarParcela", "Row", params).then(function() {
                    //ScreenService.showMessage("Apuração de faturamento realizada com sucesso!");
                    ScreenService.showMessage("Desmembramento de parcela realizado com sucesso!");
                    ScreenService.closePopup();
                    ScreenService.closeSwipe().then(function() {
                        wdgApuracao.reload();
                    });
                },
                function(error) {
                    Utilities.showErrorMessage(error);
                });
            }, null);
        }
    };

    /*----------------------------------
     | --- Procedure Faturamento   --- |
    -----------------------------------*/
    this.validarAgrupamento = function(widget) {
        var params = {
            "NRSEQCONTRATO" : '_'+widget.currentRow.NRSEQCONTRATO+'_',
            "NRPARCFATURA"  : widget.currentRow.NRPARCFATURA,
            "NRCONTRATOFCT" : widget.currentRow.NRCONTRATOFCT,
            "NRFATURACONT"  : widget.currentRow.NRFATURACONT,
            "DTINICIO"      : widget.currentRow.DTINICOMPET,
            "DTFIM"         : widget.currentRow.DTFINCOMPET
        };

        General.callRestEngine("product#/fct_vencprevcont_widget/validarAgrupamentoFat", "Row", params).then(function(response) {
            if (!response.dataset || response.dataset.dados.length == 0) {
                self.prepararApurarParcelas(widget);
            } else {
                var contratos = "";
                var agrupamento = response.dataset.dados;
                agrupamento.forEach(function(contrato) {
                    contratos += "- "+contrato.NRCONTRATOFCT+"<br>";
                });
                
                ScreenService.showMessage("Operação bloqueada!<br>Não é possível apurar parcelas ligadas a um agrupamento sem que todos os seus contratos estejam aprovados e suas medições referentes a data de competência sejam realizadas.<br><br>Devem ser verificados os seguintes contratos: <br><br>" + contratos);
            }
        }).catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    
    
    this.validarAgrupamentoMarcados = function(widget) {
        var vencimentosMarcados = widget.getCheckedRows();
        var params = {
            "VENCIMENTOS": vencimentosMarcados
        };
            
        General.callRestEngine("product#/fct_vencprevcont_widget/validarAgrupamentoFatMarcados", "Row", params).then(function(response) {
            
            var resultado = response.dataset.retorno.dados;
            if (resultado == "OK") {
                this.prepararApurarParcelasMarcadas(widget);
            }
            else { 
                ScreenService.showMessage("Operação bloqueada!<br>Não é possível apurar parcelas ligadas a um agrupamento sem que todos os seus contratos estejam aprovados e suas medições referentes a data de competência sejam realizadas.<br><br>Devem ser verificados os seguintes contratos: <br><br>" + resultado);
            }
              
        }.bind(this));
    };
    
    
     /**
     * Prepara os parâmetros para faturar o vencimento selecionado.
     * 
     * @param {object} widget 
     * @param {String} tipo de apuracao - o que será apurado (aceita as str = PARCELA, RATEIO, ITEM)
     */
    this.prepararApurarParcelas = function(widget, tipoApuracao) {
        if (tipoApuracao === undefined) {
            tipoApuracao = 'PARCELA';
        }

        try{
            if(widget.currentRow.VRVENCIMENTO == 0) {
                ScreenService.confirmMessage("Operação bloqueada!<br><br>Não é possível apurar parcelas com valor zerado.<br><br>Deseja alterar a situação desta parcela para \'Faturada\'?", "question", function() {
                    widget.currentRow.IDSITUACAO = '3';
                    General.callRestEngine("product#/fct_faturacont_widget/alterarStatusParcelasZeradas", "Row", widget.currentRow).then(function(response){
                        ScreenService.closePopup();
                        ScreenService.showMessage("Operação realizada com sucesso!");
                        this.controlaVisibilidadeAct(widget);
                    }.bind(this));
                }.bind(this));
            } else {
                tipoApuracao = tipoApuracao.toUpperCase();
                ScreenService.confirmMessage(getMensagem().APURACAO.CONFIRMACAO[tipoApuracao], "question", function() {
                    ScreenService.closePopup();
                    var wdgParcela = templateManager.container.getWidget("wdgApuracaoFaturamento"); 
                    
                    var nritfaturacont = null;
                    var rateios        = null;
                    var situacao       = widget.currentRow.IDSITUACAO;
                    var vencimentos    = '_' + widget.currentRow.NRFATURACONT + '_';
                    
                    if (tipoApuracao == 'ITEM') {
                        nritfaturacont = '_' + widget.currentRow.NRITFATURACONT + '_';
                    } else if (tipoApuracao == 'RATEIO') {
                        situacao = widget.currentRow.IDSITUVENCRATEIO;
                        rateios  = '_' + widget.currentRow.NRRATEIOITEMFAT + '_';
                    }
                    
                    if (situacao != "1") {
                        ScreenService.showMessage(getMensagem().APURACAO.VALIDACAO.PARCNAOPENDENTE[tipoApuracao]);
                    } else {
                        this.gerarFaturamentoApuracao(vencimentos, rateios, nritfaturacont, false).then(function(success) {
                            ScreenService.closePopup();
                            ScreenService.showMessage(success);
                            ScreenService.closeSwipe().then(function() {
                                widget.reload();
                                if (rateios || nritfaturacont) {
                                    wdgParcela.reload().then(function() {
                                        this.controlaVisibilidadeAct(wdgParcela);
                                    }.bind(this));
                                }
                            }.bind(this));
                        }.bind(this));
                    }
                }.bind(this));
            }
        }catch(err){
            ScreenService.showMessage('Operação bloqueada. <br><br>' + err);
        }
    };
  
    /**
     * Prepara os parâmetros para faturar o(s) vencimento(s) marcado(s).
     * 
     * @param {object} widget 
     * @param {String} tipo de apuracao - o que será apurado (aceita as str = PARCELASMARCADAS, RATEIOSMARCADOS)
     */
    this.prepararApurarParcelasMarcadas = function(widget, tipoApuracao) {
        if (tipoApuracao === undefined) {
            tipoApuracao = 'PARCELASMARCADAS';
        }

        tipoApuracao            = tipoApuracao.toUpperCase();
        var vencimentosMarcados = widget.getCheckedRows();
        var wdgParcela          = templateManager.container.getWidget("wdgApuracaoFaturamento"); 
        
        var vencimentos = '_';
        var rateios     = '_';
        
        vencimentosMarcados.forEach(function(contFatura) {
            if(tipoApuracao == 'RATEIOSMARCADOS') {
                if (vencimentos == '_')
                    vencimentos = "_" + contFatura.NRFATURACONT + "_";
                
                if (contFatura.NRRATEIOITEMFAT != null)
                    rateios = rateios + contFatura.NRRATEIOITEMFAT + '_';
            }
            else {    
                /* -- Concatena os Vencimentos -- */
                if (contFatura.NRFATURACONT != null)
                    vencimentos  = vencimentos + contFatura.NRFATURACONT + '_';
            }
        });
    
        if (rateios == '_') rateios = null;
        if (vencimentos == '_') vencimentos = null;
        
        this.gerarFaturamentoApuracao(vencimentos, rateios, null, true).then(function(success) {
            ScreenService.showMessage(success);
            ScreenService.closePopup();
            widget.unSelectAll();
            widget.reload();
            
            if(tipoApuracao == 'PARCELASMARCADAS') {
                this.controlaVisibilidadeAct(widget);
            } else {
                wdgParcela.reload().then(function() {
                    this.controlaVisibilidadeAct(wdgParcela);
                }.bind(this));
            }
        }.bind(this));

    };
    /**
     * Fatura os vencimentos.
     */
    this.gerarFaturamentoApuracao = function(nrfaturacont, nrrateioitfatura, nritfaturacont, multiplosregistros) {
        var deferred = ZHPromise.defer();
        var params = {
            "P_NRFATURACONT"    : nrfaturacont,
            "P_NRRATEIOITEMFAT" : nrrateioitfatura,
            "P_NRITFATURACONT"  : nritfaturacont
        };
    
        Utilities.callRestEngineWithTimeout("product#/fct_vencprevcont_widget/gerarFaturamentoApuracao", "Row", params, 600000).then(function() {
            deferred.resolve("Apuração de faturamento realizada com sucesso!");
        }, function(error) {
            console.error(error);
            
            if (multiplosregistros) {
                ScreenService.showMessage("Operação bloqueada! <br/><br/>Ocorreram erros na realização do processo. As parcelas que não foram apuradas serão exibidas na lista a seguir.");
                this.popupErroApuraParcelas();
                var wdgApuracaoFaturamento = templateManager.container.getWidget("wdgApuracaoFaturamento");
                wdgApuracaoFaturamento.unSelectAll();
                wdgApuracaoFaturamento.reload();
            }
            else {
                var filtro = [
                      {
                        "name": "CDOPERADOR",
                        "operator": "=",
                        "value": "OPERADOR_LOGADO"
                      }
                ];
                
                localStorageEngine.download("product#/crud/fct_errosapuracaofatura", filtro).then(function(dataset) {
                    ScreenService.showMessage("Operação bloqueada!<br><br>"+dataset[0].DSERRO);
                });
                
                
            }
                
            deferred.reject();
        }.bind(this));
        
        return deferred.promise;
    };
    
    /*--------------------------------------------
     | --- Procedure Cancelar Faturamento   --- |
    --------------------------------------------*/
    
     /**
     * Prepara os parâmetros para cancelar o faturamento do vencimento selecionado.
     * @param {String} tipo de apuracao - o que será apurado (aceita as str = PARCELA, RATEIO, ITEM)
     * @param {bool} rateioUnico  - determina se a apuracao do rateio está ligada a outros rateios (utilizado apenas para rateios)
     */
    this.prepararCancelarApuracao = function(widget, rateioUnico, tipoApuracao) {
        if (tipoApuracao === undefined) {
            tipoApuracao = 'PARCELA';
        }
        tipoApuracao       = tipoApuracao.toUpperCase();
        var msgConfirmacao = (tipoApuracao != "RATEIO" || rateioUnico) ? getMensagem().CANCELAMENTO.CONFIRMACAO[tipoApuracao]
                             : getMensagem().CANCELAMENTO.VALIDACAO.RATEIOAPURACAO;
            
        ScreenService.confirmMessage(msgConfirmacao, "question", function() {
            var situacao = (tipoApuracao == 'RATEIO') ? widget.currentRow.IDSITUVENCRATEIO : widget.currentRow.IDSITUACAO;
            if (!["2", "6"].includes(situacao)) {
                ScreenService.showMessage(getMensagem()["CANCELAMENTO"]["VALIDACAO"]["SITUACAONAOAPUR"][tipoApuracao]);
            } else {
                var nrseqcontrato  = widget.currentRow.NRSEQCONTRATO;
                var nrfaturacont   = widget.currentRow.NRFATURACONT;
                var wdgFaturamento = templateManager.container.getWidget("wdgApuracaoFaturamento"); 
              
                var contratos      = "_" + nrseqcontrato + "_";
                var vencimentos    = "_" + nrfaturacont + "_";
                var dtInicial      = wdgFaturamento.currentRow.DTINICOMPET;
                var dtFinal        = wdgFaturamento.currentRow.DTFINCOMPET;
                var rateios        = null;
                var nritfaturacont = null;
                
                if(tipoApuracao == 'RATEIO')
                    rateios = '_' + widget.currentRow.NRRATEIOITEMFAT + '_';
                else if(tipoApuracao == 'ITEM')
                    nritfaturacont = '_' + widget.currentRow.NRITFATURACONT + "_"; 
            
                this.cancelarFaturamentoApuracao(contratos, dtInicial, dtFinal, vencimentos, rateios, nritfaturacont).then(function() {
                    ScreenService.showMessage("Apuração de faturamento cancelada com sucesso!");
                    ScreenService.closeSwipe().then(function() {
                        widget.reload();
                        if(tipoApuracao == 'PARCELA') {
                            this.controlaVisibilidadeAct(widget);
                        } else {
                            wdgFaturamento.reload().then(function() {
                                this.controlaVisibilidadeAct(wdgFaturamento);
                            }.bind(this));
                        }
                    }.bind(this));
                }.bind(this))
                .catch(function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
            }
        }.bind(this));
    };
    
    
    /**
     * Prepara os parâmertos para chamar o cancelamento dos vencimentos.
     * 
     * @param {object} wdgFaturamento
     * @param {String} tipo de apuracao - o que será apurado (aceita as str = PARCELA, RATEIO, ITEM)
     * @param {Array} rateioUnico - determina se não há outros rateios ligados a apuração
     *                              se o cancelamento não for de um rateio é passado null
     */
    this.prepararCancelarApuracaoMarcados = function(widget, rateioUnico, tipoApuracao) {
        if (tipoApuracao === undefined) {
            tipoApuracao = 'PARCELASMARCADAS';
        }
        var vencimentosMarcados = widget.getCheckedRows();
        var wdgParcela = templateManager.container.getWidget("wdgApuracaoFaturamento"); 
        
        var contratos   = "_";
        var vencimentos = "_";
        var rateios     = "_";
        var msgConfirma = null;
        var dtInicial   = (tipoApuracao == 'PARCELASMARCADAS') ? null : moment(wdgParcela.currentRow.DTINICOMPET, 'DD/MM/YYYY');
        var dtFinal     = (tipoApuracao == 'PARCELASMARCADAS') ? null : moment(wdgParcela.currentRow.DTFINCOMPET, 'DD/MM/YYYY');
        var idSituacao  = (tipoApuracao == 'PARCELASMARCADAS') ? "IDSITUACAO" : "IDSITUVENCRATEIO";
        
        var todosVencimentosFaturados = vencimentosMarcados.filter(function(row) {
            return ["2", "6"].includes(row[idSituacao]);
        });
        
        var vencimentosOutras = vencimentosMarcados.filter(function(row) {
            return ["1", "3", "4"].includes(row[idSituacao]);
        });
        
        if (todosVencimentosFaturados.length == 0) {
            ScreenService.showMessage(getMensagem().CANCELAMENTO.VALIDACAO.SITUACAONAOAPUR[tipoApuracao]);
        } else {
            if(tipoApuracao == 'RATEIOSMARCADOS' && !rateioUnico)
                msgConfirma = getMensagem().CANCELAMENTO.VALIDACAO.RATEIOAPURACAO;
            else 
                msgConfirma = (vencimentosOutras.length == 0) ? getMensagem().CANCELAMENTO.CONFIRMACAO[tipoApuracao]
                              : getMensagem().CANCELAMENTO.VALIDACAO.PARCDIFAPURADA[tipoApuracao];
            
            ScreenService.confirmMessage(msgConfirma, "question", function() {
                todosVencimentosFaturados.forEach(function(rowVencimento) {
                    if(tipoApuracao == 'RATEIOSMARCADOS') {
                        if(contratos == '_') contratos     = "_" + rowVencimento.NRSEQCONTRATO + "_";
                        if(vencimentos == '_') vencimentos = "_" + rowVencimento.NRFATURACONT + "_";
                        if (rowVencimento.NRRATEIOITEMFAT != null)
                            rateios = rateios + rowVencimento.NRRATEIOITEMFAT + '_';
                    } else {
                        // -- Concatena os Contratos --
                        if (rowVencimento.NRSEQCONTRATO != null) 
                            contratos = contratos + rowVencimento.NRSEQCONTRATO + "_";
                        
                        // -- Concatena os Vencimentos --
                        if (rowVencimento.NRFATURACONT != null) 
                            vencimentos = vencimentos + rowVencimento.NRFATURACONT + "_";

                        // -- Busca menor data inicial --
                        if (rowVencimento.DTINICOMPET != null) {
                            var dtIniAux = moment(rowVencimento.DTINICOMPET, "DD/MM/YYYY");
                            if (dtInicial != null) {
                                var diffIni = dtIniAux.diff(dtInicial);
                                if (diffIni < 0) {
                                    dtInicial = dtIniAux;
                                }
                            } else {
                                dtInicial = dtIniAux;
                            }
                        }
                        
                        // -- Busca maior data final --
                        if (rowVencimento.DTFINCOMPET != null) {
                            var dtFinAux = moment(rowVencimento.DTFINCOMPET,"DD/MM/YYYY");
                            if (dtFinal != null) {
                            
                                var diffFin = dtFinAux.diff(dtFinal);
                                if (diffFin > 0) {
                                    dtFinal = dtFinAux;
                                }
                            } else {
                                dtFinal = dtFinAux;    
                            }
                        }
                    }
                });
            
                if (contratos == '_') contratos = null;
                if (rateios == '_') rateios = null;
                if (vencimentos == '_') vencimentos = null;
                dtInicial = dtInicial.format("DD/MM/YYYY");
                dtFinal   = dtFinal.format("DD/MM/YYYY");
                
                this.cancelarFaturamentoApuracao(contratos, dtInicial, dtFinal, vencimentos, rateios, null).then(function() {
                    ScreenService.showMessage("Apuração de faturamento cancelada com sucesso!");
                    ScreenService.closePopup();
                    widget.unSelectAll();
                    widget.reload();
                    
                    if(tipoApuracao == 'PARCELASMARCADAS') {
                        this.controlaVisibilidadeAct(widget);
                    } else {
                        wdgParcela.reload().then(function() {
                            this.controlaVisibilidadeAct(wdgParcela);
                        }.bind(this));
                    }
                }.bind(this)).catch(function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
            }.bind(this));
        }
    };
    
    
     /* Cancela a apuração do faturamento.
     */
    this.cancelarFaturamentoApuracao = function(nrseqcontrato, dataini, datafin, nrfaturacont, nrrateioitfatura, nritfaturacont) {
        var params = {
            "P_NRSEQCONTRATO"   : nrseqcontrato,
            "P_DATAINI"         : dataini,
            "P_DATAFIN"         : datafin,
            "P_NRFATURACONT"    : nrfaturacont,
            "P_NRRATEIOITEMFAT" : nrrateioitfatura,
            "P_NRITFATURACONT"  : nritfaturacont
        };
        
        return General.callRestEngine("product#/fct_vencprevcont_widget/cancelarFaturamentoApuracao", "Row", params);
    };
    
    /**
     * Valida se a ação de agrupar parcelas para faturamento pode aparecer na tela
     * @params {Object} widget - widget apuração de faturamento
    **/
    this.validarParcelasParaAgrupamento = function(widget){
        if(widget.dataSource.getCheckedRows().length <= 1)
            widget.getAction("agrupar_parcelas").isVisible = false;
        else 
            widget.getAction("agrupar_parcelas").isVisible = true;
        templateManager.updateTemplate();
    };
    /**
     * Manda validar e agrupar as parcelas para serem faturadas juntas
     * @params{Object} widget - widget apuração de faturamento
    **/
    this.agruparParcelasParaFaturamento = function(widget){
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente agrupar as parcelas selecionadas?"), "question", function(){ 
            var nrfaturaconts  = [];
            var permiteagrupar = true;
            widget.dataSource.getCheckedRows().forEach(function(parcela){
                nrfaturaconts.push(parcela.NRFATURACONT); 
                if(parcela.IDSITUACAO != 1)
                    permiteagrupar = false;    
            });
            
            if(!permiteagrupar){
                ScreenService.showMessage("Operção bloqueada!<br>Não é possível agrupar parcelas com a situação diferente de 'Pendente'.");
            } else {
                var row = {
                    "NRFATURACONT": nrfaturaconts,
                    "IDACAO"      : "AGRUPA"
                };
                General.callRestEngine("product#/fct_faturacont_completo/agruparParcelasParaFaturamento", "Row", row).then(function(response){
                    ScreenService.closePopup();
                    widget.reload();
                    widget.dataSource.clearCheckedRows();
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }
        }.bind(this));
    };
    /**
     * Desagrupa contratos ou parcelas 
     * @params{Object} widget - widget dados do agrupamento de contratos
    **/
    this.desagruparContratoParcela = function(widget){
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente desagrupar os contratos selecionados?"), "question", function(){ 
            var wdgApuracao = templateManager.container.getWidget("wdgApuracaoFaturamento");
            if(wdgApuracao.currentRow.IDSITUACAO != "1"){
                ScreenService.showMessage("Operação bloqueada!<br>Não é possível desagrupar contratos de uma parcela com a situação diferente de 'Pendente'");
            } else {
                var nrseqcontrato = [];
                
                widget.dataSource.getCheckedRows().forEach(function(parcela){
                   nrseqcontrato.push(parcela.NRSEQCONTRATO); 
                });
                
                var row = {
                    "NRFATURACONT" : wdgApuracao.currentRow.NRFATURACONT,
                    "NRSEQCONTRATO": nrseqcontrato,
                    "IDACAO"       : "DESAGRUPA"
                };
                
                General.callRestEngine("product#/fct_faturacont_completo/desagruparParcelasFaturamento", "Row", row).then(function(response){
                    widget.dataSource.clearCheckedRows();
                    ScreenService.closeSwipe();
                    wdgApuracao.reload();
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }
        }.bind(this));
    };
    /**
     * Valida a situação do rateio
     * @param{Object} wdgRateio - widget de rateio
    **/
    this.verificarSituacaoRateio = function(wdgRateio) {
        var wdgFaturamento  = templateManager.container.getWidget("wdgApuracaoFaturamento");
        var situacao        = wdgRateio.currentRow.IDSITUVENCRATEIO;
        var dstipoapuracao  = wdgFaturamento.currentRow.DSTIPOAPURACAO;
        var situacaoParcela = wdgFaturamento.currentRow.IDSITUACAO;
        
        var actionApura   = wdgRateio.getAction("apurarFaturamentoRateio");
        var actionCancela = wdgRateio.getAction("cancelarApuracaoRateio");
        
        if (situacao == '1') {
            actionApura.isVisible     = (situacaoParcela == '1' || dstipoapuracao == 'RAT');
            actionCancela.isVisible   = false;
        } else if (situacao == '2') {
            actionApura.isVisible     = false;
            actionCancela.isVisible   = (dstipoapuracao == 'RAT');
        } else if (situacao == '5') {
            actionApura.isVisible     = false;
            actionCancela.isVisible   = false;
        } else {
            actionApura.isVisible     = false;
            actionCancela.isVisible   = false;
        }
        
        /* Bloqueio de Reajuste */ 
        if (bloqReajuste == 'S') {
            
            actionApura.isVisible     = false;
            actionCancela.isVisible   = false;
        }
        
    };
    
    /**
     * Valida se o rateio pode ser cancelado
     * @param {Object} widget - widget de rateio
    **/
    this.validarCancelamentoDeApuracaoRateio = function(widget, isMultiplo) {
        var params = [
            {
                name: "NRFATURACONT",
                operator: "=",
                value: templateManager.container.getWidget("wdgApuracaoFaturamento").currentRow.NRFATURACONT
            },
            {
                name: "NRAPURACAO",
                operator: "IN",
                value: widget.currentRow.NRAPURACAO ? widget.currentRow.NRAPURACAO.split("_") : null
            },
            {
                name: "NRRATEIOITEMFAT",
                operator: "NOT_IN",
                value: widget.currentRow.NRRATEIOITEMFAT ? widget.currentRow.NRRATEIOITEMFAT.split("_") : null
            }
        ];
        
        if(isMultiplo) {
            var rowsRateio = widget.getCheckedRows();
            var apuracoes  = [];
            var nrrateios  = [];
            rowsRateio.forEach(function(rateio) {
                if(rateio.NRAPURACAO && apuracoes.indexOf(rateio.NRAPURACAO) == -1) 
                    apuracoes = apuracoes.concat(rateio.NRAPURACAO.split("_"));  
                nrrateios = nrrateios.concat(rateio.NRRATEIOITEMFAT.split("_"));  
            });
            
            params[1].value = apuracoes;
            params[2].value = nrrateios;
        }
        
        General.callRestEngine("product#/crud/fct_rateioitfaturacont_widget", "FilterData", params).then(function(response) {
            var rateiosApuracao = response.dataset.fct_rateioitfaturacont_widget;
            var rateioUnico     = (rateiosApuracao.length == 0);
            if(isMultiplo) 
                this.prepararCancelarApuracaoMarcados(widget, rateioUnico, 'RATEIOSMARCADOS');
            else 
                this.prepararCancelarApuracao(widget, rateioUnico, 'RATEIO');
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    
    /**
     * Controla as ações de apurar e cancelar apuração da tela de itens
     * @param{Object} widget - widget de itens 
    **/
    this.validarVibilidadeAcoesItens = function(widget) {
        var wdgFaturamento  = templateManager.container.getWidget("wdgApuracaoFaturamento");
        var situacao        = widget.currentRow.IDSITUACAO;
        var dstipoapuracao  = wdgFaturamento.currentRow.DSTIPOAPURACAO;
        var situacaoParcela = wdgFaturamento.currentRow.IDSITUACAO;
        
        var actionApura   = widget.getAction("apurar_item");
        var actionCancela = widget.getAction("cancelar_apuracao_item");
        
        if (situacao == '1') {
            actionApura.isVisible     = (situacaoParcela == '1' || dstipoapuracao == 'ITEM');
            actionCancela.isVisible   = false;
        } else if (situacao == '6') {
            actionApura.isVisible     = (dstipoapuracao == 'ITEM');
            actionCancela.isVisible   = false;
        } else if (situacao == '2') {
            actionApura.isVisible     = false;
            actionCancela.isVisible   = (dstipoapuracao == 'ITEM');
        } else if (situacao == '5') {
            actionApura.isVisible     = false;
            actionCancela.isVisible   = false;
        } else {
            actionApura.isVisible     = false;
            actionCancela.isVisible   = false;
        }
        
        /* Bloqueio de Reajuste */ 
        if (bloqReajuste == 'S') {
            
            actionApura.isVisible     = false;
            actionCancela.isVisible   = false;
        }
    };
    
    /**
     * Retorna uma mensagem de confimação ou validação dos
     * cancelamentos ou apurações das parcelas, rateios, itens
    **/
    var getMensagem = function() {
        var mensagens = {
            "APURACAO": {
                "CONFIRMACAO": {
                    "PARCELA"         : "Deseja realmente apurar a parcela?",
                    "RATEIO"          : "Deseja realmente apurar o rateio?",
                    "ITEM"            : "Deseja realmente apurar o item?",
                    "PARCELASMARCADAS": "Deseja realmente apurar as parcelas selecionadas?",
                    "RATEIOSMARCADOS" : "Deseja realmente apurar os rateios selecionados?"
                },
                "VALIDACAO": {
                    "PARCNAOPENDENTE": {
                        "PARCELA"         : "Operação bloqueada.<br/>Só podem ser faturadas parcelas com a situação 'Pendente'.",
                        "RATEIO"          : "Operação bloqueada.<br/>Só podem ser faturados rateios com a situação 'Pendente'.",
                        "ITEM"            : "Operação bloqueada.<br/>Só podem ser faturados itens com a situação 'Pendente'.",
                        "PARCELASMARCADAS": "Operação bloqueada.<br/>Não há nenhuma parcela selecionada de situação 'Pendente' com o valor maior que zero para realizar a apuração.",
                        "RATEIOSMARCADOS" : "Operação bloqueada.<br/>Um ou mais rateios não estão pendentes."
                    },
                    "PARCDIFPENDENTE": {
                        "PARCELASMARCADAS": "Deseja realmente apurar as parcelas selecionadas?<br><br>Obs: Algumas das parcelas selecionadas não estão com a situação 'Pendente'. Estas parcelas serão desconsideradas.",
                        "RATEIOSMARCADOS" : "Deseja realmente apurar os rateios selecionados?<br><br>Obs: Alguns dos rateios selecionados não estão com a situação 'Pendente'. Estes rateios serão desconsiderados."
                    }
                }
            },
            "CANCELAMENTO": {
                "CONFIRMACAO": {
                    "PARCELA"         : "Deseja realmente cancelar a apuração da parcela?",
                    "RATEIO"          : "Deseja realmente cancelar a apuração do rateio?",
                    "ITEM"            : "Deseja realmente cancelar a apuração do item?",
                    "PARCELASMARCADAS": "Deseja realmente cancelar a apuração das parcelas selecionadas?",
                    "RATEIOSMARCADOS" : "Deseja realmente cancelar a apuração dos rateios selecionados?"
                },
                "VALIDACAO": {
                    "SITUACAONAOAPUR": {
                        "PARCELA"          : "Operação bloqueada!<br>A parcela não está apurada.",
                        "RATEIO"           : "Operação bloqueada!<br>O rateio não está apurado.",
                        "ITEM"             : "Operação bloqueada!<br>O item não está apurado.",
                        "PARCELASMARCADAS" : "Operação bloqueada!<br>As parcelas não estão apuradas.",
                        "RATEIOSMARCADOS"  : "Operação bloqueada!<br>Os rateios não estão apurados."
                    },
                    "PARCDIFAPURADA": {
                        "PARCELASMARCADAS": "Deseja realmente cancelar a apuração as parcelas selecionadas?<br><br>Obs: Algumas das parcelas selecionadas não estão com a situação 'Apurada'. Estas parcelas serão desconsideradas.",
                        "RATEIOSMARCADOS" : "Deseja realmente cancelar a apuração dos rateios selecionados?<br><br>Obs: Alguns dos rateios selecionados não estão com a situação 'Apurada'. Estes rateios serão desconsiderados."
                    },
                    "RATEIOAPURACAO": "Atenção!<br><br>O cancelamento dos rateios selecionados acarretará no cancelamento de todos os demais rateios ligados as notas de apuração dos mesmos.<br><br> Deseja cancelar a apuração dos rateios?"
                }
            }
        };
        
        return mensagens; 
    };
    
    /**
     * Carrega os dados na tela e faz as validações necessárias
     * @params{Object} widget - widget de rateio
    **/
    this.widgetRateioOnEnter = function(widget) {
        var wdgFaturamento = templateManager.container.getWidget("wdgApuracaoFaturamento");
        var idsituacao     = wdgFaturamento.currentRow.IDSITUACAO;
        var idtipoapuracao = wdgFaturamento.currentRow.DSTIPOAPURACAO;
        
        widget.getAction("apurarFaturamentoRateioMarcados").isVisible = (idsituacao == '1' || idtipoapuracao == 'RAT');
        widget.getAction("cancelarApuracaoRateioMarcados").isVisible  = (idsituacao == '1' || idtipoapuracao == 'RAT');
        widget.reload();
    };
    
    this.validaReajusteContr = function(widget){
       
        var params = [];
        
        General.callRestEngine("product#/fct_vencprevcont_widget/validarReajuste", "Row", params).then(function(response){
            
            if (response.dataset.dados.length > 0) {
            
                bloqReajuste = 'S';
                
                ScreenService.showCustomDialog("Atenção!<br><br>Não será possível gerar apurações de faturamento. <br><br>Existem contratos com pendência em reajuste de valores que devem ser regularizados antes da realização das apurações.", 'WARNING', null,
                [
                    {
                        "label": "Exibir clientes pendentes",
                        "code": function() {
                            
                             var wdgContPendReaj = templateManager.container.getWidget("wdgContPendReaj");
                             ScreenService.openPopup(wdgContPendReaj).then(function() {
                                
                                $('.zh-new-grid-wdgContPendReaj').parent().parent().parent().parent().parent().parent().addClass('cp-high-modal');
                                
                             });
                             
                             wdgContPendReaj.reload();
                            
                        }.bind(this)
                    },
                    {
                        "label": "OK",
                        "code": function() {}
                    }
                ]);
            }
            else {
               bloqReajuste = 'N'; 
            }
            
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        }); 
    
    };
    
    this.fecharPopupValReajCont = function(){
        $('.zh-new-grid-wdgContPendReaj').parent().parent().parent().parent().parent().parent().removeClass('cp-high-modal');
        ScreenService.closePopup();
    };
    
    /**
     * Valida se serão alterados os status de parcelas com valor zerado
     * @params{Array} vencimentos com valor zerado
    **/
    this.validarParcelasZeradas = function(vencimentosZerados) {
        var deferred = ZHPromise.defer();
        if(vencimentosZerados.length == 0) {
            deferred.resolve(false);
        } else {
            ScreenService.showCustomDialog("Atenção!<br><br>Algumas das parcelas selecionadas não serão apuradas porque estão com valor zerado.<br><br>Deseja alterar a situação destas parcelas para \'Faturada\'?", "question", null, [
                {
                    "label": "Sim",
                    "code": function() {
                        deferred.resolve(true);
                    }.bind(this)
                },
                {
                    "label": "Não",
                    "code": function() {
                        deferred.resolve(false);
                    }
                },
                {
                    "label": "Cancelar",
                    "code": function() {
                        deferred.resolve(null);
                    }
                }
            ]);
        }
        
        return deferred.promise;
    };
    
    /**
     * Valida se há parcelas com status diferente de pendentes quando 
     * solicitado a apuração de múltiplas parcelas
     * @params{Array} vencimentosOutras   - array de parcelas não pendentes
     * @params{Array} vencimentosZerados  - array de parcelas zeradas
     * @params{String} tipoApuracao       - Tipo de apuração (parcela, rateio ou item)
    **/
    this.validarParcelasDiferentesPendente = function(vencimentosOutras, vencimentosZerados, tipoApuracao) {
        var deferred = ZHPromise.defer();
        var msgConfirma = null;
        
        if(vencimentosOutras.length > 0)  
            msgConfirma = getMensagem().APURACAO.VALIDACAO.PARCDIFPENDENTE[tipoApuracao];
        else if(vencimentosZerados.length == 0)
            msgConfirma = getMensagem().APURACAO.CONFIRMACAO[tipoApuracao];
        
        if(!msgConfirma) {
            deferred.resolve(true);
        } else {
            ScreenService.confirmMessage(msgConfirma, "question", function() {
                deferred.resolve(true);
            }, function() {
                deferred.resolve(false);
            });
        }
        
        return deferred.promise;
    };
    
    /**
     * Abre o popup de alteração da data de saída da nota fiscal
     * 
     * @params{Object} - popup de data de saída da nota fiscal
    **/
    this.abrirPopupAlterarDataSaida = function(widget) {
        var nrlancamentos        = [];
        var nrfaturacont         = [];
        var existeParcDifApurada = false;

        widget.dataSource.getCheckedRows().forEach(function(parcela) {
            if(parcela.IDSITUACAO != "2") { 
                if(!existeParcDifApurada) existeParcDifApurada = true;
            } else {
                nrfaturacont.push(parcela.NRFATURACONT);
                nrlancamentos = nrlancamentos.concat(parcela.NRLANCTONF.split(", "));
            }
        });
        
        ScreenService.closePopup();
        if(existeParcDifApurada) {
            var mensagem = "Atenção!<br><br>Não é possível alterar a Data de Saída das notas fiscais de parcelas com situação diferente de 'Apurada'";
            mensagem     = mensagem + ((nrfaturacont.length == 0) ? "." : ", por isso durante o processo elas parcelas serão ignoradas.");
            
            ScreenService.showMessage(mensagem);
        }
        
        if(nrfaturacont.length > 0) {
            var widgetPopup       = templateManager.container.getWidget("wdgPopupAlterarDataSaida");
            widgetPopup.isVisible = true;
            
            ScreenService.openPopup(widgetPopup).then(function() {
                widgetPopup.dataSource.data = [];
                widgetPopup.newRow();
                
                widgetPopup.currentRow.NEWDTSAIDANF = moment().format("DD/MM/YYYY");
                
                var fieldNotaFiscal = widgetPopup.getField("NRLANCTONF");
                fieldNotaFiscal.dataSourceFilter[0].value = nrfaturacont;
                fieldNotaFiscal.dataSourceFilter[1].value = ['P', 'D'];
                fieldNotaFiscal.reload().then(function() {
                    fieldNotaFiscal.dataSource.selectAll();
                    
                    widgetPopup.currentRow.NRLANCTONF   = [];
                    widgetPopup.currentRow.NRNOTAFISC   = [];
                    widgetPopup.currentRow.DTENTRSAID   = [];
                    widgetPopup.currentRow.NRFATURACONT = [];
                    widgetPopup.currentRow.CDFILIAL     = [];
                    
                    fieldNotaFiscal.dataSource.data.forEach(function(nota) {
                        widgetPopup.currentRow.NRLANCTONF.push(nota.NRLANCTONF);
                        widgetPopup.currentRow.NRNOTAFISC.push(nota.NRNOTAFISC);
                        widgetPopup.currentRow.DTENTRSAID.push(nota.DTENTRSAID);
                        widgetPopup.currentRow.NRFATURACONT.push(nota.NRFATURACONT);
                        widgetPopup.currentRow.CDFILIAL.push(nota.CDFILIAL);
                    });
                });
                
            });
        }
    };
    
    /**
     * Valida se a data definida é superior a data atual
     * @params{Object} widget - popup de alteração da data de saída da nf
    **/
    this.validarNovaDataDeSaidaNF = function(widget) {
        if(widget.currentRow.NEWDTSAIDANF) {
            var dataAtual     = moment();
            var novaDataSaida = moment(widget.currentRow.NEWDTSAIDANF, "DD/MM/YYYY");
            
            if(dataAtual > novaDataSaida) {
                ScreenService.showMessage("Operação bloqueada!<br><br>A nova data de saída definida deve ser igual ou maior que a data atual.");
                widget.currentRow.NEWDTSAIDANF = dataAtual.format("DD/MM/YYYY");
            }
        }
    };
    
    /**
     * Valida e salva as alterações realizadas nas datas finais das notas fiscais
     * @params{Object} widget - popup de alteração da data de saída
    **/
    this.salvarAlteracaoDataDeSaida = function(widget) {
        if(!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            ScreenService.confirmMessage("A alteração da data de saída de uma nota fiscal a retornará para o status 'Em Digitação'.<br>Deseja continuar?", 
                "question", function() {
                    widget.currentRow.NOTASFISCAIS = widget.getField("NRLANCTONF").dataSource.getCheckedRows();
                    General.callRestEngine("product#/fct_faturacont_completo/alterarDataSaidaDaNotaFiscal", "Row", widget.currentRow).then(function() {
                        ScreenService.showMessage("Processo realizado com sucesso.");
                        ScreenService.closePopup();
                    }.bind(this))
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            );
        }
    };

    /**
     * Ação para cancelar o faturamento apenas das parcelas com situação "Faturada" de valor zerado.
     * @param {Object} wdgApuracaoFaturamento - widget de Apuração de Faturamento
     */
    this.cancelarFaturamento = function(wdgApuracaoFaturamento) {
        try{
            var nrlanctonf = wdgApuracaoFaturamento.currentRow.NRLANCTONF;
            var vrvencimento = wdgApuracaoFaturamento.currentRow.VRVENCIMENTO;
    
            if (nrlanctonf == null && vrvencimento == 0) {
                wdgApuracaoFaturamento.currentRow.changed = true;
                wdgApuracaoFaturamento.currentRow.IDSITUACAO = "1";
                General.callRestEngine("product#/fct_faturacont_widget/alterarStatusParcelasZeradas", "Row", wdgApuracaoFaturamento.currentRow).then(function(reponse){
                    wdgApuracaoFaturamento.currentRow.changed = false;
                    ScreenService.showMessage("Operação realizada com sucesso!");
                    this.controlaVisibilidadeAct(wdgApuracaoFaturamento);
                }.bind(this));
            }
            else {
                ScreenService.showMessage("Operação bloqueada. <br><br>Esta ação é permitida somente para parcelas faturadas de valor zerado.");
            }
        }catch(err){
            ScreenService.showMessage('Operação bloqueada. <br><br>' + err);
        }
    };
    
    this.popupErroApuraParcelas = function(dados) {
        ScreenService.closePopup();
        var wdgErroApuraParcelas = templateManager.container.getWidget('wdgErroApuraParcelas');
        wdgErroApuraParcelas.dataSource.data = dados;
        wdgErroApuraParcelas.isVisible = true;
         
        ScreenService.openPopup(wdgErroApuraParcelas).then(function() {
            $('.zh-new-grid-wdgErroApuraParcelas').parent().parent().parent().parent().parent().parent().addClass('cp-high-modal');
            wdgErroApuraParcelas.reload();
        });
        
    };
    
    this.fecharErroApuraParcelas = function(){
        $('.zh-new-grid-wdgErroApuraParcelas').parent().parent().parent().parent().parent().parent().removeClass('cp-high-modal');
        
        var wdgErroApuraParcelas = templateManager.container.getWidget('wdgErroApuraParcelas');
        wdgErroApuraParcelas.isVisible = true;
        ScreenService.closePopup();
    };
    
    this.setarNumeroParcelas = function(_widget){
        var widget = templateManager.container.getWidget("wdgDadosAgrupamento");
        console.log('A função que eu vou alterar tem como widget : ' + widget.label) ;
        var data = widget.dataSource.data;
        data.forEach(function(contrato, indice){
            contrato.NRPARCELA = (indice+1) + contrato.NRPARCELA;
        });
        widget.dataSource.data = data;
    };
    
    this.voltarGridPrincipal = function() {
        ScreenService.closeSwipe().then(function() {
            templateManager.container.getWidget("wdgApuracaoFaturamento").activate();
        });
    };


    this.abrirPopupwdgNegociacaoApuraFaturamento = function (inserindo) {
        var wdgNegociacaoApuraFaturamento = templateManager.container.getWidget("wdgNegociacaoApuraFaturamento");
        wdgNegociacaoApuraFaturamento.dataSource.data = [];
        ScreenService.openPopup(wdgNegociacaoApuraFaturamento).then(function () {
            if (inserindo == true) {
                wdgNegociacaoApuraFaturamento.newRow();
                wdgNegociacaoApuraFaturamento.template = 'widget/form.html';
                wdgNegociacaoApuraFaturamento.label = "Negociar Apuração de Faturamento";
            } else {
                wdgNegociacaoApuraFaturamento.view();
                wdgNegociacaoApuraFaturamento.template = 'widget/form-view.html';
                wdgNegociacaoApuraFaturamento.label = "Visualizar Negociação";
                this.carregarWdgViewNegociacao();
            }
        }.bind(this));
    };

    this.carregarWdgViewNegociacao = function () {
        var wdgApuracao = templateManager.container.getWidget("wdgApuracaoFaturamento");
        nrparcelaApuracao = [wdgApuracao.currentRow.NRFATURACONT];
        var params = {
            "NRFATURACONT" : nrparcelaApuracao,
        };

        General.callRestEngine("product#/fct_faturacont_widget/getViewNegociacao", "Row", params).then(function(response) {
            var wdgNegociacaoApuraFaturamento = templateManager.container.getWidget("wdgNegociacaoApuraFaturamento");
            wdgNegociacaoApuraFaturamento.currentRow.IDTIPONEGOCIA = response.dataset.getViewNegociacao[0].IDTIPONEGOCIA;
            wdgNegociacaoApuraFaturamento.currentRow.VRPARCELANEGOCIA = response.dataset.getViewNegociacao[0].VRPARCELANEGOCIA;
            wdgNegociacaoApuraFaturamento.currentRow.NMOPERADOR = response.dataset.getViewNegociacao[0].NMOPERADOR;
            wdgNegociacaoApuraFaturamento.currentRow.DSMOTIVOPARALISACAO = response.dataset.getViewNegociacao[0].DSMOTIVOPARALISACAO;
            wdgNegociacaoApuraFaturamento.currentRow.DSOBSNEGOCIA = response.dataset.getViewNegociacao[0].DSOBSNEGOCIA;
        }.bind(this)); 
    };

    this.saveNegociarApuracao = function(widget){
        if (widget.isValid()) {
            var wdgApuracao = templateManager.container.getWidget("wdgApuracaoFaturamento");
            var dtinicompet = wdgApuracao.currentRow.DTINICOMPET;
            dtinicompet = dtinicompet.split(' ')[0];
            var dtfincompet = wdgApuracao.currentRow.DTFINCOMPET;
            dtfincompet = dtfincompet.split(' ')[0];

            var parcelas = {
                "NRFATURACONT": wdgApuracao.currentRow.NRFATURACONT
            };

            var negociacao = {
                "NRNEGCONTRATO"         : null,
                "NRPARCNEGOCIO"         : wdgApuracao.currentRow.NRPARCFATURA,
                "NRAGRUPAMENTO"         : (wdgApuracao.currentRow.IDPARCAGRUP == 'S') ? wdgApuracao.currentRow.NRSEQCONTRATO : null,
                "IDTPFATURA"            : wdgApuracao.currentRow.IDTPFATURA,
                "CDFILIAL"              : wdgApuracao.currentRow.CDFILIALFATURA,
                "NRPARCFATURA"          : wdgApuracao.currentRow.NRPARCFATURA,
                "CDTIPOOPERFATURA"      : wdgApuracao.currentRow.CDTIPOOPERFATURA,
                "CDSERIENF"             : wdgApuracao.currentRow.CDSERIENF,
                "CDEMPRESA"             : wdgApuracao.currentRow.CDEMPRESA,
                "CDINSCESTA"            : wdgApuracao.currentRow.CDINSCESTA,
                "CDINSCMUNI"            : wdgApuracao.currentRow.CDINSCMUNI,
                "NRSEQCONTRATO"         : (wdgApuracao.currentRow.IDPARCAGRUP == 'N') ? wdgApuracao.currentRow.NRSEQCONTRATO : null,
                "NRITEMOCONTRATO"       : null, 
                "NRITEMELEMENTO"        : null, 
                "DSVARIAVELNEGOCIA"     : 'Apuração de Faturamento', 
                "IDTIPONEGOCIA"         : widget.currentRow.IDTIPONEGOCIA,
                "DSOBSNEGOCIA"          : widget.currentRow.DSOBSNEGOCIA,
                "CDOPERSOLICINEGOCIA"   : widget.currentRow.CDOPERSOLICINEGOCIA,
                "NRMOTIVOPARALISACAO"   : widget.currentRow.NRMOTIVOPARALISACAO,
                "IDREALIZARNEGOCIACAO"  : 'A',
                "PAERCELASAPUR"         : parcelas,
                "VRPARCELANEGOCIA"      : widget.currentRow.VRPARCELANEGOCIA
            };

            var params = {
                "NEGOCIACAO" : negociacao,
                "PARCELAS": parcelas
            };
            
            General.callRestEngine('product#/fct_negcontrato_widget/saveDadosNegContrato', "Row", params).then(function(){
                    ScreenService.showMessage("Processo realizado com sucesso!");
                    ScreenService.closePopup();
                    wdgApuracao.reload();
            }.bind(this), 
            function(error) {
                Utilities.showErrorMessage(error);
            }.bind(this)); 
        } else {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
    };

    this.excluirParcelaNegApur = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Deseja realmente excluir esta negociação?'), 'question', function (success) {
            var wdgApuracao = templateManager.container.getWidget("wdgApuracaoFaturamento");
            var params = {
                "NRFATURACONT": widget.currentRow.NRFATURACONT,
                "IDSITUACAO" : widget.currentRow.IDSITUACAO,
                "VRVENCIMENTO" : widget.currentRow.VRVENCIMENTO,
                "NRPARCELASAPURNEGCONTRATO" : widget.currentRow.NRPARCELASAPURNEGCONTRATO
            };
            General.callRestEngine("product#/fct_faturacont_widget/excluirParcelaNegApur", "Row", params).then(function() {
                    
                ScreenService.showMessage("Processo realizado com sucesso!"); 
                widget.reload();
                wdgApuracao.reload();
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            }); 
        });
    };

    this.filtraServico = function(widget) {
        var wdgApuracao = templateManager.container.getWidget("wdgApuracaoFaturamento");
        var fieldServico = wdgApuracao.getField("CDSERVORCA"); 
        if( widget.currentRow.NRTIPOORCAMENTO.length > 0) {
            
            fieldServico.dataSourceFilter[0].value = 'S';
            fieldServico.dataSourceFilter[1].value = widget.currentRow.NRTIPOORCAMENTO;
            fieldServico.reload();
        }
        else {
            fieldServico.dataSourceFilter[0].value = 'N';
            fieldServico.dataSourceFilter[1].value = []; 
            fieldServico.reload();
        }
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Sm00011000_Apuracao_Faturamento_Controller', Sm00011000_Apuracao_Faturamento_Controller);
});

// FILE: ./assets/js/controllers/Sm00012000_Medicao_Faturamento_Controller.js
function Sm00012000_Medicao_Faturamento_Controller(ScreenService, templateManager, Utilities, General, ZhFilterPreferences, ZHPromise) {
    
    this.carregarWidgetsItens = function() {
        
        var wdgMedicaoFaturamento = templateManager.container.getWidget("wdgMedicaoFaturamento");
        
        var nrMedicaoFatura = wdgMedicaoFaturamento.currentRow.NRMEDICAOFATURA;
         
        var situacao = wdgMedicaoFaturamento.currentRow.IDSITUACAO;
        
        var wdgItensMedicao = templateManager.container.getWidget("wdgItensMedicao");
        wdgItensMedicao.dataSourceFilter[0].value = nrMedicaoFatura;
        wdgItensMedicao.reload();
        
        var wdgMedidasVariaveis = templateManager.container.getWidget("wdgMedidasVariaveis");
        wdgMedidasVariaveis.dataSourceFilter[0].value = nrMedicaoFatura;
        wdgMedidasVariaveis.list();
        wdgMedidasVariaveis.reload();
        this.controlarCamposInlineEditable(situacao);

    };
    
    this.controlarCamposInlineEditable = function(situacao) {
        
        var wdgMedidasVariaveis = templateManager.container.getWidget("wdgMedidasVariaveis");

        if (situacao == "1" || situacao == "2") {
            wdgMedidasVariaveis.getField("QTMEDIDAVARIAVEL").inlineEditable = "number";
        }
        else {
            delete wdgMedidasVariaveis.getField("QTMEDIDAVARIAVEL").inlineEditable;
        }
    };
    
    this.cancelarEdicaoMedidasVariaveis = function(widget) {
        var wdgMedidasVariaveis = templateManager.container.getWidget("wdgMedidasVariaveis");
        wdgMedidasVariaveis.list();
        wdgMedidasVariaveis.reload();
    };
    
    
    this.salvarEdicaoMedidasVariaveis = function() {
        
        var wdgMedidasVariaveis = templateManager.container.getWidget("wdgMedidasVariaveis");
        
        General.widgetSave(wdgMedidasVariaveis).then(function() {
            wdgMedidasVariaveis.list();
            wdgMedidasVariaveis.reload();
        })
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    }; 

    /**
     * Faz o cálculo das medições marcadas
     */
    this.calcularMedicoesMarcadas = function(wdgMedicaoFaturamento) {
        var rows = wdgMedicaoFaturamento.getCheckedRows();
        
        var medicoesPendentes = rows.filter(function(row) {
            return ["1", "6"].includes(row.IDSITUACAO);
        });

        if (medicoesPendentes.length == 0) {
            ScreenService.showMessage('Operação bloqueada! <br/><br/>Não há nenhuma medição com situação "Pendente" ou "Cancelada" selecionada.');
        }
        else {
            var medicoesNaoPendentes = rows.filter(function(row) {
                return !(["1", "6"].includes(row.IDSITUACAO));
            });
    
            var obs = "";
            if (medicoesNaoPendentes.length > 0) {
                obs = '<br/><br/>OBS: Algumas das medições selecionadas não estão com a situação "Pendente" ou "Cancelada". Estas medições serão desconsideradas.';
            }
            
            ScreenService.confirmMessage("Deseja realmente realizar o cálculo das medições selecionadas?" + obs, "question", function() {
                var params = [];
                
                medicoesPendentes.forEach(function(row) {
                    params.push({
                        "NRMEDICAOFATURA": row.NRMEDICAOFATURA,
                        "NRSEQCONTRATO": row.NRSEQCONTRATO
                    });
                });
                
                General.callRestEngine("product#/fct_medicaofatura/calcularMultiplasMedicoes", "DataSet", params).then(function() {
                    ScreenService.showMessage("Operação realizada com sucesso!");
                    ScreenService.closePopup();
                    wdgMedicaoFaturamento.reload();
                    //wdgMedicaoFaturamento.dataSource.unSelectAll();
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            });
        }
    };
    
    this.calcularMedicao = function() {
        var wdgMedicaoFaturamento = templateManager.container.getWidget("wdgMedicaoFaturamento");
        var params = {
            "NRMEDICAOFATURA": wdgMedicaoFaturamento.currentRow.NRMEDICAOFATURA,
            "NRSEQCONTRATO": wdgMedicaoFaturamento.currentRow.NRSEQCONTRATO
        };
        
        General.callRestEngine("product#/fct_medicaofatura/recalcularItensMedicao", "Row", params).then(function() {
            ScreenService.showMessage("Operação realizada com sucesso!");
            ScreenService.closePopup();
            wdgMedicaoFaturamento.reload().then(function() {
                this.controlaVisibilidadeAct(wdgMedicaoFaturamento);
                this.carregarWidgetsItens();
            }.bind(this));
            
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };

    this.calcularQuantidadeParaMedicao = function(widget) {
        var qtmedidavariavel = Utilities.tirarMascaraValores(widget.currentRow.QTMEDIDAVARIAVEL);
        var franquia = widget.currentRow.QTFRANQUIA;
        var qtde = 0;

        if (widget.currentRow.IDELEMENTORAIZ && qtmedidavariavel < widget.currentRow.QTMEDIDAVENDIDA) {
            qtmedidavariavel = widget.currentRow.QTMEDIDAVENDIDA;
        }
        
        if (qtmedidavariavel > franquia) {
            qtde = qtmedidavariavel - widget.currentRow.QTFRANQUIA;
        }

        widget.currentRow.QTMEDIDAMEDICAO = qtde;
        if (widget.currentRow.IDMULTIPVARIACAO == "N") {

            var params = {
                "NRSEQCONTRATO": widget.currentRow.NRSEQCONTRATO,
                "CDSERVORCA": widget.currentRow.CDSERVORCA,
                "CDTPMEDIDASERV": widget.currentRow.CDTPMEDIDASERV,
                "CDTPMEDIDAVARIAVEL": widget.currentRow.CDTPMEDIDAVARIAVEL,
                "QTMEDIDA": qtde
            };

            General.callRestEngine("product#/fct_medicaofatura/buscarFaixaPrecoSemMultiplicacao", "Row", params).then(function(response) {

                widget.currentRow.VRTOTAL = response.dataset.buscarFaixaPrecoSemMultiplicacao.VRPRECOVARIACAO;

            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
        else if (widget.currentRow.IDMULTIPVARIACAO == "S" || !widget.currentRow.IDELEMENTORAIZ) {
            widget.currentRow.VRTOTAL = (qtde * widget.currentRow.VRUNITARIO).toFixed(2);
        }
    };
    
    this.gerarApuracaoFatura = function() {
        var wdgMedicaoFaturamento = templateManager.container.getWidget("wdgMedicaoFaturamento");
        
        var situacao = wdgMedicaoFaturamento.currentRow.IDSITUACAO;
        if (situacao == 3){
            ScreenService.confirmMessage("Deseja realmente gerar a parcela referente à medição?", "question", function() {
                var medicao   = "_" + wdgMedicaoFaturamento.currentRow.NRMEDICAOFATURA + "_";
                var contratos = "_" + wdgMedicaoFaturamento.currentRow.NRSEQCONTRATO + "_"; 
                var params = {
                    "NRMEDICAOFATURA": medicao,
                    "NRSEQCONTRATO"  : contratos
                };
                
                General.callRestEngine("product#/fct_medicaofatura/gerarApuracaoFatura", "Row", params).then(function(success) {
                    ScreenService.closePopup();
                    ScreenService.closeSwipe();
                    ScreenService.showMessage("Operação realizada com sucesso!");
                    wdgMedicaoFaturamento.reload();
                }).catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            });
        } else {
            ScreenService.showMessage("Operação Bloqueada. Só é permitido gerar parcela para as medições que estejam com a sitação 'Calculada'.");
        }
    };
    
    
    this.gerarApuracaoFaturaMarcados = function() {
        var wdgMedicaoFaturamento = templateManager.container.getWidget("wdgMedicaoFaturamento");
        var medicoes = wdgMedicaoFaturamento.getCheckedRows();
        
        var medicao = '_';
        var contratos = '_';
        var msgConfirma = null;
        
        var medicoesCalculadas = medicoes.filter(function(row) {
            return ["3"].includes(row.IDSITUACAO);
        });
        
        var medicoesOutras = medicoes.filter(function(row) {
            return ["1", "2", "4"].includes(row.IDSITUACAO);
        });
        
        if (medicoesCalculadas.length == 0) {
            msgConfirma = "Operação bloqueada! <br/>As medições marcadas não estão com a situação 'Calculada'.";
            ScreenService.showMessage(msgConfirma);
        }
        else {
            if (medicoesOutras.length == 0) {
                msgConfirma = "Deseja realmente gerar as parcelas referentes as medições selecionadas?";  
            }
            else {
                msgConfirma = "Deseja realmente gerar as parcelas referentes as medições selecionadas?<br><br> Obs: Algumas das medições selecionadas não estão com a situação 'Calculada'. Essas medições serão desconsideradas.";
            }
            
            ScreenService.confirmMessage(msgConfirma, "question", function() {
                medicoesCalculadas.forEach(function(medFatura){
                    /* -- Concatena os Contratos -- */
                    if (medFatura.NRSEQCONTRATO != null)
                        contratos = contratos + medFatura.NRSEQCONTRATO + '_';
    
                    /* -- Concatena as medições -- */
                    if (medFatura.NRMEDICAOFATURA != null)
                        medicao = medicao + medFatura.NRMEDICAOFATURA + '_';
                        
                    if (contratos == '_') 
                        contratos = null;
                    
                    if (medicao == '_') 
                        medicao = null;
                });
                
                var params = {
                    "NRMEDICAOFATURA":medicao,
                    "NRSEQCONTRATO": contratos
                };

                General.callRestEngine("product#/fct_medicaofatura/gerarApuracaoFatura", "Row", params).then(function(success) {
                    ScreenService.closePopup();
                    ScreenService.showMessage("Operação realizada com sucesso!");
                    wdgMedicaoFaturamento.reload();
                })
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            });
        }
    };
    
    /*
      Faz o controle da visibilidade das actions da tela
    */
    this.controlaVisibilidadeAct = function(widget) {

        var situacao = widget.currentRow.IDSITUACAO;
        var wdgItensDashboard = templateManager.container.getWidget("wdgItensDashboard");
        var actCalcularMedicao = wdgItensDashboard.getAction("calcular");
        var actioncancelarMedicao = wdgItensDashboard.getAction('cancelarMedicao');
        var actionImportarMedicaoFM = wdgItensDashboard.getAction('importarMedicaoFM');
        var actionConfirmarMedicao = widget.getAction('confirmarMedicao');
        var cancelarApuracaoMedicao = widget.getAction('cancelarApuracaoMedicao');
        var actCancMedicao = widget.getAction('cancMedicao');
        
        /* Pendente */
        if (situacao == '1') {
            actionConfirmarMedicao.isVisible = false;
            actioncancelarMedicao.isVisible = false;
            cancelarApuracaoMedicao.isVisible = false;
            actCalcularMedicao.isVisible = true;
            actionImportarMedicaoFM.isVisible = true;
            actCancMedicao.isVisible = true;
        }
        /* Parcialmente Calculada */
        else if (situacao == '2') {
            actionConfirmarMedicao.isVisible = false;
            actioncancelarMedicao.isVisible = true;
            cancelarApuracaoMedicao.isVisible = false;
            actCalcularMedicao.isVisible = false;
            actionImportarMedicaoFM.isVisible = false;
            actCancMedicao.isVisible = true;
        }
        /* Calculado */
        else if (situacao == '3') {
            actionConfirmarMedicao.isVisible = true;
            actioncancelarMedicao.isVisible = true;
            cancelarApuracaoMedicao.isVisible = false;
            actCalcularMedicao.isVisible = false;
            actionImportarMedicaoFM.isVisible = false;
            actCancMedicao.isVisible = true;
        }
        /* Parcela Gerada */
        else if (situacao == '4') {
            actionConfirmarMedicao.isVisible = false;
            actioncancelarMedicao.isVisible = false;
            cancelarApuracaoMedicao.isVisible = true;
            actCalcularMedicao.isVisible = false;
            actionImportarMedicaoFM.isVisible = false;
            actCancMedicao.isVisible = false;
        }
        /* Paralisada */
        else if(situacao == '5') {
            actionConfirmarMedicao.isVisible = false;
            actioncancelarMedicao.isVisible = false;
            cancelarApuracaoMedicao.isVisible = false;
            actCalcularMedicao.isVisible = false;
            actionImportarMedicaoFM.isVisible = false;
            actCancMedicao.isVisible = false;
        }
        /* '6' - Cancelada */
        else {
            actionConfirmarMedicao.isVisible = false;
            actioncancelarMedicao.isVisible = false;
            cancelarApuracaoMedicao.isVisible = false;
            actCalcularMedicao.isVisible = true;
            actionImportarMedicaoFM.isVisible = false;
            actCancMedicao.isVisible = false;
        }
    };
    
    /**
     * Cancela o cálculo da medição
     */
    this.cancelarMedicao = function() {
        var wdgMedicaoFaturamento = templateManager.container.getWidget("wdgMedicaoFaturamento");
        
        ScreenService.confirmMessage("Deseja realmente cancelar o cálculo a medição?", "question", function() {
            
            var params = {
                "NRMEDICAOFATURA": wdgMedicaoFaturamento.currentRow.NRMEDICAOFATURA,
                "NRSEQCONTRATO": wdgMedicaoFaturamento.currentRow.NRSEQCONTRATO
            };
            
            General.callRestEngine("product#/fct_medicaofatura/validaFatura", "Row", params).then(function(response) {
                
                var valida = response.dataset.VALIDA.QUANTIDADE;
        
                if (valida > 0) {
                    ScreenService.showMessage("Operação Bloqueada. Medição já possui parcela criada."); 
                }
                else {
                    var medicao = '_'+wdgMedicaoFaturamento.currentRow.NRMEDICAOFATURA+'_';
                    var contratos = '_'+wdgMedicaoFaturamento.currentRow.NRSEQCONTRATO+'_';
                    
                    var params = {
                        "NRMEDICAOFATURA":medicao,
                        "NRSEQCONTRATO": contratos
                    };
                    
                    General.callRestEngine("product#/fct_medicaofatura/cancelarMedicao", "Row", params).then(function(success) {
                        ScreenService.closePopup();
                        //ScreenService.closeSwipe();
                        ScreenService.showMessage("Operação realizada com sucesso!");
                        wdgMedicaoFaturamento.reload().then(function() {
                            this.controlaVisibilidadeAct(wdgMedicaoFaturamento);
                            this.carregarWidgetsItens();
                        }.bind(this));
                    }.bind(this))
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }.bind(this));
    };
    
    
    this.cancelarMedicaoMarcados = function() {
        var wdgMedicaoFaturamento = templateManager.container.getWidget("wdgMedicaoFaturamento");
        var medicoes = wdgMedicaoFaturamento.getCheckedRows();
        
        var medicao = '_';
        var contratos = '_';
        var msgConfirma = null;
        
        var medicoesCalculadas = medicoes.filter(function(row) {
            return ["3"].includes(row.IDSITUACAO);
        });
        
        var medicoesOutras = medicoes.filter(function(row) {
            return ["1", "2", "4"].includes(row.IDSITUACAO);
        });
        
        if (medicoesCalculadas.length == 0) {
            ScreenService.showMessage("Operação bloqueada.<br/>as medições marcadas não estão calculadas.");
        }
        else {
            
            if (medicoesOutras.length == 0){
              
                msgConfirma = "Deseja realmente cancelar o cálculo das medições selecionadas?";  
            }
            else{
                
                msgConfirma = "Deseja realmente cancelar o cálculo das medições selecionadas?<br><br>Obs: Algumas das medições selecionadas não estão com a situação 'Calculada'. Estas medições serão desconsideradas.";
            }
            
            ScreenService.confirmMessage(msgConfirma, "question", function() {
               
                medicoesCalculadas.forEach(function(medFatura){
                    
                    /* -- Concatena os Contratos -- */
                    if (medFatura.NRSEQCONTRATO != null){
                        contratos = contratos + medFatura.NRSEQCONTRATO + '_';
                    }
    
                    /* -- Concatena as medições -- */
                    if (medFatura.NRMEDICAOFATURA != null){
                        medicao = medicao+medFatura.NRMEDICAOFATURA+'_';
                    }

                    if (contratos == '_') {
                        contratos = null;
                    }
                    
                    if (medicao == '_') {
                        medicao = null;
                    }
               
                });
                
                var params = {
                    "NRMEDICAOFATURA":medicao,
                    "NRSEQCONTRATO": contratos
                };
                        
                General.callRestEngine("product#/fct_medicaofatura/cancelarMedicao", "Row", params).then(function(success) {
                    ScreenService.closePopup();
                    ScreenService.showMessage("Operação realizada com sucesso!");
                    wdgMedicaoFaturamento.reload();
                })
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
   
            });
                
        }
    };
    
    
    this.cancelarApuracaoMedicao = function() {
        var wdgMedicaoFaturamento = templateManager.container.getWidget("wdgMedicaoFaturamento");
        
        var situacao = wdgMedicaoFaturamento.currentRow.IDSITUACAO;
        
        if (situacao == 4){
            
            
            ScreenService.confirmMessage("Deseja realmente cancelar a parcela da medição?", "question", function() {
                
                var params = {
                    "NRMEDICAOFATURA": wdgMedicaoFaturamento.currentRow.NRMEDICAOFATURA,
                    "NRSEQCONTRATO": wdgMedicaoFaturamento.currentRow.NRSEQCONTRATO
                };
                
                General.callRestEngine("product#/fct_medicaofatura/validaFatura", "Row", params).then(function(response) {
                    
                    var valida = response.dataset.VALIDA.QUANTIDADE;
            
                    if (valida == 0){ 
                       ScreenService.showMessage("Operação Bloqueada. Medição não possui parcela criada."); 
                    }
                    else{
                        
                        var medicao = '_'+wdgMedicaoFaturamento.currentRow.NRMEDICAOFATURA+'_';
                        var contratos = '_'+wdgMedicaoFaturamento.currentRow.NRSEQCONTRATO+'_';
                        
                        var params = {
                            "NRMEDICAOFATURA":medicao,
                            "NRSEQCONTRATO": contratos
                        };
                        
                        General.callRestEngine("product#/fct_medicaofatura/cancelarApuracaoMedicao", "Row", params).then(function(success) {
                            ScreenService.closeSwipe();
                            ScreenService.showMessage("Operação realizada com sucesso!");
                            wdgMedicaoFaturamento.reload();
                        })
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }
                
                })
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            });
        }
        else {
            ScreenService.showMessage("Operação Bloqueada. Só é permitido cancelar parcela de medições que estejam com a sitação 'Parcela Gerada'.");
        }
    };
    
    
    this.cancelarApuracaoMedicaoMarcados = function() {
        var wdgMedicaoFaturamento = templateManager.container.getWidget("wdgMedicaoFaturamento");
        var medicoes = wdgMedicaoFaturamento.getCheckedRows();
        
        var medicao = '_';
        var contratos = '_';
        var msgConfirma = null;
        
        var medicoesParcGerado = medicoes.filter(function(row) {
            return ["4"].includes(row.IDSITUACAO);
        });
        
        var medicoesOutras = medicoes.filter(function(row) {
            return ["1", "2", "3"].includes(row.IDSITUACAO);
        });
        
        if (medicoesParcGerado.length == 0) {
            ScreenService.showMessage("Operação bloqueada.<br/>as medições marcadas não possuem parcelas geradas.");
        }
        else {
            
            if (medicoesOutras.length == 0){
              
                msgConfirma = "Deseja realmente cancelar as parcelas das medições selecionadas?";  
            }
            else{
                
                msgConfirma = "Deseja realmente cancelar as parcelas das medições selecionadas?<br><br>Obs: Algumas das medições selecionadas não estão com a situação 'Parcela Gerada'. Estas medições serão desconsideradas.";
            }
            
            ScreenService.confirmMessage(msgConfirma, "question", function() {
                
                medicoesParcGerado.forEach(function(medFatura){
                    
                    /* -- Concatena os Contratos -- */
                    if (medFatura.NRSEQCONTRATO != null){
                        contratos = contratos + medFatura.NRSEQCONTRATO + '_';
                    }
    
                    /* -- Concatena as medições -- */
                    if (medFatura.NRMEDICAOFATURA != null){
                        medicao = medicao+medFatura.NRMEDICAOFATURA+'_';
                    }

                    if (contratos == '_') {
                        contratos = null;
                    }
                    
                    if (medicao == '_') {
                        medicao = null;
                    }
               
                });
                
                var params = {
                    "NRMEDICAOFATURA":medicao,
                    "NRSEQCONTRATO": contratos
                };
                        
                General.callRestEngine("product#/fct_medicaofatura/cancelarApuracaoMedicao", "Row", params).then(function(success) {
                    ScreenService.showMessage("Operação realizada com sucesso!");
                    ScreenService.closePopup();
                    wdgMedicaoFaturamento.reload();
                })
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
   
            });
                
        }
    };
    
    /**
     * Abre a popup de justificativa para cancelar a medição
     */
    this.validarCancelarMedicao = function(wdgMedicaoFaturamento, multiplos) {
        if (multiplos) {
            var rowsPendentes = wdgMedicaoFaturamento.getCheckedRows().filter(function(row) {
                return ["1", "3"].includes(row.IDSITUACAO);
            });
            
            var outrasRows = wdgMedicaoFaturamento.getCheckedRows().filter(function(row) {
                return !(["1", "3"].includes(row.IDSITUACAO));
            });
            
            if (rowsPendentes.length == 0) {
                ScreenService.showMessage('Operação bloqueada! <br/><br/>Só é possível cancelar medições com situação "Pendente" ou "Calculada".');
            }
            else if (outrasRows.length > 0) {
                ScreenService.confirmMessage('Atenção! <br/><br/>Existem medições marcadas com situação diferente de "Pendente" e "Calculada". Essas medições serão ignoradas. <br/>Deseja continuar?', "question", function() {
                    this.abrirPopupCancelarMedicao(wdgMedicaoFaturamento, multiplos);
                }.bind(this));
            }
            else {
                this.abrirPopupCancelarMedicao(wdgMedicaoFaturamento, multiplos);
            }
        }
        else {
            this.abrirPopupCancelarMedicao(wdgMedicaoFaturamento, multiplos);
        }
    };
    
    this.abrirPopupCancelarMedicao = function(wdgMedicaoFaturamento, multiplos) {
        ScreenService.closePopup();
        var wdgCancelarMedicao = templateManager.container.getWidget("wdgCancelarMedicao");
        wdgCancelarMedicao.isVisible = true;
        ScreenService.openPopup(wdgCancelarMedicao);
        wdgCancelarMedicao.dataSource.data = [];
        wdgCancelarMedicao.newRow();
        wdgCancelarMedicao.currentRow.MULTIPLOS = multiplos;
    };
    
    /**
     * Altera a situação da medição para 'Cancelada'
     */
    this.cancMedicao = function(wdgCancelarMedicao) {
        if (wdgCancelarMedicao.isValid()) {
            var multiplos = wdgCancelarMedicao.currentRow.MULTIPLOS;
        
            var msgConfirma = "Deseja realmente cancelar ";
            if (multiplos)
                msgConfirma += "as medições selecionadas?";
            else
                msgConfirma += "esta medição?";
            
            ScreenService.confirmMessage(msgConfirma, "question", function() {
                var wdgMedicaoFaturamento = templateManager.container.getWidget("wdgMedicaoFaturamento");
                var justificativa = wdgCancelarMedicao.currentRow.DSJUSTIFICACANC;
                var medicoes = [];
                
                if (multiplos) {
                    wdgMedicaoFaturamento.getCheckedRows().forEach(function(row) {
                        if (["1", "3"].includes(row.IDSITUACAO)) {
                            medicoes.push(row.NRMEDICAOFATURA);
                        }
                    });
                }
                else {
                    medicoes = [wdgMedicaoFaturamento.currentRow.NRMEDICAOFATURA];
                }
                
                var params = {
                    "MEDICOES"       : medicoes,
                    "DSJUSTIFICACANC": justificativa
                };
                
                General.callRestEngine("product#/fct_medicaofatura/cancelarMedicoes", "Row", params).then(function() {
                    ScreenService.showMessage("Processo realizado com sucesso!");
                    ScreenService.closePopup();
                    wdgMedicaoFaturamento.reload().then(function() {
                        if (!multiplos) {
                            this.controlaVisibilidadeAct(wdgMedicaoFaturamento);
                        }
                    }.bind(this));
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }.bind(this));
        }
    };
    
    this.voltarParaMedicaoGrid = function() {
        ScreenService.closeSwipe().then(function() {
            templateManager.container.getWidget("wdgMedicaoFaturamento").activate();
        });
    };

    /**
     * Cancela o cálculo da medição
     */
    this.importarMedicaoFM = function() {
        
        ScreenService.confirmMessage("Deseja realmente importar os dados do módulo 'FM' para realização do cálculo da medição?", "question", function() {
            
            var wdgMedicaoFaturamento = templateManager.container.getWidget("wdgMedicaoFaturamento");

            var params = {
                "NRMEDICAOFATURA": wdgMedicaoFaturamento.currentRow.NRMEDICAOFATURA,
                "NRSEQCONTRATO": wdgMedicaoFaturamento.currentRow.NRSEQCONTRATO
            };
  
            General.callRestEngine("product#/IntegracaoFM/importarMedicaoFM", "Row", params).then(function(resultado) {
                if (resultado.dataset.importarMedicaoFM.resultado == 'OK') {
                    ScreenService.closePopup();
                    ScreenService.showMessage("Operação realizada com sucesso!");
                    wdgMedicaoFaturamento.reload().then(function() {
                        this.controlaVisibilidadeAct(wdgMedicaoFaturamento);
                        this.carregarWidgetsItens();
                    }.bind(this));
                }
                else {
                    ScreenService.showMessage("Operação bloqueada!<br><br>Ocorreu uma falha na integração dos dados da medição de faturamento do módulo 'FM' para o módulo 'SM'.<br><br> Motivo: "+resultado.dataset.importarMedicaoFM.resultado);
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });

        }.bind(this));
    };

    this.abrirPopupwdgNegociacaoMedicaoFaturamento = function (inserindo) {
        var wdgNegociacaoMedicaoFaturamento = templateManager.container.getWidget("wdgNegociacaoMedicaoFaturamento");
        wdgNegociacaoMedicaoFaturamento.dataSource.data = [];
        ScreenService.openPopup(wdgNegociacaoMedicaoFaturamento).then(function () {
            if (inserindo == true) {
                wdgNegociacaoMedicaoFaturamento.newRow();
                wdgNegociacaoMedicaoFaturamento.template = 'widget/form.html';
                wdgNegociacaoMedicaoFaturamento.label = "Negociar Medição de Faturamento";
            } else {
                wdgNegociacaoMedicaoFaturamento.view();
                wdgNegociacaoMedicaoFaturamento.template = 'widget/form-view.html';
                wdgNegociacaoMedicaoFaturamento.label = "Visualizar Negociação";
                this.carregarWdgViewNegociacao();
            }
        }.bind(this));
    };

    this.carregarWdgViewNegociacao = function () {
        var wdgItensMedicao = templateManager.container.getWidget("wdgItensMedicao");
        nrparcelasnegcontr = [wdgItensMedicao.currentRow.NRPARCELASNEGCONTR];
        var params = {
            "NRPARCELASNEGCONTR" : nrparcelasnegcontr,
        };

        General.callRestEngine("product#/fct_medicaofatura/getViewNegociacao", "Row", params).then(function(response) {
            var wdgNegociacaoMedicaoFaturamento = templateManager.container.getWidget("wdgNegociacaoMedicaoFaturamento");
            wdgNegociacaoMedicaoFaturamento.currentRow.IDTIPONEGOCIA = response.dataset.getViewNegociacao[0].IDTIPONEGOCIA;
            wdgNegociacaoMedicaoFaturamento.currentRow.IDMULTIPLICAQTMED = response.dataset.getViewNegociacao[0].IDMULTIPLICAQTMED;
            wdgNegociacaoMedicaoFaturamento.currentRow.VRPARCELANEGOCIA = response.dataset.getViewNegociacao[0].VRPARCELANEGOCIA;
            wdgNegociacaoMedicaoFaturamento.currentRow.NMOPERADOR = response.dataset.getViewNegociacao[0].NMOPERADOR;
            wdgNegociacaoMedicaoFaturamento.currentRow.DSMOTIVOPARALISACAO = response.dataset.getViewNegociacao[0].DSMOTIVOPARALISACAO;
            wdgNegociacaoMedicaoFaturamento.currentRow.DSOBSNEGOCIA = response.dataset.getViewNegociacao[0].DSOBSNEGOCIA;
        }.bind(this)); 
    };

    this.saveNegociarMedicao = function(widget){
        if (widget.isValid()) {
            var wdgItensMedicao = templateManager.container.getWidget("wdgItensMedicao");
            var wdgMedicaoFaturamento = templateManager.container.getWidget("wdgMedicaoFaturamento");
            var varNegociacao = this.preencheVariavelNegociacao(wdgItensMedicao);
            var dtinicompet = wdgMedicaoFaturamento.currentRow.DTINICOMPET;
            dtinicompet = dtinicompet.split(' ')[0];
            var dtfincompet = wdgMedicaoFaturamento.currentRow.DTFINCOMPET;
            dtfincompet = dtfincompet.split(' ')[0];
            var negociacao = {
                "NRNEGCONTRATO"         : wdgItensMedicao.currentRow.NRNEGCONTRATO,
                "NRPARCNEGOCIO"         : wdgMedicaoFaturamento.currentRow.NRPARCFATURA,
                "NRAGRUPAMENTO"         : wdgItensMedicao.currentRow.NRAGRUPAMENTO,
                "IDTPFATURA"            : wdgItensMedicao.currentRow.IDTPFATURA,
                "CDFILIAL"              : wdgMedicaoFaturamento.currentRow.CDFILIALFATURA,
                "NRPARCFATURA"          : wdgMedicaoFaturamento.currentRow.NRPARCFATURA,
                "CDTIPOOPERFATURA"      : wdgMedicaoFaturamento.currentRow.CDTIPOOPERFATURA,
                "CDSERIENF"             : wdgMedicaoFaturamento.currentRow.CDSERIENF,
                "CDEMPRESA"             : wdgMedicaoFaturamento.currentRow.CDEMPRESA,
                "CDINSCESTA"            : wdgMedicaoFaturamento.currentRow.CDINSCESTA,
                "CDINSCMUNI"            : wdgMedicaoFaturamento.currentRow.CDINSCMUNI,
                "NRSEQCONTRATO"         : wdgItensMedicao.currentRow.NRSEQCONTRATO,
                "NRITEMOCONTRATO"       : wdgItensMedicao.currentRow.NRITEMOCONTRATO,
                "NRITEMELEMENTO"        : wdgItensMedicao.currentRow.NRITEMELEMENTO,
                "DSVARIAVELNEGOCIA"     : varNegociacao,
                "IDTIPONEGOCIA"         : widget.currentRow.IDTIPONEGOCIA,
                "DSOBSNEGOCIA"          : widget.currentRow.DSOBSNEGOCIA,
                "CDOPERSOLICINEGOCIA"   : widget.currentRow.CDOPERSOLICINEGOCIA,
                "NRMOTIVOPARALISACAO"   : widget.currentRow.NRMOTIVOPARALISACAO,
                "IDREALIZARNEGOCIACAO"  : 'M',
                "PAERCELASAPUR"         : null
            };
            var parcelas = [{
                "DTINIPARCNEGOCIA"      : dtinicompet,
                "DTFIMPARCNEGOCIA"      : dtfincompet,
                "IDMULTIPLICAQTMED"     : widget.currentRow.IDMULTIPLICAQTMED,
                "VRPARCELANEGOCIA"      : widget.currentRow.VRPARCELANEGOCIA,
            }];
            var params = {
                "NEGOCIACAO" : negociacao,
                "PARCELAS": parcelas
            };
            
            General.callRestEngine('product#/fct_negcontrato_widget/saveDadosNegContrato', "Row", params).then(function(){
                ScreenService.closePopup()
                    ScreenService.showMessage("Processo realizado com sucesso!");
                    wdgItensMedicao.reload();
                    wdgMedicaoFaturamento.reload();
            }.bind(this), 
            function(error) {
                Utilities.showErrorMessage(error);
            }.bind(this)); 
        } else {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
    };

    this.preencheVariavelNegociacao = function(widget){
        
        var varNegociacao = '';
        
        var nrItemContrato = widget.currentRow.NRITEMOCONTRATO;
        var nmItemContrato = widget.currentRow.NMITEMCONTRATO;
        
        if (nrItemContrato){
            
            varNegociacao = 'Serviço - '+nmItemContrato;
        }
        
        widget.currentRow.DSVARIAVELNEGOCIA = varNegociacao;
        
        return varNegociacao;
        
    };

    this.excluirParcela = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Deseja realmente excluir esta negociação?'), 'question', function (success) {
            var wdgMedicaoFaturamento = templateManager.container.getWidget("wdgMedicaoFaturamento");
            var params = {
                "NRPARCELASNEGCONTR" : widget.currentRow.NRPARCELASNEGCONTR,
                "NRNEGCONTRATO"      : widget.currentRow.NRNEGCONTRATO,
                "NRITMEDICAOFATURA"  : widget.currentRow.NRITMEDICAOFATURA,
                "NRMEDICAOFATURA": wdgMedicaoFaturamento.currentRow.NRMEDICAOFATURA,
                "NRSEQCONTRATO": wdgMedicaoFaturamento.currentRow.NRSEQCONTRATO,
            };
            General.callRestEngine("product#/fct_medicaofatura/excluirParcelaMedicao", "Row", params).then(function() {
                    
                ScreenService.showMessage("Processo realizado com sucesso!"); 
                widget.reload();
                wdgMedicaoFaturamento.reload();
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            }); 
        });
    };

    /**
     * Abre a janela para a escolha dos arquivos a serem anexados 
     * 
     * @param {object} widget widget de anexos.
     */
    this.downloadArquivo = function (widget) {
        var nmarquivo = widget.currentRow.NMARQUIVO;
        var dscaminho = widget.currentRow.DSCAMINHO;
        var type = widget.currentRow.DSTIPO;

        Utilities.downloadFile(nmarquivo, dscaminho, type);
    };

    this.abrirAnexoSwipe = function (widget) {
        widget.isVisible = true;
        ScreenService.openSwipe(widget);
    };

    /**
     * Volta para o modo list da tela de contrato.
     * 
     * @param {object} widget - Widget de Contrato form
     */
    this.voltarParaMedicaoGrid = function () {
        ScreenService.closeSwipe().then(function () {
            var wdgMedicao = templateManager.container.getWidget("wdgMedicaoFaturamento");
            wdgMedicao.activate();
            wdgMedicao.refreshWidget();
        });
    };

    this.abreWidgetAddNewTipoAnexo = function (widget) {
        ScreenService.closePopup();
        var wdgPopup = templateManager.container.getWidget("wdgNewITipoAnexo");
        wdgPopup.isVisible = true;
        wdgPopup.dataSource.data = [];
        ScreenService.openPopup(wdgPopup).then(function () {
            wdgPopup.activate();
            Utilities.formWidgetNewRow(wdgPopup);
            wdgPopup.nmwdganterior = widget.name;
        });
    };

    this.cancelaWidgetTipoAnexo = function (widget) {
        var wdgAnterior = templateManager.container.getWidget(widget.nmwdganterior);
        ScreenService.closePopup();
        wdgAnterior.activate();
    };

    this.salvarNovoTipoAnexo = function (widget) {
        var wdgAnterior = templateManager.container.getWidget(widget.nmwdganterior);

        General.widgetSave(widget).then(function (response) {
            wdgAnterior.currentRow.NRTIPOANEXO = response.dataset.columnsToReturn[0].NRTIPOANEXO;
            wdgAnterior.currentRow.NMTIPOANEXO = widget.currentRow.NMTIPOANEXO;
            widget.isVisible = false;
            ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
            ScreenService.closePopup();
            wdgAnterior.activate();
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };

    /**
     * Realiza o Anexo dos arquivos
     * 
     * @param {object} widget widget de anexos
     */
    this.uploadArquivo = function (widget) {
        var currentRow = widget.currentRow;
        var widgetAnexosMedicao = templateManager.container.getWidget("medicaoAnexos");

        var files = currentRow.DSCAMINHO;

        if (currentRow.DSCAMINHO && currentRow.DSCAMINHO.length !== 0) {
            for (var i = 0; i < files.length; i++) {
                this.setValoresAnexos(widget, files, i);
                General.widgetSave(widget).then(function () {
                    this.fecharAnexoSwipe(widget);
                    widgetAnexosMedicao.reload();
                }.bind(this));
            }
        }
        else {
            ScreenService.showMessage("Selecione um arquivo para anexar à medição.");
        }
    };

    this.setValoresAnexos = function (widget, dscaminho, indexFile) {
        var widgetMedicao = templateManager.container.getWidget("wdgMedicaoFaturamento");
        widget.currentRow.NRMEDICAOFATURA = widgetMedicao.currentRow.NRMEDICAOFATURA;
        widget.currentRow.NMARQUIVO = dscaminho[indexFile].name;
        widget.currentRow.DSTIPO = dscaminho[indexFile].name.replace(/^(.*)\.(.*)$/, "$2");
        widget.currentRow.VRTAMANHO = dscaminho[indexFile].size;
        widget.currentRow.DSCAMINHO = [dscaminho[indexFile]];
        widget.currentRow.NRORG = widgetMedicao.currentRow.NRORG;
    };

    /**
     * Fecha o Swipe para realizar o Anexo dos arquivos
     * 
     * @param {object} widget widget de anexos
     */
    this.fecharAnexoSwipe = function (widget) {
        var medicaoAnexos = templateManager.container.getWidget("medicaoAnexos");
        ScreenService.closeSwipe().then(function () {
            widget.isVisible = false;
            medicaoAnexos.list();
            medicaoAnexos.activate();
        });
    };

    this.cancelarAnexoMedicao = function(widget) {
        General.widgetCancelForm(widget);
    };

    this.filtraServico = function(widget) {
        var wdgApuracao = templateManager.container.getWidget("wdgMedicaoFaturamento");
        var fieldServico = wdgApuracao.getField("CDSERVORCA"); 
        if( widget.currentRow.NRTIPOORCAMENTO.length > 0) {
            
            fieldServico.dataSourceFilter[0].value = 'S';
            fieldServico.dataSourceFilter[1].value = widget.currentRow.NRTIPOORCAMENTO;
            fieldServico.reload();
        }
        else {
            fieldServico.dataSourceFilter[0].value = 'N';
            fieldServico.dataSourceFilter[1].value = []; 
            fieldServico.reload();
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00012000_Medicao_Faturamento_Controller", Sm00012000_Medicao_Faturamento_Controller);
});

// FILE: ./assets/js/controllers/Sm00013000_GeracaoProposta_Controller.js
function Sm00013000_GeracaoProposta_Controller(General, ScreenService, templateManager, Utilities, Sm00001000_LancamentoProposta_Service) {
    
    var getWidgetGeracaoProposta = function() {
        return templateManager.container.getWidget("wdgGeracaoProposta");
    };
    
    var getWidgetJsonIntegracao = function() {
        return templateManager.container.getWidget("wdgJson");
    };
    
    this.salvarWdgGeracaoProposta = function(wdgGeracaoProposta) {
        if (wdgGeracaoProposta.currentRow.__is_new) {
            wdgGeracaoProposta.currentRow.MUDOU_ESCOPOS = true;
        }
        else {
            var newEscopos = wdgGeracaoProposta.currentRow.NRTIPOORCAMENTO;
            var bkpEscopos = wdgGeracaoProposta.backupRow.NRTIPOORCAMENTO;
            if (!Utilities.arraysSaoIguais(newEscopos, bkpEscopos)) {
                wdgGeracaoProposta.currentRow.MUDOU_ESCOPOS = true;
            }
        }
        
        Utilities.widgetSaveAndSetColumnsToReturn(wdgGeracaoProposta).then(function() {
            wdgGeracaoProposta.reload();
        });
    };
    
    this.fieldModeloOnChange = function() {
        var wdgGeracaoProposta = getWidgetGeracaoProposta();
        wdgGeracaoProposta.getField("NRTIPOORCAMENTO").clearValue();
        wdgGeracaoProposta.getField("NRTIPOORCAMENTO").reload();
    };
    
    this.editarWdgGeracaoProposta = function(wdgGeracaoProposta) {
        General.widgetEdit(wdgGeracaoProposta);
        wdgGeracaoProposta.getField("NRTIPOORCAMENTO").reload();
    };
    
    this.wdgTipoMedidaVariavelOnEnterTab = function(wdgTipoMedidaVariavel) {
        var wdgGeracaoProposta = getWidgetGeracaoProposta();
        var nrgeracaoproposta = wdgGeracaoProposta.currentRow.NRGERACAOPROPOSTA;
        var pos = Utilities.getFilterPosition(wdgTipoMedidaVariavel.dataSourceFilter, "NRGERACAOPROPOSTA");
        wdgTipoMedidaVariavel.dataSourceFilter[pos].value = nrgeracaoproposta;
        Utilities.carregarWidgetOnEnterTab(wdgTipoMedidaVariavel);
    };
    
    this.editarWdgElementos = function(wdgElementos) {
        var wdgGeracaoProposta = getWidgetGeracaoProposta();
        var elementosSalvos = [];
        if (wdgElementos.dataSource.data)
            elementosSalvos = Util.clone(wdgElementos.dataSource.data, []);
            
        var params = [
            {
                "name": "NRORCAMENTOMODELO",
                "operator": "=",
                "value": wdgGeracaoProposta.currentRow.NRORCAMENTOMODELO
            },
            {
                "name": "NRTIPOORCAMENTO",
                "operator": "IN",
                "value": wdgGeracaoProposta.currentRow.NRTIPOORCAMENTO
            }
        ];
        General.callRestEngine("product#/crud/composicao_elemento_geracao_proposta_edit", 'FilterData', params).then(function(response){
            wdgElementos.dataSource.data = response.dataset.composicao_elemento_geracao_proposta_edit;
            //wdgElementos.edit();
            wdgElementos.editing = true;
            wdgElementos._changeMode(ViewMode.EDIT);
            
            wdgElementos.floatingControl.pdfAction = false;
			wdgElementos.floatingControl.xlsAction = false;
			wdgElementos.floatingControl.csvAction = false;
			wdgElementos.floatingControl.searchAction = false;
			wdgElementos.floatingControl.refreshAction = false;
            
            wdgElementos.showCheckbox = true;
            
            if (elementosSalvos.length > 0)
                this.marcarElementosEscolhidos(wdgElementos, elementosSalvos);
            
            //this.controlarAcessibilidadeTabbed("edit", false, wdgElementos);
        }.bind(this));
    };
    
    this.cancelarWdgElementos = function(wdgElementos) {
        //widget.dataSourceFilter[0].operator = "=";
        //widget.dataSourceFilter[0].value    = "S";

        wdgElementos.list();
        wdgElementos.reload().then(function() {
            wdgElementos.showCheckbox = false;
            wdgElementos.unSelectAll();
            //this.controlarAcessibilidadeTabbed("view", true, widget);
            wdgElementos.activate();
            
            wdgElementos.floatingControl.pdfAction = true;
			wdgElementos.floatingControl.xlsAction = true;
			wdgElementos.floatingControl.csvAction = true;
			wdgElementos.floatingControl.searchAction = true;
			wdgElementos.floatingControl.refreshAction = true;
        }.bind(this));
    };
    
    this.marcarElementosEscolhidos = function(widget, elementos) {
        widget.dataSource.data.forEach(function(row) {
            if (row.TIPO == 'C') row.OLDCOMPPROPOSTA = 'N';
            elementos.forEach(function(elemento) {
                if (elemento["IDELEMENTO"] == row["IDELEMENTO"] && elemento["NRCOMPSERVICO"] == row["NRCOMPSERVICO"]) {
                    if (row.TIPO == 'C') row.OLDCOMPPROPOSTA = 'S';
                    row.__isSelected = true;
                    widget.dataSource.addCheckedRows(row);
                }
            });
        });
    };
    
    this.salvarWdgElementos = function(wdgElementos) {
        Sm00001000_LancamentoProposta_Service.validarObrigatoriedadeItensComposicao(wdgElementos.dataSource.data).then(function(msg) {
            if (msg) {
                ScreenService.showMessage(msg);
            }
            else {
                var elementos   = [];
                var composicoes = [];
                var wdgGeracaoProposta = getWidgetGeracaoProposta();
                var nrgeracaoproposta = wdgGeracaoProposta.currentRow.NRGERACAOPROPOSTA;
                var nrtipoorcamento = wdgGeracaoProposta.currentRow.NRTIPOORCAMENTO;
                
                wdgElementos.dataSource.data.forEach(function(row) {
                    if(row.__isSelected) {
                        if (row.TIPO == "E") {
                            elementos.push({
                                "NRCOMPSERVICO"    : row.NRCOMPSERVICO,
                                "NRTIPOORCAMENTO"  : row.NRTIPOORCAMENTO,
                                "NRITCOMPSERVICO"  : row.NRITCOMPSERVICO,
                                "IDELEMENTO"       : row.IDELEMENTO,
                                "IDELEMENTOPAI"    : row.IDELEMENTOPAI,
                                "IDTIPOELEMENTO"   : row.IDTIPOELEMENTO,
                                "IDSERVOBRIGCOMP"  : row.IDSERVOBRIGCOMP,
                                "IDESCOLHIDO"      : (row.__isSelected) ? "S" : "N",
                                "NRORG"            : row.NRORG
                            });
                        } else {
                            composicoes.push({
                                "NRCOMPSERVICO"    : row.NRCOMPSERVICO,
                                "NRCOMPSERVICOPAI" : row.IDARVOREPAI,
                                "IDCOMPOBRIGATORIA": row.IDSERVOBRIGCOMP,
                                "IDESCOLHIDO"      : (row.__isSelected) ? "S" : "N",
                                "NRORG"            : row.NRORG
                            });
                        }
                    }
                }.bind(this));
        
                var params = {
                    "NRGERACAOPROPOSTA": nrgeracaoproposta,
                    "ELEMENTOS"        : elementos,
                    "COMPOSICOES"      : composicoes,
                    "ESCOPOS"          : nrtipoorcamento
                };
                
                General.callRestEngine("product#/fct_geracaoproposta_modelo/inserirComposicoesElementosMedidas", "Row", params).then(function() {
                    this.cancelarWdgElementos(wdgElementos);
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
        }.bind(this));
    };
    
    /*this.getElementoPaiServico = function(row){
        if (row.__isSelected && row._parent && row._parent.TIPO == 'C')
            return null;
        return row.IDARVORE;
    };*/
    
    this.salvarTipoMedidaVariavel = function(wdgTipoMedidaVariavel) {
        wdgTipoMedidaVariavel.saveRow().then(function() {
            wdgTipoMedidaVariavel.list();
            wdgTipoMedidaVariavel.reload();
        }).catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    
    this.wdgServicosEscopoOnEnterTab = function(wdgServicosEscopo) {
        var wdgGeracaoProposta = getWidgetGeracaoProposta();
        var escopos = wdgGeracaoProposta.currentRow.NRTIPOORCAMENTO;
        var pos = Utilities.getFilterPosition(wdgServicosEscopo.dataSourceFilter, "NRTIPOORCAMENTO");
        wdgServicosEscopo.dataSourceFilter[pos].value = escopos;
        Utilities.carregarWidgetOnEnterTab(wdgServicosEscopo);
    };
    
    this.salvarServicoEscopo = function(wdgServicosEscopo) {
        if (wdgServicosEscopo.currentRow.EXISTE == "N") {
            wdgServicosEscopo.currentRow.__is_new = true;
        }
            
        General.widgetSave(wdgServicosEscopo);
    };
    
    this.gerarJson = function(nomeWidgetAtiva) {
        var wdgGeracaoProposta = getWidgetGeracaoProposta();
        var params = {
            "NRGERACAOPROPOSTA": wdgGeracaoProposta.currentRow.NRGERACAOPROPOSTA
        };
        
        General.callRestEngine("product#/fct_geracaoproposta_modelo/gerarJsonIntegracao", "Row", params).then(function(response) {
            var wdgJson = getWidgetJsonIntegracao();
            wdgJson.isVisible = true;
            ScreenService.openSwipe(wdgJson).then(function() {
                wdgJson.activate();
                wdgJson.view();
                
                var txjson = JSON.stringify(response.dataset.retorno.json, null, 4);
                wdgJson.currentRow.TXJSON = txjson;
                wdgJson.currentRow.WIDGET_ANTERIOR = nomeWidgetAtiva;
            });
        })
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    
    this.fecharWdgJson = function(wdgJson) {
        ScreenService.closeSwipe();
        wdgJson.isVisible = false;
        var widgetAnterior = templateManager.container.getWidget(wdgJson.currentRow.WIDGET_ANTERIOR);
        widgetAnterior.activate();
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00013000_GeracaoProposta_Controller", Sm00013000_GeracaoProposta_Controller);
});

// FILE: ./assets/js/controllers/Sm00015000_Apropriacao_Recursos_Controller.js
function Sm00015000_Apropriacao_Recursos_Controller(General, ScreenService, templateManager, Utilities) {

    /**
     * Ao adicionar novo registro seta os campo como apenas leitura para serem configurados
     * de acordo com o tipo que será selecionado
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
    this.abreCampos = function(widget) {
        var tipo = widget.getField('IDTIPOAPONTAMENTO').value();
        if (tipo == "3") {
            widget.getField("DSOBSAPONTAMENTO").validations = {
                "required": true
            };
            widget.getField("DSOBSAPONTAMENTO").readOnly = false;
            widget.getField("DTINICIAL").readOnly = true;
            widget.getField("DTFIM").readOnly = true;
            widget.getField("IDMOTIVOSAIDA").readOnly = true;
            widget.getField("IDMOTIVOSAIDA").validations = {};
            widget.currentRow.IDMOTIVOSAIDA = null;
            widget.currentRow.DTINICIAL = null;
            widget.currentRow.DTFIM = null;
            widget.currentRow.QTTEMPOAPONTADO = null;
        }
        else if (tipo == "1") {
            widget.getField("DTINICIAL").readOnly = false;
            widget.getField("DTFIM").readOnly = false;
            widget.getField("DSOBSAPONTAMENTO").readOnly = false;
            widget.getField("DSOBSAPONTAMENTO").validations = {};
            widget.getField("IDMOTIVOSAIDA").readOnly = true;
            widget.getField("IDMOTIVOSAIDA").validations = {};
            widget.currentRow.IDMOTIVOSAIDA = null;
        }
        else if (tipo == "2") {
            widget.getField("IDMOTIVOSAIDA").readOnly = false;
            widget.getField("DTINICIAL").readOnly = false;
            widget.getField("DTFIM").readOnly = false;
            widget.getField("DSOBSAPONTAMENTO").readOnly = false;
            widget.getField("IDMOTIVOSAIDA").validations = {
                "required": true
            };
            widget.getField("DSOBSAPONTAMENTO").validations = {};
           
        }
    };
    /**
     * configura o campo como obrigatio de acordo com o tipo selecinado
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
    this.requiredCampo = function(widget) {
        var tipo = widget.getField('IDTIPOAPONTAMENTO').value();
        var saida = widget.getField('IDMOTIVOSAIDA').value();
            widget.getField("DSOBSAPONTAMENTO").validations = {};
        if (saida == 3 && tipo == 2) {
            widget.getField("DSOBSAPONTAMENTO").validations = {
                "required": true
            };
       }else{
             widget.getField("DSOBSAPONTAMENTO").validations = {};
        }
    };
    /**
     * Abre o form para novo registro de apontamento reconfigurando de acordo com o tipo
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
    this.novoApontamento = function(widget) {
  
        General.widgetNewRow(widget);
    
        this.validaçãoDeCampoTipo(widget).then(function(response) {
                
                var tipo = response.dataset.dados.IDTIPOAPONTAMENTO;
                if (response.dataset.dados.IDTIPOAPONTAMENTO) {
                    
                    if (tipo == "1") {
                        widget.getField("IDTIPOAPONTAMENTO").value("2");
                        widget.getField("IDTIPOAPONTAMENTO").readOnly = true;
                        
                        this.abreCampos(widget);
                    }
                    else {
                        widget.getField("IDTIPOAPONTAMENTO").value("1");
                        widget.getField("IDTIPOAPONTAMENTO").readOnly = true;
                       
                        this.abreCampos(widget);
                    }
                }
                else {
                    widget.getField("IDTIPOAPONTAMENTO").value(null);
                    this.abreCampos(widget);
                    widget.getField("IDTIPOAPONTAMENTO").readOnly = false;
                    widget.getField("IDTIPOAPONTAMENTO").dataSource = {
                        "data": [{
                            "value": "1",
                            "name": "Entrada"
                        },
                        {
                            "value": "2",
                            "name": "Saida"
                        },
                        {
                            "value": "3",
                            "name": "Falta"
                        }]
                    };
                }
            }.bind(this))
            .catch(function(error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });

    };
    this.validarAcoes = function(widget){
        var wdgApontamento = templateManager.container.getWidget("gridLançamentos"); 
        var situacao = widget.getField('IDSTATUSAPROPRIACAO').value();
        if(situacao == 3){
            wdgApontamento.getAction("add").isVisible = false;
            wdgApontamento.getAction("edit").isVisible = false;
            wdgApontamento.getAction("DELETE").isVisible = false;
            wdgApontamento.getAction("confirmar").isVisible = false;
            }else 
                if(situacao == 1){
                    wdgApontamento.getAction("confirmar").isVisible = false;
                    wdgApontamento.getAction("add").isVisible = true;
                    wdgApontamento.getAction("edit").isVisible = true;
                    wdgApontamento.getAction("DELETE").isVisible = true;
                }
                else
                    if(situacao == 2){
                        wdgApontamento.getAction("add").isVisible = true;
                        wdgApontamento.getAction("edit").isVisible = true;
                        wdgApontamento.getAction("DELETE").isVisible = true;
                        wdgApontamento.getAction("confirmar").isVisible = true;
                        }
    };
    
     this.confirmarApontamento = function(widget){
         var params = {
            "NRAPROPAGENDAMENTO": widget.parent.currentRow.NRAPROPAGENDAMENTO,
            "NRORG": widget.parent.currentRow.NRORG
        };
         General.callRestEngine("product#/fct_apontamentoapropagen_widget/atualizarSituacao", "Row", params).then(function(){
             widget.getAction("confirmar").isVisible = false;
             widget.getAction("add").isVisible = false;
             widget.getAction("edit").isVisible = false;
             widget.getAction("DELETE").isVisible = false;
             widget.parent.currentRow.IDSTATUSAPROPRIACAO = 3;
             
             
             ScreenService.closeSwipe();
             ScreenService.showMessage("Apontamento confirmado com sucesso.");
             
             widget.reload();
         });
    };
    /**
     *  A função "validarHoraInicial" realiza a validação da hora para que não 
     *  seja menor que "00:00" nem maior que "23:59" e que o segunda parte da hora nos digitos minutos 
     *  nao seja maior "59".
     * 
    */
    var numTempoIni;
    this.validarHoraInicial = function(widget){
            var hora = widget.getField('DTINICIAL').value();
             numTempoIni='';
            var strHora="";
        
            for (var i = 0; i < hora.length; i++) {
                if(hora[i]!=':'){
                    
                numTempoIni=hora[i];
                strHora=strHora+numTempoIni;
                }
            }
        numTempoIni = parseInt(strHora);
        if (numTempoIni < 0 || numTempoIni > 2359) {
            widget.currentRow.DTINICIAL = null;
        } else {
            var Num = 0;
            if ((Num = parseInt(strHora[2]) >= 6)) {
                widget.currentRow.DTINICIAL = null;
            }
        }
            
            
    };
    /**
     *  A função "validarHorafim" realiza a validação da hora para que não 
     *  seja menor que "00:00" nem maior que "23:59" e que o segunda parte da hora nos digitos minutos 
     *  nao seja maior "59".
     * 
    */
    this.validarHorafim = function(widget){
            var hora = widget.getField('DTFIM').value();
            var numTempo='';
            var strHora="";
        
            for (var i = 0; i < hora.length; i++) {
                if(hora[i]!=':'){
                    
                numTempo=hora[i];
                strHora=strHora+numTempo;
                }
            }
            numTempo = parseInt(strHora);
                if (numTempo < 0 || numTempo > 2359) {    
                    widget.currentRow.DTFIM = null;
                }else{
                    var Num = 0;
                    if((Num = parseInt(strHora[2])>=6)){
                        widget.currentRow.DTFIM = null;
                }
            }
            
            if(numTempo < numTempoIni){
                widget.currentRow.DTFIM = null;
                ScreenService.showMessage("Operação bloqueada.<br/>A hora fim nâo pode esta inferior a hora inicial.");
            }
    };
    /**
     * Faz a validação do campo tipo no banco
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
    this.validaçãoDeCampoTipo = function(widget) {
        var gridControleApontamento = templateManager.container.getWidget("gridControleApontamento");
        var params = {
            "NRAPROPAGENDAMENTO": gridControleApontamento.currentRow.NRAPROPAGENDAMENTO,
            "NRORG": gridControleApontamento.currentRow.NRORG
        };

        return General.callRestEngine("product#/fct_apontamentoapropagen_widget/validacaoDeCampoTipo", "Row", params);
    };
    /**
     * Cancela um novo registro configuranode novamente o campos com as confioguraçoes iniciais
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
    this.cancelamento = function(widget) {
        widget.currentRow.IDTIPOAPONTAMENTO = null;
        widget.currentRow.IDMOTIVOSAIDA = null;
        widget.getField("IDMOTIVOSAIDA").readOnly = true;
        widget.getField("IDMOTIVOSAIDA").validations = {};
        widget.currentRow.DTINICIAL = null;
        widget.getField("DTINICIAL").readOnly = true;
        widget.currentRow.DTFIM = null;
        widget.getField("DTFIM").readOnly = true;
        widget.currentRow.QTTEMPOAPONT = null;
        widget.getField("QTTEMPOAPONT").readOnly = true;
        widget.currentRow.DSOBSAPONTAMENTO = null;
        widget.getField("DSOBSAPONTAMENTO").readOnly = true;
        widget.getField("DSOBSAPONTAMENTO").validations = {};
        General.widgetCancelForm(widget);
    };
    /**
     * salva um novo regidtro de apontamento atualizando no banco o campo da tabela de apropriação com as
     * horas que esta sendo apropradas
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
   this.saveApontamento = function(widget) {
    if (widget.isValid()) {
        this.validaHorarios(widget).then(function(response) {
                if (widget.currentRow.DTHRINICIOAPONT >= response.dataset.dados.DTHRFIMCIOAPONT || !response.dataset.dados.DTHRFIMCIOAPONT) {
                    widget.currentRow.__columns_to_return = ["NRAPONTAMENTOAPROPAGEN", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO", "DTULTATU", "NRORGULTATU", "CDOPERULTATU"];
                    General.widgetSave(widget).then(function(response) {
                        if (response.dataset) {
                            widget.currentRow.NRAPONTAMENTOAPROPAGEN = response.dataset.columnsToReturn[0].NRAPONTAMENTOAPROPAGEN;
                            widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                            widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                            widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                            widget.currentRow.DTULTATU = response.dataset.columnsToReturn[0].DTULTATU;
                            widget.currentRow.CDOPERULTATU = response.dataset.columnsToReturn[0].CDOPERULTATU;
                        }
                        this.atualizarQtdHorasApropriadas(widget).then(function(response) {
                                widget.parent.currentRow.QTHORASAPROPRIADAS = response.dataset.dados.QTHORASAPROPRIADAS;
                                widget.parent.currentRow.QTHORAS = response.dataset.dados.QTHORAS;
                            })
                            .catch(function(error) {
                                console.log(error);
                                ScreenService.showMessage(error.data.error);
                            });
                        this.verificarApontamento(widget).then(function(response) {
                                if (response.dataset.dados.LANCADO) {
                                    this.atualizarSituacaoDoRecurso(widget).then(function(response) {
                                            widget.parent.currentRow.IDSTATUSAPROPRIACAO = response.dataset.dados.IDSTATUSAPROPRIACAO;
                                            widget.getAction("confirmar").isVisible = true;
                                        })
                                        .catch(function(error) {
                                            console.log(error);
                                            ScreenService.showMessage(error.data.error);
                                        });
                                }
                            }.bind(this))
                            .catch(function(error) {
                                console.log(error);
                                ScreenService.showMessage(error.data.error);
                            });

                    }.bind(this));
                }
                else {
                    ScreenService.showMessage("Operação bloqueada.<br/>A hora final esta anterior a hora inicial.");
                }

            }.bind(this))
            .catch(function(error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });
    }
};
    /**
     * Exclui um regidtro de apontamento atualizando no banco o campo da tabela de apropriação com as
     * horas que foram excluidas
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
    this.deleteApontamento = function(widget) {

        General.deleteCrud(widget).then(function() {
            this.atualizarQtdHorasApropriadas(widget).then(function(response){
                widget.parent.currentRow.QTHORASAPROPRIADAS = response.dataset.dados.QTHORASAPROPRIADAS;
                widget.parent.currentRow.QTHORAS = response.dataset.dados.QTHORAS;
            })
            .catch(function(error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });
        }.bind(this));

    };
    /**
     * Atualizando no banco o campo da tabela de apropriação com as
     * horas que foram excluidas ou adicionadas
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
    this.atualizarQtdHorasApropriadas = function(widget) {
        var params = {
            "NRAPROPAGENDAMENTO": widget.currentRow.NRAPROPAGENDAMENTO,
            "NRORG": widget.currentRow.NRORG
        };

        return General.callRestEngine("product#/fct_apropagendamento_widget/atualizarQtdHorasApropriadas", "Row", params);
    };
     /**
     * Atualizando no banco o campo da tabela de apropriação com as
     * horas que foram excluidas ou adicionadas
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
    this.atualizarSituacaoDoRecurso = function(widget) {
        var params = {
            "NRAPROPAGENDAMENTO": widget.currentRow.NRAPROPAGENDAMENTO,
            "NRORG": widget.currentRow.NRORG
        };

        return General.callRestEngine("product#/fct_apropagendamento_widget/atualizarSituacaoDoRecurso", "Row", params);
    };
    
    /**
     * 
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
    this.validaHorarios = function(widget) {
        var gridControleApontamento = templateManager.container.getWidget("gridControleApontamento");
        var params = {
            "NRAPROPAGENDAMENTO": gridControleApontamento.currentRow.NRAPROPAGENDAMENTO,
            "NRORG": gridControleApontamento.currentRow.NRORG
        };
    
        return  General.callRestEngine("product#/fct_apontamentoapropagen_widget/validaHorarios", "Row", params);
       //console.log(valor);
    };
    
    /**
     * Verifica se o recurso ja possui algum apomtamento
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
    this.verificarApontamento = function(widget) {
        var params = {
            "NRAPROPAGENDAMENTO": widget.parent.currentRow.NRAPROPAGENDAMENTO,
            "NRORG": widget.parent.currentRow.NRORG
        };
    
        return  General.callRestEngine("product#/fct_apontamentoapropagen_widget/verificarApontamento", "Row", params);
       //console.log(valor);
    };
    
    /**
     * Calcula o intevalo de tempo entre cada entrada e saida para seta o campo Tempo
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
    this.calculaTempo = function(widget) {
        var campoInicio = widget.getField("DTINICIAL").value();
        var campoFim = widget.getField("DTFIM").value();
        if(campoInicio && campoFim){
        var inicio = widget.getField("DTHRINICIOAPONT").value();
        var fim = widget.getField("DTHRFIMCIOAPONT").value();

        if (inicio && fim) {
            var horaIni = moment(inicio, 'DD/MM/YYYY hh:mm');
            var horaFin = moment(fim, 'DD/MM/YYYY hh:mm');
            var diferencaSegundos = horaFin.diff(horaIni, 'hh:mm') / 1000;
            widget.currentRow.QTTEMPOAPONTADO = this.toHHMM(diferencaSegundos);
            widget.currentRow.QTTEMPOAPONT = diferencaSegundos / 60 / 60; // this.toHHMM(tempoApomtado/1000);
            }
        }else{
             widget.currentRow.QTTEMPOAPONTADO = "00:00";
            widget.currentRow.QTTEMPOAPONT = null;
        }
    };
    /**
     * Gera a dataHora inicial do apontamento concartenando com as horas apontadas no formato que slva no banco
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
    this.gerardatainicial = function(widget) {
        var dataApontada = widget.parent.getField("DTAPROPRIACAO").value().split(" ")[0];
        var inicio = widget.getField("DTINICIAL").value();

        widget.currentRow.DTHRINICIOAPONT = dataApontada + " " + inicio + ":00";
    };
    /**
     * Gera a dataHora final do apontamento concartenando com as horas apontadas no formato que slva no banco
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
    this.gerarDatafinal = function(widget) {
        var dataApontada = widget.parent.getField("DTAPROPRIACAO").value().split(" ")[0];
        var fim = widget.getField("DTFIM").value();

        widget.currentRow.DTHRFIMCIOAPONT = dataApontada + " " + fim + ":00";
    };
    /**
     * Converte para o formato hora/minuto um valor de tempo dado em segundos
     * 
     * @param {float} valor de tempo em segundos
     */
    this.toHHMM = function(tempoEmSegundos) {
        var sec_num = parseInt(tempoEmSegundos, 10);
        var hours = Math.floor(sec_num / 3600);
        var minutes = Math.floor((sec_num - (hours * 3600)) / 60);
        if (hours < 10) {
            hours = "0" + hours;
        }
        if (minutes < 10) {
            minutes = "0" + minutes;
        }
        return hours + ':' + minutes;
    };
    /**
     * configura os demais campos de horas quando o tipo de apontamento for falta
     * 
     * @param {object} widget widget de 
     * apontamento de horas
     */
    this.configuraFalta = function(widget) {
        widget.currentRow.DTHRINICIOAPONT = null;
        widget.currentRow.DTHRFIMCIOAPONT = null;
        widget.currentRow.QTTEMPOAPONT = 0;
    };
    /* --- FUNÇÕES DA PÁGINA  ANEXAR JUSTIFICATIVA --- */
    
    /**
     * Habilita a aba Anexar JKustificativa de acordo com o tipo falta e Saida com justificativa 
     * 
     * @param {Boolean,Boolean} widget field do 
     * campo tipo
     */
    this.abrirFechaWdgAnexoJustificativa = function(WidgetAberta,WidgetFechada) {
        var anexoJustificativa = templateManager.container.getWidget("wdgAnexoJustificativa");
        var lançamentos = templateManager.container.getWidget("gridLançamentos");
        var tipo = lançamentos.currentRow.IDTIPOAPONTAMENTO;
        var saida = lançamentos.currentRow.IDMOTIVOSAIDA;
        
        if(tipo == 3 || (tipo == 2 && saida == 3)){
        anexoJustificativa.isVisible = WidgetAberta;
        }else{
            anexoJustificativa.isVisible = WidgetFechada;
        }
    };
    /**
     * Torna a widget anexar visivel e abre o Swipe
     * 
     * @param {object} widget widget de anexos
     */
     this.abrirAnexoSwipe = function(widget){
        widget.isVisible = true;
        ScreenService.openSwipe(widget);
    };
    
    /**
     * Realiza o Anexo dos arquivos
     * 
     * @param {object} widget widget de anexos
     */
    this.uploadArquivo = function (widget)
    {
        var currentRow = widget.currentRow;
        var files = currentRow.DSCAMINHO;
        
        if (currentRow.DSCAMINHO && currentRow.DSCAMINHO.length !== 0) {
            for (var i = 0; i < files.length; i++) {
                this.setValoresAnexos(widget, i);
                General.widgetSave(widget).then(function() {
                    this.fecharAnexoSwipe(widget);
                    widget.parent.reload();
                }.bind(this));
            }
        }
        else {
            ScreenService.showMessage("Selecione um arquivo para anexar ao apontamento.");
        }
    };
    /**
     * Recebe os valores dos campos a serem salvos no registro de anexo.
     * 
     * @param {object} widget widget de anexos
     */
    this.setValoresAnexos = function(widget, indexFile){
        
        var currentRow = widget.currentRow;
        var wdgApontamento = widget.parent.parent;
        
        currentRow.NRAPONTAMENTOAPROPAGEN = wdgApontamento.currentRow.NRAPONTAMENTOAPROPAGEN;
        currentRow.NMARQUIVO = currentRow.DSCAMINHO[indexFile].name.replace(/^(.*)\.(.*)$/, "$1");
        currentRow.DSTIPO = currentRow.DSCAMINHO[indexFile].name.replace(/^(.*)\.(.*)$/, "$2");
        currentRow.VRTAMANHO = currentRow.DSCAMINHO[indexFile].size;
        currentRow.NRORG = wdgApontamento.currentRow.NRORG;
        currentRow.INDICE = indexFile;
            
    };
    /**
     * Abre a janela para a escolha dos arquivos a serem anexados 
     * 
     * @param {object} widget widget de anexos.
     */
    this.downloadArquivo = function(widget) {
        var nmarquivo = widget.currentRow.NMANEXOPADRAO;
        var dscaminho = widget.currentRow.DSCAMINHOANEXOPADRAO;
        var type      = this.getFileNameWithoutExtension(nmarquivo);
        
        Utilities.downloadFile(nmarquivo, dscaminho, type); 
    };
     /**
     * Fecha o Swipe para realizar o Anexo dos arquivos
     * 
     * @param {object} widget widget de anexos
     */
    this.fecharAnexoSwipe = function(widget)
    {
        ScreenService.closeSwipe().then(function(){
            
            widget.isVisible = false;
            widget.parent.list();
            widget.parent.activate();
           
        });
    };
}
Configuration(function(ContextRegister) {
    ContextRegister.register('Sm00015000_Apropriacao_Recursos_Controller', Sm00015000_Apropriacao_Recursos_Controller);
});


// FILE: ./assets/js/controllers/Sm00015002_Tipo_Apontamento_Controller.js
function Sm00015002_Tipo_Apontamento_Controller(General, ScreenService, templateManager ) {
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget
    */
    this.salvarTipoApontamento = function(widget){
        
        widget.currentRow.__columns_to_return = ["NRSEQTIPOAPONT", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset && response.dataset.columnsToReturn[0]){
                widget.currentRow.NRSEQTIPOAPONT = response.dataset.columnsToReturn[0].NRSEQTIPOAPONT;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
            
        }.bind(this));
        
    };
    
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00015002_Tipo_Apontamento_Controller', Sm00015002_Tipo_Apontamento_Controller);
});

// FILE: ./assets/js/controllers/Sm00015003_Tipo_Registro_Horas_Controller.js
function Sm00015003_Tipo_Registro_Horas_Controller(General, ScreenService, templateManager ) {
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget
    */
    this.salvarTipoRegistro = function(widget){
        
        widget.currentRow.__columns_to_return = ["NRSEQTPREGIS", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset && response.dataset.columnsToReturn[0]){
                widget.currentRow.NRSEQTPREGIS = response.dataset.columnsToReturn[0].NRSEQTPREGIS;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
            
        }.bind(this));
        
    };
    
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00015003_Tipo_Registro_Horas_Controller', Sm00015003_Tipo_Registro_Horas_Controller);
});

// FILE: ./assets/js/controllers/Sm00015004_Agendamento_Recurso_Controller.js
function Sm00015004_Agendamento_Recurso_Controller(General, ScreenService, templateManager, Utilities, restEngine, ZHPromise) {
    // Função para Agendamento de Recurso
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Modelo de proposta para salvar 
     * o novo registro.
     */

    this.saveAgendamentodoRecursos = function(widget) {
        var dataIni = widget.currentRow.DTINIAGENDAMENTO;
        var dataFin = widget.currentRow.DTFIMAGENDAMENTO;
        var valida = true;

        if ((dataIni != "") && (dataFin != "")) {

            dataIni = moment(dataIni, "DD/MM/YYYY");
            dataFin = moment(dataFin, "DD/MM/YYYY");

            if (dataFin.diff(dataIni) < 0) {
                valida = false;
            }
        }

        if (valida == true) {
            widget.currentRow.__columns_to_return = ["NRSEQCONTRATO", "NRAGENDRECURSOCONT", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO", "DTULTATU", "NRORGULTATU", "CDOPERULTATU"];
            General.widgetSave(widget).then(function(response) {
                if (response.dataset) {
                    widget.currentRow.NRAGENDRECURSOCONT = response.dataset.columnsToReturn[0].NRAGENDRECURSOCONT;
                    widget.currentRow.NRSEQCONTRATO      = response.dataset.columnsToReturn[0].NRSEQCONTRATO;
                    widget.currentRow.NRORG              = response.dataset.columnsToReturn[0].NRORG;
                    widget.currentRow.DTINCLUSAO         = response.dataset.columnsToReturn[0].DTINCLUSAO;
                    widget.currentRow.CDOPERINCLUSAO     = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                    widget.currentRow.DTULTATU           = response.dataset.columnsToReturn[0].DTULTATU;
                    widget.currentRow.CDOPERULTATU       = response.dataset.columnsToReturn[0].CDOPERULTATU;
                }
            });
        } else {
            ScreenService.showMessage('A data final nao pode ser anterior à data inicial!');
        }
    };
    /**
     *Realiza filtro no campo item de contrado da tela de Agendamento
     * 
     * @param {object} widget widget de 
     * Equipe de Trabalho 
     */
    this.filtrosDoContrato = function(widget) {
        var campoLocalAtendimneto = widget.getField("NMESTRUTURAM");
        campoLocalAtendimneto.dataSourceFilter[0].value = widget.currentRow.NRSEQCONTRATO;
        campoLocalAtendimneto.reload();
    };

    /**
     *Cria novo registro de Agendamento
     * 
     * @param {object} widget widget de 
     * Equipe de Trabalho 
     */
    this.novoAgendamento = function(widget) {
        General.widgetNewRow(widget);
        this.filtrosDoContrato(widget);
    };
    //funções para equipe de trabalho salvarEquipeDeTrabalho
    /**
     *Tratamento para que a data de alocação de recurso nao seja inferior ou superior as datas ininial ou
     * final do agendamento
     * 
     * @param {object} widget widget de 
     * Equipe de Trabalho 
     * @param {booleano} dataInicial boolean do 
     * FieldOnChange
     */

    this.validarData = function(widget, dataInicial) {
        var wdgAgendamento = templateManager.container.getWidget("gridAgendamentoRecursos");
        var dataIniAgend   = moment(wdgAgendamento.currentRow.DTINIAGENDAMENTO, "DD/MM/YYYY");
        var dataFimAgend   = moment(wdgAgendamento.currentRow.DTFIMAGENDAMENTO, "DD/MM/YYYY");
        var dataIniRecu    = moment(widget.currentRow.DTINIAGENDRECURSO, "DD/MM/YYYY");
        var dataFimRecu    = moment(widget.currentRow.DTFIMAGENDRECURSO, "DD/MM/YYYY");
        
        var dataValidacao  = (dataInicial) ? dataIniRecu : dataFimRecu;
        if(widget.currentRow.DTFIMAGENDRECURSO && widget.currentRow.DTINIAGENDRECURSO && (!dataFimRecu.diff(dataIniRecu) || dataFimRecu.diff(dataIniRecu) < 0)) {
            ScreenService.showMessage("Operação bloqueada!<br>A Data Inicial não pode ser maior que a Data Final do agendamento.");
            widget.currentRow.DTINIAGENDRECURSO = null; 
            widget.currentRow.DTFIMAGENDRECURSO = null;
        } else {
            var diffDtInicial = dataValidacao.diff(dataIniAgend);
            if((!diffDtInicial && diffDtInicial != 0) || diffDtInicial < 0) {
                ScreenService.showMessage("Operação bloqueada! <br> Data selecionada inferior a data inicial do Agendamento: " + wdgAgendamento.currentRow.DTINIAGENDAMENTO.split(" ")[0]);
                widget.currentRow[((dataInicial) ? "DTINIAGENDRECURSO" : "DTFIMAGENDRECURSO")] = null; 
            } else {
                var diffDtFinal = dataValidacao.diff(dataFimAgend);
                if(wdgAgendamento.currentRow.DTFIMAGENDAMENTO && ((diffDtFinal != 0 && !diffDtFinal) || diffDtFinal > 0)) {
                    ScreenService.showMessage("Operação bloqueada! <br> Data selecionada superior a data final do Agendamento: " + wdgAgendamento.currentRow.DTFIMAGENDAMENTO.split(" ")[0]);
                    widget.currentRow[((dataInicial) ? "DTINIAGENDRECURSO" : "DTFIMAGENDRECURSO")] = null; 
                }
            }
        }
    };
    
    /**
     *Abre novo registro realizando os filtros de cada campo select e 
     * com inserção automatica no campo Recurso superior
     * 
     * @param {object} wdgEquipeDeTrabalho widget de 
     * Equipe de Trabalho 
     */
    this.novaEquipeDeTrabalho = function(wdgEquipeDeTrabalho) {
        var mode        = wdgEquipeDeTrabalho.mode;
        var elemSup     = wdgEquipeDeTrabalho.currentRow.NRRECURSOSAGENDCONT;
        var elemSupName = wdgEquipeDeTrabalho.currentRow.NMRECURSO;
        
        General.widgetNewRow(wdgEquipeDeTrabalho);
        this.filtrosDeItens(wdgEquipeDeTrabalho);
        this.filtrosDeRecursoSuperior(wdgEquipeDeTrabalho);
        this.inserirRecursoSuperior(wdgEquipeDeTrabalho, mode, elemSup, elemSupName);
        this.atualizaRecurso(wdgEquipeDeTrabalho);
        this.habilitaEscala(wdgEquipeDeTrabalho);
        this.habilitaEdicaoFieldEquipe(wdgEquipeDeTrabalho, true);
        wdgEquipeDeTrabalho.getField("CHAVE").readOnly = true;
    };
    /**
     *Atuliza o select do campo Recuros antes de cada novo registro
     * 
     * @param {object} wdgEquipeDeTrabalho widget de 
     * Equipe de Trabalho 
     */
    this.atualizaRecurso = function(wdgEquipeDeTrabalho) {
        wdgEquipeDeTrabalho.getField("CHAVE").reload();
    };

    /**
     * Insere altomaticamente o recurso superior quando adiciona outro recurso para o recurso corrente
     * 
     * @param {object} wdgEquipeDeTrabalho widget de 
     * Equipe de Trabalho 
     * @param {object} mode traz o modo atual da view
     * @param {number} elemSup traz o Numero do recurso superior
     * @param {string} elemSupName traz o nome do recurso superior
     */
    this.inserirRecursoSuperior = function(wdgEquipeDeTrabalho, mode, elemSup, elemSupName) {
        if (mode == "view") {
            wdgEquipeDeTrabalho.currentRow.NRRECURSOSUPERIOR = elemSup;
            wdgEquipeDeTrabalho.currentRow.NMRECURSOSUPERIOR = elemSupName;
        }
    };
    /**
     *Abre o "edit" de Equipede trabalho junto com outras açoes de Filtro
     * 
     * @param {object} wdgEquipeDeTrabalho widget de 
     * Equipe de Trabalho 
     */
    this.editarEquipeDeTrabalho = function(wdgEquipeDeTrabalho) {
        General.widgetEdit(wdgEquipeDeTrabalho);
        this.habilitaEscala(wdgEquipeDeTrabalho);
        this.filtrosDeItens(wdgEquipeDeTrabalho);
        this.filtrosDeRecursoSuperior(wdgEquipeDeTrabalho);
        this.habilitaEdicaoFieldEquipe(wdgEquipeDeTrabalho, false);
    };
    
    /**
     * Habilita/desabilita as fields de recurso
     * @params{Object} widget widget equipe de trabalho
     * @params{bool} isVisible como a field deve estar(true/false)
    **/
    this.habilitaEdicaoFieldEquipe = function(widget, isVisible){
        widget.getField("CHAVE").showOnForm     = isVisible;
        widget.getField("NMRECURSO").showOnForm = !isVisible;
    };
    
    /**
     * Realiza o filtro para o campo select de Itens do Contrato 
     * 
     * @param {object} widget widget de 
     * Equipe de Trabalho 
     */
    this.filtrosDeItens = function(wdgEquipeDeTrabalho) {
        var campoItemContrato = wdgEquipeDeTrabalho.getField("NMITEMCONTRATO");
        campoItemContrato.dataSourceFilter[0].value = templateManager.container.getWidget("gridAgendamentoRecursos").currentRow.NRSEQCONTRATO;
        campoItemContrato.reload();
    };
    /**
     * Realiza o filtro para o campo select de Recurso superior
     * 
     * @param {object} widget widget de 
     * Equipe de Trabalho 
     */
    this.filtrosDeRecursoSuperior = function(wdgEquipeDeTrabalho) {
        var campoRecursoSuperior = wdgEquipeDeTrabalho.getField("NMRECURSOSUPERIOR");
        campoRecursoSuperior.dataSourceFilter[0].value = wdgEquipeDeTrabalho.parent.currentRow.NRAGENDRECURSOCONT;
        campoRecursoSuperior.reload();
    };
    
    /**
     * Cancela formulário de equipe de trabalho
    **/
    this.cancelFormEquipeTrabalho = function(wdgEquipeDeTrabalho){
        var visivel = wdgEquipeDeTrabalho.currentRow.__is_new;
        this.habilitaEdicaoFieldEquipe(wdgEquipeDeTrabalho, visivel);
        this.voltarWidgetEquipeTrabGrid();
        General.widgetCancelForm(wdgEquipeDeTrabalho);
    };
    
     /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Equipe de Trabalho para salvar 
     * o novo registro.
     */
    this.salvarEquipeDeTrabalho = function(wdgEquipeDeTrabalho) {
        var deferred = ZHPromise.defer();
        if (wdgEquipeDeTrabalho.isValid()) {
            this.setDias(wdgEquipeDeTrabalho);
            wdgEquipeDeTrabalho.currentRow.NRSEQCONTRATO = templateManager.container.getWidget("gridAgendamentoRecursos").currentRow.NRSEQCONTRATO;
            if (wdgEquipeDeTrabalho.isNewRow()) {
                Utilities.inserirMultiplos(wdgEquipeDeTrabalho, "CHAVE").then(function(response) {
                    ScreenService.closeSwipe().then(function() {
                        wdgEquipeDeTrabalho.reload();
                    });
                    
                    wdgEquipeDeTrabalho.widgets.forEach(function(wid) {
                        wid.enabled = true;
                    });
                    
                    this.habilitaEdicaoFieldEquipe(wdgEquipeDeTrabalho, false);
                    deferred.resolve(response);
                }.bind(this)).catch(function(error) {
                    deferred.reject();
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
            } else {
                General.widgetSave(wdgEquipeDeTrabalho).then(function(response) {
                    if(response.dataset.columnsToReturn.length > 0)
                        wdgEquipeDeTrabalho.currentRow.NRRECURSOSAGENDCONT = response.dataset.columnsToReturn[0].NRRECURSOSAGENDCONT;
                    deferred.resolve(response);
                }).catch(function(error) {
                    deferred.reject();
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
            }
        }
        
        return deferred.promise;
    };
    /**
     * Exclui uma equipe de trabalho se a situação for de ainda alocada
     * 
     * @param {object} widget widget de 
     * Equipe de Trabalho 
     */
    this.excluirEquipeDeTrabalho = function(wdgEquipeDeTrabalho) {
        General.deleteCrud(wdgEquipeDeTrabalho).then(function() {
            wdgEquipeDeTrabalho.reload();
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };

    /**
     * Seta para cada dia da semana sua apropiação 
     * Se sim seta "S"
     * Se não seta "N"
     * 
     * @param {object} widget widget de 
     * Equipe de Trabalho 
     */
    this.setDias = function(wdgEquipeDeTrabalho) {
        var dataIni = moment(wdgEquipeDeTrabalho.currentRow.DTINIAGENDRECURSO, "DD/MM/YYYY");
        var dataFin = moment(wdgEquipeDeTrabalho.currentRow.DTFIMAGENDRECURSO, "DD/MM/YYYY");
        
        if (dataFin.diff(dataIni) > 0) {
            var diasSemana   = wdgEquipeDeTrabalho.getField("DIASDASEMANA").dataSource.data;
            var diasMarcados = wdgEquipeDeTrabalho.currentRow.DIASDASEMANA;
    
            diasSemana.forEach(function(dia) {
                var nomeDia = dia.value;
                if (diasMarcados.includes(nomeDia))
                    wdgEquipeDeTrabalho.currentRow[nomeDia] = "S";
                else
                    wdgEquipeDeTrabalho.currentRow[nomeDia] = "N";
            });
        }
    };

    this.wdgEquipeDeTrabalhoAfterMoveRow = function(wdgEquipeDeTrabalho) {
        this.verificaStatus(wdgEquipeDeTrabalho);
        this.controlarAcoesSubstituicaoCobertura(wdgEquipeDeTrabalho);
    };

    /**
     * Valida o estatus das ações do recurso de acordo com sua alocação
     * false a ações "edit" e "desalocar" são desabilitadas
     * true a ações "edit" e "desalocar" são habilitadas
     * 
     * @param {object} widget widget de 
     * Equipe de Trabalho 
     */
    this.verificaStatus = function(wdgEquipeDeTrabalho) {
        var status = wdgEquipeDeTrabalho.currentRow.IDSITUARECGEND;
        if (status == "2") {
            wdgEquipeDeTrabalho.getAction("edit").isVisible = false;
            wdgEquipeDeTrabalho.getAction("desalocar").isVisible = false;
            wdgEquipeDeTrabalho.getAction("delete").isVisible = false;
        } else {
            wdgEquipeDeTrabalho.getAction("edit").isVisible = true;
            wdgEquipeDeTrabalho.getAction("desalocar").isVisible = true;
            wdgEquipeDeTrabalho.getAction("delete").isVisible = false;
        }
    };

    this.controlarAcoesSubstituicaoCobertura = function(wdgEquipeDeTrabalho) {
        var situacao = wdgEquipeDeTrabalho.currentRow.IDSITUARECGEND;
        var actSubstituicao = wdgEquipeDeTrabalho.getAction("gerar_substituicao");
        var actCobertura = wdgEquipeDeTrabalho.getAction("gerar_cobertura");
            
        if (situacao == "1") {
            var dataAtual = moment().startOf("day");
            var dtfimagendrecurso = wdgEquipeDeTrabalho.currentRow.DTFIMAGENDRECURSO.split(" ")[0];
            dtfimagendrecurso = moment(dtfimagendrecurso + " 00:00:00", "DD/MM/YYYY HH:mm:ss");
            
            var diff = dataAtual.diff(dtfimagendrecurso) / 1000 / 60 / 60 / 24;

            if (diff <= 0) {
                actSubstituicao.isVisible = true;
                actCobertura.isVisible = true;
            } else {
                actSubstituicao.isVisible = false;
                actCobertura.isVisible = false;
            }
        } else {
            actSubstituicao.isVisible = false;
            actCobertura.isVisible = false;
        }
    };

    /**
     * Cancela a desalocação de recurso 
     * 
     * @param {object} widget widget de 
     * Equipe de Trabalho 
     */
    this.cancelarDesalocacao = function(wdgDesalocarRecurso) {
        wdgDesalocarRecurso.dataSource.data.pop();
        ScreenService.closePopup();
    };

    /**
     * Abri a ação de desalocação de resurso
     * 
     * @param {object} widget widget de 
     * Equipe de Trabalho 
     */
    this.abrirWdgDesalocaRecurso = function(multiplos) {
        var DesalocarRecur = templateManager.container.getWidget("desalocarRecurso");
        var wdgEquipeDeTrabalho = templateManager.container.getWidget("equipeDeTrabalho");
        var linhasMarcadas = wdgEquipeDeTrabalho.getCheckedRows();
        var existeDesalocado = linhasMarcadas.some(function(row) {
            return row.IDSITUARECGEND == "2";
        });
        
        if (existeDesalocado) {
            ScreenService.confirmMessage("Alguns recursos selecionados já estão desalocados. Estes recursos serão desconsiderados.<br/>Deseja proseguir?", "question", function() {
                DesalocarRecur.isVisible = true;
                ScreenService.openPopup(DesalocarRecur).then(function() {
                    DesalocarRecur.newRow();
                    DesalocarRecur.currentRow.MULTIPLOS = multiplos;
                    if(wdgEquipeDeTrabalho.currentRow.DTINIAGENDRECURSO.split(" ")[0] > moment().format('DD/MM/YYYY'))
                        DesalocarRecur.currentRow.DTDEVAGENDRECURCON = wdgEquipeDeTrabalho.currentRow.DTINIAGENDRECURSO;
            
                }.bind(this));
            }, function() {
                this.desmacarSelecao(wdgEquipeDeTrabalho);
            });
        } else {
            DesalocarRecur.isVisible = true;
            ScreenService.openPopup(DesalocarRecur).then(function() {
                DesalocarRecur.newRow();
                DesalocarRecur.currentRow.MULTIPLOS = multiplos;
                if(!multiplos && wdgEquipeDeTrabalho.currentRow.DTINIAGENDRECURSO.split(" ")[0] > moment().format('DD/MM/YYYY'))
                    DesalocarRecur.currentRow.DTDEVAGENDRECURCON = wdgEquipeDeTrabalho.currentRow.DTINIAGENDRECURSO;
            
            });
        }
    };

    /**
     *Realiza a desmacação das checkedBox ao marcar multiplas Rows
     * 
     * @param {object} wdgEquipeDeTrabalho widget de 
     * Equipe de Trabalho 
     */
    this.desmacarSelecao = function(wdgEquipeDeTrabalho) {
        var linhasMarcadas = wdgEquipeDeTrabalho.getCheckedRows();
        if ((linhasMarcadas.some(function(row) {
                return row.IDSITUARECGEND == "2";
            })) > 0) {
            wdgEquipeDeTrabalho.unSelectAll();
        }
    };

    /**
     * Permite a desalocar multiplos recursos conforme o numero de ncheckedRows
     * 
     * @param {object} wdgEquipeDeTrabalho widget de 
     * Equipe de Trabalho 
     */
    this.saveDesalocacaoMultiplos = function(wdgDesalocarRecurso) {
        if (wdgDesalocarRecurso.isValid()) {
            var wdgRecursos = templateManager.container.getWidget("equipeDeTrabalho");
            var recursosParaExcluir = wdgRecursos.getCheckedRows();
    
            var parametros = {
                "RECURSOS": recursosParaExcluir,
                "DTDEVAGENDRECURCON": wdgDesalocarRecurso.currentRow.DTDEVAGENDRECURCON
            };
    
            General.callRestEngine("product#/fct_devagendrecurcon_widget/saveDesalocacaoMultiplos", "Row", parametros).then(function() {
                ScreenService.closePopup();
                this.fecharWdgDesalocaRecurso(wdgDesalocarRecurso);
                wdgRecursos.unSelectAll();
                wdgRecursos.reload();
                ScreenService.showMessage("Processo realizado com sucesso!");
            }.bind(this)).catch(function(error) {
                console.error(error);
                var msg = error.error;
                if (msg == "APROPRIACAO_COM_APONTAMENTO") {
                    ScreenService.showMessage("Operação bloqueada!<br/>Não é permitido desalocar recurso para uma data que já esteja com apontamento");
                }
                else {
                    ScreenService.showMessage(msg);
                }
            });
        }
    };

    /**
     * Salva novo registro de Desalocão
     * 
     * @param {object} widget widget de 
     * Equipe de Trabalho 
     */
    this.saveDesalocacao = function(wdgDesalocarRecurso) {
        if (wdgDesalocarRecurso.currentRow.MULTIPLOS) {
            this.saveDesalocacaoMultiplos(wdgDesalocarRecurso);
        } else {
            var wdgRecursos = templateManager.container.getWidget("equipeDeTrabalho");

            var dataIni = moment(wdgRecursos.currentRow.DTINIAGENDRECURSO, "DD/MM/YYYY");
            var dataFin = moment(wdgRecursos.currentRow.DTFIMAGENDRECURSO, "DD/MM/YYYY");
            var dataRef = moment(wdgDesalocarRecurso.currentRow.DTDEVAGENDRECURCON, "DD/MM/YYYY");
            if (dataRef.diff(dataIni) >= 0 && dataRef.diff(dataFin) <= 0) {
                var parametros = {
                    "NRRECURSOSAGENDCONT": wdgRecursos.currentRow.NRRECURSOSAGENDCONT,
                    "DTDEVAGENDRECURCON": wdgDesalocarRecurso.currentRow.DTDEVAGENDRECURCON
                };
                
                General.callRestEngine("product#/fct_devagendrecurcon_widget/saveDesalocacao", "Row", parametros).then(function(sucesso) {
                    this.fecharWdgDesalocaRecurso(wdgDesalocarRecurso);
                    ScreenService.showMessage("Processo realizado com sucesso!");
                    wdgRecursos.reload().then(function() {
                        this.verificaStatus(wdgRecursos);
                    }.bind(this));
                }.bind(this)).catch(function(error) {
                    console.error(error);
                    var msg = error.error;
                    if (msg == "APROPRIACAO_COM_APONTAMENTO") {
                        ScreenService.showMessage("Operação bloqueada! <br> Não é permitido desalocar recurso para uma data que já esteja com apontamento");
                    }
                    else {
                        ScreenService.showMessage(msg);
                    }
                });
            }
            else {
                ScreenService.showMessage("Data informada fora do periodo agendado!");
            }
        }
    };
    /**
     * Fecha a ação de de desalocação de recurso
     * 
     * @param {object} widget widget de 
     * Equipe de Trabalho 
     */
    this.fecharWdgDesalocaRecurso = function(wdgDesalocarRecurso) {
        ScreenService.closePopup();
        wdgDesalocarRecurso.isVisible = false;
    };

    //Funçoes para Tarefas da equipe de trabalho
    /**
     *  A função "ordenar" recebe a próxima a ordenação da tarefa 
     *  que está sendo inserida para a habilidade.
     */

    this.ordenar = function(widget) {
        var widEquipe = templateManager.container.getWidget("equipeDeTrabalho");
        var nrRecursosAgendCont = widEquipe.currentRow.NRRECURSOSAGENDCONT;
        var params = {
            "NRRECURSOSAGENDCONT": nrRecursosAgendCont
        };
        //Busca o nuemro do ultimo registro
        General.callRestEngine('product#/fct_tarefasagendcont_widget/recebeOrdenacao', "Row", params).then(function(response) {
                var ordenacao = response.dataset.ORDENACAO.IDORDENACAO;

                if (ordenacao == 0) {
                    ordenacao = 10;
                } else {
                    if (ordenacao % 10 == 0) {
                        ordenacao = ordenacao + 10;
                    }
                    else {
                        while ((ordenacao % 10) != 0) {
                            ordenacao = ordenacao + 1;
                        }
                    }
                }
                widget.currentRow.IDORDENACAO = ordenacao;
            },
            function(error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });
    };
    /**
     *  A função "validarHora" realiza a validação da hora para que não 
     *  seja menor que "00:00" nem maior que "23:59" e que o segundo digito da hora 
     *  nao seja maior "6".
     * 
     */
    this.validarHora = function(widget, nameField) {
        if(widget.currentRow[nameField]){
            if(moment(widget.currentRow[nameField], "HH:mm").format("HH:mm") == "Invalid date"){
                widget.currentRow[nameField] = "";
                ScreenService.showMessage(ScreenService.i18n("Operação bloqueada!<br>Hora inválida."));
            }
        }
    };
    
    /**
     * Salva o novo registro de Tarefa
     * 
     * @param {object} widget widget 
     */
    this.salvarTarefa = function(widget) {
        var nrRecursosAgendCont = templateManager.container.getWidget("equipeDeTrabalho").currentRow.NRRECURSOSAGENDCONT;
        widget.currentRow.__columns_to_return = ["NRRECURSOSAGENDCONT", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        var parametros = {
            "IDORDENACAO": widget.currentRow.IDORDENACAO,
            "NRRECURSOSAGENDCONT": nrRecursosAgendCont
        };
        /* Valida se já existe a ordenação da tarefa que está sendo inserida */
        var hora = widget.currentRow.IDHORATAREFA;
        widget.currentRow.IDHORATAREFA = Utilities.transformaHorasEmSegundos(hora);
        General.callRestEngine("product#/fct_tarefasagendcont/validar", "Row", parametros).then(function(TemSim) {
            if (TemSim.dataset.validar.resultado) {
                General.widgetSave(widget).then(function(response) {
                    console.log(widget, response);
                    if (response.dataset && response.dataset.columnsToReturn[0]) {
                        widget.currentRow.CDHABILITSERV = response.dataset.columnsToReturn[0].NRRECURSOSAGENDCONT;
                        widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                        widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                        widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                        widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                    }
                    if (widget.mode == "list")
                        widget.reload();

                }.bind(this));
            }
            else {
                ScreenService.showMessage("Operação bloqueada.<br/>Já existe uma tarefa cadastrada com a ordem informada.");
            }
        }).catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Da um novo registro na aba de Hora Extra.
     */
    this.novaHoraExtra = function(wdgHoraExtra) {
        General.widgetNewRow(wdgHoraExtra);
        alterarTemplateCampoDiaDaSemana(wdgHoraExtra, true);
    };

    /**
     * Salva o registro da aba Hora Extra, validando registros duplicados.
     */
    this.salvarHoraExtra = function(wdgHoraExtra) {
        if (wdgHoraExtra.isNewRow()) {
            Utilities.inserirMultiplosComNovoCodigo(wdgHoraExtra, "IDDIASEMANA").then(function() {
                    ScreenService.closeSwipe().then(function() {
                        wdgHoraExtra.reload();
                    });
                })
                .catch(function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
        } else {
            //Valida se já existe um registro com os mesmos dias da semana e quantidade de horas inicial informados.
            Utilities.validarCamposDuplicados(wdgHoraExtra, ["NRAGENRECHORAEXTRA"], ["IDDIASEMANA", "QTHORASINI"]).then(function(response) {
                if (response) {
                    General.widgetSave(wdgHoraExtra).then(function() {
                            alterarTemplateCampoDiaDaSemana(wdgHoraExtra, false);
                        })
                        .catch(function(error) {
                            console.error(error);
                            ScreenService.showMessage(error.data.error);
                        });
                }
                else {
                    ScreenService.showMessage("Operação bloqueada.<br/>Já existe um registro com esta Quantidade de Horas Inicial para o Dia da Semana informado.");
                }
            });
        }
    };

    /**
     * Cancela a edição/inserção do registro na aba Hora Extra.
     */
    this.cancelarHoraExtra = function(wdgHoraExtra) {
        General.widgetCancelForm(wdgHoraExtra);
        alterarTemplateCampoDiaDaSemana(wdgHoraExtra, false);
    };

    /**
     * Altera o template do campo Dia da Semana da aba Hora Extra
     * para select-multiple ou select-native.
     * Ao inserir, deve ser multiple. Ao editar, deve ser native.
     */
    var alterarTemplateCampoDiaDaSemana = function(wdgHoraExtra, multiplo) {
        wdgHoraExtra.getField("IDDIASEMANA").template = (multiplo == true) ? "field/select-multiple.html" : "field/select-native.html";
    };

    /**
     * Valida se o campo Percentual da aba Hora Extra é negativo.
     * Não deve ser possível informar um percentual negativo.
     */
    this.validarPercentualNegativo = function(fieldPercentual) {
        var valor = fieldPercentual.value();
        valor = parseFloat(valor.replace(",", "."));
        if (valor < 0) {
            fieldPercentual.value(0);
        }
    };
    
    /**
     * Verifica a disponibilidade do recurso em um determinado período
     * ou num grupo de datas 
     * @params{Object} widget equipe de trabalho
     * @params{int} quantidade de dias verificados
     * @params{Array/String} dataInicial se form um período recebe uma string 
     *  caso seja um grupo de datas recebe um array
    **/
    this.validaDisponibilidadeRec = function(widget, qtDiasTrab, dataInicial, diasDaSemana){
        var datasPeriodo = [];
        var dias = [
            {
                "Domingo"      : "IDREPETDOMINGO",
                "Segunda-feira": "IDREPETSEGUNDA", 
                "Terça-feira"  : "IDREPETTERCA", 
                "Quarta-feira" : "IDREPETQUARTA", 
                "Quinta-feira" : "IDREPETQUINTA", 
                "Sexta-feira"  : "IDREPETSEXTA",
                "Sábado"       : "IDREPETSABADO"
            }
        ];

        if(dataInicial instanceof Array){
            dataInicial.forEach(function(data){
                var row = {
                    "CHAVE"              : widget.currentRow.CHAVE, 
                    "NRRECURSOSAGENDCONT": widget.currentRow.NRRECURSOSAGENDCONT ? widget.currentRow.NRRECURSOSAGENDCONT : null,
                    "DTENTRADA"          : data,
                    "DIASDASEMANA"       : dias[0][moment(data, "DD/MM/YYYY").format("dddd")]
                };
                
                datasPeriodo.push(row);
            });
        } else {
            for(var i = 0; i < qtDiasTrab; i++){
                var dataMarcacao = moment(dataInicial, "DD/MM/YYYY");
                var row = {
                    "CHAVE"              : widget.currentRow.CHAVE, 
                    "NRRECURSOSAGENDCONT": widget.currentRow.NRRECURSOSAGENDCONT ? widget.currentRow.NRRECURSOSAGENDCONT : null,
                    "DTENTRADA"          : dataMarcacao.add(i, "days").format("DD/MM/YYYY"),
                    "DIASDASEMANA"       : dias[0][dataMarcacao.format("dddd")]
                };
                
                if(diasDaSemana.indexOf(row.DIASDASEMANA) !== -1)
                    datasPeriodo.push(row);
            }
        }

        return General.callRestEngine("product#/fct_recursosagendcont_widget/verificaDispRescurso", "DataSet", datasPeriodo);
    };
    
    /**
     * Filtra a field de recursos de acordo com a data inicial escolhida 
     * @params{Object} widget equipe de trabalho
     * @params{String} nome do campo de data inicial 
     * @params{String} nameField nome da field de recurso
    **/
    this.filtraRecursoEquipeTrab = function(widget, nameCampoDataIni, nameField){
        var dataInicial    = widget.currentRow[nameCampoDataIni];
        var dataFinal      = widget.name ===  "wdgCoberturaRecurso" ? widget.currentRow.DATAFINAL : templateManager.container.getWidget("equipeDeTrabalho").currentRow.DTFIMAGENDRECURSO;
        var chavesRecursos = widget.currentRow.CHAVE;
        var diasDaSemana   = widget.currentRow.DIASDASEMANA ? widget.currentRow.DIASDASEMANA : templateManager.container.getWidget("equipeDeTrabalho").currentRow.DIASDASEMANA;

        if(dataInicial && chavesRecursos && chavesRecursos.length > 0 && diasDaSemana && diasDaSemana.length > 0 && dataFinal){
            if(widget.name === "equipeDeTrabalho" && !widget.currentRow.__is_new && JSON.stringify(widget.backupRow.DIASDASEMANA) !== JSON.stringify(widget.currentRow.DIASDASEMANA))
                diasDaSemana = widget.currentRow.DIASDASEMANA.filter(function(dia){
                    return widget.backupRow.DIASDASEMANA.indexOf(dia) === -1;
                });
                
            var qtDiasTrab = this.getDias(dataInicial, dataFinal);
            this.validaDisponibilidadeRec(widget, qtDiasTrab, dataInicial, diasDaSemana).then(function(response){
                if(response.dataset && response.dataset.fct_recursosagendcont_widget && !response.dataset.fct_recursosagendcont_widget.DISPONIBILIDADE){
                    if(widget.currentRow.__is_new){
                        widget.currentRow.CHAVE      = [];
                        widget.currentRow[nameField] = (widget.currentRow[nameField] instanceof Array) ? [] : "";
                    } else {
                        widget.currentRow.DTINIAGENDRECURSO = widget.backupRow.DTINIAGENDRECURSO;
                        widget.currentRow.DTFIMAGENDRECURSO = widget.backupRow.DTFIMAGENDRECURSO;
                        widget.currentRow.DIASDASEMANA      = widget.backupRow.DIASDASEMANA;
                    }
                    
                    ScreenService.showMessage("Operação bloqueada.<br>Alguns Recursos selecionados não estarão disponíveis para o período selecionado.");
                }
            }.bind(this))
            .catch(function(error) {
                widget.currentRow.CHAVE      = [];
                widget.currentRow[nameField] = (widget.currentRow[nameField] instanceof Array) ? [] : "";
                ScreenService.showMessage("Operação bloqueada.<br>Falha ao validar disponibilidade do recurso.");
            });
        }
    };
    
    /**
     * Abre o popup de substituição de recursos
     * @params{Object} widget wdgEquipeDeTrabalho widget equipe de trabalho
    **/
    this.abrirWidgetSubstituicaoRecurso = function(wdgEquipeDeTrabalho) {
        var wdgSubstituicaoRecurso = templateManager.container.getWidget("wdgSubstituicaoRecurso");
        ScreenService.closePopup();
        ScreenService.openPopup(wdgSubstituicaoRecurso).then(function() {
            wdgSubstituicaoRecurso.dataSource.data = [];
            wdgSubstituicaoRecurso.newRow();
            if(wdgEquipeDeTrabalho.currentRow.DTINIAGENDRECURSO.split(" ")[0] > moment().format('DD/MM/YYYY'))
                wdgSubstituicaoRecurso.currentRow.DATASUBSTITUICAO = wdgEquipeDeTrabalho.currentRow.DTINIAGENDRECURSO;
            
            wdgSubstituicaoRecurso.currentRow.NRAGENDRECURSOCONT  = wdgEquipeDeTrabalho.currentRow.NRAGENDRECURSOCONT;
            wdgSubstituicaoRecurso.currentRow.NRRECURSOSAGENDCONT = wdgEquipeDeTrabalho.currentRow.NRRECURSOSAGENDCONT;
            wdgSubstituicaoRecurso.currentRow.NMRECURSOANTE       = wdgEquipeDeTrabalho.currentRow.NMRECURSO;
            wdgSubstituicaoRecurso.currentRow.IDTPRECURSOANTE     = wdgEquipeDeTrabalho.currentRow.IDTPRECURSO;
            wdgSubstituicaoRecurso.currentRow.CHAVEANTE           = wdgEquipeDeTrabalho.currentRow.CHAVE;
            wdgSubstituicaoRecurso.currentRow.CDEMPRESAANTE       = wdgEquipeDeTrabalho.currentRow.CDEMPRESA;
            wdgSubstituicaoRecurso.currentRow.IDESCALATRAB        = wdgEquipeDeTrabalho.currentRow.IDESCALATRAB;
            wdgSubstituicaoRecurso.currentRow.HRJORNADADIARIA     = wdgEquipeDeTrabalho.currentRow.HRJORNADADIARIA;
            wdgSubstituicaoRecurso.currentRow.HRDESCANSO          = wdgEquipeDeTrabalho.currentRow.HRDESCANSO;
            wdgSubstituicaoRecurso.currentRow.HRINTERVALO          = wdgEquipeDeTrabalho.currentRow.HRINTERVALO;
        });
    };
    
    this.abrirWidgetCoberturaRecurso = function(wdgEquipeDeTrabalho) {
        var wdgCoberturaRecurso = templateManager.container.getWidget("wdgCoberturaRecurso");
        ScreenService.closePopup();
        ScreenService.openPopup(wdgCoberturaRecurso).then(function() {
            wdgCoberturaRecurso.dataSource.data = [];
            wdgCoberturaRecurso.newRow();
            if(wdgEquipeDeTrabalho.currentRow.DTINIAGENDRECURSO.split(" ")[0] > moment().format('DD/MM/YYYY')){
                wdgCoberturaRecurso.currentRow.DATAINICIAL     = wdgEquipeDeTrabalho.currentRow.DTINIAGENDRECURSO;
                wdgCoberturaRecurso.currentRow.DATAFINAL       = wdgEquipeDeTrabalho.currentRow.DTINIAGENDRECURSO;
            }
            
            wdgCoberturaRecurso.currentRow.NRAGENDRECURSOCONT  = wdgEquipeDeTrabalho.currentRow.NRAGENDRECURSOCONT;
            wdgCoberturaRecurso.currentRow.NRRECURSOSAGENDCONT = wdgEquipeDeTrabalho.currentRow.NRRECURSOSAGENDCONT;
            wdgCoberturaRecurso.currentRow.NMRECURSOANTE       = wdgEquipeDeTrabalho.currentRow.NMRECURSO;
            wdgCoberturaRecurso.currentRow.IDTPRECURSOANTE     = wdgEquipeDeTrabalho.currentRow.IDTPRECURSO;
            wdgCoberturaRecurso.currentRow.CHAVEANTE           = wdgEquipeDeTrabalho.currentRow.CHAVE;
            wdgCoberturaRecurso.currentRow.CDEMPRESAANTE       = wdgEquipeDeTrabalho.currentRow.CDEMPRESA;
            wdgCoberturaRecurso.currentRow.IDESCALATRAB        = wdgEquipeDeTrabalho.currentRow.IDESCALATRAB;
            wdgCoberturaRecurso.currentRow.HRJORNADADIARIA     = wdgEquipeDeTrabalho.currentRow.HRJORNADADIARIA;
            wdgCoberturaRecurso.currentRow.HRDESCANSO          = wdgEquipeDeTrabalho.currentRow.HRDESCANSO;
            wdgCoberturaRecurso.currentRow.HRINTERVALO          = wdgEquipeDeTrabalho.currentRow.HRINTERVALO;
        });
    };

    /**
     * Salva a substituição do recurso e recarrega a widget de Equipe de Trabalho.
     */
    this.salvarSubstituicaoRecurso = function(wdgSubstituicaoRecurso) {
        if (wdgSubstituicaoRecurso.isValid()) {
            console.log(wdgSubstituicaoRecurso.currentRow);
            General.callRestEngine("product#/fct_agendrecursocont_widget/salvarSubstituicaoRecurso", "Row", wdgSubstituicaoRecurso.currentRow)
            .then(function() {
                    ScreenService.showMessage("Processo realizado com sucesso!");
                    ScreenService.closePopup();
                    ScreenService.closeSwipe();
                    var wdgEquipeDeTrabalho = templateManager.container.getWidget("equipeDeTrabalho");
                    wdgEquipeDeTrabalho.reload();
            }.bind(this))
            .catch(function(error) {
                console.error(error);
                var msg = error.error;
                if (msg == "APROPRIACAO_COM_APONTAMENTO") {
                    ScreenService.showMessage("Operação bloqueada! <br> Não é permitido desalocar recurso para uma data que já esteja com apontamento");
                } else {
                    ScreenService.showMessage(msg);
                }
            });
        }
    };
    
    this.salvarCoberturaRecurso = function(wdgCoberturaRecurso) {
        if (wdgCoberturaRecurso.isValid()) {
            var periodoInvalido = validarPeriodoCoberturaRecurso(wdgCoberturaRecurso.currentRow);
            if (periodoInvalido) {
                ScreenService.showMessage("Operação bloqueada.<br/>Data Final não pode ser anterior à Data Inicial.");
            } else {
                General.callRestEngine("product#/fct_agendrecursocont_widget/salvarCoberturaRecurso", "Row", wdgCoberturaRecurso.currentRow)
                .then(function() {
                    ScreenService.showMessage("Processo realizado com sucesso!");
                    ScreenService.closePopup();
                    ScreenService.closeSwipe();
                    var wdgEquipeDeTrabalho = templateManager.container.getWidget("equipeDeTrabalho");
                    wdgEquipeDeTrabalho.reload();
                }.bind(this))
                .catch(function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
            }
        }
    };
    
    var validarPeriodoCoberturaRecurso = function(rowCobertura) {
        var dataInicialCobertura = moment(rowCobertura.DATAINICIAL, "DD/MM/YYYY");
        var dataFinalCobertura = moment(rowCobertura.DATAFINAL, "DD/MM/YYYY");
        var diff = dataFinalCobertura.diff(dataInicialCobertura) / 1000 / 60 / 60 / 24;
        return (diff < 0);
    };
    
    this.sincronizarRecurso = function(widget) {
        var wdgAgendamento = templateManager.container.getWidget("gridAgendamentoRecursos");
        
        ScreenService.confirmMessage("Deseja Sincronizar Recursos com o produto HCM?", "question", function() {
        
        var dtReferencia =  widget.currentRow.DTREFERENCIA;
        dtReferencia = "01/"+moment(dtReferencia, "DD/MM/YYYY").format("MM/YYYY"); 
        
        var parametroRestEngine = [
            {
                name: "NRORG",
                operator: "=",
                value: wdgAgendamento.currentRow.NRORG
            },
            {
                name: "CDTIPOINSCRICAO",
                operator: "=",
                value: "CPF"
            },
            {
                name: "DTREFERENCIA",
                operator: "=",
                value: dtReferencia
            }
        ];
        
        General.callRestEngine("product#/sincronizarRecurso", "FilterData", parametroRestEngine).then(function(response) {
            ScreenService.showMessage("Recursos importados com sucesso!");            
                        
        }.bind(this))
        .catch(function(error){
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });       

         
        }, null);
    };
    
    /**
     * -----------------------------------------
     *   FUNÇÕES DA WIDGET ESCALA DE TRABALHO
     * -----------------------------------------
    **/
    
    /**
     * carrega os dados da widget escala de trabalho
     * @params{Object} widget widget escala de trabalho 
    **/
    this.carregarWidgetEscala = function(widget){
        var widgetEquipe = templateManager.container.getWidget("equipeDeTrabalho");
        if(widgetEquipe.mode !== "edit"){
            this.acessibilidadeWdgEscalaTrab(false);
            var params = [
                {
                    name: "NRRECURSOSAGENDCONT",
                    operator: "=",
                    value: widget.parent.currentRow.NRRECURSOSAGENDCONT
                }
            ];
            
            General.callRestEngine("product#/crud/fct_escalarecursosagendcont_widget", "FilterData", params).then(function(response) {
                widget.dataSource.data = response.dataset.fct_escalarecursosagendcont_widget;
                if(response.dataset.fct_escalarecursosagendcont_widget.length > 0 && widgetEquipe.currentRow.IDSITUARECGEND === "1"){
                    widget.getAction("editarEscalaTrab").isVisible = true;
                } else {
                    widget.getAction("editarEscalaTrab").isVisible = false;
                }
            }.bind(this))
            .catch(function(error){
                console.error(error);
            });
        } else {
            widget.getAction("editarEscalaTrab").isVisible = true;
        }
    };
    
    /**
     * Habilita o vampo Escala de trabalho se ultilizar essa forma de agendamento
     */
    this.habilitaEscala = function(widget) {
        widget.getAction("AVANCAR").isVisible            = false;
        widget.getAction("SALVAR").isVisible             = true;

        if(widget.currentRow.IDESCALATRAB == "M"){
            widget.getField("HRDESCANSO").validations        = {"required": true};
            widget.getField("HRINTERVALO").validations       = {"required": true};
            widget.getField("HRJORNADADIARIA").validations   = {"required": true};
            widget.getField("HRPRIMEIRAENTRADA").validations = {"required": true};
            widget.getField("HRDESCANSO").readOnly           = false;
            widget.getField("HRINTERVALO").readOnly          = false;
            widget.getField("HRJORNADADIARIA").readOnly      = false;
            widget.getField("HRPRIMEIRASAIDA").readOnly      = true;
            widget.getField("HRSEGUNDAENTRADA").readOnly     = true;
            widget.getField("HRSEGUNDASAIDA").readOnly       = true;
            widget.getField("NRTEMPOTOLERANCIA").readOnly    = true;
            if(widget.currentRow.__is_new){
                widget.getAction("SALVAR").isVisible  = false;
                widget.getAction("AVANCAR").isVisible = true;
            }

        } else {
            widget.getField("HRJORNADADIARIA").validations   = {};
            widget.getField("HRDESCANSO").validations        = {};
            widget.getField("HRPRIMEIRAENTRADA").validations = {};
            widget.getField("HRINTERVALO").validations = {};
            widget.getField("HRJORNADADIARIA").readOnly      = true;
            widget.getField("HRDESCANSO").readOnly           = true;
            widget.getField("HRINTERVALO").readOnly          = true;
            widget.getField("HRPRIMEIRASAIDA").readOnly      = false;
            widget.getField("HRSEGUNDAENTRADA").readOnly     = false;
            widget.getField("HRSEGUNDASAIDA").readOnly       = false;
            widget.getField("NRTEMPOTOLERANCIA").readOnly    = false;
            widget.currentRow.HRJORNADADIARIA                = "";
            widget.currentRow.HRDESCANSO                     = "";
        }
    };
    
    /**
    * Habilita a widget de escala de trabalho quando clicado em avançar
    * @params{Object} widget widget equipe de trabalho
    **/
    this.abreWidgetEscalaTabalho = function(widget){
        var gridEscalaTrab = templateManager.container.getWidget("gridEscalaTrab");
        if(widget.isValid()){
            if(widget.currentRow.HRINTERVALO > widget.currentRow.HRJORNADADIARIA){
                ScreenService.showMessage("Operação bloqueada!<br>Tempo de intervalo excede o tempo da jornada de trabalho.");
            } else {
                widget.enabled = false;
                gridEscalaTrab.enabled = true;
                widget.openTab("gridEscalaTrab");
                this.acessibilidadeWdgEscalaTrab(true);
                gridEscalaTrab.currentRow.NMRECURSOSAGENDCONT = widget.currentRow.NMRECURSO;
                gridEscalaTrab.currentRow.HRINICIAL           = widget.currentRow.HRPRIMEIRAENTRADA;
                gridEscalaTrab.currentRow.HRINTERVALO         = widget.currentRow.HRINTERVALO;
                this.gerarEscalaEmLista(gridEscalaTrab);
            }
        }
    };
    
    /**
     * Volta para a widget equipe de trabalho
     * durante a inserção/edição do dados
     * @params{Object} widget widget escala de trabalho
    **/
    this.voltarWidgetEquipeTrab  = function(widget){
        widget.enabled           = false;
        widget.dataSource.data   = [];
        
        var equipeDeTrabalho     = templateManager.container.getWidget("equipeDeTrabalho");
        equipeDeTrabalho.enabled = true;
        equipeDeTrabalho.activate().then(function(){
            setTimeout(function(){
                angular.element("label.zh-tab-cell")[3].click(); //pega a 3 widget (equipedetrabalho)
            }, 100);
        });
        //wdgAgend.openTab("equipeDeTrabalho");// A função não consegue abrir a widget pai
    };
    
    /**
     * Cancela o popup
     * @params{Object} widget widget popup escala de trabalho
    **/
    this.cancelarPopup = function(widget){
        widget.isVisible = false;
        widget.dataSource.data = [];
        ScreenService.closePopup();
    };
    
    /**
     * Gera a row da escala que será adicionada no dataSource 
     * da widget escala de trabalho
     * @params tempoJornada{string} tempo da jornada de trabalho,
     * @params tempoDescanso{string} tempo de descaso entre jornadas ,
     * @params inicioExpediente{moment} inicio do expediente,
     * @params fimExpediente{moment} fim do expediente,
    **/
    this.geraRowEscala = function(tempoJornada, tempoDescanso, inicioExpediente, fimExpediente){
        var widget = templateManager.container.getWidget("equipeDeTrabalho");
        var  row = {
            "HRDESCANSO"         : tempoJornada,
            "HRJORNADADIARIA"    : tempoDescanso,
            "HRINICIAL"          : fimExpediente ? inicioExpediente.format("HH:mm") : "",
            "HRFINAL"            : fimExpediente ? fimExpediente.format("HH:mm") : "",
            "DTESCALAENTRADA"    : inicioExpediente.format("DD/MM/YYYY"),
            "DTESCALASAIDA"      : fimExpediente ? fimExpediente.format("DD/MM/YYYY") : "",
            "DIASDASEMANA"       : widget.currentRow.DIASDASEMANA,
            "NRPARCNEPESSOA"     : (widget.currentRow.IDTPRECURSO === "1") ? widget.currentRow.CHAVE : null,
            "NRSEQCONTRATO"      : templateManager.container.getWidget("gridAgendamentoRecursos").currentRow.NRSEQCONTRATO,
            "IDDESCANSOREMU"     : widget.currentRow.IDDESCANSOREMU,
            "DTINIAGENDRECURSO"  : widget.currentRow.DTINIAGENDRECURSO,
            "DTFIMAGENDRECURSO"  : widget.currentRow.DTFIMAGENDRECURSO,
            "HRINTERVALO"        : fimExpediente ? widget.currentRow.HRINTERVALO : ""
        };
            
        return row;
    };
    
    /**
     * Verifica os dias da semana na qual o recurso trabalha 
     * @params{Object} widget widget escala de trabalho
     * @params{String} dia dia da semana
    **/
    this.validaDiasDaSemana = function(dia){
        var wdgEvolucao = templateManager.container.getWidget("equipeDeTrabalho");
        var valida = -1;
        
        switch(dia){
            case "Domingo": 
                valida = wdgEvolucao.currentRow.DIASDASEMANA.indexOf("IDREPETDOMINGO");
                break;
            case "Segunda-feira": 
                valida = wdgEvolucao.currentRow.DIASDASEMANA.indexOf("IDREPETSEGUNDA");
                break;
            case "Terça-feira": 
                valida = wdgEvolucao.currentRow.DIASDASEMANA.indexOf("IDREPETTERCA");
                break;
            case "Quarta-feira": 
                valida = wdgEvolucao.currentRow.DIASDASEMANA.indexOf("IDREPETQUARTA");
                break;
            case "Quinta-feira": 
                valida = wdgEvolucao.currentRow.DIASDASEMANA.indexOf("IDREPETQUINTA");
                break;
            case "Sexta-feira": 
                valida = wdgEvolucao.currentRow.DIASDASEMANA.indexOf("IDREPETSEXTA");
                break;
            case "Sábado": 
                valida = wdgEvolucao.currentRow.DIASDASEMANA.indexOf("IDREPETSABADO");
                break;
        }
        
        return valida;
    };
    
    /**
     * Valida se o período de trabalho entrou ou não no 
     * horário de verão
    **/
    this.validaHorarioVerao = function(fimExpediente, inicioExpediente, hrDescanso){
        hrDescanso     = parseInt(hrDescanso.split(":")[0]);
        var hrIniDia   = parseInt(moment(inicioExpediente, "DD/MM/YYYY HH:mm").format("HH"));
        var diaIni     = parseInt(moment(inicioExpediente, "DD/MM/YYYY HH:mm").format("DD"));
        var hrFimDia   = parseInt(moment(fimExpediente, "DD/MM/YYYY HH:mm").format("HH"));
        var diafim     = parseInt(moment(fimExpediente, "DD/MM/YYYY HH:mm").format("DD"));
        var duracao    = hrDescanso;
        var diferenca  = 0;
        
        if(hrIniDia >= hrFimDia)
            duracao = ((Math.abs((diaIni - diafim)) * 24) - hrFimDia) + hrIniDia;

        if(Math.abs(duracao) !== hrDescanso)
            diferenca = hrDescanso - duracao;

        return diferenca;
    };
    
    
    /**
     * Gera a Escala de trabalho do recurso
     * @params{Object} widget widget escala de trabalho
    **/
    this.gerarEscalaEmLista = function(widget){
        var wdgEquipe = templateManager.container.getWidget("equipeDeTrabalho");
        
        var inicial          = wdgEquipe.currentRow.DTINIAGENDRECURSO;
        var final            = wdgEquipe.currentRow.DTFIMAGENDRECURSO;
        var dias             = this.getDias(inicial, final);
        var inicioExpediente = moment(inicial + " " + widget.currentRow.HRINICIAL, "DD/MM/YYYY HH:mm");
        var tempoDescanso    = wdgEquipe.currentRow.HRDESCANSO;
        var jornada          = wdgEquipe.currentRow.HRJORNADADIARIA;
        var intervalo        = widget.currentRow.HRINTERVALO;
        var fimExpediente    = null;
        var dadosEscala      = [];

        jornada   = jornada.split(":");
        intervalo = intervalo.split(":");
        var tempoJornada = parseInt(jornada[0]) + parseInt(intervalo[0]) + ":" + parseInt(jornada[1]) + parseInt(intervalo[1]);

        if(widget.currentRow.HRINICIAL != "" && widget.isValid()){
            for (var i = 0; i < dias; i++) {
                fimExpediente = moment(inicioExpediente.clone().add(tempoJornada.split(":")[0], "h").format("DD/MM/YYYY HH:mm") , "DD/MM/YYYY HH:mm");
                fimExpediente.add(tempoJornada.split(":")[1], "m");
                
                if(this.getDias(fimExpediente.format("DD/MM/YYYY"), final) >= 0 || this.getDias(inicioExpediente.format("DD/MM/YYYY"), final) >= 0){
                    var diaSemana   = inicioExpediente.format("dddd");
                    var verificaDia = this.validaDiasDaSemana(diaSemana);
                    var row; 
                    
                    if(verificaDia >= 0)
                        row = this.geraRowEscala(tempoJornada, tempoDescanso, inicioExpediente, fimExpediente);
                    else 
                        row = this.geraRowEscala(tempoJornada, tempoDescanso, inicioExpediente, null);
                    
                    dadosEscala.push(row);
                    inicioExpediente = moment(fimExpediente.clone().add(tempoDescanso.split(":")[0], "h").format("DD/MM/YYYY HH:mm") , "DD/MM/YYYY HH:mm");
                    inicioExpediente.add(tempoDescanso.split(":")[1], "m");
                    var validaDesc = this.validaHorarioVerao(fimExpediente, inicioExpediente, tempoDescanso);
                    inicioExpediente.add(validaDesc, "h");
                    
                    var diasDescanso = inicioExpediente.diff(fimExpediente, "days");
                    if(diasDescanso >= 1){
                        for (var j = diasDescanso; j >= 1; j--){
                            var inicioDescanso = inicioExpediente.clone().subtract(j, "d");
                            if(inicioDescanso.format("DD/MM/YYYY") <= moment(final, "DD/MM/YYYY").format("DD/MM/YYYY")){
                                row = this.geraRowEscala(tempoJornada, tempoDescanso, inicioDescanso, null);
                                dadosEscala.push(row);
                            }
                        }
                        
                    }
                }
            }
            
            widget.dataSource.data = dadosEscala;
            widget.list();
        }
    };

    /**
     * Retorna a quantidade de dias a partir de uma data final e inicial
     * @params{Date} inicial data inicial
     * @params{Date} final data final
    **/
    this.getDias = function(inicial, final){
        var dateInicial = moment(inicial, "DD/MM/YYYY");
        var dateFinal   = moment(final, "DD/MM/YYYY");
        return (dateFinal.diff(dateInicial)/1000/60/60/24)+1;
    };
    
    /**
     * Altera acessibilidade das actions da widget Escala de trabalho
     * @params{Object} widget widget equipe de trabalho
    **/
    this.acessibilidadeWdgEscalaTrab = function(habilitar){
        var gridEscalaTrab = templateManager.container.getWidget("gridEscalaTrab");
        gridEscalaTrab.getAction("Voltar_wdgEquipetrab").isVisible   = habilitar;
        gridEscalaTrab.getAction("gerarEscalaTrabalho").isVisible    = habilitar;
        gridEscalaTrab.getAction("salvarTudo").isVisible             = habilitar;
        gridEscalaTrab.getAction("VoltarEscalaTrab").isVisible       = !habilitar;
        
        if(habilitar){
            gridEscalaTrab.floatingControl = false;
        } else {
            gridEscalaTrab.floatingControl = {
                "customizationAction": {
                    "filter": false,
                    "layout": false,
                    "view": false
                }
            };
        }
    }; 
    
    /**
     * Volta a widget escala de trabalho para seu estado normal
    **/
    this.voltarWidgetEquipeTrabGrid = function(){
        var gridEscalaTrab = templateManager.container.getWidget("gridEscalaTrab");
        gridEscalaTrab.template = "widget/grid.html";
        gridEscalaTrab.list();
        this.acessibilidadeWdgEscalaTrab(false);
    };
    
    /**
     * Verifica se a alteração ocorrida na escala de trabalho 
     * é proveniente de uma edição da widget equipe + escala de trabalho 
     * ou somente da escala
     * @params{Object} widgetEscala widget escala de trabalho
     * @params{String} data da alteração
     * @params{String} data final do expediente
     * @params{String} hora horario final do expediente
     * @params{Object} widget widget popup de edição da escala
    **/
    this.validaEdicaoEscala = function(widgetEscala, dataAltera, dtfimExpediente, hrfimExpediente, widget, intervalo){
        var rows = {};
        var wdgEquipeDeTrabalho = templateManager.container.getWidget("equipeDeTrabalho");
        if(widgetEscala.parent.mode === "edit"){
            dataAltera = moment(dataAltera, "DD/MM/YYYY").format("DD/MM/YYYY");
            widgetEscala.dataSource.data.forEach(function(dado){
                dado.DTESCALAENTRADA = moment(dado.DTESCALAENTRADA, "DD/MM/YYYY").format("DD/MM/YYYY");
                if(dado.DTESCALAENTRADA === dataAltera){
                    dado.DTESCALAENTRADA     = dataAltera;
                    dado.DTESCALASAIDA       = dtfimExpediente;
                    dado.HRFINAL             = hrfimExpediente;
                    dado.HRINICIAL           = widget.currentRow.HRINICIAL;
                    dado.NRRECURSOSAGENDCONT = wdgEquipeDeTrabalho.currentRow.NRRECURSOSAGENDCONT;
                    dado.HRINTERVALO         = intervalo;
                }
            });
        } else {
            rows = {
                "DTESCALAENTRADA"    : dataAltera,
                "DTESCALASAIDA"      : dtfimExpediente,
                "HRFINAL"            : hrfimExpediente,
                "HRINICIAL"          : widget.currentRow.HRINICIAL,
                "NRRECURSOSAGENDCONT": wdgEquipeDeTrabalho.currentRow.NRRECURSOSAGENDCONT,
                "HRINTERVALO"        : intervalo
            };

            widgetEscala.dataSource.data.forEach(function(dado){
                if(moment(dado.DTESCALAENTRADA, "DD/MM/YYYY").format("DD/MM/YYYY") === dataAltera){
                    if(!dado.HRINICIAL && widget.currentRow.HRINICIAL)
                        rows.NEWDATATRAB = true;
                    else
                        rows.NEWDATATRAB = false;
                }
            });  
        }
        
        return rows;
    };
    
    /**
     * Faz a validação da hora e data para verificar a disponibilidade do recurso antes 
     * de permitir a alteração da escala de trabalho 
     * @params{Object} widget Popup alteração da escala
    **/
    this.validaHoraAlteracaoEscala = function(widget){
        var widgetEquipe = templateManager.container.getWidget("equipeDeTrabalho");
        if(!widget.currentRow.HRINICIAL){
            this.alterarEscalaTrab(widget);
        } else {
            this.validaDisponibilidadeRec(widgetEquipe, widget.currentRow.DTESCALAENTRADA.length, widget.currentRow.DTESCALAENTRADA, null).then(function(response){
                if(response.dataset && response.dataset.fct_recursosagendcont_widget && response.dataset.fct_recursosagendcont_widget.DISPONIBILIDADE){
                    this.alterarEscalaTrab(widget);
                } else {
                    widget.currentRow.DTESCALAENTRADA = [];
                    ScreenService.showMessage("Operação bloqueada.<br>Alguns Recursos selecionados não estarão disponíveis para o periodo selecionado.");
                }
            }.bind(this));
        }
    };
    
    /**
     * Verifica se o usuário deseja gerar a escala de trabalho novamente
     * @params{Object} widget widget popup escala de trabalho
    **/
    this.alterarEscalaTrab = function(widget){
        var widgetEscala   = templateManager.container.getWidget("gridEscalaTrab");
        var datasAlteracao = widget.currentRow.DTESCALAENTRADA;
        var rows = [];
        
        if(widget.isValid()){
            ScreenService.confirmMessage("Alterações feitas na Escala de Trabalho podem causar mudanças na quantidade de horas semanais trabalhadas pelo recurso. Deseja continuar?", "question", function(){
                datasAlteracao.forEach(function(dataAltera){
                    if(!widget.currentRow.HRINICIAL && !widget.currentRow.HRINTERVALO){
                        rows.push(this.validaEdicaoEscala(widgetEscala, dataAltera, "", "", widget, ""));
                    } else { 
                        if(!widget.currentRow.HRINICIAL)
                            widget.currentRow.HRINICIAL = widgetEscala.parent.currentRow.HRPRIMEIRAENTRADA;

                        var jornada;
                        var validaDia = 0;
                        var intervalo = widget.currentRow.HRINTERVALO ? widget.currentRow.HRINTERVALO : widgetEscala.parent.currentRow.HRINTERVALO;
                        intervalo = intervalo.split(":");
                        
                        if(widget.currentRow.HRFINAL === ""){
                            jornada = widgetEscala.parent.currentRow.HRJORNADADIARIA.split(":");
                            var tempoJornada = parseInt(jornada[0]) + parseInt(intervalo[0]) + ":" + parseInt(jornada[1]) + parseInt(intervalo[1]);
                            jornada = moment(tempoJornada, "HH:mm");
                            if(parseInt(widgetEscala.parent.currentRow.HRJORNADADIARIA.split(":")[0]) >= 24)
                                validaDia = parseInt(parseInt(widgetEscala.parent.currentRow.HRJORNADADIARIA.split(":")[0])/24);
                        } else {
                            var horaInicio = moment(widget.currentRow.HRINICIAL, "HH:mm");
                            var horaFim    = moment(widget.currentRow.HRFINAL, "HH:mm");
                            if(horaFim.format("HH:mm") > horaInicio.format("HH:mm")){
                                horaFim.subtract(horaInicio.format("HH"), "h");
                            } else {
                                validaDia += 1;
                                horaFim.add((24 - horaInicio.format("HH")), "h");
                            }
                            
                            horaFim.add(intervalo[0], "hours");
                            horaFim.add(intervalo[1], "minutes");
                            jornada = horaFim.subtract(horaInicio.format("mm"), "m");
                        }
                        
                        var diaInicial    = moment(dataAltera, "DD/MM/YYYY");
                        var fimExpediente = moment((diaInicial.format("DD/MM/YYYY") + " " + widget.currentRow.HRINICIAL), "DD/MM/YYYY HH:mm");
                        fimExpediente.add(jornada.format("HH"), "h");
                        fimExpediente.add(jornada.format("mm"), "m");
                        fimExpediente.add(validaDia, "d");
                        rows.push(this.validaEdicaoEscala(widgetEscala, dataAltera, fimExpediente.format("DD/MM/YYYY"), fimExpediente.format("HH:mm"), widget, intervalo.join(":")));
                    }
                }.bind(this));
                if(widgetEscala.parent.mode === "view"){
                    widgetEscala.dataSource.data     = rows;
                    widgetEscala.currentRow.__is_new = false;
                    this.salvarEscalaTrabalho(widgetEscala, null, "EDIT").then(function(){
                        widgetEscala.activate();
                        widgetEscala.reload();
                    });
                } else {
                    widgetEscala.currentRow.__is_new = true;
                }
                this.cancelarPopup(widget);
            }.bind(this));
        }
    };
    
    /**
     * Abre um popup para editar a escala de trabalho gerada
     * @params{Object} widget widget escala de trabalho
    **/
    this.editarEscalaDeTrabalho = function(widget){
        var popupEscala                  = templateManager.container.getWidget("popupEscalaTrablho");
        var fieldPopupEscala             = popupEscala.getField("DTESCALAENTRADA");
        fieldPopupEscala.dataSource.data = [];
        
        widget.dataSource.data.forEach(function(dado){
            fieldPopupEscala.dataSource.data.push(
                { 
                    "DTESCALAENTRADA": moment(dado.DTESCALAENTRADA, "DD/MM/YYYY").format("DD/MM/YYYY"),
                    "DIASEMANA"      : moment(dado.DTESCALAENTRADA, "DD/MM/YYYY").format("dddd"),
                    "HRINICIAL"      : moment(dado.HRINICIAL, "HH:mm").format("HH:mm")
                }
            ); 
        });
        
        ScreenService.openPopup(popupEscala).then(function(){
            popupEscala.newRow();
            popupEscala.currentRow.HRINTERVALO = templateManager.container.getWidget("equipeDeTrabalho").currentRow.HRINTERVALO;
        });
    };
    
    /**
     * Salva os dados da tela de escala de trabalho 
     * @params{Object} widget widget escala de trabalho
    **/
    this.salvarEscalaTrabalho = function(widget, response, metodo){
        var deferred = ZHPromise.defer();
        var widEquipe = templateManager.container.getWidget("equipeDeTrabalho");
        var nrRecursosAgendCont = (!widEquipe.currentRow.__is_new) ? widEquipe.currentRow.NRRECURSOSAGENDCONT : response.dataset.NRRECURSOSAGENDCONT;  
        var diasSemais  = ["Domingo","Segunda-feira", "Terça-feira", "Quarta-feira", "Quinta-feira", "Sexta-feira", "Sábado"];
        widget.dataSource.data.forEach(function(dado){
            dado.NRPARCNEPESSOA      = (widEquipe.currentRow.IDTPRECURSO == "1") ? widEquipe.currentRow.CHAVE : null;
            dado.NRRECURSOSAGENDCONT = nrRecursosAgendCont;
            dado.DIASDASEMANA        = metodo === "INSERT" ? widEquipe.currentRow.DIASDASEMANA : diasSemais.indexOf(moment(dado.DTESCALAENTRADA, "DD/MM/YYYY").format("dddd")) + 1;
            dado.NRPARCNEPESSOA      = (widEquipe.currentRow.IDTPRECURSO == "1") ? widEquipe.currentRow.CHAVE : null;
            dado.NRSEQCONTRATO       = templateManager.container.getWidget("gridAgendamentoRecursos").currentRow.NRSEQCONTRATO;
            dado.IDDESCANSOREMU      = widEquipe.currentRow.IDDESCANSOREMU;
            dado.DTINIAGENDRECURSO   = moment(widEquipe.currentRow.DTINIAGENDRECURSO, "DD/MM/YYYY").format("DD/MM/YYYY");
            dado.DTFIMAGENDRECURSO   = moment(widEquipe.currentRow.DTFIMAGENDRECURSO, "DD/MM/YYYY").format("DD/MM/YYYY");
        });
        
        if(metodo === "INSERT"){
            var params = {
                "NRRECURSOSAGENDCONT": nrRecursosAgendCont,
                "DTENTRADA": widEquipe.currentRow.DTINIAGENDRECURSO
            };
            
            General.callRestEngine("product#/fct_escalarecursosagendcont/deletarEscala", "Row", params).then(function(response) {
                General.callRestEngine("product#/fct_escalarecursoagendcont/saveEscalaTrabalho", "DataSet", widget.dataSource.data).then(function(){
                    deferred.resolve();
                }).catch(function(error){
                    deferred.reject();
                    Utilities.showErrorMessage(error);
                }.bind(this));
            }.bind(this)).catch(function(error){
                this.voltarAcessibilidadeWdgs(widEquipe, widget);
                deferred.reject();
                Utilities.showErrorMessage(error);
            }.bind(this));
                
        } else {
            General.callRestEngine("product#/fct_escalarecursoagendcont/saveEdicaoEscalaTrabalho", "DataSet", widget.dataSource.data).then(function(){
                deferred.resolve();
            }).catch(function(error){
                this.voltarAcessibilidadeWdgs(widEquipe, widget);
                deferred.reject();
                Utilities.showErrorMessage(error);
            }.bind(this));
        }
        
        return deferred.promise;
    };
    
    /**
     * Salva todos os dados das telas Escala de trabalho e Equipe de trabalho
     * @params{Object} widget widget escala de trabalho
    **/
    this.salvarEquipeEscalaTabalho = function(widget){
        var widEquipe = templateManager.container.getWidget("equipeDeTrabalho");
        this.salvarEquipeDeTrabalho(widEquipe).then(function(response){
            this.salvarEscalaTrabalho(widget, response, "INSERT").then(function(){
            }.bind(this))
            .catch(function(error){
                console.error(error);
                ScreenService.showMessage("Operação bloqueada.<br>Falha ao inserir Escala de Trabalho.");
            }.bind(this));
        }.bind(this))
        .catch(function(error){
            this.voltarAcessibilidadeWdgs(widEquipe, widget);
            console.error(error);
            Utilities.showErrorMessage(error);
        });
    };
    
    /**
     * Volta as widgets da tela de equipe de trabalho
     * para o estado de visualização ou list
     * @params{Object} widEquipe widget equipe de trabalho
     * @params{Object} widget widget escala de trabalho
    **/
    this.voltarAcessibilidadeWdgs = function(widEquipe, widget){
        this.acessibilidadeWdgEscalaTrab(false);
        this.voltarWidgetEquipeTrabGrid();
        widEquipe.activate();
        widEquipe.openTab(widEquipe.name);
        widEquipe.enabled = true;
        widEquipe.widgets.forEach(function(wid){
            wid.enabled = true;
        });
    };
    
    this.abrirWidgetSincronizarRecurso = function() {
        var wdgSincronizarRecurso = templateManager.container.getWidget("wdgSincronizarRecurso");
        ScreenService.closePopup();
        ScreenService.openPopup(wdgSincronizarRecurso).then(function() {
            wdgSincronizarRecurso.dataSource.data = [];
            wdgSincronizarRecurso.newRow();
            wdgSincronizarRecurso.currentRow.DTREFERENCIA = moment().format('DD/MM/YYYY');
        
        });
    };
    /**
     * Efetua as validações necessárias dos campos de data 
     * na tela de equipe de trabalho
     * @params{Object} widget - widget de equipe de trabalho
    **/
    this.validarFieldDeDatasEquipeTrabalho = function(widget, nameField) {
        if(widget.currentRow[nameField])
            this.validarData(widget, (nameField == 'DTINIAGENDRECURSO')); 
            
        this.filtraRecursoEquipeTrab(widget, nameField, 'CHAVE'); 
        var diasSemana = widget.currentRow.DIASDASEMANA;
        var readOnly   = !(widget.currentRow.DTINIAGENDRECURSO && widget.currentRow.DTFIMAGENDRECURSO && (diasSemana && diasSemana.length > 0));
        var fieldChave = widget.getField("CHAVE");
        
        if(!readOnly) {
            var index;
            fieldChave.dataSourceFilter.forEach(function(filter) {
                index = diasSemana.indexOf(filter.name);
                filter.value = (index != -1) ? 'S': 'N'; 
            });
            
            fieldChave.dataSourceFilter[7].value = widget.currentRow.DTINIAGENDRECURSO; 
            fieldChave.dataSourceFilter[8].value = widget.currentRow.DTFIMAGENDRECURSO;
            fieldChave.reload(); 
        }
        
        fieldChave.readOnly = readOnly;
        if(readOnly) widget.currentRow.CHAVE = "";
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Sm00015004_Agendamento_Recurso_Controller', Sm00015004_Agendamento_Recurso_Controller);
});


// FILE: ./assets/js/controllers/Sm00015005_Motivo_Apropriacao_Controller.js
function Sm00015005_Motivo_Apropriacao_Controller(General, ScreenService, templateManager ) {
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget
    */
    this.salvarMotivoAprop = function(widget){
        
        widget.currentRow.__columns_to_return = ["NRMOTIVOAGENDA", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset && response.dataset.columnsToReturn[0]){
                widget.currentRow.NRMOTIVOAGENDA = response.dataset.columnsToReturn[0].NRMOTIVOAGENDA;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
            
        }.bind(this));
        
    };
    
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00015005_Motivo_Apropriacao_Controller', Sm00015005_Motivo_Apropriacao_Controller);
});

// FILE: ./assets/js/controllers/Sm00015006_Status_Apropriacao_Controller.js
function Sm00015006_Status_Apropriacao_Controller(General) {
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget
    */
    this.salvarStatusAprop = function(widget) {
        
        widget.currentRow.__columns_to_return = ["NRSTATAPROAGEN", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response) {
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                widget.currentRow.NRSTATAPROAGEN = response.dataset.columnsToReturn[0].NRSTATAPROAGEN;
                widget.currentRow.NRORG          = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO     = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO  = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
        });
    };
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00015006_Status_Apropriacao_Controller', Sm00015006_Status_Apropriacao_Controller);
});

// FILE: ./assets/js/controllers/sm00015007_calendario_agendamento_Controller.js
function sm00015007_calendario_agendamento_Controller(General, ScreenService, Utilities,templateManager, ApplicationContext) {
    var backUpactions;
    var actionReturnCalendar;
    var filterRow = {};
    
     /**
     *Fecha o Swipe e ativa a widget da tela de calendário
     * 
     * @param {object} widget widget 
    */
    this.Fechar = function(widget){
        var widgetCalendar = templateManager.container.getWidget('widCalendario');
        ScreenService.closeSwipe();
        widgetCalendar.activate();
     };
     
     /**
     *Abre o card ao clicar em um data no calendário
     * 
     * @param {null} nulo
    */
     this.openAndEditSwipeCardView = function(){
        var widgetCalendar = templateManager.container.getWidget('widCalendario');
        var widgetNewCommitment = templateManager.container.getWidget('WDGT_NEW_COMMITMENT');
        
        var cardCurrentRow = angular.copy(widgetCalendar.getEventsWidget().currentRow);
        
        this.getActionReturnCalendar();
        this.returnDefaultActionsContainer();
        
        ScreenService.openSwipe(widgetNewCommitment).then(function(){

            widgetNewCommitment.list();
            widgetNewCommitment.template = 'widget/form-view.html';
            widgetNewCommitment.activate();

            widgetNewCommitment.currentRow = cardCurrentRow;
            
        }.bind(this));
        
        widgetNewCommitment.reload(); 
        
    };
     /**
     *Retorna uma instância do objeto card para a função que abre o card
     * 
     * @param {null} nulo 
    */
    this.getActionReturnCalendar = function(){
        
        actionReturnCalendar = templateManager.container.actions;
        
    };
     /**
     *Retorna uma ação para a propriedade action do container
     * 
     * @param {null} nulo 
    */
    this.returnDefaultActionsContainer = function(){
        templateManager.container.actions = backUpactions;    
    };
    
    /**
     *Função que abre o filtro na widget mapa de agendamento
     * 
     * @param {nulo} widget de 
     * Mapa
     */
    this.openFilter = function(widgetCalendar) {
        var ZhFilterPreferences = ApplicationContext.ZhFilterPreferences;
        ZhFilterPreferences.openFilterWidget(widgetCalendar, filterRow);
    };
    
    
    /**
     * Guarda os dados da última filtragem 
     * @params{Object} widgetFilter widget de filtro
    **/
    this.backupFilter = function(widgetFilter){
        var fieldsToApplyDefaultValue = widgetFilter.fields;
        fieldsToApplyDefaultValue.forEach(function(fieldName) {
            var field = widgetFilter.getField(fieldName.name);
            if (field) {
                filterRow[fieldName.name] = field.value();
            }
        }.bind(this));
    };

}
Configuration(function(ContextRegister)
{
    ContextRegister.register('sm00015007_calendario_agendamento_Controller', sm00015007_calendario_agendamento_Controller);
});

// FILE: ./assets/js/controllers/Sm00019000_RelatorioCentroCusto_Controller.js
function Sm00019000_RelatorioCentroCusto_Controller(templateManager, ScreenService, ZhFilterPreferences, General, Utilities, ZHPromise, localStorageEngine) {
    
    var self = this;
    var containerLabel = null;
    
    var getWidgetRelatorioCentroCusto = function() {
        return templateManager.container.getWidget("gridCentroCusto");
    };
    
    this.setContainerLabel = function(container) {
        containerLabel = container.label;
    };
    
    this.wdgRelatorioOnEnterOnce = function(widget) {
        ZhFilterPreferences.openFilterWidget(widget);
        var dataAtual = moment().format("DD/MM/YYYY");
        this.buscarDataHoraUltimaAtualizacaoView().then(function(dataAtualizacao) {
            if (!dataAtualizacao || dataAtualizacao.split(" ")[0] != dataAtual) {
                self.atualizarView();
            }
        });
    };
    
    this.buscarDataHoraUltimaAtualizacaoView = function() {
        var deferred = ZHPromise.defer();
        
        localStorageEngine.download("product#/crud/fct_parametrogeral").then(function(dataset) {
            var data = dataset[0]["DTULTATUMVCCUSTOCONT"];
            if (data)
                self.atualizarLabelContainerDataAtualizacao(data);
            
            deferred.resolve(data);
        })
        .catch(function(error) {
            Utilities.showErrorMessage(error);
            deferred.reject(error);
        });
        
        return deferred.promise;
    };
    
    this.atualizarLabelContainerDataAtualizacao = function(data) {
        templateManager.container.label = containerLabel + " (Atualizado em " + data + ")";
    };
    
    this.atualizarView = function(reloadWidget) {
        if (reloadWidget === undefined) {
            reloadWidget = false;
        }

        ScreenService.changeLoadingMessage(ScreenService.i18n("Atualizando dados do relatório..."));
        General.callRestEngine("product#/fct_centro_custo_relatorio/atualizarView", "Row", {}).then(function() {
            self.buscarDataHoraUltimaAtualizacaoView();
            if (reloadWidget) {
                getWidgetRelatorioCentroCusto().reload();
            }
        })
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        })
        .finally(function() {
            ScreenService.restoreDefaultLoadingMessage();
        });
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register("Sm00019000_RelatorioCentroCusto_Controller", Sm00019000_RelatorioCentroCusto_Controller);
});

// FILE: ./assets/js/controllers/Sm00022000_Formacao_Preco_Servico_Controller.js
function Sm00022000_Formacao_Preco_Servico_Controller(General, ScreenService, templateManager, ZHPromise, Utilities) {
    
    /**
     * Salva o novo registro da página "Formação de Preço"
     * 
     * @param {object} widget widget
    */
    this.salvarFormPrecoServico = function(widget){
        
        widget.currentRow.__columns_to_return = ["NRFORMPRECOSERVICO", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset && response.dataset.columnsToReturn[0]){
                widget.currentRow.NRFORMPRECOSERVICO = response.dataset.columnsToReturn[0].NRFORMPRECOSERVICO;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
            
            ScreenService.closeSwipe();
            widget.reload();
        });
    };
    
    /**
     * Seta os campos Padrões para a página de cálculo da Formação de Preços.
     * 
     * @param {object} widget widget
    */ 
    
    this.setaCamposPadraoCalculo = function(widget,  elemSup, elemSupName){
        
        /* === Código da Formção === */
        
        var formacaoPreco = templateManager.container.getWidget("gridFormacaoPreco");
        
        var nrformprecoservico = formacaoPreco.currentRow.NRFORMPRECOSERVICO;
        
        if (elemSup != null) {
            widget.currentRow.NRCALCFORMPRECSERVPAI = elemSup;
        }
        
        if (elemSupName != null){
            widget.currentRow.DSREFECALCULOPAI = elemSupName;
        }
        
        /* === Ordenação === */
        
        var params = {
            "NRFORMPRECOSERVICO" : nrformprecoservico,
            "TABELA"             : null 
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/recebeOrdenacao', "Row", params).then(function(response){
          
            var ordenacao = response.dataset.ORDENACAO.IDORDENACAO;
              
            if (ordenacao == 0){
                ordenacao =  10;
            }
            else {
                ordenacao = ordenacao + 10;
            }
              
            widget.currentRow.IDORDENACAO = ordenacao;
        }, 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    /**
     * Salva o novo registro da página "Cálculo"
     * 
     * @param {object} widget widget
     */
    
    this.salvarCalculoPreco = function(widget){
        
        widget.currentRow.__columns_to_return = ["NRCALCFORMPRECSERV", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        widget.currentRow.NRFORMPRECOSERVICO = widget.currentRow.NRFORMPRECOSERVICO;
        
        var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
        
        this.preencheReferencia(widget);
        this.calculaItem(widget).then(function(){
                
            General.widgetSave(widget).then(function(response){
                if (response.dataset && response.dataset.columnsToReturn[0]){
                    widget.currentRow.NRCALCFORMPRECSERV = response.dataset.columnsToReturn[0].NRCALCFORMPRECSERV;
                    widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                    widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                    widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                    widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                }
                
                /* == Somatório dos Elementos Filhos ==*/
                if (tipoCalculo == 'S'){
                    var params = {
                        "NRCALCFORMPRECSERV" : widget.currentRow.NRCALCFORMPRECSERV,
                        "TABELA"             : null
                    };
                    General.callRestEngine('product#/fct_calcformprecserv_widget/calcSomatorioFilhos', "Row", params);  
                }
                
                ScreenService.closeSwipe();
                widget.reload();
            });
        });
    };
    
    /**
     * Funções antes de inserir
     * 
     * @param {object} widget widget
     */

    this.novoCalculo = function(widget) {
        if (widget.mode == "view") {
            var elemSup     = widget.currentRow.NRCALCFORMPRECSERV;
            var elemSupName = widget.currentRow.DSREFECALCULO;
            
            General.widgetNewRow(widget);
            this.habilitaExpressao(widget);
            this.bloquearDadosMedida(widget);
            this.setaCamposPadraoCalculo(widget, elemSup, elemSupName);
        } else {
            General.widgetNewRow(widget);
            this.bloquearDadosMedida(widget);
            this.habilitaExpressao(widget);
            this.setaCamposPadraoCalculo(widget, null, null);
        }
        
        this.bloquearDadosMedida(widget);
    };
    
    /**
     * Funções antes de Editar
     * 
     * @param {object} widget widget
     */
    
    this.EditarCalculo = function(widget) {
        General.widgetEdit(widget);
        this.habilitaExpressao(widget); 
    };
    
    
    
    /**
     * Realiza o controle de Percentual do field "VRPERCCALCULO"
     * 
     * @param {object} widget widget
    */
    this.Validapercentual = function(widget){
        
        var vrdesconto = widget.getField('VRPERCCALCULO').value();
        
        /* Aplica Max e Min Value*/
        
        if (vrdesconto < 0 || vrdesconto > 100) {    
            /* Desconto */
            widget.currentRow.VRPERCCALCULO = null;
        }
    };
    
    
    /**
     * habilita / desabilita o campo "Expressão"
     * 
     * @param {object} widget widget
     */
    
    this.habilitaExpressao = function(widget){
        widget.currentRow.IDTIPOCALCULO = (widget.currentRow.IDTIPOCALCULO) ? widget.currentRow.IDTIPOCALCULO : "V";
        var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
        if (tipoCalculo == 'E') {
            /* Expressão  */
            widget.getField("DSEXPRESSAOCALC").readOnly = false;
            widget.getField("DSEXPRESSAOCALC").validations = {'required':true};
            
            /* Percentual  */
            widget.getField("VRPERCCALCULO").readOnly = false;
            widget.getField("VRPERCCALCULO").validations = {};
            
            /* Valor Unitário  */
            widget.getField("VRUNITARIO").readOnly = true;
            widget.getField("VRUNITARIO").validations = {};
            widget.currentRow.VRUNITARIO = null;
            
            widget.getField("VRTOTAL").validations = {};
        } else if (tipoCalculo == 'S'){
            /* Expressão  */
            widget.getField("DSEXPRESSAOCALC").readOnly = true;
            widget.getField("DSEXPRESSAOCALC").validations = {};
            widget.currentRow.DSEXPRESSAOCALC = null;
            
            /* Percentual  */
            widget.getField("VRPERCCALCULO").readOnly = true;
            widget.getField("VRPERCCALCULO").validations = {};
            widget.currentRow.VRPERCCALCULO = null;
            
            /* Valor Unitário  */
            widget.getField("VRUNITARIO").readOnly = true;
            widget.getField("VRUNITARIO").validations = {};
            widget.currentRow.VRUNITARIO = null;
            
            widget.getField("VRTOTAL").validations = {};
        } else {
            /* Expressão  */
            widget.getField("DSEXPRESSAOCALC").readOnly = true;
            widget.getField("DSEXPRESSAOCALC").validations = {};
            widget.currentRow.DSEXPRESSAOCALC = null;
            
            /* Percentual  */
            widget.getField("VRPERCCALCULO").readOnly = true;
            widget.getField("VRPERCCALCULO").validations = {};
            widget.currentRow.VRPERCCALCULO = null;
            
            /* Valor Unitário  */
            widget.getField("VRUNITARIO").readOnly = false;
            widget.getField("VRUNITARIO").validations = {'required':true};
            
            widget.getField("VRTOTAL").validations = {'required':true};
        }
    };
    
    
    
     /**
     * Realiza o controle dos campos de "Medida" de acordo 
     * com o field "Utiliza Medida?"
     * 
     * @param {object} widget widget de 
     * Itens para pegar o field "IDUSAMEDIDA".
    */
    
    this.bloquearDadosMedida = function(widget){
        
        var usamedida = widget.getField('IDUSAMEDIDA').value();
        
        /* Bloqueia a Modalidade de pagamento*/
        if (usamedida == 'N') {
            
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = true;
            widget.getField('DSTPMEDIDASERV').setValue(null);
            widget.getField('DSTPMEDIDASERV').validations = {};
            
            widget.getField('CDTPMEDIDASERV').setValue(null);
            widget.getField('CDTPMEDIDASERV').validations = {};
            
            /* Quantidade de Medida */
            widget.getField('QTMEDIDACALC').readOnly = true;
            widget.getField('QTMEDIDACALC').setValue(null);
            widget.getField('QTMEDIDACALC').validations = {};
            
        } 
        /* Desbloqueia a Medida*/
        else {
            
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = false;
            widget.getField('DSTPMEDIDASERV').validations = {"required": true};
            
            /* Quantidade de Medida */
            widget.getField('QTMEDIDACALC').readOnly = false;
            widget.getField('QTMEDIDACALC').validations = {"required": true};
        }

    };
    /**
     * Realiza o cálculo do campo "Valor Unitário"
     * 
     * @param {object} widget widget
     */

    this.calculaItem = function(widget){
        var defered = ZHPromise.defer();
        
        var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
        
        /* == Expressão ==*/
        if (tipoCalculo == 'E') {
            this.calculaExpressao(widget).then(function(){
                
                this.calculaValorTotal(widget);
                defered.resolve();
            }.bind(this));
        }
        /* == Somatório dos Elementos Filhos ==*/
        else if (tipoCalculo == 'S'){
            
            defered.resolve();
        }
        /* == Valor Fixo ==*/
        else{
            this.calculaValorTotal(widget);
            defered.resolve();
        }
        
        return defered;
    };
    
    /**
     * Realiza o cálculo descrito no campo "Expressão" e retorna para o campo "Valor Unitário"
     * 
     * @param {object} widget widget
     */

    this.calculaExpressao = function(widget){
        var defered = ZHPromise.defer();
        
        var formacaoPreco = templateManager.container.getWidget("gridFormacaoPreco");
        var nrformprecoservico = formacaoPreco.currentRow.NRFORMPRECOSERVICO;
        
        var expressao = widget.currentRow.DSEXPRESSAOCALC;
        var percentual = widget.currentRow.VRPERCCALCULO;
        var vrunit = 0;
        
        var params = {
            "NRFORMPRECOSERVICO" : nrformprecoservico,
            "TABELA"             : null
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/buscaIdentValores', "Row", params).then(function(response){
          
            var identVal = response.dataset.IDENTVAL;
            
            identVal.forEach(function(identfica){
                
                var identificador = '['+identfica.IDITEMCALCULO+']';
                var valor = identfica.VRTOTAL;

                expressao = expressao.split(identificador).join(valor);    
            });
            
            expressao = expressao.split("%").join("/100");
            
            if (percentual != null){
               
               vrunit = eval(expressao);
               vrunit = (vrunit*percentual)/100;
            }
            else{
                
                vrunit = eval(expressao);
            }
            
            widget.currentRow.VRUNITARIO =  vrunit;
            
            defered.resolve();
        }, 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
            defered.reject();
        });
        return defered;
    };
    
  
    /**
     * Realiza a validação  do campo "identificador". Não podem haver dois identificadores iguais em uma mesma composição.
     * 
     * @param {object} widget widget
     */

    this.validaIdentificador = function(widget){

        var formacaoPreco = templateManager.container.getWidget("gridFormacaoPreco");
        var nrformprecoservico = formacaoPreco.currentRow.NRFORMPRECOSERVICO;
        var IdItemCalculo = widget.currentRow.IDITEMCALCULO;
        var gridCalculo = templateManager.container.getWidget("gridCalculo");
        var nrCalcFormPrecServ = gridCalculo.currentRow.NRCALCFORMPRECSERV;
        /* === Ordenação === */
        
        var params = {
            "NRFORMPRECOSERVICO": nrformprecoservico,
            "IDITEMCALCULO"     : IdItemCalculo,
            "NRCALCFORMPRECSERV": nrCalcFormPrecServ,
            "TABELA"            : null,
            "NRITEMORCAMENTO"   : null
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/validaIdentificador', "Row", params).then(function(response){

            var identificador = response.dataset.IDENTIFICADOR.VALIDA;
              
            if (identificador != 0){
                widget.currentRow.IDITEMCALCULO = null;
                ScreenService.showMessage('O identificador "'+IdItemCalculo+'" já está sendo utilizado para outro elemento.'); 
            }
            else{
                this.salvarCalculoPreco(widget);
            }
        }.bind(this), 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });    
    };
    
    /**
     * Realiza o cálculo do Valor Total
     * 
     * @param {object} widget widget
     */
     
    this.calculaValorTotal = function(widget){
        var vrUnit = widget.currentRow.VRUNITARIO;
        if (vrUnit) {
            vrUnit = vrUnit.toString();
            var primeiraPos = vrUnit.indexOf(".");
            var ultimaPos = vrUnit.lastIndexOf(".");
            if (primeiraPos == ultimaPos) {
                if (vrUnit.includes(",")) {
                    vrUnit = parseFloat(vrUnit.replace(".", "").replace(",", "."));
                }
                else {
                    vrUnit = parseFloat(vrUnit.replace(",", "."));
                }
            }
            else {
                var parteInicial = vrUnit.substr(0, primeiraPos + 1);
                var parteFinal = vrUnit.substr(primeiraPos + 1);
                vrUnit = parseFloat(parteInicial.replace(".", "") + parteFinal);
            }
        }
        else {
            vrUnit = 0;
        }
        
        var qtCalc    = widget.currentRow.QTCALCULO;
        var usamedida = widget.currentRow.IDUSAMEDIDA;
        var qtmedida  = widget.currentRow.QTMEDIDACALC;
        var vrTot     = 0;
        
        if (usamedida == 'S'){
            vrTot = (vrUnit*qtCalc)*qtmedida ; 
        } else {
            vrTot = vrUnit*qtCalc;
        }
        
        widget.currentRow.VRTOTAL = vrTot;
    };
    
    /**
     * Recebe o valor do campo de código da Formação para filtro de Data Source.
     * 
     * @param {object} widget widget
     */
    
    this.recebeValorFormacao = function(widget){
        
        
        var formacaoPreco = templateManager.container.getWidget("gridFormacaoPreco");
        var nrformprecoservico = formacaoPreco.currentRow.NRFORMPRECOSERVICO;
        var gridCalculo = templateManager.container.getWidget("gridCalculo");
        var nrCalcFormPrecServ = gridCalculo.currentRow.NRCALCFORMPRECSERV;
        
        widget.getField("DSREFECALCULOPAI").dataSourceFilter[0].value = nrformprecoservico;
        widget.getField("DSREFECALCULOPAI").dataSourceFilter[1].value = nrCalcFormPrecServ;
        widget.getField("DSREFECALCULOPAI").reload();
    };
    
    
     /**
     * Exclui os elementos e os filhos.
     * 
     * @param {object} widget widget
     */
    
    this.deletarPaieFilhos = function(widget){
         
         var gridCalculo = templateManager.container.getWidget("gridCalculo");
        var nrCalcFormPrecServ = gridCalculo.currentRow.NRCALCFORMPRECSERV;
         
         ScreenService.confirmMessage("A exclusão deste elemento também irá excluir os elementos inferiores a ele. Confirma exclusão?", "question", function() {
                                
            var params = {
                "NRCALCFORMPRECSERV" : nrCalcFormPrecServ
            };
            
            General.callRestEngine('product#/fct_calcformprecserv_widget/deletarPaieFilhos', "Row", params).then(function(){
            
                ScreenService.showMessage('Exclusão realizada com sucesso.'); 
                ScreenService.closeSwipe();
                widget.reload();
              
            }, 
            function(error) {
                console.log(error);
                ScreenService.showMessage(error.data.error);
            });    
            
        }.bind(this));
    };
    
    this.preencheReferencia = function(widget){
        
        var referencia =  widget.currentRow.DSREFECALCULO;
        var identificador =  widget.currentRow.IDITEMCALCULO;
        var elemento =  widget.currentRow.NMELEMENTO;
        
        if (referencia == "") {
            
            if(identificador != ""){
                
                referencia = '['+identificador+'] - ';
            }
            
            if(elemento != ""){
                
                referencia = referencia+elemento;
            }
            
            widget.currentRow.DSREFECALCULO = referencia;
        }
        
    };
    
        /**
     * Abre a widget de Cadastramento Automático.
     * 
     * @param {object} widget
     */
    this.abrirWdgCadastramentoAutomatico = function(widget) {
        ScreenService.closePopup();
        var wdgCadAuto = templateManager.container.getWidget("wdgCadastramentoAutomatico");
        wdgCadAuto.isVisible = true;
        ScreenService.openSwipe(wdgCadAuto).then(function() {
            wdgCadAuto.edit();
        });
    };
    
       /**
     * Fecha a widget de Cadastramento Automático.
     * 
     * @param {object} widget
     */
    this.fecharWdgCadastramentoAutomatico = function(widget) {
        widget.isVisible = false;
        this.limpaCamposCadAuto(widget);
        ScreenService.goBack();
        templateManager.container.getWidget("gridCalculo").activate();
    };
    
    /**
     * Salva o Cadastramento Automático, inserindo múltiplos elementos de uma vez.
     * 
     * @param {object} widget
     */
    this.salvarCadastramentoAutomatico = function(widget) {
       
       var params = {
            "IDELEMENTO"      : widget.currentRow.IDELEMENTO,
            "NMSERVORCA"      : widget.currentRow.NMSERVORCA,
            "IDELEMENTOPAI"   : widget.currentRow.IDELEMENTOPAI,
            "IDITEMCALCULO"   : widget.currentRow.IDITEMCALCULO,
            "IDORDENAELEMENTO": widget.currentRow.IDORDENAELEMENTO,
            "DSEXPRESSAOCALC" : widget.currentRow.DSEXPRESSAOCALC,
            "IDTIPOCALCULO"   : widget.currentRow.IDTIPOCALCULO,
            "VRPERCCALCULO"   : widget.currentRow.VRPERCCALCULO,
            "FORMACAO"        : templateManager.container.getWidget("gridFormacaoPreco").currentRow.NRFORMPRECOSERVICO
        };
            
        General.callRestEngine('product#/fct_calcformprecserv_widget/cadastramentoAutomatico', "Row",params).then(function(){
            
            ScreenService.showMessage('Cadastramento automático realizado com sucesso.'); 
            this.fecharWdgCadastramentoAutomatico(widget);
            templateManager.container.getWidget("gridCalculo").reload();
          
        }.bind(this), 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        }); 
    };
    
    /**
     * A função "CopiarFormacaoPreco" executa a procedure
     * "FCT_COPIA_FORM_PRECO".
     */
    
    this.copiarFormacaoPreco = function(widget){
        
        var formacaoPreco = templateManager.container.getWidget("gridFormacaoPreco");
        var nrformprecoservico = formacaoPreco.currentRow.NRFORMPRECOSERVICO;
         
        var params = {
            "NRFORMPRECOSERVICO" : nrformprecoservico
        }; 
        
        General.callRestEngine('product#/fct_formprecoservico_widget/executarProcedureFctCopiaFormPreco', "Row", params).then(function(){
            //ScreenService.closePopup();
            ScreenService.showMessage('Cópia da formação de preço realizada com sucesso.'); 
            ScreenService.closeSwipe();
            widget.reload();
          
        }, 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });    
    };
    
    
    this.recalculo = function (ind, recalc, widget){
    
        if (ind <= recalc.length-1){
            /* == Expressão ==*/
            if (recalc[ind].IDTIPOCALCULO == 'E') {
                this.recalculaExpressao(recalc[ind].NRFORMPRECOSERVICO, recalc[ind].DSEXPRESSAOCALC, recalc[ind].VRPERCCALCULO).then(function(response){
                    var vrunit = response;
                    var qtCalc = recalc[ind].QTCALCULO;
                    var usamedida = recalc[ind].IDUSAMEDIDA;
                    var qtmedida = recalc[ind].QTMEDIDACALC;
                    var vrTot = 0;
                    
                    if (usamedida == 'S'){
                        vrTot = (vrunit*qtCalc)*qtmedida ; 
                    
                    }
                    else {
                        vrTot = vrunit*qtCalc;
                    }
                              
                    var params = {
                        "NRCALCFORMPRECSERV" : recalc[ind].NRCALCFORMPRECSERV,
                        "VRUNITARIO" : vrunit,
                        "QTCALCULO" : qtCalc,
                        "QTMEDIDACALC" : qtmedida,
                        "VRTOTAL" : vrTot
                    };
                                    
                    General.callRestEngine('product#/fct_calcformprecserv_widget/salvarRecalculo', "Row", params).then(function() {
                        this.recalculo(ind+1, recalc, widget);
                    }.bind(this),
                    function(error) {
                        console.log(error);
                        ScreenService.showMessage(error.data.error);
                    });
                    
                }.bind(this));
            }
            /* == Somatório dos Elementos Filhos ==*/
            else if (recalc[ind].IDTIPOCALCULO == 'S'){
                this.recalculaSomatorio(recalc[ind].NRCALCFORMPRECSERV).then(function(response){
                    var vrunit = response;
                    var qtCalc = recalc[ind].QTCALCULO;
                    var usamedida = recalc[ind].IDUSAMEDIDA;
                    var qtmedida = recalc[ind].QTMEDIDACALC;
                    var vrTot = 0;
                    
                    if (usamedida == 'S'){
                        vrTot = (vrunit*qtCalc)*qtmedida ; 
                    
                    }
                    else {
                        vrTot = vrunit*qtCalc;
                    }
                    
                    var params = {
                        "NRCALCFORMPRECSERV" : recalc[ind].NRCALCFORMPRECSERV,
                        "VRUNITARIO" : vrunit,
                        "QTCALCULO" : qtCalc,
                        "QTMEDIDACALC" : qtmedida,
                        "VRTOTAL" : vrTot
                    };
                    
                    General.callRestEngine('product#/fct_calcformprecserv_widget/salvarRecalculo', "Row", params).then(function() {
                        this.recalculo(ind+1, recalc, widget);   
                    }.bind(this),
                    function(error) {
                        console.log(error);
                        ScreenService.showMessage(error.data.error);
                    });   
                    
                }.bind(this));
            }
            /* == Valor Fixo ==*/
            else {
                this.recalculo(ind+1, recalc, widget); 
            }
        }
        else{
            ScreenService.closeSwipe();
            widget.reload();
        }

    };
    
    
    
    /**
     * A função "recalcular" realiza o recalculo do valor dos elementos.
     * 
     * @param {object} widget widget
     */
    
    this.recalcular = function(widget){
        
        var gridCalculo = templateManager.container.getWidget("gridCalculo");
        var nrCalcFormPrecServ = gridCalculo.currentRow.NRCALCFORMPRECSERV;
        
        var params = {
            "NRCALCFORMPRECSERV" : nrCalcFormPrecServ
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/recalculoElemento', "Row", params).then(function() {
            ScreenService.showMessage("Processo realizado com sucesso!");
            ScreenService.closeSwipe().then(function() {
                widget.reload();
            });
        })
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
        
    };
    
     /**
     * Realiza o cálculo descrito no campo "Expressão" e retorna para o campo "Valor Unitário"
     * 
     * @param {object} widget widget
     */

    this.recalculaExpressao = function(NrFormPrecoServico, expressao, percentual){
        var defered = ZHPromise.defer();
        var vrunit = 0;
        
        var params = {
            "NRFORMPRECOSERVICO" : NrFormPrecoServico,
            "TABELA"             : null
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/buscaIdentValores', "Row", params).then(function(response){
          
            var identVal = response.dataset.IDENTVAL;
            
            identVal.forEach(function(identfica){
                
                var identificador = '['+identfica.IDITEMCALCULO+']';
                var valor = identfica.VRTOTAL;

                expressao = expressao.split(identificador).join(valor);    
            });
            
            expressao = expressao.split("%").join("/100");
            
            
            if (percentual != null){
               
               vrunit = eval(expressao);
               vrunit = (vrunit*percentual)/100;
            }
            else{
                
                vrunit = eval(expressao);
            }

            
            defered.resolve(vrunit);
        }, 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
            defered.reject();
        });
        return defered;
    };
    
    
    this.recalculaSomatorio = function(nrCalcFormPrecServ){
        var defered = ZHPromise.defer();
        
        var params = {
            "NRCALCFORMPRECSERV" : nrCalcFormPrecServ
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/somatorioRecalculo', "Row", params).then(function(response){
            
            var vrUnit = response.dataset.SOMATORIO.VRSOMA;
            
            defered.resolve(vrUnit);    
        },
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
            defered.reject();
        });
        return defered;  
    };
    
    this.limpaCamposCadAuto = function(widget){
    
        widget.currentRow.IDELEMENTO = null;
        widget.currentRow.NMSERVORCA = null;
        widget.currentRow.IDELEMENTOPAI = null;
        widget.currentRow.IDORDENAELEMENTO = null;
        widget.currentRow.IDITEMCALCULO = null;
        
    };
    
    this.recalcularArvore = function(widget) {
        var formacaoPreco = templateManager.container.getWidget("gridFormacaoPreco");
        var nrformprecoservico = formacaoPreco.currentRow.NRFORMPRECOSERVICO;
        
        var params = {
            "NRFORMPRECOSERVICO" : nrformprecoservico,
            "TABELA"             : null,
            "NRITEMORCAMENTO"    : null
        };
        
        General.callRestEngine("product#/fct_calcformprecserv_widget/recalcularArvore", "Row", params).then(function() {
            ScreenService.closePopup();
            ScreenService.showMessage("Processo realizado com sucesso!");
            widget.reload();

        })
        .catch(function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });
    };
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00022000_Formacao_Preco_Servico_Controller', Sm00022000_Formacao_Preco_Servico_Controller);
});

// FILE: ./assets/js/controllers/Sm00024000_Modelo_Documentos_Controller.js
function Sm00024000_Modelo_Documentos_Controller(General, ScreenService, ZhFilterPreferences, templateManager, Utilities) {

    var self = this;
    /**
     * Salva o novo registro da página "Documento"
     * 
     * @param {object} widget widget de 
     * Documento para salvar o 
     * novo registro.
    */
    this.salvarWidget = function(widget){
        if(widget.currentRow.IDATIVO == 'N'){
            var row = {"NRSEQMODDOCFCT": widget.currentRow.NRSEQMODDOCFCT, "NRORG": widget.currentRow.NRORG};
            General.callRestEngine("product#/fct_modelodoc_widget/validarInativarModelo", "Row", row).then(function(validacao) {
                if(validacao.dataset.contratosComModelo.length > 0) {
                    ScreenService.showMessage("Operação bloqueada!<br><br>O modelo de documento não pode ser inativado pois está sendo utilizado");
                } else {
                    self.salvarDocumento(widget);

                }
            });
        } else {
            this.salvarDocumento(widget);
        }
    };
    
    this.salvarDocumento = function(wdgModelo) {
        wdgModelo.currentRow.__columns_to_return = ["NRSEQMODDOCFCT", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        var isNew = wdgModelo.currentRow.__is_new;
        var dscaminhoarq = wdgModelo.currentRow.DSCAMINHOARQ;
        
        if (!isNew && !(dscaminhoarq instanceof Array)) {
            wdgModelo.currentRow.DSCAMINHOARQ = [{
                "name": wdgModelo.currentRow.NMARQUIVO,
                "path": dscaminhoarq,
                "size": wdgModelo.currentRow.VRTAMANHO * 1024,
                "type": wdgModelo.currentRow.DSTIPO
            }];
        }
        
        General.widgetSave(wdgModelo).then(function(response) {
            if (response.dataset) {
                wdgModelo.currentRow.NRSEQMODDOCFCT = response.dataset.columnsToReturn[0].NRSEQMODDOCFCT;
                wdgModelo.currentRow.NRORG          = response.dataset.columnsToReturn[0].NRORG;
                wdgModelo.currentRow.DTINCLUSAO     = response.dataset.columnsToReturn[0].DTINCLUSAO;
                wdgModelo.currentRow.NRORGINCLUSAO  = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                wdgModelo.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
            wdgModelo.reload().then(function() {
                this.verificarSeExisteAnexo(wdgModelo);
            }.bind(this));
        }.bind(this),
        function(error) {
            console.error(error);
            if (error.data)
                ScreenService.showMessage(error.data.error);
            else
                ScreenService.showMessage(error);
        });
    };

    this.WdgOnEnterOnce = function(widget){
        ZhFilterPreferences.openFilterWidget(widget).then(function(){
            var widget = templateManager.container.getActiveWidget();
            widget.currentRow.IDATIVO = 'S';
        });
    };

    /**
     * Abre uma nova janela para fazer o download do documento.
     */
    this.downloadDocumento = function(wdgModelo) {
        ScreenService.closePopup();
        var params = { "NRSEQMODDOCFCT": wdgModelo.currentRow.NRSEQMODDOCFCT };
        General.callRestEngine("product#/fct_modelodoc_widget/downloadDocFileServer", "Row", params).then(function(response) {
            if(response.dataset && response.dataset.URLDOCUMENTO && response.dataset.URLDOCUMENTO.length > 0) {
                var urlDocumento = window.backendIndexUrl + response.dataset.URLDOCUMENTO[0]; 
                window.open(urlDocumento);
            }
        }).catch(function(error){
            window.open(error);
        });
    };
    
    this.verificarSeExisteAnexo = function(wdgModelo) {
        var actDownload = wdgModelo.getAction("download");
        var actExcluirAnexo = wdgModelo.getAction("excluir_anexo");
        var fieldAnexo = wdgModelo.getField("DSCAMINHOARQ");
        
        var existe = (wdgModelo.currentRow.NMARQUIVO) ? true : false;
        actDownload.isVisible     = existe;
        actExcluirAnexo.isVisible = existe;
        fieldAnexo.isVisible      = !existe;
    };
    
    this.excluirAnexo = function(wdgModelo) {
        var params = {
            "NRSEQMODDOCFCT": wdgModelo.currentRow.NRSEQMODDOCFCT,
            "DSCAMINHOARQ"  : wdgModelo.currentRow.DSCAMINHOARQ,
            "NRORG"         : wdgModelo.currentRow.NRORG
        };
        
        General.callRestEngine("product#/fct_modelodoc_widget/excluirAnexo", "Row", params).then(function() {
            ScreenService.showMessage("Anexo excluído com sucesso!");
            ScreenService.closePopup();
            wdgModelo.currentRow.NMARQUIVO    = null;
            wdgModelo.currentRow.DSCAMINHOARQ = null;
            wdgModelo.currentRow.VRTAMANHO    = null;
            wdgModelo.currentRow.DSTIPO       = null;
            this.verificarSeExisteAnexo(wdgModelo);
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    /**
     * Limpa a row para evitar erros ao editar ou adicionar anexos.
    */
    this.limparCurrentRowModeloDeDocumento = function(widget){
        if(widget != null || widget != undefined){
            widget.dataSource.data.forEach(function(row) {row.changed = false;});
        }
    };
}

Configuration(function(ContextRegister)
{
    ContextRegister.register("Sm00024000_Modelo_Documentos_Controller", Sm00024000_Modelo_Documentos_Controller);
});

// FILE: ./assets/js/controllers/Sm00024000_ParametrizacaoResultadoContrato.js
function Sm00024000_ParametrizacaoResultadoContrato(General, ScreenService,Utilities) {

    /**
     * Salva dierentes registros de acordo com a função da widget
     * 
     * @param {object, nunber} widget widget de 
     * gridDespesa, gridOperacoes, gridProduto 
     */
    this.salvaParamentros = function(widget,funcao){
        if(funcao == 1){
            this.salvarMaoDeObra(widget,"CDCONTCTBL","/fct_maoobracontctbl/save");
        }else
            if(funcao == 2){
                    this.salvarOperacoes(widget,"CDTIPOOPER","/fct_tpoperdespdiv/save");
            }else
                if(funcao == 3){
                    this.salvarProdutos(widget,"CDPRODUTO","/fct_elementoproddesp/save");
                }
        };
    /**
     * Função para salvar multiplos da widget MaoDeObra
     * 
     * @param {object, string, string} Função da função
     * salvarParamentros
     */
    this.salvarMaoDeObra = function(widget,chave,rota){
     
       var parametros = {
            "CDSERVORCA": widget.currentRow.CDSERVORCA,
            "CDCONTCTBL" : widget.currentRow.CDCONTCTBL
       };
       
        General.callRestEngine("product#/fct_maoobracontctbl/validar","Row",parametros).then(function(NaoTem) {
            if(NaoTem.dataset.validar.resultado){
                Utilities.inserirMultiplos(widget, chave, null, rota).then(function(){
                    ScreenService.closeSwipe().then(function(){
                        widget.reload();
                    });
                });
            }
            else{
                ScreenService.showMessage("Operação bloqueada. <br>Uma ou mais Contas Contabil já foram cadastradas para este Elemento.");
            }
        }).catch(function(error){
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
        
    };
    
     /**
     * Função para salvar multiplos da widget Produtos
     * 
     * @param {object, string, string} Função da função
     * salvarParamentros
     */
    this.salvarProdutos = function(widget,chave,rota){
     
       var parametros = {
            "CDSERVORCA": widget.currentRow.CDSERVORCA,
            "CDPRODUTO" : widget.currentRow.CDPRODUTO
       };
       
        General.callRestEngine("product#/fct_elementoproddesp/validar","Row",parametros).then(function(NaoTem) {
            if(NaoTem.dataset.validar.resultado){
                Utilities.inserirMultiplos(widget, chave, null, rota).then(function(){
                    ScreenService.closeSwipe().then(function(){
                        widget.reload();
                    });
                });
            }
            else{
                ScreenService.showMessage("Operação bloqueada. <b>Um ou mais Produtos já foram cadastrados para este Elemento.");
            }
        }).catch(function(error){
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
        
    };
    /**
     * Função para salvar multiplos da widget Operacoes
     * 
     * @param {object, string, string} Função da função
     * salvarParamentros
     */
    this.salvarOperacoes = function(widget,chave,rota){
         Utilities.inserirMultiplos(widget, chave, null, rota).then(function(){
                    ScreenService.closeSwipe().then(function(){
                        widget.reload();
                    });
                });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Sm00024000_ParametrizacaoResultadoContrato', Sm00024000_ParametrizacaoResultadoContrato);
});       



// FILE: ./assets/js/controllers/Sm00026000_Demonstrativo_Resultado_Contrato.js
function Sm00026000_Demonstrativo_Resultado_Contrato(General, ScreenService, Utilities) {
    
    /**
     * Função que gera o relatorio de acordo com os parâmentros estabelecidos
     * 
     * @param {object} widget widget 
    */
    
    this.gerarRelatorio = function(widget) {
    if (widget.isValid()) {
        var NRORG = "_"+widget.currentRow.NRORG.join("_")+"_";
        var NRSEQCONTRATO = "_"+widget.currentRow.NRSEQCONTRATO.join("_")+"_";
        var CDFILIAL = "_"+widget.currentRow.CDFILIAL.join("_")+"_";
        var NRPARCNEGOCIO = "_"+widget.currentRow.NRPARCNEGOCIO.join("_")+"_";
        var CDCENTCUST = "_"+widget.currentRow.CDCENTCUST.join("_")+"_";
        
        var params = {
            "NRORG" : NRORG,
            "INIAPURACAO" : widget.currentRow.INIAPURACAO,
            "FIMAPURACAO" : widget.currentRow.FIMAPURACAO,
            "NRSEQCONTRATO" : NRSEQCONTRATO,
            "UNIDADE" : CDFILIAL,
            "PARCNEGOCIO" : NRPARCNEGOCIO,
            "CENTCUST" : CDCENTCUST
        };
        var nomeRelatorio = widget.currentRow.NMRELATORIO_LIST;
             ScreenService.closePopup();
        Utilities.openReport("product#/geraBirtReport", nomeRelatorio, params);
           
        } 
        else {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
    };
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00026000_Demonstrativo_Resultado_Contrato', Sm00026000_Demonstrativo_Resultado_Contrato);
});

// FILE: ./assets/js/controllers/Sm00029000_Local_Atendimento_Controller.js
function Sm00029000_Local_Atendimento_Controller(General, ScreenService, templateManager, Utilities) {

    const MSG_ALERTA_INTEGRACAO_ERP_FILIAL = 'Atenção!<br><br> A integração com o ERP para salvar a Unidade não foi executada pois não há URL parametrizada. <br>Favor conferir o parâmetro "URL para integração (ERP - Unidades)" na tela "Parametrização SM".';
    const MSG_FILIAL_VINCULADA_OUTRO_LOCAL = 'Operação Bloqueada! <br><br>A Unidade informada já está relacionada a outro Local de Atendimento.';

    /**
     * Busca o endereço através do CEP digitado.
     * Função foi modularizada para poder receber vários tipos de endereços.
     * @param {object} widget
     */
    this.buscarEndereco = function (widget, tipo) {
        var fields = this.verificarEformatarComponente(widget, tipo);
      //  widget = widget.widget;
       // widget.currentRow.cep = cep;
        cep = fields[12];
        cdpais = fields[0];
        sgestado = fields[2];
        cdmunicipio = fields[4];
        if (cep) {
            Utilities.buscarCep(widget.currentRow[cep]).then(function (response) {
                var endereco = response.dataset.ENDERECO;
                if (endereco.NMBAIRRO && !endereco.CDBAIRRO) {

                    idNovoCodigo = "BAIRRO" + widget.currentRow[cdpais] + widget.currentRow[sgestado] + widget.currentRow[cdmunicipio];
                    console.log(idNovoCodigo);
                    var params = {
                        "CDPAIS": endereco.CDPAIS,
                        "SGESTADO": endereco.SGESTADO,
                        "CDBAIRRO": endereco.CDBAIRRO,
                        "NMBAIRRO": endereco.NMBAIRRO,
                        "CDMUNICIPIO": endereco.CDMUNICIPIO,
                        "__is_new": true,
                        "__is_new_code": true,
                        "__new_code_pk": idNovoCodigo,
                        "__table_pk": "CDBAIRRO",
                        "__columns_to_return": ["CDBAIRRO"]
                    };
                    General.callRestEngine('product#/crud/bairro/save', "Row", params).then(function (response) {
                        widget.currentRow[fields[6]] = response.dataset.ENDERECO.CDBAIRRO;
                    },
                        function (error) {
                            ScreenService.showMessage(error.data.error);
                        });
                }

                widget.currentRow[fields[7]] = endereco.CDBAIRRO;
                widget.currentRow[fields[4]] = endereco.CDMUNICIPIO;
                widget.currentRow[fields[0]] = endereco.CDPAIS;
                widget.currentRow[fields[10]] = endereco.DSENDERECO;
                widget.currentRow[fields[7]] = endereco.NMBAIRRO;
                widget.currentRow[fields[3]] = endereco.NMESTADO;
                widget.currentRow[fields[5]] = endereco.NMMUNICIPIO;
                widget.currentRow[fields[1]] = endereco.NMPAIS;
                widget.currentRow[fields[2]] = endereco.SGESTADO;

                if (response.messages) {
                    var msg = ScreenService.i18n(response.messages[0].message);
                    ScreenService.showMessage(msg);
                }

                angular.element('#footer').click();
            },
                function (error) {
                    console.error(error);
                    if (error.data) {
                        var mensagem = error.data.error;
                        if (error.data.error === "Erro de configuração de proxy!")
                            mensagem = "Endereço não encotrado.";

                        ScreenService.showMessage(mensagem);
                    } else {
                        ScreenService.showMessage("Falha na conexão.");
                    }
                });
        }
        else {
            fields.forEach(function (field) {
                widget.currentRow[field] = "";
            });
        }
    };

    /**
     * Verifica qual o tipo de endereço que está sendo alterado para formatar as variáveis
     * @param {object} widget
     * @returns {array} fields retorna as propriedades do objeto da curretoRow com os nomes alterados.
     */
    this.verificarEformatarComponente = function (widget, tipo) {
        /*if ( tipo == '') {
            cep = widget.currentRow.NRCEPENDERECO;
        } else if (tipo == '_COBRANCA') {
            cep = widget.currentRow.NRCEPENDERECO_COBRANCA;
        } else if ( tipo == '_EXTERNO') {
            cep = widget.currentRow.NRCEPENDERECO_EXTERNO;
        } else if ( tipo == '_ENTREGA') {
            cep = widget.currentRow.NRCEPENDERECO_ENTREGA;
        } */

        var fields = [
            "CDPAIS" + tipo,           //0
            "NMPAIS" + tipo,           //1
            "SGESTADO" + tipo,         //2
            "NMESTADO" + tipo,         //3
            "CDMUNICIPIO" + tipo,      //4 
            "NMMUNICIPIO" + tipo,      //5
            "CDBAIRRO" + tipo,         //6
            "NMBAIRROENDERECO" + tipo, //7
            "CDLOGRADOURO" + tipo,     //8
            "DSLOGRADOURO" + tipo,     //9
            "DSENDERECO" + tipo,       //10
            "NRIMOVELENDERECO" + tipo, //11
            "NRCEPENDERECO"+ tipo,     //12
            tipo,                      //13
            "NRIMOVELENDERECO" + tipo, //14
            "DSCOMPLEENDERECO" + tipo, //15

        ];
        return fields;
    }

    /**
   * Controla o filtro e o valor dos campos de endereço ao mudar o País.
   * 
   * @param {object} widget
   */
    this.controlarCamposEnderecoPorPais = function (widget, tipo) {
        fields = this.verificarEformatarComponente(widget, tipo);
        widget.currentRow[fields[2]] = null;
        widget.currentRow[fields[3]] = null;
        widget.currentRow[fields[4]] = null;
        widget.currentRow[fields[5]] = null;
        widget.currentRow[fields[6]] = null;
        widget.currentRow[fields[7]] = null;
        widget.currentRow[fields[14]] = null;
        widget.currentRow[fields[10]] = null;

        var campoEstado = widget.getField(fields[3]);
        campoEstado.reload();
    };

    /**
 * Controla o filtro e o valor dos campos de endereço ao mudar o Estado.
 * 
 * @param {object} widget
 */
    this.controlarCamposEnderecoPorEstado = function (widget, tipo) {
        fields = this.verificarEformatarComponente(widget, tipo);
        widget.currentRow[fields[4]] = null;
        widget.currentRow[fields[5]] = null;
        widget.currentRow[fields[6]] = null;
        widget.currentRow[fields[7]] = null;

        var campoMunicipio = widget.getField(fields[5]);
        campoMunicipio.reload();
    };

    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o Município.
     * 
     * @param {object} widget
     */
    this.controlarCamposEnderecoPorMunicipio = function (widget, tipo) {
        fields = this.verificarEformatarComponente(widget, tipo);
        widget.currentRow[fields[6]] = null;
        widget.currentRow[fields[7]] = null;

        var campoBairro = widget.getField(fields[7]);
        campoBairro.reload();
    };

    /**
    * Recebe os endereços e seta nos fieldsgroups corretos..
    * 
    * @param {object} widget
    */
    this.popularEnderecos = function (widget) {
        params = widget.getParent();
        widget.fields.forEach(function (values) {
            values.setValue(null);
        });
        General.callRestEngine('product#/findEnderecos', "Row", {
            NRPARCNEGOCIO: params.currentRow.NRPARCNEGOCIO,
        }).then(function (response) {
            const retorno = response.dataset.retorno.data;
            retorno.forEach(function (element) {
                if (element.CDTIPOENDERECO == 'PRINCIPAL') {
                    tipo = '';
                } else if (element.CDTIPOENDERECO == 'COBRANCA') {
                    tipo = '_COBRANCA';
                } else if (element.CDTIPOENDERECO == 'COMERCIAL') {
                    tipo = '_EXTERNO';
                } else if (element.CDTIPOENDERECO == 'ENTREGA') {
                    tipo = '_ENTREGA';
                } else if (element.CDTIPOENDERECO == 'OUTROS') {
                    tipo = '_outros';
                }

                widget.getField('NRCEPENDERECO' + tipo).setValue(element.NRCEPENDERECO);
                widget.getField('CDPAIS' + tipo).setValue(element.CDPAIS);
                widget.getField('NMPAIS' + tipo).setValue(element.NMPAIS);
                widget.getField('SGESTADO' + tipo).setValue(element.SGESTADO);
                widget.getField('NMESTADO' + tipo).setValue(element.NMESTADO);
                widget.getField('NMMUNICIPIO' + tipo).setValue(element.NMMUNICIPIO);
                widget.getField('CDMUNICIPIO' + tipo).setValue(element.CDMUNICIPIO);
                widget.getField('DSENDERECO' + tipo).setValue(element.DSENDERECO);
                widget.getField('CDBAIRRO' + tipo).setValue(element.CDBAIRRO);
                widget.getField('DSREFERENCIAENDE' + tipo).setValue(element.DSREFERENCIAENDE);
                widget.getField('NRIMOVELENDERECO' + tipo).setValue(element.NRIMOVELENDERECO);
                widget.getField('NMBAIRROENDERECO' + tipo).setValue(element.NMBAIRROENDERECO);
                widget.getField('NMBAIRROENDERECO' + tipo).setValue(element.NMBAIRROENDERECO);
                widget.getField('DSEMAILESTRUTURA' + tipo).setValue(element.DSEMAILESTRUTURA);
                widget.getField('DSCOMPLEENDERECO' + tipo).setValue(element.DSCOMPLEENDERECO);
            });
        },
            function (error) {
                ScreenService.showMessage(error.data.error);
            });
        widget.reload();
    }

    this.editEnderecos = function (widget) {
        General.widgetEdit(widget);
    }

    this.salvarEnderecos = function (widget) {
        var nr;
        if (widget.getParent().currentRow.NRPARCNEGOCIO == "") {
            nr = window.localStorage.getItem("NRPARCNEGOCIO");
        } else {
            nr = widget.getParent().currentRow.NRPARCNEGOCIO;
        }
        params = { ENDERECOS: widget.currentRow, NRPARCNEGOCIO: nr };
        General.callRestEngine('product#/insertUpdateAdress', "Row", params).then(function () {
            General.widgetCancelForm(widget);
            (nr != undefined) ? window.localStorage.removeItem("NRPARCNEGOCIO") : '';
            var wdgEndereco = templateManager.container.getWidget("wDEnderecos");
            wdgEndereco.reload();
        },
            function (error) {
                ScreenService.showMessage(error.data.error);
            });
    }

    /**
     * Realiza à rotina de inserção de todas as informações em suas respectivas tabelas.
     * 
     * @param {object} widget widget 
    */
    this.inserirTudo = function (widget, vinculaClie) {

        var params = {
            "NMESTRUTURAM": widget.currentRow.NMESTRUTURAM,
            "NRESTRUTURAMSUP": widget.currentRow.NRESTRUTURAMSUP,
            "NRTIPOESTRUTURA": widget.currentRow.NRTIPOESTRUTURA,
            "NRINSCRICAOPARC": widget.currentRow.NRINSCRICAOPARC.replace(/[\D]+/g, ''),
            "CDTIPOINSCRICAO": widget.currentRow.CDTIPOINSCRICAO,
            "CDCENTCUST": widget.currentRow.CDCENTCUST,
            "IDCONVERTIDOCLIENTE": widget.currentRow.IDCONVERTIDOCLIENTE,
            "IDCONVERTIDOFILIAL": widget.currentRow.IDCONVERTIDOFILIAL,
            "CDFILIAL": widget.currentRow.CDFILIAL,
            "CDEMPRESA": widget.currentRow.CDEMPRESA,
            "NMEMPRESA": widget.currentRow.NMEMPRESA,
            "CDINSCESTA": widget.currentRow.CDINSCESTA,
            "CDINSCMUNI": widget.currentRow.CDINSCMUNI,
            "NMRAZSOCFILI": widget.currentRow.NMRAZSOCFILI,
            "DSEMAILESTRUTURA": widget.currentRow.DSEMAILESTRUTURA,
            "CDPAIS": widget.currentRow.CDPAIS,
            "SGESTADO": widget.currentRow.SGESTADO,
            "CDMUNICIPIO": widget.currentRow.CDMUNICIPIO,
            "DSENDERECO": widget.currentRow.DSENDERECO,
            "NRIMOVELENDERECO": widget.currentRow.NRIMOVELENDERECO,
            "CDBAIRRO": widget.currentRow.CDBAIRRO,
            "NMBAIRROENDERECO": widget.currentRow.NMBAIRROENDERECO,
            "NRCEPENDERECO": widget.currentRow.NRCEPENDERECO,
            "DSCOMPLEENDERECO": widget.currentRow.DSCOMPLEENDERECO,
            "DSREFERENCIAENDE": widget.currentRow.DSREFERENCIAENDE,
            "CDPAIS_COBRANCA": widget.currentRow.CDPAIS_COBRANCA,
            "SGESTADO_COBRANCA": widget.currentRow.SGESTADO_COBRANCA,
            "CDMUNICIPIO_COBRANCA": widget.currentRow.CDMUNICIPIO_COBRANCA,
            "DSENDERECO_COBRANCA": widget.currentRow.DSENDERECO_COBRANCA,
            "NRIMOVELENDERECO_COBRANCA": widget.currentRow.NRIMOVELENDERECO_COBRANCA,
            "CDBAIRRO_COBRANCA": widget.currentRow.CDBAIRRO_COBRANCA,
            "NMBAIRROENDERECO_COBRANCA": widget.currentRow.NMBAIRROENDERECO_COBRANCA,
            "NRCEPENDERECO_COBRANCA": widget.currentRow.NRCEPENDERECO_COBRANCA,
            "DSCOMPLEENDERECO_COBRANCA": widget.currentRow.DSCOMPLEENDERECO_COBRANCA,
            "DSREFERENCIAENDE_COBRANCA": widget.currentRow.DSREFERENCIAENDE_COBRANCA,
            "CDPAIS_EXTERIOR": widget.currentRow.CDPAIS_EXTERIOR,
            "SGESTADO_EXTERIOR": widget.currentRow.SGESTADO_EXTERIOR,
            "CDMUNICIPIO_EXTERIOR": widget.currentRow.CDMUNICIPIO_EXTERIOR,
            "DSENDERECO_EXTERIOR": widget.currentRow.DSENDERECO_EXTERIOR,
            "NRIMOVELENDERECO_EXTERIOR": widget.currentRow.NRIMOVELENDERECO_EXTERIOR,
            "CDBAIRRO_EXTERIOR": widget.currentRow.CDBAIRRO_EXTERIOR,
            "NMBAIRROENDERECO_EXTERIOR": widget.currentRow.NMBAIRROENDERECO_EXTERIOR,
            "NRCEPENDERECO_EXTERIOR": widget.currentRow.NRCEPENDERECO_EXTERIOR,
            "DSCOMPLEENDERECO_EXTERIOR": widget.currentRow.DSCOMPLEENDERECO_EXTERIOR,
            "DSREFERENCIAENDE_EXTERIOR": widget.currentRow.DSREFERENCIAENDE_EXTERIOR,
            "CDPAIS_ENTREGA": widget.currentRow.CDPAIS_ENTREGA,
            "SGESTADO_ENTREGA": widget.currentRow.SGESTADO_ENTREGA,
            "CDMUNICIPIO_ENTREGA": widget.currentRow.CDMUNICIPIO_ENTREGA,
            "DSENDERECO_ENTREGA": widget.currentRow.DSENDERECO_ENTREGA,
            "NRIMOVELENDERECO_ENTREGA": widget.currentRow.NRIMOVELENDERECO_ENTREGA,
            "CDBAIRRO_ENTREGA": widget.currentRow.CDBAIRRO_ENTREGA,
            "NMBAIRROENDERECO_ENTREGA": widget.currentRow.NMBAIRROENDERECO_ENTREGA,
            "NRCEPENDERECO_ENTREGA": widget.currentRow.NRCEPENDERECO_ENTREGA,
            "DSCOMPLEENDERECO_ENTREGA": widget.currentRow.DSCOMPLEENDERECO_ENTREGA,
            "DSREFERENCIAENDE_ENTREGA": widget.currentRow.DSREFERENCIAENDE_ENTREGA,
            "VINCULA_CLIENTE_EXISTENTE": vinculaClie,
            "CRIA_CLIENTE": widget.currentRow.IDCRIANOVOCLIENTE,
            "CLIENTE": widget.currentRow.CLIENTE,
            "CRIA_FILIAL": widget.currentRow.IDCRIANOVAUNIDADE,
            "FILIAL": widget.currentRow.FILIAL
        };
        
        General.callRestEngine('product#/estruturam_widget/inserirTudo', "Row", params).then(function (response) {
            window.localStorage.setItem('NRPARCNEGOCIO', response.dataset.retorno.NRPARCNEGOCIO);

            if (response.dataset.retorno.chamouIntegracaoERP === false) {
                ScreenService.showMessage(MSG_ALERTA_INTEGRACAO_ERP_FILIAL);
            }
            ScreenService.closeSwipe();
            widget.reload();
        }).catch(function (error) {
            if (error.data && error.data.error == "FILIAL_VINCULADA_OUTRO_LOCAL") {
                ScreenService.showMessage(MSG_FILIAL_VINCULADA_OUTRO_LOCAL);
            }
            else {
                Utilities.showErrorMessage(error);
            }
        });
    };

    /**
    * Realiza à rotina de edição de todas as informações em suas respectivas tabelas.
    * 
    * @param {object} widget widget 
   */

    this.editarTudo = function (widget, vinculaClie) {
        var params = {
            "NRESTRUTURAM": widget.currentRow.NRESTRUTURAM,
            "NMESTRUTURAM": widget.currentRow.NMESTRUTURAM,
            "NRESTRUTURAH": widget.currentRow.NRESTRUTURAH,
            "NRPARCNEGOCIO": widget.currentRow.NRPARCNEGOCIO,
            "NRHIERARQUIAORG": widget.currentRow.NRHIERARQUIAORG,
            "NRESTRUTURAMSUP": widget.currentRow.NRESTRUTURAMSUP,
            "NRTIPOESTRUTURA": widget.currentRow.NRTIPOESTRUTURA,
            "NRINSCRICAOPARC": widget.currentRow.NRINSCRICAOPARC,
            "CDTIPOINSCRICAO": widget.currentRow.CDTIPOINSCRICAO,
            "NMRAZSOCFILI": widget.currentRow.NMRAZSOCFILI,
            "CDCENTCUST": widget.currentRow.CDCENTCUST,
            "NRENDERECOPARC": widget.currentRow.NRENDERECOPARC,
            "DSENDERECO": widget.currentRow.DSENDERECO,
            "NRIMOVELENDERECO": widget.currentRow.NRIMOVELENDERECO,
            "DSCOMPLEENDERECO": widget.currentRow.DSCOMPLEENDERECO,
            "CDPAIS": widget.currentRow.CDPAIS,
            "SGESTADO": widget.currentRow.SGESTADO,
            "CDMUNICIPIO": widget.currentRow.CDMUNICIPIO,
            "CDBAIRRO": widget.currentRow.CDBAIRRO,
            "NMBAIRROENDERECO": widget.currentRow.NMBAIRROENDERECO,
            "NRCEPENDERECO": widget.currentRow.NRCEPENDERECO,
            "DSREFERENCIAENDE": widget.currentRow.DSREFERENCIAENDE,
            "IDCONVERTIDOCLIENTE": widget.currentRow.IDCONVERTIDOCLIENTE,
            "IDCONVERTIDOFILIAL": widget.currentRow.IDCONVERTIDOFILIAL,
            "CDFILIAL": widget.currentRow.CDFILIAL,
            "CDEMPRESA": widget.currentRow.CDEMPRESA,
            "NMEMPRESA": widget.currentRow.NMEMPRESA,
            "CDINSCESTA": widget.currentRow.CDINSCESTA,
            "CDINSCMUNI": widget.currentRow.CDINSCMUNI,
            "DSEMAILESTRUTURA": widget.currentRow.DSEMAILESTRUTURA,
            "CDPAIS_COBRANCA": widget.currentRow.CDPAIS_COBRANCA,
            "SGESTADO_COBRANCA": widget.currentRow.SGESTADO_COBRANCA,
            "CDMUNICIPIO_COBRANCA": widget.currentRow.CDMUNICIPIO_COBRANCA,
            "DSENDERECO_COBRANCA": widget.currentRow.DSENDERECO_COBRANCA,
            "NRIMOVELENDERECO_COBRANCA": widget.currentRow.NRIMOVELENDERECO_COBRANCA,
            "CDBAIRRO_COBRANCA": widget.currentRow.CDBAIRRO_COBRANCA,
            "NMBAIRROENDERECO_COBRANCA": widget.currentRow.NMBAIRROENDERECO_COBRANCA,
            "NRCEPENDERECO_COBRANCA": widget.currentRow.NRCEPENDERECO_COBRANCA,
            "DSCOMPLEENDERECO_COBRANCA": widget.currentRow.DSCOMPLEENDERECO_COBRANCA,
            "DSREFERENCIAENDE_COBRANCA": widget.currentRow.DSREFERENCIAENDE_COBRANCA,
            "CDPAIS_EXTERIOR": widget.currentRow.CDPAIS_EXTERIOR,
            "SGESTADO_EXTERIOR": widget.currentRow.SGESTADO_EXTERIOR,
            "CDMUNICIPIO_EXTERIOR": widget.currentRow.CDMUNICIPIO_EXTERIOR,
            "DSENDERECO_EXTERIOR": widget.currentRow.DSENDERECO_EXTERIOR,
            "NRIMOVELENDERECO_EXTERIOR": widget.currentRow.NRIMOVELENDERECO_EXTERIOR,
            "CDBAIRRO_EXTERIOR": widget.currentRow.CDBAIRRO_EXTERIOR,
            "NMBAIRROENDERECO_EXTERIOR": widget.currentRow.NMBAIRROENDERECO_EXTERIOR,
            "NRCEPENDERECO_EXTERIOR": widget.currentRow.NRCEPENDERECO_EXTERIOR,
            "DSCOMPLEENDERECO_EXTERIOR": widget.currentRow.DSCOMPLEENDERECO_EXTERIOR,
            "DSREFERENCIAENDE_EXTERIOR": widget.currentRow.DSREFERENCIAENDE_EXTERIOR,
            "CDPAIS_ENTREGA": widget.currentRow.CDPAIS_ENTREGA,
            "SGESTADO_ENTREGA": widget.currentRow.SGESTADO_ENTREGA,
            "CDMUNICIPIO_ENTREGA": widget.currentRow.CDMUNICIPIO_ENTREGA,
            "DSENDERECO_ENTREGA": widget.currentRow.DSENDERECO_ENTREGA,
            "NRIMOVELENDERECO_ENTREGA": widget.currentRow.NRIMOVELENDERECO_ENTREGA,
            "CDBAIRRO_ENTREGA": widget.currentRow.CDBAIRRO_ENTREGA,
            "NMBAIRROENDERECO_ENTREGA": widget.currentRow.NMBAIRROENDERECO_ENTREGA,
            "NRCEPENDERECO_ENTREGA": widget.currentRow.NRCEPENDERECO_ENTREGA,
            "DSCOMPLEENDERECO_ENTREGA": widget.currentRow.DSCOMPLEENDERECO_ENTREGA,
            "DSREFERENCIAENDE_ENTREGA": widget.currentRow.DSREFERENCIAENDE_ENTREGA,
            "VINCULA_CLIENTE_EXISTENTE": vinculaClie,
            "CRIA_CLIENTE": widget.currentRow.IDCRIANOVOCLIENTE,
            "CLIENTE": widget.currentRow.CLIENTE,
            "CRIA_FILIAL": widget.currentRow.IDCRIANOVAUNIDADE,
            "FILIAL": widget.currentRow.FILIAL
        };

        General.callRestEngine('product#/estruturam_widget/editarTudo', "Row", params).then(function (response) {
            ScreenService.closeSwipe();
            widget.reload();
            if (response.dataset.retorno.chamouIntegracaoERP === false) {
                ScreenService.showMessage(MSG_ALERTA_INTEGRACAO_ERP_FILIAL);
            }
        }).catch(function (error) {
            if (error.data && error.data.error == "FILIAL_VINCULADA_OUTRO_LOCAL") {
                ScreenService.showMessage(MSG_FILIAL_VINCULADA_OUTRO_LOCAL);
            }
            else {
                Utilities.showErrorMessage(error);
            }
        });
    };

    /**
    * Realiza à rotina de exclusão de todas as informações em suas respectivas tabelas.
    * 
    * @param {object} widget widget 
   */

    this.deletarTudo = function (widget) {
        ScreenService.confirmMessage("Atenção! <br/><br/>A exclusão deste local de atendimento também irá excluir os locais de atendimento inferiores a ele. Confirma exclusão?", "question", function () {
            var params = {
                "NRESTRUTURAM": widget.currentRow.NRESTRUTURAM,
                "NRPARCNEGOCIO": widget.currentRow.NRPARCNEGOCIO
            };

            General.callRestEngine('product#/estruturam_widget/deletarTudo', "Row", params).then(function () {
                ScreenService.closeSwipe();
                widget.reload();
            }).catch(function (error) {
                Utilities.showErrorMessage(error);
            });

        });
    };

    /**
     * Salva os dados de Inserção ou Edição.
     * 
     * @param {object} widget widget 
    */

    this.salvarTudo = function (widget) {
        if (widget.isValid()) {

            var inscricao =  widget.currentRow.NRINSCRICAOPARC;
            var idConvCliente =  widget.currentRow.IDCONVERTIDOCLIENTE;

            if (idConvCliente == 'S') {
                var params = {
                    "NRINSCRICAOPARC": inscricao,
                };

                General.callRestEngine('product#/estruturam_widget/verificarExisteClienteComMesmaParceria', "Row", params).then(function (response) {
                
                    if (response.dataset.retorno.validaCliente == 'BLOQUEIA') {
                        ScreenService.showMessage("Operação bloqueada! <br>Já existe um cliente ativo com esse CNPJ.");
                    }
                    else if (response.dataset.retorno.validaCliente == 'PERGUNTA') {

                        ScreenService.showCustomDialog("Já existe um cliente ativo com esse CNPJ. <br><br> Deseja vincular o local de atendimento a este cliente?", 'CONFIRMATION', null, [
                            {
                                "label": "Sim",
                                "code": function () {
                                    if (widget.currentRow.__is_new) {
                                        this.inserirTudo(widget,'S'); //S = Sim
                                    } else {
                                        this.editarTudo(widget, 'S'); //S = Sim
                                    }
                                }.bind(this)
                            },
                            {
                                "label": "Não",
                                "code": function () {
                                    if (widget.currentRow.__is_new) {
                                        this.inserirTudo(widget, 'I'); //I = Ignorar
                                    } else {
                                        this.editarTudo(widget, 'I'); //I = Ignorar
                                    };
                                }.bind(this)
                            },
                            {
                                "label": "Cancelar",
                                "code": function () {
                                }.bind(this)
                            }
                        ]);

                    }
                    else {
                        if (widget.currentRow.__is_new) {
                            this.inserirTudo(widget,'N'); //N = Não
                        } else {
                            this.editarTudo(widget, 'N'); //N = Não
                        }
                    }

                }.bind(this)).catch(function (error) {
                    Utilities.showErrorMessage(error);
                });
            }
            else {
                if (widget.currentRow.__is_new) {
                    this.inserirTudo(widget, 'N'); //N = Não
                } else {
                    this.editarTudo(widget, 'N'); //N = Não
                }  
            }
        }
        else {
            ScreenService.showMessage("Preencha todos os campos corretamente!");
        }
    };

    /**
     * Funções antes de inserir
     * 
     * @param {object} widget widget
     */
    this.novoCalculo = function (widget) {
        if (widget.mode == "view") {
            var LocalAtend = widget.currentRow.NRESTRUTURAM;
            var LocalAtendName = widget.currentRow.NMESTRUTURAM;

            General.widgetNewRow(widget);
            widget.currentRow.NRESTRUTURAMSUP = LocalAtend;
            widget.currentRow.NMESTRUTURAMSUP = LocalAtendName;
        }
        else {
            General.widgetNewRow(widget);
        }
        this.idconvertidoclienteOnChange(widget);
        this.convertidoEmUnidadeOnChange(widget);
    };

    this.verificaAction = function (widget, action) {
        if (action === "AVANCAR") {
            this.avancarComposicao(widget);
        }
    };

    this.avancarComposicao = function (widget) {
        if (widget.isValid()) {
            this.inserirTudo(widget, 'N');
        } else {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
    };

    this.localAtendimentoEdit = function (widget) {

        General.widgetEdit(widget);
        // widget.getField("NMESTADO").reload();
        // widget.getField("NMMUNICIPIO").reload();
        // widget.getField("NMBAIRROENDERECO").reload();
        this.idconvertidoclienteOnChange(widget);
        this.convertidoEmUnidadeOnChange(widget);
    };

    this.abreWidgetAddBairro = function () {
        var widgetLocalAtendimento = templateManager.container.getWidget("treeGridLocalAtend");
        var cdpais = widgetLocalAtendimento.currentRow.CDPAIS;
        var sgestado = widgetLocalAtendimento.currentRow.SGESTADO;
        var cdmunicipio = widgetLocalAtendimento.currentRow.CDMUNICIPIO;

        if (!cdpais || !sgestado || !cdmunicipio) {
            Utilities.showErrorMessage("Favor preencher os campos 'País', 'Estado' e 'Município'.");
        } else {
            ScreenService.closePopup();
            var wdgPopup = templateManager.container.getWidget("wdgNewBairro");
            wdgPopup.isVisible = true;
            wdgPopup.dataSource.data = [];
            ScreenService.openPopup(wdgPopup).then(function () {
                wdgPopup.activate();
                Utilities.formWidgetNewRow(wdgPopup);
            });
        }
    };

    this.salvarWidgetBairro = function (widgetPopupNovoBairro) {
        var widgetLocalAtendimento = templateManager.container.getWidget("treeGridLocalAtend");
        var cdpais = widgetLocalAtendimento.currentRow.CDPAIS;
        var sgestado = widgetLocalAtendimento.currentRow.SGESTADO;
        var cdmunicipio = widgetLocalAtendimento.currentRow.CDMUNICIPIO;

        widgetPopupNovoBairro.currentRow.CDPAIS = cdpais;
        widgetPopupNovoBairro.currentRow.SGESTADO = sgestado;
        widgetPopupNovoBairro.currentRow.CDMUNICIPIO = cdmunicipio;

        var idNovoCodigo = "BAIRRO" + cdpais + sgestado + cdmunicipio;

        widgetPopupNovoBairro.paramsNewCode = {
            "__is_new_code": true,
            "__new_code_pk": idNovoCodigo,
            "__table_pk": "CDBAIRRO",
            "__columns_to_return": ["CDBAIRRO"]
        };

        General.widgetSave(widgetPopupNovoBairro).then(function (response) {
            widgetLocalAtendimento.currentRow.CDBAIRRO = response.dataset.columnsToReturn[0].CDBAIRRO;
            widgetLocalAtendimento.currentRow.NMBAIRROENDERECO = widgetPopupNovoBairro.currentRow.NMBAIRRO;
            ScreenService.closePopup();
            widgetPopupNovoBairro.isVisible = false;
            ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
            widgetLocalAtendimento.activate();
        }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
    };

    this.nrinscricaoOnChange = function (field) {
        var currentRow = field.widget.currentRow;
        if (field.value()) {

            var validations = General.inscricaoFormat(field.value());

            var tipoInscricao = {
                "F": "CPF",
                "I": "LIVRE",
                "J": "CNPJ",
                "O": "CODIGO"
            };

            var tipoInscricaoParceiro = tipoInscricao[validations.tipo.cod];

            if (tipoInscricaoParceiro === undefined) {
                ScreenService.showMessage(ScreenService.i18n("Ocorreu um erro ao validar a inscrição."));
            }
            else {
                currentRow.CDTIPOINSCRICAO = tipoInscricaoParceiro;
                currentRow.NRINSCRICAOPARC = (["CPF", "CNPJ"].includes(tipoInscricaoParceiro)) ? validations.parameter : validations.value;
            }
        }
        else {
            currentRow.CDTIPOINSCRICAO = null;
            currentRow.NRINSCRICAOPARC = null;
        }
    };

    this.idconvertidoclienteOnChange = function (widget) {
        var currentRow = widget.currentRow;
        var validations = null;

        if (currentRow.IDCONVERTIDOCLIENTE == "S") {

            widget.getField("IDCRIANOVOCLIENTE").readOnly = false
            widget.getField("IDCRIANOVOCLIENTE").validations = { "required": true };

            if (currentRow.IDCRIANOVOCLIENTE == "S") {
                validations = { "required": true };
                widget.getField("NRINSCRICAOPARC").readOnly = false
                this.alterarObrigatoriedadeCamposConvertidoEmCliente(widget, validations);

                widget.getField("NMCLIENTE").readOnly = true;
                widget.getField("NMCLIENTE").validations = {};
            }
            else {
                validations = {};
                widget.getField("NRINSCRICAOPARC").readOnly = true
                this.alterarObrigatoriedadeCamposConvertidoEmCliente(widget, validations);

                widget.getField("NMCLIENTE").readOnly = false;
                widget.getField("NMCLIENTE").validations = { "required": true };
            }

        }
        else {
            validations = {};
            widget.getField("NRINSCRICAOPARC").readOnly = false
            this.alterarObrigatoriedadeCamposConvertidoEmCliente(widget, validations);

            widget.getField("IDCRIANOVOCLIENTE").readOnly = true
            widget.getField("IDCRIANOVOCLIENTE").validations = {};
            widget.currentRow.IDCRIANOVOCLIENTE = null;

            widget.getField("NMCLIENTE").readOnly = true;
            widget.getField("NMCLIENTE").validations = {};
        }
        
    };

    this.alterarObrigatoriedadeCamposConvertidoEmCliente = function (widget, validations) {
        var tela = widget;
        var fields = [
            tela.getField("NMESTADO"),
            tela.getField("NMMUNICIPIO"),
            tela.getField("NMPAIS"),
            tela.getField("DSENDERECO"),
            widget.getField("NRINSCRICAOPARC")
        ];

        fields.forEach(function (field) {
            field.validations = validations;
        });

        if(widget.currentRow.IDCONVERTIDOFILIAL == "S") {
            widget.getField("NRINSCRICAOPARC").readOnly = false;
            widget.getField("NRINSCRICAOPARC").validations = { "required": true };
        };
    };

    this.filtrarLocalAtendimentoSuperior = function (fieldLocalAtendSup) {
        var wdgLocalAtendimento = fieldLocalAtendSup.widget;
        var nrestruturam = wdgLocalAtendimento.currentRow.NRESTRUTURAM;
        if (nrestruturam) {
            fieldLocalAtendSup.dataSourceFilter = [{
                "name": "NRESTRUTURAM",
                "operator": "<>",
                "value": nrestruturam
            }];
        }
        else {
            fieldLocalAtendSup.dataSourceFilter = [];
        }

        fieldLocalAtendSup.reload();
    };

    this.convertidoEmUnidadeOnChange = function (widget) {
        
        var currentRow = widget.currentRow;
        var validations = null;
        var readOnly = null;

        if (currentRow.IDCONVERTIDOFILIAL == "S") {

            widget.getField("IDCRIANOVAUNIDADE").readOnly = false
            widget.getField("IDCRIANOVAUNIDADE").validations = { "required": true };

            if (currentRow.IDCRIANOVAUNIDADE == "S") {
                validations = { "required": true };
                readOnly = false;
                this.alterarObrigatoriedadeCamposConvertidoEmUnidade(widget, validations, readOnly);

                widget.getField("NMFILIAL").readOnly = true;
                widget.getField("NMFILIAL").validations = {};
            }
            else {
                validations = {};
                readOnly = true;
                this.alterarObrigatoriedadeCamposConvertidoEmUnidade(widget, validations, readOnly);

                widget.getField("NMFILIAL").readOnly = false;
                widget.getField("NMFILIAL").validations = { "required": true };
            }
        }
        else {
            validations = {};
            readOnly = true;
            this.alterarObrigatoriedadeCamposConvertidoEmUnidade(widget, validations, readOnly);

            widget.getField("IDCRIANOVAUNIDADE").readOnly = true
            widget.getField("IDCRIANOVAUNIDADE").validations = {};
            widget.currentRow.IDCRIANOVAUNIDADE = null;

            widget.getField("NMFILIAL").readOnly = true;
            widget.getField("NMFILIAL").validations = {};
        }
       
    };

    this.alterarObrigatoriedadeCamposConvertidoEmUnidade = function (widget, validations, readOnly) {
        var fieldsUnidade = [
            widget.getField("CDFILIAL"), //Código da Unidade
            widget.getField("NMEMPRESA"), //Empresa
            widget.getField("CDINSCESTA"), //Inscrição Estadual
            widget.getField("CDINSCMUNI"), //Inscrição Municipal
            widget.getField("NMRAZSOCFILI"), //Razão Social
            widget.getField("NRINSCRICAOPARC") //Inscrição Jurídica
        ];

        var fieldsEndereco =[
            widget.getField("CDPAIS"),
            widget.getField("SGESTADO"),
            widget.getField("CDMUNICIPIO"),
            widget.getField("DSENDERECO"),
            widget.getField("NRIMOVELENDERECO"),
            widget.getField("NMBAIRROENDERECO"),
            widget.getField("NRCEPENDERECO")
        ];

        fieldsUnidade.forEach(function (field) {
            field.validations = validations;
            field.readOnly = readOnly;
        });

        fieldsEndereco.forEach(function (field) {
            field.validations = validations;
        });

        if(widget.currentRow.IDCONVERTIDOCLIENTE == "S") {
            widget.getField("NRINSCRICAOPARC").readOnly = false;
            widget.getField("NRINSCRICAOPARC").validations = { "required": true };
        };
    };

    this.empresaOnChange = function (widget) {
        widget.getField("CDINSCESTA").reload();
        widget.getField("CDINSCMUNI").reload();
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Sm00029000_Local_Atendimento_Controller', Sm00029000_Local_Atendimento_Controller);
});

// FILE: ./assets/js/controllers/Sm00029001_Tipo_Local_Atendimento_Controller.js
function Sm00029001_Tipo_Local_Atendimento_Controller(General, ScreenService, templateManager ) {
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget
    */
    this.salvarTipoLocalAtend = function(widget){
        
        widget.currentRow.__columns_to_return = ["NRTIPOESTRUTURA", "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset && response.dataset.columnsToReturn[0]){
                widget.currentRow.NRTIPOESTRUTURA = response.dataset.columnsToReturn[0].NRTIPOESTRUTURA;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
            
        }.bind(this));
        
    };
    
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00029001_Tipo_Local_Atendimento_Controller', Sm00029001_Tipo_Local_Atendimento_Controller);
});

// FILE: ./assets/js/controllers/Sm00031000_MapaRecursosAgendados_Controller.js
function Sm00031000_MapaRecursosAgendados_Controller(restEngine, ScreenService, templateManager, localStorageEngine, ApplicationContext) {
   
    /**
     * Variaveis globais para o metodos initmap
     * 
     * @param {nulo} widget de 
     * Mapa de Agendamento
    */
    var map;
    var marker;
    var markers = [];
    var locaisAtend = [];
    var latlngbounds;
    var infowindow;
    var primeiraVez = true;
    var self = this;
    var filterRow = {};
    
   /**
     * função que retorna a url do mapa da widget ativa
     * 
     * @param {nulo} widget de 
     * Mapa de Agendamento
     */
    this.getSRC = function() {
        var widget = templateManager.container.getActiveWidget();
        return widget.mapUrl;
    };
     /**
     * função que retorna a url com a chave da API do google maps 
     * 
     * @param {nulo} widget de 
     * Mapa de Agendamento
     */
    this.getMap = function() {
        return "https://maps.googleapis.com/maps/api/js?key=AIzaSyA5vRtFek_17V-Q6t4bhTYpSCSunxDPzzM&callback=initMap()";
    };
    
     /**
     *Função recursiva que gera os marcadores no mapa de acordo com os resultados no dataset
     * 
     * @param {Dataset, interger, string } widget de 
     * Mapa de Agendamento
     **/
    this.adicionarMarcador = function(dataset, indice, geocoder) {
        if (indice < dataset.length) {
            var row = dataset[indice];
            if (locaisAtend.includes(row.NRESTRUTURAM)) {
                self.adicionarMarcador(dataset, indice + 1, geocoder);
            } else {
                locaisAtend.push(row.NRESTRUTURAM);
                if (row.DSLATITUDE && row.DSLONGITUDE) {
                    marker = new google.maps.Marker({
                        position: new google.maps.LatLng(row.DSLATITUDE, row.DSLONGITUDE),
                        title: "Recursos alocados",
                        map: map
                    });
                    latlngbounds.extend(marker.position);
                    markers.push(marker);

                    self.configMarcador(marker, row.AGENDAMENTO, row.NMESTRUTURAM, row.NRESTRUTURAM);
                    self.adicionarMarcador(dataset, indice + 1, geocoder);
                   
                } else {
                    var address = row.DSENDERECOCOMPLETO;
                    if (row.DSENDERECO){
                        geocoder.geocode({
                            'address': address
                        }, function(results, status) {
                            if (status == 'OK') {
                                marker = new google.maps.Marker({
                                    map: map,
                                    position: results[0].geometry.location
                                });
                               latlngbounds.extend(marker.position);
                                markers.push(marker);
                                
                               
                                self.configMarcador(marker, row.AGENDAMENTO, row.NMESTRUTURAM, row.NRESTRUTURAM);
                                self.adicionarMarcador(dataset, indice + 1, geocoder);
                                
                            } else {
                                console.error(status);
                                ScreenService.showMessage("Alguns locais de atendimento não foram mostrados devido à parâmetros de endereço não reconhecidos");
                            }
                        });
                   } else {
                        ScreenService.showMessage("Alguns Locais de Atendimento não possuem endereço cadastrados");
                   } 
                }
            }
             
        } else {
            self.agrupaMarcadores(map,markers);
            if (markers.length == 1) {
                map.setCenter(markers[0].position);
                map.setZoom(18);
            }
            else {
               map.fitBounds(latlngbounds);
            }
        }
    };
      
     /**
     *Função que agrupa os marcadores mais próximos de acordo com o zoom da tela no mapa
     * 
     * @param {object, Array } widget de 
     * Mapa de Agendamento
     **/
    this.agrupaMarcadores = function(map,marker){
        var markerCluster = new MarkerClusterer(map, markers, {imagePath: 'https://developers.google.com/maps/documentation/javascript/examples/markerclusterer/m'});
    };
    
    /**
     *Função que inicializa o  mapa na widget 
     *Inicia a busca da longitude e latitude dos locais de atendimento 
     *mo mapa
     * 
     * @param {Dataset } widget de 
     * Mapa de Agendamento
     **/
    this.initMap = function(dataset) {
        primeiraVez = false;
        var center = {
            lat: -19.931590,
            lng: -43.939584
        };
        
        templateManager.mapPreviousPosition = [];
        map = new google.maps.Map(document.getElementById('map'), {
            center: center,
            zoom: 12
        });
        var geocoder = new google.maps.Geocoder();
        latlngbounds = new google.maps.LatLngBounds();
        
        markers = [];
        locaisAtend = [];
        if (!(dataset instanceof Array)) {
            localStorageEngine.download("product#/crud/fct_mapa_agendamento", []).then(function(dataset) {
                if(dataset.length > 0){
                    self.adicionarMarcador(dataset, 0, geocoder);
                }

            }).catch(function(error) {
                console.error(error);
            });
        } else {
            if(dataset.length > 0){
                self.adicionarMarcador(dataset, 0, geocoder);
            }
        }

    };
    
     /**
     *Função que configura o marcardores com as informações do infobox
     * 
     * @param {object, string, integer, string } widget de 
     * Mapa de Agendamento
     */
    this.configMarcador = function(marker, AGENDAMENTO, NMESTRUTURAM, NRESTRUTURAM) {
       infowindow = new google.maps.InfoWindow();
        marker.addListener('mouseover', function() {
            infowindow.setContent(NMESTRUTURAM);
            infowindow.open(map, marker);
        });
        marker.addListener('mouseout', function() {
            infowindow.setContent(NMESTRUTURAM);
            infowindow.close(map, marker);
        });
        marker.addListener('click', function() {
            map.setCenter(marker.getPosition());
            self.setaParamentros(NRESTRUTURAM);
            var wdgrecursosNoMapaTabbed = templateManager.container.getWidget('wdgrecursosNoMapaTabbed');
            var wdgRecursosNoMapa = templateManager.container.getWidget('wdgrecursosNoMapa');
            ScreenService.openSwipe(wdgrecursosNoMapaTabbed).then(function() {
                wdgRecursosNoMapa.reload();
                ScreenService.openSwipe(wdgrecursosNoMapaTabbed);
            });
        });
    };
    
    /**
     * seta o parametro para o filtro de locais de atedimentos na widget wdgRecursosNoMapa
     * 
     * @param {interger} widget de 
     * wdgRecursosNoMapa
     */
    this.setaParamentros = function(localDeatendimento) {
        var wdgRecursosNoMapa = templateManager.container.getWidget('wdgrecursosNoMapa');
        wdgRecursosNoMapa.dataSourceFilter[0].value = localDeatendimento;
    };
    
     /**
     *Função que abre o filtro na widget mapa de agendamento
     * 
     * @param {nulo} widget de 
     * Mapa
     */
    this.openFilter = function() {
        var filterMap = templateManager.container.getWidget('filterMap');
        var ZhFilterPreferences = ApplicationContext.ZhFilterPreferences;

        ZhFilterPreferences.openFilterWidget(filterMap, filterRow);
    };
    
    /**
     * Guarda os dados da última filtragem 
     * @params{Object} widgetFilter widget de filtro
    **/
    this.backupFilter = function(widgetFilter){
        var fieldsToApplyDefaultValue = widgetFilter.fields;
        fieldsToApplyDefaultValue.forEach(function(fieldName) {
            var field = widgetFilter.getField(fieldName.name);
            if (field) {
                filterRow[fieldName.name] = field.value();
            }
        }.bind(this));
    };
    
     /**
     *Função que fecha o Swipe dos marcadores e reativa a widiget
     * 
     *@param {Object} widget widget de 
     * wdgrecursosNoMapa
     */
    this.voltar = function(widget) {
        var mapa = templateManager.container.getWidget('mapa');
        mapa.activate();
        ScreenService.closeSwipe();
    };
    
     /**
     *Função que recarrega o mapa entre navegaçôes no sistema
     * 
     *@param {Object} widget widget de 
     * Filto
     
    this.recarregar = function(widiget){
        if (primeiraVez == false) {
            var filtro = templateManager.container.getWidget('filterMap');
            filtro.reload();
        }

    };
    **/
}
Configuration(function(ContextRegister) {
    ContextRegister.register('Sm00031000_MapaRecursosAgendados_Controller', Sm00031000_MapaRecursosAgendados_Controller);
});


// FILE: ./assets/js/controllers/Sm00034000_Maximo_Carencia_Proposta_Controller.js
function Sm00034000_Maximo_Carencia_Proposta_Controller(General, ScreenService, Sm00034000_Maximo_Carencia_Proposta_Service, templateManager, localStorageEngine) {
    
    var arrayDasFaixasAtuais = [];
    
    /**
     * Salva o novo registro
     * e seta IDATIVO
     * 
     * @param {object} widget widget de 
     * Carência Máxima por Operador 
     * para salvar o novo registro
    */
    this.saveCarenciaOperador = function(widget){
        
        widget.currentRow.__columns_to_return = ["NRCARENCIAOPER", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                widget.currentRow.NRCARENCIAOPER = response.dataset.columnsToReturn[0].NRCARENCIAOPER;
                widget.currentRow.NRORG          = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO     = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO  = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
            }
            
        }.bind(this));
    };
    

    /**
     * A função "validaCarenciaOperador" verifica se já existe 
     * uma carencia cadastrada para o operador indicado.
     */

    this.validaCarenciaOperador = function(widget){
        
       var parcOperador = widget.currentRow.NRPARCNEGOCIO;
       
       if (widget.currentRow.__is_new == true){
           widget.currentRow.NRDAISMAXCARENCIA = 9999;
           Sm00034000_Maximo_Carencia_Proposta_Service.validaCarenciaOperador(parcOperador).then(function(response){
                if (response.dataset.VALIDAOPER.VALIDA == false) {
                      this.saveCarenciaOperador(widget);  
                }
                else{
                      ScreenService.showMessage('Operação Bloqueada. <br>Já existe carência máxima cadastrada para o operador informado.'); 
                }
    
           }.bind(this)).catch(function(response){
               ScreenService.showMessage(response.data.error);
           });
           
       }
       else{
            this.saveCarenciaOperador(widget); 
       }
       
    };
       /**
     * Função utilizada para validar a current row da widget Faixa de Desconto
     * @params {Row} a current row da widget Faixa de Desconto
     * @returns {Boolean} widget esta valida?
     * 
    */
    this.validaRowSaveFaixaDesconto = function(row){
        var rowPai = this.pegarRowDaWidget("gridCarenciaOperProp");
        var params = {
            "NRCARENCIAOPER" : rowPai.NRCARENCIAOPER,
            "VRINI" : row.VRPERCDESCDE,
            "VRFIN" : row.VRPERCDESCATE
        };
        return General.callRestEngine("product#/fct_carenciaperfil_perfilacesso/pegarBetweenCarencia", "Row", params);
    };
    
    /**
     * Função utilizada para gerar a row quw deverá ser usada no save da Faixa de Desconto
     * @params {Row} a currentRow da widget de Faixa de Desconto
     * @returns {Object} usado para salvar no currentRestEngine
    */
    this.getParamsSaveFaixaDesconto = function(row){
        return (row.__is_new) ? {
            "NRCARENCIAOPER"    : row.NRCARENCIAOPER,
            "VRPERCDESCATE"     : row.VRPERCDESCATE,
            "VRPERCDESCDE"      : row.VRPERCDESCDE,
            "NRCARENCIAFAIXA"   : row.NRPARCELASMAX,
            "NRPARCELASMAX"     : row.NRPARCELASTOTAL,
            "INSERIR"           : row.__is_new
        } : {
            "NRCARENCIAOPER"        : row.NRCARENCIAOPER,
            "VRPERCDESCATE"         : row.VRPERCDESCATE,
            "VRPERCDESCDE"          : row.VRPERCDESCDE,
            "NRCARENCIAFAIXA"       : row.NRPARCELASMAX,
            "NRPARCELASMAX"         : row.NRPARCELASTOTAL,
            "INSERIR"               : false,
            "NRCARENCIAFAIXADESC"   : row.NRCARENCIAFAIXADESC
        };
    };
    
    this.salvarEMostrarMensagem = function(pode,widget,params){
        if(pode.status){
           General.callRestEngine("product#/fct_carenciaperfil_perfilacesso/saveFaixaDesc", "Row", params).then(function(response) {
                widget.list();
                widget.reload();
                var resposta =  response.dataset.request.resulted;
                ScreenService.closeSwipe();
                ScreenService.showMessage(ScreenService.i18n(resposta));
            }).catch(function(err){
                ScreenService.showMessage(ScreenService.i18n(err));
            });
        } else{
            ScreenService.showMessage(ScreenService.i18n(pode.message));
        }
    };
    
    /**
     * Função pública que valida e salva a widget da Faixa de Desconto 
     * @params {Widget} widget da Faixa de Desconto
     * @returns {Object | Error} Um objeto representando a resposta do backend ou então um erro
    */
    this.validaFaixaDeDescontoEInsere = function(widget){
        this.puxarOperador(widget);
        var row = widget.currentRow;
        var params =  this.getParamsSaveFaixaDesconto(row);
        if(row.__is_new){
            this.validaRowSaveFaixaDesconto(row).then(function(response){
                var pode = this.pegarStatusAtualDeFaixaDesconto(row, response);
                this.salvarEMostrarMensagem(pode, widget,params);
            }.bind(this));
        }else{
            var response =  {"dataset":{"request":{"resulted":"0"}}};
            var pode = this.pegarStatusAtualDeFaixaDesconto(row, response);
            this.salvarEMostrarMensagem(pode, widget,params);
        }
    };
    
    /**
     * Salva a faixa de desconto 
     * @params {Widget} widget da faixa de desconto que deve ser salva
    */
    this.saveFaixadeDesconto = function(widget){
        try{
            this.validaFaixaDeDescontoEInsere(widget);
        }catch(e){
            ScreenService.showMessage(ScreenService.i18n(e));
        }
    }; 
    
    /**
     * Puxa o campo do numero e nome do perfil da widget pai. 
     * @params {Widget} widget da faixa de desconto que deve ser alterada
    */
    this.puxarOperador = function(widget){
        var rowPai = this.pegarRowDaWidget("gridCarenciaOperProp");
        if(rowPai){
            widget.currentRow.NRCARENCIAOPER = rowPai.NRCARENCIAOPER;
            widget.currentRow.NMPRINCIPALPARC = rowPai.NMPRINCIPALPARC;
            widget.currentRow.NRPARCELASTOTAL = rowPai.NRPARCELASMAX;
            widget.currentRow.VRPERCMAXDE = rowPai.VRPERCMAXATE;
        }
    };
    
    
    this.filtrarDataSourceFaixaDesconto = function(widget){
        var rowPai = this.pegarRowDaWidget("gridCarenciaOperProp");
        this.pegarDataSourceFiltrado(rowPai.NRCARENCIAOPER).then(function(data){
            widget.dataSource.data = data;
        });
    };
    
    this.pegarNomeAcaoAtivaCarencia = function(widget){
        this.chamadaBackendAcaoCarencia(true);
    };
    
    
    this.alterarNomeAcaoCarencia = function(widget){
        this.chamadaBackendAcaoCarencia(false);
    };
    
    this.pegarDataSourceFiltrado = function(nrparcnegocio){
        var filter = [
            {
                "name" : "NRCARENCIAOPER",
                "operator" : "=",
                "value" : nrparcnegocio 
            }
        ];
        return this.pegarDataSource("fct_carenciafaixadesconto_operador", filter);
    };
    
    
    this.acaoAdicionarFaixaDesconto = function(widget){
        // var quantidadeFaixas = this.pegarWidget("descPorParcelas").dataSource.data.length;    
        // var carenciaMaxima = this.pegarWidget("gridCarenciaOperProp").currentRow.NRPARCELASMAX;
        // if(quantidadeFaixas < carenciaMaxima){
           General.widgetNewRow(widget);
           this.puxarOperador(widget);
        // }else{
        //     ScreenService.showMessage(ScreenService.i18n("Não é possível adicionar mais faixas de descontos pois excedeu o limite máximo para este Operador."));
        // }
    };
    
    /* REGION UTILS*/
    
    this.filterDataSource = function(data, condicao){
        var result = data.filter(function(d){
            return condicao(d);
        }) ;
        return result;
    };
    
    /**
     * Função usada para encapsular a função de pegar uma widget baseada no nome.
     * @params {string} nome da widget
     * @returns Widget widget encontrada pelo nome ou gera erro quando nao encontrada.
    */
    this.pegarWidget = function(nome){
        return templateManager.container.getWidget(nome);
    };
    
    /**
     * Pega a row de uma determinada widget. 
     * @params {string} nome da widget 
     * @returns Object | null  a row ou nulo
    */
    this.pegarRowDaWidget = function(nome){
        var widget = this.pegarWidget(nome);
        return (widget) ? widget.currentRow : null; 
    };
    
    /**
     * Funcao usada para encapsular o procedimento de chamada do backend as rotas são quase iguais e o funcionamento é o mesmo portanto foram unidos em uma unica chamada de metodo.
     * @params {boolean} deve usar a rota de pegar? Senão usara a rota de atualizar
    */
    this.chamadaBackendAcaoCarencia = function(pegar) {
        if (pegar === undefined) {
            pegar = false;
        }
        var params = {};
        var resposta = false;
       General.callRestEngine('product#/fct_parametrogeral_widget/' + ((pegar) ? 'pegar':'atualizar') + 'IdControleCarencia', "Row", params).then(function(response) {
            if (!pegar)
                ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
                
            resposta =  ( response.dataset.request.resulted !== "N");
            var ativarWdg = templateManager.container.getWidget('gridCarenciaOperProp').getAction("habCarenciaMaxima");
            ativarWdg.label = resposta ? "Desativ. Controle Carência" : "Ativar Controle de Carência";  
        }).catch(function(err){
            console.log(err);
        });
    };
    
    /**
     * Pega a maior/menor widget baseada em uma condicao
     * @params{Array} array de dados a ser percorrido
     * @params{Function} função usada no teste que retorna boolean e recebe dois parametros
     * @return Widget que passou nos testes 
    */
    this.pegarWidgetPorCondicao = function (arr, condicao) {
        return arr.reduce(function (p, c) {
            return ( condicao(p,c) ? p : c );
        });
    };
    
    this.pegarDataSource = function(rota, filter){
        if (filter === undefined) {
            filter = [];
        }

        var rotaReal = 'product#/crud/' + rota;
        return localStorageEngine.download(rotaReal, filter);
    };
    
    this.pegarStatusAtualDeFaixaDesconto = function(row, response){
        var status = { status : true };   
        const block = "Operação bloqueada! <br> Essa faixa não pode ser inserida pois";
        if(row.NRPARCELASMAX > parseFloat(row.NRPARCELASTOTAL)  || parseFloat(row.VRPERCDESCATE) > parseFloat(row.VRPERCMAXDE)){
            status = { status : false, message: block + 'os valores inseridos representando o número máximo de parcelas ou o percentual máximo de desconto ultrapassam o limite parametrizado na aba da Carência Máxima do Operador.' };
        } else if(response.dataset.request.resulted !== "0"){
            status = { status : false, message: block + 'já existe uma faixa de desconto cadastrado nesse período.'};
        }
        return status;
    };
    
    /* ENDREGION UTILS*/
    
}
Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00034000_Maximo_Carencia_Proposta_Controller', Sm00034000_Maximo_Carencia_Proposta_Controller);
});

// FILE: ./assets/js/controllers/Sm00035000_Maximo_Carencia_Proposta_perfilAcesso_Controller.js
function Sm00035000_Maximo_Carencia_Proposta_perfilAcesso_Controller(General, ScreenService, restEngine, templateManager, localStorageEngine) {
    
    var arrayDasFaixasAtuais = [];
    
    /**
     * Salva o novo registro
     * e seta IDATIVO
     * 
     * @param {object} widget widget de 
     * Carência Máxima por Operador 
     * para salvar o novo registro
    */
    this.saveCarenciaPerfilAcesso = function(widget){
        
        widget.currentRow.__columns_to_return = ["NRCARENCIAPERFIL", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                widget.currentRow.NRCARENCIAPERFIL = response.dataset.columnsToReturn[0].NRCARENCIAPERFIL;
                widget.currentRow.NRORG          = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO     = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO  = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
            }
            
        }.bind(this));
    };
    

    /**
     * A função "validaCarenciaOperador" verifica se já existe 
     * uma carencia cadastrada para o operador indicado.
     */

    this.validaCarenciaPerfilAcesso = function(widget){
        
       var perfilDeAcesso = widget.currentRow.NRPERFILACESSO;
       
       if (widget.currentRow.__is_new == true){
           this.validaCarencia(perfilDeAcesso).then(function(response){
                if (response.dataset.VALIDAPERF.VALIDA == false) {
                      this.saveCarenciaPerfilAcesso(widget);  
                }
                else{
                      ScreenService.showMessage('Operação Bloqueada. Já existe carência máxima cadastrada para o perfil de acesso selecionado.'); 
                }
    
           }.bind(this)).catch(function(response){
               ScreenService.showMessage(response.data.error);
           });
           
       }
       else{
            this.saveCarenciaPerfilAcesso(widget); 
       }
       
    };
    
     this.validaCarencia = function(perfilDeAcesso){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'NRPERFILACESSO': perfilDeAcesso}
        };
        
        return restEngine.post('product#/fct_carenciaperfil_perfilacesso/validaCarenciaPerfilAcesso', paramsRestEngine);
        
    };
    
    /**
     * Função utilizada para validar a current row da widget Faixa de Desconto
     * @params {Row} a current row da widget Faixa de Desconto
     * @returns {Boolean} widget esta valida?
     * 
    */
    this.validaRowSaveFaixaDesconto = function(row){
        var rowPai = this.pegarRowDaWidget("gridCarenciaPerfProp");
        var params = {
            "NRPERFILACESSO" : rowPai.NRCARENCIAPERFIL,
            "VRINI" : row.VRPERCDESCDE,
            "VRFIN" : row.VRPERCDESCATE
        };
        return General.callRestEngine("product#/fct_carenciaperfil_perfilacesso/pegarBetweenCarencia", "Row", params);
    };
    
    /**
     * Função utilizada para gerar a row quw deverá ser usada no save da Faixa de Desconto
     * @params {Row} a currentRow da widget de Faixa de Desconto
     * @returns {Object} usado para salvar no currentRestEngine
    */
    this.getParamsSaveFaixaDesconto = function(row){
        return (row.__is_new) ? {
            "NRPERFILACESSO"    : row.NRCARENCIAPERFIL,
            "VRPERCDESCATE"     : row.VRPERCDESCATE,
            "VRPERCDESCDE"      : row.VRPERCDESCDE,
            "NRCARENCIAFAIXA"   : row.NRPARCELASMAX,
            "NRPARCELASMAX"     : row.NRPARCELASTOTAL,
            "INSERIR"           : true 
        } : {
            "NRPERFILACESSO"        : row.NRCARENCIAPERFIL,
            "VRPERCDESCATE"         : row.VRPERCDESCATE,
            "VRPERCDESCDE"          : row.VRPERCDESCDE,
            "NRCARENCIAFAIXA"       : row.NRPARCELASMAX,
            "NRPARCELASMAX"         : row.NRPARCELASTOTAL,
            "INSERIR"               : false,
            "NRCARENCIAFAIXADESC"   : row.NRCARENCIAFAIXADESC
        };
    };
    
    this.salvarEMostrarMensagem = function(pode,widget,params){
        if(pode.status){
           General.callRestEngine("product#/fct_carenciaperfil_perfilacesso/saveFaixaDesc", "Row", params).then(function(response) {
                widget.list();
                widget.reload();
                var resposta =  response.dataset.request.resulted;
                ScreenService.closeSwipe();
                ScreenService.showMessage(ScreenService.i18n(resposta));
            }).catch(function(err){
                ScreenService.showMessage(ScreenService.i18n(err));
            });
        } else{
            ScreenService.showMessage(ScreenService.i18n(pode.message));
        }
    };
    
    /**
     * Função pública que valida e salva a widget da Faixa de Desconto 
     * @params {Widget} widget da Faixa de Desconto
     * @returns {Object | Error} Um objeto representando a resposta do backend ou então um erro
    */
    this.validaFaixaDeDescontoEInsere = function(widget){
        this.puxarPerfilAcesso(widget);
        var row = widget.currentRow;
        var params =  this.getParamsSaveFaixaDesconto(row);
        if(row.__is_new){
            this.validaRowSaveFaixaDesconto(row).then(function(response){
                var pode = this.pegarStatusAtualDeFaixaDesconto(row, response);
                this.salvarEMostrarMensagem(pode,widget,params);
            }.bind(this));
        }else{
            var response =  {"dataset":{"request":{"resulted":"0"}}};
            var pode = this.pegarStatusAtualDeFaixaDesconto(row, response);
            this.salvarEMostrarMensagem(pode,widget,params);
        }
    };
    
    /**
     * Salva a faixa de desconto 
     * @params {Widget} widget da faixa de desconto que deve ser salva
    */
    this.saveFaixadeDesconto = function(widget){
        try{
            this.validaFaixaDeDescontoEInsere(widget);
        }catch(e){
            ScreenService.showMessage(ScreenService.i18n(e));
        }
    };
    
    /**
     * Puxa o campo do numero e nome do perfil da widget pai. 
     * @params {Widget} widget da faixa de desconto que deve ser alterada
    */
    this.puxarPerfilAcesso = function(widget){
        var rowPai = this.pegarRowDaWidget("gridCarenciaPerfProp");
        if(rowPai){
            widget.currentRow.NMPERFILACESSO = rowPai.NMPERFILACESSO;
            widget.currentRow.NRCARENCIAPERFIL = rowPai.NRCARENCIAPERFIL;
            widget.currentRow.NRPARCELASTOTAL = rowPai.NRPARCELASMAX;
            widget.currentRow.VRPERCMAXADE = rowPai.VRPERCMAXATE;
        }
    };
    
    
    this.filtrarDataSourceFaixaDesconto = function(widget){
        var rowPai = this.pegarRowDaWidget("gridCarenciaPerfProp");
        this.pegarDataSourceFiltrado(rowPai.NRPERFILACESSO).then(function(data){
            widget.dataSource.data = data;
        });
    };
    
    this.pegarDataSourceFiltrado = function(nrperfil){
        var filter = [
            {
                "name" : "NRCARENCIAPERFIL",
                "operator" : "=",
                "value" : nrperfil 
            }
        ];
        return this.pegarDataSource("fct_carenciafaixadesconto", filter);
    };
    
    
    this.pegarNomeAcaoAtivaCarencia = function(widget){
        this.chamadaBackendAcaoCarencia(true);
    };
    
    
    this.alterarNomeAcaoCarencia = function(widget){
        this.chamadaBackendAcaoCarencia(false);
    };
    
    this.acaoAdicionarFaixaDesconto = function(widget){
        // var quantidadeFaixas = this.pegarWidget("descPorParcelas").dataSource.data.length;    
        // var carenciaMaxima = this.pegarWidget("gridCarenciaPerfProp").currentRow.NRPARCELASMAX;
        // if(quantidadeFaixas < carenciaMaxima){
           General.widgetNewRow(widget);
           this.puxarPerfilAcesso(widget);
        // }else{
        //     ScreenService.showMessage(ScreenService.i18n("Não é possível adicionar mais faixas de descontos pois excedeu o limite máximo para este Perfil."));
        // }
    };
    
    /* REGION UTILS*/
    
    /**
     * Pega e faz o download de um datasource
     * @params {String} nome do datasource
     * @params {Object} filtro a ser passado
     * @return Promise download do datasource
    */
    this.pegarDataSource = function(rota, filter){
        if (filter === undefined) {
            filter = [];
        }

        var rotaReal = 'product#/crud/'+ rota;
        return localStorageEngine.download(rotaReal, filter);
    };
    
    /**
     * Função usada para encapsular a função de pegar uma widget baseada no nome.
     * @params {string} nome da widget
     * @returns Widget widget encontrada pelo nome ou gera erro quando nao encontrada.
    */
    this.pegarWidget = function(nome){
        return templateManager.container.getWidget(nome);
    };
    
    /**
     * Pega a row de uma determinada widget. 
     * @params {string} nome da widget 
     * @returns Object | null  a row ou nulo
    */
    this.pegarRowDaWidget = function(nome){
        var widget = this.pegarWidget(nome);
        return (widget) ? widget.currentRow : null; 
    };
    
    
    this.chamadaBackendAcaoCarencia = function(pegar){
        if (pegar === undefined) {
            pegar = false;
        }

        var params = {};
        var resposta = false;
       General.callRestEngine('product#/fct_parametrogeral_widget/' + (pegar) ? 'pegar':'atualizar' + 'IdControleCarencia', "Row", params).then(function(response) {
            if (!pegar)
                ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
                
            resposta =  ( response.dataset.request.resulted === "N") ;
            var ativarWdg = templateManager.container.getWidget('gridCarenciaPerfProp').getAction("habCarenciaMaxima");
            ativarWdg.label = resposta ? "Ativar Controle de Carência" : "Desativ. Controle Carência";  
        }).catch(function(err){
            console.log(err);
        });
    };
    
    /**
     * Pega a maior/menor widget baseada em uma condicao
     * @params{Array} array de dados a ser percorrido
     * @params{Function} função usada no teste que retorna boolean e recebe dois parametros
     * @return Widget que passou nos testes 
    */
    this.pegarMaiorWidget = function (arr, condicao) {
      return arr.reduce(function (p, c) {
        return ( condicao(p,c) ? p : c );
      });
    };
    
    this.pegarStatusAtualDeFaixaDesconto = function(row, response){
        var status = { status : true };   
        const block = "Operação bloqueada! <br> Essa faixa não pode ser inserida pois";
        if(row.NRPARCELASMAX > parseFloat(row.NRPARCELASTOTAL)  || parseFloat(row.VRPERCDESCATE) > parseFloat(row.VRPERCMAXADE)){
            status = { status : false, message: block + 'os valores inseridos representando o número máximo de parcelas ou o percentual máximo de desconto ultrapassam o limite parametrizado na aba da Carência Máxima do Operador.' };
        } else if(response.dataset.request.resulted !== "0"){
            status = { status : false, message: block + 'já existe uma faixa de desconto cadastrado nesse período.'};
        }
        return status;
    };
    /* ENDREGION UTILS*/
}
Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00035000_Maximo_Carencia_Proposta_perfilAcesso_Controller', Sm00035000_Maximo_Carencia_Proposta_perfilAcesso_Controller);
});

// FILE: ./assets/js/controllers/Sm00041000_Modalidade_Pagamento_Controller.js
function Sm00041000_Modalidade_Pagamento_Controller(Utilities, General, templateManager) {
    
    /**
     * Função para habilitar a visualização 
     * do 'field group' referente ao módulo
     * selecionado.
     * 
     * @param {object} widget widget onde se
     * localiza os FieldGroups.
     * 
    */
    this.activeGroupByTheSelectedModel = function(widget){
            
        var fieldGroups = widget.fieldGroups;
        
        var currentRowNewModalityOfPayment = widget.currentRow;
        
        var indexGroupFixedPaymentDay = Utilities.getFieldGroupPosition(fieldGroups, "PAGAMENTO_FIXO_DIA");
        var groupFixedPaymentDay = fieldGroups[indexGroupFixedPaymentDay];
        
        var indexGroupFormOfPayment = Utilities.getFieldGroupPosition(fieldGroups, "FORMA_PAGAMENTO");
        var groupFormOfPayment = fieldGroups[indexGroupFormOfPayment];
        
        if(currentRowNewModalityOfPayment.IDTPFORPGTOMOD === "F"){
            
            groupFixedPaymentDay.isVisible = true;
            
            groupFormOfPayment.isVisible = false;
            this.clearValuesFieldGorup(widget, groupFormOfPayment.name);
            
        }else if(currentRowNewModalityOfPayment.IDTPFORPGTOMOD === "V"){
            
            groupFormOfPayment.isVisible = true;
            
            groupFixedPaymentDay.isVisible = false;
            this.clearValuesFieldGorup(widget, groupFixedPaymentDay.name);
            
        }else{
            
            groupFormOfPayment.isVisible = false;
            groupFixedPaymentDay.isVisible = false;
            
            this.clearValuesFieldGorup(widget, groupFormOfPayment.name);
            this.clearValuesFieldGorup(widget, groupFixedPaymentDay.name);
            
        }
        
    };
    
    /**
     * Função para ocultar os fieldGroups
     * ligados ao tipo de modalidade quando
     * não estiver em um abiente de edição
     * (para não esconder informações no modo 'view').
     * 
     * @param {object} widget widget onde se
     * localiza os fieldGroups.
    */
    this.hideFieldGroupsOfModality = function(widget){
        
        var fieldGroups = widget.fieldGroups;
        
        var indexGroupFixedPaymentDay = Utilities.getFieldGroupPosition(fieldGroups, "PAGAMENTO_FIXO_DIA");
        var groupFixedPaymentDay = fieldGroups[indexGroupFixedPaymentDay];
        
        var indexGroupFormOfPayment = Utilities.getFieldGroupPosition(fieldGroups, "FORMA_PAGAMENTO");
        var groupFormOfPayment = fieldGroups[indexGroupFormOfPayment];
        
        if(widget.currentRow.__is_new === true){
            
            groupFixedPaymentDay.isVisible = false;
            groupFormOfPayment.isVisible = false;
        }    
    }; 
    
    /**
     * Limpa os fields de um determinado FieldGroup.
     * 
     * @param {object} widget widget onde se localiza
     * o fieldGroup.
     * 
     * @param {String} fieldGroupName nome do FieldGroup
     * que terá os campos limpados.
    */
    this.clearValuesFieldGorup = function(widget, fieldGroupName){
        
        widget.fields.forEach(function(field){
            
            if(field.fieldGroup === fieldGroupName){
                //Utilities.clearFieldValue(field);
                field.value(null);
            }
            
        });
    };
    
    /**
     * Função para validar valores da currentRow 
     * e salvar uma nova modalidade de pagamento,
     * mostra o field group correto no modo 'edit',
     * e bloqueia a widget 'PRAZO_PAGAMENTO' caso 
     * necessário e seta a modalidade para 'A vista'
     * caso o campo esteja vazio(BANCO MAL FEITO).
     * 
     * @param {obejct} widget widget na qual será 
     * pego os valores para inserir uma nova modalidade
     * de pagamento.
    */
    this.checkValuesAndSaveModalityPayment = function(widget){
            
        this.checkIfModeIsInSight(widget);
        
        General.widgetSave(widget).then(function(){
            
            this.hideFieldGroupsOfModality(widget);
            this.activeWidgetPaymentDeadline(widget);
            
            if(!widget.currentRow.IDTPFORPGTOMOD){
                widget.currentRow.IDTPFORPGTOMOD = "A";
            }
            
            widget.reload();
            
        }.bind(this));
    };
    
    /**
     * Função para setar 'vazio' para o campo
     * 'Tipo de modalidade' caso a modalidade selecionada
     * for 'À vista'.
     * 
     * @param {object} widget widget onde se
     * localiza a field 'Tipo de modalidade'.
    */
    this.checkIfModeIsInSight = function(widget){
        
        var currentRow = widget.currentRow;
        
        if(currentRow.IDTPFORPGTOMOD === "A"){
            currentRow.IDTPFORPGTOMOD = "";
        }
          
    };
    
    /**
     * Função para ativar a widget 'PRAZO_PAGAMENTO'
     * quando o tipo da modalidade for 'Forma de pagamento'.
     * 
    */
    this.activeWidgetPaymentDeadline = function(){
        
        var widget = templateManager.container.getWidget('PRAZO_PAGAMENTO');
        
        var widModalityPayment = widget.parent;
        
        if(widModalityPayment.currentRow.IDTPFORPGTOMOD === "V"){
            
            widget.enabled = true;
            
        }else{
            
            widget.enabled = false;
            
        }
            
    };
    
    /**
     * Função para setar os valores necessários para
     * o cadastro de um novo prazo de pagamento.
     * 
     * @param {objetc(widget)} widget widget ativa, usada
     * para o cadastro.
    */
    this.setValuesForNewPaymentDeadLine = function(widget){
    
        //this.setOperatorNrorgInCurrentRow(widget);
        
        this.setModalityOfPaymentInDeadLine(widget);
        
    };
    
    /**
     * Função para setar o nrorg do operador logado
     * na currentRow da tela de cadastro de prazo de
     * pagamento.
     * 
     * @param {object(widget)} widget widget que receberá
     * o nrorg em sua currentRow.
    */
    /*this.setOperatorNrorgInCurrentRow = function(paymentDeadlineWidget){
        
        var currentRow = paymentDeadlineWidget.currentRow;
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        currentRow.NRORG = loggedOperatorInfos.NRORG;
    };*/
    
    /**
     * Função para setar o código da modalidade responsável
     * pelo prazo a ser criado.
     * 
     * @param {object} widget widget 'PRAZO_PAGAMENTO'
     * 
    */
    this.setModalityOfPaymentInDeadLine = function(widget){
        
        var modPaymentCurrentRow = widget.parent.currentRow;
        var deadlinePaymentCurrentRow = widget.currentRow;
        
        deadlinePaymentCurrentRow.CDMODPGTO = modPaymentCurrentRow.CDMODPGTO;
        deadlinePaymentCurrentRow.DSMODPGTO = modPaymentCurrentRow.DSMODPGTO;
        
    };
    
    /**
     * Função para inserir um filtro na widget
     * 'PRAZO_PAGAMENTO'.
     * 
     * @param {object} widModalityPayment widget princital
     * (MODALIDADE_PAGAMENTO)
    */
    this.insertDeadLineFilterValue = function(widModalityPayment){
        
        var widDeadLinePayment = templateManager.container.getWidget('PRAZO_PAGAMENTO');
        
        var CDMODPGTOfilterObjectIndex = Utilities.getFilterPosition(widDeadLinePayment.dataSourceFilter, 'CDMODPGTO');    
        
        widDeadLinePayment.dataSourceFilter[CDMODPGTOfilterObjectIndex].value = widModalityPayment.currentRow.CDMODPGTO;
    };
    
}

Configuration(function(ContextRegister){
   ContextRegister.register("Sm00041000_Modalidade_Pagamento_Controller", Sm00041000_Modalidade_Pagamento_Controller); 
});




// FILE: ./assets/js/controllers/Sm00042001_resultado_Controller.js
function Sm00042001_resultado_Controller(ScreenService, templateManager, Operador_Repository) {
	var self = this;


	self.setDefaultValueOperadorResultado = function(field, parametro)
	{
        Operador_Repository.getOperadorLogado().then(function(operator)
        {
        	if(operator[0] === undefined || operator[0] === null)
            {
                ScreenService.showMessage("Erro ao realizar a operação.");
                return;
            }

					field.defaultValue = operator[0][parametro];
        },function(error)
        {
            ScreenService.showMessage("Erro ao realizar a operação.");
        });
	};
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Sm00042001_resultado_Controller', Sm00042001_resultado_Controller);
});

// FILE: ./assets/js/controllers/Sm00042002_Motivo_Evolucao_Controller.js
function Sm00042002_Motivo_Evolucao_Controller(ScreenService, templateManager, Operador_Repository, General, Utilities) {

    /**
     * Função para abrir a widget de seleção
     * de arquivos.
    */
    this.openAttachPopup = function(){
        
        var fileUploadWidget = templateManager.container.getWidget('WDGT_SELECAO_ARQUIVOS');
        ScreenService.openPopup(fileUploadWidget);
            
    };
    
    /**
     * Função para fechar o Popup de upload de imagens.
     * 
     * @param {object} widget widget de upload de imagens.
    */
    this.closeAttachPopup = function(widget){
        ScreenService.closePopup();
        widget.isVisible = false;
        widget.parent.view();
        widget.parent.activate();
    };
    
    /**
     * Função para fazer o download de um arquivo
     * salvo no servidor.
     * 
     * @param {object} widget widget que contém
     * as informações do arquivo a ser baixado 
     * em sua currentRow.
    */    
    this.downloadFile = function(widget){
        var nmarquivo = widget.currentRow.NMANEXOPADRAO;
        var dscaminho = widget.currentRow.DSCAMINHOANEXOPADRAO;
        var type      = this.getFileNameWithoutExtension(nmarquivo);
        
        Utilities.downloadFile(nmarquivo, dscaminho, type);   
    };
    
    /**
     * Função para retornar o nome do arquivo sem
     * sua extenção.
     * 
     * @param {string} fileName nome completo do arquivo.
    */
    this.getFileNameWithoutExtension = function(fileName){
        return fileName.replace(/^(.*)\.(.*)$/, "$1");
    };
    
    this.changeActionDefaultAttachment = function(widget){
        var currentRow = widget.currentRow;
        
        if(Util.isEmptyOrBlank(currentRow.DSCAMINHOANEXOPADRAO) && Util.isEmptyOrBlank(currentRow.ATTACHMENT)){
            widget.getAction('DOWNLOAD').isVisible = false;
            widget.getAction('UPLOAD').isVisible = true;
            widget.getAction('UPLOAD').label = 'Adicionar arquivo padrão';
        }else{
            widget.getAction('DOWNLOAD').isVisible = true;
            widget.getAction('UPLOAD').isVisible = true;
            widget.getAction('UPLOAD').label = 'Modificar arquivo padrão';
        }
        
        widget.activate();
    };
    
    
    /**
     * Seta o anexo na widget de motivo e salva.
    */
    this.uploadDefaultFile = function(widget){
        widget.parent.currentRow.ATTACHMENT = widget.currentRow.DSCAMINHO;
        
        General.widgetSave(widget.parent).then(function(){
            ScreenService.showMessage('Arquivo salvo com sucesso!');
            this.closeAttachPopup(widget);
            this.changeActionDefaultAttachment(widget.parent);
            widget.parent.reload();    
        }.bind(this));
    };
    
	this.setDefaultValueOperador = function(field, parametro)
	{
        Operador_Repository.getOperadorLogado().then(function(operator)
        {
        	if(operator[0] === undefined || operator[0] === null)
            {
                ScreenService.showMessage("Erro ao realizar a operação.");
                return;
            }

					field.defaultValue = operator[0][parametro];
        },function(error)
        {
            ScreenService.showMessage("Erro ao realizar a operação.");
        });
	};
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Sm00042002_Motivo_Evolucao_Controller', Sm00042002_Motivo_Evolucao_Controller);
});

// FILE: ./assets/js/controllers/Sm00042003_tipo_contatoController.js
function Sm00042003_tipo_contatoController(ScreenService, templateManager, Operador_Repository) {
	var self = this;


	self.setDefaultValueOperadorResultado = function(field, parametro)
	{
        Operador_Repository.getOperadorLogado().then(function(operator)
        {
        	if(operator[0] === undefined || operator[0] === null)
            {
                ScreenService.showMessage("Erro ao realizar a operação.");
                return;
            }

					field.defaultValue = operator[0][parametro];
        },function(error)
        {
            ScreenService.showMessage("Erro ao realizar a operação.");
        });
	};
	
    this.showWonLostField = function(widget){
    	if(widget.currentRow.IDRESULTOPORTUNI === 'SIM'){
    		widget.getField('IDGANHAPERDE').isVisible = true;
    		widget.getField('IDGANHAPERDE').validations = {required:true};
    		widget.getField('IDRESULTOPORTUNI').class = "8";
    	}else{
    		widget.getField('IDGANHAPERDE').isVisible = false;
    		widget.getField('IDGANHAPERDE').validations = {};
    		widget.getField('IDRESULTOPORTUNI').class = "12";
    	}
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Sm00042003_tipo_contatoController', Sm00042003_tipo_contatoController);
});

// FILE: ./assets/js/controllers/Sm00043000_ParametrizacaoMetas_Controller.js
function Sm00043000_ParametrizacaoMetas_Controller(General, ScreenService, Utilities){
   
    /** 
     * Controla os campos de quantidade de dias e período fixo 
     * a partir da opção selecionada no campo de tipo de período
     * @param {Object} widget widget parametrização de metas 
     **/
    this.validaTpPeriodo = function(widget){
        if(!widget.currentRow.TPPERIODOMETA){
            widget.getField("CDPERIODOFIXO").readOnly     = true;
            widget.getField("QTDIASCORRIDOS").readOnly    = true;
            widget.getField("CDPERIODOFIXO").validations  = {};
            widget.getField("QTDIASCORRIDOS").validations = {};
        } else {
            if(widget.currentRow.TPPERIODOMETA === '1'){
                widget.getField("CDPERIODOFIXO").readOnly     = false;
                widget.getField("CDPERIODOFIXO").validations  = {"required": true};
                widget.getField("QTDIASCORRIDOS").readOnly    = true;
                widget.getField("QTDIASCORRIDOS").validations = {};
            } else {
                widget.getField("CDPERIODOFIXO").readOnly     = true;
                widget.getField("CDPERIODOFIXO").validations  = {};
                widget.getField("QTDIASCORRIDOS").readOnly    = false;
                widget.getField("QTDIASCORRIDOS").validations = {"required": true};
            }
        }
    };

    /***
     * @param {Object} widget widget parametrização de metas 
    */
   this.validaTpMeta = function(widget){
        if(!widget.currentRow.TPMETA){
            widget.getField("TPMEDIDAMETA").validations  = {};
            widget.getField("TPPERIODOMETA").validations = {};
        } else { 
            widget.getField("TPMEDIDAMETA").validations  = {"required": true};
            widget.getField("TPPERIODOMETA").validations = {"required": true};
            if(widget.currentRow.TPMETA === '1' || widget.currentRow.TPMETA === '2'){
                widget.getField("TPMEDIDAMETA").value("2");
                widget.getField("TPMEDIDAMETA").readOnly = true;
            } else {
                if(widget.currentRow.TPMETA === "7"){
                    widget.getField("TPMEDIDAMETA").value("1");
                    widget.getField("TPMEDIDAMETA").readOnly = true;       
                } else {
                    widget.getField("TPMEDIDAMETA").readOnly = false;
                }
            }
        }
    };

    /**
     * Salva um novo registro na tabela de parametrização de metas
     * @param {Object} widget widget parametrização de metas
    **/
    this.saveParamsMeta = function(widget){
       General.widgetSave(widget).then(function(response){
            if(response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0)
                widget.currentRow.NRPARAMSMETAS = response.dataset.columnsToReturn[0].NRPARAMSMETAS;
        }.bind(this))
        .catch(function(error){
           Utilities.showErrorMessage(error);
        });
   };

    /**
     * Salva os operadores ligados as metas e seus respectivos valores a
     * serem alcançados
     * @param {Object} widget widget parametrização de metas
    **/
    this.saveTpPerfilParamsMeta = function(widget, nameCdTipo){
        var nameDataSource = widget.dataSource.name;
        Utilities.validarCamposDuplicados(widget, ["NRPARAMPERFMETA"], [nameCdTipo, "NRPARAMSMETAS"], "product#/crud/fct_paramsperfilmeta").then(function(valida){
            if(!valida){
                var tipoPerfil = {
                    "CDOPERADORMETA": "Operador já cadastrado ",
                    "NRPERFILACESSO": "Perfil de Acesso já cadastrado ",
                    "CDFILIAL"      : "Unidade já cadastrada "
                };
                
                ScreenService.showMessage(ScreenService.i18n("Operação bloqueada.<br><br>"+ tipoPerfil[nameCdTipo] +"nesta meta."));
            } else {
                if(!widget.currentRow.__is_new){
                    widget.dataSource.name = "product#/crud/fct_paramsperfilmeta";
                    widget.saveRow().then(function(){
                        widget.dataSource.name = nameDataSource;
                        widget.view();
                        widget.reload();
                    });
                } else {
                    Utilities.inserirMultiplosComNovoCodigo(widget, nameCdTipo, null, "product#/crud/fct_paramsperfilmeta/save").then(function(response){
                        if(response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0){
                                widget.currentRow.NRORG           = response.dataset.columnsToReturn[0].NRORG;   
                                widget.currentRow.NRPARAMPERFMETA = response.dataset.columnsToReturn[0].NRPARAMPERFMETA;   
                                widget.currentRow.NRPARAMSMETAS   = response.dataset.columnsToReturn[0].NRPARAMSMETAS;       
                        }
                        
                        ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso."));
                        ScreenService.closeSwipe();
                        widget.currentRow.__is_new = false;
                        widget.reload();
                    }.bind(this))
                    .catch(function(error){
                        Utilities.showErrorMessage(error);
                    });
                }
            }
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
   };
}

Configuration(function(ContextRegister){
    ContextRegister.register('Sm00043000_ParametrizacaoMetas_Controller', Sm00043000_ParametrizacaoMetas_Controller);
});

// FILE: ./assets/js/controllers/Sm00044000_RelatorioLocalAtend_Controller.js
function Sm00044000_RelatorioLocalAtend_Controller(templateManager, Utilities, Operador_Repository){
    /**
     * Recupera o NRORG do operador logado assim que ele entra na tela
     * @param {Object} widget widget relatorio
     **/
    this.setNrorgLogado = function (widget){
        if(!widget.currentRow.NRORG){
            Operador_Repository.getOperadorLogado().then(function(success) {
                widget.currentRow.NRORG = success[0].NRORG;
            });
        }
    };
    
    /**
     * Monta os parametrôs e manda gerar o relatório
     * @param {Object} widget widget relatorio
     **/
    this.gerarRelatorioLocalAtend = function(widget){
        if(widget.isValid()){
            var params = {
                "NRSEQCONTRATO"      : (widget.currentRow.NRSEQCONTRATO && widget.currentRow.NRSEQCONTRATO.length > 0)           ? ("_" + widget.currentRow.NRSEQCONTRATO.join("_") + "_")      : "T",
                "NRPARCNEGOCIO"      : (widget.currentRow.NRPARCNEGOCIO && widget.currentRow.NRPARCNEGOCIO.length > 0)           ? ("_" + widget.currentRow.NRPARCNEGOCIO.join("_") + "_")      : "T",
                "NRFORMPRECOSERVICO" : (widget.currentRow.NRFORMPRECOSERVICO && widget.currentRow.NRFORMPRECOSERVICO.length > 0) ? ("_" + widget.currentRow.NRFORMPRECOSERVICO.join("_") + "_") : "T",
                "NRCALCFORMSERVPREC" : (widget.currentRow.NRCALCFORMSERVPREC && widget.currentRow.NRCALCFORMSERVPREC.length > 0) ? ("_" + widget.currentRow.NRCALCFORMSERVPREC.join("_") + "_") : "T",
                "NRESTRUTURAM"       : (widget.currentRow.NRESTRUTURAM && widget.currentRow.NRESTRUTURAM.length > 0)             ? ("_" + widget.currentRow.NRESTRUTURAM.join("_") + "_")       : "T",
                "NRORG"              :  widget.currentRow.NRORG
            };
            Utilities.openReport("product#/geraBirtReport", widget.currentRow.DSTPRELATORIO, params);
        }
    };

    /**
     * Retorna os dados concatenados no dataSource de contrato
     * @param {Object} widget widget de relatorio
     * @param {String} nameDado nome do dado que deseja retornar
     **/
    this.retornaArrayDadoContrato = function(widget, nameDado){
        var values= [];
        var dados = [];
        if(widget.currentRow[nameDado])
            dados = widget.currentRow[nameDado];
        else 
            if(widget.getField("NRSEQCONTRATO").dataSource.getCheckedRows().length !== 0)
                dados = widget.getField("NRSEQCONTRATO").dataSource.getCheckedRows();
            else 
                dados = widget.getField("NRSEQCONTRATO").dataSource.data;
        dados.forEach(function(value){
            if(value instanceof Object)
                value = value[nameDado];
            value.toString().split(",").forEach(function(value){
                values.push(value);
            });
        }.bind(this));
        values = values.filter(function(value, index){
            return values.indexOf(value) === index;
        });
        return values;
    };
    
    /**
     * Retorna se a field do parametro foi a primeira
     * a fazer o filtro da tela
     * @param {Object} widget widget relatorio
     * @param {String} nameField nome da field que será verificada
     **/
    this.retornaFiltroReferencial = function(widget, nameField){
        if(!widget.getField(nameField).dataSourceFilter[0])
            return true;
        else 
            if(!widget.getField(nameField).dataSourceFilter[0].value && !widget.getField(nameField).dataSourceFilter[1])
                return true;
            else 
                if(nameField === "NRPARCENEGOCIO" && !widget.getField(nameField).dataSourceFilter[1])
                    return true;
        return false;
    };

    /**
     * Monta e aplica o filtro nas fields
     * @param {Object} widget widget relatorio
     * @param {bool} rowfield indica se há ou não dados na row
     * @param {sring} nameCampo nome do campo que receberá o filtro
     * @param {string} nameFilter nome do filtro no dataSourceFilter  
     **/
    this.criaFilter = function(widget, rowfield, nameCampo, nameFilter){
        nameFilter = nameFilter ? nameFilter : nameCampo;
        var valueFilter = null;
        var index = Utilities.getFilterPosition(widget.getField(nameCampo).dataSourceFilter, nameFilter);
        if(!rowfield){
            if(index || index === 0){
                widget.getField(nameCampo).dataSourceFilter.splice(index, 1);
            }
        } else {
            valueFilter = widget.currentRow[nameFilter];
            if(nameCampo === "NRSEQCONTRATO")
                valueFilter = (valueFilter) ? valueFilter.join("_") : valueFilter;
            else 
                if(["NRFORMPRECOSERVICO", "NRPARCNEGOCIO"].indexOf(nameCampo) !== -1)
                    valueFilter = this.retornaArrayDadoContrato(widget, nameCampo);
            widget.getField(nameCampo).dataSourceFilter[((index || index === 0) ? index : (widget.getField(nameCampo).dataSourceFilter.length))] = {
                name: nameFilter,
                operator: "IN",
                value: valueFilter
            };
        }
        if(rowfield && Array.isArray(valueFilter) && widget.currentRow[nameCampo] && widget.currentRow[nameCampo].length > 0){
            var row = widget.currentRow[nameCampo];
            widget.currentRow[nameCampo] = valueFilter.filter(function(value){
                return (row == value);
            });
        }
        templateManager.updateTemplate();
        return widget.getField(nameCampo).reload();
    };

    /**
     * Valida as informações da field e manda aplicar o filtro 
     * @param {Object} widget widget relatorio
     * @param {string} fieldFilter field obteve seu valor alterado
    **/
    this.filtraFields = function(widget, fieldFilter){
        var rowfield = (widget.currentRow[fieldFilter.name].length === 0) ? false : true;
        switch (fieldFilter.name) {
            case "NRFORMPRECOSERVICO":
                this.criaFilter(widget, rowfield, "NRESTRUTURAM", null);  
                if(!rowfield){
                    widget.currentRow.NRCALCFORMPRECSERV = [];
                    widget.getField("NRCALCFORMPRECSERV").readOnly = !rowfield;
                } else {
                    widget.getField("NRCALCFORMPRECSERV").readOnly = !rowfield;
                    this.criaFilter(widget, rowfield, "NRCALCFORMPRECSERV", "NRFORMPRECOSERVICO");
                }
                if(this.retornaFiltroReferencial(widget, fieldFilter.name)){
                    if(rowfield){
                        this.criaFilter(widget, rowfield, "NRSEQCONTRATO", "NRFORMPRECOSERVICO").then(function(){
                            this.criaFilter(widget, rowfield, "NRPARCNEGOCIO", null);
                        }.bind(this));
                    } else {
                        widget.currentRow.NRSEQCONTRATO = [];
                        widget.getField("NRSEQCONTRATO").dataSourceFilter[0].value = null;
                        widget.getField("NRSEQCONTRATO").reload();
                        this.criaFilter(widget, false, "NRPARCNEGOCIO", null);
                    }
                }
            break;
            case "NRSEQCONTRATO":
                if(this.retornaFiltroReferencial(widget, fieldFilter.name)){
                    this.criaFilter(widget, rowfield, "NRPARCNEGOCIO", null);
                    this.criaFilter(widget, rowfield, "NRFORMPRECOSERVICO", null).then(function(){
                        this.criaFilter(widget, rowfield, "NRESTRUTURAM", null); 
                        widget.getField("NRCALCFORMPRECSERV").readOnly = true;
                    }.bind(this));
                } else {
                    if(fieldFilter.dataSourceFilter[0].value && !fieldFilter.dataSourceFilter[1]){
                        this.criaFilter(widget, rowfield, "NRPARCNEGOCIO", null);
                    } else {
                        this.criaFilter(widget, rowfield, "NRFORMPRECOSERVICO", null).then(function(){
                            this.criaFilter(widget, rowfield, "NRESTRUTURAM", null);
                        }.bind(this));
                    }
                }
            break;
            case "NRPARCNEGOCIO": 
                if(this.retornaFiltroReferencial(widget, fieldFilter.name)){
                    this.criaFilter(widget, rowfield, "NRSEQCONTRATO", "NRPARCNEGOCIO").then(function(){
                        this.criaFilter(widget, rowfield, "NRFORMPRECOSERVICO", null).then(function(){
                            this.criaFilter(widget, rowfield, "NRESTRUTURAM", null); 
                            widget.getField("NRCALCFORMPRECSERV").readOnly = true;
                        }.bind(this));
                    }.bind(this));
                } else {
                    if(!widget.currentRow.NRSEQCONTRATO || widget.currentRow.NRSEQCONTRATO.length === 0){
                        this.criaFilter(widget, rowfield, "NRSEQCONTRATO", "NRPARCNEGOCIO").then(function(){
                            if(!widget.currentRow.NRFORMPRECOSERVICO || widget.currentRow.NRFORMPRECOSERVICO.length === 0){
                                this.criaFilter(widget, rowfield, "NRFORMPRECOSERVICO", null).then(function(){ 
                                    widget.getField("NRCALCFORMPRECSERV").readOnly = true;
                                    this.criaFilter(widget, rowfield, "NRESTRUTURAM", null);
                                }.bind(this)); 
                            }    
                        }.bind(this));
                    }
                }
            break;
            default:
                if(this.retornaFiltroReferencial(widget, fieldFilter.name)){
                    this.criaFilter(widget, rowfield, "NRFORMPRECOSERVICO", "NRESTRUTURAM").then(function(){
                        widget.getField("NRCALCFORMPRECSERV").readOnly = true;
                        this.criaFilter(widget, rowfield, "NRSEQCONTRATO", "NRFORMPRECOSERVICO").then(function(){
                            this.criaFilter(widget, rowfield, "NRPARCNEGOCIO", null); 
                        }.bind(this));
                    }.bind(this));
                } else {
                    if(!widget.currentRow.NRFORMPRECOSERVICO || widget.currentRow.NRFORMPRECOSERVICO.length === 0){
                        this.criaFilter(widget, rowfield, "NRFORMPRECOSERVICO", "NRESTRUTURAM").then(function(){
                            this.criaFilter(widget, rowfield, "NRSEQCONTRATO", "NRFORMPRECOSERVICO").then(function(){
                                this.criaFilter(widget, rowfield, "NRPARCNEGOCIO", null);
                            }.bind(this));
                        }.bind(this)); 
                    }
                }
            break;
        }
    };
}
Configuration(function(ContextRegister){
    ContextRegister.register('Sm00044000_RelatorioLocalAtend_Controller', Sm00044000_RelatorioLocalAtend_Controller);
});

// FILE: ./assets/js/controllers/Sm00045000_Despesa_Controller.js
function Sm00045000_Despesa_Controller(General, Utilities,ScreenService, templateManager){
    /**
     * Salva uma nova despesa
     * @params{Object} widget - widget despesas
    **/
    this.SalvarDespesa = function(widget){
        General.widgetSave(widget).then(function(response){
            if(response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0){
                widget.currentRow.CDDESPESA = response.dataset.columnsToReturn[0].CDDESPESA;
                widget.currentRow.NRORG     = response.dataset.columnsToReturn[0].NRORG;
            }
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Valida e salva um serviço relaciona a uma despesa
     * @params{Object} widget - widget serviço
    **/
    this.salvarServicoDespesa = function(widget){
        var wdgDespesas = templateManager.container.getWidget("wdgDespesas");
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(widget, null, ["CDDESPESA", "NRCOMPSERVICO"]).then(function(response){
                if(!response){
                    ScreenService.showMessage("Operação bloqueada.<br/>O Serviço selecionado já está ligado a esta despesa.");
                } else {
                    Utilities.inserirMultiplosComNovoCodigo(widget, "NRCOMPSERVICO").then(function(){
                        widget.reload().then(function(){
                            wdgDespesas.enabled = true;
                            wdgDespesas.widgets.forEach(function(wid){
                                if(wid.name !== widget.name){
                                    wid.enabled = true;
                                }
                            });
                            widget.list();
                        });
                    });
                }
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Deleta mútiplos registros na tela de formação ou de serviço
     * @params{Object} widget - widget serviço
    **/
    this.deletarMultiplos = function(widget){
        widget.dataSource.getCheckedRows().forEach(function(row){
           widget.currentRow = row;
           widget.removeRow(); 
        });
    };
    /**
     * Cancela o formulário de inserção de um novo serviço ou formção 
     * @params{Object} widget - widget servico ou formação de preço
    **/
    this.cancelarFormulario = function(widget){
        var wdgDespesas = templateManager.container.getWidget("wdgDespesas");
        wdgDespesas.enabled = true;
        wdgDespesas.widgets.forEach(function(wid){
            if(wid.name !== widget.name){
                wid.enabled = true;
            }
        });
        widget.dataSource.data.pop();
        widget.list();
    };
    /**
     * Salva uma nova formação de preço ligada a despesa
     * @params{Object} widget - widget formação de preço
    **/
    this.salvarFormacaoDespesas = function(widget){
        var wdgDespesas = templateManager.container.getWidget("wdgDespesas");
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(widget, null, ["CDDESPESA", "NRFORMPRECOSERVICO"]).then(function(response){
                if(!response){
                    ScreenService.showMessage("Operação bloqueada.<br/>A Formação de Preço selecionada já está ligada a esta despesa.");
                } else {
                    Utilities.inserirMultiplosComNovoCodigo(widget, "NRFORMPRECOSERVICO").then(function(){
                        widget.reload().then(function(){
                            wdgDespesas.enabled = true;
                            wdgDespesas.widgets.forEach(function(wid){
                                if(wid.name !== widget.name){
                                    wid.enabled = true;
                                }
                            });
                            widget.list();
                        });
                    });
                }
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Habilita a tela para a insersão de um novo serviço ou formação de preço
     * @params{Object} widget - widget formação de preço ou serviço
    **/
    this.adicionarServForm = function(widget){
        var wdgDespesas = templateManager.container.getWidget("wdgDespesas");
        wdgDespesas.enabled = false;
        wdgDespesas.widgets.forEach(function(wid){
            if(wid.name !== widget.name){
                wid.enabled = false;
            }
        });
        General.widgetNewRow(widget);
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register("Sm00045000_Despesa_Controller", Sm00045000_Despesa_Controller);
});

// FILE: ./assets/js/controllers/Sm00046000_Investimentos_Controller.js
function Sm00046000_Investimentos_Controller(templateManager, ScreenService, General, Utilities){
    /**
     * Abre um popup para a inserção de uma nova categoria
    **/
    this.adicionarCategoria = function(){
        ScreenService.closePopup();
        var popupCat = templateManager.container.getWidget("wdgInserirCategoria");
        popupCat.isVisible = true;
        ScreenService.openPopup(popupCat).then(function(){
            popupCat.edit();
            popupCat.newRow();
            popupCat.activate();
        });
    };
    /**
     * Cancela a inserção de uma nova categoria 
     * durante a criação de um investimento
    **/
    this.cancelarInsersaoCategoria = function(){
        var wdgTabbed    = templateManager.container.getWidget("wdgInvertimentogrid");
        var wdgCategoria = templateManager.container.getWidget("wdgInserirCategoria");
        
        wdgCategoria.isVisible = false;
        ScreenService.closePopup();
        wdgTabbed.enabled = true;
        wdgTabbed.activate();
    };
    /**
     * Manda salvar uma nova categoria e a coloca na currentRow 
     * da tela de investimentos
     * @params{Object} widget - widget popup categoria
    **/
    this.salvarNovaCategoria = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgInvest = templateManager.container.getWidget("wdgInvertimentogrid");
            var row = widget.currentRow;
            if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
                widget.currentRow.__using_unit      = widget.paramsNewCode.__using_unit || false;
                widget.currentRow.__new_code_pk     = widget.paramsNewCode.__new_code_pk;
                widget.currentRow.__table_pk        = widget.paramsNewCode.__table_pk;
                widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
                widget.currentRow.__cast_number     = widget.paramsNewCode.__cast_number || false;
                if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                    widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
            }
            
            widget.saveRow().then(function(response){
                ScreenService.showMessage("Categoria inserida com sucesso!");
                if(response.dataset && response.dataset.columnsToReturn.length > 0)
                    wdgInvest.currentRow.CDCATINVEST = response.dataset.columnsToReturn[0].CDCATINVEST;
                
                wdgInvest.currentRow.NMCATINVEST = row.NMCATINVEST;
                var fieldCat = wdgInvest.getField("NMCATINVEST");
                fieldCat.dataSource.reload().then(function(){
                    fieldCat.dataSource.data.forEach(function(categoria){
                        if(categoria.CDCATINVEST === response.dataset.columnsToReturn[0].CDCATINVEST)
                            categoria.__isSelected = true;
                    });
                });
                
                widget.dataSource.data = [];
                this.cancelarInsersaoCategoria();
            }.bind(this))
            .catch(function(error){
                ScreenService.showMessage("Operação bloqueada!<br>Falha ao inserir categoria.");
            });
        }
    };
    /**
     * Salva um novo investimento
     * @params{Object} widget - widget tabbed investimento
    **/
    this.salvarInvestimento = function(widget){
        General.widgetSave(widget).then(function(response){
            if(response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0){
                widget.currentRow.NRINVESTIMENTO = response.dataset.columnsToReturn[0].NRINVESTIMENTO;
                widget.currentRow.NRORG          = response.dataset.columnsToReturn[0].NRORG;
            }
        });
    };
    /**
     * Cancela o formulário de inserção de um novo serviço ou formção 
     * @params{Object} widget - widget servico ou formação de preço
    **/
    this.cancelarFormulario = function(widget){
        var wdgInvest = templateManager.container.getWidget("wdgInvertimentogrid");
        wdgInvest.widgets.forEach(function(wid){
            if(wid.name !== widget.name){
                wid.enabled = true;
            }
        });
        wdgInvest.enabled = true;
        widget.dataSource.data.pop();
        widget.list();
    };
    /**
     * Deleta mútiplos registros na tela de formação ou de serviço
     * @params{Object} widget - widget serviço
    **/
    this.deletarMultiplos = function(widget){
        widget.dataSource.getCheckedRows().forEach(function(row){
           widget.currentRow = row;
           widget.removeRow(); 
        });
    };
    /**
     * Valida e salva um serviço relaciona a um investimento
     * @params{Object} widget - widget serviço
    **/
    this.salvarServicoInvestimento = function(widget){
        var wdgInvest = templateManager.container.getWidget("wdgInvertimentogrid");
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(widget, null, ["NRINVESTIMENTO", "NRCOMPSERVICO"]).then(function(response){
                if(!response){
                    ScreenService.showMessage("Operação bloqueada.<br/>O Serviço selecionado já está ligado a este investimento.");
                } else {
                    Utilities.inserirMultiplosComNovoCodigo(widget, "NRCOMPSERVICO").then(function(){
                        widget.reload().then(function(){
                            wdgInvest.widgets.forEach(function(wid){
                                if(wid.name !== widget.name){
                                    wid.enabled = true;
                                }
                            });
                            wdgInvest.enabled = true;
                            widget.list();
                        });
                    });
                }
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Salva uma nova formação de preço ligada a um investimento
     * @params{Object} widget - widget formação de preço
    **/
    this.salvarFormacaoInvestimento = function(widget){
        var wdgInvest = templateManager.container.getWidget("wdgInvertimentogrid");
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(widget, null, ["NRINVESTIMENTO", "NRFORMPRECOSERVICO"]).then(function(response){
                if(!response){
                    ScreenService.showMessage("Operação bloqueada.<br/>A Formação de Preço selecionada já está ligada a este investimento.");
                } else {
                    Utilities.inserirMultiplosComNovoCodigo(widget, "NRFORMPRECOSERVICO").then(function(){
                        widget.reload().then(function(){
                            wdgInvest.widgets.forEach(function(wid){
                                if(wid.name !== widget.name){
                                    wid.enabled = true;
                                }
                            });
                            wdgInvest.enabled = true;
                            widget.list();
                        });
                    });
                }
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Habilita a tela para a insersão de um novo serviço ou formação de preço
     * @params{Object} widget - widget formação de preço ou serviço
    **/
    this.adicionarServForm = function(widget){
        var wdgInvestGrid = templateManager.container.getWidget("wdgInvertimentogrid");
        wdgInvestGrid.widgets.forEach(function(wid){
            if(wid.name !== widget.name){
                wid.enabled = false;
            }
        });
        wdgInvestGrid.enabled = false;
        General.widgetNewRow(widget);
        widget.currentRow.NRINVESTIMENTO = wdgInvestGrid.currentRow.NRINVESTIMENTO;
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register("Sm00046000_Investimentos_Controller", Sm00046000_Investimentos_Controller);
});

// FILE: ./assets/js/controllers/Sm00047000_Taxas_Controller.js
function Sm00047000_Taxas_Controller(General, Utilities, templateManager, ScreenService){
    /**
     * Habilita a tela para receber uma nova taxa
     * @params{Object} widget - widget grid taxas 
    **/
    this.adicionarTaxa = function(widget){
        var wdgTaxasInserindo = templateManager.container.getWidget("wdgTaxasInserindo"); 
        wdgTaxasInserindo.isVisible = true;
        ScreenService.openSwipe(wdgTaxasInserindo).then(function(){
            wdgTaxasInserindo.newRow();
            wdgTaxasInserindo.activate();
        });
    };
    /**
     * Cancela a inserção uma nova taxa
     * @params{Object} widget - widget taxas inserindo 
    **/
    this.cancelarInsercaoTaxas = function(widget){
        var wdgTaxasGrid = templateManager.container.getWidget("wdgTaxas");
        widget.isVisible = false;
        ScreenService.closeSwipe().then(function(){
            wdgTaxasGrid.activate();
            wdgTaxasGrid.reload();
        });
    };
    /**
     * Deleta as taxas selecionadas no grid
     * @params{Object} widget - widget grid taxas 
    **/
    this.deletarTaxas = function(widget){
        widget.dataSource.getCheckedRows().forEach(function(row){
            widget.currentRow = row;
            widget.removeRow();
        });
        
        widget.reload();
    };
    /**
     * Salvar as edições realizadas no grid 
     * @params{Object} widget - widget taxas grid
    **/
    this.salvarEdicaoTaxas = function(widget){
        widget.saveRow().then(function(){
            widget.reload();
            widget.list();
        });
    };
    /**
     * Salva uma nova taxa
     * @params{Object} widget - widget taxas inserindo
    **/    
    this.salvarTaxas = function(widget){
        var wdgTaxasGrid = templateManager.container.getWidget("wdgTaxas"); 
        var isNew = widget.currentRow.__is_new;
        if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
            widget.currentRow.__using_unit      = widget.paramsNewCode.__using_unit || false;
            widget.currentRow.__new_code_pk     = widget.paramsNewCode.__new_code_pk;
            widget.currentRow.__table_pk        = widget.paramsNewCode.__table_pk;
            widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
            widget.currentRow.__cast_number     = widget.paramsNewCode.__cast_number || false;
            if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
        }
        
        widget.saveRow().then(function(response){
            if(isNew){
                widget.isVisible = false;
                widget.currentRow = {};
                widget.dataSource.data = [];
                ScreenService.closeSwipe().then(function(){
                    wdgTaxasGrid.activate();
                    wdgTaxasGrid.reload();
                });
            }
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register("Sm00047000_Taxas_Controller", Sm00047000_Taxas_Controller);
});

// FILE: ./assets/js/controllers/Sm00048000_imposto_Controller.js
function Sm00048000_imposto_Controller(Utilities, templateManager, ScreenService, General, ZHPromise){
    /**
     * cancela o formulário de impostos
     * @params{Object} widget - widget impostos
    **/
    this.cancelarFormImposto = function(widget){
        var wdgCompServ = templateManager.container.getWidget("wdgServicoRelac");
        widget.getAction("avancar_imposto").isVisible           = true;
        widget.getAction("salvar_imposto").isVisible            = false;
        wdgCompServ.getAction("save_impostocomp").isVisible     = false;
        wdgCompServ.getAction("save_composicao").isVisible      = true;
        wdgCompServ.getAction("voltar_compimposto").isVisible   = false;
        wdgCompServ.getAction("cancelar_servimposto").isVisible = true;
        General.widgetCancelForm(widget);
    };
    /**
     * Avança para a widget de composição de serviço para 
     * ser adicionado os serviços ligados ao imposto
     * @params{Object} widget - widget imposto
    **/
    this.avancarImpostoRelacao = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            widget.enabled = false;
            var wdgServico = templateManager.container.getWidget("wdgServicoRelac");
            wdgServico.enabled   = true;
            wdgServico.activate().then(function(){
                wdgServico.getAction("save_impostocomp").isVisible     = true;
                wdgServico.getAction("save_composicao").isVisible      = false;
                wdgServico.getAction("voltar_compimposto").isVisible   = true;
                wdgServico.getAction("cancelar_servimposto").isVisible = false;
                wdgServico.newRow();
            });
        }
    };
    /**
     * Habilita a tela para a edição de um imposto
     * @params{Object} widget - widget imposto
    **/
    this.editarImposto = function(widget){
        widget.getAction("avancar_imposto").isVisible = false;
        widget.getAction("salvar_imposto").isVisible  = true;
        General.widgetEdit(widget);
    };
    /**
     * Deleta mútiplos registros na tela de serviços ou na tela de imposto
     * @params{String} nameWidget - nome da widget que chamou a função 
    **/
    this.deletarMultiplos = function(nameWidget){
        ScreenService.confirmMessage("Excluir Registro?", "question", function(){
            var widget     = templateManager.container.getWidget("wdgImpostos");
            var widgetServ = templateManager.container.getWidget("wdgServicoRelac");
            var nrimposto = [];
            widgetServ.dataSource.getCheckedRows().forEach(function(servico){
                nrimposto.push(servico.NRIMPOSTOSERV);
            });
            var params = {
                "CDIMPOSTO": widget.currentRow.CDIMPOSTO,
                "NRIMPOSTOSERV": (nameWidget === "wdgImpostos") ? null : nrimposto
            };
            General.callRestEngine("product#/fct_relacimpostoserv_widget/deleteImposto", "Row", params).then(function(){
                if(nameWidget === "wdgImpostos"){ 
                    ScreenService.closeSwipe();
                    widget.list();
                    widget.reload();
                } else {
                    widgetServ.getField("NRCOMPSERVICO").dataSource.clearCheckedRows();
                    widgetServ.currentRow = {};
                    widgetServ.reload();
                }
            }).catch(function(error){
                Utilities.showErrorMessage(error);
            });
        });
    };
    /**
     * Salva a edição apenas da widget de imposto
    **/
    this.salvarEdicaoImposto = function(){
        var widget      = templateManager.container.getWidget("wdgImpostos");
        var wdgCompServ = templateManager.container.getWidget("wdgServicoRelac");
        var params = {
            "CDIMPOSTO": widget.currentRow.CDIMPOSTO,
            "VRPRECIMPOSTO": widget.currentRow.VRPRECIMPOSTO
        };
        
        General.callRestEngine("product#/fct_relacimpostoserv_widget/salvarEdicaoImposto", "Row", params).then(function(){
            wdgCompServ.enabled = true;
            widget.view();
        }).catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Volta para a widget de imposto
     * @params{Object} widget - widget composição de serviço
    **/
    this.voltarWidgetImposto = function(widget){
        widget.enabled = false;
        var wdgImposto = templateManager.container.getWidget("wdgImpostos");
        wdgImposto.enabled = true;
        wdgImposto.activate().then(function(){
            setTimeout(function(){
                angular.element("label.zh-tab-cell")[0].click();
            }, 100);
        });
    };
    /**
     * salva várias composições de serviço ligadas ao imposto
     * @params{Object} widget - widget composição de serviço
    **/
    this.salvarMultiplasComposicoes = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var deferred   = ZHPromise.defer();
            var wdgImposto = templateManager.container.getWidget("wdgImpostos");
            widget.currentRow.CDIMPOSTO     = wdgImposto.currentRow.CDIMPOSTO;
            widget.currentRow.VRPRECIMPOSTO = wdgImposto.currentRow.VRPRECIMPOSTO;
            Utilities.validarCamposDuplicados(widget, null, ["CDIMPOSTO", "NRCOMPSERVICO"]).then(function(response){
                if(!response){
                    ScreenService.showMessage("Operação bloqueada.<br/>A composição de serviço selecionada já está ligada ao imposto.");
                } else {
                    Utilities.validarCamposDuplicados(widget, null, ["CDIMPOSTO", "NRCOMPSERVICO"]).then(function(response){
                        if(!response){
                            ScreenService.showMessage("Operação bloqueada.<br/>A composição de serviço selecionada já está ligada ao imposto.");
                        } else {
                            Utilities.inserirMultiplosComNovoCodigo(widget, "NRCOMPSERVICO").then(function(){
                                widget.getField("NRCOMPSERVICO").dataSource.clearCheckedRows();
                                widget.currentRow = {};
                                widget.list();
                                widget.reload();
                                
                                wdgImposto.getAction("avancar_imposto").isVisible  = true;
                                wdgImposto.getAction("salvar_imposto").isVisible   = false;
                                
                                widget.getAction("save_composicao").isVisible      = true;
                                widget.getAction("save_impostocomp").isVisible     = false;
                                widget.getAction("voltar_compimposto").isVisible   = false;
                                widget.getAction("cancelar_servimposto").isVisible = true;
                                ScreenService.showMessage("Composições de Serviço salvas com sucesso.");
                                deferred.resolve();
                            }.bind(this));
                        }
                    });
                }
            }.bind(this))
            .catch(function(error){
                deferred.reject();
            });
            return deferred.promise;
        }
    };
    /**
     * Salva uma nova relação entre imposto e serviço
     * @params{Object} widget - widget composição de serviço
    **/
    this.salvarRelacImpostoServ = function(widget){
        var wdgImposto = templateManager.container.getWidget("wdgImpostos");
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(wdgImposto, null, ["CDIMPOSTO"], "product#/crud/fct_relacimpostoserv_widget").then(function(sucess){
                if(!sucess){
                    ScreenService.showMessage("Operação bloqueada.<br/>O Imposto selecionado já foi adicionado.");
                } else {
                    this.salvarMultiplasComposicoes(widget).then(function(){
                        ScreenService.closeSwipe().then(function(){
                            wdgImposto.enabled = true;
                            wdgImposto.list();
                            wdgImposto.reload();
                        }.bind(this));   
                    }.bind(this))
                    .catch(function(error){
                        Utilities.showErrorMessage(error);
                    });
                }
            }.bind(this));
        }
    };
    /**
     * Cancela o formulário de inserção de novas composições de serviço
     * ligadas a um imposto
     * @params{Object} widget - widget composição de serviço
    **/
    this.cancelarInsercaoCompServico = function(widget){
        widget.dataSource.data.pop();
        widget.getField("NRCOMPSERVICO").dataSource.clearCheckedRows();
        widget.currentRow = {};
        widget.list();
    };
    /**
     * controla a acessibilidade, as actions visiveis e habilita a tela
     * para a inserção de uma nova composição
     * @params{Object} widget - widget composição de serviço
    **/
    this.adicionarNovaComposicao = function(widget){
        var wdgImposto = templateManager.container.getWidget("wdgImpostos");
        wdgImposto.widgets.forEach(function(wdg){
            wdg.isVisible = (wdg.name === widget.name);
        });
        widget.getAction("save_composicao").isVisible      = true;
        widget.getAction("save_impostocomp").isVisible     = false;
        widget.getAction("voltar_compimposto").isVisible   = false;
        widget.getAction("cancelar_servimposto").isVisible = true;
        widget.newRow();
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register("Sm00048000_imposto_Controller", Sm00048000_imposto_Controller);    
});

// FILE: ./assets/js/controllers/Sm00049000_AgrupamentoContratosFaturamento_Controller.js
function Sm00049000_AgrupamentoContratosFaturamento_Controller(General, Utilities, templateManager, ScreenService, ZHPromise){
    
    /**
     * Valida se os escopos selecionados possuem parametrizações de 
     * faturamento similares
    **/
    this.validarEscoposAgrupamento = function(widget) {
        var deferred = ZHPromise.defer(); 
        var escopos  = widget.currentRow.NRTIPOORCAMENTO;
        
        if(!escopos || escopos.length <= 1) {
            deferred.resolve();
        } else {
            var params = { 
                "NRTIPOORCAMENTO": escopos
            };
            
            General.callRestEngine("product#/fct_agrupamento_widget/validarEscoposAgrupamento", "Row", params).then(function(response) {
                if(response.dataset.fct_agrupamento_widget.IDESCSEMELHANTES) {
                    deferred.resolve();
                } else {
                    ScreenService.confirmMessage("Atenção!<br><br>Os escopos selecionados possuem parametrizações específicas para os serviços ligados a eles, podendo impossibilitar o agrupamento das parcelas com diferentes modelos de faturamento.<br>Ex: As parcelas de um faturamento 'Periódico' não serão unidas as parcelas de 'Parcelamentos'.<br><br>Deseja criar o agrupamento?", 
                    'question', function() {
                        deferred.resolve();
                    });
                }
            }).catch(function(error) {
                var mensagem = "Falha ao validar os parâmetros de faturamento dos Escopos selecionados.";
                if(error && error.error) {
                    if(error.error == "PARAMS_DIFERENTE")
                        mensagem = "Operação bloqueada!<br><br>Não é possível criar agrupamentos com escopos que possuem diferentes parametrizações de faturamento.<br><br>Certifique-se que as informações de Tipo de Operação, Série da Nota Fiscal, Unidade para Faturamento e Tipo de Cálculo para Data de Saída são iguais";
                    else if(error.error == "COMPET_DIFERENTE")
                        mensagem = "Operação bloqueada!<br><br>Não é possível criar agrupamentos com escopos que possuem diferentes parametrizações de competência do faturamento.";
                } 
                
                deferred.reject(mensagem);
            });
        }
        
        return deferred.promise;
    };
    
    /**
     * Salva um novo agrupamento
     * @params{Object} widget - widget de agrupamento
    **/
    this.salvarAgrupamento = function(widget) {
        if(!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            if(!widget.currentRow.__is_new) {
                General.widgetSave(widget);
            } else {
                this.validarEscoposAgrupamento(widget).then(function() {
                    General.callRestEngine("product#/fct_agrupamento_widget/salvarAgrupamento", "Row", widget.currentRow).then(function(response){
                        widget.currentRow.changed = false;
                        widget.currentRow.__is_new = false;
                        
                        if(response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0){
                            widget.currentRow.NRAGRUPAMENTO = response.dataset.columnsToReturn[0].NRAGRUPAMENTO;
                            widget.currentRow.NRORG         = response.dataset.columnsToReturn[0].NRORG;
                        }
                    
                        ScreenService.closeSwipe().then(function() {
                            widget.widgets.forEach(function(widFilha) {
                                widFilha.enabled = true;
                            });
                        });
                    
                    }.bind(this));
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
        }
    };
    
    /**
     * Adiciona um filtro na field de contrato e habilita o template para 
     * receber um novo registro
     * @params{Object} widget - widget de contratos
    **/
    this.contratosNewRow = function(widget){
        var wdgAgrupamento = templateManager.container.getWidget("wdgDadosAgrupamento");
        var fieldContrato  = widget.getField("NRSEQCONTRATO");
        
        fieldContrato.dataSourceFilter[0].value = wdgAgrupamento.currentRow.NRTIPOORCAMENTO;
        fieldContrato.dataSourceFilter[1].value = wdgAgrupamento.currentRow.NRPARCFATURA;
        
        General.widgetNewRow(widget);
        wdgAgrupamento.enabled = false;
    };
    /**
     * Cancela a inserção de um novo contrato
    **/
    this.cancelarFormContratos = function(widget){
        var wdgAgrupamento = templateManager.container.getWidget("wdgDadosAgrupamento");
        
        wdgAgrupamento.enabled = true;
        widget.currentRow      = {};
        
        widget.list();
        widget.reload();
    };
    /**
     * Coloca a widget de contratos em modo list
     * @params{Object} widget - widget de contratos
    **/
    this.habilitarWidgetContratosGrid = function(){
        var widget    = templateManager.container.getWidget("wdgContratosAgrup");
        var wdgInvest = templateManager.container.getWidget("wdgDadosAgrupamento");
        
        widget.reload().then(function(){
            wdgInvest.widgets.forEach(function(wid){
                if(wid.name !== widget.name){
                    wid.enabled = true;
                }
            });
            
            wdgInvest.enabled = true;
            widget.list();
        });
    };
    /**
     * Salva a edição do grupo de nota fiscal
    **/
    this.salvarContratosInline = function(widget){
        widget.saveRow().then(function(){
            this.habilitarWidgetContratosGrid();
        }.bind(this));
    };
    /**
     * Valida se há contratos aprovados para regerar o faturamento
     * @params{Object} widget - widget de contratos
    **/
    this.validarContratosParaAgrupamento = function(widget) {
        if(!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(widget, ["NRAGRUPAMENTOCONT"], ["NRSEQCONTRATO", "NRORG"], null).then(function(validacao){
                if(!validacao) {
                    ScreenService.showMessage("Operação bloqueada.<br><br>O contrato selecionado já participa de um agrupamento.");
                } else {
                    var idgerarateio = widget.currentRow.IDGERARATEIO;
                    var codContratosFct = widget.currentRow.NRCONTRATOFCT;
                    var contratosGeraParcLocalAtend = [];
                    
                    if (Array.isArray(idgerarateio) && idgerarateio.length > 0) {
                        idgerarateio.forEach(function(value, index) {
                            if (value == "S") {
                                contratosGeraParcLocalAtend.push(codContratosFct[index]);
                            }
                        });
                    }
                    
                    if (contratosGeraParcLocalAtend.length > 0) {
                        ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br/><br/>Os seguintes contratos não podem ser agrupados pois estão parametrizados para gerar parcelas separadas por local de atendimento: <br/><br/>" + contratosGeraParcLocalAtend.join("<br/>")));
                    }
                    else {
                        var filterRow = [
                            {
                                name: "NRSEQCONTRATO",
                                operator: "IN",
                                value: widget.currentRow.NRSEQCONTRATO
                            },
                            {
                                name: "IDSITUACAOCONTRATO",
                                operator: "=",
                                value: 'A'
                            }
                        ];
                        
                        General.callRestEngine("product#/crud/fct_contrato", "FilterData", filterRow).then(function(response){
                            if(!response.dataset || !response.dataset.fct_contrato || response.dataset.fct_contrato.length ==  0) {
                                this.salvarContratosAgrupamento(widget, []);
                            } else {
                                var nrcontratos    = [];
                                var nrseqcontratos = [];
                                response.dataset.fct_contrato.forEach(function(contrato){
                                    nrcontratos.push(contrato.NRCONTRATOFCT);
                                    nrseqcontratos.push(contrato.NRSEQCONTRATO);
                                });
                                
                                if(nrcontratos) nrcontratos = nrcontratos.join("<br/>");
                                ScreenService.confirmMessage(ScreenService.i18n("Atenção!<br><br>O agrupamento de contratos aprovados resultará na regeração do faturamento para agrupar suas parcelas de situação pendente. Contratos afetados: <br/>" + nrcontratos + "<br/><br/>Deseja prosseguir?"), 'question', function(){
                                    this.salvarContratosAgrupamento(widget, nrseqcontratos);
                                }.bind(this));
                            }
                        }.bind(this));
                    }
                }
            }.bind(this));
        }
    };
    /**
     * Salva um novo contrato na relação entre agrupamento e contrato
     * @params{Object} widget - widget de contratos
    **/
    this.salvarContratosAgrupamento = function(widget, contAprovados){
        var params = {
            "NRSEQCONTRATO": widget.getField("NRSEQCONTRATO").dataSource.getCheckedRows(),
            "NRAGRUPAMENTO": widget.currentRow.NRAGRUPAMENTO
        }; 
         
        General.callRestEngine("product#/fct_relacagrupamentocont/validarContratosAgrupamento", "Row", params).then(function(response){
            if(contAprovados.length == 0){
                Utilities.inserirMultiplosComNovoCodigo(widget, "NRSEQCONTRATO").then(function(){
                    templateManager.container.getWidget("wdgDadosAgrupamento").enabled = true;
                    this.habilitarWidgetContratosGrid();
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            } else {
                var params = {  
                    "NRSEQCONTRATO": widget.getField("NRSEQCONTRATO").dataSource.getCheckedRows(),
                    "NRAGRUPAMENTO": widget.currentRow.NRAGRUPAMENTO
                };
                
                General.callRestEngine('product#/fct_relacagrupamentocont/salvaContratosAgrupamento', "Row", params).then(function(){
                    this.habilitarWidgetContratosGrid();
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }
        }.bind(this))
        .catch(function(error){
            if(error && error.error && error.error == "PARC_DIFERENTES")
                ScreenService.showMessage("Operação bloqueada!<br><br>Não é possível agrupar os contratos selecionados, porque possuem diferentes Parceiros para Faturamento.");
            else 
                Utilities.showErrorMessage(error);
        });
    };
    
    /**
     * Abre a widget listando as diferenças entre as parametrizações do contratos selecionados
     * @params{Array} dataset = dataset de diferenças
    **/
    this.abrirWdgDetalhesParametrizacao = function(dataSet){
        var wdgDetalhes = templateManager.container.getWidget("wdgDetalhesParametrizacao");
        
        wdgDetalhes.isVisible = true;
        ScreenService.openSwipe(wdgDetalhes).then(function() {
            wdgDetalhes.list();
            wdgDetalhes.dataSource.data = dataSet;
            wdgDetalhes.activate();
        });
    };
    /**
     * valida a retirada de um contrato do agrupamento
    **/
    this.retirarContratoAgrupamento = function(widget){
        ScreenService.confirmMessage(ScreenService.i18n("Atenção!<br><br>A exclusão de um contrato do agrupamento resultará no desmembramento das parcelas pendentes do faturamento em que ele participa.<br><br>Deseja continuar?"), "question", function(){
            var nrseqcontratos = [];
            var nragrupamento  = templateManager.container.getWidget("wdgDadosAgrupamento").currentRow.NRAGRUPAMENTO;
            widget.dataSource.getCheckedRows().forEach(function(contratos){
                nrseqcontratos.push(contratos.NRSEQCONTRATO);
            });
            
            this.excluirAgrupamentoOuContrato(nrseqcontratos, nragrupamento).then(function(){
                widget.dataSource.clearCheckedRows();
                widget.reload();
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }.bind(this));
    };
    
    /**
     * valida a exclusao de um agrupamento
    **/
    this.deletarAgrupamento = function(widget){
        ScreenService.confirmMessage(ScreenService.i18n("Atenção!<br><br>A exclusão do agrupamento resultará na regeração das parcelas pendentes do faturamento.<br><br>Deseja continuar?"), "question", function(){
            var nrseqcontratos = [];
            var nragrupamento  = templateManager.container.getWidget("wdgDadosAgrupamento").currentRow.NRAGRUPAMENTO;
            widget.dataSource.getCheckedRows().forEach(function(contratos){
                nrseqcontratos.push(contratos.NRSEQCONTRATO);
            });
            
            this.excluirAgrupamentoOuContrato(nrseqcontratos, nragrupamento).then(function(){
                ScreenService.closeSwipe();
                widget.reload();
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }.bind(this));
    };
    /**
     * Manda excluir um contrato ou um agrupamento 
     * no faturamento
    **/
    this.excluirAgrupamentoOuContrato = function(nrseqcontratos, nragrupamento){
        var row = {
            "NRAGRUPAMENTO": templateManager.container.getWidget("wdgDadosAgrupamento").currentRow.NRAGRUPAMENTO,
            "NRSEQCONTRATO": nrseqcontratos,
            "IDACAO"       : "DESAGRUPA"
        };
        
        return General.callRestEngine("product#/fct_faturacont_completo/retirarContratoAgrupamentoFat", "Row", row);
    };
    
    /**
     * Habilita a ediçao dos dados de um agrupamento
     * 
     * @params{Object} widget  - widget de dados gerais do agrupamento
     * @params{bool} isVisible - determina se a validação é oara uma edição ou não
    **/
    this.validarFieldEscopoAgrupamento = function(widget, isVisible) {
        widget.getField("NRTIPOORCAMENTO").showOnForm = !isVisible;
        widget.getField("NMTIPOORCAMENTO").showOnForm = isVisible;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00049000_AgrupamentoContratosFaturamento_Controller", Sm00049000_AgrupamentoContratosFaturamento_Controller);
});

// FILE: ./assets/js/controllers/Sm00050000_QuantidadeParcelasOperador_Controller.js
function Sm00050000_QuantidadeParcelasOperador_Controller(Utilities, ScreenService, General){
    /**
     * Salva um novo registro na tabela de quantidade 
     * máxima de parcelas por operador
     * @params{Object} widget - widget quantidade maxima de parcelas
    **/
    this.salvarQtParcelaOperador = function(widget){
        if(!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(widget, ["NRQTPARCMAXIMAOPER", "NRORG"], ["CDOPERADOR"]).then(function(response){
                if(!response){
                    ScreenService.showMessage("Operação bloqueada!<br>Alguns operadores selecionados já estão cadastrados.");
                } else {
                    if(!widget.currentRow.__is_new){
                        General.widgetSave(widget);
                    } else {
                        Utilities.inserirMultiplosComNovoCodigo(widget, "CDOPERADOR").then(function(){
                            this.habilitarFieldsSelectMultiple(widget, false);
                            ScreenService.closeSwipe();
                            widget.list();
                            widget.reload().then(function(){
                                ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
                            });
                        }.bind(this))
                        .catch(function(error){
                            Utilities.showErrorMessage(error);
                        });
                    }
                }
            }.bind(this));
        }
    };
    /**
     * troca as fields de operador para os templates se adaptarem as ações
     * @params {Object} widget  - widget quantidade maxima
     * @params {bool} isVisible - define se a field de select muitiplo deverá aparecer
    **/
    this.habilitarFieldsSelectMultiple = function(widget, isVisible){
        widget.getField("CDOPERADOR").showOnForm = isVisible;
        widget.getField("NMOPERADOR").showOnForm = !isVisible;
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('Sm00050000_QuantidadeParcelasOperador_Controller', Sm00050000_QuantidadeParcelasOperador_Controller);
});

// FILE: ./assets/js/controllers/Sm00051000_QuantidadeParcelasPerfilAcesso_Controller.js
function Sm00051000_QuantidadeParcelasPerfilAcesso_Controller(Utilities, ScreenService, General){
    /**
     * Salva um novo registro na tabela de quantidade 
     * máxima de parcelas por operador
     * @params{Object} widget - widget quantidade maxima de parcelas
    **/
    this.salvarQtParcelaOperador = function(widget){
        if(!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(widget, ["NRQTPARCMAXPERFACESS", "NRORG"], ["NRPERFILACESSO"]).then(function(response){
                if(!response){
                    ScreenService.showMessage("Operação bloqueada!<br>Alguns perfis de acesso selecionados já estão cadastrados.");
                } else {
                    if(!widget.currentRow.__is_new){
                        General.widgetSave(widget);
                    } else {
                        Utilities.inserirMultiplosComNovoCodigo(widget, "NRPERFILACESSO").then(function(){
                            this.habilitarFieldsSelectMultiple(widget, false);
                            ScreenService.closeSwipe();
                            widget.list();
                            widget.reload().then(function(){
                                ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
                            });
                        }.bind(this))
                        .catch(function(error){
                            Utilities.showErrorMessage(error);
                        });
                    }
                }
            }.bind(this));
        }
    };
    /**
     * troca as fields de operador para os templates se adaptarem as ações
     * @params {Object} widget  - widget quantidade maxima
     * @params {bool} isVisible - define se a field de select muitiplo deverá aparecer
    **/
    this.habilitarFieldsSelectMultiple = function(widget, isVisible){
        widget.getField("NRPERFILACESSO").showOnForm = isVisible;
        widget.getField("NMPERFILACESSO").showOnForm = !isVisible;
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('Sm00051000_QuantidadeParcelasPerfilAcesso_Controller', Sm00051000_QuantidadeParcelasPerfilAcesso_Controller);
});

// FILE: ./assets/js/controllers/Sm00053000_ReducaoQuantidadeMedidaPerfilAcesso_Controller.js
function Sm00053000_ReducaoQuantidadeMedidaPerfilAcesso_Controller(ScreenService, Utilities) {
    /**
     * Salva um novo perfil de acesso para a edição da medida da proposta
     * @params{Object} widget - widget redução da quantidade de medida da proposta
    **/
    this.salvarPerfAcessParaReducao = function(widget) {
        if(!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(widget, null, ["NRPERFILACESSO"]).then(function(response) {
                if(!response){
                    ScreenService.showMessage("Operação bloqueada.<br/>O Perfil de Acesso selecionado já possui permissão para reduzir a quantidade de medida das propostas e contratos.");
                } else {
                    Utilities.inserirMultiplosComNovoCodigo(widget, "NRPERFILACESSO").then(function(){
                        ScreenService.closeSwipe();
                        widget.reload();
                        widget.list();
                    });
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('Sm00053000_ReducaoQuantidadeMedidaPerfilAcesso_Controller', Sm00053000_ReducaoQuantidadeMedidaPerfilAcesso_Controller);
});

// FILE: ./assets/js/controllers/Sm00054000_ReducaoQuantidadeMedidaOperador_Controller.js
function Sm00054000_ReducaoQuantidadeMedidaOperador_Controller(ScreenService, Utilities) {
    /**
     * Salva um novo perfil de acesso para a edição da medida da proposta
     * @params{Object} widget - widget redução da quantidade de medida da proposta
    **/
    this.salvarOperadorParaReducao = function(widget) {
        if(!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(widget, null, ["CDOPERADOR"]).then(function(response) {
                if(!response){
                    ScreenService.showMessage("Operação bloqueada.<br/>O Operador selecionado já possui permissão para reduzir a quantidade de medida das propostas e contratos.");
                } else {
                    Utilities.inserirMultiplosComNovoCodigo(widget, "CDOPERADOR").then(function(){
                        ScreenService.closeSwipe();
                        widget.reload();
                        widget.list();
                    });
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('Sm00054000_ReducaoQuantidadeMedidaOperador_Controller', Sm00054000_ReducaoQuantidadeMedidaOperador_Controller);
});

// FILE: ./assets/js/controllers/Sm00055000_TrocaIncricaoParceiro_Controller.js
function Sm00055000_TrocaIncricaoParceiro_Controller(ScreenService, templateManager, General, Utilities) {
    
    this.validarVisibilidadeActions = function(widget) {
        widget.getAction("baixar_termo").isVisible = widget.currentRow.DSCAMINHOARQ != null;
    };
    
    this.abrirPopupDeTrocaInscricao = function(widget) {
        var popup = templateManager.container.getWidget("popupAlterarInscricao");
        
        popup.isVisible = true;
        ScreenService.openPopup(popup).then(function() {
            popup.dataSource.data = [];
            popup.newRow();
            popup.activate();
            popup.currentRow.NRPARCNEGOCIO   = widget.currentRow.NRPARCNEGOCIO;
            popup.currentRow.NMPRINCIPALPARC = widget.currentRow.NMPRINCIPALPARC;
            popup.currentRow.NROLDINSCRICAO  = widget.currentRow.NRINSCRICAOPARC;
            popup.currentRow.NMSECUNDARIPARC = widget.currentRow.NMSECUNDARIPARC;
            widget.currentRow.IDINATIVAPARC  = 'Y';
        });
    };
    
    this.cancelaPopup = function(widgetName){
        ScreenService.closePopup();
        templateManager.container.getWidget(widgetName).activate();
    };
    
    this.validarContratosDoParceiro = function(widget) {
        ScreenService.closePopup();
        var params = { "NRPARCNEGOCIO": widget.currentRow.NRPARCNEGOCIO };
        
        General.callRestEngine("product#/parcnegocio/getPermissaoAlteracaoInscParceiro", "Row", params).then(function(response) {
            if(response.dataset && response.dataset.parcnegocio && response.dataset.parcnegocio.PERMITEALTSM)
                this.abrirPopupDeTrocaInscricao(widget);
            else 
                ScreenService.showMessage("Operação bloqueada!<br>Não é possível alterar o número de inscrição de parceiros que possuam contratos Delphi sem vínculo com o SM.");
        }.bind(this));
    };
    
    this.alterarInscricaoParceiro = function(widget) {
        if(!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var inscricao = widget.currentRow.NRINSCRICAOPARC;
            widget.currentRow.NRINSCRICAOPARC = inscricao.replace(/[./-]/g, '');
            if(widget.currentRow.NROLDINSCRICAO == widget.currentRow.NRINSCRICAOPARC) {
                ScreenService.showMessage("Operação bloqueada!<br><br>Não houve alteração no número de inscrição do parceiro.");
            } else {
                var params = { 
                    "NRPARCNEGOCIO"  : widget.currentRow.NRPARCNEGOCIO,
                    "NRNEWINSCRICAO" : widget.currentRow.NRINSCRICAOPARC,
                    "NROLDINSCRICAO" : widget.currentRow.NROLDINSCRICAO,
                    "NMPRINCIPALPARC": widget.currentRow.NMPRINCIPALPARC,
                    "NMSECUNDARIPARC": widget.currentRow.NMSECUNDARIPARC,
                    "DSJUSTIFICATIVA": widget.currentRow.DSJUSTIFICATIVA,
                    "IDINATIVAPARC"  : widget.currentRow.IDINATIVAPARC == 'Y' ? "S" : "N'",
                    "DSCAMINHOARQ"   : widget.currentRow.DSCAMINHOARQ
                };
                
                General.callRestEngine("product#/parcnegocio/validarInscricaoParceiro", "Row", params).then(function(response) {
                    
                    var valida = response.dataset.alterainscricao.VALIDA;
                    
                    if(valida == "OK"){
                        
                        General.callRestEngine("product#/parcnegocio/alterarInscricaoParceiro", "Row", params).then(function(response) {
                            ScreenService.closePopup();
                            var widget = templateManager.container.getWidget("wdgTrocaInscricaoParceiro");
                            
                            ScreenService.showMessage("Processo realizado com sucesso!");
                            ScreenService.closeSwipe();
                            widget.activate();
                            widget.reload();
                        }.bind(this))
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    } else if(valida == "TEM_PARC") {
                        
                        ScreenService.confirmMessage('Atenção!<br><br>Já existe um Parceiro de Negócio ativo com a inscrição informada. Esse Parceiro será considerado.<br><br>Deseja continuar?', 'question', function(){
                               General.callRestEngine("product#/parcnegocio/alterarInscricaoParceiro", "Row", params).then(function(response) {
                                ScreenService.closePopup();
                                var widget = templateManager.container.getWidget("wdgTrocaInscricaoParceiro");
                                
                                ScreenService.showMessage("Processo realizado com sucesso!");
                                ScreenService.closeSwipe();
                                widget.activate();
                                widget.reload();
                            }.bind(this))
                            .catch(function(error) {
                                Utilities.showErrorMessage(error);
                            }); 
                        });
                    } else {
                        ScreenService.showMessage("Operação bloqueada!<br><br>Já existe uma organização ativa para a inscrição informada.");

                    }
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
        }
    };
    
    this.downloadDocumento = function(widget) {
        ScreenService.closePopup();
        var params = { "NRINSCRICAOPARC": widget.currentRow.NRINSCRICAOPARC };
        General.callRestEngine("product#/parcnegocio/getTermoAnuenciaParceiro", "Row", params).then(function(response) {
            if(response.dataset && response.dataset.URLDOCUMENTO && response.dataset.URLDOCUMENTO.length > 0) {
                var documentos   = response.dataset.URLDOCUMENTO;
                documentos.forEach(function(link) {
                    var urlDocumento = window.backendIndexUrl + link; 
                    window.open(urlDocumento);
                });
            }
        }).catch(function(error){
            window.open(error);
        });
    };
    
    this.validarInscricao = function(field) {
        var validations = General.inscricaoFormat(field.value());
        if (validations.tipo) {
            field.value(validations.value);
        }
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('Sm00055000_TrocaIncricaoParceiro_Controller', Sm00055000_TrocaIncricaoParceiro_Controller);
});

// FILE: ./assets/js/controllers/Sm00070000_HierarquiaOperador_Controller.js
function Sm00070000_HierarquiaOperador_Controller(Utilities, templateManager, ScreenService, General, ZHPromise) {

    /**
     * Procedimento para recuperar a posição
     * de um objeto dentro de um array.
     * 
     * @param {object} array array onde será feito
     * a varredura.
     * 
     * @param {object} objectName nome do objeto a ser
     * buscado.
     */
    this.getObjectPositionInArrayByName = function(array, objectName) {

        var indexFieldObject;

        array.forEach(function(value, i) {
            if (value.name === objectName) {
                indexFieldObject = i;
            }
        });

        return indexFieldObject;

    };

    /**
     * Defini o filtro do grid de operadores da hirarquia
     * selecionada.
     * 
     * @param {object} widget widget com a lista dos operadores
     * relacionado a hierarquia selecionada('WDGT_LISTA_OPERADORES_HIERARQUIA').
     */
    this.setHierarchyOperatorsWidgetFiler = function(widget) {

        var treeGridWidget = widget.parent.parent;
        var treeGridWidgetCurrentRow = treeGridWidget.currentRow;

        var parentHierarchyNumberIndex = this.getObjectPositionInArrayByName(widget.dataSourceFilter, "NRHIERARQUIAPAI");

        widget.dataSourceFilter[parentHierarchyNumberIndex].value = treeGridWidgetCurrentRow.NRHIERARQUIAOPER;
    };


    /**
     * Salva uma nova hierarquia.
     * 
     * @param {object} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
     */
    this.saveHierarchy = function(widget) {

        var currentRow = widget.currentRow;

        currentRow.__columns_to_return = ["NRHIERARQUIAOPER"];

        if (widget.isValid()) {

            if (!Boolean(currentRow.NRHIERARQUIAOPERRESP)) {

                General.widgetSave(widget).then(function(response) {

                    currentRow.NRHIERARQUIAOPERRESP = response.dataset.columnsToReturn[0].NRHIERARQUIAOPER;
                    this.saveHierarchyOperators(widget).then(function() {
                        widget.reload();
                    });

                }.bind(this));

            }
            else {

                this.saveHierarchyOperators(widget).then(function() {
                    ScreenService.closeSwipe().then(function() {
                        widget.reload();
                    });
                });

            }

            if (Boolean(widget.ISTRANSFER)) {

                this.returnOperRespFieldDefaultFilter(widget.getField('DSHIERARQUIAOPER'));
                this.deleteTransferParamOnWidget(widget);

            }

        }

    };

    /**
     * Salva os operadores de uma hierarquia.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
     */
    this.saveHierarchyOperators = function(widget) {

        var currentRow = widget.currentRow;

        if (Boolean(currentRow.CDOPERADORESHIERAR) && currentRow.CDOPERADORESHIERAR.length > 0) {

            var hierarchyOperators = this.prepareHierarchyOperatorDatasToSave(currentRow);
            return General.callRestEngine("product#/crud/crm_hierarquiaoper/save", "DataSet", hierarchyOperators);

        }
        else {

            var zhPromise = ZHPromise.defer();
            zhPromise.resolve();

            return zhPromise.promise;
        }

    };

    /**
     * Define os parâmetros para gerar um novo código
     * para o registro a ser salvo.
     * 
     * @param {object} row objeto que será persistido.
     * 
     * @param {String} tableName tabela referência para
     * gerar o novo código
     * 
     * @param {String} tablePk nome da Primary Key da tabela
     * que recebera o código gerado.
     * 
     * @param {string} columnToReturn coluna a ser retornada.
     */
    this.setColumnToReturnParamsOnRow = function(row, tableName, tablePk, columnToReturn) {
        row.__is_new_code = true;
        row.__new_code_pk = tableName;
        row.__table_pk = tablePk;
        row.__new_code_length = 20;
        row.__columns_to_return = [columnToReturn];

    };

    /**
     * Prepara os registros dos operadores para serem
     * salvos na hierarquia em caso.
     * 
     * @param {object} currentRow currentRow da hierarquia
     * que está sendo criada;
     */
    this.prepareHierarchyOperatorDatasToSave = function(currentRow) {

        var operators = [];
        var object;

        currentRow.CDOPERADORESHIERAR.forEach(function(operator, i) {

            object = {};

            this.setColumnToReturnParamsOnRow(object, "CRM_HIERARQUIAOPER", "NRHIERARQUIAOPER", "NRHIERARQUIAOPER");

            object.__is_new = "__is_new";
            object.CDOPERADOR = currentRow.CDOPERADORESHIERAR[i];
            object.DSHIERARQUIAOPER = currentRow.NMOPERADORHIERAR[i].toUpperCase();
            object.NRHIERARQUIAPAI = currentRow.NRHIERARQUIAOPERRESP;
            object.IDOPERMASTER = 'N';

            operators.push(object);

        }.bind(this));

        return operators;

    };

    /**
     * Tira a possibilidade do operador selecionado como
     * responsável ser selecionado como operador relacionado.
     * 
     * @param {object} field field de operadores relacionados.
     */
    this.setRelatedOperatorsFilter = function(field) {

        var widget = field.widget;
        var currentRow = widget.currentRow;

        var fieldFilter = field.dataSourceFilter;
        var cdoperadorFilterIndex = this.getObjectPositionInArrayByName(fieldFilter, "CDOPERADOR");

        fieldFilter[cdoperadorFilterIndex].value = currentRow.CDOPERADOR;

    };

    /**
     * Executa as funções da widget no evento
     * "WidgetAfterMoveRow".
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
     */
    this.widgetAfterMoveRow = function(widget) {

        this.setActionToDeleteVisibilityBySelectedHierarchy(widget);

    };

    /**
     * Define a visibilidade da ação de excluir hierarquias.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
     */
    this.setActionToDeleteVisibilityBySelectedHierarchy = function(widget) {

        var currentRow = widget.currentRow;

        var actionToDeleteIndex = this.getObjectPositionInArrayByName(widget.actions, "DELETAR_HIERARQUIA");

        if (currentRow._childCount !== 0) {
            widget.actions[actionToDeleteIndex].isVisible = false;
        }
        else {
            widget.actions[actionToDeleteIndex].isVisible = true;
        }

    };

    /**
     * Habilita a edição de uma hierarquia.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
     */
    this.editHierarchy = function(widget) {

        var currentRow = widget.currentRow;

        if (currentRow._childCount !== 0) {
            this.transferHierarchy(widget);
        }
        else {
            General.widgetEdit(widget);
            this.returnOperRespFieldDefaultFilter(widget.getField('DSHIERARQUIAOPER'));
            this.deleteTransferParamOnWidget(widget);
        }

    };

    /**
     * Habilita a transferência de uma hierarquia.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
     */
    this.transferHierarchy = function(widget) {

        General.widgetEdit(widget);

        this.setTransferParamsOnWidget(widget);

    };

    /**
     * Função para remover um objeto do array(filtro)
     * recebido como parâmetro.
     * 
     * @param {object} filter array com os objetos(filtros).
     * 
     * @param {int} objectIndex índice do objecto a ser removido
     * do array(filtro).
     */
    this.deleteFilterObject = function(filter, objectIndex) {

        if (objectIndex === 0) {
            filter.shift();
        }
        else if (objectIndex === filter.length - 1) {
            filter.pop();
        }
        else {
            filter = filter.slice(0, objectIndex).concat(filter.slice(objectIndex + 1));
        }

        return filter;
    };

    /**
     * Retorna o filtro do campo "Operador responsável"
     * para o padrão.
     * 
     * @param {object} field campo de operador responsável.
     */
    this.returnOperRespFieldDefaultFilter = function(field) {

        var fieldFilter = field.dataSourceFilter;
        var nrhierarquiaoperFilterIndex = this.getObjectPositionInArrayByName(fieldFilter, "NRHIERARQUIAOPER");

        if (nrhierarquiaoperFilterIndex === 0 || nrhierarquiaoperFilterIndex) {

            field.dataSourceFilter = this.deleteFilterObject(fieldFilter, nrhierarquiaoperFilterIndex);
            field.reload();

        }

    };

    /**
     * Define um parâmetro na widget indicando uma
     * trânferencia de hierarquia.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
     */
    this.setTransferParamsOnWidget = function(widget) {

        widget.ISTRANSFER = true;

    };

    /**
     * Exclui o parâmetro na widget indicando uma
     * trânferencia de hierarquia.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
     */
    this.deleteTransferParamOnWidget = function(widget) {

        if (Boolean(widget.ISTRANSFER)) {
            delete widget.ISTRANSFER;
        }

    };

    /**
     * Deleta uma hierarquia e atualiza a widget
     * em seguida.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
     */
    this.deleteHierarchy = function(widget) {

        General.deleteCrud(widget).then(function() {
            widget.reload();
        });

    };

    /**
     * Define o filtro do campo "Hierarquia Superior" para
     * retornar apenas hierarquias superiores ou no mesmo nível
     * da hierarquia selecionada.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
     */
    this.setSuperiorHierarchyFilter = function(widget) {

        var currentRow = widget.currentRow;

        var SuperiorHierarchyField = widget.getField('DSHIERARQUIAOPERPAI');
        var fieldFilter = SuperiorHierarchyField.dataSourceFilter;

        var nrhierarquiaoperFilterIndex = this.getObjectPositionInArrayByName(fieldFilter, "P_HIERARQUIATUAL");
        
        if(currentRow.NRHIERARQUIAOPER != ""){
            fieldFilter[nrhierarquiaoperFilterIndex].value = currentRow.NRHIERARQUIAOPER;
        } else {
            var ultimaHierarquia = this.pegaUltimoHierarquia(widget);
            fieldFilter[nrhierarquiaoperFilterIndex].value = ultimaHierarquia;
        }
        
        SuperiorHierarchyField.reload();

    };
    
    this.pegaUltimoHierarquia = function(widget){
        var ultimo = 1;
        
        widget.dataSource.data.forEach(function(dado){
            if(dado.NRHIERARQUIAOPER > ultimo){
                ultimo = dado.NRHIERARQUIAOPER;
            }
        });
        
        return (ultimo+1);
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Sm00070000_HierarquiaOperador_Controller', Sm00070000_HierarquiaOperador_Controller);
});


// FILE: ./assets/js/controllers/SmNext_Controller.js
function SmNext_Controller(LibUtilities) {

    this.redirectOnContainerAfterinit = function(widget, rotaDaTela) {
        var token     = window.localStorage.getItem('LOGIN_TOKEN');
        var hash      = window.localStorage.getItem('LOGIN_HASH') || '0';
        var user      = window.localStorage.getItem('NEXTSESSION_ENCRYPTED');
        var languange = window.localStorage.getItem('LOGIN_LANGUAGE');
        var productId = window.projectId;

        var url = window.urlSmNext + rotaDaTela + "?header=0&menu=0" +
                                                  "&token=" + token +
                                                  "&hash=" + hash +
                                                  "&user=" + user +
                                                  "&languange=" + languange +
                                                  "&productId=" + productId +
                                                  "&isZhAngular=1";


        clearTimeout(LibUtilities.expiredSessionMethod);

        widget.sourceUrl = url;
    };
}
//redirecionarSMNEXT_Controllerr

window.Configuration(function(ContextRegister) {
    ContextRegister.register('SmNext_Controller',  SmNext_Controller);
});


// FILE: ./assets/js/controllers/zhFavorites.js
var ZeedhiFields = angular.module("ZeedhiFields");

ZeedhiFields.directive('zhNavbarDefaultHeader', function(templateManager, ScreenService, eventAggregator, $compile, ApplicationContext) {
	return {
		restrict: 'C',
		link: function($scope, element) {
		    $scope.favoritesList = [];
		    $scope.lastDeletedItem = undefined;
		    
		    var favoritesText = 'Favoritos';
		    var favoritesAddText = 'Adicionar aos Favoritos';
		    var favoritesRemoveText = 'Remover dos Favoritos';
		    var favoritesUndoText = 'DESFAZER';

			// TRANSLATE - Adicionar essas variaveis a lista da aplicação (words.jon)
		    // var favoritesText = ScreenService.i18n('FAVORITES');
		    // var favoritesAddText = ScreenService.i18n('ADD_TO_FAVORITES');
		    // var favoritesRemoveText = ScreenService.i18n('REMOVE_FROM_FAVORITES');
		    // var favoritesUndoText = ScreenService.i18n('UNDO');
		    
		    var favoritesContainer;
		    
		    var startHollowCode = '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 30 30" style="enable-background:new 0 0 30 30;" xml:space="preserve">'+
				'	<style type="text/css">'+
				'		.st0{fill:#FFFFFF;}'+
				'	</style>'+
				'	<g>'+
				'		<path class="st0" d="M15,4.21l3.51,7.1l7.84,1.14l-5.67,5.53l1.34,7.81L15,22.1l-7.01,3.69l1.34-7.81l-5.67-5.53l7.84-1.14L15,4.21'+
				'			 M15,2.21c-0.76,0-1.46,0.43-1.79,1.11l-3.04,6.16l-6.8,0.99c-0.75,0.11-1.38,0.64-1.61,1.36c-0.24,0.72-0.04,1.52,0.51,2.05'+
				'			l4.92,4.8l-1.16,6.77c-0.13,0.75,0.18,1.51,0.8,1.96c0.35,0.25,0.76,0.38,1.18,0.38c0.32,0,0.64-0.08,0.93-0.23l6.08-3.2l6.08,3.2'+
				'			c0.29,0.15,0.61,0.23,0.93,0.23c0.41,0,0.83-0.13,1.18-0.38c0.62-0.45,0.92-1.21,0.8-1.96l-1.16-6.77l4.92-4.8'+
				'			c0.55-0.53,0.74-1.33,0.51-2.05c-0.24-0.72-0.86-1.25-1.61-1.36l-6.8-0.99l-3.04-6.16C16.46,2.64,15.76,2.21,15,2.21L15,2.21z"/>'+
				'	</g>'+
				'</svg>';
		    var startFilledCode = '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 30 30" style="enable-background:new 0 0 30 30;" xml:space="preserve">'+
				'	<style type="text/css">'+
				'		.st0{fill:#FFFFFF;}'+
				'	</style>'+
				'	<g>'+
				'		<path class="st0" d="M28.25,11.83c-0.24-0.72-0.86-1.25-1.61-1.36l-6.8-0.99l-3.04-6.16C16.46,2.64,15.76,2.21,15,2.21'+
				'			s-1.46,0.43-1.79,1.11l-3.04,6.16l-6.8,0.99c-0.75,0.11-1.38,0.64-1.61,1.36c-0.24,0.72-0.04,1.52,0.51,2.05l4.92,4.8l-1.16,6.77'+
				'			c-0.13,0.75,0.18,1.51,0.8,1.96c0.35,0.25,0.76,0.38,1.18,0.38c0.32,0,0.64-0.08,0.93-0.23l6.08-3.2l6.08,3.2'+
				'			c0.29,0.15,0.61,0.23,0.93,0.23c0.41,0,0.83-0.13,1.18-0.38c0.62-0.45,0.92-1.21,0.8-1.96l-1.16-6.77l4.92-4.8'+
				'			C28.29,13.35,28.48,12.56,28.25,11.83z"/>'+
				'	</g>'+
				'</svg>';
		    
		    function createVisualElements() {
    			var headerContainer = $(element).children()[0];
    			headerContainer.style.width = 'calc(100% - 50px)';
    			
    			favoritesContainer = $('<div class="zh-favorites"></div>').appendTo(element);
    			
    			var favoritesButtonElement = $compile(
    				'<div class="zh-favorites-button" ng-click="toggleList()">'+
    				'  <div class="zh-favorites-button-star" ng-if="!isFavorited">'+
    				'    '+startHollowCode+
    				'  </div>'+
    				'  <div class="zh-favorites-button-star" ng-if="isFavorited">'+
    				'    '+startFilledCode+
    				'  </div>'+
    				'  <div class="zh-favorites-button-text">'+favoritesText+'</div>'+
    				'</div>'
    			)($scope);
    			var favoritesButton = $(favoritesButtonElement).appendTo(favoritesContainer);
    			
    			var favoritesListElement = $compile(
    				'<div class="zh-favorites-list" ng-class="{\'opened\': opened}">'+
    				'  <div class="zh-favorites-list-button" ng-if="!isFavorited" ng-click="addToFavorites()">'+favoritesAddText+'</div>'+
    				'  <div class="zh-favorites-list-button" ng-if="isFavorited" ng-click="removeFromFavorites()">'+favoritesRemoveText+'</div>'+
    				'  <div class="zh-favorites-list-undo" ng-show="lastDeletedItem" ng-click="undo()">'+favoritesUndoText+'</div>'+
    				'  <div class="zh-favorites-list-items" ng-show="favoritesList.length > 0">'+
    				'	 <div class="zh-favorites-list-item" ng-repeat="item in favoritesList track by item.id" ng-click="openItem(item)">'+
    				'      {{ item.label }}'+
    				'      <span class="zh-favorites-list-item-delete" data-zh-icon="trash icon-white no-border" ng-click="deleteItem(item, $index, $event)"></span>'+
    				'    </div>'+
    				'  </div>'+
    				'</div>'
    			)($scope);
    			var favoritesList = $(favoritesListElement).appendTo(favoritesContainer);
		    }
		    
		    function checkIsFavorited(containerId) {
		    	var oldIsFavorited = $scope.isFavorited;
			    $scope.isFavorited = $scope.favoritesList.findIndex(function(item) { item.id === containerId }) !== -1;
			    if (oldIsFavorited !== $scope.isFavorited) {
			    	templateManager.updateTemplate();
			    }
		    }
		    
		    $scope.addToFavorites = function() {
		    	var item = {
		    		id: templateManager.container.id,
		    		name: templateManager.container.fileName.split('.json')[0],
		    		label: templateManager.container.label,
		    	};
		    	$scope.favoritesList.push(item);
		    	$scope.isFavorited = true;
		    	eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item: item, status: 'add' }});
		    };
		    
		    $scope.removeFromFavorites = function() {
		    	var index = $scope.favoritesList.findIndex(function(item) { item.id === templateManager.container.id });
		    	if (index > -1) {
			    	var item = $scope.favoritesList.splice(index, 1);
			    	$scope.isFavorited = false;
			    	eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item: item, status: 'delete' }});
		    	}
		    };
		    
		    $scope.openItem = function(item) {
		    	ApplicationContext.LibUtilities.openWindow(item.name);
	    		$scope.opened = false;
		    };
		    
		    var undoTimeout;
		    
		    $scope.deleteItem = function(item, index, event) {
		    	$scope.favoritesList.splice(index, 1);
		    	$scope.lastDeletedItem = [item, index ];

		    	clearTimeout(undoTimeout);
		    	undoTimeout = setTimeout(function() {
		    		$scope.lastDeletedItem = undefined;
		    		templateManager.updateTemplate();
		    	}, 10000);

			    eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item: item, status: 'delete' }});

		    	if (item.id === templateManager.container.id) {
		    		$scope.isFavorited = false;
		    	}

		    	event.stopImmediatePropagation();
		    };
		    
		    $scope.undo = function() {
		    	if ($scope.lastDeletedItem) {
		    		$scope.favoritesList.splice($scope.lastDeletedItem.index, 0, $scope.lastDeletedItem.item);
		    	}
		    	$scope.lastDeletedItem = undefined;
		    };

		    $scope.toggleList = function() {
		    	$scope.opened = !$scope.opened;
		    };
		    
		    document.addEventListener('mousedown', function (evt) {
		    	if ($(favoritesContainer).find(evt.target).length === 0 && $scope.opened) {
		    		$scope.opened = false;
			    	templateManager.updateTemplate();
		    	}	
		    });
		    
		    eventAggregator.subscribe(eventAggregator.ON_CONTAINER_OPEN, function(args) {
		    	checkIsFavorited(args.container.id);
		    });
		    
		    eventAggregator.subscribe('ZhFavoritesOnLoad', function (favorites) {
		    	
		    	$scope.favoritesList = favorites;
			    if (templateManager.container) {
			    	checkIsFavorited(templateManager.container.id);
			    }
		    });

		    createVisualElements();
		}
	};
});
